function chgtab = scenarios_ACTIVSg200
%SCENARIOS_ACTIVSG200  Returns a change table suitable for use with APPLY_CHANGES
%
%   This change table provides zonal load changes for 8760 hours
%   for case_ACTIVSg200 for the year 2017. Requires swapping the BUS_AREA
%   and ZONE columns in the BUS matrix of the MATPOWER case struct before
%   applying via APPLY_CHANGES.
%
%   One-line diagrams and other data formats available at:
%       https://electricgrids.engr.tamu.edu

%   Created from data in ACTIVSg200.tsb.

%   MATPOWER
%   Copyright (c) 2017 by A.B. Birchfield, T. Xu, K.M. Gegner, K.S. Shetye,
%   and T.J. Overbye
%   Licensed under the Creative Commons Attribution 4.0 International license,
%   https://creativecommons.org/licenses/by/4.0/

define_constants;

%% Change Table
%	label	prob	table	row	col	chgtype	newval
chgtab = [
	1	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.8;
	1	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.2;
	1	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.5;
	1	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.2;
	1	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	1	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	228.5;
	2	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	378.4;
	2	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.9;
	2	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.2;
	2	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.4;
	2	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.9;
	2	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.8;
	3	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	374.3;
	3	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.8;
	3	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190;
	3	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193;
	3	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	3	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.6;
	4	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	378.7;
	4	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.1;
	4	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.8;
	4	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192;
	4	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	4	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.3;
	5	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	378.3;
	5	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133;
	5	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.3;
	5	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.5;
	5	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.7;
	5	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311.5;
	6	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394.6;
	6	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.3;
	6	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.6;
	6	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.8;
	6	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.9;
	6	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.8;
	7	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403.3;
	7	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.5;
	7	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.6;
	7	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.1;
	7	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61;
	7	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.8;
	8	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.5;
	8	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146;
	8	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.2;
	8	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.4;
	8	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	8	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.7;
	9	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.6;
	9	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153;
	9	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.6;
	9	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.2;
	9	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	9	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.1;
	10	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.9;
	10	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.9;
	10	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.1;
	10	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.9;
	10	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	10	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.7;
	11	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.4;
	11	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.9;
	11	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.6;
	11	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.6;
	11	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	11	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.6;
	12	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.2;
	12	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161;
	12	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205;
	12	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.9;
	12	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	12	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.8;
	13	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.2;
	13	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160;
	13	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.4;
	13	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.1;
	13	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	13	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.4;
	14	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.9;
	14	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.2;
	14	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.8;
	14	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.5;
	14	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.1;
	14	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.6;
	15	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.4;
	15	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158;
	15	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.8;
	15	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.4;
	15	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.8;
	15	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.7;
	16	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.1;
	16	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161;
	16	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.1;
	16	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.2;
	16	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68;
	16	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349;
	17	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.5;
	17	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.1;
	17	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.2;
	17	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.4;
	17	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.5;
	17	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.7;
	18	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.4;
	18	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.4;
	18	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.7;
	18	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.9;
	18	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.4;
	18	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.3;
	19	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.3;
	19	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.4;
	19	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.7;
	19	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.7;
	19	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.5;
	19	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.3;
	20	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.5;
	20	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.3;
	20	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.9;
	20	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.3;
	20	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.4;
	20	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.4;
	21	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.5;
	21	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.6;
	21	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.4;
	21	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.5;
	21	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.6;
	21	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.1;
	22	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.1;
	22	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.5;
	22	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.4;
	22	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.1;
	22	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.8;
	22	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372;
	23	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464;
	23	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.6;
	23	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.9;
	23	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.9;
	23	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.8;
	23	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.8;
	24	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443;
	24	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.7;
	24	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.2;
	24	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205;
	24	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	24	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.6;
	25	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.3;
	25	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.3;
	25	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.8;
	25	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.9;
	25	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	25	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.5;
	26	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.6;
	26	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134;
	26	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.7;
	26	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.1;
	26	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	26	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.7;
	27	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.8;
	27	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.3;
	27	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.3;
	27	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.4;
	27	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	27	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.9;
	28	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.7;
	28	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.6;
	28	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.8;
	28	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.9;
	28	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	28	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.9;
	29	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.3;
	29	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.4;
	29	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.8;
	29	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198;
	29	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	29	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.1;
	30	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.1;
	30	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.2;
	30	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.5;
	30	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.7;
	30	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	30	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.7;
	31	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.3;
	31	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.6;
	31	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.3;
	31	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.9;
	31	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.1;
	31	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.5;
	32	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.1;
	32	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168;
	32	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.1;
	32	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.9;
	32	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.1;
	32	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.3;
	33	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.8;
	33	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173;
	33	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.7;
	33	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.3;
	33	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.9;
	33	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.3;
	34	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.1;
	34	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.5;
	34	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.1;
	34	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.3;
	34	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73;
	34	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.2;
	35	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.2;
	35	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.2;
	35	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.7;
	35	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.5;
	35	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.3;
	35	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383;
	36	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.5;
	36	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.4;
	36	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.1;
	36	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.7;
	36	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.1;
	36	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384;
	37	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.1;
	37	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.7;
	37	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.6;
	37	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.9;
	37	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.1;
	37	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.7;
	38	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502;
	38	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.2;
	38	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.5;
	38	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.9;
	38	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68;
	38	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376;
	39	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.5;
	39	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.7;
	39	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.9;
	39	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.2;
	39	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	39	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.8;
	40	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.9;
	40	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.7;
	40	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.5;
	40	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.1;
	40	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	40	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380;
	41	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528;
	41	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.9;
	41	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.4;
	41	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252;
	41	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	41	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.1;
	42	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549.5;
	42	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	203.1;
	42	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	259.3;
	42	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265.6;
	42	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	42	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.5;
	43	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	553.6;
	43	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.6;
	43	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.8;
	43	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.6;
	43	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.1;
	43	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.7;
	44	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.6;
	44	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.9;
	44	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.2;
	44	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.6;
	44	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.8;
	44	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.8;
	45	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.1;
	45	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186;
	45	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.6;
	45	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.7;
	45	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.7;
	45	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.2;
	46	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.9;
	46	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.3;
	46	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.3;
	46	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.5;
	46	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	46	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384;
	47	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.1;
	47	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.5;
	47	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.4;
	47	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.9;
	47	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.9;
	47	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.9;
	48	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471;
	48	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.9;
	48	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.6;
	48	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.2;
	48	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	48	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.9;
	49	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455;
	49	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.6;
	49	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.4;
	49	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.4;
	49	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	49	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.3;
	50	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.6;
	50	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.9;
	50	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.3;
	50	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.9;
	50	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	50	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.1;
	51	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.5;
	51	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.8;
	51	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.8;
	51	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.7;
	51	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	51	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.7;
	52	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.3;
	52	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.6;
	52	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.9;
	52	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199;
	52	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.2;
	52	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.9;
	53	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.4;
	53	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149;
	53	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.2;
	53	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204;
	53	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	53	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.7;
	54	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.7;
	54	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.5;
	54	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.7;
	54	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.8;
	54	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.4;
	54	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.4;
	55	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490;
	55	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168;
	55	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.5;
	55	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.6;
	55	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	55	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.9;
	56	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.3;
	56	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.2;
	56	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.3;
	56	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.3;
	56	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.7;
	56	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.4;
	57	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.6;
	57	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.6;
	57	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.1;
	57	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.2;
	57	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.2;
	57	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.3;
	58	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.8;
	58	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.4;
	58	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.6;
	58	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.2;
	58	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73;
	58	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.4;
	59	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.7;
	59	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180;
	59	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.2;
	59	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.9;
	59	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.2;
	59	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.2;
	60	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.4;
	60	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.4;
	60	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.4;
	60	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.5;
	60	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.5;
	60	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.3;
	61	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.7;
	61	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.8;
	61	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.5;
	61	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.5;
	61	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	61	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.5;
	62	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.4;
	62	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.6;
	62	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.1;
	62	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237;
	62	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	62	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.4;
	63	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518;
	63	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175;
	63	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.7;
	63	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.6;
	63	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	63	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.8;
	64	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.5;
	64	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.7;
	64	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.5;
	64	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.9;
	64	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	64	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.3;
	65	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.2;
	65	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192;
	65	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.4;
	65	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.5;
	65	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	65	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.1;
	66	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.4;
	66	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	199.7;
	66	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.4;
	66	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.7;
	66	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	66	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.1;
	67	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.2;
	67	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	199.5;
	67	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255;
	67	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.2;
	67	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.8;
	67	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.7;
	68	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.5;
	68	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.8;
	68	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.4;
	68	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.3;
	68	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	68	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.3;
	69	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.4;
	69	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.5;
	69	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244;
	69	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.5;
	69	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.5;
	69	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.3;
	70	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.9;
	70	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180;
	70	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.8;
	70	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.6;
	70	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.1;
	70	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.2;
	71	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.3;
	71	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.1;
	71	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.5;
	71	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.7;
	71	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.3;
	71	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.1;
	72	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.3;
	72	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.8;
	72	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.4;
	72	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.8;
	72	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	72	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.4;
	73	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.8;
	73	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.8;
	73	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.5;
	73	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.1;
	73	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	73	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.8;
	74	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.1;
	74	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.4;
	74	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.9;
	74	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.1;
	74	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	74	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.8;
	75	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.1;
	75	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.4;
	75	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198;
	75	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.4;
	75	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	75	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.9;
	76	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.4;
	76	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.6;
	76	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.5;
	76	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.2;
	76	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	76	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.7;
	77	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.7;
	77	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.6;
	77	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.8;
	77	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208;
	77	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.2;
	77	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.9;
	78	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.1;
	78	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.9;
	78	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.3;
	78	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.5;
	78	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.5;
	78	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.3;
	79	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.7;
	79	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.7;
	79	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.7;
	79	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228;
	79	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70;
	79	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.8;
	80	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523;
	80	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.7;
	80	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.2;
	80	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.4;
	80	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.9;
	80	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.8;
	81	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.5;
	81	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.5;
	81	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.9;
	81	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.7;
	81	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.5;
	81	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.2;
	82	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525.5;
	82	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.2;
	82	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.4;
	82	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.6;
	82	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.1;
	82	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.6;
	83	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535;
	83	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.9;
	83	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.3;
	83	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.1;
	83	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.2;
	83	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.7;
	84	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527;
	84	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.1;
	84	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.4;
	84	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.1;
	84	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.2;
	84	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.1;
	85	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526.9;
	85	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.6;
	85	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.2;
	85	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.1;
	85	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78;
	85	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.4;
	86	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.2;
	86	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.9;
	86	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234;
	86	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.6;
	86	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.4;
	86	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.4;
	87	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.6;
	87	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.5;
	87	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.8;
	87	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.6;
	87	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.1;
	87	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383;
	88	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.6;
	88	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.7;
	88	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.8;
	88	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.8;
	88	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.2;
	88	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.9;
	89	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.6;
	89	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	201.1;
	89	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.5;
	89	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.8;
	89	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.7;
	89	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398;
	90	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.1;
	90	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	205.5;
	90	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.7;
	90	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	263.9;
	90	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.7;
	90	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.1;
	91	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.1;
	91	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	203.9;
	91	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.3;
	91	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.1;
	91	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.6;
	91	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.9;
	92	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.9;
	92	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	199.2;
	92	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.2;
	92	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256;
	92	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.3;
	92	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.1;
	93	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	533.1;
	93	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.2;
	93	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.8;
	93	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.5;
	93	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.8;
	93	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.6;
	94	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.8;
	94	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.2;
	94	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.8;
	94	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.6;
	94	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.1;
	94	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.8;
	95	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494;
	95	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.7;
	95	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.8;
	95	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.1;
	95	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.8;
	95	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.8;
	96	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473;
	96	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.2;
	96	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216;
	96	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.9;
	96	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.8;
	96	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.3;
	97	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.3;
	97	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.4;
	97	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.1;
	97	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.5;
	97	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.1;
	97	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.3;
	98	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.4;
	98	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.9;
	98	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.3;
	98	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.1;
	98	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.8;
	98	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.8;
	99	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.1;
	99	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.2;
	99	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.5;
	99	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.5;
	99	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.7;
	99	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.4;
	100	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.1;
	100	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.3;
	100	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.4;
	100	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.7;
	100	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.7;
	100	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.2;
	101	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.8;
	101	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.8;
	101	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.6;
	101	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.4;
	101	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85;
	101	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.6;
	102	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.9;
	102	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.3;
	102	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211;
	102	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217;
	102	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.3;
	102	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.6;
	103	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518;
	103	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.4;
	103	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.3;
	103	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.3;
	103	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.9;
	103	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.5;
	104	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.8;
	104	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.7;
	104	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.2;
	104	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.7;
	104	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.9;
	104	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.2;
	105	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.5;
	105	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.6;
	105	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.8;
	105	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.3;
	105	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96.6;
	105	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.8;
	106	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.3;
	106	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.7;
	106	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.1;
	106	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.2;
	106	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96.3;
	106	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.8;
	107	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.5;
	107	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184;
	107	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.1;
	107	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.7;
	107	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.4;
	107	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.4;
	108	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	534.8;
	108	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.6;
	108	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.7;
	108	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.4;
	108	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.6;
	108	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.7;
	109	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.3;
	109	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.5;
	109	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.8;
	109	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.3;
	109	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.1;
	109	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.7;
	110	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527;
	110	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.9;
	110	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.5;
	110	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.7;
	110	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.7;
	110	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392;
	111	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.7;
	111	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181;
	111	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.8;
	111	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.9;
	111	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.8;
	111	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.3;
	112	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.8;
	112	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.9;
	112	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.2;
	112	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.5;
	112	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.5;
	112	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.7;
	113	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.2;
	113	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.4;
	113	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.7;
	113	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.7;
	113	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.4;
	113	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.1;
	114	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.2;
	114	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198.3;
	114	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.8;
	114	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260;
	114	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.7;
	114	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.6;
	115	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559;
	115	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	201.2;
	115	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.6;
	115	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262;
	115	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.4;
	115	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.5;
	116	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	557.6;
	116	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.6;
	116	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.2;
	116	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.7;
	116	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.5;
	116	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.4;
	117	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.5;
	117	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.1;
	117	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.9;
	117	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.6;
	117	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.9;
	117	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.6;
	118	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525;
	118	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.5;
	118	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234;
	118	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238;
	118	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.9;
	118	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.7;
	119	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.9;
	119	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.1;
	119	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.8;
	119	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.4;
	119	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.4;
	119	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.7;
	120	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.5;
	120	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.1;
	120	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.6;
	120	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.6;
	120	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.1;
	120	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.8;
	121	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.3;
	121	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.9;
	121	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.3;
	121	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.1;
	121	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.9;
	121	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.6;
	122	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.9;
	122	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.9;
	122	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.6;
	122	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.9;
	122	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.8;
	122	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.3;
	123	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.4;
	123	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.2;
	123	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.2;
	123	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201;
	123	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.8;
	123	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.3;
	124	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.6;
	124	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.1;
	124	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.8;
	124	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.8;
	124	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.3;
	124	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.6;
	125	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.6;
	125	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.3;
	125	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.9;
	125	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.4;
	125	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.8;
	125	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.2;
	126	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.8;
	126	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.6;
	126	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210;
	126	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.7;
	126	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.7;
	126	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.1;
	127	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.2;
	127	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.8;
	127	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220;
	127	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.5;
	127	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.5;
	127	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.5;
	128	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.4;
	128	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.8;
	128	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.9;
	128	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.8;
	128	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.7;
	128	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.7;
	129	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526.6;
	129	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.1;
	129	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.2;
	129	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.9;
	129	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96.1;
	129	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.6;
	130	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	534.2;
	130	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181;
	130	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.9;
	130	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.8;
	130	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96.7;
	130	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.6;
	131	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526.2;
	131	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.4;
	131	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.6;
	131	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.5;
	131	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.4;
	131	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390;
	132	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.3;
	132	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.1;
	132	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.5;
	132	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.3;
	132	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.7;
	132	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.7;
	133	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.3;
	133	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.5;
	133	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.9;
	133	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.3;
	133	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.7;
	133	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.7;
	134	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.8;
	134	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.9;
	134	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.7;
	134	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.1;
	134	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.7;
	134	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.1;
	135	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.7;
	135	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.8;
	135	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.1;
	135	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.1;
	135	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.2;
	135	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.3;
	136	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.8;
	136	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.2;
	136	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.6;
	136	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.6;
	136	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75;
	136	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.1;
	137	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.7;
	137	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.2;
	137	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.7;
	137	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.5;
	137	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.8;
	137	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391;
	138	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.5;
	138	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.7;
	138	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.5;
	138	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.1;
	138	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.3;
	138	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.6;
	139	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.4;
	139	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.3;
	139	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.5;
	139	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259;
	139	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.6;
	139	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.3;
	140	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549.8;
	140	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.1;
	140	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.6;
	140	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.1;
	140	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.2;
	140	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.8;
	141	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	533.5;
	141	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.8;
	141	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242;
	141	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.2;
	141	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.4;
	141	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.7;
	142	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.2;
	142	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.6;
	142	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.6;
	142	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.1;
	142	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.3;
	142	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.1;
	143	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.1;
	143	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.4;
	143	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.1;
	143	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.5;
	143	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.3;
	143	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.6;
	144	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.1;
	144	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.4;
	144	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219;
	144	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.1;
	144	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.9;
	144	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.8;
	145	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.1;
	145	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.2;
	145	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205;
	145	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.5;
	145	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.3;
	145	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.5;
	146	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.2;
	146	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.6;
	146	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.5;
	146	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.3;
	146	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.6;
	146	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346;
	147	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.7;
	147	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.5;
	147	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.5;
	147	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.4;
	147	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	147	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.2;
	148	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.9;
	148	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.6;
	148	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.6;
	148	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.9;
	148	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.6;
	148	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.1;
	149	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.7;
	149	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.5;
	149	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.7;
	149	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.1;
	149	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.6;
	149	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.4;
	150	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.9;
	150	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.4;
	150	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.6;
	150	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.6;
	150	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	150	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359;
	151	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.5;
	151	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.3;
	151	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.9;
	151	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.2;
	151	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.6;
	151	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366;
	152	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.2;
	152	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.5;
	152	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.9;
	152	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.3;
	152	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.5;
	152	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.5;
	153	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.3;
	153	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.6;
	153	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.3;
	153	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.8;
	153	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.2;
	153	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.5;
	154	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503;
	154	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180;
	154	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.5;
	154	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.8;
	154	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.3;
	154	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.6;
	155	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.3;
	155	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.1;
	155	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.6;
	155	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.5;
	155	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.2;
	155	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.1;
	156	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.8;
	156	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.3;
	156	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.1;
	156	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.7;
	156	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.3;
	156	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.4;
	157	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495;
	157	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.5;
	157	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.3;
	157	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223;
	157	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.5;
	157	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.9;
	158	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.9;
	158	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.5;
	158	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.2;
	158	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.9;
	158	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.1;
	158	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371;
	159	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487;
	159	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.8;
	159	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.5;
	159	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.2;
	159	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.1;
	159	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.2;
	160	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.4;
	160	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.6;
	160	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.6;
	160	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.9;
	160	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	160	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.9;
	161	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.8;
	161	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.6;
	161	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.5;
	161	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240;
	161	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72;
	161	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.2;
	162	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.8;
	162	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.1;
	162	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.2;
	162	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.7;
	162	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.6;
	162	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.8;
	163	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528.3;
	163	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.7;
	163	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.6;
	163	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.2;
	163	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.3;
	163	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.8;
	164	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522.1;
	164	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.7;
	164	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.6;
	164	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.9;
	164	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.4;
	164	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.8;
	165	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.1;
	165	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.7;
	165	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.7;
	165	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.1;
	165	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.9;
	165	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.4;
	166	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.7;
	166	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.7;
	166	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.6;
	166	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.4;
	166	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.7;
	166	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.1;
	167	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.2;
	167	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.4;
	167	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.4;
	167	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.2;
	167	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.4;
	167	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.7;
	168	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.1;
	168	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158;
	168	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.9;
	168	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.4;
	168	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.5;
	168	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.4;
	169	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.5;
	169	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.9;
	169	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.8;
	169	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.5;
	169	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	169	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.7;
	170	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.1;
	170	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.1;
	170	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.6;
	170	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.4;
	170	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	170	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.2;
	171	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.7;
	171	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.8;
	171	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.2;
	171	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.1;
	171	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	171	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.2;
	172	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.2;
	172	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.2;
	172	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.2;
	172	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.5;
	172	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	172	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.1;
	173	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.4;
	173	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.2;
	173	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.8;
	173	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200;
	173	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	173	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334;
	174	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	420.8;
	174	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.5;
	174	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.9;
	174	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.6;
	174	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	174	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.5;
	175	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.2;
	175	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.6;
	175	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.5;
	175	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.9;
	175	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.8;
	175	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.1;
	176	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443;
	176	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.7;
	176	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.5;
	176	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.7;
	176	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71;
	176	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.1;
	177	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.6;
	177	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.9;
	177	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.2;
	177	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.9;
	177	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.2;
	177	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352;
	178	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.7;
	178	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.3;
	178	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.8;
	178	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.2;
	178	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.2;
	178	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.7;
	179	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.4;
	179	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.9;
	179	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.7;
	179	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.1;
	179	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.8;
	179	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.3;
	180	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.2;
	180	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.2;
	180	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209;
	180	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.4;
	180	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.7;
	180	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.5;
	181	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.6;
	181	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172;
	181	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.1;
	181	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.4;
	181	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.6;
	181	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.4;
	182	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.2;
	182	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.4;
	182	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.3;
	182	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.4;
	182	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.5;
	182	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.2;
	183	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.7;
	183	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.5;
	183	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.2;
	183	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219;
	183	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.9;
	183	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360;
	184	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.8;
	184	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.9;
	184	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.2;
	184	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.8;
	184	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	184	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.6;
	185	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.4;
	185	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.3;
	185	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.1;
	185	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240;
	185	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.3;
	185	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.1;
	186	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.2;
	186	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.7;
	186	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244;
	186	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.2;
	186	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.7;
	186	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.6;
	187	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	524.3;
	187	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.8;
	187	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.3;
	187	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.2;
	187	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.5;
	187	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.4;
	188	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.9;
	188	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.8;
	188	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.7;
	188	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.7;
	188	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.2;
	188	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.3;
	189	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507;
	189	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.1;
	189	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.6;
	189	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.2;
	189	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.1;
	189	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.4;
	190	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.5;
	190	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.5;
	190	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.9;
	190	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.3;
	190	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	190	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374;
	191	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.5;
	191	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.5;
	191	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.9;
	191	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216;
	191	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.2;
	191	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.4;
	192	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.2;
	192	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.1;
	192	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.5;
	192	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.4;
	192	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	192	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.9;
	193	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.8;
	193	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.2;
	193	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.2;
	193	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.4;
	193	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	193	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345;
	194	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.6;
	194	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138;
	194	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.1;
	194	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.8;
	194	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	194	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.1;
	195	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.2;
	195	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.8;
	195	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.3;
	195	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.6;
	195	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.2;
	195	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.5;
	196	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	444.1;
	196	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.3;
	196	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.2;
	196	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.6;
	196	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61;
	196	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.7;
	197	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.2;
	197	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.3;
	197	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.8;
	197	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.4;
	197	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	197	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.8;
	198	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.7;
	198	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.9;
	198	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.3;
	198	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.3;
	198	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	198	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.5;
	199	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.6;
	199	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.4;
	199	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.2;
	199	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.3;
	199	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	199	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.8;
	200	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.4;
	200	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.6;
	200	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.4;
	200	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.4;
	200	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	200	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.5;
	201	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	574.9;
	201	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.7;
	201	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.8;
	201	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.8;
	201	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	201	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.6;
	202	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	575.4;
	202	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.6;
	202	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.9;
	202	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.7;
	202	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	202	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.5;
	203	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564.3;
	203	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.7;
	203	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.6;
	203	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.4;
	203	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	203	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.8;
	204	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559.3;
	204	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.5;
	204	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.5;
	204	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.2;
	204	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	204	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405;
	205	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	561.5;
	205	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.1;
	205	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.3;
	205	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.6;
	205	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	205	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406;
	206	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.1;
	206	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.6;
	206	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.3;
	206	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.6;
	206	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	206	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.6;
	207	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.7;
	207	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.8;
	207	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.1;
	207	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240;
	207	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	207	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.8;
	208	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566.1;
	208	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.8;
	208	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.7;
	208	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.2;
	208	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	208	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.2;
	209	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.5;
	209	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.2;
	209	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.4;
	209	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.7;
	209	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	209	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420.2;
	210	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	605;
	210	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	203.4;
	210	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.8;
	210	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267.8;
	210	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	210	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	432.3;
	211	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	620.6;
	211	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	206.1;
	211	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262;
	211	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	266.5;
	211	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	211	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	439.2;
	212	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	619.7;
	212	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	205.9;
	212	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.3;
	212	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.5;
	212	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	212	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	437.5;
	213	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	600.3;
	213	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	199.5;
	213	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250;
	213	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.8;
	213	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.2;
	213	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	427;
	214	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	581.7;
	214	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.5;
	214	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.5;
	214	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.6;
	214	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	214	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.8;
	215	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.3;
	215	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181;
	215	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226;
	215	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.1;
	215	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	215	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.4;
	216	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.9;
	216	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.2;
	216	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.9;
	216	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.8;
	216	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.1;
	216	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.8;
	217	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.8;
	217	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.2;
	217	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.6;
	217	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.6;
	217	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	217	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.4;
	218	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.3;
	218	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.6;
	218	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.8;
	218	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.1;
	218	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.6;
	218	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370;
	219	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.6;
	219	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.2;
	219	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.7;
	219	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.4;
	219	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.7;
	219	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.1;
	220	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.7;
	220	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.3;
	220	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.3;
	220	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.8;
	220	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55;
	220	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.4;
	221	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.6;
	221	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.8;
	221	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.9;
	221	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.1;
	221	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.8;
	221	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.9;
	222	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.7;
	222	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.2;
	222	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.6;
	222	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.3;
	222	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55;
	222	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.4;
	223	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	593;
	223	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	201.5;
	223	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.8;
	223	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.9;
	223	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.6;
	223	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.7;
	224	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	620.8;
	224	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	207;
	224	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.8;
	224	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.8;
	224	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.4;
	224	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	431.8;
	225	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	626.3;
	225	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	204.6;
	225	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.4;
	225	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.8;
	225	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.3;
	225	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	435.6;
	226	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	614.7;
	226	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	202.8;
	226	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.6;
	226	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252;
	226	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59;
	226	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	431.2;
	227	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	600.3;
	227	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	201.7;
	227	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.7;
	227	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251;
	227	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	227	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.2;
	228	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.5;
	228	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	199;
	228	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.8;
	228	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.1;
	228	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	228	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.8;
	229	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.3;
	229	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.6;
	229	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.7;
	229	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249;
	229	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	229	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418;
	230	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577;
	230	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.7;
	230	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.2;
	230	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.2;
	230	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	230	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.3;
	231	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.2;
	231	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.5;
	231	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.1;
	231	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.4;
	231	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.1;
	231	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.6;
	232	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	561.7;
	232	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.9;
	232	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.2;
	232	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.7;
	232	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.9;
	232	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.6;
	233	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.6;
	233	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	207.3;
	233	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.6;
	233	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	266.2;
	233	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60;
	233	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.3;
	234	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	593;
	234	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	215;
	234	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	268.1;
	234	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.1;
	234	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61;
	234	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	429.8;
	235	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	602.7;
	235	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	216.4;
	235	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	266.4;
	235	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	269.9;
	235	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	235	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	434;
	236	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	601;
	236	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	213.4;
	236	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.1;
	236	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.7;
	236	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	236	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	431.6;
	237	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588;
	237	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	206.6;
	237	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.8;
	237	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.8;
	237	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	237	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.3;
	238	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577;
	238	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.5;
	238	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.8;
	238	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.8;
	238	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	238	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.5;
	239	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	554.2;
	239	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.7;
	239	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.4;
	239	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230;
	239	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	239	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.4;
	240	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.5;
	240	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.8;
	240	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.9;
	240	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.8;
	240	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	240	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.1;
	241	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.6;
	241	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.1;
	241	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.7;
	241	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.1;
	241	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	241	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.8;
	242	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.3;
	242	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.9;
	242	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.6;
	242	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.3;
	242	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	242	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.1;
	243	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508;
	243	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.2;
	243	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.8;
	243	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.9;
	243	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	243	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.7;
	244	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521;
	244	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.1;
	244	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207;
	244	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210;
	244	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	244	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.7;
	245	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537.3;
	245	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176;
	245	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.7;
	245	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.1;
	245	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	245	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387;
	246	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.4;
	246	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.2;
	246	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.5;
	246	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.2;
	246	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	246	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.1;
	247	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577.7;
	247	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	203.3;
	247	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.2;
	247	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240;
	247	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.5;
	247	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.4;
	248	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	602.1;
	248	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	206.5;
	248	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.3;
	248	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.5;
	248	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.7;
	248	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	424.5;
	249	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	594.1;
	249	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	206.3;
	249	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.3;
	249	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.9;
	249	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.1;
	249	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.7;
	250	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	585.9;
	250	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	201.6;
	250	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.1;
	250	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.6;
	250	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.8;
	250	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.5;
	251	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	585.5;
	251	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198.6;
	251	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.1;
	251	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.2;
	251	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82;
	251	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.8;
	252	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	568.9;
	252	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.3;
	252	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.9;
	252	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.7;
	252	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.3;
	252	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.2;
	253	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559.9;
	253	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193;
	253	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.3;
	253	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.5;
	253	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.6;
	253	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.6;
	254	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.4;
	254	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.2;
	254	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.5;
	254	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.4;
	254	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.1;
	254	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.7;
	255	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.4;
	255	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.1;
	255	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.5;
	255	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.6;
	255	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.2;
	255	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.3;
	256	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	554.1;
	256	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.5;
	256	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.1;
	256	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.3;
	256	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.2;
	256	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.8;
	257	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552.6;
	257	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	206;
	257	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	259.9;
	257	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.7;
	257	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73;
	257	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.9;
	258	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	578.9;
	258	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	212.5;
	258	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	265.4;
	258	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	270.3;
	258	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.6;
	258	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.5;
	259	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	590.8;
	259	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	213;
	259	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	264.6;
	259	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	268.3;
	259	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.7;
	259	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	428.4;
	260	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	587.8;
	260	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	209.7;
	260	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	259.2;
	260	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.9;
	260	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.3;
	260	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.1;
	261	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	571.2;
	261	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	204.1;
	261	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.5;
	261	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.8;
	261	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.2;
	261	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.3;
	262	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	553.6;
	262	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.7;
	262	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.3;
	262	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.2;
	262	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.5;
	262	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.8;
	263	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528.7;
	263	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.4;
	263	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.1;
	263	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.8;
	263	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.5;
	263	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.1;
	264	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.2;
	264	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173;
	264	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.6;
	264	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.4;
	264	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.4;
	264	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.4;
	265	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.3;
	265	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.6;
	265	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.4;
	265	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.3;
	265	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.2;
	265	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.4;
	266	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.6;
	266	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.1;
	266	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.6;
	266	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.2;
	266	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.5;
	266	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.2;
	267	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.8;
	267	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161;
	267	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.3;
	267	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.8;
	267	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.2;
	267	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.5;
	268	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.5;
	268	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.8;
	268	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.6;
	268	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.4;
	268	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74;
	268	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.4;
	269	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.7;
	269	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.7;
	269	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.9;
	269	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212;
	269	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.1;
	269	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.9;
	270	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525.6;
	270	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.8;
	270	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.4;
	270	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.8;
	270	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.8;
	270	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.4;
	271	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566.2;
	271	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.2;
	271	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.9;
	271	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.2;
	271	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.7;
	271	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.7;
	272	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	581.1;
	272	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	199.9;
	272	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.3;
	272	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.8;
	272	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.7;
	272	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416;
	273	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586;
	273	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.2;
	273	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.9;
	273	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.1;
	273	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.2;
	273	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.4;
	274	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	580;
	274	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.5;
	274	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.9;
	274	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.7;
	274	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.5;
	274	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.3;
	275	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566.7;
	275	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.5;
	275	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.7;
	275	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.9;
	275	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.5;
	275	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.5;
	276	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547.5;
	276	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.5;
	276	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.1;
	276	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.5;
	276	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.2;
	276	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.2;
	277	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.5;
	277	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187;
	277	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.8;
	277	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.5;
	277	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.5;
	277	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.5;
	278	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.6;
	278	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.7;
	278	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.9;
	278	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.6;
	278	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.5;
	278	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.5;
	279	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.5;
	279	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.9;
	279	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241;
	279	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.3;
	279	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.8;
	279	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.2;
	280	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531;
	280	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.5;
	280	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.8;
	280	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250;
	280	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.4;
	280	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.4;
	281	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	546.7;
	281	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.7;
	281	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256;
	281	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.5;
	281	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.8;
	281	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.1;
	282	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	555.9;
	282	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	204;
	282	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.8;
	282	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267.9;
	282	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.9;
	282	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.7;
	283	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562;
	283	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	204.3;
	283	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.5;
	283	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265.6;
	283	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.4;
	283	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416;
	284	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	563.4;
	284	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	202.3;
	284	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.4;
	284	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.1;
	284	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78;
	284	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.5;
	285	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564.7;
	285	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.8;
	285	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.3;
	285	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.5;
	285	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79;
	285	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.1;
	286	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.4;
	286	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.9;
	286	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.3;
	286	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.3;
	286	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.5;
	286	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399;
	287	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.6;
	287	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.1;
	287	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.8;
	287	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.8;
	287	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.2;
	287	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384;
	288	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.2;
	288	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.1;
	288	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.8;
	288	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.4;
	288	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74;
	288	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375;
	289	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.7;
	289	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.1;
	289	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.3;
	289	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.4;
	289	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.9;
	289	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.9;
	290	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.6;
	290	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.7;
	290	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.7;
	290	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.4;
	290	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.9;
	290	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.5;
	291	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.3;
	291	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.9;
	291	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.4;
	291	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.8;
	291	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.5;
	291	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.4;
	292	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.2;
	292	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.6;
	292	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.7;
	292	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.7;
	292	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74;
	292	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.4;
	293	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.4;
	293	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.6;
	293	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.9;
	293	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.3;
	293	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.4;
	293	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.4;
	294	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510;
	294	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.6;
	294	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.6;
	294	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.7;
	294	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78;
	294	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.6;
	295	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.1;
	295	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.5;
	295	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.6;
	295	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.8;
	295	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.9;
	295	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.2;
	296	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.3;
	296	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.7;
	296	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.7;
	296	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.2;
	296	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.9;
	296	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.9;
	297	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.8;
	297	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.8;
	297	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244;
	297	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.9;
	297	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.4;
	297	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.8;
	298	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548.6;
	298	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.3;
	298	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.1;
	298	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.7;
	298	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91;
	298	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.6;
	299	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.1;
	299	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.8;
	299	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.6;
	299	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.6;
	299	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.4;
	299	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.6;
	300	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.3;
	300	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189;
	300	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.9;
	300	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.8;
	300	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.5;
	300	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.3;
	301	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528;
	301	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.2;
	301	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.1;
	301	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.8;
	301	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.3;
	301	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.9;
	302	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525.6;
	302	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185;
	302	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.3;
	302	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.7;
	302	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.4;
	302	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.2;
	303	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526.8;
	303	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.1;
	303	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.1;
	303	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.7;
	303	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.7;
	303	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.1;
	304	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.8;
	304	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189;
	304	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.5;
	304	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.7;
	304	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80;
	304	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.3;
	305	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.3;
	305	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.8;
	305	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.1;
	305	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.1;
	305	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81;
	305	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.4;
	306	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	553.4;
	306	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	201.3;
	306	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.9;
	306	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.6;
	306	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.4;
	306	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.5;
	307	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549.7;
	307	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.4;
	307	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.6;
	307	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.5;
	307	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.2;
	307	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.3;
	308	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.5;
	308	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193;
	308	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.3;
	308	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.8;
	308	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.1;
	308	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.3;
	309	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532;
	309	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.7;
	309	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.4;
	309	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.9;
	309	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.2;
	309	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.6;
	310	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.6;
	310	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.5;
	310	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.1;
	310	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.6;
	310	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.3;
	310	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.2;
	311	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.2;
	311	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.6;
	311	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.4;
	311	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.7;
	311	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.9;
	311	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.2;
	312	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.8;
	312	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.8;
	312	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.5;
	312	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.4;
	312	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.4;
	312	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.2;
	313	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.5;
	313	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.4;
	313	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.2;
	313	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.5;
	313	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78;
	313	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.2;
	314	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.7;
	314	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.4;
	314	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.9;
	314	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.6;
	314	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.2;
	314	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.2;
	315	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447.7;
	315	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.8;
	315	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.3;
	315	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.3;
	315	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.9;
	315	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.7;
	316	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.9;
	316	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.6;
	316	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.9;
	316	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.2;
	316	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.3;
	316	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347;
	317	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.5;
	317	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.5;
	317	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.2;
	317	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.5;
	317	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.8;
	317	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.1;
	318	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.2;
	318	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161;
	318	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.9;
	318	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.8;
	318	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.4;
	318	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360;
	319	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.1;
	319	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164;
	319	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.1;
	319	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.6;
	319	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.8;
	319	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.3;
	320	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.4;
	320	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.9;
	320	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.2;
	320	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.2;
	320	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.5;
	320	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.8;
	321	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491;
	321	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.8;
	321	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.6;
	321	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.1;
	321	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84;
	321	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.8;
	322	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.5;
	322	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.5;
	322	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.4;
	322	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.2;
	322	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.7;
	322	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.4;
	323	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.3;
	323	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.8;
	323	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.1;
	323	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.9;
	323	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.8;
	323	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.2;
	324	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.9;
	324	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.2;
	324	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.2;
	324	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.4;
	324	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.1;
	324	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.5;
	325	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.5;
	325	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179;
	325	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.7;
	325	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.7;
	325	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.2;
	325	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.2;
	326	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.5;
	326	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.7;
	326	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.4;
	326	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.2;
	326	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.7;
	326	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.5;
	327	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473;
	327	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.2;
	327	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.7;
	327	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.6;
	327	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.2;
	327	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.6;
	328	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.5;
	328	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.1;
	328	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.3;
	328	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.4;
	328	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.5;
	328	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.2;
	329	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.5;
	329	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.8;
	329	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.1;
	329	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.9;
	329	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	329	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.1;
	330	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.9;
	330	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.4;
	330	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.9;
	330	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.7;
	330	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.7;
	330	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.9;
	331	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.1;
	331	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.9;
	331	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.8;
	331	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.9;
	331	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.2;
	331	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.6;
	332	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.5;
	332	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.4;
	332	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.2;
	332	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.2;
	332	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.3;
	332	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.4;
	333	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.5;
	333	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.7;
	333	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.9;
	333	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.9;
	333	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.1;
	333	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.8;
	334	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.6;
	334	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.9;
	334	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.1;
	334	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.6;
	334	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.5;
	334	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.3;
	335	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.8;
	335	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.9;
	335	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.2;
	335	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.4;
	335	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.1;
	335	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.4;
	336	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.3;
	336	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.5;
	336	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.5;
	336	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.4;
	336	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.2;
	336	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.5;
	337	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.8;
	337	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.8;
	337	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.9;
	337	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.3;
	337	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	337	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.5;
	338	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418;
	338	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.3;
	338	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.4;
	338	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.9;
	338	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	338	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.3;
	339	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.3;
	339	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.9;
	339	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.9;
	339	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.6;
	339	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	339	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.2;
	340	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417.2;
	340	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.3;
	340	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.6;
	340	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.6;
	340	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	340	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.4;
	341	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	426.8;
	341	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.5;
	341	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.8;
	341	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.8;
	341	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	341	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.2;
	342	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.7;
	342	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.5;
	342	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.5;
	342	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.7;
	342	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	342	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.3;
	343	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.6;
	343	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.7;
	343	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.7;
	343	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206;
	343	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	343	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.6;
	344	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.4;
	344	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.9;
	344	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.6;
	344	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.3;
	344	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73;
	344	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.6;
	345	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.7;
	345	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.8;
	345	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.8;
	345	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211;
	345	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.9;
	345	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.9;
	346	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.1;
	346	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.2;
	346	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.4;
	346	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215;
	346	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.4;
	346	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.1;
	347	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.7;
	347	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.4;
	347	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.6;
	347	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.2;
	347	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.1;
	347	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.4;
	348	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.6;
	348	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.2;
	348	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.8;
	348	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.6;
	348	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.2;
	348	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.3;
	349	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.9;
	349	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.5;
	349	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216;
	349	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.5;
	349	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.9;
	349	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.3;
	350	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.2;
	350	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.5;
	350	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216;
	350	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.7;
	350	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.7;
	350	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.3;
	351	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.5;
	351	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.2;
	351	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.8;
	351	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.9;
	351	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.2;
	351	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.4;
	352	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.9;
	352	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.4;
	352	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.3;
	352	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.7;
	352	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.8;
	352	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.1;
	353	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.1;
	353	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.1;
	353	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.4;
	353	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.6;
	353	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.6;
	353	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.8;
	354	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503;
	354	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.9;
	354	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.4;
	354	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.7;
	354	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.7;
	354	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.1;
	355	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.7;
	355	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.7;
	355	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.8;
	355	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246;
	355	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.9;
	355	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.1;
	356	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.8;
	356	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185;
	356	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.3;
	356	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.2;
	356	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.6;
	356	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.6;
	357	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.5;
	357	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.3;
	357	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.1;
	357	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.2;
	357	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	357	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.6;
	358	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475;
	358	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.8;
	358	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.7;
	358	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.7;
	358	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.4;
	358	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.5;
	359	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.4;
	359	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.9;
	359	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213;
	359	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.8;
	359	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	359	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.8;
	360	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442.7;
	360	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.5;
	360	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.1;
	360	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.9;
	360	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	360	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.4;
	361	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428.6;
	361	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.8;
	361	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.5;
	361	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.8;
	361	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	361	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.7;
	362	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.1;
	362	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.2;
	362	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.6;
	362	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.7;
	362	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	362	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.3;
	363	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.3;
	363	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.8;
	363	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.1;
	363	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.8;
	363	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.7;
	363	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.7;
	364	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.2;
	364	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.3;
	364	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.5;
	364	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.4;
	364	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	364	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.5;
	365	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.2;
	365	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152;
	365	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.9;
	365	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.9;
	365	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	365	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.2;
	366	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.6;
	366	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.7;
	366	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.5;
	366	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.7;
	366	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.6;
	366	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.5;
	367	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.5;
	367	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.6;
	367	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.9;
	367	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.9;
	367	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	367	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376;
	368	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.7;
	368	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.9;
	368	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237;
	368	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.3;
	368	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	368	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.4;
	369	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550;
	369	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.4;
	369	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.3;
	369	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.8;
	369	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	369	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.7;
	370	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.5;
	370	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.3;
	370	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.3;
	370	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.4;
	370	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	370	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.6;
	371	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.9;
	371	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.5;
	371	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.2;
	371	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246;
	371	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.8;
	371	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.2;
	372	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.9;
	372	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.6;
	372	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239;
	372	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.8;
	372	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.7;
	372	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.1;
	373	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.1;
	373	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.9;
	373	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238;
	373	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.6;
	373	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.1;
	373	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.4;
	374	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	524.6;
	374	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.4;
	374	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.5;
	374	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.3;
	374	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	374	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.9;
	375	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522.8;
	375	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177;
	375	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.8;
	375	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.1;
	375	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	375	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.3;
	376	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	524.8;
	376	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.7;
	376	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.8;
	376	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.5;
	376	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	376	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.8;
	377	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.2;
	377	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.6;
	377	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.6;
	377	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.4;
	377	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	377	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.6;
	378	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.6;
	378	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.9;
	378	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	259.5;
	378	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265;
	378	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	378	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.2;
	379	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559.4;
	379	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.1;
	379	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.2;
	379	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.1;
	379	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.1;
	379	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413;
	380	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	555.3;
	380	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.7;
	380	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.1;
	380	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256;
	380	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	380	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.5;
	381	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.6;
	381	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.6;
	381	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.7;
	381	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.4;
	381	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.7;
	381	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.6;
	382	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525.7;
	382	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.2;
	382	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.6;
	382	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.6;
	382	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68;
	382	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.7;
	383	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.2;
	383	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.5;
	383	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217;
	383	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.8;
	383	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	383	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.6;
	384	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.1;
	384	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.7;
	384	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.3;
	384	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.8;
	384	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	384	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.1;
	385	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.3;
	385	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.5;
	385	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.2;
	385	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.8;
	385	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	385	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.3;
	386	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442.4;
	386	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145;
	386	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.7;
	386	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.2;
	386	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	386	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343;
	387	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	444.1;
	387	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.2;
	387	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.6;
	387	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.3;
	387	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	387	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.9;
	388	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.7;
	388	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.6;
	388	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196;
	388	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.7;
	388	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	388	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.4;
	389	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.4;
	389	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.5;
	389	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.8;
	389	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.2;
	389	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	389	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.2;
	390	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.3;
	390	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.5;
	390	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.1;
	390	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.7;
	390	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	390	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.2;
	391	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.1;
	391	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.8;
	391	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.6;
	391	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.7;
	391	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	391	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.6;
	392	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	557.7;
	392	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.6;
	392	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.9;
	392	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245;
	392	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.1;
	392	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404;
	393	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.5;
	393	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.6;
	393	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.1;
	393	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249;
	393	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.2;
	393	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408;
	394	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.8;
	394	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.3;
	394	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.4;
	394	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.1;
	394	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71;
	394	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.2;
	395	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.5;
	395	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.6;
	395	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.4;
	395	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.8;
	395	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.9;
	395	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.7;
	396	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.7;
	396	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.1;
	396	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.2;
	396	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.2;
	396	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.5;
	396	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400;
	397	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.9;
	397	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.6;
	397	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.3;
	397	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.1;
	397	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.5;
	397	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.7;
	398	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.7;
	398	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.8;
	398	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.9;
	398	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.8;
	398	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.2;
	398	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.8;
	399	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.9;
	399	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.2;
	399	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.4;
	399	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.9;
	399	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76;
	399	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.7;
	400	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523;
	400	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.9;
	400	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.5;
	400	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.8;
	400	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.9;
	400	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.4;
	401	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.7;
	401	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.4;
	401	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.5;
	401	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.6;
	401	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.7;
	401	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.3;
	402	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.2;
	402	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198.8;
	402	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.8;
	402	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.9;
	402	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.7;
	402	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.7;
	403	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	561.9;
	403	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	199.7;
	403	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.4;
	403	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259;
	403	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.5;
	403	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.8;
	404	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	569.3;
	404	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198;
	404	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250;
	404	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.6;
	404	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79;
	404	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.9;
	405	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	555.5;
	405	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190;
	405	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.7;
	405	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.7;
	405	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.2;
	405	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.4;
	406	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.7;
	406	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.2;
	406	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232;
	406	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.4;
	406	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.1;
	406	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.3;
	407	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.2;
	407	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.3;
	407	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.3;
	407	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.3;
	407	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.2;
	407	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.2;
	408	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.2;
	408	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.6;
	408	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.1;
	408	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.8;
	408	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.7;
	408	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.4;
	409	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.7;
	409	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.8;
	409	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.8;
	409	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.4;
	409	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.1;
	409	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.4;
	410	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.3;
	410	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.4;
	410	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.9;
	410	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.9;
	410	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.1;
	410	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.9;
	411	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.9;
	411	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.5;
	411	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.6;
	411	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.8;
	411	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.4;
	411	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.6;
	412	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.5;
	412	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.9;
	412	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.1;
	412	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.4;
	412	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	412	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.7;
	413	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.5;
	413	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.4;
	413	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.8;
	413	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.6;
	413	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.3;
	413	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.5;
	414	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.2;
	414	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.9;
	414	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.5;
	414	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.3;
	414	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.4;
	414	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.5;
	415	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.6;
	415	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.7;
	415	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.2;
	415	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.9;
	415	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.5;
	415	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.4;
	416	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	581.2;
	416	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.2;
	416	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.1;
	416	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.4;
	416	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.8;
	416	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.6;
	417	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.8;
	417	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.6;
	417	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.8;
	417	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.2;
	417	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.3;
	417	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.3;
	418	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566.4;
	418	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.6;
	418	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.8;
	418	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.2;
	418	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.6;
	418	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.8;
	419	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559.1;
	419	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.5;
	419	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.9;
	419	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.8;
	419	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.5;
	419	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.7;
	420	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552.1;
	420	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.4;
	420	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.1;
	420	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.6;
	420	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.3;
	420	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.1;
	421	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.9;
	421	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.7;
	421	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.6;
	421	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.1;
	421	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.8;
	421	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.3;
	422	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539;
	422	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.9;
	422	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239;
	422	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.1;
	422	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.5;
	422	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.1;
	423	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522.4;
	423	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185;
	423	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.2;
	423	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.3;
	423	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.7;
	423	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391;
	424	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526.9;
	424	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.8;
	424	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.4;
	424	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.5;
	424	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.5;
	424	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.4;
	425	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	540.7;
	425	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198.5;
	425	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.7;
	425	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.6;
	425	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85;
	425	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.8;
	426	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	557.1;
	426	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	206.3;
	426	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	259;
	426	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	263.3;
	426	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.6;
	426	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.3;
	427	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.8;
	427	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	205.2;
	427	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.4;
	427	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.1;
	427	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.8;
	427	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.9;
	428	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559.9;
	428	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	201.9;
	428	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.2;
	428	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.7;
	428	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.4;
	428	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.7;
	429	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	557;
	429	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.9;
	429	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.9;
	429	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.3;
	429	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.7;
	429	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.4;
	430	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.6;
	430	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.7;
	430	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.4;
	430	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.2;
	430	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.8;
	430	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.4;
	431	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.9;
	431	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.8;
	431	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.2;
	431	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.1;
	431	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.2;
	431	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.7;
	432	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.5;
	432	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.5;
	432	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.8;
	432	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.4;
	432	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.9;
	432	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370;
	433	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.9;
	433	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.7;
	433	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.8;
	433	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205;
	433	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90;
	433	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362;
	434	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.1;
	434	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155;
	434	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.6;
	434	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.7;
	434	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.4;
	434	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.1;
	435	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.2;
	435	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.3;
	435	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.3;
	435	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.6;
	435	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.1;
	435	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.4;
	436	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.6;
	436	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.6;
	436	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.9;
	436	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.6;
	436	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.8;
	436	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.5;
	437	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.8;
	437	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.6;
	437	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.2;
	437	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209;
	437	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.7;
	437	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.6;
	438	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.6;
	438	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.8;
	438	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.8;
	438	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.1;
	438	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.6;
	438	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.4;
	439	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.8;
	439	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.4;
	439	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232;
	439	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237;
	439	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.2;
	439	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.1;
	440	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.9;
	440	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.9;
	440	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.3;
	440	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.6;
	440	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	98.6;
	440	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.3;
	441	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	555.1;
	441	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.9;
	441	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.3;
	441	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.4;
	441	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	101.7;
	441	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.5;
	442	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	554.5;
	442	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.7;
	442	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.6;
	442	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.9;
	442	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	101;
	442	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.5;
	443	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.6;
	443	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.6;
	443	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.2;
	443	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.8;
	443	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	98;
	443	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.5;
	444	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.9;
	444	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.2;
	444	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241;
	444	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.3;
	444	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.2;
	444	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.8;
	445	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518.2;
	445	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.9;
	445	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.4;
	445	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.4;
	445	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.8;
	445	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.3;
	446	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.8;
	446	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.8;
	446	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.5;
	446	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.1;
	446	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.7;
	446	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.5;
	447	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504;
	447	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.9;
	447	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.8;
	447	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244;
	447	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.4;
	447	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.6;
	448	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.5;
	448	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.1;
	448	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243;
	448	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.6;
	448	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.8;
	448	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388;
	449	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525.9;
	449	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.7;
	449	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.7;
	449	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.6;
	449	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.5;
	449	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.4;
	450	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	534.9;
	450	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	202.7;
	450	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.8;
	450	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.7;
	450	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.3;
	450	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.4;
	451	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.5;
	451	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	202.8;
	451	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.4;
	451	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261;
	451	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.5;
	451	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.5;
	452	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.2;
	452	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	200.4;
	452	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.2;
	452	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.4;
	452	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.5;
	452	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.5;
	453	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.2;
	453	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.3;
	453	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.3;
	453	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.4;
	453	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.9;
	453	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396;
	454	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509;
	454	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.6;
	454	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.2;
	454	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.2;
	454	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.3;
	454	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.2;
	455	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481;
	455	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.3;
	455	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.1;
	455	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.4;
	455	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.8;
	455	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.8;
	456	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.5;
	456	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.3;
	456	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.9;
	456	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.4;
	456	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.9;
	456	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.1;
	457	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.4;
	457	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.4;
	457	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.7;
	457	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.6;
	457	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87;
	457	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.6;
	458	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.1;
	458	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.2;
	458	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.7;
	458	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.4;
	458	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.5;
	458	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.2;
	459	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.8;
	459	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.7;
	459	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.1;
	459	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201;
	459	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.6;
	459	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.2;
	460	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.8;
	460	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.4;
	460	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.2;
	460	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.2;
	460	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.5;
	460	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.8;
	461	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.1;
	461	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.6;
	461	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.4;
	461	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.8;
	461	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.4;
	461	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.3;
	462	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.8;
	462	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.5;
	462	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.3;
	462	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.1;
	462	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.6;
	462	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.4;
	463	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525.9;
	463	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.9;
	463	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.6;
	463	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.6;
	463	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.7;
	463	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.6;
	464	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	540.9;
	464	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.9;
	464	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.3;
	464	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.9;
	464	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.1;
	464	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.8;
	465	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.5;
	465	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.7;
	465	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.6;
	465	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.1;
	465	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.3;
	465	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400;
	466	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.3;
	466	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.4;
	466	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.7;
	466	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.6;
	466	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.8;
	466	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.5;
	467	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.7;
	467	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.8;
	467	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.4;
	467	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.6;
	467	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.4;
	467	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.3;
	468	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.6;
	468	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.9;
	468	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.8;
	468	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.1;
	468	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.3;
	468	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.1;
	469	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.8;
	469	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182;
	469	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.6;
	469	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.5;
	469	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.9;
	469	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.2;
	470	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.8;
	470	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.5;
	470	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.1;
	470	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.9;
	470	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.5;
	470	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.3;
	471	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.3;
	471	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.8;
	471	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.6;
	471	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.2;
	471	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.7;
	471	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.8;
	472	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.4;
	472	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.8;
	472	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.2;
	472	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.6;
	472	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.5;
	472	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.9;
	473	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.5;
	473	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.3;
	473	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.5;
	473	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.1;
	473	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.6;
	473	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.4;
	474	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.8;
	474	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.9;
	474	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.9;
	474	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.5;
	474	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.2;
	474	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391;
	475	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528.5;
	475	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.9;
	475	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.7;
	475	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257;
	475	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88;
	475	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.3;
	476	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530;
	476	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.2;
	476	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245;
	476	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.9;
	476	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.7;
	476	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.6;
	477	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514;
	477	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.9;
	477	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.7;
	477	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.7;
	477	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88;
	477	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388;
	478	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.3;
	478	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.8;
	478	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.2;
	478	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.7;
	478	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.2;
	478	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.7;
	479	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.9;
	479	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.4;
	479	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.2;
	479	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.8;
	479	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.8;
	479	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.9;
	480	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.7;
	480	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166;
	480	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.8;
	480	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221;
	480	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.8;
	480	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358;
	481	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.5;
	481	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.6;
	481	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204;
	481	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.5;
	481	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.7;
	481	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.2;
	482	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	430;
	482	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154;
	482	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.9;
	482	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.1;
	482	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.6;
	482	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.5;
	483	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.4;
	483	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154;
	483	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.7;
	483	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.7;
	483	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.5;
	483	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.1;
	484	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.7;
	484	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.7;
	484	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.4;
	484	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.7;
	484	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.1;
	484	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.8;
	485	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457;
	485	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.9;
	485	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.8;
	485	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.1;
	485	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.9;
	485	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.9;
	486	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.9;
	486	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.3;
	486	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.5;
	486	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.9;
	486	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.5;
	486	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.3;
	487	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484;
	487	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.9;
	487	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.7;
	487	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.6;
	487	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82;
	487	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.5;
	488	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492;
	488	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.3;
	488	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.9;
	488	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.1;
	488	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.7;
	488	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.9;
	489	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.4;
	489	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.1;
	489	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.3;
	489	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.5;
	489	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.5;
	489	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.4;
	490	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.1;
	490	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.8;
	490	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.2;
	490	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.6;
	490	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.9;
	490	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.8;
	491	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.1;
	491	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185;
	491	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.7;
	491	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.2;
	491	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88;
	491	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.4;
	492	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.3;
	492	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.5;
	492	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.6;
	492	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.5;
	492	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.3;
	492	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.4;
	493	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.5;
	493	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.1;
	493	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218;
	493	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.3;
	493	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84;
	493	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.5;
	494	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.4;
	494	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.9;
	494	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.7;
	494	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.5;
	494	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.9;
	494	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.1;
	495	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.8;
	495	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.7;
	495	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215;
	495	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.8;
	495	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.1;
	495	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363;
	496	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.7;
	496	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177;
	496	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219;
	496	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.1;
	496	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.6;
	496	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.7;
	497	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.7;
	497	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.6;
	497	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.9;
	497	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.6;
	497	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81;
	497	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.6;
	498	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497;
	498	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.6;
	498	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.9;
	498	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.3;
	498	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.2;
	498	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.3;
	499	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.5;
	499	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.2;
	499	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237;
	499	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.4;
	499	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.3;
	499	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.6;
	500	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.1;
	500	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.4;
	500	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.4;
	500	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.1;
	500	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.8;
	500	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.2;
	501	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.8;
	501	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.4;
	501	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.6;
	501	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.4;
	501	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.5;
	501	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.2;
	502	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.8;
	502	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172;
	502	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222;
	502	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.4;
	502	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.3;
	502	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.9;
	503	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.7;
	503	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.3;
	503	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.5;
	503	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219;
	503	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.1;
	503	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.7;
	504	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	430.1;
	504	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.4;
	504	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.1;
	504	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.1;
	504	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.9;
	504	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.7;
	505	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.1;
	505	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.6;
	505	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.2;
	505	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.5;
	505	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.6;
	505	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.8;
	506	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.2;
	506	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150;
	506	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.6;
	506	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.3;
	506	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.5;
	506	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.9;
	507	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.6;
	507	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.1;
	507	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.4;
	507	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.9;
	507	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.4;
	507	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.9;
	508	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417.5;
	508	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.8;
	508	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.9;
	508	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.3;
	508	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.8;
	508	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.7;
	509	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.9;
	509	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.7;
	509	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.6;
	509	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203;
	509	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.3;
	509	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.8;
	510	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.7;
	510	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.9;
	510	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.6;
	510	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.4;
	510	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.7;
	510	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.5;
	511	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.9;
	511	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.8;
	511	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.5;
	511	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.8;
	511	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85;
	511	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.6;
	512	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.2;
	512	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.1;
	512	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.5;
	512	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.6;
	512	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88;
	512	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.8;
	513	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.8;
	513	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174;
	513	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.3;
	513	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212;
	513	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.5;
	513	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.4;
	514	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.1;
	514	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.2;
	514	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.4;
	514	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.2;
	514	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.7;
	514	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.8;
	515	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.4;
	515	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.9;
	515	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.6;
	515	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.7;
	515	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.2;
	515	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363;
	516	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.1;
	516	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177;
	516	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.3;
	516	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219;
	516	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.4;
	516	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.3;
	517	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464;
	517	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.6;
	517	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.9;
	517	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.9;
	517	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.8;
	517	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.5;
	518	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.8;
	518	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174;
	518	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.2;
	518	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221;
	518	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.7;
	518	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.8;
	519	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463;
	519	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.4;
	519	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.4;
	519	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221;
	519	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.6;
	519	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.8;
	520	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.5;
	520	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.2;
	520	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.8;
	520	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.1;
	520	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.3;
	520	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.9;
	521	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.2;
	521	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.2;
	521	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233;
	521	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.2;
	521	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.2;
	521	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.6;
	522	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.9;
	522	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.1;
	522	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.2;
	522	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.4;
	522	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.5;
	522	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.6;
	523	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.9;
	523	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.7;
	523	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241;
	523	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.5;
	523	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.1;
	523	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388;
	524	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.8;
	524	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.9;
	524	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.5;
	524	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.8;
	524	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.6;
	524	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.9;
	525	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.9;
	525	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.7;
	525	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.1;
	525	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.9;
	525	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92;
	525	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.6;
	526	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.7;
	526	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.3;
	526	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226;
	526	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.6;
	526	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.6;
	526	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.3;
	527	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.4;
	527	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163;
	527	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.9;
	527	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.2;
	527	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.2;
	527	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.2;
	528	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.9;
	528	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.3;
	528	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.5;
	528	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.3;
	528	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.4;
	528	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.6;
	529	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.3;
	529	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.8;
	529	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.4;
	529	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.8;
	529	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.3;
	529	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.3;
	530	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.8;
	530	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.5;
	530	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.6;
	530	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.9;
	530	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86;
	530	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.8;
	531	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449;
	531	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.2;
	531	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.4;
	531	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.3;
	531	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.9;
	531	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.4;
	532	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.3;
	532	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.6;
	532	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.1;
	532	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.3;
	532	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.1;
	532	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.9;
	533	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.5;
	533	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.9;
	533	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.2;
	533	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.6;
	533	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.3;
	533	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.2;
	534	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.1;
	534	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.8;
	534	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.4;
	534	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.3;
	534	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.3;
	534	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.2;
	535	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.8;
	535	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.4;
	535	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.3;
	535	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.5;
	535	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.6;
	535	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.3;
	536	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.2;
	536	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191;
	536	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.8;
	536	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.4;
	536	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.5;
	536	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.7;
	537	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549.3;
	537	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.9;
	537	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.9;
	537	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.4;
	537	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.5;
	537	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.6;
	538	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	553.9;
	538	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.6;
	538	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.9;
	538	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.8;
	538	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.8;
	538	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.1;
	539	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	554.2;
	539	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.4;
	539	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.5;
	539	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.8;
	539	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.3;
	539	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.8;
	540	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	557.6;
	540	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.9;
	540	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.2;
	540	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.9;
	540	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.1;
	540	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.6;
	541	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.8;
	541	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.1;
	541	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.3;
	541	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.5;
	541	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.6;
	541	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.2;
	542	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.3;
	542	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.6;
	542	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.3;
	542	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.4;
	542	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87;
	542	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.5;
	543	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.9;
	543	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190;
	543	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.6;
	543	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.5;
	543	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.5;
	543	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.5;
	544	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	563.2;
	544	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.1;
	544	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.9;
	544	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.8;
	544	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.4;
	544	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.7;
	545	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	569.7;
	545	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	202;
	545	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.2;
	545	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.8;
	545	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.7;
	545	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.3;
	546	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	581.9;
	546	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	206.5;
	546	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.6;
	546	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.5;
	546	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.5;
	546	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422;
	547	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586.4;
	547	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	205.7;
	547	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.5;
	547	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264;
	547	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91;
	547	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.9;
	548	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	582.1;
	548	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	203;
	548	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.6;
	548	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.1;
	548	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91;
	548	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420.6;
	549	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.3;
	549	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.8;
	549	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.7;
	549	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.8;
	549	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92;
	549	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.9;
	550	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	554.9;
	550	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.7;
	550	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.6;
	550	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.2;
	550	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.3;
	550	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.9;
	551	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.2;
	551	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.4;
	551	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.2;
	551	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.2;
	551	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.3;
	551	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.6;
	552	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.2;
	552	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.9;
	552	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.1;
	552	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.7;
	552	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.6;
	552	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.6;
	553	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.6;
	553	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.5;
	553	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.2;
	553	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.6;
	553	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.3;
	553	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.8;
	554	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497;
	554	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.8;
	554	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.4;
	554	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.4;
	554	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.5;
	554	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.8;
	555	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.1;
	555	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156;
	555	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.6;
	555	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.6;
	555	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.1;
	555	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.2;
	556	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.6;
	556	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.1;
	556	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.1;
	556	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.2;
	556	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.4;
	556	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.3;
	557	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.9;
	557	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.8;
	557	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205;
	557	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.3;
	557	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.8;
	557	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.7;
	558	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.4;
	558	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.5;
	558	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.8;
	558	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.6;
	558	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.2;
	558	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.2;
	559	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552.8;
	559	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.1;
	559	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.7;
	559	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.9;
	559	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.6;
	559	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.8;
	560	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.5;
	560	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	201.4;
	560	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.4;
	560	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.5;
	560	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.7;
	560	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416;
	561	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	599.8;
	561	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	201.2;
	561	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.7;
	561	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.1;
	561	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.7;
	561	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	424.1;
	562	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.5;
	562	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	199.5;
	562	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.6;
	562	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251;
	562	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.5;
	562	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.8;
	563	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	587.3;
	563	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198.3;
	563	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.1;
	563	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.8;
	563	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.5;
	563	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.6;
	564	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	583.9;
	564	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196;
	564	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.3;
	564	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.6;
	564	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.6;
	564	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.4;
	565	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	574.7;
	565	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.3;
	565	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.5;
	565	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.6;
	565	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.8;
	565	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.2;
	566	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.7;
	566	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.8;
	566	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.7;
	566	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.4;
	566	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.2;
	566	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.4;
	567	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570;
	567	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.2;
	567	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.1;
	567	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.6;
	567	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.6;
	567	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.2;
	568	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.5;
	568	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.3;
	568	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.1;
	568	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.4;
	568	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.8;
	568	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.3;
	569	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	575.7;
	569	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	205.6;
	569	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.8;
	569	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.9;
	569	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.2;
	569	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.9;
	570	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589.7;
	570	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	213.1;
	570	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.7;
	570	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	266.4;
	570	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.7;
	570	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.5;
	571	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	605.9;
	571	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	214.6;
	571	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.2;
	571	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.6;
	571	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.5;
	571	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	432.6;
	572	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	605.1;
	572	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	211.3;
	572	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.2;
	572	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.3;
	572	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83;
	572	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	430.9;
	573	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	596.3;
	573	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	204.5;
	573	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.3;
	573	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.2;
	573	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.5;
	573	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	424;
	574	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	585.2;
	574	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.2;
	574	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.3;
	574	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.3;
	574	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.9;
	574	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.7;
	575	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564;
	575	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.8;
	575	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.4;
	575	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.2;
	575	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.6;
	575	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.6;
	576	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.6;
	576	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.8;
	576	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.4;
	576	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.1;
	576	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.6;
	576	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.9;
	577	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.9;
	577	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.6;
	577	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.9;
	577	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.3;
	577	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81;
	577	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.7;
	578	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522.6;
	578	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.4;
	578	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.6;
	578	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206;
	578	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.9;
	578	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.9;
	579	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.4;
	579	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.5;
	579	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202;
	579	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.7;
	579	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81;
	579	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.3;
	580	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526.3;
	580	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.5;
	580	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.2;
	580	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.6;
	580	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.7;
	580	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.9;
	581	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	533.3;
	581	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.9;
	581	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.8;
	581	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.4;
	581	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.8;
	581	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.6;
	582	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564.7;
	582	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.8;
	582	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.5;
	582	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.1;
	582	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86;
	582	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.4;
	583	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	600.6;
	583	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	203.8;
	583	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.5;
	583	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.6;
	583	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89;
	583	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.2;
	584	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	623.9;
	584	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	209.4;
	584	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.1;
	584	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.4;
	584	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.3;
	584	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	432.1;
	585	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	623.1;
	585	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	210.5;
	585	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.1;
	585	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253;
	585	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.8;
	585	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	433.8;
	586	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	610.9;
	586	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	205.9;
	586	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.6;
	586	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.7;
	586	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.5;
	586	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	429.3;
	587	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	594.6;
	587	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	203.9;
	587	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.6;
	587	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.5;
	587	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.1;
	587	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.5;
	588	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589.9;
	588	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	200.8;
	588	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.2;
	588	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.9;
	588	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.9;
	588	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.8;
	589	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	583.1;
	589	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	199.1;
	589	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.5;
	589	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.6;
	589	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.8;
	589	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417;
	590	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	581.2;
	590	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.8;
	590	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.3;
	590	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.7;
	590	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80;
	590	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.7;
	591	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.8;
	591	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.8;
	591	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.7;
	591	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247;
	591	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.9;
	591	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.3;
	592	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577.5;
	592	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198;
	592	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.6;
	592	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.7;
	592	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.5;
	592	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.4;
	593	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588.3;
	593	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	207.3;
	593	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.4;
	593	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.3;
	593	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.2;
	593	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.3;
	594	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	602.9;
	594	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	216.7;
	594	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	263;
	594	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267.7;
	594	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.8;
	594	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	431.3;
	595	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	603.4;
	595	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	216.8;
	595	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.9;
	595	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267.2;
	595	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.4;
	595	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	432.1;
	596	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	604.7;
	596	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	213.9;
	596	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.6;
	596	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.2;
	596	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.4;
	596	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	431.1;
	597	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.6;
	597	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	206.6;
	597	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.8;
	597	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.9;
	597	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.8;
	597	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.9;
	598	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566.8;
	598	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.7;
	598	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.2;
	598	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.8;
	598	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.3;
	598	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.7;
	599	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.8;
	599	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183;
	599	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.5;
	599	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.2;
	599	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.7;
	599	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395;
	600	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.9;
	600	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.4;
	600	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.3;
	600	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.3;
	600	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75;
	600	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.7;
	601	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.8;
	601	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.8;
	601	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207;
	601	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.2;
	601	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	601	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.9;
	602	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.7;
	602	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.8;
	602	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203;
	602	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.3;
	602	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	602	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.5;
	603	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.8;
	603	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.6;
	603	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.4;
	603	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.6;
	603	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	603	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.8;
	604	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.6;
	604	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168;
	604	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.3;
	604	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.4;
	604	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.8;
	604	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.4;
	605	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.9;
	605	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.4;
	605	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.6;
	605	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.1;
	605	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.3;
	605	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.4;
	606	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.4;
	606	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.8;
	606	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.1;
	606	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224;
	606	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.5;
	606	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.1;
	607	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	573;
	607	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	202;
	607	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.1;
	607	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.1;
	607	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.5;
	607	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.9;
	608	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589.3;
	608	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	207.1;
	608	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.6;
	608	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.7;
	608	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.8;
	608	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.9;
	609	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589.3;
	609	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	205;
	609	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248;
	609	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.5;
	609	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.7;
	609	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.4;
	610	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579.9;
	610	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	202.2;
	610	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.4;
	610	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.4;
	610	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.9;
	610	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.7;
	611	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	581.8;
	611	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	199.2;
	611	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.4;
	611	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.6;
	611	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.1;
	611	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.3;
	612	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.3;
	612	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.4;
	612	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.9;
	612	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.4;
	612	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.3;
	612	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.5;
	613	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566;
	613	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.8;
	613	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.2;
	613	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.5;
	613	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.8;
	613	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.1;
	614	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.2;
	614	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.9;
	614	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.5;
	614	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.2;
	614	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73;
	614	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.7;
	615	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	557.4;
	615	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.5;
	615	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.3;
	615	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.6;
	615	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.6;
	615	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407;
	616	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	561.8;
	616	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198;
	616	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.5;
	616	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.4;
	616	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	616	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.4;
	617	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566.3;
	617	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	207.5;
	617	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257;
	617	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.6;
	617	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	617	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.3;
	618	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577;
	618	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	213.2;
	618	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.6;
	618	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	266.3;
	618	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72;
	618	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.4;
	619	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	583.7;
	619	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	213;
	619	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.7;
	619	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265.3;
	619	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.5;
	619	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	424.8;
	620	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.6;
	620	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	211.3;
	620	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.9;
	620	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.4;
	620	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.4;
	620	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.4;
	621	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.4;
	621	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	204.4;
	621	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250;
	621	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253;
	621	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.9;
	621	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420.4;
	622	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566.5;
	622	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195;
	622	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.3;
	622	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.2;
	622	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.6;
	622	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.8;
	623	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.8;
	623	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.3;
	623	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.2;
	623	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.9;
	623	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	623	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.6;
	624	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.7;
	624	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.1;
	624	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.8;
	624	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.1;
	624	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	624	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383;
	625	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.6;
	625	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.5;
	625	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.3;
	625	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.1;
	625	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	625	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.3;
	626	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.7;
	626	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.1;
	626	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.4;
	626	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.3;
	626	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	626	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.8;
	627	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.3;
	627	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.4;
	627	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.3;
	627	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.5;
	627	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	627	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.4;
	628	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.9;
	628	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.5;
	628	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.5;
	628	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.8;
	628	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	628	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.9;
	629	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.8;
	629	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.9;
	629	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210;
	629	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.4;
	629	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	629	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.8;
	630	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548.5;
	630	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.7;
	630	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.8;
	630	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.6;
	630	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	630	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395;
	631	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	580.5;
	631	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	201.7;
	631	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.7;
	631	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.3;
	631	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.1;
	631	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.7;
	632	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	599;
	632	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	207.8;
	632	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.6;
	632	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.2;
	632	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.8;
	632	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.8;
	633	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	605;
	633	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	205.7;
	633	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.6;
	633	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.1;
	633	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75;
	633	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	427.7;
	634	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	593.9;
	634	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	204.1;
	634	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249;
	634	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.2;
	634	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.4;
	634	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.7;
	635	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589.6;
	635	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	201.2;
	635	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.7;
	635	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252;
	635	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.9;
	635	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.7;
	636	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	590.7;
	636	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198;
	636	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.8;
	636	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.4;
	636	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.4;
	636	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.2;
	637	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.1;
	637	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.7;
	637	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.1;
	637	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.3;
	637	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.3;
	637	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.9;
	638	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	576.4;
	638	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.8;
	638	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.4;
	638	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.4;
	638	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.1;
	638	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.3;
	639	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	571.9;
	639	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.8;
	639	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.4;
	639	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.3;
	639	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.7;
	639	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.7;
	640	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.4;
	640	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.2;
	640	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.3;
	640	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.8;
	640	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	640	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.8;
	641	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	569.1;
	641	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	202.4;
	641	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.8;
	641	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.2;
	641	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.1;
	641	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415;
	642	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	583.4;
	642	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	207.5;
	642	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.1;
	642	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.3;
	642	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.4;
	642	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423;
	643	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597;
	643	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	206.2;
	643	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255;
	643	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.8;
	643	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.7;
	643	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	427.9;
	644	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	587.6;
	644	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	202.2;
	644	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.6;
	644	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253;
	644	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.3;
	644	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422;
	645	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577;
	645	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196;
	645	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.7;
	645	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.1;
	645	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.4;
	645	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.5;
	646	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.6;
	646	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.2;
	646	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.5;
	646	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.7;
	646	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.2;
	646	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401;
	647	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526.7;
	647	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.5;
	647	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.9;
	647	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.4;
	647	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.6;
	647	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.5;
	648	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.6;
	648	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174;
	648	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.3;
	648	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.9;
	648	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.3;
	648	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.5;
	649	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.9;
	649	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.4;
	649	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.9;
	649	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.9;
	649	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.7;
	649	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.8;
	650	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.1;
	650	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.7;
	650	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.9;
	650	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.2;
	650	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73;
	650	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.9;
	651	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.8;
	651	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.3;
	651	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.9;
	651	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.1;
	651	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.5;
	651	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.7;
	652	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.4;
	652	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.8;
	652	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.7;
	652	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.1;
	652	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75;
	652	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.1;
	653	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.8;
	653	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.5;
	653	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.2;
	653	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208;
	653	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.2;
	653	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.7;
	654	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.7;
	654	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.9;
	654	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.3;
	654	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.2;
	654	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.4;
	654	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.1;
	655	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.1;
	655	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.4;
	655	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.6;
	655	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.3;
	655	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.1;
	655	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.4;
	656	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523;
	656	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.6;
	656	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.7;
	656	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.1;
	656	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.8;
	656	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.2;
	657	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.7;
	657	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.2;
	657	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.3;
	657	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.3;
	657	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.6;
	657	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.8;
	658	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	533.4;
	658	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.7;
	658	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.7;
	658	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.4;
	658	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.7;
	658	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.1;
	659	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536;
	659	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191;
	659	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.4;
	659	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.6;
	659	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86;
	659	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.8;
	660	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.4;
	660	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.1;
	660	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.9;
	660	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.1;
	660	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.2;
	660	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.6;
	661	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528.9;
	661	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.3;
	661	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.6;
	661	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.8;
	661	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.3;
	661	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.4;
	662	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.3;
	662	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.4;
	662	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.7;
	662	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.9;
	662	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.1;
	662	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.6;
	663	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.1;
	663	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.6;
	663	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.1;
	663	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.9;
	663	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.1;
	663	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.7;
	664	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.7;
	664	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.4;
	664	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.9;
	664	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.1;
	664	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72;
	664	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.5;
	665	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.3;
	665	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.2;
	665	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.5;
	665	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.7;
	665	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.3;
	665	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.1;
	666	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.7;
	666	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.7;
	666	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.2;
	666	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.7;
	666	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.5;
	666	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.6;
	667	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.7;
	667	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197;
	667	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246;
	667	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.9;
	667	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77;
	667	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.7;
	668	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.7;
	668	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.6;
	668	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.6;
	668	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.7;
	668	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.4;
	668	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.5;
	669	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.5;
	669	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.6;
	669	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.2;
	669	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.6;
	669	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.4;
	669	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393;
	670	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.3;
	670	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.6;
	670	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.7;
	670	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.9;
	670	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.8;
	670	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.1;
	671	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.7;
	671	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.3;
	671	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.4;
	671	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.3;
	671	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.9;
	671	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.7;
	672	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.2;
	672	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169;
	672	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.2;
	672	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.6;
	672	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.7;
	672	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.5;
	673	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.7;
	673	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.8;
	673	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.4;
	673	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.2;
	673	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.4;
	673	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.7;
	674	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.8;
	674	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.5;
	674	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.4;
	674	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.1;
	674	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	674	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.2;
	675	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.1;
	675	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.3;
	675	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.6;
	675	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.6;
	675	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.5;
	675	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.9;
	676	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.6;
	676	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.7;
	676	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.1;
	676	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202;
	676	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.5;
	676	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.8;
	677	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.9;
	677	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.4;
	677	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.6;
	677	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.7;
	677	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.4;
	677	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.6;
	678	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.7;
	678	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169;
	678	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.7;
	678	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208;
	678	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.8;
	678	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.3;
	679	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.8;
	679	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.6;
	679	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.5;
	679	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.4;
	679	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.9;
	679	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.8;
	680	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528.8;
	680	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179;
	680	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.4;
	680	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.8;
	680	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.5;
	680	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.4;
	681	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.5;
	681	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.1;
	681	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.5;
	681	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.4;
	681	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.9;
	681	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.3;
	682	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.2;
	682	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.5;
	682	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.3;
	682	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.3;
	682	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84;
	682	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.1;
	683	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	534.4;
	683	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.1;
	683	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.1;
	683	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.3;
	683	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.8;
	683	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.7;
	684	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.6;
	684	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.2;
	684	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.8;
	684	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215;
	684	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79;
	684	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.2;
	685	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514;
	685	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183;
	685	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.7;
	685	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.9;
	685	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.9;
	685	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.5;
	686	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.1;
	686	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.2;
	686	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.6;
	686	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.9;
	686	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.9;
	686	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.9;
	687	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504;
	687	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.7;
	687	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.1;
	687	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.6;
	687	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78;
	687	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.3;
	688	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515;
	688	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.1;
	688	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218;
	688	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.5;
	688	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.2;
	688	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.9;
	689	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528.5;
	689	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.2;
	689	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.6;
	689	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.4;
	689	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.5;
	689	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.3;
	690	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.5;
	690	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.5;
	690	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.8;
	690	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.8;
	690	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.4;
	690	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.4;
	691	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	555.7;
	691	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	200.5;
	691	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.8;
	691	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.6;
	691	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.9;
	691	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.9;
	692	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.7;
	692	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198;
	692	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242;
	692	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.8;
	692	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.4;
	692	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.7;
	693	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537.6;
	693	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.8;
	693	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.9;
	693	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.7;
	693	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.3;
	693	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.7;
	694	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.4;
	694	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.8;
	694	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.8;
	694	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.6;
	694	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.6;
	694	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.4;
	695	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.3;
	695	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.9;
	695	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.1;
	695	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.3;
	695	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.1;
	695	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377;
	696	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.8;
	696	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.4;
	696	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.1;
	696	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214;
	696	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.1;
	696	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.5;
	697	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.6;
	697	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.9;
	697	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.5;
	697	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.8;
	697	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.5;
	697	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.2;
	698	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477;
	698	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156;
	698	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.6;
	698	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.5;
	698	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75;
	698	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.1;
	699	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.3;
	699	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.6;
	699	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.1;
	699	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.7;
	699	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.9;
	699	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.9;
	700	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484;
	700	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.9;
	700	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.5;
	700	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.4;
	700	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.9;
	700	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.1;
	701	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.3;
	701	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.2;
	701	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.3;
	701	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.3;
	701	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.6;
	701	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.4;
	702	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.5;
	702	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.7;
	702	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212;
	702	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.7;
	702	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.9;
	702	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.1;
	703	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547.7;
	703	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.6;
	703	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.5;
	703	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.3;
	703	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.4;
	703	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.2;
	704	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	575.7;
	704	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	201.9;
	704	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.6;
	704	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.6;
	704	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.1;
	704	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.2;
	705	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589.5;
	705	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198.4;
	705	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.5;
	705	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.9;
	705	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.1;
	705	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420.3;
	706	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.3;
	706	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.7;
	706	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245;
	706	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.3;
	706	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.7;
	706	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	424.4;
	707	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	590;
	707	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198.2;
	707	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.9;
	707	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.3;
	707	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.6;
	707	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421;
	708	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	578.6;
	708	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.7;
	708	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.3;
	708	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.4;
	708	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.1;
	708	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.2;
	709	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	574.8;
	709	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.3;
	709	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.5;
	709	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.5;
	709	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.7;
	709	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.6;
	710	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572;
	710	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.4;
	710	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.1;
	710	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.8;
	710	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.4;
	710	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.3;
	711	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.4;
	711	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.1;
	711	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.2;
	711	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.7;
	711	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.3;
	711	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.6;
	712	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	571.9;
	712	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.9;
	712	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.1;
	712	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.8;
	712	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.5;
	712	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.9;
	713	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	578.3;
	713	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	201.1;
	713	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.7;
	713	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.4;
	713	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.8;
	713	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.5;
	714	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.6;
	714	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	208.6;
	714	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	259.2;
	714	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.2;
	714	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	714	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.4;
	715	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	591.5;
	715	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	208.8;
	715	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.8;
	715	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	263.5;
	715	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.9;
	715	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	426.8;
	716	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.2;
	716	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	206.8;
	716	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.8;
	716	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.5;
	716	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.7;
	716	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	428.1;
	717	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	581;
	717	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198.7;
	717	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.9;
	717	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.2;
	717	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.5;
	717	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.1;
	718	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564.8;
	718	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.7;
	718	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.1;
	718	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.1;
	718	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.1;
	718	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.5;
	719	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544;
	719	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.3;
	719	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.7;
	719	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.3;
	719	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.3;
	719	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.1;
	720	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.9;
	720	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.5;
	720	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.2;
	720	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.5;
	720	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	720	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378;
	721	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.2;
	721	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160;
	721	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.2;
	721	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.1;
	721	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.6;
	721	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.3;
	722	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.1;
	722	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.5;
	722	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.2;
	722	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.1;
	722	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	722	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.2;
	723	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.2;
	723	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.5;
	723	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.9;
	723	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204;
	723	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.1;
	723	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.5;
	724	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.1;
	724	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.4;
	724	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.8;
	724	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.8;
	724	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	724	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.3;
	725	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.5;
	725	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.4;
	725	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.2;
	725	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.7;
	725	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	725	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.2;
	726	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.5;
	726	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179;
	726	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.4;
	726	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.6;
	726	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.5;
	726	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.9;
	727	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549.8;
	727	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.3;
	727	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.1;
	727	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.5;
	727	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.5;
	727	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.1;
	728	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579.6;
	728	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.6;
	728	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.3;
	728	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244;
	728	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70;
	728	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413;
	729	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	587.8;
	729	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198.6;
	729	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.5;
	729	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.7;
	729	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.4;
	729	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.2;
	730	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	573;
	730	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.4;
	730	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.9;
	730	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.8;
	730	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.8;
	730	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.3;
	731	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559.7;
	731	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.9;
	731	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.1;
	731	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.4;
	731	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	731	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406;
	732	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559.7;
	732	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.4;
	732	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.7;
	732	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.9;
	732	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.4;
	732	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.7;
	733	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547.9;
	733	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.9;
	733	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.2;
	733	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.2;
	733	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	733	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401;
	734	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.5;
	734	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.6;
	734	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236;
	734	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.3;
	734	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	734	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.8;
	735	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.2;
	735	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.7;
	735	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236;
	735	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.8;
	735	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	735	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.1;
	736	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.3;
	736	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.9;
	736	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.9;
	736	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.6;
	736	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	736	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.3;
	737	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	553.6;
	737	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.9;
	737	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.3;
	737	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.7;
	737	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	737	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.1;
	738	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.9;
	738	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	202.5;
	738	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.4;
	738	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.9;
	738	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	738	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.1;
	739	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579.7;
	739	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	202.2;
	739	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.7;
	739	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.1;
	739	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	739	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420.6;
	740	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579;
	740	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	199.4;
	740	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250;
	740	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.2;
	740	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	740	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.8;
	741	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.6;
	741	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.2;
	741	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.8;
	741	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.2;
	741	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	741	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.4;
	742	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.2;
	742	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182;
	742	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.8;
	742	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.8;
	742	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	742	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397;
	743	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.3;
	743	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.5;
	743	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217;
	743	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221;
	743	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	743	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.5;
	744	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498;
	744	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.7;
	744	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.1;
	744	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215;
	744	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	744	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.5;
	745	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487;
	745	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.7;
	745	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.4;
	745	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.9;
	745	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60;
	745	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.6;
	746	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.8;
	746	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.6;
	746	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.4;
	746	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.6;
	746	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	746	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.5;
	747	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.5;
	747	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.7;
	747	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.1;
	747	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.6;
	747	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.6;
	747	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.2;
	748	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498;
	748	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.6;
	748	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.2;
	748	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.6;
	748	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	748	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.8;
	749	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518.6;
	749	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.2;
	749	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.3;
	749	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.9;
	749	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	749	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.9;
	750	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.9;
	750	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.1;
	750	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.9;
	750	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.9;
	750	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	750	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.7;
	751	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	583.4;
	751	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.6;
	751	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.2;
	751	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.1;
	751	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.2;
	751	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.5;
	752	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	601.9;
	752	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.1;
	752	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.7;
	752	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.4;
	752	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	752	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.6;
	753	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	605.6;
	753	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.7;
	753	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.5;
	753	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.5;
	753	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	753	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	424.8;
	754	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	593.4;
	754	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.4;
	754	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.8;
	754	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.7;
	754	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	754	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420.5;
	755	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.3;
	755	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.1;
	755	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.8;
	755	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247;
	755	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	755	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417;
	756	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.6;
	756	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184;
	756	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.5;
	756	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.7;
	756	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	756	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.6;
	757	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577;
	757	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.7;
	757	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.3;
	757	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245;
	757	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	757	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.1;
	758	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566.4;
	758	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.8;
	758	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.3;
	758	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.5;
	758	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	758	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.5;
	759	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.5;
	759	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.2;
	759	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237;
	759	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.1;
	759	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	759	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.6;
	760	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.7;
	760	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181;
	760	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.8;
	760	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244;
	760	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	760	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.4;
	761	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570;
	761	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.2;
	761	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.1;
	761	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.8;
	761	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	761	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.5;
	762	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	580;
	762	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.2;
	762	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.2;
	762	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260;
	762	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	762	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.6;
	763	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.9;
	763	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.1;
	763	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.7;
	763	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.8;
	763	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.3;
	763	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	427.3;
	764	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598;
	764	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196;
	764	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.7;
	764	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.1;
	764	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.2;
	764	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.7;
	765	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	582.4;
	765	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.6;
	765	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.4;
	765	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.3;
	765	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.2;
	765	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.2;
	766	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	557.1;
	766	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.1;
	766	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.2;
	766	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.7;
	766	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	766	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.2;
	767	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.9;
	767	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.5;
	767	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.2;
	767	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.7;
	767	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	767	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.3;
	768	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	524.2;
	768	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.5;
	768	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.7;
	768	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.7;
	768	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	768	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.4;
	769	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.8;
	769	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.4;
	769	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.8;
	769	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.4;
	769	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	769	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.5;
	770	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.7;
	770	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.2;
	770	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.8;
	770	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.6;
	770	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.3;
	770	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.7;
	771	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.3;
	771	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.2;
	771	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.6;
	771	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.3;
	771	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.8;
	771	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.7;
	772	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.3;
	772	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.4;
	772	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.4;
	772	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.5;
	772	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59;
	772	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.3;
	773	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.4;
	773	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.7;
	773	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.4;
	773	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.6;
	773	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.1;
	773	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.2;
	774	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	587.9;
	774	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.5;
	774	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.4;
	774	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221;
	774	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	774	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.2;
	775	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.3;
	775	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.5;
	775	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.5;
	775	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.9;
	775	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	775	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.5;
	776	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	600.1;
	776	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	199.1;
	776	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.3;
	776	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246;
	776	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	776	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.5;
	777	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	604.9;
	777	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.1;
	777	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.2;
	777	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.4;
	777	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74;
	777	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.5;
	778	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	587.4;
	778	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.5;
	778	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242;
	778	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.1;
	778	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.4;
	778	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.6;
	779	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	595.9;
	779	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.7;
	779	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.4;
	779	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.3;
	779	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.6;
	779	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.8;
	780	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	578.1;
	780	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.2;
	780	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.9;
	780	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.8;
	780	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.9;
	780	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.9;
	781	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.3;
	781	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.6;
	781	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.6;
	781	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.8;
	781	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.3;
	781	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.2;
	782	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.1;
	782	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.6;
	782	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.5;
	782	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.5;
	782	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.8;
	782	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402;
	783	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.2;
	783	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.8;
	783	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.5;
	783	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.1;
	783	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	783	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.2;
	784	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	557.8;
	784	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.2;
	784	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.3;
	784	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.1;
	784	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	784	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.8;
	785	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	563.1;
	785	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.2;
	785	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.4;
	785	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.5;
	785	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.2;
	785	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.6;
	786	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577.3;
	786	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	204.4;
	786	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.3;
	786	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261;
	786	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	786	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.2;
	787	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	582.3;
	787	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	205;
	787	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.8;
	787	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.7;
	787	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.4;
	787	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.8;
	788	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.5;
	788	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	203.4;
	788	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.2;
	788	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.8;
	788	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	788	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	426.4;
	789	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	575.2;
	789	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.4;
	789	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.6;
	789	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.1;
	789	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.8;
	789	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414;
	790	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559.2;
	790	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.4;
	790	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.7;
	790	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.2;
	790	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.6;
	790	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.5;
	791	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.6;
	791	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.8;
	791	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.3;
	791	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.3;
	791	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.4;
	791	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.8;
	792	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483;
	792	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.3;
	792	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.5;
	792	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.4;
	792	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	792	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.5;
	793	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.5;
	793	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.2;
	793	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202;
	793	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.4;
	793	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	793	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.1;
	794	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.9;
	794	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.2;
	794	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.3;
	794	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203;
	794	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	794	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.4;
	795	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497;
	795	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.9;
	795	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.7;
	795	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.3;
	795	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	795	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.3;
	796	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.4;
	796	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158;
	796	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.5;
	796	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.1;
	796	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	796	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.9;
	797	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518.6;
	797	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.5;
	797	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.4;
	797	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.5;
	797	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	797	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378;
	798	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.5;
	798	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.6;
	798	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.7;
	798	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.3;
	798	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	798	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.3;
	799	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.5;
	799	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.4;
	799	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.7;
	799	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.8;
	799	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	799	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.7;
	800	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	596.6;
	800	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.3;
	800	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.2;
	800	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.2;
	800	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.3;
	800	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420.3;
	801	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.8;
	801	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.6;
	801	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.3;
	801	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.7;
	801	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.5;
	801	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420.1;
	802	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	594.7;
	802	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.9;
	802	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.5;
	802	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.9;
	802	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	802	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.4;
	803	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	594.6;
	803	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190;
	803	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.1;
	803	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.7;
	803	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.3;
	803	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421;
	804	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	585;
	804	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.5;
	804	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.2;
	804	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.3;
	804	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.8;
	804	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.3;
	805	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	581.4;
	805	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.9;
	805	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.1;
	805	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.2;
	805	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.4;
	805	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.4;
	806	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.4;
	806	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.3;
	806	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.9;
	806	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.5;
	806	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68;
	806	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406;
	807	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565;
	807	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.6;
	807	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.7;
	807	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.2;
	807	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	807	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.2;
	808	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	568.6;
	808	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.3;
	808	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.6;
	808	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.7;
	808	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	808	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.2;
	809	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.6;
	809	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191;
	809	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.3;
	809	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.8;
	809	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.8;
	809	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.9;
	810	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	573.8;
	810	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.6;
	810	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.7;
	810	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256;
	810	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.2;
	810	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.7;
	811	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588.9;
	811	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198.3;
	811	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.5;
	811	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255;
	811	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	811	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.5;
	812	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577.1;
	812	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.9;
	812	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.2;
	812	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.4;
	812	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.8;
	812	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416;
	813	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566.4;
	813	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.4;
	813	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.9;
	813	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.2;
	813	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.6;
	813	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.9;
	814	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.9;
	814	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.2;
	814	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230;
	814	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234;
	814	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	814	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.1;
	815	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526;
	815	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.6;
	815	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.7;
	815	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.4;
	815	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	815	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386;
	816	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.5;
	816	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.3;
	816	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.2;
	816	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.9;
	816	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	816	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.2;
	817	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.5;
	817	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.2;
	817	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205;
	817	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.1;
	817	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.5;
	817	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.4;
	818	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.1;
	818	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.8;
	818	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.1;
	818	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203;
	818	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.4;
	818	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.7;
	819	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.9;
	819	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.2;
	819	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199;
	819	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.1;
	819	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.2;
	819	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.2;
	820	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.1;
	820	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.2;
	820	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.5;
	820	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.5;
	820	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.9;
	820	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.3;
	821	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519;
	821	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.5;
	821	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.7;
	821	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207;
	821	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.7;
	821	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378;
	822	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.6;
	822	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.7;
	822	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.5;
	822	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.1;
	822	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56;
	822	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.7;
	823	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527;
	823	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.1;
	823	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.4;
	823	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.9;
	823	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58;
	823	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.3;
	824	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.7;
	824	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.6;
	824	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.4;
	824	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.6;
	824	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	824	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.4;
	825	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	540.1;
	825	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.6;
	825	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.2;
	825	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.8;
	825	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	825	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.9;
	826	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.9;
	826	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.4;
	826	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.5;
	826	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.1;
	826	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	826	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.2;
	827	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.8;
	827	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.7;
	827	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.9;
	827	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.3;
	827	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.1;
	827	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.9;
	828	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.7;
	828	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.4;
	828	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.2;
	828	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.5;
	828	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	828	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.5;
	829	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.7;
	829	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.5;
	829	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.1;
	829	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.5;
	829	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	829	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.5;
	830	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.1;
	830	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.3;
	830	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.4;
	830	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.4;
	830	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.3;
	830	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.3;
	831	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.9;
	831	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.8;
	831	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.4;
	831	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220;
	831	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69;
	831	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.3;
	832	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.3;
	832	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.3;
	832	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.5;
	832	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.8;
	832	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	832	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372;
	833	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.5;
	833	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.6;
	833	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.5;
	833	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.1;
	833	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.5;
	833	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.6;
	834	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.8;
	834	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.7;
	834	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.5;
	834	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.9;
	834	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	834	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.2;
	835	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.8;
	835	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.4;
	835	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.7;
	835	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.5;
	835	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	835	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392;
	836	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.3;
	836	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.1;
	836	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.1;
	836	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.1;
	836	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.4;
	836	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.2;
	837	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518.9;
	837	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.7;
	837	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232;
	837	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.5;
	837	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.1;
	837	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.2;
	838	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488;
	838	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.8;
	838	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.4;
	838	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.5;
	838	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	838	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.9;
	839	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448;
	839	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.7;
	839	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216;
	839	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.1;
	839	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.8;
	839	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.7;
	840	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.2;
	840	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.5;
	840	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.5;
	840	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.1;
	840	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	840	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.6;
	841	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.5;
	841	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.1;
	841	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.4;
	841	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.4;
	841	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	841	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.4;
	842	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.1;
	842	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.8;
	842	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.6;
	842	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.7;
	842	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	842	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337;
	843	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417.5;
	843	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.1;
	843	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.7;
	843	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.8;
	843	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	843	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.7;
	844	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	426.1;
	844	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.4;
	844	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.5;
	844	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.7;
	844	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	844	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.3;
	845	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.2;
	845	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.7;
	845	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.6;
	845	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201;
	845	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	845	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.3;
	846	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.2;
	846	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159;
	846	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.7;
	846	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.2;
	846	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68;
	846	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.3;
	847	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.9;
	847	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.4;
	847	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.6;
	847	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.9;
	847	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	847	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.2;
	848	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.2;
	848	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167;
	848	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.3;
	848	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.3;
	848	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.7;
	848	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355;
	849	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.1;
	849	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.8;
	849	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207;
	849	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.7;
	849	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.8;
	849	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358;
	850	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.2;
	850	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.6;
	850	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.1;
	850	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.5;
	850	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.6;
	850	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.5;
	851	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.3;
	851	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.5;
	851	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.4;
	851	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.2;
	851	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.5;
	851	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.2;
	852	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.8;
	852	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.4;
	852	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208;
	852	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.5;
	852	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78;
	852	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.5;
	853	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.9;
	853	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.3;
	853	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210;
	853	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.1;
	853	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.4;
	853	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.8;
	854	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.6;
	854	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174;
	854	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.7;
	854	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.9;
	854	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.5;
	854	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362;
	855	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.1;
	855	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.7;
	855	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.9;
	855	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213;
	855	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.4;
	855	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.5;
	856	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.7;
	856	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.5;
	856	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.3;
	856	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.6;
	856	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.9;
	856	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.4;
	857	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.2;
	857	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180;
	857	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.2;
	857	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.1;
	857	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.9;
	857	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.5;
	858	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.1;
	858	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.1;
	858	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.7;
	858	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.6;
	858	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.8;
	858	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.2;
	859	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.7;
	859	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.4;
	859	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.7;
	859	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.8;
	859	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.5;
	859	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.5;
	860	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.6;
	860	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.8;
	860	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.2;
	860	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.9;
	860	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.7;
	860	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.7;
	861	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492;
	861	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.4;
	861	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.3;
	861	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234;
	861	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.2;
	861	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.7;
	862	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.7;
	862	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.5;
	862	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.3;
	862	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.7;
	862	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.4;
	862	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.8;
	863	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.8;
	863	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.7;
	863	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.5;
	863	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.7;
	863	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.1;
	863	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.1;
	864	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.7;
	864	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.5;
	864	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.3;
	864	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.2;
	864	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.1;
	864	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.1;
	865	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.5;
	865	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.4;
	865	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.4;
	865	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.7;
	865	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	865	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.1;
	866	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.6;
	866	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.4;
	866	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.7;
	866	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.2;
	866	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71;
	866	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.9;
	867	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.4;
	867	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.8;
	867	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.8;
	867	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.1;
	867	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	867	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.8;
	868	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447;
	868	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.4;
	868	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.3;
	868	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.7;
	868	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.5;
	868	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.2;
	869	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.1;
	869	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.7;
	869	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.3;
	869	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.9;
	869	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.7;
	869	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.8;
	870	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.8;
	870	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.7;
	870	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.8;
	870	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212;
	870	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.7;
	870	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361;
	871	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.5;
	871	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.6;
	871	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.2;
	871	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.4;
	871	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.7;
	871	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.6;
	872	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543;
	872	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189;
	872	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.6;
	872	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240;
	872	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.3;
	872	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.6;
	873	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.2;
	873	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186;
	873	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.2;
	873	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.2;
	873	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.8;
	873	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.4;
	874	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	553.4;
	874	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.4;
	874	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.4;
	874	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.1;
	874	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.7;
	874	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.7;
	875	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559;
	875	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.7;
	875	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.4;
	875	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.7;
	875	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.4;
	875	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.4;
	876	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.4;
	876	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.1;
	876	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.3;
	876	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.5;
	876	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.6;
	876	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.1;
	877	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543;
	877	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181;
	877	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.9;
	877	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.8;
	877	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.7;
	877	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.2;
	878	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537;
	878	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.3;
	878	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236;
	878	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.1;
	878	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.6;
	878	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.9;
	879	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	534.4;
	879	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.5;
	879	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.8;
	879	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.3;
	879	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68;
	879	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.9;
	880	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.3;
	880	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.3;
	880	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.9;
	880	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.1;
	880	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.8;
	880	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.6;
	881	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.2;
	881	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.9;
	881	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.5;
	881	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.8;
	881	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	881	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.9;
	882	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.2;
	882	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.2;
	882	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.5;
	882	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.4;
	882	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.4;
	882	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.4;
	883	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.9;
	883	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	200.6;
	883	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.4;
	883	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.2;
	883	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.3;
	883	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.1;
	884	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.8;
	884	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	199.3;
	884	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250;
	884	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254;
	884	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73;
	884	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.7;
	885	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.5;
	885	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.8;
	885	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.5;
	885	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.3;
	885	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.3;
	885	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.4;
	886	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.7;
	886	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.6;
	886	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.1;
	886	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.3;
	886	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.8;
	886	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.8;
	887	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520;
	887	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170;
	887	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.9;
	887	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.8;
	887	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74;
	887	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.7;
	888	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.5;
	888	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.3;
	888	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.4;
	888	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.8;
	888	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.4;
	888	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.2;
	889	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.3;
	889	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.8;
	889	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.7;
	889	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204;
	889	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.2;
	889	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.3;
	890	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.1;
	890	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.4;
	890	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.2;
	890	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.2;
	890	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	890	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.1;
	891	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.3;
	891	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.8;
	891	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.5;
	891	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.4;
	891	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.9;
	891	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.6;
	892	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485;
	892	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.8;
	892	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.7;
	892	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.6;
	892	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.5;
	892	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.5;
	893	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.2;
	893	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.7;
	893	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.1;
	893	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.1;
	893	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.5;
	893	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371;
	894	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.2;
	894	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.9;
	894	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.1;
	894	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219;
	894	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.6;
	894	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.8;
	895	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.4;
	895	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.5;
	895	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.1;
	895	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.5;
	895	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.4;
	895	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.5;
	896	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	594.6;
	896	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198.5;
	896	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.1;
	896	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.4;
	896	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.3;
	896	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.6;
	897	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	595.8;
	897	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.4;
	897	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.7;
	897	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.4;
	897	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.8;
	897	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420.6;
	898	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.8;
	898	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.8;
	898	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.5;
	898	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.8;
	898	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.5;
	898	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.7;
	899	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	569.5;
	899	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.5;
	899	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.1;
	899	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.3;
	899	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.1;
	899	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.6;
	900	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	553.9;
	900	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.4;
	900	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.9;
	900	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.3;
	900	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.5;
	900	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.2;
	901	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.9;
	901	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.4;
	901	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.9;
	901	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.7;
	901	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.8;
	901	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.6;
	902	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538;
	902	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.2;
	902	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.5;
	902	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.2;
	902	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.4;
	902	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.3;
	903	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535;
	903	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.3;
	903	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.6;
	903	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.4;
	903	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	903	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.5;
	904	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.6;
	904	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.8;
	904	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.1;
	904	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.5;
	904	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.5;
	904	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.5;
	905	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	546;
	905	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190;
	905	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.5;
	905	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.1;
	905	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.9;
	905	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.5;
	906	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549.5;
	906	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.8;
	906	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.8;
	906	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.2;
	906	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.2;
	906	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.9;
	907	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.3;
	907	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	199.1;
	907	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.9;
	907	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.8;
	907	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.6;
	907	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.4;
	908	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.5;
	908	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198.1;
	908	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.9;
	908	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.5;
	908	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.4;
	908	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.1;
	909	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.9;
	909	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.2;
	909	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.4;
	909	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.5;
	909	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.9;
	909	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402;
	910	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.3;
	910	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.7;
	910	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.6;
	910	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.2;
	910	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.1;
	910	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.1;
	911	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500;
	911	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.7;
	911	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218;
	911	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.5;
	911	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.5;
	911	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.6;
	912	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.4;
	912	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.5;
	912	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.3;
	912	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.6;
	912	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	912	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.6;
	913	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.6;
	913	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.5;
	913	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.8;
	913	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.7;
	913	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	913	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.7;
	914	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.9;
	914	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.2;
	914	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.6;
	914	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.5;
	914	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	914	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.1;
	915	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.3;
	915	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.8;
	915	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.5;
	915	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.6;
	915	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	915	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.3;
	916	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.2;
	916	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.3;
	916	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.2;
	916	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.2;
	916	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	916	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.2;
	917	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.2;
	917	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.4;
	917	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.8;
	917	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.2;
	917	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	917	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.2;
	918	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.9;
	918	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.9;
	918	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.9;
	918	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.1;
	918	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.1;
	918	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372;
	919	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528.6;
	919	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.2;
	919	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.6;
	919	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.3;
	919	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.8;
	919	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.4;
	920	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	546.6;
	920	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.2;
	920	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.2;
	920	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241;
	920	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.9;
	920	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.5;
	921	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548;
	921	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.5;
	921	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.2;
	921	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.5;
	921	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.4;
	921	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.2;
	922	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.7;
	922	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.8;
	922	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.3;
	922	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.1;
	922	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.2;
	922	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.6;
	923	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525;
	923	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.1;
	923	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.2;
	923	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.2;
	923	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.8;
	923	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.1;
	924	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526.1;
	924	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.8;
	924	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241;
	924	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.2;
	924	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.8;
	924	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.2;
	925	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.3;
	925	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.4;
	925	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.7;
	925	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.2;
	925	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.1;
	925	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.8;
	926	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.4;
	926	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.5;
	926	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.6;
	926	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.2;
	926	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.5;
	926	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.1;
	927	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.8;
	927	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.5;
	927	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.6;
	927	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.4;
	927	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.6;
	927	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.5;
	928	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.9;
	928	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.6;
	928	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.6;
	928	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.8;
	928	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.6;
	928	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.5;
	929	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.9;
	929	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.5;
	929	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247;
	929	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.2;
	929	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.7;
	929	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.2;
	930	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.8;
	930	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.7;
	930	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.3;
	930	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.5;
	930	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.8;
	930	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.2;
	931	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.6;
	931	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198;
	931	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.5;
	931	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.9;
	931	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.9;
	931	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.7;
	932	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.9;
	932	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.5;
	932	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.6;
	932	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.8;
	932	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.7;
	932	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.3;
	933	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.1;
	933	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.8;
	933	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.4;
	933	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.4;
	933	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.1;
	933	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.9;
	934	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.7;
	934	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.9;
	934	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.5;
	934	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.1;
	934	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.3;
	934	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.5;
	935	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.1;
	935	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.8;
	935	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.5;
	935	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.7;
	935	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.8;
	935	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.2;
	936	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.3;
	936	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.9;
	936	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.7;
	936	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.8;
	936	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	936	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.3;
	937	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.5;
	937	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151;
	937	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.2;
	937	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.8;
	937	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.3;
	937	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.4;
	938	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450;
	938	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.7;
	938	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.7;
	938	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.2;
	938	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	938	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.1;
	939	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.2;
	939	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.8;
	939	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.9;
	939	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.6;
	939	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	939	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.3;
	940	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464;
	940	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.3;
	940	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.6;
	940	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.2;
	940	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	940	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.3;
	941	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.6;
	941	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.6;
	941	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.5;
	941	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.3;
	941	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	941	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.7;
	942	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.7;
	942	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.5;
	942	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214;
	942	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.7;
	942	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68;
	942	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.9;
	943	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	546.8;
	943	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.6;
	943	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228;
	943	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.1;
	943	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	943	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.6;
	944	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	567.1;
	944	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.6;
	944	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.2;
	944	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.8;
	944	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.1;
	944	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.2;
	945	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	569.4;
	945	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.4;
	945	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.8;
	945	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.2;
	945	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.3;
	945	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.7;
	946	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	557.6;
	946	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.7;
	946	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.4;
	946	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.8;
	946	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.5;
	946	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.1;
	947	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.9;
	947	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.1;
	947	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.1;
	947	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.4;
	947	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79;
	947	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.2;
	948	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.6;
	948	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.6;
	948	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.5;
	948	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.4;
	948	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.8;
	948	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.6;
	949	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.1;
	949	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.8;
	949	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.1;
	949	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.4;
	949	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.1;
	949	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.3;
	950	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.2;
	950	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178;
	950	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.1;
	950	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.3;
	950	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.6;
	950	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.5;
	951	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.1;
	951	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.3;
	951	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.8;
	951	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.1;
	951	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.7;
	951	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.5;
	952	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528.5;
	952	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.3;
	952	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.8;
	952	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.1;
	952	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.5;
	952	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.4;
	953	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.7;
	953	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.7;
	953	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.1;
	953	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.7;
	953	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.7;
	953	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.8;
	954	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552.7;
	954	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.4;
	954	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.8;
	954	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.6;
	954	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.1;
	954	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.9;
	955	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	571.7;
	955	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	201;
	955	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.8;
	955	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.1;
	955	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.9;
	955	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.8;
	956	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.2;
	956	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	199.7;
	956	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.6;
	956	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.9;
	956	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.4;
	956	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.9;
	957	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.8;
	957	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.9;
	957	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244;
	957	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.9;
	957	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.3;
	957	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.8;
	958	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.4;
	958	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.6;
	958	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233;
	958	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.5;
	958	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.8;
	958	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.1;
	959	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.5;
	959	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.6;
	959	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.2;
	959	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.4;
	959	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.9;
	959	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.8;
	960	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502;
	960	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.4;
	960	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.5;
	960	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.7;
	960	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.8;
	960	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.6;
	961	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490;
	961	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.4;
	961	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.9;
	961	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.4;
	961	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.7;
	961	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.3;
	962	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.3;
	962	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.4;
	962	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.8;
	962	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204;
	962	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.7;
	962	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.1;
	963	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.5;
	963	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.4;
	963	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.5;
	963	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.5;
	963	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.5;
	963	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.4;
	964	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.5;
	964	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.7;
	964	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.1;
	964	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.1;
	964	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	964	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.5;
	965	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.3;
	965	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.4;
	965	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.6;
	965	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211;
	965	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.9;
	965	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.8;
	966	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.9;
	966	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.9;
	966	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.8;
	966	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.8;
	966	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74;
	966	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.9;
	967	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.5;
	967	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.2;
	967	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.6;
	967	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.9;
	967	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.2;
	967	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.8;
	968	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.6;
	968	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.8;
	968	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.6;
	968	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.3;
	968	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.5;
	968	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.5;
	969	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	587.8;
	969	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.7;
	969	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.4;
	969	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.3;
	969	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85;
	969	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.9;
	970	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	578;
	970	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194;
	970	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.7;
	970	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.1;
	970	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.9;
	970	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.2;
	971	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564.3;
	971	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.1;
	971	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.2;
	971	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.9;
	971	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.2;
	971	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.1;
	972	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.8;
	972	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.5;
	972	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.3;
	972	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.5;
	972	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.8;
	972	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.4;
	973	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.8;
	973	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.2;
	973	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.4;
	973	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.9;
	973	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.5;
	973	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.8;
	974	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	553.4;
	974	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183;
	974	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.5;
	974	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.7;
	974	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.8;
	974	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.4;
	975	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	554.9;
	975	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.8;
	975	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.9;
	975	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.4;
	975	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.4;
	975	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.9;
	976	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549.3;
	976	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.4;
	976	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.2;
	976	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.5;
	976	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.8;
	976	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.6;
	977	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.2;
	977	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.4;
	977	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.8;
	977	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.5;
	977	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.5;
	977	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.8;
	978	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566.3;
	978	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.4;
	978	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.5;
	978	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.4;
	978	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.9;
	978	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.3;
	979	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	568.5;
	979	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.8;
	979	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.6;
	979	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.5;
	979	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80;
	979	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.8;
	980	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	568.4;
	980	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.5;
	980	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.5;
	980	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.7;
	980	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.8;
	980	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.4;
	981	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.1;
	981	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.4;
	981	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.2;
	981	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.5;
	981	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.7;
	981	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.1;
	982	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.5;
	982	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.3;
	982	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229;
	982	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.8;
	982	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.1;
	982	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.5;
	983	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525.7;
	983	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.9;
	983	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.6;
	983	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.5;
	983	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.4;
	983	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.5;
	984	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.9;
	984	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.6;
	984	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216;
	984	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.4;
	984	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.2;
	984	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.5;
	985	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.3;
	985	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.2;
	985	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.9;
	985	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208;
	985	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.4;
	985	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.9;
	986	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480;
	986	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.7;
	986	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.9;
	986	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.8;
	986	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77;
	986	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.7;
	987	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.7;
	987	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.4;
	987	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198;
	987	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.2;
	987	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.9;
	987	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.9;
	988	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.2;
	988	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.7;
	988	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.2;
	988	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.4;
	988	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.4;
	988	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.2;
	989	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.6;
	989	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.7;
	989	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.8;
	989	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.2;
	989	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.9;
	989	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.7;
	990	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	533.5;
	990	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.9;
	990	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208;
	990	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213;
	990	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.4;
	990	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.1;
	991	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537;
	991	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.6;
	991	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.2;
	991	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216;
	991	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.1;
	991	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.1;
	992	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.5;
	992	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.2;
	992	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.1;
	992	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.5;
	992	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.2;
	992	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.8;
	993	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.1;
	993	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.8;
	993	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.7;
	993	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.9;
	993	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.5;
	993	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.6;
	994	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.7;
	994	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.8;
	994	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.2;
	994	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.7;
	994	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.1;
	994	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390;
	995	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.2;
	995	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.2;
	995	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.1;
	995	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.1;
	995	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.2;
	995	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.1;
	996	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526.5;
	996	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.3;
	996	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.3;
	996	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.4;
	996	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.2;
	996	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.1;
	997	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518.1;
	997	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.6;
	997	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.5;
	997	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.6;
	997	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91;
	997	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.7;
	998	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.9;
	998	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.9;
	998	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.4;
	998	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.3;
	998	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.5;
	998	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.5;
	999	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.1;
	999	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.7;
	999	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216;
	999	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.8;
	999	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.7;
	999	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.9;
	1000	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.7;
	1000	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.4;
	1000	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.2;
	1000	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.8;
	1000	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.1;
	1000	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379;
	1001	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.2;
	1001	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.9;
	1001	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.1;
	1001	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.2;
	1001	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.3;
	1001	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376;
	1002	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.9;
	1002	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.6;
	1002	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.4;
	1002	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.8;
	1002	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.2;
	1002	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.5;
	1003	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528.6;
	1003	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.6;
	1003	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.3;
	1003	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.3;
	1003	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.1;
	1003	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.5;
	1004	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.1;
	1004	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187;
	1004	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.5;
	1004	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.6;
	1004	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.4;
	1004	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.3;
	1005	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525.4;
	1005	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.8;
	1005	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.5;
	1005	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.2;
	1005	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.1;
	1005	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.3;
	1006	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508;
	1006	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.6;
	1006	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.9;
	1006	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.4;
	1006	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.2;
	1006	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.9;
	1007	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.5;
	1007	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.4;
	1007	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.4;
	1007	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.9;
	1007	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91;
	1007	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.7;
	1008	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.8;
	1008	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.3;
	1008	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.7;
	1008	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.1;
	1008	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.6;
	1008	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.2;
	1009	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.2;
	1009	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.6;
	1009	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.9;
	1009	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.5;
	1009	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.3;
	1009	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.1;
	1010	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.8;
	1010	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.2;
	1010	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.2;
	1010	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.6;
	1010	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.1;
	1010	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.6;
	1011	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.1;
	1011	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.9;
	1011	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.7;
	1011	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.3;
	1011	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.2;
	1011	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.4;
	1012	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.9;
	1012	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.6;
	1012	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.7;
	1012	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.4;
	1012	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.3;
	1012	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.4;
	1013	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.3;
	1013	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.7;
	1013	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.7;
	1013	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.5;
	1013	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.4;
	1013	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.7;
	1014	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.9;
	1014	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.1;
	1014	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.7;
	1014	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.7;
	1014	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.8;
	1014	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361;
	1015	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.8;
	1015	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171;
	1015	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.3;
	1015	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.8;
	1015	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.4;
	1015	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.8;
	1016	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.7;
	1016	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.8;
	1016	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.8;
	1016	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.5;
	1016	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.5;
	1016	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.6;
	1017	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.5;
	1017	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.1;
	1017	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.3;
	1017	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.2;
	1017	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.5;
	1017	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.5;
	1018	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.5;
	1018	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.6;
	1018	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.6;
	1018	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.5;
	1018	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.5;
	1018	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.8;
	1019	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.8;
	1019	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.2;
	1019	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.7;
	1019	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.8;
	1019	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.5;
	1019	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.6;
	1020	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.2;
	1020	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.5;
	1020	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211;
	1020	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.8;
	1020	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.1;
	1020	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.1;
	1021	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.1;
	1021	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.2;
	1021	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211;
	1021	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.1;
	1021	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.1;
	1021	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.4;
	1022	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.9;
	1022	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.5;
	1022	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.5;
	1022	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.5;
	1022	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.2;
	1022	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.7;
	1023	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466;
	1023	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.5;
	1023	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.6;
	1023	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.4;
	1023	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.8;
	1023	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.5;
	1024	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.8;
	1024	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176;
	1024	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.9;
	1024	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.3;
	1024	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.9;
	1024	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.6;
	1025	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.1;
	1025	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.5;
	1025	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.9;
	1025	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.3;
	1025	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.3;
	1025	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.3;
	1026	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.1;
	1026	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.6;
	1026	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.5;
	1026	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.1;
	1026	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.4;
	1026	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.2;
	1027	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.3;
	1027	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.1;
	1027	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.1;
	1027	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.9;
	1027	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.6;
	1027	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389;
	1028	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518.3;
	1028	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190;
	1028	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.5;
	1028	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.7;
	1028	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.2;
	1028	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.1;
	1029	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.4;
	1029	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.1;
	1029	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.8;
	1029	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.9;
	1029	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.3;
	1029	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.2;
	1030	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.9;
	1030	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.8;
	1030	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.5;
	1030	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.2;
	1030	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.3;
	1030	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.1;
	1031	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.1;
	1031	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.4;
	1031	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214;
	1031	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.3;
	1031	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.1;
	1031	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.7;
	1032	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.2;
	1032	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.4;
	1032	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209;
	1032	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.2;
	1032	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.1;
	1032	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.4;
	1033	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.4;
	1033	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153;
	1033	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.9;
	1033	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.5;
	1033	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.5;
	1033	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352;
	1034	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.4;
	1034	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.4;
	1034	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.2;
	1034	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.4;
	1034	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.3;
	1034	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.1;
	1035	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.1;
	1035	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.3;
	1035	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.6;
	1035	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.5;
	1035	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.8;
	1035	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.3;
	1036	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.8;
	1036	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.5;
	1036	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.8;
	1036	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.8;
	1036	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77;
	1036	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.5;
	1037	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.5;
	1037	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.9;
	1037	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.8;
	1037	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.6;
	1037	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.2;
	1037	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.2;
	1038	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.5;
	1038	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.4;
	1038	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.8;
	1038	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213;
	1038	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.4;
	1038	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.2;
	1039	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.2;
	1039	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.8;
	1039	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.2;
	1039	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.5;
	1039	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84;
	1039	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.8;
	1040	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.3;
	1040	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.4;
	1040	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.4;
	1040	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.9;
	1040	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.3;
	1040	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.7;
	1041	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	576.8;
	1041	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.4;
	1041	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.5;
	1041	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.8;
	1041	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.3;
	1041	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.6;
	1042	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	569;
	1042	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.2;
	1042	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.7;
	1042	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.1;
	1042	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.9;
	1042	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.3;
	1043	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564.1;
	1043	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.3;
	1043	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.2;
	1043	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246;
	1043	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.5;
	1043	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.6;
	1044	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	553;
	1044	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.5;
	1044	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.1;
	1044	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.5;
	1044	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91;
	1044	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.9;
	1045	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	546.5;
	1045	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.9;
	1045	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.7;
	1045	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.5;
	1045	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.2;
	1045	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.8;
	1046	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.8;
	1046	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.1;
	1046	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.4;
	1046	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.2;
	1046	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.5;
	1046	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.4;
	1047	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	553.2;
	1047	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.4;
	1047	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.8;
	1047	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.3;
	1047	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.5;
	1047	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.5;
	1048	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547.6;
	1048	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.4;
	1048	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.7;
	1048	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.5;
	1048	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.4;
	1048	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.3;
	1049	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	569.4;
	1049	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.9;
	1049	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.5;
	1049	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.3;
	1049	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.9;
	1049	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.8;
	1050	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	580.7;
	1050	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	199.1;
	1050	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.7;
	1050	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.2;
	1050	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.4;
	1050	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420;
	1051	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588;
	1051	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	199.8;
	1051	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.3;
	1051	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259;
	1051	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.5;
	1051	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	424.2;
	1052	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588.7;
	1052	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198.1;
	1052	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.8;
	1052	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.5;
	1052	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96.3;
	1052	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.3;
	1053	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	573;
	1053	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.4;
	1053	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.7;
	1053	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.4;
	1053	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	97.8;
	1053	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.2;
	1054	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.7;
	1054	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.4;
	1054	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.8;
	1054	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.2;
	1054	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	98.5;
	1054	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.9;
	1055	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	534.9;
	1055	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.9;
	1055	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.3;
	1055	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.6;
	1055	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	97.3;
	1055	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.5;
	1056	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.4;
	1056	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.8;
	1056	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.1;
	1056	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214;
	1056	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96.4;
	1056	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.2;
	1057	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.9;
	1057	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.5;
	1057	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.7;
	1057	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.4;
	1057	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.9;
	1057	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.5;
	1058	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.9;
	1058	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.6;
	1058	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.9;
	1058	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.9;
	1058	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.7;
	1058	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.5;
	1059	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.1;
	1059	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.2;
	1059	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.3;
	1059	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.9;
	1059	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.6;
	1059	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.4;
	1060	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.4;
	1060	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.1;
	1060	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.5;
	1060	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203;
	1060	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.1;
	1060	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.7;
	1061	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.8;
	1061	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.5;
	1061	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.3;
	1061	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.4;
	1061	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.6;
	1061	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.7;
	1062	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.7;
	1062	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.8;
	1062	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.8;
	1062	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.7;
	1062	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96.1;
	1062	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.7;
	1063	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566.4;
	1063	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.3;
	1063	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.5;
	1063	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.6;
	1063	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96.7;
	1063	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.2;
	1064	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.6;
	1064	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194;
	1064	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.3;
	1064	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.3;
	1064	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	98.2;
	1064	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420.4;
	1065	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.5;
	1065	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.6;
	1065	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.4;
	1065	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247;
	1065	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	98.2;
	1065	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422;
	1066	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588.5;
	1066	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.5;
	1066	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.4;
	1066	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.5;
	1066	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	98;
	1066	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.8;
	1067	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	576.5;
	1067	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.9;
	1067	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.9;
	1067	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246;
	1067	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96.9;
	1067	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.9;
	1068	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577.5;
	1068	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.7;
	1068	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.4;
	1068	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.5;
	1068	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.9;
	1068	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.9;
	1069	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	573.8;
	1069	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.8;
	1069	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.7;
	1069	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.2;
	1069	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.8;
	1069	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.7;
	1070	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559.7;
	1070	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.1;
	1070	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.2;
	1070	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.9;
	1070	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89;
	1070	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.3;
	1071	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.5;
	1071	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.9;
	1071	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.1;
	1071	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.6;
	1071	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.5;
	1071	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.3;
	1072	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	567.5;
	1072	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183;
	1072	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.3;
	1072	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.6;
	1072	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.1;
	1072	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.7;
	1073	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	569.4;
	1073	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.3;
	1073	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.7;
	1073	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.1;
	1073	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.1;
	1073	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.1;
	1074	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579.4;
	1074	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	199.9;
	1074	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.7;
	1074	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.6;
	1074	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.6;
	1074	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.7;
	1075	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	595.1;
	1075	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	200.9;
	1075	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.1;
	1075	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257;
	1075	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.3;
	1075	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.4;
	1076	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	604.2;
	1076	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	199.4;
	1076	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.2;
	1076	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.7;
	1076	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.3;
	1076	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	428.3;
	1077	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	580.5;
	1077	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.2;
	1077	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.1;
	1077	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.1;
	1077	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.6;
	1077	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.2;
	1078	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	561.4;
	1078	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.9;
	1078	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.3;
	1078	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.8;
	1078	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91;
	1078	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.8;
	1079	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.7;
	1079	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.5;
	1079	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.5;
	1079	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.7;
	1079	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.6;
	1079	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.1;
	1080	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.1;
	1080	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.5;
	1080	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.8;
	1080	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.9;
	1080	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.9;
	1080	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.5;
	1081	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.4;
	1081	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.2;
	1081	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.4;
	1081	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.3;
	1081	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.3;
	1081	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.9;
	1082	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.9;
	1082	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.9;
	1082	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.1;
	1082	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.1;
	1082	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.4;
	1082	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.8;
	1083	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.4;
	1083	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.3;
	1083	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.1;
	1083	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202;
	1083	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.3;
	1083	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.8;
	1084	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.4;
	1084	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.6;
	1084	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.1;
	1084	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.8;
	1084	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.8;
	1084	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.6;
	1085	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538;
	1085	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.8;
	1085	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.7;
	1085	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.4;
	1085	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.5;
	1085	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.8;
	1086	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	561.5;
	1086	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.5;
	1086	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217;
	1086	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.2;
	1086	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.4;
	1086	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.8;
	1087	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	591.9;
	1087	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.3;
	1087	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.2;
	1087	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.7;
	1087	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.4;
	1087	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.5;
	1088	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	622.2;
	1088	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.7;
	1088	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240;
	1088	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.7;
	1088	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.4;
	1088	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	430.1;
	1089	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	614.7;
	1089	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.8;
	1089	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.4;
	1089	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.8;
	1089	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93;
	1089	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	428.3;
	1090	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	600.2;
	1090	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.8;
	1090	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.9;
	1090	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.2;
	1090	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.3;
	1090	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.7;
	1091	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579.9;
	1091	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.2;
	1091	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.2;
	1091	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.2;
	1091	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94;
	1091	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.3;
	1092	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562;
	1092	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.6;
	1092	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.8;
	1092	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.9;
	1092	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.5;
	1092	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.4;
	1093	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.2;
	1093	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.1;
	1093	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.7;
	1093	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.8;
	1093	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.7;
	1093	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406;
	1094	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.5;
	1094	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.9;
	1094	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.4;
	1094	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.1;
	1094	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.3;
	1094	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.1;
	1095	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.6;
	1095	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.6;
	1095	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.3;
	1095	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.9;
	1095	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.7;
	1095	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.8;
	1096	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	554;
	1096	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.8;
	1096	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.7;
	1096	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.3;
	1096	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89;
	1096	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.1;
	1097	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560;
	1097	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.5;
	1097	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243;
	1097	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.9;
	1097	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.6;
	1097	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.2;
	1098	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.2;
	1098	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.4;
	1098	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.7;
	1098	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.4;
	1098	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.6;
	1098	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.1;
	1099	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.1;
	1099	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	199.6;
	1099	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252;
	1099	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.6;
	1099	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.3;
	1099	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421;
	1100	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589.4;
	1100	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198.1;
	1100	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.3;
	1100	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.5;
	1100	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.4;
	1100	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422;
	1101	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.3;
	1101	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.1;
	1101	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.3;
	1101	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.2;
	1101	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.7;
	1101	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.1;
	1102	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.2;
	1102	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.5;
	1102	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.1;
	1102	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.1;
	1102	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.6;
	1102	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.6;
	1103	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	540.3;
	1103	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.4;
	1103	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.2;
	1103	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.2;
	1103	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.8;
	1103	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.5;
	1104	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.9;
	1104	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.6;
	1104	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.8;
	1104	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.4;
	1104	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.5;
	1104	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.5;
	1105	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.1;
	1105	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.1;
	1105	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202;
	1105	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.4;
	1105	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.7;
	1105	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.5;
	1106	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.1;
	1106	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.1;
	1106	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.1;
	1106	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.4;
	1106	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.6;
	1106	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.4;
	1107	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.2;
	1107	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.5;
	1107	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.5;
	1107	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.9;
	1107	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.2;
	1107	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366;
	1108	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.5;
	1108	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.6;
	1108	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200;
	1108	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.6;
	1108	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.8;
	1108	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.7;
	1109	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528.3;
	1109	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.4;
	1109	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.8;
	1109	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.8;
	1109	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	1109	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.5;
	1110	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559.6;
	1110	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.5;
	1110	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.7;
	1110	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.4;
	1110	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	1110	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.4;
	1111	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	583.8;
	1111	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.5;
	1111	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.3;
	1111	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.6;
	1111	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	1111	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.3;
	1112	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	600.6;
	1112	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.2;
	1112	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.1;
	1112	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.1;
	1112	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.2;
	1112	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420.6;
	1113	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.9;
	1113	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.7;
	1113	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239;
	1113	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.6;
	1113	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.8;
	1113	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.1;
	1114	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	593.2;
	1114	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.5;
	1114	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.4;
	1114	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.1;
	1114	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.5;
	1114	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.9;
	1115	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.7;
	1115	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.4;
	1115	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.3;
	1115	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.5;
	1115	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.7;
	1115	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.9;
	1116	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	587.4;
	1116	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.8;
	1116	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.1;
	1116	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.3;
	1116	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.1;
	1116	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.2;
	1117	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.3;
	1117	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.5;
	1117	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.4;
	1117	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.8;
	1117	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.6;
	1117	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.5;
	1118	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	567.4;
	1118	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.2;
	1118	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.5;
	1118	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.8;
	1118	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.9;
	1118	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.4;
	1119	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	563.1;
	1119	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.3;
	1119	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.1;
	1119	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.5;
	1119	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.1;
	1119	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.5;
	1120	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.3;
	1120	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.4;
	1120	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.7;
	1120	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.1;
	1120	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71;
	1120	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.7;
	1121	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566;
	1121	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.9;
	1121	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.7;
	1121	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.7;
	1121	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	1121	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.3;
	1122	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	576.1;
	1122	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	201.2;
	1122	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.8;
	1122	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.7;
	1122	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	1122	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.8;
	1123	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589;
	1123	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	202.1;
	1123	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.1;
	1123	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255;
	1123	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	1123	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423;
	1124	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589.3;
	1124	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	202.5;
	1124	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.8;
	1124	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251;
	1124	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.3;
	1124	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.6;
	1125	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	581.3;
	1125	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.4;
	1125	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.8;
	1125	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.9;
	1125	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.7;
	1125	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.6;
	1126	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566.1;
	1126	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.2;
	1126	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.9;
	1126	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.7;
	1126	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.3;
	1126	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.8;
	1127	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.8;
	1127	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.2;
	1127	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.2;
	1127	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.8;
	1127	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.9;
	1127	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.1;
	1128	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.1;
	1128	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.1;
	1128	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.7;
	1128	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.1;
	1128	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	1128	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.2;
	1129	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.3;
	1129	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.7;
	1129	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.6;
	1129	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.6;
	1129	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.7;
	1129	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.5;
	1130	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.2;
	1130	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158;
	1130	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.2;
	1130	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.9;
	1130	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	1130	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.3;
	1131	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.2;
	1131	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.3;
	1131	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.4;
	1131	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.3;
	1131	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	1131	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.5;
	1132	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.3;
	1132	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.8;
	1132	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.2;
	1132	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.1;
	1132	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	1132	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.2;
	1133	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.5;
	1133	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.9;
	1133	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.7;
	1133	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.2;
	1133	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	1133	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.8;
	1134	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.9;
	1134	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.8;
	1134	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.2;
	1134	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.2;
	1134	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.8;
	1134	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.2;
	1135	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589.9;
	1135	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.5;
	1135	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.1;
	1135	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.3;
	1135	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.9;
	1135	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.1;
	1136	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	605.5;
	1136	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.7;
	1136	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.1;
	1136	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.1;
	1136	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.5;
	1136	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.1;
	1137	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	604.8;
	1137	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.4;
	1137	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.9;
	1137	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.8;
	1137	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.4;
	1137	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.9;
	1138	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588.6;
	1138	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.9;
	1138	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.6;
	1138	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.9;
	1138	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81;
	1138	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.7;
	1139	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	563.8;
	1139	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.4;
	1139	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.2;
	1139	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241;
	1139	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.4;
	1139	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.6;
	1140	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	557.6;
	1140	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188;
	1140	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.5;
	1140	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.4;
	1140	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.5;
	1140	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.5;
	1141	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	546;
	1141	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.2;
	1141	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.9;
	1141	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.5;
	1141	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75;
	1141	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.5;
	1142	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.4;
	1142	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.1;
	1142	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.1;
	1142	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.7;
	1142	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.4;
	1142	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.4;
	1143	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.7;
	1143	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.3;
	1143	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.9;
	1143	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.2;
	1143	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.6;
	1143	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.3;
	1144	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.7;
	1144	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.9;
	1144	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.7;
	1144	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.9;
	1144	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.5;
	1144	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.6;
	1145	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.2;
	1145	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.5;
	1145	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238;
	1145	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.3;
	1145	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.3;
	1145	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395;
	1146	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.9;
	1146	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.6;
	1146	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245;
	1146	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.6;
	1146	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.7;
	1146	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.5;
	1147	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.1;
	1147	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.7;
	1147	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.9;
	1147	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.2;
	1147	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.9;
	1147	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410;
	1148	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.5;
	1148	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.4;
	1148	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.9;
	1148	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.1;
	1148	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.7;
	1148	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.8;
	1149	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559.1;
	1149	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.8;
	1149	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.9;
	1149	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.5;
	1149	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.6;
	1149	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.4;
	1150	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	540.5;
	1150	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.7;
	1150	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.4;
	1150	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.8;
	1150	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.4;
	1150	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.6;
	1151	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.5;
	1151	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.2;
	1151	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219;
	1151	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.3;
	1151	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.7;
	1151	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.2;
	1152	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.5;
	1152	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168;
	1152	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.7;
	1152	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.8;
	1152	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.3;
	1152	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.1;
	1153	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.6;
	1153	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.9;
	1153	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.5;
	1153	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.9;
	1153	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	1153	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.7;
	1154	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.6;
	1154	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.4;
	1154	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.1;
	1154	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.9;
	1154	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.3;
	1154	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.8;
	1155	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.2;
	1155	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.2;
	1155	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.7;
	1155	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.4;
	1155	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.7;
	1155	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.4;
	1156	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485;
	1156	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157;
	1156	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199;
	1156	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201;
	1156	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.5;
	1156	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.3;
	1157	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.8;
	1157	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160;
	1157	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.7;
	1157	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.7;
	1157	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.4;
	1157	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.2;
	1158	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.9;
	1158	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.4;
	1158	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.7;
	1158	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.3;
	1158	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.8;
	1158	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.3;
	1159	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.9;
	1159	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.4;
	1159	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.8;
	1159	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213;
	1159	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.9;
	1159	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.6;
	1160	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528;
	1160	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.1;
	1160	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.4;
	1160	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.2;
	1160	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80;
	1160	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.8;
	1161	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528.2;
	1161	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.1;
	1161	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.8;
	1161	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.6;
	1161	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.8;
	1161	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.4;
	1162	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	533.9;
	1162	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.9;
	1162	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.6;
	1162	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.4;
	1162	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.9;
	1162	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.6;
	1163	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537.3;
	1163	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.5;
	1163	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221;
	1163	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224;
	1163	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.6;
	1163	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.6;
	1164	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.8;
	1164	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.1;
	1164	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.8;
	1164	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.8;
	1164	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.2;
	1164	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.2;
	1165	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.8;
	1165	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.3;
	1165	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.4;
	1165	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.2;
	1165	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.2;
	1165	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.6;
	1166	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	524.2;
	1166	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.8;
	1166	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.9;
	1166	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.4;
	1166	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.5;
	1166	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.5;
	1167	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520;
	1167	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.1;
	1167	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.4;
	1167	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.5;
	1167	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.4;
	1167	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.5;
	1168	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.6;
	1168	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.5;
	1168	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.2;
	1168	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.8;
	1168	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.5;
	1168	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.1;
	1169	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.4;
	1169	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.4;
	1169	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.8;
	1169	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230;
	1169	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.5;
	1169	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.9;
	1170	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.1;
	1170	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.7;
	1170	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236;
	1170	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.7;
	1170	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.6;
	1170	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398;
	1171	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566.3;
	1171	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197;
	1171	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.7;
	1171	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.9;
	1171	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.7;
	1171	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.4;
	1172	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572;
	1172	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.8;
	1172	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.6;
	1172	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.4;
	1172	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.1;
	1172	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.1;
	1173	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.6;
	1173	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.1;
	1173	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.6;
	1173	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.9;
	1173	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.4;
	1173	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.2;
	1174	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.4;
	1174	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.7;
	1174	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.3;
	1174	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.3;
	1174	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.1;
	1174	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.3;
	1175	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518.8;
	1175	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.1;
	1175	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.4;
	1175	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.2;
	1175	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.1;
	1175	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.1;
	1176	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.9;
	1176	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.7;
	1176	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.3;
	1176	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.8;
	1176	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.4;
	1176	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.2;
	1177	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.2;
	1177	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.5;
	1177	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.4;
	1177	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.2;
	1177	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.9;
	1177	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.2;
	1178	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.5;
	1178	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.1;
	1178	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.4;
	1178	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.6;
	1178	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.9;
	1178	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.8;
	1179	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.6;
	1179	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.7;
	1179	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.8;
	1179	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.9;
	1179	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.7;
	1179	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.4;
	1180	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.1;
	1180	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.5;
	1180	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.4;
	1180	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.7;
	1180	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.5;
	1180	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370;
	1181	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.7;
	1181	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.4;
	1181	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.2;
	1181	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.3;
	1181	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.7;
	1181	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.1;
	1182	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.4;
	1182	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.9;
	1182	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.3;
	1182	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.6;
	1182	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.3;
	1182	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.2;
	1183	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.3;
	1183	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.1;
	1183	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.4;
	1183	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.1;
	1183	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85;
	1183	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.4;
	1184	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537;
	1184	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.8;
	1184	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.2;
	1184	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.9;
	1184	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.9;
	1184	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387;
	1185	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.3;
	1185	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.7;
	1185	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.1;
	1185	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.3;
	1185	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91;
	1185	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.3;
	1186	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549;
	1186	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.1;
	1186	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.3;
	1186	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.2;
	1186	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.3;
	1186	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.6;
	1187	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.5;
	1187	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195;
	1187	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.8;
	1187	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.6;
	1187	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.2;
	1187	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.2;
	1188	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.4;
	1188	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.3;
	1188	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.9;
	1188	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.9;
	1188	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.5;
	1188	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.6;
	1189	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	533.5;
	1189	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.6;
	1189	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.8;
	1189	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.2;
	1189	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.4;
	1189	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.2;
	1190	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526.4;
	1190	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.9;
	1190	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.4;
	1190	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.7;
	1190	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.9;
	1190	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.3;
	1191	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.4;
	1191	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.7;
	1191	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.9;
	1191	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.6;
	1191	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.6;
	1191	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.9;
	1192	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.4;
	1192	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.3;
	1192	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.6;
	1192	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.5;
	1192	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.9;
	1192	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.7;
	1193	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530;
	1193	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.1;
	1193	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.6;
	1193	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.7;
	1193	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.7;
	1193	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.2;
	1194	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.8;
	1194	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	202.5;
	1194	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.1;
	1194	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.6;
	1194	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.9;
	1194	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.8;
	1195	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.2;
	1195	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	207.6;
	1195	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.4;
	1195	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.4;
	1195	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.7;
	1195	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.5;
	1196	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	575.4;
	1196	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	207.9;
	1196	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.1;
	1196	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.3;
	1196	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.3;
	1196	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.7;
	1197	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	571.1;
	1197	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	203.2;
	1197	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238;
	1197	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.6;
	1197	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.2;
	1197	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.8;
	1198	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552.2;
	1198	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.3;
	1198	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.1;
	1198	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.2;
	1198	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.8;
	1198	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400;
	1199	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525.7;
	1199	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.1;
	1199	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.5;
	1199	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.5;
	1199	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.9;
	1199	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.9;
	1200	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.9;
	1200	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.6;
	1200	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218;
	1200	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.5;
	1200	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.1;
	1200	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.7;
	1201	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.1;
	1201	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174;
	1201	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207;
	1201	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.2;
	1201	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	1201	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.1;
	1202	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508;
	1202	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.1;
	1202	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.6;
	1202	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.6;
	1202	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	1202	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.1;
	1203	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508;
	1203	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.2;
	1203	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.8;
	1203	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.3;
	1203	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.8;
	1203	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373;
	1204	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.5;
	1204	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176;
	1204	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.3;
	1204	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.8;
	1204	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	1204	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.1;
	1205	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.9;
	1205	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.5;
	1205	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.9;
	1205	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.6;
	1205	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.2;
	1205	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.9;
	1206	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552.5;
	1206	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.9;
	1206	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.5;
	1206	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.1;
	1206	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	1206	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.4;
	1207	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	583.1;
	1207	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	203.5;
	1207	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.7;
	1207	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.4;
	1207	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70;
	1207	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.2;
	1208	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	606.1;
	1208	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	211.9;
	1208	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.7;
	1208	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.3;
	1208	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	1208	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420.5;
	1209	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	605;
	1209	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	213.1;
	1209	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234;
	1209	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236;
	1209	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.8;
	1209	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.4;
	1210	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	602.1;
	1210	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	211.5;
	1210	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237;
	1210	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.3;
	1210	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	1210	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.4;
	1211	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	603.4;
	1211	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	209.1;
	1211	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.9;
	1211	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.3;
	1211	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	1211	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.7;
	1212	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.3;
	1212	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	205.9;
	1212	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.5;
	1212	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.8;
	1212	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	1212	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420.5;
	1213	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586.2;
	1213	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	201.2;
	1213	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.4;
	1213	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.4;
	1213	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.7;
	1213	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.2;
	1214	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	574.1;
	1214	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.7;
	1214	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.2;
	1214	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.2;
	1214	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	1214	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.2;
	1215	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	567.6;
	1215	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.1;
	1215	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.1;
	1215	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.3;
	1215	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.2;
	1215	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.4;
	1216	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566.9;
	1216	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.6;
	1216	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233;
	1216	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.6;
	1216	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	1216	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.3;
	1217	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.4;
	1217	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	202.8;
	1217	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239;
	1217	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.4;
	1217	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	1217	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.2;
	1218	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577.4;
	1218	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	212.8;
	1218	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.4;
	1218	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.4;
	1218	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	1218	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.5;
	1219	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	594.2;
	1219	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	216.8;
	1219	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.9;
	1219	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.4;
	1219	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	1219	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	424.4;
	1220	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.3;
	1220	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	212.5;
	1220	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.5;
	1220	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.3;
	1220	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	1220	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	424.9;
	1221	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589;
	1221	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	206.1;
	1221	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.2;
	1221	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.6;
	1221	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	1221	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.3;
	1222	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	554;
	1222	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.5;
	1222	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.3;
	1222	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.5;
	1222	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	1222	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.9;
	1223	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537.8;
	1223	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.1;
	1223	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.9;
	1223	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.2;
	1223	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	1223	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.5;
	1224	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522.9;
	1224	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.1;
	1224	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.2;
	1224	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.5;
	1224	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	1224	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.9;
	1225	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.7;
	1225	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.7;
	1225	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204;
	1225	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.2;
	1225	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	1225	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375;
	1226	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.2;
	1226	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.2;
	1226	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.5;
	1226	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.7;
	1226	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.9;
	1226	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.2;
	1227	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.4;
	1227	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.2;
	1227	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.3;
	1227	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.8;
	1227	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.6;
	1227	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.6;
	1228	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.3;
	1228	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.4;
	1228	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.5;
	1228	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.9;
	1228	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	1228	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.9;
	1229	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.5;
	1229	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.7;
	1229	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.8;
	1229	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.9;
	1229	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	1229	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.2;
	1230	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.4;
	1230	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.1;
	1230	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.9;
	1230	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.4;
	1230	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.9;
	1230	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.2;
	1231	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.9;
	1231	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195;
	1231	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.4;
	1231	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.9;
	1231	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	1231	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397;
	1232	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	573.1;
	1232	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.9;
	1232	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.5;
	1232	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.8;
	1232	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	1232	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.7;
	1233	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586.8;
	1233	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.2;
	1233	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.1;
	1233	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.5;
	1233	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	1233	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.9;
	1234	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	575.2;
	1234	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.6;
	1234	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.6;
	1234	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.6;
	1234	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	1234	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.5;
	1235	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.2;
	1235	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.9;
	1235	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.6;
	1235	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.2;
	1235	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	1235	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.1;
	1236	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	561.5;
	1236	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.1;
	1236	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239;
	1236	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.9;
	1236	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	1236	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.3;
	1237	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	554.2;
	1237	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.4;
	1237	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.7;
	1237	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.6;
	1237	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	1237	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.2;
	1238	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.4;
	1238	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.8;
	1238	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.7;
	1238	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.7;
	1238	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	1238	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.5;
	1239	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542;
	1239	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.8;
	1239	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.3;
	1239	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.3;
	1239	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	1239	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.1;
	1240	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.1;
	1240	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.8;
	1240	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.9;
	1240	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241;
	1240	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	1240	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.8;
	1241	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548.2;
	1241	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.9;
	1241	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.4;
	1241	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.3;
	1241	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	1241	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.9;
	1242	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552.9;
	1242	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198.9;
	1242	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.3;
	1242	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.5;
	1242	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	1242	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.8;
	1243	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	569.1;
	1243	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	199.9;
	1243	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.1;
	1243	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.1;
	1243	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	1243	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.9;
	1244	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.3;
	1244	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198.7;
	1244	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.5;
	1244	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.6;
	1244	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	1244	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.5;
	1245	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	561.3;
	1245	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.7;
	1245	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.9;
	1245	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.1;
	1245	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	1245	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.4;
	1246	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	540.9;
	1246	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.2;
	1246	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.7;
	1246	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.7;
	1246	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	1246	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.9;
	1247	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.2;
	1247	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171;
	1247	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.4;
	1247	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.1;
	1247	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	1247	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.7;
	1248	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.8;
	1248	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.2;
	1248	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.2;
	1248	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.6;
	1248	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	1248	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.9;
	1249	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.9;
	1249	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.9;
	1249	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.4;
	1249	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.8;
	1249	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	1249	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358;
	1250	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.6;
	1250	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.7;
	1250	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.3;
	1250	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.1;
	1250	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	1250	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.1;
	1251	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469;
	1251	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.1;
	1251	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.1;
	1251	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.3;
	1251	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.6;
	1251	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.5;
	1252	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475;
	1252	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.1;
	1252	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199;
	1252	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.4;
	1252	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.1;
	1252	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.8;
	1253	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.3;
	1253	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.9;
	1253	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.1;
	1253	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.8;
	1253	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.4;
	1253	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.8;
	1254	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.9;
	1254	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.3;
	1254	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.3;
	1254	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.6;
	1254	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.3;
	1254	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.1;
	1255	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552.6;
	1255	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.4;
	1255	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.9;
	1255	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.7;
	1255	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	1255	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.3;
	1256	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566.6;
	1256	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.1;
	1256	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235;
	1256	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.7;
	1256	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	1256	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.2;
	1257	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	561.8;
	1257	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.8;
	1257	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238;
	1257	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.1;
	1257	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.7;
	1257	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.8;
	1258	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559.4;
	1258	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.1;
	1258	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.2;
	1258	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.3;
	1258	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.9;
	1258	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.3;
	1259	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552.1;
	1259	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.9;
	1259	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.8;
	1259	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.5;
	1259	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.9;
	1259	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.9;
	1260	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537.9;
	1260	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.2;
	1260	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.1;
	1260	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.2;
	1260	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	1260	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.3;
	1261	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528.6;
	1261	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.5;
	1261	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.4;
	1261	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.4;
	1261	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.6;
	1261	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.7;
	1262	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531;
	1262	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.2;
	1262	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.8;
	1262	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.6;
	1262	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	1262	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.2;
	1263	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535;
	1263	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.2;
	1263	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235;
	1263	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239;
	1263	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	1263	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.2;
	1264	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	533;
	1264	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.5;
	1264	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.5;
	1264	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.5;
	1264	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	1264	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.5;
	1265	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536;
	1265	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.6;
	1265	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.9;
	1265	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.1;
	1265	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	1265	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.8;
	1266	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548;
	1266	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.7;
	1266	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.8;
	1266	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.8;
	1266	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	1266	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.2;
	1267	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	557.5;
	1267	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	201.8;
	1267	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.9;
	1267	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.8;
	1267	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	1267	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.6;
	1268	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559.8;
	1268	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198.7;
	1268	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.3;
	1268	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.2;
	1268	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.7;
	1268	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.8;
	1269	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	546.9;
	1269	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.5;
	1269	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.9;
	1269	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.7;
	1269	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.8;
	1269	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.1;
	1270	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537.7;
	1270	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.1;
	1270	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.4;
	1270	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.8;
	1270	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	1270	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.7;
	1271	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.8;
	1271	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.1;
	1271	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.7;
	1271	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.1;
	1271	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.3;
	1271	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.1;
	1272	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.2;
	1272	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.4;
	1272	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.6;
	1272	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.8;
	1272	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	1272	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371;
	1273	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.8;
	1273	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.3;
	1273	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201;
	1273	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.9;
	1273	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	1273	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.2;
	1274	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.8;
	1274	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.6;
	1274	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.6;
	1274	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.7;
	1274	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	1274	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.8;
	1275	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487;
	1275	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154;
	1275	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.5;
	1275	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.5;
	1275	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.7;
	1275	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.2;
	1276	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.6;
	1276	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.4;
	1276	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.9;
	1276	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.2;
	1276	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	1276	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.5;
	1277	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521;
	1277	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.7;
	1277	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.2;
	1277	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.9;
	1277	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.3;
	1277	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.7;
	1278	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564.9;
	1278	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179;
	1278	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.3;
	1278	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.5;
	1278	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	1278	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.3;
	1279	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.6;
	1279	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.2;
	1279	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.8;
	1279	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.2;
	1279	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	1279	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.7;
	1280	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	606.8;
	1280	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.9;
	1280	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233;
	1280	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.2;
	1280	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.1;
	1280	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.5;
	1281	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	605.9;
	1281	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.8;
	1281	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.4;
	1281	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.4;
	1281	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.5;
	1281	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.6;
	1282	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	580.4;
	1282	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.8;
	1282	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.1;
	1282	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.4;
	1282	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.7;
	1282	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.8;
	1283	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	574.5;
	1283	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.1;
	1283	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.9;
	1283	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.3;
	1283	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.5;
	1283	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.5;
	1284	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.1;
	1284	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.8;
	1284	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.2;
	1284	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.3;
	1284	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.3;
	1284	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.5;
	1285	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	568.8;
	1285	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.7;
	1285	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.6;
	1285	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.1;
	1285	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.9;
	1285	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.7;
	1286	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.1;
	1286	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.6;
	1286	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.7;
	1286	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.3;
	1286	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.8;
	1286	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.1;
	1287	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	563;
	1287	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.9;
	1287	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.9;
	1287	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.2;
	1287	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.5;
	1287	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405;
	1288	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.3;
	1288	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.2;
	1288	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233;
	1288	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.5;
	1288	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.9;
	1288	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.1;
	1289	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.5;
	1289	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.8;
	1289	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.2;
	1289	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.5;
	1289	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.9;
	1289	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.4;
	1290	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.2;
	1290	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198.1;
	1290	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.8;
	1290	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.7;
	1290	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.5;
	1290	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.3;
	1291	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	583.4;
	1291	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	201.8;
	1291	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.2;
	1291	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.3;
	1291	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.9;
	1291	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.5;
	1292	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.8;
	1292	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	201.7;
	1292	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.3;
	1292	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.3;
	1292	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.3;
	1292	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.7;
	1293	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	583.3;
	1293	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.3;
	1293	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.8;
	1293	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.5;
	1293	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.2;
	1293	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.4;
	1294	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559.3;
	1294	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187;
	1294	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.5;
	1294	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.8;
	1294	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.4;
	1294	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.2;
	1295	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537.9;
	1295	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.3;
	1295	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.7;
	1295	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221;
	1295	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.2;
	1295	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.8;
	1296	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.9;
	1296	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.8;
	1296	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.7;
	1296	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.6;
	1296	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	1296	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.7;
	1297	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.6;
	1297	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163;
	1297	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.8;
	1297	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.6;
	1297	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	1297	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374;
	1298	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.3;
	1298	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.3;
	1298	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.2;
	1298	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.7;
	1298	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	1298	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.5;
	1299	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.7;
	1299	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.9;
	1299	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.5;
	1299	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202;
	1299	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	1299	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.8;
	1300	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.9;
	1300	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.4;
	1300	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.1;
	1300	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.8;
	1300	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	1300	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.4;
	1301	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522.1;
	1301	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.4;
	1301	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.6;
	1301	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.6;
	1301	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.2;
	1301	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.8;
	1302	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547.9;
	1302	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.1;
	1302	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.7;
	1302	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219;
	1302	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	1302	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393;
	1303	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.3;
	1303	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.6;
	1303	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.7;
	1303	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.2;
	1303	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	1303	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404;
	1304	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	585;
	1304	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	199;
	1304	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.5;
	1304	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.5;
	1304	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.2;
	1304	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.1;
	1305	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	582.6;
	1305	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198;
	1305	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241;
	1305	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.1;
	1305	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.8;
	1305	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.8;
	1306	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579.5;
	1306	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.3;
	1306	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243;
	1306	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.7;
	1306	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.3;
	1306	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.3;
	1307	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	567.1;
	1307	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.7;
	1307	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.1;
	1307	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.6;
	1307	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.8;
	1307	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.7;
	1308	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559.7;
	1308	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.9;
	1308	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.5;
	1308	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.1;
	1308	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	1308	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.5;
	1309	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.3;
	1309	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.5;
	1309	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.7;
	1309	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.2;
	1309	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	1309	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408;
	1310	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.5;
	1310	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.8;
	1310	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.3;
	1310	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.7;
	1310	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.1;
	1310	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.3;
	1311	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528.6;
	1311	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.1;
	1311	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.5;
	1311	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.3;
	1311	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	1311	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.9;
	1312	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.7;
	1312	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.5;
	1312	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.9;
	1312	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.2;
	1312	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.1;
	1312	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.2;
	1313	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.6;
	1313	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.5;
	1313	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.1;
	1313	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.6;
	1313	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	1313	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.6;
	1314	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.4;
	1314	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.6;
	1314	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248;
	1314	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.5;
	1314	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	1314	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.8;
	1315	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552.1;
	1315	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.3;
	1315	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.3;
	1315	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.9;
	1315	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	1315	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.5;
	1316	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.6;
	1316	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.3;
	1316	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.6;
	1316	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.3;
	1316	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.8;
	1316	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.4;
	1317	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547;
	1317	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.1;
	1317	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.6;
	1317	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.3;
	1317	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	1317	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.8;
	1318	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525.8;
	1318	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.1;
	1318	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.4;
	1318	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.6;
	1318	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	1318	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388;
	1319	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.9;
	1319	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.9;
	1319	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.3;
	1319	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.5;
	1319	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	1319	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.3;
	1320	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.7;
	1320	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.8;
	1320	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.6;
	1320	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.3;
	1320	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	1320	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368;
	1321	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.8;
	1321	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.1;
	1321	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.6;
	1321	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.9;
	1321	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.8;
	1321	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.7;
	1322	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.1;
	1322	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.6;
	1322	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.9;
	1322	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.1;
	1322	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.5;
	1322	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.1;
	1323	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.6;
	1323	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.8;
	1323	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.4;
	1323	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.8;
	1323	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.1;
	1323	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.8;
	1324	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470;
	1324	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.3;
	1324	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.9;
	1324	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.4;
	1324	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54;
	1324	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.9;
	1325	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.2;
	1325	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157;
	1325	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.3;
	1325	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204;
	1325	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.2;
	1325	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.7;
	1326	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.1;
	1326	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.2;
	1326	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.6;
	1326	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.5;
	1326	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.3;
	1326	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.6;
	1327	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.1;
	1327	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.1;
	1327	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.3;
	1327	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.3;
	1327	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.9;
	1327	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.5;
	1328	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.7;
	1328	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.5;
	1328	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.3;
	1328	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.2;
	1328	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	1328	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.6;
	1329	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.1;
	1329	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.5;
	1329	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.8;
	1329	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219;
	1329	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	1329	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377;
	1330	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.9;
	1330	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.1;
	1330	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.6;
	1330	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221;
	1330	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	1330	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.6;
	1331	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.4;
	1331	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.8;
	1331	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.8;
	1331	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221;
	1331	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71;
	1331	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380;
	1332	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.5;
	1332	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.5;
	1332	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.8;
	1332	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.9;
	1332	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	1332	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.1;
	1333	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.8;
	1333	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.4;
	1333	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.2;
	1333	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219;
	1333	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.4;
	1333	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.1;
	1334	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.1;
	1334	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.7;
	1334	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214;
	1334	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.3;
	1334	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71;
	1334	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.5;
	1335	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.1;
	1335	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.1;
	1335	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.6;
	1335	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.9;
	1335	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	1335	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.8;
	1336	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.8;
	1336	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.8;
	1336	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.6;
	1336	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.3;
	1336	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	1336	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.4;
	1337	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.3;
	1337	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.2;
	1337	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.1;
	1337	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.1;
	1337	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.9;
	1337	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.2;
	1338	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.1;
	1338	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.4;
	1338	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.9;
	1338	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.3;
	1338	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.3;
	1338	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372;
	1339	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.1;
	1339	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.9;
	1339	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.5;
	1339	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.9;
	1339	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.7;
	1339	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.9;
	1340	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.3;
	1340	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176;
	1340	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.6;
	1340	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.7;
	1340	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.3;
	1340	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.5;
	1341	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.4;
	1341	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.9;
	1341	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.7;
	1341	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.7;
	1341	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.6;
	1341	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371;
	1342	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.6;
	1342	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.4;
	1342	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.1;
	1342	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223;
	1342	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.6;
	1342	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.7;
	1343	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.4;
	1343	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.8;
	1343	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.7;
	1343	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.5;
	1343	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.7;
	1343	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.8;
	1344	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.7;
	1344	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153;
	1344	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.9;
	1344	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.3;
	1344	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	1344	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.1;
	1345	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.9;
	1345	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.6;
	1345	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.5;
	1345	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.2;
	1345	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	1345	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.9;
	1346	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.3;
	1346	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.7;
	1346	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193;
	1346	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196;
	1346	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	1346	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.3;
	1347	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.9;
	1347	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.2;
	1347	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.2;
	1347	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.9;
	1347	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	1347	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.6;
	1348	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.4;
	1348	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.4;
	1348	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.3;
	1348	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.6;
	1348	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	1348	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.6;
	1349	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441;
	1349	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.1;
	1349	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.8;
	1349	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.7;
	1349	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	1349	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.3;
	1350	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.6;
	1350	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.2;
	1350	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.1;
	1350	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.2;
	1350	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69;
	1350	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.7;
	1351	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.3;
	1351	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.6;
	1351	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199;
	1351	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.1;
	1351	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.5;
	1351	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.4;
	1352	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.1;
	1352	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.9;
	1352	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.5;
	1352	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.5;
	1352	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77;
	1352	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.8;
	1353	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.7;
	1353	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.3;
	1353	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.4;
	1353	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.5;
	1353	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.1;
	1353	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356;
	1354	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.6;
	1354	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.9;
	1354	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.1;
	1354	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.7;
	1354	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.2;
	1354	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.7;
	1355	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.5;
	1355	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.9;
	1355	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.7;
	1355	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.9;
	1355	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.6;
	1355	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358;
	1356	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.5;
	1356	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.4;
	1356	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204;
	1356	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.7;
	1356	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.7;
	1356	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.1;
	1357	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.7;
	1357	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.3;
	1357	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.7;
	1357	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.8;
	1357	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.9;
	1357	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.8;
	1358	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.9;
	1358	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.1;
	1358	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.7;
	1358	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.6;
	1358	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.2;
	1358	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.1;
	1359	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.5;
	1359	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.2;
	1359	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.7;
	1359	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.9;
	1359	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.9;
	1359	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351;
	1360	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.7;
	1360	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.5;
	1360	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.2;
	1360	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.6;
	1360	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.1;
	1360	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.1;
	1361	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.9;
	1361	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.7;
	1361	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.5;
	1361	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.7;
	1361	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.2;
	1361	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.8;
	1362	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.1;
	1362	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.1;
	1362	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.2;
	1362	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.7;
	1362	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.7;
	1362	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.2;
	1363	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.2;
	1363	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179;
	1363	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.1;
	1363	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.8;
	1363	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.2;
	1363	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.6;
	1364	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508;
	1364	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.7;
	1364	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.3;
	1364	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.8;
	1364	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.3;
	1364	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.2;
	1365	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.7;
	1365	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.3;
	1365	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.4;
	1365	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.6;
	1365	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.8;
	1365	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.8;
	1366	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.6;
	1366	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.2;
	1366	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.7;
	1366	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.5;
	1366	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78;
	1366	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371;
	1367	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.7;
	1367	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.2;
	1367	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.5;
	1367	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.2;
	1367	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.8;
	1367	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364;
	1368	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.2;
	1368	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.6;
	1368	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.5;
	1368	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207;
	1368	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.7;
	1368	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.1;
	1369	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459;
	1369	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.8;
	1369	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.8;
	1369	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.7;
	1369	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.7;
	1369	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.8;
	1370	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.6;
	1370	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.6;
	1370	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.7;
	1370	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195;
	1370	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	1370	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.6;
	1371	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.7;
	1371	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.2;
	1371	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.4;
	1371	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.5;
	1371	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.4;
	1371	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.4;
	1372	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.7;
	1372	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.5;
	1372	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.3;
	1372	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.4;
	1372	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.7;
	1372	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.9;
	1373	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.1;
	1373	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152;
	1373	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.8;
	1373	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.9;
	1373	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.7;
	1373	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.3;
	1374	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.9;
	1374	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.4;
	1374	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.1;
	1374	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.7;
	1374	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.1;
	1374	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.5;
	1375	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.3;
	1375	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.2;
	1375	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.9;
	1375	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.5;
	1375	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.9;
	1375	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.4;
	1376	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564.5;
	1376	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.3;
	1376	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.3;
	1376	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.3;
	1376	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.3;
	1376	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.4;
	1377	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	576.3;
	1377	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.9;
	1377	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.7;
	1377	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.1;
	1377	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.5;
	1377	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.9;
	1378	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	576.5;
	1378	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.4;
	1378	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.6;
	1378	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.3;
	1378	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.1;
	1378	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.3;
	1379	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	557.5;
	1379	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.5;
	1379	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.4;
	1379	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.8;
	1379	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.2;
	1379	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.6;
	1380	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.8;
	1380	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179;
	1380	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.9;
	1380	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.2;
	1380	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.9;
	1380	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.3;
	1381	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.2;
	1381	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.6;
	1381	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.9;
	1381	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237;
	1381	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73;
	1381	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.1;
	1382	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.2;
	1382	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.5;
	1382	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.3;
	1382	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.3;
	1382	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.4;
	1382	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.4;
	1383	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.4;
	1383	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.4;
	1383	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.4;
	1383	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.2;
	1383	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.9;
	1383	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388;
	1384	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.3;
	1384	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.9;
	1384	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.4;
	1384	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.3;
	1384	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	1384	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.7;
	1385	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.3;
	1385	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.6;
	1385	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.6;
	1385	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.6;
	1385	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	1385	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.8;
	1386	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.8;
	1386	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.8;
	1386	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.2;
	1386	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.7;
	1386	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	1386	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.7;
	1387	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	533.1;
	1387	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.9;
	1387	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.2;
	1387	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.7;
	1387	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.8;
	1387	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.8;
	1388	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.3;
	1388	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195;
	1388	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.1;
	1388	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.1;
	1388	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	1388	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.6;
	1389	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	540.3;
	1389	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.7;
	1389	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.4;
	1389	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.3;
	1389	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.6;
	1389	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.4;
	1390	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542;
	1390	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.3;
	1390	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.5;
	1390	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.1;
	1390	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.2;
	1390	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397;
	1391	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.8;
	1391	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.1;
	1391	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.9;
	1391	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.9;
	1391	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	1391	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384;
	1392	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.1;
	1392	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.9;
	1392	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.2;
	1392	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212;
	1392	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.5;
	1392	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.1;
	1393	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.1;
	1393	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.9;
	1393	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.7;
	1393	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.5;
	1393	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69;
	1393	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.5;
	1394	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.7;
	1394	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.9;
	1394	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197;
	1394	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.8;
	1394	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.2;
	1394	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358;
	1395	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.2;
	1395	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.7;
	1395	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.1;
	1395	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.1;
	1395	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	1395	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.9;
	1396	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.7;
	1396	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159;
	1396	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.2;
	1396	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.8;
	1396	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68;
	1396	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.3;
	1397	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.3;
	1397	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.4;
	1397	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.5;
	1397	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208;
	1397	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.4;
	1397	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.5;
	1398	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.8;
	1398	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180;
	1398	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.8;
	1398	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219;
	1398	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.1;
	1398	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381;
	1399	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547.2;
	1399	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.3;
	1399	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.9;
	1399	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.5;
	1399	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	1399	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.7;
	1400	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	569.4;
	1400	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.8;
	1400	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.4;
	1400	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.9;
	1400	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	1400	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.4;
	1401	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	571.8;
	1401	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.9;
	1401	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.1;
	1401	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.5;
	1401	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.5;
	1401	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411;
	1402	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.4;
	1402	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.6;
	1402	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.9;
	1402	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.5;
	1402	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.5;
	1402	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.5;
	1403	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547.1;
	1403	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.5;
	1403	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.9;
	1403	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.5;
	1403	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.1;
	1403	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.9;
	1404	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.1;
	1404	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.3;
	1404	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239;
	1404	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.3;
	1404	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	1404	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.3;
	1405	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.7;
	1405	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.5;
	1405	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.6;
	1405	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.7;
	1405	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	1405	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395;
	1406	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.9;
	1406	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182;
	1406	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.6;
	1406	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.9;
	1406	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	1406	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.3;
	1407	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.2;
	1407	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.8;
	1407	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.8;
	1407	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.9;
	1407	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.4;
	1407	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.3;
	1408	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520;
	1408	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.1;
	1408	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.1;
	1408	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.1;
	1408	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.9;
	1408	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.6;
	1409	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.2;
	1409	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.3;
	1409	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.9;
	1409	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.8;
	1409	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60;
	1409	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.5;
	1410	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.4;
	1410	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	199.2;
	1410	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.2;
	1410	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.2;
	1410	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	1410	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.5;
	1411	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	553.3;
	1411	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	202.8;
	1411	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.2;
	1411	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.6;
	1411	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.3;
	1411	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.3;
	1412	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.2;
	1412	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	200.7;
	1412	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.8;
	1412	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.5;
	1412	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	1412	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410;
	1413	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548.8;
	1413	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.6;
	1413	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.1;
	1413	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.1;
	1413	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	1413	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.8;
	1414	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.6;
	1414	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183;
	1414	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.3;
	1414	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.3;
	1414	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	1414	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.5;
	1415	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.2;
	1415	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.9;
	1415	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.3;
	1415	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.8;
	1415	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	1415	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.3;
	1416	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.8;
	1416	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.4;
	1416	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.3;
	1416	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.6;
	1416	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	1416	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.4;
	1417	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.4;
	1417	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.9;
	1417	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201;
	1417	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.2;
	1417	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.4;
	1417	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.9;
	1418	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.1;
	1418	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.6;
	1418	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.5;
	1418	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.2;
	1418	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	1418	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.7;
	1419	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.8;
	1419	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.4;
	1419	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.4;
	1419	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.5;
	1419	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.7;
	1419	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.4;
	1420	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.8;
	1420	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.5;
	1420	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.9;
	1420	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.2;
	1420	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.5;
	1420	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.3;
	1421	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.9;
	1421	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.7;
	1421	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.6;
	1421	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.3;
	1421	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.1;
	1421	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.3;
	1422	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.4;
	1422	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.6;
	1422	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215;
	1422	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.1;
	1422	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	1422	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380;
	1423	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548.9;
	1423	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.7;
	1423	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.3;
	1423	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.7;
	1423	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	1423	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.4;
	1424	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	573.6;
	1424	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.5;
	1424	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.4;
	1424	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241;
	1424	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	1424	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.9;
	1425	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.2;
	1425	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.5;
	1425	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.9;
	1425	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.2;
	1425	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	1425	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.5;
	1426	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549.1;
	1426	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.5;
	1426	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.5;
	1426	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.3;
	1426	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	1426	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.5;
	1427	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.3;
	1427	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186;
	1427	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.5;
	1427	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.1;
	1427	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.2;
	1427	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.8;
	1428	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.5;
	1428	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.4;
	1428	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.5;
	1428	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.2;
	1428	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	1428	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396;
	1429	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537.7;
	1429	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.4;
	1429	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.7;
	1429	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.2;
	1429	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.2;
	1429	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.3;
	1430	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.3;
	1430	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.1;
	1430	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.1;
	1430	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.7;
	1430	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	1430	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.2;
	1431	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.8;
	1431	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.9;
	1431	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.4;
	1431	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.3;
	1431	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	1431	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.8;
	1432	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.8;
	1432	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.2;
	1432	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.5;
	1432	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.5;
	1432	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	1432	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.4;
	1433	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.2;
	1433	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.2;
	1433	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.5;
	1433	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.1;
	1433	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	1433	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.9;
	1434	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.7;
	1434	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.6;
	1434	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.7;
	1434	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.2;
	1434	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	1434	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.7;
	1435	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.5;
	1435	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.5;
	1435	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.5;
	1435	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.7;
	1435	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	1435	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.9;
	1436	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	561.8;
	1436	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.4;
	1436	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.6;
	1436	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.6;
	1436	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.2;
	1436	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.8;
	1437	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.8;
	1437	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.2;
	1437	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.2;
	1437	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244;
	1437	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	1437	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.3;
	1438	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.1;
	1438	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.2;
	1438	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.3;
	1438	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.7;
	1438	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	1438	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.7;
	1439	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.6;
	1439	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.4;
	1439	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.4;
	1439	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.7;
	1439	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	1439	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.3;
	1440	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.7;
	1440	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.5;
	1440	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.4;
	1440	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.8;
	1440	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	1440	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.1;
	1441	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.3;
	1441	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.8;
	1441	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.3;
	1441	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203;
	1441	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	1441	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.2;
	1442	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.5;
	1442	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.6;
	1442	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.4;
	1442	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.6;
	1442	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	1442	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355;
	1443	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.7;
	1443	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.8;
	1443	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.3;
	1443	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.4;
	1443	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	1443	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.1;
	1444	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.1;
	1444	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.5;
	1444	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.4;
	1444	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.8;
	1444	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	1444	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.9;
	1445	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.7;
	1445	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.3;
	1445	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.2;
	1445	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.1;
	1445	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	1445	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.5;
	1446	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522;
	1446	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.7;
	1446	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.2;
	1446	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.6;
	1446	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	1446	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.6;
	1447	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.7;
	1447	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.7;
	1447	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.5;
	1447	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.3;
	1447	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	1447	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.8;
	1448	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	563.8;
	1448	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.2;
	1448	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.9;
	1448	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.4;
	1448	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	1448	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406;
	1449	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.4;
	1449	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.6;
	1449	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.4;
	1449	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.3;
	1449	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74;
	1449	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.1;
	1450	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	554.6;
	1450	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.4;
	1450	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.2;
	1450	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.3;
	1450	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.4;
	1450	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.2;
	1451	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549.4;
	1451	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.3;
	1451	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.9;
	1451	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.6;
	1451	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.6;
	1451	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.1;
	1452	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547.1;
	1452	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.6;
	1452	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239;
	1452	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.4;
	1452	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	1452	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.9;
	1453	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547.8;
	1453	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.4;
	1453	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.1;
	1453	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.8;
	1453	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	1453	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.2;
	1454	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	553.6;
	1454	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.4;
	1454	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.1;
	1454	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.7;
	1454	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	1454	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.3;
	1455	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549.8;
	1455	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.4;
	1455	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236;
	1455	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.3;
	1455	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	1455	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.4;
	1456	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	555.1;
	1456	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.2;
	1456	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.1;
	1456	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.8;
	1456	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	1456	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.2;
	1457	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	563.4;
	1457	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.6;
	1457	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.9;
	1457	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.7;
	1457	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	1457	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.9;
	1458	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	576.3;
	1458	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.2;
	1458	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.3;
	1458	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.5;
	1458	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	1458	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.8;
	1459	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	587;
	1459	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	200.7;
	1459	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.9;
	1459	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.7;
	1459	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	1459	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422;
	1460	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.7;
	1460	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	199.8;
	1460	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.7;
	1460	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.3;
	1460	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	1460	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.9;
	1461	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586.9;
	1461	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.6;
	1461	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.8;
	1461	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.9;
	1461	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	1461	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.7;
	1462	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	569.7;
	1462	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.1;
	1462	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.4;
	1462	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.1;
	1462	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	1462	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.1;
	1463	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.8;
	1463	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172;
	1463	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.5;
	1463	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221;
	1463	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	1463	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.6;
	1464	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537.1;
	1464	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.2;
	1464	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.2;
	1464	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.5;
	1464	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.1;
	1464	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.7;
	1465	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.8;
	1465	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.3;
	1465	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.8;
	1465	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.6;
	1465	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.2;
	1465	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.3;
	1466	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.2;
	1466	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.7;
	1466	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.3;
	1466	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202;
	1466	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56;
	1466	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372;
	1467	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.5;
	1467	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.3;
	1467	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.6;
	1467	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.1;
	1467	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	1467	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.1;
	1468	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522.4;
	1468	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.2;
	1468	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.4;
	1468	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.2;
	1468	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	1468	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.2;
	1469	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.7;
	1469	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.4;
	1469	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.1;
	1469	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.5;
	1469	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.2;
	1469	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386;
	1470	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566.1;
	1470	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.2;
	1470	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.8;
	1470	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.5;
	1470	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.7;
	1470	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.3;
	1471	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	596.3;
	1471	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.6;
	1471	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.3;
	1471	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.3;
	1471	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58;
	1471	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.7;
	1472	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	617.2;
	1472	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.3;
	1472	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.1;
	1472	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.7;
	1472	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	1472	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	428.2;
	1473	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	613.6;
	1473	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.1;
	1473	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.5;
	1473	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.9;
	1473	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	1473	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	427.8;
	1474	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	603.3;
	1474	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.3;
	1474	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.2;
	1474	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.1;
	1474	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68;
	1474	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	424.1;
	1475	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588;
	1475	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.7;
	1475	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.6;
	1475	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.4;
	1475	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.4;
	1475	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.8;
	1476	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	580;
	1476	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.2;
	1476	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.7;
	1476	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.4;
	1476	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.1;
	1476	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.2;
	1477	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	571.5;
	1477	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183;
	1477	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.1;
	1477	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.8;
	1477	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	1477	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.4;
	1478	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	561.2;
	1478	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.9;
	1478	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.9;
	1478	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.6;
	1478	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	1478	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.9;
	1479	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552.6;
	1479	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.3;
	1479	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.6;
	1479	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.1;
	1479	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69;
	1479	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.4;
	1480	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	546.3;
	1480	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.7;
	1480	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.2;
	1480	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.7;
	1480	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.8;
	1480	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.9;
	1481	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.3;
	1481	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.3;
	1481	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.9;
	1481	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.7;
	1481	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	1481	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.7;
	1482	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	557.4;
	1482	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192;
	1482	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246;
	1482	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.7;
	1482	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	1482	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.4;
	1483	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564.7;
	1483	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.5;
	1483	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.8;
	1483	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.4;
	1483	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	1483	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.3;
	1484	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.8;
	1484	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.7;
	1484	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.1;
	1484	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248;
	1484	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.7;
	1484	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.8;
	1485	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566.7;
	1485	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.6;
	1485	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.7;
	1485	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.6;
	1485	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.2;
	1485	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.8;
	1486	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.1;
	1486	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.6;
	1486	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.2;
	1486	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.5;
	1486	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.9;
	1486	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.4;
	1487	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.4;
	1487	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172;
	1487	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220;
	1487	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.3;
	1487	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.1;
	1487	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.1;
	1488	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.6;
	1488	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169;
	1488	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.2;
	1488	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.8;
	1488	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.2;
	1488	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.4;
	1489	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.2;
	1489	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.5;
	1489	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.2;
	1489	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.4;
	1489	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	1489	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373;
	1490	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.6;
	1490	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.3;
	1490	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.4;
	1490	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.5;
	1490	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	1490	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.1;
	1491	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.2;
	1491	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.4;
	1491	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.9;
	1491	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.2;
	1491	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	1491	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.4;
	1492	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.3;
	1492	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.3;
	1492	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.8;
	1492	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.2;
	1492	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.3;
	1492	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.8;
	1493	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.2;
	1493	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.2;
	1493	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.6;
	1493	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205;
	1493	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.1;
	1493	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.8;
	1494	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526.1;
	1494	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.1;
	1494	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.6;
	1494	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.1;
	1494	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.5;
	1494	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.5;
	1495	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.1;
	1495	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.6;
	1495	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.1;
	1495	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.7;
	1495	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.9;
	1495	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.1;
	1496	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.6;
	1496	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175;
	1496	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.5;
	1496	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215;
	1496	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.8;
	1496	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.5;
	1497	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	546.7;
	1497	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.1;
	1497	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.4;
	1497	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.6;
	1497	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.4;
	1497	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.1;
	1498	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542;
	1498	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.2;
	1498	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.7;
	1498	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223;
	1498	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.5;
	1498	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.5;
	1499	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.3;
	1499	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.5;
	1499	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.9;
	1499	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.2;
	1499	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.3;
	1499	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.5;
	1500	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525.7;
	1500	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.9;
	1500	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.3;
	1500	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.2;
	1500	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.9;
	1500	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.9;
	1501	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518.2;
	1501	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.7;
	1501	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.2;
	1501	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.6;
	1501	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.4;
	1501	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.5;
	1502	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.5;
	1502	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.4;
	1502	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.2;
	1502	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.8;
	1502	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.8;
	1502	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.5;
	1503	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.3;
	1503	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.6;
	1503	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.4;
	1503	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.2;
	1503	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	1503	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.8;
	1504	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.3;
	1504	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.6;
	1504	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.1;
	1504	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.9;
	1504	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.4;
	1504	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.2;
	1505	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.6;
	1505	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.4;
	1505	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.6;
	1505	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.5;
	1505	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72;
	1505	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.8;
	1506	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.1;
	1506	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.5;
	1506	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.9;
	1506	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229;
	1506	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.1;
	1506	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.5;
	1507	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.4;
	1507	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.9;
	1507	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.1;
	1507	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.5;
	1507	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.3;
	1507	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.4;
	1508	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.7;
	1508	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.1;
	1508	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.8;
	1508	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234;
	1508	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.4;
	1508	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.8;
	1509	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.6;
	1509	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178;
	1509	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.6;
	1509	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.5;
	1509	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.6;
	1509	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.6;
	1510	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.8;
	1510	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.9;
	1510	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.9;
	1510	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.7;
	1510	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.3;
	1510	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.1;
	1511	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476;
	1511	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.3;
	1511	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.3;
	1511	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215;
	1511	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.1;
	1511	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.6;
	1512	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.2;
	1512	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.7;
	1512	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211;
	1512	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.3;
	1512	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.9;
	1512	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.6;
	1513	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.9;
	1513	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.8;
	1513	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.2;
	1513	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.6;
	1513	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.9;
	1513	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.4;
	1514	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.7;
	1514	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151;
	1514	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.7;
	1514	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.2;
	1514	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70;
	1514	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.9;
	1515	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.8;
	1515	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.7;
	1515	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.3;
	1515	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.9;
	1515	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.9;
	1515	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.4;
	1516	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.1;
	1516	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.2;
	1516	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.2;
	1516	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.9;
	1516	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70;
	1516	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.5;
	1517	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.7;
	1517	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.2;
	1517	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.9;
	1517	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.5;
	1517	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71;
	1517	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.2;
	1518	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.4;
	1518	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.3;
	1518	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199;
	1518	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.6;
	1518	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.5;
	1518	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.7;
	1519	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.4;
	1519	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.1;
	1519	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.4;
	1519	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.3;
	1519	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.2;
	1519	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.3;
	1520	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.9;
	1520	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.8;
	1520	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.6;
	1520	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.3;
	1520	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.4;
	1520	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.1;
	1521	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.2;
	1521	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.8;
	1521	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.7;
	1521	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.1;
	1521	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.9;
	1521	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.9;
	1522	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.8;
	1522	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.2;
	1522	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208;
	1522	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.9;
	1522	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.8;
	1522	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.5;
	1523	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.8;
	1523	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.3;
	1523	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.9;
	1523	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.4;
	1523	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82;
	1523	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366;
	1524	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.2;
	1524	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.7;
	1524	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.2;
	1524	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.3;
	1524	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.1;
	1524	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.8;
	1525	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.8;
	1525	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.4;
	1525	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.9;
	1525	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.9;
	1525	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.6;
	1525	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.4;
	1526	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.6;
	1526	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.5;
	1526	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.1;
	1526	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.1;
	1526	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.5;
	1526	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.2;
	1527	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.6;
	1527	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.1;
	1527	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.8;
	1527	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.7;
	1527	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.8;
	1527	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.7;
	1528	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.6;
	1528	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.3;
	1528	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.6;
	1528	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.8;
	1528	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.6;
	1528	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368;
	1529	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.3;
	1529	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.8;
	1529	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.8;
	1529	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221;
	1529	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79;
	1529	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.9;
	1530	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.8;
	1530	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.6;
	1530	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.6;
	1530	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234;
	1530	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.6;
	1530	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.5;
	1531	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.8;
	1531	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.5;
	1531	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237;
	1531	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.3;
	1531	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.4;
	1531	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.5;
	1532	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.1;
	1532	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.2;
	1532	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.4;
	1532	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238;
	1532	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79;
	1532	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.9;
	1533	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.1;
	1533	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.6;
	1533	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.4;
	1533	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.7;
	1533	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.5;
	1533	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.6;
	1534	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.7;
	1534	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.4;
	1534	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.9;
	1534	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223;
	1534	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.4;
	1534	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.6;
	1535	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.7;
	1535	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.4;
	1535	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.1;
	1535	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.7;
	1535	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.6;
	1535	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363;
	1536	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.9;
	1536	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.3;
	1536	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.1;
	1536	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.9;
	1536	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.7;
	1536	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.2;
	1537	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.6;
	1537	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.6;
	1537	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.3;
	1537	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.2;
	1537	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.5;
	1537	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.9;
	1538	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.2;
	1538	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.3;
	1538	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.5;
	1538	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.6;
	1538	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.6;
	1538	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.7;
	1539	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.9;
	1539	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.7;
	1539	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.4;
	1539	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.9;
	1539	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.8;
	1539	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.4;
	1540	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.1;
	1540	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.6;
	1540	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.2;
	1540	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199;
	1540	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.6;
	1540	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.8;
	1541	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.6;
	1541	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.5;
	1541	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.6;
	1541	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.4;
	1541	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.3;
	1541	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.8;
	1542	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490;
	1542	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.7;
	1542	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.2;
	1542	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.5;
	1542	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.6;
	1542	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.1;
	1543	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526.8;
	1543	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.3;
	1543	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.9;
	1543	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.3;
	1543	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75;
	1543	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386;
	1544	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552.5;
	1544	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.6;
	1544	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.1;
	1544	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.7;
	1544	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.7;
	1544	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.2;
	1545	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564.2;
	1545	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.2;
	1545	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.6;
	1545	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.5;
	1545	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.3;
	1545	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.7;
	1546	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.9;
	1546	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.7;
	1546	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.8;
	1546	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.6;
	1546	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.2;
	1546	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.7;
	1547	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	555.7;
	1547	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188;
	1547	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.9;
	1547	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.1;
	1547	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.5;
	1547	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.9;
	1548	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547.7;
	1548	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.4;
	1548	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.9;
	1548	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.4;
	1548	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.4;
	1548	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.8;
	1549	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.3;
	1549	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.3;
	1549	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.7;
	1549	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.1;
	1549	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.8;
	1549	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.6;
	1550	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	540.9;
	1550	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.2;
	1550	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.3;
	1550	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.2;
	1550	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.8;
	1550	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397;
	1551	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.9;
	1551	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175;
	1551	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.9;
	1551	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.5;
	1551	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.2;
	1551	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.7;
	1552	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.2;
	1552	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.3;
	1552	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.9;
	1552	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.3;
	1552	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	1552	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.7;
	1553	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.1;
	1553	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181;
	1553	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.7;
	1553	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242;
	1553	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.1;
	1553	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.8;
	1554	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	540.3;
	1554	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190;
	1554	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.6;
	1554	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249;
	1554	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	1554	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.5;
	1555	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	557.4;
	1555	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.6;
	1555	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.1;
	1555	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.6;
	1555	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	1555	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.9;
	1556	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.1;
	1556	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.4;
	1556	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.9;
	1556	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.5;
	1556	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	1556	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.9;
	1557	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	561.5;
	1557	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.6;
	1557	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.3;
	1557	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.7;
	1557	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.5;
	1557	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.1;
	1558	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547.3;
	1558	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.7;
	1558	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.7;
	1558	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.4;
	1558	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.1;
	1558	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.8;
	1559	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	524;
	1559	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.7;
	1559	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.5;
	1559	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.1;
	1559	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	1559	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.5;
	1560	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.4;
	1560	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.8;
	1560	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.6;
	1560	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214;
	1560	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.1;
	1560	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.4;
	1561	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.1;
	1561	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.3;
	1561	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.3;
	1561	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.6;
	1561	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.5;
	1561	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.6;
	1562	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.6;
	1562	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.1;
	1562	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.7;
	1562	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.4;
	1562	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	1562	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.6;
	1563	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.7;
	1563	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.9;
	1563	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.1;
	1563	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.3;
	1563	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68;
	1563	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.9;
	1564	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.5;
	1564	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.3;
	1564	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.7;
	1564	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.8;
	1564	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	1564	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.4;
	1565	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.5;
	1565	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.3;
	1565	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205;
	1565	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.6;
	1565	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.1;
	1565	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.9;
	1566	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.9;
	1566	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.6;
	1566	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.2;
	1566	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.5;
	1566	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	1566	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.9;
	1567	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	574.2;
	1567	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.4;
	1567	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.1;
	1567	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.7;
	1567	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.6;
	1567	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.3;
	1568	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	591.6;
	1568	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.5;
	1568	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.5;
	1568	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.8;
	1568	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	1568	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.8;
	1569	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588.8;
	1569	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.8;
	1569	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.4;
	1569	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.4;
	1569	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	1569	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.1;
	1570	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577.5;
	1570	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.6;
	1570	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.1;
	1570	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242;
	1570	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73;
	1570	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.6;
	1571	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.8;
	1571	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.9;
	1571	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.4;
	1571	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.4;
	1571	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.9;
	1571	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.4;
	1572	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547.8;
	1572	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.1;
	1572	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.7;
	1572	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.8;
	1572	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73;
	1572	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.6;
	1573	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542;
	1573	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.8;
	1573	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.9;
	1573	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.3;
	1573	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72;
	1573	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.7;
	1574	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.2;
	1574	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.5;
	1574	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.9;
	1574	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.1;
	1574	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.9;
	1574	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.1;
	1575	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	534.8;
	1575	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.4;
	1575	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.8;
	1575	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.9;
	1575	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	1575	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.5;
	1576	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525.3;
	1576	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.5;
	1576	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.5;
	1576	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.6;
	1576	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.1;
	1576	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.2;
	1577	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.1;
	1577	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.4;
	1577	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237;
	1577	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.4;
	1577	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	1577	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.4;
	1578	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522.6;
	1578	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.5;
	1578	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246;
	1578	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.5;
	1578	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	1578	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.1;
	1579	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537;
	1579	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.7;
	1579	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.7;
	1579	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.6;
	1579	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.8;
	1579	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.2;
	1580	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.6;
	1580	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.1;
	1580	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.8;
	1580	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.5;
	1580	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.1;
	1580	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.9;
	1581	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.9;
	1581	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.5;
	1581	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.5;
	1581	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243;
	1581	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.2;
	1581	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.3;
	1582	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.4;
	1582	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176;
	1582	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.3;
	1582	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.3;
	1582	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.9;
	1582	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.6;
	1583	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.8;
	1583	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.9;
	1583	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.3;
	1583	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.4;
	1583	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.4;
	1583	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.8;
	1584	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.9;
	1584	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.8;
	1584	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.4;
	1584	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.1;
	1584	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	1584	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.2;
	1585	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.8;
	1585	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.2;
	1585	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.6;
	1585	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.1;
	1585	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.6;
	1585	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.3;
	1586	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.1;
	1586	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.4;
	1586	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.6;
	1586	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.2;
	1586	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.2;
	1586	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349;
	1587	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.6;
	1587	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.3;
	1587	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.4;
	1587	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.1;
	1587	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.1;
	1587	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.1;
	1588	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.1;
	1588	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.2;
	1588	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.5;
	1588	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.1;
	1588	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	1588	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.3;
	1589	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.8;
	1589	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.6;
	1589	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.1;
	1589	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.7;
	1589	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.1;
	1589	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.1;
	1590	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.1;
	1590	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.5;
	1590	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.3;
	1590	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.2;
	1590	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.8;
	1590	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379;
	1591	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.5;
	1591	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.1;
	1591	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.8;
	1591	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.7;
	1591	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73;
	1591	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.5;
	1592	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564.6;
	1592	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.6;
	1592	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.7;
	1592	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.5;
	1592	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.9;
	1592	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.5;
	1593	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	568.1;
	1593	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.1;
	1593	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.4;
	1593	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.1;
	1593	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.6;
	1593	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.3;
	1594	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	554;
	1594	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.4;
	1594	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.4;
	1594	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.5;
	1594	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.3;
	1594	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.8;
	1595	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.2;
	1595	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.3;
	1595	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.4;
	1595	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.2;
	1595	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.9;
	1595	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.6;
	1596	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.5;
	1596	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.8;
	1596	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.2;
	1596	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.7;
	1596	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	1596	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.9;
	1597	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.7;
	1597	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.3;
	1597	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.7;
	1597	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.5;
	1597	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.1;
	1597	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.2;
	1598	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.5;
	1598	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.1;
	1598	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233;
	1598	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.4;
	1598	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	1598	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.7;
	1599	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525;
	1599	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.6;
	1599	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.7;
	1599	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.5;
	1599	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	1599	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.5;
	1600	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.4;
	1600	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.4;
	1600	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.3;
	1600	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.4;
	1600	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	1600	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.5;
	1601	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518.5;
	1601	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.2;
	1601	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.2;
	1601	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.6;
	1601	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	1601	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389;
	1602	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525.2;
	1602	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.6;
	1602	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.4;
	1602	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.4;
	1602	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	1602	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.1;
	1603	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537.7;
	1603	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.5;
	1603	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.3;
	1603	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.6;
	1603	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	1603	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401;
	1604	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549.6;
	1604	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.1;
	1604	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.9;
	1604	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.2;
	1604	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68;
	1604	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.9;
	1605	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.8;
	1605	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.8;
	1605	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238;
	1605	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.8;
	1605	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	1605	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.9;
	1606	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.4;
	1606	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.7;
	1606	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.6;
	1606	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.7;
	1606	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.5;
	1606	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.8;
	1607	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.9;
	1607	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.2;
	1607	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.8;
	1607	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.9;
	1607	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	1607	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.3;
	1608	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.9;
	1608	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.5;
	1608	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.6;
	1608	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.3;
	1608	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	1608	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.4;
	1609	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.1;
	1609	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.3;
	1609	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.4;
	1609	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.8;
	1609	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	1609	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.4;
	1610	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.6;
	1610	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.2;
	1610	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.3;
	1610	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.8;
	1610	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59;
	1610	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.1;
	1611	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.7;
	1611	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146;
	1611	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.6;
	1611	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.9;
	1611	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.9;
	1611	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.6;
	1612	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.2;
	1612	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.7;
	1612	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.3;
	1612	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.9;
	1612	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.4;
	1612	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.2;
	1613	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.6;
	1613	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.8;
	1613	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.7;
	1613	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.4;
	1613	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56;
	1613	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.7;
	1614	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.6;
	1614	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.7;
	1614	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.5;
	1614	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.5;
	1614	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.5;
	1614	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.4;
	1615	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548.6;
	1615	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.4;
	1615	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.2;
	1615	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.5;
	1615	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	1615	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396;
	1616	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.4;
	1616	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.6;
	1616	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.3;
	1616	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.1;
	1616	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	1616	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.5;
	1617	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.4;
	1617	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185;
	1617	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.8;
	1617	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.4;
	1617	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	1617	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.8;
	1618	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562;
	1618	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184;
	1618	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.9;
	1618	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.1;
	1618	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	1618	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.3;
	1619	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	563.1;
	1619	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.2;
	1619	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.7;
	1619	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.3;
	1619	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	1619	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.8;
	1620	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	553.5;
	1620	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.4;
	1620	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.9;
	1620	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.2;
	1620	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	1620	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.1;
	1621	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537;
	1621	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.9;
	1621	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.1;
	1621	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.4;
	1621	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	1621	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.6;
	1622	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522.2;
	1622	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.8;
	1622	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.9;
	1622	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.2;
	1622	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	1622	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.6;
	1623	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513;
	1623	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.3;
	1623	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.9;
	1623	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.9;
	1623	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	1623	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.9;
	1624	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.6;
	1624	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.3;
	1624	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.5;
	1624	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.7;
	1624	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	1624	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.1;
	1625	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506;
	1625	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.4;
	1625	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239;
	1625	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.9;
	1625	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	1625	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.5;
	1626	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.8;
	1626	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.3;
	1626	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.1;
	1626	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.7;
	1626	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	1626	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.5;
	1627	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537.4;
	1627	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194;
	1627	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.6;
	1627	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.4;
	1627	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	1627	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402;
	1628	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.6;
	1628	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.8;
	1628	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.7;
	1628	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.1;
	1628	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	1628	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.6;
	1629	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549;
	1629	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.7;
	1629	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.9;
	1629	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.1;
	1629	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	1629	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.6;
	1630	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.7;
	1630	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.9;
	1630	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.2;
	1630	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.1;
	1630	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	1630	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.2;
	1631	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.6;
	1631	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.8;
	1631	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.4;
	1631	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.9;
	1631	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	1631	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.2;
	1632	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.3;
	1632	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.7;
	1632	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.2;
	1632	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.7;
	1632	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.5;
	1632	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.8;
	1633	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.2;
	1633	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.1;
	1633	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.6;
	1633	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.6;
	1633	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.3;
	1633	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.3;
	1634	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.1;
	1634	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.6;
	1634	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.9;
	1634	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.9;
	1634	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.8;
	1634	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.2;
	1635	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.4;
	1635	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150;
	1635	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.9;
	1635	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.7;
	1635	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.8;
	1635	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.5;
	1636	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.8;
	1636	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.5;
	1636	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.4;
	1636	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.1;
	1636	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.4;
	1636	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360;
	1637	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.9;
	1637	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.5;
	1637	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.9;
	1637	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.8;
	1637	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.7;
	1637	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.4;
	1638	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.8;
	1638	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.3;
	1638	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.7;
	1638	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.3;
	1638	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.8;
	1638	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.2;
	1639	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559.5;
	1639	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.7;
	1639	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.6;
	1639	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.5;
	1639	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.1;
	1639	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.9;
	1640	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	571.1;
	1640	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.4;
	1640	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.8;
	1640	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.2;
	1640	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	1640	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.5;
	1641	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.9;
	1641	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190;
	1641	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.4;
	1641	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.7;
	1641	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	1641	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.2;
	1642	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	546.6;
	1642	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.4;
	1642	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241;
	1642	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.2;
	1642	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	1642	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.4;
	1643	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.2;
	1643	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.9;
	1643	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.4;
	1643	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243;
	1643	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	1643	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.1;
	1644	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522.4;
	1644	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.9;
	1644	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.9;
	1644	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.8;
	1644	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	1644	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391;
	1645	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.2;
	1645	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.7;
	1645	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.2;
	1645	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240;
	1645	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	1645	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.1;
	1646	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.4;
	1646	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.7;
	1646	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.1;
	1646	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.1;
	1646	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	1646	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.2;
	1647	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.7;
	1647	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.3;
	1647	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.8;
	1647	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.2;
	1647	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	1647	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379;
	1648	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.6;
	1648	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173;
	1648	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.4;
	1648	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.8;
	1648	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	1648	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.9;
	1649	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.9;
	1649	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.7;
	1649	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.1;
	1649	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.2;
	1649	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	1649	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.8;
	1650	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.2;
	1650	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.4;
	1650	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.8;
	1650	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.6;
	1650	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	1650	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.4;
	1651	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.5;
	1651	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.6;
	1651	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.9;
	1651	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.2;
	1651	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	1651	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.4;
	1652	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.3;
	1652	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.6;
	1652	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.7;
	1652	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.1;
	1652	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	1652	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.9;
	1653	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.9;
	1653	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.1;
	1653	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.8;
	1653	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238;
	1653	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	1653	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.2;
	1654	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.2;
	1654	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.1;
	1654	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.9;
	1654	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.6;
	1654	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	1654	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379;
	1655	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.5;
	1655	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.3;
	1655	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.5;
	1655	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.9;
	1655	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.2;
	1655	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.6;
	1656	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.6;
	1656	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.1;
	1656	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.1;
	1656	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.6;
	1656	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57;
	1656	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.5;
	1657	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.7;
	1657	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.3;
	1657	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.3;
	1657	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204;
	1657	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55;
	1657	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352;
	1658	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.1;
	1658	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.3;
	1658	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.2;
	1658	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.7;
	1658	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.5;
	1658	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.1;
	1659	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.6;
	1659	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.2;
	1659	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.7;
	1659	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.3;
	1659	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.9;
	1659	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.8;
	1660	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.9;
	1660	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.2;
	1660	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.2;
	1660	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.2;
	1660	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.1;
	1660	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344;
	1661	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464;
	1661	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.9;
	1661	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.8;
	1661	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.7;
	1661	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.5;
	1661	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.4;
	1662	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.4;
	1662	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.5;
	1662	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.3;
	1662	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.5;
	1662	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.4;
	1662	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.4;
	1663	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.8;
	1663	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170;
	1663	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.8;
	1663	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.4;
	1663	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.5;
	1663	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.9;
	1664	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.8;
	1664	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172;
	1664	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.2;
	1664	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.5;
	1664	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.6;
	1664	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368;
	1665	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.4;
	1665	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.2;
	1665	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.1;
	1665	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.8;
	1665	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.9;
	1665	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.1;
	1666	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.6;
	1666	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.8;
	1666	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.6;
	1666	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.5;
	1666	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.7;
	1666	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.1;
	1667	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478;
	1667	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.7;
	1667	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213;
	1667	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.9;
	1667	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	1667	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.8;
	1668	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.2;
	1668	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.9;
	1668	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.2;
	1668	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215;
	1668	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	1668	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.7;
	1669	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.5;
	1669	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.9;
	1669	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.4;
	1669	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.3;
	1669	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	1669	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.7;
	1670	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	444.8;
	1670	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.4;
	1670	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.3;
	1670	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.1;
	1670	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	1670	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.5;
	1671	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.4;
	1671	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.1;
	1671	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.7;
	1671	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.7;
	1671	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	1671	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.1;
	1672	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429;
	1672	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.6;
	1672	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.1;
	1672	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.6;
	1672	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	1672	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.9;
	1673	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.8;
	1673	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.5;
	1673	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.4;
	1673	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.1;
	1673	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	1673	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.9;
	1674	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.9;
	1674	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.4;
	1674	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.5;
	1674	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.8;
	1674	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	1674	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.5;
	1675	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.1;
	1675	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.1;
	1675	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.8;
	1675	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.6;
	1675	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	1675	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.2;
	1676	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.3;
	1676	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.3;
	1676	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.5;
	1676	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.9;
	1676	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	1676	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.6;
	1677	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.9;
	1677	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.6;
	1677	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.3;
	1677	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.7;
	1677	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	1677	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.9;
	1678	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.7;
	1678	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.6;
	1678	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.7;
	1678	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.5;
	1678	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	1678	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.5;
	1679	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.4;
	1679	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.6;
	1679	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.8;
	1679	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.2;
	1679	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	1679	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.8;
	1680	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417.2;
	1680	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.4;
	1680	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.5;
	1680	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.4;
	1680	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.4;
	1680	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.3;
	1681	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	406;
	1681	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.9;
	1681	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.6;
	1681	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.8;
	1681	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.4;
	1681	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.3;
	1682	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391.9;
	1682	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.4;
	1682	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.2;
	1682	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.9;
	1682	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.8;
	1682	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.4;
	1683	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	395.2;
	1683	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.6;
	1683	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.5;
	1683	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.7;
	1683	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.1;
	1683	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.2;
	1684	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.3;
	1684	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.9;
	1684	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192;
	1684	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194;
	1684	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.6;
	1684	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.7;
	1685	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411;
	1685	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.7;
	1685	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.4;
	1685	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.3;
	1685	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.5;
	1685	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.6;
	1686	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.6;
	1686	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.7;
	1686	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.9;
	1686	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.7;
	1686	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.2;
	1686	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.1;
	1687	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.3;
	1687	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150;
	1687	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.9;
	1687	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.9;
	1687	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	1687	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.3;
	1688	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	427.2;
	1688	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.6;
	1688	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.7;
	1688	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.4;
	1688	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.6;
	1688	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.7;
	1689	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.9;
	1689	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.7;
	1689	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.6;
	1689	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.1;
	1689	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.7;
	1689	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.3;
	1690	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.6;
	1690	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.4;
	1690	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.1;
	1690	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.1;
	1690	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	1690	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345;
	1691	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441;
	1691	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.4;
	1691	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.5;
	1691	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.3;
	1691	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	1691	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.3;
	1692	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.4;
	1692	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.3;
	1692	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.2;
	1692	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.3;
	1692	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	1692	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.3;
	1693	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.3;
	1693	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159;
	1693	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.1;
	1693	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.5;
	1693	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	1693	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.3;
	1694	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.1;
	1694	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154;
	1694	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.7;
	1694	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203;
	1694	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	1694	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.7;
	1695	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.4;
	1695	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.9;
	1695	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.7;
	1695	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205;
	1695	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	1695	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.2;
	1696	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.4;
	1696	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.3;
	1696	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.9;
	1696	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.4;
	1696	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	1696	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.9;
	1697	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.3;
	1697	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.2;
	1697	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.7;
	1697	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.6;
	1697	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	1697	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.5;
	1698	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.6;
	1698	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.1;
	1698	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.2;
	1698	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.6;
	1698	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	1698	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.4;
	1699	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428.3;
	1699	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.2;
	1699	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.7;
	1699	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.7;
	1699	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	1699	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.6;
	1700	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.7;
	1700	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.5;
	1700	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.7;
	1700	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232;
	1700	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	1700	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.5;
	1701	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.8;
	1701	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.3;
	1701	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.6;
	1701	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.5;
	1701	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	1701	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.2;
	1702	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.4;
	1702	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.1;
	1702	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.5;
	1702	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.6;
	1702	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.2;
	1702	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.3;
	1703	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434;
	1703	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.8;
	1703	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.1;
	1703	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.2;
	1703	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.1;
	1703	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346;
	1704	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.9;
	1704	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.7;
	1704	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.3;
	1704	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.3;
	1704	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.7;
	1704	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.5;
	1705	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	402.9;
	1705	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.6;
	1705	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.9;
	1705	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.1;
	1705	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.1;
	1705	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.3;
	1706	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	387.6;
	1706	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135;
	1706	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.3;
	1706	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.4;
	1706	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.3;
	1706	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320;
	1707	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	387.2;
	1707	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135;
	1707	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.2;
	1707	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.6;
	1707	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.3;
	1707	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.3;
	1708	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392.2;
	1708	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135;
	1708	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.7;
	1708	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.6;
	1708	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.7;
	1708	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.8;
	1709	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.8;
	1709	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142;
	1709	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.3;
	1709	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.5;
	1709	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.8;
	1709	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.4;
	1710	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.6;
	1710	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.8;
	1710	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201;
	1710	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.3;
	1710	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.2;
	1710	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.8;
	1711	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.7;
	1711	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.9;
	1711	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.1;
	1711	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.2;
	1711	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.7;
	1711	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.9;
	1712	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.6;
	1712	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.2;
	1712	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.8;
	1712	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.5;
	1712	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.7;
	1712	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.1;
	1713	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.9;
	1713	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.8;
	1713	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.9;
	1713	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.1;
	1713	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.8;
	1713	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.3;
	1714	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.1;
	1714	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.1;
	1714	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.6;
	1714	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.1;
	1714	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.2;
	1714	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.9;
	1715	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.1;
	1715	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.8;
	1715	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.4;
	1715	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.5;
	1715	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	1715	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.6;
	1716	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.6;
	1716	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.3;
	1716	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.5;
	1716	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.4;
	1716	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	1716	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.2;
	1717	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.9;
	1717	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.8;
	1717	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227;
	1717	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233;
	1717	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	1717	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.9;
	1718	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.1;
	1718	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157;
	1718	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.4;
	1718	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232;
	1718	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	1718	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.7;
	1719	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.7;
	1719	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.3;
	1719	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.7;
	1719	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.1;
	1719	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	1719	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.4;
	1720	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.9;
	1720	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.7;
	1720	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.4;
	1720	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.1;
	1720	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.7;
	1720	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.8;
	1721	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.9;
	1721	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.6;
	1721	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.5;
	1721	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.8;
	1721	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	1721	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.3;
	1722	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.5;
	1722	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.7;
	1722	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.2;
	1722	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.8;
	1722	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	1722	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363;
	1723	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.6;
	1723	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.4;
	1723	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233;
	1723	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.7;
	1723	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	1723	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.7;
	1724	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.3;
	1724	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.7;
	1724	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.2;
	1724	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.4;
	1724	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	1724	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.6;
	1725	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.3;
	1725	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.7;
	1725	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.1;
	1725	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.1;
	1725	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	1725	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.4;
	1726	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.7;
	1726	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.8;
	1726	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223;
	1726	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.2;
	1726	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	1726	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.6;
	1727	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.2;
	1727	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.9;
	1727	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.9;
	1727	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.2;
	1727	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	1727	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.5;
	1728	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.8;
	1728	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.2;
	1728	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202;
	1728	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.9;
	1728	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.8;
	1728	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.5;
	1729	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.5;
	1729	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.3;
	1729	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.6;
	1729	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198;
	1729	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.4;
	1729	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.8;
	1730	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.6;
	1730	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.8;
	1730	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.7;
	1730	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.2;
	1730	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55;
	1730	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.8;
	1731	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392.9;
	1731	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.9;
	1731	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.4;
	1731	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.1;
	1731	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.7;
	1731	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.7;
	1732	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.4;
	1732	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.7;
	1732	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.7;
	1732	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.5;
	1732	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.8;
	1732	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.1;
	1733	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.3;
	1733	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.3;
	1733	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192;
	1733	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.5;
	1733	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.2;
	1733	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.6;
	1734	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.9;
	1734	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.4;
	1734	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.7;
	1734	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.8;
	1734	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.8;
	1734	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.3;
	1735	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.4;
	1735	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.6;
	1735	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.5;
	1735	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224;
	1735	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54;
	1735	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.6;
	1736	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508;
	1736	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.7;
	1736	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.3;
	1736	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.6;
	1736	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.1;
	1736	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.5;
	1737	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518.3;
	1737	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.9;
	1737	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.7;
	1737	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.1;
	1737	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.7;
	1737	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386;
	1738	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.5;
	1738	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.8;
	1738	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.3;
	1738	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.9;
	1738	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	1738	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.4;
	1739	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.7;
	1739	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162;
	1739	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.4;
	1739	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234;
	1739	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	1739	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.9;
	1740	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.5;
	1740	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.9;
	1740	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.1;
	1740	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234;
	1740	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.3;
	1740	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.3;
	1741	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.3;
	1741	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.7;
	1741	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.3;
	1741	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.1;
	1741	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61;
	1741	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369;
	1742	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.4;
	1742	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.3;
	1742	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.7;
	1742	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.9;
	1742	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	1742	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.9;
	1743	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.7;
	1743	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.7;
	1743	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228;
	1743	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.6;
	1743	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	1743	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.5;
	1744	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.8;
	1744	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.9;
	1744	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.7;
	1744	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232;
	1744	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	1744	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.5;
	1745	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.1;
	1745	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.2;
	1745	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.6;
	1745	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.4;
	1745	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	1745	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.1;
	1746	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.2;
	1746	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.8;
	1746	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228;
	1746	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233;
	1746	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	1746	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.3;
	1747	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.1;
	1747	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.7;
	1747	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.8;
	1747	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.3;
	1747	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	1747	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.9;
	1748	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.1;
	1748	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.1;
	1748	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.8;
	1748	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.8;
	1748	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	1748	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.5;
	1749	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.9;
	1749	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162;
	1749	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.7;
	1749	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.4;
	1749	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	1749	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.6;
	1750	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.3;
	1750	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.7;
	1750	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.2;
	1750	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.5;
	1750	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	1750	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.4;
	1751	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.4;
	1751	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.2;
	1751	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.5;
	1751	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.5;
	1751	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	1751	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.4;
	1752	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.2;
	1752	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.4;
	1752	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201;
	1752	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.7;
	1752	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.1;
	1752	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.8;
	1753	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.1;
	1753	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125;
	1753	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191;
	1753	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.4;
	1753	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58;
	1753	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.5;
	1754	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394.7;
	1754	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.1;
	1754	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.1;
	1754	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.3;
	1754	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.1;
	1754	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.4;
	1755	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	402;
	1755	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.4;
	1755	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.1;
	1755	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.4;
	1755	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.7;
	1755	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323;
	1756	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.5;
	1756	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.2;
	1756	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.7;
	1756	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191;
	1756	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	1756	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.2;
	1757	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	430.8;
	1757	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.5;
	1757	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.4;
	1757	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.2;
	1757	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	1757	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.2;
	1758	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.1;
	1758	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.2;
	1758	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.7;
	1758	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.2;
	1758	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55;
	1758	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.4;
	1759	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500;
	1759	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.4;
	1759	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.2;
	1759	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.8;
	1759	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.2;
	1759	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.3;
	1760	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.8;
	1760	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.3;
	1760	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.2;
	1760	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.3;
	1760	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	1760	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.3;
	1761	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.1;
	1761	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161;
	1761	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.9;
	1761	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.7;
	1761	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	1761	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.8;
	1762	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.7;
	1762	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.3;
	1762	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.2;
	1762	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.8;
	1762	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.5;
	1762	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.8;
	1763	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.9;
	1763	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.2;
	1763	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.8;
	1763	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.4;
	1763	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.8;
	1763	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.6;
	1764	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.5;
	1764	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.2;
	1764	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.9;
	1764	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.2;
	1764	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	1764	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.2;
	1765	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.8;
	1765	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.2;
	1765	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.9;
	1765	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.4;
	1765	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	1765	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.1;
	1766	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.1;
	1766	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.5;
	1766	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.9;
	1766	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.9;
	1766	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	1766	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.2;
	1767	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498;
	1767	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.6;
	1767	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.4;
	1767	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.1;
	1767	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	1767	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.4;
	1768	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.3;
	1768	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152;
	1768	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225;
	1768	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.6;
	1768	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	1768	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.9;
	1769	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.2;
	1769	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.3;
	1769	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.7;
	1769	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.5;
	1769	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	1769	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.5;
	1770	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.2;
	1770	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158;
	1770	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.9;
	1770	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.6;
	1770	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	1770	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374;
	1771	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.2;
	1771	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162;
	1771	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.4;
	1771	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.1;
	1771	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	1771	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.1;
	1772	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.7;
	1772	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.1;
	1772	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.3;
	1772	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.1;
	1772	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	1772	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.8;
	1773	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.1;
	1773	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.2;
	1773	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.1;
	1773	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.4;
	1773	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	1773	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.9;
	1774	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.7;
	1774	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.5;
	1774	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.3;
	1774	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.8;
	1774	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	1774	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.1;
	1775	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.2;
	1775	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.2;
	1775	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.8;
	1775	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.3;
	1775	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	1775	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361;
	1776	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.9;
	1776	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140;
	1776	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.5;
	1776	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.6;
	1776	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.4;
	1776	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.1;
	1777	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.4;
	1777	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.1;
	1777	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.6;
	1777	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.3;
	1777	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.1;
	1777	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.3;
	1778	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.2;
	1778	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.2;
	1778	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.9;
	1778	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.5;
	1778	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.3;
	1778	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.4;
	1779	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447;
	1779	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.4;
	1779	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.1;
	1779	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.7;
	1779	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.3;
	1779	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.9;
	1780	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455;
	1780	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.4;
	1780	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.8;
	1780	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.5;
	1780	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.7;
	1780	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.9;
	1781	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.8;
	1781	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.1;
	1781	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.5;
	1781	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.4;
	1781	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.4;
	1781	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.4;
	1782	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.5;
	1782	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.8;
	1782	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.8;
	1782	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.2;
	1782	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.1;
	1782	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369;
	1783	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.8;
	1783	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.1;
	1783	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.7;
	1783	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.3;
	1783	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.2;
	1783	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.8;
	1784	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.4;
	1784	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.5;
	1784	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.3;
	1784	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.5;
	1784	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	1784	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.6;
	1785	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	553.9;
	1785	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173;
	1785	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.9;
	1785	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.7;
	1785	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	1785	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.5;
	1786	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547.7;
	1786	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.5;
	1786	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.7;
	1786	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.6;
	1786	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.5;
	1786	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399;
	1787	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.9;
	1787	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.2;
	1787	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.8;
	1787	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.4;
	1787	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	1787	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.7;
	1788	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.8;
	1788	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.5;
	1788	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.1;
	1788	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.5;
	1788	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	1788	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.3;
	1789	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.9;
	1789	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.3;
	1789	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.1;
	1789	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236;
	1789	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.9;
	1789	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.6;
	1790	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.1;
	1790	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.8;
	1790	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.3;
	1790	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.2;
	1790	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.3;
	1790	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.3;
	1791	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.7;
	1791	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.7;
	1791	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.3;
	1791	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.8;
	1791	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	1791	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.4;
	1792	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.5;
	1792	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.1;
	1792	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.5;
	1792	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232;
	1792	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.2;
	1792	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.2;
	1793	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.9;
	1793	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.4;
	1793	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.7;
	1793	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.7;
	1793	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	1793	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373;
	1794	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.7;
	1794	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.1;
	1794	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.8;
	1794	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.1;
	1794	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	1794	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.4;
	1795	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.5;
	1795	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.7;
	1795	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.8;
	1795	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.6;
	1795	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	1795	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.4;
	1796	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516;
	1796	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.6;
	1796	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242;
	1796	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245;
	1796	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	1796	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.5;
	1797	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.5;
	1797	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.1;
	1797	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238;
	1797	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.4;
	1797	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	1797	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.1;
	1798	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.5;
	1798	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165;
	1798	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.9;
	1798	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.9;
	1798	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	1798	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.6;
	1799	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472;
	1799	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.9;
	1799	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214;
	1799	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.9;
	1799	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	1799	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.5;
	1800	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.1;
	1800	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.3;
	1800	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.4;
	1800	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.2;
	1800	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	1800	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351;
	1801	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.7;
	1801	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.7;
	1801	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.4;
	1801	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.6;
	1801	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.8;
	1801	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.4;
	1802	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	426.6;
	1802	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.1;
	1802	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.4;
	1802	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.7;
	1802	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.6;
	1802	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.5;
	1803	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.1;
	1803	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.6;
	1803	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.2;
	1803	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.3;
	1803	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.4;
	1803	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.9;
	1804	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431;
	1804	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138;
	1804	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.8;
	1804	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.1;
	1804	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.5;
	1804	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.9;
	1805	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.3;
	1805	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.8;
	1805	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.5;
	1805	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.1;
	1805	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.5;
	1805	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.7;
	1806	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.8;
	1806	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.9;
	1806	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209;
	1806	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.3;
	1806	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.8;
	1806	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.2;
	1807	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506;
	1807	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.4;
	1807	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.2;
	1807	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.7;
	1807	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	1807	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.4;
	1808	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.7;
	1808	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.6;
	1808	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232;
	1808	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.9;
	1808	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	1808	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.5;
	1809	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548;
	1809	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.5;
	1809	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.1;
	1809	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.2;
	1809	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.1;
	1809	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.6;
	1810	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549;
	1810	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174;
	1810	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.5;
	1810	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.6;
	1810	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.5;
	1810	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.8;
	1811	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586.5;
	1811	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.7;
	1811	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.4;
	1811	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237;
	1811	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.7;
	1811	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.5;
	1812	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	575;
	1812	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.3;
	1812	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.5;
	1812	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.9;
	1812	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	1812	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409;
	1813	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.5;
	1813	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.4;
	1813	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.3;
	1813	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234;
	1813	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	1813	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403;
	1814	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549.6;
	1814	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.6;
	1814	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228;
	1814	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.8;
	1814	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	1814	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.9;
	1815	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548.4;
	1815	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.2;
	1815	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.7;
	1815	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.7;
	1815	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	1815	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.8;
	1816	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537.5;
	1816	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.5;
	1816	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.8;
	1816	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.8;
	1816	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	1816	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.1;
	1817	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.1;
	1817	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.9;
	1817	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.2;
	1817	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.7;
	1817	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	1817	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.2;
	1818	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531;
	1818	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.5;
	1818	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.7;
	1818	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.3;
	1818	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	1818	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.6;
	1819	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.9;
	1819	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.4;
	1819	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.3;
	1819	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235;
	1819	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	1819	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.4;
	1820	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.6;
	1820	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.4;
	1820	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.6;
	1820	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.8;
	1820	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	1820	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.8;
	1821	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547.6;
	1821	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161;
	1821	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.2;
	1821	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.6;
	1821	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	1821	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.9;
	1822	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.4;
	1822	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.2;
	1822	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.2;
	1822	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.1;
	1822	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	1822	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.5;
	1823	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503;
	1823	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145;
	1823	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.9;
	1823	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.4;
	1823	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	1823	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373;
	1824	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.8;
	1824	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.4;
	1824	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.4;
	1824	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.6;
	1824	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	1824	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.9;
	1825	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.9;
	1825	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.6;
	1825	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.4;
	1825	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.8;
	1825	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.2;
	1825	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.4;
	1826	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.8;
	1826	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.5;
	1826	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189;
	1826	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.5;
	1826	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.1;
	1826	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.8;
	1827	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.3;
	1827	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.8;
	1827	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.7;
	1827	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.1;
	1827	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.1;
	1827	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344;
	1828	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.4;
	1828	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.6;
	1828	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.6;
	1828	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.3;
	1828	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.3;
	1828	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.5;
	1829	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.4;
	1829	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.2;
	1829	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.9;
	1829	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.4;
	1829	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.8;
	1829	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.6;
	1830	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.5;
	1830	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.9;
	1830	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.7;
	1830	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.9;
	1830	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.8;
	1830	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.2;
	1831	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496;
	1831	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.3;
	1831	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202;
	1831	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.1;
	1831	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.2;
	1831	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.9;
	1832	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.3;
	1832	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.4;
	1832	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201;
	1832	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.7;
	1832	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.7;
	1832	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.9;
	1833	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518.3;
	1833	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.2;
	1833	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.4;
	1833	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.9;
	1833	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	1833	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378;
	1834	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.9;
	1834	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.1;
	1834	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.8;
	1834	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.8;
	1834	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	1834	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.8;
	1835	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.2;
	1835	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.9;
	1835	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.9;
	1835	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.8;
	1835	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	1835	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.5;
	1836	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.4;
	1836	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.6;
	1836	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.8;
	1836	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.4;
	1836	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	1836	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.6;
	1837	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522.6;
	1837	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.7;
	1837	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.5;
	1837	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.4;
	1837	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	1837	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.6;
	1838	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.1;
	1838	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.4;
	1838	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.5;
	1838	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.2;
	1838	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	1838	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.6;
	1839	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.4;
	1839	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.9;
	1839	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.9;
	1839	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.2;
	1839	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	1839	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.2;
	1840	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.5;
	1840	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.4;
	1840	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.6;
	1840	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.9;
	1840	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	1840	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.3;
	1841	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.9;
	1841	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.7;
	1841	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.1;
	1841	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.9;
	1841	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	1841	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366;
	1842	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.6;
	1842	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.4;
	1842	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.9;
	1842	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.6;
	1842	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	1842	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.6;
	1843	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.6;
	1843	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.7;
	1843	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.2;
	1843	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.8;
	1843	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	1843	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.1;
	1844	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.4;
	1844	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.9;
	1844	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.1;
	1844	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.6;
	1844	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	1844	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.6;
	1845	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.1;
	1845	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.9;
	1845	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.8;
	1845	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.4;
	1845	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	1845	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.6;
	1846	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.7;
	1846	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.2;
	1846	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.5;
	1846	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.2;
	1846	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	1846	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.3;
	1847	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.8;
	1847	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.5;
	1847	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205;
	1847	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.4;
	1847	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	1847	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.3;
	1848	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.7;
	1848	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.5;
	1848	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.7;
	1848	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207;
	1848	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.5;
	1848	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.9;
	1849	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.7;
	1849	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.1;
	1849	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.3;
	1849	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.6;
	1849	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56;
	1849	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.1;
	1850	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423;
	1850	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.4;
	1850	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.4;
	1850	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190;
	1850	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.6;
	1850	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.9;
	1851	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.7;
	1851	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.6;
	1851	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.1;
	1851	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.5;
	1851	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.7;
	1851	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332;
	1852	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.7;
	1852	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.1;
	1852	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186;
	1852	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.5;
	1852	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.7;
	1852	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335;
	1853	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.2;
	1853	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.1;
	1853	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.4;
	1853	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.1;
	1853	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.1;
	1853	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.2;
	1854	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.6;
	1854	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.6;
	1854	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.1;
	1854	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.9;
	1854	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.5;
	1854	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.1;
	1855	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469;
	1855	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.5;
	1855	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.3;
	1855	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.2;
	1855	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	1855	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.3;
	1856	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.7;
	1856	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.8;
	1856	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.6;
	1856	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.3;
	1856	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	1856	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.8;
	1857	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485;
	1857	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.4;
	1857	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.6;
	1857	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.8;
	1857	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	1857	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.7;
	1858	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.3;
	1858	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.3;
	1858	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.8;
	1858	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.8;
	1858	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	1858	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.9;
	1859	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.8;
	1859	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.7;
	1859	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.5;
	1859	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.1;
	1859	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.1;
	1859	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.4;
	1860	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.1;
	1860	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.9;
	1860	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.9;
	1860	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.2;
	1860	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	1860	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.9;
	1861	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.2;
	1861	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.8;
	1861	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.4;
	1861	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.9;
	1861	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	1861	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371;
	1862	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.7;
	1862	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.3;
	1862	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.5;
	1862	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.7;
	1862	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	1862	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371;
	1863	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.4;
	1863	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.1;
	1863	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.5;
	1863	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.8;
	1863	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	1863	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.6;
	1864	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.5;
	1864	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.8;
	1864	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.8;
	1864	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.7;
	1864	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	1864	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.6;
	1865	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.8;
	1865	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.6;
	1865	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.9;
	1865	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.6;
	1865	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.4;
	1865	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.3;
	1866	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.1;
	1866	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157;
	1866	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208;
	1866	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.8;
	1866	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	1866	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.3;
	1867	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.7;
	1867	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.3;
	1867	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.6;
	1867	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.9;
	1867	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	1867	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.5;
	1868	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.2;
	1868	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.2;
	1868	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.6;
	1868	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.8;
	1868	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	1868	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.4;
	1869	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.2;
	1869	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.1;
	1869	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226;
	1869	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.1;
	1869	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	1869	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.7;
	1870	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526.2;
	1870	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159;
	1870	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.4;
	1870	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.3;
	1870	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	1870	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.4;
	1871	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.4;
	1871	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.8;
	1871	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205;
	1871	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.7;
	1871	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60;
	1871	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.1;
	1872	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.3;
	1872	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.3;
	1872	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.1;
	1872	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.6;
	1872	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.8;
	1872	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.8;
	1873	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.2;
	1873	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.8;
	1873	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.8;
	1873	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.7;
	1873	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.9;
	1873	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.8;
	1874	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.8;
	1874	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.5;
	1874	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.8;
	1874	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.3;
	1874	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.9;
	1874	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.4;
	1875	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.7;
	1875	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.4;
	1875	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.4;
	1875	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.6;
	1875	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.1;
	1875	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.9;
	1876	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.2;
	1876	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.4;
	1876	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.7;
	1876	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.9;
	1876	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.7;
	1876	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.4;
	1877	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.6;
	1877	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.6;
	1877	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.2;
	1877	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194;
	1877	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.3;
	1877	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.7;
	1878	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.7;
	1878	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.7;
	1878	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.4;
	1878	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.5;
	1878	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.5;
	1878	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.5;
	1879	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.8;
	1879	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.9;
	1879	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.2;
	1879	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.7;
	1879	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59;
	1879	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.5;
	1880	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.4;
	1880	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182;
	1880	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.2;
	1880	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.4;
	1880	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	1880	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.8;
	1881	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548;
	1881	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179;
	1881	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236;
	1881	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.4;
	1881	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	1881	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.9;
	1882	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.1;
	1882	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.1;
	1882	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.8;
	1882	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.2;
	1882	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	1882	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.4;
	1883	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.5;
	1883	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.6;
	1883	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.4;
	1883	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.4;
	1883	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	1883	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.6;
	1884	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.8;
	1884	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.2;
	1884	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.7;
	1884	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.6;
	1884	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	1884	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.5;
	1885	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.2;
	1885	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.9;
	1885	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.9;
	1885	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.6;
	1885	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.7;
	1885	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.9;
	1886	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.9;
	1886	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.6;
	1886	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.8;
	1886	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.2;
	1886	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.5;
	1886	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.2;
	1887	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506;
	1887	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.5;
	1887	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.3;
	1887	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.6;
	1887	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	1887	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.3;
	1888	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.1;
	1888	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.8;
	1888	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231;
	1888	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.4;
	1888	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	1888	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.6;
	1889	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.8;
	1889	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.2;
	1889	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.7;
	1889	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.9;
	1889	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.1;
	1889	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.3;
	1890	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.9;
	1890	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.6;
	1890	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.1;
	1890	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.3;
	1890	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.5;
	1890	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.3;
	1891	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.7;
	1891	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.4;
	1891	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239;
	1891	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.7;
	1891	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	1891	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.1;
	1892	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.3;
	1892	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.6;
	1892	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.7;
	1892	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245;
	1892	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.8;
	1892	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.7;
	1893	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.9;
	1893	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.4;
	1893	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.3;
	1893	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240;
	1893	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	1893	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.1;
	1894	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.3;
	1894	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.6;
	1894	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226;
	1894	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.3;
	1894	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.5;
	1894	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.3;
	1895	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.5;
	1895	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.1;
	1895	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.2;
	1895	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.6;
	1895	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.3;
	1895	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.6;
	1896	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.9;
	1896	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145;
	1896	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.4;
	1896	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.3;
	1896	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.9;
	1896	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.5;
	1897	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.5;
	1897	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138;
	1897	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.5;
	1897	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.3;
	1897	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	1897	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.9;
	1898	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	402.1;
	1898	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.3;
	1898	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.3;
	1898	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.6;
	1898	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	1898	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.9;
	1899	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	402.5;
	1899	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.4;
	1899	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190;
	1899	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.6;
	1899	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	1899	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.9;
	1900	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.8;
	1900	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.6;
	1900	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.6;
	1900	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.2;
	1900	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	1900	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.4;
	1901	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.2;
	1901	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139;
	1901	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195;
	1901	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.1;
	1901	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	1901	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.3;
	1902	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.6;
	1902	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.1;
	1902	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.9;
	1902	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.6;
	1902	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	1902	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.5;
	1903	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.6;
	1903	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.8;
	1903	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.8;
	1903	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.6;
	1903	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	1903	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.2;
	1904	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.2;
	1904	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.3;
	1904	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.4;
	1904	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235;
	1904	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	1904	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.6;
	1905	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.1;
	1905	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.6;
	1905	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.1;
	1905	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.4;
	1905	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.1;
	1905	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.3;
	1906	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	524.8;
	1906	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.8;
	1906	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.6;
	1906	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.1;
	1906	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.1;
	1906	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.2;
	1907	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.1;
	1907	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.1;
	1907	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.8;
	1907	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.3;
	1907	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.3;
	1907	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.6;
	1908	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.8;
	1908	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165;
	1908	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.2;
	1908	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.6;
	1908	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	1908	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.5;
	1909	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.6;
	1909	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.2;
	1909	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.5;
	1909	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.8;
	1909	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.4;
	1909	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387;
	1910	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.6;
	1910	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.1;
	1910	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.3;
	1910	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.9;
	1910	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.2;
	1910	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.9;
	1911	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.5;
	1911	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.7;
	1911	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.6;
	1911	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.9;
	1911	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.7;
	1911	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.6;
	1912	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.6;
	1912	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.3;
	1912	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.9;
	1912	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.8;
	1912	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	1912	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.2;
	1913	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.6;
	1913	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.9;
	1913	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226;
	1913	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230;
	1913	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.8;
	1913	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.3;
	1914	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.4;
	1914	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.2;
	1914	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.4;
	1914	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.4;
	1914	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.9;
	1914	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.3;
	1915	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.5;
	1915	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.3;
	1915	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.3;
	1915	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.3;
	1915	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.9;
	1915	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.8;
	1916	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.6;
	1916	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.1;
	1916	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236;
	1916	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.5;
	1916	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.4;
	1916	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.8;
	1917	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.9;
	1917	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.3;
	1917	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.3;
	1917	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.1;
	1917	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74;
	1917	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.8;
	1918	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.8;
	1918	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.4;
	1918	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.2;
	1918	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.5;
	1918	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.6;
	1918	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.7;
	1919	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.1;
	1919	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.5;
	1919	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.7;
	1919	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.7;
	1919	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.1;
	1919	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.2;
	1920	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.1;
	1920	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.7;
	1920	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.3;
	1920	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208;
	1920	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	1920	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.4;
	1921	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.8;
	1921	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.4;
	1921	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.6;
	1921	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.9;
	1921	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.9;
	1921	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.5;
	1922	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431;
	1922	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.2;
	1922	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.7;
	1922	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.1;
	1922	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	1922	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.5;
	1923	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.3;
	1923	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.3;
	1923	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.6;
	1923	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.9;
	1923	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	1923	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.4;
	1924	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.9;
	1924	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.8;
	1924	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.2;
	1924	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.9;
	1924	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	1924	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340;
	1925	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447.8;
	1925	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.7;
	1925	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.3;
	1925	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.4;
	1925	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	1925	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.4;
	1926	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.4;
	1926	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.2;
	1926	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.3;
	1926	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.8;
	1926	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	1926	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.3;
	1927	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.7;
	1927	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.8;
	1927	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.3;
	1927	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.4;
	1927	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69;
	1927	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380;
	1928	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.5;
	1928	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.1;
	1928	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.6;
	1928	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235;
	1928	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.6;
	1928	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.8;
	1929	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547.9;
	1929	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.9;
	1929	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.8;
	1929	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.5;
	1929	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.9;
	1929	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.9;
	1930	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.8;
	1930	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168;
	1930	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.4;
	1930	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.8;
	1930	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79;
	1930	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.5;
	1931	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.1;
	1931	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.3;
	1931	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.5;
	1931	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.8;
	1931	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.5;
	1931	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.2;
	1932	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.5;
	1932	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.7;
	1932	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.8;
	1932	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.6;
	1932	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.2;
	1932	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.4;
	1933	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513;
	1933	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.3;
	1933	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.4;
	1933	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234;
	1933	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	1933	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.3;
	1934	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.5;
	1934	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.7;
	1934	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.1;
	1934	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.5;
	1934	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	1934	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.4;
	1935	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.9;
	1935	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161;
	1935	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.4;
	1935	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.9;
	1935	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.4;
	1935	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382;
	1936	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.4;
	1936	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.5;
	1936	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.6;
	1936	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.8;
	1936	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.1;
	1936	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.2;
	1937	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.9;
	1937	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.1;
	1937	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.5;
	1937	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.1;
	1937	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	1937	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.8;
	1938	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.7;
	1938	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.9;
	1938	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.2;
	1938	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.8;
	1938	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	1938	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.5;
	1939	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.6;
	1939	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.6;
	1939	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.2;
	1939	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.4;
	1939	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	1939	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.1;
	1940	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512;
	1940	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.1;
	1940	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.6;
	1940	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.9;
	1940	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	1940	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.5;
	1941	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.2;
	1941	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.9;
	1941	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.6;
	1941	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.1;
	1941	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.4;
	1941	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.1;
	1942	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.7;
	1942	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.1;
	1942	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.1;
	1942	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.4;
	1942	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	1942	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.7;
	1943	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.7;
	1943	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.5;
	1943	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.8;
	1943	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.6;
	1943	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	1943	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.2;
	1944	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.5;
	1944	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.8;
	1944	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.3;
	1944	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.9;
	1944	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.2;
	1944	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.6;
	1945	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.5;
	1945	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.6;
	1945	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.2;
	1945	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.4;
	1945	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	1945	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.8;
	1946	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.9;
	1946	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.8;
	1946	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.6;
	1946	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193;
	1946	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	1946	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.7;
	1947	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.2;
	1947	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.6;
	1947	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.2;
	1947	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.5;
	1947	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	1947	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.5;
	1948	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.2;
	1948	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.1;
	1948	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.6;
	1948	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.6;
	1948	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	1948	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.8;
	1949	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459;
	1949	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.5;
	1949	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194;
	1949	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199;
	1949	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	1949	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350;
	1950	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486;
	1950	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.5;
	1950	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.8;
	1950	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.8;
	1950	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	1950	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.1;
	1951	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.2;
	1951	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.7;
	1951	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.2;
	1951	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.4;
	1951	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.4;
	1951	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.4;
	1952	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548.3;
	1952	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.9;
	1952	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.7;
	1952	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.4;
	1952	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.1;
	1952	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.9;
	1953	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552.8;
	1953	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.7;
	1953	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.3;
	1953	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.1;
	1953	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.6;
	1953	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.3;
	1954	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.2;
	1954	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.4;
	1954	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.6;
	1954	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.7;
	1954	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.9;
	1954	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.8;
	1955	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	540.7;
	1955	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.4;
	1955	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.8;
	1955	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.3;
	1955	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.2;
	1955	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.2;
	1956	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547.9;
	1956	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.8;
	1956	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.2;
	1956	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.7;
	1956	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	1956	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.4;
	1957	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	534.4;
	1957	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.9;
	1957	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.1;
	1957	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.1;
	1957	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.8;
	1957	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.4;
	1958	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530;
	1958	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.2;
	1958	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.2;
	1958	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.7;
	1958	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	1958	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.7;
	1959	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	524.8;
	1959	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.6;
	1959	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.1;
	1959	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232;
	1959	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	1959	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.1;
	1960	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521;
	1960	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158;
	1960	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.5;
	1960	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.1;
	1960	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	1960	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.5;
	1961	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.1;
	1961	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.7;
	1961	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.4;
	1961	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.7;
	1961	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	1961	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.8;
	1962	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.4;
	1962	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.4;
	1962	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.3;
	1962	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.3;
	1962	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	1962	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.9;
	1963	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.2;
	1963	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.1;
	1963	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.5;
	1963	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.5;
	1963	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	1963	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.4;
	1964	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.9;
	1964	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.7;
	1964	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.5;
	1964	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.9;
	1964	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	1964	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.3;
	1965	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528.8;
	1965	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.2;
	1965	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.7;
	1965	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239;
	1965	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	1965	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.7;
	1966	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.4;
	1966	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.1;
	1966	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.7;
	1966	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.3;
	1966	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	1966	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.7;
	1967	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.9;
	1967	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.3;
	1967	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.6;
	1967	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.2;
	1967	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	1967	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.3;
	1968	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.4;
	1968	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.3;
	1968	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.8;
	1968	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.2;
	1968	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.4;
	1968	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.1;
	1969	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.8;
	1969	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.4;
	1969	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.1;
	1969	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.9;
	1969	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	1969	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.6;
	1970	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.2;
	1970	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.5;
	1970	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190;
	1970	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.5;
	1970	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.6;
	1970	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.3;
	1971	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.2;
	1971	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.3;
	1971	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.7;
	1971	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193;
	1971	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.7;
	1971	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.1;
	1972	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.4;
	1972	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.2;
	1972	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.4;
	1972	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.8;
	1972	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.4;
	1972	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.4;
	1973	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.9;
	1973	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.8;
	1973	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195;
	1973	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.8;
	1973	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.3;
	1973	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.9;
	1974	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.5;
	1974	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.2;
	1974	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.7;
	1974	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.4;
	1974	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.5;
	1974	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.9;
	1975	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.6;
	1975	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.6;
	1975	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221;
	1975	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.1;
	1975	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.3;
	1975	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.2;
	1976	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547;
	1976	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.3;
	1976	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.8;
	1976	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.7;
	1976	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	1976	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.1;
	1977	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	553.9;
	1977	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.5;
	1977	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.1;
	1977	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.8;
	1977	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	1977	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.3;
	1978	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.9;
	1978	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168;
	1978	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.7;
	1978	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.7;
	1978	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	1978	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.4;
	1979	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528;
	1979	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.6;
	1979	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.4;
	1979	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.7;
	1979	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	1979	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390;
	1980	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522.5;
	1980	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.1;
	1980	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.7;
	1980	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.1;
	1980	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	1980	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.8;
	1981	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.6;
	1981	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.6;
	1981	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230;
	1981	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.6;
	1981	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	1981	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.7;
	1982	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.5;
	1982	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.1;
	1982	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.7;
	1982	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.8;
	1982	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	1982	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382;
	1983	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.7;
	1983	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.7;
	1983	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.9;
	1983	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.9;
	1983	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	1983	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.8;
	1984	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.6;
	1984	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.3;
	1984	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.1;
	1984	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.5;
	1984	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	1984	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.5;
	1985	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.9;
	1985	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161;
	1985	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.3;
	1985	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.8;
	1985	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	1985	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.2;
	1986	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.7;
	1986	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.7;
	1986	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.8;
	1986	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.4;
	1986	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.7;
	1986	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.7;
	1987	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.3;
	1987	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.5;
	1987	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.9;
	1987	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.1;
	1987	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.4;
	1987	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.3;
	1988	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478;
	1988	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.2;
	1988	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.4;
	1988	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.8;
	1988	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	1988	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.1;
	1989	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.4;
	1989	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164;
	1989	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.9;
	1989	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.1;
	1989	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	1989	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.2;
	1990	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.5;
	1990	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.2;
	1990	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.5;
	1990	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.4;
	1990	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.2;
	1990	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.5;
	1991	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.4;
	1991	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.1;
	1991	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.4;
	1991	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.4;
	1991	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.9;
	1991	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.2;
	1992	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.6;
	1992	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.3;
	1992	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.7;
	1992	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.2;
	1992	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.7;
	1992	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.4;
	1993	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.6;
	1993	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.9;
	1993	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.5;
	1993	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.7;
	1993	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55;
	1993	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.1;
	1994	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403;
	1994	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.4;
	1994	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.2;
	1994	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.3;
	1994	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.2;
	1994	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.7;
	1995	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.8;
	1995	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.3;
	1995	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.5;
	1995	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.9;
	1995	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.6;
	1995	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.9;
	1996	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.6;
	1996	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.6;
	1996	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.9;
	1996	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.6;
	1996	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.6;
	1996	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.5;
	1997	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418;
	1997	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.4;
	1997	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.6;
	1997	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.3;
	1997	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.1;
	1997	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333;
	1998	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442.6;
	1998	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.7;
	1998	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.5;
	1998	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.7;
	1998	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.4;
	1998	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.3;
	1999	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.9;
	1999	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.6;
	1999	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.5;
	1999	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.6;
	1999	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.2;
	1999	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.8;
	2000	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.5;
	2000	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147;
	2000	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.4;
	2000	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.8;
	2000	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	2000	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.6;
	2001	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.7;
	2001	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.5;
	2001	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.1;
	2001	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.7;
	2001	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.2;
	2001	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.7;
	2002	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.1;
	2002	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.2;
	2002	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.6;
	2002	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.5;
	2002	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	2002	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.6;
	2003	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.7;
	2003	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.1;
	2003	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210;
	2003	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.1;
	2003	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	2003	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.9;
	2004	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.6;
	2004	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.9;
	2004	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.6;
	2004	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.4;
	2004	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	2004	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.9;
	2005	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.2;
	2005	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.8;
	2005	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.9;
	2005	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.7;
	2005	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	2005	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.6;
	2006	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454;
	2006	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.6;
	2006	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.9;
	2006	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.6;
	2006	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	2006	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.8;
	2007	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442.9;
	2007	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.8;
	2007	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.4;
	2007	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.7;
	2007	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	2007	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.5;
	2008	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442.5;
	2008	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149;
	2008	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201;
	2008	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.6;
	2008	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	2008	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.1;
	2009	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442.7;
	2009	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.1;
	2009	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.2;
	2009	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.5;
	2009	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	2009	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.5;
	2010	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.7;
	2010	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.1;
	2010	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.7;
	2010	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.3;
	2010	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	2010	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.6;
	2011	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.6;
	2011	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.2;
	2011	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.8;
	2011	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.1;
	2011	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	2011	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.2;
	2012	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.4;
	2012	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.1;
	2012	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.3;
	2012	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.3;
	2012	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.7;
	2012	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.1;
	2013	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.3;
	2013	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.9;
	2013	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.9;
	2013	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.8;
	2013	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	2013	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363;
	2014	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.7;
	2014	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.7;
	2014	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.9;
	2014	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.5;
	2014	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	2014	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.1;
	2015	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.2;
	2015	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.3;
	2015	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.7;
	2015	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.9;
	2015	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	2015	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.3;
	2016	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.1;
	2016	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.3;
	2016	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.4;
	2016	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.2;
	2016	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	2016	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.1;
	2017	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417.9;
	2017	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.1;
	2017	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191;
	2017	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.3;
	2017	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.7;
	2017	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.6;
	2018	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.1;
	2018	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.3;
	2018	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.7;
	2018	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.1;
	2018	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.8;
	2018	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.6;
	2019	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	396;
	2019	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.5;
	2019	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.2;
	2019	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.8;
	2019	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.8;
	2019	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.4;
	2020	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.6;
	2020	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.7;
	2020	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.3;
	2020	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.9;
	2020	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.2;
	2020	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.3;
	2021	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.4;
	2021	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.4;
	2021	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.9;
	2021	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.8;
	2021	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.2;
	2021	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.7;
	2022	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429;
	2022	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.4;
	2022	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191;
	2022	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196;
	2022	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.7;
	2022	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.2;
	2023	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449;
	2023	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.3;
	2023	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.9;
	2023	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.7;
	2023	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.7;
	2023	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.7;
	2024	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.8;
	2024	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.7;
	2024	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.8;
	2024	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.2;
	2024	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.3;
	2024	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.4;
	2025	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.7;
	2025	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.2;
	2025	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.8;
	2025	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.3;
	2025	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.5;
	2025	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.6;
	2026	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.3;
	2026	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.9;
	2026	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.8;
	2026	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.2;
	2026	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.1;
	2026	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.5;
	2027	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.6;
	2027	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.5;
	2027	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.5;
	2027	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.6;
	2027	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	2027	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.9;
	2028	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.7;
	2028	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.1;
	2028	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.5;
	2028	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.5;
	2028	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	2028	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.9;
	2029	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.9;
	2029	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.8;
	2029	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.8;
	2029	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.1;
	2029	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	2029	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.4;
	2030	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.8;
	2030	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152;
	2030	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.6;
	2030	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.4;
	2030	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	2030	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.1;
	2031	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.9;
	2031	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.7;
	2031	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.8;
	2031	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.7;
	2031	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	2031	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.5;
	2032	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.3;
	2032	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.4;
	2032	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.9;
	2032	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.3;
	2032	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	2032	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.5;
	2033	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.5;
	2033	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151;
	2033	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.2;
	2033	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.1;
	2033	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	2033	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353;
	2034	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.7;
	2034	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.6;
	2034	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.2;
	2034	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210;
	2034	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	2034	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356;
	2035	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.8;
	2035	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.6;
	2035	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.7;
	2035	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221;
	2035	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.2;
	2035	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.6;
	2036	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.6;
	2036	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.6;
	2036	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.3;
	2036	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.5;
	2036	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	2036	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.1;
	2037	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.1;
	2037	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.6;
	2037	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.6;
	2037	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230;
	2037	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.3;
	2037	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.2;
	2038	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.7;
	2038	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.5;
	2038	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.4;
	2038	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.9;
	2038	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	2038	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372;
	2039	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.6;
	2039	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148;
	2039	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.7;
	2039	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.2;
	2039	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59;
	2039	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.5;
	2040	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.9;
	2040	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.9;
	2040	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.8;
	2040	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.2;
	2040	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.9;
	2040	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.9;
	2041	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.3;
	2041	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.4;
	2041	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.3;
	2041	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.3;
	2041	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55;
	2041	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.9;
	2042	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.5;
	2042	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.7;
	2042	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.7;
	2042	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.4;
	2042	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.2;
	2042	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.6;
	2043	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448;
	2043	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.1;
	2043	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.8;
	2043	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.1;
	2043	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.9;
	2043	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343;
	2044	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.7;
	2044	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.9;
	2044	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.8;
	2044	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.2;
	2044	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.7;
	2044	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.7;
	2045	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.3;
	2045	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.2;
	2045	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.8;
	2045	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.3;
	2045	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.4;
	2045	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.3;
	2046	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.9;
	2046	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.5;
	2046	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.5;
	2046	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.6;
	2046	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.9;
	2046	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.1;
	2047	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537.5;
	2047	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.8;
	2047	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.4;
	2047	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.1;
	2047	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	2047	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.4;
	2048	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	575;
	2048	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.7;
	2048	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.5;
	2048	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.6;
	2048	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.5;
	2048	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.2;
	2049	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	591.3;
	2049	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.7;
	2049	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.5;
	2049	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.4;
	2049	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59;
	2049	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417;
	2050	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589.4;
	2050	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.1;
	2050	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.2;
	2050	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.4;
	2050	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.6;
	2050	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.1;
	2051	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	574.6;
	2051	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.4;
	2051	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.7;
	2051	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.8;
	2051	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.3;
	2051	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.8;
	2052	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.5;
	2052	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174;
	2052	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.4;
	2052	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.5;
	2052	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	2052	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403;
	2053	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	557.9;
	2053	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.6;
	2053	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.4;
	2053	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.5;
	2053	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	2053	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.9;
	2054	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.7;
	2054	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.9;
	2054	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.7;
	2054	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.8;
	2054	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	2054	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.2;
	2055	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.5;
	2055	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.1;
	2055	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.1;
	2055	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.9;
	2055	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	2055	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.3;
	2056	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	534.9;
	2056	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.5;
	2056	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.4;
	2056	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.1;
	2056	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	2056	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.6;
	2057	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537.5;
	2057	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.7;
	2057	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.5;
	2057	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.9;
	2057	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	2057	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.3;
	2058	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.8;
	2058	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.6;
	2058	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.8;
	2058	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.6;
	2058	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.1;
	2058	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.2;
	2059	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.1;
	2059	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.4;
	2059	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.9;
	2059	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.3;
	2059	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.8;
	2059	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.6;
	2060	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.8;
	2060	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.7;
	2060	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.8;
	2060	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.3;
	2060	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	2060	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.3;
	2061	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.8;
	2061	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.7;
	2061	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.3;
	2061	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.3;
	2061	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.1;
	2061	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.3;
	2062	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.8;
	2062	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.4;
	2062	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227;
	2062	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.7;
	2062	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	2062	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.7;
	2063	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.1;
	2063	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.2;
	2063	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.2;
	2063	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.3;
	2063	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60;
	2063	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.2;
	2064	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.2;
	2064	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.3;
	2064	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.5;
	2064	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.4;
	2064	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	2064	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.5;
	2065	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.9;
	2065	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.7;
	2065	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.6;
	2065	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.6;
	2065	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.2;
	2065	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357;
	2066	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.9;
	2066	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.2;
	2066	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.4;
	2066	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.9;
	2066	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.8;
	2066	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.9;
	2067	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.5;
	2067	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.5;
	2067	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.2;
	2067	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.9;
	2067	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.7;
	2067	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.8;
	2068	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473;
	2068	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.2;
	2068	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.6;
	2068	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.2;
	2068	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.3;
	2068	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.8;
	2069	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.5;
	2069	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.8;
	2069	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.1;
	2069	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.4;
	2069	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.1;
	2069	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.8;
	2070	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.3;
	2070	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.5;
	2070	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.3;
	2070	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.1;
	2070	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.9;
	2070	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.9;
	2071	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564.8;
	2071	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183;
	2071	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.5;
	2071	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.4;
	2071	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.9;
	2071	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.9;
	2072	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	590.5;
	2072	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.2;
	2072	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.4;
	2072	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.3;
	2072	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.3;
	2072	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.4;
	2073	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	596.8;
	2073	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.7;
	2073	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.5;
	2073	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.9;
	2073	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	2073	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.6;
	2074	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577.5;
	2074	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.7;
	2074	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.5;
	2074	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.1;
	2074	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.7;
	2074	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.3;
	2075	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	575.3;
	2075	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176;
	2075	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.6;
	2075	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.6;
	2075	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	2075	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.5;
	2076	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	575.5;
	2076	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.9;
	2076	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227;
	2076	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.4;
	2076	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.4;
	2076	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.5;
	2077	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	524.1;
	2077	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.5;
	2077	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.2;
	2077	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.2;
	2077	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	2077	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.7;
	2078	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.5;
	2078	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.4;
	2078	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.9;
	2078	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230;
	2078	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.4;
	2078	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.4;
	2079	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518;
	2079	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.7;
	2079	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.7;
	2079	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.4;
	2079	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.3;
	2079	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.8;
	2080	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.3;
	2080	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.1;
	2080	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.8;
	2080	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.5;
	2080	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.1;
	2080	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.3;
	2081	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.4;
	2081	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165;
	2081	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.4;
	2081	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.7;
	2081	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.4;
	2081	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.2;
	2082	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500;
	2082	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.1;
	2082	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.7;
	2082	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.9;
	2082	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	2082	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.6;
	2083	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.2;
	2083	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.3;
	2083	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.6;
	2083	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.3;
	2083	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	2083	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380;
	2084	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.1;
	2084	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.2;
	2084	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.1;
	2084	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.8;
	2084	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	2084	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.2;
	2085	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526.1;
	2085	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.9;
	2085	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.8;
	2085	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.9;
	2085	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.7;
	2085	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.9;
	2086	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522.7;
	2086	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165;
	2086	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.6;
	2086	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227;
	2086	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	2086	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.4;
	2087	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.9;
	2087	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.3;
	2087	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.2;
	2087	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.5;
	2087	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	2087	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.3;
	2088	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.2;
	2088	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.1;
	2088	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.4;
	2088	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.4;
	2088	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	2088	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.5;
	2089	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464;
	2089	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.4;
	2089	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.7;
	2089	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.5;
	2089	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.9;
	2089	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.5;
	2090	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452;
	2090	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.6;
	2090	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.5;
	2090	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.2;
	2090	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56;
	2090	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.4;
	2091	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.3;
	2091	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.9;
	2091	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.1;
	2091	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.9;
	2091	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.4;
	2091	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.4;
	2092	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.5;
	2092	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.9;
	2092	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.3;
	2092	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.6;
	2092	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	2092	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.4;
	2093	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.3;
	2093	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.6;
	2093	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.5;
	2093	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.7;
	2093	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.5;
	2093	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.7;
	2094	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504;
	2094	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.3;
	2094	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.3;
	2094	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.2;
	2094	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.8;
	2094	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.4;
	2095	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528.7;
	2095	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.3;
	2095	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.1;
	2095	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.3;
	2095	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.8;
	2095	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.8;
	2096	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.9;
	2096	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.2;
	2096	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.2;
	2096	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.2;
	2096	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	2096	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.7;
	2097	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552.2;
	2097	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.7;
	2097	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.6;
	2097	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.3;
	2097	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	2097	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.9;
	2098	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	554.1;
	2098	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.7;
	2098	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.1;
	2098	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.8;
	2098	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	2098	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.1;
	2099	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549.2;
	2099	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.3;
	2099	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.8;
	2099	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.6;
	2099	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.2;
	2099	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.9;
	2100	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.2;
	2100	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.9;
	2100	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.2;
	2100	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.7;
	2100	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	2100	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.5;
	2101	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.3;
	2101	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.8;
	2101	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.2;
	2101	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.5;
	2101	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	2101	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.1;
	2102	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514;
	2102	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165;
	2102	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.2;
	2102	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.7;
	2102	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	2102	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.4;
	2103	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.5;
	2103	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163;
	2103	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.8;
	2103	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.4;
	2103	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	2103	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.8;
	2104	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518.7;
	2104	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.8;
	2104	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225;
	2104	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.3;
	2104	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	2104	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.6;
	2105	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.1;
	2105	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.7;
	2105	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.7;
	2105	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.6;
	2105	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	2105	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.4;
	2106	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502;
	2106	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.2;
	2106	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.5;
	2106	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.1;
	2106	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.6;
	2106	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.2;
	2107	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.8;
	2107	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.9;
	2107	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.4;
	2107	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.2;
	2107	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61;
	2107	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.3;
	2108	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.7;
	2108	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.7;
	2108	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.3;
	2108	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.5;
	2108	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	2108	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.8;
	2109	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.1;
	2109	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.2;
	2109	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.9;
	2109	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.4;
	2109	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	2109	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.9;
	2110	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.7;
	2110	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.2;
	2110	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.3;
	2110	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.4;
	2110	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	2110	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.3;
	2111	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.6;
	2111	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.8;
	2111	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.7;
	2111	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.9;
	2111	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	2111	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.1;
	2112	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.3;
	2112	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.9;
	2112	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.3;
	2112	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.2;
	2112	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.1;
	2112	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359;
	2113	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.9;
	2113	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.9;
	2113	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.7;
	2113	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.2;
	2113	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.3;
	2113	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.4;
	2114	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.5;
	2114	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.7;
	2114	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.2;
	2114	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.5;
	2114	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.1;
	2114	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.7;
	2115	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.4;
	2115	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.9;
	2115	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.9;
	2115	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.1;
	2115	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.5;
	2115	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.3;
	2116	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442.7;
	2116	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.2;
	2116	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.1;
	2116	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.5;
	2116	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.2;
	2116	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.5;
	2117	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.6;
	2117	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.7;
	2117	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.3;
	2117	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198;
	2117	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54;
	2117	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.5;
	2118	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.2;
	2118	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.1;
	2118	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.4;
	2118	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.7;
	2118	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.6;
	2118	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.9;
	2119	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520;
	2119	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.8;
	2119	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.5;
	2119	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.5;
	2119	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.4;
	2119	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.4;
	2120	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	540.4;
	2120	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.4;
	2120	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226;
	2120	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.6;
	2120	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	2120	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.8;
	2121	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542;
	2121	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.1;
	2121	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.2;
	2121	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.8;
	2121	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	2121	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.9;
	2122	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547.2;
	2122	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.2;
	2122	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.9;
	2122	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234;
	2122	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.2;
	2122	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.5;
	2123	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	534.7;
	2123	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.7;
	2123	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.8;
	2123	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.4;
	2123	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	2123	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.9;
	2124	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.9;
	2124	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.8;
	2124	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.7;
	2124	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233;
	2124	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	2124	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.1;
	2125	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.7;
	2125	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.9;
	2125	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.3;
	2125	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.9;
	2125	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	2125	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.3;
	2126	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.6;
	2126	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.5;
	2126	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.9;
	2126	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.5;
	2126	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	2126	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.9;
	2127	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.5;
	2127	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158;
	2127	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.4;
	2127	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.3;
	2127	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	2127	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.2;
	2128	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.5;
	2128	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.5;
	2128	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224;
	2128	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.9;
	2128	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	2128	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.1;
	2129	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.3;
	2129	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160;
	2129	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.1;
	2129	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.9;
	2129	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.3;
	2129	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.8;
	2130	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.9;
	2130	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.4;
	2130	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.8;
	2130	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.9;
	2130	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.6;
	2130	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.8;
	2131	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.6;
	2131	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.4;
	2131	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.6;
	2131	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.8;
	2131	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.4;
	2131	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.9;
	2132	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518;
	2132	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.2;
	2132	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.7;
	2132	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.2;
	2132	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	2132	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387;
	2133	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522.2;
	2133	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.8;
	2133	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.8;
	2133	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.3;
	2133	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	2133	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388;
	2134	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.9;
	2134	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.3;
	2134	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.9;
	2134	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224;
	2134	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	2134	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.2;
	2135	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.7;
	2135	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146;
	2135	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.4;
	2135	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.1;
	2135	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	2135	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.2;
	2136	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.6;
	2136	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.5;
	2136	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.3;
	2136	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.6;
	2136	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	2136	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.8;
	2137	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.2;
	2137	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.2;
	2137	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.6;
	2137	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.5;
	2137	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	2137	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.3;
	2138	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.1;
	2138	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.1;
	2138	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.9;
	2138	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.9;
	2138	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.3;
	2138	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.3;
	2139	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.4;
	2139	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.4;
	2139	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.2;
	2139	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.6;
	2139	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.7;
	2139	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.9;
	2140	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	427;
	2140	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.3;
	2140	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.5;
	2140	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.6;
	2140	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.4;
	2140	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.9;
	2141	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.6;
	2141	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.4;
	2141	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.5;
	2141	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.3;
	2141	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.1;
	2141	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.9;
	2142	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.9;
	2142	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.6;
	2142	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.3;
	2142	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.5;
	2142	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.4;
	2142	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.2;
	2143	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.6;
	2143	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.9;
	2143	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.4;
	2143	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220;
	2143	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	2143	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367;
	2144	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.3;
	2144	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.2;
	2144	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.7;
	2144	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.4;
	2144	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.2;
	2144	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.2;
	2145	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.1;
	2145	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.9;
	2145	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.3;
	2145	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.5;
	2145	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.6;
	2145	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.4;
	2146	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.2;
	2146	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.4;
	2146	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.5;
	2146	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231;
	2146	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72;
	2146	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.7;
	2147	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508;
	2147	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.9;
	2147	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.5;
	2147	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.8;
	2147	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.5;
	2147	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.8;
	2148	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.2;
	2148	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.4;
	2148	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.4;
	2148	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230;
	2148	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	2148	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.7;
	2149	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.8;
	2149	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.3;
	2149	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.4;
	2149	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.7;
	2149	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.2;
	2149	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.3;
	2150	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.2;
	2150	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.4;
	2150	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.4;
	2150	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.2;
	2150	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	2150	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.9;
	2151	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.9;
	2151	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.7;
	2151	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.9;
	2151	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.8;
	2151	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	2151	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.2;
	2152	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.7;
	2152	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.1;
	2152	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.4;
	2152	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.9;
	2152	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	2152	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.2;
	2153	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.2;
	2153	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.8;
	2153	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.4;
	2153	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226;
	2153	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	2153	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.4;
	2154	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.1;
	2154	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.4;
	2154	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.7;
	2154	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.3;
	2154	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	2154	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.5;
	2155	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.4;
	2155	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.4;
	2155	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.5;
	2155	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.8;
	2155	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	2155	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.9;
	2156	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.8;
	2156	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.8;
	2156	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.6;
	2156	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230;
	2156	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	2156	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.1;
	2157	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.5;
	2157	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.3;
	2157	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.4;
	2157	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.4;
	2157	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	2157	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.7;
	2158	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493;
	2158	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.5;
	2158	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.1;
	2158	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.7;
	2158	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	2158	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.8;
	2159	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.8;
	2159	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.9;
	2159	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206;
	2159	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.2;
	2159	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	2159	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360;
	2160	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.9;
	2160	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.6;
	2160	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.8;
	2160	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.4;
	2160	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	2160	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.7;
	2161	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.4;
	2161	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.2;
	2161	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.9;
	2161	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195;
	2161	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.7;
	2161	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.5;
	2162	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.1;
	2162	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.3;
	2162	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.4;
	2162	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.5;
	2162	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.3;
	2162	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.7;
	2163	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.3;
	2163	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.8;
	2163	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.8;
	2163	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.9;
	2163	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.3;
	2163	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.2;
	2164	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.3;
	2164	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.3;
	2164	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.3;
	2164	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.6;
	2164	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	2164	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.5;
	2165	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.1;
	2165	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.2;
	2165	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.1;
	2165	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.3;
	2165	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.7;
	2165	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.1;
	2166	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464;
	2166	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.1;
	2166	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.2;
	2166	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.1;
	2166	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.6;
	2166	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352;
	2167	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.5;
	2167	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.7;
	2167	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.2;
	2167	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203;
	2167	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58;
	2167	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.2;
	2168	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.2;
	2168	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.9;
	2168	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.7;
	2168	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.5;
	2168	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	2168	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.9;
	2169	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.5;
	2169	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.2;
	2169	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.1;
	2169	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.1;
	2169	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	2169	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.6;
	2170	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.2;
	2170	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.3;
	2170	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.5;
	2170	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.1;
	2170	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	2170	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.7;
	2171	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.5;
	2171	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.4;
	2171	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.3;
	2171	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.7;
	2171	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	2171	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.6;
	2172	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.7;
	2172	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.8;
	2172	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.9;
	2172	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.9;
	2172	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	2172	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.4;
	2173	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.7;
	2173	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.1;
	2173	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.1;
	2173	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.2;
	2173	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	2173	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.6;
	2174	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.5;
	2174	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.6;
	2174	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.3;
	2174	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.6;
	2174	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	2174	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.3;
	2175	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.6;
	2175	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.5;
	2175	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206;
	2175	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.1;
	2175	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	2175	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.6;
	2176	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.6;
	2176	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149;
	2176	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.6;
	2176	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.8;
	2176	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	2176	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.4;
	2177	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.2;
	2177	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149;
	2177	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.9;
	2177	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.9;
	2177	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	2177	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.7;
	2178	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428;
	2178	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.3;
	2178	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.2;
	2178	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.8;
	2178	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	2178	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.2;
	2179	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.2;
	2179	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.4;
	2179	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.8;
	2179	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.1;
	2179	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	2179	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.9;
	2180	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.7;
	2180	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.2;
	2180	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.4;
	2180	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.6;
	2180	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.3;
	2180	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.4;
	2181	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.9;
	2181	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.1;
	2181	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.8;
	2181	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.4;
	2181	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	2181	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350;
	2182	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.2;
	2182	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151;
	2182	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214;
	2182	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.1;
	2182	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	2182	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.8;
	2183	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.8;
	2183	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.5;
	2183	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.7;
	2183	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.8;
	2183	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	2183	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.6;
	2184	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	406.1;
	2184	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.3;
	2184	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.9;
	2184	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.1;
	2184	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.9;
	2184	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332;
	2185	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391.6;
	2185	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.7;
	2185	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192;
	2185	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.3;
	2185	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.3;
	2185	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.7;
	2186	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	387.5;
	2186	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.6;
	2186	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.8;
	2186	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.3;
	2186	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54;
	2186	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.3;
	2187	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.5;
	2187	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.5;
	2187	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.3;
	2187	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.2;
	2187	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.2;
	2187	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.7;
	2188	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391.2;
	2188	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.6;
	2188	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.8;
	2188	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.1;
	2188	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.4;
	2188	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.8;
	2189	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.5;
	2189	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.9;
	2189	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.2;
	2189	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.5;
	2189	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.6;
	2189	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.4;
	2190	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	406.6;
	2190	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137;
	2190	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.1;
	2190	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.7;
	2190	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.1;
	2190	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.4;
	2191	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.1;
	2191	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.7;
	2191	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.5;
	2191	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.8;
	2191	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.2;
	2191	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.4;
	2192	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.9;
	2192	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.4;
	2192	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.8;
	2192	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.8;
	2192	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.9;
	2192	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.3;
	2193	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.3;
	2193	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156;
	2193	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.9;
	2193	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.9;
	2193	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.3;
	2193	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.6;
	2194	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.6;
	2194	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.3;
	2194	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.3;
	2194	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.6;
	2194	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	2194	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.7;
	2195	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.5;
	2195	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.6;
	2195	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.2;
	2195	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.3;
	2195	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	2195	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.5;
	2196	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.2;
	2196	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.8;
	2196	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.9;
	2196	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.9;
	2196	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	2196	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.5;
	2197	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.2;
	2197	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.1;
	2197	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.7;
	2197	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.9;
	2197	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	2197	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.9;
	2198	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.9;
	2198	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.7;
	2198	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.1;
	2198	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203;
	2198	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	2198	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.6;
	2199	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	402.7;
	2199	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.7;
	2199	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.7;
	2199	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.8;
	2199	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	2199	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.2;
	2200	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392.7;
	2200	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.4;
	2200	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199;
	2200	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.2;
	2200	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	2200	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.4;
	2201	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	396.1;
	2201	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.8;
	2201	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.3;
	2201	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204;
	2201	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	2201	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.7;
	2202	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	396;
	2202	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.4;
	2202	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.1;
	2202	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208;
	2202	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	2202	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.5;
	2203	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404;
	2203	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.6;
	2203	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.5;
	2203	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218;
	2203	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	2203	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.9;
	2204	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	420.6;
	2204	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.8;
	2204	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.9;
	2204	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.8;
	2204	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	2204	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.9;
	2205	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.9;
	2205	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.4;
	2205	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220;
	2205	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.7;
	2205	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	2205	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.8;
	2206	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	430.2;
	2206	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.7;
	2206	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.9;
	2206	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.2;
	2206	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	2206	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.3;
	2207	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.2;
	2207	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142;
	2207	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.2;
	2207	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.8;
	2207	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.4;
	2207	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.3;
	2208	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.9;
	2208	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.2;
	2208	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.9;
	2208	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.3;
	2208	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	2208	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.2;
	2209	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.7;
	2209	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.3;
	2209	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.7;
	2209	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.8;
	2209	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.5;
	2209	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.1;
	2210	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	387.5;
	2210	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.2;
	2210	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.6;
	2210	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.1;
	2210	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.7;
	2210	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.4;
	2211	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	387;
	2211	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.7;
	2211	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.3;
	2211	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.4;
	2211	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.8;
	2211	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317;
	2212	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	386.9;
	2212	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.9;
	2212	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.7;
	2212	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.1;
	2212	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.8;
	2212	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.6;
	2213	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.2;
	2213	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.3;
	2213	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.6;
	2213	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.1;
	2213	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	50.9;
	2213	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.8;
	2214	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.6;
	2214	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.6;
	2214	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.6;
	2214	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.2;
	2214	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.3;
	2214	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.2;
	2215	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458;
	2215	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.2;
	2215	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.5;
	2215	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.7;
	2215	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.2;
	2215	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.8;
	2216	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.9;
	2216	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.9;
	2216	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.9;
	2216	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.9;
	2216	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.4;
	2216	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.2;
	2217	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.9;
	2217	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.5;
	2217	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.3;
	2217	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.8;
	2217	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.2;
	2217	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.5;
	2218	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.5;
	2218	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.2;
	2218	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.5;
	2218	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.2;
	2218	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.5;
	2218	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.2;
	2219	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.5;
	2219	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.8;
	2219	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230;
	2219	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235;
	2219	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.6;
	2219	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.4;
	2220	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.2;
	2220	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.8;
	2220	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.4;
	2220	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.8;
	2220	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	2220	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.2;
	2221	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.2;
	2221	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.6;
	2221	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.1;
	2221	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.5;
	2221	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	2221	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.5;
	2222	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.4;
	2222	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.8;
	2222	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.2;
	2222	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.9;
	2222	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61;
	2222	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.5;
	2223	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.9;
	2223	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156;
	2223	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.5;
	2223	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.2;
	2223	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.3;
	2223	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.5;
	2224	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.3;
	2224	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.8;
	2224	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.3;
	2224	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.8;
	2224	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	2224	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.2;
	2225	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.2;
	2225	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.2;
	2225	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222;
	2225	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.7;
	2225	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.4;
	2225	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.6;
	2226	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.1;
	2226	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.2;
	2226	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.6;
	2226	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.1;
	2226	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	2226	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.9;
	2227	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.8;
	2227	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.4;
	2227	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.4;
	2227	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230;
	2227	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.4;
	2227	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.7;
	2228	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.5;
	2228	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.5;
	2228	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.7;
	2228	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.6;
	2228	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	2228	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.7;
	2229	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.2;
	2229	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.3;
	2229	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.5;
	2229	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.6;
	2229	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.3;
	2229	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.5;
	2230	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.2;
	2230	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.9;
	2230	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.2;
	2230	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225;
	2230	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	2230	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.3;
	2231	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.8;
	2231	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.7;
	2231	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.1;
	2231	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.8;
	2231	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.3;
	2231	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362;
	2232	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.5;
	2232	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.4;
	2232	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.8;
	2232	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.3;
	2232	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.7;
	2232	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.7;
	2233	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447.8;
	2233	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.1;
	2233	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.9;
	2233	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.5;
	2233	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56;
	2233	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.8;
	2234	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.3;
	2234	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.4;
	2234	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187;
	2234	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.6;
	2234	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.9;
	2234	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.7;
	2235	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	430.3;
	2235	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.2;
	2235	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187;
	2235	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.8;
	2235	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.5;
	2235	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.1;
	2236	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.2;
	2236	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.8;
	2236	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188;
	2236	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.7;
	2236	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.1;
	2236	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.6;
	2237	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.8;
	2237	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.9;
	2237	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.2;
	2237	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.2;
	2237	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.7;
	2237	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.4;
	2238	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.6;
	2238	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.2;
	2238	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.7;
	2238	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.1;
	2238	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.6;
	2238	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.9;
	2239	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.8;
	2239	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.5;
	2239	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.1;
	2239	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222;
	2239	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.3;
	2239	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.9;
	2240	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549.9;
	2240	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.5;
	2240	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.7;
	2240	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.5;
	2240	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	2240	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.7;
	2241	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.7;
	2241	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.6;
	2241	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.9;
	2241	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.5;
	2241	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	2241	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.9;
	2242	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552.9;
	2242	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.3;
	2242	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.2;
	2242	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.1;
	2242	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	2242	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.8;
	2243	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	534.9;
	2243	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.5;
	2243	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.9;
	2243	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.8;
	2243	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.7;
	2243	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.6;
	2244	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.2;
	2244	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163;
	2244	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.6;
	2244	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.5;
	2244	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	2244	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.5;
	2245	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.5;
	2245	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.7;
	2245	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.7;
	2245	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.4;
	2245	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.1;
	2245	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.9;
	2246	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510;
	2246	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.9;
	2246	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.4;
	2246	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228;
	2246	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	2246	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.7;
	2247	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505;
	2247	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.4;
	2247	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.4;
	2247	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.3;
	2247	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	2247	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.9;
	2248	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.4;
	2248	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.8;
	2248	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.7;
	2248	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228;
	2248	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	2248	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376;
	2249	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.4;
	2249	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.9;
	2249	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.8;
	2249	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.5;
	2249	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	2249	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.8;
	2250	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.7;
	2250	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.2;
	2250	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.6;
	2250	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.4;
	2250	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.3;
	2250	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.8;
	2251	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.1;
	2251	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.8;
	2251	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.8;
	2251	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.3;
	2251	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	2251	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.9;
	2252	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.9;
	2252	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169;
	2252	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234;
	2252	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.1;
	2252	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	2252	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.3;
	2253	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.5;
	2253	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.2;
	2253	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.1;
	2253	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236;
	2253	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	2253	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.9;
	2254	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.5;
	2254	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160;
	2254	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.8;
	2254	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.3;
	2254	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	2254	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.9;
	2255	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.9;
	2255	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.7;
	2255	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.5;
	2255	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.9;
	2255	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	2255	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.9;
	2256	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.2;
	2256	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.7;
	2256	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.2;
	2256	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.4;
	2256	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	2256	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.7;
	2257	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447;
	2257	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.4;
	2257	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.3;
	2257	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195;
	2257	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.4;
	2257	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.9;
	2258	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.6;
	2258	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131;
	2258	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187;
	2258	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.7;
	2258	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.5;
	2258	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.6;
	2259	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.5;
	2259	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.4;
	2259	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.8;
	2259	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.6;
	2259	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.4;
	2259	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.8;
	2260	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.7;
	2260	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.1;
	2260	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188;
	2260	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.8;
	2260	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.2;
	2260	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.3;
	2261	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.6;
	2261	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.6;
	2261	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.2;
	2261	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.5;
	2261	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	2261	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.5;
	2262	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.8;
	2262	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.3;
	2262	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.1;
	2262	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.8;
	2262	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.5;
	2262	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367;
	2263	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528.9;
	2263	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.5;
	2263	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.6;
	2263	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223;
	2263	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	2263	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.6;
	2264	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	555.8;
	2264	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.3;
	2264	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226;
	2264	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.3;
	2264	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	2264	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.6;
	2265	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.2;
	2265	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.5;
	2265	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229;
	2265	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.5;
	2265	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	2265	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.3;
	2266	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.1;
	2266	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.4;
	2266	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.6;
	2266	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233;
	2266	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69;
	2266	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.5;
	2267	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	534.2;
	2267	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.5;
	2267	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.5;
	2267	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.4;
	2267	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	2267	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.7;
	2268	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.8;
	2268	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.4;
	2268	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225;
	2268	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.6;
	2268	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	2268	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.9;
	2269	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.5;
	2269	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.3;
	2269	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225;
	2269	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.3;
	2269	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	2269	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.5;
	2270	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.2;
	2270	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155;
	2270	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.8;
	2270	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229;
	2270	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	2270	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.8;
	2271	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.7;
	2271	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.8;
	2271	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.5;
	2271	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.4;
	2271	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	2271	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.5;
	2272	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.1;
	2272	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.5;
	2272	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.2;
	2272	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.7;
	2272	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	2272	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.2;
	2273	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.7;
	2273	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.2;
	2273	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.1;
	2273	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.9;
	2273	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	2273	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373;
	2274	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.5;
	2274	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.5;
	2274	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.7;
	2274	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.6;
	2274	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	2274	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.5;
	2275	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482;
	2275	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.2;
	2275	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.3;
	2275	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.8;
	2275	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	2275	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.3;
	2276	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.3;
	2276	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.9;
	2276	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.9;
	2276	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.8;
	2276	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	2276	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.3;
	2277	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.8;
	2277	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.5;
	2277	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.7;
	2277	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.2;
	2277	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	2277	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.2;
	2278	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.8;
	2278	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.2;
	2278	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.8;
	2278	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.9;
	2278	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	2278	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.9;
	2279	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.3;
	2279	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.1;
	2279	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.2;
	2279	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.1;
	2279	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	2279	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.3;
	2280	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.4;
	2280	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.4;
	2280	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.7;
	2280	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.6;
	2280	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.4;
	2280	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.2;
	2281	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.7;
	2281	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.2;
	2281	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191;
	2281	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.3;
	2281	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.1;
	2281	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.1;
	2282	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	426;
	2282	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.6;
	2282	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187;
	2282	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.4;
	2282	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.6;
	2282	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.6;
	2283	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437;
	2283	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.7;
	2283	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.4;
	2283	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.9;
	2283	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.6;
	2283	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.8;
	2284	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.8;
	2284	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129;
	2284	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.6;
	2284	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191;
	2284	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.9;
	2284	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.8;
	2285	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.7;
	2285	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.8;
	2285	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.4;
	2285	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.1;
	2285	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.5;
	2285	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.4;
	2286	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.5;
	2286	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.2;
	2286	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.2;
	2286	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.4;
	2286	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.7;
	2286	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.3;
	2287	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.4;
	2287	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.4;
	2287	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.4;
	2287	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.3;
	2287	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	2287	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380;
	2288	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.3;
	2288	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.5;
	2288	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.3;
	2288	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.4;
	2288	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.7;
	2288	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.6;
	2289	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.5;
	2289	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.8;
	2289	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.6;
	2289	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.8;
	2289	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.7;
	2289	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.2;
	2290	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528.1;
	2290	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.9;
	2290	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.5;
	2290	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.1;
	2290	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.9;
	2290	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.4;
	2291	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.3;
	2291	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159;
	2291	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.7;
	2291	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.3;
	2291	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.6;
	2291	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378;
	2292	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.6;
	2292	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154;
	2292	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.1;
	2292	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.9;
	2292	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.9;
	2292	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.8;
	2293	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.6;
	2293	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.2;
	2293	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.1;
	2293	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.1;
	2293	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.3;
	2293	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.3;
	2294	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481;
	2294	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149;
	2294	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.2;
	2294	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.7;
	2294	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.5;
	2294	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.3;
	2295	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.8;
	2295	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.3;
	2295	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.8;
	2295	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.1;
	2295	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70;
	2295	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.5;
	2296	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.7;
	2296	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146;
	2296	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.6;
	2296	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.7;
	2296	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	2296	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.9;
	2297	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.6;
	2297	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.3;
	2297	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.3;
	2297	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.4;
	2297	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	2297	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.4;
	2298	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.2;
	2298	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.3;
	2298	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.3;
	2298	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224;
	2298	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	2298	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.8;
	2299	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.5;
	2299	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153;
	2299	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224;
	2299	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.5;
	2299	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	2299	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.1;
	2300	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.8;
	2300	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.2;
	2300	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230;
	2300	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.2;
	2300	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	2300	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.6;
	2301	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.3;
	2301	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156;
	2301	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.7;
	2301	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.3;
	2301	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	2301	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.6;
	2302	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.8;
	2302	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.5;
	2302	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.2;
	2302	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.1;
	2302	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.8;
	2302	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.8;
	2303	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.7;
	2303	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.5;
	2303	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.4;
	2303	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.1;
	2303	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	2303	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.7;
	2304	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.6;
	2304	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.4;
	2304	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.2;
	2304	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.6;
	2304	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	2304	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.5;
	2305	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.7;
	2305	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.5;
	2305	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.8;
	2305	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.8;
	2305	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	2305	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.2;
	2306	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403.7;
	2306	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.3;
	2306	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.9;
	2306	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.2;
	2306	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	2306	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.3;
	2307	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	402;
	2307	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.1;
	2307	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.2;
	2307	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.4;
	2307	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	2307	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323;
	2308	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.3;
	2308	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.7;
	2308	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.2;
	2308	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.8;
	2308	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	2308	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.1;
	2309	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	426.1;
	2309	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.9;
	2309	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.3;
	2309	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.2;
	2309	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	2309	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.7;
	2310	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442.7;
	2310	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.6;
	2310	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.4;
	2310	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204;
	2310	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	2310	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.8;
	2311	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.7;
	2311	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.8;
	2311	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.2;
	2311	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.3;
	2311	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.1;
	2311	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362;
	2312	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490;
	2312	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.4;
	2312	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.8;
	2312	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.3;
	2312	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.1;
	2312	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.3;
	2313	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.7;
	2313	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.3;
	2313	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.3;
	2313	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.4;
	2313	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.6;
	2313	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.8;
	2314	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.8;
	2314	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.8;
	2314	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.6;
	2314	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.4;
	2314	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.8;
	2314	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.8;
	2315	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.9;
	2315	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.5;
	2315	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.1;
	2315	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.5;
	2315	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.5;
	2315	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.1;
	2316	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.2;
	2316	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.9;
	2316	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.9;
	2316	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.8;
	2316	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.7;
	2316	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.3;
	2317	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.9;
	2317	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.5;
	2317	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.1;
	2317	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.2;
	2317	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.3;
	2317	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360;
	2318	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461;
	2318	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146;
	2318	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.8;
	2318	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.2;
	2318	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	2318	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.3;
	2319	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.8;
	2319	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144;
	2319	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.6;
	2319	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.6;
	2319	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	2319	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.2;
	2320	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.4;
	2320	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.5;
	2320	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.9;
	2320	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.1;
	2320	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	2320	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356;
	2321	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442.7;
	2321	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.3;
	2321	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.7;
	2321	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.2;
	2321	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	2321	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.7;
	2322	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.5;
	2322	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.7;
	2322	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.8;
	2322	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.7;
	2322	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	2322	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.1;
	2323	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433;
	2323	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.4;
	2323	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.7;
	2323	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.5;
	2323	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	2323	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.2;
	2324	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.9;
	2324	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152;
	2324	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227;
	2324	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.9;
	2324	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	2324	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.6;
	2325	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.7;
	2325	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.1;
	2325	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.2;
	2325	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.3;
	2325	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	2325	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.7;
	2326	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.4;
	2326	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.4;
	2326	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.1;
	2326	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219;
	2326	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	2326	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.3;
	2327	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.7;
	2327	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.7;
	2327	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.7;
	2327	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.4;
	2327	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	2327	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.4;
	2328	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417.1;
	2328	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.1;
	2328	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.7;
	2328	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206;
	2328	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	2328	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.6;
	2329	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403.2;
	2329	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.3;
	2329	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.6;
	2329	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.4;
	2329	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.9;
	2329	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.4;
	2330	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.7;
	2330	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.2;
	2330	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.6;
	2330	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.3;
	2330	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.7;
	2330	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.4;
	2331	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.9;
	2331	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.9;
	2331	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.6;
	2331	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.4;
	2331	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.6;
	2331	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.1;
	2332	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397;
	2332	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.7;
	2332	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.4;
	2332	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.7;
	2332	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.4;
	2332	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.2;
	2333	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.4;
	2333	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.7;
	2333	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186;
	2333	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.4;
	2333	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56;
	2333	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.6;
	2334	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.8;
	2334	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.7;
	2334	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.3;
	2334	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.1;
	2334	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.7;
	2334	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336;
	2335	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.4;
	2335	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.2;
	2335	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.1;
	2335	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.4;
	2335	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.9;
	2335	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.1;
	2336	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	444.2;
	2336	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.2;
	2336	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.7;
	2336	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.3;
	2336	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	2336	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.6;
	2337	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.2;
	2337	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.1;
	2337	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.5;
	2337	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.4;
	2337	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	2337	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.6;
	2338	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.3;
	2338	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.5;
	2338	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.6;
	2338	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207;
	2338	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	2338	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355;
	2339	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.6;
	2339	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.9;
	2339	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.2;
	2339	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.9;
	2339	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	2339	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.8;
	2340	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.2;
	2340	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.8;
	2340	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.8;
	2340	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.1;
	2340	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	2340	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.4;
	2341	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.8;
	2341	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.2;
	2341	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.4;
	2341	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.4;
	2341	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	2341	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.1;
	2342	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.4;
	2342	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.4;
	2342	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.9;
	2342	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.7;
	2342	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	2342	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.6;
	2343	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.4;
	2343	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.7;
	2343	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.3;
	2343	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.6;
	2343	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	2343	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.8;
	2344	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.2;
	2344	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.4;
	2344	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.8;
	2344	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.8;
	2344	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	2344	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336;
	2345	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.9;
	2345	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.9;
	2345	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.5;
	2345	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.5;
	2345	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	2345	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.6;
	2346	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.1;
	2346	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.8;
	2346	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.9;
	2346	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.7;
	2346	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	2346	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338;
	2347	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423;
	2347	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.9;
	2347	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.3;
	2347	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.2;
	2347	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.3;
	2347	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.9;
	2348	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.9;
	2348	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.2;
	2348	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.9;
	2348	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.7;
	2348	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	2348	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.4;
	2349	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.8;
	2349	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.4;
	2349	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.6;
	2349	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.2;
	2349	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	2349	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.9;
	2350	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432;
	2350	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.4;
	2350	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210;
	2350	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.2;
	2350	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	2350	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.1;
	2351	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.9;
	2351	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.2;
	2351	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.4;
	2351	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.3;
	2351	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61;
	2351	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.8;
	2352	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.3;
	2352	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.5;
	2352	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.9;
	2352	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.5;
	2352	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60;
	2352	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.8;
	2353	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	385.4;
	2353	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.5;
	2353	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.8;
	2353	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.5;
	2353	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.4;
	2353	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.8;
	2354	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	378.5;
	2354	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.4;
	2354	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.6;
	2354	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187;
	2354	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.8;
	2354	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.2;
	2355	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379.7;
	2355	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122;
	2355	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.2;
	2355	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.3;
	2355	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.3;
	2355	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.9;
	2356	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.4;
	2356	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.9;
	2356	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.4;
	2356	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.6;
	2356	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.3;
	2356	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.5;
	2357	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	387;
	2357	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.3;
	2357	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.8;
	2357	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187;
	2357	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.8;
	2357	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.6;
	2358	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	400.5;
	2358	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.6;
	2358	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.2;
	2358	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.5;
	2358	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.8;
	2358	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.4;
	2359	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.4;
	2359	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.8;
	2359	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.6;
	2359	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.9;
	2359	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	2359	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.1;
	2360	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.7;
	2360	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.8;
	2360	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.7;
	2360	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.9;
	2360	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	2360	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.7;
	2361	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.7;
	2361	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.3;
	2361	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.7;
	2361	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.6;
	2361	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	2361	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.3;
	2362	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.2;
	2362	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.7;
	2362	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.5;
	2362	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.5;
	2362	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.6;
	2362	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.8;
	2363	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.4;
	2363	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.4;
	2363	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.6;
	2363	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.2;
	2363	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.5;
	2363	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.2;
	2364	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.8;
	2364	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.2;
	2364	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.3;
	2364	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.7;
	2364	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.2;
	2364	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.7;
	2365	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.1;
	2365	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.7;
	2365	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191;
	2365	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.5;
	2365	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	2365	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.4;
	2366	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	401.4;
	2366	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141;
	2366	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.8;
	2366	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.9;
	2366	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	2366	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326;
	2367	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392;
	2367	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137;
	2367	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.4;
	2367	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.4;
	2367	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	2367	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.5;
	2368	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	384.9;
	2368	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.2;
	2368	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.3;
	2368	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.5;
	2368	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	2368	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.3;
	2369	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	389.7;
	2369	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.4;
	2369	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.4;
	2369	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.8;
	2369	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	2369	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.7;
	2370	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	385.5;
	2370	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136;
	2370	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.3;
	2370	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.9;
	2370	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	2370	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.3;
	2371	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.5;
	2371	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.3;
	2371	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.4;
	2371	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.1;
	2371	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.6;
	2371	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.8;
	2372	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.2;
	2372	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.9;
	2372	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.7;
	2372	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.6;
	2372	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	2372	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.3;
	2373	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.4;
	2373	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.9;
	2373	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.4;
	2373	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.6;
	2373	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	2373	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.8;
	2374	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.3;
	2374	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.3;
	2374	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.3;
	2374	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.5;
	2374	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	2374	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.3;
	2375	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.5;
	2375	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.6;
	2375	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.5;
	2375	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.5;
	2375	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	2375	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331;
	2376	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.7;
	2376	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.2;
	2376	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.3;
	2376	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198;
	2376	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	2376	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.9;
	2377	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379.7;
	2377	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120;
	2377	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.2;
	2377	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.5;
	2377	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	2377	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.7;
	2378	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	366.1;
	2378	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.8;
	2378	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.1;
	2378	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.9;
	2378	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.9;
	2378	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308;
	2379	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	365.3;
	2379	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.3;
	2379	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181;
	2379	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.6;
	2379	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	2379	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	307.3;
	2380	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	367.9;
	2380	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.2;
	2380	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.1;
	2380	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.7;
	2380	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.6;
	2380	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.5;
	2381	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379.4;
	2381	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.2;
	2381	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.6;
	2381	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.2;
	2381	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	2381	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.4;
	2382	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403.9;
	2382	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.8;
	2382	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.5;
	2382	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.5;
	2382	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.4;
	2382	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.9;
	2383	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	444.3;
	2383	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154;
	2383	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.4;
	2383	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.9;
	2383	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.7;
	2383	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.7;
	2384	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.6;
	2384	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.3;
	2384	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.6;
	2384	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.4;
	2384	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	2384	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.8;
	2385	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.1;
	2385	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162;
	2385	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.1;
	2385	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228;
	2385	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	2385	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.4;
	2386	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.3;
	2386	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.6;
	2386	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.5;
	2386	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.8;
	2386	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.9;
	2386	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.1;
	2387	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.7;
	2387	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.5;
	2387	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.3;
	2387	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.5;
	2387	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.2;
	2387	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.2;
	2388	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.2;
	2388	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.6;
	2388	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.8;
	2388	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.1;
	2388	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	2388	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.1;
	2389	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.8;
	2389	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.7;
	2389	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.5;
	2389	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.8;
	2389	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.6;
	2389	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.1;
	2390	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.4;
	2390	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.8;
	2390	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.3;
	2390	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.3;
	2390	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.6;
	2390	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.8;
	2391	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.1;
	2391	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.9;
	2391	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.4;
	2391	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229;
	2391	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.5;
	2391	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.8;
	2392	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.8;
	2392	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.6;
	2392	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.5;
	2392	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.6;
	2392	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	2392	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.4;
	2393	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.4;
	2393	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.9;
	2393	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.3;
	2393	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.8;
	2393	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.5;
	2393	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.1;
	2394	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467;
	2394	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.2;
	2394	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.8;
	2394	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.5;
	2394	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.2;
	2394	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.1;
	2395	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.9;
	2395	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.2;
	2395	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.9;
	2395	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232;
	2395	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	2395	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.5;
	2396	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.6;
	2396	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.6;
	2396	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.7;
	2396	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.4;
	2396	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.3;
	2396	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.8;
	2397	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.3;
	2397	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.4;
	2397	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.3;
	2397	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232;
	2397	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.9;
	2397	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.7;
	2398	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.3;
	2398	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151;
	2398	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.7;
	2398	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.7;
	2398	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70;
	2398	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.9;
	2399	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.3;
	2399	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140;
	2399	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.5;
	2399	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.1;
	2399	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.4;
	2399	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.2;
	2400	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.3;
	2400	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.8;
	2400	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.2;
	2400	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.4;
	2400	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.4;
	2400	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.6;
	2401	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.3;
	2401	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.6;
	2401	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.5;
	2401	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.7;
	2401	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	2401	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.5;
	2402	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.3;
	2402	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.1;
	2402	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.9;
	2402	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.9;
	2402	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	2402	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.8;
	2403	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.4;
	2403	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.2;
	2403	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.1;
	2403	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188;
	2403	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	2403	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.8;
	2404	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.2;
	2404	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.2;
	2404	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.7;
	2404	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.4;
	2404	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	2404	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.7;
	2405	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.5;
	2405	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.3;
	2405	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.9;
	2405	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.9;
	2405	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	2405	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.9;
	2406	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.7;
	2406	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.5;
	2406	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.4;
	2406	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.9;
	2406	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	2406	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.3;
	2407	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.9;
	2407	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.2;
	2407	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.8;
	2407	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.8;
	2407	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69;
	2407	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.6;
	2408	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515;
	2408	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.8;
	2408	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.6;
	2408	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.8;
	2408	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.3;
	2408	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.7;
	2409	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	524.8;
	2409	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.6;
	2409	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.9;
	2409	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.2;
	2409	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.9;
	2409	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.1;
	2410	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.5;
	2410	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155;
	2410	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.8;
	2410	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.2;
	2410	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.9;
	2410	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.2;
	2411	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.7;
	2411	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.2;
	2411	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.8;
	2411	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.5;
	2411	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	2411	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380;
	2412	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.6;
	2412	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.6;
	2412	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.9;
	2412	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.5;
	2412	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.1;
	2412	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.2;
	2413	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.8;
	2413	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.8;
	2413	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.2;
	2413	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.6;
	2413	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.1;
	2413	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.3;
	2414	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.5;
	2414	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.2;
	2414	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224;
	2414	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.2;
	2414	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	2414	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.5;
	2415	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.6;
	2415	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.2;
	2415	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.7;
	2415	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.3;
	2415	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	2415	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.4;
	2416	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.6;
	2416	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.3;
	2416	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.3;
	2416	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.6;
	2416	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	2416	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.7;
	2417	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.2;
	2417	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.8;
	2417	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.4;
	2417	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.7;
	2417	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.6;
	2417	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.2;
	2418	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.4;
	2418	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.2;
	2418	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.8;
	2418	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.3;
	2418	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	2418	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.9;
	2419	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.5;
	2419	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.5;
	2419	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.7;
	2419	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.5;
	2419	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	2419	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.3;
	2420	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.3;
	2420	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.8;
	2420	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.1;
	2420	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.3;
	2420	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	2420	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.2;
	2421	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.3;
	2421	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160;
	2421	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.1;
	2421	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.4;
	2421	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	2421	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.6;
	2422	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481;
	2422	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.5;
	2422	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.4;
	2422	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.5;
	2422	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	2422	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.1;
	2423	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.6;
	2423	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.8;
	2423	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.7;
	2423	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.7;
	2423	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	2423	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.3;
	2424	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.8;
	2424	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.3;
	2424	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.8;
	2424	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.6;
	2424	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	2424	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.5;
	2425	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	426.2;
	2425	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.1;
	2425	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.6;
	2425	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.7;
	2425	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	2425	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.3;
	2426	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424;
	2426	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.9;
	2426	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.3;
	2426	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.7;
	2426	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.4;
	2426	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.9;
	2427	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.9;
	2427	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122;
	2427	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.4;
	2427	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.8;
	2427	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.8;
	2427	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.3;
	2428	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428.7;
	2428	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.9;
	2428	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.5;
	2428	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.9;
	2428	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.4;
	2428	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.7;
	2429	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.1;
	2429	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131;
	2429	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.9;
	2429	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.8;
	2429	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.4;
	2429	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.8;
	2430	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.1;
	2430	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.5;
	2430	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.8;
	2430	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.2;
	2430	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	2430	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.2;
	2431	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.7;
	2431	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.8;
	2431	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211;
	2431	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.2;
	2431	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	2431	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.5;
	2432	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.2;
	2432	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.5;
	2432	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.5;
	2432	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.1;
	2432	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	2432	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.6;
	2433	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.9;
	2433	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162;
	2433	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.7;
	2433	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.5;
	2433	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	2433	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.5;
	2434	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.7;
	2434	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.6;
	2434	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.6;
	2434	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.5;
	2434	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.1;
	2434	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.8;
	2435	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506;
	2435	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159;
	2435	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225;
	2435	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231;
	2435	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	2435	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.7;
	2436	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.5;
	2436	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.9;
	2436	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.1;
	2436	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231;
	2436	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.8;
	2436	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.6;
	2437	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.8;
	2437	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.5;
	2437	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.1;
	2437	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.8;
	2437	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	2437	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371;
	2438	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.2;
	2438	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.2;
	2438	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.1;
	2438	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.5;
	2438	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	2438	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.5;
	2439	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.2;
	2439	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.7;
	2439	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.6;
	2439	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.5;
	2439	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	2439	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.1;
	2440	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.7;
	2440	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154;
	2440	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.5;
	2440	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.8;
	2440	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	2440	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.2;
	2441	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.7;
	2441	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.6;
	2441	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.3;
	2441	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.7;
	2441	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.3;
	2441	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359;
	2442	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.7;
	2442	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.6;
	2442	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221;
	2442	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226;
	2442	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	2442	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.5;
	2443	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.7;
	2443	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.8;
	2443	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.8;
	2443	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.6;
	2443	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	2443	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.1;
	2444	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.1;
	2444	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163;
	2444	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.5;
	2444	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235;
	2444	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	2444	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.1;
	2445	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.5;
	2445	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.6;
	2445	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.3;
	2445	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.2;
	2445	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	2445	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.9;
	2446	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.7;
	2446	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.2;
	2446	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218;
	2446	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.4;
	2446	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	2446	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.9;
	2447	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.2;
	2447	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.4;
	2447	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.3;
	2447	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.5;
	2447	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61;
	2447	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.4;
	2448	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.4;
	2448	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.9;
	2448	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.2;
	2448	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.3;
	2448	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	2448	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.6;
	2449	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.6;
	2449	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.2;
	2449	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.9;
	2449	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.4;
	2449	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.8;
	2449	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.2;
	2450	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	386.1;
	2450	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.1;
	2450	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.2;
	2450	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.6;
	2450	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.4;
	2450	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.2;
	2451	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392.9;
	2451	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.3;
	2451	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.6;
	2451	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188;
	2451	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.5;
	2451	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.5;
	2452	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392.9;
	2452	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.5;
	2452	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.5;
	2452	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.2;
	2452	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.8;
	2452	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.8;
	2453	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.1;
	2453	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.9;
	2453	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.7;
	2453	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.6;
	2453	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.2;
	2453	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.1;
	2454	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	444.8;
	2454	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.3;
	2454	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.2;
	2454	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.8;
	2454	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.9;
	2454	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.6;
	2455	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.9;
	2455	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.8;
	2455	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210;
	2455	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.1;
	2455	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.4;
	2455	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.8;
	2456	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.8;
	2456	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.5;
	2456	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.2;
	2456	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.1;
	2456	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	2456	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.5;
	2457	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.3;
	2457	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.4;
	2457	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.7;
	2457	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.2;
	2457	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.2;
	2457	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.9;
	2458	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.5;
	2458	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.8;
	2458	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.6;
	2458	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.6;
	2458	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	2458	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374;
	2459	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.1;
	2459	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.4;
	2459	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.2;
	2459	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.2;
	2459	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	2459	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.4;
	2460	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.4;
	2460	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.9;
	2460	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.2;
	2460	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.6;
	2460	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	2460	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.5;
	2461	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.1;
	2461	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.4;
	2461	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.2;
	2461	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.9;
	2461	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	2461	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.3;
	2462	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.6;
	2462	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.1;
	2462	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.3;
	2462	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.8;
	2462	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	2462	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.8;
	2463	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.5;
	2463	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152;
	2463	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.7;
	2463	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.7;
	2463	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	2463	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.3;
	2464	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.5;
	2464	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.6;
	2464	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.2;
	2464	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.3;
	2464	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	2464	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.5;
	2465	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462;
	2465	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.4;
	2465	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.9;
	2465	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.2;
	2465	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	2465	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.5;
	2466	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.3;
	2466	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.3;
	2466	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.9;
	2466	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.1;
	2466	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	2466	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.8;
	2467	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.6;
	2467	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.2;
	2467	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.2;
	2467	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.4;
	2467	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	2467	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.9;
	2468	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.5;
	2468	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162;
	2468	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.1;
	2468	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.9;
	2468	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.7;
	2468	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.2;
	2469	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.6;
	2469	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.8;
	2469	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.7;
	2469	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.4;
	2469	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	2469	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.6;
	2470	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.1;
	2470	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.7;
	2470	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.7;
	2470	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.5;
	2470	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	2470	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.2;
	2471	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.8;
	2471	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.7;
	2471	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.8;
	2471	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.3;
	2471	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	2471	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.6;
	2472	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428.2;
	2472	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.7;
	2472	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.1;
	2472	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.1;
	2472	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	2472	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.7;
	2473	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	420.7;
	2473	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.4;
	2473	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.9;
	2473	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.4;
	2473	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.9;
	2473	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.3;
	2474	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.5;
	2474	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.2;
	2474	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.5;
	2474	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.7;
	2474	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.4;
	2474	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.3;
	2475	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.3;
	2475	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.2;
	2475	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.5;
	2475	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.8;
	2475	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55;
	2475	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.1;
	2476	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	406.5;
	2476	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121;
	2476	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.2;
	2476	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.6;
	2476	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.6;
	2476	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325;
	2477	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424;
	2477	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.6;
	2477	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.8;
	2477	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.4;
	2477	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56;
	2477	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.7;
	2478	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.3;
	2478	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.1;
	2478	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.4;
	2478	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.4;
	2478	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.5;
	2478	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.9;
	2479	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.5;
	2479	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.4;
	2479	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.6;
	2479	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.8;
	2479	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	2479	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.9;
	2480	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.8;
	2480	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159;
	2480	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.8;
	2480	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.9;
	2480	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	2480	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.7;
	2481	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.5;
	2481	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158;
	2481	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.4;
	2481	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.8;
	2481	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.2;
	2481	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.6;
	2482	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.9;
	2482	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.1;
	2482	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.4;
	2482	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.4;
	2482	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.5;
	2482	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.7;
	2483	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.3;
	2483	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158;
	2483	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.6;
	2483	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.4;
	2483	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	2483	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.1;
	2484	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.8;
	2484	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155;
	2484	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.6;
	2484	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.4;
	2484	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	2484	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.4;
	2485	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.1;
	2485	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.6;
	2485	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.5;
	2485	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.3;
	2485	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	2485	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.1;
	2486	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.4;
	2486	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.1;
	2486	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.1;
	2486	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.9;
	2486	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	2486	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.4;
	2487	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.7;
	2487	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.6;
	2487	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.8;
	2487	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.4;
	2487	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	2487	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.1;
	2488	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.3;
	2488	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.8;
	2488	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.6;
	2488	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227;
	2488	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	2488	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.2;
	2489	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.3;
	2489	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.9;
	2489	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.2;
	2489	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.3;
	2489	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.7;
	2489	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.4;
	2490	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.9;
	2490	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.3;
	2490	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.6;
	2490	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.4;
	2490	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.6;
	2490	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359;
	2491	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.8;
	2491	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.2;
	2491	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.7;
	2491	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225;
	2491	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	2491	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.9;
	2492	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.2;
	2492	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154;
	2492	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.4;
	2492	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.6;
	2492	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	2492	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.2;
	2493	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.7;
	2493	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.9;
	2493	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.6;
	2493	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.3;
	2493	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	2493	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.6;
	2494	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.4;
	2494	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.5;
	2494	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.9;
	2494	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.4;
	2494	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	2494	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.5;
	2495	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.4;
	2495	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.6;
	2495	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.4;
	2495	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.8;
	2495	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	2495	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.3;
	2496	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	427.6;
	2496	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.2;
	2496	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.3;
	2496	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.2;
	2496	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	2496	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.3;
	2497	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.1;
	2497	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.7;
	2497	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.7;
	2497	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.8;
	2497	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.6;
	2497	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.6;
	2498	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.6;
	2498	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.3;
	2498	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.1;
	2498	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.5;
	2498	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.7;
	2498	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.6;
	2499	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.2;
	2499	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.7;
	2499	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.7;
	2499	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187;
	2499	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.2;
	2499	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.1;
	2500	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417.3;
	2500	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.8;
	2500	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.4;
	2500	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187;
	2500	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	2500	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.3;
	2501	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.1;
	2501	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.3;
	2501	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.4;
	2501	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190;
	2501	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.8;
	2501	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.8;
	2502	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.8;
	2502	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.1;
	2502	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.4;
	2502	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195;
	2502	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	2502	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.4;
	2503	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454;
	2503	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.7;
	2503	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.3;
	2503	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.7;
	2503	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	2503	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.9;
	2504	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.3;
	2504	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.5;
	2504	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.3;
	2504	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.5;
	2504	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	2504	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.1;
	2505	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468;
	2505	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.3;
	2505	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.2;
	2505	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.4;
	2505	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.8;
	2505	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.6;
	2506	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.1;
	2506	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.3;
	2506	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.1;
	2506	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.3;
	2506	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	2506	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.9;
	2507	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483;
	2507	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.2;
	2507	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.6;
	2507	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.8;
	2507	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.1;
	2507	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.9;
	2508	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.3;
	2508	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146;
	2508	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.7;
	2508	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.3;
	2508	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	2508	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357;
	2509	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469;
	2509	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.3;
	2509	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.5;
	2509	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.1;
	2509	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	2509	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.3;
	2510	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.3;
	2510	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.6;
	2510	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.2;
	2510	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206;
	2510	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	2510	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.7;
	2511	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.4;
	2511	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.7;
	2511	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.2;
	2511	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.9;
	2511	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	2511	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.4;
	2512	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.9;
	2512	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.4;
	2512	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.5;
	2512	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.5;
	2512	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	2512	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.4;
	2513	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	444.8;
	2513	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.4;
	2513	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.3;
	2513	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.4;
	2513	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.6;
	2513	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.1;
	2514	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.9;
	2514	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.4;
	2514	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.8;
	2514	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.9;
	2514	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	2514	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.8;
	2515	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.3;
	2515	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.6;
	2515	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.1;
	2515	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.1;
	2515	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	2515	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.9;
	2516	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.4;
	2516	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.9;
	2516	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.7;
	2516	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.8;
	2516	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	2516	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.9;
	2517	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.8;
	2517	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.4;
	2517	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.9;
	2517	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.9;
	2517	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.4;
	2517	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.2;
	2518	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455;
	2518	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.5;
	2518	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.9;
	2518	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.8;
	2518	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	2518	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.8;
	2519	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428.3;
	2519	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.9;
	2519	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.9;
	2519	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.7;
	2519	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.3;
	2519	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.1;
	2520	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	400.8;
	2520	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.8;
	2520	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.4;
	2520	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.9;
	2520	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.3;
	2520	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.4;
	2521	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	387;
	2521	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.1;
	2521	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.4;
	2521	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190;
	2521	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.9;
	2521	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.7;
	2522	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	382.1;
	2522	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.7;
	2522	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.5;
	2522	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.4;
	2522	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.6;
	2522	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.9;
	2523	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	380.8;
	2523	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.8;
	2523	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.4;
	2523	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.1;
	2523	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.6;
	2523	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.4;
	2524	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	384;
	2524	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.3;
	2524	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.4;
	2524	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.1;
	2524	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.4;
	2524	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.5;
	2525	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392.4;
	2525	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.9;
	2525	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.1;
	2525	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.8;
	2525	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.7;
	2525	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.8;
	2526	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	406.9;
	2526	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.8;
	2526	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.6;
	2526	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.4;
	2526	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.8;
	2526	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.2;
	2527	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.6;
	2527	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.3;
	2527	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.3;
	2527	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190;
	2527	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	2527	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.4;
	2528	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.7;
	2528	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.9;
	2528	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.6;
	2528	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.5;
	2528	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	2528	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.5;
	2529	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.4;
	2529	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.1;
	2529	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.5;
	2529	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.8;
	2529	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.8;
	2529	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.9;
	2530	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.2;
	2530	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.4;
	2530	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193;
	2530	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.5;
	2530	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.4;
	2530	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.8;
	2531	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	430.2;
	2531	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.9;
	2531	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.7;
	2531	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.9;
	2531	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	2531	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.4;
	2532	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.8;
	2532	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.4;
	2532	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.5;
	2532	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.7;
	2532	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	2532	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.3;
	2533	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.7;
	2533	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.9;
	2533	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.6;
	2533	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.8;
	2533	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	2533	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.2;
	2534	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.2;
	2534	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.8;
	2534	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.5;
	2534	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.3;
	2534	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	2534	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.9;
	2535	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.2;
	2535	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.4;
	2535	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.6;
	2535	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.7;
	2535	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.3;
	2535	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.9;
	2536	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.3;
	2536	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.2;
	2536	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.6;
	2536	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.4;
	2536	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.6;
	2536	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.7;
	2537	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.6;
	2537	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132;
	2537	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.4;
	2537	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201;
	2537	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	2537	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.6;
	2538	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.6;
	2538	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.2;
	2538	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.7;
	2538	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.5;
	2538	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	2538	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.6;
	2539	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442.7;
	2539	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.5;
	2539	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.5;
	2539	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.9;
	2539	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	2539	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347;
	2540	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.3;
	2540	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.5;
	2540	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.1;
	2540	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.2;
	2540	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.3;
	2540	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.1;
	2541	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.5;
	2541	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.7;
	2541	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.5;
	2541	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.4;
	2541	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.2;
	2541	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.1;
	2542	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.6;
	2542	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.5;
	2542	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.8;
	2542	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.7;
	2542	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	2542	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.9;
	2543	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.7;
	2543	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.8;
	2543	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.4;
	2543	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.2;
	2543	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.3;
	2543	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.8;
	2544	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403.1;
	2544	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.7;
	2544	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.8;
	2544	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.5;
	2544	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.1;
	2544	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.5;
	2545	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	395.4;
	2545	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.1;
	2545	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.8;
	2545	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.8;
	2545	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.8;
	2545	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.7;
	2546	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	381.3;
	2546	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.9;
	2546	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181.6;
	2546	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.7;
	2546	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.2;
	2546	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.3;
	2547	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	374.5;
	2547	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110;
	2547	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.9;
	2547	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.4;
	2547	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55;
	2547	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311.1;
	2548	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	377;
	2548	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.6;
	2548	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181.8;
	2548	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.6;
	2548	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.5;
	2548	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.2;
	2549	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	382.2;
	2549	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116;
	2549	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.3;
	2549	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.4;
	2549	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.5;
	2549	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.5;
	2550	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394.5;
	2550	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.9;
	2550	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.9;
	2550	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.4;
	2550	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.5;
	2550	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.9;
	2551	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.2;
	2551	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.9;
	2551	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.5;
	2551	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.6;
	2551	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	2551	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.6;
	2552	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.7;
	2552	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.9;
	2552	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.7;
	2552	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.3;
	2552	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.2;
	2552	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.9;
	2553	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.5;
	2553	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.7;
	2553	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.7;
	2553	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.2;
	2553	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	2553	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.3;
	2554	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.9;
	2554	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.8;
	2554	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.3;
	2554	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.3;
	2554	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	2554	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.6;
	2555	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.4;
	2555	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.6;
	2555	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.6;
	2555	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.8;
	2555	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	2555	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.5;
	2556	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.7;
	2556	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.6;
	2556	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.9;
	2556	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.6;
	2556	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.2;
	2556	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.8;
	2557	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.4;
	2557	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.8;
	2557	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.4;
	2557	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.6;
	2557	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.4;
	2557	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.4;
	2558	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.4;
	2558	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146;
	2558	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.3;
	2558	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.8;
	2558	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.8;
	2558	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.4;
	2559	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.1;
	2559	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.9;
	2559	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.9;
	2559	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.3;
	2559	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	2559	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.3;
	2560	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.7;
	2560	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.9;
	2560	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.1;
	2560	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.1;
	2560	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	2560	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.6;
	2561	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504;
	2561	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.1;
	2561	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.2;
	2561	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240;
	2561	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.1;
	2561	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.6;
	2562	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.4;
	2562	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.1;
	2562	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.8;
	2562	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.8;
	2562	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.8;
	2562	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.7;
	2563	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.7;
	2563	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.8;
	2563	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.6;
	2563	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.5;
	2563	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.6;
	2563	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.5;
	2564	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.4;
	2564	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.8;
	2564	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.8;
	2564	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.1;
	2564	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.8;
	2564	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.4;
	2565	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.4;
	2565	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.4;
	2565	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.7;
	2565	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.4;
	2565	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	2565	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.9;
	2566	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.1;
	2566	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.1;
	2566	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.8;
	2566	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.3;
	2566	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.9;
	2566	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.1;
	2567	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.5;
	2567	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.3;
	2567	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.8;
	2567	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.3;
	2567	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58;
	2567	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.1;
	2568	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447;
	2568	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.3;
	2568	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.5;
	2568	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.9;
	2568	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.7;
	2568	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.7;
	2569	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.6;
	2569	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.7;
	2569	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.7;
	2569	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.9;
	2569	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.4;
	2569	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.3;
	2570	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.6;
	2570	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109;
	2570	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.8;
	2570	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.9;
	2570	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.8;
	2570	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.1;
	2571	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.6;
	2571	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.7;
	2571	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.8;
	2571	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.3;
	2571	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.2;
	2571	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.2;
	2572	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.6;
	2572	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.7;
	2572	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.2;
	2572	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.7;
	2572	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.2;
	2572	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.5;
	2573	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417.1;
	2573	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.8;
	2573	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.5;
	2573	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.5;
	2573	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.9;
	2573	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.7;
	2574	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.9;
	2574	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.7;
	2574	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.5;
	2574	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.8;
	2574	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.2;
	2574	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.8;
	2575	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.3;
	2575	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.6;
	2575	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.2;
	2575	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.2;
	2575	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.6;
	2575	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.5;
	2576	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.7;
	2576	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.2;
	2576	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.7;
	2576	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.9;
	2576	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	2576	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.5;
	2577	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.2;
	2577	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.1;
	2577	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.8;
	2577	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234;
	2577	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	2577	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.1;
	2578	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.5;
	2578	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147;
	2578	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.2;
	2578	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.8;
	2578	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	2578	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.4;
	2579	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515;
	2579	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.7;
	2579	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.2;
	2579	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.5;
	2579	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.2;
	2579	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.6;
	2580	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.5;
	2580	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.6;
	2580	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.8;
	2580	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.7;
	2580	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	2580	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.9;
	2581	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.4;
	2581	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.4;
	2581	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.1;
	2581	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.5;
	2581	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	2581	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.7;
	2582	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.3;
	2582	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.5;
	2582	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.6;
	2582	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.5;
	2582	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	2582	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.9;
	2583	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.7;
	2583	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.2;
	2583	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.9;
	2583	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.7;
	2583	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	2583	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.4;
	2584	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.6;
	2584	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143;
	2584	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.5;
	2584	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.8;
	2584	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.5;
	2584	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373;
	2585	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.8;
	2585	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.1;
	2585	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.3;
	2585	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.2;
	2585	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.4;
	2585	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.1;
	2586	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.8;
	2586	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.4;
	2586	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.9;
	2586	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.4;
	2586	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.4;
	2586	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.7;
	2587	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.5;
	2587	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.5;
	2587	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.8;
	2587	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.5;
	2587	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.6;
	2587	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.3;
	2588	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.5;
	2588	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.5;
	2588	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.5;
	2588	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.2;
	2588	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.1;
	2588	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.2;
	2589	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.4;
	2589	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.4;
	2589	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.4;
	2589	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.4;
	2589	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	2589	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.9;
	2590	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476;
	2590	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.7;
	2590	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.5;
	2590	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.2;
	2590	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	2590	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.8;
	2591	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.7;
	2591	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.6;
	2591	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.9;
	2591	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.3;
	2591	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	2591	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.6;
	2592	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	427.3;
	2592	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.9;
	2592	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.7;
	2592	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.5;
	2592	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.2;
	2592	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.2;
	2593	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.3;
	2593	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.5;
	2593	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.7;
	2593	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.6;
	2593	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.6;
	2593	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.9;
	2594	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	400.9;
	2594	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.9;
	2594	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.6;
	2594	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.2;
	2594	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.7;
	2594	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.2;
	2595	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403.4;
	2595	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.3;
	2595	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.3;
	2595	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187;
	2595	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.5;
	2595	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.8;
	2596	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415;
	2596	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.5;
	2596	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.9;
	2596	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.6;
	2596	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.5;
	2596	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.6;
	2597	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	426;
	2597	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.2;
	2597	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.5;
	2597	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.6;
	2597	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.8;
	2597	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.9;
	2598	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455;
	2598	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138;
	2598	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.7;
	2598	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.5;
	2598	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.9;
	2598	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.8;
	2599	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498;
	2599	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.9;
	2599	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.3;
	2599	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.9;
	2599	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.4;
	2599	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.1;
	2600	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.8;
	2600	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158;
	2600	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221;
	2600	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227;
	2600	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	2600	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.2;
	2601	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500;
	2601	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.2;
	2601	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.4;
	2601	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.3;
	2601	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	2601	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.7;
	2602	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.4;
	2602	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155;
	2602	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.8;
	2602	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.1;
	2602	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	2602	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.3;
	2603	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.9;
	2603	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.8;
	2603	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225;
	2603	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.3;
	2603	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	2603	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.5;
	2604	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.7;
	2604	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.6;
	2604	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.5;
	2604	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.3;
	2604	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	2604	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.5;
	2605	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.3;
	2605	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.6;
	2605	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.7;
	2605	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.4;
	2605	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	2605	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.3;
	2606	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.6;
	2606	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.9;
	2606	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.4;
	2606	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.1;
	2606	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	2606	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.9;
	2607	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.3;
	2607	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.3;
	2607	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.1;
	2607	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.5;
	2607	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	2607	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.5;
	2608	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.5;
	2608	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.3;
	2608	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.3;
	2608	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.6;
	2608	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	2608	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.5;
	2609	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	430.8;
	2609	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145;
	2609	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221;
	2609	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226;
	2609	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	2609	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.9;
	2610	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428.7;
	2610	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.5;
	2610	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.2;
	2610	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.7;
	2610	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	2610	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.8;
	2611	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.6;
	2611	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.4;
	2611	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.3;
	2611	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.6;
	2611	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	2611	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.7;
	2612	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.9;
	2612	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.2;
	2612	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.3;
	2612	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.5;
	2612	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	2612	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.1;
	2613	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.8;
	2613	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.6;
	2613	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.3;
	2613	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.4;
	2613	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	2613	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.1;
	2614	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.3;
	2614	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.2;
	2614	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.5;
	2614	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.4;
	2614	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	2614	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.2;
	2615	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.7;
	2615	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.5;
	2615	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.6;
	2615	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.8;
	2615	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61;
	2615	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.2;
	2616	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393;
	2616	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.5;
	2616	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.8;
	2616	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.9;
	2616	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	2616	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.5;
	2617	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	387;
	2617	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.3;
	2617	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.7;
	2617	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.5;
	2617	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.1;
	2617	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.4;
	2618	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	381.8;
	2618	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.6;
	2618	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.9;
	2618	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.5;
	2618	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.2;
	2618	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.1;
	2619	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	390.3;
	2619	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.4;
	2619	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.2;
	2619	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.8;
	2619	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.8;
	2619	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.9;
	2620	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	388.5;
	2620	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.6;
	2620	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.9;
	2620	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.5;
	2620	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51;
	2620	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.5;
	2621	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.3;
	2621	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.3;
	2621	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.2;
	2621	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.1;
	2621	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.1;
	2621	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.2;
	2622	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.4;
	2622	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.1;
	2622	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.7;
	2622	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.3;
	2622	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.8;
	2622	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.8;
	2623	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.2;
	2623	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.6;
	2623	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.6;
	2623	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.3;
	2623	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.9;
	2623	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.3;
	2624	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.7;
	2624	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.1;
	2624	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.8;
	2624	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.6;
	2624	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.2;
	2624	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.3;
	2625	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.8;
	2625	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.6;
	2625	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.2;
	2625	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.3;
	2625	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	2625	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.9;
	2626	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463;
	2626	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.1;
	2626	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.9;
	2626	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229;
	2626	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	2626	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363;
	2627	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.9;
	2627	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.3;
	2627	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.8;
	2627	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.5;
	2627	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	2627	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.4;
	2628	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.6;
	2628	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.7;
	2628	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.3;
	2628	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.2;
	2628	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	2628	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.3;
	2629	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.9;
	2629	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.8;
	2629	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.1;
	2629	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.9;
	2629	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	2629	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.6;
	2630	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.7;
	2630	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.7;
	2630	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.8;
	2630	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.6;
	2630	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	2630	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.8;
	2631	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.1;
	2631	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.8;
	2631	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225;
	2631	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.2;
	2631	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	2631	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357;
	2632	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.9;
	2632	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.5;
	2632	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.4;
	2632	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.5;
	2632	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	2632	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.3;
	2633	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.9;
	2633	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.9;
	2633	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.8;
	2633	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.6;
	2633	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	2633	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.5;
	2634	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.3;
	2634	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.6;
	2634	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.3;
	2634	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.7;
	2634	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	2634	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.3;
	2635	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.6;
	2635	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.8;
	2635	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.2;
	2635	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.8;
	2635	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	2635	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.3;
	2636	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.5;
	2636	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.9;
	2636	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.8;
	2636	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.8;
	2636	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	2636	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353;
	2637	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447.5;
	2637	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.1;
	2637	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.9;
	2637	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.8;
	2637	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	2637	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.6;
	2638	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.1;
	2638	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.4;
	2638	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.1;
	2638	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.6;
	2638	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	2638	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.8;
	2639	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.2;
	2639	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131;
	2639	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.4;
	2639	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.1;
	2639	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	2639	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.3;
	2640	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398;
	2640	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.4;
	2640	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.4;
	2640	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.8;
	2640	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.7;
	2640	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.7;
	2641	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	382.9;
	2641	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.4;
	2641	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.5;
	2641	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.3;
	2641	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.5;
	2641	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.1;
	2642	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	368;
	2642	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.4;
	2642	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.2;
	2642	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.1;
	2642	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.1;
	2642	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309.6;
	2643	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	375.4;
	2643	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112;
	2643	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.7;
	2643	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.5;
	2643	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.2;
	2643	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.1;
	2644	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	384.3;
	2644	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114;
	2644	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.8;
	2644	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.8;
	2644	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	49.9;
	2644	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.1;
	2645	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	395;
	2645	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.9;
	2645	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188;
	2645	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.3;
	2645	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	50.6;
	2645	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.2;
	2646	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.9;
	2646	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.3;
	2646	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.1;
	2646	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.4;
	2646	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	50.9;
	2646	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.8;
	2647	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447.5;
	2647	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.1;
	2647	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.5;
	2647	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.7;
	2647	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54;
	2647	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.3;
	2648	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.1;
	2648	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.4;
	2648	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.1;
	2648	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.8;
	2648	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.5;
	2648	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.5;
	2649	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.2;
	2649	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.1;
	2649	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.8;
	2649	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.4;
	2649	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.1;
	2649	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.4;
	2650	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.5;
	2650	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147;
	2650	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.8;
	2650	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.4;
	2650	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	2650	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.9;
	2651	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.7;
	2651	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.6;
	2651	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.7;
	2651	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.5;
	2651	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	2651	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.9;
	2652	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447.2;
	2652	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.7;
	2652	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.8;
	2652	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.3;
	2652	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	2652	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356;
	2653	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.5;
	2653	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.4;
	2653	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.1;
	2653	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.8;
	2653	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	2653	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.9;
	2654	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.6;
	2654	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.4;
	2654	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.5;
	2654	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227;
	2654	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	2654	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.2;
	2655	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	426.9;
	2655	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.3;
	2655	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.5;
	2655	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.4;
	2655	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	2655	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.4;
	2656	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	426.9;
	2656	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.1;
	2656	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.5;
	2656	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.6;
	2656	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	2656	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.1;
	2657	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.6;
	2657	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.9;
	2657	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.5;
	2657	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.3;
	2657	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	2657	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.4;
	2658	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.3;
	2658	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.8;
	2658	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.8;
	2658	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.7;
	2658	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.3;
	2658	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.5;
	2659	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419;
	2659	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.4;
	2659	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.1;
	2659	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.4;
	2659	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	2659	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.2;
	2660	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.3;
	2660	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.3;
	2660	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.7;
	2660	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.8;
	2660	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	2660	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.7;
	2661	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	420.3;
	2661	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.6;
	2661	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.9;
	2661	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.7;
	2661	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	2661	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.5;
	2662	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.2;
	2662	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.9;
	2662	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215;
	2662	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218;
	2662	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	2662	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339;
	2663	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.1;
	2663	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.2;
	2663	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.2;
	2663	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.9;
	2663	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.8;
	2663	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.4;
	2664	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	380.6;
	2664	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.6;
	2664	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.2;
	2664	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.8;
	2664	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58;
	2664	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.9;
	2665	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	369.4;
	2665	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.9;
	2665	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.1;
	2665	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.9;
	2665	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	2665	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.6;
	2666	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	354.7;
	2666	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.9;
	2666	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.3;
	2666	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.9;
	2666	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.6;
	2666	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	304.4;
	2667	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	359.7;
	2667	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	105.9;
	2667	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181.4;
	2667	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185;
	2667	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.7;
	2667	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	305.5;
	2668	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	365.8;
	2668	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.6;
	2668	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.8;
	2668	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.7;
	2668	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	50.9;
	2668	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308;
	2669	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379;
	2669	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.4;
	2669	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.6;
	2669	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.3;
	2669	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.7;
	2669	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.3;
	2670	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392.7;
	2670	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.5;
	2670	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.6;
	2670	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.9;
	2670	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.6;
	2670	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322;
	2671	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.1;
	2671	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.8;
	2671	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192;
	2671	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.5;
	2671	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55;
	2671	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.4;
	2672	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404;
	2672	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.1;
	2672	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.9;
	2672	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.4;
	2672	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.9;
	2672	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.6;
	2673	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.8;
	2673	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.1;
	2673	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.6;
	2673	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203;
	2673	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61;
	2673	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.3;
	2674	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	420.2;
	2674	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.1;
	2674	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.7;
	2674	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207;
	2674	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	2674	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.6;
	2675	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.4;
	2675	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.1;
	2675	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.2;
	2675	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.5;
	2675	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	2675	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.5;
	2676	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418;
	2676	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.8;
	2676	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.1;
	2676	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.2;
	2676	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	2676	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.8;
	2677	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.2;
	2677	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.3;
	2677	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.4;
	2677	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.9;
	2677	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	2677	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.5;
	2678	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	402.1;
	2678	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.2;
	2678	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.6;
	2678	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.2;
	2678	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	2678	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.8;
	2679	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	390.5;
	2679	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.4;
	2679	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.4;
	2679	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.9;
	2679	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	2679	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.2;
	2680	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	389.6;
	2680	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.2;
	2680	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.2;
	2680	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.2;
	2680	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	2680	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.2;
	2681	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392.9;
	2681	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.7;
	2681	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204;
	2681	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208;
	2681	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	2681	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.4;
	2682	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	387.9;
	2682	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.5;
	2682	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.6;
	2682	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.7;
	2682	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	2682	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.7;
	2683	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392.7;
	2683	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130;
	2683	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.6;
	2683	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.3;
	2683	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	2683	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.5;
	2684	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392.1;
	2684	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.8;
	2684	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.6;
	2684	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.1;
	2684	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	2684	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.6;
	2685	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	385.3;
	2685	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.9;
	2685	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.7;
	2685	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.5;
	2685	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	2685	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.6;
	2686	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	389.3;
	2686	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.2;
	2686	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.5;
	2686	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.1;
	2686	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	2686	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.1;
	2687	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	371.8;
	2687	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121;
	2687	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.9;
	2687	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.4;
	2687	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.2;
	2687	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.6;
	2688	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	360.7;
	2688	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.9;
	2688	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.9;
	2688	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.8;
	2688	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.6;
	2688	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.1;
	2689	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	347;
	2689	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.9;
	2689	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.7;
	2689	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.2;
	2689	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.7;
	2689	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	303.7;
	2690	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	335.3;
	2690	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.2;
	2690	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.8;
	2690	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.4;
	2690	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.3;
	2690	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	295.9;
	2691	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	330;
	2691	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.2;
	2691	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	178.7;
	2691	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	182.1;
	2691	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.9;
	2691	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	292.8;
	2692	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	333.5;
	2692	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.2;
	2692	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	178.8;
	2692	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	182.3;
	2692	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.5;
	2692	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	293.9;
	2693	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	341.7;
	2693	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.4;
	2693	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	179.7;
	2693	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	183.3;
	2693	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51;
	2693	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	297.9;
	2694	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	365.4;
	2694	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.5;
	2694	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.2;
	2694	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.7;
	2694	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.3;
	2694	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309;
	2695	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	376.5;
	2695	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.2;
	2695	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.4;
	2695	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187;
	2695	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.8;
	2695	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.2;
	2696	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	384.6;
	2696	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.3;
	2696	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.8;
	2696	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189;
	2696	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57;
	2696	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.7;
	2697	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	385.3;
	2697	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.1;
	2697	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.2;
	2697	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.1;
	2697	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.1;
	2697	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.7;
	2698	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	390.3;
	2698	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.3;
	2698	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190;
	2698	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.7;
	2698	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.3;
	2698	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.2;
	2699	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.5;
	2699	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.3;
	2699	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.8;
	2699	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.5;
	2699	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	2699	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324;
	2700	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	387;
	2700	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.6;
	2700	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.7;
	2700	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.5;
	2700	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	2700	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.6;
	2701	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392.1;
	2701	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.1;
	2701	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.8;
	2701	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.8;
	2701	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	2701	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.5;
	2702	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	382.1;
	2702	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.1;
	2702	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.9;
	2702	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.2;
	2702	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	2702	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.7;
	2703	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.7;
	2703	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.8;
	2703	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.9;
	2703	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.3;
	2703	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	2703	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.5;
	2704	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	374.9;
	2704	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.8;
	2704	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.8;
	2704	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.3;
	2704	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	2704	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.4;
	2705	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	384.1;
	2705	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.7;
	2705	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.2;
	2705	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.2;
	2705	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	2705	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.9;
	2706	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391.7;
	2706	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130;
	2706	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.8;
	2706	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.7;
	2706	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	2706	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.5;
	2707	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	385.7;
	2707	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.2;
	2707	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.3;
	2707	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.1;
	2707	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	2707	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.7;
	2708	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.6;
	2708	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.3;
	2708	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.6;
	2708	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219;
	2708	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	2708	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.2;
	2709	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.8;
	2709	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.1;
	2709	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.1;
	2709	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.1;
	2709	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	2709	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.9;
	2710	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.6;
	2710	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.3;
	2710	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.7;
	2710	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.9;
	2710	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.4;
	2710	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.6;
	2711	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392.9;
	2711	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.6;
	2711	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.1;
	2711	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201;
	2711	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.1;
	2711	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.4;
	2712	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	368.9;
	2712	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.8;
	2712	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.6;
	2712	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.9;
	2712	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	2712	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.2;
	2713	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	360.9;
	2713	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.3;
	2713	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.1;
	2713	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.2;
	2713	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.2;
	2713	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	307.1;
	2714	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	346.2;
	2714	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.9;
	2714	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.6;
	2714	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.5;
	2714	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.7;
	2714	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	299.5;
	2715	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	354.3;
	2715	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.5;
	2715	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	178.5;
	2715	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	182;
	2715	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.2;
	2715	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	302.1;
	2716	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	357.1;
	2716	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.2;
	2716	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.4;
	2716	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	183.9;
	2716	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.2;
	2716	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	303.5;
	2717	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	373.9;
	2717	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.2;
	2717	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.9;
	2717	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.7;
	2717	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.1;
	2717	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311.2;
	2718	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	401.7;
	2718	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.4;
	2718	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.9;
	2718	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.8;
	2718	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.2;
	2718	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325;
	2719	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.9;
	2719	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.8;
	2719	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.2;
	2719	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209;
	2719	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.7;
	2719	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.3;
	2720	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.8;
	2720	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.3;
	2720	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.4;
	2720	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.6;
	2720	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.8;
	2720	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.1;
	2721	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.4;
	2721	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.9;
	2721	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.7;
	2721	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.6;
	2721	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.1;
	2721	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.9;
	2722	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.5;
	2722	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.1;
	2722	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.1;
	2722	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.5;
	2722	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.7;
	2722	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.7;
	2723	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.1;
	2723	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.8;
	2723	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.7;
	2723	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.7;
	2723	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.7;
	2723	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.1;
	2724	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.5;
	2724	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.5;
	2724	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.9;
	2724	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.5;
	2724	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61;
	2724	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.1;
	2725	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.4;
	2725	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146;
	2725	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.6;
	2725	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.7;
	2725	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	2725	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.8;
	2726	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.1;
	2726	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.8;
	2726	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.6;
	2726	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.5;
	2726	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.6;
	2726	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.8;
	2727	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.1;
	2727	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.7;
	2727	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.7;
	2727	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.2;
	2727	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.2;
	2727	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.9;
	2728	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.1;
	2728	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.4;
	2728	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.8;
	2728	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232;
	2728	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	2728	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.3;
	2729	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.3;
	2729	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.4;
	2729	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.7;
	2729	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.8;
	2729	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59;
	2729	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.8;
	2730	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.7;
	2730	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.2;
	2730	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.1;
	2730	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.9;
	2730	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	2730	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361;
	2731	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.4;
	2731	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147;
	2731	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.5;
	2731	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.9;
	2731	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.1;
	2731	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.1;
	2732	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.3;
	2732	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.6;
	2732	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.4;
	2732	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.7;
	2732	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	2732	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.2;
	2733	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.2;
	2733	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.3;
	2733	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.3;
	2733	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.5;
	2733	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	2733	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362;
	2734	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.6;
	2734	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.6;
	2734	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.1;
	2734	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.5;
	2734	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	2734	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.7;
	2735	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.1;
	2735	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.1;
	2735	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.7;
	2735	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.4;
	2735	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.6;
	2735	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.1;
	2736	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403.3;
	2736	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.9;
	2736	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.9;
	2736	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.5;
	2736	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.2;
	2736	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.4;
	2737	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.8;
	2737	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.8;
	2737	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.6;
	2737	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.9;
	2737	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.4;
	2737	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.8;
	2738	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	387.2;
	2738	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.9;
	2738	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.7;
	2738	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.2;
	2738	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.1;
	2738	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.6;
	2739	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	390.8;
	2739	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.6;
	2739	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.4;
	2739	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.2;
	2739	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.5;
	2739	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.7;
	2740	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	389.6;
	2740	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.7;
	2740	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.9;
	2740	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.6;
	2740	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	50.8;
	2740	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.7;
	2741	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	401;
	2741	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.7;
	2741	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.9;
	2741	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191;
	2741	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	50.6;
	2741	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.5;
	2742	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	420.2;
	2742	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.2;
	2742	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.3;
	2742	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.8;
	2742	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.3;
	2742	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.6;
	2743	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.3;
	2743	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.4;
	2743	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.4;
	2743	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.8;
	2743	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.2;
	2743	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.1;
	2744	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.5;
	2744	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.1;
	2744	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220;
	2744	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.1;
	2744	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.7;
	2744	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.8;
	2745	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.9;
	2745	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.9;
	2745	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.3;
	2745	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.9;
	2745	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.6;
	2745	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.3;
	2746	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.1;
	2746	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.2;
	2746	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.7;
	2746	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.1;
	2746	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.1;
	2746	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.4;
	2747	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.5;
	2747	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.1;
	2747	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.8;
	2747	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234;
	2747	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.1;
	2747	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.2;
	2748	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.1;
	2748	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.8;
	2748	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.3;
	2748	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233;
	2748	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.8;
	2748	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.9;
	2749	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454;
	2749	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145;
	2749	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.4;
	2749	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.2;
	2749	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	2749	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.4;
	2750	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.5;
	2750	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.2;
	2750	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.5;
	2750	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.9;
	2750	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	2750	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.4;
	2751	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.4;
	2751	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.3;
	2751	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.7;
	2751	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.9;
	2751	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.7;
	2751	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.1;
	2752	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447.3;
	2752	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.3;
	2752	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.8;
	2752	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.8;
	2752	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.3;
	2752	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.1;
	2753	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.6;
	2753	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.5;
	2753	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.1;
	2753	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.5;
	2753	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.5;
	2753	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.8;
	2754	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.1;
	2754	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143;
	2754	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.5;
	2754	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.9;
	2754	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.4;
	2754	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353;
	2755	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.9;
	2755	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.3;
	2755	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.8;
	2755	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.8;
	2755	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	2755	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.6;
	2756	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.4;
	2756	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.6;
	2756	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.9;
	2756	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.8;
	2756	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	2756	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.7;
	2757	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.4;
	2757	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.3;
	2757	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.8;
	2757	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.5;
	2757	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	2757	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.1;
	2758	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.9;
	2758	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.2;
	2758	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.7;
	2758	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.2;
	2758	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	2758	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348;
	2759	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.9;
	2759	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130;
	2759	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.2;
	2759	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.5;
	2759	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.6;
	2759	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.1;
	2760	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	402.8;
	2760	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.7;
	2760	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.6;
	2760	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.4;
	2760	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.3;
	2760	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.3;
	2761	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	395.2;
	2761	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.4;
	2761	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.6;
	2761	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193;
	2761	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.1;
	2761	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.4;
	2762	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	381;
	2762	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.3;
	2762	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181.1;
	2762	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.1;
	2762	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.4;
	2762	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.2;
	2763	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	386;
	2763	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.4;
	2763	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181.6;
	2763	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.7;
	2763	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	50.9;
	2763	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315;
	2764	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	388.1;
	2764	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.7;
	2764	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182;
	2764	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.1;
	2764	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51;
	2764	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.2;
	2765	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.1;
	2765	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.9;
	2765	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.9;
	2765	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.1;
	2765	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	50.8;
	2765	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.8;
	2766	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.3;
	2766	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136;
	2766	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.2;
	2766	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.2;
	2766	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.1;
	2766	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.4;
	2767	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.3;
	2767	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.6;
	2767	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.4;
	2767	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.3;
	2767	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.7;
	2767	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.9;
	2768	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.9;
	2768	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.7;
	2768	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.7;
	2768	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.8;
	2768	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.6;
	2768	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.9;
	2769	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.1;
	2769	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.9;
	2769	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.9;
	2769	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.5;
	2769	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	2769	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363;
	2770	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.6;
	2770	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.8;
	2770	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.6;
	2770	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.3;
	2770	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	2770	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.3;
	2771	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.6;
	2771	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.3;
	2771	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.6;
	2771	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.8;
	2771	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	2771	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.4;
	2772	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.3;
	2772	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.2;
	2772	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.9;
	2772	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.2;
	2772	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	2772	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.3;
	2773	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.7;
	2773	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146;
	2773	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.3;
	2773	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.9;
	2773	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	2773	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.2;
	2774	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.7;
	2774	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.1;
	2774	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.7;
	2774	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.2;
	2774	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	2774	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.1;
	2775	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.1;
	2775	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.7;
	2775	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.9;
	2775	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.8;
	2775	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	2775	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.3;
	2776	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.6;
	2776	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.1;
	2776	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.6;
	2776	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.7;
	2776	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	2776	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.1;
	2777	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.1;
	2777	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.9;
	2777	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.3;
	2777	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.7;
	2777	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	2777	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.4;
	2778	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.6;
	2778	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.3;
	2778	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.7;
	2778	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.4;
	2778	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	2778	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.6;
	2779	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.7;
	2779	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.6;
	2779	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.6;
	2779	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.3;
	2779	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	2779	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.1;
	2780	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.8;
	2780	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.7;
	2780	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.4;
	2780	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.2;
	2780	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	2780	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358;
	2781	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.1;
	2781	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.2;
	2781	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.1;
	2781	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.8;
	2781	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	2781	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362;
	2782	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.7;
	2782	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.4;
	2782	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.8;
	2782	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.2;
	2782	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	2782	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.7;
	2783	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	444.5;
	2783	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131;
	2783	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.2;
	2783	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209;
	2783	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	2783	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.6;
	2784	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.5;
	2784	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.6;
	2784	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197;
	2784	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.4;
	2784	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	2784	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.6;
	2785	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.7;
	2785	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.1;
	2785	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.2;
	2785	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.4;
	2785	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.4;
	2785	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.1;
	2786	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.4;
	2786	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.5;
	2786	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182;
	2786	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.2;
	2786	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.2;
	2786	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.4;
	2787	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.8;
	2787	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.6;
	2787	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181.5;
	2787	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.6;
	2787	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.9;
	2787	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.6;
	2788	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421;
	2788	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.5;
	2788	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.7;
	2788	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.4;
	2788	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.7;
	2788	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.9;
	2789	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.4;
	2789	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.6;
	2789	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.1;
	2789	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.1;
	2789	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.1;
	2789	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.1;
	2790	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.9;
	2790	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.3;
	2790	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.8;
	2790	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.3;
	2790	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52;
	2790	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.6;
	2791	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.3;
	2791	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.9;
	2791	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.9;
	2791	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.8;
	2791	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.1;
	2791	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.1;
	2792	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.1;
	2792	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.9;
	2792	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.3;
	2792	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.2;
	2792	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.6;
	2792	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.8;
	2793	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.7;
	2793	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.4;
	2793	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.9;
	2793	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.1;
	2793	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	2793	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372;
	2794	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.6;
	2794	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.1;
	2794	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.6;
	2794	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.4;
	2794	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	2794	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.9;
	2795	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.9;
	2795	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.5;
	2795	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.6;
	2795	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.6;
	2795	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	2795	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.1;
	2796	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.3;
	2796	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.7;
	2796	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.2;
	2796	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.5;
	2796	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	2796	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.4;
	2797	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.3;
	2797	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.7;
	2797	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.6;
	2797	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.5;
	2797	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	2797	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.9;
	2798	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459;
	2798	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.3;
	2798	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.5;
	2798	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.1;
	2798	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.4;
	2798	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.2;
	2799	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.4;
	2799	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.3;
	2799	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.9;
	2799	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.2;
	2799	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	2799	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.9;
	2800	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.1;
	2800	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.7;
	2800	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223;
	2800	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.1;
	2800	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.1;
	2800	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.8;
	2801	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.8;
	2801	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.6;
	2801	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.1;
	2801	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.7;
	2801	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.4;
	2801	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.2;
	2802	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.6;
	2802	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148;
	2802	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.3;
	2802	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.5;
	2802	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.1;
	2802	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.1;
	2803	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.2;
	2803	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.6;
	2803	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.1;
	2803	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.5;
	2803	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.9;
	2803	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.5;
	2804	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465;
	2804	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.9;
	2804	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.2;
	2804	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.5;
	2804	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	2804	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.5;
	2805	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.1;
	2805	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.5;
	2805	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228;
	2805	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.4;
	2805	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.9;
	2805	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.3;
	2806	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.7;
	2806	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.9;
	2806	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218;
	2806	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224;
	2806	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.4;
	2806	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.3;
	2807	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.2;
	2807	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.8;
	2807	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.5;
	2807	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.7;
	2807	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	2807	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.1;
	2808	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.7;
	2808	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.9;
	2808	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.2;
	2808	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.9;
	2808	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	2808	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.4;
	2809	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403.6;
	2809	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.3;
	2809	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.8;
	2809	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.2;
	2809	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	2809	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.5;
	2810	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	388.5;
	2810	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.5;
	2810	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.7;
	2810	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.7;
	2810	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.3;
	2810	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.8;
	2811	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.7;
	2811	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.8;
	2811	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.2;
	2811	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.9;
	2811	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.2;
	2811	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.3;
	2812	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408;
	2812	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118;
	2812	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.5;
	2812	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.3;
	2812	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.2;
	2812	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.2;
	2813	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	427;
	2813	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.9;
	2813	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.8;
	2813	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.8;
	2813	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.1;
	2813	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.4;
	2814	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.6;
	2814	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.7;
	2814	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.9;
	2814	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.2;
	2814	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.7;
	2814	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.5;
	2815	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.4;
	2815	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.1;
	2815	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.6;
	2815	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.9;
	2815	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.1;
	2815	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.5;
	2816	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.9;
	2816	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.4;
	2816	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.2;
	2816	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.3;
	2816	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.6;
	2816	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.6;
	2817	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.3;
	2817	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.5;
	2817	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.1;
	2817	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.1;
	2817	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.1;
	2817	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.5;
	2818	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.2;
	2818	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.6;
	2818	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224;
	2818	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.3;
	2818	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	2818	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.5;
	2819	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.3;
	2819	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150;
	2819	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.4;
	2819	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.9;
	2819	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	2819	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.2;
	2820	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.3;
	2820	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.4;
	2820	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.2;
	2820	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.2;
	2820	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	2820	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.3;
	2821	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.1;
	2821	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.5;
	2821	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.4;
	2821	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.5;
	2821	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	2821	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.6;
	2822	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452;
	2822	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144;
	2822	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.8;
	2822	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.7;
	2822	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	2822	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.9;
	2823	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465;
	2823	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.8;
	2823	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.2;
	2823	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.2;
	2823	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	2823	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.3;
	2824	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.3;
	2824	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.7;
	2824	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.7;
	2824	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.4;
	2824	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	2824	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.3;
	2825	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.7;
	2825	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.7;
	2825	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.3;
	2825	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.7;
	2825	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	2825	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.2;
	2826	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.6;
	2826	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.4;
	2826	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221;
	2826	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.3;
	2826	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	2826	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352;
	2827	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436;
	2827	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.5;
	2827	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.1;
	2827	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226;
	2827	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	2827	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.6;
	2828	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.3;
	2828	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.6;
	2828	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.6;
	2828	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231;
	2828	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	2828	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.8;
	2829	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.6;
	2829	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.7;
	2829	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.4;
	2829	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.9;
	2829	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	2829	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.7;
	2830	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.5;
	2830	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.4;
	2830	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219;
	2830	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222;
	2830	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	2830	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350;
	2831	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419;
	2831	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.8;
	2831	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.2;
	2831	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.8;
	2831	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	2831	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.4;
	2832	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.2;
	2832	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.5;
	2832	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.5;
	2832	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.4;
	2832	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	2832	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.2;
	2833	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.5;
	2833	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.9;
	2833	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.4;
	2833	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.7;
	2833	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.4;
	2833	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.2;
	2834	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	387;
	2834	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.2;
	2834	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.1;
	2834	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.1;
	2834	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.5;
	2834	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.5;
	2835	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	385.7;
	2835	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.3;
	2835	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.1;
	2835	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187;
	2835	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.9;
	2835	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.7;
	2836	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	387.6;
	2836	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.3;
	2836	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.5;
	2836	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.5;
	2836	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.9;
	2836	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.7;
	2837	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.9;
	2837	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.7;
	2837	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.9;
	2837	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.9;
	2837	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.6;
	2837	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324;
	2838	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.2;
	2838	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.7;
	2838	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.2;
	2838	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.1;
	2838	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53;
	2838	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.9;
	2839	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.8;
	2839	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.2;
	2839	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.6;
	2839	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.7;
	2839	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.8;
	2839	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.4;
	2840	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.3;
	2840	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.2;
	2840	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.9;
	2840	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.8;
	2840	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	2840	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.4;
	2841	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422;
	2841	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.6;
	2841	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.4;
	2841	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.6;
	2841	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	2841	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338;
	2842	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.9;
	2842	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.4;
	2842	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204;
	2842	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.3;
	2842	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	2842	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.1;
	2843	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	426.9;
	2843	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.5;
	2843	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203;
	2843	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.6;
	2843	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	2843	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.1;
	2844	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428.4;
	2844	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.4;
	2844	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.4;
	2844	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.9;
	2844	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	2844	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.3;
	2845	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.1;
	2845	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.9;
	2845	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.3;
	2845	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.6;
	2845	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	2845	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.4;
	2846	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.9;
	2846	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.5;
	2846	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.9;
	2846	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.2;
	2846	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	2846	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.7;
	2847	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403;
	2847	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.9;
	2847	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.8;
	2847	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.3;
	2847	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	2847	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.4;
	2848	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.6;
	2848	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.2;
	2848	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.8;
	2848	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.1;
	2848	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	2848	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.7;
	2849	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398;
	2849	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.3;
	2849	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.4;
	2849	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.1;
	2849	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	2849	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.2;
	2850	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.3;
	2850	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.2;
	2850	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.3;
	2850	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.7;
	2850	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	2850	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.1;
	2851	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	395.4;
	2851	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.5;
	2851	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.3;
	2851	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.4;
	2851	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	2851	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.8;
	2852	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.2;
	2852	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132;
	2852	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.5;
	2852	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.7;
	2852	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	2852	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.3;
	2853	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.5;
	2853	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.3;
	2853	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.8;
	2853	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.1;
	2853	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	2853	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332;
	2854	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	400.7;
	2854	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.2;
	2854	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.3;
	2854	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.9;
	2854	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	2854	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.1;
	2855	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	390;
	2855	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.2;
	2855	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.2;
	2855	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.2;
	2855	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	2855	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.7;
	2856	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	377.7;
	2856	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.3;
	2856	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197;
	2856	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.1;
	2856	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	2856	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.6;
	2857	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	361.7;
	2857	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.3;
	2857	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.4;
	2857	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187;
	2857	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	2857	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.7;
	2858	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	350.6;
	2858	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.6;
	2858	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.1;
	2858	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	183.4;
	2858	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.6;
	2858	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	301.7;
	2859	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	348.5;
	2859	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.4;
	2859	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	178.7;
	2859	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	181.7;
	2859	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.9;
	2859	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	300.1;
	2860	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	349.3;
	2860	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110;
	2860	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	178.6;
	2860	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	181.8;
	2860	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.5;
	2860	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	299.9;
	2861	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	354.5;
	2861	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.3;
	2861	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180;
	2861	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	182.9;
	2861	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.7;
	2861	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	303;
	2862	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	361.7;
	2862	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.5;
	2862	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182;
	2862	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.3;
	2862	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.4;
	2862	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	307.3;
	2863	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	371.6;
	2863	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.5;
	2863	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.6;
	2863	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.1;
	2863	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58;
	2863	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.2;
	2864	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	378;
	2864	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.9;
	2864	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.2;
	2864	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.7;
	2864	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	2864	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.2;
	2865	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393;
	2865	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.8;
	2865	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.2;
	2865	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.5;
	2865	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	2865	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.7;
	2866	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.4;
	2866	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.1;
	2866	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.1;
	2866	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.5;
	2866	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	2866	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.9;
	2867	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	400.2;
	2867	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.8;
	2867	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.5;
	2867	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.8;
	2867	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.9;
	2867	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.1;
	2868	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.7;
	2868	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132;
	2868	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.3;
	2868	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.6;
	2868	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.8;
	2868	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341;
	2869	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.3;
	2869	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.5;
	2869	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.2;
	2869	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.9;
	2869	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	2869	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.6;
	2870	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.2;
	2870	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.1;
	2870	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.6;
	2870	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.3;
	2870	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	2870	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.7;
	2871	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	402.9;
	2871	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127;
	2871	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.3;
	2871	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.2;
	2871	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	2871	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.3;
	2872	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.1;
	2872	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.6;
	2872	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.3;
	2872	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.3;
	2872	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	2872	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.8;
	2873	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.9;
	2873	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.3;
	2873	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.7;
	2873	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201;
	2873	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	2873	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.5;
	2874	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.3;
	2874	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.4;
	2874	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.7;
	2874	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204;
	2874	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	2874	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.4;
	2875	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.4;
	2875	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.3;
	2875	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.6;
	2875	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209;
	2875	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	2875	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.1;
	2876	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	396.9;
	2876	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.4;
	2876	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.8;
	2876	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.1;
	2876	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	2876	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.2;
	2877	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.8;
	2877	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.4;
	2877	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216;
	2877	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.3;
	2877	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	2877	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.2;
	2878	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.8;
	2878	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.1;
	2878	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.2;
	2878	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.4;
	2878	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	2878	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.7;
	2879	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392.7;
	2879	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.9;
	2879	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.8;
	2879	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200;
	2879	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	2879	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.6;
	2880	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	376.6;
	2880	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.8;
	2880	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.5;
	2880	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.7;
	2880	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	2880	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.4;
	2881	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	360.2;
	2881	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.8;
	2881	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	179.6;
	2881	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	183;
	2881	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.8;
	2881	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	306.7;
	2882	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	354.5;
	2882	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.1;
	2882	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	179.7;
	2882	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	183.4;
	2882	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.8;
	2882	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	302.5;
	2883	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	349.8;
	2883	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.2;
	2883	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	178.3;
	2883	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	181.9;
	2883	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.9;
	2883	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	300.4;
	2884	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	355.3;
	2884	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.3;
	2884	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	179.6;
	2884	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	183;
	2884	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.2;
	2884	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	303;
	2885	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	368;
	2885	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.4;
	2885	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.9;
	2885	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.4;
	2885	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.2;
	2885	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309.1;
	2886	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.4;
	2886	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.4;
	2886	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.7;
	2886	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.6;
	2886	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.3;
	2886	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.6;
	2887	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.3;
	2887	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.5;
	2887	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.3;
	2887	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.1;
	2887	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.4;
	2887	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.2;
	2888	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458;
	2888	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.1;
	2888	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.4;
	2888	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.2;
	2888	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	2888	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.6;
	2889	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.7;
	2889	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.6;
	2889	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.1;
	2889	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.3;
	2889	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.2;
	2889	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.5;
	2890	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.8;
	2890	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.5;
	2890	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223;
	2890	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.3;
	2890	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	2890	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.3;
	2891	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.9;
	2891	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.3;
	2891	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.2;
	2891	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.9;
	2891	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	2891	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.5;
	2892	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459;
	2892	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.5;
	2892	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.2;
	2892	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.1;
	2892	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	2892	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.5;
	2893	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.2;
	2893	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.1;
	2893	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224;
	2893	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.4;
	2893	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.3;
	2893	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.8;
	2894	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.1;
	2894	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.4;
	2894	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.3;
	2894	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.7;
	2894	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.6;
	2894	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.2;
	2895	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.6;
	2895	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.6;
	2895	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.7;
	2895	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.8;
	2895	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61;
	2895	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.6;
	2896	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.3;
	2896	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.3;
	2896	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.4;
	2896	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.6;
	2896	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	2896	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.8;
	2897	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.3;
	2897	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.1;
	2897	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.4;
	2897	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.3;
	2897	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.3;
	2897	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.3;
	2898	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442.4;
	2898	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145;
	2898	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.3;
	2898	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.6;
	2898	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.1;
	2898	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.9;
	2899	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.3;
	2899	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.4;
	2899	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.3;
	2899	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.1;
	2899	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	2899	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.2;
	2900	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.5;
	2900	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.7;
	2900	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.2;
	2900	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.8;
	2900	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	2900	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.2;
	2901	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.1;
	2901	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.5;
	2901	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.3;
	2901	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.8;
	2901	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.4;
	2901	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.8;
	2902	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.5;
	2902	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.1;
	2902	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218;
	2902	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.6;
	2902	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	2902	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368;
	2903	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.6;
	2903	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.4;
	2903	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.8;
	2903	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.3;
	2903	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	2903	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.3;
	2904	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.2;
	2904	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.1;
	2904	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195;
	2904	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.2;
	2904	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.1;
	2904	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.4;
	2905	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	368.4;
	2905	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.1;
	2905	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.4;
	2905	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.4;
	2905	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.6;
	2905	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311.2;
	2906	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	377.8;
	2906	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.6;
	2906	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.8;
	2906	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.5;
	2906	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.1;
	2906	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.3;
	2907	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	382.1;
	2907	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109;
	2907	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	179.9;
	2907	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	183.8;
	2907	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.1;
	2907	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.7;
	2908	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	375.2;
	2908	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.3;
	2908	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.6;
	2908	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.4;
	2908	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.3;
	2908	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311;
	2909	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	385.5;
	2909	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.4;
	2909	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.8;
	2909	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189;
	2909	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.5;
	2909	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.4;
	2910	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.7;
	2910	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.2;
	2910	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.3;
	2910	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.9;
	2910	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.5;
	2910	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.2;
	2911	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.2;
	2911	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.3;
	2911	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.8;
	2911	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.3;
	2911	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.9;
	2911	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.3;
	2912	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.5;
	2912	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.2;
	2912	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.8;
	2912	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226;
	2912	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56;
	2912	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.6;
	2913	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.9;
	2913	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.8;
	2913	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.7;
	2913	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230;
	2913	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.4;
	2913	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.7;
	2914	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.6;
	2914	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.7;
	2914	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.3;
	2914	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232;
	2914	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.7;
	2914	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.4;
	2915	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.7;
	2915	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.6;
	2915	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.5;
	2915	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.9;
	2915	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	2915	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.4;
	2916	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.1;
	2916	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.6;
	2916	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.9;
	2916	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.3;
	2916	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.3;
	2916	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.7;
	2917	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.1;
	2917	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.2;
	2917	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227;
	2917	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.8;
	2917	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	2917	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.3;
	2918	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440;
	2918	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.2;
	2918	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.7;
	2918	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234;
	2918	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	2918	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.8;
	2919	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.7;
	2919	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.1;
	2919	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.8;
	2919	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.7;
	2919	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	2919	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.1;
	2920	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.2;
	2920	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.6;
	2920	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.4;
	2920	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.6;
	2920	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	2920	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.1;
	2921	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442.5;
	2921	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.3;
	2921	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.9;
	2921	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.7;
	2921	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	2921	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.4;
	2922	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.8;
	2922	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.1;
	2922	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.7;
	2922	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.5;
	2922	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.4;
	2922	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.1;
	2923	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.5;
	2923	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.7;
	2923	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226;
	2923	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.3;
	2923	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	2923	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.5;
	2924	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431;
	2924	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.3;
	2924	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.3;
	2924	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.8;
	2924	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	2924	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.8;
	2925	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.6;
	2925	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.1;
	2925	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.1;
	2925	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.6;
	2925	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	2925	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.6;
	2926	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.1;
	2926	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.1;
	2926	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.8;
	2926	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223;
	2926	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	2926	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.2;
	2927	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	402.8;
	2927	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.4;
	2927	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.6;
	2927	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.3;
	2927	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	2927	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.4;
	2928	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	371;
	2928	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.4;
	2928	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.7;
	2928	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.9;
	2928	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	2928	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.3;
	2929	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.4;
	2929	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.4;
	2929	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187;
	2929	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.9;
	2929	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.7;
	2929	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.2;
	2930	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	376.1;
	2930	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.2;
	2930	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181.6;
	2930	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.8;
	2930	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.5;
	2930	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311.6;
	2931	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	367.5;
	2931	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.3;
	2931	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181.7;
	2931	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.9;
	2931	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.7;
	2931	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	307.9;
	2932	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	368.7;
	2932	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.6;
	2932	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.2;
	2932	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.3;
	2932	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.5;
	2932	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309.3;
	2933	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	384.6;
	2933	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.3;
	2933	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.5;
	2933	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191;
	2933	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.7;
	2933	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.3;
	2934	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.1;
	2934	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.9;
	2934	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.4;
	2934	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.6;
	2934	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.4;
	2934	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.3;
	2935	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.2;
	2935	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.2;
	2935	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.1;
	2935	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216;
	2935	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.5;
	2935	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.1;
	2936	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.5;
	2936	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.9;
	2936	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.3;
	2936	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.4;
	2936	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	2936	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.7;
	2937	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.5;
	2937	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.3;
	2937	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.3;
	2937	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.2;
	2937	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	2937	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.7;
	2938	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.4;
	2938	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.5;
	2938	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.2;
	2938	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233;
	2938	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	2938	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.2;
	2939	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.9;
	2939	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.1;
	2939	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.5;
	2939	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.2;
	2939	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	2939	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.6;
	2940	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	426.4;
	2940	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.9;
	2940	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.6;
	2940	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.2;
	2940	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	2940	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.8;
	2941	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.7;
	2941	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.9;
	2941	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.6;
	2941	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.1;
	2941	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	2941	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.9;
	2942	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.9;
	2942	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.4;
	2942	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.5;
	2942	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.8;
	2942	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	2942	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.8;
	2943	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.9;
	2943	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.2;
	2943	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.1;
	2943	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.9;
	2943	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	2943	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.9;
	2944	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.2;
	2944	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.8;
	2944	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.4;
	2944	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.1;
	2944	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	2944	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363;
	2945	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460;
	2945	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.9;
	2945	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.1;
	2945	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.6;
	2945	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	2945	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.7;
	2946	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.2;
	2946	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144;
	2946	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.7;
	2946	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.2;
	2946	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	2946	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.6;
	2947	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.9;
	2947	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.9;
	2947	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.9;
	2947	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.1;
	2947	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	2947	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.2;
	2948	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.8;
	2948	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.4;
	2948	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.5;
	2948	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.5;
	2948	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	2948	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.3;
	2949	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.2;
	2949	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.7;
	2949	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228;
	2949	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.1;
	2949	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	2949	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.3;
	2950	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.1;
	2950	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.8;
	2950	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.6;
	2950	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.1;
	2950	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	2950	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.4;
	2951	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	427.6;
	2951	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.4;
	2951	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.8;
	2951	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208;
	2951	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	2951	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.7;
	2952	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.6;
	2952	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.1;
	2952	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.8;
	2952	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.2;
	2952	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61;
	2952	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.6;
	2953	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.2;
	2953	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.7;
	2953	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.1;
	2953	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.5;
	2953	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	2953	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.6;
	2954	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	367.5;
	2954	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108;
	2954	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181.8;
	2954	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.9;
	2954	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.4;
	2954	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.2;
	2955	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	354.1;
	2955	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.8;
	2955	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181.7;
	2955	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.5;
	2955	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.1;
	2955	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	302.8;
	2956	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	357;
	2956	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.4;
	2956	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.2;
	2956	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.2;
	2956	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.3;
	2956	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	304.8;
	2957	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	371.7;
	2957	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.5;
	2957	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.3;
	2957	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.8;
	2957	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.4;
	2957	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.3;
	2958	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.5;
	2958	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.1;
	2958	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.7;
	2958	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.5;
	2958	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.5;
	2958	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.9;
	2959	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.4;
	2959	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.3;
	2959	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.3;
	2959	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.6;
	2959	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.9;
	2959	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.5;
	2960	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.9;
	2960	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.7;
	2960	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.6;
	2960	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.6;
	2960	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	2960	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.7;
	2961	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.8;
	2961	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.2;
	2961	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.1;
	2961	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.2;
	2961	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	2961	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.9;
	2962	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.6;
	2962	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.4;
	2962	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.3;
	2962	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.1;
	2962	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	2962	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.5;
	2963	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.2;
	2963	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144;
	2963	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.2;
	2963	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.9;
	2963	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	2963	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.3;
	2964	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.2;
	2964	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.7;
	2964	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.8;
	2964	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.6;
	2964	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	2964	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.6;
	2965	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.1;
	2965	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.7;
	2965	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.5;
	2965	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.4;
	2965	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	2965	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.6;
	2966	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.3;
	2966	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142;
	2966	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.7;
	2966	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.4;
	2966	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	2966	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.3;
	2967	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.5;
	2967	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.5;
	2967	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.5;
	2967	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.3;
	2967	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.2;
	2967	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.2;
	2968	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.5;
	2968	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142;
	2968	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.2;
	2968	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.7;
	2968	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	2968	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.4;
	2969	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.5;
	2969	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.1;
	2969	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.3;
	2969	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.2;
	2969	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	2969	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369;
	2970	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.3;
	2970	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.5;
	2970	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.9;
	2970	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.8;
	2970	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	2970	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.2;
	2971	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.3;
	2971	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.1;
	2971	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225;
	2971	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.9;
	2971	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	2971	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.6;
	2972	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.9;
	2972	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.1;
	2972	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230;
	2972	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.9;
	2972	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	2972	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.8;
	2973	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.9;
	2973	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.4;
	2973	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229;
	2973	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.8;
	2973	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	2973	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.7;
	2974	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.6;
	2974	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.9;
	2974	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.8;
	2974	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.9;
	2974	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	2974	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.2;
	2975	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442.8;
	2975	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.5;
	2975	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.1;
	2975	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.3;
	2975	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	2975	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.1;
	2976	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.9;
	2976	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.4;
	2976	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.6;
	2976	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.5;
	2976	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.4;
	2976	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.4;
	2977	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.7;
	2977	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.9;
	2977	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188;
	2977	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.3;
	2977	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.5;
	2977	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.7;
	2978	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	376.3;
	2978	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.8;
	2978	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.1;
	2978	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.6;
	2978	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.5;
	2978	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.1;
	2979	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	378.5;
	2979	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	105.4;
	2979	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.8;
	2979	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.4;
	2979	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.1;
	2979	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.4;
	2980	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	388.8;
	2980	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.3;
	2980	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.7;
	2980	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.4;
	2980	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	50.7;
	2980	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.8;
	2981	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392.3;
	2981	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.3;
	2981	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.4;
	2981	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.4;
	2981	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.8;
	2981	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.9;
	2982	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421;
	2982	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.3;
	2982	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.4;
	2982	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.9;
	2982	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.6;
	2982	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.4;
	2983	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.8;
	2983	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.2;
	2983	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.5;
	2983	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.4;
	2983	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	2983	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.4;
	2984	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.1;
	2984	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.1;
	2984	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.5;
	2984	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.3;
	2984	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.6;
	2984	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.1;
	2985	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.2;
	2985	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.1;
	2985	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226;
	2985	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.8;
	2985	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	2985	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372;
	2986	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.5;
	2986	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.2;
	2986	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.3;
	2986	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.2;
	2986	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	2986	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.8;
	2987	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.8;
	2987	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.3;
	2987	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.6;
	2987	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234;
	2987	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	2987	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.2;
	2988	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.4;
	2988	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.3;
	2988	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.3;
	2988	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235;
	2988	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	2988	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.7;
	2989	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.3;
	2989	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143;
	2989	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.2;
	2989	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.3;
	2989	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	2989	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.2;
	2990	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.1;
	2990	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.9;
	2990	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.1;
	2990	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.6;
	2990	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	2990	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.3;
	2991	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.7;
	2991	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.8;
	2991	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.2;
	2991	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.5;
	2991	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	2991	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.5;
	2992	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.2;
	2992	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.1;
	2992	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.7;
	2992	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.3;
	2992	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	2992	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.1;
	2993	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.6;
	2993	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.3;
	2993	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.8;
	2993	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.9;
	2993	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	2993	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.9;
	2994	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.5;
	2994	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.8;
	2994	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.9;
	2994	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.8;
	2994	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	2994	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.7;
	2995	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.1;
	2995	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.6;
	2995	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.6;
	2995	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.7;
	2995	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	2995	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.6;
	2996	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.7;
	2996	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143;
	2996	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.7;
	2996	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.1;
	2996	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	2996	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.9;
	2997	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.7;
	2997	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142;
	2997	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.3;
	2997	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.1;
	2997	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	2997	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.9;
	2998	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442.3;
	2998	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.9;
	2998	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.2;
	2998	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.6;
	2998	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	2998	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.1;
	2999	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.4;
	2999	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.3;
	2999	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.4;
	2999	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.6;
	2999	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	2999	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.3;
	3000	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.7;
	3000	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.5;
	3000	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.6;
	3000	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.3;
	3000	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	3000	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329;
	3001	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379.2;
	3001	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.2;
	3001	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.9;
	3001	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.2;
	3001	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.8;
	3001	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.5;
	3002	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	359.7;
	3002	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	105;
	3002	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181.3;
	3002	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.8;
	3002	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.7;
	3002	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	306.8;
	3003	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	353.4;
	3003	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	104.9;
	3003	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182;
	3003	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.4;
	3003	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.7;
	3003	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	303.4;
	3004	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	359.8;
	3004	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	105.5;
	3004	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181.1;
	3004	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.8;
	3004	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.4;
	3004	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	305.2;
	3005	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	368.9;
	3005	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.3;
	3005	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.6;
	3005	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187;
	3005	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.4;
	3005	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310.6;
	3006	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	386.1;
	3006	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.3;
	3006	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.2;
	3006	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192;
	3006	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.6;
	3006	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.2;
	3007	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.9;
	3007	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.4;
	3007	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.9;
	3007	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.9;
	3007	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57;
	3007	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.4;
	3008	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.2;
	3008	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.7;
	3008	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.9;
	3008	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.1;
	3008	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.2;
	3008	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.4;
	3009	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.6;
	3009	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.5;
	3009	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200;
	3009	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.9;
	3009	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	3009	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.1;
	3010	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434;
	3010	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.1;
	3010	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.2;
	3010	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.9;
	3010	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	3010	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.7;
	3011	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.5;
	3011	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133;
	3011	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204;
	3011	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.9;
	3011	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	3011	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.5;
	3012	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.6;
	3012	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.6;
	3012	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.7;
	3012	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.9;
	3012	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.1;
	3012	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.6;
	3013	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.6;
	3013	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.7;
	3013	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.2;
	3013	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.7;
	3013	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	3013	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.9;
	3014	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.3;
	3014	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.8;
	3014	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.6;
	3014	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210;
	3014	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	3014	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.5;
	3015	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.6;
	3015	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127;
	3015	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.1;
	3015	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.4;
	3015	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.2;
	3015	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.5;
	3016	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.9;
	3016	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.1;
	3016	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.6;
	3016	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.8;
	3016	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	3016	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.3;
	3017	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418;
	3017	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.8;
	3017	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.1;
	3017	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.3;
	3017	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	3017	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.9;
	3018	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417.3;
	3018	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.4;
	3018	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.2;
	3018	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.1;
	3018	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	3018	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.7;
	3019	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.3;
	3019	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.2;
	3019	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.7;
	3019	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.9;
	3019	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	3019	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.6;
	3020	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.5;
	3020	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.3;
	3020	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213;
	3020	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.2;
	3020	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	3020	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.9;
	3021	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.4;
	3021	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.5;
	3021	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.8;
	3021	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.9;
	3021	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	3021	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.1;
	3022	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	401.9;
	3022	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.9;
	3022	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209;
	3022	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.6;
	3022	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	3022	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.7;
	3023	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	380.2;
	3023	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118;
	3023	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.3;
	3023	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.8;
	3023	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	3023	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.7;
	3024	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	358.4;
	3024	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115;
	3024	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.4;
	3024	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.3;
	3024	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.3;
	3024	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310.4;
	3025	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	338.7;
	3025	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	105.4;
	3025	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.2;
	3025	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.5;
	3025	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.4;
	3025	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	299.9;
	3026	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	320;
	3026	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	103.2;
	3026	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.9;
	3026	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.6;
	3026	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.8;
	3026	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	290.5;
	3027	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	322.7;
	3027	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	101.3;
	3027	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180;
	3027	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	183.1;
	3027	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.2;
	3027	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	290.8;
	3028	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	310.8;
	3028	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	101.6;
	3028	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.7;
	3028	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	183.7;
	3028	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.2;
	3028	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	285.7;
	3029	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	327.7;
	3029	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	103.6;
	3029	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.9;
	3029	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.2;
	3029	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.8;
	3029	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	292.8;
	3030	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	337.9;
	3030	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.8;
	3030	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183;
	3030	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186;
	3030	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.2;
	3030	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	298.3;
	3031	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	345.4;
	3031	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.4;
	3031	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.4;
	3031	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.9;
	3031	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.8;
	3031	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	302.4;
	3032	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	353.9;
	3032	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.8;
	3032	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.5;
	3032	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.3;
	3032	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	3032	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	306.4;
	3033	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	367.8;
	3033	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.2;
	3033	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.3;
	3033	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.5;
	3033	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	3033	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.2;
	3034	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	362;
	3034	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.8;
	3034	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196;
	3034	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.1;
	3034	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	3034	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.1;
	3035	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	366;
	3035	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.1;
	3035	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197;
	3035	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.1;
	3035	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.1;
	3035	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.8;
	3036	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	376.3;
	3036	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.9;
	3036	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.5;
	3036	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.4;
	3036	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.7;
	3036	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318;
	3037	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	386.9;
	3037	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.6;
	3037	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.2;
	3037	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.1;
	3037	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.2;
	3037	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.3;
	3038	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	378.2;
	3038	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.6;
	3038	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.4;
	3038	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.8;
	3038	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.8;
	3038	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.5;
	3039	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379.7;
	3039	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.9;
	3039	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.5;
	3039	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.2;
	3039	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	3039	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.3;
	3040	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	381.3;
	3040	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.5;
	3040	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.6;
	3040	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.1;
	3040	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	3040	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.7;
	3041	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	381.9;
	3041	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.5;
	3041	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.2;
	3041	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.4;
	3041	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	3041	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.6;
	3042	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	386.8;
	3042	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.9;
	3042	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.1;
	3042	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.5;
	3042	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	3042	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.7;
	3043	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.9;
	3043	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.5;
	3043	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.2;
	3043	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.7;
	3043	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	3043	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.4;
	3044	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403.7;
	3044	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.8;
	3044	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.4;
	3044	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.7;
	3044	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	3044	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.2;
	3045	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.2;
	3045	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.7;
	3045	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.8;
	3045	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.4;
	3045	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	3045	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.3;
	3046	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	395.9;
	3046	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.4;
	3046	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.5;
	3046	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.6;
	3046	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	3046	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.3;
	3047	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391.1;
	3047	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.4;
	3047	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.2;
	3047	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.8;
	3047	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.4;
	3047	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.7;
	3048	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	370;
	3048	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.3;
	3048	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.8;
	3048	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.4;
	3048	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	3048	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.3;
	3049	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	348.3;
	3049	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.4;
	3049	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.4;
	3049	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.4;
	3049	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.1;
	3049	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	303.4;
	3050	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	341.4;
	3050	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	105.3;
	3050	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181.6;
	3050	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.3;
	3050	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.3;
	3050	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	298.7;
	3051	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	344.9;
	3051	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	104.4;
	3051	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.5;
	3051	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184;
	3051	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.5;
	3051	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	299.3;
	3052	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	345.9;
	3052	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	105.7;
	3052	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181.3;
	3052	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.6;
	3052	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.1;
	3052	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	299.8;
	3053	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	347.2;
	3053	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110;
	3053	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.5;
	3053	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187;
	3053	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.2;
	3053	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	301.2;
	3054	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	370;
	3054	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.7;
	3054	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.3;
	3054	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.8;
	3054	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.7;
	3054	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.8;
	3055	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.1;
	3055	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.1;
	3055	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.9;
	3055	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.5;
	3055	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.4;
	3055	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.3;
	3056	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.1;
	3056	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.8;
	3056	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.6;
	3056	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.8;
	3056	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.6;
	3056	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.3;
	3057	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.1;
	3057	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.7;
	3057	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.3;
	3057	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.3;
	3057	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	3057	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.5;
	3058	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.1;
	3058	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.2;
	3058	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.1;
	3058	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.4;
	3058	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61;
	3058	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.6;
	3059	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.5;
	3059	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.1;
	3059	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.1;
	3059	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.2;
	3059	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	3059	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.1;
	3060	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.3;
	3060	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.2;
	3060	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.4;
	3060	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.7;
	3060	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	3060	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.8;
	3061	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.4;
	3061	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145;
	3061	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.4;
	3061	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.3;
	3061	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	3061	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.4;
	3062	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.4;
	3062	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.8;
	3062	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.1;
	3062	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.2;
	3062	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	3062	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.4;
	3063	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.5;
	3063	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.7;
	3063	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.5;
	3063	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.5;
	3063	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	3063	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370;
	3064	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.8;
	3064	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.5;
	3064	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.7;
	3064	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241;
	3064	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	3064	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.7;
	3065	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467;
	3065	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.8;
	3065	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.8;
	3065	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.8;
	3065	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.9;
	3065	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.6;
	3066	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.5;
	3066	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.9;
	3066	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.2;
	3066	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.6;
	3066	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	3066	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.1;
	3067	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.9;
	3067	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.4;
	3067	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.5;
	3067	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.4;
	3067	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.5;
	3067	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.6;
	3068	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.9;
	3068	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.9;
	3068	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.4;
	3068	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.5;
	3068	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.4;
	3068	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.4;
	3069	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.5;
	3069	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.3;
	3069	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.4;
	3069	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.2;
	3069	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	3069	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.6;
	3070	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.4;
	3070	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.4;
	3070	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228;
	3070	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231;
	3070	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.1;
	3070	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.1;
	3071	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	426.2;
	3071	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.2;
	3071	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.4;
	3071	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.7;
	3071	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.8;
	3071	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.6;
	3072	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	406.9;
	3072	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.7;
	3072	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.1;
	3072	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.3;
	3072	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.4;
	3072	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.2;
	3073	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	390.4;
	3073	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.8;
	3073	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.2;
	3073	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.3;
	3073	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.5;
	3073	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.6;
	3074	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	366.3;
	3074	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.8;
	3074	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.3;
	3074	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.5;
	3074	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.7;
	3074	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310.3;
	3075	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	361.5;
	3075	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.8;
	3075	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.3;
	3075	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.1;
	3075	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.5;
	3075	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	307.6;
	3076	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	365.8;
	3076	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.5;
	3076	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.9;
	3076	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188;
	3076	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.7;
	3076	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309.1;
	3077	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	380;
	3077	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.6;
	3077	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.5;
	3077	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.8;
	3077	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.9;
	3077	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.9;
	3078	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.5;
	3078	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.5;
	3078	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.2;
	3078	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.1;
	3078	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52;
	3078	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.3;
	3079	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.8;
	3079	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.7;
	3079	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.4;
	3079	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215;
	3079	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.6;
	3079	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.6;
	3080	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.8;
	3080	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.5;
	3080	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.3;
	3080	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.4;
	3080	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.8;
	3080	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.6;
	3081	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479;
	3081	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.3;
	3081	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.7;
	3081	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.4;
	3081	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55;
	3081	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.4;
	3082	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.2;
	3082	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.4;
	3082	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.7;
	3082	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.2;
	3082	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.9;
	3082	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.4;
	3083	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.3;
	3083	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148;
	3083	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.2;
	3083	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.8;
	3083	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	3083	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.9;
	3084	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.3;
	3084	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.6;
	3084	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.3;
	3084	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.1;
	3084	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	3084	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.7;
	3085	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.9;
	3085	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.9;
	3085	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.2;
	3085	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.2;
	3085	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.9;
	3085	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.3;
	3086	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.2;
	3086	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.2;
	3086	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.1;
	3086	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.6;
	3086	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.5;
	3086	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.7;
	3087	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.9;
	3087	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.1;
	3087	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.1;
	3087	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.4;
	3087	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	3087	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.6;
	3088	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.1;
	3088	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.1;
	3088	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242;
	3088	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.5;
	3088	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.4;
	3088	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.2;
	3089	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.6;
	3089	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.2;
	3089	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.4;
	3089	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.5;
	3089	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.4;
	3089	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.4;
	3090	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.4;
	3090	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.3;
	3090	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.6;
	3090	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.8;
	3090	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.1;
	3090	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.4;
	3091	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.1;
	3091	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.4;
	3091	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.7;
	3091	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.7;
	3091	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.4;
	3091	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.9;
	3092	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442.9;
	3092	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.9;
	3092	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.9;
	3092	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.6;
	3092	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58;
	3092	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.1;
	3093	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.1;
	3093	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.8;
	3093	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.7;
	3093	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.7;
	3093	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58;
	3093	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.9;
	3094	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452;
	3094	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.2;
	3094	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.1;
	3094	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.7;
	3094	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	3094	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.6;
	3095	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.9;
	3095	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128;
	3095	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.8;
	3095	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.4;
	3095	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.7;
	3095	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346;
	3096	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.2;
	3096	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.7;
	3096	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.6;
	3096	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.8;
	3096	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.2;
	3096	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.4;
	3097	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	388.5;
	3097	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112;
	3097	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.4;
	3097	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.4;
	3097	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	3097	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.6;
	3098	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	367.4;
	3098	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.2;
	3098	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.2;
	3098	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.9;
	3098	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.8;
	3098	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310.1;
	3099	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	361.1;
	3099	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.9;
	3099	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186;
	3099	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189;
	3099	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.2;
	3099	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	306.9;
	3100	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	362.7;
	3100	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.2;
	3100	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.3;
	3100	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.4;
	3100	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.2;
	3100	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308;
	3101	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	375.4;
	3101	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.8;
	3101	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.3;
	3101	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.4;
	3101	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.1;
	3101	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.6;
	3102	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.6;
	3102	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.4;
	3102	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.1;
	3102	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.4;
	3102	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.3;
	3102	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.7;
	3103	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.3;
	3103	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.7;
	3103	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.2;
	3103	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.3;
	3103	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.1;
	3103	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345;
	3104	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.5;
	3104	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.4;
	3104	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227;
	3104	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.3;
	3104	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	3104	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.1;
	3105	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.2;
	3105	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.1;
	3105	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.3;
	3105	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.5;
	3105	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	3105	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.2;
	3106	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.5;
	3106	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.1;
	3106	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.2;
	3106	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.5;
	3106	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	3106	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.1;
	3107	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.3;
	3107	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.7;
	3107	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.9;
	3107	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.4;
	3107	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	3107	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.7;
	3108	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.4;
	3108	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146;
	3108	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.9;
	3108	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.1;
	3108	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	3108	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.4;
	3109	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467;
	3109	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.5;
	3109	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.4;
	3109	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.4;
	3109	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	3109	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.8;
	3110	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.6;
	3110	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.3;
	3110	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.6;
	3110	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243;
	3110	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	3110	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370;
	3111	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.5;
	3111	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.8;
	3111	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238;
	3111	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.9;
	3111	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	3111	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.5;
	3112	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.8;
	3112	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.5;
	3112	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237;
	3112	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.4;
	3112	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	3112	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.7;
	3113	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.8;
	3113	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.2;
	3113	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.2;
	3113	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241;
	3113	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	3113	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.1;
	3114	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.9;
	3114	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.7;
	3114	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.9;
	3114	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238;
	3114	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	3114	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.9;
	3115	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.4;
	3115	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.6;
	3115	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.9;
	3115	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.9;
	3115	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	3115	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357;
	3116	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.8;
	3116	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.8;
	3116	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.4;
	3116	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.8;
	3116	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	3116	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.6;
	3117	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.7;
	3117	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148;
	3117	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.9;
	3117	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.5;
	3117	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	3117	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.8;
	3118	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.5;
	3118	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.4;
	3118	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.7;
	3118	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.8;
	3118	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	3118	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.3;
	3119	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.1;
	3119	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126;
	3119	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.8;
	3119	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.3;
	3119	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	3119	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340;
	3120	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	388.1;
	3120	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.1;
	3120	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.7;
	3120	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.4;
	3120	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.5;
	3120	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.1;
	3121	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	365.6;
	3121	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.7;
	3121	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.9;
	3121	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.8;
	3121	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.6;
	3121	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.1;
	3122	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	350.3;
	3122	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.3;
	3122	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.9;
	3122	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.2;
	3122	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.5;
	3122	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	303;
	3123	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	347.1;
	3123	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	105.5;
	3123	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.3;
	3123	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.6;
	3123	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.6;
	3123	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	301.3;
	3124	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	348.3;
	3124	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.6;
	3124	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.3;
	3124	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.6;
	3124	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.8;
	3124	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	302.1;
	3125	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	355.5;
	3125	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.2;
	3125	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.1;
	3125	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.6;
	3125	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.1;
	3125	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	306.5;
	3126	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	386.6;
	3126	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.2;
	3126	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.1;
	3126	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.8;
	3126	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.6;
	3126	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.3;
	3127	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428.6;
	3127	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.3;
	3127	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212;
	3127	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.2;
	3127	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.1;
	3127	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.7;
	3128	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.4;
	3128	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.6;
	3128	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.4;
	3128	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.4;
	3128	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.5;
	3128	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.3;
	3129	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.9;
	3129	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.1;
	3129	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230;
	3129	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.9;
	3129	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	3129	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363;
	3130	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.1;
	3130	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.1;
	3130	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.1;
	3130	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.1;
	3130	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	3130	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368;
	3131	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.1;
	3131	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.7;
	3131	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.9;
	3131	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.1;
	3131	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	3131	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.8;
	3132	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.3;
	3132	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.5;
	3132	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.9;
	3132	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.3;
	3132	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	3132	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.1;
	3133	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.4;
	3133	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.8;
	3133	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236;
	3133	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.8;
	3133	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	3133	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.5;
	3134	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.4;
	3134	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.7;
	3134	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.4;
	3134	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.1;
	3134	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	3134	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.8;
	3135	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.4;
	3135	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143;
	3135	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237;
	3135	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244;
	3135	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	3135	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.3;
	3136	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.5;
	3136	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.2;
	3136	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238;
	3136	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.5;
	3136	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	3136	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.5;
	3137	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.1;
	3137	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.5;
	3137	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.4;
	3137	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.3;
	3137	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	3137	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.3;
	3138	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.5;
	3138	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.2;
	3138	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.1;
	3138	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.4;
	3138	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	3138	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.5;
	3139	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471;
	3139	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144;
	3139	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.3;
	3139	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.4;
	3139	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	3139	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.3;
	3140	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.1;
	3140	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.5;
	3140	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.5;
	3140	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.7;
	3140	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	3140	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.4;
	3141	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471;
	3141	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.1;
	3141	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.2;
	3141	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.1;
	3141	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	3141	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.8;
	3142	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.3;
	3142	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.4;
	3142	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.4;
	3142	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.5;
	3142	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	3142	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.2;
	3143	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.2;
	3143	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.1;
	3143	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.6;
	3143	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.1;
	3143	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60;
	3143	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.8;
	3144	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.7;
	3144	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.7;
	3144	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.1;
	3144	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.8;
	3144	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.1;
	3144	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333;
	3145	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391.3;
	3145	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.6;
	3145	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.5;
	3145	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.7;
	3145	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.3;
	3145	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.8;
	3146	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	365;
	3146	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107;
	3146	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186;
	3146	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.2;
	3146	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54;
	3146	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309.3;
	3147	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	361.2;
	3147	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	105.6;
	3147	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.3;
	3147	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.6;
	3147	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52;
	3147	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	307.2;
	3148	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	364.6;
	3148	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.7;
	3148	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.5;
	3148	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.8;
	3148	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.8;
	3148	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.9;
	3149	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	369.2;
	3149	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.7;
	3149	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.8;
	3149	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.3;
	3149	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.9;
	3149	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.3;
	3150	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	389.8;
	3150	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.5;
	3150	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.5;
	3150	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.1;
	3150	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.6;
	3150	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.6;
	3151	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421;
	3151	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.8;
	3151	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.2;
	3151	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.8;
	3151	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55;
	3151	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.8;
	3152	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.5;
	3152	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.3;
	3152	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.3;
	3152	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.9;
	3152	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	3152	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.9;
	3153	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.1;
	3153	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143;
	3153	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.5;
	3153	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.7;
	3153	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	3153	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.4;
	3154	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.3;
	3154	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.8;
	3154	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.9;
	3154	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.2;
	3154	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	3154	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.3;
	3155	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.8;
	3155	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145;
	3155	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.1;
	3155	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.5;
	3155	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	3155	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.9;
	3156	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.1;
	3156	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.1;
	3156	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.9;
	3156	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.9;
	3156	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	3156	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.9;
	3157	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.3;
	3157	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.3;
	3157	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.3;
	3157	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.9;
	3157	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	3157	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.2;
	3158	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.2;
	3158	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.3;
	3158	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.8;
	3158	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.3;
	3158	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	3158	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.3;
	3159	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.4;
	3159	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.5;
	3159	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.6;
	3159	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.5;
	3159	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	3159	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.6;
	3160	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.4;
	3160	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.2;
	3160	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.3;
	3160	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.1;
	3160	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	3160	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.4;
	3161	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.2;
	3161	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.4;
	3161	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.1;
	3161	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.2;
	3161	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	3161	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369;
	3162	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.5;
	3162	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.4;
	3162	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.8;
	3162	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.8;
	3162	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	3162	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.1;
	3163	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447.9;
	3163	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.4;
	3163	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.6;
	3163	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242;
	3163	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	3163	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.8;
	3164	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	444.8;
	3164	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.2;
	3164	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.2;
	3164	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.4;
	3164	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	3164	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.8;
	3165	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.5;
	3165	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.7;
	3165	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.1;
	3165	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.4;
	3165	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	3165	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.1;
	3166	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.6;
	3166	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.1;
	3166	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.1;
	3166	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.1;
	3166	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	3166	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.4;
	3167	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414;
	3167	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.4;
	3167	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.7;
	3167	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218;
	3167	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	3167	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.2;
	3168	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	388.1;
	3168	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.1;
	3168	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.1;
	3168	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.3;
	3168	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	3168	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.8;
	3169	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	372.7;
	3169	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.1;
	3169	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.4;
	3169	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.8;
	3169	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.4;
	3169	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.8;
	3170	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	342.9;
	3170	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.9;
	3170	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.8;
	3170	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.2;
	3170	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.4;
	3170	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	302.4;
	3171	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	339.8;
	3171	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106;
	3171	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.5;
	3171	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.2;
	3171	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.9;
	3171	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	300.1;
	3172	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	340.2;
	3172	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.1;
	3172	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.2;
	3172	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.1;
	3172	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.4;
	3172	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	299.9;
	3173	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	349;
	3173	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.5;
	3173	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.8;
	3173	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.7;
	3173	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.1;
	3173	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	304.7;
	3174	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	359.8;
	3174	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.8;
	3174	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193;
	3174	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.7;
	3174	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.1;
	3174	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310.9;
	3175	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	373.7;
	3175	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.3;
	3175	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.8;
	3175	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.9;
	3175	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.3;
	3175	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.4;
	3176	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	378;
	3176	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.2;
	3176	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.8;
	3176	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.1;
	3176	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.4;
	3176	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.7;
	3177	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	395.6;
	3177	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.1;
	3177	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.6;
	3177	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.6;
	3177	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	3177	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.9;
	3178	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.9;
	3178	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.8;
	3178	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.1;
	3178	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219;
	3178	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	3178	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.7;
	3179	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.5;
	3179	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.9;
	3179	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.6;
	3179	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.3;
	3179	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	3179	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.3;
	3180	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.8;
	3180	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.1;
	3180	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.1;
	3180	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.1;
	3180	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	3180	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.1;
	3181	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	430.1;
	3181	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.8;
	3181	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.1;
	3181	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.2;
	3181	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	3181	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.7;
	3182	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.7;
	3182	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.6;
	3182	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.1;
	3182	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.4;
	3182	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	3182	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.3;
	3183	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	427.4;
	3183	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.1;
	3183	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.9;
	3183	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223;
	3183	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	3183	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.9;
	3184	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.6;
	3184	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.6;
	3184	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.1;
	3184	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.9;
	3184	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	3184	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.5;
	3185	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	430.2;
	3185	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.7;
	3185	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.7;
	3185	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223;
	3185	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	3185	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.4;
	3186	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.4;
	3186	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.1;
	3186	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.5;
	3186	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.6;
	3186	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	3186	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.9;
	3187	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	426.9;
	3187	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.7;
	3187	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.9;
	3187	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.7;
	3187	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	3187	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.1;
	3188	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.8;
	3188	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.6;
	3188	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.6;
	3188	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228;
	3188	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	3188	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.4;
	3189	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.5;
	3189	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.8;
	3189	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.2;
	3189	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.3;
	3189	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	3189	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.3;
	3190	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.7;
	3190	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.1;
	3190	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.9;
	3190	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.5;
	3190	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	3190	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.9;
	3191	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391.6;
	3191	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.3;
	3191	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.6;
	3191	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.4;
	3191	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	3191	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.6;
	3192	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	369.7;
	3192	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.5;
	3192	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.8;
	3192	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.5;
	3192	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59;
	3192	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319;
	3193	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	349.2;
	3193	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109;
	3193	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.2;
	3193	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.8;
	3193	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.5;
	3193	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	307.8;
	3194	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	328.8;
	3194	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	102.5;
	3194	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.7;
	3194	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.3;
	3194	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.6;
	3194	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	296.4;
	3195	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	323;
	3195	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	101;
	3195	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.6;
	3195	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.2;
	3195	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.7;
	3195	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	292.9;
	3196	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	322.9;
	3196	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	100.8;
	3196	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.4;
	3196	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.2;
	3196	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.6;
	3196	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	292.3;
	3197	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	330.2;
	3197	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	102;
	3197	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.7;
	3197	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.5;
	3197	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.9;
	3197	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	295.5;
	3198	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	338.7;
	3198	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	104.7;
	3198	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.9;
	3198	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.1;
	3198	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.2;
	3198	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	300.2;
	3199	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	348.8;
	3199	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.4;
	3199	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.4;
	3199	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.1;
	3199	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.6;
	3199	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	305.2;
	3200	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	354.6;
	3200	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.8;
	3200	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192;
	3200	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.7;
	3200	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.1;
	3200	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.5;
	3201	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	380.1;
	3201	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.8;
	3201	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.4;
	3201	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202;
	3201	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	3201	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.4;
	3202	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	402.2;
	3202	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.7;
	3202	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.9;
	3202	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.7;
	3202	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	3202	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.2;
	3203	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.6;
	3203	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.2;
	3203	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.9;
	3203	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.4;
	3203	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	3203	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.7;
	3204	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	406.9;
	3204	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.3;
	3204	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.8;
	3204	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215;
	3204	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	3204	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335;
	3205	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.2;
	3205	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.9;
	3205	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.1;
	3205	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.9;
	3205	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.2;
	3205	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.6;
	3206	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.8;
	3206	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.6;
	3206	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.7;
	3206	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221;
	3206	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	3206	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.6;
	3207	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.1;
	3207	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.7;
	3207	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.8;
	3207	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.5;
	3207	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	3207	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.7;
	3208	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.1;
	3208	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.9;
	3208	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.4;
	3208	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.6;
	3208	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	3208	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342;
	3209	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.6;
	3209	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.4;
	3209	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.4;
	3209	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.5;
	3209	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	3209	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345;
	3210	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.7;
	3210	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.8;
	3210	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.3;
	3210	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.6;
	3210	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	3210	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345;
	3211	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.6;
	3211	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.1;
	3211	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.8;
	3211	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.6;
	3211	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	3211	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.2;
	3212	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.7;
	3212	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.7;
	3212	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.6;
	3212	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239;
	3212	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	3212	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.7;
	3213	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.5;
	3213	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144;
	3213	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.5;
	3213	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.2;
	3213	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	3213	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.8;
	3214	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.5;
	3214	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.7;
	3214	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.8;
	3214	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.6;
	3214	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	3214	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.1;
	3215	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.2;
	3215	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.2;
	3215	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.8;
	3215	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.7;
	3215	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	3215	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.3;
	3216	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392.7;
	3216	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.8;
	3216	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.6;
	3216	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.1;
	3216	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	3216	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.2;
	3217	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	373.8;
	3217	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109;
	3217	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.5;
	3217	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.5;
	3217	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.7;
	3217	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318;
	3218	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	350.6;
	3218	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107;
	3218	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.8;
	3218	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.6;
	3218	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.7;
	3218	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	305.6;
	3219	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	347;
	3219	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	105.7;
	3219	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.3;
	3219	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.8;
	3219	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.7;
	3219	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	303.3;
	3220	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	348.2;
	3220	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.3;
	3220	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189;
	3220	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.9;
	3220	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.2;
	3220	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	303.6;
	3221	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	362.1;
	3221	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.4;
	3221	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.4;
	3221	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.4;
	3221	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.2;
	3221	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310;
	3222	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	380.4;
	3222	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.1;
	3222	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.1;
	3222	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.6;
	3222	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.3;
	3222	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.5;
	3223	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421;
	3223	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.7;
	3223	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.3;
	3223	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.2;
	3223	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.3;
	3223	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340;
	3224	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.9;
	3224	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.9;
	3224	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.7;
	3224	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235;
	3224	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	3224	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.5;
	3225	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.2;
	3225	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150;
	3225	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.6;
	3225	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.4;
	3225	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.3;
	3225	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.9;
	3226	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.3;
	3226	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.9;
	3226	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242;
	3226	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.2;
	3226	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.1;
	3226	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.7;
	3227	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.9;
	3227	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.6;
	3227	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.2;
	3227	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.6;
	3227	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	3227	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.6;
	3228	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.6;
	3228	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.2;
	3228	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.9;
	3228	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.6;
	3228	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	3228	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.8;
	3229	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.7;
	3229	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.4;
	3229	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.3;
	3229	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.3;
	3229	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	3229	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.5;
	3230	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.4;
	3230	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.3;
	3230	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.2;
	3230	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.3;
	3230	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	3230	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.3;
	3231	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.1;
	3231	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.1;
	3231	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	259.5;
	3231	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260;
	3231	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	3231	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.3;
	3232	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.9;
	3232	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.2;
	3232	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.9;
	3232	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.5;
	3232	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	3232	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.4;
	3233	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.5;
	3233	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.3;
	3233	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	259.8;
	3233	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261;
	3233	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	3233	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.6;
	3234	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.9;
	3234	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.1;
	3234	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.8;
	3234	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.6;
	3234	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	3234	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.9;
	3235	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.1;
	3235	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.6;
	3235	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.1;
	3235	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.9;
	3235	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	3235	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.2;
	3236	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.8;
	3236	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.5;
	3236	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.5;
	3236	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.5;
	3236	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.3;
	3236	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.4;
	3237	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.2;
	3237	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.9;
	3237	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.6;
	3237	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.9;
	3237	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.7;
	3237	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.4;
	3238	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.9;
	3238	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.1;
	3238	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.5;
	3238	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.4;
	3238	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	3238	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376;
	3239	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.7;
	3239	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.6;
	3239	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.4;
	3239	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.8;
	3239	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58;
	3239	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.6;
	3240	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421;
	3240	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129;
	3240	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.5;
	3240	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.1;
	3240	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.2;
	3240	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.2;
	3241	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	400.4;
	3241	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.6;
	3241	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.7;
	3241	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.5;
	3241	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.9;
	3241	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.9;
	3242	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	371.6;
	3242	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113;
	3242	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194;
	3242	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.9;
	3242	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53;
	3242	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.8;
	3243	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	377.9;
	3243	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.7;
	3243	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.1;
	3243	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.4;
	3243	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.5;
	3243	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.5;
	3244	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	377.7;
	3244	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.5;
	3244	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.2;
	3244	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.4;
	3244	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.2;
	3244	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.5;
	3245	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	384.1;
	3245	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.8;
	3245	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.8;
	3245	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.6;
	3245	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.6;
	3245	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.8;
	3246	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.5;
	3246	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.2;
	3246	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.3;
	3246	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.8;
	3246	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.2;
	3246	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334;
	3247	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.9;
	3247	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.4;
	3247	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.5;
	3247	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.4;
	3247	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.5;
	3247	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.9;
	3248	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488;
	3248	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.1;
	3248	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.1;
	3248	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.6;
	3248	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	3248	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.8;
	3249	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.9;
	3249	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.4;
	3249	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.4;
	3249	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.6;
	3249	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.3;
	3249	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.4;
	3250	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.1;
	3250	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.3;
	3250	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.6;
	3250	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.4;
	3250	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.4;
	3250	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388;
	3251	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.7;
	3251	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153;
	3251	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.4;
	3251	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.8;
	3251	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	3251	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.6;
	3252	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.1;
	3252	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.2;
	3252	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.8;
	3252	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.6;
	3252	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.4;
	3252	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.7;
	3253	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.8;
	3253	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.4;
	3253	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.2;
	3253	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.5;
	3253	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	3253	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383;
	3254	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485;
	3254	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152;
	3254	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.8;
	3254	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.7;
	3254	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	3254	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.3;
	3255	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.4;
	3255	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152;
	3255	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.6;
	3255	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.5;
	3255	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	3255	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.4;
	3256	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.4;
	3256	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.7;
	3256	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254;
	3256	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.8;
	3256	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	3256	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386;
	3257	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.6;
	3257	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.2;
	3257	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.9;
	3257	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.4;
	3257	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	3257	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.4;
	3258	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.9;
	3258	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.9;
	3258	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.5;
	3258	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.1;
	3258	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	3258	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.8;
	3259	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.4;
	3259	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.2;
	3259	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.7;
	3259	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.4;
	3259	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	3259	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.6;
	3260	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.8;
	3260	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.9;
	3260	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248;
	3260	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.8;
	3260	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.6;
	3260	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.1;
	3261	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.9;
	3261	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.1;
	3261	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246;
	3261	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.8;
	3261	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	3261	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.9;
	3262	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.6;
	3262	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144;
	3262	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.3;
	3262	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.9;
	3262	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	3262	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.3;
	3263	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.2;
	3263	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.2;
	3263	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.9;
	3263	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.7;
	3263	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	3263	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.9;
	3264	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.5;
	3264	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.4;
	3264	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.6;
	3264	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.5;
	3264	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.5;
	3264	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.7;
	3265	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	396;
	3265	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.7;
	3265	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.1;
	3265	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.2;
	3265	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.6;
	3265	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.8;
	3266	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	372.8;
	3266	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.1;
	3266	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.3;
	3266	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.3;
	3266	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.2;
	3266	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.6;
	3267	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	371.3;
	3267	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.1;
	3267	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.9;
	3267	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.2;
	3267	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.4;
	3267	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311.5;
	3268	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	367.2;
	3268	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.2;
	3268	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.1;
	3268	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.4;
	3268	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.9;
	3268	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310;
	3269	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	377.7;
	3269	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113;
	3269	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.4;
	3269	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.5;
	3269	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53;
	3269	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.5;
	3270	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403.3;
	3270	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.6;
	3270	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.8;
	3270	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.9;
	3270	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	3270	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.8;
	3271	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.9;
	3271	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.3;
	3271	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.4;
	3271	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.7;
	3271	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.8;
	3271	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.3;
	3272	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.4;
	3272	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143;
	3272	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.9;
	3272	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.7;
	3272	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	3272	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.4;
	3273	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.7;
	3273	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.5;
	3273	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.7;
	3273	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.4;
	3273	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	3273	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.8;
	3274	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473;
	3274	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.9;
	3274	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.6;
	3274	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.9;
	3274	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	3274	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.8;
	3275	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.2;
	3275	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.5;
	3275	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.3;
	3275	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.3;
	3275	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	3275	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.5;
	3276	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.4;
	3276	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.2;
	3276	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.9;
	3276	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.2;
	3276	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	3276	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.9;
	3277	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.4;
	3277	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.5;
	3277	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.9;
	3277	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248;
	3277	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	3277	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.7;
	3278	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.5;
	3278	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.1;
	3278	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.7;
	3278	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249;
	3278	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	3278	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.6;
	3279	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472;
	3279	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.4;
	3279	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.7;
	3279	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.7;
	3279	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	3279	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.7;
	3280	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.4;
	3280	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.2;
	3280	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.4;
	3280	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.6;
	3280	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	3280	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.5;
	3281	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.5;
	3281	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.9;
	3281	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.2;
	3281	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.7;
	3281	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	3281	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.9;
	3282	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.3;
	3282	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146;
	3282	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.3;
	3282	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.1;
	3282	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	3282	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.1;
	3283	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.6;
	3283	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.8;
	3283	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.2;
	3283	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.4;
	3283	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	3283	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.3;
	3284	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.3;
	3284	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.3;
	3284	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.4;
	3284	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.7;
	3284	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	3284	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.9;
	3285	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.9;
	3285	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.2;
	3285	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.9;
	3285	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.9;
	3285	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	3285	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.5;
	3286	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.5;
	3286	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140;
	3286	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.9;
	3286	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.4;
	3286	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	3286	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.6;
	3287	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	420.5;
	3287	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.8;
	3287	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.4;
	3287	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.1;
	3287	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	3287	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.1;
	3288	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.5;
	3288	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.1;
	3288	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.6;
	3288	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.2;
	3288	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	3288	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.4;
	3289	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	385.6;
	3289	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.2;
	3289	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.6;
	3289	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.8;
	3289	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.3;
	3289	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.4;
	3290	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	372.6;
	3290	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.8;
	3290	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.6;
	3290	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.5;
	3290	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54;
	3290	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.1;
	3291	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	362.2;
	3291	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.1;
	3291	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.4;
	3291	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.9;
	3291	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53;
	3291	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.3;
	3292	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	363.9;
	3292	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.3;
	3292	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.1;
	3292	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190;
	3292	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.5;
	3292	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309.1;
	3293	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	381;
	3293	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112;
	3293	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.5;
	3293	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.5;
	3293	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.7;
	3293	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.5;
	3294	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.7;
	3294	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.2;
	3294	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.1;
	3294	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.9;
	3294	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.3;
	3294	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.4;
	3295	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.3;
	3295	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.9;
	3295	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.9;
	3295	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.1;
	3295	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.9;
	3295	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.5;
	3296	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.7;
	3296	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.5;
	3296	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.2;
	3296	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.2;
	3296	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	3296	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.5;
	3297	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.8;
	3297	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.8;
	3297	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.3;
	3297	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238;
	3297	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	3297	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.7;
	3298	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.8;
	3298	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.9;
	3298	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.4;
	3298	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.5;
	3298	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	3298	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.6;
	3299	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.3;
	3299	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.4;
	3299	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.2;
	3299	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.5;
	3299	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	3299	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.9;
	3300	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.3;
	3300	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.7;
	3300	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241;
	3300	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.9;
	3300	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	3300	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.7;
	3301	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457;
	3301	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.7;
	3301	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244;
	3301	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.1;
	3301	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	3301	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.6;
	3302	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.6;
	3302	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146;
	3302	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.2;
	3302	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.2;
	3302	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	3302	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.6;
	3303	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.3;
	3303	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.4;
	3303	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.6;
	3303	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.2;
	3303	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	3303	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.3;
	3304	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.3;
	3304	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.3;
	3304	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.1;
	3304	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.5;
	3304	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	3304	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.9;
	3305	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.6;
	3305	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.7;
	3305	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.5;
	3305	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.3;
	3305	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	3305	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.8;
	3306	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.4;
	3306	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.1;
	3306	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.7;
	3306	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.9;
	3306	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	3306	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.5;
	3307	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.9;
	3307	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.8;
	3307	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.9;
	3307	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.7;
	3307	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	3307	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.5;
	3308	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.6;
	3308	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.6;
	3308	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.8;
	3308	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.9;
	3308	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	3308	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365;
	3309	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.7;
	3309	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148;
	3309	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.1;
	3309	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.8;
	3309	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	3309	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.9;
	3310	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436;
	3310	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.2;
	3310	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.6;
	3310	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.1;
	3310	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	3310	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.6;
	3311	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.3;
	3311	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.5;
	3311	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.8;
	3311	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.9;
	3311	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.7;
	3311	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.6;
	3312	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394.3;
	3312	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.8;
	3312	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.2;
	3312	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.3;
	3312	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	3312	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.8;
	3313	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	376.2;
	3313	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.9;
	3313	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.4;
	3313	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.2;
	3313	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.9;
	3313	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.5;
	3314	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	360.4;
	3314	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.4;
	3314	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.4;
	3314	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.8;
	3314	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.7;
	3314	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309;
	3315	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	350.8;
	3315	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	105.7;
	3315	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.7;
	3315	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.4;
	3315	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.7;
	3315	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	304.4;
	3316	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	360.1;
	3316	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.4;
	3316	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.2;
	3316	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191;
	3316	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.3;
	3316	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.1;
	3317	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	368.6;
	3317	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111;
	3317	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.5;
	3317	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.5;
	3317	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.9;
	3317	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.8;
	3318	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.6;
	3318	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121;
	3318	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.5;
	3318	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.2;
	3318	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	3318	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.7;
	3319	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.9;
	3319	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.1;
	3319	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.1;
	3319	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.8;
	3319	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.4;
	3319	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339;
	3320	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.5;
	3320	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.2;
	3320	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225;
	3320	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.6;
	3320	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59;
	3320	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.8;
	3321	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.8;
	3321	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143;
	3321	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.3;
	3321	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240;
	3321	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	3321	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.8;
	3322	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.1;
	3322	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.7;
	3322	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.5;
	3322	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.1;
	3322	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	3322	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.9;
	3323	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.6;
	3323	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.1;
	3323	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.9;
	3323	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.4;
	3323	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	3323	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.7;
	3324	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.9;
	3324	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.1;
	3324	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.2;
	3324	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.2;
	3324	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	3324	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.9;
	3325	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.1;
	3325	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.4;
	3325	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.7;
	3325	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249;
	3325	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	3325	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.6;
	3326	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.8;
	3326	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.6;
	3326	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.1;
	3326	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.5;
	3326	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	3326	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.7;
	3327	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.4;
	3327	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.9;
	3327	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.4;
	3327	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.6;
	3327	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	3327	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.5;
	3328	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.2;
	3328	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.7;
	3328	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.8;
	3328	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.6;
	3328	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	3328	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.1;
	3329	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.7;
	3329	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.3;
	3329	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.4;
	3329	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.7;
	3329	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	3329	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366;
	3330	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.7;
	3330	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.3;
	3330	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.4;
	3330	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244;
	3330	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	3330	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.4;
	3331	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.8;
	3331	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.3;
	3331	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.7;
	3331	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.7;
	3331	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	3331	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.1;
	3332	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.7;
	3332	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.4;
	3332	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.8;
	3332	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.6;
	3332	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	3332	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.3;
	3333	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	427.4;
	3333	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.8;
	3333	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.1;
	3333	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.6;
	3333	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.6;
	3333	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.8;
	3334	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	430.1;
	3334	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.9;
	3334	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.9;
	3334	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228;
	3334	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	3334	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.4;
	3335	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422;
	3335	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122;
	3335	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213;
	3335	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.2;
	3335	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	3335	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.1;
	3336	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.1;
	3336	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.4;
	3336	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.3;
	3336	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.9;
	3336	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.8;
	3336	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.7;
	3337	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	390.5;
	3337	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.2;
	3337	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.3;
	3337	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.4;
	3337	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.8;
	3337	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.9;
	3338	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	347.9;
	3338	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	101.9;
	3338	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.7;
	3338	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.8;
	3338	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.6;
	3338	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	303.4;
	3339	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	348.2;
	3339	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	102.2;
	3339	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185;
	3339	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.4;
	3339	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.4;
	3339	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	302.5;
	3340	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	352.3;
	3340	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	101.8;
	3340	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.6;
	3340	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.1;
	3340	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.8;
	3340	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	304;
	3341	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	357.8;
	3341	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	104.1;
	3341	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.8;
	3341	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.3;
	3341	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.9;
	3341	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	307;
	3342	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	374.5;
	3342	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111;
	3342	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.9;
	3342	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.7;
	3342	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.5;
	3342	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.4;
	3343	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.2;
	3343	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.8;
	3343	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.3;
	3343	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.5;
	3343	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.8;
	3343	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.8;
	3344	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	388.6;
	3344	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.6;
	3344	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.1;
	3344	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.4;
	3344	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.8;
	3344	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.5;
	3345	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	401.7;
	3345	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.1;
	3345	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.5;
	3345	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.6;
	3345	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.6;
	3345	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.8;
	3346	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.5;
	3346	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.5;
	3346	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.7;
	3346	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.6;
	3346	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	3346	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.6;
	3347	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.8;
	3347	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.7;
	3347	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.1;
	3347	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.8;
	3347	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	3347	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.1;
	3348	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.8;
	3348	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.4;
	3348	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.3;
	3348	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.4;
	3348	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	3348	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.1;
	3349	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.5;
	3349	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.6;
	3349	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.2;
	3349	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.1;
	3349	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	3349	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356;
	3350	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.5;
	3350	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.9;
	3350	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.8;
	3350	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.4;
	3350	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	3350	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.4;
	3351	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.4;
	3351	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.8;
	3351	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.1;
	3351	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.1;
	3351	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	3351	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.1;
	3352	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.7;
	3352	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.5;
	3352	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.7;
	3352	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.6;
	3352	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	3352	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.1;
	3353	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.6;
	3353	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.6;
	3353	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.6;
	3353	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.2;
	3353	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	3353	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.4;
	3354	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.5;
	3354	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.5;
	3354	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.3;
	3354	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.1;
	3354	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	3354	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.8;
	3355	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.2;
	3355	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.5;
	3355	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.7;
	3355	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.4;
	3355	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	3355	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.5;
	3356	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.8;
	3356	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.3;
	3356	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.3;
	3356	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.1;
	3356	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	3356	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.8;
	3357	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431;
	3357	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137;
	3357	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.3;
	3357	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.2;
	3357	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	3357	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.9;
	3358	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	426.3;
	3358	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.5;
	3358	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.8;
	3358	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.7;
	3358	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	3358	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.1;
	3359	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.5;
	3359	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.4;
	3359	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.5;
	3359	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214;
	3359	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	3359	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.5;
	3360	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	384.5;
	3360	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.2;
	3360	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.2;
	3360	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.4;
	3360	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59;
	3360	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.2;
	3361	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	360.1;
	3361	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.7;
	3361	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.3;
	3361	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.8;
	3361	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.1;
	3361	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311.4;
	3362	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	342;
	3362	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	105.5;
	3362	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180;
	3362	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.9;
	3362	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.9;
	3362	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	299.3;
	3363	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	345.3;
	3363	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	101.9;
	3363	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.8;
	3363	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.9;
	3363	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.2;
	3363	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	299.5;
	3364	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	338;
	3364	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	102.4;
	3364	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	179.3;
	3364	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	183.6;
	3364	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.6;
	3364	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	296.2;
	3365	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	343;
	3365	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	103.6;
	3365	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.2;
	3365	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.3;
	3365	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.5;
	3365	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	298.9;
	3366	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	349.9;
	3366	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.3;
	3366	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181.3;
	3366	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.7;
	3366	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.4;
	3366	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	302.5;
	3367	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	357.7;
	3367	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.4;
	3367	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.2;
	3367	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.8;
	3367	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	3367	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	306.8;
	3368	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	366;
	3368	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.7;
	3368	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.4;
	3368	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.3;
	3368	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.5;
	3368	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311.3;
	3369	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	387;
	3369	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.6;
	3369	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.5;
	3369	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.4;
	3369	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.2;
	3369	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.8;
	3370	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.7;
	3370	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.3;
	3370	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.1;
	3370	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.1;
	3370	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	3370	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.1;
	3371	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.6;
	3371	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.7;
	3371	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.4;
	3371	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.8;
	3371	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	3371	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.7;
	3372	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.4;
	3372	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.1;
	3372	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.7;
	3372	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.5;
	3372	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	3372	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.7;
	3373	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435;
	3373	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.8;
	3373	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.3;
	3373	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.4;
	3373	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	3373	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.7;
	3374	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.1;
	3374	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.2;
	3374	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.2;
	3374	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.8;
	3374	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	3374	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.7;
	3375	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.6;
	3375	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.6;
	3375	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.5;
	3375	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.4;
	3375	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.2;
	3375	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.1;
	3376	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.3;
	3376	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.9;
	3376	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.3;
	3376	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.5;
	3376	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	3376	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.2;
	3377	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.8;
	3377	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.2;
	3377	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.2;
	3377	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.9;
	3377	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	3377	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.9;
	3378	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.4;
	3378	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.3;
	3378	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.3;
	3378	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.8;
	3378	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.2;
	3378	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.9;
	3379	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.6;
	3379	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.4;
	3379	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225;
	3379	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.2;
	3379	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	3379	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.4;
	3380	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447.5;
	3380	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.9;
	3380	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.5;
	3380	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.7;
	3380	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	3380	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.2;
	3381	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.1;
	3381	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.2;
	3381	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.5;
	3381	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.7;
	3381	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	3381	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.3;
	3382	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.6;
	3382	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.8;
	3382	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.4;
	3382	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.4;
	3382	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	3382	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354;
	3383	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.4;
	3383	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.1;
	3383	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.5;
	3383	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.4;
	3383	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.7;
	3383	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.8;
	3384	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	400.3;
	3384	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.8;
	3384	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.9;
	3384	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.9;
	3384	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.8;
	3384	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.1;
	3385	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	382.7;
	3385	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.3;
	3385	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.7;
	3385	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.6;
	3385	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.5;
	3385	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.1;
	3386	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	365.7;
	3386	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.7;
	3386	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194;
	3386	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.2;
	3386	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54;
	3386	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310.6;
	3387	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	364;
	3387	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108;
	3387	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.9;
	3387	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.7;
	3387	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.1;
	3387	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309.5;
	3388	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	361.3;
	3388	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.8;
	3388	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.7;
	3388	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.6;
	3388	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.2;
	3388	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.3;
	3389	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	374.4;
	3389	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.3;
	3389	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.1;
	3389	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192;
	3389	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	50.8;
	3389	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.2;
	3390	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394;
	3390	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.3;
	3390	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.1;
	3390	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.5;
	3390	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.4;
	3390	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.1;
	3391	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.2;
	3391	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.2;
	3391	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.4;
	3391	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.4;
	3391	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.3;
	3391	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340;
	3392	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.9;
	3392	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.4;
	3392	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.3;
	3392	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.7;
	3392	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.7;
	3392	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.5;
	3393	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.6;
	3393	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.8;
	3393	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.7;
	3393	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.7;
	3393	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.6;
	3393	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.3;
	3394	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.5;
	3394	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.1;
	3394	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.6;
	3394	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.3;
	3394	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.5;
	3394	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.1;
	3395	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.9;
	3395	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.5;
	3395	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.1;
	3395	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.2;
	3395	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	3395	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.6;
	3396	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.7;
	3396	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.9;
	3396	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243;
	3396	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.9;
	3396	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.3;
	3396	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.1;
	3397	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.3;
	3397	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.7;
	3397	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.5;
	3397	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.1;
	3397	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	3397	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.6;
	3398	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.5;
	3398	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.5;
	3398	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243;
	3398	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.5;
	3398	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	3398	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.7;
	3399	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.8;
	3399	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.4;
	3399	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.9;
	3399	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.9;
	3399	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	3399	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.4;
	3400	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.1;
	3400	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154;
	3400	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.4;
	3400	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.3;
	3400	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	3400	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.2;
	3401	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.2;
	3401	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.3;
	3401	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.7;
	3401	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248;
	3401	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	3401	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.5;
	3402	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.3;
	3402	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.2;
	3402	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.2;
	3402	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.1;
	3402	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	3402	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395;
	3403	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525.5;
	3403	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.9;
	3403	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.8;
	3403	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.6;
	3403	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.3;
	3403	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.7;
	3404	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.5;
	3404	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.1;
	3404	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.9;
	3404	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.1;
	3404	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	3404	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.3;
	3405	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.7;
	3405	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.5;
	3405	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.3;
	3405	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.5;
	3405	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	3405	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.5;
	3406	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.6;
	3406	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.5;
	3406	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233;
	3406	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.3;
	3406	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	3406	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.3;
	3407	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474;
	3407	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.6;
	3407	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.4;
	3407	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.9;
	3407	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	3407	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.8;
	3408	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.4;
	3408	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126;
	3408	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.7;
	3408	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.2;
	3408	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	3408	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.4;
	3409	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.8;
	3409	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.1;
	3409	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.4;
	3409	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.7;
	3409	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.5;
	3409	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.7;
	3410	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398;
	3410	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.2;
	3410	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.6;
	3410	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.8;
	3410	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.9;
	3410	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.4;
	3411	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	382.8;
	3411	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.5;
	3411	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187;
	3411	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.5;
	3411	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.1;
	3411	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.8;
	3412	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	372.5;
	3412	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.1;
	3412	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.2;
	3412	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.5;
	3412	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.9;
	3412	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.5;
	3413	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	384.5;
	3413	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.1;
	3413	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.3;
	3413	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.4;
	3413	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.4;
	3413	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.6;
	3414	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394.9;
	3414	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.7;
	3414	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.3;
	3414	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.3;
	3414	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.9;
	3414	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.4;
	3415	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.8;
	3415	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.2;
	3415	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.1;
	3415	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220;
	3415	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.6;
	3415	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.5;
	3416	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.1;
	3416	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.2;
	3416	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.2;
	3416	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.6;
	3416	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.7;
	3416	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.6;
	3417	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.3;
	3417	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.9;
	3417	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.6;
	3417	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246;
	3417	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.4;
	3417	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.4;
	3418	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.9;
	3418	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.6;
	3418	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245;
	3418	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.6;
	3418	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	3418	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.7;
	3419	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.7;
	3419	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.6;
	3419	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.8;
	3419	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.4;
	3419	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	3419	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.9;
	3420	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.5;
	3420	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.9;
	3420	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.9;
	3420	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.7;
	3420	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61;
	3420	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.4;
	3421	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.4;
	3421	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.2;
	3421	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.7;
	3421	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.7;
	3421	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.4;
	3421	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.8;
	3422	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.6;
	3422	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.9;
	3422	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	259.3;
	3422	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264;
	3422	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	3422	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.4;
	3423	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.1;
	3423	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.6;
	3423	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.1;
	3423	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265.9;
	3423	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	3423	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.1;
	3424	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.9;
	3424	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.5;
	3424	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	266.9;
	3424	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	271.4;
	3424	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.3;
	3424	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.3;
	3425	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.1;
	3425	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.5;
	3425	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	269.1;
	3425	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	273.8;
	3425	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	3425	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.6;
	3426	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.3;
	3426	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.6;
	3426	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	267.9;
	3426	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	271.9;
	3426	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.1;
	3426	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.8;
	3427	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.1;
	3427	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.7;
	3427	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	263.3;
	3427	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267;
	3427	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	3427	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.4;
	3428	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.9;
	3428	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.2;
	3428	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	264.1;
	3428	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267.5;
	3428	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	3428	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.9;
	3429	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.3;
	3429	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.7;
	3429	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.1;
	3429	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265;
	3429	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	3429	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.8;
	3430	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.5;
	3430	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.3;
	3430	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.1;
	3430	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.5;
	3430	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	3430	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.7;
	3431	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.6;
	3431	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.7;
	3431	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.8;
	3431	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.4;
	3431	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	3431	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.3;
	3432	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.6;
	3432	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.9;
	3432	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.6;
	3432	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224;
	3432	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	3432	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.9;
	3433	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.4;
	3433	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.6;
	3433	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.3;
	3433	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.6;
	3433	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.5;
	3433	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337;
	3434	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	376.3;
	3434	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.9;
	3434	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.9;
	3434	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.3;
	3434	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.4;
	3434	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.2;
	3435	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	371.1;
	3435	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.8;
	3435	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.6;
	3435	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.1;
	3435	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52;
	3435	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.6;
	3436	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	377.3;
	3436	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.9;
	3436	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.1;
	3436	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.6;
	3436	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.7;
	3436	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317;
	3437	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394;
	3437	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.4;
	3437	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.2;
	3437	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.9;
	3437	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.9;
	3437	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.8;
	3438	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	420.6;
	3438	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.3;
	3438	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.3;
	3438	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.5;
	3438	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.1;
	3438	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.6;
	3439	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.9;
	3439	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.4;
	3439	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221;
	3439	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.1;
	3439	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	3439	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.7;
	3440	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470;
	3440	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.1;
	3440	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234;
	3440	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.2;
	3440	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.4;
	3440	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.7;
	3441	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.1;
	3441	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.8;
	3441	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.1;
	3441	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.8;
	3441	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.4;
	3441	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.6;
	3442	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503;
	3442	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.8;
	3442	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247;
	3442	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.1;
	3442	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	3442	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.6;
	3443	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502;
	3443	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.5;
	3443	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.9;
	3443	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.7;
	3443	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	3443	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.3;
	3444	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504;
	3444	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.7;
	3444	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.2;
	3444	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.5;
	3444	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	3444	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.1;
	3445	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.1;
	3445	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.4;
	3445	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.2;
	3445	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.7;
	3445	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	3445	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388;
	3446	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.8;
	3446	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.7;
	3446	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.6;
	3446	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.8;
	3446	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	3446	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.1;
	3447	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.3;
	3447	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.4;
	3447	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.8;
	3447	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	269.1;
	3447	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	3447	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.6;
	3448	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.9;
	3448	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.9;
	3448	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.5;
	3448	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	271.6;
	3448	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	3448	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.7;
	3449	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.9;
	3449	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.4;
	3449	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.1;
	3449	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.2;
	3449	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	3449	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.4;
	3450	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.6;
	3450	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.8;
	3450	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.9;
	3450	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265.9;
	3450	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	3450	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.2;
	3451	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.3;
	3451	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.2;
	3451	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.1;
	3451	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.3;
	3451	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	3451	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.2;
	3452	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.9;
	3452	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.6;
	3452	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.7;
	3452	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.1;
	3452	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	3452	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.4;
	3453	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.4;
	3453	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.7;
	3453	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249;
	3453	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256;
	3453	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	3453	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.6;
	3454	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.4;
	3454	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.5;
	3454	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.3;
	3454	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.1;
	3454	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	3454	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.9;
	3455	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.1;
	3455	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.5;
	3455	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.5;
	3455	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.5;
	3455	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	3455	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.5;
	3456	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415;
	3456	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.7;
	3456	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.6;
	3456	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.9;
	3456	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.6;
	3456	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.9;
	3457	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.3;
	3457	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.3;
	3457	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.4;
	3457	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.6;
	3457	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.8;
	3457	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.2;
	3458	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	369.1;
	3458	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.6;
	3458	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.8;
	3458	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.7;
	3458	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54;
	3458	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.8;
	3459	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	370.3;
	3459	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109;
	3459	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.3;
	3459	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.2;
	3459	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.7;
	3459	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.5;
	3460	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	368.1;
	3460	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110;
	3460	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.2;
	3460	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.2;
	3460	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.3;
	3460	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.8;
	3461	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	373.5;
	3461	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.6;
	3461	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.8;
	3461	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199;
	3461	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.1;
	3461	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.3;
	3462	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.3;
	3462	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.5;
	3462	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.2;
	3462	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.7;
	3462	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.1;
	3462	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.4;
	3463	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440;
	3463	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.9;
	3463	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.4;
	3463	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.9;
	3463	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.7;
	3463	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.5;
	3464	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.7;
	3464	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147;
	3464	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.5;
	3464	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.7;
	3464	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58;
	3464	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.2;
	3465	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.1;
	3465	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.2;
	3465	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.5;
	3465	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.1;
	3465	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	3465	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.6;
	3466	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.3;
	3466	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.9;
	3466	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.9;
	3466	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.6;
	3466	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	3466	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.8;
	3467	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.5;
	3467	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.1;
	3467	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.5;
	3467	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261;
	3467	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	3467	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.3;
	3468	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.7;
	3468	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155;
	3468	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.9;
	3468	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.9;
	3468	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	3468	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.6;
	3469	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.4;
	3469	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.4;
	3469	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.5;
	3469	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	263.9;
	3469	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	3469	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.1;
	3470	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.7;
	3470	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.6;
	3470	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251;
	3470	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	263.2;
	3470	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	3470	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.7;
	3471	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.7;
	3471	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.6;
	3471	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251;
	3471	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265;
	3471	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68;
	3471	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.5;
	3472	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.6;
	3472	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.3;
	3472	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.4;
	3472	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267.1;
	3472	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	3472	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.8;
	3473	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.6;
	3473	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153;
	3473	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.6;
	3473	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.8;
	3473	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.1;
	3473	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.5;
	3474	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.1;
	3474	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.3;
	3474	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.3;
	3474	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.3;
	3474	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.4;
	3474	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.5;
	3475	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492;
	3475	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149;
	3475	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.1;
	3475	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.6;
	3475	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.3;
	3475	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.9;
	3476	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.1;
	3476	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.2;
	3476	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.8;
	3476	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.9;
	3476	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	3476	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.3;
	3477	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.6;
	3477	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.9;
	3477	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.4;
	3477	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.8;
	3477	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.8;
	3477	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.5;
	3478	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.1;
	3478	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143;
	3478	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.6;
	3478	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.4;
	3478	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	3478	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.5;
	3479	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.6;
	3479	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.7;
	3479	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.6;
	3479	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.9;
	3479	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	3479	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.3;
	3480	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.1;
	3480	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.4;
	3480	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.9;
	3480	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.7;
	3480	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	3480	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.4;
	3481	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	396.5;
	3481	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.2;
	3481	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.7;
	3481	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.8;
	3481	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.3;
	3481	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.6;
	3482	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	363.9;
	3482	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.2;
	3482	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.1;
	3482	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.3;
	3482	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.9;
	3482	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310.6;
	3483	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	366.3;
	3483	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.1;
	3483	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189;
	3483	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.2;
	3483	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.5;
	3483	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311;
	3484	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	361.4;
	3484	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109;
	3484	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.1;
	3484	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.2;
	3484	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.8;
	3484	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309.4;
	3485	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	376.6;
	3485	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.1;
	3485	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.2;
	3485	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.4;
	3485	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.6;
	3485	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.2;
	3486	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.6;
	3486	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.8;
	3486	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.8;
	3486	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.4;
	3486	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.9;
	3486	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.8;
	3487	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	420.8;
	3487	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.8;
	3487	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.4;
	3487	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.4;
	3487	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.7;
	3487	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.3;
	3488	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.9;
	3488	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.3;
	3488	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.9;
	3488	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.7;
	3488	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.5;
	3488	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.9;
	3489	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.8;
	3489	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145;
	3489	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.6;
	3489	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.8;
	3489	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	3489	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.7;
	3490	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.4;
	3490	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.7;
	3490	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.5;
	3490	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.4;
	3490	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	3490	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379;
	3491	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.2;
	3491	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.8;
	3491	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.7;
	3491	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.7;
	3491	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	3491	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.1;
	3492	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.7;
	3492	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.7;
	3492	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.1;
	3492	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.4;
	3492	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.2;
	3492	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.1;
	3493	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.4;
	3493	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.4;
	3493	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.6;
	3493	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.7;
	3493	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.5;
	3493	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.2;
	3494	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.3;
	3494	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.8;
	3494	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.1;
	3494	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.1;
	3494	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.1;
	3494	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.3;
	3495	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.1;
	3495	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.7;
	3495	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.8;
	3495	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.4;
	3495	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.8;
	3495	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.5;
	3496	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.1;
	3496	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.5;
	3496	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.9;
	3496	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.6;
	3496	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75;
	3496	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.5;
	3497	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.7;
	3497	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.9;
	3497	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241;
	3497	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259;
	3497	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.1;
	3497	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.3;
	3498	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.3;
	3498	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144;
	3498	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.3;
	3498	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.6;
	3498	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.1;
	3498	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.8;
	3499	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.5;
	3499	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.7;
	3499	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.1;
	3499	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.7;
	3499	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.2;
	3499	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.8;
	3500	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.9;
	3500	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.6;
	3500	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.9;
	3500	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.1;
	3500	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.8;
	3500	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366;
	3501	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.9;
	3501	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.4;
	3501	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.1;
	3501	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.1;
	3501	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.2;
	3501	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.1;
	3502	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.3;
	3502	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.3;
	3502	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.4;
	3502	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.8;
	3502	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.4;
	3502	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360;
	3503	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.2;
	3503	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.2;
	3503	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.2;
	3503	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.4;
	3503	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.3;
	3503	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.8;
	3504	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.5;
	3504	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.5;
	3504	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.8;
	3504	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.5;
	3504	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	3504	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.5;
	3505	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	382;
	3505	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111;
	3505	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.8;
	3505	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.9;
	3505	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.7;
	3505	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.9;
	3506	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	362.6;
	3506	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.7;
	3506	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.8;
	3506	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.7;
	3506	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58;
	3506	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310.1;
	3507	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	346.2;
	3507	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	105.6;
	3507	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.8;
	3507	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.4;
	3507	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.4;
	3507	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	302.6;
	3508	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	347.1;
	3508	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	105.5;
	3508	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.7;
	3508	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.3;
	3508	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.5;
	3508	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	303;
	3509	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	364.1;
	3509	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.4;
	3509	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.1;
	3509	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.1;
	3509	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.7;
	3509	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310.6;
	3510	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	374.8;
	3510	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.1;
	3510	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.9;
	3510	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.8;
	3510	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	3510	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.4;
	3511	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.7;
	3511	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.5;
	3511	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197;
	3511	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.1;
	3511	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.6;
	3511	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.3;
	3512	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	386.8;
	3512	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.8;
	3512	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.9;
	3512	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.1;
	3512	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	3512	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325;
	3513	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.3;
	3513	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.3;
	3513	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.8;
	3513	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.5;
	3513	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	3513	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332;
	3514	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.6;
	3514	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.4;
	3514	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.6;
	3514	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.8;
	3514	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	3514	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.3;
	3515	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415;
	3515	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.6;
	3515	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.8;
	3515	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.3;
	3515	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.9;
	3515	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.5;
	3516	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.8;
	3516	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.7;
	3516	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.6;
	3516	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.4;
	3516	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.6;
	3516	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.7;
	3517	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421;
	3517	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.6;
	3517	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.7;
	3517	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239;
	3517	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77;
	3517	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346;
	3518	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.3;
	3518	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.5;
	3518	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.3;
	3518	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.9;
	3518	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.1;
	3518	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340;
	3519	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.1;
	3519	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.7;
	3519	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226;
	3519	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.2;
	3519	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.9;
	3519	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.5;
	3520	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.6;
	3520	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.2;
	3520	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.9;
	3520	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.4;
	3520	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.8;
	3520	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.9;
	3521	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.7;
	3521	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.7;
	3521	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.4;
	3521	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237;
	3521	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.4;
	3521	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.1;
	3522	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428.4;
	3522	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.1;
	3522	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.6;
	3522	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.9;
	3522	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.7;
	3522	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.8;
	3523	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	426.6;
	3523	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.6;
	3523	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.1;
	3523	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.9;
	3523	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.3;
	3523	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.7;
	3524	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.7;
	3524	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.5;
	3524	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.1;
	3524	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.1;
	3524	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83;
	3524	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.2;
	3525	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	400.4;
	3525	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.6;
	3525	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226;
	3525	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.3;
	3525	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.4;
	3525	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337;
	3526	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	388;
	3526	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.6;
	3526	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.9;
	3526	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.3;
	3526	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.1;
	3526	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331;
	3527	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	384.7;
	3527	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.4;
	3527	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.6;
	3527	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.1;
	3527	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.7;
	3527	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.4;
	3528	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	354.8;
	3528	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.1;
	3528	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.5;
	3528	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216;
	3528	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	3528	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.5;
	3529	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	330;
	3529	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.2;
	3529	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.1;
	3529	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.9;
	3529	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	3529	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	299.8;
	3530	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	327.7;
	3530	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	103.8;
	3530	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.8;
	3530	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.4;
	3530	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	3530	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	296.2;
	3531	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	324.6;
	3531	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	102.7;
	3531	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.7;
	3531	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.5;
	3531	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	3531	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	294.1;
	3532	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	326.7;
	3532	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	102.2;
	3532	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.1;
	3532	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.9;
	3532	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	3532	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	294.5;
	3533	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	330.7;
	3533	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	104.1;
	3533	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.3;
	3533	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.6;
	3533	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.1;
	3533	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	296.4;
	3534	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	333.1;
	3534	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.5;
	3534	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.4;
	3534	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.7;
	3534	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.8;
	3534	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	297.8;
	3535	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	330.7;
	3535	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.3;
	3535	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.9;
	3535	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.1;
	3535	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	3535	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	298.1;
	3536	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	342.6;
	3536	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.1;
	3536	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.9;
	3536	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.8;
	3536	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	3536	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	304;
	3537	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	365;
	3537	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.4;
	3537	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.4;
	3537	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.3;
	3537	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	3537	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.9;
	3538	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	375.9;
	3538	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.2;
	3538	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.4;
	3538	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.1;
	3538	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.7;
	3538	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.7;
	3539	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	390;
	3539	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.3;
	3539	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.2;
	3539	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.2;
	3539	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.4;
	3539	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.8;
	3540	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	395.2;
	3540	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.1;
	3540	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212;
	3540	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.6;
	3540	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.6;
	3540	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.6;
	3541	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.4;
	3541	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.8;
	3541	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.6;
	3541	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.5;
	3541	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.3;
	3541	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.2;
	3542	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.7;
	3542	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.7;
	3542	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.6;
	3542	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.8;
	3542	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.8;
	3542	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.1;
	3543	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.9;
	3543	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.3;
	3543	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.1;
	3543	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237;
	3543	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.5;
	3543	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.4;
	3544	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.1;
	3544	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.9;
	3544	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.7;
	3544	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.3;
	3544	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.2;
	3544	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.8;
	3545	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.4;
	3545	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.9;
	3545	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.2;
	3545	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.1;
	3545	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.7;
	3545	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.1;
	3546	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.9;
	3546	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.3;
	3546	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.2;
	3546	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.1;
	3546	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.2;
	3546	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.8;
	3547	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.2;
	3547	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.8;
	3547	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.8;
	3547	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.1;
	3547	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.4;
	3547	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344;
	3548	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.8;
	3548	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138;
	3548	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.4;
	3548	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.5;
	3548	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83;
	3548	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.7;
	3549	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	400.9;
	3549	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.9;
	3549	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.5;
	3549	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.5;
	3549	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.1;
	3549	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.5;
	3550	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	401.1;
	3550	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.2;
	3550	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.4;
	3550	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.2;
	3550	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.8;
	3550	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.9;
	3551	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383;
	3551	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.6;
	3551	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.5;
	3551	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225;
	3551	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.3;
	3551	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.7;
	3552	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	372.8;
	3552	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.9;
	3552	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.9;
	3552	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.5;
	3552	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69;
	3552	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.8;
	3553	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	361.5;
	3553	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.1;
	3553	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.7;
	3553	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.4;
	3553	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	3553	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.7;
	3554	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	324.4;
	3554	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	104.5;
	3554	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.5;
	3554	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.6;
	3554	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.6;
	3554	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	295.9;
	3555	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	330.1;
	3555	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	102.8;
	3555	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.6;
	3555	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.3;
	3555	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.7;
	3555	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	296.5;
	3556	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	318.8;
	3556	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	102.7;
	3556	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.1;
	3556	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.7;
	3556	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.7;
	3556	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	291.8;
	3557	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	316.8;
	3557	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	104;
	3557	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.6;
	3557	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195;
	3557	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.8;
	3557	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	291.6;
	3558	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	326.2;
	3558	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.9;
	3558	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.7;
	3558	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.1;
	3558	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.9;
	3558	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	297.4;
	3559	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	345.3;
	3559	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.1;
	3559	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.2;
	3559	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.5;
	3559	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57;
	3559	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	306.3;
	3560	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	359.6;
	3560	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.8;
	3560	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.1;
	3560	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.3;
	3560	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.6;
	3560	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.9;
	3561	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	380.3;
	3561	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.9;
	3561	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.3;
	3561	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.1;
	3561	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	3561	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.8;
	3562	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.7;
	3562	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.5;
	3562	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.2;
	3562	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.9;
	3562	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	3562	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.2;
	3563	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	427.8;
	3563	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.5;
	3563	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.1;
	3563	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.7;
	3563	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.7;
	3563	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351;
	3564	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442.5;
	3564	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.9;
	3564	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241;
	3564	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.8;
	3564	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.9;
	3564	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359;
	3565	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.7;
	3565	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.5;
	3565	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.6;
	3565	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.7;
	3565	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.2;
	3565	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.1;
	3566	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.6;
	3566	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.3;
	3566	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.4;
	3566	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	269.7;
	3566	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.6;
	3566	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.3;
	3567	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.4;
	3567	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.8;
	3567	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	263.7;
	3567	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	275.6;
	3567	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.6;
	3567	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.6;
	3568	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466;
	3568	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.3;
	3568	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	271;
	3568	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	283.1;
	3568	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.4;
	3568	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.8;
	3569	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.7;
	3569	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.5;
	3569	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	274.3;
	3569	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	286.4;
	3569	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82;
	3569	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.2;
	3570	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.2;
	3570	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.7;
	3570	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	274.8;
	3570	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	286.5;
	3570	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.7;
	3570	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385;
	3571	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463;
	3571	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.2;
	3571	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	271.7;
	3571	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	282.3;
	3571	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.5;
	3571	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.3;
	3572	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.7;
	3572	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.8;
	3572	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	270.4;
	3572	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	280.5;
	3572	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.1;
	3572	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.5;
	3573	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461;
	3573	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.4;
	3573	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	267.7;
	3573	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	277.6;
	3573	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.1;
	3573	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.9;
	3574	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.1;
	3574	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.5;
	3574	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.7;
	3574	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267.2;
	3574	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.5;
	3574	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381;
	3575	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.1;
	3575	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.8;
	3575	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.7;
	3575	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246;
	3575	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.1;
	3575	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.8;
	3576	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423;
	3576	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.3;
	3576	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.9;
	3576	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234;
	3576	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	3576	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.9;
	3577	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	400.6;
	3577	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.6;
	3577	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.7;
	3577	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.7;
	3577	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	3577	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.7;
	3578	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	384.2;
	3578	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.9;
	3578	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.1;
	3578	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207;
	3578	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.1;
	3578	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.7;
	3579	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	371.2;
	3579	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.5;
	3579	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.9;
	3579	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.1;
	3579	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.1;
	3579	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.7;
	3580	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	369.3;
	3580	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.5;
	3580	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.2;
	3580	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.1;
	3580	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.5;
	3580	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.4;
	3581	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	382.4;
	3581	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.3;
	3581	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.6;
	3581	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.6;
	3581	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.9;
	3581	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.8;
	3582	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	396.4;
	3582	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.9;
	3582	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204;
	3582	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.5;
	3582	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.1;
	3582	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.9;
	3583	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.2;
	3583	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.6;
	3583	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.2;
	3583	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.6;
	3583	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.7;
	3583	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.5;
	3584	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.8;
	3584	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.8;
	3584	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.8;
	3584	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.3;
	3584	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.2;
	3584	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.6;
	3585	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.3;
	3585	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.6;
	3585	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.6;
	3585	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267.7;
	3585	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.4;
	3585	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390;
	3586	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.4;
	3586	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.1;
	3586	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	268.9;
	3586	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	279.5;
	3586	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	3586	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.1;
	3587	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549.5;
	3587	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.1;
	3587	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	279.9;
	3587	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	291.3;
	3587	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	3587	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.9;
	3588	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552.3;
	3588	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.8;
	3588	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	289.3;
	3588	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	301.3;
	3588	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	3588	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420.1;
	3589	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549.9;
	3589	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.7;
	3589	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	297.7;
	3589	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	310.4;
	3589	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.7;
	3589	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.7;
	3590	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.6;
	3590	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.7;
	3590	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	304.8;
	3590	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	318.2;
	3590	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.4;
	3590	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.8;
	3591	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538;
	3591	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.1;
	3591	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	306.4;
	3591	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	320.7;
	3591	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.8;
	3591	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.4;
	3592	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.3;
	3592	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.2;
	3592	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	309.3;
	3592	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	324.1;
	3592	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.1;
	3592	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.9;
	3593	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.5;
	3593	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.5;
	3593	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	314.6;
	3593	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	329;
	3593	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.8;
	3593	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.6;
	3594	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	546.2;
	3594	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.2;
	3594	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	308;
	3594	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	320.2;
	3594	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.3;
	3594	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	427.1;
	3595	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537.2;
	3595	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.7;
	3595	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	296.6;
	3595	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	307.2;
	3595	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73;
	3595	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420.1;
	3596	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	534.1;
	3596	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.7;
	3596	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	294.8;
	3596	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	304.5;
	3596	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	3596	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.8;
	3597	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	534.6;
	3597	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.2;
	3597	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	283.8;
	3597	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	293.4;
	3597	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.1;
	3597	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414;
	3598	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526.1;
	3598	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.2;
	3598	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	266;
	3598	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	274.3;
	3598	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.7;
	3598	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.9;
	3599	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.5;
	3599	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.9;
	3599	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247;
	3599	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.5;
	3599	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	3599	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.3;
	3600	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.6;
	3600	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.2;
	3600	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.6;
	3600	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.8;
	3600	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	3600	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.6;
	3601	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.2;
	3601	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.9;
	3601	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.8;
	3601	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.1;
	3601	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	3601	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.9;
	3602	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.3;
	3602	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.3;
	3602	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.6;
	3602	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.8;
	3602	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.1;
	3602	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.4;
	3603	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.7;
	3603	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.2;
	3603	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.2;
	3603	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.7;
	3603	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.5;
	3603	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.6;
	3604	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.1;
	3604	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.3;
	3604	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.3;
	3604	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.3;
	3604	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.2;
	3604	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.4;
	3605	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	400;
	3605	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.7;
	3605	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.5;
	3605	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.3;
	3605	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.1;
	3605	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.7;
	3606	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417;
	3606	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.1;
	3606	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213;
	3606	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.6;
	3606	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.9;
	3606	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.2;
	3607	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.5;
	3607	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.9;
	3607	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.3;
	3607	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.2;
	3607	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.5;
	3607	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.6;
	3608	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.3;
	3608	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.3;
	3608	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.4;
	3608	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.7;
	3608	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.8;
	3608	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384;
	3609	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532;
	3609	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.3;
	3609	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.1;
	3609	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267.2;
	3609	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	3609	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.3;
	3610	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.6;
	3610	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.8;
	3610	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.8;
	3610	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	274.6;
	3610	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	3610	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.6;
	3611	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550;
	3611	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.4;
	3611	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	267.2;
	3611	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	280.8;
	3611	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	3611	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413;
	3612	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548.8;
	3612	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.9;
	3612	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	269.4;
	3612	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	286.8;
	3612	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	3612	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.1;
	3613	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.4;
	3613	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.7;
	3613	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	274.8;
	3613	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	294.4;
	3613	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.8;
	3613	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420.3;
	3614	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	553.6;
	3614	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.7;
	3614	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	278.7;
	3614	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	300.7;
	3614	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.9;
	3614	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.4;
	3615	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	561.7;
	3615	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.9;
	3615	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	286.1;
	3615	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	307.9;
	3615	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.9;
	3615	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.6;
	3616	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572;
	3616	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.6;
	3616	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	293.2;
	3616	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	311.8;
	3616	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	3616	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	429.9;
	3617	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	575.3;
	3617	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.6;
	3617	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	296;
	3617	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	312.7;
	3617	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68;
	3617	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	432.8;
	3618	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	603.2;
	3618	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.4;
	3618	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	292.5;
	3618	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	306.6;
	3618	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.9;
	3618	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	443.1;
	3619	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	587.3;
	3619	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.6;
	3619	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	285.9;
	3619	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	301.3;
	3619	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	3619	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	434.5;
	3620	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	554.9;
	3620	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.5;
	3620	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	278.7;
	3620	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	293.2;
	3620	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	3620	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.8;
	3621	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539;
	3621	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.6;
	3621	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	274.6;
	3621	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	288.5;
	3621	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.8;
	3621	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.5;
	3622	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.8;
	3622	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.3;
	3622	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.8;
	3622	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	274.5;
	3622	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.1;
	3622	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.1;
	3623	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498;
	3623	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.4;
	3623	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.7;
	3623	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252;
	3623	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	3623	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.4;
	3624	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.1;
	3624	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.6;
	3624	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.5;
	3624	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.9;
	3624	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	3624	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.2;
	3625	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438;
	3625	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.5;
	3625	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.4;
	3625	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.6;
	3625	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.7;
	3625	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.8;
	3626	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.3;
	3626	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.4;
	3626	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.8;
	3626	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.4;
	3626	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.7;
	3626	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.7;
	3627	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.5;
	3627	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.4;
	3627	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.9;
	3627	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.9;
	3627	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.5;
	3627	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.8;
	3628	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.1;
	3628	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.7;
	3628	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.6;
	3628	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.5;
	3628	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.7;
	3628	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.3;
	3629	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.8;
	3629	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.2;
	3629	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199;
	3629	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.9;
	3629	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.8;
	3629	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333;
	3630	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.5;
	3630	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.7;
	3630	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208;
	3630	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.4;
	3630	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.5;
	3630	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.1;
	3631	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.9;
	3631	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.7;
	3631	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.4;
	3631	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.3;
	3631	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.1;
	3631	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.4;
	3632	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.7;
	3632	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.5;
	3632	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242;
	3632	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.1;
	3632	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.9;
	3632	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.3;
	3633	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.5;
	3633	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.4;
	3633	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255;
	3633	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	263.3;
	3633	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	3633	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.1;
	3634	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522;
	3634	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.8;
	3634	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	265.9;
	3634	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	273.3;
	3634	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68;
	3634	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.1;
	3635	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	534.2;
	3635	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160;
	3635	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	274.2;
	3635	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	281.6;
	3635	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.8;
	3635	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.7;
	3636	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.3;
	3636	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.1;
	3636	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	284.8;
	3636	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	289.8;
	3636	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.6;
	3636	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.4;
	3637	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	540;
	3637	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.2;
	3637	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	295.1;
	3637	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	299.5;
	3637	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.8;
	3637	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.1;
	3638	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.3;
	3638	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.1;
	3638	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	304.7;
	3638	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	306.7;
	3638	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79;
	3638	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.7;
	3639	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.6;
	3639	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.7;
	3639	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	314;
	3639	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	313.8;
	3639	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.6;
	3639	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	429.8;
	3640	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566.4;
	3640	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.9;
	3640	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	321.2;
	3640	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	320.9;
	3640	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.4;
	3640	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	438.2;
	3641	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	580.5;
	3641	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.4;
	3641	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	319.9;
	3641	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	318.4;
	3641	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.3;
	3641	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	445;
	3642	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	568.8;
	3642	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171;
	3642	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	311.4;
	3642	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	309.7;
	3642	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.9;
	3642	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	438.1;
	3643	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.1;
	3643	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.6;
	3643	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	301;
	3643	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	298;
	3643	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.2;
	3643	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	430.7;
	3644	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.7;
	3644	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.4;
	3644	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	292.6;
	3644	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	290.4;
	3644	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.3;
	3644	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.5;
	3645	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.3;
	3645	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.5;
	3645	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	283.7;
	3645	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	282;
	3645	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.1;
	3645	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.8;
	3646	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512;
	3646	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.6;
	3646	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	267.6;
	3646	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265;
	3646	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.7;
	3646	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.8;
	3647	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.8;
	3647	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.6;
	3647	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.1;
	3647	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.6;
	3647	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.2;
	3647	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.7;
	3648	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.9;
	3648	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.8;
	3648	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234;
	3648	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.9;
	3648	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.5;
	3648	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.1;
	3649	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.1;
	3649	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.1;
	3649	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.4;
	3649	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.3;
	3649	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	3649	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.4;
	3650	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	401.4;
	3650	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116;
	3650	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.9;
	3650	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.5;
	3650	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	3650	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331;
	3651	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.5;
	3651	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.3;
	3651	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.5;
	3651	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.2;
	3651	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	3651	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329;
	3652	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394.1;
	3652	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.2;
	3652	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.1;
	3652	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.4;
	3652	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.8;
	3652	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.2;
	3653	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403.4;
	3653	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.4;
	3653	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.9;
	3653	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.2;
	3653	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56;
	3653	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331;
	3654	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.7;
	3654	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.6;
	3654	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.3;
	3654	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.2;
	3654	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.9;
	3654	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.7;
	3655	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.5;
	3655	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.8;
	3655	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.3;
	3655	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228;
	3655	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.6;
	3655	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362;
	3656	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.1;
	3656	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.1;
	3656	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.1;
	3656	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.6;
	3656	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.4;
	3656	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.3;
	3657	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.3;
	3657	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.7;
	3657	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.3;
	3657	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.1;
	3657	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	3657	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.9;
	3658	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.6;
	3658	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.9;
	3658	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	265.2;
	3658	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	266.9;
	3658	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71;
	3658	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.5;
	3659	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.3;
	3659	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.8;
	3659	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	275.9;
	3659	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	276.7;
	3659	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.4;
	3659	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.4;
	3660	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550;
	3660	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164;
	3660	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	288.6;
	3660	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	287.9;
	3660	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.4;
	3660	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.6;
	3661	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.9;
	3661	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.4;
	3661	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	298;
	3661	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	296;
	3661	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.6;
	3661	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	426.7;
	3662	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586.7;
	3662	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.2;
	3662	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	304.5;
	3662	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	301.6;
	3662	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.1;
	3662	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	441.6;
	3663	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	590.2;
	3663	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169;
	3663	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	308;
	3663	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	304.7;
	3663	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.8;
	3663	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	444.2;
	3664	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	580.6;
	3664	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.6;
	3664	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	309.8;
	3664	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	307.1;
	3664	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.6;
	3664	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	441.1;
	3665	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.6;
	3665	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.5;
	3665	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	309.1;
	3665	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	305.4;
	3665	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.5;
	3665	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	446.4;
	3666	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.5;
	3666	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.7;
	3666	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	298.1;
	3666	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	295.3;
	3666	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.7;
	3666	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	434.3;
	3667	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.7;
	3667	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.9;
	3667	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	288.3;
	3667	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	285.3;
	3667	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.5;
	3667	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.5;
	3668	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.7;
	3668	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.8;
	3668	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	279.8;
	3668	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	277.9;
	3668	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.5;
	3668	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.3;
	3669	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.3;
	3669	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.9;
	3669	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	271.9;
	3669	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	270.6;
	3669	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.2;
	3669	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.8;
	3670	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.4;
	3670	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.8;
	3670	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.7;
	3670	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.3;
	3670	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.1;
	3670	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.9;
	3671	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.1;
	3671	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.8;
	3671	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.6;
	3671	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.6;
	3671	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	3671	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379;
	3672	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.7;
	3672	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.4;
	3672	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.7;
	3672	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236;
	3672	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	3672	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.4;
	3673	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.6;
	3673	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125;
	3673	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.3;
	3673	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.5;
	3673	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	3673	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.6;
	3674	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.7;
	3674	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.5;
	3674	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.3;
	3674	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.6;
	3674	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.3;
	3674	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.2;
	3675	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	389.1;
	3675	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.8;
	3675	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.6;
	3675	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.9;
	3675	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.8;
	3675	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325;
	3676	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	374.4;
	3676	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.1;
	3676	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.1;
	3676	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.7;
	3676	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.3;
	3676	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.6;
	3677	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	377.6;
	3677	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.1;
	3677	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.1;
	3677	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.7;
	3677	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.2;
	3677	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.1;
	3678	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	396.8;
	3678	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.6;
	3678	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.7;
	3678	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.9;
	3678	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.5;
	3678	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.2;
	3679	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.8;
	3679	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.5;
	3679	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.1;
	3679	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.7;
	3679	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.7;
	3679	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.2;
	3680	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.2;
	3680	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.9;
	3680	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.6;
	3680	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.5;
	3680	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	3680	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.7;
	3681	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.2;
	3681	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141;
	3681	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.5;
	3681	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230;
	3681	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	3681	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.6;
	3682	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.1;
	3682	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.9;
	3682	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.4;
	3682	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.9;
	3682	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.3;
	3682	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.4;
	3683	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.4;
	3683	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156;
	3683	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.6;
	3683	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255;
	3683	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	3683	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.2;
	3684	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.7;
	3684	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.4;
	3684	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	267;
	3684	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.3;
	3684	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.1;
	3684	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.7;
	3685	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.7;
	3685	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.7;
	3685	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	276.8;
	3685	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.6;
	3685	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.3;
	3685	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.7;
	3686	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.7;
	3686	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164;
	3686	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	283.9;
	3686	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	279;
	3686	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.7;
	3686	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.5;
	3687	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.8;
	3687	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.5;
	3687	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	290.6;
	3687	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	284.6;
	3687	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.5;
	3687	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.5;
	3688	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	534.8;
	3688	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.9;
	3688	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	291.9;
	3688	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	285.5;
	3688	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.7;
	3688	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.3;
	3689	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.7;
	3689	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.3;
	3689	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	287.7;
	3689	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	281.1;
	3689	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.9;
	3689	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.3;
	3690	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.8;
	3690	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.7;
	3690	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	279.8;
	3690	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	274.3;
	3690	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.1;
	3690	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.4;
	3691	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.3;
	3691	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.7;
	3691	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	271.9;
	3691	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267.9;
	3691	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.3;
	3691	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.6;
	3692	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.1;
	3692	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.7;
	3692	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	267.2;
	3692	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265;
	3692	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74;
	3692	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.1;
	3693	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.6;
	3693	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.3;
	3693	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.3;
	3693	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.7;
	3693	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	3693	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.8;
	3694	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.1;
	3694	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.5;
	3694	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.2;
	3694	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.9;
	3694	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.3;
	3694	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.7;
	3695	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.2;
	3695	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.7;
	3695	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.9;
	3695	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240;
	3695	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	3695	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.1;
	3696	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.9;
	3696	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.6;
	3696	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.8;
	3696	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.4;
	3696	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	3696	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.1;
	3697	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	396.8;
	3697	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.7;
	3697	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.5;
	3697	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.8;
	3697	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	3697	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335;
	3698	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	365.8;
	3698	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118;
	3698	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.8;
	3698	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.8;
	3698	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.6;
	3698	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.7;
	3699	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	350.3;
	3699	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.5;
	3699	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.1;
	3699	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.7;
	3699	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.3;
	3699	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310.6;
	3700	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	350.9;
	3700	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.8;
	3700	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.8;
	3700	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.6;
	3700	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.7;
	3700	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310;
	3701	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	352.9;
	3701	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.6;
	3701	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.2;
	3701	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.1;
	3701	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.1;
	3701	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310.5;
	3702	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	357.8;
	3702	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.5;
	3702	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.1;
	3702	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.4;
	3702	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.2;
	3702	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.6;
	3703	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	364.6;
	3703	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.5;
	3703	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203;
	3703	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.2;
	3703	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.6;
	3703	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.8;
	3704	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	381;
	3704	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128;
	3704	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210;
	3704	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.7;
	3704	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	3704	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326;
	3705	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.4;
	3705	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.1;
	3705	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.7;
	3705	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.2;
	3705	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	3705	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.3;
	3706	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.7;
	3706	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.2;
	3706	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.7;
	3706	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.7;
	3706	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	3706	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.8;
	3707	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.9;
	3707	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.9;
	3707	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.5;
	3707	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.4;
	3707	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	3707	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.4;
	3708	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.4;
	3708	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.9;
	3708	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.3;
	3708	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.1;
	3708	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.5;
	3708	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.1;
	3709	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.8;
	3709	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.1;
	3709	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.5;
	3709	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.4;
	3709	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	3709	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381;
	3710	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.1;
	3710	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.1;
	3710	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	268.2;
	3710	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264;
	3710	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.6;
	3710	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.4;
	3711	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.2;
	3711	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.4;
	3711	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	272.9;
	3711	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267.8;
	3711	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.1;
	3711	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.8;
	3712	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488;
	3712	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.5;
	3712	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	278.4;
	3712	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.6;
	3712	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	3712	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.2;
	3713	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.5;
	3713	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.8;
	3713	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	271.2;
	3713	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	266.4;
	3713	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.3;
	3713	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.8;
	3714	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.4;
	3714	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.6;
	3714	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	265;
	3714	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.3;
	3714	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.1;
	3714	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.6;
	3715	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.4;
	3715	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155;
	3715	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	264.9;
	3715	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.8;
	3715	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.4;
	3715	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.1;
	3716	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.1;
	3716	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.9;
	3716	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.7;
	3716	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.3;
	3716	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.6;
	3716	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.7;
	3717	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.6;
	3717	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.8;
	3717	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258;
	3717	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.3;
	3717	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.2;
	3717	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.5;
	3718	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.1;
	3718	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.2;
	3718	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.9;
	3718	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.4;
	3718	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	3718	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.6;
	3719	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.8;
	3719	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.2;
	3719	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230;
	3719	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.8;
	3719	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	3719	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.7;
	3720	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417.5;
	3720	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.2;
	3720	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.8;
	3720	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.4;
	3720	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	3720	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.1;
	3721	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394.2;
	3721	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.7;
	3721	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.2;
	3721	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.8;
	3721	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	3721	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.4;
	3722	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	365.7;
	3722	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.9;
	3722	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200;
	3722	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.9;
	3722	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.3;
	3722	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.4;
	3723	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	357;
	3723	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.3;
	3723	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.6;
	3723	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.4;
	3723	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.1;
	3723	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310.5;
	3724	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	353.4;
	3724	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.8;
	3724	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.6;
	3724	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.8;
	3724	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.4;
	3724	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.5;
	3725	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	356.6;
	3725	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.8;
	3725	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.1;
	3725	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.5;
	3725	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.1;
	3725	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310.3;
	3726	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	370.4;
	3726	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121;
	3726	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.2;
	3726	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206;
	3726	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.7;
	3726	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318;
	3727	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403.9;
	3727	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.2;
	3727	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.1;
	3727	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223;
	3727	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.1;
	3727	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.3;
	3728	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.1;
	3728	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146;
	3728	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.1;
	3728	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.5;
	3728	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.1;
	3728	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.2;
	3729	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.7;
	3729	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.3;
	3729	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.1;
	3729	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.9;
	3729	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	3729	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370;
	3730	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.3;
	3730	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.6;
	3730	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.8;
	3730	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.8;
	3730	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	3730	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.9;
	3731	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488;
	3731	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.2;
	3731	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.5;
	3731	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.9;
	3731	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	3731	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.1;
	3732	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.2;
	3732	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.3;
	3732	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.9;
	3732	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	263.4;
	3732	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	3732	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.2;
	3733	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.9;
	3733	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.7;
	3733	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.5;
	3733	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	266.7;
	3733	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.4;
	3733	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.6;
	3734	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.5;
	3734	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.5;
	3734	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	267.4;
	3734	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	273.8;
	3734	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.1;
	3734	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.5;
	3735	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.4;
	3735	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.3;
	3735	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	272.5;
	3735	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	279.2;
	3735	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76;
	3735	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.3;
	3736	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.3;
	3736	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.9;
	3736	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	271.4;
	3736	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	279.2;
	3736	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.7;
	3736	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.9;
	3737	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.3;
	3737	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.7;
	3737	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	271.2;
	3737	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	280.3;
	3737	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.3;
	3737	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.5;
	3738	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.1;
	3738	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.1;
	3738	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	268.6;
	3738	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	277.4;
	3738	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.2;
	3738	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395;
	3739	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500;
	3739	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.4;
	3739	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.4;
	3739	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	270;
	3739	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.2;
	3739	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.6;
	3740	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.8;
	3740	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.2;
	3740	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.4;
	3740	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265.7;
	3740	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77;
	3740	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.9;
	3741	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.6;
	3741	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.2;
	3741	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	259.1;
	3741	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265;
	3741	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.1;
	3741	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.8;
	3742	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.7;
	3742	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.5;
	3742	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.7;
	3742	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.2;
	3742	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.6;
	3742	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.3;
	3743	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.4;
	3743	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.3;
	3743	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.1;
	3743	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.8;
	3743	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.7;
	3743	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.9;
	3744	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.6;
	3744	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128;
	3744	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.9;
	3744	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.4;
	3744	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.3;
	3744	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.7;
	3745	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399;
	3745	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117;
	3745	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.1;
	3745	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.7;
	3745	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	3745	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.7;
	3746	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	372.8;
	3746	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.1;
	3746	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.2;
	3746	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.9;
	3746	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	3746	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.3;
	3747	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	362.9;
	3747	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.3;
	3747	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.2;
	3747	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.7;
	3747	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	3747	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311;
	3748	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	360.8;
	3748	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.4;
	3748	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.6;
	3748	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.1;
	3748	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.1;
	3748	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309.9;
	3749	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	370;
	3749	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.5;
	3749	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.8;
	3749	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.6;
	3749	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.7;
	3749	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.5;
	3750	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	389.6;
	3750	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.8;
	3750	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.8;
	3750	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.1;
	3750	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57;
	3750	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.7;
	3751	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.2;
	3751	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.8;
	3751	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.9;
	3751	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.6;
	3751	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57;
	3751	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.5;
	3752	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.8;
	3752	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145;
	3752	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.1;
	3752	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.1;
	3752	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.8;
	3752	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.3;
	3753	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.9;
	3753	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.6;
	3753	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.6;
	3753	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.8;
	3753	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	3753	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.3;
	3754	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.2;
	3754	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.5;
	3754	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.9;
	3754	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.9;
	3754	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	3754	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.6;
	3755	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.6;
	3755	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.1;
	3755	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.3;
	3755	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	270.3;
	3755	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	3755	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391;
	3756	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.8;
	3756	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.1;
	3756	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	266.6;
	3756	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	277.9;
	3756	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.2;
	3756	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.2;
	3757	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518;
	3757	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.5;
	3757	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	273.1;
	3757	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	285.2;
	3757	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.9;
	3757	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.6;
	3758	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.8;
	3758	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.1;
	3758	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	278.3;
	3758	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	291.2;
	3758	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.3;
	3758	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.2;
	3759	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.7;
	3759	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.4;
	3759	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	280;
	3759	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	294.3;
	3759	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.2;
	3759	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.4;
	3760	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525.4;
	3760	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160;
	3760	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	282.9;
	3760	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	298.3;
	3760	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.9;
	3760	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409;
	3761	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.8;
	3761	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.7;
	3761	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	283.3;
	3761	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	299.1;
	3761	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.4;
	3761	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.1;
	3762	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525.3;
	3762	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.9;
	3762	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	276.2;
	3762	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	291.3;
	3762	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.8;
	3762	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407;
	3763	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.8;
	3763	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.2;
	3763	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	269.1;
	3763	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	282.9;
	3763	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80;
	3763	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.8;
	3764	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.5;
	3764	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160;
	3764	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	263.7;
	3764	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	277.5;
	3764	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.6;
	3764	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.1;
	3765	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.1;
	3765	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.3;
	3765	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	259;
	3765	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.1;
	3765	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.1;
	3765	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.6;
	3766	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.8;
	3766	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.7;
	3766	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245;
	3766	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257;
	3766	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.5;
	3766	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.2;
	3767	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.8;
	3767	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.1;
	3767	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227;
	3767	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.2;
	3767	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	3767	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.1;
	3768	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	426.2;
	3768	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.4;
	3768	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.2;
	3768	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.5;
	3768	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	3768	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.4;
	3769	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	396.7;
	3769	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119;
	3769	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.6;
	3769	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.2;
	3769	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	3769	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.2;
	3770	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379.5;
	3770	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.6;
	3770	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193;
	3770	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.9;
	3770	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	3770	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.4;
	3771	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	372.6;
	3771	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111;
	3771	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.5;
	3771	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.3;
	3771	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	3771	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314;
	3772	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	369.5;
	3772	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.2;
	3772	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190;
	3772	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.3;
	3772	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.4;
	3772	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.4;
	3773	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	378.4;
	3773	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.2;
	3773	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.2;
	3773	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.5;
	3773	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.7;
	3773	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317;
	3774	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404;
	3774	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.7;
	3774	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.3;
	3774	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.2;
	3774	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.5;
	3774	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.3;
	3775	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.2;
	3775	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.2;
	3775	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.4;
	3775	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.2;
	3775	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.7;
	3775	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.2;
	3776	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.4;
	3776	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.5;
	3776	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.5;
	3776	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.5;
	3776	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	3776	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.4;
	3777	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474;
	3777	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.8;
	3777	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.9;
	3777	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.2;
	3777	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	3777	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.2;
	3778	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.3;
	3778	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151;
	3778	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.4;
	3778	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.6;
	3778	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72;
	3778	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.2;
	3779	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.1;
	3779	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.2;
	3779	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.6;
	3779	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.7;
	3779	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.3;
	3779	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.9;
	3780	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.9;
	3780	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.8;
	3780	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.7;
	3780	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	280.5;
	3780	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.4;
	3780	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.4;
	3781	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.8;
	3781	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.6;
	3781	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	266;
	3781	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	287.6;
	3781	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.4;
	3781	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.5;
	3782	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518.4;
	3782	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.3;
	3782	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	270.9;
	3782	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	293.8;
	3782	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.8;
	3782	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.5;
	3783	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.3;
	3783	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.1;
	3783	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	276.7;
	3783	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	301.3;
	3783	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86;
	3783	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.6;
	3784	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.2;
	3784	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.8;
	3784	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	278.5;
	3784	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	304.4;
	3784	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.2;
	3784	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.7;
	3785	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.5;
	3785	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163;
	3785	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	276.3;
	3785	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	302.4;
	3785	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.7;
	3785	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.8;
	3786	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.3;
	3786	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.4;
	3786	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	272.9;
	3786	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	296.9;
	3786	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.8;
	3786	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.9;
	3787	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.1;
	3787	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.1;
	3787	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	263.8;
	3787	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	286.6;
	3787	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.9;
	3787	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.4;
	3788	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.8;
	3788	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.2;
	3788	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260;
	3788	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	281.2;
	3788	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.1;
	3788	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.2;
	3789	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.3;
	3789	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.6;
	3789	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.8;
	3789	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	275.2;
	3789	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.5;
	3789	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387;
	3790	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.9;
	3790	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152;
	3790	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.4;
	3790	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260;
	3790	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.3;
	3790	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.7;
	3791	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463;
	3791	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.3;
	3791	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.2;
	3791	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.8;
	3791	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74;
	3791	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.8;
	3792	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.7;
	3792	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.6;
	3792	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.7;
	3792	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233;
	3792	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.5;
	3792	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349;
	3793	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414;
	3793	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.4;
	3793	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199;
	3793	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213;
	3793	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	3793	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.2;
	3794	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.2;
	3794	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.1;
	3794	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.4;
	3794	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.9;
	3794	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	3794	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319;
	3795	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379.8;
	3795	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.5;
	3795	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.1;
	3795	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.5;
	3795	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	3795	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.8;
	3796	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	376.7;
	3796	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.7;
	3796	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.3;
	3796	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.3;
	3796	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	3796	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.4;
	3797	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394.4;
	3797	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.4;
	3797	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.4;
	3797	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.2;
	3797	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	3797	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.8;
	3798	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.7;
	3798	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125;
	3798	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.2;
	3798	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.8;
	3798	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.3;
	3798	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.3;
	3799	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.3;
	3799	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.5;
	3799	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.8;
	3799	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.6;
	3799	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61;
	3799	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.8;
	3800	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.2;
	3800	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144;
	3800	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.9;
	3800	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246;
	3800	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	3800	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.4;
	3801	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481;
	3801	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.5;
	3801	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.5;
	3801	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.9;
	3801	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.3;
	3801	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.2;
	3802	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.6;
	3802	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.9;
	3802	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.4;
	3802	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267.8;
	3802	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.4;
	3802	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.1;
	3803	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.9;
	3803	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.9;
	3803	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.3;
	3803	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	275.6;
	3803	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.7;
	3803	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.9;
	3804	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.6;
	3804	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.3;
	3804	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.9;
	3804	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	281.6;
	3804	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.1;
	3804	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.3;
	3805	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.5;
	3805	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.3;
	3805	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.1;
	3805	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	286.6;
	3805	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.3;
	3805	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.5;
	3806	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.2;
	3806	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.8;
	3806	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.8;
	3806	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	290.6;
	3806	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.3;
	3806	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.8;
	3807	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.2;
	3807	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.2;
	3807	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	263.7;
	3807	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	295.4;
	3807	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.4;
	3807	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.5;
	3808	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.3;
	3808	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.7;
	3808	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	266.6;
	3808	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	297.7;
	3808	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.5;
	3808	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.9;
	3809	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520;
	3809	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.5;
	3809	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	268.5;
	3809	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	294.5;
	3809	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.1;
	3809	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.4;
	3810	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523;
	3810	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.6;
	3810	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.9;
	3810	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	285;
	3810	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.9;
	3810	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.2;
	3811	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.3;
	3811	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.2;
	3811	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.1;
	3811	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.3;
	3811	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.2;
	3811	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.4;
	3812	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.8;
	3812	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.7;
	3812	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.9;
	3812	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267.3;
	3812	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.8;
	3812	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.5;
	3813	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.5;
	3813	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.7;
	3813	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.2;
	3813	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.8;
	3813	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.8;
	3813	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382;
	3814	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.3;
	3814	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.9;
	3814	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.1;
	3814	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.5;
	3814	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.3;
	3814	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.6;
	3815	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.9;
	3815	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.1;
	3815	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220;
	3815	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.5;
	3815	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.8;
	3815	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.1;
	3816	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.5;
	3816	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.2;
	3816	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.4;
	3816	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.4;
	3816	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	3816	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353;
	3817	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.2;
	3817	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.9;
	3817	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.1;
	3817	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.8;
	3817	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	3817	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.6;
	3818	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	400.6;
	3818	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112;
	3818	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.9;
	3818	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.3;
	3818	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.4;
	3818	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.6;
	3819	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	387.9;
	3819	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.6;
	3819	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.6;
	3819	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.5;
	3819	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.4;
	3819	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.6;
	3820	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.9;
	3820	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.4;
	3820	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.2;
	3820	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.2;
	3820	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.8;
	3820	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.1;
	3821	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.4;
	3821	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.7;
	3821	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.7;
	3821	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.9;
	3821	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.2;
	3821	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.1;
	3822	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.8;
	3822	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.5;
	3822	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.1;
	3822	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.1;
	3822	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.7;
	3822	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335;
	3823	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.9;
	3823	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.8;
	3823	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.8;
	3823	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226;
	3823	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.6;
	3823	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.8;
	3824	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.2;
	3824	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.5;
	3824	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.4;
	3824	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.3;
	3824	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.7;
	3824	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.8;
	3825	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.7;
	3825	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.1;
	3825	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.4;
	3825	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.4;
	3825	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	3825	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.3;
	3826	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.7;
	3826	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.8;
	3826	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.5;
	3826	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.7;
	3826	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	3826	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.8;
	3827	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.2;
	3827	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.8;
	3827	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.2;
	3827	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267.6;
	3827	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	3827	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.5;
	3828	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498;
	3828	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.7;
	3828	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.4;
	3828	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	269.7;
	3828	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.6;
	3828	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.3;
	3829	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.8;
	3829	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.5;
	3829	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.6;
	3829	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	273.9;
	3829	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.9;
	3829	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395;
	3830	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.4;
	3830	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.9;
	3830	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.8;
	3830	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	277.4;
	3830	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.5;
	3830	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.7;
	3831	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.8;
	3831	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.1;
	3831	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.1;
	3831	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	278.6;
	3831	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.7;
	3831	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.6;
	3832	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.1;
	3832	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.6;
	3832	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.3;
	3832	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	278.9;
	3832	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.3;
	3832	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.8;
	3833	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.9;
	3833	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.3;
	3833	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.5;
	3833	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	276.3;
	3833	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	3833	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.2;
	3834	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.1;
	3834	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154;
	3834	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.5;
	3834	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	266.7;
	3834	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.3;
	3834	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.9;
	3835	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.1;
	3835	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.7;
	3835	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.6;
	3835	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.7;
	3835	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	3835	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.4;
	3836	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.7;
	3836	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.3;
	3836	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.1;
	3836	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.9;
	3836	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	3836	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.6;
	3837	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490;
	3837	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.9;
	3837	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.1;
	3837	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.8;
	3837	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	3837	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.9;
	3838	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.7;
	3838	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.4;
	3838	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.6;
	3838	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.4;
	3838	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	3838	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.3;
	3839	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.1;
	3839	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.5;
	3839	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.7;
	3839	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231;
	3839	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	3839	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.5;
	3840	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.7;
	3840	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.4;
	3840	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.1;
	3840	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.3;
	3840	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	3840	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.6;
	3841	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.4;
	3841	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.9;
	3841	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200;
	3841	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209;
	3841	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.4;
	3841	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.5;
	3842	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	377.1;
	3842	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.7;
	3842	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.9;
	3842	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.1;
	3842	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.1;
	3842	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.8;
	3843	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	358.9;
	3843	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.6;
	3843	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.4;
	3843	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197;
	3843	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52;
	3843	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.5;
	3844	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	360.4;
	3844	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.7;
	3844	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.4;
	3844	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.6;
	3844	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.9;
	3844	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309.4;
	3845	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	357.9;
	3845	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.9;
	3845	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.2;
	3845	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.9;
	3845	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.5;
	3845	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309.1;
	3846	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	373.4;
	3846	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.9;
	3846	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.7;
	3846	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.2;
	3846	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.1;
	3846	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.9;
	3847	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	372.4;
	3847	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.1;
	3847	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.9;
	3847	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.1;
	3847	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54;
	3847	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.8;
	3848	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	386.6;
	3848	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.8;
	3848	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.9;
	3848	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.9;
	3848	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57;
	3848	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.9;
	3849	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.7;
	3849	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.3;
	3849	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.7;
	3849	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.5;
	3849	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	3849	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.1;
	3850	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.5;
	3850	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.9;
	3850	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.5;
	3850	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233;
	3850	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	3850	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.1;
	3851	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.2;
	3851	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.2;
	3851	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.5;
	3851	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238;
	3851	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	3851	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.3;
	3852	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.7;
	3852	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.9;
	3852	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.2;
	3852	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.2;
	3852	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	3852	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.7;
	3853	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.9;
	3853	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146;
	3853	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.9;
	3853	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.2;
	3853	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.8;
	3853	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.4;
	3854	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.3;
	3854	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.6;
	3854	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.2;
	3854	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.8;
	3854	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.4;
	3854	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.5;
	3855	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.5;
	3855	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.9;
	3855	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.2;
	3855	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.5;
	3855	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70;
	3855	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.3;
	3856	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.6;
	3856	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.3;
	3856	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.1;
	3856	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.3;
	3856	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	3856	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.9;
	3857	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.1;
	3857	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.9;
	3857	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.7;
	3857	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.5;
	3857	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72;
	3857	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.2;
	3858	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485;
	3858	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.6;
	3858	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.9;
	3858	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.6;
	3858	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.8;
	3858	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382;
	3859	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486;
	3859	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.9;
	3859	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.7;
	3859	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252;
	3859	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73;
	3859	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.4;
	3860	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.9;
	3860	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.8;
	3860	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.5;
	3860	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.3;
	3860	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.1;
	3860	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.4;
	3861	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.1;
	3861	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145;
	3861	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.6;
	3861	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.2;
	3861	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.5;
	3861	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373;
	3862	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.5;
	3862	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.3;
	3862	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237;
	3862	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.6;
	3862	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	3862	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.4;
	3863	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.9;
	3863	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.1;
	3863	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.5;
	3863	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.6;
	3863	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	3863	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.5;
	3864	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.8;
	3864	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.1;
	3864	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.5;
	3864	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.1;
	3864	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	3864	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.4;
	3865	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	385.2;
	3865	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.3;
	3865	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.7;
	3865	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.7;
	3865	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	3865	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.9;
	3866	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	350.6;
	3866	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.6;
	3866	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.8;
	3866	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.8;
	3866	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.9;
	3866	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	307.9;
	3867	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	331.8;
	3867	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106;
	3867	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.2;
	3867	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.3;
	3867	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.5;
	3867	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	298.9;
	3868	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	330.5;
	3868	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	105;
	3868	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.8;
	3868	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192;
	3868	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.1;
	3868	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	297.9;
	3869	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	330.1;
	3869	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	105.5;
	3869	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189;
	3869	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.3;
	3869	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.9;
	3869	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	297.7;
	3870	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	345.3;
	3870	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.5;
	3870	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.8;
	3870	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.7;
	3870	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.6;
	3870	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	304.4;
	3871	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	339.3;
	3871	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.7;
	3871	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.7;
	3871	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.8;
	3871	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.5;
	3871	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	303.3;
	3872	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	367.4;
	3872	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.3;
	3872	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.8;
	3872	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.4;
	3872	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.5;
	3872	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.8;
	3873	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.4;
	3873	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.5;
	3873	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208;
	3873	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.4;
	3873	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	3873	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.7;
	3874	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.2;
	3874	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.3;
	3874	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.9;
	3874	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.9;
	3874	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	3874	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.9;
	3875	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.6;
	3875	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.9;
	3875	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.7;
	3875	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.9;
	3875	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	3875	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.5;
	3876	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.9;
	3876	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.8;
	3876	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.3;
	3876	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.7;
	3876	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	3876	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.3;
	3877	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454;
	3877	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.7;
	3877	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.3;
	3877	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.9;
	3877	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.4;
	3877	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.3;
	3878	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.8;
	3878	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.8;
	3878	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.8;
	3878	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.8;
	3878	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.9;
	3878	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369;
	3879	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.9;
	3879	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.6;
	3879	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.9;
	3879	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.8;
	3879	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	3879	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.8;
	3880	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479;
	3880	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.9;
	3880	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	259.1;
	3880	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.5;
	3880	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	3880	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.1;
	3881	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.2;
	3881	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154;
	3881	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	259.3;
	3881	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.9;
	3881	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.8;
	3881	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.5;
	3882	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.5;
	3882	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.3;
	3882	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.2;
	3882	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255;
	3882	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.1;
	3882	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.9;
	3883	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.7;
	3883	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.1;
	3883	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.2;
	3883	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.1;
	3883	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.5;
	3883	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.6;
	3884	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.4;
	3884	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.4;
	3884	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.6;
	3884	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.3;
	3884	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	3884	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.4;
	3885	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.3;
	3885	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.1;
	3885	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.7;
	3885	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.9;
	3885	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.8;
	3885	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.1;
	3886	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.3;
	3886	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151;
	3886	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.4;
	3886	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.2;
	3886	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	3886	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.9;
	3887	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.8;
	3887	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.9;
	3887	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.4;
	3887	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227;
	3887	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	3887	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.2;
	3888	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.7;
	3888	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.5;
	3888	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.5;
	3888	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.2;
	3888	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	3888	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.7;
	3889	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	406.4;
	3889	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.4;
	3889	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.5;
	3889	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.4;
	3889	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.1;
	3889	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334;
	3890	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379;
	3890	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.8;
	3890	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194;
	3890	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.3;
	3890	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.4;
	3890	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.8;
	3891	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	377.5;
	3891	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.3;
	3891	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193;
	3891	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.6;
	3891	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.8;
	3891	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.2;
	3892	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	374.1;
	3892	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.5;
	3892	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.7;
	3892	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.6;
	3892	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.7;
	3892	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.3;
	3893	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379.7;
	3893	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.3;
	3893	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.8;
	3893	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.8;
	3893	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.5;
	3893	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.4;
	3894	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	395.9;
	3894	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.2;
	3894	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201;
	3894	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.3;
	3894	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.4;
	3894	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.7;
	3895	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.9;
	3895	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.3;
	3895	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.2;
	3895	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.2;
	3895	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	3895	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.6;
	3896	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.9;
	3896	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150;
	3896	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.4;
	3896	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.7;
	3896	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.7;
	3896	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.1;
	3897	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.8;
	3897	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.7;
	3897	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.2;
	3897	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.5;
	3897	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58;
	3897	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.2;
	3898	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526.3;
	3898	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.8;
	3898	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258;
	3898	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.2;
	3898	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	3898	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.4;
	3899	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.1;
	3899	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162;
	3899	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	263.2;
	3899	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	266.3;
	3899	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	3899	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.8;
	3900	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	553.5;
	3900	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.6;
	3900	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	265.4;
	3900	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	268.4;
	3900	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	3900	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.5;
	3901	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552.4;
	3901	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165;
	3901	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	268.4;
	3901	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	270.8;
	3901	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	3901	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.4;
	3902	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559.1;
	3902	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.9;
	3902	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	271.6;
	3902	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	273.8;
	3902	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.8;
	3902	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.3;
	3903	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.9;
	3903	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.5;
	3903	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	275.4;
	3903	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	277.8;
	3903	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.9;
	3903	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.6;
	3904	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559.6;
	3904	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.3;
	3904	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	279.1;
	3904	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	281.1;
	3904	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.9;
	3904	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420.8;
	3905	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549.1;
	3905	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.3;
	3905	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	278.1;
	3905	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	280.3;
	3905	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.7;
	3905	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.7;
	3906	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.2;
	3906	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.2;
	3906	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	277.7;
	3906	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	277.8;
	3906	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.2;
	3906	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.5;
	3907	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528.1;
	3907	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.9;
	3907	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	272.2;
	3907	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	271.7;
	3907	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83;
	3907	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406;
	3908	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	524.4;
	3908	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166;
	3908	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	266.8;
	3908	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267.1;
	3908	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.7;
	3908	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.7;
	3909	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.2;
	3909	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.4;
	3909	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	263.5;
	3909	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264;
	3909	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.1;
	3909	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397;
	3910	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.3;
	3910	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.2;
	3910	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.9;
	3910	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.7;
	3910	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.6;
	3910	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.5;
	3911	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.6;
	3911	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.3;
	3911	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.8;
	3911	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234;
	3911	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.3;
	3911	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.2;
	3912	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.6;
	3912	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.6;
	3912	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.8;
	3912	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.1;
	3912	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.4;
	3912	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358;
	3913	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.9;
	3913	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.9;
	3913	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.9;
	3913	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.1;
	3913	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.5;
	3913	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.6;
	3914	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.4;
	3914	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.2;
	3914	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200;
	3914	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.1;
	3914	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	3914	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.1;
	3915	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.7;
	3915	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.8;
	3915	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.7;
	3915	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.6;
	3915	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	3915	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.6;
	3916	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.8;
	3916	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.9;
	3916	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196;
	3916	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.9;
	3916	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	3916	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.8;
	3917	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.6;
	3917	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117;
	3917	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198;
	3917	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.4;
	3917	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	3917	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.4;
	3918	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.9;
	3918	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.6;
	3918	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.4;
	3918	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.7;
	3918	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.2;
	3918	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.6;
	3919	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452;
	3919	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.4;
	3919	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.7;
	3919	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.9;
	3919	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60;
	3919	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.4;
	3920	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472;
	3920	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.3;
	3920	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.8;
	3920	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246;
	3920	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	3920	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.8;
	3921	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.6;
	3921	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.1;
	3921	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.3;
	3921	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.2;
	3921	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	3921	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.4;
	3922	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.6;
	3922	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.2;
	3922	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	264;
	3922	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	266.2;
	3922	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	3922	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.7;
	3923	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.6;
	3923	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160;
	3923	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	270.3;
	3923	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	271.6;
	3923	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.8;
	3923	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.5;
	3924	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.1;
	3924	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.4;
	3924	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	276.1;
	3924	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	276.3;
	3924	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.4;
	3924	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.8;
	3925	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.8;
	3925	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.3;
	3925	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	281.2;
	3925	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	281;
	3925	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.5;
	3925	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.7;
	3926	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.7;
	3926	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.8;
	3926	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	287.6;
	3926	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	286.5;
	3926	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85;
	3926	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.1;
	3927	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548.1;
	3927	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.1;
	3927	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	290.6;
	3927	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	288.5;
	3927	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.6;
	3927	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420.6;
	3928	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547.6;
	3928	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163;
	3928	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	287.3;
	3928	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	285.7;
	3928	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.8;
	3928	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.8;
	3929	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.6;
	3929	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.9;
	3929	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	284.6;
	3929	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	282.6;
	3929	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.4;
	3929	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416;
	3930	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.7;
	3930	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.8;
	3930	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	282.9;
	3930	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	280.3;
	3930	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.7;
	3930	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.6;
	3931	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526.8;
	3931	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.7;
	3931	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	274.4;
	3931	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.9;
	3931	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.9;
	3931	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.1;
	3932	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518.8;
	3932	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.4;
	3932	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	267.5;
	3932	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267.1;
	3932	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.5;
	3932	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.2;
	3933	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.5;
	3933	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.8;
	3933	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.9;
	3933	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	263;
	3933	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.1;
	3933	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.8;
	3934	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.1;
	3934	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150;
	3934	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.9;
	3934	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.1;
	3934	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.7;
	3934	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.6;
	3935	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.6;
	3935	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.8;
	3935	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.3;
	3935	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.5;
	3935	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.6;
	3935	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.2;
	3936	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.7;
	3936	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.9;
	3936	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.1;
	3936	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224;
	3936	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.5;
	3936	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.4;
	3937	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	430.8;
	3937	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.7;
	3937	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.6;
	3937	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.4;
	3937	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	3937	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.8;
	3938	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.4;
	3938	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.7;
	3938	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.4;
	3938	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.8;
	3938	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60;
	3938	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.6;
	3939	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	380.6;
	3939	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.4;
	3939	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.9;
	3939	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.6;
	3939	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.2;
	3939	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.6;
	3940	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	378.1;
	3940	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.7;
	3940	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.9;
	3940	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.8;
	3940	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.5;
	3940	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.3;
	3941	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391;
	3941	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.9;
	3941	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.6;
	3941	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.9;
	3941	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.9;
	3941	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.5;
	3942	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.6;
	3942	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.5;
	3942	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.7;
	3942	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.5;
	3942	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.6;
	3942	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.2;
	3943	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.4;
	3943	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.7;
	3943	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.2;
	3943	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.7;
	3943	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.4;
	3943	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.5;
	3944	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468;
	3944	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144;
	3944	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.9;
	3944	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.3;
	3944	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	3944	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.4;
	3945	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.3;
	3945	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.7;
	3945	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.4;
	3945	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251;
	3945	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	3945	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.1;
	3946	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.5;
	3946	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.9;
	3946	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.6;
	3946	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257;
	3946	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	3946	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.7;
	3947	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.9;
	3947	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.5;
	3947	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.8;
	3947	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.7;
	3947	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	3947	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397;
	3948	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525.2;
	3948	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.1;
	3948	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.1;
	3948	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265.1;
	3948	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.5;
	3948	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.3;
	3949	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.8;
	3949	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.6;
	3949	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	263.8;
	3949	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	268.5;
	3949	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.9;
	3949	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.8;
	3950	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.6;
	3950	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.3;
	3950	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	265.5;
	3950	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	271.2;
	3950	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.4;
	3950	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.2;
	3951	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	534.2;
	3951	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.5;
	3951	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	266.2;
	3951	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.9;
	3951	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75;
	3951	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.5;
	3952	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	524.8;
	3952	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.5;
	3952	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	268.5;
	3952	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	275.9;
	3952	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.5;
	3952	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.1;
	3953	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.9;
	3953	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.3;
	3953	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	272.7;
	3953	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	279.6;
	3953	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.2;
	3953	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.3;
	3954	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.7;
	3954	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157;
	3954	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	266.9;
	3954	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	274.3;
	3954	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.6;
	3954	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.4;
	3955	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525.6;
	3955	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.1;
	3955	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.8;
	3955	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267.2;
	3955	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.7;
	3955	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.3;
	3956	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518.7;
	3956	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155;
	3956	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.6;
	3956	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	263;
	3956	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75;
	3956	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.5;
	3957	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.4;
	3957	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.1;
	3957	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254;
	3957	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.7;
	3957	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.3;
	3957	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.3;
	3958	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.6;
	3958	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149;
	3958	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.3;
	3958	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.3;
	3958	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	3958	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.7;
	3959	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484;
	3959	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136;
	3959	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.1;
	3959	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.3;
	3959	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.5;
	3959	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.5;
	3960	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456;
	3960	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.2;
	3960	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218;
	3960	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.8;
	3960	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	3960	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.5;
	3961	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441;
	3961	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.1;
	3961	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.1;
	3961	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.9;
	3961	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	3961	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.8;
	3962	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	406.7;
	3962	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.3;
	3962	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.9;
	3962	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.2;
	3962	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59;
	3962	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.1;
	3963	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.8;
	3963	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110;
	3963	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.7;
	3963	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.1;
	3963	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.5;
	3963	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.1;
	3964	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392.1;
	3964	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.5;
	3964	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.1;
	3964	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.7;
	3964	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.5;
	3964	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.6;
	3965	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	400.1;
	3965	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.6;
	3965	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.4;
	3965	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.7;
	3965	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.9;
	3965	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.2;
	3966	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410;
	3966	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.1;
	3966	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.2;
	3966	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.2;
	3966	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.1;
	3966	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.2;
	3967	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	430.7;
	3967	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.8;
	3967	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219;
	3967	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224;
	3967	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.5;
	3967	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.4;
	3968	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.2;
	3968	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.2;
	3968	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.7;
	3968	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.7;
	3968	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60;
	3968	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.3;
	3969	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.6;
	3969	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.7;
	3969	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.4;
	3969	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.1;
	3969	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	3969	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.8;
	3970	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.7;
	3970	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.7;
	3970	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.3;
	3970	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.7;
	3970	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	3970	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.6;
	3971	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.1;
	3971	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.7;
	3971	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.8;
	3971	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.4;
	3971	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	3971	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.9;
	3972	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.1;
	3972	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.8;
	3972	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.1;
	3972	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	268.8;
	3972	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.8;
	3972	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.5;
	3973	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.3;
	3973	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.4;
	3973	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	264.3;
	3973	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	273.7;
	3973	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.1;
	3973	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.7;
	3974	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552.7;
	3974	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.9;
	3974	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	268.9;
	3974	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	278.3;
	3974	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.7;
	3974	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.5;
	3975	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552.4;
	3975	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.1;
	3975	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	272.2;
	3975	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	281.9;
	3975	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.6;
	3975	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.2;
	3976	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548;
	3976	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.9;
	3976	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	276.2;
	3976	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	284.1;
	3976	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.5;
	3976	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.7;
	3977	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548.4;
	3977	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.4;
	3977	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	279.2;
	3977	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	285.1;
	3977	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.9;
	3977	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.1;
	3978	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.4;
	3978	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.4;
	3978	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	275.1;
	3978	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	278.8;
	3978	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.9;
	3978	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.5;
	3979	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532;
	3979	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160;
	3979	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	269.3;
	3979	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.4;
	3979	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.4;
	3979	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.9;
	3980	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.1;
	3980	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.5;
	3980	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	266.5;
	3980	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	270.7;
	3980	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76;
	3980	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.3;
	3981	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.4;
	3981	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.4;
	3981	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.6;
	3981	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265.8;
	3981	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.9;
	3981	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.1;
	3982	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.4;
	3982	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153;
	3982	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.4;
	3982	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253;
	3982	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.1;
	3982	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.8;
	3983	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.2;
	3983	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.6;
	3983	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.5;
	3983	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.1;
	3983	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68;
	3983	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.2;
	3984	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.8;
	3984	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.9;
	3984	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.9;
	3984	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.5;
	3984	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	3984	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.9;
	3985	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442.5;
	3985	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.5;
	3985	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.3;
	3985	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.7;
	3985	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	3985	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.1;
	3986	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.3;
	3986	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.7;
	3986	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.1;
	3986	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.8;
	3986	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57;
	3986	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.6;
	3987	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.4;
	3987	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.2;
	3987	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.6;
	3987	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.3;
	3987	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.3;
	3987	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333;
	3988	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	406.4;
	3988	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112;
	3988	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196;
	3988	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.9;
	3988	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.6;
	3988	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.2;
	3989	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.4;
	3989	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.2;
	3989	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198;
	3989	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.1;
	3989	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54;
	3989	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.6;
	3990	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.6;
	3990	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.8;
	3990	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.3;
	3990	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.6;
	3990	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.6;
	3990	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.2;
	3991	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.2;
	3991	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.3;
	3991	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222;
	3991	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.4;
	3991	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.3;
	3991	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362;
	3992	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.7;
	3992	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.5;
	3992	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.1;
	3992	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.5;
	3992	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.1;
	3992	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.7;
	3993	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.5;
	3993	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.3;
	3993	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.6;
	3993	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.7;
	3993	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	3993	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.3;
	3994	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.5;
	3994	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154;
	3994	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257;
	3994	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	263.6;
	3994	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	3994	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.8;
	3995	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	563.4;
	3995	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.5;
	3995	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261;
	3995	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	268.1;
	3995	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	3995	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.1;
	3996	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.1;
	3996	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.5;
	3996	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	265;
	3996	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.7;
	3996	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70;
	3996	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.6;
	3997	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	571.3;
	3997	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164;
	3997	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	271.2;
	3997	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	278.8;
	3997	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	3997	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.4;
	3998	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	585.6;
	3998	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.5;
	3998	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	275.9;
	3998	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	281.8;
	3998	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.1;
	3998	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	430.1;
	3999	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588.8;
	3999	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.1;
	3999	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	282.2;
	3999	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	285.9;
	3999	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.5;
	3999	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	433.2;
	4000	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.5;
	4000	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.2;
	4000	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	290;
	4000	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	292.3;
	4000	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.9;
	4000	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	439.1;
	4001	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	593.6;
	4001	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.5;
	4001	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	292.7;
	4001	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	293;
	4001	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.4;
	4001	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	439.4;
	4002	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588.4;
	4002	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.4;
	4002	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	284.9;
	4002	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	284.1;
	4002	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76;
	4002	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	435.6;
	4003	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	571.8;
	4003	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.8;
	4003	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	274.8;
	4003	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	274.7;
	4003	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.9;
	4003	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.3;
	4004	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.9;
	4004	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.7;
	4004	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	266.1;
	4004	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	266.9;
	4004	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.7;
	4004	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.6;
	4005	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.9;
	4005	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.3;
	4005	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.9;
	4005	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.2;
	4005	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.5;
	4005	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.4;
	4006	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525.8;
	4006	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.9;
	4006	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.4;
	4006	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.5;
	4006	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72;
	4006	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.4;
	4007	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.2;
	4007	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.1;
	4007	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.6;
	4007	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.9;
	4007	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.6;
	4007	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381;
	4008	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.6;
	4008	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.8;
	4008	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.2;
	4008	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.5;
	4008	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	4008	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.7;
	4009	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.2;
	4009	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.8;
	4009	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.6;
	4009	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.9;
	4009	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	4009	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.8;
	4010	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.8;
	4010	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.3;
	4010	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.2;
	4010	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.5;
	4010	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	4010	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.5;
	4011	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	401.5;
	4011	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.9;
	4011	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.5;
	4011	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.3;
	4011	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58;
	4011	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.8;
	4012	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403.7;
	4012	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.5;
	4012	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.9;
	4012	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.9;
	4012	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57;
	4012	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.5;
	4013	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.7;
	4013	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.4;
	4013	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.2;
	4013	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.6;
	4013	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.5;
	4013	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.3;
	4014	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.4;
	4014	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.7;
	4014	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.2;
	4014	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.7;
	4014	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.2;
	4014	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.4;
	4015	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412;
	4015	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.4;
	4015	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.5;
	4015	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.2;
	4015	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.9;
	4015	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.7;
	4016	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.8;
	4016	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.6;
	4016	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.1;
	4016	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.1;
	4016	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.2;
	4016	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.5;
	4017	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.9;
	4017	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.8;
	4017	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.3;
	4017	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.8;
	4017	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	4017	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.3;
	4018	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.4;
	4018	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.4;
	4018	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.2;
	4018	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.6;
	4018	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	4018	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.2;
	4019	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.6;
	4019	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.5;
	4019	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.6;
	4019	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.5;
	4019	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.3;
	4019	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.8;
	4020	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.6;
	4020	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.1;
	4020	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.1;
	4020	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.4;
	4020	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.8;
	4020	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.5;
	4021	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522;
	4021	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.7;
	4021	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.9;
	4021	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.8;
	4021	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.9;
	4021	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.6;
	4022	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.9;
	4022	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.5;
	4022	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257;
	4022	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.2;
	4022	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.3;
	4022	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.6;
	4023	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537.3;
	4023	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.4;
	4023	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.9;
	4023	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	269.6;
	4023	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.3;
	4023	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.5;
	4024	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537.5;
	4024	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.5;
	4024	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	264.9;
	4024	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	270.8;
	4024	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.5;
	4024	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.7;
	4025	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	540.7;
	4025	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.3;
	4025	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	264.4;
	4025	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	269.8;
	4025	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.2;
	4025	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.9;
	4026	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.4;
	4026	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.5;
	4026	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	264.2;
	4026	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	269.2;
	4026	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.7;
	4026	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.9;
	4027	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.4;
	4027	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.3;
	4027	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.8;
	4027	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267.5;
	4027	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71;
	4027	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399;
	4028	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.9;
	4028	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.8;
	4028	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.4;
	4028	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.3;
	4028	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.9;
	4028	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.9;
	4029	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.6;
	4029	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.9;
	4029	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.8;
	4029	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.3;
	4029	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.2;
	4029	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.6;
	4030	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.5;
	4030	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.8;
	4030	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.4;
	4030	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.1;
	4030	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	4030	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.2;
	4031	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.4;
	4031	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.7;
	4031	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.5;
	4031	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.3;
	4031	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	4031	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.8;
	4032	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440;
	4032	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.3;
	4032	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.2;
	4032	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.2;
	4032	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	4032	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.9;
	4033	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.1;
	4033	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.4;
	4033	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.9;
	4033	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212;
	4033	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	4033	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.5;
	4034	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	376.7;
	4034	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.6;
	4034	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.5;
	4034	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.9;
	4034	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.1;
	4034	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.8;
	4035	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	367.3;
	4035	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.5;
	4035	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.7;
	4035	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.3;
	4035	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.6;
	4035	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.3;
	4036	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	365.9;
	4036	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.3;
	4036	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.7;
	4036	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.5;
	4036	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.7;
	4036	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.7;
	4037	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	367.2;
	4037	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.7;
	4037	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.1;
	4037	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.8;
	4037	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.6;
	4037	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.2;
	4038	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	374.7;
	4038	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.2;
	4038	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.2;
	4038	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.1;
	4038	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.5;
	4038	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.7;
	4039	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	369.9;
	4039	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.1;
	4039	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.7;
	4039	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200;
	4039	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.6;
	4039	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.3;
	4040	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379.7;
	4040	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.4;
	4040	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.1;
	4040	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.6;
	4040	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.9;
	4040	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.3;
	4041	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	406.4;
	4041	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.8;
	4041	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.7;
	4041	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.7;
	4041	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	4041	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.2;
	4042	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.8;
	4042	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.3;
	4042	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.4;
	4042	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.8;
	4042	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	4042	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.7;
	4043	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.6;
	4043	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140;
	4043	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.5;
	4043	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.3;
	4043	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	4043	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.2;
	4044	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476;
	4044	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.6;
	4044	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.7;
	4044	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.5;
	4044	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.7;
	4044	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.7;
	4045	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.6;
	4045	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.8;
	4045	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.3;
	4045	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.9;
	4045	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.9;
	4045	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378;
	4046	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.2;
	4046	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.4;
	4046	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.6;
	4046	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.5;
	4046	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	4046	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.3;
	4047	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.3;
	4047	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.6;
	4047	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256;
	4047	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.7;
	4047	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.3;
	4047	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.5;
	4048	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499;
	4048	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.8;
	4048	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	264;
	4048	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	270.4;
	4048	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75;
	4048	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.9;
	4049	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.3;
	4049	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.9;
	4049	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	265.5;
	4049	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.6;
	4049	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.6;
	4049	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393;
	4050	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.6;
	4050	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.2;
	4050	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	263.7;
	4050	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	270.1;
	4050	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.4;
	4050	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.3;
	4051	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.2;
	4051	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.4;
	4051	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	259.6;
	4051	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265.7;
	4051	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76;
	4051	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.9;
	4052	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.7;
	4052	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.5;
	4052	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.2;
	4052	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.3;
	4052	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74;
	4052	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390;
	4053	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.3;
	4053	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.1;
	4053	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.5;
	4053	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.8;
	4053	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.4;
	4053	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.2;
	4054	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.3;
	4054	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.9;
	4054	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.9;
	4054	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.5;
	4054	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	4054	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.4;
	4055	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.4;
	4055	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.8;
	4055	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.7;
	4055	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.2;
	4055	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	4055	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.5;
	4056	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.2;
	4056	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.1;
	4056	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.5;
	4056	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.2;
	4056	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.3;
	4056	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352;
	4057	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417.2;
	4057	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.6;
	4057	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.5;
	4057	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.4;
	4057	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	4057	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.3;
	4058	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	386.5;
	4058	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.9;
	4058	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.8;
	4058	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.5;
	4058	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.2;
	4058	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.1;
	4059	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	370.6;
	4059	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.2;
	4059	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.9;
	4059	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.5;
	4059	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.7;
	4059	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.7;
	4060	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	371.4;
	4060	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.5;
	4060	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192;
	4060	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.7;
	4060	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.4;
	4060	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.5;
	4061	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	375.3;
	4061	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.6;
	4061	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.2;
	4061	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.8;
	4061	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.7;
	4061	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.6;
	4062	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394.8;
	4062	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.2;
	4062	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.4;
	4062	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.6;
	4062	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.5;
	4062	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.2;
	4063	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.6;
	4063	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.8;
	4063	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.9;
	4063	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.9;
	4063	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54;
	4063	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.1;
	4064	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.5;
	4064	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.1;
	4064	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.5;
	4064	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.2;
	4064	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.2;
	4064	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.5;
	4065	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.4;
	4065	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.9;
	4065	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.5;
	4065	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257;
	4065	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.8;
	4065	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.8;
	4066	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552;
	4066	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.3;
	4066	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.8;
	4066	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264;
	4066	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	4066	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.3;
	4067	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	581.9;
	4067	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.6;
	4067	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	266.5;
	4067	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	271.8;
	4067	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	4067	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.6;
	4068	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.7;
	4068	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.4;
	4068	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	273.2;
	4068	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	278.7;
	4068	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.4;
	4068	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	434;
	4069	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	624.9;
	4069	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.7;
	4069	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	278.6;
	4069	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	285;
	4069	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	4069	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	446.9;
	4070	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	631.8;
	4070	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.5;
	4070	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	286.8;
	4070	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	293.6;
	4070	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.4;
	4070	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	452.1;
	4071	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	642.6;
	4071	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.2;
	4071	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	295.7;
	4071	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	302.4;
	4071	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.8;
	4071	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	459.2;
	4072	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	617;
	4072	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.4;
	4072	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	298.1;
	4072	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	305.9;
	4072	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.5;
	4072	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	450.3;
	4073	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	615.3;
	4073	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.7;
	4073	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	297.8;
	4073	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	305.2;
	4073	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.7;
	4073	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	450.2;
	4074	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	605;
	4074	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171;
	4074	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	289.5;
	4074	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	297.4;
	4074	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76;
	4074	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	443.9;
	4075	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589.1;
	4075	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168;
	4075	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	277.6;
	4075	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	284.7;
	4075	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.3;
	4075	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	433.7;
	4076	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	567.2;
	4076	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.4;
	4076	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	268.8;
	4076	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	275.5;
	4076	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75;
	4076	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.2;
	4077	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552.3;
	4077	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.9;
	4077	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	267.3;
	4077	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	273.5;
	4077	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.3;
	4077	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.5;
	4078	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	546.2;
	4078	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.7;
	4078	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.6;
	4078	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.6;
	4078	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	4078	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.2;
	4079	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522.8;
	4079	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.9;
	4079	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.1;
	4079	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.7;
	4079	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	4079	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.8;
	4080	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.2;
	4080	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.6;
	4080	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.2;
	4080	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.4;
	4080	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	4080	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370;
	4081	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.6;
	4081	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.9;
	4081	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.6;
	4081	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.6;
	4081	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.2;
	4081	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.4;
	4082	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428.3;
	4082	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.3;
	4082	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.7;
	4082	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.7;
	4082	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	4082	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.5;
	4083	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.8;
	4083	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.5;
	4083	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.9;
	4083	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.2;
	4083	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.9;
	4083	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.2;
	4084	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.3;
	4084	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.9;
	4084	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.2;
	4084	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.5;
	4084	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.9;
	4084	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.8;
	4085	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.7;
	4085	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.7;
	4085	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.4;
	4085	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202;
	4085	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.8;
	4085	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.2;
	4086	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.3;
	4086	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.9;
	4086	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.1;
	4086	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.5;
	4086	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.5;
	4086	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.7;
	4087	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.6;
	4087	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.6;
	4087	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.3;
	4087	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.7;
	4087	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.8;
	4087	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.9;
	4088	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.8;
	4088	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.5;
	4088	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.4;
	4088	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250;
	4088	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.4;
	4088	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.6;
	4089	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.5;
	4089	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.9;
	4089	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257;
	4089	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.8;
	4089	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57;
	4089	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.1;
	4090	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	574;
	4090	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.5;
	4090	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	265.1;
	4090	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	273.8;
	4090	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60;
	4090	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.1;
	4091	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588.3;
	4091	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.4;
	4091	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	273.3;
	4091	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	281.5;
	4091	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	4091	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	430.2;
	4092	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	601.1;
	4092	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.8;
	4092	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	281.7;
	4092	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	290;
	4092	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.5;
	4092	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	437.8;
	4093	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	610.3;
	4093	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.2;
	4093	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	290.2;
	4093	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	297.9;
	4093	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.1;
	4093	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	444.8;
	4094	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	620.1;
	4094	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.8;
	4094	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	297.6;
	4094	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	305.3;
	4094	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.9;
	4094	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	451.5;
	4095	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	619.1;
	4095	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.5;
	4095	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	297;
	4095	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	305.9;
	4095	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75;
	4095	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	451.8;
	4096	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	634.7;
	4096	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.6;
	4096	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	297.5;
	4096	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	307.7;
	4096	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.4;
	4096	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	458.1;
	4097	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	640.6;
	4097	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.9;
	4097	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	300.9;
	4097	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	310.9;
	4097	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.1;
	4097	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	461;
	4098	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	638;
	4098	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.3;
	4098	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	298.7;
	4098	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	305.9;
	4098	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79;
	4098	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	459.2;
	4099	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	625.3;
	4099	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.6;
	4099	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	288.9;
	4099	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	295.8;
	4099	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.6;
	4099	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	451.5;
	4100	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	608.7;
	4100	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.1;
	4100	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	287.5;
	4100	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	293.9;
	4100	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.4;
	4100	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	443.2;
	4101	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	591.3;
	4101	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.4;
	4101	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	281.5;
	4101	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	288.2;
	4101	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.2;
	4101	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	434.4;
	4102	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.9;
	4102	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165;
	4102	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	266;
	4102	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	271.8;
	4102	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.2;
	4102	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.7;
	4103	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.4;
	4103	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.9;
	4103	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.3;
	4103	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.6;
	4103	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.5;
	4103	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.6;
	4104	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.8;
	4104	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.2;
	4104	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.3;
	4104	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.6;
	4104	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.9;
	4104	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.7;
	4105	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.4;
	4105	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.6;
	4105	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.4;
	4105	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.8;
	4105	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	4105	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.6;
	4106	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.1;
	4106	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.6;
	4106	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.1;
	4106	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210;
	4106	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	4106	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.8;
	4107	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428.8;
	4107	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.5;
	4107	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.7;
	4107	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.6;
	4107	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.6;
	4107	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.9;
	4108	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.6;
	4108	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.2;
	4108	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.8;
	4108	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.7;
	4108	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.5;
	4108	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.3;
	4109	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	427.4;
	4109	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.1;
	4109	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.8;
	4109	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.7;
	4109	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.4;
	4109	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.1;
	4110	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.1;
	4110	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.4;
	4110	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.1;
	4110	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.5;
	4110	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.9;
	4110	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.5;
	4111	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.6;
	4111	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.3;
	4111	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.6;
	4111	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.6;
	4111	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.1;
	4111	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.3;
	4112	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.4;
	4112	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.2;
	4112	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.1;
	4112	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.1;
	4112	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.2;
	4112	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.9;
	4113	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518.7;
	4113	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.6;
	4113	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	267.1;
	4113	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	273.2;
	4113	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	4113	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.3;
	4114	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.3;
	4114	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.5;
	4114	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	280.2;
	4114	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	285.8;
	4114	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70;
	4114	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.4;
	4115	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.9;
	4115	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.4;
	4115	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	293.2;
	4115	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	296.8;
	4115	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.2;
	4115	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422;
	4116	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	571.1;
	4116	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.6;
	4116	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	302.7;
	4116	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	304.5;
	4116	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.3;
	4116	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	433.1;
	4117	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	567.9;
	4117	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.8;
	4117	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	312.1;
	4117	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	313.4;
	4117	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.2;
	4117	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	435.7;
	4118	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	591.3;
	4118	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.5;
	4118	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	320.8;
	4118	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	321.6;
	4118	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.1;
	4118	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	448.5;
	4119	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586.4;
	4119	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.2;
	4119	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	329.2;
	4119	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	328.7;
	4119	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.8;
	4119	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	449.2;
	4120	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	602.5;
	4120	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.2;
	4120	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	336.4;
	4120	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	335.4;
	4120	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.1;
	4120	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	458.3;
	4121	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	590.3;
	4121	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186;
	4121	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	335.7;
	4121	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	334.8;
	4121	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.7;
	4121	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	454.3;
	4122	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	580.1;
	4122	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.3;
	4122	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	325.8;
	4122	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	324.9;
	4122	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.1;
	4122	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	447.8;
	4123	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577.9;
	4123	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.3;
	4123	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	317.3;
	4123	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	316.5;
	4123	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.3;
	4123	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	443;
	4124	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	563.1;
	4124	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.2;
	4124	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	305.9;
	4124	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	306.5;
	4124	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.6;
	4124	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	433.2;
	4125	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547;
	4125	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.1;
	4125	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	294.5;
	4125	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	296.1;
	4125	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.1;
	4125	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.3;
	4126	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.1;
	4126	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172;
	4126	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	281.3;
	4126	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	281.8;
	4126	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.3;
	4126	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.1;
	4127	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516;
	4127	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.4;
	4127	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.2;
	4127	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259;
	4127	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.2;
	4127	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398;
	4128	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.8;
	4128	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.8;
	4128	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248;
	4128	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.1;
	4128	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.9;
	4128	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.9;
	4129	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.9;
	4129	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.1;
	4129	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.4;
	4129	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.4;
	4129	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	4129	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.4;
	4130	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.4;
	4130	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.1;
	4130	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.5;
	4130	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.2;
	4130	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	4130	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.8;
	4131	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.7;
	4131	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.9;
	4131	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.6;
	4131	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.3;
	4131	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	4131	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.5;
	4132	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403.3;
	4132	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.3;
	4132	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.7;
	4132	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.7;
	4132	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.2;
	4132	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.1;
	4133	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.6;
	4133	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.5;
	4133	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.6;
	4133	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.2;
	4133	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	4133	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.7;
	4134	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.9;
	4134	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.3;
	4134	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220;
	4134	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.7;
	4134	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.9;
	4134	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.9;
	4135	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.7;
	4135	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.6;
	4135	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.6;
	4135	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244;
	4135	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61;
	4135	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.3;
	4136	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.1;
	4136	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153;
	4136	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.9;
	4136	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.5;
	4136	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	4136	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385;
	4137	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518.1;
	4137	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.1;
	4137	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	270.3;
	4137	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	275.7;
	4137	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.1;
	4137	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.2;
	4138	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.9;
	4138	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.2;
	4138	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	280.9;
	4138	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	285.1;
	4138	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.2;
	4138	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.7;
	4139	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	568.6;
	4139	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.7;
	4139	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	289;
	4139	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	292.8;
	4139	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.1;
	4139	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	429.3;
	4140	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	567.2;
	4140	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.3;
	4140	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	293.7;
	4140	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	296.7;
	4140	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.9;
	4140	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	429.8;
	4141	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	575.7;
	4141	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174;
	4141	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	299.3;
	4141	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	300.8;
	4141	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.9;
	4141	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	435.7;
	4142	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579.2;
	4142	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.9;
	4142	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	303.2;
	4142	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	304;
	4142	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.2;
	4142	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	438.3;
	4143	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	585.5;
	4143	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.6;
	4143	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	306;
	4143	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	306.5;
	4143	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.2;
	4143	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	441.6;
	4144	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	587.3;
	4144	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.3;
	4144	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	309.5;
	4144	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	309.1;
	4144	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.7;
	4144	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	443;
	4145	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588.8;
	4145	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.5;
	4145	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	306.2;
	4145	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	304.7;
	4145	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.5;
	4145	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	443.4;
	4146	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	575.6;
	4146	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.6;
	4146	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	298.8;
	4146	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	297.6;
	4146	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.8;
	4146	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	436;
	4147	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572;
	4147	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.6;
	4147	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	292.8;
	4147	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	291.9;
	4147	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.7;
	4147	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	431.5;
	4148	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565;
	4148	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.4;
	4148	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	285.3;
	4148	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	285.5;
	4148	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.4;
	4148	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.6;
	4149	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548.8;
	4149	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.4;
	4149	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	280.5;
	4149	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	280.8;
	4149	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.1;
	4149	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.1;
	4150	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.9;
	4150	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.7;
	4150	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	268.5;
	4150	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267.9;
	4150	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.9;
	4150	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.6;
	4151	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.9;
	4151	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.8;
	4151	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.4;
	4151	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.2;
	4151	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.1;
	4151	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.5;
	4152	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.1;
	4152	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.5;
	4152	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.4;
	4152	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.2;
	4152	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	4152	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.8;
	4153	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.1;
	4153	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.7;
	4153	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.4;
	4153	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.5;
	4153	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	4153	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.9;
	4154	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	420;
	4154	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.1;
	4154	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.8;
	4154	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.1;
	4154	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.2;
	4154	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.4;
	4155	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.5;
	4155	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.8;
	4155	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205;
	4155	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207;
	4155	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	4155	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.7;
	4156	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.3;
	4156	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.8;
	4156	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.9;
	4156	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.2;
	4156	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.1;
	4156	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.7;
	4157	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.7;
	4157	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119;
	4157	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.7;
	4157	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.3;
	4157	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.3;
	4157	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.9;
	4158	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	426.3;
	4158	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.4;
	4158	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214;
	4158	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.7;
	4158	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.2;
	4158	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.2;
	4159	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.9;
	4159	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.7;
	4159	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230;
	4159	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.1;
	4159	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	4159	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.1;
	4160	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.9;
	4160	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149;
	4160	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.8;
	4160	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.7;
	4160	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	4160	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.3;
	4161	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.3;
	4161	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.2;
	4161	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.4;
	4161	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.1;
	4161	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	4161	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.6;
	4162	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.8;
	4162	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.4;
	4162	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	273.4;
	4162	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	275.8;
	4162	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.4;
	4162	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.7;
	4163	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.2;
	4163	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.4;
	4163	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	283.7;
	4163	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	284.8;
	4163	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.6;
	4163	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.7;
	4164	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558;
	4164	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.2;
	4164	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	296.1;
	4164	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	295.6;
	4164	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.1;
	4164	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	426.6;
	4165	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577.1;
	4165	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.1;
	4165	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	306.2;
	4165	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	304.1;
	4165	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.2;
	4165	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	438.5;
	4166	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	587.6;
	4166	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.7;
	4166	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	309;
	4166	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	305.5;
	4166	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.6;
	4166	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	444.2;
	4167	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	594.5;
	4167	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.2;
	4167	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	316.4;
	4167	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	311.9;
	4167	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.7;
	4167	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	448.9;
	4168	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589.5;
	4168	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.7;
	4168	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	321.7;
	4168	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	315.9;
	4168	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.6;
	4168	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	448.6;
	4169	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	594.8;
	4169	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.4;
	4169	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	317;
	4169	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	310.8;
	4169	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.6;
	4169	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	450.4;
	4170	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	585.6;
	4170	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.5;
	4170	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	301.7;
	4170	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	297.1;
	4170	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78;
	4170	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	442.3;
	4171	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	567.3;
	4171	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167;
	4171	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	288.6;
	4171	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	285;
	4171	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.3;
	4171	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	429.5;
	4172	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.1;
	4172	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.7;
	4172	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	277.8;
	4172	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	276.4;
	4172	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76;
	4172	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.3;
	4173	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.6;
	4173	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.3;
	4173	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	270;
	4173	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	269.5;
	4173	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.9;
	4173	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.1;
	4174	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.4;
	4174	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.5;
	4174	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.5;
	4174	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.3;
	4174	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.3;
	4174	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391;
	4175	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.6;
	4175	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.8;
	4175	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.6;
	4175	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.7;
	4175	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	4175	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.2;
	4176	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.2;
	4176	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.8;
	4176	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.7;
	4176	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.6;
	4176	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	4176	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.1;
	4177	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433;
	4177	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.6;
	4177	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.7;
	4177	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.5;
	4177	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	4177	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.3;
	4178	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393;
	4178	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.1;
	4178	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.1;
	4178	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208;
	4178	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60;
	4178	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.9;
	4179	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379;
	4179	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.3;
	4179	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.7;
	4179	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.7;
	4179	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.6;
	4179	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.8;
	4180	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	371.5;
	4180	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.7;
	4180	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.1;
	4180	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.7;
	4180	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.5;
	4180	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.5;
	4181	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379.9;
	4181	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.6;
	4181	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.4;
	4181	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.1;
	4181	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.2;
	4181	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321;
	4182	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	387.4;
	4182	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.3;
	4182	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.9;
	4182	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.2;
	4182	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.2;
	4182	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.6;
	4183	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	400.6;
	4183	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.2;
	4183	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.5;
	4183	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.4;
	4183	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	4183	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.2;
	4184	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.6;
	4184	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.5;
	4184	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214;
	4184	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218;
	4184	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	4184	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.6;
	4185	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.2;
	4185	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.1;
	4185	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.7;
	4185	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.4;
	4185	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	4185	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.6;
	4186	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.2;
	4186	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.9;
	4186	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.7;
	4186	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.2;
	4186	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70;
	4186	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.8;
	4187	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.7;
	4187	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.6;
	4187	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.6;
	4187	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.2;
	4187	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.3;
	4187	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.2;
	4188	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.7;
	4188	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.8;
	4188	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.6;
	4188	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.7;
	4188	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.9;
	4188	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.4;
	4189	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.4;
	4189	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.2;
	4189	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.2;
	4189	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265.7;
	4189	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76;
	4189	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.1;
	4190	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.9;
	4190	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.7;
	4190	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	267;
	4190	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	271.6;
	4190	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.1;
	4190	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.4;
	4191	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.3;
	4191	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.3;
	4191	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	272.7;
	4191	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	276.8;
	4191	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79;
	4191	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.7;
	4192	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.4;
	4192	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.1;
	4192	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	274.9;
	4192	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	278;
	4192	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.1;
	4192	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.9;
	4193	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.6;
	4193	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160;
	4193	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	273.2;
	4193	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	276.3;
	4193	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.1;
	4193	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.8;
	4194	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.7;
	4194	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159;
	4194	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	272.5;
	4194	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	275.2;
	4194	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.2;
	4194	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.2;
	4195	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.4;
	4195	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.7;
	4195	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	267.1;
	4195	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	271;
	4195	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.1;
	4195	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.8;
	4196	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.8;
	4196	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.3;
	4196	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.1;
	4196	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	263.7;
	4196	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.4;
	4196	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.8;
	4197	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.8;
	4197	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.1;
	4197	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.5;
	4197	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.1;
	4197	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.9;
	4197	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.7;
	4198	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.8;
	4198	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.3;
	4198	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.6;
	4198	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.7;
	4198	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.7;
	4198	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.3;
	4199	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439;
	4199	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.6;
	4199	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.1;
	4199	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.2;
	4199	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	4199	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.7;
	4200	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	427.4;
	4200	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.7;
	4200	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.1;
	4200	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.4;
	4200	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	4200	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348;
	4201	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.2;
	4201	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.7;
	4201	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.3;
	4201	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.7;
	4201	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.2;
	4201	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.3;
	4202	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	362.7;
	4202	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.7;
	4202	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.3;
	4202	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.5;
	4202	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	4202	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.3;
	4203	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	347;
	4203	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.3;
	4203	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.1;
	4203	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.7;
	4203	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	4203	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	306.6;
	4204	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	352.4;
	4204	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.8;
	4204	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.1;
	4204	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.4;
	4204	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	4204	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	307.9;
	4205	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	354.8;
	4205	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.2;
	4205	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.1;
	4205	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.5;
	4205	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.4;
	4205	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.9;
	4206	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	355.5;
	4206	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.8;
	4206	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.4;
	4206	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.5;
	4206	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59;
	4206	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309.9;
	4207	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	362.4;
	4207	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.3;
	4207	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.6;
	4207	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.2;
	4207	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.2;
	4207	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.5;
	4208	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	364.9;
	4208	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.6;
	4208	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.2;
	4208	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.9;
	4208	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	4208	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.3;
	4209	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	388.5;
	4209	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.7;
	4209	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.8;
	4209	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.9;
	4209	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.1;
	4209	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.6;
	4210	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.8;
	4210	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.1;
	4210	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.9;
	4210	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.4;
	4210	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	4210	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.4;
	4211	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.8;
	4211	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.6;
	4211	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.1;
	4211	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.1;
	4211	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.1;
	4211	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.9;
	4212	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.1;
	4212	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.1;
	4212	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.5;
	4212	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.5;
	4212	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.1;
	4212	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.3;
	4213	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.4;
	4213	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.6;
	4213	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	269.5;
	4213	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	271.9;
	4213	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.6;
	4213	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.9;
	4214	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.6;
	4214	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.2;
	4214	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	283.6;
	4214	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	283.4;
	4214	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.8;
	4214	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.2;
	4215	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.3;
	4215	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.1;
	4215	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	291.6;
	4215	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	289.5;
	4215	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.3;
	4215	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.8;
	4216	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.4;
	4216	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.6;
	4216	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	295.6;
	4216	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	294;
	4216	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.7;
	4216	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.1;
	4217	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.7;
	4217	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.5;
	4217	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	298.3;
	4217	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	295.4;
	4217	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.4;
	4217	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.9;
	4218	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	533.2;
	4218	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.6;
	4218	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	298.3;
	4218	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	296.3;
	4218	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.9;
	4218	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.4;
	4219	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	533.5;
	4219	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.5;
	4219	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	292.2;
	4219	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	290.6;
	4219	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.7;
	4219	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.4;
	4220	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518.4;
	4220	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.9;
	4220	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	279.4;
	4220	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	279;
	4220	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.4;
	4220	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.5;
	4221	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.9;
	4221	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171;
	4221	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	277.9;
	4221	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	278.4;
	4221	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.4;
	4221	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.8;
	4222	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.2;
	4222	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.4;
	4222	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	268.2;
	4222	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	268.7;
	4222	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.4;
	4222	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.2;
	4223	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.1;
	4223	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.2;
	4223	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.2;
	4223	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.1;
	4223	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.6;
	4223	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.3;
	4224	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.1;
	4224	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.6;
	4224	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.8;
	4224	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.6;
	4224	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	4224	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.9;
	4225	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.7;
	4225	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132;
	4225	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.9;
	4225	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.8;
	4225	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	4225	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.1;
	4226	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	382.7;
	4226	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.4;
	4226	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.2;
	4226	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.7;
	4226	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	4226	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.4;
	4227	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	380.5;
	4227	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.8;
	4227	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.1;
	4227	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.8;
	4227	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.3;
	4227	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.9;
	4228	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379.6;
	4228	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.7;
	4228	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.8;
	4228	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.6;
	4228	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	4228	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.9;
	4229	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	385.3;
	4229	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.9;
	4229	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.3;
	4229	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.4;
	4229	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.5;
	4229	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.5;
	4230	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392;
	4230	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.8;
	4230	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.7;
	4230	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.1;
	4230	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.5;
	4230	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.9;
	4231	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421;
	4231	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.9;
	4231	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226;
	4231	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.4;
	4231	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.2;
	4231	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.4;
	4232	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.2;
	4232	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.7;
	4232	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.2;
	4232	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.7;
	4232	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.3;
	4232	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.7;
	4233	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.7;
	4233	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.7;
	4233	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	268.5;
	4233	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.8;
	4233	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	4233	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.7;
	4234	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	533.4;
	4234	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.4;
	4234	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	281.7;
	4234	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	286;
	4234	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	4234	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.2;
	4235	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	555.1;
	4235	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.8;
	4235	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	293.5;
	4235	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	297.5;
	4235	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.1;
	4235	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.2;
	4236	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	575.4;
	4236	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.6;
	4236	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	306.2;
	4236	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	309.5;
	4236	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.1;
	4236	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	437.1;
	4237	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	596.1;
	4237	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.2;
	4237	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	315.7;
	4237	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	318.5;
	4237	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74;
	4237	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	449.3;
	4238	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	615.9;
	4238	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.5;
	4238	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	324.6;
	4238	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	328;
	4238	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.1;
	4238	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	460.2;
	4239	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	632.6;
	4239	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.5;
	4239	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	334.1;
	4239	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	336.2;
	4239	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.5;
	4239	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	470;
	4240	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	626.2;
	4240	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.4;
	4240	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	337.6;
	4240	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	340.2;
	4240	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78;
	4240	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	469.2;
	4241	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	641.2;
	4241	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.5;
	4241	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	332.3;
	4241	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	334.7;
	4241	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.7;
	4241	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	474.3;
	4242	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	648.1;
	4242	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.8;
	4242	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	324.4;
	4242	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	325.6;
	4242	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.4;
	4242	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	474.1;
	4243	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	631.3;
	4243	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.8;
	4243	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	311;
	4243	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	311.7;
	4243	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.1;
	4243	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	462.7;
	4244	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	607.8;
	4244	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.6;
	4244	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	303.8;
	4244	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	305.4;
	4244	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.2;
	4244	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	449.3;
	4245	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.7;
	4245	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.3;
	4245	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	292.9;
	4245	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	295.4;
	4245	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.1;
	4245	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	439.5;
	4246	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	569.9;
	4246	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.3;
	4246	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	279.3;
	4246	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	280.7;
	4246	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.7;
	4246	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.7;
	4247	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552;
	4247	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.3;
	4247	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.9;
	4247	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.3;
	4247	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	4247	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.4;
	4248	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.1;
	4248	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.8;
	4248	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.8;
	4248	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.8;
	4248	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	4248	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.7;
	4249	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.8;
	4249	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.9;
	4249	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.3;
	4249	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.5;
	4249	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	4249	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373;
	4250	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.2;
	4250	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.9;
	4250	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.6;
	4250	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.3;
	4250	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	4250	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353;
	4251	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.1;
	4251	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121;
	4251	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.3;
	4251	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.6;
	4251	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.9;
	4251	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.7;
	4252	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	430.7;
	4252	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.9;
	4252	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.4;
	4252	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.9;
	4252	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.4;
	4252	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.9;
	4253	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.8;
	4253	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.4;
	4253	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.2;
	4253	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.6;
	4253	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.1;
	4253	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.1;
	4254	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.8;
	4254	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.2;
	4254	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212;
	4254	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.8;
	4254	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.2;
	4254	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.9;
	4255	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.1;
	4255	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.9;
	4255	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.6;
	4255	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.2;
	4255	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.9;
	4255	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.8;
	4256	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.2;
	4256	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.2;
	4256	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.8;
	4256	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.9;
	4256	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.6;
	4256	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.8;
	4257	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.3;
	4257	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.5;
	4257	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	265.4;
	4257	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	273.2;
	4257	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57;
	4257	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.7;
	4258	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	573;
	4258	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.2;
	4258	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	263.5;
	4258	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	273.4;
	4258	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.9;
	4258	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.9;
	4259	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	596.8;
	4259	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.3;
	4259	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	299.3;
	4259	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	305.9;
	4259	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	4259	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	440.2;
	4260	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	611.4;
	4260	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.5;
	4260	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	305.3;
	4260	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	312.9;
	4260	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.1;
	4260	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	450.6;
	4261	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	642.1;
	4261	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.5;
	4261	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	316.3;
	4261	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	323.1;
	4261	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.8;
	4261	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	467.2;
	4262	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	653.6;
	4262	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.8;
	4262	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	322.8;
	4262	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	327.9;
	4262	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.9;
	4262	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	474.3;
	4263	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	646.7;
	4263	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.2;
	4263	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	327.7;
	4263	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	333.5;
	4263	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.4;
	4263	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	473.2;
	4264	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	655.3;
	4264	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.2;
	4264	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	334.2;
	4264	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	337.8;
	4264	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.2;
	4264	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	479;
	4265	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	654.2;
	4265	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.3;
	4265	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	332;
	4265	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	337.2;
	4265	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.6;
	4265	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	478.2;
	4266	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	641.5;
	4266	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.6;
	4266	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	321.5;
	4266	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	325.9;
	4266	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77;
	4266	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	469.9;
	4267	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	610.3;
	4267	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.9;
	4267	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	311.7;
	4267	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	315.8;
	4267	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.1;
	4267	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	453.7;
	4268	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	591.6;
	4268	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.8;
	4268	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	303.5;
	4268	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	308.3;
	4268	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.4;
	4268	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	443;
	4269	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.1;
	4269	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.8;
	4269	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	294.8;
	4269	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	299.8;
	4269	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.2;
	4269	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	431;
	4270	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548.6;
	4270	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.9;
	4270	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	277.7;
	4270	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	282.7;
	4270	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.1;
	4270	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.1;
	4271	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.8;
	4271	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.6;
	4271	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.2;
	4271	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.3;
	4271	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.1;
	4271	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.5;
	4272	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.3;
	4272	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.1;
	4272	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.3;
	4272	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.9;
	4272	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.2;
	4272	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.9;
	4273	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.5;
	4273	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.8;
	4273	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.7;
	4273	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.5;
	4273	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	4273	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.6;
	4274	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.8;
	4274	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122;
	4274	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.6;
	4274	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.6;
	4274	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.8;
	4274	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.3;
	4275	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417.5;
	4275	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119;
	4275	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.1;
	4275	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.8;
	4275	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.2;
	4275	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.4;
	4276	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.5;
	4276	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.7;
	4276	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.5;
	4276	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.3;
	4276	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55;
	4276	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.2;
	4277	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.6;
	4277	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.9;
	4277	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.3;
	4277	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.8;
	4277	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	4277	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.6;
	4278	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.7;
	4278	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.3;
	4278	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.6;
	4278	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.9;
	4278	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.5;
	4278	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.9;
	4279	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468;
	4279	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.4;
	4279	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.1;
	4279	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.7;
	4279	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.7;
	4279	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.3;
	4280	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.4;
	4280	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150;
	4280	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249;
	4280	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.7;
	4280	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.5;
	4280	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.5;
	4281	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516;
	4281	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.9;
	4281	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.7;
	4281	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	269.5;
	4281	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	4281	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.3;
	4282	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.8;
	4282	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.8;
	4282	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	288.6;
	4282	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	294;
	4282	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	4282	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.8;
	4283	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.3;
	4283	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169;
	4283	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	289.6;
	4283	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	296.4;
	4283	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.6;
	4283	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	428.6;
	4284	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	574.3;
	4284	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.1;
	4284	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	301.2;
	4284	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	306.7;
	4284	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.7;
	4284	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	435.2;
	4285	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577.6;
	4285	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.8;
	4285	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	304.9;
	4285	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	310.6;
	4285	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.1;
	4285	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	438.9;
	4286	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	580.9;
	4286	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.1;
	4286	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	313.4;
	4286	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	318.7;
	4286	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.8;
	4286	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	442.4;
	4287	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	585;
	4287	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.6;
	4287	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	325.5;
	4287	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	329.1;
	4287	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.3;
	4287	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	447;
	4288	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	596.8;
	4288	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.8;
	4288	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	324.4;
	4288	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	328.4;
	4288	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.1;
	4288	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	453;
	4289	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	604.3;
	4289	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.1;
	4289	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	328.8;
	4289	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	330.9;
	4289	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.4;
	4289	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	457.1;
	4290	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586.1;
	4290	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.7;
	4290	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	323.2;
	4290	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	324;
	4290	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90;
	4290	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	447.7;
	4291	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	581.7;
	4291	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.8;
	4291	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	311.1;
	4291	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	311.8;
	4291	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.9;
	4291	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	442.3;
	4292	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	569;
	4292	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174;
	4292	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	300.8;
	4292	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	301.4;
	4292	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.7;
	4292	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	433.4;
	4293	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	554.5;
	4293	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.4;
	4293	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	291.6;
	4293	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	292.1;
	4293	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.8;
	4293	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.7;
	4294	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.8;
	4294	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.1;
	4294	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	276.8;
	4294	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	277.1;
	4294	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.8;
	4294	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.6;
	4295	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.2;
	4295	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.3;
	4295	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.6;
	4295	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.9;
	4295	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.7;
	4295	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.4;
	4296	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.8;
	4296	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.2;
	4296	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243;
	4296	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.7;
	4296	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.3;
	4296	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.4;
	4297	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.7;
	4297	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.3;
	4297	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222;
	4297	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.4;
	4297	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.9;
	4297	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.3;
	4298	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.6;
	4298	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.5;
	4298	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.7;
	4298	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.6;
	4298	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	4298	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.9;
	4299	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.7;
	4299	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.6;
	4299	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206;
	4299	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.9;
	4299	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	4299	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.8;
	4300	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.8;
	4300	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.5;
	4300	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204;
	4300	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.6;
	4300	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	4300	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335;
	4301	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417.4;
	4301	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.4;
	4301	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.9;
	4301	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.5;
	4301	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61;
	4301	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.4;
	4302	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.3;
	4302	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.9;
	4302	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.6;
	4302	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.1;
	4302	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.3;
	4302	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.4;
	4303	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.7;
	4303	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137;
	4303	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.5;
	4303	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.4;
	4303	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	4303	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.7;
	4304	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.9;
	4304	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.5;
	4304	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.7;
	4304	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.2;
	4304	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	4304	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.8;
	4305	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	524.7;
	4305	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.2;
	4305	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.2;
	4305	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265.4;
	4305	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	4305	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.7;
	4306	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.1;
	4306	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.7;
	4306	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	285.5;
	4306	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	287.8;
	4306	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.1;
	4306	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.6;
	4307	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	557.1;
	4307	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167;
	4307	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	289.3;
	4307	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	292.6;
	4307	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.2;
	4307	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.8;
	4308	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566.9;
	4308	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.5;
	4308	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	304.1;
	4308	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	305.7;
	4308	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.6;
	4308	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	433.3;
	4309	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	567.1;
	4309	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.6;
	4309	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	314.7;
	4309	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	315.5;
	4309	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.9;
	4309	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	437.8;
	4310	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566.7;
	4310	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.1;
	4310	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	326.4;
	4310	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	327.1;
	4310	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.1;
	4310	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	441;
	4311	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579.7;
	4311	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.9;
	4311	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	332.7;
	4311	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	332.7;
	4311	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.2;
	4311	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	449.3;
	4312	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	591.3;
	4312	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.9;
	4312	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	335.4;
	4312	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	336.1;
	4312	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.2;
	4312	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	455.2;
	4313	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	582.8;
	4313	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.8;
	4313	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	335.3;
	4313	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	334.9;
	4313	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.5;
	4313	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	452.1;
	4314	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579.7;
	4314	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.2;
	4314	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	325.1;
	4314	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	325.3;
	4314	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.1;
	4314	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	448;
	4315	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	569.8;
	4315	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177;
	4315	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	312.6;
	4315	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	312.6;
	4315	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.5;
	4315	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	439.1;
	4316	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	568.6;
	4316	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.1;
	4316	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	300.9;
	4316	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	300.6;
	4316	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.4;
	4316	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	433.8;
	4317	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	554.1;
	4317	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.1;
	4317	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	294.5;
	4317	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	294.1;
	4317	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.6;
	4317	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	424.5;
	4318	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.9;
	4318	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.2;
	4318	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	279.8;
	4318	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	278.9;
	4318	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.6;
	4318	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.1;
	4319	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.2;
	4319	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.8;
	4319	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.8;
	4319	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.7;
	4319	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.1;
	4319	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.1;
	4320	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.9;
	4320	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.9;
	4320	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251;
	4320	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.3;
	4320	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.5;
	4320	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.2;
	4321	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.1;
	4321	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132;
	4321	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.9;
	4321	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.3;
	4321	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	4321	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.2;
	4322	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.8;
	4322	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.2;
	4322	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.8;
	4322	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.8;
	4322	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	4322	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350;
	4323	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	427.9;
	4323	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.3;
	4323	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.1;
	4323	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.4;
	4323	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.2;
	4323	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.6;
	4324	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	427.5;
	4324	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.7;
	4324	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.6;
	4324	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.1;
	4324	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59;
	4324	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344;
	4325	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.8;
	4325	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.7;
	4325	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.2;
	4325	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.6;
	4325	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58;
	4325	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.3;
	4326	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.1;
	4326	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.3;
	4326	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.1;
	4326	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.9;
	4326	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.1;
	4326	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.5;
	4327	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.6;
	4327	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.9;
	4327	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.5;
	4327	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.8;
	4327	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.1;
	4327	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.5;
	4328	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.3;
	4328	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.2;
	4328	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.4;
	4328	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.1;
	4328	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	4328	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.3;
	4329	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525;
	4329	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.4;
	4329	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	268.8;
	4329	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	269.8;
	4329	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	4329	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.6;
	4330	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.7;
	4330	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.4;
	4330	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	284;
	4330	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	284.8;
	4330	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.9;
	4330	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.7;
	4331	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.2;
	4331	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.7;
	4331	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	296.7;
	4331	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	296.5;
	4331	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.9;
	4331	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	430.7;
	4332	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	574.7;
	4332	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.2;
	4332	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	310.4;
	4332	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	309.4;
	4332	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.3;
	4332	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	438.8;
	4333	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	585.1;
	4333	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182;
	4333	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	321.9;
	4333	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	320.6;
	4333	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.1;
	4333	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	447.6;
	4334	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	594.4;
	4334	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.8;
	4334	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	333.5;
	4334	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	331.3;
	4334	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.8;
	4334	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	455.2;
	4335	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	602.9;
	4335	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.2;
	4335	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	339.1;
	4335	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	337;
	4335	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.1;
	4335	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	461.1;
	4336	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	605.9;
	4336	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.5;
	4336	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	344.6;
	4336	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	341.7;
	4336	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.3;
	4336	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	463.7;
	4337	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	595.6;
	4337	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.3;
	4337	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	341.6;
	4337	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	338.6;
	4337	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.9;
	4337	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	459.4;
	4338	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	576;
	4338	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.4;
	4338	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	333.2;
	4338	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	330.3;
	4338	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.5;
	4338	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	448.7;
	4339	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.8;
	4339	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.9;
	4339	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	318.8;
	4339	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	315.5;
	4339	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.2;
	4339	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	437.2;
	4340	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.2;
	4340	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.9;
	4340	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	305.6;
	4340	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	303.2;
	4340	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.7;
	4340	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.1;
	4341	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.6;
	4341	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173;
	4341	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	294.7;
	4341	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	293.1;
	4341	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.8;
	4341	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407;
	4342	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.7;
	4342	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.7;
	4342	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	281.3;
	4342	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	279.2;
	4342	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.2;
	4342	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.7;
	4343	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.4;
	4343	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.2;
	4343	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261;
	4343	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.7;
	4343	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.7;
	4343	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382;
	4344	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.1;
	4344	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.3;
	4344	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.4;
	4344	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.3;
	4344	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	4344	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.2;
	4345	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417.9;
	4345	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.6;
	4345	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.3;
	4345	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.3;
	4345	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	4345	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.5;
	4346	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.9;
	4346	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.7;
	4346	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218;
	4346	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.2;
	4346	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.3;
	4346	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.3;
	4347	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	372.7;
	4347	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.3;
	4347	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.3;
	4347	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.9;
	4347	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	4347	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.4;
	4348	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	373.7;
	4348	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.6;
	4348	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.2;
	4348	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.1;
	4348	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.1;
	4348	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.3;
	4349	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	375.5;
	4349	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.2;
	4349	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.2;
	4349	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.1;
	4349	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.7;
	4349	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.1;
	4350	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	389.4;
	4350	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.3;
	4350	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.3;
	4350	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.7;
	4350	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56;
	4350	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.5;
	4351	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	402.4;
	4351	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.8;
	4351	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.7;
	4351	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.5;
	4351	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.8;
	4351	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.9;
	4352	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.2;
	4352	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.5;
	4352	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.9;
	4352	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.6;
	4352	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	4352	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.5;
	4353	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.9;
	4353	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.7;
	4353	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.2;
	4353	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.9;
	4353	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	4353	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.5;
	4354	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.4;
	4354	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.5;
	4354	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.5;
	4354	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.2;
	4354	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	4354	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377;
	4355	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.7;
	4355	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.1;
	4355	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	271.3;
	4355	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	273.2;
	4355	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.6;
	4355	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.9;
	4356	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.9;
	4356	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.9;
	4356	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	280.3;
	4356	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	282.4;
	4356	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.8;
	4356	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.9;
	4357	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.4;
	4357	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.6;
	4357	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	289.8;
	4357	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	292;
	4357	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.9;
	4357	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411;
	4358	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	533.7;
	4358	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.9;
	4358	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	296.3;
	4358	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	298.9;
	4358	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.9;
	4358	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.7;
	4359	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	534;
	4359	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.9;
	4359	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	300;
	4359	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	302.8;
	4359	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85;
	4359	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.7;
	4360	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549.2;
	4360	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.2;
	4360	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	302.6;
	4360	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	305.6;
	4360	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.3;
	4360	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	426.1;
	4361	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552.7;
	4361	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.2;
	4361	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	298.7;
	4361	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	302.5;
	4361	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88;
	4361	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	426.7;
	4362	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	555.3;
	4362	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.2;
	4362	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	289.9;
	4362	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	293.4;
	4362	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.1;
	4362	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	424.6;
	4363	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.2;
	4363	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.1;
	4363	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	282.8;
	4363	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	286.2;
	4363	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.7;
	4363	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.5;
	4364	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.1;
	4364	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.9;
	4364	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	277;
	4364	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	279.9;
	4364	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.5;
	4364	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.8;
	4365	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518.9;
	4365	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161;
	4365	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	268.4;
	4365	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	271.5;
	4365	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.8;
	4365	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.2;
	4366	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.3;
	4366	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.4;
	4366	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.5;
	4366	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.9;
	4366	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.8;
	4366	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.4;
	4367	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.8;
	4367	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.8;
	4367	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.7;
	4367	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.9;
	4367	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.4;
	4367	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.5;
	4368	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.2;
	4368	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.6;
	4368	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238;
	4368	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.2;
	4368	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	4368	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.9;
	4369	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.5;
	4369	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.3;
	4369	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.1;
	4369	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.6;
	4369	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	4369	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.9;
	4370	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.5;
	4370	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118;
	4370	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.6;
	4370	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.3;
	4370	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	4370	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.9;
	4371	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.9;
	4371	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.5;
	4371	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.9;
	4371	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.8;
	4371	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.4;
	4371	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.2;
	4372	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	378.6;
	4372	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.6;
	4372	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.3;
	4372	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.1;
	4372	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	4372	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.1;
	4373	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	365.3;
	4373	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.6;
	4373	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.7;
	4373	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.8;
	4373	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	4373	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.7;
	4374	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	372.4;
	4374	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.1;
	4374	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.2;
	4374	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.4;
	4374	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.4;
	4374	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.3;
	4375	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	371.6;
	4375	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.9;
	4375	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.3;
	4375	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207;
	4375	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	4375	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.8;
	4376	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.4;
	4376	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.5;
	4376	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213;
	4376	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.2;
	4376	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	4376	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.9;
	4377	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.1;
	4377	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.6;
	4377	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.1;
	4377	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.8;
	4377	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	4377	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.6;
	4378	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.2;
	4378	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.9;
	4378	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.6;
	4378	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242;
	4378	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.9;
	4378	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.7;
	4379	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.7;
	4379	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.4;
	4379	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.5;
	4379	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.9;
	4379	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.8;
	4379	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.4;
	4380	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.7;
	4380	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.6;
	4380	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	270.5;
	4380	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	268.1;
	4380	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.5;
	4380	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.7;
	4381	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.9;
	4381	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.6;
	4381	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	282.4;
	4381	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	279;
	4381	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.5;
	4381	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.2;
	4382	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.8;
	4382	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.3;
	4382	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	290.3;
	4382	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	285.2;
	4382	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.1;
	4382	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407;
	4383	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518.9;
	4383	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.9;
	4383	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	293.3;
	4383	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	287.5;
	4383	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.2;
	4383	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.4;
	4384	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526.7;
	4384	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171;
	4384	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	289.8;
	4384	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	284.8;
	4384	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.6;
	4384	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.7;
	4385	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528.6;
	4385	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.1;
	4385	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	289.2;
	4385	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	284.1;
	4385	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.5;
	4385	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.9;
	4386	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528;
	4386	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.7;
	4386	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	280;
	4386	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	275.8;
	4386	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.5;
	4386	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.2;
	4387	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.7;
	4387	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.4;
	4387	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	280;
	4387	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	276.3;
	4387	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.2;
	4387	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.1;
	4388	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.5;
	4388	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.5;
	4388	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	275.3;
	4388	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.6;
	4388	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86;
	4388	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395;
	4389	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.5;
	4389	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.1;
	4389	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	271;
	4389	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	269.9;
	4389	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.3;
	4389	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.2;
	4390	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.3;
	4390	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.9;
	4390	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.6;
	4390	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.9;
	4390	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77;
	4390	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.7;
	4391	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.2;
	4391	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.4;
	4391	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.5;
	4391	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.6;
	4391	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.6;
	4391	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370;
	4392	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.5;
	4392	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.8;
	4392	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.1;
	4392	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.2;
	4392	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	4392	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.2;
	4393	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410;
	4393	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.1;
	4393	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.6;
	4393	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.6;
	4393	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	4393	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.1;
	4394	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.4;
	4394	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.9;
	4394	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.2;
	4394	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.4;
	4394	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	4394	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.5;
	4395	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	371.9;
	4395	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.8;
	4395	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.3;
	4395	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.9;
	4395	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61;
	4395	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.5;
	4396	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	367.1;
	4396	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.7;
	4396	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.6;
	4396	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.6;
	4396	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	4396	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.6;
	4397	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	376.4;
	4397	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120;
	4397	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.2;
	4397	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.8;
	4397	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.4;
	4397	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.1;
	4398	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.1;
	4398	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.5;
	4398	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210;
	4398	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.4;
	4398	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	4398	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.9;
	4399	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.5;
	4399	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.1;
	4399	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225;
	4399	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.2;
	4399	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	4399	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.1;
	4400	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447.5;
	4400	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.5;
	4400	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.6;
	4400	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.1;
	4400	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	4400	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.7;
	4401	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.8;
	4401	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162;
	4401	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	264.5;
	4401	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.7;
	4401	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	4401	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.9;
	4402	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.8;
	4402	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.6;
	4402	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	276.7;
	4402	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	275.4;
	4402	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	4402	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402;
	4403	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	533.4;
	4403	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.2;
	4403	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	284.7;
	4403	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	285.1;
	4403	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70;
	4403	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.4;
	4404	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.1;
	4404	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.4;
	4404	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	291.6;
	4404	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	293.4;
	4404	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.2;
	4404	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	427.7;
	4405	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	583.7;
	4405	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.2;
	4405	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	298.3;
	4405	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	300.8;
	4405	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.8;
	4405	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	438.5;
	4406	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	594.3;
	4406	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.3;
	4406	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	305.9;
	4406	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	308.3;
	4406	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.4;
	4406	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	444.8;
	4407	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	608.3;
	4407	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.5;
	4407	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	312;
	4407	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	314.6;
	4407	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.4;
	4407	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	452.7;
	4408	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	607.9;
	4408	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.2;
	4408	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	317.8;
	4408	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	321;
	4408	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76;
	4408	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	453.8;
	4409	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	606.3;
	4409	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.5;
	4409	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	312.6;
	4409	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	316.2;
	4409	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.6;
	4409	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	452.7;
	4410	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.4;
	4410	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.5;
	4410	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	312.3;
	4410	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	314.9;
	4410	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.4;
	4410	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	447.6;
	4411	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588.7;
	4411	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.9;
	4411	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	300.7;
	4411	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	301.6;
	4411	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75;
	4411	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	440.9;
	4412	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	582.5;
	4412	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.2;
	4412	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	290.2;
	4412	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	292;
	4412	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.1;
	4412	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	434.9;
	4413	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.6;
	4413	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.4;
	4413	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	283.9;
	4413	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	286;
	4413	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.7;
	4413	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423;
	4414	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.2;
	4414	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.7;
	4414	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	271.4;
	4414	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.4;
	4414	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	4414	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.6;
	4415	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.6;
	4415	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.6;
	4415	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.1;
	4415	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.9;
	4415	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.5;
	4415	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.2;
	4416	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.2;
	4416	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.4;
	4416	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.8;
	4416	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.1;
	4416	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	4416	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.7;
	4417	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.2;
	4417	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.8;
	4417	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.6;
	4417	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.5;
	4417	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	4417	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.1;
	4418	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.3;
	4418	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.4;
	4418	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.9;
	4418	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.2;
	4418	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	4418	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343;
	4419	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.2;
	4419	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.4;
	4419	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208;
	4419	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.7;
	4419	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.8;
	4419	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.9;
	4420	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.2;
	4420	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.7;
	4420	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.7;
	4420	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.5;
	4420	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.7;
	4420	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.4;
	4421	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	420;
	4421	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.6;
	4421	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.6;
	4421	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.5;
	4421	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.5;
	4421	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340;
	4422	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.4;
	4422	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.5;
	4422	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.5;
	4422	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.2;
	4422	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.6;
	4422	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.8;
	4423	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455;
	4423	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.5;
	4423	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.5;
	4423	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.7;
	4423	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55;
	4423	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.3;
	4424	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496;
	4424	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158;
	4424	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.7;
	4424	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.2;
	4424	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.7;
	4424	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.1;
	4425	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.3;
	4425	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.1;
	4425	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	271.6;
	4425	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	275.6;
	4425	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.5;
	4425	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.8;
	4426	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	557.2;
	4426	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.5;
	4426	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	283.5;
	4426	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	288;
	4426	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.9;
	4426	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.4;
	4427	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	576;
	4427	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.4;
	4427	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	294.8;
	4427	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	299.3;
	4427	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	4427	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	434.3;
	4428	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584;
	4428	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.8;
	4428	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	308.2;
	4428	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	312.1;
	4428	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.3;
	4428	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	441.3;
	4429	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.4;
	4429	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.2;
	4429	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	316.1;
	4429	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	320.2;
	4429	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	4429	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	448;
	4430	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.7;
	4430	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.6;
	4430	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	319.1;
	4430	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	324.8;
	4430	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	4430	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	451.6;
	4431	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	594.1;
	4431	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.4;
	4431	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	323;
	4431	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	329.9;
	4431	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	4431	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	451.2;
	4432	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	599.9;
	4432	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.4;
	4432	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	327.9;
	4432	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	334.9;
	4432	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	4432	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	454.7;
	4433	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	594.2;
	4433	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.7;
	4433	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	322.7;
	4433	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	331.2;
	4433	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	4433	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	451.5;
	4434	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	601;
	4434	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.5;
	4434	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	317.5;
	4434	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	324.3;
	4434	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	4434	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	452.4;
	4435	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	596.9;
	4435	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.8;
	4435	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	305.8;
	4435	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	312.5;
	4435	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	4435	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	447.2;
	4436	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	580.4;
	4436	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.1;
	4436	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	300.6;
	4436	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	305.7;
	4436	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	4436	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	437.7;
	4437	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.4;
	4437	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.1;
	4437	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	296.3;
	4437	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	299.8;
	4437	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	4437	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	428.3;
	4438	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547;
	4438	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.5;
	4438	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	284.9;
	4438	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	287.6;
	4438	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.3;
	4438	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.8;
	4439	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.5;
	4439	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.5;
	4439	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	264.1;
	4439	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265.4;
	4439	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.4;
	4439	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.4;
	4440	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.6;
	4440	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.5;
	4440	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.4;
	4440	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.4;
	4440	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	4440	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.6;
	4441	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.1;
	4441	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141;
	4441	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.2;
	4441	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238;
	4441	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.6;
	4441	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.3;
	4442	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.4;
	4442	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.1;
	4442	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.2;
	4442	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.7;
	4442	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.8;
	4442	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.9;
	4443	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.3;
	4443	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.1;
	4443	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.8;
	4443	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.1;
	4443	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.2;
	4443	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.9;
	4444	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.4;
	4444	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127;
	4444	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.1;
	4444	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.1;
	4444	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.1;
	4444	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.8;
	4445	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.1;
	4445	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.1;
	4445	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.5;
	4445	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.8;
	4445	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54;
	4445	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.5;
	4446	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	426.8;
	4446	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.7;
	4446	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.8;
	4446	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.3;
	4446	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54;
	4446	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349;
	4447	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.8;
	4447	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.6;
	4447	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.9;
	4447	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.3;
	4447	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.6;
	4447	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.5;
	4448	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.1;
	4448	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.9;
	4448	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.2;
	4448	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.9;
	4448	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.8;
	4448	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.5;
	4449	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.6;
	4449	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.6;
	4449	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	269;
	4449	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	274.7;
	4449	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.9;
	4449	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.6;
	4450	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.3;
	4450	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.2;
	4450	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	285.9;
	4450	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	292.2;
	4450	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	4450	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.8;
	4451	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	561.2;
	4451	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.1;
	4451	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	302.1;
	4451	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	308.2;
	4451	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	4451	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	429.5;
	4452	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572;
	4452	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.1;
	4452	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	313.2;
	4452	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	320.3;
	4452	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	4452	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	437.8;
	4453	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.8;
	4453	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.3;
	4453	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	327.5;
	4453	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	334.4;
	4453	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	4453	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	448;
	4454	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.4;
	4454	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.8;
	4454	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	337.9;
	4454	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	345.6;
	4454	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.2;
	4454	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	455;
	4455	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	599.8;
	4455	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.3;
	4455	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	344.1;
	4455	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	352.1;
	4455	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	4455	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	460.4;
	4456	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	601.4;
	4456	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.8;
	4456	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	345.8;
	4456	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	354.1;
	4456	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.5;
	4456	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	462.1;
	4457	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	604.3;
	4457	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.5;
	4457	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	346.8;
	4457	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	354.3;
	4457	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	4457	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	463.8;
	4458	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	596.1;
	4458	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186;
	4458	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	330.5;
	4458	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	338;
	4458	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	4458	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	456.7;
	4459	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	578;
	4459	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185;
	4459	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	321.1;
	4459	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	328.1;
	4459	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	4459	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	444.7;
	4460	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.9;
	4460	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.4;
	4460	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	312.1;
	4460	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	318.5;
	4460	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	4460	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	432.3;
	4461	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545;
	4461	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.2;
	4461	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	301.5;
	4461	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	308.1;
	4461	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	4461	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.9;
	4462	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	534.9;
	4462	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.9;
	4462	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	284;
	4462	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	290.3;
	4462	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	4462	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.3;
	4463	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.8;
	4463	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.8;
	4463	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	268.1;
	4463	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	271.9;
	4463	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	4463	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.1;
	4464	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474;
	4464	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.8;
	4464	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	259.5;
	4464	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.3;
	4464	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	4464	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379;
	4465	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.7;
	4465	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.8;
	4465	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.4;
	4465	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.8;
	4465	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	4465	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.5;
	4466	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.7;
	4466	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.4;
	4466	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.3;
	4466	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.3;
	4466	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.4;
	4466	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.1;
	4467	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	396.6;
	4467	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.1;
	4467	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.3;
	4467	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.9;
	4467	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.8;
	4467	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.4;
	4468	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391.2;
	4468	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.4;
	4468	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.5;
	4468	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220;
	4468	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	4468	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.7;
	4469	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.5;
	4469	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.7;
	4469	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.8;
	4469	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.5;
	4469	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	4469	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.9;
	4470	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417.7;
	4470	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.7;
	4470	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.8;
	4470	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.3;
	4470	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.4;
	4470	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.4;
	4471	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.4;
	4471	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.8;
	4471	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.3;
	4471	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.6;
	4471	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	4471	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.2;
	4472	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.8;
	4472	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156;
	4472	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.4;
	4472	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.6;
	4472	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	4472	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.2;
	4473	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.3;
	4473	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.2;
	4473	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	275.2;
	4473	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	283.1;
	4473	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	4473	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.4;
	4474	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.5;
	4474	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.3;
	4474	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	291.4;
	4474	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	299.5;
	4474	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72;
	4474	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.2;
	4475	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.1;
	4475	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.2;
	4475	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	305.2;
	4475	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	314.7;
	4475	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.9;
	4475	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.5;
	4476	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.7;
	4476	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.3;
	4476	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	318.3;
	4476	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	327.9;
	4476	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.8;
	4476	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.9;
	4477	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.3;
	4477	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.1;
	4477	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	327.6;
	4477	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	337.9;
	4477	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.8;
	4477	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	430.4;
	4478	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.5;
	4478	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.5;
	4478	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	338.4;
	4478	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	348.6;
	4478	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.4;
	4478	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	443;
	4479	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	580.8;
	4479	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.8;
	4479	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	344.2;
	4479	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	353;
	4479	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.7;
	4479	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	453.8;
	4480	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589.5;
	4480	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.1;
	4480	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	346.1;
	4480	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	354.5;
	4480	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.7;
	4480	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	458;
	4481	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577.6;
	4481	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191;
	4481	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	341.1;
	4481	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	348.8;
	4481	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.2;
	4481	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	452.8;
	4482	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.5;
	4482	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.4;
	4482	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	335.1;
	4482	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	341.1;
	4482	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.2;
	4482	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	447.9;
	4483	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549.1;
	4483	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.7;
	4483	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	323.9;
	4483	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	329;
	4483	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.1;
	4483	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	434.5;
	4484	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.3;
	4484	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.5;
	4484	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	306.6;
	4484	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	313.4;
	4484	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.6;
	4484	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.5;
	4485	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514;
	4485	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.3;
	4485	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	295.5;
	4485	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	302.8;
	4485	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90;
	4485	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.2;
	4486	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.7;
	4486	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.3;
	4486	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	280.3;
	4486	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	285.9;
	4486	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.7;
	4486	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.3;
	4487	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.2;
	4487	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.7;
	4487	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.8;
	4487	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	266.6;
	4487	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.4;
	4487	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.9;
	4488	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.1;
	4488	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153;
	4488	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.8;
	4488	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.9;
	4488	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.3;
	4488	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.7;
	4489	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.7;
	4489	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.4;
	4489	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.8;
	4489	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.3;
	4489	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.1;
	4489	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.8;
	4490	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	389.9;
	4490	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.4;
	4490	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.6;
	4490	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.9;
	4490	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.9;
	4490	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.4;
	4491	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	372.5;
	4491	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.7;
	4491	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.8;
	4491	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.7;
	4491	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.1;
	4491	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.5;
	4492	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	364.9;
	4492	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.9;
	4492	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.7;
	4492	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.1;
	4492	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	4492	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.8;
	4493	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	384.8;
	4493	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.1;
	4493	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.3;
	4493	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.1;
	4493	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	4493	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.2;
	4494	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.1;
	4494	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.2;
	4494	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.7;
	4494	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.8;
	4494	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	4494	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.7;
	4495	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.5;
	4495	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.9;
	4495	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.1;
	4495	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.6;
	4495	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	4495	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.3;
	4496	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446;
	4496	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.4;
	4496	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243;
	4496	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252;
	4496	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	4496	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.3;
	4497	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.6;
	4497	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.9;
	4497	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.1;
	4497	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	269;
	4497	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.4;
	4497	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.3;
	4498	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.3;
	4498	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.5;
	4498	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	273.1;
	4498	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	281.6;
	4498	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.1;
	4498	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.5;
	4499	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.3;
	4499	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.4;
	4499	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	286.8;
	4499	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	295.5;
	4499	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.9;
	4499	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.8;
	4500	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	540.4;
	4500	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.2;
	4500	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	298.6;
	4500	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	308;
	4500	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.9;
	4500	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.5;
	4501	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	554.1;
	4501	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.5;
	4501	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	305.7;
	4501	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	314.5;
	4501	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.6;
	4501	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	430.2;
	4502	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	576.5;
	4502	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.4;
	4502	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	310.1;
	4502	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	318.2;
	4502	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.5;
	4502	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	440.8;
	4503	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.8;
	4503	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.8;
	4503	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	315.2;
	4503	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	323.7;
	4503	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	97.5;
	4503	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	439.6;
	4504	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	546.8;
	4504	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.6;
	4504	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	316.1;
	4504	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	323.5;
	4504	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	98.5;
	4504	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	430.4;
	4505	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	587.9;
	4505	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.7;
	4505	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	314;
	4505	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	319;
	4505	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	99;
	4505	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	446.1;
	4506	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	573;
	4506	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.4;
	4506	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	307.6;
	4506	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	307.9;
	4506	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	99.5;
	4506	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	437.3;
	4507	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	568;
	4507	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.4;
	4507	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	296.7;
	4507	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	297.2;
	4507	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	97.4;
	4507	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	431.7;
	4508	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.4;
	4508	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.7;
	4508	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	286.5;
	4508	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	287.3;
	4508	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96.7;
	4508	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.5;
	4509	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522.5;
	4509	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.9;
	4509	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	279;
	4509	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	280.1;
	4509	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.8;
	4509	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406;
	4510	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.3;
	4510	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.2;
	4510	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	270.2;
	4510	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	270;
	4510	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.9;
	4510	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.6;
	4511	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.8;
	4511	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.8;
	4511	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.9;
	4511	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.4;
	4511	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.6;
	4511	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.9;
	4512	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.4;
	4512	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.3;
	4512	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.2;
	4512	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.9;
	4512	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.9;
	4512	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.2;
	4513	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.3;
	4513	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.3;
	4513	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.3;
	4513	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.6;
	4513	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.3;
	4513	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.9;
	4514	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.8;
	4514	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.5;
	4514	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.7;
	4514	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.9;
	4514	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72;
	4514	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.3;
	4515	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	389.6;
	4515	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.5;
	4515	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.8;
	4515	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.5;
	4515	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	4515	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.4;
	4516	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	384.7;
	4516	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.2;
	4516	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.3;
	4516	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.3;
	4516	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	4516	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.9;
	4517	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	389.8;
	4517	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.5;
	4517	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.7;
	4517	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.8;
	4517	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	4517	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.1;
	4518	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.5;
	4518	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.7;
	4518	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.8;
	4518	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.6;
	4518	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	4518	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.2;
	4519	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	406.5;
	4519	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.4;
	4519	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.9;
	4519	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.2;
	4519	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.2;
	4519	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.5;
	4520	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.1;
	4520	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.4;
	4520	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.8;
	4520	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.9;
	4520	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.4;
	4520	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.6;
	4521	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	444.7;
	4521	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.5;
	4521	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.3;
	4521	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.7;
	4521	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.4;
	4521	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.9;
	4522	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.5;
	4522	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.5;
	4522	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255;
	4522	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.5;
	4522	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.3;
	4522	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.8;
	4523	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525;
	4523	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.8;
	4523	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	266.1;
	4523	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.1;
	4523	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.7;
	4523	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.7;
	4524	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.7;
	4524	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169;
	4524	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	275.5;
	4524	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	281.3;
	4524	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.2;
	4524	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.3;
	4525	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.2;
	4525	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.8;
	4525	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	284.4;
	4525	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	290.5;
	4525	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.9;
	4525	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418;
	4526	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.1;
	4526	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.9;
	4526	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	285.4;
	4526	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	292;
	4526	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	97.4;
	4526	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.7;
	4527	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.3;
	4527	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.7;
	4527	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	284.5;
	4527	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	292.1;
	4527	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	99;
	4527	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.5;
	4528	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.3;
	4528	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.3;
	4528	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	285.2;
	4528	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	292.9;
	4528	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	100.7;
	4528	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.1;
	4529	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	567.1;
	4529	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.7;
	4529	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	286.5;
	4529	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	293.1;
	4529	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	100.8;
	4529	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	427.2;
	4530	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.5;
	4530	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.3;
	4530	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	284.8;
	4530	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	291.6;
	4530	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	100.9;
	4530	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.5;
	4531	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	524.2;
	4531	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163;
	4531	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	280;
	4531	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	286.2;
	4531	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	99.5;
	4531	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.9;
	4532	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.3;
	4532	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161;
	4532	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	274.5;
	4532	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	279.6;
	4532	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	97.4;
	4532	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.5;
	4533	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.8;
	4533	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.3;
	4533	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	269.4;
	4533	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.9;
	4533	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.6;
	4533	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.2;
	4534	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.4;
	4534	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.3;
	4534	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.6;
	4534	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	263.7;
	4534	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91;
	4534	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.7;
	4535	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.3;
	4535	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.2;
	4535	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246;
	4535	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.4;
	4535	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.1;
	4535	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.1;
	4536	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421;
	4536	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.9;
	4536	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.2;
	4536	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.9;
	4536	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.1;
	4536	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.5;
	4537	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394.6;
	4537	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.9;
	4537	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223;
	4537	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.8;
	4537	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.2;
	4537	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.6;
	4538	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	367.8;
	4538	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.5;
	4538	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.9;
	4538	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.6;
	4538	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.9;
	4538	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322;
	4539	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	367.2;
	4539	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.5;
	4539	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.9;
	4539	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.2;
	4539	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.3;
	4539	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.5;
	4540	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	361;
	4540	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116;
	4540	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.5;
	4540	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.8;
	4540	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	4540	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.8;
	4541	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	360.9;
	4541	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.2;
	4541	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.7;
	4541	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.9;
	4541	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	4541	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.5;
	4542	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	364.7;
	4542	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.3;
	4542	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204;
	4542	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.3;
	4542	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	4542	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.5;
	4543	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	372.9;
	4543	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.2;
	4543	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.3;
	4543	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.1;
	4543	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	4543	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.7;
	4544	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	385.4;
	4544	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.6;
	4544	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.9;
	4544	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.7;
	4544	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.4;
	4544	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.1;
	4545	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.1;
	4545	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.2;
	4545	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.4;
	4545	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.6;
	4545	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.5;
	4545	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.4;
	4546	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.4;
	4546	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.1;
	4546	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.9;
	4546	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248;
	4546	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.1;
	4546	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.2;
	4547	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.9;
	4547	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.8;
	4547	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.8;
	4547	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	263.7;
	4547	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.6;
	4547	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.9;
	4548	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.2;
	4548	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.1;
	4548	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	272.7;
	4548	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	274.2;
	4548	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.5;
	4548	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.4;
	4549	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.4;
	4549	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.3;
	4549	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	283.7;
	4549	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	285.9;
	4549	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.8;
	4549	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.2;
	4550	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.2;
	4550	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.7;
	4550	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	293.4;
	4550	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	296;
	4550	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	97;
	4550	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.8;
	4551	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.7;
	4551	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.3;
	4551	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	303.5;
	4551	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	306.3;
	4551	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	98.9;
	4551	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.4;
	4552	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.7;
	4552	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173;
	4552	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	311.4;
	4552	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	314.3;
	4552	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	99.9;
	4552	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.1;
	4553	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.3;
	4553	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.9;
	4553	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	311.5;
	4553	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	313.9;
	4553	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	99.8;
	4553	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407;
	4554	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.3;
	4554	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.4;
	4554	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	310;
	4554	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	313;
	4554	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	97.4;
	4554	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.5;
	4555	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.7;
	4555	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169;
	4555	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	303.5;
	4555	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	306.1;
	4555	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.4;
	4555	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.5;
	4556	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.4;
	4556	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.1;
	4556	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	296;
	4556	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	298.7;
	4556	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.7;
	4556	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.3;
	4557	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.9;
	4557	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.6;
	4557	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	291.9;
	4557	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	295;
	4557	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.7;
	4557	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.5;
	4558	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.3;
	4558	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160;
	4558	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	277.4;
	4558	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	280.5;
	4558	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.9;
	4558	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.6;
	4559	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.2;
	4559	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.6;
	4559	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.3;
	4559	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.4;
	4559	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.8;
	4559	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.5;
	4560	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.7;
	4560	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.5;
	4560	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.6;
	4560	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.5;
	4560	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.7;
	4560	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.6;
	4561	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.2;
	4561	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.6;
	4561	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230;
	4561	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232;
	4561	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.7;
	4561	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.8;
	4562	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	371;
	4562	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.9;
	4562	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.8;
	4562	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.1;
	4562	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.7;
	4562	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.6;
	4563	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	360.4;
	4563	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.8;
	4563	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.3;
	4563	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.7;
	4563	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	4563	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.1;
	4564	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	363.4;
	4564	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.4;
	4564	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.6;
	4564	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.3;
	4564	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	4564	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.9;
	4565	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	370.4;
	4565	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.8;
	4565	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.9;
	4565	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.6;
	4565	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	4565	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.1;
	4566	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	386.8;
	4566	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.3;
	4566	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.3;
	4566	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.4;
	4566	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	4566	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.7;
	4567	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.2;
	4567	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.2;
	4567	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.9;
	4567	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.5;
	4567	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	4567	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.9;
	4568	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.1;
	4568	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.9;
	4568	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.8;
	4568	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.8;
	4568	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	4568	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.1;
	4569	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.6;
	4569	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.8;
	4569	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	269.6;
	4569	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	276.4;
	4569	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	4569	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.5;
	4570	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518.4;
	4570	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.6;
	4570	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	282.2;
	4570	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	289.9;
	4570	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.5;
	4570	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.4;
	4571	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541;
	4571	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.1;
	4571	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	298.1;
	4571	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	306.8;
	4571	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.1;
	4571	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421;
	4572	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.4;
	4572	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.4;
	4572	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	311.1;
	4572	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	321.1;
	4572	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.3;
	4572	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	431.2;
	4573	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	576.7;
	4573	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182;
	4573	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	322.9;
	4573	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	334.1;
	4573	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.4;
	4573	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	443.8;
	4574	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.1;
	4574	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.9;
	4574	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	331.6;
	4574	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	343.3;
	4574	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.5;
	4574	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	455.7;
	4575	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	603;
	4575	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.3;
	4575	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	340.1;
	4575	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	350.6;
	4575	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.9;
	4575	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	460.6;
	4576	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	615;
	4576	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.2;
	4576	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	347.4;
	4576	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	357.3;
	4576	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.4;
	4576	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	467.9;
	4577	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	595.7;
	4577	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.5;
	4577	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	347.2;
	4577	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	357.1;
	4577	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.7;
	4577	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	460.8;
	4578	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	591.3;
	4578	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.7;
	4578	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	339.5;
	4578	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	348.7;
	4578	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.5;
	4578	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	457.1;
	4579	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	581;
	4579	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.5;
	4579	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	324.2;
	4579	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	332.1;
	4579	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.7;
	4579	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	448.3;
	4580	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564.4;
	4580	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.5;
	4580	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	314.3;
	4580	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	323.5;
	4580	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85;
	4580	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	437.3;
	4581	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	557.3;
	4581	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.6;
	4581	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	306.3;
	4581	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	315.7;
	4581	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.7;
	4581	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	430.8;
	4582	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.9;
	4582	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169;
	4582	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	295.2;
	4582	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	303.4;
	4582	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.8;
	4582	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.8;
	4583	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.6;
	4583	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.1;
	4583	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	270.8;
	4583	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	277.7;
	4583	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.9;
	4583	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.5;
	4584	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.8;
	4584	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.3;
	4584	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.7;
	4584	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	268.3;
	4584	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.6;
	4584	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.3;
	4585	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.2;
	4585	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.7;
	4585	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.6;
	4585	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.7;
	4585	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.1;
	4585	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.3;
	4586	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	427.6;
	4586	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.3;
	4586	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.9;
	4586	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.8;
	4586	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	4586	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.7;
	4587	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.5;
	4587	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.7;
	4587	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.3;
	4587	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.9;
	4587	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	4587	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.6;
	4588	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.4;
	4588	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.5;
	4588	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.1;
	4588	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223;
	4588	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	4588	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.3;
	4589	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.6;
	4589	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.2;
	4589	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.6;
	4589	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.5;
	4589	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60;
	4589	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.8;
	4590	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.7;
	4590	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.5;
	4590	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.6;
	4590	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.1;
	4590	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.1;
	4590	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.2;
	4591	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.7;
	4591	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.6;
	4591	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.3;
	4591	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.8;
	4591	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59;
	4591	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.2;
	4592	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.7;
	4592	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.9;
	4592	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.6;
	4592	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.6;
	4592	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.7;
	4592	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.6;
	4593	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525.1;
	4593	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.8;
	4593	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	280;
	4593	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	290.5;
	4593	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	4593	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.6;
	4594	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.7;
	4594	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.9;
	4594	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	297.3;
	4594	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	307.9;
	4594	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	4594	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	424.4;
	4595	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	573;
	4595	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179;
	4595	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	310.9;
	4595	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	322;
	4595	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	4595	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	438.7;
	4596	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	595.7;
	4596	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.9;
	4596	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	326;
	4596	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	336.6;
	4596	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.3;
	4596	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	452.5;
	4597	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	613.3;
	4597	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.3;
	4597	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	338.1;
	4597	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	348.9;
	4597	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.2;
	4597	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	464.4;
	4598	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	613.3;
	4598	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.1;
	4598	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	349.5;
	4598	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	360.1;
	4598	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.4;
	4598	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	468.2;
	4599	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	626.5;
	4599	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.8;
	4599	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	359.6;
	4599	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	370.2;
	4599	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.1;
	4599	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	477.1;
	4600	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	626.9;
	4600	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.9;
	4600	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	364.1;
	4600	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	374.5;
	4600	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.1;
	4600	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	478.8;
	4601	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	627.2;
	4601	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.8;
	4601	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	364.8;
	4601	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	374.2;
	4601	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.5;
	4601	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	479.7;
	4602	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	626.5;
	4602	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.8;
	4602	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	354.2;
	4602	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	362.5;
	4602	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.5;
	4602	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	476.8;
	4603	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	620;
	4603	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.1;
	4603	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	345;
	4603	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	352.7;
	4603	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82;
	4603	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	470.5;
	4604	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	603.2;
	4604	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180;
	4604	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	331.8;
	4604	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	339;
	4604	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.6;
	4604	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	459.1;
	4605	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586.3;
	4605	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.4;
	4605	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	324.5;
	4605	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	330.6;
	4605	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.6;
	4605	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	448.7;
	4606	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.9;
	4606	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.6;
	4606	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	305.5;
	4606	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	309.9;
	4606	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.3;
	4606	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	433.7;
	4607	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.7;
	4607	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.3;
	4607	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	282.1;
	4607	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	285.5;
	4607	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.8;
	4607	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.5;
	4608	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.2;
	4608	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.1;
	4608	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	268;
	4608	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	273.3;
	4608	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	4608	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.7;
	4609	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.2;
	4609	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.2;
	4609	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.5;
	4609	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.3;
	4609	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	4609	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.6;
	4610	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423;
	4610	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.8;
	4610	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.3;
	4610	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.9;
	4610	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	4610	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.7;
	4611	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.6;
	4611	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.8;
	4611	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.7;
	4611	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.8;
	4611	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60;
	4611	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.1;
	4612	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404;
	4612	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.7;
	4612	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.5;
	4612	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.5;
	4612	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59;
	4612	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.7;
	4613	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.6;
	4613	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.3;
	4613	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.2;
	4613	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.3;
	4613	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.5;
	4613	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.9;
	4614	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.2;
	4614	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.3;
	4614	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.7;
	4614	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.4;
	4614	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.7;
	4614	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.9;
	4615	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.6;
	4615	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.3;
	4615	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249;
	4615	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.7;
	4615	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.8;
	4615	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.3;
	4616	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.9;
	4616	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.1;
	4616	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	268.5;
	4616	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	274.2;
	4616	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	4616	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.8;
	4617	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513;
	4617	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.3;
	4617	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	280.2;
	4617	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	286.1;
	4617	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	4617	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.7;
	4618	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.4;
	4618	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.5;
	4618	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	307.8;
	4618	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	310.3;
	4618	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.1;
	4618	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.2;
	4619	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	555.3;
	4619	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.2;
	4619	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	318.4;
	4619	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	319.8;
	4619	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.1;
	4619	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	434.1;
	4620	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566.1;
	4620	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.7;
	4620	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	332.5;
	4620	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	331.7;
	4620	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.9;
	4620	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	442.9;
	4621	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579;
	4621	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.6;
	4621	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	343.7;
	4621	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	342.6;
	4621	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81;
	4621	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	452.8;
	4622	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588.6;
	4622	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.6;
	4622	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	356.6;
	4622	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	355.4;
	4622	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.7;
	4622	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	460.9;
	4623	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	599.8;
	4623	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.8;
	4623	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	360.6;
	4623	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	361.8;
	4623	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.2;
	4623	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	467.7;
	4624	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	602.3;
	4624	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.9;
	4624	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	364.4;
	4624	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	365.6;
	4624	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.3;
	4624	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	470;
	4625	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	602.3;
	4625	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.5;
	4625	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	357.2;
	4625	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	359.5;
	4625	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.4;
	4625	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	468.8;
	4626	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	594.1;
	4626	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.6;
	4626	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	347.3;
	4626	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	348.7;
	4626	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89;
	4626	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	461.5;
	4627	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.1;
	4627	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.5;
	4627	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	333.5;
	4627	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	334.2;
	4627	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.5;
	4627	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	446.7;
	4628	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.8;
	4628	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.2;
	4628	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	326.1;
	4628	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	323.4;
	4628	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.2;
	4628	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	435.3;
	4629	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.5;
	4629	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186;
	4629	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	313.8;
	4629	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	312.3;
	4629	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.3;
	4629	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	426.8;
	4630	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522.1;
	4630	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.4;
	4630	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	297.1;
	4630	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	294.1;
	4630	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.8;
	4630	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.6;
	4631	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.4;
	4631	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164;
	4631	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	275.4;
	4631	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.8;
	4631	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.2;
	4631	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.5;
	4632	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.3;
	4632	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.2;
	4632	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	267.1;
	4632	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	266.1;
	4632	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	4632	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.1;
	4633	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.8;
	4633	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.6;
	4633	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.5;
	4633	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.1;
	4633	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	4633	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.9;
	4634	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.1;
	4634	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.5;
	4634	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.3;
	4634	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.8;
	4634	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	4634	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.1;
	4635	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391.3;
	4635	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.1;
	4635	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223;
	4635	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222;
	4635	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	4635	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.3;
	4636	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	380;
	4636	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.1;
	4636	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.2;
	4636	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.4;
	4636	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	4636	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.9;
	4637	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393;
	4637	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.5;
	4637	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.5;
	4637	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222;
	4637	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	4637	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.2;
	4638	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.1;
	4638	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.3;
	4638	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.6;
	4638	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.5;
	4638	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58;
	4638	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351;
	4639	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.6;
	4639	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.8;
	4639	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.3;
	4639	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.1;
	4639	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59;
	4639	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.9;
	4640	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.4;
	4640	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.4;
	4640	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.5;
	4640	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	263.4;
	4640	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	4640	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.6;
	4641	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509;
	4641	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.8;
	4641	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	286.4;
	4641	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	286.4;
	4641	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	4641	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.3;
	4642	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.1;
	4642	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.3;
	4642	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	290.9;
	4642	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	291.3;
	4642	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.7;
	4642	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420.8;
	4643	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562;
	4643	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.9;
	4643	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	303.9;
	4643	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	303.8;
	4643	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.4;
	4643	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	432.7;
	4644	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.1;
	4644	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185;
	4644	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	313.1;
	4644	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	312.9;
	4644	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.7;
	4644	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	439.6;
	4645	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579.8;
	4645	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.9;
	4645	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	322.8;
	4645	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	322.9;
	4645	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.4;
	4645	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	446.4;
	4646	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	593;
	4646	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.4;
	4646	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	330.4;
	4646	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	330.4;
	4646	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.9;
	4646	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	454.3;
	4647	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	596.5;
	4647	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.6;
	4647	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	339.7;
	4647	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	338.9;
	4647	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.5;
	4647	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	458.1;
	4648	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	601.1;
	4648	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.5;
	4648	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	341.8;
	4648	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	341.4;
	4648	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.2;
	4648	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	461.3;
	4649	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	595.3;
	4649	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	200.1;
	4649	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	339.3;
	4649	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	338.5;
	4649	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.1;
	4649	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	458.5;
	4650	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588.6;
	4650	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.9;
	4650	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	333.9;
	4650	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	331;
	4650	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.2;
	4650	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	453.3;
	4651	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	580.2;
	4651	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.2;
	4651	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	315.5;
	4651	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	313.4;
	4651	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.9;
	4651	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	444.6;
	4652	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	573;
	4652	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.4;
	4652	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	303.4;
	4652	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	302.6;
	4652	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.5;
	4652	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	436.5;
	4653	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.6;
	4653	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.3;
	4653	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	296.2;
	4653	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	295.1;
	4653	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.9;
	4653	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.5;
	4654	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537.3;
	4654	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.3;
	4654	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	281.5;
	4654	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	280.2;
	4654	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.6;
	4654	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.6;
	4655	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.8;
	4655	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.3;
	4655	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.9;
	4655	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.1;
	4655	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.3;
	4655	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.5;
	4656	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.7;
	4656	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157;
	4656	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.3;
	4656	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.6;
	4656	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.1;
	4656	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.4;
	4657	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.8;
	4657	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.2;
	4657	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.5;
	4657	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.1;
	4657	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69;
	4657	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.4;
	4658	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.6;
	4658	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.5;
	4658	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.6;
	4658	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220;
	4658	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	4658	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.1;
	4659	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	400.4;
	4659	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.7;
	4659	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.3;
	4659	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.5;
	4659	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	4659	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.9;
	4660	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.2;
	4660	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.5;
	4660	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.3;
	4660	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.9;
	4660	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	4660	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.1;
	4661	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.6;
	4661	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.9;
	4661	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.8;
	4661	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.7;
	4661	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	4661	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.9;
	4662	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417.4;
	4662	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.8;
	4662	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223;
	4662	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.2;
	4662	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	4662	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.1;
	4663	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.1;
	4663	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.6;
	4663	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.9;
	4663	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.9;
	4663	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	4663	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.1;
	4664	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.7;
	4664	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.3;
	4664	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.4;
	4664	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.4;
	4664	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	4664	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.2;
	4665	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.2;
	4665	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.4;
	4665	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	275.3;
	4665	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	277.1;
	4665	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69;
	4665	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404;
	4666	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.8;
	4666	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.6;
	4666	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	286.9;
	4666	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	289.2;
	4666	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76;
	4666	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.9;
	4667	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	557.6;
	4667	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178;
	4667	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	300.4;
	4667	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	302.6;
	4667	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.5;
	4667	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	429.4;
	4668	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	576.7;
	4668	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.6;
	4668	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	314;
	4668	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	316.2;
	4668	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.1;
	4668	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	441.3;
	4669	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	596.8;
	4669	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.2;
	4669	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	324.2;
	4669	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	326.3;
	4669	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.9;
	4669	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	453.8;
	4670	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	611.2;
	4670	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.2;
	4670	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	330.2;
	4670	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	332.6;
	4670	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.6;
	4670	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	461.9;
	4671	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	607.3;
	4671	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.4;
	4671	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	334.7;
	4671	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	337.2;
	4671	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.1;
	4671	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	461.5;
	4672	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	603.1;
	4672	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.3;
	4672	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	332.2;
	4672	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	335.7;
	4672	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.3;
	4672	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	459.4;
	4673	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	604;
	4673	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.7;
	4673	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	331.1;
	4673	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	333.8;
	4673	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.8;
	4673	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	459;
	4674	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	602.5;
	4674	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.1;
	4674	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	314.3;
	4674	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	318.3;
	4674	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87;
	4674	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	453.8;
	4675	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	590.1;
	4675	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.9;
	4675	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	304.6;
	4675	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	307.2;
	4675	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.7;
	4675	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	443.7;
	4676	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	569.1;
	4676	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.4;
	4676	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	294.8;
	4676	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	298;
	4676	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.6;
	4676	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	431.1;
	4677	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	555.2;
	4677	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.3;
	4677	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	288.1;
	4677	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	290.3;
	4677	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.6;
	4677	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.1;
	4678	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.5;
	4678	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.1;
	4678	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	272.2;
	4678	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	274.3;
	4678	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.5;
	4678	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.6;
	4679	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.2;
	4679	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.2;
	4679	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256;
	4679	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.1;
	4679	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.4;
	4679	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.3;
	4680	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.7;
	4680	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151;
	4680	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.4;
	4680	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.8;
	4680	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.5;
	4680	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.1;
	4681	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447.8;
	4681	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.4;
	4681	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225;
	4681	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.9;
	4681	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.4;
	4681	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.5;
	4682	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.5;
	4682	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.1;
	4682	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217;
	4682	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.5;
	4682	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	4682	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.4;
	4683	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394.8;
	4683	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.3;
	4683	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.3;
	4683	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.1;
	4683	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	4683	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.3;
	4684	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	387.3;
	4684	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.9;
	4684	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.9;
	4684	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.8;
	4684	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	4684	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.3;
	4685	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	386.5;
	4685	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.2;
	4685	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.1;
	4685	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.9;
	4685	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	4685	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.3;
	4686	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394.3;
	4686	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.9;
	4686	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.5;
	4686	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.5;
	4686	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	4686	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.9;
	4687	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.4;
	4687	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.8;
	4687	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.1;
	4687	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220;
	4687	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	4687	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.9;
	4688	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.6;
	4688	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.9;
	4688	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.3;
	4688	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.6;
	4688	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	4688	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.6;
	4689	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.7;
	4689	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.1;
	4689	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.5;
	4689	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.1;
	4689	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.6;
	4689	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.7;
	4690	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.8;
	4690	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.9;
	4690	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.2;
	4690	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.9;
	4690	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.6;
	4690	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.9;
	4691	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513;
	4691	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.5;
	4691	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261;
	4691	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265.9;
	4691	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.6;
	4691	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.3;
	4692	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538;
	4692	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.8;
	4692	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	268.3;
	4692	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	274.7;
	4692	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.2;
	4692	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.4;
	4693	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.5;
	4693	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.1;
	4693	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	276.5;
	4693	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	283.7;
	4693	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.3;
	4693	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420.8;
	4694	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559.5;
	4694	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175;
	4694	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	284.1;
	4694	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	291.9;
	4694	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.1;
	4694	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.6;
	4695	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564.8;
	4695	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.6;
	4695	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	289.3;
	4695	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	296.6;
	4695	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	98.3;
	4695	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	427.6;
	4696	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566.1;
	4696	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.8;
	4696	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	291.9;
	4696	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	299.6;
	4696	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	100.9;
	4696	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	428.4;
	4697	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	571.4;
	4697	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.5;
	4697	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	291;
	4697	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	298.4;
	4697	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	102.2;
	4697	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	430.7;
	4698	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	571.1;
	4698	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.1;
	4698	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	286;
	4698	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	293.1;
	4698	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	101.7;
	4698	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	428.8;
	4699	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	555.5;
	4699	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.2;
	4699	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	282.3;
	4699	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	288.9;
	4699	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	100.3;
	4699	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420;
	4700	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.8;
	4700	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.3;
	4700	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	275.4;
	4700	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	281.2;
	4700	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	98;
	4700	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.2;
	4701	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.4;
	4701	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.2;
	4701	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	269.8;
	4701	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	274.9;
	4701	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.8;
	4701	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.2;
	4702	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.3;
	4702	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.5;
	4702	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	259.7;
	4702	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.1;
	4702	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.8;
	4702	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.9;
	4703	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.9;
	4703	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.6;
	4703	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.2;
	4703	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.4;
	4703	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.1;
	4703	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.4;
	4704	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.9;
	4704	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.8;
	4704	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.8;
	4704	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.1;
	4704	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.9;
	4704	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.7;
	4705	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409;
	4705	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134;
	4705	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.5;
	4705	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.8;
	4705	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.8;
	4705	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.2;
	4706	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	374.4;
	4706	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.5;
	4706	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.3;
	4706	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.5;
	4706	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.1;
	4706	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.3;
	4707	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	362.6;
	4707	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.6;
	4707	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.7;
	4707	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211;
	4707	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72;
	4707	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.9;
	4708	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	358.3;
	4708	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.9;
	4708	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.4;
	4708	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.6;
	4708	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.5;
	4708	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.2;
	4709	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	367.9;
	4709	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.2;
	4709	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.7;
	4709	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.6;
	4709	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.9;
	4709	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.1;
	4710	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	374.7;
	4710	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.2;
	4710	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.6;
	4710	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.4;
	4710	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	4710	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.7;
	4711	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379.1;
	4711	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.8;
	4711	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.5;
	4711	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.9;
	4711	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.8;
	4711	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.1;
	4712	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	400;
	4712	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.1;
	4712	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220;
	4712	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.7;
	4712	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.3;
	4712	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.6;
	4713	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.9;
	4713	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.4;
	4713	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.9;
	4713	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.8;
	4713	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.4;
	4713	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355;
	4714	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.9;
	4714	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.8;
	4714	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.9;
	4714	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.5;
	4714	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.5;
	4714	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.7;
	4715	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.9;
	4715	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.1;
	4715	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.7;
	4715	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	268.3;
	4715	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.5;
	4715	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.2;
	4716	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.6;
	4716	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.1;
	4716	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	277.4;
	4716	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	282.9;
	4716	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.8;
	4716	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.7;
	4717	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.5;
	4717	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.3;
	4717	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	289.8;
	4717	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	295.5;
	4717	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.6;
	4717	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.9;
	4718	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.7;
	4718	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.9;
	4718	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	299.2;
	4718	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	305.6;
	4718	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96.7;
	4718	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.9;
	4719	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.7;
	4719	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.2;
	4719	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	308.7;
	4719	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	315.6;
	4719	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	97.4;
	4719	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	428.6;
	4720	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564.5;
	4720	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.5;
	4720	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	311.1;
	4720	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	319;
	4720	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	98.4;
	4720	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	435.1;
	4721	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.8;
	4721	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.4;
	4721	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	314;
	4721	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	321.4;
	4721	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	99.4;
	4721	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	435.4;
	4722	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.8;
	4722	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.1;
	4722	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	308.4;
	4722	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	315.6;
	4722	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	98.2;
	4722	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	434.8;
	4723	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562;
	4723	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.6;
	4723	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	306.3;
	4723	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	312.6;
	4723	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	97.5;
	4723	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	431.9;
	4724	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.1;
	4724	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.2;
	4724	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	303.1;
	4724	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	308.4;
	4724	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.3;
	4724	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.8;
	4725	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.3;
	4725	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.6;
	4725	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	297.2;
	4725	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	303.1;
	4725	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.5;
	4725	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.1;
	4726	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.1;
	4726	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.1;
	4726	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	285.6;
	4726	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	289.9;
	4726	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.5;
	4726	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.1;
	4727	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.5;
	4727	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.2;
	4727	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	265.9;
	4727	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	269.1;
	4727	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.3;
	4727	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.3;
	4728	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.8;
	4728	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.1;
	4728	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.2;
	4728	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.6;
	4728	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.1;
	4728	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375;
	4729	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436;
	4729	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.1;
	4729	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.4;
	4729	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.8;
	4729	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75;
	4729	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.7;
	4730	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.2;
	4730	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.8;
	4730	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.5;
	4730	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.9;
	4730	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.1;
	4730	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.4;
	4731	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.8;
	4731	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.6;
	4731	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.9;
	4731	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.1;
	4731	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.2;
	4731	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.4;
	4732	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	382.3;
	4732	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.3;
	4732	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.7;
	4732	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.9;
	4732	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	4732	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.6;
	4733	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	385.9;
	4733	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.8;
	4733	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.5;
	4733	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.4;
	4733	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	4733	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330;
	4734	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.8;
	4734	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.7;
	4734	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.5;
	4734	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.6;
	4734	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	4734	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.1;
	4735	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.5;
	4735	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.8;
	4735	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.8;
	4735	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.2;
	4735	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	4735	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.4;
	4736	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.3;
	4736	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.4;
	4736	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	263;
	4736	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	269.7;
	4736	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	4736	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383;
	4737	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.7;
	4737	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.6;
	4737	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	285.5;
	4737	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	292.6;
	4737	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.9;
	4737	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.2;
	4738	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.2;
	4738	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.3;
	4738	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	298.8;
	4738	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	306.8;
	4738	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.2;
	4738	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	431;
	4739	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586;
	4739	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.8;
	4739	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	314.5;
	4739	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	323.7;
	4739	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.7;
	4739	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	444.6;
	4740	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	605.8;
	4740	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.5;
	4740	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	328;
	4740	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	338;
	4740	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.3;
	4740	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	457;
	4741	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	621.6;
	4741	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.4;
	4741	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	339.5;
	4741	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	350.2;
	4741	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.1;
	4741	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	468.1;
	4742	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	640.1;
	4742	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.6;
	4742	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	347.7;
	4742	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	358.3;
	4742	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.6;
	4742	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	478.6;
	4743	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	654.7;
	4743	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	199.4;
	4743	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	352.2;
	4743	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	362.2;
	4743	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.5;
	4743	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	486.5;
	4744	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	663.1;
	4744	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	202.1;
	4744	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	360.5;
	4744	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	367.1;
	4744	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.3;
	4744	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	491.8;
	4745	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	659.8;
	4745	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	202.9;
	4745	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	360.2;
	4745	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	364.9;
	4745	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93;
	4745	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	491.3;
	4746	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	652.9;
	4746	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	203.5;
	4746	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	354.2;
	4746	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	357.9;
	4746	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.1;
	4746	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	486.5;
	4747	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	641.5;
	4747	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	200;
	4747	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	338;
	4747	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	341.6;
	4747	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.9;
	4747	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	477.3;
	4748	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	630.2;
	4748	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197;
	4748	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	324.3;
	4748	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	329.4;
	4748	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.5;
	4748	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	467.5;
	4749	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	617.4;
	4749	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.1;
	4749	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	312.3;
	4749	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	318.7;
	4749	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.1;
	4749	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	457.3;
	4750	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	576.1;
	4750	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.5;
	4750	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	297.1;
	4750	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	301.5;
	4750	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81;
	4750	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	435.9;
	4751	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549.8;
	4751	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.2;
	4751	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	271;
	4751	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	275;
	4751	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.4;
	4751	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417;
	4752	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.5;
	4752	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.4;
	4752	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	259.1;
	4752	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.6;
	4752	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	4752	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.1;
	4753	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.4;
	4753	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.6;
	4753	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.8;
	4753	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.4;
	4753	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	4753	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.6;
	4754	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.5;
	4754	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.7;
	4754	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.7;
	4754	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.2;
	4754	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	4754	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.2;
	4755	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.2;
	4755	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.4;
	4755	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.4;
	4755	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224;
	4755	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	4755	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.3;
	4756	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.4;
	4756	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.4;
	4756	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.6;
	4756	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.4;
	4756	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	4756	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.6;
	4757	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.5;
	4757	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.4;
	4757	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.4;
	4757	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.4;
	4757	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	4757	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.9;
	4758	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.9;
	4758	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.2;
	4758	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.6;
	4758	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.9;
	4758	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	4758	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.3;
	4759	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.3;
	4759	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.6;
	4759	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.5;
	4759	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251;
	4759	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	4759	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.4;
	4760	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.1;
	4760	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.4;
	4760	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	264.8;
	4760	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.8;
	4760	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	4760	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.9;
	4761	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	569.4;
	4761	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171;
	4761	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	284.8;
	4761	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	293;
	4761	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	4761	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	426.3;
	4762	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.3;
	4762	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.5;
	4762	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	301.7;
	4762	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	309.9;
	4762	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	4762	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	444.8;
	4763	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	623.6;
	4763	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.1;
	4763	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	318.1;
	4763	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	326;
	4763	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72;
	4763	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	460.6;
	4764	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	645;
	4764	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.9;
	4764	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	329.3;
	4764	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	337.8;
	4764	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.3;
	4764	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	473.1;
	4765	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	667;
	4765	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.4;
	4765	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	335.8;
	4765	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	345.7;
	4765	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.9;
	4765	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	485.4;
	4766	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	666.6;
	4766	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	200.9;
	4766	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	351.6;
	4766	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	361;
	4766	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.7;
	4766	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	489.5;
	4767	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	671.3;
	4767	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	203.9;
	4767	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	354.4;
	4767	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	363.8;
	4767	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.7;
	4767	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	493.5;
	4768	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	660.3;
	4768	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	207.8;
	4768	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	357;
	4768	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	366.6;
	4768	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.8;
	4768	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	489.6;
	4769	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	654.2;
	4769	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	210.5;
	4769	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	352.7;
	4769	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	362.5;
	4769	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.2;
	4769	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	486.6;
	4770	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	637.3;
	4770	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	210.5;
	4770	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	348.7;
	4770	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	356.7;
	4770	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.7;
	4770	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	478;
	4771	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	620.1;
	4771	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	206.5;
	4771	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	337;
	4771	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	344;
	4771	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.3;
	4771	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	467.4;
	4772	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	608.3;
	4772	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	201.9;
	4772	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	326;
	4772	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	332.6;
	4772	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.2;
	4772	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	458.5;
	4773	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.7;
	4773	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198.5;
	4773	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	317.5;
	4773	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	322.9;
	4773	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87;
	4773	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	448.5;
	4774	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	590.5;
	4774	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.9;
	4774	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	301.3;
	4774	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	305.4;
	4774	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.6;
	4774	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	442.9;
	4775	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	563.7;
	4775	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.3;
	4775	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	280.4;
	4775	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	283.8;
	4775	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.5;
	4775	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	424.6;
	4776	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.9;
	4776	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167;
	4776	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	271.3;
	4776	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	274.9;
	4776	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.9;
	4776	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.3;
	4777	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.3;
	4777	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.2;
	4777	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.3;
	4777	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.8;
	4777	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.5;
	4777	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.4;
	4778	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.7;
	4778	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.5;
	4778	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.9;
	4778	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237;
	4778	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68;
	4778	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.5;
	4779	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447.2;
	4779	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.4;
	4779	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.8;
	4779	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.3;
	4779	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	4779	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.4;
	4780	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.5;
	4780	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.2;
	4780	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.9;
	4780	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.9;
	4780	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	4780	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.3;
	4781	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457;
	4781	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.3;
	4781	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.5;
	4781	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.1;
	4781	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	4781	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.9;
	4782	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.1;
	4782	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.3;
	4782	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.9;
	4782	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.9;
	4782	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	4782	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.9;
	4783	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.9;
	4783	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.3;
	4783	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.3;
	4783	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.1;
	4783	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	4783	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.1;
	4784	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.7;
	4784	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.4;
	4784	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	275.2;
	4784	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	279.9;
	4784	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	4784	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.9;
	4785	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	561.5;
	4785	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.1;
	4785	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	294.8;
	4785	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	299.4;
	4785	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.6;
	4785	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	427.3;
	4786	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	583;
	4786	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.7;
	4786	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	312.9;
	4786	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	318.8;
	4786	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.3;
	4786	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	442.9;
	4787	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	595.5;
	4787	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.8;
	4787	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	326.3;
	4787	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	334;
	4787	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.4;
	4787	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	453.4;
	4788	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	613.8;
	4788	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.8;
	4788	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	340.3;
	4788	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	347.7;
	4788	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.1;
	4788	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	465.1;
	4789	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	608.9;
	4789	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	201.1;
	4789	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	351.2;
	4789	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	358.5;
	4789	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96.4;
	4789	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	467.2;
	4790	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	602.1;
	4790	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	202;
	4790	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	359.3;
	4790	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	366.3;
	4790	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	99.1;
	4790	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	467.8;
	4791	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	600.6;
	4791	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.6;
	4791	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	355.6;
	4791	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	363.9;
	4791	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	101.9;
	4791	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	467.4;
	4792	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	608;
	4792	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.6;
	4792	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	355;
	4792	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	363.5;
	4792	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	103.4;
	4792	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	469.5;
	4793	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	619.8;
	4793	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198.8;
	4793	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	350.6;
	4793	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	359.3;
	4793	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	103.8;
	4793	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	472.9;
	4794	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	627.3;
	4794	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.3;
	4794	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	339.4;
	4794	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	347;
	4794	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	103.3;
	4794	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	472.2;
	4795	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	626.9;
	4795	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.5;
	4795	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	324.4;
	4795	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	332.1;
	4795	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	97.2;
	4795	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	466.3;
	4796	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	611.3;
	4796	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.8;
	4796	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	313.3;
	4796	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	321.2;
	4796	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.1;
	4796	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	455;
	4797	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	585.3;
	4797	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.7;
	4797	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	304.9;
	4797	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	312.5;
	4797	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.6;
	4797	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	440.3;
	4798	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	569.8;
	4798	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.1;
	4798	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	291.4;
	4798	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	297.7;
	4798	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.2;
	4798	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	429.6;
	4799	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.6;
	4799	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.1;
	4799	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	269.8;
	4799	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	275;
	4799	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.2;
	4799	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.4;
	4800	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.4;
	4800	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.1;
	4800	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.2;
	4800	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267;
	4800	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.7;
	4800	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.9;
	4801	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.4;
	4801	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.7;
	4801	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.2;
	4801	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.6;
	4801	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.9;
	4801	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.4;
	4802	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439;
	4802	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.4;
	4802	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.5;
	4802	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.5;
	4802	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.1;
	4802	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.1;
	4803	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.4;
	4803	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.8;
	4803	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.1;
	4803	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.1;
	4803	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	4803	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.1;
	4804	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.6;
	4804	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.5;
	4804	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.8;
	4804	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.1;
	4804	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	4804	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.6;
	4805	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.9;
	4805	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.6;
	4805	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.7;
	4805	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.5;
	4805	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	4805	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.2;
	4806	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.9;
	4806	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.7;
	4806	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230;
	4806	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.7;
	4806	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	4806	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.4;
	4807	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.2;
	4807	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.6;
	4807	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.3;
	4807	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.5;
	4807	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	4807	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.4;
	4808	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522;
	4808	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164;
	4808	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	266.3;
	4808	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	274.7;
	4808	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	4808	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.4;
	4809	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	546.2;
	4809	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.6;
	4809	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	283.5;
	4809	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	293.1;
	4809	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	4809	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.6;
	4810	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.6;
	4810	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.3;
	4810	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	297.6;
	4810	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	308.9;
	4810	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.5;
	4810	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	432.1;
	4811	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	591.2;
	4811	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.8;
	4811	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	307.4;
	4811	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	320.8;
	4811	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83;
	4811	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	446.2;
	4812	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	595.6;
	4812	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.3;
	4812	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	313;
	4812	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	329;
	4812	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.1;
	4812	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	450;
	4813	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	595;
	4813	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.4;
	4813	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	322.9;
	4813	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	340.8;
	4813	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.9;
	4813	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	452.8;
	4814	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	612.1;
	4814	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.9;
	4814	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	322.5;
	4814	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	342.6;
	4814	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.8;
	4814	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	460.3;
	4815	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	605.7;
	4815	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.9;
	4815	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	330.8;
	4815	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	350.3;
	4815	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.7;
	4815	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	458.9;
	4816	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	616.4;
	4816	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.3;
	4816	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	337.1;
	4816	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	356;
	4816	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.2;
	4816	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	464.7;
	4817	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	610.1;
	4817	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.4;
	4817	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	337.7;
	4817	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	355.8;
	4817	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.7;
	4817	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	462.9;
	4818	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	612.8;
	4818	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.6;
	4818	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	323.4;
	4818	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	341.8;
	4818	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.8;
	4818	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	460.3;
	4819	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	612.2;
	4819	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191;
	4819	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	315.9;
	4819	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	332.8;
	4819	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.5;
	4819	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	456.1;
	4820	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.5;
	4820	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190;
	4820	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	310.3;
	4820	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	326.5;
	4820	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.2;
	4820	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	447.1;
	4821	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	580.8;
	4821	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.4;
	4821	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	301;
	4821	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	317.1;
	4821	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.5;
	4821	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	437.1;
	4822	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	569.1;
	4822	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.6;
	4822	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	284.8;
	4822	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	300.1;
	4822	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.4;
	4822	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	427.9;
	4823	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.7;
	4823	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.1;
	4823	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	264.3;
	4823	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	278.8;
	4823	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.1;
	4823	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.3;
	4824	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.5;
	4824	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.9;
	4824	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.7;
	4824	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.8;
	4824	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.4;
	4824	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.9;
	4825	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.3;
	4825	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141;
	4825	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.7;
	4825	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247;
	4825	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	4825	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375;
	4826	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.3;
	4826	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.3;
	4826	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225;
	4826	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.5;
	4826	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	4826	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.4;
	4827	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	420.8;
	4827	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.8;
	4827	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.4;
	4827	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.7;
	4827	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	4827	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.5;
	4828	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.8;
	4828	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.7;
	4828	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.8;
	4828	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.5;
	4828	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	4828	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.8;
	4829	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.3;
	4829	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131;
	4829	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.8;
	4829	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.4;
	4829	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.4;
	4829	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.5;
	4830	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456;
	4830	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.1;
	4830	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.6;
	4830	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.3;
	4830	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.1;
	4830	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.4;
	4831	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.4;
	4831	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.8;
	4831	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.2;
	4831	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.1;
	4831	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	4831	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.3;
	4832	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.1;
	4832	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.3;
	4832	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	259.1;
	4832	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	274.6;
	4832	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	4832	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.2;
	4833	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.9;
	4833	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.5;
	4833	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	275.7;
	4833	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	293.7;
	4833	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	4833	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.1;
	4834	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.1;
	4834	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.8;
	4834	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	290.9;
	4834	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	311;
	4834	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74;
	4834	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	431.1;
	4835	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589.4;
	4835	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.2;
	4835	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	304.5;
	4835	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	326;
	4835	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79;
	4835	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	443.2;
	4836	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	596.5;
	4836	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.2;
	4836	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	313.1;
	4836	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	335.6;
	4836	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.2;
	4836	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	449;
	4837	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	601.4;
	4837	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.3;
	4837	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	321.1;
	4837	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	344.8;
	4837	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.9;
	4837	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	454.3;
	4838	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	606.6;
	4838	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.8;
	4838	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	325.2;
	4838	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	348.2;
	4838	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.5;
	4838	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	458.1;
	4839	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	613.8;
	4839	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.2;
	4839	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	332.7;
	4839	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	354.8;
	4839	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.4;
	4839	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	462.7;
	4840	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	622.6;
	4840	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.4;
	4840	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	332.9;
	4840	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	355.4;
	4840	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.7;
	4840	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	466.3;
	4841	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	620.9;
	4841	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.9;
	4841	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	326.8;
	4841	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	348.4;
	4841	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.3;
	4841	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	464.7;
	4842	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	616.5;
	4842	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.1;
	4842	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	321.3;
	4842	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	341.1;
	4842	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.5;
	4842	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	460.2;
	4843	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	599.7;
	4843	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.4;
	4843	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	313;
	4843	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	331.3;
	4843	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.3;
	4843	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	450.1;
	4844	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579.2;
	4844	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.9;
	4844	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	301.1;
	4844	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	318.9;
	4844	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89;
	4844	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	438;
	4845	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560;
	4845	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.3;
	4845	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	292.7;
	4845	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	309.3;
	4845	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.1;
	4845	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	426.8;
	4846	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548.8;
	4846	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.9;
	4846	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	279.4;
	4846	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	293.8;
	4846	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.9;
	4846	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418;
	4847	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.4;
	4847	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.6;
	4847	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	263.6;
	4847	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	276.2;
	4847	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.7;
	4847	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.8;
	4848	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.2;
	4848	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.8;
	4848	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.3;
	4848	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	270.7;
	4848	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.3;
	4848	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.8;
	4849	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.9;
	4849	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.2;
	4849	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.1;
	4849	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.2;
	4849	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	4849	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.6;
	4850	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	401.9;
	4850	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.6;
	4850	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.6;
	4850	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.8;
	4850	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	4850	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.8;
	4851	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394;
	4851	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.7;
	4851	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.4;
	4851	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.8;
	4851	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.2;
	4851	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.7;
	4852	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.1;
	4852	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127;
	4852	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.2;
	4852	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.4;
	4852	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	4852	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.4;
	4853	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	401.7;
	4853	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.7;
	4853	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.3;
	4853	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.4;
	4853	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.1;
	4853	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.2;
	4854	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.4;
	4854	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.7;
	4854	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.1;
	4854	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.6;
	4854	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.5;
	4854	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.1;
	4855	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.5;
	4855	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.6;
	4855	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.5;
	4855	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.1;
	4855	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	4855	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.6;
	4856	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442.7;
	4856	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.2;
	4856	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.4;
	4856	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243;
	4856	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	4856	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.5;
	4857	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470;
	4857	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.1;
	4857	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.9;
	4857	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.6;
	4857	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	4857	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.4;
	4858	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.4;
	4858	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159;
	4858	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.9;
	4858	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	270.4;
	4858	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.7;
	4858	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.6;
	4859	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.4;
	4859	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.7;
	4859	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	270.5;
	4859	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	282.4;
	4859	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.1;
	4859	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.4;
	4860	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.9;
	4860	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.9;
	4860	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	280.9;
	4860	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	294.6;
	4860	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.1;
	4860	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.2;
	4861	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.1;
	4861	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.5;
	4861	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	290.3;
	4861	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	304.9;
	4861	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.4;
	4861	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.9;
	4862	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.2;
	4862	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.1;
	4862	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	296.4;
	4862	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	311.6;
	4862	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.9;
	4862	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.5;
	4863	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	554.5;
	4863	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.6;
	4863	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	302.9;
	4863	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	315.9;
	4863	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.6;
	4863	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	428.5;
	4864	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	555.9;
	4864	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.9;
	4864	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	309.7;
	4864	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	321.7;
	4864	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.1;
	4864	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	430.9;
	4865	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549.2;
	4865	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.6;
	4865	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	310.8;
	4865	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	321.5;
	4865	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.9;
	4865	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	429.4;
	4866	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.7;
	4866	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.6;
	4866	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	306.3;
	4866	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	318;
	4866	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.8;
	4866	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	428.6;
	4867	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537.7;
	4867	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.8;
	4867	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	294.3;
	4867	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	307.3;
	4867	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.2;
	4867	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.3;
	4868	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	524.9;
	4868	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172;
	4868	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	286.5;
	4868	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	299.6;
	4868	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.5;
	4868	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.5;
	4869	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.5;
	4869	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167;
	4869	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	282.7;
	4869	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	294.7;
	4869	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.9;
	4869	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.5;
	4870	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.1;
	4870	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.2;
	4870	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	273.9;
	4870	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	284.6;
	4870	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.4;
	4870	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.1;
	4871	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.4;
	4871	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.9;
	4871	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.6;
	4871	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267.5;
	4871	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.6;
	4871	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.3;
	4872	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.6;
	4872	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.3;
	4872	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.9;
	4872	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	263.3;
	4872	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.1;
	4872	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.4;
	4873	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403.6;
	4873	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.2;
	4873	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.4;
	4873	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239;
	4873	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.3;
	4873	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.9;
	4874	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	363.2;
	4874	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.5;
	4874	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.8;
	4874	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.2;
	4874	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	4874	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.7;
	4875	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	348.2;
	4875	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.1;
	4875	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.6;
	4875	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224;
	4875	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	4875	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.7;
	4876	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	344.9;
	4876	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.4;
	4876	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.4;
	4876	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221;
	4876	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	4876	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.9;
	4877	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	361.8;
	4877	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.3;
	4877	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.3;
	4877	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219;
	4877	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	4877	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.3;
	4878	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	365.1;
	4878	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.9;
	4878	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.4;
	4878	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.3;
	4878	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.4;
	4878	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.8;
	4879	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	373.2;
	4879	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.2;
	4879	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213;
	4879	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.8;
	4879	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	4879	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.9;
	4880	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	380.4;
	4880	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132;
	4880	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.3;
	4880	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.5;
	4880	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	4880	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.3;
	4881	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.2;
	4881	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.7;
	4881	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233;
	4881	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.1;
	4881	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.5;
	4881	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344;
	4882	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.9;
	4882	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.7;
	4882	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.6;
	4882	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.1;
	4882	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.8;
	4882	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.6;
	4883	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.7;
	4883	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.5;
	4883	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	263.7;
	4883	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	273;
	4883	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.4;
	4883	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.4;
	4884	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.5;
	4884	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.1;
	4884	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	278.9;
	4884	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	287.9;
	4884	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.4;
	4884	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.9;
	4885	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	524.7;
	4885	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.4;
	4885	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	294.2;
	4885	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	300.3;
	4885	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.3;
	4885	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.6;
	4886	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.2;
	4886	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.4;
	4886	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	308.6;
	4886	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	312.5;
	4886	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.1;
	4886	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.9;
	4887	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548.4;
	4887	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.2;
	4887	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	315.8;
	4887	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	320.6;
	4887	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.3;
	4887	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	430.4;
	4888	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558;
	4888	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.8;
	4888	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	322.4;
	4888	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	322.5;
	4888	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96.6;
	4888	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	436.4;
	4889	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564.5;
	4889	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184;
	4889	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	322.4;
	4889	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	324.7;
	4889	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	97.9;
	4889	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	440.1;
	4890	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	561.3;
	4890	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.9;
	4890	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	321;
	4890	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	324;
	4890	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96.9;
	4890	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	437.8;
	4891	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552;
	4891	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.2;
	4891	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	315.8;
	4891	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	319.8;
	4891	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96;
	4891	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	432;
	4892	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.6;
	4892	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.5;
	4892	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	308.5;
	4892	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	312.4;
	4892	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.6;
	4892	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	426.2;
	4893	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539;
	4893	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.9;
	4893	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	304.3;
	4893	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	308.3;
	4893	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.7;
	4893	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.4;
	4894	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526.3;
	4894	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.2;
	4894	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	290.5;
	4894	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	293.4;
	4894	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.5;
	4894	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.7;
	4895	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.9;
	4895	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.8;
	4895	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	270.3;
	4895	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	273.5;
	4895	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.6;
	4895	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.1;
	4896	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.6;
	4896	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.6;
	4896	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.6;
	4896	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	263.8;
	4896	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80;
	4896	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.3;
	4897	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.7;
	4897	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.1;
	4897	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.4;
	4897	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240;
	4897	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.4;
	4897	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.6;
	4898	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.7;
	4898	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.5;
	4898	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.9;
	4898	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.1;
	4898	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73;
	4898	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.2;
	4899	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	395;
	4899	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.2;
	4899	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.5;
	4899	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.4;
	4899	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.8;
	4899	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.9;
	4900	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.3;
	4900	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.1;
	4900	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220;
	4900	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225;
	4900	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.1;
	4900	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.1;
	4901	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	387.6;
	4901	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.6;
	4901	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.5;
	4901	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.6;
	4901	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	4901	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.9;
	4902	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.4;
	4902	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136;
	4902	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.4;
	4902	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.2;
	4902	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	4902	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.3;
	4903	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	444.1;
	4903	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.8;
	4903	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.9;
	4903	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.1;
	4903	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	4903	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.7;
	4904	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.4;
	4904	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.4;
	4904	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.8;
	4904	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267.8;
	4904	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.5;
	4904	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.9;
	4905	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.9;
	4905	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170;
	4905	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	284.6;
	4905	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	289.7;
	4905	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.1;
	4905	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.3;
	4906	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.1;
	4906	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.9;
	4906	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	299;
	4906	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	302.7;
	4906	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.5;
	4906	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.1;
	4907	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577.5;
	4907	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.1;
	4907	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	314.1;
	4907	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	317.2;
	4907	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.1;
	4907	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	441.5;
	4908	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	555.9;
	4908	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.1;
	4908	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	332.4;
	4908	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	332;
	4908	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.9;
	4908	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	438.4;
	4909	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	605.4;
	4909	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198.5;
	4909	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	345.1;
	4909	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	342.8;
	4909	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.4;
	4909	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	463.8;
	4910	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	617.2;
	4910	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	201.2;
	4910	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	357.4;
	4910	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	351.4;
	4910	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96.7;
	4910	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	473.2;
	4911	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	625.4;
	4911	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	202;
	4911	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	355.9;
	4911	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	347.6;
	4911	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	97.9;
	4911	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	477.5;
	4912	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	658;
	4912	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	202.4;
	4912	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	356.3;
	4912	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	348.5;
	4912	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	99.4;
	4912	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	490.3;
	4913	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	633;
	4913	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	203.9;
	4913	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	355.6;
	4913	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	347.9;
	4913	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	99.3;
	4913	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	479.7;
	4914	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	632.4;
	4914	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	204.3;
	4914	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	340.5;
	4914	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	333;
	4914	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	99.8;
	4914	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	474.8;
	4915	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	625.3;
	4915	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	200.8;
	4915	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	329;
	4915	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	321.6;
	4915	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	98.9;
	4915	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	467.4;
	4916	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	608.8;
	4916	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.8;
	4916	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	324.1;
	4916	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	318.4;
	4916	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96.4;
	4916	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	457.3;
	4917	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.2;
	4917	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.2;
	4917	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	312.6;
	4917	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	308.4;
	4917	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.4;
	4917	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	449.5;
	4918	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	581.9;
	4918	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.9;
	4918	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	295.9;
	4918	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	292;
	4918	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.6;
	4918	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	437.8;
	4919	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	555;
	4919	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.1;
	4919	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	273.5;
	4919	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	270.1;
	4919	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.3;
	4919	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.3;
	4920	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.2;
	4920	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.5;
	4920	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.5;
	4920	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.7;
	4920	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.1;
	4920	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.1;
	4921	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.1;
	4921	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.4;
	4921	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.4;
	4921	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.9;
	4921	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.2;
	4921	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.3;
	4922	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.7;
	4922	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.9;
	4922	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228;
	4922	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.1;
	4922	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	4922	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350;
	4923	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.3;
	4923	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.5;
	4923	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.5;
	4923	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.2;
	4923	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69;
	4923	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.6;
	4924	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403.4;
	4924	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.9;
	4924	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.3;
	4924	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.4;
	4924	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	4924	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.3;
	4925	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.9;
	4925	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.8;
	4925	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.2;
	4925	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224;
	4925	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	4925	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.8;
	4926	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.4;
	4926	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.9;
	4926	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.1;
	4926	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.5;
	4926	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	4926	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.2;
	4927	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.5;
	4927	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.7;
	4927	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246;
	4927	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248;
	4927	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	4927	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368;
	4928	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.1;
	4928	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.9;
	4928	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	266.8;
	4928	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	269.1;
	4928	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	4928	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.7;
	4929	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.2;
	4929	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.7;
	4929	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	285.8;
	4929	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	286.9;
	4929	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.4;
	4929	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406;
	4930	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.3;
	4930	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.4;
	4930	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	299.4;
	4930	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	300.9;
	4930	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.5;
	4930	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.3;
	4931	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	567.2;
	4931	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.2;
	4931	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	316.8;
	4931	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	318.4;
	4931	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.6;
	4931	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	437.9;
	4932	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	608.7;
	4932	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.9;
	4932	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	328.9;
	4932	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	329;
	4932	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.6;
	4932	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	459.3;
	4933	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	612.9;
	4933	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.1;
	4933	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	340.1;
	4933	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	342.8;
	4933	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.7;
	4933	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	465.1;
	4934	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	622.6;
	4934	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198.3;
	4934	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	345.4;
	4934	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	349.4;
	4934	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.5;
	4934	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	471.8;
	4935	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	634.6;
	4935	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	201.2;
	4935	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	355.2;
	4935	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	358.6;
	4935	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.8;
	4935	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	478.9;
	4936	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	632.4;
	4936	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	202.5;
	4936	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	361.9;
	4936	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	367;
	4936	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.7;
	4936	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	479.6;
	4937	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	624.3;
	4937	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	205.1;
	4937	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	356.3;
	4937	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	361.1;
	4937	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.5;
	4937	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	476.3;
	4938	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	613.8;
	4938	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	202.3;
	4938	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	351.5;
	4938	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	353;
	4938	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.2;
	4938	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	469.5;
	4939	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	594.2;
	4939	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.1;
	4939	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	337.7;
	4939	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	338.5;
	4939	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.2;
	4939	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	457.3;
	4940	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.6;
	4940	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191;
	4940	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	323.2;
	4940	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	326.2;
	4940	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.3;
	4940	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	442.9;
	4941	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548.6;
	4941	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.7;
	4941	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	308;
	4941	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	309.8;
	4941	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.8;
	4941	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	428.9;
	4942	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525.7;
	4942	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.4;
	4942	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	292.8;
	4942	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	293.5;
	4942	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.4;
	4942	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.2;
	4943	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.3;
	4943	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.4;
	4943	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	269.3;
	4943	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	270.2;
	4943	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.8;
	4943	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.7;
	4944	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.8;
	4944	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.1;
	4944	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.6;
	4944	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.6;
	4944	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.9;
	4944	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.4;
	4945	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.6;
	4945	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.3;
	4945	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.6;
	4945	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.7;
	4945	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.2;
	4945	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.4;
	4946	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.4;
	4946	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.2;
	4946	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220;
	4946	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.5;
	4946	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.3;
	4946	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.3;
	4947	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	384.8;
	4947	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.8;
	4947	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215;
	4947	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.2;
	4947	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.5;
	4947	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.5;
	4948	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	365.8;
	4948	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.3;
	4948	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.3;
	4948	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.2;
	4948	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	4948	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.8;
	4949	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	381;
	4949	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.1;
	4949	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215;
	4949	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.6;
	4949	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	4949	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.3;
	4950	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.6;
	4950	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.1;
	4950	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.8;
	4950	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.9;
	4950	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	4950	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.8;
	4951	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.8;
	4951	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.1;
	4951	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.9;
	4951	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.4;
	4951	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	4951	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.5;
	4952	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.8;
	4952	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.8;
	4952	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255;
	4952	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.8;
	4952	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.4;
	4952	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.4;
	4953	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.2;
	4953	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.9;
	4953	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	271.6;
	4953	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	275.5;
	4953	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.6;
	4953	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.2;
	4954	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.7;
	4954	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.2;
	4954	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	286;
	4954	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	289.5;
	4954	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.2;
	4954	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.9;
	4955	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.7;
	4955	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172;
	4955	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	299.2;
	4955	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	301.2;
	4955	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.8;
	4955	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.9;
	4956	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	563.4;
	4956	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.3;
	4956	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	308.5;
	4956	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	311.4;
	4956	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95;
	4956	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	434.3;
	4957	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.9;
	4957	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.7;
	4957	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	319.3;
	4957	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	320.6;
	4957	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	100;
	4957	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	436.1;
	4958	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.2;
	4958	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183;
	4958	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	327.9;
	4958	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	329.7;
	4958	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	103;
	4958	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	443.7;
	4959	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	576.3;
	4959	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184;
	4959	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	337.5;
	4959	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	338.8;
	4959	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	106.3;
	4959	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	449;
	4960	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.8;
	4960	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.3;
	4960	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	343.7;
	4960	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	345;
	4960	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	107.5;
	4960	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	437.7;
	4961	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	555.6;
	4961	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188;
	4961	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	340.4;
	4961	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	341.7;
	4961	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	107.9;
	4961	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	443.1;
	4962	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	553.8;
	4962	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185;
	4962	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	334.6;
	4962	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	335.6;
	4962	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	107.6;
	4962	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	440.3;
	4963	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.2;
	4963	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.4;
	4963	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	320.7;
	4963	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	322.5;
	4963	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	103.8;
	4963	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	437.3;
	4964	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	546;
	4964	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.3;
	4964	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	305.2;
	4964	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	306.4;
	4964	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	101.7;
	4964	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	426.8;
	4965	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526.9;
	4965	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.5;
	4965	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	293.3;
	4965	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	295.5;
	4965	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	97.5;
	4965	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.6;
	4966	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.7;
	4966	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.4;
	4966	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	277.1;
	4966	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	279.1;
	4966	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.1;
	4966	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400;
	4967	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.8;
	4967	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.2;
	4967	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.8;
	4967	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.3;
	4967	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.1;
	4967	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.7;
	4968	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.8;
	4968	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.2;
	4968	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.2;
	4968	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.9;
	4968	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.7;
	4968	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.3;
	4969	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.1;
	4969	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129;
	4969	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222;
	4969	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.4;
	4969	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.2;
	4969	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.5;
	4970	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	388.3;
	4970	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124;
	4970	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.4;
	4970	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.7;
	4970	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.7;
	4970	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.4;
	4971	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	384.4;
	4971	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.2;
	4971	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.4;
	4971	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.8;
	4971	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	4971	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.7;
	4972	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	387.6;
	4972	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.7;
	4972	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.7;
	4972	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.4;
	4972	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.8;
	4972	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.3;
	4973	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	401.8;
	4973	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.8;
	4973	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.1;
	4973	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.1;
	4973	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	4973	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.7;
	4974	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.5;
	4974	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.3;
	4974	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.4;
	4974	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.6;
	4974	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	4974	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.8;
	4975	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.3;
	4975	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.6;
	4975	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.5;
	4975	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.1;
	4975	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.8;
	4975	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.6;
	4976	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.1;
	4976	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.6;
	4976	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.1;
	4976	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.3;
	4976	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.7;
	4976	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.2;
	4977	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.5;
	4977	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.9;
	4977	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.3;
	4977	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265.4;
	4977	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.5;
	4977	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390;
	4978	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518.4;
	4978	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161;
	4978	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	270.6;
	4978	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	276.6;
	4978	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.6;
	4978	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.5;
	4979	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.2;
	4979	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.4;
	4979	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	278.4;
	4979	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	285.3;
	4979	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.2;
	4979	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.7;
	4980	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	554.2;
	4980	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.2;
	4980	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	286.5;
	4980	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	294.4;
	4980	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.7;
	4980	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.7;
	4981	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.1;
	4981	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.1;
	4981	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	295.3;
	4981	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	303.1;
	4981	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.7;
	4981	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	427.9;
	4982	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.9;
	4982	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.5;
	4982	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	302.8;
	4982	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	312.4;
	4982	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	99.6;
	4982	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	433.9;
	4983	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.7;
	4983	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.3;
	4983	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	313.7;
	4983	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	323.2;
	4983	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	102.1;
	4983	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	438.7;
	4984	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	567.3;
	4984	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.3;
	4984	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	321.5;
	4984	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	327.9;
	4984	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	103.4;
	4984	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	440.2;
	4985	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559.5;
	4985	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.2;
	4985	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	324.3;
	4985	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	327.5;
	4985	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	103.1;
	4985	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	438.3;
	4986	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564.9;
	4986	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.6;
	4986	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	317.5;
	4986	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	320.2;
	4986	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	103;
	4986	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	438.5;
	4987	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552.6;
	4987	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.7;
	4987	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	306.2;
	4987	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	309.1;
	4987	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	100.8;
	4987	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	429.2;
	4988	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547;
	4988	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.7;
	4988	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	292.4;
	4988	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	296.5;
	4988	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	99.1;
	4988	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.9;
	4989	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	524.3;
	4989	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.3;
	4989	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	285;
	4989	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	290.1;
	4989	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.6;
	4989	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.9;
	4990	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.9;
	4990	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.1;
	4990	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	268.5;
	4990	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	274;
	4990	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.3;
	4990	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.6;
	4991	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.5;
	4991	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.8;
	4991	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.8;
	4991	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.5;
	4991	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.5;
	4991	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.3;
	4992	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.1;
	4992	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.3;
	4992	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.9;
	4992	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.9;
	4992	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.4;
	4992	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.7;
	4993	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.8;
	4993	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.8;
	4993	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.6;
	4993	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.7;
	4993	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.3;
	4993	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.5;
	4994	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	400.3;
	4994	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.1;
	4994	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.7;
	4994	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.7;
	4994	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.2;
	4994	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.8;
	4995	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	388.9;
	4995	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.4;
	4995	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208;
	4995	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.2;
	4995	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72;
	4995	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.7;
	4996	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	389.1;
	4996	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.3;
	4996	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.6;
	4996	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213;
	4996	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.8;
	4996	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.8;
	4997	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391.7;
	4997	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.1;
	4997	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.9;
	4997	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.3;
	4997	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70;
	4997	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.8;
	4998	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.2;
	4998	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.5;
	4998	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.6;
	4998	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223;
	4998	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.8;
	4998	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.7;
	4999	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.9;
	4999	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.9;
	4999	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.9;
	4999	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.5;
	4999	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.9;
	4999	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.2;
	5000	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.9;
	5000	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.2;
	5000	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.7;
	5000	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.9;
	5000	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.9;
	5000	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374;
	5001	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.6;
	5001	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.8;
	5001	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.2;
	5001	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	268.1;
	5001	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.3;
	5001	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390;
	5002	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.8;
	5002	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.6;
	5002	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	275.4;
	5002	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	281.1;
	5002	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.2;
	5002	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.6;
	5003	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.2;
	5003	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.1;
	5003	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	290;
	5003	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	293.8;
	5003	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.9;
	5003	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.4;
	5004	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.1;
	5004	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.9;
	5004	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	304.6;
	5004	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	307.1;
	5004	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.3;
	5004	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.2;
	5005	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.5;
	5005	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.5;
	5005	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	317.3;
	5005	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	320.1;
	5005	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.9;
	5005	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	424.7;
	5006	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	540.9;
	5006	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.4;
	5006	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	326.9;
	5006	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	328.9;
	5006	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	99.9;
	5006	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	432.3;
	5007	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	546.3;
	5007	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.5;
	5007	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	336.4;
	5007	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	337.9;
	5007	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	101.6;
	5007	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	437.3;
	5008	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	568;
	5008	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.2;
	5008	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	342.2;
	5008	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	344.8;
	5008	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	102.4;
	5008	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	448.2;
	5009	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	573.2;
	5009	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.7;
	5009	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	341.6;
	5009	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	343.7;
	5009	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	103.1;
	5009	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	450.7;
	5010	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579.3;
	5010	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.8;
	5010	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	333;
	5010	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	336.5;
	5010	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	101.8;
	5010	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	450.7;
	5011	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	567.3;
	5011	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.8;
	5011	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	321.3;
	5011	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	324.7;
	5011	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	100.1;
	5011	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	441.3;
	5012	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.1;
	5012	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.2;
	5012	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	307.8;
	5012	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	310.8;
	5012	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	97.3;
	5012	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	426.5;
	5013	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.8;
	5013	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.5;
	5013	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	299.1;
	5013	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	302.3;
	5013	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.4;
	5013	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.5;
	5014	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.8;
	5014	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.3;
	5014	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	283.9;
	5014	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	286.8;
	5014	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.2;
	5014	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.1;
	5015	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.1;
	5015	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.4;
	5015	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	265.8;
	5015	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	268.8;
	5015	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.5;
	5015	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.9;
	5016	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.3;
	5016	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.4;
	5016	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.4;
	5016	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.9;
	5016	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81;
	5016	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.4;
	5017	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.5;
	5017	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.7;
	5017	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.1;
	5017	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.8;
	5017	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.4;
	5017	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.9;
	5018	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	377.5;
	5018	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.2;
	5018	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.7;
	5018	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.5;
	5018	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.7;
	5018	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.3;
	5019	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	368.4;
	5019	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.2;
	5019	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.1;
	5019	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.4;
	5019	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.5;
	5019	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.4;
	5020	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	367;
	5020	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.1;
	5020	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214;
	5020	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.2;
	5020	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.8;
	5020	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.7;
	5021	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	363.8;
	5021	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.4;
	5021	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.7;
	5021	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.9;
	5021	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	5021	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.8;
	5022	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	374.1;
	5022	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122;
	5022	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.3;
	5022	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.8;
	5022	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	5022	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326;
	5023	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	385;
	5023	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.3;
	5023	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.5;
	5023	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.5;
	5023	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.1;
	5023	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.8;
	5024	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	387.2;
	5024	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.9;
	5024	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.5;
	5024	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233;
	5024	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.5;
	5024	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.2;
	5025	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.8;
	5025	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.5;
	5025	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.8;
	5025	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.4;
	5025	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.4;
	5025	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.1;
	5026	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.5;
	5026	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.9;
	5026	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.3;
	5026	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.2;
	5026	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.7;
	5026	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.6;
	5027	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.7;
	5027	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162;
	5027	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	269.9;
	5027	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	277.1;
	5027	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.9;
	5027	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.5;
	5028	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.8;
	5028	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166;
	5028	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	278.1;
	5028	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	285.7;
	5028	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.4;
	5028	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.6;
	5029	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.8;
	5029	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.2;
	5029	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	284.1;
	5029	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	293;
	5029	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	97.8;
	5029	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.4;
	5030	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.8;
	5030	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173;
	5030	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	293.5;
	5030	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	302.4;
	5030	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	98.3;
	5030	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.9;
	5031	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.8;
	5031	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.6;
	5031	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	299.4;
	5031	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	307.9;
	5031	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.8;
	5031	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.1;
	5032	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.9;
	5032	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.6;
	5032	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	299.5;
	5032	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	308.3;
	5032	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	99.2;
	5032	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.2;
	5033	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542;
	5033	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.5;
	5033	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	298.3;
	5033	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	308.1;
	5033	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.7;
	5033	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.4;
	5034	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.6;
	5034	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.9;
	5034	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	292;
	5034	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	299.9;
	5034	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	98.4;
	5034	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420.5;
	5035	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536;
	5035	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.4;
	5035	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	286.1;
	5035	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	293.5;
	5035	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94;
	5035	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.3;
	5036	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.9;
	5036	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.8;
	5036	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	281.8;
	5036	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	288.5;
	5036	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.5;
	5036	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.2;
	5037	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.4;
	5037	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.1;
	5037	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	276;
	5037	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	281.5;
	5037	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.4;
	5037	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398;
	5038	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.3;
	5038	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.9;
	5038	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	264.8;
	5038	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	269.2;
	5038	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.7;
	5038	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.1;
	5039	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.5;
	5039	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.8;
	5039	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.6;
	5039	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.8;
	5039	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.2;
	5039	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.3;
	5040	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.2;
	5040	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.3;
	5040	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.9;
	5040	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.1;
	5040	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.8;
	5040	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.5;
	5041	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.1;
	5041	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.2;
	5041	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.8;
	5041	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.2;
	5041	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.9;
	5041	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.1;
	5042	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	370.3;
	5042	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.3;
	5042	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.7;
	5042	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.1;
	5042	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	5042	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.5;
	5043	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	354.2;
	5043	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.3;
	5043	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.7;
	5043	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.4;
	5043	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	5043	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.4;
	5044	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	344.6;
	5044	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122;
	5044	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.8;
	5044	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.1;
	5044	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	5044	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309.5;
	5045	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	347.1;
	5045	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.2;
	5045	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.9;
	5045	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.4;
	5045	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	5045	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310.5;
	5046	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	353.3;
	5046	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.1;
	5046	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.1;
	5046	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.5;
	5046	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	5046	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.7;
	5047	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	360.7;
	5047	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.3;
	5047	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.6;
	5047	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.2;
	5047	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	5047	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.9;
	5048	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	369.6;
	5048	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133;
	5048	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.3;
	5048	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.5;
	5048	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	5048	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324;
	5049	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392.7;
	5049	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.5;
	5049	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.9;
	5049	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.1;
	5049	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.7;
	5049	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.9;
	5050	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.8;
	5050	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.4;
	5050	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.4;
	5050	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.6;
	5050	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.3;
	5050	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.3;
	5051	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.7;
	5051	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.5;
	5051	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.4;
	5051	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.6;
	5051	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.9;
	5051	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.5;
	5052	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.7;
	5052	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.8;
	5052	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.7;
	5052	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	266.2;
	5052	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.6;
	5052	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.8;
	5053	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525.1;
	5053	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.4;
	5053	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	272.5;
	5053	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	279.7;
	5053	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.3;
	5053	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.9;
	5054	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.9;
	5054	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.5;
	5054	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	285.2;
	5054	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	292.4;
	5054	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.8;
	5054	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.4;
	5055	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550;
	5055	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.5;
	5055	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	295.6;
	5055	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	303;
	5055	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.3;
	5055	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.7;
	5056	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.6;
	5056	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.9;
	5056	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	301.9;
	5056	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	309.1;
	5056	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.3;
	5056	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.7;
	5057	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.1;
	5057	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.8;
	5057	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	305.6;
	5057	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	312.7;
	5057	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.7;
	5057	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	424.1;
	5058	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.9;
	5058	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186;
	5058	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	304.4;
	5058	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	310.3;
	5058	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.2;
	5058	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.2;
	5059	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.5;
	5059	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.2;
	5059	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	301;
	5059	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	306.4;
	5059	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.6;
	5059	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.6;
	5060	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.1;
	5060	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.4;
	5060	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	295.3;
	5060	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	300.5;
	5060	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.7;
	5060	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.2;
	5061	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.1;
	5061	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.2;
	5061	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	290.9;
	5061	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	296.7;
	5061	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.1;
	5061	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.3;
	5062	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.6;
	5062	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.1;
	5062	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	281.6;
	5062	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	286;
	5062	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.7;
	5062	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.3;
	5063	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.1;
	5063	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.5;
	5063	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.9;
	5063	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.8;
	5063	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.1;
	5063	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.8;
	5064	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.3;
	5064	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.5;
	5064	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.3;
	5064	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.7;
	5064	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.6;
	5064	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.5;
	5065	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.4;
	5065	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.1;
	5065	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.8;
	5065	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.2;
	5065	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	5065	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.1;
	5066	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	372.9;
	5066	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.2;
	5066	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.2;
	5066	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.1;
	5066	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.4;
	5066	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.4;
	5067	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	364;
	5067	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.1;
	5067	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.1;
	5067	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222;
	5067	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	5067	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.3;
	5068	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	356.2;
	5068	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.8;
	5068	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.2;
	5068	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220;
	5068	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	5068	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.2;
	5069	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	357.3;
	5069	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.9;
	5069	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.4;
	5069	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221;
	5069	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	5069	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319;
	5070	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	373.4;
	5070	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.7;
	5070	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.5;
	5070	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.4;
	5070	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	5070	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.3;
	5071	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.5;
	5071	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.2;
	5071	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.9;
	5071	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243;
	5071	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	5071	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.1;
	5072	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.1;
	5072	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.8;
	5072	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	259.4;
	5072	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.8;
	5072	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	5072	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.6;
	5073	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.1;
	5073	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.6;
	5073	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	277.2;
	5073	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	283.2;
	5073	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	5073	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.3;
	5074	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.5;
	5074	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.1;
	5074	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	287.3;
	5074	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	294.9;
	5074	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	5074	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.1;
	5075	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	555.7;
	5075	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.4;
	5075	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	300.2;
	5075	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	308.8;
	5075	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.9;
	5075	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	427.9;
	5076	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.9;
	5076	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.6;
	5076	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	311.7;
	5076	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	321.9;
	5076	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.9;
	5076	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	438.3;
	5077	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	596.1;
	5077	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.8;
	5077	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	327.2;
	5077	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	337.8;
	5077	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.4;
	5077	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	452.9;
	5078	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	601.9;
	5078	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.6;
	5078	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	340.5;
	5078	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	350.7;
	5078	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.6;
	5078	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	460.2;
	5079	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	609.7;
	5079	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.5;
	5079	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	349.7;
	5079	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	359.6;
	5079	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.6;
	5079	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	466.9;
	5080	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	623.4;
	5080	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.7;
	5080	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	355.9;
	5080	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	365.2;
	5080	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89;
	5080	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	474.5;
	5081	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	619.2;
	5081	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.1;
	5081	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	345.4;
	5081	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	355.7;
	5081	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.6;
	5081	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	471.1;
	5082	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	616;
	5082	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.9;
	5082	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	335.2;
	5082	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	344;
	5082	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.5;
	5082	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	465.8;
	5083	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	611.7;
	5083	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.2;
	5083	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	314.7;
	5083	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	323.1;
	5083	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.9;
	5083	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	458;
	5084	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	593;
	5084	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.2;
	5084	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	302.1;
	5084	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	309.3;
	5084	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.3;
	5084	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	444.7;
	5085	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577.4;
	5085	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.4;
	5085	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	290.6;
	5085	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	297.2;
	5085	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.4;
	5085	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	433.5;
	5086	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.3;
	5086	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.5;
	5086	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	276.4;
	5086	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	281.4;
	5086	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.8;
	5086	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.4;
	5087	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525.7;
	5087	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.8;
	5087	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.8;
	5087	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.1;
	5087	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.1;
	5087	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.3;
	5088	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.5;
	5088	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.1;
	5088	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.9;
	5088	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.8;
	5088	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.2;
	5088	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378;
	5089	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.7;
	5089	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.3;
	5089	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.7;
	5089	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232;
	5089	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	5089	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.4;
	5090	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.1;
	5090	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.5;
	5090	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.9;
	5090	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.9;
	5090	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	5090	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.7;
	5091	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.7;
	5091	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.1;
	5091	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.8;
	5091	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214;
	5091	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	5091	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.1;
	5092	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391.5;
	5092	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.5;
	5092	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.6;
	5092	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.2;
	5092	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	5092	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.6;
	5093	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.8;
	5093	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.5;
	5093	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.1;
	5093	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.1;
	5093	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	5093	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.1;
	5094	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.2;
	5094	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.8;
	5094	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.7;
	5094	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.9;
	5094	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	5094	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.8;
	5095	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.3;
	5095	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.1;
	5095	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.1;
	5095	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.2;
	5095	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	5095	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.7;
	5096	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.5;
	5096	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.1;
	5096	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.5;
	5096	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.1;
	5096	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	5096	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.1;
	5097	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.1;
	5097	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159;
	5097	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	266.8;
	5097	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	273.6;
	5097	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	5097	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.2;
	5098	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.3;
	5098	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.6;
	5098	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	281.6;
	5098	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	288.8;
	5098	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.1;
	5098	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.2;
	5099	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.5;
	5099	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.3;
	5099	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	291.4;
	5099	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	299.4;
	5099	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.2;
	5099	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.7;
	5100	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	582;
	5100	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.5;
	5100	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	303.9;
	5100	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	312.1;
	5100	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.9;
	5100	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	439;
	5101	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.2;
	5101	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.7;
	5101	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	313.4;
	5101	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	322;
	5101	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.8;
	5101	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	449.5;
	5102	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	607.3;
	5102	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.2;
	5102	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	319.6;
	5102	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	328.9;
	5102	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.1;
	5102	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	455.8;
	5103	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	619.5;
	5103	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.9;
	5103	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	327.6;
	5103	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	337;
	5103	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.2;
	5103	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	463.1;
	5104	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	617.2;
	5104	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.7;
	5104	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	326;
	5104	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	335.9;
	5104	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.2;
	5104	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	462.5;
	5105	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	618.2;
	5105	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.1;
	5105	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	327.9;
	5105	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	336.4;
	5105	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.2;
	5105	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	462.5;
	5106	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	614.3;
	5106	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.5;
	5106	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	320.1;
	5106	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	328.4;
	5106	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.4;
	5106	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	458.6;
	5107	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	610.5;
	5107	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.1;
	5107	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	311.5;
	5107	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	317.1;
	5107	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.6;
	5107	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	453.5;
	5108	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	593.2;
	5108	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.3;
	5108	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	304.6;
	5108	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	310.2;
	5108	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.8;
	5108	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	443.5;
	5109	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	581;
	5109	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.1;
	5109	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	294.9;
	5109	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	299.7;
	5109	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.8;
	5109	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	435.3;
	5110	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	554.8;
	5110	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.4;
	5110	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	278.3;
	5110	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	281.4;
	5110	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87;
	5110	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.8;
	5111	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515;
	5111	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.4;
	5111	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.4;
	5111	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.6;
	5111	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.8;
	5111	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.8;
	5112	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.5;
	5112	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.6;
	5112	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.3;
	5112	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250;
	5112	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.7;
	5112	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.1;
	5113	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448;
	5113	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.1;
	5113	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.5;
	5113	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.1;
	5113	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.9;
	5113	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.1;
	5114	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.6;
	5114	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.8;
	5114	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214;
	5114	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.8;
	5114	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.2;
	5114	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.2;
	5115	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.9;
	5115	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.2;
	5115	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.5;
	5115	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.4;
	5115	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.3;
	5115	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.6;
	5116	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	388.8;
	5116	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.4;
	5116	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.8;
	5116	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.6;
	5116	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	5116	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.8;
	5117	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.7;
	5117	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.1;
	5117	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.2;
	5117	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.2;
	5117	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	5117	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.6;
	5118	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417.6;
	5118	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.8;
	5118	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.9;
	5118	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.2;
	5118	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	5118	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.3;
	5119	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.5;
	5119	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.4;
	5119	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.3;
	5119	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.7;
	5119	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	5119	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357;
	5120	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.7;
	5120	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.4;
	5120	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.4;
	5120	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.1;
	5120	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.4;
	5120	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.5;
	5121	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.1;
	5121	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.5;
	5121	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	263.7;
	5121	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	268.7;
	5121	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.2;
	5121	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.7;
	5122	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537.4;
	5122	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.1;
	5122	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	278.7;
	5122	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	283.8;
	5122	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.6;
	5122	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.2;
	5123	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.1;
	5123	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.8;
	5123	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	293.9;
	5123	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	298.6;
	5123	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.4;
	5123	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	428.3;
	5124	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	587.1;
	5124	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.3;
	5124	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	308.2;
	5124	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	313;
	5124	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.3;
	5124	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	441.7;
	5125	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	599.8;
	5125	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.7;
	5125	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	319.8;
	5125	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	325.6;
	5125	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.4;
	5125	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	451.6;
	5126	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	620.2;
	5126	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.3;
	5126	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	330.2;
	5126	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	335.3;
	5126	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96.2;
	5126	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	463.6;
	5127	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	630.8;
	5127	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.7;
	5127	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	333.4;
	5127	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	336.8;
	5127	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	97.9;
	5127	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	469.6;
	5128	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	627.2;
	5128	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.1;
	5128	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	332.9;
	5128	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	333.7;
	5128	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	99.8;
	5128	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	468.1;
	5129	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	630.4;
	5129	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.5;
	5129	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	333.6;
	5129	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	331.6;
	5129	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	99.9;
	5129	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	469.1;
	5130	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	622.9;
	5130	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.9;
	5130	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	324.1;
	5130	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	321.6;
	5130	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	99.7;
	5130	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	463.4;
	5131	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	611;
	5131	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.9;
	5131	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	314.3;
	5131	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	310.9;
	5131	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96.9;
	5131	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	454.7;
	5132	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	594.1;
	5132	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.2;
	5132	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	306.9;
	5132	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	305.3;
	5132	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.1;
	5132	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	444.6;
	5133	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	575.9;
	5133	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.5;
	5133	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	299.6;
	5133	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	297.7;
	5133	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.4;
	5133	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	434.2;
	5134	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.5;
	5134	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.9;
	5134	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	283.2;
	5134	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	281.4;
	5134	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.9;
	5134	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.6;
	5135	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528.6;
	5135	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.2;
	5135	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.6;
	5135	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.2;
	5135	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.3;
	5135	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.8;
	5136	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.8;
	5136	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.4;
	5136	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.7;
	5136	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.2;
	5136	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.4;
	5136	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.6;
	5137	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.4;
	5137	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.2;
	5137	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.5;
	5137	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229;
	5137	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.3;
	5137	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369;
	5138	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.2;
	5138	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132;
	5138	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.8;
	5138	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.4;
	5138	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	5138	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.8;
	5139	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	388.2;
	5139	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.9;
	5139	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.9;
	5139	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.5;
	5139	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	5139	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.9;
	5140	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	390.1;
	5140	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.3;
	5140	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.5;
	5140	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.4;
	5140	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.3;
	5140	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.9;
	5141	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.1;
	5141	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.2;
	5141	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215;
	5141	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.4;
	5141	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.7;
	5141	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.6;
	5142	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.1;
	5142	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.3;
	5142	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.6;
	5142	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.8;
	5142	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	5142	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.9;
	5143	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.6;
	5143	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148;
	5143	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.2;
	5143	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.4;
	5143	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	5143	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365;
	5144	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.4;
	5144	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.8;
	5144	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.4;
	5144	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.6;
	5144	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	5144	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379;
	5145	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.7;
	5145	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.1;
	5145	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	272.9;
	5145	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	276.1;
	5145	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	5145	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.7;
	5146	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	534.5;
	5146	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.4;
	5146	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	285.9;
	5146	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	288;
	5146	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.5;
	5146	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415;
	5147	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.1;
	5147	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.3;
	5147	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	298.1;
	5147	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	300.3;
	5147	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.8;
	5147	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.4;
	5148	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.6;
	5148	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.7;
	5148	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	307.7;
	5148	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	310.3;
	5148	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.1;
	5148	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	434.6;
	5149	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	580.9;
	5149	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.3;
	5149	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	316.4;
	5149	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	318.7;
	5149	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.2;
	5149	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	444.2;
	5150	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	581.9;
	5150	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.7;
	5150	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	324.8;
	5150	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	326.7;
	5150	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.3;
	5150	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	447.7;
	5151	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.1;
	5151	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.7;
	5151	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	334.5;
	5151	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	335.3;
	5151	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.2;
	5151	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	451.2;
	5152	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.2;
	5152	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.7;
	5152	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	341.3;
	5152	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	341.5;
	5152	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.8;
	5152	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	456.7;
	5153	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	590.7;
	5153	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.7;
	5153	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	334.4;
	5153	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	334.6;
	5153	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.6;
	5153	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	455.2;
	5154	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	591.9;
	5154	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.2;
	5154	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	322.1;
	5154	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	322.2;
	5154	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.3;
	5154	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	451.7;
	5155	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	576.4;
	5155	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.5;
	5155	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	310.8;
	5155	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	311.3;
	5155	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.5;
	5155	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	440.7;
	5156	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548.6;
	5156	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.3;
	5156	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	301.3;
	5156	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	302.9;
	5156	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.1;
	5156	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.5;
	5157	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.9;
	5157	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.4;
	5157	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	295.3;
	5157	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	296.9;
	5157	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.3;
	5157	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411;
	5158	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.5;
	5158	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.1;
	5158	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	282.2;
	5158	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	282.7;
	5158	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.3;
	5158	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.2;
	5159	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.1;
	5159	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160;
	5159	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262;
	5159	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.5;
	5159	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.9;
	5159	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.7;
	5160	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.7;
	5160	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.1;
	5160	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.4;
	5160	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.6;
	5160	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	5160	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.5;
	5161	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	426;
	5161	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.1;
	5161	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.3;
	5161	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.1;
	5161	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	5161	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.6;
	5162	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	382.2;
	5162	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.6;
	5162	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.3;
	5162	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.2;
	5162	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	5162	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.2;
	5163	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	363.4;
	5163	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127;
	5163	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.1;
	5163	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.3;
	5163	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60;
	5163	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.9;
	5164	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	358;
	5164	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.1;
	5164	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.8;
	5164	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.4;
	5164	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.8;
	5164	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318;
	5165	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	371.2;
	5165	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.5;
	5165	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.7;
	5165	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.7;
	5165	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.7;
	5165	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324;
	5166	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	401.8;
	5166	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.1;
	5166	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.6;
	5166	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.3;
	5166	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	5166	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.6;
	5167	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.9;
	5167	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144;
	5167	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.4;
	5167	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.5;
	5167	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	5167	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.4;
	5168	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.6;
	5168	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153;
	5168	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.7;
	5168	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.2;
	5168	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	5168	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.8;
	5169	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.5;
	5169	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.7;
	5169	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	268.8;
	5169	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.9;
	5169	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	5169	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.1;
	5170	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.8;
	5170	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.5;
	5170	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	284.3;
	5170	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	289.1;
	5170	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.7;
	5170	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.3;
	5171	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.6;
	5171	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.1;
	5171	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	297.5;
	5171	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	302.5;
	5171	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.9;
	5171	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420.5;
	5172	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.2;
	5172	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.9;
	5172	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	312.2;
	5172	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	316.9;
	5172	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.6;
	5172	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	433;
	5173	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579.1;
	5173	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.1;
	5173	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	326.5;
	5173	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	330.6;
	5173	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.9;
	5173	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	446.6;
	5174	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586;
	5174	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.2;
	5174	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	334.4;
	5174	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	338.1;
	5174	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.2;
	5174	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	452.5;
	5175	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	596.5;
	5175	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.3;
	5175	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	335.9;
	5175	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	338.4;
	5175	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.6;
	5175	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	457.5;
	5176	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	604.2;
	5176	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.1;
	5176	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	334.1;
	5176	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	338.1;
	5176	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.1;
	5176	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	460;
	5177	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	604.3;
	5177	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.9;
	5177	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	330.6;
	5177	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	334.6;
	5177	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.4;
	5177	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	459;
	5178	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	596.3;
	5178	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190;
	5178	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	324.4;
	5178	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	327.6;
	5178	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85;
	5178	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	453.2;
	5179	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	578.8;
	5179	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.5;
	5179	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	314.1;
	5179	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	317;
	5179	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.3;
	5179	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	442.4;
	5180	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.5;
	5180	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.4;
	5180	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	305.2;
	5180	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	307.3;
	5180	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.9;
	5180	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	429.6;
	5181	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.7;
	5181	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.2;
	5181	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	294.8;
	5181	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	297.2;
	5181	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.2;
	5181	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.2;
	5182	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.4;
	5182	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.7;
	5182	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	279.1;
	5182	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	281.1;
	5182	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.3;
	5182	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.2;
	5183	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.8;
	5183	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.1;
	5183	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.2;
	5183	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.7;
	5183	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	5183	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.3;
	5184	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.8;
	5184	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.5;
	5184	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.3;
	5184	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.3;
	5184	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	5184	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.3;
	5185	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.1;
	5185	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.8;
	5185	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.8;
	5185	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.2;
	5185	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	5185	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.6;
	5186	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	381.1;
	5186	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.3;
	5186	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.9;
	5186	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.7;
	5186	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.2;
	5186	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.1;
	5187	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	375.3;
	5187	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.6;
	5187	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.1;
	5187	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216;
	5187	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.5;
	5187	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.6;
	5188	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	368;
	5188	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.6;
	5188	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.9;
	5188	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.1;
	5188	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.6;
	5188	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.7;
	5189	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	372.4;
	5189	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.9;
	5189	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.1;
	5189	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.2;
	5189	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.2;
	5189	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.1;
	5190	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	377.8;
	5190	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.5;
	5190	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.5;
	5190	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.8;
	5190	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.5;
	5190	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.9;
	5191	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	388.5;
	5191	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.8;
	5191	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.5;
	5191	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.3;
	5191	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.6;
	5191	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.5;
	5192	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403.9;
	5192	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.9;
	5192	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.9;
	5192	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.9;
	5192	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	5192	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.9;
	5193	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.1;
	5193	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151;
	5193	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241;
	5193	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.9;
	5193	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	5193	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.4;
	5194	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.5;
	5194	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.9;
	5194	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.5;
	5194	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260;
	5194	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.1;
	5194	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375;
	5195	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.3;
	5195	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.8;
	5195	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	268;
	5195	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.8;
	5195	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.1;
	5195	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.5;
	5196	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510;
	5196	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.8;
	5196	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	277.4;
	5196	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	282;
	5196	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.8;
	5196	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.4;
	5197	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	524.7;
	5197	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.5;
	5197	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	285.1;
	5197	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	290.5;
	5197	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.2;
	5197	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.8;
	5198	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528.8;
	5198	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.1;
	5198	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	292.7;
	5198	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	297.4;
	5198	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.9;
	5198	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.1;
	5199	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.9;
	5199	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.3;
	5199	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	298.4;
	5199	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	301;
	5199	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.1;
	5199	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.7;
	5200	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	534;
	5200	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.1;
	5200	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	300.4;
	5200	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	301.6;
	5200	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.7;
	5200	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.8;
	5201	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.1;
	5201	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.5;
	5201	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	298.5;
	5201	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	298.7;
	5201	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.1;
	5201	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.4;
	5202	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522.7;
	5202	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.1;
	5202	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	286.3;
	5202	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	286.2;
	5202	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87;
	5202	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.5;
	5203	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.3;
	5203	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.1;
	5203	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	278.5;
	5203	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	279.1;
	5203	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.4;
	5203	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.6;
	5204	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.3;
	5204	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.9;
	5204	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	274.8;
	5204	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	276.9;
	5204	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83;
	5204	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389;
	5205	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.9;
	5205	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.6;
	5205	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	269.8;
	5205	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.2;
	5205	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.2;
	5205	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.4;
	5206	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.7;
	5206	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.6;
	5206	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.5;
	5206	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.3;
	5206	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.2;
	5206	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372;
	5207	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.2;
	5207	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146;
	5207	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.3;
	5207	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.3;
	5207	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.4;
	5207	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.9;
	5208	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.8;
	5208	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.7;
	5208	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.9;
	5208	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.4;
	5208	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	5208	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347;
	5209	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	382.2;
	5209	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.3;
	5209	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222;
	5209	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.9;
	5209	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	5209	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.1;
	5210	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	346.3;
	5210	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.3;
	5210	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.4;
	5210	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.3;
	5210	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.4;
	5210	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.6;
	5211	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	335;
	5211	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.8;
	5211	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.9;
	5211	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213;
	5211	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.5;
	5211	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	307.5;
	5212	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	328.8;
	5212	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.9;
	5212	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208;
	5212	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.2;
	5212	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.5;
	5212	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	304.1;
	5213	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	336;
	5213	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.9;
	5213	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.1;
	5213	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.9;
	5213	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.4;
	5213	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	306.9;
	5214	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	343.1;
	5214	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.1;
	5214	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.4;
	5214	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.2;
	5214	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57;
	5214	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310.5;
	5215	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	357.3;
	5215	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123;
	5215	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211;
	5215	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.5;
	5215	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.7;
	5215	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.7;
	5216	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	365.4;
	5216	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.1;
	5216	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.5;
	5216	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.1;
	5216	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.4;
	5216	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.5;
	5217	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	389.2;
	5217	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.1;
	5217	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.6;
	5217	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.2;
	5217	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	5217	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337;
	5218	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.5;
	5218	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.1;
	5218	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.6;
	5218	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247;
	5218	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.1;
	5218	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.3;
	5219	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.7;
	5219	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156;
	5219	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.9;
	5219	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260;
	5219	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.9;
	5219	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.9;
	5220	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.3;
	5220	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.5;
	5220	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.9;
	5220	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	270.4;
	5220	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.5;
	5220	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.4;
	5221	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.7;
	5221	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.8;
	5221	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	269.4;
	5221	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	278.4;
	5221	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.6;
	5221	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381;
	5222	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.4;
	5222	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.7;
	5222	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	277;
	5222	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	285.9;
	5222	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.6;
	5222	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.6;
	5223	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482;
	5223	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.4;
	5223	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	283.6;
	5223	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	294.5;
	5223	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.9;
	5223	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.4;
	5224	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.7;
	5224	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168;
	5224	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	285.6;
	5224	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	296.2;
	5224	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.2;
	5224	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.4;
	5225	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.7;
	5225	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.5;
	5225	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	283.4;
	5225	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	294.3;
	5225	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.7;
	5225	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.8;
	5226	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518.5;
	5226	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.7;
	5226	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	286.7;
	5226	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	295.7;
	5226	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.2;
	5226	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.2;
	5227	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.6;
	5227	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.2;
	5227	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	283.7;
	5227	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	293.6;
	5227	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.8;
	5227	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.5;
	5228	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.6;
	5228	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.3;
	5228	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	275.7;
	5228	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	286.6;
	5228	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.9;
	5228	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.1;
	5229	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.4;
	5229	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.7;
	5229	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	272.8;
	5229	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	281.7;
	5229	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.7;
	5229	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.1;
	5230	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.7;
	5230	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.6;
	5230	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261;
	5230	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	266.8;
	5230	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.4;
	5230	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.9;
	5231	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.9;
	5231	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.3;
	5231	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.9;
	5231	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.9;
	5231	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.9;
	5231	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.3;
	5232	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	430.3;
	5232	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.4;
	5232	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.8;
	5232	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.3;
	5232	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	5232	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.2;
	5233	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	401.2;
	5233	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.8;
	5233	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.1;
	5233	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.5;
	5233	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.8;
	5233	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.3;
	5234	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	369.7;
	5234	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.6;
	5234	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.4;
	5234	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.1;
	5234	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	5234	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.9;
	5235	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	359.4;
	5235	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.4;
	5235	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.9;
	5235	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.7;
	5235	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	5235	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.6;
	5236	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	349;
	5236	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.5;
	5236	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.2;
	5236	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.8;
	5236	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	5236	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311.2;
	5237	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	352.7;
	5237	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.7;
	5237	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.1;
	5237	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.1;
	5237	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	5237	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.5;
	5238	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	369.7;
	5238	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.5;
	5238	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.5;
	5238	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220;
	5238	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	5238	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.5;
	5239	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391.7;
	5239	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.2;
	5239	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.3;
	5239	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.5;
	5239	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	5239	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.4;
	5240	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.2;
	5240	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155;
	5240	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.9;
	5240	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.7;
	5240	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	5240	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.3;
	5241	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.4;
	5241	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.8;
	5241	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	266.7;
	5241	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	275.2;
	5241	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	5241	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377;
	5242	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479;
	5242	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.3;
	5242	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	278.8;
	5242	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	288.1;
	5242	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.4;
	5242	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.9;
	5243	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.3;
	5243	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171;
	5243	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	287.4;
	5243	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	298.1;
	5243	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.1;
	5243	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.2;
	5244	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537.1;
	5244	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.1;
	5244	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	295.2;
	5244	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	306.9;
	5244	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.7;
	5244	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.2;
	5245	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.9;
	5245	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.2;
	5245	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	304.8;
	5245	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	316.1;
	5245	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.8;
	5245	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.6;
	5246	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	554.9;
	5246	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.4;
	5246	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	311.5;
	5246	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	323.2;
	5246	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.2;
	5246	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	432.1;
	5247	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.7;
	5247	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.3;
	5247	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	319.7;
	5247	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	331.7;
	5247	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.1;
	5247	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	441.1;
	5248	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579.8;
	5248	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.2;
	5248	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	324.3;
	5248	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	336.1;
	5248	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.3;
	5248	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	445.7;
	5249	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	574.5;
	5249	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.8;
	5249	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	326.8;
	5249	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	334.8;
	5249	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.7;
	5249	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	444.3;
	5250	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566;
	5250	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.4;
	5250	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	320.8;
	5250	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	326.8;
	5250	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.1;
	5250	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	439.3;
	5251	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	563.2;
	5251	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.7;
	5251	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	304.5;
	5251	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	311.3;
	5251	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90;
	5251	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	433.9;
	5252	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547.6;
	5252	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.3;
	5252	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	297.7;
	5252	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	303.4;
	5252	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89;
	5252	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.9;
	5253	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.2;
	5253	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.5;
	5253	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	289.4;
	5253	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	294.5;
	5253	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.4;
	5253	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.7;
	5254	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518;
	5254	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.1;
	5254	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	274.8;
	5254	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	279.9;
	5254	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.3;
	5254	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.4;
	5255	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.3;
	5255	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.2;
	5255	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.6;
	5255	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.4;
	5255	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.8;
	5255	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.6;
	5256	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.4;
	5256	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.8;
	5256	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.7;
	5256	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.5;
	5256	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.9;
	5256	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.2;
	5257	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.8;
	5257	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.9;
	5257	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.8;
	5257	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230;
	5257	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73;
	5257	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.1;
	5258	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	366.6;
	5258	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.9;
	5258	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.4;
	5258	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.7;
	5258	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.3;
	5258	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.6;
	5259	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	335.4;
	5259	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.9;
	5259	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.1;
	5259	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.9;
	5259	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	5259	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	306.5;
	5260	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	334.8;
	5260	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.7;
	5260	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.7;
	5260	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.5;
	5260	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	5260	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	305.7;
	5261	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	360.3;
	5261	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.1;
	5261	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.1;
	5261	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.1;
	5261	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	5261	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.8;
	5262	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	396.8;
	5262	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.2;
	5262	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.6;
	5262	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.5;
	5262	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	5262	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.5;
	5263	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.3;
	5263	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.6;
	5263	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.8;
	5263	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.8;
	5263	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	5263	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.3;
	5264	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.3;
	5264	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.9;
	5264	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.3;
	5264	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.6;
	5264	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	5264	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.8;
	5265	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.5;
	5265	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.2;
	5265	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.6;
	5265	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	270;
	5265	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	5265	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.9;
	5266	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.8;
	5266	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.9;
	5266	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	274.3;
	5266	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	282;
	5266	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	5266	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.7;
	5267	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.7;
	5267	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.3;
	5267	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	287.5;
	5267	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	295.6;
	5267	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.8;
	5267	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.9;
	5268	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.3;
	5268	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.7;
	5268	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	296.9;
	5268	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	306;
	5268	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.4;
	5268	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.9;
	5269	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566.9;
	5269	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.4;
	5269	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	309.9;
	5269	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	316.6;
	5269	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.1;
	5269	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	434.6;
	5270	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	583.4;
	5270	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.4;
	5270	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	321.1;
	5270	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	325.2;
	5270	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.6;
	5270	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	445.1;
	5271	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	610;
	5271	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.1;
	5271	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	328.2;
	5271	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	329.3;
	5271	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.1;
	5271	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	458.6;
	5272	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	615.3;
	5272	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.4;
	5272	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	328.8;
	5272	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	327.3;
	5272	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.2;
	5272	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	461.4;
	5273	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	607.8;
	5273	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.4;
	5273	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	328.5;
	5273	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	325;
	5273	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.1;
	5273	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	458.5;
	5274	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	607.8;
	5274	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.1;
	5274	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	319.5;
	5274	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	318.2;
	5274	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.7;
	5274	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	456.1;
	5275	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	600;
	5275	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182;
	5275	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	306.3;
	5275	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	305.4;
	5275	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.6;
	5275	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	448.4;
	5276	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579;
	5276	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.4;
	5276	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	297.9;
	5276	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	298.6;
	5276	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.2;
	5276	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	435.8;
	5277	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.8;
	5277	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.2;
	5277	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	288.1;
	5277	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	289.3;
	5277	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.1;
	5277	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.1;
	5278	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545;
	5278	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.2;
	5278	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	272.4;
	5278	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	274.7;
	5278	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.3;
	5278	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.7;
	5279	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.5;
	5279	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.6;
	5279	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.3;
	5279	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.4;
	5279	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.3;
	5279	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.1;
	5280	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.2;
	5280	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.3;
	5280	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.8;
	5280	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246;
	5280	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.5;
	5280	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370;
	5281	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.1;
	5281	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.3;
	5281	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.8;
	5281	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.8;
	5281	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	5281	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.5;
	5282	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.3;
	5282	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.4;
	5282	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.4;
	5282	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214;
	5282	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	5282	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.6;
	5283	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	386.8;
	5283	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.3;
	5283	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.3;
	5283	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.6;
	5283	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	5283	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.9;
	5284	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	378;
	5284	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.3;
	5284	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.2;
	5284	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.9;
	5284	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.5;
	5284	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.9;
	5285	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	401.8;
	5285	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.1;
	5285	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.9;
	5285	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.6;
	5285	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.1;
	5285	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.4;
	5286	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.7;
	5286	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131;
	5286	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.3;
	5286	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.4;
	5286	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.8;
	5286	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.6;
	5287	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.6;
	5287	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.9;
	5287	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.2;
	5287	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.9;
	5287	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.1;
	5287	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.9;
	5288	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.3;
	5288	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.5;
	5288	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.1;
	5288	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.9;
	5288	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	5288	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.7;
	5289	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.9;
	5289	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.9;
	5289	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.6;
	5289	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267.7;
	5289	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.2;
	5289	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.4;
	5290	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.4;
	5290	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.6;
	5290	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	275;
	5290	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	281.8;
	5290	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.2;
	5290	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.8;
	5291	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.4;
	5291	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.3;
	5291	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	284.4;
	5291	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	291.1;
	5291	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.5;
	5291	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.6;
	5292	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	575.3;
	5292	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.8;
	5292	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	295.1;
	5292	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	300.5;
	5292	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.8;
	5292	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	432.7;
	5293	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	594.3;
	5293	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.4;
	5293	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	305.5;
	5293	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	311.6;
	5293	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.2;
	5293	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	444.4;
	5294	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	610.7;
	5294	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.6;
	5294	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	313.3;
	5294	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	319.2;
	5294	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.6;
	5294	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	453.9;
	5295	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	617.7;
	5295	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.5;
	5295	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	319.8;
	5295	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	325.7;
	5295	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.9;
	5295	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	458.8;
	5296	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	609.7;
	5296	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.7;
	5296	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	325.2;
	5296	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	330.6;
	5296	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.7;
	5296	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	457.2;
	5297	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	611.2;
	5297	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.4;
	5297	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	323;
	5297	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	329.6;
	5297	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.4;
	5297	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	457.9;
	5298	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	608.8;
	5298	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.7;
	5298	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	308.9;
	5298	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	315.6;
	5298	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.7;
	5298	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	453.3;
	5299	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	590.7;
	5299	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.1;
	5299	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	300;
	5299	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	305.3;
	5299	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.6;
	5299	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	441.7;
	5300	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	569.2;
	5300	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.7;
	5300	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	293.3;
	5300	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	297.9;
	5300	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.3;
	5300	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	429.7;
	5301	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.2;
	5301	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.1;
	5301	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	283.4;
	5301	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	288.2;
	5301	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.5;
	5301	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.2;
	5302	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.8;
	5302	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.8;
	5302	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	267.8;
	5302	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	271.3;
	5302	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.3;
	5302	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.9;
	5303	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.2;
	5303	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.8;
	5303	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.2;
	5303	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252;
	5303	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.7;
	5303	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.8;
	5304	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462;
	5304	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.3;
	5304	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.5;
	5304	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.3;
	5304	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.4;
	5304	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.2;
	5305	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.1;
	5305	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.7;
	5305	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.3;
	5305	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.4;
	5305	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	5305	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.8;
	5306	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.7;
	5306	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.3;
	5306	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.2;
	5306	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.4;
	5306	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.2;
	5306	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.7;
	5307	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	385.9;
	5307	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.5;
	5307	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.2;
	5307	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.4;
	5307	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.1;
	5307	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.4;
	5308	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391.6;
	5308	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.7;
	5308	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.4;
	5308	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.5;
	5308	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.1;
	5308	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.6;
	5309	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	401.2;
	5309	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.2;
	5309	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207;
	5309	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.5;
	5309	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.7;
	5309	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.4;
	5310	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.8;
	5310	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.7;
	5310	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.2;
	5310	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.7;
	5310	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.7;
	5310	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.4;
	5311	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.1;
	5311	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.2;
	5311	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.1;
	5311	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.5;
	5311	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.4;
	5311	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.8;
	5312	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.6;
	5312	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.3;
	5312	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.8;
	5312	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.7;
	5312	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	5312	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.1;
	5313	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.4;
	5313	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.9;
	5313	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.2;
	5313	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265.5;
	5313	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	5313	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.7;
	5314	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528.6;
	5314	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163;
	5314	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	275.3;
	5314	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	279.3;
	5314	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.7;
	5314	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408;
	5315	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.6;
	5315	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.7;
	5315	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	285.5;
	5315	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	289.2;
	5315	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.8;
	5315	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.3;
	5316	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.1;
	5316	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.4;
	5316	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	297.5;
	5316	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	300.1;
	5316	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.4;
	5316	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	426.8;
	5317	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	568.1;
	5317	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.7;
	5317	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	303.9;
	5317	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	306.9;
	5317	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.4;
	5317	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	434.4;
	5318	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	581.9;
	5318	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.7;
	5318	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	311.9;
	5318	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	314.4;
	5318	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.1;
	5318	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	442.7;
	5319	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.3;
	5319	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.5;
	5319	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	318.2;
	5319	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	320.4;
	5319	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.9;
	5319	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	448.7;
	5320	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	600.2;
	5320	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.1;
	5320	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	323.7;
	5320	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	325.4;
	5320	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.3;
	5320	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	453.3;
	5321	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	605.7;
	5321	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.5;
	5321	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	317.1;
	5321	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	319.1;
	5321	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.9;
	5321	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	454.6;
	5322	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	594.7;
	5322	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.9;
	5322	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	308.4;
	5322	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	309.5;
	5322	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.2;
	5322	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	446.9;
	5323	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.2;
	5323	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.8;
	5323	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	296.3;
	5323	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	297.2;
	5323	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.7;
	5323	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	432.1;
	5324	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	553.4;
	5324	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.7;
	5324	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	288.3;
	5324	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	289.2;
	5324	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.2;
	5324	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.5;
	5325	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.7;
	5325	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.4;
	5325	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	279.6;
	5325	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	280.3;
	5325	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.4;
	5325	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.1;
	5326	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	524.2;
	5326	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.1;
	5326	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	266;
	5326	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265.9;
	5326	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.4;
	5326	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.6;
	5327	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494;
	5327	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.2;
	5327	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.5;
	5327	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.2;
	5327	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.9;
	5327	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.8;
	5328	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468;
	5328	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.7;
	5328	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.4;
	5328	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.3;
	5328	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.5;
	5328	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.9;
	5329	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442.5;
	5329	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.4;
	5329	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.5;
	5329	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.4;
	5329	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	5329	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.5;
	5330	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.5;
	5330	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.3;
	5330	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.2;
	5330	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.8;
	5330	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	5330	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.5;
	5331	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.6;
	5331	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.2;
	5331	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.9;
	5331	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.5;
	5331	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	5331	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.4;
	5332	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.4;
	5332	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.2;
	5332	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.4;
	5332	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208;
	5332	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.7;
	5332	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.7;
	5333	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.4;
	5333	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.2;
	5333	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211;
	5333	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.8;
	5333	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	5333	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.8;
	5334	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	427.6;
	5334	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.6;
	5334	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.5;
	5334	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.5;
	5334	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.2;
	5334	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.1;
	5335	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456;
	5335	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.6;
	5335	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.2;
	5335	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.6;
	5335	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	5335	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.3;
	5336	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.7;
	5336	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.4;
	5336	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.2;
	5336	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.7;
	5336	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.1;
	5336	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.1;
	5337	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.1;
	5337	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.3;
	5337	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	264.7;
	5337	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	266.2;
	5337	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	5337	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.2;
	5338	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.7;
	5338	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.4;
	5338	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	276.2;
	5338	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	277.6;
	5338	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.1;
	5338	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.4;
	5339	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.8;
	5339	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.5;
	5339	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	285.7;
	5339	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	287.4;
	5339	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.8;
	5339	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.6;
	5340	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.5;
	5340	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.3;
	5340	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	290.5;
	5340	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	291.8;
	5340	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.3;
	5340	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.8;
	5341	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	576.7;
	5341	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.6;
	5341	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	294.6;
	5341	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	296.2;
	5341	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.4;
	5341	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	435;
	5342	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.2;
	5342	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.1;
	5342	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	300.9;
	5342	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	303.4;
	5342	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96.3;
	5342	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	440;
	5343	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	594.9;
	5343	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.7;
	5343	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	307.8;
	5343	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	310.2;
	5343	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	99;
	5343	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	446.2;
	5344	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.2;
	5344	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.3;
	5344	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	313.3;
	5344	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	315.2;
	5344	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	98.5;
	5344	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	448.6;
	5345	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589.3;
	5345	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.6;
	5345	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	311.2;
	5345	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	312.9;
	5345	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	100.5;
	5345	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	445.2;
	5346	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	581.8;
	5346	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.1;
	5346	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	302.9;
	5346	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	304.2;
	5346	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	99.1;
	5346	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	439.5;
	5347	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564;
	5347	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.7;
	5347	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	293.4;
	5347	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	293.9;
	5347	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	98.8;
	5347	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	428.5;
	5348	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.4;
	5348	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.5;
	5348	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	283.7;
	5348	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	284.8;
	5348	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	97;
	5348	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.6;
	5349	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	534.5;
	5349	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.2;
	5349	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	277.8;
	5349	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	277.4;
	5349	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.2;
	5349	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.3;
	5350	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.3;
	5350	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.5;
	5350	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	265.1;
	5350	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265;
	5350	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.1;
	5350	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.8;
	5351	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.8;
	5351	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147;
	5351	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.3;
	5351	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.8;
	5351	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.3;
	5351	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.6;
	5352	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.1;
	5352	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.2;
	5352	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243;
	5352	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.9;
	5352	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.1;
	5352	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.3;
	5353	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.8;
	5353	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.5;
	5353	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.4;
	5353	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.3;
	5353	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.1;
	5353	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.1;
	5354	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	386.4;
	5354	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.6;
	5354	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.7;
	5354	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.3;
	5354	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73;
	5354	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.6;
	5355	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379.7;
	5355	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.6;
	5355	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208;
	5355	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.8;
	5355	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	5355	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.5;
	5356	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	374.4;
	5356	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.1;
	5356	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.4;
	5356	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.4;
	5356	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.3;
	5356	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.6;
	5357	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	378;
	5357	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.6;
	5357	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.3;
	5357	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.2;
	5357	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.1;
	5357	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.5;
	5358	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392.1;
	5358	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.6;
	5358	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.4;
	5358	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.5;
	5358	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.8;
	5358	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.6;
	5359	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.3;
	5359	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128;
	5359	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.8;
	5359	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.7;
	5359	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.1;
	5359	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.2;
	5360	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.6;
	5360	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.6;
	5360	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.3;
	5360	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.9;
	5360	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.3;
	5360	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.7;
	5361	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	430.4;
	5361	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.8;
	5361	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.3;
	5361	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.7;
	5361	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.8;
	5361	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355;
	5362	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.9;
	5362	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.4;
	5362	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249;
	5362	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.1;
	5362	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83;
	5362	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.9;
	5363	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.5;
	5363	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.4;
	5363	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.7;
	5363	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	263.1;
	5363	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.6;
	5363	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.1;
	5364	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.3;
	5364	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.2;
	5364	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	272.9;
	5364	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	273.9;
	5364	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94;
	5364	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392;
	5365	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.8;
	5365	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.5;
	5365	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	281.4;
	5365	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	282.8;
	5365	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	97.9;
	5365	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.7;
	5366	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.8;
	5366	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.8;
	5366	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	289.9;
	5366	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	291.1;
	5366	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	100.7;
	5366	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.9;
	5367	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507;
	5367	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.4;
	5367	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	300.1;
	5367	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	300.6;
	5367	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	103.5;
	5367	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.6;
	5368	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.3;
	5368	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.4;
	5368	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	302.2;
	5368	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	301.6;
	5368	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	104.6;
	5368	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.2;
	5369	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.8;
	5369	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173;
	5369	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	302;
	5369	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	301.4;
	5369	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	105.3;
	5369	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.4;
	5370	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.6;
	5370	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.4;
	5370	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	289.3;
	5370	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	287.8;
	5370	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	105.1;
	5370	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.5;
	5371	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.8;
	5371	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.6;
	5371	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	284.6;
	5371	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	283;
	5371	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	102.5;
	5371	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397;
	5372	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.8;
	5372	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.8;
	5372	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	280.5;
	5372	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	280.1;
	5372	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	100.8;
	5372	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.5;
	5373	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465;
	5373	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.8;
	5373	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	272.9;
	5373	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	273.4;
	5373	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96.3;
	5373	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381;
	5374	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447.1;
	5374	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.7;
	5374	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.7;
	5374	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.5;
	5374	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.4;
	5374	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.9;
	5375	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.4;
	5375	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.5;
	5375	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.8;
	5375	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.6;
	5375	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.2;
	5375	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.4;
	5376	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.5;
	5376	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.8;
	5376	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.6;
	5376	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.9;
	5376	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.1;
	5376	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340;
	5377	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	370.5;
	5377	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125;
	5377	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.6;
	5377	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.9;
	5377	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.8;
	5377	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.7;
	5378	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	340.2;
	5378	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.1;
	5378	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.2;
	5378	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.4;
	5378	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.5;
	5378	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309.8;
	5379	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	328.4;
	5379	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.5;
	5379	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.3;
	5379	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.6;
	5379	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.2;
	5379	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	303;
	5380	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	323.8;
	5380	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.1;
	5380	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.7;
	5380	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.4;
	5380	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	5380	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	300;
	5381	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	328.4;
	5381	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.7;
	5381	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.1;
	5381	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.7;
	5381	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.9;
	5381	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	301.7;
	5382	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	336.5;
	5382	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.7;
	5382	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201;
	5382	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.9;
	5382	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	5382	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	305.4;
	5383	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	341.5;
	5383	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.5;
	5383	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202;
	5383	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206;
	5383	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.4;
	5383	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.3;
	5384	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	342.9;
	5384	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.9;
	5384	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.6;
	5384	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.9;
	5384	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.9;
	5384	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311.3;
	5385	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	363.9;
	5385	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.7;
	5385	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.5;
	5385	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.4;
	5385	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.4;
	5385	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.5;
	5386	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	390.9;
	5386	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.3;
	5386	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.5;
	5386	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.1;
	5386	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.9;
	5386	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.5;
	5387	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.8;
	5387	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.9;
	5387	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.1;
	5387	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.6;
	5387	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.2;
	5387	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.6;
	5388	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.9;
	5388	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.9;
	5388	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.3;
	5388	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.3;
	5388	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.1;
	5388	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.6;
	5389	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.8;
	5389	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.7;
	5389	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	270.3;
	5389	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	268.9;
	5389	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	98.6;
	5389	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.1;
	5390	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488;
	5390	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.1;
	5390	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	281.4;
	5390	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	277.6;
	5390	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	101.9;
	5390	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.3;
	5391	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497;
	5391	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.9;
	5391	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	288.8;
	5391	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	283.5;
	5391	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	103.4;
	5391	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.8;
	5392	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.8;
	5392	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.9;
	5392	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	291.9;
	5392	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	286.2;
	5392	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	105.2;
	5392	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.1;
	5393	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.6;
	5393	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.5;
	5393	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	291.1;
	5393	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	284.7;
	5393	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	105.7;
	5393	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.2;
	5394	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.3;
	5394	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168;
	5394	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	291.1;
	5394	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	284.7;
	5394	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	105;
	5394	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.2;
	5395	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.6;
	5395	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.6;
	5395	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	286.7;
	5395	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	281.8;
	5395	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	104.1;
	5395	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408;
	5396	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.1;
	5396	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.5;
	5396	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	280.8;
	5396	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	278.3;
	5396	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	100.6;
	5396	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.6;
	5397	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.7;
	5397	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.3;
	5397	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	274.2;
	5397	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.6;
	5397	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	98.2;
	5397	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.3;
	5398	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.3;
	5398	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159;
	5398	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.7;
	5398	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261;
	5398	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.3;
	5398	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.4;
	5399	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.2;
	5399	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146;
	5399	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.8;
	5399	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.4;
	5399	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.9;
	5399	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.3;
	5400	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.5;
	5400	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.8;
	5400	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.8;
	5400	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.9;
	5400	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.3;
	5400	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355;
	5401	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403.9;
	5401	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.8;
	5401	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.5;
	5401	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.9;
	5401	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.9;
	5401	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.7;
	5402	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	363.1;
	5402	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.1;
	5402	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.9;
	5402	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.1;
	5402	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.3;
	5402	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.8;
	5403	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	350.3;
	5403	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.3;
	5403	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205;
	5403	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.1;
	5403	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.2;
	5403	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311.1;
	5404	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	346.4;
	5404	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.7;
	5404	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.9;
	5404	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.2;
	5404	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.9;
	5404	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309;
	5405	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	348.7;
	5405	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.6;
	5405	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.1;
	5405	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.8;
	5405	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.7;
	5405	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310.4;
	5406	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	372.3;
	5406	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.9;
	5406	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.8;
	5406	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.2;
	5406	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	5406	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322;
	5407	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.5;
	5407	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136;
	5407	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.7;
	5407	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.2;
	5407	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.1;
	5407	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.4;
	5408	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447.1;
	5408	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.5;
	5408	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243;
	5408	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248;
	5408	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.9;
	5408	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.2;
	5409	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.8;
	5409	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.9;
	5409	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.1;
	5409	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265.6;
	5409	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.3;
	5409	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384;
	5410	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.7;
	5410	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.3;
	5410	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	275.5;
	5410	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	278.4;
	5410	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.2;
	5410	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.5;
	5411	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.7;
	5411	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.2;
	5411	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	286.8;
	5411	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	288.5;
	5411	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87;
	5411	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.5;
	5412	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.9;
	5412	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.2;
	5412	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	298.2;
	5412	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	298.5;
	5412	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92;
	5412	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.6;
	5413	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	557.5;
	5413	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.1;
	5413	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	307.5;
	5413	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	306.2;
	5413	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	97;
	5413	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	431.4;
	5414	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	575.1;
	5414	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.2;
	5414	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	316.6;
	5414	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	313.5;
	5414	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	99.5;
	5414	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	441.7;
	5415	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	578.7;
	5415	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.1;
	5415	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	322.8;
	5415	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	318.9;
	5415	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	101.9;
	5415	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	445.3;
	5416	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	582.3;
	5416	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.4;
	5416	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	324.5;
	5416	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	321.6;
	5416	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	102.7;
	5416	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	447.3;
	5417	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	571.5;
	5417	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.6;
	5417	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	322.7;
	5417	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	319;
	5417	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	103.6;
	5417	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	442.6;
	5418	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570;
	5418	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.9;
	5418	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	314.9;
	5418	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	310.5;
	5418	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	103.1;
	5418	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	439.3;
	5419	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.6;
	5419	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.3;
	5419	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	299.4;
	5419	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	296.5;
	5419	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	102.5;
	5419	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	431.7;
	5420	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.8;
	5420	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.5;
	5420	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	291.2;
	5420	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	289.6;
	5420	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	100.9;
	5420	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.7;
	5421	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	540.7;
	5421	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.3;
	5421	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	284.5;
	5421	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	283.5;
	5421	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	98.5;
	5421	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.8;
	5422	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.3;
	5422	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.3;
	5422	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	269.8;
	5422	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	269.1;
	5422	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.8;
	5422	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.7;
	5423	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.2;
	5423	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.4;
	5423	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.7;
	5423	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248;
	5423	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.8;
	5423	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.2;
	5424	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.6;
	5424	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.2;
	5424	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.5;
	5424	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.9;
	5424	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.6;
	5424	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369;
	5425	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.6;
	5425	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.3;
	5425	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.5;
	5425	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.5;
	5425	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.2;
	5425	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.3;
	5426	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	386.1;
	5426	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.6;
	5426	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.7;
	5426	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.8;
	5426	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.7;
	5426	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.8;
	5427	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	367.2;
	5427	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.3;
	5427	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.5;
	5427	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.1;
	5427	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.3;
	5427	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.8;
	5428	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	369.6;
	5428	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.6;
	5428	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.1;
	5428	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.5;
	5428	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.1;
	5428	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.2;
	5429	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	377.6;
	5429	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.5;
	5429	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209;
	5429	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211;
	5429	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.7;
	5429	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.3;
	5430	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.9;
	5430	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.1;
	5430	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.9;
	5430	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.4;
	5430	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.4;
	5430	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.3;
	5431	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.7;
	5431	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.8;
	5431	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.8;
	5431	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.3;
	5431	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.9;
	5431	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.2;
	5432	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.8;
	5432	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.9;
	5432	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.2;
	5432	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.9;
	5432	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.5;
	5432	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.8;
	5433	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.8;
	5433	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156;
	5433	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	264.7;
	5433	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	266.2;
	5433	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.8;
	5433	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389;
	5434	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.9;
	5434	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.9;
	5434	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	274.7;
	5434	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	275.4;
	5434	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.1;
	5434	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405;
	5435	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.1;
	5435	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.5;
	5435	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	281.4;
	5435	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	281.7;
	5435	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.5;
	5435	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.3;
	5436	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543;
	5436	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.9;
	5436	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	285.7;
	5436	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	286.3;
	5436	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91;
	5436	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.9;
	5437	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.5;
	5437	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.3;
	5437	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	287.7;
	5437	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	288.2;
	5437	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95;
	5437	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	427;
	5438	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.3;
	5438	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.2;
	5438	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	293.4;
	5438	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	294.5;
	5438	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	97.4;
	5438	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	431.3;
	5439	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	574.2;
	5439	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.2;
	5439	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	295.6;
	5439	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	297.9;
	5439	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	99.3;
	5439	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	433.5;
	5440	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.4;
	5440	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.7;
	5440	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	298.4;
	5440	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	300.3;
	5440	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	100.6;
	5440	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	433;
	5441	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	568.8;
	5441	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.9;
	5441	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	299.7;
	5441	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	301.4;
	5441	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	101.2;
	5441	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	431.9;
	5442	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.1;
	5442	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.1;
	5442	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	295.1;
	5442	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	297.2;
	5442	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	101;
	5442	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	427.1;
	5443	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.3;
	5443	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.6;
	5443	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	289.7;
	5443	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	290.5;
	5443	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	100.6;
	5443	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421;
	5444	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529;
	5444	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.3;
	5444	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	286.2;
	5444	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	287;
	5444	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	99.1;
	5444	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.5;
	5445	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.1;
	5445	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.4;
	5445	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	277.6;
	5445	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	279.1;
	5445	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96.9;
	5445	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409;
	5446	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.6;
	5446	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.6;
	5446	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	265.6;
	5446	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	266.4;
	5446	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.7;
	5446	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.5;
	5447	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.8;
	5447	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.6;
	5447	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.5;
	5447	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.7;
	5447	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.9;
	5447	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.4;
	5448	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.7;
	5448	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.3;
	5448	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.3;
	5448	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.2;
	5448	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.5;
	5448	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.8;
	5449	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.7;
	5449	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.3;
	5449	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.1;
	5449	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.7;
	5449	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.6;
	5449	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.4;
	5450	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	389.3;
	5450	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.2;
	5450	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.6;
	5450	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213;
	5450	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	5450	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.4;
	5451	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	384.1;
	5451	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.3;
	5451	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.7;
	5451	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.7;
	5451	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.2;
	5451	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.1;
	5452	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.6;
	5452	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.7;
	5452	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.5;
	5452	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.2;
	5452	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	5452	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.9;
	5453	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	396.5;
	5453	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.4;
	5453	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.6;
	5453	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.8;
	5453	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.2;
	5453	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.4;
	5454	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.8;
	5454	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.8;
	5454	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.6;
	5454	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.1;
	5454	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	5454	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.5;
	5455	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.3;
	5455	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.6;
	5455	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.1;
	5455	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.8;
	5455	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	5455	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.5;
	5456	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.3;
	5456	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.3;
	5456	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.9;
	5456	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.2;
	5456	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	5456	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.3;
	5457	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.3;
	5457	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.9;
	5457	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262;
	5457	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.8;
	5457	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.2;
	5457	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.1;
	5458	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.8;
	5458	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.1;
	5458	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	273.5;
	5458	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	276.9;
	5458	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.9;
	5458	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.3;
	5459	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.7;
	5459	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.7;
	5459	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	281.9;
	5459	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	286.1;
	5459	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.7;
	5459	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410;
	5460	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537.2;
	5460	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.5;
	5460	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	289.9;
	5460	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	294.8;
	5460	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.8;
	5460	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.2;
	5461	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549.4;
	5461	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.9;
	5461	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	296.5;
	5461	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	302.7;
	5461	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.6;
	5461	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.6;
	5462	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.7;
	5462	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.2;
	5462	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	302.4;
	5462	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	308.8;
	5462	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	99.4;
	5462	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	426.1;
	5463	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560;
	5463	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.1;
	5463	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	306.4;
	5463	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	313;
	5463	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	102.3;
	5463	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	431.2;
	5464	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.2;
	5464	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.9;
	5464	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	313.5;
	5464	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	319.5;
	5464	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	104.6;
	5464	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	434;
	5465	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564.4;
	5465	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.7;
	5465	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	311.3;
	5465	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	317.7;
	5465	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	104.8;
	5465	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	435.3;
	5466	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559.6;
	5466	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.1;
	5466	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	302.6;
	5466	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	307.9;
	5466	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	105.5;
	5466	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	430.7;
	5467	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.2;
	5467	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169;
	5467	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	289.5;
	5467	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	294.8;
	5467	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	102.8;
	5467	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.1;
	5468	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526;
	5468	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.3;
	5468	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	283.6;
	5468	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	288.9;
	5468	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	100.3;
	5468	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.4;
	5469	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.7;
	5469	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.9;
	5469	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	278.7;
	5469	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	283.2;
	5469	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96.9;
	5469	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.4;
	5470	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.6;
	5470	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.3;
	5470	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	264.8;
	5470	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267.9;
	5470	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.4;
	5470	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.6;
	5471	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474;
	5471	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.8;
	5471	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.5;
	5471	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.9;
	5471	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.8;
	5471	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.2;
	5472	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.8;
	5472	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.6;
	5472	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.1;
	5472	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.2;
	5472	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.8;
	5472	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.8;
	5473	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	426.9;
	5473	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.3;
	5473	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.8;
	5473	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.1;
	5473	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.2;
	5473	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.8;
	5474	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	382;
	5474	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.9;
	5474	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211;
	5474	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.8;
	5474	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	5474	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.6;
	5475	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	378.9;
	5475	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.1;
	5475	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.5;
	5475	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.7;
	5475	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	5475	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.1;
	5476	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.3;
	5476	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.4;
	5476	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.3;
	5476	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.7;
	5476	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	5476	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.5;
	5477	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391.5;
	5477	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.2;
	5477	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.4;
	5477	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211;
	5477	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	5477	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329;
	5478	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414;
	5478	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.9;
	5478	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.3;
	5478	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.5;
	5478	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	5478	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.9;
	5479	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.3;
	5479	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.9;
	5479	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.3;
	5479	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.8;
	5479	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	5479	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.5;
	5480	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.9;
	5480	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146;
	5480	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.7;
	5480	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.4;
	5480	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	5480	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.7;
	5481	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.5;
	5481	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.6;
	5481	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.1;
	5481	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.5;
	5481	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.4;
	5481	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.7;
	5482	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.4;
	5482	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156;
	5482	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	269.7;
	5482	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	273.5;
	5482	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.9;
	5482	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.4;
	5483	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.9;
	5483	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.5;
	5483	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	280.1;
	5483	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	283.5;
	5483	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.9;
	5483	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.6;
	5484	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548;
	5484	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.6;
	5484	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	289.2;
	5484	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	291.8;
	5484	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88;
	5484	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.8;
	5485	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537.9;
	5485	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.1;
	5485	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	300.9;
	5485	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	303.4;
	5485	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.6;
	5485	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420.1;
	5486	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.9;
	5486	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.8;
	5486	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	307.6;
	5486	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	311.6;
	5486	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	97;
	5486	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.5;
	5487	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.7;
	5487	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.3;
	5487	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	316.9;
	5487	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	322.1;
	5487	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	102.2;
	5487	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	431.3;
	5488	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.8;
	5488	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.5;
	5488	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	320.9;
	5488	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	326.8;
	5488	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	103.5;
	5488	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	435.4;
	5489	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	546.6;
	5489	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.8;
	5489	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	317.2;
	5489	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	322.3;
	5489	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	105.8;
	5489	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	431.6;
	5490	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537.4;
	5490	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.9;
	5490	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	312.9;
	5490	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	316.4;
	5490	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	105.2;
	5490	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.9;
	5491	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.7;
	5491	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.4;
	5491	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	301.2;
	5491	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	305.1;
	5491	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	102.8;
	5491	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.8;
	5492	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.4;
	5492	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.7;
	5492	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	289.3;
	5492	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	294.5;
	5492	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	100.9;
	5492	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.1;
	5493	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510;
	5493	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.5;
	5493	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	281.5;
	5493	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	286.4;
	5493	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96.8;
	5493	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.3;
	5494	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508;
	5494	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.7;
	5494	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	265.2;
	5494	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	269.7;
	5494	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.1;
	5494	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.6;
	5495	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.8;
	5495	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.1;
	5495	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245;
	5495	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249;
	5495	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.7;
	5495	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.4;
	5496	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.1;
	5496	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.4;
	5496	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.7;
	5496	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.8;
	5496	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.5;
	5496	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.3;
	5497	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.4;
	5497	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.3;
	5497	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.4;
	5497	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221;
	5497	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.9;
	5497	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.2;
	5498	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	382.1;
	5498	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.2;
	5498	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.9;
	5498	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.3;
	5498	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.7;
	5498	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.2;
	5499	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	386.7;
	5499	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.4;
	5499	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.8;
	5499	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.9;
	5499	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.6;
	5499	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.3;
	5500	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.4;
	5500	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.4;
	5500	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202;
	5500	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.4;
	5500	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.9;
	5500	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.1;
	5501	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.2;
	5501	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.9;
	5501	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.6;
	5501	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209;
	5501	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72;
	5501	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.4;
	5502	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.7;
	5502	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.3;
	5502	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212;
	5502	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216;
	5502	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.9;
	5502	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.6;
	5503	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	444;
	5503	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.1;
	5503	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.2;
	5503	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.3;
	5503	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.3;
	5503	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.2;
	5504	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.6;
	5504	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.8;
	5504	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.5;
	5504	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.2;
	5504	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.4;
	5504	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367;
	5505	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.6;
	5505	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.9;
	5505	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.7;
	5505	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.8;
	5505	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80;
	5505	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.3;
	5506	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.8;
	5506	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160;
	5506	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	267.1;
	5506	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.5;
	5506	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.1;
	5506	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.6;
	5507	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.4;
	5507	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.5;
	5507	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	279.9;
	5507	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	284.6;
	5507	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.7;
	5507	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.3;
	5508	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.2;
	5508	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.2;
	5508	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	291;
	5508	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	295.9;
	5508	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.5;
	5508	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.3;
	5509	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.1;
	5509	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.5;
	5509	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	301.8;
	5509	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	307.5;
	5509	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	99.3;
	5509	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.8;
	5510	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	546;
	5510	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.4;
	5510	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	312.8;
	5510	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	319.4;
	5510	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	101.5;
	5510	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	428.2;
	5511	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552.2;
	5511	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.5;
	5511	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	319.8;
	5511	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	326.5;
	5511	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	101.9;
	5511	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	433.3;
	5512	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	554.6;
	5512	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.3;
	5512	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	321;
	5512	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	327.4;
	5512	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	103.3;
	5512	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	435.1;
	5513	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	553.2;
	5513	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.3;
	5513	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	319.8;
	5513	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	324.5;
	5513	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	102.9;
	5513	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	434.4;
	5514	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.6;
	5514	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.7;
	5514	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	310;
	5514	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	312.3;
	5514	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	102.2;
	5514	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	428.1;
	5515	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528.5;
	5515	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.8;
	5515	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	302.1;
	5515	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	302.7;
	5515	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	100.9;
	5515	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.9;
	5516	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.4;
	5516	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.7;
	5516	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	297.3;
	5516	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	297.7;
	5516	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	98.2;
	5516	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.5;
	5517	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.6;
	5517	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.5;
	5517	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	277.8;
	5517	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	277.9;
	5517	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.2;
	5517	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.7;
	5518	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.5;
	5518	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.4;
	5518	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	265.2;
	5518	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.7;
	5518	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.4;
	5518	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.8;
	5519	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.7;
	5519	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.4;
	5519	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.1;
	5519	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249;
	5519	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.6;
	5519	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.4;
	5520	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.4;
	5520	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.9;
	5520	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.5;
	5520	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.4;
	5520	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.6;
	5520	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.4;
	5521	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.1;
	5521	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.4;
	5521	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.4;
	5521	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.6;
	5521	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.9;
	5521	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.8;
	5522	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379.7;
	5522	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.7;
	5522	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.3;
	5522	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.4;
	5522	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.5;
	5522	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.4;
	5523	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	380.4;
	5523	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.8;
	5523	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.9;
	5523	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.1;
	5523	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	5523	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.6;
	5524	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	384.6;
	5524	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.7;
	5524	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.3;
	5524	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.5;
	5524	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	5524	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.4;
	5525	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	369.1;
	5525	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118;
	5525	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.7;
	5525	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.1;
	5525	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	5525	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.6;
	5526	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	380.7;
	5526	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.3;
	5526	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.4;
	5526	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.1;
	5526	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	5526	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.5;
	5527	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	385.1;
	5527	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.9;
	5527	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214;
	5527	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.3;
	5527	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	5527	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329;
	5528	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394;
	5528	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.4;
	5528	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.2;
	5528	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.4;
	5528	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.4;
	5528	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.4;
	5529	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.7;
	5529	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143;
	5529	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.3;
	5529	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.1;
	5529	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73;
	5529	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.6;
	5530	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.4;
	5530	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.1;
	5530	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.1;
	5530	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.9;
	5530	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.7;
	5530	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.3;
	5531	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.3;
	5531	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.4;
	5531	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	259.6;
	5531	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.4;
	5531	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.7;
	5531	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.4;
	5532	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.4;
	5532	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.6;
	5532	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	270.4;
	5532	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	273.7;
	5532	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.9;
	5532	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.4;
	5533	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.7;
	5533	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.4;
	5533	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	281.7;
	5533	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	285.1;
	5533	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.6;
	5533	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.5;
	5534	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.6;
	5534	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.5;
	5534	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	291;
	5534	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	293.9;
	5534	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.5;
	5534	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.6;
	5535	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.2;
	5535	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.3;
	5535	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	295.4;
	5535	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	298.9;
	5535	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.5;
	5535	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.9;
	5536	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.7;
	5536	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.7;
	5536	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	299.2;
	5536	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	302.2;
	5536	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.5;
	5536	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409;
	5537	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.4;
	5537	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169;
	5537	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	301.5;
	5537	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	304.1;
	5537	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96.2;
	5537	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.8;
	5538	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.6;
	5538	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.5;
	5538	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	298;
	5538	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	299.8;
	5538	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.3;
	5538	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.8;
	5539	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.3;
	5539	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.5;
	5539	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	291;
	5539	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	293;
	5539	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.9;
	5539	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.5;
	5540	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.3;
	5540	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.3;
	5540	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	283.5;
	5540	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	285.4;
	5540	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.3;
	5540	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389;
	5541	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.4;
	5541	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.1;
	5541	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	280.3;
	5541	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	281.8;
	5541	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.4;
	5541	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382;
	5542	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.6;
	5542	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153;
	5542	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	266.1;
	5542	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267.1;
	5542	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.2;
	5542	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.3;
	5543	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.7;
	5543	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.7;
	5543	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.6;
	5543	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.2;
	5543	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.1;
	5543	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354;
	5544	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	395.4;
	5544	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139;
	5544	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.4;
	5544	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.5;
	5544	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.2;
	5544	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.5;
	5545	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	377.3;
	5545	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.4;
	5545	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.8;
	5545	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.9;
	5545	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.3;
	5545	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.6;
	5546	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	356.3;
	5546	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.9;
	5546	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.3;
	5546	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.7;
	5546	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.7;
	5546	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.9;
	5547	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	346.9;
	5547	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119;
	5547	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.5;
	5547	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211;
	5547	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	5547	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.2;
	5548	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	335.9;
	5548	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.6;
	5548	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.3;
	5548	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.1;
	5548	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	5548	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	306.5;
	5549	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	341.7;
	5549	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.4;
	5549	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205;
	5549	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.1;
	5549	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	5549	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.3;
	5550	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	355;
	5550	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.4;
	5550	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205;
	5550	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.2;
	5550	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	5550	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.1;
	5551	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	360.1;
	5551	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.2;
	5551	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.7;
	5551	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.3;
	5551	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	5551	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.2;
	5552	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	371.3;
	5552	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.6;
	5552	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213;
	5552	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.8;
	5552	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	5552	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.7;
	5553	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	388.3;
	5553	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.9;
	5553	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.1;
	5553	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.8;
	5553	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.5;
	5553	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.8;
	5554	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.8;
	5554	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.1;
	5554	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.4;
	5554	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.4;
	5554	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.9;
	5554	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.9;
	5555	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.9;
	5555	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.4;
	5555	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.3;
	5555	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.1;
	5555	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.9;
	5555	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.4;
	5556	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.9;
	5556	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.4;
	5556	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.5;
	5556	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264;
	5556	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.6;
	5556	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.7;
	5557	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.6;
	5557	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.1;
	5557	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	269.2;
	5557	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	274.5;
	5557	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.4;
	5557	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.7;
	5558	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.1;
	5558	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.7;
	5558	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	277.4;
	5558	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	280.7;
	5558	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93;
	5558	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.1;
	5559	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486;
	5559	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.2;
	5559	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	283;
	5559	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	287;
	5559	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.4;
	5559	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.2;
	5560	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.1;
	5560	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.5;
	5560	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	288.4;
	5560	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	292.4;
	5560	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.7;
	5560	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.7;
	5561	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.9;
	5561	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.4;
	5561	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	290;
	5561	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	292.5;
	5561	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	97;
	5561	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.4;
	5562	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.2;
	5562	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165;
	5562	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	288.1;
	5562	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	289.8;
	5562	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.5;
	5562	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397;
	5563	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.4;
	5563	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.8;
	5563	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	285.8;
	5563	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	287.7;
	5563	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92;
	5563	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.2;
	5564	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.6;
	5564	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.8;
	5564	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	281.3;
	5564	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	283.3;
	5564	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.4;
	5564	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.3;
	5565	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.8;
	5565	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.3;
	5565	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	276.4;
	5565	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	277.5;
	5565	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.7;
	5565	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.7;
	5566	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.5;
	5566	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.1;
	5566	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	264.1;
	5566	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265.1;
	5566	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.4;
	5566	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.3;
	5567	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.6;
	5567	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.6;
	5567	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.8;
	5567	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.6;
	5567	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.3;
	5567	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.2;
	5568	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.1;
	5568	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.2;
	5568	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.4;
	5568	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.8;
	5568	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.4;
	5568	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.8;
	5569	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	400.8;
	5569	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127;
	5569	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.3;
	5569	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.9;
	5569	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.8;
	5569	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.9;
	5570	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	358.7;
	5570	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121;
	5570	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.4;
	5570	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.5;
	5570	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	5570	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.1;
	5571	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	353.4;
	5571	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.3;
	5571	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.4;
	5571	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.4;
	5571	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	5571	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.4;
	5572	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	346.3;
	5572	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118;
	5572	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.1;
	5572	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.7;
	5572	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.3;
	5572	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309.7;
	5573	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	362.5;
	5573	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.4;
	5573	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.1;
	5573	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.4;
	5573	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	5573	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.6;
	5574	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	380.7;
	5574	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.5;
	5574	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.4;
	5574	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215;
	5574	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	5574	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.8;
	5575	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421;
	5575	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.5;
	5575	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.6;
	5575	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.9;
	5575	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.7;
	5575	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.4;
	5576	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.5;
	5576	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.3;
	5576	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.6;
	5576	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.4;
	5576	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	5576	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.1;
	5577	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.2;
	5577	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.3;
	5577	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.9;
	5577	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.3;
	5577	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	5577	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.6;
	5578	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.4;
	5578	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.5;
	5578	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	273.7;
	5578	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	276.3;
	5578	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.8;
	5578	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.5;
	5579	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.3;
	5579	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.9;
	5579	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	285.8;
	5579	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	288;
	5579	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74;
	5579	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411;
	5580	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549.3;
	5580	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.4;
	5580	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	298.8;
	5580	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	299;
	5580	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.4;
	5580	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	424;
	5581	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.5;
	5581	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.8;
	5581	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	308.9;
	5581	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	307.2;
	5581	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.2;
	5581	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	433.6;
	5582	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.1;
	5582	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.7;
	5582	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	316.6;
	5582	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	315.6;
	5582	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.5;
	5582	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	439.8;
	5583	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	576.8;
	5583	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.5;
	5583	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	323.3;
	5583	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	321.8;
	5583	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.6;
	5583	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	444.5;
	5584	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577.3;
	5584	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.3;
	5584	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	327.9;
	5584	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	324.9;
	5584	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.8;
	5584	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	446.2;
	5585	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	580.4;
	5585	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.9;
	5585	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	328.9;
	5585	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	325.9;
	5585	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91;
	5585	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	447.9;
	5586	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	573.4;
	5586	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.6;
	5586	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	322.4;
	5586	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	318.4;
	5586	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.8;
	5586	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	443.4;
	5587	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559;
	5587	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.8;
	5587	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	311.6;
	5587	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	306.5;
	5587	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.2;
	5587	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	434.2;
	5588	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.5;
	5588	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.2;
	5588	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	303.2;
	5588	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	299.6;
	5588	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.7;
	5588	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	424.1;
	5589	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.6;
	5589	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175;
	5589	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	294.4;
	5589	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	291.4;
	5589	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.1;
	5589	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.6;
	5590	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525.5;
	5590	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.8;
	5590	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	279;
	5590	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	275.8;
	5590	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.9;
	5590	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.8;
	5591	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.1;
	5591	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152;
	5591	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.3;
	5591	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.2;
	5591	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.7;
	5591	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.7;
	5592	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.1;
	5592	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.1;
	5592	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244;
	5592	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.9;
	5592	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.6;
	5592	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.7;
	5593	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.7;
	5593	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.9;
	5593	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.6;
	5593	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.9;
	5593	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.7;
	5593	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.3;
	5594	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394.9;
	5594	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123;
	5594	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.7;
	5594	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.8;
	5594	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.8;
	5594	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333;
	5595	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391.1;
	5595	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.1;
	5595	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.1;
	5595	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.5;
	5595	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	5595	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.5;
	5596	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	387.5;
	5596	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.1;
	5596	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.1;
	5596	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.4;
	5596	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	5596	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.5;
	5597	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	396.1;
	5597	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.1;
	5597	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.6;
	5597	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.5;
	5597	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	5597	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.7;
	5598	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.5;
	5598	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.3;
	5598	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.7;
	5598	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.9;
	5598	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.4;
	5598	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.4;
	5599	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442.3;
	5599	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.3;
	5599	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.7;
	5599	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.2;
	5599	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	5599	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.3;
	5600	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.7;
	5600	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.6;
	5600	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.2;
	5600	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.1;
	5600	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.6;
	5600	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.5;
	5601	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.6;
	5601	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156;
	5601	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	265.5;
	5601	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	266;
	5601	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	5601	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.4;
	5602	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.9;
	5602	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.2;
	5602	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	277;
	5602	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	277;
	5602	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.1;
	5602	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.6;
	5603	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	540.6;
	5603	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169;
	5603	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	289.2;
	5603	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	288.2;
	5603	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.5;
	5603	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.5;
	5604	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566.4;
	5604	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175;
	5604	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	302.2;
	5604	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	299.5;
	5604	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79;
	5604	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	431.9;
	5605	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	583.3;
	5605	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.3;
	5605	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	315;
	5605	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	311;
	5605	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.5;
	5605	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	443.3;
	5606	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	595.9;
	5606	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.3;
	5606	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	326.7;
	5606	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	322.5;
	5606	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.4;
	5606	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	452.6;
	5607	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	596.6;
	5607	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.4;
	5607	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	333.2;
	5607	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	328.5;
	5607	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.1;
	5607	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	455.4;
	5608	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	602.3;
	5608	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.8;
	5608	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	338.8;
	5608	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	332.9;
	5608	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.9;
	5608	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	459.6;
	5609	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	599.9;
	5609	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.9;
	5609	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	339;
	5609	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	332.1;
	5609	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.2;
	5609	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	459.2;
	5610	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	594;
	5610	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.9;
	5610	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	330.9;
	5610	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	324.7;
	5610	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.3;
	5610	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	454.9;
	5611	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	574.6;
	5611	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.3;
	5611	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	320.3;
	5611	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	314;
	5611	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.6;
	5611	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	443.4;
	5612	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.3;
	5612	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.4;
	5612	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	309.3;
	5612	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	304.5;
	5612	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.3;
	5612	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	432.5;
	5613	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548.8;
	5613	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.1;
	5613	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	301.8;
	5613	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	296.9;
	5613	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.9;
	5613	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.5;
	5614	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.4;
	5614	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170;
	5614	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	282.5;
	5614	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	278.1;
	5614	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.9;
	5614	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.2;
	5615	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.9;
	5615	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.8;
	5615	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	263;
	5615	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.9;
	5615	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.2;
	5615	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.5;
	5616	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.4;
	5616	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.7;
	5616	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.7;
	5616	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.6;
	5616	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	5616	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.9;
	5617	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.2;
	5617	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.7;
	5617	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.2;
	5617	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.3;
	5617	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	5617	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.3;
	5618	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.4;
	5618	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.9;
	5618	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220;
	5618	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.4;
	5618	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	5618	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.1;
	5619	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	396.5;
	5619	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.8;
	5619	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.3;
	5619	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.8;
	5619	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61;
	5619	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.2;
	5620	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394;
	5620	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.5;
	5620	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.7;
	5620	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.6;
	5620	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.5;
	5620	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.5;
	5621	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.6;
	5621	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.1;
	5621	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217;
	5621	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.4;
	5621	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.7;
	5621	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.4;
	5622	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	420.9;
	5622	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.7;
	5622	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.8;
	5622	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.4;
	5622	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.8;
	5622	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.7;
	5623	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.1;
	5623	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.3;
	5623	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.6;
	5623	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.7;
	5623	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	5623	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.4;
	5624	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.5;
	5624	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.7;
	5624	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.2;
	5624	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.4;
	5624	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.2;
	5624	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.6;
	5625	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.6;
	5625	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.1;
	5625	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	269.8;
	5625	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	269.7;
	5625	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	5625	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.1;
	5626	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	524.1;
	5626	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.5;
	5626	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	284.1;
	5626	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	282.8;
	5626	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.8;
	5626	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.8;
	5627	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	546.8;
	5627	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.9;
	5627	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	298.5;
	5627	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	296.5;
	5627	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.9;
	5627	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.7;
	5628	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	555.3;
	5628	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.4;
	5628	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	312.3;
	5628	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	309.3;
	5628	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.5;
	5628	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	430.7;
	5629	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.9;
	5629	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.4;
	5629	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	322.9;
	5629	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	319.1;
	5629	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.8;
	5629	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	437.6;
	5630	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	573.8;
	5630	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.6;
	5630	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	331.2;
	5630	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	327.3;
	5630	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84;
	5630	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	446.1;
	5631	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586.8;
	5631	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.2;
	5631	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	341.7;
	5631	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	336.8;
	5631	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.8;
	5631	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	454.5;
	5632	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.1;
	5632	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.4;
	5632	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	345;
	5632	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	340.3;
	5632	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.7;
	5632	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	460.8;
	5633	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	601.2;
	5633	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.3;
	5633	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	337.6;
	5633	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	333.3;
	5633	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.6;
	5633	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	460.8;
	5634	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592;
	5634	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.8;
	5634	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	322.5;
	5634	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	318.8;
	5634	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.6;
	5634	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	452;
	5635	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579.4;
	5635	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.8;
	5635	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	310;
	5635	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	306.7;
	5635	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.7;
	5635	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	441.7;
	5636	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564.5;
	5636	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.2;
	5636	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	303.8;
	5636	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	301.7;
	5636	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.9;
	5636	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	431.6;
	5637	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562;
	5637	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.1;
	5637	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	290.9;
	5637	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	289.6;
	5637	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.6;
	5637	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	426.6;
	5638	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.7;
	5638	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.1;
	5638	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	276.7;
	5638	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	275.3;
	5638	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.9;
	5638	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.7;
	5639	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.8;
	5639	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.6;
	5639	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255;
	5639	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.2;
	5639	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.1;
	5639	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.4;
	5640	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.5;
	5640	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.3;
	5640	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.3;
	5640	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.8;
	5640	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.9;
	5640	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.5;
	5641	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	444.6;
	5641	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.5;
	5641	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228;
	5641	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.2;
	5641	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	5641	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358;
	5642	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.1;
	5642	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.3;
	5642	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.9;
	5642	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.6;
	5642	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.3;
	5642	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.6;
	5643	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	402.6;
	5643	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.1;
	5643	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.8;
	5643	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214;
	5643	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.2;
	5643	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.9;
	5644	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.9;
	5644	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127;
	5644	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.5;
	5644	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.2;
	5644	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.6;
	5644	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.8;
	5645	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.6;
	5645	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.3;
	5645	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.3;
	5645	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.3;
	5645	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.2;
	5645	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.1;
	5646	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	426.9;
	5646	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138;
	5646	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.4;
	5646	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.7;
	5646	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.1;
	5646	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.5;
	5647	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.9;
	5647	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.1;
	5647	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.3;
	5647	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.7;
	5647	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.5;
	5647	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.6;
	5648	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483;
	5648	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.5;
	5648	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.2;
	5648	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.2;
	5648	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.1;
	5648	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.1;
	5649	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.4;
	5649	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.9;
	5649	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	270.8;
	5649	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	274.8;
	5649	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	5649	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.4;
	5650	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.9;
	5650	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.9;
	5650	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	283.4;
	5650	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	286.8;
	5650	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.1;
	5650	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.6;
	5651	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	563.3;
	5651	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.2;
	5651	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	293.9;
	5651	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	296.3;
	5651	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.8;
	5651	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	428.8;
	5652	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	569.2;
	5652	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.6;
	5652	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	303.2;
	5652	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	304.1;
	5652	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.3;
	5652	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	434.3;
	5653	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577.2;
	5653	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.6;
	5653	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	310.5;
	5653	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	309.9;
	5653	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.2;
	5653	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	441;
	5654	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589.7;
	5654	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.3;
	5654	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	317;
	5654	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	315.6;
	5654	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.8;
	5654	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	448.2;
	5655	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	600.5;
	5655	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.1;
	5655	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	321.5;
	5655	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	318.5;
	5655	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.8;
	5655	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	453.6;
	5656	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	601.1;
	5656	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193;
	5656	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	319.7;
	5656	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	317;
	5656	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.4;
	5656	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	453.5;
	5657	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	604.6;
	5657	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.1;
	5657	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	318;
	5657	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	315.4;
	5657	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.6;
	5657	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	454.2;
	5658	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	601.2;
	5658	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.2;
	5658	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	314.7;
	5658	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	311.9;
	5658	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.5;
	5658	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	450.7;
	5659	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588;
	5659	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.5;
	5659	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	301.9;
	5659	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	300.7;
	5659	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.6;
	5659	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	441.1;
	5660	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.8;
	5660	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.3;
	5660	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	298.3;
	5660	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	298.3;
	5660	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.9;
	5660	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	432.4;
	5661	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	555.7;
	5661	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.9;
	5661	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	289.2;
	5661	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	289.6;
	5661	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.9;
	5661	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.7;
	5662	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535;
	5662	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170;
	5662	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	275.3;
	5662	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	275.1;
	5662	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75;
	5662	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.2;
	5663	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.8;
	5663	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.4;
	5663	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.6;
	5663	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257;
	5663	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	5663	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.6;
	5664	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.3;
	5664	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.3;
	5664	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.6;
	5664	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.8;
	5664	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	5664	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.4;
	5665	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.8;
	5665	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.5;
	5665	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.2;
	5665	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.5;
	5665	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.3;
	5665	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.2;
	5666	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394;
	5666	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.9;
	5666	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.8;
	5666	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220;
	5666	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	5666	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.6;
	5667	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	387.5;
	5667	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.3;
	5667	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.3;
	5667	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215;
	5667	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57;
	5667	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.1;
	5668	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	382.7;
	5668	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.3;
	5668	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.4;
	5668	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.2;
	5668	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.8;
	5668	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.9;
	5669	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.2;
	5669	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.9;
	5669	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.3;
	5669	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.5;
	5669	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.1;
	5669	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.9;
	5670	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.2;
	5670	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.5;
	5670	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.4;
	5670	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.1;
	5670	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.3;
	5670	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.8;
	5671	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447.7;
	5671	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.7;
	5671	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.7;
	5671	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.2;
	5671	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.3;
	5671	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.2;
	5672	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474;
	5672	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.6;
	5672	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.6;
	5672	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.7;
	5672	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	5672	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.9;
	5673	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.4;
	5673	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.6;
	5673	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	263.6;
	5673	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	268.1;
	5673	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	5673	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.8;
	5674	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.2;
	5674	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.9;
	5674	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	275.1;
	5674	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	280.4;
	5674	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.3;
	5674	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406;
	5675	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	540.2;
	5675	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.7;
	5675	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	284.4;
	5675	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	289.7;
	5675	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.4;
	5675	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.7;
	5676	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.6;
	5676	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.3;
	5676	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	293;
	5676	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	299;
	5676	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.7;
	5676	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	426.5;
	5677	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	580.4;
	5677	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.8;
	5677	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	303.5;
	5677	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	309.5;
	5677	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.9;
	5677	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	439;
	5678	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.3;
	5678	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.5;
	5678	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	311.6;
	5678	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	317.7;
	5678	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.2;
	5678	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	446.5;
	5679	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	605.1;
	5679	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.7;
	5679	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	316.7;
	5679	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	322.7;
	5679	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.4;
	5679	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	453.4;
	5680	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	621.8;
	5680	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.4;
	5680	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	318.8;
	5680	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	324.3;
	5680	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.8;
	5680	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	460.6;
	5681	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	625;
	5681	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.1;
	5681	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	316.6;
	5681	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	321.6;
	5681	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.1;
	5681	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	461.3;
	5682	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	614.3;
	5682	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.4;
	5682	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	307.9;
	5682	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	311.5;
	5682	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.2;
	5682	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	454.2;
	5683	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	594;
	5683	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.2;
	5683	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	298.4;
	5683	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	302.6;
	5683	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.5;
	5683	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	442.4;
	5684	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	573.4;
	5684	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.7;
	5684	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	287.1;
	5684	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	292.1;
	5684	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.9;
	5684	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	429.9;
	5685	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	557.9;
	5685	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.1;
	5685	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	277.4;
	5685	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	282.1;
	5685	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.4;
	5685	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.7;
	5686	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.2;
	5686	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.9;
	5686	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.3;
	5686	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265.9;
	5686	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.9;
	5686	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.9;
	5687	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.7;
	5687	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.3;
	5687	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.5;
	5687	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.6;
	5687	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.7;
	5687	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.2;
	5688	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.7;
	5688	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.1;
	5688	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.8;
	5688	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.4;
	5688	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.1;
	5688	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.6;
	5689	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	426.3;
	5689	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.7;
	5689	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.9;
	5689	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.6;
	5689	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	5689	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.6;
	5690	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.1;
	5690	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.3;
	5690	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.5;
	5690	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.9;
	5690	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	5690	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.6;
	5691	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391.9;
	5691	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.6;
	5691	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204;
	5691	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208;
	5691	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.4;
	5691	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.6;
	5692	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	388.5;
	5692	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.5;
	5692	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.3;
	5692	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.3;
	5692	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.7;
	5692	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.5;
	5693	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	389.2;
	5693	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117;
	5693	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.7;
	5693	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.2;
	5693	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.8;
	5693	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.2;
	5694	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	395.5;
	5694	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122;
	5694	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.6;
	5694	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.4;
	5694	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.6;
	5694	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.5;
	5695	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.2;
	5695	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.9;
	5695	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.1;
	5695	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.2;
	5695	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.6;
	5695	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.3;
	5696	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	427.6;
	5696	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.6;
	5696	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.5;
	5696	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225;
	5696	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	5696	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348;
	5697	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.9;
	5697	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.4;
	5697	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.8;
	5697	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.6;
	5697	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	5697	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357;
	5698	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.1;
	5698	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.5;
	5698	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.2;
	5698	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.4;
	5698	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.2;
	5698	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.2;
	5699	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.3;
	5699	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.7;
	5699	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.9;
	5699	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.5;
	5699	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.7;
	5699	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.6;
	5700	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.7;
	5700	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.1;
	5700	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	259.7;
	5700	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265.6;
	5700	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.2;
	5700	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.7;
	5701	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.4;
	5701	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.1;
	5701	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	268.3;
	5701	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	273.2;
	5701	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85;
	5701	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.9;
	5702	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548.5;
	5702	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.2;
	5702	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	274.9;
	5702	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	278.1;
	5702	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.6;
	5702	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.7;
	5703	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556;
	5703	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.7;
	5703	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	278.4;
	5703	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	280.4;
	5703	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.3;
	5703	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420.1;
	5704	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	571.8;
	5704	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.1;
	5704	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	280.3;
	5704	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	281.5;
	5704	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.1;
	5704	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	426.9;
	5705	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.6;
	5705	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.4;
	5705	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	277.9;
	5705	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	279.4;
	5705	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.4;
	5705	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	426.6;
	5706	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	568.2;
	5706	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.3;
	5706	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	270.9;
	5706	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	273.1;
	5706	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.7;
	5706	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.3;
	5707	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547.4;
	5707	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.6;
	5707	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	268.6;
	5707	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.7;
	5707	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.6;
	5707	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.1;
	5708	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.2;
	5708	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.4;
	5708	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	264.7;
	5708	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	270.2;
	5708	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87;
	5708	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.8;
	5709	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.2;
	5709	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.1;
	5709	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.6;
	5709	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.2;
	5709	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.5;
	5709	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.8;
	5710	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.9;
	5710	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149;
	5710	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.2;
	5710	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.3;
	5710	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.9;
	5710	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.1;
	5711	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.6;
	5711	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.1;
	5711	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.5;
	5711	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.1;
	5711	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.1;
	5711	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.1;
	5712	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.2;
	5712	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.6;
	5712	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228;
	5712	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.9;
	5712	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74;
	5712	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.5;
	5713	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	401.7;
	5713	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.2;
	5713	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.2;
	5713	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.5;
	5713	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.6;
	5713	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.6;
	5714	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	361.4;
	5714	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.2;
	5714	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.6;
	5714	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.2;
	5714	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	5714	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315;
	5715	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	354.2;
	5715	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.2;
	5715	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.8;
	5715	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.6;
	5715	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	5715	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310.7;
	5716	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	349.2;
	5716	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.4;
	5716	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.1;
	5716	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.6;
	5716	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	5716	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.1;
	5717	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	349;
	5717	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.5;
	5717	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.6;
	5717	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.7;
	5717	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	5717	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.1;
	5718	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	362.7;
	5718	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.1;
	5718	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.6;
	5718	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.1;
	5718	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	5718	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.9;
	5719	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	368.9;
	5719	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.7;
	5719	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.7;
	5719	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.8;
	5719	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	5719	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.3;
	5720	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	375.6;
	5720	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.8;
	5720	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.3;
	5720	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.1;
	5720	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.4;
	5720	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.6;
	5721	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.2;
	5721	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.2;
	5721	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.7;
	5721	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.2;
	5721	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.5;
	5721	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.5;
	5722	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.8;
	5722	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143;
	5722	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.8;
	5722	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.1;
	5722	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.5;
	5722	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.2;
	5723	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.2;
	5723	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.5;
	5723	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.1;
	5723	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.1;
	5723	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.4;
	5723	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.1;
	5724	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.7;
	5724	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153;
	5724	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.2;
	5724	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249;
	5724	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90;
	5724	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.7;
	5725	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.5;
	5725	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.9;
	5725	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	259.8;
	5725	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.7;
	5725	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.9;
	5725	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.9;
	5726	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.8;
	5726	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.1;
	5726	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	269.1;
	5726	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	268;
	5726	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	98.5;
	5726	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.9;
	5727	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.4;
	5727	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159;
	5727	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	277;
	5727	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	274;
	5727	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	100.3;
	5727	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.2;
	5728	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.2;
	5728	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.8;
	5728	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	282.9;
	5728	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	277.7;
	5728	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	101.6;
	5728	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.5;
	5729	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.2;
	5729	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.3;
	5729	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	282.2;
	5729	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	275.8;
	5729	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	101.8;
	5729	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413;
	5730	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.3;
	5730	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163;
	5730	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	279.4;
	5730	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.9;
	5730	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	101;
	5730	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.9;
	5731	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	524.9;
	5731	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162;
	5731	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	272;
	5731	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265.8;
	5731	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	99.2;
	5731	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.9;
	5732	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.6;
	5732	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.6;
	5732	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	268.7;
	5732	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.1;
	5732	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96.2;
	5732	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.8;
	5733	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.9;
	5733	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.6;
	5733	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	266.4;
	5733	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.1;
	5733	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.4;
	5733	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.4;
	5734	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.1;
	5734	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.8;
	5734	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254;
	5734	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.1;
	5734	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.4;
	5734	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.8;
	5735	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.3;
	5735	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.9;
	5735	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238;
	5735	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.8;
	5735	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.4;
	5735	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.8;
	5736	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.3;
	5736	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.6;
	5736	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231;
	5736	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.2;
	5736	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.4;
	5736	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.2;
	5737	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.6;
	5737	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.7;
	5737	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.1;
	5737	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.7;
	5737	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.2;
	5737	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.5;
	5738	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	381.8;
	5738	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.2;
	5738	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.5;
	5738	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.7;
	5738	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	5738	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.6;
	5739	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	369.9;
	5739	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.6;
	5739	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.4;
	5739	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202;
	5739	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	5739	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.4;
	5740	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	369.8;
	5740	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.8;
	5740	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.2;
	5740	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.6;
	5740	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	5740	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.9;
	5741	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	374.7;
	5741	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.9;
	5741	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.6;
	5741	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.1;
	5741	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	5741	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.2;
	5742	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.8;
	5742	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.7;
	5742	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.1;
	5742	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.8;
	5742	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	5742	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.3;
	5743	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442.4;
	5743	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.4;
	5743	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.2;
	5743	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.1;
	5743	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	5743	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.3;
	5744	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.4;
	5744	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.1;
	5744	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.7;
	5744	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.4;
	5744	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	5744	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.7;
	5745	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.5;
	5745	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155;
	5745	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.6;
	5745	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.5;
	5745	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	5745	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.1;
	5746	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.8;
	5746	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.7;
	5746	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	269.5;
	5746	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267;
	5746	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71;
	5746	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.7;
	5747	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.1;
	5747	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.7;
	5747	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	279.2;
	5747	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	276.1;
	5747	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.2;
	5747	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.2;
	5748	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.8;
	5748	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.3;
	5748	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	287.3;
	5748	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	283.6;
	5748	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81;
	5748	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.2;
	5749	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.1;
	5749	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.6;
	5749	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	293.8;
	5749	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	289.6;
	5749	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.6;
	5749	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	426.1;
	5750	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.9;
	5750	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.5;
	5750	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	302.4;
	5750	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	297.6;
	5750	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.8;
	5750	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	434.7;
	5751	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.3;
	5751	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.3;
	5751	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	310;
	5751	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	304.6;
	5751	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.7;
	5751	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	442.3;
	5752	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589.2;
	5752	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.3;
	5752	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	313.9;
	5752	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	308.5;
	5752	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.3;
	5752	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	445.6;
	5753	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	585.6;
	5753	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.4;
	5753	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	310.6;
	5753	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	305;
	5753	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.5;
	5753	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	443.7;
	5754	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.3;
	5754	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.3;
	5754	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	302.5;
	5754	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	297.3;
	5754	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.3;
	5754	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	435.6;
	5755	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566;
	5755	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.8;
	5755	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	294.2;
	5755	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	289.3;
	5755	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.8;
	5755	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	430.2;
	5756	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.5;
	5756	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.8;
	5756	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	288.9;
	5756	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	285.9;
	5756	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.6;
	5756	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.7;
	5757	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536;
	5757	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.1;
	5757	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	280.1;
	5757	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	277.2;
	5757	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.2;
	5757	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.7;
	5758	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.3;
	5758	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.1;
	5758	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	266.3;
	5758	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	263.5;
	5758	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.9;
	5758	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.4;
	5759	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.2;
	5759	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.6;
	5759	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.8;
	5759	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.2;
	5759	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.3;
	5759	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.5;
	5760	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.1;
	5760	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.1;
	5760	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.5;
	5760	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.5;
	5760	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.2;
	5760	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.7;
	5761	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415;
	5761	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132;
	5761	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.9;
	5761	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.2;
	5761	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.4;
	5761	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.3;
	5762	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	374.5;
	5762	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.7;
	5762	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210;
	5762	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209;
	5762	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	5762	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.9;
	5763	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	378.4;
	5763	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.6;
	5763	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.5;
	5763	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.1;
	5763	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	5763	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.2;
	5764	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	373.8;
	5764	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.4;
	5764	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.6;
	5764	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.9;
	5764	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60;
	5764	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.9;
	5765	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379.4;
	5765	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.4;
	5765	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.4;
	5765	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.2;
	5765	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	5765	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324;
	5766	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.4;
	5766	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.2;
	5766	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.6;
	5766	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.5;
	5766	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.8;
	5766	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.3;
	5767	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.1;
	5767	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.6;
	5767	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233;
	5767	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.5;
	5767	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59;
	5767	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.4;
	5768	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474;
	5768	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.7;
	5768	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.3;
	5768	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.9;
	5768	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.9;
	5768	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.2;
	5769	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.6;
	5769	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.4;
	5769	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	264.4;
	5769	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.2;
	5769	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	5769	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.3;
	5770	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.4;
	5770	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162;
	5770	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	277.5;
	5770	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	276.2;
	5770	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	5770	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398;
	5771	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525.2;
	5771	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.4;
	5771	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	289.2;
	5771	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	287.3;
	5771	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70;
	5771	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.6;
	5772	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	540.3;
	5772	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173;
	5772	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	303;
	5772	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	300;
	5772	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.8;
	5772	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.7;
	5773	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.2;
	5773	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.6;
	5773	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	314.1;
	5773	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	310.7;
	5773	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.2;
	5773	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	426.9;
	5774	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	546.1;
	5774	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.3;
	5774	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	325.1;
	5774	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	320.7;
	5774	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.2;
	5774	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	432.3;
	5775	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559.5;
	5775	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.9;
	5775	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	334;
	5775	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	329.1;
	5775	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.1;
	5775	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	440.9;
	5776	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.8;
	5776	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.8;
	5776	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	334.9;
	5776	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	330.4;
	5776	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.8;
	5776	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	447.3;
	5777	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	569.1;
	5777	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.7;
	5777	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	335.8;
	5777	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	331.3;
	5777	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.9;
	5777	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	446;
	5778	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	568.2;
	5778	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186;
	5778	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	327.9;
	5778	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	323.1;
	5778	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88;
	5778	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	443.2;
	5779	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	561.9;
	5779	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.7;
	5779	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	312.4;
	5779	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	308.7;
	5779	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.4;
	5779	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	436.1;
	5780	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.7;
	5780	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.9;
	5780	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	302.8;
	5780	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	299.4;
	5780	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.2;
	5780	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.9;
	5781	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.9;
	5781	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.4;
	5781	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	290;
	5781	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	288.2;
	5781	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.8;
	5781	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.8;
	5782	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.7;
	5782	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164;
	5782	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	273.3;
	5782	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	270.8;
	5782	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.5;
	5782	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.6;
	5783	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.8;
	5783	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.1;
	5783	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.7;
	5783	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.1;
	5783	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.7;
	5783	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.2;
	5784	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.2;
	5784	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.5;
	5784	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.3;
	5784	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239;
	5784	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	5784	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.9;
	5785	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.3;
	5785	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.8;
	5785	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223;
	5785	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.1;
	5785	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	5785	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.8;
	5786	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	401.3;
	5786	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.5;
	5786	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.3;
	5786	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.3;
	5786	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	5786	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.7;
	5787	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394.9;
	5787	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.9;
	5787	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.1;
	5787	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.6;
	5787	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	5787	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.3;
	5788	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.7;
	5788	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.6;
	5788	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.8;
	5788	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.5;
	5788	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.7;
	5788	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.5;
	5789	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414;
	5789	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.6;
	5789	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.9;
	5789	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.9;
	5789	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.6;
	5789	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.6;
	5790	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.8;
	5790	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.5;
	5790	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.2;
	5790	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.1;
	5790	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.2;
	5790	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.9;
	5791	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.9;
	5791	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141;
	5791	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.9;
	5791	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.4;
	5791	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.6;
	5791	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.3;
	5792	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.4;
	5792	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.2;
	5792	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.7;
	5792	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.9;
	5792	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	5792	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.1;
	5793	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.6;
	5793	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154;
	5793	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.8;
	5793	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.6;
	5793	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	5793	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.2;
	5794	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.5;
	5794	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.4;
	5794	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	273.4;
	5794	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	274.8;
	5794	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72;
	5794	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.8;
	5795	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	555.5;
	5795	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.8;
	5795	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	286.2;
	5795	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	287.6;
	5795	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.6;
	5795	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.7;
	5796	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556;
	5796	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.2;
	5796	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	296.2;
	5796	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	297.4;
	5796	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.7;
	5796	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.5;
	5797	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572;
	5797	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.1;
	5797	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	309.4;
	5797	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	309.9;
	5797	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.3;
	5797	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	436.6;
	5798	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	583.8;
	5798	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.3;
	5798	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	315.1;
	5798	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	316;
	5798	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.7;
	5798	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	444.2;
	5799	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	591.9;
	5799	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177;
	5799	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	319.3;
	5799	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	320.6;
	5799	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.8;
	5799	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	449;
	5800	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	590;
	5800	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.8;
	5800	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	321.3;
	5800	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	322;
	5800	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.8;
	5800	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	448.9;
	5801	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	585.9;
	5801	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183;
	5801	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	320.6;
	5801	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	321.3;
	5801	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.9;
	5801	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	447.3;
	5802	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	576.7;
	5802	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.7;
	5802	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	311.8;
	5802	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	310.8;
	5802	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.2;
	5802	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	441.1;
	5803	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566.2;
	5803	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.8;
	5803	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	301.8;
	5803	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	301.6;
	5803	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.2;
	5803	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	432.6;
	5804	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.3;
	5804	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.5;
	5804	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	296.2;
	5804	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	296.5;
	5804	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.7;
	5804	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.2;
	5805	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549.4;
	5805	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.5;
	5805	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	285.5;
	5805	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	285.7;
	5805	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.7;
	5805	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.9;
	5806	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.7;
	5806	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.2;
	5806	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	269.4;
	5806	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	269.6;
	5806	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.2;
	5806	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.5;
	5807	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.6;
	5807	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.3;
	5807	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.6;
	5807	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.8;
	5807	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.7;
	5807	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.3;
	5808	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.2;
	5808	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.6;
	5808	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.6;
	5808	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.7;
	5808	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.1;
	5808	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.7;
	5809	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.8;
	5809	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129;
	5809	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219;
	5809	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.4;
	5809	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.4;
	5809	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.5;
	5810	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.1;
	5810	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.6;
	5810	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.8;
	5810	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.7;
	5810	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	5810	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.8;
	5811	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	396.3;
	5811	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.6;
	5811	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.1;
	5811	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.5;
	5811	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	5811	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.5;
	5812	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.6;
	5812	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.3;
	5812	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.6;
	5812	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.2;
	5812	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	5812	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.6;
	5813	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.7;
	5813	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.3;
	5813	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.8;
	5813	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.9;
	5813	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60;
	5813	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.6;
	5814	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.9;
	5814	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.8;
	5814	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.8;
	5814	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216;
	5814	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.2;
	5814	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.5;
	5815	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.3;
	5815	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.4;
	5815	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.5;
	5815	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.6;
	5815	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.4;
	5815	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.6;
	5816	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.2;
	5816	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.4;
	5816	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242;
	5816	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.4;
	5816	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	5816	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377;
	5817	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512;
	5817	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.4;
	5817	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.2;
	5817	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260;
	5817	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	5817	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.4;
	5818	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528.4;
	5818	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.6;
	5818	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	266.9;
	5818	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	271.1;
	5818	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.3;
	5818	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.7;
	5819	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544;
	5819	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.1;
	5819	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	276.4;
	5819	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	280.8;
	5819	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.9;
	5819	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415;
	5820	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559.8;
	5820	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165;
	5820	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	286;
	5820	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	290.4;
	5820	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.2;
	5820	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	424.1;
	5821	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579;
	5821	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.5;
	5821	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	290.9;
	5821	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	296.4;
	5821	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.3;
	5821	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	434.8;
	5822	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.9;
	5822	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.4;
	5822	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	298;
	5822	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	302.9;
	5822	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.5;
	5822	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	442.2;
	5823	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.3;
	5823	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.5;
	5823	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	306.1;
	5823	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	309;
	5823	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.5;
	5823	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	444.1;
	5824	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.7;
	5824	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.2;
	5824	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	310.8;
	5824	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	312;
	5824	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.1;
	5824	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	447.7;
	5825	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	603.5;
	5825	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173;
	5825	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	305;
	5825	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	305.5;
	5825	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.5;
	5825	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	449.1;
	5826	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	599.4;
	5826	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.1;
	5826	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	292.1;
	5826	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	293.5;
	5826	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.5;
	5826	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	443.1;
	5827	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	587;
	5827	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.2;
	5827	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	280.8;
	5827	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	282.8;
	5827	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.2;
	5827	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	433.1;
	5828	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	576.6;
	5828	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.2;
	5828	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	275.9;
	5828	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	277.9;
	5828	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.8;
	5828	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.9;
	5829	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	567.3;
	5829	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.2;
	5829	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	269.3;
	5829	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	270.8;
	5829	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.6;
	5829	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.4;
	5830	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547.5;
	5830	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.1;
	5830	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.6;
	5830	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.1;
	5830	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.5;
	5830	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.6;
	5831	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.5;
	5831	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.7;
	5831	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.2;
	5831	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.4;
	5831	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.6;
	5831	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.1;
	5832	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.9;
	5832	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.5;
	5832	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.8;
	5832	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234;
	5832	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	5832	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.5;
	5833	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.9;
	5833	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.3;
	5833	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.5;
	5833	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.3;
	5833	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	5833	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.4;
	5834	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.9;
	5834	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.4;
	5834	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.6;
	5834	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.7;
	5834	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	5834	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.9;
	5835	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	402.4;
	5835	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.9;
	5835	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201;
	5835	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203;
	5835	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	5835	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.2;
	5836	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	386.1;
	5836	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.8;
	5836	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.3;
	5836	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.3;
	5836	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.1;
	5836	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.4;
	5837	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.6;
	5837	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.2;
	5837	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.8;
	5837	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.1;
	5837	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.6;
	5837	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.2;
	5838	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.7;
	5838	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.4;
	5838	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.7;
	5838	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.2;
	5838	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.6;
	5838	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.8;
	5839	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.3;
	5839	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139;
	5839	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.9;
	5839	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.8;
	5839	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	5839	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.4;
	5840	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.8;
	5840	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.5;
	5840	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.8;
	5840	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.1;
	5840	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	5840	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.5;
	5841	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.5;
	5841	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.4;
	5841	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.5;
	5841	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.9;
	5841	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68;
	5841	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.7;
	5842	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.6;
	5842	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.9;
	5842	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	266;
	5842	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	268.7;
	5842	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.6;
	5842	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.6;
	5843	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548.2;
	5843	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.8;
	5843	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	277.6;
	5843	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	279.6;
	5843	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77;
	5843	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.5;
	5844	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564.3;
	5844	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.4;
	5844	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	288.7;
	5844	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	290.1;
	5844	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.1;
	5844	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	426.4;
	5845	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	582.9;
	5845	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.4;
	5845	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	296.6;
	5845	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	297.4;
	5845	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.3;
	5845	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	437.6;
	5846	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	590.8;
	5846	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.5;
	5846	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	309.6;
	5846	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	309;
	5846	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.2;
	5846	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	444.4;
	5847	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	604.6;
	5847	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.7;
	5847	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	315.9;
	5847	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	314.9;
	5847	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.3;
	5847	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	452.3;
	5848	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	611.5;
	5848	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.1;
	5848	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	318.5;
	5848	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	316.6;
	5848	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.3;
	5848	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	456.3;
	5849	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579.5;
	5849	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.8;
	5849	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	317.8;
	5849	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	314.3;
	5849	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.9;
	5849	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	443.7;
	5850	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	607.6;
	5850	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.7;
	5850	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	309.2;
	5850	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	305.3;
	5850	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.5;
	5850	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	452.1;
	5851	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	608.8;
	5851	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.4;
	5851	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	299.2;
	5851	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	296.1;
	5851	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.2;
	5851	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	448.5;
	5852	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	581.4;
	5852	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.2;
	5852	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	289.5;
	5852	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	288.2;
	5852	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.2;
	5852	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	433.8;
	5853	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	563.1;
	5853	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.2;
	5853	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	278.3;
	5853	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	276.7;
	5853	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.6;
	5853	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.5;
	5854	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.6;
	5854	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.9;
	5854	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.1;
	5854	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261;
	5854	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.4;
	5854	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.9;
	5855	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505;
	5855	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.1;
	5855	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.1;
	5855	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.4;
	5855	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.8;
	5855	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.9;
	5856	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.1;
	5856	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.8;
	5856	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.2;
	5856	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.8;
	5856	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.3;
	5856	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.2;
	5857	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.7;
	5857	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.9;
	5857	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.9;
	5857	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.5;
	5857	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.5;
	5857	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.7;
	5858	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394.9;
	5858	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.8;
	5858	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.8;
	5858	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.7;
	5858	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	5858	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331;
	5859	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391.4;
	5859	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.9;
	5859	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.2;
	5859	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205;
	5859	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	5859	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.8;
	5860	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391.6;
	5860	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.8;
	5860	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.8;
	5860	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.8;
	5860	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	5860	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.9;
	5861	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394.8;
	5861	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.2;
	5861	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.8;
	5861	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205;
	5861	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	5861	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329;
	5862	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.1;
	5862	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.8;
	5862	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.3;
	5862	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.8;
	5862	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	5862	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.4;
	5863	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.7;
	5863	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.4;
	5863	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.1;
	5863	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.2;
	5863	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	5863	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340;
	5864	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.1;
	5864	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.3;
	5864	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.6;
	5864	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.8;
	5864	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.5;
	5864	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.7;
	5865	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.1;
	5865	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.8;
	5865	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.4;
	5865	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.1;
	5865	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.1;
	5865	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.5;
	5866	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.1;
	5866	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.9;
	5866	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.9;
	5866	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.6;
	5866	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.7;
	5866	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.8;
	5867	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498;
	5867	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.8;
	5867	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.6;
	5867	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.6;
	5867	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.2;
	5867	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.7;
	5868	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.8;
	5868	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160;
	5868	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	266.6;
	5868	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267.3;
	5868	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.1;
	5868	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.1;
	5869	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.2;
	5869	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.8;
	5869	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	279;
	5869	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	279.4;
	5869	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.4;
	5869	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413;
	5870	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	553.3;
	5870	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.5;
	5870	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	281.9;
	5870	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	282.9;
	5870	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.1;
	5870	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420.8;
	5871	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.3;
	5871	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.7;
	5871	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	286.1;
	5871	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	287.8;
	5871	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.7;
	5871	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.7;
	5872	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	563.9;
	5872	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.2;
	5872	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	292;
	5872	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	294.1;
	5872	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.8;
	5872	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	427.1;
	5873	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	595.3;
	5873	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.1;
	5873	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	292.1;
	5873	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	294.6;
	5873	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.4;
	5873	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	440.4;
	5874	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	594.8;
	5874	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.1;
	5874	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	286.8;
	5874	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	288.7;
	5874	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.2;
	5874	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	438.2;
	5875	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577.6;
	5875	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.1;
	5875	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	284.1;
	5875	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	286.6;
	5875	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.8;
	5875	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	429.1;
	5876	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	563.6;
	5876	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.2;
	5876	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	281.4;
	5876	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	284.3;
	5876	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.7;
	5876	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.1;
	5877	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.5;
	5877	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.9;
	5877	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	272.4;
	5877	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	275.4;
	5877	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.7;
	5877	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.1;
	5878	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525;
	5878	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.9;
	5878	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.9;
	5878	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265.3;
	5878	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.4;
	5878	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.8;
	5879	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.8;
	5879	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.1;
	5879	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.6;
	5879	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.9;
	5879	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.1;
	5879	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.2;
	5880	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.7;
	5880	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140;
	5880	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.3;
	5880	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.7;
	5880	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.1;
	5880	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.1;
	5881	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.4;
	5881	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.3;
	5881	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.2;
	5881	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225;
	5881	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	5881	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.4;
	5882	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	380.1;
	5882	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.8;
	5882	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.7;
	5882	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.4;
	5882	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	5882	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.7;
	5883	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	367.3;
	5883	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.1;
	5883	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.1;
	5883	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.8;
	5883	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.1;
	5883	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.4;
	5884	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	356.8;
	5884	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117;
	5884	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.6;
	5884	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.3;
	5884	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.1;
	5884	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.2;
	5885	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	351.5;
	5885	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.6;
	5885	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.7;
	5885	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.7;
	5885	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56;
	5885	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.3;
	5886	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	364.5;
	5886	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.7;
	5886	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.6;
	5886	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.6;
	5886	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.6;
	5886	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.4;
	5887	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.5;
	5887	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.6;
	5887	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.5;
	5887	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.7;
	5887	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.4;
	5887	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.1;
	5888	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	389;
	5888	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.2;
	5888	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.6;
	5888	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.9;
	5888	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	5888	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.2;
	5889	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.7;
	5889	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.1;
	5889	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.3;
	5889	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.6;
	5889	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	5889	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.2;
	5890	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	444.8;
	5890	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.1;
	5890	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.3;
	5890	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241;
	5890	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	5890	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.2;
	5891	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.9;
	5891	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154;
	5891	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.7;
	5891	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.2;
	5891	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	5891	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.6;
	5892	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.1;
	5892	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.3;
	5892	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.1;
	5892	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	263.6;
	5892	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.5;
	5892	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.7;
	5893	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.3;
	5893	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.9;
	5893	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	269.4;
	5893	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	275.4;
	5893	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.5;
	5893	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.5;
	5894	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.1;
	5894	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.8;
	5894	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	283.3;
	5894	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	288.8;
	5894	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.6;
	5894	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.3;
	5895	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.8;
	5895	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.5;
	5895	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	296.3;
	5895	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	299.1;
	5895	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.2;
	5895	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.5;
	5896	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.8;
	5896	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.2;
	5896	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	300.6;
	5896	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	302.9;
	5896	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.3;
	5896	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413;
	5897	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.5;
	5897	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.2;
	5897	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	297.8;
	5897	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	299.8;
	5897	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78;
	5897	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.7;
	5898	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535;
	5898	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.5;
	5898	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	293.1;
	5898	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	294.8;
	5898	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.9;
	5898	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.2;
	5899	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526.2;
	5899	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.3;
	5899	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	291.4;
	5899	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	294.1;
	5899	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.8;
	5899	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.7;
	5900	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517;
	5900	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.1;
	5900	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	288.4;
	5900	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	292.1;
	5900	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.9;
	5900	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.6;
	5901	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.5;
	5901	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.6;
	5901	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	282;
	5901	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	285.8;
	5901	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71;
	5901	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.7;
	5902	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.8;
	5902	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.3;
	5902	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	269.8;
	5902	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	273.3;
	5902	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.5;
	5902	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.9;
	5903	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.8;
	5903	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.4;
	5903	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.9;
	5903	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.6;
	5903	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	5903	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.4;
	5904	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.3;
	5904	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.5;
	5904	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.8;
	5904	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.4;
	5904	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	5904	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.7;
	5905	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	389.1;
	5905	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.9;
	5905	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.3;
	5905	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.5;
	5905	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.4;
	5905	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.8;
	5906	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	364.4;
	5906	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.6;
	5906	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219;
	5906	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.4;
	5906	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.9;
	5906	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.4;
	5907	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	351.3;
	5907	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.8;
	5907	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.2;
	5907	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.4;
	5907	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.4;
	5907	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.7;
	5908	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	346;
	5908	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.6;
	5908	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.1;
	5908	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.2;
	5908	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.6;
	5908	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.4;
	5909	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	351;
	5909	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.8;
	5909	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.2;
	5909	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.1;
	5909	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.8;
	5909	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.5;
	5910	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	358.4;
	5910	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.4;
	5910	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219;
	5910	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.1;
	5910	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.3;
	5910	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.1;
	5911	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	373.5;
	5911	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.3;
	5911	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.2;
	5911	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.8;
	5911	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.4;
	5911	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329;
	5912	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.1;
	5912	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.1;
	5912	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.2;
	5912	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.7;
	5912	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.5;
	5912	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.8;
	5913	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428.2;
	5913	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.5;
	5913	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.6;
	5913	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256;
	5913	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60;
	5913	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.2;
	5914	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.1;
	5914	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.8;
	5914	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	267.3;
	5914	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	269.8;
	5914	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	5914	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.4;
	5915	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.8;
	5915	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.3;
	5915	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	281.2;
	5915	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	283.8;
	5915	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	5915	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.9;
	5916	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.1;
	5916	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.6;
	5916	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	293.9;
	5916	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	295.9;
	5916	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	5916	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.3;
	5917	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.5;
	5917	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.2;
	5917	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	303.2;
	5917	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	304.5;
	5917	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.5;
	5917	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	424;
	5918	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.1;
	5918	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.4;
	5918	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	312.7;
	5918	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	311.5;
	5918	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.2;
	5918	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	432.3;
	5919	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	578.4;
	5919	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.5;
	5919	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	313.6;
	5919	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	312.9;
	5919	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.8;
	5919	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	442.6;
	5920	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588.7;
	5920	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.1;
	5920	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	315;
	5920	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	314.6;
	5920	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	5920	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	447;
	5921	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	587.2;
	5921	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.6;
	5921	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	319.5;
	5921	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	318.7;
	5921	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.8;
	5921	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	447.1;
	5922	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577;
	5922	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.8;
	5922	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	317.6;
	5922	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	317.1;
	5922	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.1;
	5922	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	442;
	5923	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	561.9;
	5923	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.7;
	5923	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	311.4;
	5923	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	311.3;
	5923	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.3;
	5923	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	434;
	5924	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.4;
	5924	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187;
	5924	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	303.7;
	5924	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	305.4;
	5924	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.3;
	5924	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.5;
	5925	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537;
	5925	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.3;
	5925	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	296.3;
	5925	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	297.5;
	5925	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	5925	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.8;
	5926	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522.9;
	5926	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.1;
	5926	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	280;
	5926	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	281;
	5926	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	5926	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.2;
	5927	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.6;
	5927	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.9;
	5927	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.5;
	5927	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.2;
	5927	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	5927	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.2;
	5928	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447.1;
	5928	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.6;
	5928	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.4;
	5928	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.4;
	5928	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.3;
	5928	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.3;
	5929	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.2;
	5929	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.4;
	5929	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.9;
	5929	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.4;
	5929	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	5929	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.9;
	5930	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	385.1;
	5930	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.5;
	5930	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.4;
	5930	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.1;
	5930	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.6;
	5930	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.5;
	5931	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	375;
	5931	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.2;
	5931	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.8;
	5931	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213;
	5931	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.4;
	5931	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.4;
	5932	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	373.7;
	5932	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.5;
	5932	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.6;
	5932	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211;
	5932	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.8;
	5932	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322;
	5933	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379.4;
	5933	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.6;
	5933	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.2;
	5933	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.6;
	5933	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.8;
	5933	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.2;
	5934	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	406.8;
	5934	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.8;
	5934	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.8;
	5934	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.1;
	5934	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.8;
	5934	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.8;
	5935	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.5;
	5935	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.5;
	5935	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.3;
	5935	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.6;
	5935	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.2;
	5935	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.1;
	5936	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.4;
	5936	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.7;
	5936	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.3;
	5936	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.4;
	5936	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.8;
	5936	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.1;
	5937	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.5;
	5937	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.8;
	5937	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	272.7;
	5937	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	274.1;
	5937	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.2;
	5937	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.9;
	5938	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.7;
	5938	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.7;
	5938	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	285.7;
	5938	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	285.6;
	5938	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	5938	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420.6;
	5939	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.8;
	5939	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.2;
	5939	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	298.3;
	5939	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	296.3;
	5939	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	5939	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	433.6;
	5940	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	590.2;
	5940	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.7;
	5940	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	311.3;
	5940	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	306.7;
	5940	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	5940	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	444.5;
	5941	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	593.8;
	5941	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.7;
	5941	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	320.8;
	5941	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	314.2;
	5941	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	5941	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	450.3;
	5942	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	603;
	5942	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.8;
	5942	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	329.3;
	5942	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	321;
	5942	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	5942	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	457.3;
	5943	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	611.5;
	5943	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.4;
	5943	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	336.1;
	5943	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	327;
	5943	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.3;
	5943	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	463;
	5944	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	614.5;
	5944	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.4;
	5944	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	343.9;
	5944	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	333.3;
	5944	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.9;
	5944	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	466.3;
	5945	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	610.7;
	5945	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.4;
	5945	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	341;
	5945	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	330.2;
	5945	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	5945	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	464.9;
	5946	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	604.8;
	5946	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.5;
	5946	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	332.7;
	5946	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	322.5;
	5946	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	5946	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	460.1;
	5947	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	591.4;
	5947	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190;
	5947	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	319.8;
	5947	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	312.6;
	5947	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70;
	5947	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	450.5;
	5948	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	580.3;
	5948	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.9;
	5948	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	313.7;
	5948	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	307.4;
	5948	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.8;
	5948	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	442.4;
	5949	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579.4;
	5949	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.5;
	5949	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	303;
	5949	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	296.4;
	5949	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.5;
	5949	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	438.4;
	5950	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.2;
	5950	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.6;
	5950	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	284.6;
	5950	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	278.5;
	5950	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	5950	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.3;
	5951	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.7;
	5951	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.8;
	5951	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.6;
	5951	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.2;
	5951	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	5951	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.5;
	5952	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.8;
	5952	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.7;
	5952	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252;
	5952	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249;
	5952	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	5952	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376;
	5953	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.2;
	5953	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136;
	5953	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235;
	5953	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.1;
	5953	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.1;
	5953	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.1;
	5954	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.5;
	5954	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.6;
	5954	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.5;
	5954	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.7;
	5954	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.1;
	5954	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.2;
	5955	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	406.4;
	5955	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.9;
	5955	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.2;
	5955	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213;
	5955	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.9;
	5955	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.6;
	5956	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.3;
	5956	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124;
	5956	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.6;
	5956	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.7;
	5956	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.3;
	5956	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.7;
	5957	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.7;
	5957	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.7;
	5957	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.2;
	5957	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.9;
	5957	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.4;
	5957	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.8;
	5958	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.8;
	5958	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.6;
	5958	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226;
	5958	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.1;
	5958	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.1;
	5958	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.8;
	5959	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.9;
	5959	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.1;
	5959	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.7;
	5959	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.9;
	5959	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.9;
	5959	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.2;
	5960	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499;
	5960	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.7;
	5960	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.1;
	5960	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.8;
	5960	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.4;
	5960	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.5;
	5961	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.1;
	5961	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160;
	5961	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	274.3;
	5961	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	273.6;
	5961	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.2;
	5961	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.7;
	5962	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528.8;
	5962	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.8;
	5962	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	285.7;
	5962	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	283.2;
	5962	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	5962	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.6;
	5963	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.9;
	5963	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.8;
	5963	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	293.6;
	5963	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	290.2;
	5963	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.3;
	5963	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	426.3;
	5964	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	557.5;
	5964	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173;
	5964	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	302.1;
	5964	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	297.3;
	5964	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	5964	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	429;
	5965	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	571.9;
	5965	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.7;
	5965	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	311.8;
	5965	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	305.8;
	5965	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	5965	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	438.5;
	5966	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588.6;
	5966	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.4;
	5966	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	321.4;
	5966	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	314.1;
	5966	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	5966	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	448.4;
	5967	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	605.6;
	5967	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.1;
	5967	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	329.5;
	5967	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	320.6;
	5967	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.4;
	5967	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	457.4;
	5968	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	614.1;
	5968	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185;
	5968	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	334.9;
	5968	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	325.3;
	5968	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	5968	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	463;
	5969	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	617.5;
	5969	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.6;
	5969	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	336.2;
	5969	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	326.1;
	5969	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.3;
	5969	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	465.3;
	5970	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	608.9;
	5970	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.9;
	5970	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	326.5;
	5970	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	317.4;
	5970	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	5970	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	459.5;
	5971	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	593.7;
	5971	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.3;
	5971	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	314.9;
	5971	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	308.2;
	5971	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.6;
	5971	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	448.8;
	5972	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579.8;
	5972	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.4;
	5972	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	305.4;
	5972	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	300.3;
	5972	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.1;
	5972	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	439.4;
	5973	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	574.8;
	5973	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.4;
	5973	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	296;
	5973	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	290.6;
	5973	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.9;
	5973	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	433.7;
	5974	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.1;
	5974	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.6;
	5974	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	276.2;
	5974	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272;
	5974	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.8;
	5974	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.9;
	5975	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.6;
	5975	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.5;
	5975	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.1;
	5975	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.8;
	5975	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	5975	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.9;
	5976	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.4;
	5976	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.4;
	5976	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.3;
	5976	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.6;
	5976	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	5976	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.3;
	5977	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.4;
	5977	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.4;
	5977	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.7;
	5977	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220;
	5977	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.9;
	5977	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.7;
	5978	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428.7;
	5978	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.2;
	5978	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213;
	5978	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.3;
	5978	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.1;
	5978	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.5;
	5979	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.4;
	5979	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.6;
	5979	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.8;
	5979	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.9;
	5979	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.7;
	5979	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341;
	5980	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.7;
	5980	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.7;
	5980	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.8;
	5980	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.5;
	5980	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.6;
	5980	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.6;
	5981	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.5;
	5981	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126;
	5981	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.2;
	5981	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212;
	5981	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.7;
	5981	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.6;
	5982	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.3;
	5982	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.2;
	5982	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.4;
	5982	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.3;
	5982	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.9;
	5982	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.5;
	5983	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.4;
	5983	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148;
	5983	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.6;
	5983	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.6;
	5983	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.9;
	5983	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.6;
	5984	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.9;
	5984	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.4;
	5984	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.5;
	5984	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.9;
	5984	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.9;
	5984	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.7;
	5985	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548.1;
	5985	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.9;
	5985	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	264.4;
	5985	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265.3;
	5985	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	5985	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.7;
	5986	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	569.8;
	5986	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.8;
	5986	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	274.3;
	5986	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	275;
	5986	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.7;
	5986	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	424.5;
	5987	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	580.6;
	5987	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.8;
	5987	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	283;
	5987	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	283.2;
	5987	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.3;
	5987	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	431.7;
	5988	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	603.5;
	5988	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.3;
	5988	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	292.1;
	5988	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	292.3;
	5988	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.9;
	5988	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	443.2;
	5989	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	618.8;
	5989	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.3;
	5989	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	300.7;
	5989	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	300.2;
	5989	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.6;
	5989	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	453;
	5990	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	635.3;
	5990	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.8;
	5990	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	310.6;
	5990	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	310.6;
	5990	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.7;
	5990	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	463;
	5991	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	645.9;
	5991	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.7;
	5991	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	320.2;
	5991	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	318.6;
	5991	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.8;
	5991	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	469.9;
	5992	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	649;
	5992	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.9;
	5992	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	322.4;
	5992	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	320.7;
	5992	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.4;
	5992	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	472.5;
	5993	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	642.8;
	5993	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.4;
	5993	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	318.9;
	5993	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	318.3;
	5993	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.7;
	5993	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	469.6;
	5994	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	627.6;
	5994	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.6;
	5994	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	309.9;
	5994	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	309.9;
	5994	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.8;
	5994	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	460.5;
	5995	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	605.2;
	5995	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.4;
	5995	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	299.5;
	5995	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	300.8;
	5995	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.6;
	5995	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	447.2;
	5996	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589.7;
	5996	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.5;
	5996	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	296.6;
	5996	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	298.5;
	5996	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.3;
	5996	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	438.6;
	5997	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	575.3;
	5997	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.8;
	5997	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	285.5;
	5997	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	287.3;
	5997	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.5;
	5997	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	429.1;
	5998	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	546.8;
	5998	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.3;
	5998	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	268.2;
	5998	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	269.5;
	5998	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.6;
	5998	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.2;
	5999	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.8;
	5999	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.1;
	5999	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.4;
	5999	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.6;
	5999	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.9;
	5999	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.7;
	6000	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.4;
	6000	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.6;
	6000	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.8;
	6000	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.5;
	6000	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	6000	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.8;
	6001	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.1;
	6001	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.7;
	6001	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.8;
	6001	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.9;
	6001	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	6001	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.8;
	6002	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	420.6;
	6002	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.2;
	6002	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.9;
	6002	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.1;
	6002	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	6002	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.1;
	6003	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417.5;
	6003	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.2;
	6003	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.4;
	6003	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.5;
	6003	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	6003	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.3;
	6004	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.3;
	6004	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.5;
	6004	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.6;
	6004	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.7;
	6004	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.1;
	6004	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.3;
	6005	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.6;
	6005	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.5;
	6005	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208;
	6005	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.5;
	6005	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.3;
	6005	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.4;
	6006	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.4;
	6006	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134;
	6006	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.5;
	6006	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.1;
	6006	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.9;
	6006	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.9;
	6007	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.7;
	6007	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.5;
	6007	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.7;
	6007	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.3;
	6007	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	6007	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.1;
	6008	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.2;
	6008	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.7;
	6008	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.1;
	6008	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.5;
	6008	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	6008	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.7;
	6009	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.9;
	6009	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.5;
	6009	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.4;
	6009	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.2;
	6009	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	6009	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.9;
	6010	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.7;
	6010	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.6;
	6010	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	272.6;
	6010	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	276.1;
	6010	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.9;
	6010	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.2;
	6011	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.4;
	6011	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.1;
	6011	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	283.5;
	6011	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	287.2;
	6011	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.3;
	6011	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.7;
	6012	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.7;
	6012	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.1;
	6012	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	294.7;
	6012	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	298.5;
	6012	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.9;
	6012	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.3;
	6013	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.8;
	6013	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.2;
	6013	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	303.3;
	6013	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	306.9;
	6013	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.8;
	6013	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	432.2;
	6014	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.4;
	6014	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.1;
	6014	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	308.1;
	6014	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	312.2;
	6014	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.7;
	6014	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	438.3;
	6015	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	567;
	6015	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.6;
	6015	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	312.3;
	6015	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	317.8;
	6015	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.4;
	6015	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	437.3;
	6016	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559.2;
	6016	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.5;
	6016	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	313.7;
	6016	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	320.2;
	6016	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.3;
	6016	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	434.9;
	6017	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	553.6;
	6017	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.9;
	6017	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	310.9;
	6017	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	317;
	6017	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.7;
	6017	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	432.1;
	6018	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.1;
	6018	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.7;
	6018	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	300.6;
	6018	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	306.4;
	6018	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.1;
	6018	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	427.7;
	6019	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.7;
	6019	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.1;
	6019	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	293.1;
	6019	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	298.1;
	6019	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.7;
	6019	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419;
	6020	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	524.3;
	6020	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.1;
	6020	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	286.5;
	6020	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	290.8;
	6020	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.1;
	6020	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.8;
	6021	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.2;
	6021	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.7;
	6021	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	274.1;
	6021	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	278.2;
	6021	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85;
	6021	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.1;
	6022	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.3;
	6022	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.9;
	6022	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	259.9;
	6022	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	263.6;
	6022	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.8;
	6022	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.2;
	6023	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.6;
	6023	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.5;
	6023	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.2;
	6023	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.8;
	6023	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.2;
	6023	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.9;
	6024	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	430.3;
	6024	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.3;
	6024	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.9;
	6024	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.9;
	6024	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.8;
	6024	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.6;
	6025	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.3;
	6025	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.2;
	6025	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.5;
	6025	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.5;
	6025	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69;
	6025	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.8;
	6026	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.5;
	6026	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.3;
	6026	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.7;
	6026	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.4;
	6026	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	6026	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.5;
	6027	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	374.8;
	6027	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.9;
	6027	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206;
	6027	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.9;
	6027	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	6027	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.6;
	6028	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	380;
	6028	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.4;
	6028	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.5;
	6028	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.1;
	6028	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	6028	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.8;
	6029	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	387.5;
	6029	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.1;
	6029	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.5;
	6029	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.5;
	6029	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	6029	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.5;
	6030	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	400.6;
	6030	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.1;
	6030	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.9;
	6030	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.8;
	6030	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.3;
	6030	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.7;
	6031	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.2;
	6031	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.7;
	6031	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.8;
	6031	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.5;
	6031	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	6031	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.6;
	6032	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.2;
	6032	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.6;
	6032	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.3;
	6032	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.2;
	6032	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.1;
	6032	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342;
	6033	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.5;
	6033	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.3;
	6033	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.4;
	6033	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.7;
	6033	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.5;
	6033	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.2;
	6034	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442;
	6034	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.5;
	6034	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.9;
	6034	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.3;
	6034	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.7;
	6034	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.7;
	6035	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.2;
	6035	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154;
	6035	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.1;
	6035	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.1;
	6035	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.3;
	6035	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.2;
	6036	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.4;
	6036	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.4;
	6036	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.4;
	6036	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265.5;
	6036	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.9;
	6036	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.9;
	6037	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.2;
	6037	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.3;
	6037	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	270.6;
	6037	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	275.5;
	6037	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.9;
	6037	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.6;
	6038	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.9;
	6038	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.7;
	6038	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	274.9;
	6038	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	279.4;
	6038	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.3;
	6038	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.5;
	6039	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.5;
	6039	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.3;
	6039	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	277.8;
	6039	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	283;
	6039	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	97.9;
	6039	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.2;
	6040	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.7;
	6040	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.5;
	6040	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	283.2;
	6040	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	286.5;
	6040	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	100.8;
	6040	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387;
	6041	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473;
	6041	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162;
	6041	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	279;
	6041	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	282.2;
	6041	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	102.2;
	6041	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.3;
	6042	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.7;
	6042	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.8;
	6042	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	271.4;
	6042	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	274.5;
	6042	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	101.5;
	6042	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.4;
	6043	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.2;
	6043	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155;
	6043	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	268.2;
	6043	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	271.4;
	6043	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	98.5;
	6043	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.1;
	6044	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.9;
	6044	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.3;
	6044	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	265;
	6044	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267.7;
	6044	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96.2;
	6044	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.4;
	6045	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.3;
	6045	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.4;
	6045	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.8;
	6045	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.1;
	6045	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.9;
	6045	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.7;
	6046	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.7;
	6046	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.9;
	6046	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.3;
	6046	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.4;
	6046	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89;
	6046	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.5;
	6047	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.2;
	6047	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.6;
	6047	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.3;
	6047	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.3;
	6047	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.9;
	6047	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.4;
	6048	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	378.6;
	6048	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.6;
	6048	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.8;
	6048	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.8;
	6048	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.9;
	6048	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.3;
	6049	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	357.4;
	6049	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.1;
	6049	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.9;
	6049	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.4;
	6049	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.5;
	6049	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.3;
	6050	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	338.8;
	6050	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.6;
	6050	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.1;
	6050	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.2;
	6050	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.6;
	6050	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	304.8;
	6051	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	335.6;
	6051	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.8;
	6051	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.1;
	6051	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.3;
	6051	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.1;
	6051	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	301.7;
	6052	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	329.7;
	6052	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.1;
	6052	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.1;
	6052	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.3;
	6052	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	6052	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	298.8;
	6053	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	340.2;
	6053	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.2;
	6053	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.1;
	6053	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.1;
	6053	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.1;
	6053	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	303.4;
	6054	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	342.9;
	6054	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.5;
	6054	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.9;
	6054	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198;
	6054	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	6054	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	305.9;
	6055	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	357.9;
	6055	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.9;
	6055	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.4;
	6055	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.3;
	6055	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	6055	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.4;
	6056	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	356.4;
	6056	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.3;
	6056	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.4;
	6056	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.5;
	6056	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.3;
	6056	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.3;
	6057	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	378;
	6057	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.5;
	6057	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.2;
	6057	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214;
	6057	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.8;
	6057	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.9;
	6058	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.6;
	6058	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.5;
	6058	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.6;
	6058	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.8;
	6058	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.6;
	6058	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.2;
	6059	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425;
	6059	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.9;
	6059	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.6;
	6059	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.1;
	6059	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.7;
	6059	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.3;
	6060	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.2;
	6060	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.5;
	6060	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.4;
	6060	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.3;
	6060	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92;
	6060	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.3;
	6061	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.4;
	6061	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.2;
	6061	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.9;
	6061	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.6;
	6061	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	98.2;
	6061	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368;
	6062	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.1;
	6062	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.7;
	6062	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.2;
	6062	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.7;
	6062	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	101.3;
	6062	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.4;
	6063	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.7;
	6063	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.1;
	6063	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.1;
	6063	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.6;
	6063	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	103.3;
	6063	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.9;
	6064	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.7;
	6064	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.8;
	6064	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261;
	6064	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.2;
	6064	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	104;
	6064	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.5;
	6065	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.6;
	6065	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.9;
	6065	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	263.2;
	6065	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.7;
	6065	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	103.5;
	6065	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.6;
	6066	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.8;
	6066	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.7;
	6066	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.1;
	6066	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.6;
	6066	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	102.1;
	6066	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.5;
	6067	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467;
	6067	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.3;
	6067	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.4;
	6067	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.6;
	6067	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	98;
	6067	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.9;
	6068	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.4;
	6068	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.7;
	6068	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.4;
	6068	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.8;
	6068	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.6;
	6068	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.3;
	6069	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.9;
	6069	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.1;
	6069	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.5;
	6069	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.9;
	6069	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.5;
	6069	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369;
	6070	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.4;
	6070	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.3;
	6070	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.4;
	6070	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.4;
	6070	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.9;
	6070	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.3;
	6071	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.4;
	6071	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130;
	6071	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220;
	6071	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.1;
	6071	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.6;
	6071	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.2;
	6072	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.8;
	6072	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.3;
	6072	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.3;
	6072	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.7;
	6072	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.9;
	6072	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.2;
	6073	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	372.5;
	6073	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.4;
	6073	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.7;
	6073	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.8;
	6073	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.5;
	6073	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.9;
	6074	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	356.6;
	6074	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.5;
	6074	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.4;
	6074	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198;
	6074	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.1;
	6074	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309.4;
	6075	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	348.3;
	6075	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.3;
	6075	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.8;
	6075	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194;
	6075	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.1;
	6075	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	304.9;
	6076	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	350.2;
	6076	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.3;
	6076	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.8;
	6076	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.2;
	6076	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	6076	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	306;
	6077	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	353.6;
	6077	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.6;
	6077	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.4;
	6077	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199;
	6077	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	6077	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.8;
	6078	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	389.1;
	6078	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118;
	6078	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.9;
	6078	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.1;
	6078	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	6078	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.1;
	6079	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.7;
	6079	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.4;
	6079	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.9;
	6079	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.3;
	6079	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	6079	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340;
	6080	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.2;
	6080	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.6;
	6080	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.7;
	6080	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.4;
	6080	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	6080	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.3;
	6081	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.7;
	6081	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.9;
	6081	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.3;
	6081	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.5;
	6081	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.7;
	6081	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.7;
	6082	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.7;
	6082	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143;
	6082	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.7;
	6082	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.9;
	6082	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.3;
	6082	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.3;
	6083	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.5;
	6083	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.5;
	6083	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.8;
	6083	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.4;
	6083	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.8;
	6083	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.6;
	6084	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.1;
	6084	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.7;
	6084	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.2;
	6084	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.9;
	6084	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.4;
	6084	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.9;
	6085	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.8;
	6085	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.4;
	6085	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.2;
	6085	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.9;
	6085	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.8;
	6085	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.3;
	6086	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.5;
	6086	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.9;
	6086	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.8;
	6086	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	273;
	6086	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.2;
	6086	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.1;
	6087	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.8;
	6087	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.5;
	6087	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	271.1;
	6087	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	280.8;
	6087	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.4;
	6087	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.2;
	6088	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	555.3;
	6088	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.5;
	6088	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	272.8;
	6088	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	283.5;
	6088	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.8;
	6088	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.4;
	6089	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.9;
	6089	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.2;
	6089	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	274.7;
	6089	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	284.5;
	6089	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.6;
	6089	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.3;
	6090	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	557.2;
	6090	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.8;
	6090	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	271.6;
	6090	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	279.9;
	6090	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.7;
	6090	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.5;
	6091	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.7;
	6091	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.4;
	6091	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	271.5;
	6091	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	278.4;
	6091	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.1;
	6091	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414;
	6092	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.9;
	6092	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.5;
	6092	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	269.8;
	6092	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	277;
	6092	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.4;
	6092	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.2;
	6093	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.4;
	6093	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.4;
	6093	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	263.8;
	6093	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	269.8;
	6093	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.1;
	6093	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.4;
	6094	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.6;
	6094	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.7;
	6094	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.1;
	6094	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.5;
	6094	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.2;
	6094	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.4;
	6095	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.4;
	6095	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.2;
	6095	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.8;
	6095	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.2;
	6095	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78;
	6095	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.3;
	6096	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.5;
	6096	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131;
	6096	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.3;
	6096	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.9;
	6096	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.1;
	6096	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.1;
	6097	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.2;
	6097	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.8;
	6097	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.9;
	6097	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.3;
	6097	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.7;
	6097	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.1;
	6098	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	389.2;
	6098	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.3;
	6098	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.4;
	6098	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204;
	6098	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.4;
	6098	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.6;
	6099	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.4;
	6099	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.5;
	6099	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.1;
	6099	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202;
	6099	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	6099	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.5;
	6100	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379.5;
	6100	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.5;
	6100	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.4;
	6100	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.3;
	6100	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	6100	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.6;
	6101	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	388;
	6101	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.9;
	6101	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.8;
	6101	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.6;
	6101	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	6101	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.7;
	6102	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.3;
	6102	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.1;
	6102	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.6;
	6102	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.8;
	6102	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	6102	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.9;
	6103	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455;
	6103	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.4;
	6103	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.9;
	6103	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230;
	6103	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	6103	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.8;
	6104	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.2;
	6104	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.8;
	6104	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.9;
	6104	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.1;
	6104	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	6104	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.6;
	6105	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.2;
	6105	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.6;
	6105	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.9;
	6105	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.1;
	6105	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.8;
	6105	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.8;
	6106	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.6;
	6106	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.1;
	6106	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.2;
	6106	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	266.1;
	6106	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.5;
	6106	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.8;
	6107	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.9;
	6107	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.2;
	6107	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	267.7;
	6107	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	274.1;
	6107	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.8;
	6107	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.8;
	6108	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	585.2;
	6108	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166;
	6108	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	276.1;
	6108	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	283.3;
	6108	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.3;
	6108	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	429.3;
	6109	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	599.1;
	6109	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.9;
	6109	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	285.3;
	6109	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	292.4;
	6109	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80;
	6109	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	438.3;
	6110	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	625.3;
	6110	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.1;
	6110	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	292.7;
	6110	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	299.6;
	6110	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.2;
	6110	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	451.4;
	6111	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	634.2;
	6111	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.4;
	6111	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	300.2;
	6111	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	307.2;
	6111	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.9;
	6111	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	457.3;
	6112	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	647.9;
	6112	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.1;
	6112	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	305;
	6112	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	311.1;
	6112	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.6;
	6112	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	464.8;
	6113	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	645.4;
	6113	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.4;
	6113	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	304.9;
	6113	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	310.6;
	6113	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.8;
	6113	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	464.4;
	6114	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	643.6;
	6114	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.3;
	6114	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	301.6;
	6114	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	305.7;
	6114	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.1;
	6114	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	462.5;
	6115	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	634.4;
	6115	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183;
	6115	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	297.3;
	6115	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	300.7;
	6115	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.9;
	6115	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	457.1;
	6116	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	620.5;
	6116	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184;
	6116	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	293.7;
	6116	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	297.7;
	6116	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76;
	6116	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	450.2;
	6117	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	603.1;
	6117	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.7;
	6117	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	286.9;
	6117	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	289.1;
	6117	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.1;
	6117	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	441.1;
	6118	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579;
	6118	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.3;
	6118	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	269.8;
	6118	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	271.6;
	6118	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.4;
	6118	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	426.2;
	6119	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.1;
	6119	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.5;
	6119	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.4;
	6119	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.2;
	6119	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	6119	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.8;
	6120	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.4;
	6120	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143;
	6120	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.4;
	6120	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.7;
	6120	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	6120	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.8;
	6121	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.8;
	6121	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.8;
	6121	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.4;
	6121	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.4;
	6121	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61;
	6121	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.1;
	6122	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428;
	6122	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.1;
	6122	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213;
	6122	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.5;
	6122	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	6122	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.2;
	6123	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.3;
	6123	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.4;
	6123	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.4;
	6123	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.1;
	6123	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.8;
	6123	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.5;
	6124	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.4;
	6124	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.8;
	6124	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.5;
	6124	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.4;
	6124	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.1;
	6124	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.1;
	6125	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.6;
	6125	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.8;
	6125	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.8;
	6125	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.6;
	6125	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.2;
	6125	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.9;
	6126	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460;
	6126	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.5;
	6126	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.2;
	6126	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.3;
	6126	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.2;
	6126	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.7;
	6127	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.2;
	6127	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.3;
	6127	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.4;
	6127	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.7;
	6127	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.8;
	6127	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384;
	6128	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.7;
	6128	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153;
	6128	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.9;
	6128	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255;
	6128	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	6128	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.2;
	6129	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547.7;
	6129	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.4;
	6129	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	265.2;
	6129	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	269.3;
	6129	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	6129	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.1;
	6130	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	569.5;
	6130	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.9;
	6130	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	275.3;
	6130	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	280;
	6130	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.1;
	6130	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.9;
	6131	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.6;
	6131	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.5;
	6131	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	287.2;
	6131	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	291.7;
	6131	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.1;
	6131	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	438;
	6132	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	618.6;
	6132	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.7;
	6132	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	299.9;
	6132	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	304.1;
	6132	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.5;
	6132	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	449.9;
	6133	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	635.2;
	6133	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.3;
	6133	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	313.9;
	6133	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	318.3;
	6133	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.6;
	6133	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	462.1;
	6134	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	641.1;
	6134	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.4;
	6134	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	325.5;
	6134	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	328.2;
	6134	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.4;
	6134	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	468.6;
	6135	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	648.2;
	6135	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.6;
	6135	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	330.4;
	6135	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	333.3;
	6135	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.2;
	6135	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	473.9;
	6136	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	655.3;
	6136	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.7;
	6136	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	332.9;
	6136	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	335.9;
	6136	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.4;
	6136	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	478.6;
	6137	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	653.5;
	6137	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.3;
	6137	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	333.3;
	6137	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	335.9;
	6137	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.1;
	6137	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	478.4;
	6138	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	639.4;
	6138	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.7;
	6138	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	324.9;
	6138	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	326.6;
	6138	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.2;
	6138	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	470.5;
	6139	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	626.3;
	6139	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188;
	6139	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	317.7;
	6139	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	320.7;
	6139	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.3;
	6139	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	462;
	6140	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	608.1;
	6140	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.4;
	6140	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	311.9;
	6140	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	315.1;
	6140	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.5;
	6140	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	452.3;
	6141	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	608.3;
	6141	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179;
	6141	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	299.5;
	6141	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	302;
	6141	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.7;
	6141	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	448.5;
	6142	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.3;
	6142	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.7;
	6142	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	280.3;
	6142	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	282.4;
	6142	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.7;
	6142	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	427.9;
	6143	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.5;
	6143	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151;
	6143	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.3;
	6143	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.1;
	6143	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	6143	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.6;
	6144	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.1;
	6144	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.6;
	6144	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.1;
	6144	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.5;
	6144	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	6144	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.6;
	6145	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.8;
	6145	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.1;
	6145	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.5;
	6145	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.7;
	6145	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	6145	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.4;
	6146	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	444.9;
	6146	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.2;
	6146	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.6;
	6146	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.5;
	6146	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	6146	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.7;
	6147	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.2;
	6147	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.1;
	6147	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.6;
	6147	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.4;
	6147	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.7;
	6147	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.1;
	6148	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.9;
	6148	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.3;
	6148	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.7;
	6148	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.5;
	6148	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.2;
	6148	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.5;
	6149	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.4;
	6149	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.7;
	6149	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.2;
	6149	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217;
	6149	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.2;
	6149	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.8;
	6150	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.4;
	6150	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.4;
	6150	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.3;
	6150	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229;
	6150	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.4;
	6150	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.5;
	6151	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.2;
	6151	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.8;
	6151	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.2;
	6151	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.1;
	6151	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57;
	6151	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.4;
	6152	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	540.3;
	6152	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.1;
	6152	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.1;
	6152	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261;
	6152	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	6152	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.4;
	6153	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548;
	6153	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.4;
	6153	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	269.1;
	6153	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	273.2;
	6153	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	6153	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.5;
	6154	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.8;
	6154	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.6;
	6154	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	280;
	6154	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	283;
	6154	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	6154	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.8;
	6155	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564.9;
	6155	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.5;
	6155	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	285.8;
	6155	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	288.6;
	6155	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.1;
	6155	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.8;
	6156	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	557.6;
	6156	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.4;
	6156	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	295;
	6156	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	295.9;
	6156	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.7;
	6156	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.4;
	6157	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	606.5;
	6157	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.1;
	6157	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	300.2;
	6157	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	298.7;
	6157	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.6;
	6157	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	448.1;
	6158	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.7;
	6158	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.9;
	6158	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	294.8;
	6158	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	293.5;
	6158	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.5;
	6158	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	442;
	6159	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	581.6;
	6159	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.6;
	6159	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	301.2;
	6159	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	299;
	6159	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.3;
	6159	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	438.2;
	6160	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548.3;
	6160	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.9;
	6160	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	309;
	6160	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	305.6;
	6160	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.9;
	6160	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	426.9;
	6161	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	555.2;
	6161	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.4;
	6161	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	313.1;
	6161	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	307.2;
	6161	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.3;
	6161	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	431.7;
	6162	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.3;
	6162	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.1;
	6162	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	307.5;
	6162	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	300.6;
	6162	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.4;
	6162	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	426.2;
	6163	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	534.7;
	6163	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.7;
	6163	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	303.3;
	6163	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	298.7;
	6163	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.7;
	6163	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420;
	6164	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522.1;
	6164	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.2;
	6164	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	296.1;
	6164	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	291.3;
	6164	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.9;
	6164	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.6;
	6165	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.5;
	6165	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.9;
	6165	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	285.8;
	6165	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	280.4;
	6165	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.8;
	6165	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.6;
	6166	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.7;
	6166	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.6;
	6166	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	268.4;
	6166	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	263.5;
	6166	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.2;
	6166	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.9;
	6167	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.6;
	6167	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.7;
	6167	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.9;
	6167	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.2;
	6167	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.9;
	6167	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.1;
	6168	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.1;
	6168	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.8;
	6168	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.2;
	6168	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.8;
	6168	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68;
	6168	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.1;
	6169	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	420.5;
	6169	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.2;
	6169	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.6;
	6169	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.3;
	6169	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	6169	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.5;
	6170	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403.1;
	6170	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.9;
	6170	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.7;
	6170	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.5;
	6170	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	6170	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.6;
	6171	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	390.6;
	6171	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.4;
	6171	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.3;
	6171	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.6;
	6171	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.9;
	6171	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.3;
	6172	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	365.2;
	6172	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.2;
	6172	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206;
	6172	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.6;
	6172	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57;
	6172	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.5;
	6173	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398;
	6173	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.4;
	6173	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.2;
	6173	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.3;
	6173	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.1;
	6173	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.8;
	6174	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.7;
	6174	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.7;
	6174	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.9;
	6174	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.6;
	6174	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57;
	6174	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.2;
	6175	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.8;
	6175	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.7;
	6175	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.1;
	6175	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.9;
	6175	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.6;
	6175	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368;
	6176	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484;
	6176	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.7;
	6176	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.9;
	6176	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.5;
	6176	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	6176	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.6;
	6177	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488;
	6177	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.6;
	6177	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.9;
	6177	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.3;
	6177	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	6177	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.4;
	6178	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.2;
	6178	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.3;
	6178	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	266.7;
	6178	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265;
	6178	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.7;
	6178	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.7;
	6179	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.3;
	6179	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.8;
	6179	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	275.9;
	6179	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.9;
	6179	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.3;
	6179	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.1;
	6180	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.9;
	6180	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.7;
	6180	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	279.9;
	6180	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	276.5;
	6180	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.6;
	6180	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.3;
	6181	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.6;
	6181	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.8;
	6181	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	283;
	6181	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	278.8;
	6181	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.8;
	6181	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.4;
	6182	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.1;
	6182	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.1;
	6182	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	296.2;
	6182	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	290.7;
	6182	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.8;
	6182	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.1;
	6183	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.6;
	6183	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.8;
	6183	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	300;
	6183	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	293.6;
	6183	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.7;
	6183	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.8;
	6184	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.6;
	6184	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162;
	6184	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	301.5;
	6184	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	294.1;
	6184	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.7;
	6184	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.3;
	6185	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.4;
	6185	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.3;
	6185	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	300.4;
	6185	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	292.5;
	6185	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	94.2;
	6185	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.9;
	6186	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.8;
	6186	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.9;
	6186	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	288.4;
	6186	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	281.6;
	6186	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.9;
	6186	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.3;
	6187	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.1;
	6187	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.8;
	6187	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	279.9;
	6187	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	275.4;
	6187	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.2;
	6187	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.2;
	6188	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459;
	6188	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.1;
	6188	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	275.7;
	6188	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.1;
	6188	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.4;
	6188	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.2;
	6189	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.9;
	6189	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.7;
	6189	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	265.6;
	6189	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.1;
	6189	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.7;
	6189	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.9;
	6190	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428.9;
	6190	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.1;
	6190	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.9;
	6190	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248;
	6190	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.8;
	6190	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.3;
	6191	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.4;
	6191	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.5;
	6191	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.9;
	6191	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.4;
	6191	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.8;
	6191	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.3;
	6192	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.4;
	6192	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.5;
	6192	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.9;
	6192	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.9;
	6192	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	6192	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.4;
	6193	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	396.1;
	6193	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118;
	6193	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210;
	6193	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.5;
	6193	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	6193	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.7;
	6194	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	378.5;
	6194	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.9;
	6194	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.1;
	6194	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.8;
	6194	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	6194	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.9;
	6195	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	374.4;
	6195	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.3;
	6195	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.2;
	6195	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200;
	6195	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.3;
	6195	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319;
	6196	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379.8;
	6196	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.5;
	6196	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.6;
	6196	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.9;
	6196	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	6196	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.2;
	6197	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	376.5;
	6197	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.8;
	6197	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.2;
	6197	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.7;
	6197	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	6197	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.5;
	6198	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.9;
	6198	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.5;
	6198	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205;
	6198	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.2;
	6198	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.6;
	6198	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.8;
	6199	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417;
	6199	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.9;
	6199	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.5;
	6199	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.4;
	6199	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.3;
	6199	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341;
	6200	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407;
	6200	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.2;
	6200	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.9;
	6200	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.7;
	6200	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	6200	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.5;
	6201	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	427.8;
	6201	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.6;
	6201	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.6;
	6201	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.4;
	6201	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	6201	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350;
	6202	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.4;
	6202	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.6;
	6202	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.4;
	6202	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.5;
	6202	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.8;
	6202	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.8;
	6203	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.5;
	6203	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.3;
	6203	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.5;
	6203	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.8;
	6203	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.4;
	6203	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.4;
	6204	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.8;
	6204	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.7;
	6204	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.3;
	6204	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.2;
	6204	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.9;
	6204	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.9;
	6205	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.8;
	6205	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.4;
	6205	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.5;
	6205	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.2;
	6205	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.1;
	6205	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.3;
	6206	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.4;
	6206	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.3;
	6206	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.3;
	6206	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249;
	6206	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.6;
	6206	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.9;
	6207	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462;
	6207	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.8;
	6207	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.8;
	6207	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.6;
	6207	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.1;
	6207	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.8;
	6208	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.2;
	6208	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.4;
	6208	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.6;
	6208	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.2;
	6208	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.6;
	6208	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.4;
	6209	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.5;
	6209	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.4;
	6209	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.3;
	6209	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.8;
	6209	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75;
	6209	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.6;
	6210	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.2;
	6210	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.3;
	6210	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.6;
	6210	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.9;
	6210	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.8;
	6210	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.9;
	6211	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.9;
	6211	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.1;
	6211	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250;
	6211	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.2;
	6211	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	6211	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.2;
	6212	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468;
	6212	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.6;
	6212	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.6;
	6212	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.9;
	6212	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.5;
	6212	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.2;
	6213	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.9;
	6213	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.2;
	6213	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.1;
	6213	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.3;
	6213	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.4;
	6213	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.4;
	6214	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.2;
	6214	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.8;
	6214	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.2;
	6214	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.6;
	6214	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	6214	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.9;
	6215	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424;
	6215	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.8;
	6215	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.5;
	6215	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.7;
	6215	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	6215	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.2;
	6216	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.7;
	6216	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.3;
	6216	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.7;
	6216	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.8;
	6216	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.6;
	6216	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.3;
	6217	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	373.4;
	6217	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.8;
	6217	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.5;
	6217	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.8;
	6217	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.9;
	6217	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.1;
	6218	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	346.3;
	6218	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.2;
	6218	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.4;
	6218	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.5;
	6218	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.9;
	6218	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.9;
	6219	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	332.8;
	6219	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.2;
	6219	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.3;
	6219	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.9;
	6219	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.1;
	6219	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	302.2;
	6220	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	349.7;
	6220	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112;
	6220	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.2;
	6220	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.2;
	6220	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.3;
	6220	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.4;
	6221	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	350.5;
	6221	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.4;
	6221	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.1;
	6221	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.1;
	6221	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.6;
	6221	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309.8;
	6222	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	366.2;
	6222	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.5;
	6222	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.8;
	6222	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.1;
	6222	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.9;
	6222	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.6;
	6223	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	375.4;
	6223	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.9;
	6223	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.9;
	6223	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.1;
	6223	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.4;
	6223	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.2;
	6224	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	382.3;
	6224	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.1;
	6224	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.9;
	6224	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.1;
	6224	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	6224	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.6;
	6225	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391.1;
	6225	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.3;
	6225	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.6;
	6225	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.1;
	6225	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	6225	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.3;
	6226	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410;
	6226	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.3;
	6226	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221;
	6226	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221;
	6226	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	6226	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.8;
	6227	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.1;
	6227	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.6;
	6227	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.9;
	6227	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.6;
	6227	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	6227	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.1;
	6228	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.7;
	6228	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.7;
	6228	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.9;
	6228	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.8;
	6228	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.4;
	6228	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.5;
	6229	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417.5;
	6229	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.2;
	6229	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.2;
	6229	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.1;
	6229	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	6229	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.1;
	6230	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.7;
	6230	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.2;
	6230	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.3;
	6230	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.4;
	6230	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.2;
	6230	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.3;
	6231	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.4;
	6231	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.8;
	6231	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.8;
	6231	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.7;
	6231	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.1;
	6231	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.2;
	6232	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.7;
	6232	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.6;
	6232	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.1;
	6232	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.5;
	6232	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.4;
	6232	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.5;
	6233	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442.1;
	6233	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.4;
	6233	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.9;
	6233	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.8;
	6233	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74;
	6233	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.5;
	6234	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	430.7;
	6234	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.9;
	6234	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240;
	6234	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.7;
	6234	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.9;
	6234	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356;
	6235	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	430.9;
	6235	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.6;
	6235	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.8;
	6235	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.7;
	6235	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.1;
	6235	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356;
	6236	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.5;
	6236	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.5;
	6236	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.7;
	6236	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.1;
	6236	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	6236	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.1;
	6237	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.3;
	6237	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.8;
	6237	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.2;
	6237	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.8;
	6237	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.4;
	6237	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.6;
	6238	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.3;
	6238	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.9;
	6238	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.3;
	6238	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.6;
	6238	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	6238	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.2;
	6239	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	406.4;
	6239	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.5;
	6239	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.8;
	6239	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212;
	6239	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	6239	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.7;
	6240	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	373.5;
	6240	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.1;
	6240	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206;
	6240	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.2;
	6240	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.7;
	6240	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.3;
	6241	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	362.9;
	6241	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.6;
	6241	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.4;
	6241	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.6;
	6241	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.9;
	6241	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.5;
	6242	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	356.2;
	6242	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	105.1;
	6242	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.5;
	6242	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.6;
	6242	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.9;
	6242	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	306.4;
	6243	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	357.3;
	6243	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	105;
	6243	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185;
	6243	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.7;
	6243	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55;
	6243	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	306.2;
	6244	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	355.1;
	6244	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	105.2;
	6244	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.8;
	6244	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.1;
	6244	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.7;
	6244	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	305;
	6245	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	358.4;
	6245	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.3;
	6245	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187;
	6245	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.4;
	6245	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53;
	6245	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	307.2;
	6246	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.2;
	6246	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118;
	6246	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.5;
	6246	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.5;
	6246	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.4;
	6246	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.5;
	6247	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.3;
	6247	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.7;
	6247	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.3;
	6247	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.3;
	6247	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.2;
	6247	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.4;
	6248	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.2;
	6248	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.2;
	6248	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.8;
	6248	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.1;
	6248	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.5;
	6248	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.2;
	6249	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.9;
	6249	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.2;
	6249	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.9;
	6249	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236;
	6249	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.1;
	6249	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367;
	6250	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.3;
	6250	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144;
	6250	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.6;
	6250	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.1;
	6250	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.2;
	6250	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.1;
	6251	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.5;
	6251	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.2;
	6251	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.3;
	6251	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.8;
	6251	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	6251	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.3;
	6252	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.3;
	6252	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.7;
	6252	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.5;
	6252	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.9;
	6252	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	6252	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.6;
	6253	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.6;
	6253	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.9;
	6253	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.8;
	6253	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248;
	6253	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	6253	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.5;
	6254	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.4;
	6254	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.4;
	6254	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.9;
	6254	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.7;
	6254	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	6254	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386;
	6255	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.7;
	6255	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.4;
	6255	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.5;
	6255	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.8;
	6255	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.1;
	6255	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.8;
	6256	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.7;
	6256	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.6;
	6256	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.8;
	6256	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.2;
	6256	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72;
	6256	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387;
	6257	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.6;
	6257	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152;
	6257	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.7;
	6257	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.5;
	6257	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.9;
	6257	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.1;
	6258	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.8;
	6258	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.2;
	6258	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.3;
	6258	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.1;
	6258	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.4;
	6258	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.9;
	6259	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.7;
	6259	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.2;
	6259	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.4;
	6259	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.7;
	6259	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.8;
	6259	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.4;
	6260	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.5;
	6260	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.3;
	6260	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.6;
	6260	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.2;
	6260	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.5;
	6260	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.4;
	6261	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.3;
	6261	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.7;
	6261	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243;
	6261	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.5;
	6261	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.5;
	6261	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.5;
	6262	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.6;
	6262	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.5;
	6262	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231;
	6262	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232;
	6262	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.5;
	6262	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.9;
	6263	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	427.8;
	6263	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128;
	6263	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.1;
	6263	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.5;
	6263	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	6263	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.8;
	6264	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.5;
	6264	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.8;
	6264	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.8;
	6264	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.9;
	6264	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	6264	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334;
	6265	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	402.4;
	6265	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.1;
	6265	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.4;
	6265	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.5;
	6265	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	6265	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.7;
	6266	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	385.7;
	6266	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.4;
	6266	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.9;
	6266	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.8;
	6266	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	6266	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.8;
	6267	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	381.1;
	6267	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.3;
	6267	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.1;
	6267	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.5;
	6267	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.4;
	6267	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.4;
	6268	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379.4;
	6268	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.2;
	6268	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.4;
	6268	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.5;
	6268	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.8;
	6268	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.2;
	6269	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	387;
	6269	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.1;
	6269	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.3;
	6269	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.2;
	6269	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.6;
	6269	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.9;
	6270	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	406.7;
	6270	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124;
	6270	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.1;
	6270	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.6;
	6270	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.3;
	6270	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.7;
	6271	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433;
	6271	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.3;
	6271	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.5;
	6271	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.5;
	6271	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.9;
	6271	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.9;
	6272	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.6;
	6272	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.3;
	6272	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.4;
	6272	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.9;
	6272	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57;
	6272	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.7;
	6273	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.9;
	6273	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.6;
	6273	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.2;
	6273	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.7;
	6273	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.9;
	6273	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.4;
	6274	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.6;
	6274	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.1;
	6274	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.4;
	6274	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.6;
	6274	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	6274	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.7;
	6275	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.8;
	6275	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.6;
	6275	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.4;
	6275	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.4;
	6275	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	6275	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.8;
	6276	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.4;
	6276	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150;
	6276	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.3;
	6276	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.9;
	6276	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	6276	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383;
	6277	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.6;
	6277	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.6;
	6277	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253;
	6277	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.9;
	6277	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.2;
	6277	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.7;
	6278	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.2;
	6278	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.7;
	6278	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.5;
	6278	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.8;
	6278	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.2;
	6278	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.8;
	6279	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.7;
	6279	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152;
	6279	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	259.3;
	6279	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.9;
	6279	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83;
	6279	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.8;
	6280	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.9;
	6280	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.9;
	6280	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.7;
	6280	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.9;
	6280	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.8;
	6280	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402;
	6281	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.4;
	6281	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.2;
	6281	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.6;
	6281	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.4;
	6281	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.6;
	6281	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.4;
	6282	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.2;
	6282	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.5;
	6282	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.5;
	6282	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.8;
	6282	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.3;
	6282	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.1;
	6283	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.4;
	6283	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.2;
	6283	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	263;
	6283	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	263.3;
	6283	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.6;
	6283	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.7;
	6284	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.4;
	6284	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.3;
	6284	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	259.1;
	6284	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.3;
	6284	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80;
	6284	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.3;
	6285	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.9;
	6285	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156;
	6285	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.3;
	6285	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.3;
	6285	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.9;
	6285	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.1;
	6286	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.9;
	6286	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.5;
	6286	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.1;
	6286	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237;
	6286	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.4;
	6286	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.9;
	6287	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447.3;
	6287	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.2;
	6287	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.5;
	6287	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.4;
	6287	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.4;
	6287	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.3;
	6288	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.9;
	6288	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.5;
	6288	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.2;
	6288	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.7;
	6288	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.5;
	6288	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.6;
	6289	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	402.7;
	6289	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.2;
	6289	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.9;
	6289	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.6;
	6289	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	6289	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.2;
	6290	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	385.2;
	6290	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.8;
	6290	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.5;
	6290	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.4;
	6290	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	6290	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.2;
	6291	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	375.6;
	6291	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.5;
	6291	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.9;
	6291	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.1;
	6291	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	6291	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.5;
	6292	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	373.8;
	6292	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110;
	6292	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.6;
	6292	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.6;
	6292	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	6292	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.4;
	6293	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	387.8;
	6293	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.4;
	6293	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.5;
	6293	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.2;
	6293	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	6293	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321;
	6294	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.2;
	6294	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.8;
	6294	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.1;
	6294	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.6;
	6294	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	6294	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.4;
	6295	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453;
	6295	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.8;
	6295	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.6;
	6295	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.9;
	6295	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	6295	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.9;
	6296	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.2;
	6296	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.2;
	6296	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.8;
	6296	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.8;
	6296	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	6296	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367;
	6297	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.8;
	6297	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.8;
	6297	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.5;
	6297	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.3;
	6297	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.3;
	6297	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.1;
	6298	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.2;
	6298	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.5;
	6298	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.3;
	6298	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.2;
	6298	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.3;
	6298	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374;
	6299	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.9;
	6299	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.1;
	6299	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.1;
	6299	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.9;
	6299	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.7;
	6299	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.5;
	6300	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.2;
	6300	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152;
	6300	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.9;
	6300	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.5;
	6300	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.1;
	6300	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.3;
	6301	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.7;
	6301	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152;
	6301	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.8;
	6301	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.7;
	6301	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77;
	6301	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.9;
	6302	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.7;
	6302	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.8;
	6302	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	265;
	6302	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	266.8;
	6302	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.5;
	6302	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.3;
	6303	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.1;
	6303	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.2;
	6303	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	271.2;
	6303	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272.3;
	6303	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.6;
	6303	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.5;
	6304	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.8;
	6304	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.7;
	6304	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	275.9;
	6304	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	275.6;
	6304	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.5;
	6304	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.7;
	6305	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490;
	6305	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.9;
	6305	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	279.4;
	6305	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	278.4;
	6305	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.2;
	6305	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.1;
	6306	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.8;
	6306	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156;
	6306	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	275.3;
	6306	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	274.4;
	6306	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.5;
	6306	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.3;
	6307	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.8;
	6307	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.7;
	6307	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	265.2;
	6307	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267.3;
	6307	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71;
	6307	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.5;
	6308	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.4;
	6308	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.8;
	6308	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.6;
	6308	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265;
	6308	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	6308	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.9;
	6309	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.9;
	6309	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.9;
	6309	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.5;
	6309	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.3;
	6309	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.7;
	6309	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.3;
	6310	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.3;
	6310	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.4;
	6310	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.8;
	6310	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.7;
	6310	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69;
	6310	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.4;
	6311	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.5;
	6311	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.2;
	6311	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.1;
	6311	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.9;
	6311	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	6311	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.9;
	6312	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.9;
	6312	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.5;
	6312	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.2;
	6312	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.4;
	6312	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	6312	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.9;
	6313	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391.3;
	6313	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.6;
	6313	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.8;
	6313	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.1;
	6313	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	6313	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.2;
	6314	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	370.2;
	6314	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.4;
	6314	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191;
	6314	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.6;
	6314	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.5;
	6314	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.3;
	6315	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	376.2;
	6315	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.6;
	6315	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.2;
	6315	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.4;
	6315	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.8;
	6315	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315;
	6316	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	381.9;
	6316	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.6;
	6316	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.3;
	6316	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.2;
	6316	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.3;
	6316	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.2;
	6317	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	396.5;
	6317	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.7;
	6317	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191;
	6317	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.3;
	6317	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.6;
	6317	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.3;
	6318	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.2;
	6318	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.3;
	6318	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.8;
	6318	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207;
	6318	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.7;
	6318	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.7;
	6319	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.9;
	6319	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137;
	6319	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.8;
	6319	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.5;
	6319	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.8;
	6319	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.3;
	6320	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457;
	6320	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.6;
	6320	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.3;
	6320	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.4;
	6320	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	6320	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.1;
	6321	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.9;
	6321	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.7;
	6321	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.9;
	6321	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.2;
	6321	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	6321	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.6;
	6322	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.4;
	6322	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.3;
	6322	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.8;
	6322	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.4;
	6322	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	6322	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.7;
	6323	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.6;
	6323	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.8;
	6323	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.3;
	6323	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.2;
	6323	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.1;
	6323	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.6;
	6324	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.6;
	6324	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.8;
	6324	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249;
	6324	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.8;
	6324	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.9;
	6324	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.3;
	6325	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.8;
	6325	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.9;
	6325	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.3;
	6325	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.8;
	6325	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.2;
	6325	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.6;
	6326	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.2;
	6326	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.3;
	6326	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.9;
	6326	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265.9;
	6326	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.1;
	6326	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.7;
	6327	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.4;
	6327	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.1;
	6327	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.5;
	6327	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	269.2;
	6327	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.3;
	6327	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390;
	6328	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.5;
	6328	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.2;
	6328	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	264.2;
	6328	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	271.1;
	6328	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.2;
	6328	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389;
	6329	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.3;
	6329	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.8;
	6329	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	265.6;
	6329	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	272;
	6329	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.3;
	6329	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.6;
	6330	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.5;
	6330	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153;
	6330	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	259.8;
	6330	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265.8;
	6330	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.5;
	6330	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.8;
	6331	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.3;
	6331	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.3;
	6331	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.4;
	6331	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	266.8;
	6331	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.8;
	6331	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.3;
	6332	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.8;
	6332	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.2;
	6332	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256;
	6332	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.5;
	6332	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.9;
	6332	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385;
	6333	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.2;
	6333	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.5;
	6333	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.9;
	6333	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.6;
	6333	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.2;
	6333	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379;
	6334	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.4;
	6334	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.7;
	6334	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233;
	6334	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.7;
	6334	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.6;
	6334	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.6;
	6335	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	430.5;
	6335	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.6;
	6335	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.3;
	6335	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.4;
	6335	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.3;
	6335	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.9;
	6336	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	402.9;
	6336	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.2;
	6336	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.6;
	6336	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.3;
	6336	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	6336	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332;
	6337	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	384.2;
	6337	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.7;
	6337	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.8;
	6337	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.5;
	6337	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	6337	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.5;
	6338	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	361.3;
	6338	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.3;
	6338	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.9;
	6338	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193;
	6338	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.9;
	6338	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309.3;
	6339	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	359.6;
	6339	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.2;
	6339	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.1;
	6339	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.7;
	6339	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58;
	6339	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	307.7;
	6340	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	361.5;
	6340	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.7;
	6340	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.3;
	6340	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.3;
	6340	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.1;
	6340	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.6;
	6341	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	376.3;
	6341	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112;
	6341	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.1;
	6341	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.2;
	6341	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.5;
	6341	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316;
	6342	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.8;
	6342	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.4;
	6342	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.4;
	6342	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.8;
	6342	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.4;
	6342	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.9;
	6343	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	444.6;
	6343	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.1;
	6343	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.1;
	6343	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.7;
	6343	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.1;
	6343	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351;
	6344	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.2;
	6344	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.6;
	6344	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.8;
	6344	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.9;
	6344	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	6344	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.5;
	6345	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.8;
	6345	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141;
	6345	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.8;
	6345	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.4;
	6345	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	6345	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.5;
	6346	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.9;
	6346	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.3;
	6346	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240;
	6346	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.9;
	6346	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	6346	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.8;
	6347	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.9;
	6347	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.5;
	6347	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.2;
	6347	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.8;
	6347	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.1;
	6347	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.2;
	6348	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.8;
	6348	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.7;
	6348	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.8;
	6348	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.7;
	6348	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.6;
	6348	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.3;
	6349	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.9;
	6349	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.5;
	6349	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.3;
	6349	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.7;
	6349	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.8;
	6349	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384;
	6350	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.4;
	6350	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.2;
	6350	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.9;
	6350	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.9;
	6350	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.2;
	6350	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.7;
	6351	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.3;
	6351	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.1;
	6351	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.4;
	6351	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	263.5;
	6351	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79;
	6351	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.4;
	6352	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.8;
	6352	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.4;
	6352	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.9;
	6352	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	266.1;
	6352	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.8;
	6352	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.1;
	6353	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.3;
	6353	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.6;
	6353	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.8;
	6353	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265;
	6353	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.3;
	6353	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.7;
	6354	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.8;
	6354	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.4;
	6354	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.5;
	6354	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.8;
	6354	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.4;
	6354	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.1;
	6355	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.5;
	6355	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.2;
	6355	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250;
	6355	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.7;
	6355	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77;
	6355	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.5;
	6356	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.1;
	6356	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.1;
	6356	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.6;
	6356	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.8;
	6356	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.1;
	6356	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.7;
	6357	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.2;
	6357	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.4;
	6357	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239;
	6357	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.9;
	6357	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.9;
	6357	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.9;
	6358	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.8;
	6358	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.6;
	6358	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.9;
	6358	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.6;
	6358	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.7;
	6358	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.9;
	6359	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.5;
	6359	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.2;
	6359	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.5;
	6359	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.1;
	6359	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.9;
	6359	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.5;
	6360	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.2;
	6360	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.5;
	6360	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.5;
	6360	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.4;
	6360	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.2;
	6360	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.6;
	6361	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.8;
	6361	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.3;
	6361	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.1;
	6361	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.6;
	6361	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	6361	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.1;
	6362	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.5;
	6362	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107;
	6362	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190;
	6362	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.7;
	6362	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	6362	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.2;
	6363	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	378.5;
	6363	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	105.6;
	6363	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.5;
	6363	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.5;
	6363	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.1;
	6363	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.1;
	6364	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	382.7;
	6364	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	104.9;
	6364	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.6;
	6364	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.5;
	6364	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.5;
	6364	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.9;
	6365	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392.2;
	6365	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.3;
	6365	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.4;
	6365	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.4;
	6365	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.9;
	6365	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.9;
	6366	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.2;
	6366	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115;
	6366	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.6;
	6366	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.2;
	6366	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.4;
	6366	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.2;
	6367	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	402.2;
	6367	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.1;
	6367	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.5;
	6367	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.6;
	6367	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.2;
	6367	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.4;
	6368	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.1;
	6368	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.2;
	6368	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.2;
	6368	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205;
	6368	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	6368	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.9;
	6369	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.3;
	6369	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.4;
	6369	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.5;
	6369	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.4;
	6369	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	6369	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341;
	6370	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.8;
	6370	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.3;
	6370	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.7;
	6370	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.9;
	6370	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	6370	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.6;
	6371	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	444;
	6371	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.4;
	6371	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.4;
	6371	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.5;
	6371	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	6371	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.8;
	6372	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.4;
	6372	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.7;
	6372	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.8;
	6372	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.1;
	6372	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.1;
	6372	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356;
	6373	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.4;
	6373	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.3;
	6373	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.2;
	6373	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.4;
	6373	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.1;
	6373	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360;
	6374	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.2;
	6374	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.1;
	6374	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.6;
	6374	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.2;
	6374	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.5;
	6374	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.8;
	6375	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447.3;
	6375	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141;
	6375	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.6;
	6375	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.4;
	6375	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.4;
	6375	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.5;
	6376	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448;
	6376	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.3;
	6376	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.2;
	6376	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.4;
	6376	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.8;
	6376	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.8;
	6377	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.3;
	6377	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.1;
	6377	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.2;
	6377	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.7;
	6377	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.2;
	6377	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.4;
	6378	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.1;
	6378	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139;
	6378	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237;
	6378	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.6;
	6378	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.7;
	6378	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.1;
	6379	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.9;
	6379	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.4;
	6379	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.6;
	6379	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.2;
	6379	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.8;
	6379	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356;
	6380	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.6;
	6380	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.1;
	6380	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.1;
	6380	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.2;
	6380	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.8;
	6380	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.2;
	6381	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.3;
	6381	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.6;
	6381	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.2;
	6381	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.7;
	6381	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.1;
	6381	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.7;
	6382	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.5;
	6382	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.9;
	6382	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.8;
	6382	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.6;
	6382	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.2;
	6382	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.2;
	6383	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392.8;
	6383	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.2;
	6383	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.3;
	6383	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.8;
	6383	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	6383	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.2;
	6384	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	374.2;
	6384	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122;
	6384	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.7;
	6384	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221;
	6384	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	6384	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.4;
	6385	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	356.2;
	6385	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.8;
	6385	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.4;
	6385	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.5;
	6385	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.2;
	6385	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.6;
	6386	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	333.9;
	6386	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.7;
	6386	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.9;
	6386	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.4;
	6386	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	6386	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	301.2;
	6387	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	330.9;
	6387	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.3;
	6387	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.1;
	6387	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.5;
	6387	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.4;
	6387	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	298.6;
	6388	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	327.9;
	6388	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	105.1;
	6388	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.8;
	6388	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.2;
	6388	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.1;
	6388	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	296.8;
	6389	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	333.5;
	6389	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.5;
	6389	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.8;
	6389	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194;
	6389	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.8;
	6389	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	299.5;
	6390	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	346.8;
	6390	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.1;
	6390	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.1;
	6390	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197;
	6390	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.6;
	6390	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	306.1;
	6391	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	350.8;
	6391	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.7;
	6391	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.2;
	6391	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.2;
	6391	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.1;
	6391	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.7;
	6392	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	354.5;
	6392	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117;
	6392	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198;
	6392	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.2;
	6392	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	6392	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311;
	6393	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	375.1;
	6393	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.9;
	6393	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.1;
	6393	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.6;
	6393	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	6393	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.2;
	6394	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	390.5;
	6394	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131;
	6394	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212;
	6394	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.5;
	6394	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	6394	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.5;
	6395	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.4;
	6395	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.4;
	6395	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.3;
	6395	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.9;
	6395	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.7;
	6395	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.6;
	6396	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415;
	6396	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.8;
	6396	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.4;
	6396	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225;
	6396	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.8;
	6396	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.4;
	6397	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.9;
	6397	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.4;
	6397	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.8;
	6397	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.7;
	6397	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.3;
	6397	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345;
	6398	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.1;
	6398	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.1;
	6398	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.4;
	6398	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.5;
	6398	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.9;
	6398	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.8;
	6399	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417.8;
	6399	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.2;
	6399	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.2;
	6399	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.6;
	6399	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.5;
	6399	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.2;
	6400	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423;
	6400	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.5;
	6400	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.5;
	6400	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.6;
	6400	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.6;
	6400	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.6;
	6401	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.6;
	6401	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.2;
	6401	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.9;
	6401	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.7;
	6401	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.8;
	6401	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350;
	6402	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428.3;
	6402	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.4;
	6402	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.2;
	6402	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.2;
	6402	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.2;
	6402	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352;
	6403	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	426.1;
	6403	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.9;
	6403	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.6;
	6403	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.6;
	6403	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.4;
	6403	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.5;
	6404	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.8;
	6404	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.1;
	6404	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.2;
	6404	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.2;
	6404	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.8;
	6404	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.7;
	6405	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.8;
	6405	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.7;
	6405	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.9;
	6405	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.9;
	6405	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.6;
	6405	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359;
	6406	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.6;
	6406	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.5;
	6406	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.9;
	6406	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.1;
	6406	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.2;
	6406	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349;
	6407	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.6;
	6407	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.6;
	6407	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.6;
	6407	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.5;
	6407	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.4;
	6407	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335;
	6408	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	378.9;
	6408	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.6;
	6408	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208;
	6408	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.7;
	6408	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	6408	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.2;
	6409	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	365.2;
	6409	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.1;
	6409	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.6;
	6409	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.8;
	6409	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	6409	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.9;
	6410	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	351.6;
	6410	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.4;
	6410	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.7;
	6410	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.1;
	6410	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	6410	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	305.6;
	6411	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	357.1;
	6411	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.3;
	6411	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.2;
	6411	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.6;
	6411	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	6411	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	307.2;
	6412	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	355.4;
	6412	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.2;
	6412	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.4;
	6412	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.1;
	6412	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.9;
	6412	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	306.5;
	6413	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	367.5;
	6413	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.8;
	6413	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192;
	6413	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.7;
	6413	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56;
	6413	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.5;
	6414	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.1;
	6414	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.7;
	6414	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.1;
	6414	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.6;
	6414	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.3;
	6414	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.3;
	6415	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.4;
	6415	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.2;
	6415	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215;
	6415	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221;
	6415	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.9;
	6415	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.4;
	6416	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.2;
	6416	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.5;
	6416	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.7;
	6416	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.3;
	6416	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	6416	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.7;
	6417	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.4;
	6417	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.7;
	6417	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.2;
	6417	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245;
	6417	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.1;
	6417	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.4;
	6418	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.4;
	6418	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.9;
	6418	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.6;
	6418	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.4;
	6418	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	6418	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.2;
	6419	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.4;
	6419	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.7;
	6419	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.2;
	6419	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.1;
	6419	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	6419	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.8;
	6420	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.2;
	6420	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.4;
	6420	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245;
	6420	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.2;
	6420	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.1;
	6420	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.4;
	6421	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489;
	6421	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150;
	6421	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.7;
	6421	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.2;
	6421	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68;
	6421	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.8;
	6422	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489;
	6422	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.8;
	6422	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.3;
	6422	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.2;
	6422	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.8;
	6422	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.3;
	6423	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.4;
	6423	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.3;
	6423	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.4;
	6423	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.2;
	6423	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	6423	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.4;
	6424	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.9;
	6424	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147;
	6424	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.9;
	6424	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.6;
	6424	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	6424	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.8;
	6425	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.1;
	6425	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156;
	6425	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.9;
	6425	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.8;
	6425	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	6425	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.3;
	6426	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.5;
	6426	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.2;
	6426	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.7;
	6426	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.1;
	6426	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	6426	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.4;
	6427	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.8;
	6427	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.8;
	6427	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.9;
	6427	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.6;
	6427	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	6427	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.1;
	6428	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.5;
	6428	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.7;
	6428	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.1;
	6428	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.7;
	6428	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	6428	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.5;
	6429	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.9;
	6429	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.3;
	6429	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240;
	6429	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.3;
	6429	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	6429	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.9;
	6430	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470;
	6430	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.4;
	6430	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.5;
	6430	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.9;
	6430	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	6430	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.8;
	6431	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.4;
	6431	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.9;
	6431	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.3;
	6431	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.5;
	6431	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	6431	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.6;
	6432	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.2;
	6432	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.9;
	6432	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.8;
	6432	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.4;
	6432	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.6;
	6432	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.4;
	6433	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394.7;
	6433	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.4;
	6433	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.7;
	6433	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.9;
	6433	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56;
	6433	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.3;
	6434	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	384.9;
	6434	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.5;
	6434	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.9;
	6434	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.2;
	6434	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54;
	6434	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.6;
	6435	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	382.3;
	6435	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.9;
	6435	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.6;
	6435	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.5;
	6435	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.3;
	6435	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.8;
	6436	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392;
	6436	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.2;
	6436	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.1;
	6436	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.7;
	6436	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.7;
	6436	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.9;
	6437	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405;
	6437	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.1;
	6437	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190;
	6437	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.4;
	6437	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.1;
	6437	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.6;
	6438	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.4;
	6438	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.7;
	6438	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.9;
	6438	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.4;
	6438	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.3;
	6438	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.4;
	6439	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.8;
	6439	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.3;
	6439	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.9;
	6439	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.6;
	6439	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.9;
	6439	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.5;
	6440	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.1;
	6440	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.9;
	6440	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.2;
	6440	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.7;
	6440	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.7;
	6440	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.4;
	6441	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.9;
	6441	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.6;
	6441	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.2;
	6441	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.5;
	6441	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	6441	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.2;
	6442	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.2;
	6442	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148;
	6442	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.1;
	6442	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244;
	6442	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.2;
	6442	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.6;
	6443	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.6;
	6443	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.7;
	6443	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.8;
	6443	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.2;
	6443	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.8;
	6443	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.3;
	6444	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.4;
	6444	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.4;
	6444	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.5;
	6444	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.3;
	6444	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.3;
	6444	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.5;
	6445	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.2;
	6445	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.6;
	6445	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.5;
	6445	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.5;
	6445	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	6445	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.5;
	6446	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.6;
	6446	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.8;
	6446	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.4;
	6446	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.8;
	6446	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	6446	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.4;
	6447	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.7;
	6447	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.2;
	6447	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.2;
	6447	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.3;
	6447	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	6447	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.6;
	6448	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509;
	6448	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152;
	6448	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253;
	6448	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	263.8;
	6448	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.3;
	6448	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391;
	6449	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.6;
	6449	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.7;
	6449	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.1;
	6449	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	263.5;
	6449	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	6449	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.1;
	6450	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.4;
	6450	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.2;
	6450	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.5;
	6450	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.9;
	6450	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	6450	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.6;
	6451	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.7;
	6451	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.1;
	6451	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.5;
	6451	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.6;
	6451	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	6451	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.4;
	6452	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498;
	6452	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.7;
	6452	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.7;
	6452	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.5;
	6452	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	6452	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.5;
	6453	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.6;
	6453	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152;
	6453	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.5;
	6453	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.6;
	6453	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	6453	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386;
	6454	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.3;
	6454	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.5;
	6454	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231;
	6454	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.4;
	6454	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.2;
	6454	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.5;
	6455	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.9;
	6455	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129;
	6455	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214;
	6455	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.5;
	6455	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.4;
	6455	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.2;
	6456	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428.2;
	6456	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.7;
	6456	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.9;
	6456	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.4;
	6456	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.3;
	6456	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.7;
	6457	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.1;
	6457	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.6;
	6457	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.8;
	6457	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.7;
	6457	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.5;
	6457	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.5;
	6458	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.4;
	6458	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109;
	6458	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.4;
	6458	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.5;
	6458	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.7;
	6458	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.1;
	6459	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.1;
	6459	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.2;
	6459	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.1;
	6459	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192;
	6459	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.3;
	6459	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.1;
	6460	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.3;
	6460	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.9;
	6460	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.7;
	6460	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.7;
	6460	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51;
	6460	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322;
	6461	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.9;
	6461	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.4;
	6461	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.3;
	6461	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.7;
	6461	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	50.7;
	6461	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.5;
	6462	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.5;
	6462	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.2;
	6462	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.7;
	6462	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.6;
	6462	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.4;
	6462	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.5;
	6463	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480;
	6463	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.7;
	6463	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.3;
	6463	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.2;
	6463	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.5;
	6463	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.2;
	6464	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.6;
	6464	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.3;
	6464	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.7;
	6464	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.6;
	6464	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.8;
	6464	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.9;
	6465	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.4;
	6465	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.6;
	6465	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.3;
	6465	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.4;
	6465	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.3;
	6465	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.5;
	6466	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.2;
	6466	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.6;
	6466	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.3;
	6466	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246;
	6466	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	6466	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.5;
	6467	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.7;
	6467	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.1;
	6467	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.6;
	6467	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.1;
	6467	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	6467	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.3;
	6468	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.4;
	6468	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.2;
	6468	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.6;
	6468	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.9;
	6468	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	6468	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.5;
	6469	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.2;
	6469	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.3;
	6469	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.3;
	6469	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.7;
	6469	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.2;
	6469	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.1;
	6470	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.5;
	6470	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.2;
	6470	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.9;
	6470	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.4;
	6470	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	6470	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.7;
	6471	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.4;
	6471	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.4;
	6471	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.1;
	6471	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.1;
	6471	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	6471	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.4;
	6472	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512;
	6472	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.2;
	6472	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.1;
	6472	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.4;
	6472	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	6472	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393;
	6473	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.9;
	6473	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.5;
	6473	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.1;
	6473	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260.7;
	6473	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	6473	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.1;
	6474	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.3;
	6474	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154;
	6474	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.2;
	6474	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.8;
	6474	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	6474	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.4;
	6475	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.5;
	6475	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157;
	6475	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.9;
	6475	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.3;
	6475	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	6475	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.3;
	6476	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.7;
	6476	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.1;
	6476	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.2;
	6476	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.2;
	6476	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	6476	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.6;
	6477	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.8;
	6477	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.8;
	6477	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.3;
	6477	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.3;
	6477	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	6477	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.8;
	6478	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.9;
	6478	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.1;
	6478	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.4;
	6478	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234;
	6478	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	6478	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.5;
	6479	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	430.1;
	6479	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.4;
	6479	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.6;
	6479	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.5;
	6479	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.3;
	6479	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.1;
	6480	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.9;
	6480	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.9;
	6480	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.8;
	6480	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.4;
	6480	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	6480	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337;
	6481	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.7;
	6481	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.8;
	6481	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.2;
	6481	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.6;
	6481	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57;
	6481	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.2;
	6482	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	382.6;
	6482	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.9;
	6482	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.7;
	6482	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.6;
	6482	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.1;
	6482	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317;
	6483	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	388;
	6483	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.7;
	6483	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.4;
	6483	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191;
	6483	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	6483	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.7;
	6484	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	390.2;
	6484	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.3;
	6484	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.7;
	6484	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.6;
	6484	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.2;
	6484	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.7;
	6485	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405;
	6485	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.2;
	6485	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.5;
	6485	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.7;
	6485	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.5;
	6485	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.3;
	6486	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428.6;
	6486	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.5;
	6486	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.3;
	6486	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.8;
	6486	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	6486	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.4;
	6487	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.3;
	6487	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.3;
	6487	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.8;
	6487	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.6;
	6487	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.2;
	6487	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.5;
	6488	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.6;
	6488	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.5;
	6488	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.2;
	6488	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.2;
	6488	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	6488	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.3;
	6489	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.5;
	6489	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.6;
	6489	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.5;
	6489	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.9;
	6489	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	6489	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.4;
	6490	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.4;
	6490	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.2;
	6490	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.8;
	6490	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.1;
	6490	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	6490	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.2;
	6491	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.4;
	6491	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.2;
	6491	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.9;
	6491	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.4;
	6491	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	6491	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.6;
	6492	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.4;
	6492	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.1;
	6492	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.8;
	6492	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.9;
	6492	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	6492	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.8;
	6493	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.9;
	6493	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.9;
	6493	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.6;
	6493	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.6;
	6493	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69;
	6493	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.7;
	6494	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.9;
	6494	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.5;
	6494	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.1;
	6494	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.4;
	6494	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.6;
	6494	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.7;
	6495	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.2;
	6495	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150;
	6495	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.1;
	6495	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.3;
	6495	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	6495	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.9;
	6496	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.6;
	6496	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.9;
	6496	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.2;
	6496	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	263.3;
	6496	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	6496	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.1;
	6497	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.2;
	6497	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.6;
	6497	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.5;
	6497	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	263.3;
	6497	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.8;
	6497	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.3;
	6498	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.1;
	6498	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.9;
	6498	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257;
	6498	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.8;
	6498	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.3;
	6498	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.8;
	6499	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.4;
	6499	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.2;
	6499	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.5;
	6499	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.1;
	6499	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	6499	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.3;
	6500	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487;
	6500	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158;
	6500	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.8;
	6500	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.8;
	6500	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.4;
	6500	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.2;
	6501	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479;
	6501	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.5;
	6501	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.4;
	6501	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.8;
	6501	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.5;
	6501	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.7;
	6502	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.4;
	6502	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.2;
	6502	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.8;
	6502	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.8;
	6502	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	6502	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.8;
	6503	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.6;
	6503	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.1;
	6503	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.2;
	6503	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.1;
	6503	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	6503	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.7;
	6504	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	402.2;
	6504	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.4;
	6504	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.9;
	6504	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.1;
	6504	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60;
	6504	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.1;
	6505	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392.9;
	6505	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.9;
	6505	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.7;
	6505	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.5;
	6505	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.7;
	6505	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.9;
	6506	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	377.8;
	6506	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.1;
	6506	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.1;
	6506	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.9;
	6506	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.8;
	6506	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.5;
	6507	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	367.3;
	6507	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107;
	6507	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.4;
	6507	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.1;
	6507	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.3;
	6507	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310.5;
	6508	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	372.4;
	6508	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.2;
	6508	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.4;
	6508	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.1;
	6508	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.8;
	6508	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.8;
	6509	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	389;
	6509	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.2;
	6509	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191;
	6509	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194;
	6509	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.5;
	6509	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.1;
	6510	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.9;
	6510	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.8;
	6510	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201;
	6510	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.5;
	6510	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.3;
	6510	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.5;
	6511	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.7;
	6511	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.7;
	6511	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.3;
	6511	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.8;
	6511	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55;
	6511	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.3;
	6512	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447.5;
	6512	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.5;
	6512	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.9;
	6512	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.1;
	6512	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	6512	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.9;
	6513	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	444;
	6513	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.5;
	6513	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.5;
	6513	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.3;
	6513	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	6513	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.3;
	6514	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	444.5;
	6514	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145;
	6514	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.9;
	6514	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.8;
	6514	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	6514	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.2;
	6515	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.4;
	6515	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.8;
	6515	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.9;
	6515	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.6;
	6515	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	6515	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.9;
	6516	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.9;
	6516	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.2;
	6516	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.7;
	6516	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247;
	6516	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.9;
	6516	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.9;
	6517	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.9;
	6517	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.7;
	6517	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.4;
	6517	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.5;
	6517	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.7;
	6517	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.2;
	6518	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.6;
	6518	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.5;
	6518	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.8;
	6518	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.4;
	6518	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	6518	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.5;
	6519	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.7;
	6519	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.7;
	6519	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250;
	6519	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.1;
	6519	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69;
	6519	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.4;
	6520	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.4;
	6520	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.2;
	6520	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.3;
	6520	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.1;
	6520	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	6520	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.7;
	6521	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.6;
	6521	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.2;
	6521	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.9;
	6521	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.8;
	6521	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.3;
	6521	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.5;
	6522	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.7;
	6522	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.2;
	6522	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249;
	6522	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.6;
	6522	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	6522	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369;
	6523	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	444.7;
	6523	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.2;
	6523	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.3;
	6523	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.9;
	6523	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.2;
	6523	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364;
	6524	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.6;
	6524	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150;
	6524	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.6;
	6524	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247;
	6524	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	6524	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.4;
	6525	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.9;
	6525	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.6;
	6525	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.1;
	6525	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.6;
	6525	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68;
	6525	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.1;
	6526	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.3;
	6526	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.3;
	6526	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.7;
	6526	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.2;
	6526	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	6526	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.9;
	6527	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.1;
	6527	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.4;
	6527	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.4;
	6527	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.4;
	6527	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	6527	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.1;
	6528	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	389.3;
	6528	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.8;
	6528	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.1;
	6528	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.9;
	6528	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	6528	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.4;
	6529	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	371.2;
	6529	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.1;
	6529	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.8;
	6529	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.7;
	6529	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.1;
	6529	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.9;
	6530	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	353.7;
	6530	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.9;
	6530	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.1;
	6530	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.7;
	6530	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.1;
	6530	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	306.6;
	6531	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	353.8;
	6531	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	104.7;
	6531	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.4;
	6531	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.6;
	6531	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.9;
	6531	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	305.6;
	6532	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	359.5;
	6532	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	104.4;
	6532	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.8;
	6532	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.3;
	6532	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54;
	6532	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	307.8;
	6533	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	366.7;
	6533	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108;
	6533	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.7;
	6533	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.2;
	6533	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54;
	6533	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311.8;
	6534	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	373.2;
	6534	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.1;
	6534	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.5;
	6534	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.6;
	6534	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.6;
	6534	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.1;
	6535	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	380.6;
	6535	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.6;
	6535	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.3;
	6535	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.4;
	6535	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.5;
	6535	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.9;
	6536	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	384.1;
	6536	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.7;
	6536	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.3;
	6536	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202;
	6536	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.9;
	6536	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.5;
	6537	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	395.7;
	6537	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.6;
	6537	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.8;
	6537	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.4;
	6537	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	6537	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.4;
	6538	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.4;
	6538	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.3;
	6538	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.2;
	6538	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.8;
	6538	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	6538	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.6;
	6539	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.3;
	6539	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.3;
	6539	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.6;
	6539	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.1;
	6539	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.4;
	6539	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.6;
	6540	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.2;
	6540	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.8;
	6540	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.4;
	6540	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.7;
	6540	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	6540	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.4;
	6541	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.1;
	6541	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.5;
	6541	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.7;
	6541	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.1;
	6541	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	6541	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.1;
	6542	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.3;
	6542	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.3;
	6542	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219;
	6542	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.4;
	6542	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.1;
	6542	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.3;
	6543	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.6;
	6543	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133;
	6543	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.7;
	6543	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.5;
	6543	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	6543	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.7;
	6544	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.5;
	6544	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.7;
	6544	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.8;
	6544	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223;
	6544	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.5;
	6544	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.4;
	6545	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.1;
	6545	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.7;
	6545	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221;
	6545	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.2;
	6545	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69;
	6545	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.3;
	6546	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.9;
	6546	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.2;
	6546	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.7;
	6546	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.1;
	6546	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.1;
	6546	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.7;
	6547	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403.3;
	6547	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.8;
	6547	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.5;
	6547	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.1;
	6547	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.1;
	6547	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.6;
	6548	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411;
	6548	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137;
	6548	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.7;
	6548	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.6;
	6548	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	6548	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.6;
	6549	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.6;
	6549	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.8;
	6549	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.7;
	6549	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.6;
	6549	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	6549	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.3;
	6550	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	386.4;
	6550	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.7;
	6550	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.8;
	6550	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.7;
	6550	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	6550	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.9;
	6551	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	365;
	6551	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.6;
	6551	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.3;
	6551	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.1;
	6551	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	6551	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.3;
	6552	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	347.6;
	6552	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.1;
	6552	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.4;
	6552	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.5;
	6552	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.7;
	6552	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	307.3;
	6553	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	332.1;
	6553	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	104.4;
	6553	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.8;
	6553	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.4;
	6553	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.3;
	6553	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	298.4;
	6554	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	311.1;
	6554	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	102.4;
	6554	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.1;
	6554	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.5;
	6554	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.3;
	6554	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	287;
	6555	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	313.4;
	6555	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	100.4;
	6555	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.5;
	6555	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	182.7;
	6555	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.9;
	6555	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	287;
	6556	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	315.2;
	6556	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	101.1;
	6556	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.7;
	6556	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	183.1;
	6556	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.2;
	6556	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	287.7;
	6557	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	319.2;
	6557	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	102.2;
	6557	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181.2;
	6557	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	183.6;
	6557	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.1;
	6557	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	290;
	6558	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	328.7;
	6558	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.7;
	6558	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.9;
	6558	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.5;
	6558	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.5;
	6558	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	295.3;
	6559	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	338.3;
	6559	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.6;
	6559	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186;
	6559	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.5;
	6559	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.8;
	6559	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	300.4;
	6560	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	342.5;
	6560	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114;
	6560	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.6;
	6560	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.3;
	6560	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.5;
	6560	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	302.5;
	6561	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	355.2;
	6561	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.9;
	6561	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.6;
	6561	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.6;
	6561	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	6561	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309.1;
	6562	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	357.5;
	6562	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.1;
	6562	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.2;
	6562	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.9;
	6562	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	6562	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311.6;
	6563	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	364.3;
	6563	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.5;
	6563	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.8;
	6563	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.4;
	6563	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	6563	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.2;
	6564	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	375;
	6564	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.7;
	6564	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.5;
	6564	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.9;
	6564	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	6564	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.2;
	6565	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	373.3;
	6565	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.5;
	6565	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.7;
	6565	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.7;
	6565	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	6565	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.8;
	6566	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.2;
	6566	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.8;
	6566	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.3;
	6566	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.7;
	6566	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.9;
	6566	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.9;
	6567	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	386.2;
	6567	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.6;
	6567	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.4;
	6567	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.3;
	6567	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	6567	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.1;
	6568	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391.6;
	6568	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.3;
	6568	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.7;
	6568	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.5;
	6568	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	6568	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.4;
	6569	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391.4;
	6569	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.7;
	6569	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.9;
	6569	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.7;
	6569	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	6569	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.2;
	6570	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392.4;
	6570	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.9;
	6570	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.7;
	6570	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.3;
	6570	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	6570	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.1;
	6571	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	401.6;
	6571	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.5;
	6571	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.1;
	6571	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.8;
	6571	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	6571	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.1;
	6572	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.6;
	6572	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145;
	6572	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.9;
	6572	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.5;
	6572	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	6572	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.1;
	6573	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414;
	6573	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.5;
	6573	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.2;
	6573	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230;
	6573	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	6573	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.7;
	6574	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.5;
	6574	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.3;
	6574	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.4;
	6574	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.8;
	6574	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	6574	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.3;
	6575	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	370.7;
	6575	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.1;
	6575	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203;
	6575	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.2;
	6575	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.6;
	6575	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.1;
	6576	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	351.3;
	6576	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115;
	6576	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.2;
	6576	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.6;
	6576	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59;
	6576	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308;
	6577	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	343.5;
	6577	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.5;
	6577	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.2;
	6577	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.7;
	6577	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.8;
	6577	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	301.8;
	6578	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	342.4;
	6578	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	104.3;
	6578	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.8;
	6578	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185;
	6578	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.8;
	6578	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	299;
	6579	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	344.2;
	6579	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	102.4;
	6579	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181.6;
	6579	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	183.7;
	6579	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.8;
	6579	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	299.3;
	6580	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	350;
	6580	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	104;
	6580	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.1;
	6580	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.5;
	6580	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.7;
	6580	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	301.6;
	6581	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	376.1;
	6581	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.8;
	6581	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.4;
	6581	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.7;
	6581	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.6;
	6581	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.3;
	6582	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	395.4;
	6582	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.4;
	6582	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.5;
	6582	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.3;
	6582	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.4;
	6582	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.6;
	6583	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439;
	6583	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.5;
	6583	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.2;
	6583	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.6;
	6583	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.2;
	6583	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.9;
	6584	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.8;
	6584	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.5;
	6584	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.1;
	6584	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.8;
	6584	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.5;
	6584	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.2;
	6585	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.4;
	6585	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.2;
	6585	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.6;
	6585	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.8;
	6585	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.8;
	6585	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.8;
	6586	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.3;
	6586	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145;
	6586	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.2;
	6586	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.1;
	6586	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.1;
	6586	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.1;
	6587	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.3;
	6587	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.6;
	6587	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.2;
	6587	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.4;
	6587	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.3;
	6587	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.2;
	6588	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.9;
	6588	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.3;
	6588	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.9;
	6588	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245;
	6588	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	6588	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.7;
	6589	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477;
	6589	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.9;
	6589	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.7;
	6589	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243;
	6589	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	6589	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.2;
	6590	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.5;
	6590	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146;
	6590	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.2;
	6590	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245;
	6590	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	6590	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.2;
	6591	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.8;
	6591	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.1;
	6591	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.3;
	6591	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.5;
	6591	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	6591	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.6;
	6592	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.4;
	6592	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146;
	6592	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246;
	6592	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.2;
	6592	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	6592	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.5;
	6593	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.1;
	6593	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.6;
	6593	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.2;
	6593	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.5;
	6593	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	6593	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374;
	6594	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.6;
	6594	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.2;
	6594	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.9;
	6594	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248;
	6594	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	6594	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.3;
	6595	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.8;
	6595	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.4;
	6595	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.5;
	6595	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.1;
	6595	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	6595	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.4;
	6596	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.2;
	6596	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.9;
	6596	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.3;
	6596	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.3;
	6596	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	6596	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.2;
	6597	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.2;
	6597	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.3;
	6597	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.4;
	6597	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.1;
	6597	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	6597	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.5;
	6598	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.1;
	6598	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.5;
	6598	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.7;
	6598	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231;
	6598	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	6598	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.7;
	6599	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.8;
	6599	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.8;
	6599	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.4;
	6599	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215;
	6599	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.7;
	6599	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344;
	6600	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.1;
	6600	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.1;
	6600	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.2;
	6600	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.4;
	6600	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	6600	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.4;
	6601	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	385.9;
	6601	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.4;
	6601	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.6;
	6601	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.9;
	6601	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.2;
	6601	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322;
	6602	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	362;
	6602	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.8;
	6602	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.4;
	6602	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.5;
	6602	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.6;
	6602	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309.5;
	6603	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	357.2;
	6603	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.7;
	6603	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.1;
	6603	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.7;
	6603	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.8;
	6603	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	306.9;
	6604	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	360.5;
	6604	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.2;
	6604	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.6;
	6604	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.9;
	6604	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.2;
	6604	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.5;
	6605	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	365.4;
	6605	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.9;
	6605	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.4;
	6605	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.5;
	6605	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.6;
	6605	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311.9;
	6606	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	390.9;
	6606	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.5;
	6606	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.8;
	6606	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206;
	6606	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.6;
	6606	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.2;
	6607	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.4;
	6607	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.6;
	6607	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.7;
	6607	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.5;
	6607	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.2;
	6607	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.2;
	6608	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.3;
	6608	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.5;
	6608	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.7;
	6608	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.8;
	6608	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.9;
	6608	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.1;
	6609	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.3;
	6609	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144;
	6609	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.8;
	6609	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.5;
	6609	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.6;
	6609	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.2;
	6610	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.6;
	6610	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.2;
	6610	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.2;
	6610	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.3;
	6610	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	6610	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.6;
	6611	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.8;
	6611	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.1;
	6611	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.5;
	6611	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.7;
	6611	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	6611	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.3;
	6612	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.7;
	6612	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.7;
	6612	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.3;
	6612	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.8;
	6612	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.4;
	6612	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.5;
	6613	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.3;
	6613	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.7;
	6613	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.8;
	6613	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.3;
	6613	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	6613	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.8;
	6614	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.5;
	6614	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.6;
	6614	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.3;
	6614	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.6;
	6614	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	6614	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.8;
	6615	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.7;
	6615	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.6;
	6615	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258;
	6615	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257;
	6615	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	6615	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.8;
	6616	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457;
	6616	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.3;
	6616	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.4;
	6616	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.4;
	6616	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	6616	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.8;
	6617	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.8;
	6617	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151;
	6617	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258;
	6617	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257;
	6617	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	6617	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.9;
	6618	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.3;
	6618	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.3;
	6618	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.5;
	6618	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.5;
	6618	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	6618	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.9;
	6619	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.4;
	6619	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.3;
	6619	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.9;
	6619	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.3;
	6619	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	6619	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.3;
	6620	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.1;
	6620	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.4;
	6620	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.7;
	6620	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.8;
	6620	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	6620	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.3;
	6621	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.6;
	6621	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.5;
	6621	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248;
	6621	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.9;
	6621	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	6621	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.2;
	6622	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.7;
	6622	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141;
	6622	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.1;
	6622	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.8;
	6622	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	6622	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.2;
	6623	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.4;
	6623	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.7;
	6623	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.4;
	6623	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.3;
	6623	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	6623	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.4;
	6624	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	389.2;
	6624	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.8;
	6624	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.5;
	6624	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.2;
	6624	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.1;
	6624	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.4;
	6625	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	378.4;
	6625	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.1;
	6625	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.6;
	6625	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.4;
	6625	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.2;
	6625	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.7;
	6626	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	363.1;
	6626	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.5;
	6626	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.3;
	6626	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.1;
	6626	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.8;
	6626	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310.1;
	6627	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	362.2;
	6627	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.8;
	6627	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.7;
	6627	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.8;
	6627	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.1;
	6627	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309;
	6628	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	366.3;
	6628	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.4;
	6628	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.5;
	6628	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.9;
	6628	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.7;
	6628	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310.9;
	6629	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	373.2;
	6629	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.4;
	6629	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.3;
	6629	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.9;
	6629	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.2;
	6629	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.4;
	6630	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.3;
	6630	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.6;
	6630	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.2;
	6630	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.6;
	6630	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.1;
	6630	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.1;
	6631	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.3;
	6631	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139;
	6631	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.8;
	6631	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.2;
	6631	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	6631	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.5;
	6632	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440;
	6632	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.2;
	6632	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.1;
	6632	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.5;
	6632	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.5;
	6632	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.7;
	6633	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.1;
	6633	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.2;
	6633	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.1;
	6633	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.5;
	6633	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.2;
	6633	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.3;
	6634	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.3;
	6634	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.2;
	6634	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.3;
	6634	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248;
	6634	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	6634	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358;
	6635	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.8;
	6635	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.8;
	6635	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.9;
	6635	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.7;
	6635	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	6635	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.2;
	6636	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.1;
	6636	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.2;
	6636	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.1;
	6636	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.2;
	6636	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	6636	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.8;
	6637	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442.1;
	6637	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.8;
	6637	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.3;
	6637	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.3;
	6637	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	6637	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.8;
	6638	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.3;
	6638	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.4;
	6638	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.8;
	6638	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.9;
	6638	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.7;
	6638	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.4;
	6639	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.6;
	6639	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.4;
	6639	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.2;
	6639	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.9;
	6639	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.8;
	6639	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.3;
	6640	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.7;
	6640	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.5;
	6640	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.3;
	6640	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.2;
	6640	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.6;
	6640	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.1;
	6641	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.8;
	6641	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.9;
	6641	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.5;
	6641	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251;
	6641	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73;
	6641	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.7;
	6642	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.9;
	6642	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.2;
	6642	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.5;
	6642	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.4;
	6642	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.5;
	6642	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.6;
	6643	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.4;
	6643	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.4;
	6643	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.8;
	6643	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.6;
	6643	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.5;
	6643	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.6;
	6644	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.3;
	6644	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.5;
	6644	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252;
	6644	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.8;
	6644	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.9;
	6644	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.1;
	6645	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.3;
	6645	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.4;
	6645	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243;
	6645	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.7;
	6645	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.4;
	6645	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.3;
	6646	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.9;
	6646	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.5;
	6646	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.4;
	6646	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.6;
	6646	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	6646	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.4;
	6647	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394.4;
	6647	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.6;
	6647	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.4;
	6647	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.6;
	6647	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	6647	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.5;
	6648	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	377.2;
	6648	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.1;
	6648	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.7;
	6648	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.2;
	6648	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.9;
	6648	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.8;
	6649	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	362.5;
	6649	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.5;
	6649	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.7;
	6649	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.7;
	6649	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.2;
	6649	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.8;
	6650	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	350.5;
	6650	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.7;
	6650	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.4;
	6650	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189;
	6650	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.6;
	6650	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	304.4;
	6651	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	352.1;
	6651	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	105.6;
	6651	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.6;
	6651	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.1;
	6651	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.8;
	6651	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	304.4;
	6652	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	349;
	6652	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.5;
	6652	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.4;
	6652	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.4;
	6652	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.4;
	6652	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	303.6;
	6653	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	355.1;
	6653	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.1;
	6653	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.3;
	6653	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.2;
	6653	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.7;
	6653	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	307.9;
	6654	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	387.1;
	6654	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.4;
	6654	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.8;
	6654	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.4;
	6654	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.4;
	6654	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.4;
	6655	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.2;
	6655	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.1;
	6655	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.5;
	6655	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.6;
	6655	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54;
	6655	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340;
	6656	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.2;
	6656	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.3;
	6656	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.6;
	6656	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.2;
	6656	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.4;
	6656	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353;
	6657	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.2;
	6657	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.8;
	6657	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.6;
	6657	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.4;
	6657	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.3;
	6657	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.1;
	6658	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.2;
	6658	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.6;
	6658	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.8;
	6658	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242;
	6658	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	6658	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.4;
	6659	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439;
	6659	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.6;
	6659	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242;
	6659	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.1;
	6659	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	6659	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.8;
	6660	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442.2;
	6660	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.3;
	6660	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.8;
	6660	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.7;
	6660	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	6660	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.4;
	6661	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.3;
	6661	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.2;
	6661	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.2;
	6661	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.4;
	6661	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	6661	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.4;
	6662	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.5;
	6662	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.7;
	6662	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.2;
	6662	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.4;
	6662	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.3;
	6662	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363;
	6663	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.7;
	6663	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.7;
	6663	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.8;
	6663	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.9;
	6663	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	6663	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360;
	6664	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.5;
	6664	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.5;
	6664	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.3;
	6664	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.4;
	6664	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.5;
	6664	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.8;
	6665	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.2;
	6665	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.8;
	6665	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.5;
	6665	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.4;
	6665	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69;
	6665	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.6;
	6666	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.9;
	6666	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.2;
	6666	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243;
	6666	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.4;
	6666	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68;
	6666	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.5;
	6667	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.3;
	6667	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.5;
	6667	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.4;
	6667	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249;
	6667	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	6667	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.5;
	6668	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.7;
	6668	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.9;
	6668	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.4;
	6668	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.4;
	6668	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	6668	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.4;
	6669	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448;
	6669	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.8;
	6669	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.8;
	6669	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.8;
	6669	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	6669	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.3;
	6670	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.3;
	6670	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.8;
	6670	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226;
	6670	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.9;
	6670	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	6670	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.3;
	6671	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.9;
	6671	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.5;
	6671	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.4;
	6671	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.7;
	6671	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	6671	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.4;
	6672	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	395;
	6672	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121;
	6672	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.2;
	6672	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.4;
	6672	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59;
	6672	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.4;
	6673	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392;
	6673	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.1;
	6673	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.7;
	6673	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.3;
	6673	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.5;
	6673	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.2;
	6674	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	363.5;
	6674	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.5;
	6674	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.8;
	6674	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.5;
	6674	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54;
	6674	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.7;
	6675	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	358.6;
	6675	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	105.1;
	6675	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.7;
	6675	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.8;
	6675	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.8;
	6675	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	306.2;
	6676	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	354.8;
	6676	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.6;
	6676	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.6;
	6676	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.7;
	6676	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.6;
	6676	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	305.1;
	6677	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	372.3;
	6677	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.6;
	6677	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.6;
	6677	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192;
	6677	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.5;
	6677	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.1;
	6678	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.7;
	6678	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.5;
	6678	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.9;
	6678	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.9;
	6678	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52;
	6678	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.6;
	6679	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.4;
	6679	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.6;
	6679	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.8;
	6679	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.9;
	6679	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.8;
	6679	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.2;
	6680	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.3;
	6680	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.3;
	6680	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.9;
	6680	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.6;
	6680	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.3;
	6680	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361;
	6681	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.5;
	6681	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.7;
	6681	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.5;
	6681	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.4;
	6681	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	6681	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370;
	6682	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.3;
	6682	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.9;
	6682	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.6;
	6682	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.2;
	6682	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	6682	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.6;
	6683	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.4;
	6683	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.9;
	6683	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.5;
	6683	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.7;
	6683	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	6683	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369;
	6684	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.4;
	6684	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.3;
	6684	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.6;
	6684	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.4;
	6684	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	6684	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.4;
	6685	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.3;
	6685	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.9;
	6685	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.5;
	6685	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.9;
	6685	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	6685	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.7;
	6686	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.6;
	6686	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.8;
	6686	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.1;
	6686	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.2;
	6686	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	6686	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.5;
	6687	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454;
	6687	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.5;
	6687	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.6;
	6687	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.6;
	6687	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	6687	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.4;
	6688	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.4;
	6688	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.1;
	6688	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.8;
	6688	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248;
	6688	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	6688	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.4;
	6689	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.3;
	6689	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.8;
	6689	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.3;
	6689	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.5;
	6689	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	6689	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.3;
	6690	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437;
	6690	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.6;
	6690	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.4;
	6690	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244;
	6690	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	6690	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.7;
	6691	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.1;
	6691	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.3;
	6691	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245;
	6691	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.1;
	6691	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	6691	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.1;
	6692	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439;
	6692	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.8;
	6692	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.3;
	6692	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.9;
	6692	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	6692	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.1;
	6693	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.2;
	6693	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.6;
	6693	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.2;
	6693	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.9;
	6693	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	6693	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.5;
	6694	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440;
	6694	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.1;
	6694	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.6;
	6694	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.3;
	6694	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	6694	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.4;
	6695	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.7;
	6695	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.6;
	6695	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.2;
	6695	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.2;
	6695	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.6;
	6695	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.7;
	6696	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391.7;
	6696	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.1;
	6696	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.7;
	6696	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.1;
	6696	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.5;
	6696	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.3;
	6697	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	377.5;
	6697	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.3;
	6697	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.1;
	6697	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196;
	6697	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.7;
	6697	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.7;
	6698	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	373.8;
	6698	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106;
	6698	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.9;
	6698	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.6;
	6698	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.8;
	6698	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.6;
	6699	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	370.3;
	6699	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	104.1;
	6699	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.7;
	6699	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.3;
	6699	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.6;
	6699	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311.4;
	6700	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	367.6;
	6700	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	105.1;
	6700	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.9;
	6700	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.8;
	6700	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.7;
	6700	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310.6;
	6701	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	377.9;
	6701	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.3;
	6701	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.1;
	6701	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.7;
	6701	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.7;
	6701	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316;
	6702	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379.3;
	6702	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.9;
	6702	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.9;
	6702	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.4;
	6702	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.8;
	6702	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.5;
	6703	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	400.5;
	6703	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.3;
	6703	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.3;
	6703	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.1;
	6703	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.2;
	6703	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.9;
	6704	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.5;
	6704	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.1;
	6704	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.8;
	6704	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.1;
	6704	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.7;
	6704	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335;
	6705	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.8;
	6705	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.1;
	6705	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.6;
	6705	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.2;
	6705	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.4;
	6705	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.9;
	6706	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.4;
	6706	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.7;
	6706	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.8;
	6706	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.1;
	6706	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	6706	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.3;
	6707	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	420.9;
	6707	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139;
	6707	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.5;
	6707	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.7;
	6707	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	6707	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344;
	6708	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.9;
	6708	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.5;
	6708	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.4;
	6708	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.5;
	6708	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	6708	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345;
	6709	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.4;
	6709	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141;
	6709	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.9;
	6709	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.5;
	6709	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	6709	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.5;
	6710	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.2;
	6710	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.8;
	6710	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.5;
	6710	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.5;
	6710	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	6710	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.2;
	6711	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.5;
	6711	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.6;
	6711	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220;
	6711	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.8;
	6711	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.8;
	6711	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.4;
	6712	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.8;
	6712	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138;
	6712	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.3;
	6712	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.3;
	6712	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.6;
	6712	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.5;
	6713	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411;
	6713	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.8;
	6713	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.8;
	6713	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.2;
	6713	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	6713	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.7;
	6714	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.1;
	6714	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138;
	6714	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.3;
	6714	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.1;
	6714	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	6714	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.3;
	6715	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.1;
	6715	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.4;
	6715	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.9;
	6715	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.9;
	6715	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.2;
	6715	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342;
	6716	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.4;
	6716	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.5;
	6716	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.3;
	6716	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.6;
	6716	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	6716	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.4;
	6717	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.3;
	6717	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.6;
	6717	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.4;
	6717	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.8;
	6717	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	6717	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.9;
	6718	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.3;
	6718	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133;
	6718	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214;
	6718	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.2;
	6718	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	6718	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.7;
	6719	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	385.1;
	6719	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.7;
	6719	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.7;
	6719	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.5;
	6719	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	6719	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.1;
	6720	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	371.8;
	6720	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121;
	6720	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203;
	6720	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.3;
	6720	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.1;
	6720	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.1;
	6721	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	357;
	6721	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.9;
	6721	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188;
	6721	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.9;
	6721	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.5;
	6721	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.3;
	6722	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	336.4;
	6722	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.6;
	6722	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.1;
	6722	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187;
	6722	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.1;
	6722	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	297.2;
	6723	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	329.9;
	6723	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	105.5;
	6723	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181.7;
	6723	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	183.8;
	6723	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.8;
	6723	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	293.6;
	6724	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	331.6;
	6724	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	105.6;
	6724	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181.8;
	6724	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	183.8;
	6724	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.5;
	6724	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	294.5;
	6725	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	339.8;
	6725	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.5;
	6725	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.7;
	6725	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.7;
	6725	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.7;
	6725	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	298.7;
	6726	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	354.7;
	6726	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.9;
	6726	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.5;
	6726	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.8;
	6726	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53;
	6726	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	306;
	6727	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	369.2;
	6727	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.4;
	6727	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.5;
	6727	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.5;
	6727	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54;
	6727	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.9;
	6728	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	374.8;
	6728	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.8;
	6728	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.1;
	6728	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192;
	6728	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.1;
	6728	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316;
	6729	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	387.7;
	6729	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.3;
	6729	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.1;
	6729	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.5;
	6729	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.3;
	6729	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.6;
	6730	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	395.9;
	6730	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.5;
	6730	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.5;
	6730	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.3;
	6730	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	6730	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.1;
	6731	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	402.9;
	6731	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.1;
	6731	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.8;
	6731	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.3;
	6731	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	6731	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.6;
	6732	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.6;
	6732	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.7;
	6732	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.7;
	6732	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.5;
	6732	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	6732	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.4;
	6733	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.2;
	6733	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.4;
	6733	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.4;
	6733	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.1;
	6733	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	6733	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.8;
	6734	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.7;
	6734	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137;
	6734	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204;
	6734	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.3;
	6734	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	6734	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.8;
	6735	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	402.5;
	6735	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.6;
	6735	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.3;
	6735	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.5;
	6735	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.8;
	6735	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331;
	6736	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	401.6;
	6736	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.7;
	6736	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.3;
	6736	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.6;
	6736	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	6736	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.2;
	6737	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.8;
	6737	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.5;
	6737	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.8;
	6737	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213;
	6737	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.2;
	6737	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.2;
	6738	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.1;
	6738	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.9;
	6738	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.4;
	6738	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.9;
	6738	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	6738	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.9;
	6739	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.6;
	6739	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.1;
	6739	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225;
	6739	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.4;
	6739	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	6739	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339;
	6740	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.3;
	6740	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.6;
	6740	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.5;
	6740	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.6;
	6740	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	6740	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.6;
	6741	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428;
	6741	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.8;
	6741	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.6;
	6741	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.2;
	6741	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	6741	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.8;
	6742	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.4;
	6742	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.6;
	6742	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.6;
	6742	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.5;
	6742	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	6742	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.6;
	6743	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391;
	6743	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.9;
	6743	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.1;
	6743	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.5;
	6743	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	6743	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.9;
	6744	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	375.4;
	6744	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.4;
	6744	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.7;
	6744	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.8;
	6744	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	6744	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.3;
	6745	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	362;
	6745	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.8;
	6745	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.1;
	6745	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186;
	6745	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.7;
	6745	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.9;
	6746	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	352.4;
	6746	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110;
	6746	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183;
	6746	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.4;
	6746	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.2;
	6746	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	302.8;
	6747	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	346.3;
	6747	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109;
	6747	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.2;
	6747	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.5;
	6747	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	6747	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	300.1;
	6748	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	350.2;
	6748	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.2;
	6748	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.9;
	6748	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.3;
	6748	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.6;
	6748	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	301.9;
	6749	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	362.3;
	6749	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.1;
	6749	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.4;
	6749	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.7;
	6749	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53;
	6749	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.2;
	6750	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	386.4;
	6750	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.7;
	6750	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195;
	6750	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.5;
	6750	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.1;
	6750	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321;
	6751	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.6;
	6751	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.9;
	6751	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.3;
	6751	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.8;
	6751	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.4;
	6751	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341;
	6752	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.4;
	6752	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.6;
	6752	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.4;
	6752	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.1;
	6752	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.9;
	6752	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.2;
	6753	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	430.9;
	6753	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.5;
	6753	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.8;
	6753	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.7;
	6753	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57;
	6753	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.6;
	6754	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.3;
	6754	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.4;
	6754	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.6;
	6754	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.1;
	6754	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	6754	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.3;
	6755	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.9;
	6755	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.6;
	6755	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.5;
	6755	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.8;
	6755	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	6755	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.9;
	6756	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.2;
	6756	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.5;
	6756	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.2;
	6756	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240;
	6756	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	6756	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.5;
	6757	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.4;
	6757	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.9;
	6757	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.8;
	6757	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.4;
	6757	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	6757	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.2;
	6758	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.9;
	6758	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.3;
	6758	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.8;
	6758	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.8;
	6758	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	6758	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.9;
	6759	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	400.4;
	6759	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.7;
	6759	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.2;
	6759	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.7;
	6759	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	6759	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.1;
	6760	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	401.2;
	6760	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.8;
	6760	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237;
	6760	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.3;
	6760	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	6760	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.7;
	6761	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.8;
	6761	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.2;
	6761	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236;
	6761	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.6;
	6761	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	6761	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.9;
	6762	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	401.1;
	6762	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.7;
	6762	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.2;
	6762	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245;
	6762	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	6762	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.5;
	6763	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.8;
	6763	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.5;
	6763	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.3;
	6763	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.9;
	6763	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	6763	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.6;
	6764	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421;
	6764	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.9;
	6764	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.2;
	6764	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.8;
	6764	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	6764	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.4;
	6765	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.1;
	6765	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.4;
	6765	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.1;
	6765	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.1;
	6765	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	6765	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.1;
	6766	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	406.8;
	6766	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.4;
	6766	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.6;
	6766	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.9;
	6766	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	6766	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.5;
	6767	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379.5;
	6767	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.5;
	6767	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.4;
	6767	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.3;
	6767	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60;
	6767	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.4;
	6768	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	353.3;
	6768	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.4;
	6768	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.3;
	6768	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.7;
	6768	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58;
	6768	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310;
	6769	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	342.9;
	6769	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.7;
	6769	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.1;
	6769	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.3;
	6769	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.8;
	6769	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	302.9;
	6770	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	371;
	6770	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.3;
	6770	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.9;
	6770	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.5;
	6770	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.1;
	6770	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311;
	6771	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	373.3;
	6771	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.5;
	6771	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.1;
	6771	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.9;
	6771	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.7;
	6771	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311.4;
	6772	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	375.9;
	6772	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109;
	6772	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.4;
	6772	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.1;
	6772	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.5;
	6772	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.7;
	6773	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	381.6;
	6773	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115;
	6773	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.6;
	6773	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.7;
	6773	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.9;
	6773	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.4;
	6774	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.2;
	6774	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.1;
	6774	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.8;
	6774	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.5;
	6774	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.6;
	6774	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.8;
	6775	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.9;
	6775	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.5;
	6775	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.7;
	6775	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.1;
	6775	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.8;
	6775	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.5;
	6776	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.2;
	6776	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.2;
	6776	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.9;
	6776	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.1;
	6776	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.7;
	6776	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.7;
	6777	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467;
	6777	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.4;
	6777	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.5;
	6777	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.1;
	6777	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.3;
	6777	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.4;
	6778	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.2;
	6778	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.6;
	6778	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230;
	6778	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.9;
	6778	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.1;
	6778	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.8;
	6779	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.8;
	6779	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.9;
	6779	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.6;
	6779	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.6;
	6779	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	6779	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.4;
	6780	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.9;
	6780	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.8;
	6780	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.9;
	6780	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.5;
	6780	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60;
	6780	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.5;
	6781	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.6;
	6781	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.7;
	6781	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.5;
	6781	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.5;
	6781	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.4;
	6781	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.4;
	6782	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.5;
	6782	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.1;
	6782	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.5;
	6782	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.7;
	6782	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.3;
	6782	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364;
	6783	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.1;
	6783	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143;
	6783	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.3;
	6783	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.7;
	6783	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.7;
	6783	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.3;
	6784	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458;
	6784	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.9;
	6784	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.6;
	6784	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.3;
	6784	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	6784	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.8;
	6785	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.9;
	6785	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.4;
	6785	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.9;
	6785	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.1;
	6785	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59;
	6785	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.9;
	6786	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.4;
	6786	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.6;
	6786	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.8;
	6786	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.3;
	6786	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	6786	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.8;
	6787	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.2;
	6787	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.4;
	6787	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.7;
	6787	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.1;
	6787	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	6787	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.2;
	6788	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468;
	6788	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.7;
	6788	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.9;
	6788	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.4;
	6788	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	6788	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.9;
	6789	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.5;
	6789	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.8;
	6789	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.5;
	6789	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.3;
	6789	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	6789	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.5;
	6790	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.2;
	6790	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.4;
	6790	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.9;
	6790	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.4;
	6790	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	6790	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.7;
	6791	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.3;
	6791	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.2;
	6791	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206;
	6791	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.3;
	6791	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	6791	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.5;
	6792	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.5;
	6792	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121;
	6792	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.4;
	6792	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.8;
	6792	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.1;
	6792	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.1;
	6793	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.2;
	6793	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.2;
	6793	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188;
	6793	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.5;
	6793	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.4;
	6793	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.9;
	6794	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	374.5;
	6794	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.3;
	6794	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.7;
	6794	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.3;
	6794	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53;
	6794	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311.2;
	6795	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	369.7;
	6795	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.6;
	6795	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.5;
	6795	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.1;
	6795	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.6;
	6795	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309;
	6796	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	375.5;
	6796	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.6;
	6796	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.4;
	6796	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.1;
	6796	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.1;
	6796	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312;
	6797	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	389;
	6797	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.8;
	6797	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.9;
	6797	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.8;
	6797	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	50.9;
	6797	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.4;
	6798	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	420.6;
	6798	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.8;
	6798	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.6;
	6798	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.4;
	6798	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.5;
	6798	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.4;
	6799	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.6;
	6799	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.6;
	6799	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.1;
	6799	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.2;
	6799	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53;
	6799	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.3;
	6800	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.9;
	6800	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.6;
	6800	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.2;
	6800	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.4;
	6800	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.7;
	6800	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.8;
	6801	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.2;
	6801	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.5;
	6801	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.8;
	6801	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.8;
	6801	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.7;
	6801	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.7;
	6802	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.4;
	6802	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.1;
	6802	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.3;
	6802	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.2;
	6802	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	6802	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.9;
	6803	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.6;
	6803	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.6;
	6803	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.4;
	6803	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.3;
	6803	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	6803	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.1;
	6804	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.9;
	6804	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144;
	6804	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.6;
	6804	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.6;
	6804	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	6804	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.1;
	6805	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.7;
	6805	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.3;
	6805	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.2;
	6805	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.9;
	6805	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	6805	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.1;
	6806	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.4;
	6806	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.3;
	6806	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232;
	6806	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235;
	6806	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	6806	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.5;
	6807	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.6;
	6807	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.2;
	6807	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.7;
	6807	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.7;
	6807	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	6807	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367;
	6808	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.9;
	6808	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.9;
	6808	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.7;
	6808	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.6;
	6808	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	6808	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.9;
	6809	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.5;
	6809	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.3;
	6809	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.9;
	6809	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.4;
	6809	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	6809	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.3;
	6810	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461;
	6810	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.3;
	6810	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.8;
	6810	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.2;
	6810	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	6810	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.3;
	6811	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.9;
	6811	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152;
	6811	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.3;
	6811	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.1;
	6811	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	6811	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370;
	6812	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.2;
	6812	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153;
	6812	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.1;
	6812	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.5;
	6812	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	6812	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377;
	6813	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.5;
	6813	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.3;
	6813	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231;
	6813	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.6;
	6813	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	6813	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.7;
	6814	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467;
	6814	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.9;
	6814	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.2;
	6814	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.3;
	6814	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	6814	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.2;
	6815	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.3;
	6815	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.1;
	6815	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.5;
	6815	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.6;
	6815	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	6815	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.6;
	6816	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.6;
	6816	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121;
	6816	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.6;
	6816	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.5;
	6816	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.8;
	6816	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.7;
	6817	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.5;
	6817	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.2;
	6817	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.2;
	6817	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.2;
	6817	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.9;
	6817	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.2;
	6818	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	389.3;
	6818	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.4;
	6818	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.5;
	6818	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.2;
	6818	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.9;
	6818	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.9;
	6819	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	386.1;
	6819	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.1;
	6819	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181.7;
	6819	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.3;
	6819	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.7;
	6819	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.5;
	6820	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	382.4;
	6820	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.3;
	6820	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.6;
	6820	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.3;
	6820	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.4;
	6820	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.8;
	6821	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409;
	6821	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.9;
	6821	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.8;
	6821	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.8;
	6821	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.9;
	6821	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.4;
	6822	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.3;
	6822	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.7;
	6822	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.3;
	6822	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.9;
	6822	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.6;
	6822	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.4;
	6823	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.4;
	6823	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141;
	6823	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.3;
	6823	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.3;
	6823	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.3;
	6823	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.5;
	6824	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.6;
	6824	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.7;
	6824	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.9;
	6824	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.2;
	6824	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	6824	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.1;
	6825	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.5;
	6825	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146;
	6825	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.6;
	6825	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.9;
	6825	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	6825	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.4;
	6826	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.4;
	6826	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.4;
	6826	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.4;
	6826	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.8;
	6826	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	6826	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.7;
	6827	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.1;
	6827	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.3;
	6827	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.5;
	6827	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233;
	6827	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	6827	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.7;
	6828	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.7;
	6828	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.9;
	6828	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.6;
	6828	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.8;
	6828	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	6828	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.3;
	6829	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.3;
	6829	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.7;
	6829	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.8;
	6829	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.8;
	6829	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	6829	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.3;
	6830	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.3;
	6830	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.9;
	6830	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.2;
	6830	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.9;
	6830	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	6830	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.4;
	6831	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.6;
	6831	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.3;
	6831	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.9;
	6831	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.9;
	6831	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	6831	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365;
	6832	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.3;
	6832	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.6;
	6832	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.3;
	6832	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.2;
	6832	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	6832	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.1;
	6833	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.1;
	6833	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.6;
	6833	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.6;
	6833	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.8;
	6833	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	6833	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.4;
	6834	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.3;
	6834	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.8;
	6834	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.1;
	6834	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.6;
	6834	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	6834	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.5;
	6835	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.1;
	6835	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.6;
	6835	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.3;
	6835	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.4;
	6835	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	6835	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.8;
	6836	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.3;
	6836	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.1;
	6836	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238;
	6836	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.2;
	6836	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	6836	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.7;
	6837	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.8;
	6837	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.6;
	6837	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.9;
	6837	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.9;
	6837	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	6837	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.2;
	6838	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.1;
	6838	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.8;
	6838	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.3;
	6838	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.9;
	6838	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	6838	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.9;
	6839	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.5;
	6839	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.9;
	6839	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.5;
	6839	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.1;
	6839	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.9;
	6839	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.8;
	6840	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428.2;
	6840	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.9;
	6840	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200;
	6840	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.4;
	6840	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.9;
	6840	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.8;
	6841	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.8;
	6841	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.1;
	6841	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.8;
	6841	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.7;
	6841	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.7;
	6841	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.2;
	6842	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.7;
	6842	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.4;
	6842	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.7;
	6842	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.7;
	6842	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53;
	6842	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.2;
	6843	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	377.9;
	6843	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.4;
	6843	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.2;
	6843	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.2;
	6843	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.1;
	6843	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.8;
	6844	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	384.7;
	6844	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.3;
	6844	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.5;
	6844	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.7;
	6844	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52;
	6844	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.2;
	6845	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.5;
	6845	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.1;
	6845	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.1;
	6845	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.6;
	6845	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.7;
	6845	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.2;
	6846	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	420.5;
	6846	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129;
	6846	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.1;
	6846	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.3;
	6846	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.4;
	6846	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.9;
	6847	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.5;
	6847	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.4;
	6847	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.6;
	6847	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.8;
	6847	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.2;
	6847	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.2;
	6848	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.2;
	6848	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.2;
	6848	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222;
	6848	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.6;
	6848	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	6848	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.7;
	6849	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.3;
	6849	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.5;
	6849	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.9;
	6849	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232;
	6849	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61;
	6849	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.9;
	6850	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.6;
	6850	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.7;
	6850	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.3;
	6850	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.4;
	6850	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	6850	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.6;
	6851	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.7;
	6851	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.2;
	6851	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.1;
	6851	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238;
	6851	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	6851	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.6;
	6852	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.7;
	6852	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.6;
	6852	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.1;
	6852	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.7;
	6852	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	6852	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.9;
	6853	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.6;
	6853	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.5;
	6853	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232;
	6853	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.8;
	6853	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	6853	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.2;
	6854	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.1;
	6854	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.2;
	6854	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.9;
	6854	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.4;
	6854	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	6854	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.5;
	6855	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.2;
	6855	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.6;
	6855	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.3;
	6855	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.9;
	6855	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	6855	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366;
	6856	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463;
	6856	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.3;
	6856	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.2;
	6856	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.5;
	6856	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	6856	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.4;
	6857	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.7;
	6857	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.5;
	6857	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.3;
	6857	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.7;
	6857	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	6857	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.4;
	6858	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.2;
	6858	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.8;
	6858	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.2;
	6858	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.2;
	6858	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	6858	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.8;
	6859	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.9;
	6859	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.9;
	6859	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.9;
	6859	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.3;
	6859	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	6859	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.3;
	6860	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.6;
	6860	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.8;
	6860	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.1;
	6860	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.2;
	6860	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	6860	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.3;
	6861	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.3;
	6861	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.3;
	6861	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.7;
	6861	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.5;
	6861	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	6861	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.2;
	6862	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428.5;
	6862	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.9;
	6862	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.6;
	6862	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.2;
	6862	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	6862	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.8;
	6863	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403;
	6863	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.7;
	6863	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.6;
	6863	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.9;
	6863	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.1;
	6863	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.5;
	6864	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	384.3;
	6864	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.3;
	6864	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.3;
	6864	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.1;
	6864	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.4;
	6864	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.6;
	6865	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	370;
	6865	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.9;
	6865	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.1;
	6865	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.6;
	6865	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.7;
	6865	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.2;
	6866	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.3;
	6866	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.5;
	6866	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.7;
	6866	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.7;
	6866	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.7;
	6866	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.5;
	6867	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.2;
	6867	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.4;
	6867	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.3;
	6867	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.5;
	6867	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.8;
	6867	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.4;
	6868	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.7;
	6868	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109;
	6868	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.8;
	6868	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.9;
	6868	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	50.7;
	6868	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.4;
	6869	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.5;
	6869	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.2;
	6869	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186;
	6869	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.1;
	6869	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.3;
	6869	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348;
	6870	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.3;
	6870	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.7;
	6870	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192;
	6870	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.7;
	6870	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.1;
	6870	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.6;
	6871	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.2;
	6871	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.6;
	6871	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.3;
	6871	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.2;
	6871	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54;
	6871	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.9;
	6872	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.3;
	6872	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.6;
	6872	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.4;
	6872	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.6;
	6872	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.5;
	6872	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.3;
	6873	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.2;
	6873	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.6;
	6873	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.2;
	6873	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.3;
	6873	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	6873	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.7;
	6874	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.3;
	6874	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.1;
	6874	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.8;
	6874	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.4;
	6874	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	6874	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.7;
	6875	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.4;
	6875	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.8;
	6875	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.2;
	6875	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.8;
	6875	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	6875	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.1;
	6876	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.6;
	6876	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136;
	6876	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.6;
	6876	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.4;
	6876	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	6876	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.5;
	6877	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.5;
	6877	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.8;
	6877	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.3;
	6877	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210;
	6877	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	6877	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.8;
	6878	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.7;
	6878	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.9;
	6878	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.9;
	6878	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.2;
	6878	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	6878	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.3;
	6879	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.5;
	6879	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.8;
	6879	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.2;
	6879	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.6;
	6879	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	6879	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.7;
	6880	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.4;
	6880	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.1;
	6880	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.7;
	6880	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208;
	6880	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	6880	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.8;
	6881	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.9;
	6881	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.3;
	6881	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.6;
	6881	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.1;
	6881	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	6881	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.9;
	6882	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.3;
	6882	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.9;
	6882	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.4;
	6882	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.7;
	6882	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	6882	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.1;
	6883	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.6;
	6883	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.6;
	6883	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.7;
	6883	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.4;
	6883	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	6883	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.5;
	6884	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.3;
	6884	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.2;
	6884	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.5;
	6884	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223;
	6884	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	6884	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.8;
	6885	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.8;
	6885	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.1;
	6885	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.7;
	6885	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219;
	6885	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	6885	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.1;
	6886	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474;
	6886	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.7;
	6886	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.9;
	6886	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.9;
	6886	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.3;
	6886	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.8;
	6887	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.6;
	6887	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.9;
	6887	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.5;
	6887	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.2;
	6887	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.2;
	6887	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.9;
	6888	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.7;
	6888	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118;
	6888	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.5;
	6888	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.2;
	6888	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.3;
	6888	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.4;
	6889	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.5;
	6889	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.9;
	6889	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.3;
	6889	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.8;
	6889	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.4;
	6889	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.6;
	6890	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	327.6;
	6890	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	105.9;
	6890	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183;
	6890	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.9;
	6890	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.1;
	6890	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	292.8;
	6891	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	316.1;
	6891	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	104.5;
	6891	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181;
	6891	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	183;
	6891	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.4;
	6891	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	287.8;
	6892	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	322.9;
	6892	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	105.4;
	6892	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.6;
	6892	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	182.6;
	6892	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.3;
	6892	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	290.6;
	6893	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	326.9;
	6893	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.6;
	6893	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181.5;
	6893	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	183.7;
	6893	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.3;
	6893	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	293.2;
	6894	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	347.4;
	6894	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.1;
	6894	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.1;
	6894	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.5;
	6894	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.1;
	6894	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	303.1;
	6895	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	359.4;
	6895	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.6;
	6895	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.4;
	6895	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.7;
	6895	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	6895	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.9;
	6896	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	363.8;
	6896	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.7;
	6896	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.3;
	6896	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.4;
	6896	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.3;
	6896	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311.3;
	6897	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	381.6;
	6897	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.1;
	6897	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.9;
	6897	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.2;
	6897	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.1;
	6897	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.7;
	6898	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	384.9;
	6898	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.6;
	6898	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.6;
	6898	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.1;
	6898	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	6898	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.5;
	6899	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.6;
	6899	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.2;
	6899	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.4;
	6899	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.8;
	6899	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	6899	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321;
	6900	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	384.9;
	6900	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.2;
	6900	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196;
	6900	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.4;
	6900	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	6900	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.4;
	6901	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.5;
	6901	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.9;
	6901	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.8;
	6901	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201;
	6901	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	6901	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.8;
	6902	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379.3;
	6902	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127;
	6902	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.5;
	6902	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.1;
	6902	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	6902	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.9;
	6903	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	371.3;
	6903	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126;
	6903	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.8;
	6903	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.4;
	6903	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	6903	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.9;
	6904	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	372.6;
	6904	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.9;
	6904	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.7;
	6904	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.5;
	6904	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	6904	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.5;
	6905	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	370.6;
	6905	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.2;
	6905	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.9;
	6905	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.5;
	6905	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	6905	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.2;
	6906	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	374.1;
	6906	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.7;
	6906	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.8;
	6906	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.9;
	6906	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	6906	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.8;
	6907	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	390;
	6907	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.3;
	6907	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.3;
	6907	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.4;
	6907	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	6907	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.2;
	6908	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	400.5;
	6908	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.2;
	6908	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.2;
	6908	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.6;
	6908	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	6908	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.8;
	6909	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	400.2;
	6909	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.3;
	6909	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.1;
	6909	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.9;
	6909	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	6909	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.9;
	6910	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	376.5;
	6910	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.4;
	6910	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210;
	6910	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.2;
	6910	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.9;
	6910	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.5;
	6911	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	352.5;
	6911	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.2;
	6911	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.4;
	6911	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.1;
	6911	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.8;
	6911	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309.6;
	6912	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	342.9;
	6912	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.8;
	6912	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.7;
	6912	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.3;
	6912	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.1;
	6912	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	303.7;
	6913	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	336.7;
	6913	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.5;
	6913	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.7;
	6913	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.6;
	6913	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.9;
	6913	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	298.6;
	6914	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	326.4;
	6914	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	104.5;
	6914	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.7;
	6914	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	182.9;
	6914	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53;
	6914	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	292.1;
	6915	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	321.8;
	6915	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	103.7;
	6915	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	179.5;
	6915	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	181.7;
	6915	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.1;
	6915	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	289.8;
	6916	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	321.9;
	6916	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	104.9;
	6916	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.4;
	6916	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	183;
	6916	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.5;
	6916	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	290.2;
	6917	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	342.7;
	6917	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.3;
	6917	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.3;
	6917	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186;
	6917	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.5;
	6917	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	299.8;
	6918	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	350.2;
	6918	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.9;
	6918	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191;
	6918	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.2;
	6918	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52;
	6918	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	304.9;
	6919	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	385.6;
	6919	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.5;
	6919	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.1;
	6919	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.3;
	6919	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.7;
	6919	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.5;
	6920	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.1;
	6920	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.5;
	6920	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.3;
	6920	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225;
	6920	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.9;
	6920	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.9;
	6921	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.3;
	6921	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144;
	6921	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.4;
	6921	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.9;
	6921	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.8;
	6921	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.8;
	6922	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.2;
	6922	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.5;
	6922	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.8;
	6922	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229;
	6922	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.6;
	6922	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.4;
	6923	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.3;
	6923	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.2;
	6923	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.2;
	6923	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.7;
	6923	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	6923	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.7;
	6924	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455;
	6924	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.5;
	6924	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.9;
	6924	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.4;
	6924	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.3;
	6924	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.7;
	6925	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.9;
	6925	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.5;
	6925	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.4;
	6925	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.8;
	6925	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	6925	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.1;
	6926	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.8;
	6926	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.9;
	6926	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.1;
	6926	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.7;
	6926	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	6926	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.6;
	6927	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.4;
	6927	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.4;
	6927	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.2;
	6927	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.5;
	6927	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.3;
	6927	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.4;
	6928	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.8;
	6928	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.9;
	6928	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.3;
	6928	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.4;
	6928	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.4;
	6928	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.9;
	6929	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.1;
	6929	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.9;
	6929	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.5;
	6929	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.2;
	6929	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	6929	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.2;
	6930	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.9;
	6930	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.3;
	6930	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.5;
	6930	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.2;
	6930	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	6930	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.2;
	6931	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.1;
	6931	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154;
	6931	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.3;
	6931	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.2;
	6931	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.8;
	6931	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.6;
	6932	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471;
	6932	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.2;
	6932	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.2;
	6932	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.1;
	6932	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	6932	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.4;
	6933	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.3;
	6933	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.4;
	6933	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.9;
	6933	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.1;
	6933	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	6933	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.1;
	6934	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.1;
	6934	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.4;
	6934	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.1;
	6934	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.2;
	6934	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.1;
	6934	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.1;
	6935	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.5;
	6935	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.2;
	6935	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.6;
	6935	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.3;
	6935	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.1;
	6935	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.1;
	6936	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.5;
	6936	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.4;
	6936	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.6;
	6936	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.9;
	6936	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56;
	6936	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327;
	6937	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	366.2;
	6937	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.4;
	6937	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.8;
	6937	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.9;
	6937	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.8;
	6937	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311.3;
	6938	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	356;
	6938	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.2;
	6938	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.6;
	6938	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.4;
	6938	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.5;
	6938	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	305.4;
	6939	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	367.4;
	6939	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.5;
	6939	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.8;
	6939	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.8;
	6939	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.3;
	6939	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309.6;
	6940	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	372.1;
	6940	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.7;
	6940	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.6;
	6940	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.6;
	6940	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	50.8;
	6940	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311.7;
	6941	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	382.1;
	6941	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.2;
	6941	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.4;
	6941	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.9;
	6941	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	50.8;
	6941	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.9;
	6942	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.5;
	6942	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.2;
	6942	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.1;
	6942	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201;
	6942	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.4;
	6942	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.1;
	6943	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440;
	6943	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.7;
	6943	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.7;
	6943	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217;
	6943	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.1;
	6943	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348;
	6944	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.5;
	6944	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.2;
	6944	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.3;
	6944	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.4;
	6944	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.6;
	6944	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368;
	6945	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.9;
	6945	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.2;
	6945	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.1;
	6945	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.6;
	6945	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.2;
	6945	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.9;
	6946	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.3;
	6946	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146;
	6946	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.6;
	6946	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.6;
	6946	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.4;
	6946	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.6;
	6947	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471;
	6947	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.5;
	6947	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.1;
	6947	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.6;
	6947	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	6947	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.8;
	6948	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.5;
	6948	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146;
	6948	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.2;
	6948	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.6;
	6948	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.4;
	6948	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.6;
	6949	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.1;
	6949	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.9;
	6949	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.1;
	6949	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.8;
	6949	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.9;
	6949	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.1;
	6950	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.3;
	6950	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.7;
	6950	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.4;
	6950	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.1;
	6950	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	6950	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.3;
	6951	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.8;
	6951	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.3;
	6951	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.8;
	6951	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.6;
	6951	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.8;
	6951	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.3;
	6952	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.3;
	6952	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.6;
	6952	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.4;
	6952	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.9;
	6952	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	6952	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.1;
	6953	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464;
	6953	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.3;
	6953	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.4;
	6953	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.6;
	6953	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	6953	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368;
	6954	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.6;
	6954	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.6;
	6954	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.8;
	6954	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.7;
	6954	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	6954	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.5;
	6955	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470;
	6955	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.3;
	6955	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.2;
	6955	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.1;
	6955	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	6955	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371;
	6956	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.5;
	6956	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157;
	6956	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.4;
	6956	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.5;
	6956	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.6;
	6956	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.2;
	6957	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.2;
	6957	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148;
	6957	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.1;
	6957	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.8;
	6957	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	6957	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.8;
	6958	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442.1;
	6958	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.9;
	6958	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.8;
	6958	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.9;
	6958	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.2;
	6958	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.2;
	6959	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	420.6;
	6959	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.3;
	6959	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.3;
	6959	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.4;
	6959	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.7;
	6959	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.7;
	6960	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.7;
	6960	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.9;
	6960	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202;
	6960	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.7;
	6960	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.9;
	6960	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.7;
	6961	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	386;
	6961	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.8;
	6961	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.1;
	6961	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.4;
	6961	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.2;
	6961	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320;
	6962	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	368.8;
	6962	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109;
	6962	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.8;
	6962	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.7;
	6962	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.6;
	6962	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310.2;
	6963	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	366;
	6963	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.9;
	6963	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.6;
	6963	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.4;
	6963	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	50.7;
	6963	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.6;
	6964	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	371.6;
	6964	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110;
	6964	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.7;
	6964	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.4;
	6964	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	50.8;
	6964	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311.5;
	6965	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	376.7;
	6965	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.3;
	6965	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.3;
	6965	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.5;
	6965	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	50.2;
	6965	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.4;
	6966	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.9;
	6966	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129;
	6966	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.2;
	6966	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.6;
	6966	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.6;
	6966	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.7;
	6967	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.4;
	6967	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.2;
	6967	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.3;
	6967	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.6;
	6967	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.6;
	6967	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.9;
	6968	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.1;
	6968	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147;
	6968	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.2;
	6968	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230;
	6968	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.4;
	6968	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.2;
	6969	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.2;
	6969	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.5;
	6969	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.5;
	6969	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.7;
	6969	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.1;
	6969	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.5;
	6970	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.4;
	6970	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148;
	6970	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.3;
	6970	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.9;
	6970	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	6970	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.4;
	6971	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458;
	6971	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.5;
	6971	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.3;
	6971	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.1;
	6971	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	6971	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.3;
	6972	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.3;
	6972	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.8;
	6972	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.3;
	6972	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.6;
	6972	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	6972	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.4;
	6973	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458;
	6973	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.6;
	6973	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.6;
	6973	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.7;
	6973	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	6973	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.3;
	6974	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.1;
	6974	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.1;
	6974	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.8;
	6974	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.1;
	6974	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	6974	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.6;
	6975	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.2;
	6975	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.6;
	6975	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.7;
	6975	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.6;
	6975	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.3;
	6975	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.2;
	6976	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.1;
	6976	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.6;
	6976	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.1;
	6976	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.5;
	6976	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	6976	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.5;
	6977	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450;
	6977	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.9;
	6977	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.7;
	6977	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.3;
	6977	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.8;
	6977	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.6;
	6978	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.2;
	6978	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.6;
	6978	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241;
	6978	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.9;
	6978	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	6978	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.4;
	6979	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.5;
	6979	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.3;
	6979	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.2;
	6979	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249;
	6979	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	6979	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.1;
	6980	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.1;
	6980	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.3;
	6980	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.6;
	6980	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.6;
	6980	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.5;
	6980	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.6;
	6981	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.7;
	6981	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.8;
	6981	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.3;
	6981	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.1;
	6981	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69;
	6981	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362;
	6982	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.4;
	6982	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.4;
	6982	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.7;
	6982	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.1;
	6982	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	6982	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.3;
	6983	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421;
	6983	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.4;
	6983	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.4;
	6983	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.9;
	6983	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	6983	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.7;
	6984	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.1;
	6984	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.6;
	6984	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.8;
	6984	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.3;
	6984	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.9;
	6984	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.4;
	6985	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	389;
	6985	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.1;
	6985	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.8;
	6985	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.8;
	6985	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.6;
	6985	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.5;
	6986	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	366.6;
	6986	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.9;
	6986	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.5;
	6986	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.6;
	6986	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.5;
	6986	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309.8;
	6987	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	360;
	6987	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.9;
	6987	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.1;
	6987	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.2;
	6987	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.4;
	6987	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	307;
	6988	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	368.6;
	6988	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.6;
	6988	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.5;
	6988	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.7;
	6988	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.3;
	6988	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311;
	6989	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	381.7;
	6989	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.5;
	6989	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.5;
	6989	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.7;
	6989	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.5;
	6989	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.1;
	6990	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419;
	6990	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.5;
	6990	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.1;
	6990	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.6;
	6990	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.8;
	6990	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.8;
	6991	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.7;
	6991	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.8;
	6991	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219;
	6991	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223;
	6991	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.8;
	6991	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.7;
	6992	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.8;
	6992	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.1;
	6992	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.7;
	6992	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.1;
	6992	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	6992	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.6;
	6993	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.1;
	6993	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.1;
	6993	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.2;
	6993	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.2;
	6993	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.7;
	6993	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.2;
	6994	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.3;
	6994	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.3;
	6994	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.5;
	6994	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.9;
	6994	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	6994	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.9;
	6995	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.4;
	6995	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146;
	6995	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.8;
	6995	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.2;
	6995	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	6995	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.5;
	6996	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.4;
	6996	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.4;
	6996	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.5;
	6996	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.4;
	6996	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	6996	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.8;
	6997	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.5;
	6997	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.5;
	6997	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.7;
	6997	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.8;
	6997	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.2;
	6997	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.7;
	6998	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.4;
	6998	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.5;
	6998	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237;
	6998	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.5;
	6998	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.9;
	6998	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.8;
	6999	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.9;
	6999	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.7;
	6999	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.8;
	6999	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.2;
	6999	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.3;
	6999	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.1;
	7000	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451;
	7000	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.4;
	7000	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.8;
	7000	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.8;
	7000	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73;
	7000	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.5;
	7001	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.5;
	7001	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.8;
	7001	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.4;
	7001	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.3;
	7001	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.9;
	7001	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.2;
	7002	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.5;
	7002	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.6;
	7002	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.9;
	7002	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.6;
	7002	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.9;
	7002	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.4;
	7003	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.6;
	7003	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.4;
	7003	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244;
	7003	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.7;
	7003	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.8;
	7003	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.7;
	7004	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.1;
	7004	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152;
	7004	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.6;
	7004	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.7;
	7004	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.3;
	7004	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.6;
	7005	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.2;
	7005	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.4;
	7005	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.7;
	7005	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.5;
	7005	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.1;
	7005	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.2;
	7006	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.7;
	7006	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.3;
	7006	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.5;
	7006	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224;
	7006	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.1;
	7006	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355;
	7007	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414;
	7007	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.3;
	7007	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.7;
	7007	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.2;
	7007	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	7007	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.3;
	7008	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394.3;
	7008	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.7;
	7008	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.5;
	7008	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.7;
	7008	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.7;
	7008	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.4;
	7009	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	380.8;
	7009	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.2;
	7009	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.6;
	7009	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.3;
	7009	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	7009	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.4;
	7010	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.6;
	7010	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.8;
	7010	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185;
	7010	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.9;
	7010	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.1;
	7010	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.4;
	7011	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.8;
	7011	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.1;
	7011	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.6;
	7011	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.6;
	7011	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	7011	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.3;
	7012	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.9;
	7012	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.7;
	7012	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.2;
	7012	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.4;
	7012	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.4;
	7012	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.2;
	7013	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.4;
	7013	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.8;
	7013	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.2;
	7013	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.4;
	7013	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53;
	7013	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.7;
	7014	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537.7;
	7014	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126;
	7014	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.4;
	7014	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.3;
	7014	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.7;
	7014	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.6;
	7015	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572;
	7015	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.8;
	7015	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.1;
	7015	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.6;
	7015	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.7;
	7015	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.8;
	7016	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	590.1;
	7016	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.8;
	7016	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225;
	7016	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.8;
	7016	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.8;
	7016	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.2;
	7017	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588.1;
	7017	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.7;
	7017	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.1;
	7017	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.3;
	7017	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	7017	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.9;
	7018	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.9;
	7018	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.1;
	7018	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.4;
	7018	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.7;
	7018	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	7018	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.7;
	7019	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566.5;
	7019	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.2;
	7019	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.2;
	7019	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.2;
	7019	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	7019	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406;
	7020	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559.5;
	7020	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143;
	7020	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.7;
	7020	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.7;
	7020	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.2;
	7020	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.3;
	7021	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	553.7;
	7021	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.9;
	7021	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.6;
	7021	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.7;
	7021	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.6;
	7021	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.8;
	7022	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.8;
	7022	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.9;
	7022	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.9;
	7022	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.8;
	7022	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.4;
	7022	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.2;
	7023	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.3;
	7023	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.5;
	7023	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.3;
	7023	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.5;
	7023	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.8;
	7023	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.6;
	7024	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	563.6;
	7024	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.2;
	7024	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.7;
	7024	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.6;
	7024	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.9;
	7024	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.1;
	7025	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552.2;
	7025	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.9;
	7025	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.2;
	7025	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.5;
	7025	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.2;
	7025	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.9;
	7026	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	554.7;
	7026	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142;
	7026	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.1;
	7026	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.1;
	7026	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78;
	7026	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.7;
	7027	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564.1;
	7027	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.7;
	7027	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.3;
	7027	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.7;
	7027	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.7;
	7027	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406;
	7028	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	569.8;
	7028	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.9;
	7028	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.7;
	7028	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.8;
	7028	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.9;
	7028	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.5;
	7029	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	571.3;
	7029	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.2;
	7029	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.2;
	7029	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.1;
	7029	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.5;
	7029	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.4;
	7030	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	554.5;
	7030	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.2;
	7030	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.8;
	7030	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.7;
	7030	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.4;
	7030	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.1;
	7031	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	533.8;
	7031	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.2;
	7031	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.1;
	7031	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.8;
	7031	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.3;
	7031	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.4;
	7032	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.4;
	7032	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.5;
	7032	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.9;
	7032	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.1;
	7032	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	7032	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.8;
	7033	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526.1;
	7033	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.6;
	7033	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190;
	7033	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.8;
	7033	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	7033	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.7;
	7034	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	365.3;
	7034	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	107.3;
	7034	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.6;
	7034	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.4;
	7034	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.6;
	7034	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308;
	7035	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	350.1;
	7035	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.7;
	7035	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.5;
	7035	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.5;
	7035	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.7;
	7035	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	301.4;
	7036	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	358.3;
	7036	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	106.9;
	7036	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.4;
	7036	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.3;
	7036	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.9;
	7036	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	305.4;
	7037	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	375.2;
	7037	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.3;
	7037	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.7;
	7037	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.6;
	7037	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.8;
	7037	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.1;
	7038	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.1;
	7038	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.6;
	7038	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.5;
	7038	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.2;
	7038	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.9;
	7038	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.4;
	7039	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	402.1;
	7039	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.7;
	7039	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.9;
	7039	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.7;
	7039	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.4;
	7039	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.5;
	7040	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.5;
	7040	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.1;
	7040	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198;
	7040	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.7;
	7040	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.9;
	7040	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333;
	7041	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.4;
	7041	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.6;
	7041	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.7;
	7041	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.1;
	7041	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	7041	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.3;
	7042	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.4;
	7042	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.1;
	7042	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.9;
	7042	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.7;
	7042	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	7042	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.5;
	7043	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.2;
	7043	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.6;
	7043	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.5;
	7043	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.2;
	7043	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	7043	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342;
	7044	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.9;
	7044	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.9;
	7044	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.2;
	7044	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.9;
	7044	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	7044	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.3;
	7045	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.6;
	7045	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.2;
	7045	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207;
	7045	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.6;
	7045	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.8;
	7045	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339;
	7046	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.4;
	7046	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.2;
	7046	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.8;
	7046	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.2;
	7046	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.2;
	7046	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.1;
	7047	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.7;
	7047	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131;
	7047	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.4;
	7047	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.2;
	7047	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.9;
	7047	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.8;
	7048	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.5;
	7048	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.3;
	7048	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.2;
	7048	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.8;
	7048	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.5;
	7048	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.1;
	7049	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.5;
	7049	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.1;
	7049	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.3;
	7049	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.2;
	7049	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.7;
	7049	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.8;
	7050	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.4;
	7050	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.2;
	7050	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.2;
	7050	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.4;
	7050	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.4;
	7050	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.3;
	7051	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.3;
	7051	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.1;
	7051	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.9;
	7051	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.4;
	7051	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76;
	7051	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.3;
	7052	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428.7;
	7052	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.4;
	7052	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.7;
	7052	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222;
	7052	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.7;
	7052	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.6;
	7053	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.2;
	7053	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.3;
	7053	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.5;
	7053	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.6;
	7053	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.1;
	7053	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.3;
	7054	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	395.1;
	7054	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.7;
	7054	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.4;
	7054	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.5;
	7054	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	7054	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.2;
	7055	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	374.2;
	7055	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.4;
	7055	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.6;
	7055	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.3;
	7055	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	7055	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.3;
	7056	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	356.7;
	7056	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.3;
	7056	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.1;
	7056	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.6;
	7056	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	7056	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309.8;
	7057	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	346.2;
	7057	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.8;
	7057	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.8;
	7057	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.6;
	7057	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	7057	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	302.7;
	7058	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	336.5;
	7058	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.4;
	7058	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182;
	7058	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	183.9;
	7058	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.7;
	7058	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	296.5;
	7059	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	340.4;
	7059	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.8;
	7059	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.5;
	7059	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	182.5;
	7059	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.9;
	7059	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	297.6;
	7060	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	344;
	7060	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.3;
	7060	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.3;
	7060	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	182.3;
	7060	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.6;
	7060	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	299.2;
	7061	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	355.5;
	7061	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.5;
	7061	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.9;
	7061	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	183;
	7061	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.4;
	7061	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	304.5;
	7062	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	368.6;
	7062	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.7;
	7062	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.4;
	7062	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.6;
	7062	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.9;
	7062	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311.4;
	7063	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	376.3;
	7063	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.5;
	7063	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.9;
	7063	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190;
	7063	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.2;
	7063	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.9;
	7064	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	389.9;
	7064	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.5;
	7064	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189;
	7064	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.8;
	7064	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.1;
	7064	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.1;
	7065	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.7;
	7065	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.5;
	7065	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192;
	7065	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.6;
	7065	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	7065	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.5;
	7066	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.4;
	7066	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.4;
	7066	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.8;
	7066	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.4;
	7066	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	7066	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325;
	7067	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	395.9;
	7067	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.5;
	7067	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.9;
	7067	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.3;
	7067	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	7067	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.1;
	7068	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	390.9;
	7068	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.6;
	7068	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.4;
	7068	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.9;
	7068	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.5;
	7068	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.2;
	7069	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	395.5;
	7069	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.4;
	7069	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.5;
	7069	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.7;
	7069	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	7069	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.6;
	7070	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	396.6;
	7070	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.5;
	7070	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196;
	7070	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.4;
	7070	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.4;
	7070	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.4;
	7071	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394.1;
	7071	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.6;
	7071	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.7;
	7071	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.9;
	7071	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.8;
	7071	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.9;
	7072	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.1;
	7072	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.5;
	7072	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.2;
	7072	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.4;
	7072	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.8;
	7072	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.3;
	7073	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	401;
	7073	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.9;
	7073	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.3;
	7073	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.4;
	7073	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.8;
	7073	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.7;
	7074	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.4;
	7074	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.3;
	7074	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.7;
	7074	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.5;
	7074	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	7074	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.5;
	7075	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419;
	7075	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.9;
	7075	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.5;
	7075	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.6;
	7075	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.3;
	7075	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.9;
	7076	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.7;
	7076	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.1;
	7076	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.5;
	7076	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.2;
	7076	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	7076	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.5;
	7077	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.5;
	7077	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.2;
	7077	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220;
	7077	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.3;
	7077	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	7077	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.6;
	7078	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412;
	7078	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139;
	7078	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.6;
	7078	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.4;
	7078	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	7078	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.6;
	7079	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	385.3;
	7079	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.2;
	7079	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.3;
	7079	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.7;
	7079	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	7079	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.8;
	7080	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	369.5;
	7080	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.6;
	7080	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.6;
	7080	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.9;
	7080	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.6;
	7080	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.5;
	7081	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	334.5;
	7081	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.9;
	7081	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.8;
	7081	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.5;
	7081	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.8;
	7081	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	298.3;
	7082	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	349.6;
	7082	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114;
	7082	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183;
	7082	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.9;
	7082	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.7;
	7082	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	302.8;
	7083	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	353.5;
	7083	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.8;
	7083	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181.1;
	7083	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	183.1;
	7083	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.1;
	7083	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	303.6;
	7084	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	360.2;
	7084	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.7;
	7084	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.4;
	7084	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.2;
	7084	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51;
	7084	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	306.8;
	7085	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	372.9;
	7085	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.6;
	7085	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.8;
	7085	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188;
	7085	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	50.7;
	7085	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.7;
	7086	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	402.1;
	7086	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.5;
	7086	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.9;
	7086	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.4;
	7086	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51;
	7086	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.4;
	7087	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447.3;
	7087	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.6;
	7087	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.9;
	7087	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.4;
	7087	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.2;
	7087	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.6;
	7088	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.9;
	7088	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.3;
	7088	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.1;
	7088	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.8;
	7088	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.6;
	7088	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.5;
	7089	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.3;
	7089	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.1;
	7089	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.4;
	7089	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.6;
	7089	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.6;
	7089	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373;
	7090	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.5;
	7090	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.2;
	7090	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.8;
	7090	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230;
	7090	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.1;
	7090	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.7;
	7091	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.9;
	7091	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.7;
	7091	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.8;
	7091	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.6;
	7091	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	7091	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.1;
	7092	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465;
	7092	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.2;
	7092	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.3;
	7092	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.2;
	7092	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	7092	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.3;
	7093	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.7;
	7093	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.4;
	7093	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.2;
	7093	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.3;
	7093	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	7093	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367;
	7094	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.3;
	7094	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.5;
	7094	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.5;
	7094	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.5;
	7094	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	7094	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364;
	7095	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.7;
	7095	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.9;
	7095	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.3;
	7095	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.7;
	7095	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.9;
	7095	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.7;
	7096	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.2;
	7096	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.6;
	7096	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.7;
	7096	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.9;
	7096	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.6;
	7096	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.9;
	7097	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.2;
	7097	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.8;
	7097	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.5;
	7097	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235;
	7097	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58;
	7097	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.7;
	7098	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.8;
	7098	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154;
	7098	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.3;
	7098	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.8;
	7098	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.2;
	7098	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.3;
	7099	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.5;
	7099	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.6;
	7099	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.2;
	7099	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245;
	7099	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	7099	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.8;
	7100	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.6;
	7100	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161;
	7100	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.7;
	7100	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.6;
	7100	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.3;
	7100	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.2;
	7101	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477;
	7101	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.2;
	7101	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.8;
	7101	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.7;
	7101	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.7;
	7101	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.3;
	7102	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.3;
	7102	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.3;
	7102	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.9;
	7102	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.3;
	7102	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.1;
	7102	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361;
	7103	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	427.8;
	7103	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.2;
	7103	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.1;
	7103	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.3;
	7103	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.4;
	7103	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.5;
	7104	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.2;
	7104	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.2;
	7104	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200;
	7104	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.1;
	7104	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.8;
	7104	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.8;
	7105	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	395;
	7105	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.8;
	7105	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.7;
	7105	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.6;
	7105	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.7;
	7105	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.2;
	7106	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	377.9;
	7106	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.3;
	7106	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182;
	7106	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.2;
	7106	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.5;
	7106	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.1;
	7107	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	364.4;
	7107	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111;
	7107	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.8;
	7107	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.8;
	7107	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.3;
	7107	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	307.5;
	7108	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	369.7;
	7108	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.7;
	7108	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.1;
	7108	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.3;
	7108	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.8;
	7108	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310.1;
	7109	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	380.3;
	7109	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119;
	7109	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.5;
	7109	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.9;
	7109	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.9;
	7109	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316;
	7110	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.4;
	7110	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.7;
	7110	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.1;
	7110	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.1;
	7110	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.2;
	7110	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.2;
	7111	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.6;
	7111	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.2;
	7111	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.4;
	7111	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.2;
	7111	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.9;
	7111	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.3;
	7112	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.2;
	7112	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151;
	7112	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.8;
	7112	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.2;
	7112	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.4;
	7112	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.6;
	7113	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.3;
	7113	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.1;
	7113	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.1;
	7113	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229;
	7113	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.2;
	7113	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.7;
	7114	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.8;
	7114	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.7;
	7114	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.4;
	7114	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.5;
	7114	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.4;
	7114	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.5;
	7115	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.6;
	7115	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.1;
	7115	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.9;
	7115	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.7;
	7115	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.1;
	7115	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.3;
	7116	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.7;
	7116	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.4;
	7116	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.1;
	7116	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.5;
	7116	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59;
	7116	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.7;
	7117	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473;
	7117	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.1;
	7117	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.7;
	7117	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234;
	7117	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	7117	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.4;
	7118	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.9;
	7118	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.2;
	7118	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.8;
	7118	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.5;
	7118	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	7118	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.4;
	7119	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.2;
	7119	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.6;
	7119	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.8;
	7119	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.4;
	7119	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	7119	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368;
	7120	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.1;
	7120	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.2;
	7120	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234;
	7120	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.6;
	7120	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	7120	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.1;
	7121	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.4;
	7121	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.2;
	7121	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.1;
	7121	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.9;
	7121	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.7;
	7121	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.9;
	7122	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.6;
	7122	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.9;
	7122	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.4;
	7122	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.4;
	7122	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58;
	7122	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.4;
	7123	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.5;
	7123	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157;
	7123	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.5;
	7123	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.7;
	7123	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59;
	7123	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.6;
	7124	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.9;
	7124	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.4;
	7124	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241;
	7124	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.8;
	7124	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60;
	7124	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381;
	7125	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.4;
	7125	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.7;
	7125	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.1;
	7125	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.7;
	7125	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	7125	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.6;
	7126	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.5;
	7126	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.6;
	7126	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.8;
	7126	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223;
	7126	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.2;
	7126	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.3;
	7127	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434;
	7127	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.3;
	7127	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.2;
	7127	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.2;
	7127	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.8;
	7127	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.1;
	7128	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.1;
	7128	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.3;
	7128	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.9;
	7128	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.8;
	7128	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.2;
	7128	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.6;
	7129	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394.5;
	7129	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	114.3;
	7129	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.5;
	7129	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.3;
	7129	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.7;
	7129	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.3;
	7130	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379.2;
	7130	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.5;
	7130	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.1;
	7130	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.8;
	7130	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.4;
	7130	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.5;
	7131	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	382.5;
	7131	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.9;
	7131	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.7;
	7131	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.6;
	7131	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	50.7;
	7131	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.7;
	7132	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	388.8;
	7132	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.2;
	7132	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.4;
	7132	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.4;
	7132	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	50.8;
	7132	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.7;
	7133	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.6;
	7133	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.5;
	7133	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.9;
	7133	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.1;
	7133	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	50.8;
	7133	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326;
	7134	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.8;
	7134	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.4;
	7134	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.4;
	7134	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.5;
	7134	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.3;
	7134	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.6;
	7135	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.1;
	7135	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.8;
	7135	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.6;
	7135	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.1;
	7135	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.1;
	7135	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.3;
	7136	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.3;
	7136	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.8;
	7136	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.4;
	7136	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.7;
	7136	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.9;
	7136	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.5;
	7137	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.1;
	7137	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148;
	7137	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.4;
	7137	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.7;
	7137	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	7137	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.7;
	7138	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.6;
	7138	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.8;
	7138	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.3;
	7138	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235;
	7138	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.3;
	7138	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.2;
	7139	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.7;
	7139	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.6;
	7139	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.2;
	7139	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.4;
	7139	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	7139	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.8;
	7140	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483;
	7140	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.7;
	7140	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.5;
	7140	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.5;
	7140	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	7140	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.2;
	7141	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.2;
	7141	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.3;
	7141	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.3;
	7141	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.5;
	7141	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	7141	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.1;
	7142	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.5;
	7142	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146;
	7142	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.3;
	7142	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.7;
	7142	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	7142	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.9;
	7143	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.8;
	7143	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.1;
	7143	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.1;
	7143	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.2;
	7143	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	7143	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.9;
	7144	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.5;
	7144	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.5;
	7144	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.5;
	7144	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.4;
	7144	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	7144	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.6;
	7145	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.2;
	7145	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.1;
	7145	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241;
	7145	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.9;
	7145	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	7145	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.5;
	7146	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.9;
	7146	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.5;
	7146	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.7;
	7146	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.1;
	7146	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	7146	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.6;
	7147	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.2;
	7147	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.4;
	7147	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.7;
	7147	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.6;
	7147	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	7147	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.4;
	7148	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.9;
	7148	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.4;
	7148	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.6;
	7148	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.6;
	7148	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	7148	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382;
	7149	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.6;
	7149	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.2;
	7149	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.2;
	7149	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.8;
	7149	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	7149	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376;
	7150	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.9;
	7150	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.6;
	7150	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.3;
	7150	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.6;
	7150	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	7150	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.4;
	7151	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.7;
	7151	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128;
	7151	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.6;
	7151	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208;
	7151	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	7151	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.5;
	7152	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.6;
	7152	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.3;
	7152	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.6;
	7152	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.7;
	7152	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.8;
	7152	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.3;
	7153	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.2;
	7153	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.5;
	7153	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.2;
	7153	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.1;
	7153	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54;
	7153	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.5;
	7154	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	385.3;
	7154	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.6;
	7154	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.1;
	7154	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186;
	7154	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.2;
	7154	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.5;
	7155	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.5;
	7155	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.4;
	7155	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.6;
	7155	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.7;
	7155	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.3;
	7155	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.4;
	7156	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	385.7;
	7156	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.8;
	7156	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.2;
	7156	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.3;
	7156	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	50.9;
	7156	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.6;
	7157	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.4;
	7157	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.4;
	7157	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.7;
	7157	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190;
	7157	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.3;
	7157	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.7;
	7158	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424;
	7158	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.7;
	7158	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.3;
	7158	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.5;
	7158	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.4;
	7158	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.3;
	7159	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.1;
	7159	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.6;
	7159	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215;
	7159	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.3;
	7159	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.7;
	7159	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.9;
	7160	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.2;
	7160	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.4;
	7160	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.4;
	7160	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.3;
	7160	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	7160	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.3;
	7161	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.8;
	7161	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.3;
	7161	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.6;
	7161	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.3;
	7161	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.2;
	7161	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.6;
	7162	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.5;
	7162	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.9;
	7162	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.8;
	7162	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.9;
	7162	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	7162	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.4;
	7163	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.6;
	7163	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.1;
	7163	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.3;
	7163	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.9;
	7163	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	7163	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.9;
	7164	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.2;
	7164	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.9;
	7164	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.8;
	7164	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.9;
	7164	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	7164	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.2;
	7165	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.3;
	7165	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.3;
	7165	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.6;
	7165	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.1;
	7165	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	7165	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.8;
	7166	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.8;
	7166	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.9;
	7166	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.3;
	7166	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.3;
	7166	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	7166	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368;
	7167	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469;
	7167	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146;
	7167	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.8;
	7167	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233;
	7167	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	7167	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.8;
	7168	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.2;
	7168	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.7;
	7168	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230;
	7168	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.6;
	7168	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	7168	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.3;
	7169	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.4;
	7169	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.8;
	7169	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.3;
	7169	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.2;
	7169	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	7169	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.8;
	7170	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.8;
	7170	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.3;
	7170	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.3;
	7170	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.3;
	7170	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	7170	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.1;
	7171	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.9;
	7171	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.2;
	7171	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.4;
	7171	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.6;
	7171	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	7171	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.9;
	7172	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.2;
	7172	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.6;
	7172	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.1;
	7172	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.8;
	7172	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	7172	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.9;
	7173	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.1;
	7173	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.8;
	7173	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.1;
	7173	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.5;
	7173	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	7173	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.1;
	7174	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.9;
	7174	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.2;
	7174	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.1;
	7174	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.3;
	7174	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	7174	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.5;
	7175	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.5;
	7175	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.1;
	7175	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.8;
	7175	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.8;
	7175	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	7175	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.3;
	7176	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.8;
	7176	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.1;
	7176	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.4;
	7176	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.2;
	7176	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	7176	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.9;
	7177	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	401.4;
	7177	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.5;
	7177	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189;
	7177	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.4;
	7177	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.1;
	7177	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326;
	7178	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	348.3;
	7178	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.1;
	7178	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.3;
	7178	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187;
	7178	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.1;
	7178	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	301.8;
	7179	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	350.1;
	7179	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.2;
	7179	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.5;
	7179	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.3;
	7179	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	7179	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	302.4;
	7180	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	360.8;
	7180	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.7;
	7180	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.1;
	7180	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.8;
	7180	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	7180	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	307.2;
	7181	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.4;
	7181	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.3;
	7181	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.2;
	7181	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.1;
	7181	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55;
	7181	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.3;
	7182	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419;
	7182	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.2;
	7182	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.1;
	7182	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204;
	7182	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.3;
	7182	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336;
	7183	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447.8;
	7183	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.2;
	7183	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.3;
	7183	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.1;
	7183	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.8;
	7183	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.5;
	7184	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459;
	7184	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.2;
	7184	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.2;
	7184	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.4;
	7184	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	7184	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.8;
	7185	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.3;
	7185	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.7;
	7185	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.7;
	7185	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233;
	7185	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	7185	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.2;
	7186	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.8;
	7186	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.1;
	7186	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.5;
	7186	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.2;
	7186	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.9;
	7186	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.1;
	7187	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.9;
	7187	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147;
	7187	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.9;
	7187	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.7;
	7187	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.4;
	7187	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.2;
	7188	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.9;
	7188	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.2;
	7188	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.6;
	7188	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.6;
	7188	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.2;
	7188	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.7;
	7189	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.6;
	7189	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.1;
	7189	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.6;
	7189	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.2;
	7189	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	7189	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.5;
	7190	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.8;
	7190	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.7;
	7190	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.2;
	7190	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.6;
	7190	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	7190	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.2;
	7191	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.3;
	7191	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.9;
	7191	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.2;
	7191	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.7;
	7191	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	7191	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.6;
	7192	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.6;
	7192	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.5;
	7192	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227;
	7192	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.2;
	7192	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	7192	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.5;
	7193	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	426.7;
	7193	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.9;
	7193	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.1;
	7193	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.5;
	7193	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	7193	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350;
	7194	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.2;
	7194	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.7;
	7194	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.2;
	7194	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.7;
	7194	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	7194	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.5;
	7195	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.2;
	7195	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.6;
	7195	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.6;
	7195	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.8;
	7195	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	7195	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.2;
	7196	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.3;
	7196	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.8;
	7196	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.7;
	7196	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.4;
	7196	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	7196	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.6;
	7197	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.5;
	7197	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.2;
	7197	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.2;
	7197	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.7;
	7197	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	7197	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.1;
	7198	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414;
	7198	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.9;
	7198	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.8;
	7198	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.2;
	7198	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	7198	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.9;
	7199	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392.7;
	7199	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.9;
	7199	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.6;
	7199	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.9;
	7199	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.2;
	7199	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.2;
	7200	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	367.9;
	7200	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.1;
	7200	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.2;
	7200	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.1;
	7200	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59;
	7200	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.3;
	7201	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	349.8;
	7201	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.2;
	7201	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.7;
	7201	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.1;
	7201	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.5;
	7201	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	306.2;
	7202	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	351.7;
	7202	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.3;
	7202	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.3;
	7202	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.6;
	7202	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.8;
	7202	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	303.9;
	7203	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	357.4;
	7203	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111;
	7203	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.5;
	7203	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.1;
	7203	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.1;
	7203	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	305.4;
	7204	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	365.5;
	7204	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	112.2;
	7204	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.2;
	7204	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.8;
	7204	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.4;
	7204	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.9;
	7205	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	377.5;
	7205	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.3;
	7205	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.6;
	7205	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.4;
	7205	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.9;
	7205	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.1;
	7206	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	386.9;
	7206	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.1;
	7206	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.5;
	7206	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.7;
	7206	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.9;
	7206	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.1;
	7207	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	395.9;
	7207	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.5;
	7207	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.6;
	7207	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.5;
	7207	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.2;
	7207	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.8;
	7208	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.9;
	7208	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.2;
	7208	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.7;
	7208	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.1;
	7208	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	7208	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.8;
	7209	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.3;
	7209	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.2;
	7209	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204;
	7209	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.5;
	7209	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	7209	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.2;
	7210	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417.5;
	7210	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.7;
	7210	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.8;
	7210	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.1;
	7210	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	7210	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.2;
	7211	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.2;
	7211	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.6;
	7211	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207;
	7211	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.6;
	7211	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	7211	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.1;
	7212	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.7;
	7212	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.5;
	7212	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.7;
	7212	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.3;
	7212	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	7212	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334;
	7213	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.1;
	7213	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.5;
	7213	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.5;
	7213	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.9;
	7213	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	7213	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.8;
	7214	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	406.9;
	7214	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.6;
	7214	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.1;
	7214	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.4;
	7214	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	7214	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.8;
	7215	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.1;
	7215	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.4;
	7215	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.9;
	7215	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.3;
	7215	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	7215	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.2;
	7216	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.8;
	7216	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.1;
	7216	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.9;
	7216	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.8;
	7216	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	7216	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.1;
	7217	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391.8;
	7217	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.5;
	7217	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205;
	7217	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.6;
	7217	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	7217	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.4;
	7218	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394.7;
	7218	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.2;
	7218	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.9;
	7218	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.1;
	7218	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	7218	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.3;
	7219	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.9;
	7219	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.2;
	7219	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.4;
	7219	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.7;
	7219	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	7219	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337;
	7220	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.1;
	7220	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.1;
	7220	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.5;
	7220	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.8;
	7220	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	7220	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.3;
	7221	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	402.6;
	7221	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.3;
	7221	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.2;
	7221	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.9;
	7221	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	7221	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.5;
	7222	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	380.1;
	7222	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.7;
	7222	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.2;
	7222	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.4;
	7222	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	7222	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.1;
	7223	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	359.8;
	7223	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.4;
	7223	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201;
	7223	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.2;
	7223	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	7223	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.8;
	7224	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	339.6;
	7224	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.6;
	7224	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.1;
	7224	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.9;
	7224	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.8;
	7224	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	303.2;
	7225	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	324.7;
	7225	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.2;
	7225	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.8;
	7225	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.3;
	7225	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.4;
	7225	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	294.7;
	7226	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	316.2;
	7226	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	110.9;
	7226	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181.4;
	7226	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	181.8;
	7226	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.5;
	7226	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	288.4;
	7227	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	316.3;
	7227	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	108.8;
	7227	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	179.3;
	7227	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	179.3;
	7227	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52;
	7227	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	287.7;
	7228	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	333.6;
	7228	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.3;
	7228	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	179.4;
	7228	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	179.7;
	7228	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.7;
	7228	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	294.6;
	7229	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	349.6;
	7229	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.7;
	7229	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.8;
	7229	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	181.4;
	7229	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.7;
	7229	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	302.1;
	7230	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	356.6;
	7230	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.1;
	7230	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.3;
	7230	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.1;
	7230	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.8;
	7230	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	306.5;
	7231	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	368;
	7231	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.4;
	7231	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.6;
	7231	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.8;
	7231	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.9;
	7231	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.4;
	7232	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	375.6;
	7232	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.9;
	7232	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.5;
	7232	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.3;
	7232	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.1;
	7232	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.9;
	7233	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	380.2;
	7233	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.6;
	7233	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.5;
	7233	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.2;
	7233	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	7233	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.3;
	7234	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	382.5;
	7234	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.3;
	7234	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.5;
	7234	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.7;
	7234	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	7234	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.1;
	7235	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	377.5;
	7235	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.9;
	7235	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198;
	7235	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.4;
	7235	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	7235	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.4;
	7236	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	377.3;
	7236	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.7;
	7236	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.7;
	7236	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.5;
	7236	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	7236	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.2;
	7237	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	384.6;
	7237	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138;
	7237	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.5;
	7237	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.7;
	7237	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	7237	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322;
	7238	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	377.7;
	7238	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.6;
	7238	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.1;
	7238	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.7;
	7238	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	7238	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.9;
	7239	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	373.7;
	7239	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.9;
	7239	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.8;
	7239	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.6;
	7239	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	7239	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.1;
	7240	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	376.4;
	7240	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.4;
	7240	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.9;
	7240	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.3;
	7240	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	7240	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.3;
	7241	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	373.3;
	7241	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.5;
	7241	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.8;
	7241	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204;
	7241	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.7;
	7241	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.5;
	7242	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	385.5;
	7242	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.6;
	7242	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.9;
	7242	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.6;
	7242	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	7242	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.9;
	7243	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.8;
	7243	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.9;
	7243	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.6;
	7243	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.4;
	7243	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.6;
	7243	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.8;
	7244	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.8;
	7244	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.6;
	7244	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.3;
	7244	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220;
	7244	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	7244	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.9;
	7245	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.8;
	7245	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.5;
	7245	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.2;
	7245	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.7;
	7245	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	7245	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.5;
	7246	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392.1;
	7246	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.1;
	7246	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.2;
	7246	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208;
	7246	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.6;
	7246	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.1;
	7247	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	361.5;
	7247	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.8;
	7247	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.1;
	7247	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.3;
	7247	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.4;
	7247	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.3;
	7248	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	358.7;
	7248	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.8;
	7248	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.7;
	7248	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193;
	7248	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.5;
	7248	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309.6;
	7249	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	350.3;
	7249	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.3;
	7249	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181.8;
	7249	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	180.8;
	7249	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.2;
	7249	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	303.3;
	7250	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	371.5;
	7250	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.6;
	7250	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.6;
	7250	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	180.5;
	7250	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56;
	7250	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309.4;
	7251	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	373;
	7251	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.4;
	7251	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.2;
	7251	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	179.7;
	7251	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.2;
	7251	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310;
	7252	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	369.8;
	7252	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.3;
	7252	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181;
	7252	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	180.6;
	7252	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.9;
	7252	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	309.2;
	7253	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	380.9;
	7253	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.4;
	7253	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.2;
	7253	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	183.8;
	7253	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.2;
	7253	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.8;
	7254	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.2;
	7254	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.1;
	7254	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.9;
	7254	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.9;
	7254	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.1;
	7254	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.1;
	7255	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.3;
	7255	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.5;
	7255	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.1;
	7255	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.9;
	7255	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.7;
	7255	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.4;
	7256	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.3;
	7256	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.6;
	7256	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.9;
	7256	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.1;
	7256	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	7256	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.1;
	7257	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.3;
	7257	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.2;
	7257	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.9;
	7257	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.6;
	7257	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61;
	7257	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.3;
	7258	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.6;
	7258	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.9;
	7258	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.4;
	7258	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.2;
	7258	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	7258	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.6;
	7259	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.6;
	7259	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.2;
	7259	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.8;
	7259	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222;
	7259	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	7259	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.8;
	7260	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.2;
	7260	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.6;
	7260	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.7;
	7260	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.3;
	7260	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	7260	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.6;
	7261	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.1;
	7261	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.9;
	7261	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.3;
	7261	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220;
	7261	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	7261	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.3;
	7262	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.4;
	7262	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.2;
	7262	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.4;
	7262	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.3;
	7262	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.3;
	7262	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.3;
	7263	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.7;
	7263	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.6;
	7263	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.4;
	7263	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.6;
	7263	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	7263	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.2;
	7264	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.6;
	7264	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.4;
	7264	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.3;
	7264	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.5;
	7264	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	7264	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.8;
	7265	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.9;
	7265	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.2;
	7265	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.4;
	7265	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.7;
	7265	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	7265	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.2;
	7266	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.9;
	7266	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.4;
	7266	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.3;
	7266	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.5;
	7266	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	7266	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.1;
	7267	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.8;
	7267	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160;
	7267	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.1;
	7267	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.6;
	7267	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.3;
	7267	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.2;
	7268	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.2;
	7268	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.7;
	7268	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.7;
	7268	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220;
	7268	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.3;
	7268	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.5;
	7269	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.2;
	7269	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.3;
	7269	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.1;
	7269	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.5;
	7269	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.4;
	7269	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.2;
	7270	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.1;
	7270	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143;
	7270	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.2;
	7270	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.8;
	7270	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.2;
	7270	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.4;
	7271	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.3;
	7271	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.4;
	7271	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.3;
	7271	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.8;
	7271	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	7271	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341;
	7272	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.8;
	7272	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.6;
	7272	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.4;
	7272	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.5;
	7272	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.5;
	7272	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.3;
	7273	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.1;
	7273	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.6;
	7273	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	173.6;
	7273	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	176.4;
	7273	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.7;
	7273	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325;
	7274	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	395.5;
	7274	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	109.3;
	7274	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	172.9;
	7274	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	175.7;
	7274	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53;
	7274	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.1;
	7275	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.1;
	7275	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	111.7;
	7275	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	171.1;
	7275	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	173.4;
	7275	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.2;
	7275	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.7;
	7276	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.6;
	7276	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113;
	7276	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	171.9;
	7276	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	174.4;
	7276	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.1;
	7276	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.1;
	7277	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.2;
	7277	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.2;
	7277	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	175.9;
	7277	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	178.3;
	7277	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.1;
	7277	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.8;
	7278	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.8;
	7278	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.9;
	7278	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.4;
	7278	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187;
	7278	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.1;
	7278	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.1;
	7279	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.5;
	7279	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.3;
	7279	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196;
	7279	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199;
	7279	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.8;
	7279	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.4;
	7280	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502;
	7280	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149;
	7280	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.3;
	7280	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.1;
	7280	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.9;
	7280	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.9;
	7281	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.2;
	7281	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.6;
	7281	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.7;
	7281	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.1;
	7281	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54;
	7281	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.1;
	7282	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.2;
	7282	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.4;
	7282	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.7;
	7282	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.4;
	7282	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.4;
	7282	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.5;
	7283	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.4;
	7283	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.3;
	7283	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.6;
	7283	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209;
	7283	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.6;
	7283	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.1;
	7284	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.4;
	7284	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.4;
	7284	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.9;
	7284	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.2;
	7284	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.1;
	7284	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362;
	7285	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.8;
	7285	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.6;
	7285	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.9;
	7285	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.1;
	7285	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.6;
	7285	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.1;
	7286	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.6;
	7286	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.9;
	7286	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.8;
	7286	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.9;
	7286	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60;
	7286	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.5;
	7287	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.9;
	7287	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.2;
	7287	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.7;
	7287	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.7;
	7287	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.1;
	7287	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.7;
	7288	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.1;
	7288	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.7;
	7288	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.2;
	7288	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.3;
	7288	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	7288	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.6;
	7289	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.1;
	7289	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.4;
	7289	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.1;
	7289	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.7;
	7289	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	7289	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.9;
	7290	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.1;
	7290	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.4;
	7290	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.4;
	7290	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.8;
	7290	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.2;
	7290	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.1;
	7291	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.9;
	7291	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.2;
	7291	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.8;
	7291	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.7;
	7291	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61;
	7291	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.4;
	7292	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.6;
	7292	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165;
	7292	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.2;
	7292	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.7;
	7292	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	7292	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.1;
	7293	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.4;
	7293	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.9;
	7293	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.9;
	7293	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.9;
	7293	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.3;
	7293	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.8;
	7294	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.3;
	7294	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.5;
	7294	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.6;
	7294	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.9;
	7294	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	7294	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.3;
	7295	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	427.7;
	7295	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.7;
	7295	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.3;
	7295	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.1;
	7295	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	7295	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.4;
	7296	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.5;
	7296	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.4;
	7296	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.3;
	7296	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.5;
	7296	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.5;
	7296	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324;
	7297	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403.9;
	7297	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.3;
	7297	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	177.3;
	7297	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	180.2;
	7297	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.6;
	7297	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.5;
	7298	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	381.4;
	7298	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.7;
	7298	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	173.6;
	7298	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	175.4;
	7298	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.8;
	7298	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310.1;
	7299	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	378.6;
	7299	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.9;
	7299	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	172.7;
	7299	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	175.2;
	7299	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51;
	7299	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.8;
	7300	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	388.8;
	7300	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.6;
	7300	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	173.7;
	7300	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	176.1;
	7300	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	50.4;
	7300	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	313.7;
	7301	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403;
	7301	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.1;
	7301	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	178.3;
	7301	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	181;
	7301	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	50.4;
	7301	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.1;
	7302	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.6;
	7302	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.2;
	7302	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.3;
	7302	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190;
	7302	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	50.9;
	7302	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.2;
	7303	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.4;
	7303	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.1;
	7303	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.2;
	7303	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.7;
	7303	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.5;
	7303	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.2;
	7304	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.6;
	7304	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.5;
	7304	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.9;
	7304	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.3;
	7304	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.5;
	7304	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.9;
	7305	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.2;
	7305	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.8;
	7305	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.4;
	7305	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.9;
	7305	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.1;
	7305	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.6;
	7306	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.2;
	7306	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.1;
	7306	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.8;
	7306	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.5;
	7306	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	7306	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.1;
	7307	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.6;
	7307	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.3;
	7307	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.5;
	7307	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.8;
	7307	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	7307	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.1;
	7308	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.5;
	7308	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.7;
	7308	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.9;
	7308	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.2;
	7308	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	7308	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.4;
	7309	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465;
	7309	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.4;
	7309	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.1;
	7309	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.9;
	7309	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	7309	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.2;
	7310	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.3;
	7310	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151;
	7310	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.6;
	7310	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.5;
	7310	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	7310	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.8;
	7311	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.8;
	7311	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.6;
	7311	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.6;
	7311	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.3;
	7311	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	7311	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.9;
	7312	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.8;
	7312	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.2;
	7312	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.2;
	7312	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.9;
	7312	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.3;
	7312	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.3;
	7313	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.9;
	7313	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.1;
	7313	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.5;
	7313	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.6;
	7313	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	7313	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.8;
	7314	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.9;
	7314	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160;
	7314	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.1;
	7314	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.2;
	7314	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	7314	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367;
	7315	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.2;
	7315	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.9;
	7315	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.4;
	7315	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.6;
	7315	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	7315	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.9;
	7316	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.5;
	7316	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.9;
	7316	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221;
	7316	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.7;
	7316	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	7316	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.8;
	7317	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.8;
	7317	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.1;
	7317	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.6;
	7317	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217;
	7317	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.7;
	7317	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.3;
	7318	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.5;
	7318	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.2;
	7318	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.9;
	7318	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.1;
	7318	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	7318	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.4;
	7319	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.2;
	7319	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.1;
	7319	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.4;
	7319	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.7;
	7319	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	7319	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.3;
	7320	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.4;
	7320	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.6;
	7320	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188;
	7320	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.5;
	7320	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	7320	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.9;
	7321	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.8;
	7321	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.7;
	7321	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	178.1;
	7321	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	180.9;
	7321	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.2;
	7321	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.9;
	7322	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	388.2;
	7322	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.1;
	7322	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	175.8;
	7322	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	178.1;
	7322	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.5;
	7322	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.4;
	7323	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	389.6;
	7323	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.7;
	7323	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	174.1;
	7323	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	177;
	7323	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.7;
	7323	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.6;
	7324	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.3;
	7324	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.7;
	7324	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	175.6;
	7324	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	178.5;
	7324	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.7;
	7324	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.2;
	7325	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.8;
	7325	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.8;
	7325	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	179;
	7325	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	182.2;
	7325	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.4;
	7325	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.5;
	7326	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.1;
	7326	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.6;
	7326	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.8;
	7326	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.7;
	7326	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.5;
	7326	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.8;
	7327	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.5;
	7327	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.6;
	7327	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.2;
	7327	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.6;
	7327	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	7327	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356;
	7328	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.7;
	7328	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155;
	7328	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.6;
	7328	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.5;
	7328	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	7328	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.3;
	7329	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.5;
	7329	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.4;
	7329	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.2;
	7329	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.1;
	7329	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	7329	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.6;
	7330	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.8;
	7330	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.8;
	7330	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.8;
	7330	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.1;
	7330	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	7330	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.5;
	7331	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.3;
	7331	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.6;
	7331	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.7;
	7331	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.9;
	7331	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.1;
	7331	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.1;
	7332	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.7;
	7332	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.7;
	7332	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.2;
	7332	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.1;
	7332	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	7332	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.6;
	7333	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.8;
	7333	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.5;
	7333	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219;
	7333	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.6;
	7333	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	7333	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.9;
	7334	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.4;
	7334	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.5;
	7334	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.1;
	7334	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.4;
	7334	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	7334	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.8;
	7335	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.8;
	7335	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.1;
	7335	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.8;
	7335	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.8;
	7335	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	7335	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.4;
	7336	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.9;
	7336	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.3;
	7336	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.3;
	7336	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.5;
	7336	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	7336	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360;
	7337	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.7;
	7337	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.1;
	7337	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.2;
	7337	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.8;
	7337	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	7337	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.7;
	7338	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.8;
	7338	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.6;
	7338	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.8;
	7338	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.1;
	7338	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	7338	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.1;
	7339	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.2;
	7339	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.8;
	7339	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.9;
	7339	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.6;
	7339	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	7339	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.4;
	7340	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.1;
	7340	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163;
	7340	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.1;
	7340	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.9;
	7340	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	7340	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376;
	7341	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.8;
	7341	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.9;
	7341	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.5;
	7341	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.1;
	7341	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	7341	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.5;
	7342	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459;
	7342	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.4;
	7342	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206;
	7342	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.2;
	7342	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	7342	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.7;
	7343	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442.9;
	7343	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.1;
	7343	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195;
	7343	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.8;
	7343	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	7343	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.7;
	7344	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428.4;
	7344	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.1;
	7344	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.6;
	7344	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.1;
	7344	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.2;
	7344	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.3;
	7345	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.1;
	7345	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.8;
	7345	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	179.4;
	7345	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	182.8;
	7345	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.4;
	7345	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.6;
	7346	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394.7;
	7346	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.6;
	7346	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	175.7;
	7346	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	178.7;
	7346	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.2;
	7346	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317;
	7347	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391;
	7347	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.5;
	7347	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	174.4;
	7347	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	177.7;
	7347	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.6;
	7347	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.2;
	7348	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391.5;
	7348	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.2;
	7348	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	176;
	7348	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	179.3;
	7348	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.1;
	7348	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.1;
	7349	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.7;
	7349	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.2;
	7349	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	179.3;
	7349	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	182.8;
	7349	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53;
	7349	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.6;
	7350	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.8;
	7350	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.7;
	7350	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.6;
	7350	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.4;
	7350	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.9;
	7350	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.5;
	7351	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.8;
	7351	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152;
	7351	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.6;
	7351	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.3;
	7351	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.8;
	7351	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.4;
	7352	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.6;
	7352	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.8;
	7352	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.8;
	7352	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.9;
	7352	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.2;
	7352	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.4;
	7353	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.1;
	7353	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.2;
	7353	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.6;
	7353	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.1;
	7353	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	7353	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.5;
	7354	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501;
	7354	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157;
	7354	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.8;
	7354	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222;
	7354	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	7354	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.6;
	7355	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.1;
	7355	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.5;
	7355	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.5;
	7355	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219;
	7355	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	7355	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.8;
	7356	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.6;
	7356	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.7;
	7356	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.9;
	7356	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.3;
	7356	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	7356	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.2;
	7357	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.1;
	7357	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.7;
	7357	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.6;
	7357	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.7;
	7357	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	7357	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.9;
	7358	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.1;
	7358	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.2;
	7358	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.5;
	7358	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.9;
	7358	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	7358	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.2;
	7359	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.4;
	7359	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.8;
	7359	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.8;
	7359	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.1;
	7359	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	7359	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.7;
	7360	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.6;
	7360	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.9;
	7360	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.5;
	7360	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.7;
	7360	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	7360	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.8;
	7361	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.1;
	7361	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147;
	7361	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.4;
	7361	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218;
	7361	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	7361	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.9;
	7362	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.5;
	7362	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.9;
	7362	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.5;
	7362	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222;
	7362	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	7362	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.8;
	7363	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.7;
	7363	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.8;
	7363	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.8;
	7363	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.8;
	7363	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	7363	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.7;
	7364	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.2;
	7364	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.8;
	7364	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220;
	7364	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.9;
	7364	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	7364	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.4;
	7365	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.2;
	7365	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.7;
	7365	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.1;
	7365	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.3;
	7365	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	7365	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.7;
	7366	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447.7;
	7366	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.4;
	7366	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.7;
	7366	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.2;
	7366	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	7366	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.1;
	7367	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417.6;
	7367	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.4;
	7367	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.1;
	7367	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.8;
	7367	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	7367	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.5;
	7368	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.6;
	7368	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.5;
	7368	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.2;
	7368	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.9;
	7368	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57;
	7368	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330;
	7369	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.5;
	7369	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.3;
	7369	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.2;
	7369	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.3;
	7369	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.2;
	7369	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.2;
	7370	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	374.7;
	7370	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.9;
	7370	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	176.7;
	7370	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	180.3;
	7370	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.3;
	7370	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310.5;
	7371	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	377.3;
	7371	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116.5;
	7371	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	177.3;
	7371	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	181;
	7371	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52;
	7371	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311.2;
	7372	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.9;
	7372	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.2;
	7372	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	177.2;
	7372	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	180.8;
	7372	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.6;
	7372	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314;
	7373	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.9;
	7373	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.6;
	7373	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	179.7;
	7373	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	183.8;
	7373	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.6;
	7373	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.9;
	7374	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409;
	7374	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.5;
	7374	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.8;
	7374	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.4;
	7374	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.4;
	7374	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.8;
	7375	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.4;
	7375	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.1;
	7375	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190;
	7375	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.5;
	7375	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.3;
	7375	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.5;
	7376	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.5;
	7376	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.2;
	7376	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.5;
	7376	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.3;
	7376	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.9;
	7376	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.1;
	7377	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435;
	7377	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.9;
	7377	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.1;
	7377	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.9;
	7377	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	7377	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.8;
	7378	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.1;
	7378	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.9;
	7378	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.2;
	7378	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.4;
	7378	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	7378	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.5;
	7379	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.5;
	7379	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.9;
	7379	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.1;
	7379	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.8;
	7379	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	7379	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.8;
	7380	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.7;
	7380	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.5;
	7380	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.1;
	7380	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.9;
	7380	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	7380	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.5;
	7381	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.3;
	7381	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.3;
	7381	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.1;
	7381	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.6;
	7381	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	7381	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.4;
	7382	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	427.1;
	7382	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.3;
	7382	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.7;
	7382	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.1;
	7382	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	7382	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.2;
	7383	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.2;
	7383	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.9;
	7383	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.7;
	7383	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.9;
	7383	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	7383	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.1;
	7384	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.9;
	7384	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.5;
	7384	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.2;
	7384	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.3;
	7384	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	7384	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.2;
	7385	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	430.9;
	7385	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.7;
	7385	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.8;
	7385	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.3;
	7385	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	7385	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.4;
	7386	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.1;
	7386	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.4;
	7386	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.3;
	7386	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.9;
	7386	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	7386	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.6;
	7387	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.5;
	7387	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.1;
	7387	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.7;
	7387	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.9;
	7387	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	7387	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.2;
	7388	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.7;
	7388	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.2;
	7388	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.9;
	7388	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.3;
	7388	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	7388	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.8;
	7389	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.6;
	7389	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.7;
	7389	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.9;
	7389	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.7;
	7389	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	7389	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.6;
	7390	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.6;
	7390	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.9;
	7390	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.1;
	7390	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.5;
	7390	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	7390	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.3;
	7391	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.5;
	7391	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.7;
	7391	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.3;
	7391	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.9;
	7391	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	7391	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.1;
	7392	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.4;
	7392	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.8;
	7392	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.3;
	7392	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.3;
	7392	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.1;
	7392	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.9;
	7393	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	384.5;
	7393	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118;
	7393	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.6;
	7393	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.9;
	7393	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.1;
	7393	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.4;
	7394	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	375.8;
	7394	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.1;
	7394	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.5;
	7394	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.6;
	7394	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.3;
	7394	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311.9;
	7395	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	369.8;
	7395	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	113.8;
	7395	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	175.6;
	7395	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	178.4;
	7395	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.3;
	7395	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	307.9;
	7396	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	372.7;
	7396	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.1;
	7396	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	176.4;
	7396	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	180.4;
	7396	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51;
	7396	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	308.9;
	7397	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	374.8;
	7397	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118;
	7397	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.2;
	7397	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	183.9;
	7397	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	50.5;
	7397	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311.1;
	7398	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383;
	7398	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.4;
	7398	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181.5;
	7398	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.7;
	7398	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.1;
	7398	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.5;
	7399	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.3;
	7399	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.6;
	7399	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.7;
	7399	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.9;
	7399	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.2;
	7399	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.8;
	7400	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.3;
	7400	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.9;
	7400	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.6;
	7400	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.4;
	7400	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.6;
	7400	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.3;
	7401	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	430.8;
	7401	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.9;
	7401	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.2;
	7401	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.4;
	7401	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	7401	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.2;
	7402	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.8;
	7402	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.6;
	7402	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.1;
	7402	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.7;
	7402	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	7402	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.7;
	7403	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.7;
	7403	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.9;
	7403	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.8;
	7403	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.6;
	7403	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	7403	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.3;
	7404	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	420.1;
	7404	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.9;
	7404	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.6;
	7404	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.4;
	7404	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	7404	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.9;
	7405	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417;
	7405	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.8;
	7405	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.5;
	7405	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.8;
	7405	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	7405	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333;
	7406	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.2;
	7406	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.8;
	7406	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.1;
	7406	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.7;
	7406	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	7406	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.8;
	7407	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418;
	7407	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.2;
	7407	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.1;
	7407	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.2;
	7407	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	7407	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334;
	7408	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.9;
	7408	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.7;
	7408	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.2;
	7408	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.6;
	7408	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	7408	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.9;
	7409	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	427.2;
	7409	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.4;
	7409	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.2;
	7409	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.5;
	7409	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.3;
	7409	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.5;
	7410	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.5;
	7410	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.9;
	7410	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.6;
	7410	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.9;
	7410	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	7410	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.8;
	7411	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.1;
	7411	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.9;
	7411	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.9;
	7411	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.5;
	7411	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.6;
	7411	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.9;
	7412	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.9;
	7412	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159;
	7412	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.7;
	7412	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.4;
	7412	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	7412	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363;
	7413	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.6;
	7413	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154;
	7413	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.7;
	7413	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.8;
	7413	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	7413	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.4;
	7414	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.8;
	7414	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.5;
	7414	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207;
	7414	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.9;
	7414	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	7414	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.8;
	7415	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.4;
	7415	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.9;
	7415	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.5;
	7415	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.8;
	7415	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	7415	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.1;
	7416	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.9;
	7416	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.5;
	7416	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.1;
	7416	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.2;
	7416	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.5;
	7416	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.9;
	7417	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	366.2;
	7417	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	105.7;
	7417	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	168.9;
	7417	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	174.1;
	7417	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.6;
	7417	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	306.3;
	7418	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	388.4;
	7418	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.1;
	7418	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.5;
	7418	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.7;
	7418	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55;
	7418	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.3;
	7419	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392.8;
	7419	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.9;
	7419	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181.4;
	7419	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.1;
	7419	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.8;
	7419	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.2;
	7420	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.1;
	7420	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.3;
	7420	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.5;
	7420	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.5;
	7420	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.9;
	7420	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.9;
	7421	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.5;
	7421	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.3;
	7421	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.5;
	7421	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.1;
	7421	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.5;
	7421	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.1;
	7422	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.7;
	7422	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.4;
	7422	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.1;
	7422	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.2;
	7422	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.5;
	7422	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.2;
	7423	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.6;
	7423	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.9;
	7423	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.6;
	7423	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.7;
	7423	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.4;
	7423	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.6;
	7424	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518.1;
	7424	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.8;
	7424	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.2;
	7424	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.9;
	7424	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	7424	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.9;
	7425	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.8;
	7425	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163;
	7425	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.9;
	7425	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.5;
	7425	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	7425	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.1;
	7426	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.4;
	7426	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.1;
	7426	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.8;
	7426	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.4;
	7426	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	7426	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.9;
	7427	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.5;
	7427	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.2;
	7427	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.7;
	7427	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.6;
	7427	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	7427	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.7;
	7428	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496;
	7428	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.2;
	7428	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.9;
	7428	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.6;
	7428	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	7428	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.4;
	7429	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495;
	7429	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.1;
	7429	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.9;
	7429	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224;
	7429	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	7429	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.3;
	7430	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.9;
	7430	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.3;
	7430	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220;
	7430	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.2;
	7430	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	7430	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.6;
	7431	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.2;
	7431	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.1;
	7431	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222;
	7431	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.7;
	7431	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	7431	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.2;
	7432	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.5;
	7432	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.8;
	7432	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.2;
	7432	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.6;
	7432	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59;
	7432	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368;
	7433	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489;
	7433	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.9;
	7433	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.6;
	7433	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.2;
	7433	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	7433	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.6;
	7434	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.7;
	7434	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170;
	7434	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.5;
	7434	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.4;
	7434	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.6;
	7434	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.5;
	7435	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515;
	7435	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.4;
	7435	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.7;
	7435	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.6;
	7435	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59;
	7435	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.2;
	7436	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.4;
	7436	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.6;
	7436	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.8;
	7436	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.7;
	7436	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	7436	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.6;
	7437	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.7;
	7437	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.6;
	7437	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.9;
	7437	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.8;
	7437	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	7437	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.2;
	7438	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.6;
	7438	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.5;
	7438	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.2;
	7438	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219;
	7438	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.2;
	7438	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.4;
	7439	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.9;
	7439	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.5;
	7439	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.2;
	7439	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.9;
	7439	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	7439	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.1;
	7440	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425;
	7440	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.5;
	7440	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.3;
	7440	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.2;
	7440	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	7440	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.7;
	7441	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.6;
	7441	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137;
	7441	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.8;
	7441	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.5;
	7441	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.2;
	7441	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.8;
	7442	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.9;
	7442	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.6;
	7442	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.2;
	7442	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	182.3;
	7442	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.9;
	7442	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.2;
	7443	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.2;
	7443	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.8;
	7443	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185;
	7443	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.4;
	7443	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.4;
	7443	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.4;
	7444	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.8;
	7444	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.7;
	7444	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.1;
	7444	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.2;
	7444	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.1;
	7444	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.5;
	7445	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.5;
	7445	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.5;
	7445	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.5;
	7445	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.2;
	7445	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.5;
	7445	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.4;
	7446	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.5;
	7446	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.8;
	7446	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.6;
	7446	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.9;
	7446	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.9;
	7446	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.6;
	7447	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.9;
	7447	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.7;
	7447	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.5;
	7447	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.3;
	7447	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.4;
	7447	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.9;
	7448	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.7;
	7448	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.3;
	7448	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.7;
	7448	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.8;
	7448	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.9;
	7448	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.1;
	7449	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.8;
	7449	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.5;
	7449	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.2;
	7449	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.8;
	7449	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.4;
	7449	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.4;
	7450	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.9;
	7450	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.9;
	7450	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.5;
	7450	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.9;
	7450	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.9;
	7450	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.8;
	7451	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.6;
	7451	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.8;
	7451	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.1;
	7451	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.1;
	7451	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	7451	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.5;
	7452	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.3;
	7452	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152;
	7452	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.4;
	7452	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.8;
	7452	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.8;
	7452	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.4;
	7453	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.3;
	7453	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.7;
	7453	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.3;
	7453	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.5;
	7453	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.9;
	7453	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.2;
	7454	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.4;
	7454	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.1;
	7454	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.2;
	7454	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.1;
	7454	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.4;
	7454	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.7;
	7455	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.1;
	7455	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.3;
	7455	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.8;
	7455	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.2;
	7455	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	7455	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363;
	7456	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.6;
	7456	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.5;
	7456	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.9;
	7456	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.1;
	7456	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.1;
	7456	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.6;
	7457	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.8;
	7457	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160;
	7457	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.7;
	7457	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.7;
	7457	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.5;
	7457	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.8;
	7458	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.7;
	7458	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.7;
	7458	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.5;
	7458	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.5;
	7458	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59;
	7458	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379;
	7459	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.8;
	7459	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.3;
	7459	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.1;
	7459	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.9;
	7459	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	7459	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382;
	7460	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.3;
	7460	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.7;
	7460	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.6;
	7460	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.4;
	7460	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.4;
	7460	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.9;
	7461	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.6;
	7461	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.4;
	7461	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.3;
	7461	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.5;
	7461	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	7461	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.2;
	7462	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.1;
	7462	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.7;
	7462	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216;
	7462	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.7;
	7462	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.6;
	7462	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.1;
	7463	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	444.3;
	7463	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136;
	7463	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.1;
	7463	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.1;
	7463	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.3;
	7463	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.1;
	7464	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	420.7;
	7464	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.7;
	7464	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.3;
	7464	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.1;
	7464	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58;
	7464	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.3;
	7465	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.4;
	7465	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.1;
	7465	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.6;
	7465	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.1;
	7465	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.5;
	7465	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.9;
	7466	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.2;
	7466	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.6;
	7466	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.1;
	7466	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	182.5;
	7466	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.7;
	7466	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.5;
	7467	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	401.9;
	7467	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.1;
	7467	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.9;
	7467	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.2;
	7467	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.7;
	7467	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.1;
	7468	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409;
	7468	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.2;
	7468	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.6;
	7468	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.1;
	7468	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.9;
	7468	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.7;
	7469	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	430.3;
	7469	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.4;
	7469	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.7;
	7469	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.3;
	7469	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.9;
	7469	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.8;
	7470	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.8;
	7470	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.3;
	7470	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.1;
	7470	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.6;
	7470	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57;
	7470	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.8;
	7471	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.2;
	7471	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.7;
	7471	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212;
	7471	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.8;
	7471	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	7471	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368;
	7472	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.3;
	7472	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.3;
	7472	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.2;
	7472	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.2;
	7472	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	7472	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.3;
	7473	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.2;
	7473	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.5;
	7473	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.5;
	7473	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.6;
	7473	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	7473	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.4;
	7474	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.9;
	7474	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.3;
	7474	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.5;
	7474	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.6;
	7474	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	7474	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.7;
	7475	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.7;
	7475	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.9;
	7475	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.7;
	7475	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230;
	7475	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.2;
	7475	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371;
	7476	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475;
	7476	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.4;
	7476	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.4;
	7476	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.4;
	7476	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	7476	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.2;
	7477	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.4;
	7477	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.7;
	7477	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229;
	7477	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.9;
	7477	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	7477	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.9;
	7478	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.7;
	7478	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.6;
	7478	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.8;
	7478	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.5;
	7478	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	7478	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.6;
	7479	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.1;
	7479	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.1;
	7479	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.7;
	7479	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.3;
	7479	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	7479	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.6;
	7480	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.7;
	7480	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.1;
	7480	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.2;
	7480	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.5;
	7480	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	7480	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.2;
	7481	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.9;
	7481	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.5;
	7481	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.2;
	7481	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.8;
	7481	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	7481	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.3;
	7482	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.4;
	7482	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.5;
	7482	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242;
	7482	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.3;
	7482	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	7482	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.5;
	7483	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.7;
	7483	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.8;
	7483	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.8;
	7483	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.6;
	7483	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	7483	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.2;
	7484	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.7;
	7484	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.4;
	7484	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.9;
	7484	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.3;
	7484	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	7484	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.7;
	7485	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.6;
	7485	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.9;
	7485	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.7;
	7485	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.7;
	7485	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.4;
	7485	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.2;
	7486	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.1;
	7486	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.7;
	7486	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216;
	7486	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.3;
	7486	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	7486	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.5;
	7487	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.5;
	7487	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.1;
	7487	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.4;
	7487	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205;
	7487	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	7487	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.1;
	7488	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.4;
	7488	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.1;
	7488	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.9;
	7488	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.5;
	7488	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	7488	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.4;
	7489	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.5;
	7489	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.3;
	7489	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.3;
	7489	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.3;
	7489	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	7489	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.5;
	7490	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	388.8;
	7490	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	116;
	7490	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.1;
	7490	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.3;
	7490	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	7490	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318;
	7491	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	390.9;
	7491	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	115.7;
	7491	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.7;
	7491	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185;
	7491	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.9;
	7491	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.8;
	7492	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.9;
	7492	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117;
	7492	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.9;
	7492	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.2;
	7492	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	7492	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.5;
	7493	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.5;
	7493	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.5;
	7493	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189;
	7493	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.7;
	7493	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	7493	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.2;
	7494	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	444.4;
	7494	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.4;
	7494	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.5;
	7494	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.7;
	7494	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	7494	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.7;
	7495	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475;
	7495	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.4;
	7495	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.7;
	7495	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.7;
	7495	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	7495	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.4;
	7496	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.8;
	7496	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.8;
	7496	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.8;
	7496	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.6;
	7496	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.8;
	7496	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.6;
	7497	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.1;
	7497	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152;
	7497	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.1;
	7497	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.4;
	7497	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.5;
	7497	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.5;
	7498	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.7;
	7498	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153;
	7498	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.4;
	7498	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.5;
	7498	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	7498	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.3;
	7499	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.1;
	7499	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.8;
	7499	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.9;
	7499	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.5;
	7499	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	7499	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.4;
	7500	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.7;
	7500	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.3;
	7500	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.3;
	7500	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235;
	7500	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	7500	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.9;
	7501	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.8;
	7501	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.1;
	7501	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.3;
	7501	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.2;
	7501	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	7501	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.4;
	7502	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.8;
	7502	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.8;
	7502	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.6;
	7502	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.6;
	7502	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	7502	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.6;
	7503	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.8;
	7503	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.7;
	7503	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.2;
	7503	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.2;
	7503	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	7503	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.6;
	7504	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.2;
	7504	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.6;
	7504	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.4;
	7504	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.2;
	7504	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	7504	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.6;
	7505	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483;
	7505	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.1;
	7505	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.8;
	7505	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.6;
	7505	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	7505	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.9;
	7506	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.1;
	7506	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.1;
	7506	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.2;
	7506	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.6;
	7506	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	7506	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.7;
	7507	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.5;
	7507	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.5;
	7507	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240;
	7507	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.3;
	7507	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	7507	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.3;
	7508	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.3;
	7508	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.8;
	7508	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.8;
	7508	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.7;
	7508	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	7508	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.5;
	7509	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.5;
	7509	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.7;
	7509	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.2;
	7509	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.2;
	7509	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	7509	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.6;
	7510	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467;
	7510	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.6;
	7510	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.3;
	7510	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.4;
	7510	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	7510	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.5;
	7511	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.6;
	7511	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.4;
	7511	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.2;
	7511	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.2;
	7511	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.4;
	7511	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.2;
	7512	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.3;
	7512	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.5;
	7512	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200;
	7512	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.2;
	7512	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.3;
	7512	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.7;
	7513	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.3;
	7513	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121;
	7513	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.4;
	7513	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.9;
	7513	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.8;
	7513	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.4;
	7514	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	400.1;
	7514	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.7;
	7514	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.4;
	7514	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.6;
	7514	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.8;
	7514	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323;
	7515	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394.9;
	7515	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.6;
	7515	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.8;
	7515	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.7;
	7515	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.5;
	7515	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.7;
	7516	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.6;
	7516	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.4;
	7516	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.9;
	7516	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.1;
	7516	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.4;
	7516	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.4;
	7517	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.6;
	7517	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.5;
	7517	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.8;
	7517	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.1;
	7517	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54;
	7517	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.8;
	7518	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449;
	7518	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.4;
	7518	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.6;
	7518	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.4;
	7518	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.2;
	7518	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.2;
	7519	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479;
	7519	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.5;
	7519	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.2;
	7519	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.7;
	7519	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57;
	7519	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.8;
	7520	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.2;
	7520	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.3;
	7520	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.5;
	7520	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223;
	7520	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.9;
	7520	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.6;
	7521	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.7;
	7521	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.4;
	7521	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.2;
	7521	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.2;
	7521	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	7521	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.5;
	7522	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.3;
	7522	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153;
	7522	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.5;
	7522	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.3;
	7522	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	7522	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.9;
	7523	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.5;
	7523	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.5;
	7523	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.9;
	7523	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.4;
	7523	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	7523	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.8;
	7524	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.2;
	7524	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.8;
	7524	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.4;
	7524	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229;
	7524	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	7524	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.2;
	7525	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.3;
	7525	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.2;
	7525	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.6;
	7525	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.2;
	7525	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	7525	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.7;
	7526	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.4;
	7526	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.9;
	7526	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.8;
	7526	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.2;
	7526	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	7526	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.9;
	7527	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.6;
	7527	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.9;
	7527	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.8;
	7527	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.9;
	7527	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	7527	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.9;
	7528	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.5;
	7528	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.3;
	7528	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.2;
	7528	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.5;
	7528	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	7528	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.8;
	7529	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.4;
	7529	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.4;
	7529	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.4;
	7529	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.2;
	7529	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	7529	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.5;
	7530	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.4;
	7530	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.7;
	7530	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.7;
	7530	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242;
	7530	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	7530	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.5;
	7531	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.6;
	7531	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.6;
	7531	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.7;
	7531	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.5;
	7531	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	7531	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.7;
	7532	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.5;
	7532	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.8;
	7532	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.8;
	7532	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.1;
	7532	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	7532	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.8;
	7533	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.4;
	7533	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.9;
	7533	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.1;
	7533	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.4;
	7533	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	7533	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.7;
	7534	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.1;
	7534	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.1;
	7534	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.7;
	7534	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.4;
	7534	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	7534	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.8;
	7535	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.2;
	7535	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.7;
	7535	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.6;
	7535	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.6;
	7535	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.7;
	7535	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.9;
	7536	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425;
	7536	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.9;
	7536	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.8;
	7536	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.3;
	7536	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	7536	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.5;
	7537	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.4;
	7537	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.4;
	7537	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.7;
	7537	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.8;
	7537	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.4;
	7537	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332;
	7538	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	400.4;
	7538	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.3;
	7538	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.5;
	7538	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.7;
	7538	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.2;
	7538	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.8;
	7539	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.4;
	7539	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.1;
	7539	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.2;
	7539	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.6;
	7539	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.2;
	7539	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.9;
	7540	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.8;
	7540	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.7;
	7540	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.1;
	7540	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.7;
	7540	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.8;
	7540	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.1;
	7541	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423;
	7541	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.8;
	7541	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.8;
	7541	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.4;
	7541	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.9;
	7541	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.9;
	7542	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.4;
	7542	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.6;
	7542	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.2;
	7542	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.3;
	7542	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.1;
	7542	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.9;
	7543	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.4;
	7543	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.3;
	7543	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.2;
	7543	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200;
	7543	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	7543	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.1;
	7544	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.9;
	7544	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141;
	7544	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.5;
	7544	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.5;
	7544	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	7544	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.2;
	7545	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.1;
	7545	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.5;
	7545	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.6;
	7545	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.6;
	7545	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	7545	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.2;
	7546	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.9;
	7546	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.7;
	7546	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.5;
	7546	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210;
	7546	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	7546	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.9;
	7547	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.5;
	7547	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.2;
	7547	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.5;
	7547	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.8;
	7547	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	7547	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.3;
	7548	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442.2;
	7548	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.1;
	7548	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.2;
	7548	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.8;
	7548	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	7548	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.4;
	7549	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.5;
	7549	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145;
	7549	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.1;
	7549	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.7;
	7549	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	7549	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.8;
	7550	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.9;
	7550	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142;
	7550	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.2;
	7550	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.7;
	7550	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	7550	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.5;
	7551	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.8;
	7551	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141;
	7551	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.4;
	7551	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.6;
	7551	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	7551	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.6;
	7552	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417.2;
	7552	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.7;
	7552	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.8;
	7552	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.4;
	7552	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	7552	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.7;
	7553	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.1;
	7553	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.4;
	7553	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.5;
	7553	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.9;
	7553	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.8;
	7553	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343;
	7554	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.9;
	7554	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.2;
	7554	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.4;
	7554	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.7;
	7554	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	7554	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.2;
	7555	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.9;
	7555	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.2;
	7555	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.6;
	7555	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.3;
	7555	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	7555	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.6;
	7556	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.6;
	7556	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.5;
	7556	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.8;
	7556	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.6;
	7556	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	7556	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.4;
	7557	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.8;
	7557	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146;
	7557	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.9;
	7557	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.4;
	7557	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	7557	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.9;
	7558	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.5;
	7558	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.2;
	7558	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.5;
	7558	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.8;
	7558	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	7558	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.5;
	7559	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399;
	7559	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.6;
	7559	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.1;
	7559	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.6;
	7559	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	7559	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328;
	7560	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	384.1;
	7560	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.3;
	7560	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.3;
	7560	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.1;
	7560	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.4;
	7560	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.5;
	7561	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	374.7;
	7561	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	121.1;
	7561	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.9;
	7561	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.2;
	7561	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.6;
	7561	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.6;
	7562	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	374.3;
	7562	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118;
	7562	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.9;
	7562	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.5;
	7562	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.7;
	7562	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312;
	7563	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	373.8;
	7563	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	118.8;
	7563	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.7;
	7563	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.3;
	7563	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.2;
	7563	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311.7;
	7564	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	374.3;
	7564	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	117.4;
	7564	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	180.3;
	7564	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	183.1;
	7564	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.3;
	7564	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	311.5;
	7565	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.3;
	7565	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.2;
	7565	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.8;
	7565	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.8;
	7565	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.1;
	7565	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316;
	7566	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.7;
	7566	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.9;
	7566	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.1;
	7566	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.2;
	7566	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.9;
	7566	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.3;
	7567	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407;
	7567	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.7;
	7567	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.7;
	7567	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.5;
	7567	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	7567	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.5;
	7568	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.3;
	7568	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.2;
	7568	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.5;
	7568	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.5;
	7568	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.2;
	7568	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.3;
	7569	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.8;
	7569	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.5;
	7569	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.9;
	7569	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.1;
	7569	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	7569	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.9;
	7570	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428.9;
	7570	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.7;
	7570	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193;
	7570	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.4;
	7570	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	7570	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.5;
	7571	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428.6;
	7571	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.2;
	7571	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.7;
	7571	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196;
	7571	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	7571	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.4;
	7572	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422;
	7572	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.6;
	7572	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.1;
	7572	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.3;
	7572	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	7572	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.3;
	7573	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421;
	7573	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.5;
	7573	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.4;
	7573	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.3;
	7573	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	7573	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.8;
	7574	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.6;
	7574	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.9;
	7574	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.2;
	7574	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.9;
	7574	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	7574	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.5;
	7575	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	420.1;
	7575	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.5;
	7575	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199;
	7575	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.6;
	7575	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	7575	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336;
	7576	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417.5;
	7576	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.9;
	7576	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.7;
	7576	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.1;
	7576	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	7576	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.2;
	7577	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.9;
	7577	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.7;
	7577	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.5;
	7577	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.4;
	7577	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	7577	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.5;
	7578	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.7;
	7578	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.8;
	7578	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.5;
	7578	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.8;
	7578	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	7578	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.9;
	7579	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.3;
	7579	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.9;
	7579	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.2;
	7579	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.7;
	7579	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	7579	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.4;
	7580	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.8;
	7580	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.5;
	7580	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.8;
	7580	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.9;
	7580	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	7580	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.9;
	7581	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.7;
	7581	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.9;
	7581	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.1;
	7581	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.8;
	7581	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	7581	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.7;
	7582	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442;
	7582	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.9;
	7582	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.1;
	7582	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.2;
	7582	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	7582	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.2;
	7583	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.4;
	7583	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132;
	7583	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.2;
	7583	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199;
	7583	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61;
	7583	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.2;
	7584	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	406.8;
	7584	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.4;
	7584	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.6;
	7584	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.2;
	7584	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	7584	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328;
	7585	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394;
	7585	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.8;
	7585	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	182.1;
	7585	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184.4;
	7585	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.5;
	7585	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.5;
	7586	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	386.7;
	7586	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.6;
	7586	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.3;
	7586	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.4;
	7586	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57;
	7586	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.1;
	7587	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	382.3;
	7587	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	120.4;
	7587	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181.8;
	7587	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	184;
	7587	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.1;
	7587	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.5;
	7588	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	378.9;
	7588	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.5;
	7588	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.2;
	7588	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.2;
	7588	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.8;
	7588	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	314.2;
	7589	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.5;
	7589	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.9;
	7589	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.4;
	7589	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.6;
	7589	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.8;
	7589	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.1;
	7590	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.8;
	7590	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.8;
	7590	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.7;
	7590	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.5;
	7590	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.6;
	7590	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.9;
	7591	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.8;
	7591	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.4;
	7591	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211;
	7591	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.3;
	7591	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61;
	7591	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.6;
	7592	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.3;
	7592	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.4;
	7592	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.5;
	7592	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.1;
	7592	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	7592	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373;
	7593	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.4;
	7593	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.4;
	7593	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.1;
	7593	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229;
	7593	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	7593	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.3;
	7594	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.5;
	7594	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.9;
	7594	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.4;
	7594	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228;
	7594	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	7594	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376;
	7595	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.1;
	7595	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.1;
	7595	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.4;
	7595	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.5;
	7595	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	7595	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.1;
	7596	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.6;
	7596	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.9;
	7596	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.7;
	7596	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.8;
	7596	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	7596	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.8;
	7597	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.6;
	7597	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.8;
	7597	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.7;
	7597	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228;
	7597	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	7597	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.1;
	7598	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470;
	7598	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.6;
	7598	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.7;
	7598	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.3;
	7598	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	7598	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.3;
	7599	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.7;
	7599	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.9;
	7599	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.4;
	7599	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.9;
	7599	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	7599	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372;
	7600	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.7;
	7600	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.6;
	7600	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.1;
	7600	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.6;
	7600	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	7600	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.6;
	7601	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.7;
	7601	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.3;
	7601	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241;
	7601	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.1;
	7601	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	7601	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.2;
	7602	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.7;
	7602	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.6;
	7602	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.7;
	7602	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249;
	7602	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	7602	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.8;
	7603	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.9;
	7603	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167;
	7603	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.4;
	7603	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246;
	7603	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	7603	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.6;
	7604	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.9;
	7604	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164;
	7604	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.6;
	7604	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.5;
	7604	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	7604	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.2;
	7605	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485;
	7605	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.3;
	7605	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.3;
	7605	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.6;
	7605	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.3;
	7605	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.7;
	7606	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.7;
	7606	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.3;
	7606	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.9;
	7606	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.8;
	7606	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	7606	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363;
	7607	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.2;
	7607	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.3;
	7607	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.6;
	7607	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.2;
	7607	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	7607	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.3;
	7608	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.2;
	7608	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.2;
	7608	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.3;
	7608	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.6;
	7608	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	7608	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.4;
	7609	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412;
	7609	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.2;
	7609	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.9;
	7609	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.1;
	7609	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	7609	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.4;
	7610	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398;
	7610	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.1;
	7610	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181.4;
	7610	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	183.5;
	7610	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	7610	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.9;
	7611	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403;
	7611	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	119.7;
	7611	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	181.7;
	7611	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	183.9;
	7611	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	7611	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.7;
	7612	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	406.9;
	7612	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	122.3;
	7612	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.9;
	7612	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.2;
	7612	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	7612	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.3;
	7613	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.3;
	7613	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.8;
	7613	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.7;
	7613	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.2;
	7613	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	7613	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.3;
	7614	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442;
	7614	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.7;
	7614	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.1;
	7614	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.9;
	7614	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	7614	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.6;
	7615	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479;
	7615	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.8;
	7615	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.6;
	7615	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.3;
	7615	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	7615	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.3;
	7616	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.5;
	7616	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.2;
	7616	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.2;
	7616	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.5;
	7616	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	7616	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.6;
	7617	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.4;
	7617	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.9;
	7617	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.7;
	7617	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230;
	7617	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	7617	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.4;
	7618	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.7;
	7618	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.5;
	7618	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.5;
	7618	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.4;
	7618	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.8;
	7618	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.8;
	7619	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482;
	7619	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.8;
	7619	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.2;
	7619	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.9;
	7619	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	7619	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.5;
	7620	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.4;
	7620	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.2;
	7620	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.7;
	7620	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.6;
	7620	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	7620	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367;
	7621	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.3;
	7621	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.1;
	7621	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.3;
	7621	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231;
	7621	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	7621	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.4;
	7622	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.4;
	7622	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.5;
	7622	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.1;
	7622	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.7;
	7622	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	7622	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.7;
	7623	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.6;
	7623	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.1;
	7623	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.9;
	7623	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.3;
	7623	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	7623	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.3;
	7624	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.8;
	7624	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.8;
	7624	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.2;
	7624	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.7;
	7624	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	7624	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.1;
	7625	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.9;
	7625	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.6;
	7625	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.6;
	7625	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.9;
	7625	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	7625	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.9;
	7626	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.9;
	7626	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.1;
	7626	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245;
	7626	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.1;
	7626	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.4;
	7626	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.5;
	7627	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.8;
	7627	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.1;
	7627	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.3;
	7627	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.2;
	7627	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	7627	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.2;
	7628	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.2;
	7628	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.9;
	7628	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.3;
	7628	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.1;
	7628	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	7628	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.8;
	7629	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.3;
	7629	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160;
	7629	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.3;
	7629	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.5;
	7629	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	7629	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.9;
	7630	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.5;
	7630	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.6;
	7630	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217;
	7630	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219;
	7630	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	7630	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361;
	7631	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442.2;
	7631	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.2;
	7631	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.7;
	7631	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.3;
	7631	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	7631	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.9;
	7632	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.3;
	7632	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.8;
	7632	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.4;
	7632	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.9;
	7632	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	7632	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.8;
	7633	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.7;
	7633	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.6;
	7633	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.4;
	7633	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.5;
	7633	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	7633	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.5;
	7634	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.5;
	7634	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124;
	7634	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.4;
	7634	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.1;
	7634	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.3;
	7634	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.3;
	7635	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.1;
	7635	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	123.7;
	7635	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.1;
	7635	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186;
	7635	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	7635	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.3;
	7636	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	401.8;
	7636	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.9;
	7636	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.9;
	7636	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.9;
	7636	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59;
	7636	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.5;
	7637	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	420.7;
	7637	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.7;
	7637	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.1;
	7637	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.3;
	7637	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.3;
	7637	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.8;
	7638	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.5;
	7638	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.6;
	7638	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.3;
	7638	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.1;
	7638	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.3;
	7638	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.5;
	7639	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.5;
	7639	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.7;
	7639	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.1;
	7639	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.5;
	7639	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	7639	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361;
	7640	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.9;
	7640	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.2;
	7640	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.7;
	7640	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.1;
	7640	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.1;
	7640	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.8;
	7641	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.9;
	7641	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.6;
	7641	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.6;
	7641	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.9;
	7641	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	7641	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376;
	7642	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.9;
	7642	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.5;
	7642	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.9;
	7642	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.4;
	7642	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.2;
	7642	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.6;
	7643	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.5;
	7643	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.4;
	7643	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.1;
	7643	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.2;
	7643	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.6;
	7643	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.8;
	7644	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.9;
	7644	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.4;
	7644	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.6;
	7644	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.7;
	7644	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.8;
	7644	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374;
	7645	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.8;
	7645	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.7;
	7645	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.6;
	7645	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.8;
	7645	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.9;
	7645	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.2;
	7646	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.5;
	7646	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.1;
	7646	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.8;
	7646	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.3;
	7646	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68;
	7646	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370;
	7647	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472;
	7647	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.2;
	7647	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224;
	7647	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.5;
	7647	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	7647	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.8;
	7648	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.6;
	7648	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.5;
	7648	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.6;
	7648	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.6;
	7648	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	7648	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.5;
	7649	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.2;
	7649	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.6;
	7649	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.9;
	7649	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.1;
	7649	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	7649	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.6;
	7650	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.9;
	7650	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.9;
	7650	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.9;
	7650	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.6;
	7650	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.3;
	7650	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385;
	7651	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.6;
	7651	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.7;
	7651	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.4;
	7651	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.6;
	7651	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.9;
	7651	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389;
	7652	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.3;
	7652	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.1;
	7652	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.9;
	7652	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.8;
	7652	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	7652	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.9;
	7653	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.1;
	7653	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.4;
	7653	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.6;
	7653	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231;
	7653	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.6;
	7653	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.3;
	7654	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.8;
	7654	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.9;
	7654	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.8;
	7654	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.7;
	7654	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.2;
	7654	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.6;
	7655	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.9;
	7655	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.9;
	7655	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.8;
	7655	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.6;
	7655	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	7655	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.5;
	7656	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.9;
	7656	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.8;
	7656	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.4;
	7656	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.9;
	7656	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	7656	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341;
	7657	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.7;
	7657	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.6;
	7657	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.3;
	7657	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.4;
	7657	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.1;
	7657	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.4;
	7658	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.1;
	7658	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.2;
	7658	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.7;
	7658	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190;
	7658	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.9;
	7658	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.4;
	7659	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410;
	7659	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.6;
	7659	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186;
	7659	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.8;
	7659	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.5;
	7659	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.5;
	7660	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.2;
	7660	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.1;
	7660	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.2;
	7660	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.9;
	7660	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.2;
	7660	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.5;
	7661	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.1;
	7661	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.7;
	7661	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.3;
	7661	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.3;
	7661	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.1;
	7661	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.1;
	7662	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.2;
	7662	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.2;
	7662	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205;
	7662	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.7;
	7662	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.8;
	7662	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.4;
	7663	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498.1;
	7663	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.2;
	7663	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.6;
	7663	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.5;
	7663	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	7663	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.7;
	7664	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.2;
	7664	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.9;
	7664	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.9;
	7664	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.3;
	7664	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	7664	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.3;
	7665	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.6;
	7665	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.8;
	7665	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.7;
	7665	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.5;
	7665	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	7665	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.9;
	7666	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.3;
	7666	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.9;
	7666	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.4;
	7666	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231;
	7666	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	7666	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.2;
	7667	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.7;
	7667	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.3;
	7667	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.3;
	7667	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.8;
	7667	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	7667	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.6;
	7668	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.6;
	7668	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.2;
	7668	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.1;
	7668	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.8;
	7668	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	7668	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.3;
	7669	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.5;
	7669	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.5;
	7669	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.5;
	7669	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.7;
	7669	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	7669	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.9;
	7670	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.9;
	7670	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.1;
	7670	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.8;
	7670	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.7;
	7670	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	7670	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.4;
	7671	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.9;
	7671	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.6;
	7671	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.4;
	7671	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.5;
	7671	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	7671	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.6;
	7672	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.8;
	7672	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.9;
	7672	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.4;
	7672	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.9;
	7672	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	7672	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.3;
	7673	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485;
	7673	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.7;
	7673	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.2;
	7673	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.7;
	7673	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	7673	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.9;
	7674	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.4;
	7674	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.6;
	7674	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.4;
	7674	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.1;
	7674	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	7674	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.3;
	7675	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.7;
	7675	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174;
	7675	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.3;
	7675	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.9;
	7675	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.2;
	7675	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.5;
	7676	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.7;
	7676	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171;
	7676	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.3;
	7676	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.2;
	7676	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	7676	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.7;
	7677	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.1;
	7677	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.6;
	7677	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.5;
	7677	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.4;
	7677	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	7677	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.6;
	7678	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.2;
	7678	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.7;
	7678	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.5;
	7678	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222;
	7678	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	7678	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.3;
	7679	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.5;
	7679	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.4;
	7679	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.4;
	7679	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.8;
	7679	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	7679	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.6;
	7680	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.4;
	7680	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.4;
	7680	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.5;
	7680	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.5;
	7680	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.2;
	7680	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.9;
	7681	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417.5;
	7681	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.6;
	7681	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.4;
	7681	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.6;
	7681	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.9;
	7681	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333;
	7682	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403.8;
	7682	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.6;
	7682	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.1;
	7682	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.7;
	7682	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.6;
	7682	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.1;
	7683	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	413.5;
	7683	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.1;
	7683	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.8;
	7683	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.4;
	7683	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.7;
	7683	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.1;
	7684	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.6;
	7684	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130;
	7684	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.9;
	7684	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.6;
	7684	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.9;
	7684	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332;
	7685	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.1;
	7685	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.3;
	7685	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.8;
	7685	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.7;
	7685	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.7;
	7685	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.4;
	7686	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.3;
	7686	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.9;
	7686	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.6;
	7686	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.2;
	7686	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	7686	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.1;
	7687	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.5;
	7687	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.2;
	7687	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.2;
	7687	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.8;
	7687	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	7687	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.6;
	7688	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.2;
	7688	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.6;
	7688	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.8;
	7688	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.7;
	7688	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	7688	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387;
	7689	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	524.2;
	7689	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.8;
	7689	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.3;
	7689	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.9;
	7689	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.3;
	7689	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.1;
	7690	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.4;
	7690	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.7;
	7690	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.5;
	7690	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.3;
	7690	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.6;
	7690	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380;
	7691	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.6;
	7691	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.8;
	7691	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.7;
	7691	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.1;
	7691	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69;
	7691	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.3;
	7692	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.6;
	7692	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.4;
	7692	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.3;
	7692	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.8;
	7692	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	7692	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.4;
	7693	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.2;
	7693	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.8;
	7693	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.1;
	7693	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.6;
	7693	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	7693	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363;
	7694	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.8;
	7694	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.5;
	7694	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.2;
	7694	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.5;
	7694	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	7694	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.2;
	7695	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.4;
	7695	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.5;
	7695	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.2;
	7695	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227;
	7695	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	7695	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.3;
	7696	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.2;
	7696	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.8;
	7696	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.6;
	7696	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.3;
	7696	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	7696	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.7;
	7697	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.7;
	7697	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.6;
	7697	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.2;
	7697	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.9;
	7697	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	7697	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.1;
	7698	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.1;
	7698	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.1;
	7698	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.5;
	7698	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.5;
	7698	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	7698	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.6;
	7699	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.9;
	7699	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.8;
	7699	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.6;
	7699	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.9;
	7699	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	7699	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.8;
	7700	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.2;
	7700	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.3;
	7700	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.9;
	7700	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.9;
	7700	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	7700	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.4;
	7701	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.4;
	7701	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.3;
	7701	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.4;
	7701	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.1;
	7701	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	7701	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.2;
	7702	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.9;
	7702	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.5;
	7702	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.9;
	7702	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.6;
	7702	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	7702	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.5;
	7703	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.4;
	7703	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.3;
	7703	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208;
	7703	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.6;
	7703	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	7703	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.8;
	7704	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	426.5;
	7704	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.5;
	7704	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204;
	7704	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.2;
	7704	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	7704	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.3;
	7705	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417.3;
	7705	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.2;
	7705	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194;
	7705	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.2;
	7705	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	7705	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.4;
	7706	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.3;
	7706	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.2;
	7706	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.4;
	7706	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.1;
	7706	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.1;
	7706	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.7;
	7707	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.6;
	7707	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.8;
	7707	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.5;
	7707	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.1;
	7707	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.6;
	7707	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.1;
	7708	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.6;
	7708	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.3;
	7708	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.4;
	7708	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.1;
	7708	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.8;
	7708	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.4;
	7709	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.4;
	7709	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.5;
	7709	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.1;
	7709	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.9;
	7709	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.2;
	7709	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.4;
	7710	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439;
	7710	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.5;
	7710	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.8;
	7710	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.1;
	7710	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	7710	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.4;
	7711	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.2;
	7711	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.3;
	7711	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.4;
	7711	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.3;
	7711	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	7711	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.1;
	7712	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.9;
	7712	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.1;
	7712	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.4;
	7712	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.1;
	7712	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	7712	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.5;
	7713	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.1;
	7713	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.2;
	7713	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.3;
	7713	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.6;
	7713	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	7713	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.1;
	7714	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.8;
	7714	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.8;
	7714	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.4;
	7714	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.6;
	7714	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	7714	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.7;
	7715	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.7;
	7715	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.2;
	7715	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.1;
	7715	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.1;
	7715	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	7715	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.8;
	7716	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.8;
	7716	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.9;
	7716	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.5;
	7716	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.4;
	7716	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	7716	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.2;
	7717	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.8;
	7717	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.5;
	7717	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.1;
	7717	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.1;
	7717	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	7717	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.5;
	7718	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.3;
	7718	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.1;
	7718	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.8;
	7718	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.5;
	7718	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	7718	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.9;
	7719	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	430.6;
	7719	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.9;
	7719	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.4;
	7719	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.3;
	7719	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	7719	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342;
	7720	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	430.3;
	7720	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.7;
	7720	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208;
	7720	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.3;
	7720	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	7720	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.2;
	7721	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	444.7;
	7721	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.3;
	7721	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.8;
	7721	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.3;
	7721	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	7721	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.3;
	7722	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465;
	7722	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.8;
	7722	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226;
	7722	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.3;
	7722	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	7722	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.4;
	7723	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.9;
	7723	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.7;
	7723	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.7;
	7723	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.5;
	7723	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	7723	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.4;
	7724	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.1;
	7724	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159;
	7724	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.9;
	7724	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.8;
	7724	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	7724	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.4;
	7725	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.5;
	7725	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.7;
	7725	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.7;
	7725	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.2;
	7725	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	7725	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355;
	7726	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.9;
	7726	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149;
	7726	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.6;
	7726	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.9;
	7726	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	7726	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.4;
	7727	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.1;
	7727	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.9;
	7727	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.4;
	7727	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.4;
	7727	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	7727	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.4;
	7728	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.5;
	7728	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.7;
	7728	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.5;
	7728	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.1;
	7728	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	7728	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.3;
	7729	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397;
	7729	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.3;
	7729	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.8;
	7729	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.6;
	7729	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.3;
	7729	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.9;
	7730	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393;
	7730	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.7;
	7730	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.2;
	7730	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.3;
	7730	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.9;
	7730	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.8;
	7731	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393;
	7731	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.9;
	7731	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.8;
	7731	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.9;
	7731	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	7731	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.7;
	7732	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	392.9;
	7732	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.8;
	7732	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.6;
	7732	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.6;
	7732	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.4;
	7732	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.9;
	7733	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403.9;
	7733	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.3;
	7733	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.2;
	7733	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.5;
	7733	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	7733	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.1;
	7734	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417;
	7734	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.7;
	7734	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.9;
	7734	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.2;
	7734	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	7734	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.6;
	7735	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.9;
	7735	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.1;
	7735	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.3;
	7735	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.3;
	7735	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	7735	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.5;
	7736	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.6;
	7736	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.1;
	7736	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.6;
	7736	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.6;
	7736	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	7736	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.2;
	7737	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.3;
	7737	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.9;
	7737	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.8;
	7737	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.2;
	7737	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.8;
	7737	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.8;
	7738	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.6;
	7738	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.5;
	7738	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.6;
	7738	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.6;
	7738	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.8;
	7738	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.6;
	7739	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.9;
	7739	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.4;
	7739	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.9;
	7739	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.8;
	7739	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.2;
	7739	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.1;
	7740	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.5;
	7740	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.1;
	7740	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.8;
	7740	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.8;
	7740	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	7740	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337;
	7741	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.3;
	7741	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.6;
	7741	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.1;
	7741	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203;
	7741	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	7741	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342;
	7742	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429;
	7742	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.8;
	7742	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.4;
	7742	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.4;
	7742	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	7742	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.6;
	7743	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	427.4;
	7743	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.5;
	7743	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.7;
	7743	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.8;
	7743	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	7743	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.3;
	7744	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.4;
	7744	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.6;
	7744	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.5;
	7744	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.4;
	7744	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	7744	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.8;
	7745	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.9;
	7745	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.6;
	7745	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.4;
	7745	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.9;
	7745	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	7745	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.5;
	7746	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.7;
	7746	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.7;
	7746	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.9;
	7746	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.8;
	7746	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	7746	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.6;
	7747	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.7;
	7747	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.2;
	7747	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.2;
	7747	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.6;
	7747	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	7747	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.7;
	7748	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.9;
	7748	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.6;
	7748	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.2;
	7748	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.6;
	7748	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	7748	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.9;
	7749	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.4;
	7749	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157;
	7749	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.7;
	7749	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224;
	7749	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	7749	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.2;
	7750	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447.1;
	7750	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.3;
	7750	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.1;
	7750	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.9;
	7750	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	7750	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.1;
	7751	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.9;
	7751	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.9;
	7751	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.7;
	7751	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.1;
	7751	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	7751	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.7;
	7752	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.5;
	7752	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.4;
	7752	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.9;
	7752	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.4;
	7752	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.3;
	7752	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330;
	7753	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	401.5;
	7753	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.6;
	7753	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.3;
	7753	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.1;
	7753	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	7753	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.7;
	7754	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	388.6;
	7754	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.1;
	7754	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	184.4;
	7754	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	186.2;
	7754	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.6;
	7754	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.8;
	7755	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.9;
	7755	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	124.8;
	7755	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	183.8;
	7755	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	185.5;
	7755	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.3;
	7755	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.2;
	7756	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	400.8;
	7756	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.7;
	7756	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.9;
	7756	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.5;
	7756	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.7;
	7756	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.6;
	7757	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.8;
	7757	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.7;
	7757	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.2;
	7757	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191;
	7757	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.5;
	7757	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.9;
	7758	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.4;
	7758	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.1;
	7758	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.7;
	7758	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.1;
	7758	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.2;
	7758	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.7;
	7759	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.1;
	7759	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160;
	7759	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.1;
	7759	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.9;
	7759	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.2;
	7759	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.7;
	7760	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505;
	7760	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.5;
	7760	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.7;
	7760	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230;
	7760	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.4;
	7760	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.6;
	7761	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.3;
	7761	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.3;
	7761	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.6;
	7761	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232;
	7761	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	7761	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.2;
	7762	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.2;
	7762	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.2;
	7762	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.2;
	7762	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.8;
	7762	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	7762	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379;
	7763	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.5;
	7763	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.1;
	7763	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.2;
	7763	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.9;
	7763	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.7;
	7763	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.1;
	7764	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.6;
	7764	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.5;
	7764	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.3;
	7764	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231;
	7764	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	7764	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.1;
	7765	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.2;
	7765	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.9;
	7765	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.9;
	7765	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.3;
	7765	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	7765	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.5;
	7766	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.8;
	7766	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.2;
	7766	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.7;
	7766	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.4;
	7766	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.9;
	7766	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.4;
	7767	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.4;
	7767	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.8;
	7767	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.3;
	7767	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.6;
	7767	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	7767	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.9;
	7768	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.1;
	7768	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.4;
	7768	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232;
	7768	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.9;
	7768	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	7768	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372;
	7769	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.9;
	7769	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166;
	7769	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243;
	7769	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.6;
	7769	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	7769	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.2;
	7770	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.4;
	7770	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.7;
	7770	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.8;
	7770	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.5;
	7770	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	7770	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.3;
	7771	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.9;
	7771	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.8;
	7771	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.6;
	7771	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.4;
	7771	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.1;
	7771	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.2;
	7772	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.1;
	7772	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.2;
	7772	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239;
	7772	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242;
	7772	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	7772	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.7;
	7773	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.2;
	7773	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.5;
	7773	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.6;
	7773	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.1;
	7773	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	7773	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.8;
	7774	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.8;
	7774	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.3;
	7774	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.5;
	7774	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.7;
	7774	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	7774	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.2;
	7775	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.5;
	7775	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.3;
	7775	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.7;
	7775	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.7;
	7775	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	7775	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.1;
	7776	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.5;
	7776	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.7;
	7776	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.3;
	7776	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201;
	7776	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	7776	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.9;
	7777	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.5;
	7777	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.5;
	7777	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.9;
	7777	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.2;
	7777	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60;
	7777	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.3;
	7778	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	426.8;
	7778	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.1;
	7778	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.3;
	7778	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.4;
	7778	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	7778	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.1;
	7779	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	430.9;
	7779	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	127.9;
	7779	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	185.6;
	7779	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	187.9;
	7779	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.3;
	7779	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.2;
	7780	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.7;
	7780	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.3;
	7780	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.6;
	7780	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.9;
	7780	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.3;
	7780	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.9;
	7781	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447;
	7781	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.9;
	7781	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.9;
	7781	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.7;
	7781	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.1;
	7781	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.8;
	7782	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.1;
	7782	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.1;
	7782	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.1;
	7782	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.8;
	7782	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	7782	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.7;
	7783	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.9;
	7783	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166;
	7783	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.7;
	7783	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.3;
	7783	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	7783	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.7;
	7784	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.5;
	7784	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.9;
	7784	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.4;
	7784	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.3;
	7784	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	7784	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.6;
	7785	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	533.8;
	7785	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.6;
	7785	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.6;
	7785	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.4;
	7785	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	7785	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.3;
	7786	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.4;
	7786	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.9;
	7786	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.5;
	7786	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.5;
	7786	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	7786	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.9;
	7787	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.4;
	7787	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.7;
	7787	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230;
	7787	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.7;
	7787	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68;
	7787	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.9;
	7788	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.6;
	7788	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.3;
	7788	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.1;
	7788	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233;
	7788	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.1;
	7788	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.5;
	7789	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.3;
	7789	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.5;
	7789	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.5;
	7789	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.7;
	7789	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.5;
	7789	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.8;
	7790	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.5;
	7790	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.8;
	7790	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.9;
	7790	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233;
	7790	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	7790	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.3;
	7791	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.6;
	7791	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.1;
	7791	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.3;
	7791	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.9;
	7791	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	7791	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.4;
	7792	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.2;
	7792	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.2;
	7792	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.9;
	7792	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239;
	7792	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	7792	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.5;
	7793	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.2;
	7793	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.5;
	7793	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.9;
	7793	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.6;
	7793	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	7793	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.7;
	7794	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.5;
	7794	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.6;
	7794	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.6;
	7794	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.3;
	7794	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	7794	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.5;
	7795	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.7;
	7795	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.8;
	7795	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.2;
	7795	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249;
	7795	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.4;
	7795	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.3;
	7796	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	540.9;
	7796	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.3;
	7796	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.7;
	7796	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.1;
	7796	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.8;
	7796	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.3;
	7797	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529;
	7797	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.1;
	7797	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.6;
	7797	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.8;
	7797	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.9;
	7797	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.6;
	7798	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.8;
	7798	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.7;
	7798	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.7;
	7798	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.6;
	7798	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.2;
	7798	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.4;
	7799	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.4;
	7799	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.1;
	7799	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.4;
	7799	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.6;
	7799	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.5;
	7799	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.4;
	7800	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458;
	7800	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.8;
	7800	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203;
	7800	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.6;
	7800	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.1;
	7800	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.2;
	7801	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	444.3;
	7801	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.5;
	7801	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.3;
	7801	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.4;
	7801	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	7801	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.4;
	7802	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.4;
	7802	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.3;
	7802	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.6;
	7802	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.5;
	7802	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	7802	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.9;
	7803	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.2;
	7803	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136;
	7803	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.3;
	7803	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.2;
	7803	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	7803	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.1;
	7804	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.3;
	7804	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.7;
	7804	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.5;
	7804	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.5;
	7804	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	7804	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.7;
	7805	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.1;
	7805	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.2;
	7805	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.2;
	7805	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198;
	7805	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	7805	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.8;
	7806	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.5;
	7806	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.1;
	7806	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.4;
	7806	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.9;
	7806	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	7806	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364;
	7807	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519;
	7807	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.1;
	7807	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.2;
	7807	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.2;
	7807	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.3;
	7807	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.9;
	7808	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	533.2;
	7808	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.3;
	7808	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.4;
	7808	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.5;
	7808	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.7;
	7808	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.5;
	7809	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.2;
	7809	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.5;
	7809	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.5;
	7809	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.1;
	7809	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.8;
	7809	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.9;
	7810	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.2;
	7810	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.1;
	7810	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.2;
	7810	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.1;
	7810	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.5;
	7810	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.1;
	7811	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.8;
	7811	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167;
	7811	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.2;
	7811	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.3;
	7811	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77;
	7811	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.5;
	7812	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488;
	7812	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.3;
	7812	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.6;
	7812	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231;
	7812	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75;
	7812	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.5;
	7813	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.7;
	7813	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162;
	7813	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.6;
	7813	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.7;
	7813	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.5;
	7813	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.7;
	7814	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.2;
	7814	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160;
	7814	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.8;
	7814	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.9;
	7814	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.2;
	7814	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.5;
	7815	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.1;
	7815	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.6;
	7815	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.7;
	7815	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232;
	7815	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	7815	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.6;
	7816	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.6;
	7816	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.5;
	7816	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.4;
	7816	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.9;
	7816	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.5;
	7816	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.8;
	7817	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.9;
	7817	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.2;
	7817	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.2;
	7817	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.7;
	7817	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	7817	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.4;
	7818	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.2;
	7818	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.1;
	7818	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.1;
	7818	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.1;
	7818	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.1;
	7818	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.6;
	7819	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.8;
	7819	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.4;
	7819	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247;
	7819	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.8;
	7819	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.9;
	7819	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.4;
	7820	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.7;
	7820	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.9;
	7820	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.9;
	7820	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.5;
	7820	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77;
	7820	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.3;
	7821	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.8;
	7821	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.2;
	7821	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.2;
	7821	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.6;
	7821	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.7;
	7821	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.6;
	7822	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.8;
	7822	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.1;
	7822	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.4;
	7822	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.6;
	7822	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.8;
	7822	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.9;
	7823	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.7;
	7823	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.1;
	7823	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.2;
	7823	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.5;
	7823	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.4;
	7823	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.3;
	7824	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.1;
	7824	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.4;
	7824	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.2;
	7824	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.7;
	7824	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.1;
	7824	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.9;
	7825	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.2;
	7825	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.7;
	7825	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.8;
	7825	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.3;
	7825	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	7825	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.5;
	7826	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391.2;
	7826	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.3;
	7826	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.6;
	7826	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.9;
	7826	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	7826	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.9;
	7827	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	391.8;
	7827	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.6;
	7827	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.4;
	7827	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.5;
	7827	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	7827	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.1;
	7828	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403.5;
	7828	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.1;
	7828	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.8;
	7828	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.3;
	7828	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	7828	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.6;
	7829	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.7;
	7829	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.7;
	7829	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.9;
	7829	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196;
	7829	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	7829	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.2;
	7830	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	444.5;
	7830	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153;
	7830	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.3;
	7830	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.8;
	7830	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.5;
	7830	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.7;
	7831	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.7;
	7831	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.4;
	7831	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.1;
	7831	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215;
	7831	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.8;
	7831	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.5;
	7832	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.4;
	7832	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.8;
	7832	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220;
	7832	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.6;
	7832	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.7;
	7832	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.8;
	7833	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.5;
	7833	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.1;
	7833	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.9;
	7833	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.7;
	7833	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.7;
	7833	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.2;
	7834	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.6;
	7834	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.4;
	7834	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.5;
	7834	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.6;
	7834	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.6;
	7834	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.5;
	7835	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.5;
	7835	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.2;
	7835	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.5;
	7835	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.1;
	7835	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.3;
	7835	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.2;
	7836	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.7;
	7836	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.3;
	7836	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.7;
	7836	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.8;
	7836	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.3;
	7836	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.8;
	7837	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.5;
	7837	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.7;
	7837	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.2;
	7837	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.7;
	7837	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.1;
	7837	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.8;
	7838	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.1;
	7838	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.9;
	7838	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.1;
	7838	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.3;
	7838	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	7838	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.2;
	7839	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452;
	7839	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.9;
	7839	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.7;
	7839	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.7;
	7839	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.9;
	7839	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.3;
	7840	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.7;
	7840	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.6;
	7840	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.4;
	7840	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.2;
	7840	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.8;
	7840	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.9;
	7841	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.5;
	7841	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.6;
	7841	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.7;
	7841	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240;
	7841	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.9;
	7841	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.3;
	7842	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.5;
	7842	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.3;
	7842	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.3;
	7842	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.8;
	7842	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.5;
	7842	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.8;
	7843	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.7;
	7843	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.7;
	7843	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240;
	7843	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.6;
	7843	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.7;
	7843	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.7;
	7844	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.3;
	7844	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.4;
	7844	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.4;
	7844	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.4;
	7844	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.3;
	7844	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.4;
	7845	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.3;
	7845	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.2;
	7845	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.3;
	7845	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.8;
	7845	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.8;
	7845	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.8;
	7846	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.5;
	7846	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.5;
	7846	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.7;
	7846	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.6;
	7846	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.7;
	7846	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.7;
	7847	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.6;
	7847	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.9;
	7847	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207;
	7847	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.6;
	7847	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.3;
	7847	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.6;
	7848	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.4;
	7848	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.4;
	7848	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202;
	7848	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.6;
	7848	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.4;
	7848	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.3;
	7849	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.3;
	7849	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131;
	7849	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.8;
	7849	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.2;
	7849	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	7849	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.7;
	7850	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	395.3;
	7850	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131;
	7850	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.9;
	7850	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.9;
	7850	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	7850	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.8;
	7851	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	398.9;
	7851	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.8;
	7851	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.9;
	7851	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189;
	7851	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	7851	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.5;
	7852	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403.7;
	7852	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.8;
	7852	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.1;
	7852	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.2;
	7852	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	7852	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.3;
	7853	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.5;
	7853	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.4;
	7853	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.3;
	7853	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.4;
	7853	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	7853	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.3;
	7854	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.9;
	7854	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.4;
	7854	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.5;
	7854	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.2;
	7854	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	7854	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.6;
	7855	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.7;
	7855	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.2;
	7855	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.8;
	7855	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.4;
	7855	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	7855	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.1;
	7856	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.7;
	7856	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.6;
	7856	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220;
	7856	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.5;
	7856	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.5;
	7856	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.1;
	7857	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.9;
	7857	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.2;
	7857	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.8;
	7857	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.7;
	7857	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78;
	7857	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.5;
	7858	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.5;
	7858	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.5;
	7858	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.1;
	7858	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.1;
	7858	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.3;
	7858	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.7;
	7859	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.2;
	7859	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.2;
	7859	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.6;
	7859	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.8;
	7859	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.2;
	7859	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.4;
	7860	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.1;
	7860	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.1;
	7860	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.4;
	7860	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.9;
	7860	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.7;
	7860	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365;
	7861	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.4;
	7861	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.2;
	7861	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223;
	7861	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.4;
	7861	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.7;
	7861	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.6;
	7862	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.5;
	7862	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.2;
	7862	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.8;
	7862	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222;
	7862	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	7862	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.7;
	7863	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.2;
	7863	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.5;
	7863	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.3;
	7863	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.5;
	7863	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.7;
	7863	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.3;
	7864	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.1;
	7864	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.1;
	7864	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.1;
	7864	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.4;
	7864	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	7864	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.6;
	7865	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.2;
	7865	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.2;
	7865	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.4;
	7865	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.4;
	7865	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	7865	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.4;
	7866	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.5;
	7866	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.9;
	7866	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.2;
	7866	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.4;
	7866	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.2;
	7866	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.8;
	7867	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490;
	7867	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.4;
	7867	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239;
	7867	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.6;
	7867	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	7867	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.3;
	7868	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.8;
	7868	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.3;
	7868	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.4;
	7868	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234;
	7868	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.9;
	7868	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.5;
	7869	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.8;
	7869	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.6;
	7869	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.1;
	7869	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.8;
	7869	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	7869	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.3;
	7870	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467;
	7870	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.9;
	7870	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.2;
	7870	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.9;
	7870	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69;
	7870	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.2;
	7871	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447.5;
	7871	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.6;
	7871	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.6;
	7871	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.3;
	7871	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	7871	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.5;
	7872	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.2;
	7872	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.1;
	7872	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.9;
	7872	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206;
	7872	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	7872	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.7;
	7873	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.3;
	7873	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.8;
	7873	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193;
	7873	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.5;
	7873	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	7873	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.2;
	7874	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.5;
	7874	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.3;
	7874	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.8;
	7874	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.7;
	7874	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.7;
	7874	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.9;
	7875	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	402.5;
	7875	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.5;
	7875	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.4;
	7875	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.3;
	7875	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	7875	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325;
	7876	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.1;
	7876	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.2;
	7876	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.9;
	7876	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.8;
	7876	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	7876	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.3;
	7877	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	422.4;
	7877	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137;
	7877	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.3;
	7877	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193;
	7877	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.2;
	7877	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.7;
	7878	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.1;
	7878	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.6;
	7878	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.1;
	7878	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.8;
	7878	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	7878	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.8;
	7879	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.8;
	7879	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.4;
	7879	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.2;
	7879	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.6;
	7879	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	7879	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.5;
	7880	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.2;
	7880	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.3;
	7880	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.3;
	7880	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.1;
	7880	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	7880	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.2;
	7881	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.3;
	7881	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.7;
	7881	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.9;
	7881	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.2;
	7881	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	7881	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357;
	7882	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.2;
	7882	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.9;
	7882	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.1;
	7882	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211;
	7882	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	7882	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.7;
	7883	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482;
	7883	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.6;
	7883	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.5;
	7883	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.5;
	7883	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	7883	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.8;
	7884	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.5;
	7884	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156;
	7884	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.9;
	7884	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.1;
	7884	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	7884	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.2;
	7885	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.7;
	7885	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154;
	7885	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209;
	7885	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.4;
	7885	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	7885	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.1;
	7886	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.7;
	7886	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.8;
	7886	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.1;
	7886	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.7;
	7886	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	7886	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.1;
	7887	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.4;
	7887	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.7;
	7887	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.6;
	7887	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209;
	7887	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.4;
	7887	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.4;
	7888	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.5;
	7888	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.8;
	7888	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.8;
	7888	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.9;
	7888	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.4;
	7888	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.5;
	7889	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.1;
	7889	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.5;
	7889	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.9;
	7889	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.1;
	7889	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.7;
	7889	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.6;
	7890	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507;
	7890	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.5;
	7890	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.9;
	7890	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.1;
	7890	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60;
	7890	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381;
	7891	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521;
	7891	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.9;
	7891	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.1;
	7891	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.3;
	7891	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	7891	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387;
	7892	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.9;
	7892	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.4;
	7892	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.8;
	7892	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228;
	7892	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.2;
	7892	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.2;
	7893	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.9;
	7893	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.6;
	7893	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.7;
	7893	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.7;
	7893	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	7893	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.8;
	7894	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.5;
	7894	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.2;
	7894	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.7;
	7894	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.8;
	7894	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.9;
	7894	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.5;
	7895	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.6;
	7895	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.8;
	7895	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.6;
	7895	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.3;
	7895	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	7895	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.6;
	7896	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.3;
	7896	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.3;
	7896	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.9;
	7896	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206;
	7896	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	7896	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.2;
	7897	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	426.9;
	7897	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.3;
	7897	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.7;
	7897	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.5;
	7897	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.4;
	7897	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.1;
	7898	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.3;
	7898	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.7;
	7898	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.9;
	7898	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.7;
	7898	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.5;
	7898	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.4;
	7899	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417.2;
	7899	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.4;
	7899	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.4;
	7899	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.3;
	7899	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.5;
	7899	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.5;
	7900	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.5;
	7900	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.9;
	7900	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.7;
	7900	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	188.9;
	7900	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	7900	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.7;
	7901	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.6;
	7901	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.2;
	7901	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.6;
	7901	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.5;
	7901	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58;
	7901	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.8;
	7902	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	444.6;
	7902	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.3;
	7902	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.9;
	7902	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.9;
	7902	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	7902	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.7;
	7903	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.2;
	7903	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.6;
	7903	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.5;
	7903	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.2;
	7903	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	7903	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.5;
	7904	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.1;
	7904	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.5;
	7904	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.4;
	7904	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.5;
	7904	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	7904	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.8;
	7905	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.9;
	7905	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.3;
	7905	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.8;
	7905	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.9;
	7905	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	7905	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.8;
	7906	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.6;
	7906	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.1;
	7906	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.2;
	7906	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.5;
	7906	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	7906	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.4;
	7907	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.7;
	7907	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.9;
	7907	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.2;
	7907	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.8;
	7907	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	7907	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.2;
	7908	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.2;
	7908	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.1;
	7908	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.9;
	7908	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.2;
	7908	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.1;
	7908	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.6;
	7909	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486;
	7909	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.6;
	7909	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204;
	7909	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.6;
	7909	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	7909	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.9;
	7910	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.4;
	7910	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.5;
	7910	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.3;
	7910	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.8;
	7910	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	7910	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.2;
	7911	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.4;
	7911	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.5;
	7911	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.3;
	7911	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.8;
	7911	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	7911	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366;
	7912	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488;
	7912	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.2;
	7912	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.8;
	7912	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.8;
	7912	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	7912	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.1;
	7913	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.8;
	7913	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.9;
	7913	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.8;
	7913	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.5;
	7913	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	7913	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.9;
	7914	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.1;
	7914	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.5;
	7914	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.2;
	7914	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.2;
	7914	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	7914	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396;
	7915	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.3;
	7915	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.4;
	7915	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235;
	7915	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.1;
	7915	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	7915	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.1;
	7916	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.8;
	7916	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.2;
	7916	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.6;
	7916	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.6;
	7916	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	7916	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.1;
	7917	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523;
	7917	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.6;
	7917	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.9;
	7917	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.6;
	7917	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	7917	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.5;
	7918	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.8;
	7918	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.6;
	7918	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.9;
	7918	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219;
	7918	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	7918	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.1;
	7919	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.3;
	7919	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.6;
	7919	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.2;
	7919	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.9;
	7919	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	7919	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.5;
	7920	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.4;
	7920	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.4;
	7920	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201;
	7920	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.3;
	7920	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	7920	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.7;
	7921	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.4;
	7921	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139;
	7921	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.6;
	7921	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.6;
	7921	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.8;
	7921	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.8;
	7922	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.6;
	7922	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.7;
	7922	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.6;
	7922	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.7;
	7922	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.7;
	7922	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.4;
	7923	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.9;
	7923	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.4;
	7923	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.9;
	7923	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190;
	7923	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.5;
	7923	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339;
	7924	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.8;
	7924	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.7;
	7924	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191;
	7924	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192;
	7924	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.7;
	7924	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.1;
	7925	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.9;
	7925	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.1;
	7925	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194;
	7925	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.4;
	7925	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.4;
	7925	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.5;
	7926	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.7;
	7926	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.9;
	7926	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.8;
	7926	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.5;
	7926	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57;
	7926	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.4;
	7927	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510;
	7927	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.9;
	7927	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.6;
	7927	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.7;
	7927	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.5;
	7927	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.6;
	7928	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	576.2;
	7928	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.3;
	7928	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.9;
	7928	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.1;
	7928	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.7;
	7928	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.7;
	7929	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.5;
	7929	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.6;
	7929	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235;
	7929	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.3;
	7929	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	7929	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.1;
	7930	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.2;
	7930	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.5;
	7930	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.8;
	7930	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.8;
	7930	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	7930	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.4;
	7931	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525;
	7931	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.3;
	7931	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.4;
	7931	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.3;
	7931	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	7931	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.3;
	7932	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	524.9;
	7932	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.8;
	7932	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.8;
	7932	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.3;
	7932	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.3;
	7932	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.6;
	7933	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525.2;
	7933	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.2;
	7933	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.9;
	7933	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.2;
	7933	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	7933	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.9;
	7934	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.7;
	7934	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.1;
	7934	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.7;
	7934	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.3;
	7934	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	7934	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.4;
	7935	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517;
	7935	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.6;
	7935	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231;
	7935	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.4;
	7935	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	7935	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.7;
	7936	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.5;
	7936	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.6;
	7936	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.8;
	7936	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.6;
	7936	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	7936	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.5;
	7937	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.6;
	7937	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.5;
	7937	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.2;
	7937	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.1;
	7937	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	7937	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405;
	7938	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.2;
	7938	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.6;
	7938	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255;
	7938	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.5;
	7938	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	7938	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.5;
	7939	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	554.5;
	7939	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.3;
	7939	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.1;
	7939	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.9;
	7939	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	7939	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.3;
	7940	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.7;
	7940	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.1;
	7940	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247;
	7940	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.8;
	7940	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.2;
	7940	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404;
	7941	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.8;
	7941	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.1;
	7941	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.2;
	7941	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.8;
	7941	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69;
	7941	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.4;
	7942	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.3;
	7942	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.7;
	7942	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229;
	7942	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230;
	7942	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	7942	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.8;
	7943	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504;
	7943	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.7;
	7943	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.5;
	7943	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.3;
	7943	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	7943	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.8;
	7944	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.9;
	7944	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.3;
	7944	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.4;
	7944	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.9;
	7944	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	7944	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.8;
	7945	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.3;
	7945	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.5;
	7945	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.2;
	7945	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.5;
	7945	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	7945	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.9;
	7946	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.4;
	7946	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.2;
	7946	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.3;
	7946	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.8;
	7946	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.7;
	7946	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.9;
	7947	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.9;
	7947	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.4;
	7947	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.7;
	7947	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.6;
	7947	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	7947	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.1;
	7948	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.4;
	7948	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.3;
	7948	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.6;
	7948	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.5;
	7948	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	7948	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.8;
	7949	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.9;
	7949	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.4;
	7949	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.4;
	7949	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.7;
	7949	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.7;
	7949	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.2;
	7950	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	498;
	7950	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.2;
	7950	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207;
	7950	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.8;
	7950	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	7950	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369;
	7951	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.3;
	7951	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.7;
	7951	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223;
	7951	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225;
	7951	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	7951	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.7;
	7952	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	553.5;
	7952	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.3;
	7952	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.6;
	7952	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.7;
	7952	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	7952	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400;
	7953	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564.7;
	7953	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.9;
	7953	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.5;
	7953	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.3;
	7953	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	7953	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.9;
	7954	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.5;
	7954	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.3;
	7954	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.8;
	7954	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.1;
	7954	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.1;
	7954	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.4;
	7955	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548.9;
	7955	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.6;
	7955	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.8;
	7955	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.3;
	7955	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	7955	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.4;
	7956	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	540.5;
	7956	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.4;
	7956	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.4;
	7956	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.3;
	7956	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	7956	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.4;
	7957	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.8;
	7957	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.3;
	7957	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.9;
	7957	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.4;
	7957	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	7957	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.6;
	7958	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.6;
	7958	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.2;
	7958	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.5;
	7958	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.2;
	7958	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	7958	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.9;
	7959	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.9;
	7959	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.9;
	7959	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.8;
	7959	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.2;
	7959	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	7959	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.5;
	7960	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.7;
	7960	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.4;
	7960	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.5;
	7960	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.2;
	7960	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	7960	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.9;
	7961	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549.3;
	7961	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186;
	7961	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.8;
	7961	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.1;
	7961	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	7961	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.8;
	7962	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	568.2;
	7962	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.5;
	7962	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.9;
	7962	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.2;
	7962	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	7962	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.1;
	7963	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	573.4;
	7963	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.1;
	7963	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.7;
	7963	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.8;
	7963	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	7963	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.2;
	7964	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570;
	7964	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.6;
	7964	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.4;
	7964	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.3;
	7964	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.4;
	7964	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.9;
	7965	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	563.8;
	7965	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.5;
	7965	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.7;
	7965	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241;
	7965	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	7965	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.4;
	7966	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.6;
	7966	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.2;
	7966	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.1;
	7966	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229;
	7966	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.9;
	7966	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.5;
	7967	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.3;
	7967	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.9;
	7967	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.4;
	7967	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.8;
	7967	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.2;
	7967	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377;
	7968	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.3;
	7968	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.5;
	7968	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.9;
	7968	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.9;
	7968	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.5;
	7968	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363;
	7969	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.1;
	7969	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.6;
	7969	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.6;
	7969	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.2;
	7969	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.2;
	7969	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.3;
	7970	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.3;
	7970	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.1;
	7970	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.8;
	7970	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.9;
	7970	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	7970	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.8;
	7971	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.7;
	7971	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.6;
	7971	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191;
	7971	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.1;
	7971	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	7971	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.1;
	7972	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470;
	7972	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.3;
	7972	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.6;
	7972	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.8;
	7972	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	7972	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.3;
	7973	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.9;
	7973	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.7;
	7973	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.4;
	7973	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.9;
	7973	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	7973	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.2;
	7974	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.9;
	7974	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.6;
	7974	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.8;
	7974	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.6;
	7974	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.7;
	7974	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367;
	7975	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	540.2;
	7975	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.3;
	7975	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.3;
	7975	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.9;
	7975	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.6;
	7975	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.3;
	7976	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543;
	7976	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.7;
	7976	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.4;
	7976	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.2;
	7976	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76;
	7976	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.2;
	7977	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.5;
	7977	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.6;
	7977	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.2;
	7977	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.8;
	7977	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.3;
	7977	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.6;
	7978	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.5;
	7978	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.6;
	7978	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.9;
	7978	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240;
	7978	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.1;
	7978	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.3;
	7979	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.9;
	7979	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.2;
	7979	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.6;
	7979	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.1;
	7979	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.8;
	7979	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.6;
	7980	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.7;
	7980	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.7;
	7980	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.3;
	7980	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.5;
	7980	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.1;
	7980	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.6;
	7981	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509;
	7981	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.3;
	7981	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.3;
	7981	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.4;
	7981	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.7;
	7981	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.1;
	7982	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.3;
	7982	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.9;
	7982	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236;
	7982	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.4;
	7982	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	7982	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.7;
	7983	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.4;
	7983	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.2;
	7983	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.3;
	7983	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.3;
	7983	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.2;
	7983	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.9;
	7984	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.7;
	7984	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.1;
	7984	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.4;
	7984	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.2;
	7984	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	7984	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.9;
	7985	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.1;
	7985	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.6;
	7985	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.3;
	7985	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.7;
	7985	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69;
	7985	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.1;
	7986	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528.5;
	7986	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.6;
	7986	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.4;
	7986	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.8;
	7986	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	7986	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.6;
	7987	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.3;
	7987	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.3;
	7987	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.4;
	7987	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.8;
	7987	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.5;
	7987	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.2;
	7988	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.7;
	7988	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.1;
	7988	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.2;
	7988	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.6;
	7988	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.5;
	7988	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.1;
	7989	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.1;
	7989	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.4;
	7989	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.5;
	7989	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.2;
	7989	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.2;
	7989	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.8;
	7990	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.6;
	7990	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.2;
	7990	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.9;
	7990	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.4;
	7990	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.9;
	7990	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.6;
	7991	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446;
	7991	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.8;
	7991	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.2;
	7991	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216;
	7991	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	7991	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.6;
	7992	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416;
	7992	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152;
	7992	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.5;
	7992	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212;
	7992	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	7992	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.5;
	7993	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	400.7;
	7993	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.7;
	7993	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.9;
	7993	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.7;
	7993	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.4;
	7993	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.4;
	7994	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383;
	7994	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.7;
	7994	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.6;
	7994	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.5;
	7994	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.1;
	7994	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.4;
	7995	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379.4;
	7995	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137;
	7995	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.8;
	7995	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.5;
	7995	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.9;
	7995	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.6;
	7996	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	385.5;
	7996	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.3;
	7996	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.8;
	7996	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.9;
	7996	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	7996	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	320.5;
	7997	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	396.4;
	7997	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143;
	7997	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.5;
	7997	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.9;
	7997	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	7997	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.5;
	7998	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.8;
	7998	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.6;
	7998	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.1;
	7998	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.6;
	7998	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.9;
	7998	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.7;
	7999	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.6;
	7999	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.1;
	7999	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.9;
	7999	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217;
	7999	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.1;
	7999	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.2;
	8000	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.8;
	8000	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.2;
	8000	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.3;
	8000	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.3;
	8000	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	8000	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.8;
	8001	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.3;
	8001	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.3;
	8001	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.5;
	8001	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.2;
	8001	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	8001	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.3;
	8002	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.6;
	8002	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.8;
	8002	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.4;
	8002	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.5;
	8002	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	8002	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.2;
	8003	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450;
	8003	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.2;
	8003	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220;
	8003	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224;
	8003	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	8003	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.9;
	8004	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.8;
	8004	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.7;
	8004	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.3;
	8004	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.8;
	8004	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	8004	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.5;
	8005	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.4;
	8005	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.1;
	8005	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.5;
	8005	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226;
	8005	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	8005	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.5;
	8006	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	426.2;
	8006	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.3;
	8006	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.2;
	8006	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225;
	8006	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	8006	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.4;
	8007	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416;
	8007	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.1;
	8007	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.6;
	8007	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.9;
	8007	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	8007	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.2;
	8008	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	414.4;
	8008	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.9;
	8008	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226;
	8008	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.5;
	8008	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	8008	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.7;
	8009	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	426.1;
	8009	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.7;
	8009	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233;
	8009	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.4;
	8009	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	8009	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.2;
	8010	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437;
	8010	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.1;
	8010	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.1;
	8010	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.2;
	8010	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	8010	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.8;
	8011	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.3;
	8011	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.7;
	8011	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.3;
	8011	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.2;
	8011	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.2;
	8011	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356;
	8012	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.4;
	8012	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.2;
	8012	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.7;
	8012	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.8;
	8012	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68;
	8012	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.3;
	8013	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.9;
	8013	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.5;
	8013	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.9;
	8013	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.3;
	8013	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	8013	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.7;
	8014	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.1;
	8014	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.5;
	8014	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.7;
	8014	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.9;
	8014	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	8014	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.6;
	8015	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.8;
	8015	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146;
	8015	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.8;
	8015	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.1;
	8015	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	8015	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.4;
	8016	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	388.9;
	8016	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143;
	8016	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.7;
	8016	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.6;
	8016	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.1;
	8016	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.5;
	8017	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	377;
	8017	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.7;
	8017	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.2;
	8017	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.8;
	8017	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.6;
	8017	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	318.2;
	8018	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	362.9;
	8018	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.2;
	8018	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.3;
	8018	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.6;
	8018	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.6;
	8018	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	310.2;
	8019	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	369.5;
	8019	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.6;
	8019	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.1;
	8019	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.8;
	8019	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.7;
	8019	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	312.6;
	8020	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	374.9;
	8020	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.5;
	8020	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.7;
	8020	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.4;
	8020	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.7;
	8020	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.3;
	8021	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	387.1;
	8021	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.4;
	8021	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.6;
	8021	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.6;
	8021	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53;
	8021	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	321.6;
	8022	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.1;
	8022	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.4;
	8022	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201;
	8022	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.8;
	8022	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.5;
	8022	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.6;
	8023	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.9;
	8023	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.6;
	8023	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.3;
	8023	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.1;
	8023	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.8;
	8023	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.4;
	8024	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	442.3;
	8024	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.3;
	8024	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.3;
	8024	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.6;
	8024	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.6;
	8024	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.9;
	8025	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.5;
	8025	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.2;
	8025	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.6;
	8025	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221;
	8025	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.4;
	8025	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.4;
	8026	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.9;
	8026	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.5;
	8026	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.6;
	8026	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.6;
	8026	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	8026	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.2;
	8027	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.2;
	8027	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.6;
	8027	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.5;
	8027	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.8;
	8027	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	8027	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.4;
	8028	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	438.9;
	8028	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.8;
	8028	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.8;
	8028	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.8;
	8028	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	8028	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.7;
	8029	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.1;
	8029	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.6;
	8029	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.7;
	8029	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222;
	8029	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	8029	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.7;
	8030	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.5;
	8030	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.8;
	8030	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.7;
	8030	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.9;
	8030	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	8030	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.5;
	8031	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428.4;
	8031	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.1;
	8031	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.4;
	8031	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.3;
	8031	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	8031	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.3;
	8032	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.9;
	8032	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.5;
	8032	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.1;
	8032	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.8;
	8032	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	8032	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.1;
	8033	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.4;
	8033	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.5;
	8033	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.2;
	8033	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.5;
	8033	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	8033	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.6;
	8034	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.4;
	8034	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.4;
	8034	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.3;
	8034	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.2;
	8034	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	8034	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.5;
	8035	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.3;
	8035	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.1;
	8035	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.1;
	8035	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.9;
	8035	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	8035	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.8;
	8036	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.7;
	8036	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.2;
	8036	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.2;
	8036	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.2;
	8036	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.1;
	8036	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.4;
	8037	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.8;
	8037	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.2;
	8037	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.7;
	8037	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.7;
	8037	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	8037	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.4;
	8038	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.3;
	8038	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.3;
	8038	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.5;
	8038	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.1;
	8038	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	8038	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.5;
	8039	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421;
	8039	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.9;
	8039	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.7;
	8039	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.9;
	8039	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	8039	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.8;
	8040	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	405.8;
	8040	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.5;
	8040	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208;
	8040	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.6;
	8040	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	8040	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.5;
	8041	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	394.6;
	8041	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.5;
	8041	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.2;
	8041	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.5;
	8041	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	56.8;
	8041	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.2;
	8042	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	377.3;
	8042	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.9;
	8042	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.4;
	8042	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.3;
	8042	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	54.1;
	8042	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.7;
	8043	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	381;
	8043	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.6;
	8043	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.4;
	8043	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.9;
	8043	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.4;
	8043	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.9;
	8044	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	385.2;
	8044	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.2;
	8044	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.5;
	8044	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.9;
	8044	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	51.8;
	8044	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319.7;
	8045	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	389.5;
	8045	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.2;
	8045	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.9;
	8045	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.7;
	8045	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	52.7;
	8045	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322.3;
	8046	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.6;
	8046	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.5;
	8046	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.6;
	8046	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.3;
	8046	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	53.1;
	8046	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.2;
	8047	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.4;
	8047	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.7;
	8047	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.2;
	8047	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.3;
	8047	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	55.3;
	8047	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.9;
	8048	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.8;
	8048	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.8;
	8048	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.9;
	8048	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.3;
	8048	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59;
	8048	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.3;
	8049	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.1;
	8049	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.5;
	8049	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.4;
	8049	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.4;
	8049	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	8049	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.2;
	8050	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.9;
	8050	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.6;
	8050	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.9;
	8050	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.6;
	8050	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	8050	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.5;
	8051	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.5;
	8051	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.3;
	8051	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.3;
	8051	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.5;
	8051	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	8051	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.1;
	8052	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.5;
	8052	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.7;
	8052	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.8;
	8052	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.6;
	8052	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	8052	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.6;
	8053	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.2;
	8053	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.3;
	8053	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.4;
	8053	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217;
	8053	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	8053	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.3;
	8054	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.2;
	8054	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.8;
	8054	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.1;
	8054	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.4;
	8054	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	8054	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.8;
	8055	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.2;
	8055	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162;
	8055	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.1;
	8055	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.6;
	8055	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	8055	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.3;
	8056	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.8;
	8056	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.7;
	8056	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.9;
	8056	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.3;
	8056	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	8056	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.1;
	8057	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.3;
	8057	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.4;
	8057	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.1;
	8057	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.7;
	8057	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	8057	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.4;
	8058	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.3;
	8058	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.7;
	8058	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.2;
	8058	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.2;
	8058	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	8058	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.7;
	8059	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.6;
	8059	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.9;
	8059	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.5;
	8059	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.2;
	8059	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	8059	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.4;
	8060	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.2;
	8060	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.5;
	8060	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.9;
	8060	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.4;
	8060	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	8060	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.3;
	8061	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.3;
	8061	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.2;
	8061	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.6;
	8061	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.4;
	8061	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.7;
	8061	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.4;
	8062	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.2;
	8062	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.4;
	8062	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.2;
	8062	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.5;
	8062	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	8062	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.6;
	8063	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428;
	8063	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.3;
	8063	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.6;
	8063	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213;
	8063	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	8063	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.8;
	8064	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	408.9;
	8064	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.7;
	8064	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.7;
	8064	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.7;
	8064	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	8064	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.5;
	8065	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	393.6;
	8065	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.1;
	8065	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.3;
	8065	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.3;
	8065	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	8065	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.6;
	8066	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	377.3;
	8066	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.6;
	8066	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.7;
	8066	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191;
	8066	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	8066	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.3;
	8067	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379.2;
	8067	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.2;
	8067	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.5;
	8067	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190;
	8067	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.8;
	8067	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.7;
	8068	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379.4;
	8068	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.2;
	8068	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.1;
	8068	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.8;
	8068	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.3;
	8068	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.8;
	8069	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	381.1;
	8069	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.4;
	8069	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.6;
	8069	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.4;
	8069	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.1;
	8069	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	317.1;
	8070	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	390.4;
	8070	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.1;
	8070	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.1;
	8070	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.3;
	8070	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.5;
	8070	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	322;
	8071	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.6;
	8071	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.7;
	8071	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.3;
	8071	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.1;
	8071	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	8071	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.2;
	8072	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	409.8;
	8072	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.9;
	8072	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196;
	8072	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.3;
	8072	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	8072	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.8;
	8073	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.1;
	8073	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.6;
	8073	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.7;
	8073	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.6;
	8073	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	8073	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.5;
	8074	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.9;
	8074	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.3;
	8074	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.9;
	8074	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.9;
	8074	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.1;
	8074	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.7;
	8075	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.4;
	8075	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.6;
	8075	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.9;
	8075	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.9;
	8075	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.1;
	8075	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.3;
	8076	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.3;
	8076	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.8;
	8076	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.5;
	8076	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.4;
	8076	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.2;
	8076	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345;
	8077	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.2;
	8077	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.7;
	8077	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.3;
	8077	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.3;
	8077	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	8077	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.3;
	8078	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.1;
	8078	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.1;
	8078	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203;
	8078	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.1;
	8078	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	8078	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.2;
	8079	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	427.6;
	8079	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.1;
	8079	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.6;
	8079	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.7;
	8079	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	8079	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.5;
	8080	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.7;
	8080	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.1;
	8080	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.6;
	8080	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.5;
	8080	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.7;
	8080	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.8;
	8081	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.3;
	8081	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.1;
	8081	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.1;
	8081	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.2;
	8081	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	8081	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358;
	8082	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.7;
	8082	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.1;
	8082	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.4;
	8082	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.7;
	8082	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	8082	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.1;
	8083	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.9;
	8083	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.1;
	8083	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.9;
	8083	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.5;
	8083	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.6;
	8083	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.2;
	8084	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.9;
	8084	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174;
	8084	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.4;
	8084	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.3;
	8084	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	8084	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.1;
	8085	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469.8;
	8085	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.1;
	8085	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.3;
	8085	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232;
	8085	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.7;
	8085	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.3;
	8086	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.2;
	8086	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.7;
	8086	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.4;
	8086	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.7;
	8086	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.8;
	8086	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.3;
	8087	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.6;
	8087	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.7;
	8087	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.7;
	8087	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.5;
	8087	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.9;
	8087	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.8;
	8088	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424;
	8088	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.9;
	8088	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.8;
	8088	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.3;
	8088	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	8088	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.4;
	8089	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403.4;
	8089	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.8;
	8089	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.1;
	8089	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.9;
	8089	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	8089	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.1;
	8090	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	402.7;
	8090	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.9;
	8090	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	186.8;
	8090	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.1;
	8090	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.6;
	8090	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	323.9;
	8091	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	406;
	8091	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.6;
	8091	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187;
	8091	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189;
	8091	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.4;
	8091	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.6;
	8092	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	403.4;
	8092	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.1;
	8092	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.8;
	8092	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.9;
	8092	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.3;
	8092	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.1;
	8093	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.1;
	8093	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139;
	8093	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.5;
	8093	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.7;
	8093	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.4;
	8093	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.7;
	8094	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.6;
	8094	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.8;
	8094	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.1;
	8094	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.9;
	8094	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	8094	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.1;
	8095	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.9;
	8095	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.5;
	8095	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.4;
	8095	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.9;
	8095	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	8095	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.5;
	8096	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.8;
	8096	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.4;
	8096	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.7;
	8096	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.2;
	8096	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	8096	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.6;
	8097	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.6;
	8097	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.2;
	8097	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.2;
	8097	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.7;
	8097	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	8097	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.3;
	8098	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.6;
	8098	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.3;
	8098	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.4;
	8098	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.3;
	8098	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	8098	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.9;
	8099	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.7;
	8099	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.5;
	8099	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.5;
	8099	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.5;
	8099	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	8099	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.9;
	8100	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504;
	8100	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.5;
	8100	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.3;
	8100	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.1;
	8100	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.5;
	8100	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.9;
	8101	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.8;
	8101	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163;
	8101	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.1;
	8101	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.1;
	8101	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	8101	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.2;
	8102	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.1;
	8102	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.5;
	8102	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.9;
	8102	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.6;
	8102	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	8102	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.6;
	8103	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.7;
	8103	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.2;
	8103	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.6;
	8103	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.6;
	8103	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	8103	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.3;
	8104	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.3;
	8104	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.6;
	8104	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.1;
	8104	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.1;
	8104	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	8104	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.8;
	8105	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522;
	8105	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.2;
	8105	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.3;
	8105	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.8;
	8105	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	8105	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.2;
	8106	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547.2;
	8106	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.4;
	8106	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.6;
	8106	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.6;
	8106	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	8106	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406;
	8107	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	555.1;
	8107	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.5;
	8107	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.6;
	8107	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.9;
	8107	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.9;
	8107	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.9;
	8108	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550;
	8108	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.6;
	8108	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.8;
	8108	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.7;
	8108	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	8108	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.3;
	8109	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541;
	8109	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.8;
	8109	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.1;
	8109	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.6;
	8109	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70;
	8109	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.9;
	8110	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.9;
	8110	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.4;
	8110	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.5;
	8110	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.5;
	8110	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.3;
	8110	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.4;
	8111	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.2;
	8111	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.6;
	8111	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.3;
	8111	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.6;
	8111	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	8111	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.9;
	8112	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.5;
	8112	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.5;
	8112	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.8;
	8112	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.8;
	8112	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	8112	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.5;
	8113	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.5;
	8113	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134;
	8113	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194;
	8113	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.8;
	8113	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	8113	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.1;
	8114	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.1;
	8114	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130;
	8114	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.5;
	8114	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.2;
	8114	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	8114	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.4;
	8115	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.1;
	8115	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.5;
	8115	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.2;
	8115	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.8;
	8115	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.3;
	8115	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.8;
	8116	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	430.6;
	8116	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.6;
	8116	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.5;
	8116	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.3;
	8116	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.6;
	8116	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.7;
	8117	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.6;
	8117	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.1;
	8117	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.8;
	8117	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.8;
	8117	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	8117	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.8;
	8118	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.2;
	8118	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.5;
	8118	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.2;
	8118	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.9;
	8118	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	8118	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.2;
	8119	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.2;
	8119	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.2;
	8119	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.2;
	8119	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.1;
	8119	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	8119	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.5;
	8120	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547.8;
	8120	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.5;
	8120	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.5;
	8120	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.8;
	8120	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	8120	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.8;
	8121	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.8;
	8121	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.2;
	8121	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.7;
	8121	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.4;
	8121	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.1;
	8121	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.3;
	8122	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	534.9;
	8122	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.2;
	8122	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235;
	8122	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238;
	8122	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.4;
	8122	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.6;
	8123	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.3;
	8123	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.2;
	8123	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.9;
	8123	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238;
	8123	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	8123	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.5;
	8124	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526.3;
	8124	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.7;
	8124	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.4;
	8124	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.3;
	8124	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.5;
	8124	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.9;
	8125	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.8;
	8125	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.2;
	8125	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.6;
	8125	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.8;
	8125	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	8125	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.8;
	8126	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.9;
	8126	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.2;
	8126	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234;
	8126	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.3;
	8126	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	8126	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.7;
	8127	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.8;
	8127	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162;
	8127	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.9;
	8127	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.1;
	8127	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.5;
	8127	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387;
	8128	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.4;
	8128	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.2;
	8128	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.9;
	8128	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.5;
	8128	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	8128	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.2;
	8129	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.5;
	8129	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.7;
	8129	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.4;
	8129	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.1;
	8129	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	8129	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.5;
	8130	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550;
	8130	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.6;
	8130	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256;
	8130	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.8;
	8130	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	8130	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.6;
	8131	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.3;
	8131	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.9;
	8131	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251;
	8131	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.9;
	8131	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70;
	8131	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.7;
	8132	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.3;
	8132	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.9;
	8132	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.6;
	8132	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.1;
	8132	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	8132	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.8;
	8133	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.2;
	8133	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.5;
	8133	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.9;
	8133	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.3;
	8133	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.2;
	8133	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.9;
	8134	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.7;
	8134	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.4;
	8134	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.2;
	8134	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.6;
	8134	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	8134	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.9;
	8135	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.2;
	8135	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.2;
	8135	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.9;
	8135	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.3;
	8135	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.1;
	8135	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365;
	8136	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.2;
	8136	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.6;
	8136	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.9;
	8136	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.1;
	8136	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	8136	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.3;
	8137	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.6;
	8137	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	129.7;
	8137	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.4;
	8137	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.1;
	8137	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	8137	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.8;
	8138	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	420.7;
	8138	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.4;
	8138	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.9;
	8138	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.6;
	8138	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	8138	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.4;
	8139	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417.2;
	8139	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	125.6;
	8139	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	187.6;
	8139	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	189.4;
	8139	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.2;
	8139	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.2;
	8140	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434;
	8140	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	126.1;
	8140	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.4;
	8140	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	190.2;
	8140	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	8140	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.8;
	8141	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.9;
	8141	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.2;
	8141	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.5;
	8141	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.5;
	8141	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.4;
	8141	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.5;
	8142	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.9;
	8142	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.2;
	8142	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205;
	8142	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.6;
	8142	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.2;
	8142	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.4;
	8143	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.8;
	8143	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.5;
	8143	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.3;
	8143	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.4;
	8143	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	8143	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.8;
	8144	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	524.6;
	8144	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162;
	8144	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.8;
	8144	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.5;
	8144	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	8144	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.7;
	8145	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	534.4;
	8145	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.3;
	8145	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.5;
	8145	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235;
	8145	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70;
	8145	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.5;
	8146	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.8;
	8146	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.7;
	8146	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.1;
	8146	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.6;
	8146	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.9;
	8146	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.2;
	8147	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525.7;
	8147	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.4;
	8147	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.1;
	8147	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.6;
	8147	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.1;
	8147	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.1;
	8148	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.3;
	8148	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.9;
	8148	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.9;
	8148	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.3;
	8148	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.4;
	8148	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.2;
	8149	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.6;
	8149	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.4;
	8149	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.1;
	8149	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.6;
	8149	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.2;
	8149	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.9;
	8150	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.5;
	8150	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.4;
	8150	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.6;
	8150	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236;
	8150	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.9;
	8150	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383;
	8151	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.6;
	8151	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.5;
	8151	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.3;
	8151	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.9;
	8151	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	8151	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.2;
	8152	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.7;
	8152	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.4;
	8152	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.4;
	8152	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.7;
	8152	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	8152	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386;
	8153	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526.7;
	8153	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.2;
	8153	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252;
	8153	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.3;
	8153	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	8153	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.6;
	8154	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	546.6;
	8154	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.5;
	8154	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.2;
	8154	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.3;
	8154	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	8154	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.9;
	8155	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.8;
	8155	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.4;
	8155	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.1;
	8155	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.3;
	8155	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	8155	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.1;
	8156	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.9;
	8156	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.7;
	8156	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.6;
	8156	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.6;
	8156	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.1;
	8156	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.6;
	8157	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528.8;
	8157	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.3;
	8157	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.3;
	8157	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241;
	8157	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.6;
	8157	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.3;
	8158	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.1;
	8158	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.8;
	8158	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.9;
	8158	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.2;
	8158	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	8158	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.8;
	8159	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.4;
	8159	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.8;
	8159	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211;
	8159	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.2;
	8159	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	8159	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.5;
	8160	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.3;
	8160	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.9;
	8160	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.3;
	8160	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.4;
	8160	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	8160	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.5;
	8161	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.2;
	8161	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.9;
	8161	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.6;
	8161	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.1;
	8161	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	8161	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.1;
	8162	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.1;
	8162	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.4;
	8162	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.7;
	8162	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191.4;
	8162	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.9;
	8162	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.8;
	8163	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.1;
	8163	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	128.3;
	8163	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.3;
	8163	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	191;
	8163	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	8163	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.9;
	8164	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.9;
	8164	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	130.4;
	8164	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.3;
	8164	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.2;
	8164	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60;
	8164	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.1;
	8165	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.7;
	8165	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.4;
	8165	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.4;
	8165	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.3;
	8165	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	8165	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.2;
	8166	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.8;
	8166	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.9;
	8166	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.6;
	8166	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.4;
	8166	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.5;
	8166	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365;
	8167	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526.6;
	8167	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.3;
	8167	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.7;
	8167	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.3;
	8167	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	8167	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.4;
	8168	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.6;
	8168	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.8;
	8168	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.6;
	8168	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.2;
	8168	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	8168	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.7;
	8169	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547.4;
	8169	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.4;
	8169	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.9;
	8169	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.2;
	8169	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.4;
	8169	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.2;
	8170	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549.6;
	8170	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.6;
	8170	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.6;
	8170	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.9;
	8170	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.4;
	8170	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.7;
	8171	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	533.6;
	8171	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.4;
	8171	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231;
	8171	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.2;
	8171	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.7;
	8171	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.1;
	8172	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.3;
	8172	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.1;
	8172	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.8;
	8172	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.8;
	8172	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	8172	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.2;
	8173	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.1;
	8173	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.1;
	8173	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.9;
	8173	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.2;
	8173	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	8173	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.5;
	8174	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.1;
	8174	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.2;
	8174	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.4;
	8174	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.2;
	8174	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.9;
	8174	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.2;
	8175	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.4;
	8175	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.4;
	8175	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.3;
	8175	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.3;
	8175	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.1;
	8175	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.2;
	8176	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.9;
	8176	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.3;
	8176	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.9;
	8176	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.1;
	8176	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.3;
	8176	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.5;
	8177	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.3;
	8177	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.5;
	8177	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.3;
	8177	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	253.5;
	8177	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.4;
	8177	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.2;
	8178	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565;
	8178	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.8;
	8178	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.6;
	8178	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.9;
	8178	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.3;
	8178	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.8;
	8179	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	575.8;
	8179	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.3;
	8179	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.4;
	8179	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257;
	8179	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.8;
	8179	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.8;
	8180	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	568.3;
	8180	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185;
	8180	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.9;
	8180	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.3;
	8180	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.2;
	8180	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.6;
	8181	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.2;
	8181	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179;
	8181	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.4;
	8181	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.6;
	8181	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.5;
	8181	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.4;
	8182	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.9;
	8182	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.9;
	8182	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.4;
	8182	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.3;
	8182	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72;
	8182	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.2;
	8183	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.1;
	8183	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.5;
	8183	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.1;
	8183	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.5;
	8183	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.2;
	8183	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.1;
	8184	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.3;
	8184	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.3;
	8184	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.8;
	8184	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.6;
	8184	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66;
	8184	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.2;
	8185	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.2;
	8185	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.9;
	8185	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.5;
	8185	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.8;
	8185	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.9;
	8185	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.1;
	8186	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.2;
	8186	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.6;
	8186	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190;
	8186	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.5;
	8186	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.8;
	8186	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341;
	8187	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447.4;
	8187	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.6;
	8187	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.3;
	8187	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.1;
	8187	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.8;
	8187	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.8;
	8188	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.7;
	8188	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.7;
	8188	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.7;
	8188	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.3;
	8188	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.4;
	8188	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.5;
	8189	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.3;
	8189	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.5;
	8189	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.8;
	8189	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200;
	8189	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.7;
	8189	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.5;
	8190	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.3;
	8190	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.1;
	8190	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.1;
	8190	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.9;
	8190	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61;
	8190	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.8;
	8191	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.2;
	8191	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.3;
	8191	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.4;
	8191	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227;
	8191	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.5;
	8191	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388;
	8192	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.3;
	8192	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.3;
	8192	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.7;
	8192	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.5;
	8192	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	8192	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399;
	8193	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	554.3;
	8193	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.2;
	8193	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.9;
	8193	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.8;
	8193	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.8;
	8193	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.3;
	8194	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.4;
	8194	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.5;
	8194	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.1;
	8194	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.1;
	8194	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.7;
	8194	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.8;
	8195	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.5;
	8195	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.7;
	8195	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.7;
	8195	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.2;
	8195	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	8195	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.9;
	8196	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537;
	8196	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.5;
	8196	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.7;
	8196	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.9;
	8196	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	8196	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.7;
	8197	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.7;
	8197	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.5;
	8197	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.8;
	8197	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.9;
	8197	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.1;
	8197	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.4;
	8198	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	524.9;
	8198	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.4;
	8198	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.9;
	8198	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.7;
	8198	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	8198	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.1;
	8199	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.9;
	8199	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.9;
	8199	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.9;
	8199	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.8;
	8199	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	8199	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.7;
	8200	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.8;
	8200	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.2;
	8200	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.3;
	8200	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.6;
	8200	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	8200	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.8;
	8201	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	534.7;
	8201	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.5;
	8201	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.5;
	8201	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255;
	8201	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.8;
	8201	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.4;
	8202	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539;
	8202	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.6;
	8202	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.5;
	8202	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.4;
	8202	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.1;
	8202	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.9;
	8203	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.7;
	8203	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.2;
	8203	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249;
	8203	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.5;
	8203	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.5;
	8203	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.9;
	8204	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	546.6;
	8204	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.6;
	8204	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.2;
	8204	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.1;
	8204	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.3;
	8204	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.7;
	8205	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526.5;
	8205	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.7;
	8205	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.3;
	8205	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.9;
	8205	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.5;
	8205	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.4;
	8206	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.5;
	8206	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.9;
	8206	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.8;
	8206	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.3;
	8206	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73;
	8206	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.5;
	8207	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.2;
	8207	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.5;
	8207	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.4;
	8207	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.4;
	8207	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	8207	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.4;
	8208	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.8;
	8208	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.6;
	8208	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210;
	8208	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.8;
	8208	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.6;
	8208	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.6;
	8209	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	441.9;
	8209	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.1;
	8209	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.4;
	8209	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.5;
	8209	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.3;
	8209	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.9;
	8210	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428.3;
	8210	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.4;
	8210	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191;
	8210	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.2;
	8210	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	8210	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.6;
	8211	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428.5;
	8211	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.8;
	8211	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.7;
	8211	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.1;
	8211	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	8211	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.5;
	8212	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.6;
	8212	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.4;
	8212	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.3;
	8212	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.8;
	8212	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	8212	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.1;
	8213	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.5;
	8213	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.8;
	8213	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.8;
	8213	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.3;
	8213	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	8213	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.5;
	8214	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.5;
	8214	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.4;
	8214	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.4;
	8214	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.7;
	8214	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.5;
	8214	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.8;
	8215	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.3;
	8215	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149;
	8215	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.2;
	8215	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.5;
	8215	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.9;
	8215	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.5;
	8216	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.8;
	8216	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149;
	8216	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.4;
	8216	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.1;
	8216	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.3;
	8216	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.6;
	8217	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.4;
	8217	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.4;
	8217	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.1;
	8217	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.7;
	8217	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78;
	8217	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.8;
	8218	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.1;
	8218	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.1;
	8218	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.1;
	8218	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.3;
	8218	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.3;
	8218	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.6;
	8219	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.6;
	8219	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.7;
	8219	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.9;
	8219	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217;
	8219	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.2;
	8219	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.4;
	8220	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.6;
	8220	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.6;
	8220	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.4;
	8220	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.2;
	8220	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.2;
	8220	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.1;
	8221	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.9;
	8221	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.6;
	8221	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.4;
	8221	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.3;
	8221	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.6;
	8221	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.9;
	8222	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.9;
	8222	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.2;
	8222	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.4;
	8222	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.7;
	8222	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.8;
	8222	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.9;
	8223	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.2;
	8223	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.5;
	8223	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.4;
	8223	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.6;
	8223	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72;
	8223	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.1;
	8224	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.4;
	8224	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.4;
	8224	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.7;
	8224	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223;
	8224	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.1;
	8224	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.6;
	8225	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.5;
	8225	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.9;
	8225	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.9;
	8225	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.3;
	8225	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.3;
	8225	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384;
	8226	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532;
	8226	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177;
	8226	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238;
	8226	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.8;
	8226	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.4;
	8226	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.1;
	8227	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	547.1;
	8227	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.4;
	8227	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.4;
	8227	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.1;
	8227	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77;
	8227	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.4;
	8228	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.7;
	8228	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.3;
	8228	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233;
	8228	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.7;
	8228	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.4;
	8228	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.8;
	8229	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.8;
	8229	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.6;
	8229	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.5;
	8229	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.7;
	8229	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78;
	8229	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.9;
	8230	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.6;
	8230	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.2;
	8230	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.4;
	8230	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.7;
	8230	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76;
	8230	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.6;
	8231	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	473.4;
	8231	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.3;
	8231	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.9;
	8231	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.8;
	8231	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75;
	8231	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.1;
	8232	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.7;
	8232	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.4;
	8232	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.5;
	8232	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.1;
	8232	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.4;
	8232	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.1;
	8233	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.9;
	8233	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.3;
	8233	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.8;
	8233	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.5;
	8233	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.9;
	8233	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342;
	8234	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	417.8;
	8234	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135;
	8234	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.1;
	8234	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.1;
	8234	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.6;
	8234	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.4;
	8235	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423;
	8235	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.5;
	8235	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.3;
	8235	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.4;
	8235	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.1;
	8235	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.1;
	8236	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.9;
	8236	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.2;
	8236	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.8;
	8236	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.9;
	8236	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.8;
	8236	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.2;
	8237	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.8;
	8237	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.2;
	8237	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.4;
	8237	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.6;
	8237	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.1;
	8237	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.2;
	8238	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.9;
	8238	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.2;
	8238	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.3;
	8238	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.6;
	8238	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.6;
	8238	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.2;
	8239	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.1;
	8239	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.7;
	8239	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.6;
	8239	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.6;
	8239	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.9;
	8239	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.1;
	8240	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465.3;
	8240	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153;
	8240	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.5;
	8240	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.1;
	8240	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.6;
	8240	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.4;
	8241	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474;
	8241	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.2;
	8241	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.2;
	8241	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.7;
	8241	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81;
	8241	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.7;
	8242	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.4;
	8242	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165;
	8242	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.3;
	8242	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.9;
	8242	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.8;
	8242	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.3;
	8243	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.9;
	8243	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.6;
	8243	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.3;
	8243	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.7;
	8243	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.3;
	8243	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.7;
	8244	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464;
	8244	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164;
	8244	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.6;
	8244	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.9;
	8244	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80;
	8244	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.4;
	8245	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.7;
	8245	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.4;
	8245	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.7;
	8245	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.2;
	8245	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.9;
	8245	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.4;
	8246	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.2;
	8246	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.2;
	8246	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.2;
	8246	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.5;
	8246	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.4;
	8246	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.6;
	8247	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458;
	8247	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.3;
	8247	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.9;
	8247	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.1;
	8247	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.6;
	8247	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.3;
	8248	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.5;
	8248	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.2;
	8248	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.2;
	8248	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.9;
	8248	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.1;
	8248	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.1;
	8249	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.3;
	8249	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.1;
	8249	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.2;
	8249	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238;
	8249	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.6;
	8249	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376;
	8250	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526.7;
	8250	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.2;
	8250	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.9;
	8250	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.4;
	8250	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.4;
	8250	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.1;
	8251	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.2;
	8251	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.5;
	8251	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.7;
	8251	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.9;
	8251	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.6;
	8251	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.8;
	8252	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.4;
	8252	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.5;
	8252	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.2;
	8252	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.7;
	8252	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.5;
	8252	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395;
	8253	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512;
	8253	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.4;
	8253	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237;
	8253	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.3;
	8253	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.8;
	8253	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.1;
	8254	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.3;
	8254	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.4;
	8254	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.4;
	8254	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.4;
	8254	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80;
	8254	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.7;
	8255	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.7;
	8255	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.7;
	8255	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.6;
	8255	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.1;
	8255	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.7;
	8255	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359;
	8256	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.3;
	8256	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.7;
	8256	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.5;
	8256	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.1;
	8256	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.7;
	8256	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	346.7;
	8257	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424;
	8257	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.1;
	8257	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.4;
	8257	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.3;
	8257	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.2;
	8257	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.9;
	8258	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.2;
	8258	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.5;
	8258	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.2;
	8258	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.9;
	8258	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.6;
	8258	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.3;
	8259	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.6;
	8259	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.9;
	8259	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.6;
	8259	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.3;
	8259	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.6;
	8259	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.8;
	8260	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	420.3;
	8260	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.3;
	8260	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.3;
	8260	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196;
	8260	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.6;
	8260	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.2;
	8261	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	429.8;
	8261	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.4;
	8261	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.9;
	8261	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.1;
	8261	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	8261	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.5;
	8262	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.3;
	8262	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156;
	8262	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.6;
	8262	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.4;
	8262	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.6;
	8262	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.4;
	8263	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	555.3;
	8263	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.6;
	8263	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.7;
	8263	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.5;
	8263	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.9;
	8263	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.7;
	8264	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.5;
	8264	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.8;
	8264	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.7;
	8264	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.3;
	8264	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.7;
	8264	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.1;
	8265	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.5;
	8265	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.3;
	8265	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.4;
	8265	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.3;
	8265	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77;
	8265	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.9;
	8266	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532;
	8266	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.6;
	8266	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.5;
	8266	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.1;
	8266	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.1;
	8266	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.2;
	8267	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.3;
	8267	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.1;
	8267	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.6;
	8267	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.2;
	8267	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77;
	8267	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.5;
	8268	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.8;
	8268	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.4;
	8268	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.9;
	8268	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.4;
	8268	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.7;
	8268	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.2;
	8269	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.3;
	8269	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.4;
	8269	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.5;
	8269	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.4;
	8269	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.1;
	8269	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.6;
	8270	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522.3;
	8270	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.2;
	8270	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.3;
	8270	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.8;
	8270	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.9;
	8270	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.6;
	8271	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.2;
	8271	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.9;
	8271	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.9;
	8271	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.7;
	8271	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.9;
	8271	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.3;
	8272	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.1;
	8272	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.7;
	8272	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.7;
	8272	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.3;
	8272	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.5;
	8272	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.4;
	8273	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.1;
	8273	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.9;
	8273	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.2;
	8273	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.9;
	8273	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	8273	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.4;
	8274	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564.5;
	8274	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.1;
	8274	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.7;
	8274	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.5;
	8274	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.3;
	8274	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.9;
	8275	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	571.1;
	8275	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195;
	8275	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258;
	8275	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.8;
	8275	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.6;
	8275	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.2;
	8276	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	568.6;
	8276	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.1;
	8276	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.9;
	8276	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.4;
	8276	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.5;
	8276	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.4;
	8277	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	553.8;
	8277	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.1;
	8277	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.8;
	8277	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.2;
	8277	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.3;
	8277	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.7;
	8278	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	533.1;
	8278	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.9;
	8278	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.8;
	8278	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.7;
	8278	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.4;
	8278	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.5;
	8279	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509.8;
	8279	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.7;
	8279	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.7;
	8279	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.9;
	8279	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.7;
	8279	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.4;
	8280	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.2;
	8280	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.4;
	8280	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.9;
	8280	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.1;
	8280	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.6;
	8280	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.4;
	8281	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.5;
	8281	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.5;
	8281	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.9;
	8281	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.6;
	8281	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.4;
	8281	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.1;
	8282	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.3;
	8282	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.4;
	8282	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.2;
	8282	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.3;
	8282	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.9;
	8282	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.2;
	8283	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.8;
	8283	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.7;
	8283	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.6;
	8283	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.8;
	8283	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73;
	8283	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.2;
	8284	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.6;
	8284	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.4;
	8284	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.7;
	8284	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.3;
	8284	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.8;
	8284	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.5;
	8285	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	469;
	8285	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.8;
	8285	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.1;
	8285	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.1;
	8285	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.2;
	8285	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355;
	8286	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.5;
	8286	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.1;
	8286	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.3;
	8286	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.6;
	8286	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.8;
	8286	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.5;
	8287	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.5;
	8287	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.1;
	8287	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.8;
	8287	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.8;
	8287	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.8;
	8287	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.5;
	8288	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	567.5;
	8288	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.3;
	8288	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.3;
	8288	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.3;
	8288	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.6;
	8288	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.5;
	8289	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.6;
	8289	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178;
	8289	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.2;
	8289	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.2;
	8289	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.5;
	8289	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410;
	8290	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.8;
	8290	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.6;
	8290	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.8;
	8290	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.5;
	8290	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.2;
	8290	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.7;
	8291	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548;
	8291	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.1;
	8291	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.9;
	8291	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.6;
	8291	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.6;
	8291	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.2;
	8292	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.1;
	8292	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.6;
	8292	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.9;
	8292	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.8;
	8292	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.8;
	8292	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.8;
	8293	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	534.3;
	8293	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.8;
	8293	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.4;
	8293	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.5;
	8293	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.1;
	8293	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.9;
	8294	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.2;
	8294	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.6;
	8294	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.8;
	8294	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.7;
	8294	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.6;
	8294	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.3;
	8295	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.1;
	8295	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.3;
	8295	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.8;
	8295	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.7;
	8295	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.5;
	8295	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.8;
	8296	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.8;
	8296	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.7;
	8296	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.7;
	8296	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.7;
	8296	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.1;
	8296	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.6;
	8297	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.1;
	8297	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.7;
	8297	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.4;
	8297	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	260;
	8297	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.5;
	8297	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.4;
	8298	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.4;
	8298	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	200.1;
	8298	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	264.1;
	8298	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267;
	8298	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.6;
	8298	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	424.7;
	8299	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.1;
	8299	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	199.2;
	8299	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.4;
	8299	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.7;
	8299	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78;
	8299	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	429.3;
	8300	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	591.4;
	8300	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.5;
	8300	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.2;
	8300	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.1;
	8300	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.4;
	8300	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	425.1;
	8301	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.2;
	8301	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.6;
	8301	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.8;
	8301	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.6;
	8301	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.1;
	8301	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.3;
	8302	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551;
	8302	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.4;
	8302	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.8;
	8302	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.1;
	8302	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.6;
	8302	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.4;
	8303	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526.8;
	8303	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163;
	8303	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218;
	8303	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220;
	8303	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75;
	8303	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.9;
	8304	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.5;
	8304	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154.7;
	8304	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.7;
	8304	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.3;
	8304	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.5;
	8304	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369;
	8305	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.9;
	8305	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.4;
	8305	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.8;
	8305	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.1;
	8305	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	8305	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.7;
	8306	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.3;
	8306	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.6;
	8306	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193;
	8306	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.5;
	8306	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	8306	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.2;
	8307	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.1;
	8307	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143;
	8307	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.8;
	8307	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.3;
	8307	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	8307	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344;
	8308	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.3;
	8308	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.4;
	8308	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.5;
	8308	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.2;
	8308	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	8308	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.2;
	8309	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.4;
	8309	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.7;
	8309	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.9;
	8309	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.7;
	8309	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	8309	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.6;
	8310	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529;
	8310	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.2;
	8310	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.6;
	8310	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.9;
	8310	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	8310	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.8;
	8311	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.5;
	8311	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.8;
	8311	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.2;
	8311	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.7;
	8311	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	8311	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.3;
	8312	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	574.7;
	8312	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.4;
	8312	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.8;
	8312	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.5;
	8312	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	8312	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.8;
	8313	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	575.6;
	8313	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.4;
	8313	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237;
	8313	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.4;
	8313	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.7;
	8313	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.4;
	8314	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	561.6;
	8314	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.1;
	8314	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.9;
	8314	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.7;
	8314	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75;
	8314	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.9;
	8315	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	557.4;
	8315	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.7;
	8315	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.2;
	8315	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.9;
	8315	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75;
	8315	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.2;
	8316	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.5;
	8316	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.5;
	8316	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.2;
	8316	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.9;
	8316	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.7;
	8316	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.1;
	8317	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.4;
	8317	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.6;
	8317	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.5;
	8317	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.1;
	8317	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.8;
	8317	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.5;
	8318	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.3;
	8318	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172;
	8318	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.2;
	8318	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.2;
	8318	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.8;
	8318	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.5;
	8319	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537;
	8319	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.9;
	8319	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.4;
	8319	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.1;
	8319	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.9;
	8319	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.6;
	8320	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	533.3;
	8320	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.1;
	8320	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.3;
	8320	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.9;
	8320	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.1;
	8320	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.4;
	8321	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.8;
	8321	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.2;
	8321	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.5;
	8321	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261;
	8321	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.1;
	8321	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.2;
	8322	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577.5;
	8322	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	199.4;
	8322	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.2;
	8322	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.2;
	8322	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.4;
	8322	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.2;
	8323	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586.1;
	8323	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	199.8;
	8323	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	259.8;
	8323	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262;
	8323	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74;
	8323	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	424.6;
	8324	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	575.7;
	8324	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.9;
	8324	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.3;
	8324	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.7;
	8324	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.1;
	8324	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418;
	8325	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564;
	8325	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189;
	8325	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.3;
	8325	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.1;
	8325	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.1;
	8325	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.1;
	8326	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.3;
	8326	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.4;
	8326	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.6;
	8326	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.8;
	8326	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74;
	8326	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.8;
	8327	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.4;
	8327	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.6;
	8327	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.5;
	8327	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.7;
	8327	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.1;
	8327	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.9;
	8328	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.7;
	8328	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.5;
	8328	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.5;
	8328	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.4;
	8328	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	8328	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.9;
	8329	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.1;
	8329	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.6;
	8329	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.7;
	8329	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.2;
	8329	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	8329	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.7;
	8330	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.8;
	8330	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.4;
	8330	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.3;
	8330	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.3;
	8330	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	8330	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.7;
	8331	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.9;
	8331	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.9;
	8331	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.8;
	8331	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.3;
	8331	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.4;
	8331	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.5;
	8332	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	468.9;
	8332	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.1;
	8332	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.7;
	8332	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.5;
	8332	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	8332	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.6;
	8333	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479;
	8333	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.3;
	8333	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.9;
	8333	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.6;
	8333	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	8333	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.7;
	8334	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	514.8;
	8334	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.1;
	8334	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.6;
	8334	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.9;
	8334	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	8334	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.8;
	8335	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.2;
	8335	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.5;
	8335	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228;
	8335	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.4;
	8335	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.3;
	8335	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.9;
	8336	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.1;
	8336	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.4;
	8336	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236;
	8336	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.3;
	8336	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.3;
	8336	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.9;
	8337	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.9;
	8337	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.9;
	8337	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.8;
	8337	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241;
	8337	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.7;
	8337	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.4;
	8338	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	557.3;
	8338	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.9;
	8338	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.2;
	8338	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.7;
	8338	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79;
	8338	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.7;
	8339	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.1;
	8339	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177;
	8339	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.9;
	8339	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.2;
	8339	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79;
	8339	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.3;
	8340	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.1;
	8340	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.1;
	8340	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.3;
	8340	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.2;
	8340	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.5;
	8340	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.2;
	8341	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	536.4;
	8341	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.3;
	8341	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.2;
	8341	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.6;
	8341	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.8;
	8341	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.4;
	8342	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.3;
	8342	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.6;
	8342	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.3;
	8342	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.3;
	8342	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.8;
	8342	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.9;
	8343	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	540.6;
	8343	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.6;
	8343	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.5;
	8343	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.3;
	8343	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	8343	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.5;
	8344	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.2;
	8344	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.9;
	8344	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.9;
	8344	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.6;
	8344	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	8344	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.7;
	8345	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.1;
	8345	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193;
	8345	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.8;
	8345	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.9;
	8345	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	8345	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.3;
	8346	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	583.6;
	8346	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	199.3;
	8346	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	261.8;
	8346	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264;
	8346	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.6;
	8346	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	424;
	8347	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.1;
	8347	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	200;
	8347	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	259.4;
	8347	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.1;
	8347	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.7;
	8347	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	429;
	8348	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.3;
	8348	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.7;
	8348	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.6;
	8348	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.3;
	8348	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.4;
	8348	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.8;
	8349	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.5;
	8349	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	192.8;
	8349	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.2;
	8349	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.6;
	8349	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.3;
	8349	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414;
	8350	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	553.5;
	8350	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.4;
	8350	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.3;
	8350	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.2;
	8350	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.7;
	8350	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.3;
	8351	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522.1;
	8351	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.2;
	8351	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.9;
	8351	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.5;
	8351	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.5;
	8351	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385;
	8352	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500;
	8352	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.5;
	8352	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.1;
	8352	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.6;
	8352	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	8352	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	372.6;
	8353	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.2;
	8353	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.2;
	8353	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.2;
	8353	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.9;
	8353	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	8353	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.9;
	8354	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.3;
	8354	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.4;
	8354	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.9;
	8354	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.8;
	8354	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.2;
	8354	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.9;
	8355	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.1;
	8355	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.4;
	8355	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.7;
	8355	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.5;
	8355	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	8355	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.3;
	8356	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.6;
	8356	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.5;
	8356	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.1;
	8356	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198;
	8356	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.1;
	8356	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.9;
	8357	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.9;
	8357	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154;
	8357	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.9;
	8357	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204;
	8357	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.7;
	8357	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.1;
	8358	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.3;
	8358	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.8;
	8358	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.7;
	8358	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.5;
	8358	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	8358	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.6;
	8359	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549.2;
	8359	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181;
	8359	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.5;
	8359	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.7;
	8359	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.1;
	8359	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.2;
	8360	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	578.2;
	8360	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185;
	8360	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.5;
	8360	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.6;
	8360	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.7;
	8360	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.3;
	8361	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572;
	8361	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.2;
	8361	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.2;
	8361	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.1;
	8361	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.8;
	8361	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.3;
	8362	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589.5;
	8362	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180;
	8362	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.6;
	8362	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.1;
	8362	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.4;
	8362	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.7;
	8363	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	578.1;
	8363	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.6;
	8363	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.9;
	8363	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.7;
	8363	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.1;
	8363	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.2;
	8364	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.2;
	8364	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.3;
	8364	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.6;
	8364	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.6;
	8364	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.2;
	8364	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407;
	8365	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	563.1;
	8365	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.5;
	8365	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.4;
	8365	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.9;
	8365	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.5;
	8365	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.7;
	8366	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	557.8;
	8366	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.5;
	8366	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.2;
	8366	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.6;
	8366	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.3;
	8366	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.5;
	8367	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548.7;
	8367	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.4;
	8367	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.3;
	8367	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.8;
	8367	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.1;
	8367	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.2;
	8368	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541.7;
	8368	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.6;
	8368	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240;
	8368	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.2;
	8368	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69;
	8368	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.2;
	8369	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.5;
	8369	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.7;
	8369	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.5;
	8369	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.9;
	8369	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.8;
	8369	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.8;
	8370	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	578.7;
	8370	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.2;
	8370	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.9;
	8370	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	263.3;
	8370	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.2;
	8370	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.6;
	8371	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.8;
	8371	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191;
	8371	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.1;
	8371	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.5;
	8371	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.2;
	8371	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.9;
	8372	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	581.8;
	8372	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.9;
	8372	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.3;
	8372	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.4;
	8372	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.7;
	8372	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.1;
	8373	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.9;
	8373	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.9;
	8373	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.2;
	8373	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.3;
	8373	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.4;
	8373	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.5;
	8374	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	540.3;
	8374	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.4;
	8374	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.1;
	8374	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.6;
	8374	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.1;
	8374	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.9;
	8375	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518.1;
	8375	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.8;
	8375	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222;
	8375	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.5;
	8375	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73;
	8375	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.9;
	8376	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.1;
	8376	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158;
	8376	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.2;
	8376	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.3;
	8376	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.3;
	8376	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.6;
	8377	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.2;
	8377	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.3;
	8377	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.8;
	8377	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204;
	8377	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	8377	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.4;
	8378	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461.4;
	8378	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.5;
	8378	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.2;
	8378	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.8;
	8378	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	8378	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.7;
	8379	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457;
	8379	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.7;
	8379	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.1;
	8379	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.8;
	8379	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.6;
	8379	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.9;
	8380	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.9;
	8380	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.7;
	8380	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.6;
	8380	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.3;
	8380	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64;
	8380	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.8;
	8381	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.3;
	8381	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.2;
	8381	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.9;
	8381	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.6;
	8381	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	8381	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.8;
	8382	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.6;
	8382	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.2;
	8382	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.8;
	8382	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207.3;
	8382	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68;
	8382	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.5;
	8383	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.2;
	8383	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.9;
	8383	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210;
	8383	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.1;
	8383	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	8383	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.8;
	8384	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520;
	8384	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.1;
	8384	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.3;
	8384	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.9;
	8384	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.2;
	8384	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.7;
	8385	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528;
	8385	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.3;
	8385	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.7;
	8385	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.9;
	8385	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.3;
	8385	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.8;
	8386	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.1;
	8386	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.4;
	8386	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.7;
	8386	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.7;
	8386	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.2;
	8386	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.8;
	8387	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.5;
	8387	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.7;
	8387	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.4;
	8387	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.7;
	8387	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.6;
	8387	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.9;
	8388	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	535.5;
	8388	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174;
	8388	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.8;
	8388	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.2;
	8388	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.4;
	8388	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.5;
	8389	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.3;
	8389	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.9;
	8389	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.7;
	8389	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.9;
	8389	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.4;
	8389	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.7;
	8390	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	528.3;
	8390	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170;
	8390	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.1;
	8390	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.1;
	8390	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.5;
	8390	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.6;
	8391	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522.3;
	8391	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.3;
	8391	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.5;
	8391	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.9;
	8391	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.7;
	8391	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385;
	8392	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.5;
	8392	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.9;
	8392	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.4;
	8392	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.4;
	8392	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.4;
	8392	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.7;
	8393	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.8;
	8393	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.1;
	8393	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.2;
	8393	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.5;
	8393	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.1;
	8393	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.4;
	8394	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.7;
	8394	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.3;
	8394	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	245.9;
	8394	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.9;
	8394	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.8;
	8394	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.4;
	8395	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564.6;
	8395	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.6;
	8395	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.4;
	8395	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.1;
	8395	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.9;
	8395	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.2;
	8396	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	553.5;
	8396	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.3;
	8396	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.5;
	8396	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.2;
	8396	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.1;
	8396	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405;
	8397	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.2;
	8397	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.6;
	8397	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.1;
	8397	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.7;
	8397	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96;
	8397	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	396.4;
	8398	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	510.6;
	8398	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.8;
	8398	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229;
	8398	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.5;
	8398	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.9;
	8398	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.4;
	8399	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476;
	8399	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158;
	8399	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.8;
	8399	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.1;
	8399	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	92.1;
	8399	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.6;
	8400	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454.8;
	8400	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.7;
	8400	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.9;
	8400	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219;
	8400	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.5;
	8400	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.4;
	8401	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.7;
	8401	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.2;
	8401	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.8;
	8401	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203;
	8401	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.1;
	8401	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.9;
	8402	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	402.9;
	8402	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.1;
	8402	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.3;
	8402	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.5;
	8402	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.9;
	8402	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.9;
	8403	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.9;
	8403	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.1;
	8403	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.4;
	8403	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195;
	8403	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.4;
	8403	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333;
	8404	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	415.7;
	8404	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.4;
	8404	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.5;
	8404	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.2;
	8404	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.2;
	8404	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.6;
	8405	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.3;
	8405	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.4;
	8405	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.8;
	8405	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.6;
	8405	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.4;
	8405	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.4;
	8406	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.1;
	8406	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.2;
	8406	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.4;
	8406	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.6;
	8406	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.7;
	8406	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.3;
	8407	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.4;
	8407	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.6;
	8407	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.2;
	8407	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.4;
	8407	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.8;
	8407	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.4;
	8408	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.7;
	8408	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.4;
	8408	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.2;
	8408	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204.2;
	8408	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.3;
	8408	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.1;
	8409	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.5;
	8409	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.5;
	8409	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.8;
	8409	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.1;
	8409	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96.2;
	8409	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.7;
	8410	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.8;
	8410	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166;
	8410	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.2;
	8410	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.1;
	8410	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	96.6;
	8410	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.3;
	8411	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.6;
	8411	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.1;
	8411	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.3;
	8411	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.9;
	8411	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	95.2;
	8411	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.7;
	8412	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.2;
	8412	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.2;
	8412	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.7;
	8412	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.2;
	8412	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	93.1;
	8412	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.2;
	8413	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.5;
	8413	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.8;
	8413	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.9;
	8413	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.6;
	8413	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	91.8;
	8413	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.1;
	8414	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.3;
	8414	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.2;
	8414	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.7;
	8414	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.9;
	8414	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.2;
	8414	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.8;
	8415	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.5;
	8415	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164;
	8415	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.1;
	8415	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.5;
	8415	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89;
	8415	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.3;
	8416	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.4;
	8416	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.9;
	8416	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.8;
	8416	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.9;
	8416	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87;
	8416	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373;
	8417	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518.6;
	8417	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.6;
	8417	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.3;
	8417	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244;
	8417	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	88.5;
	8417	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.6;
	8418	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.1;
	8418	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.3;
	8418	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.4;
	8418	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249;
	8418	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.4;
	8418	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.9;
	8419	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.5;
	8419	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187;
	8419	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.8;
	8419	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.2;
	8419	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	90.6;
	8419	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.3;
	8420	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.2;
	8420	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.7;
	8420	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.6;
	8420	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.6;
	8420	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.9;
	8420	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.7;
	8421	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	524;
	8421	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.9;
	8421	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239;
	8421	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.8;
	8421	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.6;
	8421	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.5;
	8422	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.8;
	8422	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.3;
	8422	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229;
	8422	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.6;
	8422	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.7;
	8422	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.8;
	8423	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.7;
	8423	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.5;
	8423	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.9;
	8423	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.6;
	8423	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84.7;
	8423	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.6;
	8424	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.3;
	8424	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.8;
	8424	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.4;
	8424	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.1;
	8424	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.8;
	8424	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358;
	8425	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.5;
	8425	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.7;
	8425	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.6;
	8425	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.6;
	8425	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.8;
	8425	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.1;
	8426	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.9;
	8426	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.9;
	8426	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.9;
	8426	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.9;
	8426	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.6;
	8426	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.2;
	8427	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.3;
	8427	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.4;
	8427	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.7;
	8427	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.1;
	8427	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.7;
	8427	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	339.2;
	8428	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.5;
	8428	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	135.3;
	8428	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	188.9;
	8428	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.4;
	8428	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.9;
	8428	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.2;
	8429	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.6;
	8429	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.6;
	8429	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.5;
	8429	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.6;
	8429	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.9;
	8429	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.1;
	8430	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.4;
	8430	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.3;
	8430	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.5;
	8430	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.1;
	8430	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.9;
	8430	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.4;
	8431	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.3;
	8431	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.7;
	8431	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.4;
	8431	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225;
	8431	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.1;
	8431	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.3;
	8432	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548.9;
	8432	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.6;
	8432	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.5;
	8432	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.5;
	8432	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.1;
	8432	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.7;
	8433	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	555.1;
	8433	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.3;
	8433	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.7;
	8433	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.7;
	8433	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.2;
	8433	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.8;
	8434	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.4;
	8434	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.9;
	8434	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.8;
	8434	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.2;
	8434	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.1;
	8434	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408;
	8435	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.3;
	8435	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.1;
	8435	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.2;
	8435	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.2;
	8435	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.4;
	8435	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407;
	8436	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.2;
	8436	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.5;
	8436	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.5;
	8436	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.3;
	8436	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.9;
	8436	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.5;
	8437	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	554.8;
	8437	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.6;
	8437	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.7;
	8437	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.6;
	8437	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.5;
	8437	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.8;
	8438	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537.3;
	8438	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.8;
	8438	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237;
	8438	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.3;
	8438	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.8;
	8438	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397;
	8439	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537.8;
	8439	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.6;
	8439	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.5;
	8439	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.1;
	8439	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80;
	8439	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.2;
	8440	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.7;
	8440	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.7;
	8440	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.5;
	8440	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.8;
	8440	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.6;
	8440	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.1;
	8441	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.6;
	8441	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.2;
	8441	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.2;
	8441	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.4;
	8441	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.6;
	8441	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.2;
	8442	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	578.2;
	8442	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.1;
	8442	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.1;
	8442	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265.7;
	8442	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.2;
	8442	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422;
	8443	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	580.2;
	8443	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.4;
	8443	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.9;
	8443	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265.6;
	8443	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.2;
	8443	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.2;
	8444	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584;
	8444	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.3;
	8444	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.8;
	8444	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.6;
	8444	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.6;
	8444	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.4;
	8445	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.8;
	8445	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.5;
	8445	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.9;
	8445	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.3;
	8445	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.4;
	8445	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	415.3;
	8446	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.2;
	8446	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174;
	8446	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.6;
	8446	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.3;
	8446	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.1;
	8446	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.1;
	8447	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.6;
	8447	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.8;
	8447	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.3;
	8447	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.9;
	8447	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.3;
	8447	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.5;
	8448	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.6;
	8448	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.3;
	8448	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.2;
	8448	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.5;
	8448	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	8448	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.2;
	8449	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.6;
	8449	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.2;
	8449	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.5;
	8449	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.7;
	8449	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	8449	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.5;
	8450	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435;
	8450	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.5;
	8450	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.8;
	8450	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.1;
	8450	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	8450	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.5;
	8451	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.7;
	8451	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.5;
	8451	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.8;
	8451	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.8;
	8451	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.6;
	8451	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.5;
	8452	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.4;
	8452	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.5;
	8452	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.3;
	8452	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.1;
	8452	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	8452	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.1;
	8453	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.5;
	8453	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.8;
	8453	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.9;
	8453	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.3;
	8453	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.9;
	8453	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.6;
	8454	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484.8;
	8454	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.1;
	8454	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.1;
	8454	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213;
	8454	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.2;
	8454	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.6;
	8455	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.3;
	8455	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.8;
	8455	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.6;
	8455	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.5;
	8455	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.5;
	8455	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.6;
	8456	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.6;
	8456	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.3;
	8456	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.6;
	8456	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.6;
	8456	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.8;
	8456	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.6;
	8457	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566.1;
	8457	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.9;
	8457	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.2;
	8457	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.1;
	8457	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63;
	8457	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.9;
	8458	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552.9;
	8458	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.8;
	8458	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.6;
	8458	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.1;
	8458	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.1;
	8458	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.4;
	8459	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	546.1;
	8459	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.2;
	8459	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.9;
	8459	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.7;
	8459	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	8459	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.5;
	8460	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.2;
	8460	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.3;
	8460	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.4;
	8460	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.4;
	8460	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	8460	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.6;
	8461	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.5;
	8461	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.3;
	8461	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.2;
	8461	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.9;
	8461	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	8461	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.7;
	8462	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.2;
	8462	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.6;
	8462	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237;
	8462	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.8;
	8462	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.3;
	8462	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.9;
	8463	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	533.8;
	8463	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.6;
	8463	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237;
	8463	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.2;
	8463	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.2;
	8463	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.7;
	8464	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	533.7;
	8464	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.1;
	8464	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.9;
	8464	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.5;
	8464	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	8464	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.6;
	8465	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	557.2;
	8465	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.3;
	8465	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258.5;
	8465	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.7;
	8465	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	8465	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.1;
	8466	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	575.3;
	8466	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198.3;
	8466	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	264.3;
	8466	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	267;
	8466	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	8466	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.2;
	8467	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579;
	8467	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.2;
	8467	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.6;
	8467	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	263.4;
	8467	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.1;
	8467	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.7;
	8468	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.4;
	8468	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.9;
	8468	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.4;
	8468	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.1;
	8468	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	8468	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.1;
	8469	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564.1;
	8469	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.8;
	8469	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.6;
	8469	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250;
	8469	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.9;
	8469	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.6;
	8470	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.5;
	8470	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.5;
	8470	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.4;
	8470	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.6;
	8470	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.3;
	8470	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.1;
	8471	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.4;
	8471	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.9;
	8471	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	219.4;
	8471	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.1;
	8471	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71;
	8471	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.3;
	8472	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482;
	8472	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.1;
	8472	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.2;
	8472	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.7;
	8472	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.8;
	8472	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.1;
	8473	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.2;
	8473	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.1;
	8473	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.2;
	8473	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.2;
	8473	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	8473	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.4;
	8474	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	433.9;
	8474	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.1;
	8474	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.4;
	8474	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.2;
	8474	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	8474	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.6;
	8475	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447.4;
	8475	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.9;
	8475	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.9;
	8475	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.2;
	8475	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.9;
	8475	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343.9;
	8476	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.2;
	8476	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.8;
	8476	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.8;
	8476	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.9;
	8476	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	8476	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.2;
	8477	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478;
	8477	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.3;
	8477	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.7;
	8477	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.1;
	8477	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	8477	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.1;
	8478	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.5;
	8478	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.1;
	8478	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.9;
	8478	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	213.9;
	8478	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.9;
	8478	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.6;
	8479	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	541;
	8479	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172;
	8479	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.6;
	8479	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.7;
	8479	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	8479	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.9;
	8480	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.6;
	8480	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.5;
	8480	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.9;
	8480	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238;
	8480	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.5;
	8480	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.7;
	8481	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564.5;
	8481	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.1;
	8481	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.4;
	8481	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.1;
	8481	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.9;
	8481	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.8;
	8482	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.6;
	8482	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.5;
	8482	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.2;
	8482	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.9;
	8482	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82;
	8482	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	405.4;
	8483	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	549.9;
	8483	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.7;
	8483	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.7;
	8483	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.7;
	8483	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82;
	8483	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.3;
	8484	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.7;
	8484	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.3;
	8484	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.4;
	8484	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.4;
	8484	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.1;
	8484	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.3;
	8485	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.7;
	8485	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.2;
	8485	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.6;
	8485	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.8;
	8485	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.6;
	8485	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.3;
	8486	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.4;
	8486	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.1;
	8486	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.9;
	8486	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236;
	8486	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.9;
	8486	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.9;
	8487	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.6;
	8487	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.4;
	8487	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.6;
	8487	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.7;
	8487	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74;
	8487	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	391.3;
	8488	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	530.4;
	8488	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.9;
	8488	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.3;
	8488	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.1;
	8488	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.9;
	8488	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.6;
	8489	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	548.5;
	8489	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189.1;
	8489	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.5;
	8489	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259;
	8489	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.7;
	8489	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.2;
	8490	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560;
	8490	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.2;
	8490	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.8;
	8490	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	265;
	8490	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.1;
	8490	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.7;
	8491	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	578.7;
	8491	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.5;
	8491	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.2;
	8491	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.8;
	8491	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83;
	8491	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	422.1;
	8492	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	576.1;
	8492	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.2;
	8492	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	255.5;
	8492	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.6;
	8492	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	84;
	8492	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.2;
	8493	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	568.8;
	8493	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.9;
	8493	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.2;
	8493	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.1;
	8493	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.7;
	8493	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.1;
	8494	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	545.7;
	8494	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.1;
	8494	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236;
	8494	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.2;
	8494	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.5;
	8494	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.5;
	8495	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.4;
	8495	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163;
	8495	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221;
	8495	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.9;
	8495	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.3;
	8495	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.2;
	8496	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.4;
	8496	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.9;
	8496	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.2;
	8496	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.2;
	8496	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.5;
	8496	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.8;
	8497	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	476.3;
	8497	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.2;
	8497	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.9;
	8497	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200;
	8497	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.8;
	8497	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	359.2;
	8498	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.4;
	8498	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142;
	8498	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195;
	8498	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.3;
	8498	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.9;
	8498	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.2;
	8499	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.8;
	8499	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.6;
	8499	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.5;
	8499	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.8;
	8499	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.3;
	8499	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.1;
	8500	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	467.6;
	8500	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.9;
	8500	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.8;
	8500	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.2;
	8500	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.6;
	8500	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.3;
	8501	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	484;
	8501	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.3;
	8501	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.3;
	8501	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.7;
	8501	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.9;
	8501	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.8;
	8502	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.3;
	8502	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.9;
	8502	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213;
	8502	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215;
	8502	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.7;
	8502	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.5;
	8503	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.1;
	8503	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.6;
	8503	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.3;
	8503	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.7;
	8503	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.2;
	8503	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.7;
	8504	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	568.8;
	8504	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184;
	8504	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.5;
	8504	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.5;
	8504	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	86.4;
	8504	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.3;
	8505	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.9;
	8505	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.3;
	8505	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.6;
	8505	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.1;
	8505	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.2;
	8505	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.4;
	8506	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.1;
	8506	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.7;
	8506	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.3;
	8506	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.2;
	8506	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	89.5;
	8506	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.7;
	8507	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	563.5;
	8507	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.4;
	8507	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.5;
	8507	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.9;
	8507	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	87.5;
	8507	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408;
	8508	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	554;
	8508	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.2;
	8508	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.2;
	8508	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.1;
	8508	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.8;
	8508	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.6;
	8509	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	546.5;
	8509	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.6;
	8509	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.2;
	8509	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.6;
	8509	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.4;
	8509	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	400.8;
	8510	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	550.2;
	8510	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.4;
	8510	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.4;
	8510	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.4;
	8510	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76;
	8510	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.5;
	8511	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537.1;
	8511	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.6;
	8511	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.6;
	8511	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.8;
	8511	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.9;
	8511	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.8;
	8512	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.5;
	8512	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.4;
	8512	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.3;
	8512	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.5;
	8512	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.5;
	8512	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.5;
	8513	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	574.5;
	8513	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	196.9;
	8513	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.7;
	8513	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.2;
	8513	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.4;
	8513	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	419.6;
	8514	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	599.2;
	8514	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	201.7;
	8514	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	264.8;
	8514	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	266.8;
	8514	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.6;
	8514	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	432;
	8515	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	610.1;
	8515	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	202.5;
	8515	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.8;
	8515	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.1;
	8515	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.9;
	8515	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	435.5;
	8516	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	611.3;
	8516	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	200.3;
	8516	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.3;
	8516	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.7;
	8516	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.7;
	8516	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	433.6;
	8517	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	594.9;
	8517	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.4;
	8517	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.2;
	8517	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.3;
	8517	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	83.8;
	8517	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	423.8;
	8518	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.3;
	8518	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.8;
	8518	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.3;
	8518	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.1;
	8518	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.8;
	8518	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.9;
	8519	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.3;
	8519	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.6;
	8519	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.7;
	8519	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.4;
	8519	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	81.5;
	8519	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.9;
	8520	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	513.6;
	8520	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.1;
	8520	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.6;
	8520	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.3;
	8520	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.8;
	8520	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.8;
	8521	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.3;
	8521	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.8;
	8521	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.2;
	8521	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203;
	8521	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.7;
	8521	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.8;
	8522	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	483.6;
	8522	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.7;
	8522	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.4;
	8522	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.8;
	8522	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.7;
	8522	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.4;
	8523	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478;
	8523	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.1;
	8523	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.8;
	8523	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.9;
	8523	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.1;
	8523	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.1;
	8524	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.5;
	8524	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146;
	8524	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.4;
	8524	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.3;
	8524	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.5;
	8524	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	363.8;
	8525	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.9;
	8525	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.4;
	8525	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.4;
	8525	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.4;
	8525	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.9;
	8525	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.9;
	8526	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.2;
	8526	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.4;
	8526	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.4;
	8526	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.8;
	8526	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.5;
	8526	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.8;
	8527	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	567.9;
	8527	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.6;
	8527	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.6;
	8527	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229;
	8527	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.2;
	8527	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	404.7;
	8528	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	582.3;
	8528	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.7;
	8528	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.3;
	8528	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.7;
	8528	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.6;
	8528	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	413.7;
	8529	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588.2;
	8529	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.2;
	8529	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.2;
	8529	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.1;
	8529	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.8;
	8529	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	418.3;
	8530	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	583.6;
	8530	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183;
	8530	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.6;
	8530	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.2;
	8530	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	85.8;
	8530	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	417.2;
	8531	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	578.7;
	8531	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.5;
	8531	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.5;
	8531	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.6;
	8531	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.7;
	8531	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414.4;
	8532	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	566.5;
	8532	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.9;
	8532	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.3;
	8532	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.1;
	8532	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.3;
	8532	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.1;
	8533	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	561.5;
	8533	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.1;
	8533	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.7;
	8533	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.2;
	8533	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.1;
	8533	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	407.2;
	8534	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.6;
	8534	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.4;
	8534	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.4;
	8534	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.2;
	8534	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.4;
	8534	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.3;
	8535	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.6;
	8535	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.4;
	8535	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.2;
	8535	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.5;
	8535	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	8535	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.5;
	8536	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.4;
	8536	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.6;
	8536	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.6;
	8536	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.6;
	8536	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71;
	8536	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	412.2;
	8537	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	580.2;
	8537	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	197.4;
	8537	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	258;
	8537	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.1;
	8537	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.7;
	8537	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	420.5;
	8538	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.7;
	8538	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	201.2;
	8538	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	262.5;
	8538	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	264.8;
	8538	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.1;
	8538	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	429.9;
	8539	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	613.4;
	8539	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	198.9;
	8539	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.9;
	8539	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	262.2;
	8539	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76;
	8539	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	435.9;
	8540	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.4;
	8540	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195;
	8540	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254;
	8540	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.4;
	8540	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.1;
	8540	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	427.6;
	8541	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	574.4;
	8541	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.2;
	8541	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.6;
	8541	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.6;
	8541	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.3;
	8541	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	414;
	8542	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.1;
	8542	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.3;
	8542	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.3;
	8542	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.6;
	8542	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.3;
	8542	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.4;
	8543	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518.5;
	8543	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.5;
	8543	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.9;
	8543	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.4;
	8543	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76;
	8543	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.8;
	8544	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.3;
	8544	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.1;
	8544	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.1;
	8544	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.4;
	8544	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.3;
	8544	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.5;
	8545	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.7;
	8545	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.8;
	8545	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.3;
	8545	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.5;
	8545	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.7;
	8545	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.9;
	8546	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.1;
	8546	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.8;
	8546	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.5;
	8546	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.5;
	8546	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.7;
	8546	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.1;
	8547	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	454;
	8547	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.7;
	8547	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.5;
	8547	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.5;
	8547	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.1;
	8547	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.6;
	8548	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.7;
	8548	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.5;
	8548	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.4;
	8548	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.4;
	8548	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69;
	8548	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.6;
	8549	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471;
	8549	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.2;
	8549	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.3;
	8549	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.5;
	8549	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	8549	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.2;
	8550	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.1;
	8550	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.6;
	8550	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.5;
	8550	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	206.8;
	8550	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.7;
	8550	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.2;
	8551	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.4;
	8551	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.1;
	8551	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.4;
	8551	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.5;
	8551	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.8;
	8551	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.1;
	8552	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.1;
	8552	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.9;
	8552	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.7;
	8552	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.9;
	8552	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.2;
	8552	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.2;
	8553	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.8;
	8553	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173;
	8553	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.2;
	8553	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.8;
	8553	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.8;
	8553	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.4;
	8554	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.2;
	8554	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.9;
	8554	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.9;
	8554	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.6;
	8554	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.6;
	8554	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.8;
	8555	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.6;
	8555	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.2;
	8555	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	223.6;
	8555	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.3;
	8555	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.7;
	8555	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.5;
	8556	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.4;
	8556	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.4;
	8556	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.4;
	8556	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223;
	8556	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.3;
	8556	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.6;
	8557	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.8;
	8557	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179;
	8557	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.5;
	8557	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224;
	8557	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	8557	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.8;
	8558	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522.7;
	8558	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.3;
	8558	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.4;
	8558	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.8;
	8558	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.5;
	8558	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.7;
	8559	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.4;
	8559	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.3;
	8559	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.3;
	8559	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.8;
	8559	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.2;
	8559	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.5;
	8560	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	527.4;
	8560	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.7;
	8560	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.6;
	8560	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.9;
	8560	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	8560	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.6;
	8561	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	538.4;
	8561	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.5;
	8561	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.3;
	8561	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.6;
	8561	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.5;
	8561	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.9;
	8562	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552.9;
	8562	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.2;
	8562	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248;
	8562	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.7;
	8562	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.1;
	8562	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	406.8;
	8563	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.5;
	8563	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190.7;
	8563	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	248.2;
	8563	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.3;
	8563	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	8563	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	408.4;
	8564	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.3;
	8564	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.6;
	8564	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.3;
	8564	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.8;
	8564	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.2;
	8564	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.7;
	8565	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	529.4;
	8565	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.7;
	8565	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	240.4;
	8565	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.7;
	8565	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.9;
	8565	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.7;
	8566	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.8;
	8566	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.3;
	8566	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230;
	8566	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.9;
	8566	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.1;
	8566	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.7;
	8567	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.2;
	8567	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.7;
	8567	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.7;
	8567	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.8;
	8567	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71;
	8567	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.4;
	8568	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.5;
	8568	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	156.8;
	8568	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.2;
	8568	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	221.5;
	8568	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.1;
	8568	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.7;
	8569	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	444.6;
	8569	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.8;
	8569	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.9;
	8569	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	204;
	8569	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	8569	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.3;
	8570	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.5;
	8570	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.1;
	8570	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.5;
	8570	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.9;
	8570	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	8570	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.2;
	8571	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423;
	8571	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.3;
	8571	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192;
	8571	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.9;
	8571	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	8571	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	335.3;
	8572	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	420.3;
	8572	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.9;
	8572	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.1;
	8572	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195;
	8572	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	8572	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334.3;
	8573	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434;
	8573	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.6;
	8573	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.3;
	8573	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.1;
	8573	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.1;
	8573	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.5;
	8574	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	447.1;
	8574	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.5;
	8574	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.9;
	8574	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199.9;
	8574	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.3;
	8574	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347.3;
	8575	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.5;
	8575	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.4;
	8575	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.3;
	8575	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.8;
	8575	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.1;
	8575	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352;
	8576	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.7;
	8576	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.6;
	8576	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.3;
	8576	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.2;
	8576	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77.9;
	8576	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.5;
	8577	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.7;
	8577	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.7;
	8577	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	205.6;
	8577	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.3;
	8577	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80.5;
	8577	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367.9;
	8578	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.5;
	8578	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.9;
	8578	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	210.1;
	8578	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214;
	8578	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	82.5;
	8578	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.6;
	8579	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.3;
	8579	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.9;
	8579	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.2;
	8579	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.3;
	8579	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	80;
	8579	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.3;
	8580	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	481.5;
	8580	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	165.6;
	8580	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.2;
	8580	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.8;
	8580	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.4;
	8580	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.3;
	8581	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.1;
	8581	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.4;
	8581	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.9;
	8581	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.1;
	8581	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.4;
	8581	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.2;
	8582	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.4;
	8582	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.5;
	8582	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.6;
	8582	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.6;
	8582	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72;
	8582	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.4;
	8583	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.4;
	8583	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.5;
	8583	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.3;
	8583	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.1;
	8583	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	8583	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.4;
	8584	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.4;
	8584	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.3;
	8584	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224;
	8584	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226;
	8584	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.5;
	8584	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.7;
	8585	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506;
	8585	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.9;
	8585	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.1;
	8585	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.6;
	8585	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.8;
	8585	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.6;
	8586	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	537.2;
	8586	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.5;
	8586	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.1;
	8586	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.7;
	8586	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.2;
	8586	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.8;
	8587	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.6;
	8587	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.3;
	8587	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247;
	8587	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	247.7;
	8587	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.1;
	8587	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	409.8;
	8588	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	542.2;
	8588	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.8;
	8588	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.5;
	8588	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.7;
	8588	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.7;
	8588	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.4;
	8589	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525;
	8589	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.9;
	8589	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.1;
	8589	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	243.2;
	8589	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	77;
	8589	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.7;
	8590	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.3;
	8590	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.2;
	8590	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.7;
	8590	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.2;
	8590	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.9;
	8590	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.2;
	8591	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.2;
	8591	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.6;
	8591	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.6;
	8591	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.2;
	8591	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.5;
	8591	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.9;
	8592	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	462.3;
	8592	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154;
	8592	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.8;
	8592	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.3;
	8592	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71;
	8592	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.7;
	8593	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	445.3;
	8593	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.7;
	8593	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.8;
	8593	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.7;
	8593	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	8593	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	347;
	8594	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	430.6;
	8594	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.7;
	8594	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.8;
	8594	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.8;
	8594	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.5;
	8594	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.1;
	8595	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.5;
	8595	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	139.2;
	8595	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.4;
	8595	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.2;
	8595	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	8595	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.8;
	8596	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	439.8;
	8596	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.3;
	8596	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.1;
	8596	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196;
	8596	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	8596	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.4;
	8597	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.4;
	8597	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.2;
	8597	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.3;
	8597	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.1;
	8597	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	8597	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.1;
	8598	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	466.3;
	8598	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	155.2;
	8598	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	203.7;
	8598	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.2;
	8598	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	8598	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.3;
	8599	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	504.4;
	8599	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.5;
	8599	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216;
	8599	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.4;
	8599	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	8599	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375;
	8600	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	544.7;
	8600	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.4;
	8600	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.7;
	8600	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.3;
	8600	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	8600	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.3;
	8601	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	556.2;
	8601	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.2;
	8601	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.2;
	8601	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.2;
	8601	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	8601	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.3;
	8602	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	553.1;
	8602	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.4;
	8602	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.5;
	8602	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.5;
	8602	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.5;
	8602	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	403.4;
	8603	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	543.6;
	8603	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.1;
	8603	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.2;
	8603	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.8;
	8603	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	8603	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.8;
	8604	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	533.3;
	8604	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.1;
	8604	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.7;
	8604	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235;
	8604	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.2;
	8604	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.2;
	8605	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	526.4;
	8605	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.5;
	8605	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.7;
	8605	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.6;
	8605	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.9;
	8605	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392.2;
	8606	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.4;
	8606	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.6;
	8606	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234;
	8606	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.9;
	8606	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.7;
	8606	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.9;
	8607	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.5;
	8607	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172;
	8607	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.5;
	8607	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.1;
	8607	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.5;
	8607	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	402.4;
	8608	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	539.9;
	8608	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.6;
	8608	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.5;
	8608	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.5;
	8608	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	8608	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.6;
	8609	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.1;
	8609	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.6;
	8609	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.6;
	8609	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.7;
	8609	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.3;
	8609	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	411.8;
	8610	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	585.9;
	8610	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195.2;
	8610	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	260.4;
	8610	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	261.1;
	8610	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	8610	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	424.7;
	8611	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	578.2;
	8611	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	195;
	8611	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	257.5;
	8611	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	258.5;
	8611	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.8;
	8611	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	421.1;
	8612	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	569.7;
	8612	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	193.2;
	8612	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254;
	8612	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.1;
	8612	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.4;
	8612	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	416.2;
	8613	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.1;
	8613	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187.6;
	8613	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247;
	8613	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	248.3;
	8613	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.9;
	8613	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	410.7;
	8614	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	532.5;
	8614	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.6;
	8614	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.3;
	8614	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.2;
	8614	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.4;
	8614	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.7;
	8615	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.7;
	8615	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.2;
	8615	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.7;
	8615	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.5;
	8615	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	8615	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.5;
	8616	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.7;
	8616	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.5;
	8616	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.7;
	8616	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.3;
	8616	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.7;
	8616	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364;
	8617	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	456.4;
	8617	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	148.9;
	8617	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	202.6;
	8617	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	203.8;
	8617	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	8617	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.8;
	8618	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.7;
	8618	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.9;
	8618	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.3;
	8618	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.7;
	8618	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	8618	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	340.9;
	8619	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	431.6;
	8619	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.1;
	8619	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.1;
	8619	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.5;
	8619	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	8619	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.9;
	8620	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	436.6;
	8620	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	144.6;
	8620	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.8;
	8620	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.2;
	8620	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	8620	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.6;
	8621	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.8;
	8621	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	149.1;
	8621	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199;
	8621	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.5;
	8621	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.6;
	8621	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	345.9;
	8622	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463.9;
	8622	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.7;
	8622	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.2;
	8622	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.7;
	8622	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.6;
	8622	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	356.4;
	8623	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.8;
	8623	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.2;
	8623	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.5;
	8623	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.4;
	8623	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	8623	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	369.3;
	8624	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.8;
	8624	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.5;
	8624	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	226.9;
	8624	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	229.7;
	8624	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.7;
	8624	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.9;
	8625	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	519.8;
	8625	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.9;
	8625	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.9;
	8625	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.8;
	8625	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.9;
	8625	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.7;
	8626	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	520.8;
	8626	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	179.8;
	8626	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.1;
	8626	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236;
	8626	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.5;
	8626	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.9;
	8627	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.6;
	8627	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.2;
	8627	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.7;
	8627	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.1;
	8627	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	8627	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.9;
	8628	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.6;
	8628	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178;
	8628	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.4;
	8628	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.6;
	8628	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.8;
	8628	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383;
	8629	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	499.2;
	8629	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.7;
	8629	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	230.7;
	8629	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	234.4;
	8629	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	8629	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.4;
	8630	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	492.1;
	8630	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174;
	8630	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.3;
	8630	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.8;
	8630	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	8630	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.2;
	8631	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.3;
	8631	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.3;
	8631	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233.6;
	8631	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.3;
	8631	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.6;
	8631	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374.9;
	8632	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.7;
	8632	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.5;
	8632	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.7;
	8632	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.1;
	8632	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.2;
	8632	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.3;
	8633	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.6;
	8633	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	185.5;
	8633	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	250.6;
	8633	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.6;
	8633	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.6;
	8633	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.9;
	8634	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	516.4;
	8634	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	189;
	8634	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	254.9;
	8634	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	257.3;
	8634	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.8;
	8634	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.4;
	8635	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	512.3;
	8635	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	190;
	8635	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.5;
	8635	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.8;
	8635	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	8635	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	393.5;
	8636	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.6;
	8636	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.1;
	8636	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.5;
	8636	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	251.3;
	8636	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.9;
	8636	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.6;
	8637	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.4;
	8637	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	184.2;
	8637	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.3;
	8637	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.4;
	8637	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.6;
	8637	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.1;
	8638	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.7;
	8638	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176;
	8638	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.7;
	8638	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.3;
	8638	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.8;
	8638	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.9;
	8639	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	460.7;
	8639	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.2;
	8639	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.1;
	8639	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.3;
	8639	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.3;
	8639	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.8;
	8640	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	444.6;
	8640	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.6;
	8640	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.6;
	8640	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	218.7;
	8640	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71;
	8640	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.4;
	8641	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	432.7;
	8641	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	145.4;
	8641	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.2;
	8641	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.5;
	8641	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.9;
	8641	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	343;
	8642	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	410.4;
	8642	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.3;
	8642	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.3;
	8642	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.1;
	8642	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.8;
	8642	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.8;
	8643	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.9;
	8643	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.7;
	8643	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.8;
	8643	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.2;
	8643	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.6;
	8643	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.2;
	8644	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.3;
	8644	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	143.4;
	8644	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.7;
	8644	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.2;
	8644	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.4;
	8644	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.9;
	8645	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	434.7;
	8645	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.7;
	8645	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.4;
	8645	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.2;
	8645	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69;
	8645	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.4;
	8646	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	457.3;
	8646	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.6;
	8646	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.3;
	8646	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	209.2;
	8646	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.5;
	8646	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.6;
	8647	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	485.9;
	8647	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.3;
	8647	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.1;
	8647	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219;
	8647	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	8647	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.3;
	8648	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.8;
	8648	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.6;
	8648	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225.4;
	8648	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	227.1;
	8648	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.3;
	8648	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.8;
	8649	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.6;
	8649	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.7;
	8649	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.4;
	8649	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.4;
	8649	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	8649	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.8;
	8650	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	518.9;
	8650	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181.9;
	8650	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.3;
	8650	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	236.8;
	8650	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	8650	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.2;
	8651	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517.1;
	8651	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.1;
	8651	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.1;
	8651	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238;
	8651	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.8;
	8651	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	388.4;
	8652	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	517;
	8652	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.6;
	8652	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.3;
	8652	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238;
	8652	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	8652	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.9;
	8653	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.5;
	8653	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.6;
	8653	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.5;
	8653	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.7;
	8653	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.2;
	8653	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.5;
	8654	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.3;
	8654	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.6;
	8654	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.3;
	8654	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.1;
	8654	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	8654	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384.8;
	8655	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.2;
	8655	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177;
	8655	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.1;
	8655	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.1;
	8655	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.4;
	8655	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.9;
	8656	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500.6;
	8656	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180;
	8656	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242.8;
	8656	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	245.2;
	8656	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.2;
	8656	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.8;
	8657	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.8;
	8657	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.9;
	8657	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	253.3;
	8657	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.7;
	8657	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.7;
	8657	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	389.8;
	8658	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	523.6;
	8658	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	194.2;
	8658	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	256.6;
	8658	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	259.7;
	8658	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.9;
	8658	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	398.7;
	8659	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.9;
	8659	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	191.1;
	8659	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.9;
	8659	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	255.9;
	8659	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.9;
	8659	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	401.3;
	8660	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	521.2;
	8660	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	188.4;
	8660	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.8;
	8660	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.1;
	8660	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.2;
	8660	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	395.3;
	8661	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	508.8;
	8661	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.5;
	8661	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	243.9;
	8661	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	246.6;
	8661	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.7;
	8661	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.6;
	8662	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.8;
	8662	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.8;
	8662	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235;
	8662	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237;
	8662	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.8;
	8662	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.8;
	8663	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	464.1;
	8663	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.1;
	8663	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.9;
	8663	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	223.8;
	8663	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.4;
	8663	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.6;
	8664	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440;
	8664	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	154;
	8664	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.6;
	8664	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.2;
	8664	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	8664	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350;
	8665	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.2;
	8665	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.6;
	8665	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.5;
	8665	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.9;
	8665	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62.7;
	8665	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	337.5;
	8666	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	401.7;
	8666	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.8;
	8666	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.4;
	8666	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.1;
	8666	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.5;
	8666	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	327.4;
	8667	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.3;
	8667	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.3;
	8667	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.8;
	8667	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.9;
	8667	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.8;
	8667	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	328.1;
	8668	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.3;
	8668	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	134.5;
	8668	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194;
	8668	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.3;
	8668	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.2;
	8668	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.4;
	8669	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	411.8;
	8669	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.4;
	8669	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.1;
	8669	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.9;
	8669	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.9;
	8669	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	332.5;
	8670	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	428.2;
	8670	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.2;
	8670	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204;
	8670	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.2;
	8670	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.3;
	8670	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.7;
	8671	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	455.9;
	8671	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.3;
	8671	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.3;
	8671	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.6;
	8671	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.7;
	8671	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.7;
	8672	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	478.5;
	8672	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.1;
	8672	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.1;
	8672	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.6;
	8672	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.1;
	8672	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	367;
	8673	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	489.4;
	8673	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.7;
	8673	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	227.3;
	8673	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	231.4;
	8673	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	8673	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	374;
	8674	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495.6;
	8674	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.2;
	8674	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.5;
	8674	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.6;
	8674	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.1;
	8674	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.5;
	8675	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.4;
	8675	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.5;
	8675	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233;
	8675	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.7;
	8675	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.3;
	8675	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.8;
	8676	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.2;
	8676	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	172.7;
	8676	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	234.2;
	8676	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.3;
	8676	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.1;
	8676	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	381.9;
	8677	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	497.2;
	8677	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.9;
	8677	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.8;
	8677	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.5;
	8677	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.6;
	8677	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.1;
	8678	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	496.2;
	8678	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.2;
	8678	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.9;
	8678	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.1;
	8678	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.3;
	8678	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.1;
	8679	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	493.6;
	8679	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.2;
	8679	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.7;
	8679	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	232.9;
	8679	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.8;
	8679	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.5;
	8680	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	495;
	8680	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.1;
	8680	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	235.6;
	8680	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.4;
	8680	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.6;
	8680	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.3;
	8681	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	507.7;
	8681	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.7;
	8681	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	244.9;
	8681	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	250.2;
	8681	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	64.4;
	8681	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	387.4;
	8682	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	525.1;
	8682	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	187;
	8682	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	252.5;
	8682	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	256.4;
	8682	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65;
	8682	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	397.4;
	8683	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	531.2;
	8683	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.8;
	8683	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	251.9;
	8683	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	254.9;
	8683	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	8683	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	399.9;
	8684	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	522.7;
	8684	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	182.9;
	8684	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	246.8;
	8684	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.8;
	8684	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.2;
	8684	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	394.9;
	8685	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506;
	8685	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.3;
	8685	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.4;
	8685	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244;
	8685	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.8;
	8685	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	385.8;
	8686	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	490.6;
	8686	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	170.1;
	8686	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	233;
	8686	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.3;
	8686	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.2;
	8686	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.5;
	8687	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.2;
	8687	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	157.4;
	8687	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.1;
	8687	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222;
	8687	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	63.9;
	8687	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.7;
	8688	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	452.4;
	8688	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.3;
	8688	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.3;
	8688	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.9;
	8688	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	62;
	8688	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	353.7;
	8689	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.1;
	8689	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.3;
	8689	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	200.4;
	8689	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.6;
	8689	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	59.6;
	8689	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.6;
	8690	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	418.2;
	8690	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.6;
	8690	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.4;
	8690	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	197.5;
	8690	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58.7;
	8690	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	334;
	8691	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	412.6;
	8691	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.7;
	8691	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.1;
	8691	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.5;
	8691	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.2;
	8691	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.3;
	8692	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	416.8;
	8692	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	138.6;
	8692	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194;
	8692	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.6;
	8692	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	58;
	8692	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	333.1;
	8693	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	425.9;
	8693	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	142.5;
	8693	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.5;
	8693	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.3;
	8693	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	57.8;
	8693	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.1;
	8694	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	446.2;
	8694	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.5;
	8694	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204;
	8694	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	207;
	8694	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	60.2;
	8694	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.6;
	8695	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.9;
	8695	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163;
	8695	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213;
	8695	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216;
	8695	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	61.9;
	8695	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.8;
	8696	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	488.1;
	8696	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.8;
	8696	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	220.1;
	8696	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.9;
	8696	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	8696	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.3;
	8697	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	502.7;
	8697	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173;
	8697	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.5;
	8697	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.4;
	8697	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.8;
	8697	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.7;
	8698	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	505.7;
	8698	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.1;
	8698	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.3;
	8698	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228.4;
	8698	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	8698	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	379.7;
	8699	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	509;
	8699	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176;
	8699	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.7;
	8699	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228;
	8699	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.8;
	8699	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	380.9;
	8700	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.6;
	8700	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173;
	8700	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.7;
	8700	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225.3;
	8700	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	8700	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	376.8;
	8701	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	487.7;
	8701	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.5;
	8701	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.7;
	8701	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.9;
	8701	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.1;
	8701	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	371.4;
	8702	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.6;
	8702	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.5;
	8702	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	221.5;
	8702	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	224.6;
	8702	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68;
	8702	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.2;
	8703	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	472.9;
	8703	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.5;
	8703	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	222.1;
	8703	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	225;
	8703	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.9;
	8703	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	365.7;
	8704	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.4;
	8704	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.2;
	8704	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	228.2;
	8704	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.8;
	8704	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.4;
	8704	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	366.8;
	8705	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	482.5;
	8705	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177.9;
	8705	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	241.3;
	8705	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.1;
	8705	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	8705	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	375.2;
	8706	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511.9;
	8706	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	186.7;
	8706	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	249.7;
	8706	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	252.1;
	8706	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.2;
	8706	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.5;
	8707	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.5;
	8707	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	183.8;
	8707	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	247.5;
	8707	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	249.5;
	8707	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	8707	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	392;
	8708	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	515.7;
	8708	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	180.9;
	8708	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	242;
	8708	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	244.1;
	8708	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.2;
	8708	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	390.5;
	8709	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.7;
	8709	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	176.7;
	8709	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	237.5;
	8709	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	239.1;
	8709	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.6;
	8709	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.5;
	8710	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	486.4;
	8710	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.3;
	8710	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	229.5;
	8710	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	230.9;
	8710	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.5;
	8710	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	373.7;
	8711	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	463;
	8711	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.2;
	8711	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.4;
	8711	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	219.8;
	8711	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72;
	8711	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.7;
	8712	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	440.4;
	8712	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	151.9;
	8712	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.5;
	8712	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.5;
	8712	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.6;
	8712	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.8;
	8713	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	423.5;
	8713	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	141.5;
	8713	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.1;
	8713	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.5;
	8713	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.6;
	8713	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	338.8;
	8714	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	397.4;
	8714	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.9;
	8714	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	193.3;
	8714	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.7;
	8714	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.4;
	8714	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.1;
	8715	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	396.7;
	8715	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137;
	8715	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.7;
	8715	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194.1;
	8715	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.4;
	8715	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	324.5;
	8716	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399.3;
	8716	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.8;
	8716	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	191.8;
	8716	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	194;
	8716	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.2;
	8716	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	325.8;
	8717	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	404.9;
	8717	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.6;
	8717	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	194.1;
	8717	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	196.1;
	8717	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.7;
	8717	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	329.1;
	8718	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	421.3;
	8718	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.1;
	8718	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	196.6;
	8718	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	199;
	8718	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	66.8;
	8718	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336.9;
	8719	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	437.7;
	8719	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.1;
	8719	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	201.1;
	8719	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.9;
	8719	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.2;
	8719	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	344.7;
	8720	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.9;
	8720	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.8;
	8720	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.8;
	8720	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	205.9;
	8720	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.7;
	8720	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.9;
	8721	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	458.6;
	8721	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161.9;
	8721	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.4;
	8721	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.7;
	8721	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	8721	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	354.9;
	8722	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	470.7;
	8722	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	166.1;
	8722	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	212.8;
	8722	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.4;
	8722	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.4;
	8722	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.3;
	8723	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	475.7;
	8723	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.9;
	8723	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.9;
	8723	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	215.3;
	8723	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.2;
	8723	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	364.3;
	8724	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	471.2;
	8724	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.5;
	8724	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.6;
	8724	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.1;
	8724	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.1;
	8724	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	362.5;
	8725	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	465;
	8725	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	168.1;
	8725	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	216.3;
	8725	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216;
	8725	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.1;
	8725	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	360.5;
	8726	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.1;
	8726	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	164.3;
	8726	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.8;
	8726	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.8;
	8726	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	72.1;
	8726	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358;
	8727	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.1;
	8727	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163;
	8727	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	214.6;
	8727	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.9;
	8727	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.3;
	8727	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	355.5;
	8728	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.1;
	8728	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	167.5;
	8728	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	218.8;
	8728	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	220.9;
	8728	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.5;
	8728	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358.9;
	8729	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	474.9;
	8729	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	177;
	8729	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.4;
	8729	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.7;
	8729	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	70.2;
	8729	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.9;
	8730	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	500;
	8730	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	181;
	8730	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.5;
	8730	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.7;
	8730	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.7;
	8730	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	382.1;
	8731	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	501.2;
	8731	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	178.3;
	8731	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	239.6;
	8731	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	241.8;
	8731	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.1;
	8731	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.3;
	8732	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	491.7;
	8732	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.2;
	8732	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.5;
	8732	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	238.8;
	8732	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.7;
	8732	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	378.3;
	8733	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	477.4;
	8733	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	169.9;
	8733	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.3;
	8733	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	233.4;
	8733	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.7;
	8733	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.5;
	8734	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	461;
	8734	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.1;
	8734	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	225;
	8734	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	226.3;
	8734	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.3;
	8734	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	361.4;
	8735	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.5;
	8735	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.1;
	8735	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	215.8;
	8735	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	217.4;
	8735	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.7;
	8735	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.5;
	8736	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	424.5;
	8736	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	150.3;
	8736	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	211.4;
	8736	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	214.3;
	8736	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	69.4;
	8736	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	341.3;
	8737	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	406.9;
	8737	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	137.6;
	8737	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	197.9;
	8737	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	200.9;
	8737	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67.5;
	8737	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	331.3;
	8738	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	375.1;
	8738	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.6;
	8738	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.6;
	8738	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.3;
	8738	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.8;
	8738	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.6;
	8739	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	375;
	8739	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	131.2;
	8739	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.7;
	8739	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.6;
	8739	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	8739	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	315.1;
	8740	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	379.3;
	8740	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	132.2;
	8740	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	189.5;
	8740	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	192.6;
	8740	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.5;
	8740	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	316.8;
	8741	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	383.2;
	8741	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	133.4;
	8741	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	190.6;
	8741	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	193.7;
	8741	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	65.7;
	8741	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	319;
	8742	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	399;
	8742	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	136.5;
	8742	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	192.5;
	8742	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	195.7;
	8742	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	67;
	8742	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	326.3;
	8743	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	407.8;
	8743	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	140.9;
	8743	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	195.4;
	8743	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	198.2;
	8743	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	68.9;
	8743	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	330.7;
	8744	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	419.5;
	8744	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	146.6;
	8744	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	199.1;
	8744	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	201.9;
	8744	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	71.5;
	8744	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	336;
	8745	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	435.2;
	8745	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	153.2;
	8745	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	198.9;
	8745	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	202.6;
	8745	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.1;
	8745	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	342.5;
	8746	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	448.1;
	8746	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	161;
	8746	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.6;
	8746	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.1;
	8746	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.4;
	8746	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	349.2;
	8747	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.7;
	8747	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.9;
	8747	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	204.7;
	8747	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	208.8;
	8747	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.6;
	8747	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.9;
	8748	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	449.5;
	8748	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.4;
	8748	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	206.6;
	8748	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	210.5;
	8748	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.8;
	8748	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	350.2;
	8749	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	451.1;
	8749	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	160.8;
	8749	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207;
	8749	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211;
	8749	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	75.5;
	8749	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	351.4;
	8750	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	453.2;
	8750	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	159.1;
	8750	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	207.6;
	8750	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	211.4;
	8750	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.4;
	8750	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352.6;
	8751	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	450.2;
	8751	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	158.4;
	8751	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	209.1;
	8751	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212.6;
	8751	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.9;
	8751	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	352;
	8752	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.5;
	8752	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	162.4;
	8752	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	217.5;
	8752	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	222.2;
	8752	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.3;
	8752	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	358;
	8753	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	480.1;
	8753	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	173.9;
	8753	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	232.3;
	8753	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	237.8;
	8753	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	73.2;
	8753	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	370.8;
	8754	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	503.5;
	8754	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.2;
	8754	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.4;
	8754	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.2;
	8754	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.7;
	8754	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	383.3;
	8755	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	511;
	8755	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	175.2;
	8755	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	238.1;
	8755	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	242.6;
	8755	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.9;
	8755	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	386.7;
	8756	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	506.7;
	8756	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	174.7;
	8756	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	236.1;
	8756	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	240.3;
	8756	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79;
	8756	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	384;
	8757	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	494.8;
	8757	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	171.3;
	8757	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	231.9;
	8757	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	235.9;
	8757	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	79.8;
	8757	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	377.5;
	8758	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	479.5;
	8758	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	163.1;
	8758	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	224.3;
	8758	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	228;
	8758	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	78.5;
	8758	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	368.9;
	8759	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	459.7;
	8759	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	152.1;
	8759	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	213.6;
	8759	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	216.5;
	8759	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	76.9;
	8759	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	357.7;
	8760	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	443.6;
	8760	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	147.3;
	8760	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	208.2;
	8760	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	212;
	8760	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	74.9;
	8760	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	348.2;
];
