function chgtab = scenarios_ACTIVSg2000
%SCENARIOS_ACTIVSG2000  Returns a change table suitable for use with APPLY_CHANGES
%
%   This change table provides area load changes for 8784 hours
%   for case_ACTIVSg2000 for the year 2016.
%
%   One-line diagrams and other data formats available at:
%       https://electricgrids.engr.tamu.edu

%   Created from data in ACTIVSg2000.tsb.

%   MATPOWER
%   Copyright (c) 2017 by A.B. Birchfield, T. Xu, K.M. Gegner, K.S. Shetye,
%   and T.J. Overbye
%   Licensed under the Creative Commons Attribution 4.0 International license,
%   https://creativecommons.org/licenses/by/4.0/

define_constants;

%% Change Table
%	label	prob	table	row	col	chgtype	newval
chgtab = [
	1	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.9;
	1	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	806.8;
	1	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940.1;
	1	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3318.7;
	1	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10047.2;
	1	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5441.9;
	1	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8258.4;
	1	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1467.9;
	2	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.6;
	2	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	800.5;
	2	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	933.5;
	2	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3318.8;
	2	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9893.9;
	2	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5369.3;
	2	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8132.8;
	2	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1450.6;
	3	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.3;
	3	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	799.3;
	3	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	924.1;
	3	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3271.2;
	3	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9820.6;
	3	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5298;
	3	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8023.8;
	3	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1440.4;
	4	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.5;
	4	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	803.5;
	4	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	927.7;
	4	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3234.3;
	4	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9871.4;
	4	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5266.9;
	4	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7972.6;
	4	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1443.6;
	5	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951;
	5	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	808.5;
	5	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	947.3;
	5	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3248;
	5	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10064.3;
	5	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5327.9;
	5	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8012.7;
	5	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1459.4;
	6	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.3;
	6	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	823.8;
	6	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	978;
	6	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3307.8;
	6	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10420.5;
	6	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5539.3;
	6	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8183.8;
	6	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1512.1;
	7	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.8;
	7	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	847;
	7	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1013.5;
	7	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3384.5;
	7	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10793.7;
	7	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5756.4;
	7	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8352.8;
	7	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1561.3;
	8	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.3;
	8	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	873.1;
	8	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1043.5;
	8	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3419.3;
	8	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11095.5;
	8	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5897.2;
	8	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8420.6;
	8	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1598.9;
	9	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.7;
	9	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	891.4;
	9	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1063.1;
	9	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3501.9;
	9	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11380.9;
	9	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6035.9;
	9	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8564.9;
	9	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1643.2;
	10	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	976.3;
	10	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	901.1;
	10	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1082.2;
	10	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3662.9;
	10	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11685.1;
	10	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6310.3;
	10	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8881.7;
	10	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1691.4;
	11	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	982.9;
	11	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	891;
	11	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1087.9;
	11	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3808.4;
	11	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11688.3;
	11	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6532.8;
	11	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9176.4;
	11	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1716.6;
	12	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990.3;
	12	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	868.8;
	12	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1068;
	12	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3931.3;
	12	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11615;
	12	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6545.6;
	12	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9286.7;
	12	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1707.5;
	13	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989.8;
	13	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	841.4;
	13	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1052.4;
	13	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4001.3;
	13	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11401.7;
	13	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6477.3;
	13	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9349.6;
	13	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1657.9;
	14	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	982;
	14	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	834.4;
	14	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1044.3;
	14	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4020;
	14	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11155;
	14	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6405.2;
	14	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9257.2;
	14	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1609.8;
	15	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	973;
	15	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	827.9;
	15	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1023.7;
	15	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4026.2;
	15	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10945.1;
	15	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6336;
	15	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9155.2;
	15	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1579.3;
	16	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970.7;
	16	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	805.2;
	16	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1017.3;
	16	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4047.2;
	16	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10798.8;
	16	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6369.6;
	16	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9170.3;
	16	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1575.9;
	17	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	971.8;
	17	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	803.2;
	17	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1022.4;
	17	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4125;
	17	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10900.9;
	17	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6506.7;
	17	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9260.9;
	17	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1598;
	18	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	984.9;
	18	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	852.1;
	18	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1063.9;
	18	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4291.5;
	18	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11576.6;
	18	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6828;
	18	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9722.6;
	18	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1713;
	19	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009;
	19	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	894.1;
	19	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1107.6;
	19	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4474.8;
	19	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12040.6;
	19	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7030.5;
	19	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9917;
	19	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1779.8;
	20	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1008.8;
	20	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	896.4;
	20	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1109.7;
	20	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4482.2;
	20	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11962.6;
	20	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6966.1;
	20	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9804.6;
	20	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1774.8;
	21	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1004.5;
	21	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	891.6;
	21	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1099.5;
	21	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4471.6;
	21	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11848.3;
	21	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6873;
	21	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9702.2;
	21	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1766.5;
	22	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990.5;
	22	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	880.1;
	22	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1091.3;
	22	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4393.7;
	22	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11623.1;
	22	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6712.7;
	22	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9529.8;
	22	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1732.2;
	23	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	973.3;
	23	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	858.4;
	23	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1054.3;
	23	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4286.4;
	23	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11255;
	23	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6437.7;
	23	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9275.9;
	23	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1674.7;
	24	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.3;
	24	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	833.3;
	24	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1020.1;
	24	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4114.4;
	24	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10785.9;
	24	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6118.1;
	24	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8912.8;
	24	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1600.7;
	25	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.5;
	25	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	801.2;
	25	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	993.3;
	25	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3935.1;
	25	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10303.3;
	25	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5839.2;
	25	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8614.9;
	25	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1569.7;
	26	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.1;
	26	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	792.8;
	26	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	979.4;
	26	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3830.6;
	26	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10038.5;
	26	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5636.4;
	26	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8403.5;
	26	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1546.2;
	27	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.4;
	27	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	789.6;
	27	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	973.5;
	27	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3739.3;
	27	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9856.9;
	27	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5521.5;
	27	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8283.9;
	27	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1533.3;
	28	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950;
	28	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	786.8;
	28	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	975.1;
	28	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3707.4;
	28	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9775.3;
	28	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5477.5;
	28	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8189.3;
	28	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1523;
	29	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.1;
	29	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	793.2;
	29	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	985.5;
	29	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3729;
	29	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9855.6;
	29	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5531.3;
	29	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8257.2;
	29	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1536;
	30	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.2;
	30	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	809.2;
	30	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1009.7;
	30	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3828.1;
	30	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10127.3;
	30	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5690.9;
	30	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8428.6;
	30	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1591.2;
	31	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.3;
	31	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	830.5;
	31	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1036.2;
	31	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3976.3;
	31	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10532.1;
	31	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5960.2;
	31	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8672.7;
	31	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1649.8;
	32	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	974.3;
	32	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	859.1;
	32	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1077.4;
	32	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4100.1;
	32	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10953.2;
	32	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6255.1;
	32	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8937.1;
	32	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1697.7;
	33	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	977.6;
	33	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	880.2;
	33	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1116.8;
	33	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4270.8;
	33	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11327.4;
	33	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6486.7;
	33	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9186.8;
	33	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1738.3;
	34	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990;
	34	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	901.4;
	34	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1145.3;
	34	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4495.5;
	34	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11691.7;
	34	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6796.5;
	34	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9486.4;
	34	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1789.1;
	35	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	988.9;
	35	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	906.4;
	35	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1148.2;
	35	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4674.2;
	35	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11798;
	35	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6985.9;
	35	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9640.6;
	35	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1799.3;
	36	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980;
	36	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	889.4;
	36	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1133.6;
	36	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4775;
	36	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11567.7;
	36	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6996.8;
	36	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9728.4;
	36	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1759.4;
	37	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.6;
	37	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	858.9;
	37	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1106.4;
	37	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4799.6;
	37	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11242.2;
	37	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6954.9;
	37	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9692.2;
	37	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1697.2;
	38	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948.8;
	38	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	835.1;
	38	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1074.2;
	38	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4764.4;
	38	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10832.4;
	38	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6834.3;
	38	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9488.1;
	38	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1633.7;
	39	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.5;
	39	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	814.9;
	39	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1051.3;
	39	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4718.8;
	39	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10544.1;
	39	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6740.2;
	39	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9240.5;
	39	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1597.4;
	40	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.2;
	40	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	802.3;
	40	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1049.3;
	40	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4699.2;
	40	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10477.6;
	40	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6700.2;
	40	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9123.3;
	40	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1574.4;
	41	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.1;
	41	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	813.8;
	41	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1061;
	41	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4736.1;
	41	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10575;
	41	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6773.4;
	41	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9173.9;
	41	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1597.5;
	42	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.3;
	42	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	845.3;
	42	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1091.7;
	42	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4832.7;
	42	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11190.8;
	42	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7082.4;
	42	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9644.1;
	42	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1686.2;
	43	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986.4;
	43	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	878.1;
	43	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1129.8;
	43	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5009.6;
	43	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11557.5;
	43	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7324.8;
	43	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9929.2;
	43	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1747.5;
	44	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986.8;
	44	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	878.8;
	44	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1123.1;
	44	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5000.9;
	44	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11447.9;
	44	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7283.6;
	44	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9856.7;
	44	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1742.3;
	45	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	976.1;
	45	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	867.9;
	45	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1096.5;
	45	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4962.6;
	45	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11288.1;
	45	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7181.1;
	45	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9765.1;
	45	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1731.8;
	46	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.5;
	46	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	856.8;
	46	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1064.4;
	46	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4885.3;
	46	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11003.1;
	46	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6978.7;
	46	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9583.3;
	46	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1686.3;
	47	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.7;
	47	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	836.2;
	47	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1019.9;
	47	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4750.3;
	47	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10573.3;
	47	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6664.2;
	47	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9261.9;
	47	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1613.7;
	48	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920.6;
	48	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	809.1;
	48	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	978.8;
	48	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4575.2;
	48	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10038.6;
	48	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6300.9;
	48	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8894.9;
	48	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1531.8;
	49	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.1;
	49	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	790.5;
	49	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	941.4;
	49	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4399.1;
	49	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9667.1;
	49	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6033.8;
	49	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8547.7;
	49	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1464.4;
	50	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.4;
	50	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	785.4;
	50	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	930.4;
	50	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4248.3;
	50	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9482.5;
	50	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5822.6;
	50	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8340.7;
	50	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1430.4;
	51	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.9;
	51	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	791.2;
	51	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	923.1;
	51	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4144.7;
	51	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9473;
	51	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5691.6;
	51	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8226.7;
	51	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1408.7;
	52	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.7;
	52	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	801.1;
	52	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	923.1;
	52	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4087.6;
	52	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9604.4;
	52	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5631.6;
	52	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8185.1;
	52	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1414.4;
	53	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.3;
	53	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	818.5;
	53	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	939.1;
	53	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4091.7;
	53	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9849.3;
	53	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5651.9;
	53	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8227.2;
	53	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1442.2;
	54	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924;
	54	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	845;
	54	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	969.4;
	54	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4141.1;
	54	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10258.3;
	54	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5773.8;
	54	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8393.6;
	54	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1498.3;
	55	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933;
	55	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	880.7;
	55	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1017.5;
	55	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4236.4;
	55	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10832.1;
	55	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5973.5;
	55	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8638.9;
	55	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1571.2;
	56	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.4;
	56	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	919.8;
	56	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1064.7;
	56	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4299.3;
	56	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11411.8;
	56	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6211.3;
	56	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8783.1;
	56	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1644.1;
	57	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.1;
	57	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	929.8;
	57	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1065.6;
	57	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4393.5;
	57	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11535.9;
	57	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6335.2;
	57	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8945.1;
	57	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1677;
	58	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.5;
	58	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	895.4;
	58	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1045;
	58	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4476;
	58	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11170.1;
	58	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6289;
	58	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9015.7;
	58	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1611.5;
	59	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.3;
	59	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	847.7;
	59	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1005.4;
	59	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4408.9;
	59	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10659.1;
	59	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6042.8;
	59	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8915.1;
	59	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1507;
	60	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907;
	60	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	807.1;
	60	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	958.6;
	60	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4233.5;
	60	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10144.9;
	60	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5739.2;
	60	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8757.1;
	60	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1423.5;
	61	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.7;
	61	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	775.1;
	61	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	886.6;
	61	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4107.1;
	61	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9727.7;
	61	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5500.6;
	61	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8561.6;
	61	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1377.3;
	62	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.8;
	62	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.1;
	62	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.3;
	62	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4002.7;
	62	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9338.4;
	62	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5291.5;
	62	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8360.7;
	62	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1341.1;
	63	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866.1;
	63	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.6;
	63	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	837.9;
	63	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3887;
	63	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9060.7;
	63	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5132.9;
	63	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8208.1;
	63	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1302.6;
	64	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	858.5;
	64	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.5;
	64	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	827.6;
	64	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3798.6;
	64	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8924.5;
	64	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5074.6;
	64	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8132.5;
	64	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1291.1;
	65	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	854.6;
	65	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.7;
	65	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849;
	65	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3749.9;
	65	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9028.9;
	65	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5136.2;
	65	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8138.4;
	65	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1319.7;
	66	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	865.7;
	66	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	796.6;
	66	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	888.3;
	66	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3867.5;
	66	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9765.9;
	66	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5502.2;
	66	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8553.8;
	66	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1440.7;
	67	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.8;
	67	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	860.4;
	67	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	974.2;
	67	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4170.1;
	67	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10726.6;
	67	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6099.8;
	67	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9173.8;
	67	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1592.1;
	68	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.8;
	68	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	873.7;
	68	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1014.8;
	68	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4269.9;
	68	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10965.2;
	68	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6242.7;
	68	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9287.8;
	68	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1646.2;
	69	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950;
	69	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	871.9;
	69	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1025;
	69	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4318.6;
	69	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10997.9;
	69	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6202.7;
	69	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9286.4;
	69	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1655.9;
	70	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949;
	70	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	858;
	70	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1029.1;
	70	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4327.8;
	70	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10875.1;
	70	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6118.8;
	70	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9154.3;
	70	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1645;
	71	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.5;
	71	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	829.1;
	71	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	999.4;
	71	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4207.8;
	71	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10447.3;
	71	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5846.4;
	71	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8857.4;
	71	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1583.3;
	72	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920.9;
	72	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	805.1;
	72	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	974.8;
	72	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4001.2;
	72	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9977;
	72	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5545.8;
	72	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8501.2;
	72	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1519.5;
	73	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.9;
	73	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	795.4;
	73	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	969.1;
	73	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3836.8;
	73	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9755.8;
	73	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5378.1;
	73	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8268.8;
	73	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1489.1;
	74	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.2;
	74	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	796.4;
	74	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	974.5;
	74	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3760.6;
	74	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9755.8;
	74	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5347;
	74	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8200.4;
	74	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1491.7;
	75	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.7;
	75	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	803.6;
	75	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	981.5;
	75	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3742.9;
	75	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9948.1;
	75	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5407.9;
	75	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8216.3;
	75	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1526.2;
	76	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.4;
	76	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	820.1;
	76	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1005.9;
	76	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3774.2;
	76	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10271.3;
	76	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5600.4;
	76	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8332;
	76	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1592;
	77	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.7;
	77	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	849.2;
	77	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1042.8;
	77	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3892;
	77	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10890.3;
	77	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5959.5;
	77	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8711.5;
	77	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1683.5;
	78	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.6;
	78	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	911.5;
	78	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1115.8;
	78	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4182;
	78	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12026.4;
	78	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6617.3;
	78	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9407;
	78	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1846.7;
	79	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009.6;
	79	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	993.8;
	79	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1231.4;
	79	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4658.8;
	79	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13455.7;
	79	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7450.1;
	79	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10156.5;
	79	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2036.3;
	80	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1032.4;
	80	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1041.1;
	80	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1293.1;
	80	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4789.5;
	80	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14058.7;
	80	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7781.9;
	80	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10495.5;
	80	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2083.8;
	81	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1015;
	81	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1023.1;
	81	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1262.8;
	81	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4609.8;
	81	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13662.9;
	81	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7505.9;
	81	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10454.3;
	81	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2029.3;
	82	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	993.8;
	82	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	984.6;
	82	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1197.5;
	82	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4391.1;
	82	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13023.9;
	82	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7076.1;
	82	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10335.2;
	82	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1934.5;
	83	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	971.1;
	83	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	944.5;
	83	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1127.4;
	83	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4140.9;
	83	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12409.1;
	83	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6705;
	83	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10117;
	83	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1826.5;
	84	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.5;
	84	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	897.7;
	84	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1053.8;
	84	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3845.8;
	84	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11806.7;
	84	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6344.7;
	84	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9832;
	84	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1773.9;
	85	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925.4;
	85	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	857.7;
	85	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1002.2;
	85	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3605.6;
	85	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11238.1;
	85	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6025.4;
	85	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9597.1;
	85	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1710.9;
	86	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.9;
	86	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	829.4;
	86	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	969.1;
	86	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3427.6;
	86	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10822.6;
	86	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5793.5;
	86	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9414.1;
	86	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1672.6;
	87	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.6;
	87	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	804.9;
	87	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	930;
	87	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3299.1;
	87	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10434.3;
	87	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5624.3;
	87	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9227.2;
	87	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1679.5;
	88	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.8;
	88	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	788.3;
	88	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	906.2;
	88	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3225.9;
	88	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10215;
	88	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5544.7;
	88	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9077.2;
	88	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1652.8;
	89	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.1;
	89	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	794.2;
	89	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	912.1;
	89	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3212.8;
	89	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10274.5;
	89	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5589.8;
	89	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9009.3;
	89	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1638.5;
	90	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.8;
	90	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	850.1;
	90	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	961.3;
	90	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3326.3;
	90	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11166.2;
	90	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5997.2;
	90	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9393.5;
	90	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1730.7;
	91	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	968.6;
	91	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	919.1;
	91	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1061.7;
	91	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3774;
	91	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12228.9;
	91	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6708.4;
	91	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9978;
	91	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1870.9;
	92	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986.4;
	92	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	930.8;
	92	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1092.2;
	92	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3947.2;
	92	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12442.6;
	92	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6881.8;
	92	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10021.2;
	92	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1919.7;
	93	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	996.5;
	93	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	941.6;
	93	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1108.9;
	93	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4058.5;
	93	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12506.6;
	93	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6931.3;
	93	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9975;
	93	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1926.6;
	94	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985.3;
	94	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	924.8;
	94	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1092.2;
	94	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4044.8;
	94	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12249.9;
	94	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6774;
	94	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9715.7;
	94	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1888.5;
	95	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.9;
	95	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	884.5;
	95	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1044;
	95	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3897.4;
	95	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11587.9;
	95	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6392.8;
	95	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9195.1;
	95	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1844.2;
	96	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.9;
	96	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	856.1;
	96	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1005.9;
	96	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3677.2;
	96	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10961.7;
	96	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6015.9;
	96	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8736.6;
	96	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1773.1;
	97	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.6;
	97	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	850.3;
	97	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	976.4;
	97	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3505.6;
	97	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10638.2;
	97	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5824.9;
	97	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8416.2;
	97	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1762.8;
	98	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.6;
	98	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	853.7;
	98	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	978.7;
	98	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3451.1;
	98	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10590;
	98	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5757.3;
	98	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8325.5;
	98	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1742.2;
	99	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.7;
	99	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	862.6;
	99	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	982.9;
	99	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3436.3;
	99	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10662.8;
	99	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5796.1;
	99	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8322.9;
	99	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1765.3;
	100	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.4;
	100	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	874.7;
	100	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	993.8;
	100	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3453.6;
	100	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10905.9;
	100	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5929.5;
	100	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8457.4;
	100	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1805.4;
	101	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.4;
	101	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	906.3;
	101	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1023.7;
	101	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3550.9;
	101	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11516.9;
	101	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6209.7;
	101	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8852.8;
	101	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1913.1;
	102	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.2;
	102	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	967.8;
	102	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1089.2;
	102	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3823.4;
	102	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12687;
	102	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6832.6;
	102	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9651.9;
	102	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2063.1;
	103	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.2;
	103	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1056.3;
	103	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1197.7;
	103	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4323.7;
	103	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14221.4;
	103	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7707;
	103	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10614.3;
	103	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2317.7;
	104	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1020.7;
	104	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1093.2;
	104	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1240.2;
	104	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4418.6;
	104	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14782.3;
	104	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7876.8;
	104	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10925.7;
	104	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2307.7;
	105	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1007.5;
	105	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1088.5;
	105	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1205.3;
	105	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4199.3;
	105	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14420.9;
	105	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7501.2;
	105	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10810.1;
	105	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2293;
	106	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1002.4;
	106	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1065.7;
	106	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1161.3;
	106	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4069.7;
	106	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14033.7;
	106	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7123.4;
	106	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10657.9;
	106	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2258.8;
	107	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1001.2;
	107	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1026.2;
	107	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1126.6;
	107	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3927.4;
	107	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13616.7;
	107	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6715.4;
	107	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10445.4;
	107	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2151.2;
	108	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	995.1;
	108	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	977.7;
	108	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1087.2;
	108	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3789.4;
	108	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13001;
	108	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6358.6;
	108	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10159.2;
	108	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2016.4;
	109	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.7;
	109	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	927;
	109	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1057.2;
	109	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3680.3;
	109	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12354.2;
	109	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6110.9;
	109	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9864.5;
	109	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1922.1;
	110	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.6;
	110	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	892.4;
	110	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1033.8;
	110	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3605.5;
	110	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11831.7;
	110	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6035.2;
	110	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9632.6;
	110	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1844.9;
	111	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961;
	111	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	871.4;
	111	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1017.1;
	111	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3538.9;
	111	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11306.7;
	111	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5967.5;
	111	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9394.2;
	111	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1783;
	112	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951.1;
	112	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	857.5;
	112	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1008.8;
	112	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3521;
	112	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11027.1;
	112	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6012;
	112	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9250.3;
	112	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1763.4;
	113	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.1;
	113	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	881.4;
	113	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1032.6;
	113	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3579.5;
	113	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11374.4;
	113	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6188.3;
	113	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9299.1;
	113	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1761.5;
	114	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.2;
	114	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	933.2;
	114	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1068.5;
	114	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3733.2;
	114	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12409;
	114	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6607.6;
	114	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9775.9;
	114	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1928;
	115	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1004.9;
	115	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	979.7;
	115	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1118.4;
	115	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3996.6;
	115	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13206.3;
	115	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7013.3;
	115	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10326.7;
	115	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2103.5;
	116	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.8;
	116	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	981;
	116	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1116.6;
	116	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4034;
	116	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13204.6;
	116	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6961.4;
	116	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10345.2;
	116	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2152;
	117	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.2;
	117	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	966.2;
	117	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1103.5;
	117	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4018.4;
	117	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13040.8;
	117	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6820;
	117	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10250.7;
	117	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2142.6;
	118	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000.5;
	118	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	937.5;
	118	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1061.8;
	118	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3879.6;
	118	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12552.5;
	118	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6489.9;
	118	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9931.2;
	118	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2060.8;
	119	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	973.9;
	119	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	891;
	119	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1000.4;
	119	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3625.6;
	119	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11695.7;
	119	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5983.5;
	119	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9377.1;
	119	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1954;
	120	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951.8;
	120	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	849.6;
	120	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	935.6;
	120	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3312.8;
	120	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10867;
	120	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5497.6;
	120	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8830.1;
	120	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1841.5;
	121	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.9;
	121	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	831.1;
	121	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	906.8;
	121	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3080.9;
	121	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10325.2;
	121	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5181.2;
	121	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8521.3;
	121	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1771.1;
	122	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.2;
	122	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	815.6;
	122	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	891.3;
	122	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2964.9;
	122	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10064.9;
	122	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5039.8;
	122	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8351.5;
	122	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1757.1;
	123	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.4;
	123	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	812.1;
	123	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884.7;
	123	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2900.4;
	123	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9953.9;
	123	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4974;
	123	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8259.1;
	123	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1756.6;
	124	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.5;
	124	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	812.9;
	124	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.6;
	124	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2877.6;
	124	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9964.3;
	124	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5013.8;
	124	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8257.5;
	124	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1756.9;
	125	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.1;
	125	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	830;
	125	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	899.2;
	125	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2947;
	125	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10306.7;
	125	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5191.1;
	125	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8510;
	125	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1779;
	126	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.9;
	126	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	880.7;
	126	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	950.7;
	126	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3175.7;
	126	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11203.1;
	126	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5655.7;
	126	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9161.7;
	126	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1883.2;
	127	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985.3;
	127	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	949.8;
	127	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1047.4;
	127	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3653.3;
	127	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12526.7;
	127	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6426.3;
	127	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9993.2;
	127	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2074.6;
	128	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1010.5;
	128	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	976.2;
	128	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1089.2;
	128	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3735.5;
	128	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12980.4;
	128	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6661.4;
	128	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10214.1;
	128	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2070.3;
	129	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	995.5;
	129	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	966.4;
	129	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1068.5;
	129	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3565.1;
	129	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12674.1;
	129	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6522.2;
	129	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10091.5;
	129	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2037.1;
	130	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	993.6;
	130	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	962.9;
	130	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1063.9;
	130	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3504.3;
	130	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12559.3;
	130	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6503.3;
	130	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10143.5;
	130	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2029.1;
	131	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	988.6;
	131	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	960;
	131	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1055.8;
	131	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3434.3;
	131	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12498.7;
	131	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6480.3;
	131	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10170.9;
	131	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2039.4;
	132	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981.7;
	132	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	948.4;
	132	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1051;
	132	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3360.2;
	132	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12365.7;
	132	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6434.8;
	132	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10116.7;
	132	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1995;
	133	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970.3;
	133	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	928.3;
	133	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1037.2;
	133	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3312.8;
	133	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12162.4;
	133	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6349.3;
	133	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9987.3;
	133	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1920;
	134	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.1;
	134	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	917.3;
	134	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1026.9;
	134	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3299.7;
	134	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11946;
	134	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6263.3;
	134	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9902.2;
	134	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1855.7;
	135	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944;
	135	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	906.1;
	135	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1010.4;
	135	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3260.9;
	135	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11736.1;
	135	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6192;
	135	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9750.8;
	135	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1846.4;
	136	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.5;
	136	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	901.4;
	136	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1009.2;
	136	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3240.7;
	136	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11672.5;
	136	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6205;
	136	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9688;
	136	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1830.9;
	137	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.7;
	137	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	906.7;
	137	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1023.9;
	137	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3246;
	137	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11865.4;
	137	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6355.8;
	137	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9759.9;
	137	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1880.2;
	138	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.4;
	138	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	938.8;
	138	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1045.3;
	138	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3326.3;
	138	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12554;
	138	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6676.6;
	138	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10096.6;
	138	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1945.8;
	139	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	978.6;
	139	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	964.6;
	139	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1087.3;
	139	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3557.5;
	139	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12934.1;
	139	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6932.3;
	139	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10176.5;
	139	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2009.1;
	140	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	984.6;
	140	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	954.5;
	140	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1083.6;
	140	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3581.4;
	140	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12780.7;
	140	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6819.8;
	140	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10045.3;
	140	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1945.7;
	141	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990.3;
	141	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	938.5;
	141	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1066.1;
	141	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3549.4;
	141	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12502.3;
	141	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6609.9;
	141	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9808;
	141	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1880.1;
	142	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	982.9;
	142	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	904.9;
	142	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1028.8;
	142	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3443.5;
	142	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11954.6;
	142	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6216.5;
	142	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9362.6;
	142	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1743.7;
	143	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.4;
	143	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	854.7;
	143	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	977.2;
	143	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3241.9;
	143	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11007.7;
	143	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5699.5;
	143	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8746;
	143	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1610.9;
	144	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.3;
	144	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	804.3;
	144	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	922.4;
	144	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2957;
	144	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10095.5;
	144	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5182.4;
	144	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8150.4;
	144	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1476.4;
	145	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.9;
	145	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	765.8;
	145	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	918.6;
	145	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2779.4;
	145	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9412.5;
	145	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4728.6;
	145	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7836.2;
	145	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1392.3;
	146	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.6;
	146	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.8;
	146	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	909.2;
	146	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2636.4;
	146	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9109.1;
	146	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4559.1;
	146	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7609.5;
	146	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1350;
	147	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.4;
	147	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749.3;
	147	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	909.1;
	147	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2578;
	147	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8998.5;
	147	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4428.1;
	147	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7516.7;
	147	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1320.4;
	148	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.4;
	148	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	748.1;
	148	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	910;
	148	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2551.2;
	148	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8995.2;
	148	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4451.7;
	148	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7516.9;
	148	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1307.5;
	149	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.6;
	149	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	756;
	149	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	928.9;
	149	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2596.9;
	149	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9244.6;
	149	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4640.2;
	149	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7699.3;
	149	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1336.9;
	150	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.1;
	150	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	793.6;
	150	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	981.4;
	150	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2787;
	150	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10038;
	150	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5096.5;
	150	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8247;
	150	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1435.2;
	151	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009.1;
	151	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	857.4;
	151	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1077.3;
	151	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3219.8;
	151	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11271.2;
	151	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5837.6;
	151	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9045.4;
	151	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1620;
	152	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1037.2;
	152	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	882.5;
	152	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1116.7;
	152	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3341.7;
	152	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11705.8;
	152	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6066;
	152	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9256.6;
	152	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1671.9;
	153	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1018.6;
	153	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	872.3;
	153	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1093.7;
	153	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3247.7;
	153	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11388.6;
	153	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5866.6;
	153	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9227.6;
	153	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1642.4;
	154	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000.1;
	154	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	856.3;
	154	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1072.6;
	154	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3257.8;
	154	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11219.7;
	154	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5789.4;
	154	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9319.7;
	154	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1633.5;
	155	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	991.7;
	155	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	841.8;
	155	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1040.1;
	155	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3269.6;
	155	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11022.2;
	155	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5634.1;
	155	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9374.8;
	155	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1621;
	156	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	984.1;
	156	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	819.4;
	156	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1002.1;
	156	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3245.6;
	156	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10628.4;
	156	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5488.8;
	156	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9343.5;
	156	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1590.7;
	157	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.6;
	157	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	793;
	157	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	964.6;
	157	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3209;
	157	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10225.3;
	157	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5362.7;
	157	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9314.6;
	157	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1543.3;
	158	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.7;
	158	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	775.9;
	158	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	933.5;
	158	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3206.2;
	158	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9937.8;
	158	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5334.4;
	158	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9331.8;
	158	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1510.7;
	159	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.9;
	159	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753.1;
	159	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	910.9;
	159	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3216.7;
	159	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9687.7;
	159	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5339.1;
	159	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9299.2;
	159	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1464.2;
	160	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.3;
	160	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	739.4;
	160	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	902;
	160	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3213.3;
	160	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9522.3;
	160	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5355.7;
	160	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9237.1;
	160	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1435.3;
	161	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.8;
	161	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	731.2;
	161	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897.9;
	161	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3220.4;
	161	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9536.4;
	161	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5409.7;
	161	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9163;
	161	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1438.6;
	162	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.9;
	162	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	762.9;
	162	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	904;
	162	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3208.6;
	162	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9998.9;
	162	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5533.6;
	162	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9268.5;
	162	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1503.9;
	163	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967;
	163	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	817.9;
	163	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	977.3;
	163	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3443.3;
	163	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10766.1;
	163	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5909.1;
	163	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9649.7;
	163	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1621.9;
	164	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	983.7;
	164	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	817.9;
	164	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	997;
	164	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3494.5;
	164	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10783.7;
	164	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5913.9;
	164	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9520.7;
	164	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1631.4;
	165	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	987.6;
	165	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	815.1;
	165	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1004.7;
	165	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3475.4;
	165	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10678;
	165	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5818.2;
	165	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9383.8;
	165	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1612.4;
	166	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981.9;
	166	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	795.5;
	166	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	982.8;
	166	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3380.7;
	166	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10305.5;
	166	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5562.5;
	166	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9037.7;
	166	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1552.6;
	167	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.1;
	167	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755;
	167	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940.7;
	167	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3198.7;
	167	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9572.4;
	167	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5161.5;
	167	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8522.6;
	167	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1452.8;
	168	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.8;
	168	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719.7;
	168	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	910.6;
	168	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2947.5;
	168	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8880.4;
	168	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4747;
	168	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8023.3;
	168	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1356.8;
	169	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.9;
	169	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.3;
	169	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	848.9;
	169	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2652.5;
	169	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8539;
	169	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4602.1;
	169	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7575;
	169	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1382.4;
	170	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.6;
	170	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	718.4;
	170	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849;
	170	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2556.4;
	170	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8384.2;
	170	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4503.8;
	170	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7405.8;
	170	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1364;
	171	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.6;
	171	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720.3;
	171	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.6;
	171	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2513.1;
	171	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8386.4;
	171	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4535.1;
	171	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7323.8;
	171	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1341.7;
	172	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.9;
	172	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.9;
	172	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.5;
	172	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2507.4;
	172	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8539.3;
	172	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4603.2;
	172	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7331.7;
	172	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1311.3;
	173	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.6;
	173	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753.7;
	173	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897;
	173	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2563.2;
	173	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8964.6;
	173	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4845.4;
	173	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7549.7;
	173	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1319.4;
	174	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.1;
	174	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	811;
	174	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	957.1;
	174	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2769.5;
	174	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9932.2;
	174	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5386.5;
	174	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8137.8;
	174	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1394;
	175	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.9;
	175	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	893.9;
	175	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1061.4;
	175	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3213.9;
	175	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11376.3;
	175	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6212.9;
	175	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8904.9;
	175	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1574.5;
	176	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989;
	176	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	925.6;
	176	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1117.7;
	176	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3351.3;
	176	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11969.1;
	176	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6520.7;
	176	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9211.6;
	176	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1631.5;
	177	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	982.4;
	177	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	917.3;
	177	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1078.7;
	177	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3243.3;
	177	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11604.2;
	177	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6347.5;
	177	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9232.3;
	177	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1611.2;
	178	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981.6;
	178	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	899.7;
	178	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1043.6;
	178	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3237.3;
	178	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11319.7;
	178	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6162.8;
	178	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9406.3;
	178	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1602;
	179	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	977.3;
	179	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	877.5;
	179	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1009.2;
	179	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3215.8;
	179	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11015.4;
	179	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5938.6;
	179	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9478;
	179	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1582;
	180	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	971.5;
	180	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	856.3;
	180	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	974.8;
	180	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3203.3;
	180	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10771.5;
	180	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5783;
	180	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9476.8;
	180	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1546.6;
	181	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.7;
	181	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	837.6;
	181	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	930.9;
	181	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3197.9;
	181	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10537;
	181	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5639.1;
	181	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9461.4;
	181	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1504.2;
	182	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.9;
	182	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	817.9;
	182	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	900;
	182	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3198.4;
	182	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10376.1;
	182	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5505.8;
	182	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9422.5;
	182	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1463.5;
	183	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.1;
	183	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	804.4;
	183	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.7;
	183	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3178.4;
	183	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10068.6;
	183	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5373.3;
	183	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9353.4;
	183	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1441.9;
	184	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.1;
	184	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	793.8;
	184	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	872.8;
	184	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3140.2;
	184	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9902;
	184	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5303.8;
	184	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9274.9;
	184	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1422.6;
	185	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.1;
	185	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	807.4;
	185	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	876.9;
	185	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3129.2;
	185	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10016.9;
	185	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5285;
	185	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9245.4;
	185	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1443.1;
	186	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.7;
	186	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	839;
	186	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	888.9;
	186	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3141.3;
	186	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10544.2;
	186	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5431.6;
	186	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9460.1;
	186	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1514.5;
	187	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969.6;
	187	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	869.5;
	187	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	936.6;
	187	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3321;
	187	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10996.4;
	187	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5816.3;
	187	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9591.2;
	187	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1563.1;
	188	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	976.8;
	188	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	865.8;
	188	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	946.6;
	188	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3303.9;
	188	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10885.5;
	188	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5789;
	188	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9421.3;
	188	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1542.5;
	189	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	979.2;
	189	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	857;
	189	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	948.7;
	189	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3250.3;
	189	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10785.9;
	189	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5678.5;
	189	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9321.9;
	189	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1507.1;
	190	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	982.2;
	190	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	845.3;
	190	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940.1;
	190	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3177.9;
	190	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10611.1;
	190	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5495.1;
	190	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9147.2;
	190	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1462.5;
	191	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	978.5;
	191	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	821.4;
	191	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913.5;
	191	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3052.7;
	191	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10186.1;
	191	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5241.9;
	191	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8759.9;
	191	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1389.8;
	192	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	971.2;
	192	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	795.2;
	192	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.8;
	192	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2869;
	192	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9670.8;
	192	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4904.4;
	192	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8351.1;
	192	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1317.8;
	193	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	976.6;
	193	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	765.6;
	193	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.1;
	193	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2777.3;
	193	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9077.2;
	193	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4622.9;
	193	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8086.6;
	193	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1251.6;
	194	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969.5;
	194	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	759.8;
	194	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.2;
	194	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2667.9;
	194	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8812;
	194	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4454.8;
	194	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7833.6;
	194	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1224.1;
	195	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.7;
	195	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	757.7;
	195	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	863.6;
	195	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2607.3;
	195	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8680.8;
	195	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4419.6;
	195	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7680.3;
	195	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1206.5;
	196	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970.2;
	196	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	765.3;
	196	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	883.8;
	196	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2605;
	196	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8740.1;
	196	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4475.4;
	196	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7653.9;
	196	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1201.8;
	197	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	971.2;
	197	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	781.1;
	197	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	903.7;
	197	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2660;
	197	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8921.5;
	197	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4610.6;
	197	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7701.4;
	197	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1221.3;
	198	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.4;
	198	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	814.4;
	198	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	944.1;
	198	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2800.2;
	198	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9385;
	198	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4875.2;
	198	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7897.4;
	198	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1294.7;
	199	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	988.9;
	199	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	852.3;
	199	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	997;
	199	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3010;
	199	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10049.6;
	199	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5271.2;
	199	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8243.9;
	199	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1386.8;
	200	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1007.3;
	200	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	899.5;
	200	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1058.8;
	200	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3195.1;
	200	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10858.3;
	200	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5676.3;
	200	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8546.6;
	200	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1483.9;
	201	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.2;
	201	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	941.2;
	201	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1095.5;
	201	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3313.3;
	201	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11624;
	201	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5912.2;
	201	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8904.8;
	201	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1571.5;
	202	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1022;
	202	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	963.8;
	202	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1113.1;
	202	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3352;
	202	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12273.8;
	202	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5972.6;
	202	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9264.5;
	202	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1656.9;
	203	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.9;
	203	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	982.3;
	203	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1110.9;
	203	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3319.9;
	203	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12645.8;
	203	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5905.1;
	203	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9388.1;
	203	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1706.6;
	204	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1001.9;
	204	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	985.9;
	204	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1094.1;
	204	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3267.6;
	204	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12839.2;
	204	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5779.8;
	204	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9408;
	204	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1734.8;
	205	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.6;
	205	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	970;
	205	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1078.8;
	205	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3190.5;
	205	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12890.3;
	205	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5639;
	205	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9346.8;
	205	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1740.9;
	206	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.9;
	206	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	963.1;
	206	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1059.5;
	206	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3118;
	206	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12850.6;
	206	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5505.5;
	206	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9239.6;
	206	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1749.5;
	207	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.3;
	207	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	962;
	207	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1035.2;
	207	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3058.7;
	207	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12808.4;
	207	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5417.7;
	207	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9131.9;
	207	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1761.6;
	208	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.1;
	208	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	959.5;
	208	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1027.9;
	208	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3028.1;
	208	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12875.1;
	208	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5433.7;
	208	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9121;
	208	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1767.5;
	209	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925.7;
	209	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	970.7;
	209	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1036.8;
	209	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3000.3;
	209	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13104.3;
	209	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5616.5;
	209	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9222.1;
	209	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1812.1;
	210	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935;
	210	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	991.7;
	210	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1070.4;
	210	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3050;
	210	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13621.9;
	210	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6025.2;
	210	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9783.1;
	210	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1928.4;
	211	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.3;
	211	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1021.9;
	211	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1135.1;
	211	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3335.2;
	211	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14033;
	211	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6608.7;
	211	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10325.9;
	211	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1987.2;
	212	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	987.9;
	212	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1022.8;
	212	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1142.7;
	212	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3408.9;
	212	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13955.4;
	212	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6756.2;
	212	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10350.5;
	212	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1988.7;
	213	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990.9;
	213	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1010.4;
	213	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1136.3;
	213	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3434.7;
	213	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13818.2;
	213	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6804.5;
	213	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10377.7;
	213	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1985.7;
	214	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986.3;
	214	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	990.1;
	214	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1123.1;
	214	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3464.9;
	214	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13558.6;
	214	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6779.1;
	214	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10287.1;
	214	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1949.6;
	215	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	973.9;
	215	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	960.1;
	215	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1087.2;
	215	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3455.2;
	215	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13084;
	215	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6630.7;
	215	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10053.1;
	215	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1881.6;
	216	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.9;
	216	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	935.7;
	216	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1066.6;
	216	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3420.7;
	216	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12625.5;
	216	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6431.2;
	216	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9807.6;
	216	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1812.2;
	217	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.1;
	217	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	918.9;
	217	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1051.6;
	217	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3403.2;
	217	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12173.8;
	217	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6270.8;
	217	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9553.4;
	217	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1803.3;
	218	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.4;
	218	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	915.9;
	218	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1049.5;
	218	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3374.3;
	218	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11984.9;
	218	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6257.5;
	218	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9369.1;
	218	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1773.2;
	219	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.1;
	219	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	924.8;
	219	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1053.6;
	219	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3395.3;
	219	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11954.5;
	219	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6269.2;
	219	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9278.4;
	219	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1767.6;
	220	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.1;
	220	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	935.4;
	220	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1068.8;
	220	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3450.7;
	220	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12084;
	220	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6320;
	220	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9285.4;
	220	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1789.3;
	221	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.9;
	221	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	948.9;
	221	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1096.1;
	221	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3551.9;
	221	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12359.1;
	221	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6457.7;
	221	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9389.7;
	221	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1833.3;
	222	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	974.6;
	222	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	978.3;
	222	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1138;
	222	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3709.3;
	222	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12820.3;
	222	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6738.1;
	222	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9622.9;
	222	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1915.6;
	223	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.1;
	223	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1018.3;
	223	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1188.9;
	223	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3936;
	223	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13434.3;
	223	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7086.6;
	223	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9938.6;
	223	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2013.8;
	224	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	995.1;
	224	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1055.9;
	224	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1239.6;
	224	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4146.3;
	224	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14048;
	224	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7413.2;
	224	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10130.9;
	224	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2110;
	225	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	994;
	225	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1074.9;
	225	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1235.9;
	225	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4280.6;
	225	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14328.5;
	225	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7510.7;
	225	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10291.6;
	225	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2149.8;
	226	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	983;
	226	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1054.2;
	226	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1202.9;
	226	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4273.6;
	226	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14104.1;
	226	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7297.7;
	226	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10338.8;
	226	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2106.8;
	227	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.6;
	227	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1009.9;
	227	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1141.9;
	227	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4097;
	227	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13506.5;
	227	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6952.6;
	227	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10208.3;
	227	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1993.7;
	228	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948.9;
	228	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	970.1;
	228	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1071;
	228	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3842.6;
	228	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12893;
	228	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6570.8;
	228	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9968.8;
	228	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1883.8;
	229	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.2;
	229	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	929.6;
	229	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	988.2;
	229	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3600.4;
	229	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12334.1;
	229	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6230.6;
	229	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9712.2;
	229	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1804.8;
	230	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.5;
	230	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	896.2;
	230	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	948.9;
	230	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3381.2;
	230	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11759.1;
	230	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5939.9;
	230	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9445.6;
	230	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1725.8;
	231	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.9;
	231	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	867;
	231	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	916.7;
	231	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3219.1;
	231	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11311.1;
	231	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5698.9;
	231	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9243.8;
	231	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1653;
	232	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.9;
	232	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	855.6;
	232	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	905.6;
	232	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3138.8;
	232	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11097.5;
	232	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5606.6;
	232	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9134.4;
	232	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1626.2;
	233	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900;
	233	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	863.1;
	233	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	937;
	233	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3128.8;
	233	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11269.8;
	233	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5716.8;
	233	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9187.4;
	233	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1673.4;
	234	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.4;
	234	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	932.6;
	234	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	998.6;
	234	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3308.7;
	234	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12287.2;
	234	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6269.5;
	234	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9742.5;
	234	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1834.3;
	235	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.4;
	235	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	997.8;
	235	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1078.9;
	235	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3780.4;
	235	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13428.8;
	235	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6943.7;
	235	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10498.7;
	235	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2006.3;
	236	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	976.4;
	236	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1014.2;
	236	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1118.1;
	236	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4036.5;
	236	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13747.5;
	236	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7138.9;
	236	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10644.5;
	236	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2067.4;
	237	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	983.1;
	237	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1010;
	237	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1121.9;
	237	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4198.4;
	237	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13795.4;
	237	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7120.2;
	237	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10667.9;
	237	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2057.1;
	238	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	976.6;
	238	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	982.9;
	238	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1108.8;
	238	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4232.1;
	238	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13539.5;
	238	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6966.2;
	238	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10457.2;
	238	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2003.8;
	239	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.8;
	239	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	936.1;
	239	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1054.7;
	239	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4111.3;
	239	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12900.2;
	239	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6623.3;
	239	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10003.8;
	239	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1913.3;
	240	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.1;
	240	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	900.5;
	240	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1016.1;
	240	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3896.9;
	240	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12258.6;
	240	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6244.6;
	240	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9545.9;
	240	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1843.9;
	241	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.6;
	241	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	881.1;
	241	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1000.3;
	241	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3760.6;
	241	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11884.9;
	241	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6080.6;
	241	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9274.4;
	241	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1832.4;
	242	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.8;
	242	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	876.2;
	242	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	997.7;
	242	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3691.5;
	242	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11771.5;
	242	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6032.7;
	242	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9195.7;
	242	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1834.6;
	243	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.7;
	243	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	875.1;
	243	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	992.4;
	243	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3667.2;
	243	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11823.8;
	243	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6087.5;
	243	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9221.4;
	243	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1857.7;
	244	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.1;
	244	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	881.8;
	244	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1007.4;
	244	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3679.4;
	244	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12038.4;
	244	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6192.5;
	244	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9369.8;
	244	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1923.4;
	245	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.1;
	245	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	901.5;
	245	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1033.6;
	245	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3758.7;
	245	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12453.4;
	245	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6471.4;
	245	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9739.7;
	245	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1997.1;
	246	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.6;
	246	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	958.4;
	246	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1103;
	246	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4033.2;
	246	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13445.8;
	246	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7144.4;
	246	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10488.3;
	246	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2144.3;
	247	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1002.4;
	247	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1037.3;
	247	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1211.4;
	247	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4589.5;
	247	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14898.6;
	247	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8093.4;
	247	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11476.5;
	247	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2342.7;
	248	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1029.5;
	248	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1073.6;
	248	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1250.5;
	248	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4709.5;
	248	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15329.1;
	248	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8374.7;
	248	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11707.6;
	248	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2358.4;
	249	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1014.1;
	249	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1063.1;
	249	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1219.4;
	249	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4503;
	249	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14755;
	249	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8029.9;
	249	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11475.3;
	249	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2297.4;
	250	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.9;
	250	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1050;
	250	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1164.9;
	250	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4441.4;
	250	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14217.4;
	250	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7614.7;
	250	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11228;
	250	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2217.3;
	251	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	971.2;
	251	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1026.2;
	251	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1107.3;
	251	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4380.1;
	251	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13620.3;
	251	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7217.3;
	251	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11077.5;
	251	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2090.7;
	252	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.2;
	252	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	986;
	252	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1053.3;
	252	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4277.1;
	252	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13032.3;
	252	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6874.4;
	252	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10888.1;
	252	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2016.1;
	253	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.9;
	253	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	936.2;
	253	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	998.7;
	253	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4150.1;
	253	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12491.3;
	253	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6512.2;
	253	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10594.1;
	253	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1926.6;
	254	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.9;
	254	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	894.6;
	254	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	968.4;
	254	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4015;
	254	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11861;
	254	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6200.5;
	254	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10379.9;
	254	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1836.9;
	255	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.8;
	255	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	858.5;
	255	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	945.6;
	255	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3904.7;
	255	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11268.8;
	255	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5940.9;
	255	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10140.1;
	255	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1789.3;
	256	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.1;
	256	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	834.7;
	256	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	922.5;
	256	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3866.9;
	256	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10965.9;
	256	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5820;
	256	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9927.5;
	256	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1741.3;
	257	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.6;
	257	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	845.2;
	257	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	927;
	257	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3931.2;
	257	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11065.8;
	257	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5937.4;
	257	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9896.5;
	257	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1734;
	258	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.1;
	258	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	891.1;
	258	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	968.2;
	258	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4114.5;
	258	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11885;
	258	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6313.7;
	258	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10312.7;
	258	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1818.1;
	259	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950;
	259	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	956.5;
	259	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1071.5;
	259	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4518.3;
	259	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13025;
	259	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6986.7;
	259	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10900.8;
	259	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1973.7;
	260	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.4;
	260	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	970.6;
	260	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1106.5;
	260	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4664.6;
	260	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13232.9;
	260	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7133.4;
	260	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10860;
	260	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2039.1;
	261	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	991;
	261	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	969.4;
	261	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1118.2;
	261	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4651.3;
	261	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13214.4;
	261	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7143.9;
	261	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10672;
	261	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2048.4;
	262	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985.8;
	262	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	946.2;
	262	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1096;
	262	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4512.1;
	262	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12906.2;
	262	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6971.8;
	262	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10308.4;
	262	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2011.5;
	263	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.3;
	263	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	903;
	263	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1049.4;
	263	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4217.3;
	263	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12172;
	263	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6585.3;
	263	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9738;
	263	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1965.5;
	264	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.5;
	264	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	870.4;
	264	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1024.2;
	264	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3904.3;
	264	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11557.3;
	264	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6268.9;
	264	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9301.6;
	264	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1902.3;
	265	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.8;
	265	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	852.8;
	265	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1008.6;
	265	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3676.1;
	265	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11155.9;
	265	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6078.8;
	265	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8978;
	265	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1890.6;
	266	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.1;
	266	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	850.5;
	266	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1019.3;
	266	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3578.8;
	266	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11074.6;
	266	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6030.2;
	266	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8895.9;
	266	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1891;
	267	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953;
	267	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	859;
	267	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1033;
	267	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3533.5;
	267	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11138.8;
	267	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6085.5;
	267	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8931.6;
	267	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1906;
	268	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961;
	268	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	866.3;
	268	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1048;
	268	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3546;
	268	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11352.4;
	268	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6212.3;
	268	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9022.9;
	268	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1934.2;
	269	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	971.7;
	269	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	890.2;
	269	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1083.5;
	269	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3645.8;
	269	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11845.1;
	269	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6529.6;
	269	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9307.8;
	269	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2021;
	270	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000.4;
	270	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	943.7;
	270	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1155.4;
	270	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3937.8;
	270	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12900.3;
	270	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7153.2;
	270	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10085.8;
	270	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2149.3;
	271	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1041.9;
	271	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1033.2;
	271	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1270.5;
	271	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4493.5;
	271	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14431.5;
	271	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8042.1;
	271	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10993.8;
	271	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2373.7;
	272	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1066.7;
	272	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1069.7;
	272	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1319.1;
	272	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4592.1;
	272	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14890.1;
	272	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8230.5;
	272	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11192.3;
	272	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2340.9;
	273	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1037.7;
	273	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1033.1;
	273	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1259.9;
	273	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4320.9;
	273	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13935.2;
	273	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7703.7;
	273	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10916.4;
	273	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2251.4;
	274	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1010.6;
	274	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	987.6;
	274	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1173;
	274	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4120.8;
	274	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12891.3;
	274	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7067.8;
	274	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10602;
	274	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2125.8;
	275	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990.7;
	275	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	938;
	275	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1099.9;
	275	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3907.6;
	275	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12046.5;
	275	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6547.5;
	275	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10286;
	275	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1976.6;
	276	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969;
	276	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	888.5;
	276	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1029;
	276	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3695.9;
	276	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11285.2;
	276	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6113.7;
	276	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9971.6;
	276	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1835.6;
	277	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.5;
	277	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	844.6;
	277	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	973.3;
	277	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3522.5;
	277	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10646.5;
	277	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5797.5;
	277	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9684.4;
	277	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1724.7;
	278	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.3;
	278	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	812.6;
	278	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	933.3;
	278	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3409.6;
	278	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10205.1;
	278	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5579.9;
	278	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9535.1;
	278	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1648;
	279	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.2;
	279	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777.9;
	279	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	903.7;
	279	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3345.3;
	279	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9842.6;
	279	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5414;
	279	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9377.2;
	279	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1607.8;
	280	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.6;
	280	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	759.1;
	280	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	887.5;
	280	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3293.6;
	280	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9623;
	280	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5371.8;
	280	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9282.2;
	280	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1593;
	281	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.2;
	281	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	760.3;
	281	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	895.7;
	281	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3295.1;
	281	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9653.2;
	281	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5399.5;
	281	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9212.5;
	281	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1561.4;
	282	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903;
	282	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	790.8;
	282	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	921.3;
	282	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3360.7;
	282	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10152;
	282	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5651.8;
	282	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9442.4;
	282	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1667.7;
	283	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.5;
	283	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	855.7;
	283	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1001.9;
	283	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3700;
	283	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11090.2;
	283	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6244.3;
	283	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9972.5;
	283	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1823.9;
	284	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.7;
	284	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	872.9;
	284	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1027.5;
	284	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3824.8;
	284	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11254.9;
	284	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6369.8;
	284	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9997.2;
	284	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1878.9;
	285	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	988.3;
	285	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	872.7;
	285	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1045.7;
	285	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3889.7;
	285	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11306.2;
	285	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6346.1;
	285	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9992.4;
	285	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1887;
	286	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	983.1;
	286	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	864.6;
	286	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1034.3;
	286	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3854.1;
	286	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11074.9;
	286	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6144.3;
	286	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9737.7;
	286	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1853.1;
	287	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.3;
	287	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	833.2;
	287	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	997.8;
	287	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3701.2;
	287	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10486.2;
	287	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5786.7;
	287	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9242.5;
	287	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1783.1;
	288	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.6;
	288	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	802.4;
	288	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	956.7;
	288	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3481.2;
	288	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9880.4;
	288	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5446.9;
	288	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8783.1;
	288	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1702.8;
	289	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.2;
	289	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	790.8;
	289	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	943.2;
	289	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3331.1;
	289	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9540.1;
	289	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5302.2;
	289	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8583.8;
	289	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1658.3;
	290	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.2;
	290	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	788.1;
	290	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	948.7;
	290	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3279.5;
	290	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9478.7;
	290	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5274.9;
	290	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8536.6;
	290	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1669.1;
	291	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.7;
	291	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	794.3;
	291	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	958.7;
	291	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3276.4;
	291	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9525.1;
	291	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5323.1;
	291	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8538.1;
	291	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1687.7;
	292	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.6;
	292	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	813.4;
	292	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	978.6;
	292	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3336.3;
	292	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9766.7;
	292	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5452.8;
	292	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8694.8;
	292	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1728.4;
	293	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.4;
	293	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	840.1;
	293	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1014.9;
	293	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3459.9;
	293	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10294.2;
	293	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5784.9;
	293	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9059.8;
	293	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1791.5;
	294	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	991.4;
	294	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	900.3;
	294	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1085.4;
	294	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3758.3;
	294	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11361.6;
	294	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6468.8;
	294	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9872.1;
	294	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1940.6;
	295	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1028.1;
	295	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	983.8;
	295	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1201.3;
	295	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4332.9;
	295	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12859.4;
	295	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7389.4;
	295	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10850.1;
	295	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2178.8;
	296	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1054.4;
	296	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1014.2;
	296	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1248.1;
	296	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4470.6;
	296	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13372.3;
	296	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7624.8;
	296	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11094.5;
	296	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2179.2;
	297	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1029.9;
	297	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	975.8;
	297	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1192.7;
	297	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4188.7;
	297	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12653.5;
	297	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7202;
	297	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10777.3;
	297	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2077.4;
	298	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1008.8;
	298	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	926.9;
	298	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1119.2;
	298	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3985.9;
	298	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11893.2;
	298	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6659.8;
	298	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10404.5;
	298	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1948.2;
	299	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	984.9;
	299	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	881.4;
	299	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1046.6;
	299	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3804.3;
	299	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11235;
	299	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6219.4;
	299	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10084.9;
	299	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1851.6;
	300	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.8;
	300	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	840.4;
	300	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	992.4;
	300	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3652.1;
	300	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10672.9;
	300	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5863.8;
	300	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9767.5;
	300	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1748.7;
	301	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.7;
	301	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	802.6;
	301	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	946.7;
	301	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3531.3;
	301	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10202.5;
	301	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5573.1;
	301	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9511.4;
	301	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1656.9;
	302	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.6;
	302	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	787.1;
	302	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913.6;
	302	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3447.7;
	302	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9878.5;
	302	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5385.2;
	302	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9386.6;
	302	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1592.5;
	303	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915;
	303	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	770.8;
	303	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	886;
	303	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3375.9;
	303	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9612.6;
	303	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5278.6;
	303	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9267.9;
	303	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1582.3;
	304	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.3;
	304	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	761.1;
	304	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	875.4;
	304	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3326.9;
	304	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9422.5;
	304	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5242.2;
	304	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9159.6;
	304	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1552.8;
	305	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.3;
	305	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	763.6;
	305	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.7;
	305	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3348.9;
	305	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9499.1;
	305	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5299;
	305	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9139.5;
	305	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1586.4;
	306	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.6;
	306	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	790.7;
	306	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	899.7;
	306	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3439.6;
	306	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9979.2;
	306	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5545.5;
	306	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9349.2;
	306	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1652.6;
	307	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.2;
	307	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	843.7;
	307	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	975.5;
	307	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3693.7;
	307	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10798.4;
	307	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6098.5;
	307	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9784.2;
	307	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1798.2;
	308	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962;
	308	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	842.9;
	308	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	999.8;
	308	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3756.9;
	308	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10899.5;
	308	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6207.9;
	308	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9799.5;
	308	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1789.3;
	309	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	976.3;
	309	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	840.3;
	309	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1010.7;
	309	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3762.9;
	309	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10863.1;
	309	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6178.5;
	309	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9710.3;
	309	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1778;
	310	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	978;
	310	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	820;
	310	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	991.8;
	310	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3675.2;
	310	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10572.1;
	310	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5981.9;
	310	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9432.3;
	310	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1696.9;
	311	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.3;
	311	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	781.9;
	311	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	953.2;
	311	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3454.7;
	311	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9910.3;
	311	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5616.8;
	311	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8927.4;
	311	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1600;
	312	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948.5;
	312	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.7;
	312	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	914.4;
	312	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3177.3;
	312	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9251.7;
	312	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5232.2;
	312	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8445.9;
	312	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1506.1;
	313	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.2;
	313	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.6;
	313	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	894.8;
	313	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2982;
	313	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8867.5;
	313	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4937.8;
	313	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8182;
	313	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1472;
	314	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.7;
	314	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735.2;
	314	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	886.7;
	314	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2871.3;
	314	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8714.1;
	314	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4782.5;
	314	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8045.8;
	314	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1459.4;
	315	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.8;
	315	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.4;
	315	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	895.6;
	315	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2831.4;
	315	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8739.9;
	315	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4742.2;
	315	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8020;
	315	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1456.4;
	316	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.5;
	316	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.2;
	316	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	905.7;
	316	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2829.8;
	316	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8896;
	316	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4770.1;
	316	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8029.5;
	316	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1483.3;
	317	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.3;
	317	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	767.2;
	317	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	927.9;
	317	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2892.4;
	317	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9342.6;
	317	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4949.9;
	317	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8219.4;
	317	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1545.6;
	318	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	978.6;
	318	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	817.8;
	318	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	983.5;
	318	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3114.2;
	318	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10375.8;
	318	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5450.6;
	318	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8833.3;
	318	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1671.1;
	319	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1020.2;
	319	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	895.2;
	319	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1077.8;
	319	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3598.1;
	319	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11846.6;
	319	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6217.5;
	319	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9623.5;
	319	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1881.5;
	320	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1050;
	320	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	921.3;
	320	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1111.3;
	320	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3715.5;
	320	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12350.4;
	320	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6362.5;
	320	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9824.9;
	320	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1937.1;
	321	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1032.6;
	321	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	893.5;
	321	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1075.2;
	321	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3563.8;
	321	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11811.8;
	321	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6092.4;
	321	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9737.5;
	321	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1862.2;
	322	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1014.1;
	322	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	859.9;
	322	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1049.3;
	322	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3510;
	322	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11268.2;
	322	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5887.8;
	322	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9749.4;
	322	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1783.2;
	323	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.1;
	323	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	825;
	323	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1009.7;
	323	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3465.2;
	323	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10782.4;
	323	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5773.3;
	323	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9739.6;
	323	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1698.2;
	324	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	991.7;
	324	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	795.2;
	324	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	968.7;
	324	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3404.4;
	324	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10375.6;
	324	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5605.1;
	324	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9710.7;
	324	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1622.3;
	325	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	973.3;
	325	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	768.6;
	325	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	933;
	325	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3347.4;
	325	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10059.8;
	325	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5467.7;
	325	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9648.4;
	325	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1552.4;
	326	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.9;
	326	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753.5;
	326	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	906.2;
	326	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3348.4;
	326	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9856.2;
	326	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5380.9;
	326	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9611.8;
	326	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1517.5;
	327	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948.4;
	327	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735;
	327	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	891;
	327	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3364.2;
	327	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9611.5;
	327	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5295.8;
	327	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9471.5;
	327	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1471.6;
	328	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.3;
	328	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	724.8;
	328	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	887.8;
	328	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3379.1;
	328	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9459.6;
	328	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5282.4;
	328	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9394.6;
	328	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1437.7;
	329	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.2;
	329	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723.5;
	329	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	887.1;
	329	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3397.6;
	329	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9474.4;
	329	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5330.6;
	329	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9323.3;
	329	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1440.4;
	330	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.1;
	330	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.8;
	330	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	891.7;
	330	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3391.6;
	330	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9807;
	330	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5466.9;
	330	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9483.1;
	330	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1499.8;
	331	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	971.3;
	331	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	801.5;
	331	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	955.6;
	331	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3614.9;
	331	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10575.5;
	331	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5919.4;
	331	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9842.1;
	331	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1617.7;
	332	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990.8;
	332	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	804.5;
	332	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	973.5;
	332	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3669.2;
	332	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10602.6;
	332	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5968.5;
	332	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9774.4;
	332	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1623.7;
	333	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.7;
	333	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	794.9;
	333	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	972.2;
	333	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3629.8;
	333	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10429.8;
	333	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5851;
	333	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9579.5;
	333	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1592.9;
	334	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985;
	334	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	768.3;
	334	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	945.6;
	334	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3521.3;
	334	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9997.2;
	334	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5582.7;
	334	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9184;
	334	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1526.2;
	335	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.8;
	335	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	726.9;
	335	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	905.6;
	335	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3315.5;
	335	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9281.4;
	335	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5211.2;
	335	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8644.7;
	335	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1417.5;
	336	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.1;
	336	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	694.9;
	336	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	873.2;
	336	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3028.2;
	336	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8536;
	336	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4777.1;
	336	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8087.1;
	336	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1301.3;
	337	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.3;
	337	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	691.6;
	337	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	850;
	337	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2719.9;
	337	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8080.1;
	337	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4610.4;
	337	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7639.8;
	337	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1325.5;
	338	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.1;
	338	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	685.6;
	338	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	846.5;
	338	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2629.2;
	338	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7905.5;
	338	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4451.6;
	338	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7472.6;
	338	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1329.2;
	339	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.7;
	339	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	683.5;
	339	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	844.8;
	339	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2596.2;
	339	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7874.7;
	339	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4429.5;
	339	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7366.9;
	339	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1307.1;
	340	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.8;
	340	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	696.5;
	340	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860;
	340	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2631.3;
	340	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8069.4;
	340	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4450.3;
	340	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7400.7;
	340	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1280.8;
	341	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.5;
	341	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	716.1;
	341	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889.9;
	341	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2724.6;
	341	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8499.9;
	341	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4644;
	341	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7602.1;
	341	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1282.4;
	342	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.8;
	342	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	771.8;
	342	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	951.4;
	342	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2993.7;
	342	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9487.7;
	342	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5133.7;
	342	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8209.1;
	342	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1359.2;
	343	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.5;
	343	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	858.1;
	343	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1062.1;
	343	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3526.5;
	343	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11023.3;
	343	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5951.1;
	343	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9072.9;
	343	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1551.7;
	344	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1024.7;
	344	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	892.3;
	344	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1115.8;
	344	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3708.6;
	344	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11648.4;
	344	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6256;
	344	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9392.7;
	344	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1620.2;
	345	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1010.2;
	345	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	882.4;
	345	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1084.4;
	345	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3555.5;
	345	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11387.3;
	345	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6072.4;
	345	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9404.7;
	345	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1605;
	346	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989.2;
	346	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	859;
	346	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1042.1;
	346	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3447.7;
	346	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11099.5;
	346	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5917.3;
	346	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9439.2;
	346	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1579;
	347	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	977.6;
	347	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	844.2;
	347	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1011.2;
	347	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3371.2;
	347	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10754.2;
	347	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5712.9;
	347	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9501;
	347	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1556.8;
	348	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.2;
	348	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	810.5;
	348	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	969;
	348	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3309.3;
	348	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10356.6;
	348	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5558.6;
	348	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9469.4;
	348	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1516.4;
	349	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.1;
	349	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776.3;
	349	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	920.9;
	349	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3299.2;
	349	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9980.3;
	349	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5418.6;
	349	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9376.3;
	349	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1483.9;
	350	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.7;
	350	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	750.9;
	350	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	877.4;
	350	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3351.5;
	350	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9763.6;
	350	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5325.5;
	350	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9357.6;
	350	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1457.5;
	351	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.6;
	351	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.7;
	351	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.3;
	351	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3392.2;
	351	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9504.8;
	351	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5267.3;
	351	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9338.6;
	351	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1438.7;
	352	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.5;
	352	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.1;
	352	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	850.3;
	352	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3419;
	352	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9331.3;
	352	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5251.3;
	352	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9305.8;
	352	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1412.4;
	353	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.2;
	353	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	724.5;
	353	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849;
	353	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3440.3;
	353	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9306.9;
	353	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5249.8;
	353	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9235.2;
	353	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1375.3;
	354	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901;
	354	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.8;
	354	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.5;
	354	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3367.9;
	354	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9598;
	354	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5333;
	354	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9198.4;
	354	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1403.8;
	355	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.5;
	355	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	795.6;
	355	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	918;
	355	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3463.2;
	355	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10260.9;
	355	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5699.2;
	355	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9443;
	355	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1501.5;
	356	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.5;
	356	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	801.7;
	356	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	924.7;
	356	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3433.5;
	356	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10288.6;
	356	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5698.9;
	356	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9310.4;
	356	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1511.6;
	357	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.6;
	357	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	807.4;
	357	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	924.6;
	357	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3352.5;
	357	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10296.4;
	357	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5604.8;
	357	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9084;
	357	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1521.4;
	358	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924;
	358	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	798.6;
	358	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	915.9;
	358	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3257;
	358	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10148.5;
	358	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5447.5;
	358	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8871.9;
	358	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1503.5;
	359	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917;
	359	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	783.7;
	359	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	896.8;
	359	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3151.5;
	359	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9850.6;
	359	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5207.2;
	359	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8595.3;
	359	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1466;
	360	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.1;
	360	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	767.9;
	360	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868;
	360	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2990.1;
	360	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9435.8;
	360	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4906.8;
	360	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8276.1;
	360	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1415.1;
	361	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.3;
	361	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.9;
	361	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	840;
	361	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2905.7;
	361	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8899;
	361	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4880.2;
	361	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8053.7;
	361	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1359.6;
	362	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.4;
	362	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.2;
	362	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	828.2;
	362	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2818.9;
	362	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8690.4;
	362	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4788.6;
	362	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7933.5;
	362	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1355.7;
	363	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.5;
	363	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	731.8;
	363	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824.9;
	363	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2774.4;
	363	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8616.6;
	363	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4786.1;
	363	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7901.8;
	363	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1353.6;
	364	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.6;
	364	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735.8;
	364	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	828.7;
	364	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2759.4;
	364	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8666.5;
	364	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4829.4;
	364	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7916.5;
	364	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1354.8;
	365	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.9;
	365	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	750;
	365	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	837.3;
	365	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2779;
	365	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8850.1;
	365	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4944.1;
	365	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8026.4;
	365	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1370.2;
	366	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.3;
	366	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	778;
	366	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.5;
	366	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2883.2;
	366	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9273.8;
	366	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5143.7;
	366	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8296.6;
	366	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1436.6;
	367	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.3;
	367	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	809.3;
	367	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	908;
	367	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3035.4;
	367	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9855;
	367	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5432.4;
	367	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8668.5;
	367	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1523;
	368	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.1;
	368	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	850.4;
	368	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	955.3;
	368	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3170.2;
	368	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10509.3;
	368	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5767.1;
	368	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8946.2;
	368	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1599.9;
	369	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.4;
	369	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	891.2;
	369	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	986.1;
	369	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3322.1;
	369	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11091.3;
	369	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6068.1;
	369	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9292;
	369	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1669.3;
	370	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.8;
	370	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	923.9;
	370	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	996.8;
	370	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3419.1;
	370	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11569.5;
	370	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6262.1;
	370	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9583.1;
	370	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1718.8;
	371	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.1;
	371	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	947.9;
	371	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	997.3;
	371	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3440.9;
	371	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11710.2;
	371	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6215.1;
	371	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9741.8;
	371	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1721.7;
	372	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.6;
	372	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	953.1;
	372	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	975;
	372	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3404.6;
	372	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11705;
	372	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6078.3;
	372	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9709.9;
	372	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1704.2;
	373	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927;
	373	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	946.2;
	373	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	944.6;
	373	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3352.8;
	373	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11628.5;
	373	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5812.3;
	373	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9536.1;
	373	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1684.4;
	374	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.5;
	374	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	944.3;
	374	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	924;
	374	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3283.3;
	374	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11547.5;
	374	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5473.6;
	374	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9504.1;
	374	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1668;
	375	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.7;
	375	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	933.9;
	375	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	912.7;
	375	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3212.2;
	375	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11559.7;
	375	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5225.2;
	375	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9600.1;
	375	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1658.8;
	376	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.7;
	376	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	912;
	376	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	912.5;
	376	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3190.6;
	376	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11697.2;
	376	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5147.6;
	376	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9672.2;
	376	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1643.2;
	377	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.5;
	377	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	901.3;
	377	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	924.7;
	377	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3160.4;
	377	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11816;
	377	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5243.6;
	377	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9565;
	377	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1672;
	378	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.3;
	378	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	917.9;
	378	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	958.1;
	378	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3161.9;
	378	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12312.3;
	378	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5508.3;
	378	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9607.3;
	378	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1755.1;
	379	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.4;
	379	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	963.4;
	379	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1025;
	379	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3387.6;
	379	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12923.7;
	379	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6127.6;
	379	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10026.2;
	379	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1841;
	380	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.4;
	380	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	970.9;
	380	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1046.8;
	380	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3452.8;
	380	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12953.1;
	380	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6283.1;
	380	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10012.8;
	380	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1852.7;
	381	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.9;
	381	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	966.8;
	381	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1051.7;
	381	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3424.6;
	381	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12904.7;
	381	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6317.8;
	381	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9930.5;
	381	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1845.8;
	382	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.6;
	382	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	954.1;
	382	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1051.8;
	382	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3401;
	382	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12750.2;
	382	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6247.6;
	382	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9757.7;
	382	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1818.6;
	383	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.4;
	383	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	927.2;
	383	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1012.3;
	383	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3347;
	383	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12376.4;
	383	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6071.7;
	383	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9551.9;
	383	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1761.2;
	384	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.2;
	384	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	902.7;
	384	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	984.7;
	384	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3260.6;
	384	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11990.2;
	384	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5869.5;
	384	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9276.8;
	384	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1697.1;
	385	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.9;
	385	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	876.9;
	385	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	983.7;
	385	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3198.7;
	385	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11607.8;
	385	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5709.8;
	385	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9018.1;
	385	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1694.8;
	386	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.1;
	386	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	872.2;
	386	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	988.9;
	386	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3153.9;
	386	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11516;
	386	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5659;
	386	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8862.7;
	386	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1692.7;
	387	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.6;
	387	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	875.5;
	387	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	997.8;
	387	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3164.8;
	387	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11530;
	387	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5689.3;
	387	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8779.1;
	387	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1697.3;
	388	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.8;
	388	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	882.7;
	388	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1011.7;
	388	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3222.1;
	388	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11656;
	388	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5806.5;
	388	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8765.1;
	388	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1732.7;
	389	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.7;
	389	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	894.1;
	389	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1031.5;
	389	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3311.1;
	389	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11889.3;
	389	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5995.5;
	389	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8851.8;
	389	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1783.5;
	390	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925.4;
	390	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	915.8;
	390	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1067.4;
	390	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3461.1;
	390	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12294.4;
	390	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6282.9;
	390	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9082.1;
	390	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1861.4;
	391	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.5;
	391	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	946.8;
	391	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1122;
	391	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3676.5;
	391	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12861.2;
	391	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6688.3;
	391	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9412.4;
	391	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1960.9;
	392	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.4;
	392	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	983.3;
	392	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1178.9;
	392	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3889.6;
	392	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13380.1;
	392	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7084.1;
	392	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9638.6;
	392	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2059.2;
	393	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.9;
	393	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1000.4;
	393	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1168.7;
	393	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4061.1;
	393	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13510.4;
	393	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7164.4;
	393	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9788.6;
	393	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2066.7;
	394	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.6;
	394	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	962.9;
	394	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1115.6;
	394	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4035.2;
	394	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12976.5;
	394	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6831.4;
	394	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9826.4;
	394	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1948.7;
	395	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.9;
	395	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	905.8;
	395	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1036.4;
	395	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3827.4;
	395	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12126.2;
	395	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6391.1;
	395	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9653.5;
	395	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1783.7;
	396	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.4;
	396	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	847.5;
	396	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	972.4;
	396	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3576.8;
	396	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11257.2;
	396	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5987.6;
	396	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9421.9;
	396	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1642;
	397	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.7;
	397	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	804.4;
	397	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	881.7;
	397	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3354.8;
	397	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10563.2;
	397	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5667.6;
	397	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9203.1;
	397	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1552.9;
	398	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.6;
	398	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	771.3;
	398	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	840.1;
	398	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3196;
	398	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9987.1;
	398	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5367.9;
	398	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8944;
	398	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1475.7;
	399	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	862;
	399	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742.6;
	399	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	822.6;
	399	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3092;
	399	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9555.8;
	399	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5212.1;
	399	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8743.3;
	399	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1402.8;
	400	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	853.2;
	400	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.1;
	400	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	814.7;
	400	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3036.4;
	400	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9340.5;
	400	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5183.8;
	400	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8628.1;
	400	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1375.5;
	401	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	847;
	401	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	732.3;
	401	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	844.4;
	401	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3030.3;
	401	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9369.2;
	401	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5315.8;
	401	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8662.1;
	401	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1394;
	402	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	851.5;
	402	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	782.4;
	402	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	883.4;
	402	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3114.4;
	402	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9954.7;
	402	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5653.4;
	402	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9064.1;
	402	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1507.6;
	403	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.3;
	403	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	852;
	403	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	945.9;
	403	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3454.5;
	403	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10942.8;
	403	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6194.5;
	403	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9739.7;
	403	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1677.8;
	404	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.7;
	404	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	887;
	404	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	986.9;
	404	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3627.2;
	404	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11219.1;
	404	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6384.8;
	404	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9883.1;
	404	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1731.2;
	405	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.9;
	405	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	906.5;
	405	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	984.8;
	405	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3689.7;
	405	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11269.5;
	405	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6401.7;
	405	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9875.8;
	405	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1739.6;
	406	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912;
	406	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	915.9;
	406	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	996.2;
	406	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3704.5;
	406	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11225.9;
	406	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6342.4;
	406	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9769.2;
	406	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1733.7;
	407	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.8;
	407	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	902.3;
	407	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	974;
	407	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3601.7;
	407	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10908.2;
	407	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6110;
	407	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9482.7;
	407	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1676.9;
	408	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.4;
	408	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	885.2;
	408	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	954;
	408	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3418.5;
	408	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10621.8;
	408	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5830.9;
	408	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9149.9;
	408	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1621.8;
	409	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.6;
	409	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	883.1;
	409	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	956;
	409	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3269.9;
	409	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10608.8;
	409	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5679.7;
	409	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8900.4;
	409	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1610.3;
	410	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	881.3;
	410	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	892.6;
	410	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	964.7;
	410	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3147;
	410	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10708.6;
	410	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5657.6;
	410	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8849.5;
	410	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1619.5;
	411	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889;
	411	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	908.3;
	411	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	973.5;
	411	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3121.2;
	411	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10966.7;
	411	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5725.2;
	411	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8871.5;
	411	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1663.2;
	412	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.8;
	412	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	931.1;
	412	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1000.5;
	412	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3165.9;
	412	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11390.2;
	412	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5873.4;
	412	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8992.1;
	412	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1730.8;
	413	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910;
	413	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	951.7;
	413	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1031.7;
	413	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3275.1;
	413	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12022.4;
	413	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6175.6;
	413	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9316;
	413	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1815.4;
	414	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.8;
	414	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1003.7;
	414	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1101.3;
	414	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3530.9;
	414	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13016.6;
	414	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6682.2;
	414	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9875;
	414	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1961.9;
	415	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	974.5;
	415	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1074;
	415	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1209.8;
	415	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3984;
	415	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14210.5;
	415	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7314.7;
	415	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10531.4;
	415	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2106;
	416	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.4;
	416	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1118.4;
	416	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1256.8;
	416	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4166.2;
	416	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14937.3;
	416	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7594.2;
	416	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10890.1;
	416	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2167.4;
	417	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	998.2;
	417	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1103.4;
	417	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1256.2;
	417	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4040;
	417	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14874.2;
	417	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7442.8;
	417	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10887.8;
	417	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2137.4;
	418	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1004;
	418	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1072;
	418	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1241.3;
	418	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3869.4;
	418	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14499.6;
	418	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7107.4;
	418	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10763.3;
	418	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2049.2;
	419	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	997.6;
	419	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1041.3;
	419	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1200.8;
	419	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3625.9;
	419	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13954.6;
	419	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6718.8;
	419	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10524.1;
	419	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1930.2;
	420	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	988.1;
	420	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	998.5;
	420	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1141.2;
	420	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3424.2;
	420	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13276.7;
	420	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6367.5;
	420	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10207.2;
	420	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1859.6;
	421	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.3;
	421	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	944.8;
	421	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1079.2;
	421	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3274.4;
	421	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12509.8;
	421	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6033.6;
	421	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9909.3;
	421	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1762.9;
	422	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938;
	422	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	900.6;
	422	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1017.3;
	422	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3195.8;
	422	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11863;
	422	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5759.2;
	422	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9701.9;
	422	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1681.6;
	423	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.4;
	423	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	869.6;
	423	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	961;
	423	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3146.7;
	423	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11297.6;
	423	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5567.1;
	423	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9466.6;
	423	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1651.4;
	424	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.1;
	424	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	844.9;
	424	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	937.7;
	424	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3120.2;
	424	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10926.1;
	424	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5475.5;
	424	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9316.4;
	424	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1613.3;
	425	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.8;
	425	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	846.7;
	425	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	935;
	425	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3111.7;
	425	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10933.4;
	425	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5484.5;
	425	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9253.3;
	425	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1593;
	426	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.5;
	426	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	887.6;
	426	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	953.9;
	426	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3120.5;
	426	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11622.7;
	426	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5748.6;
	426	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9508.1;
	426	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1642.1;
	427	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.9;
	427	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	959.9;
	427	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1043.9;
	427	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3403.1;
	427	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12780;
	427	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6348.1;
	427	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10102.5;
	427	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1795.6;
	428	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969.4;
	428	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	977.4;
	428	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1081.2;
	428	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3537.6;
	428	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13059.2;
	428	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6497.3;
	428	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10139.4;
	428	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1857.7;
	429	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981.1;
	429	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	983.4;
	429	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1091.1;
	429	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3578.6;
	429	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13091.7;
	429	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6424.1;
	429	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10120.9;
	429	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1871.1;
	430	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	976.4;
	430	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	962.9;
	430	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1074.9;
	430	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3512.7;
	430	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12800.4;
	430	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6218.7;
	430	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9893;
	430	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1846.4;
	431	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.2;
	431	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	925.9;
	431	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1028.7;
	431	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3305.6;
	431	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12089;
	431	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5882.9;
	431	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9436.7;
	431	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1805.4;
	432	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.1;
	432	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	891.6;
	432	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	992.3;
	432	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3043.5;
	432	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11362.6;
	432	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5493.9;
	432	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8928.4;
	432	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1724;
	433	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.1;
	433	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	875.4;
	433	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	964.4;
	433	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2851.1;
	433	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10907.1;
	433	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5212.1;
	433	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8575.9;
	433	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1696.7;
	434	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.9;
	434	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	881.6;
	434	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	960.7;
	434	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2790.9;
	434	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10756.2;
	434	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5103;
	434	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8471.9;
	434	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1684.5;
	435	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920.4;
	435	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	878.2;
	435	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	957.7;
	435	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2783.1;
	435	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10715.1;
	435	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5158.5;
	435	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8447.6;
	435	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1687.5;
	436	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.8;
	436	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	882.5;
	436	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	952.4;
	436	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2833.4;
	436	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10878.3;
	436	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5256.9;
	436	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8567.7;
	436	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1714.3;
	437	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927;
	437	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	892.6;
	437	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	959.8;
	437	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2953.3;
	437	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11229.8;
	437	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5511.7;
	437	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8876.5;
	437	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1792;
	438	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.4;
	438	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	940.9;
	438	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1010;
	438	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3250.1;
	438	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12164.7;
	438	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6103.3;
	438	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9630.8;
	438	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1920.4;
	439	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981.8;
	439	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1018.2;
	439	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1100.1;
	439	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3821.1;
	439	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13459.8;
	439	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6925;
	439	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10476.3;
	439	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2157.1;
	440	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000.1;
	440	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1038.7;
	440	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1124.8;
	440	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3934.1;
	440	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13687.4;
	440	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7085.4;
	440	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10621.8;
	440	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2127.4;
	441	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	984.1;
	441	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1018.2;
	441	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1086.1;
	441	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3689.4;
	441	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13051.6;
	441	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6710.7;
	441	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10284;
	441	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2079.9;
	442	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	978.8;
	442	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1005.6;
	442	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1049.4;
	442	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3538.3;
	442	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12622.4;
	442	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6381.9;
	442	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9985.2;
	442	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2002.9;
	443	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.3;
	443	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	979;
	443	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1020.6;
	443	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3403.9;
	443	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11990;
	443	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6129.5;
	443	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9785.9;
	443	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1887.1;
	444	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.9;
	444	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	936.6;
	444	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	990.9;
	444	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3301.2;
	444	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11410.2;
	444	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5888.2;
	444	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9629.7;
	444	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1770.7;
	445	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928;
	445	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	893.1;
	445	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	955.4;
	445	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3231;
	445	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10970;
	445	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5706.1;
	445	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9499.7;
	445	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1686.6;
	446	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.2;
	446	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	856.5;
	446	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	918.6;
	446	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3118.7;
	446	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10631;
	446	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5501.8;
	446	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9422.9;
	446	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1627.5;
	447	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.2;
	447	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	830.9;
	447	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889.7;
	447	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3101.7;
	447	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10357.6;
	447	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5331.3;
	447	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9281.6;
	447	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1597.1;
	448	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.7;
	448	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	819.8;
	448	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	869.4;
	448	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3100.6;
	448	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10102.6;
	448	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5292.2;
	448	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9218.9;
	448	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1595.7;
	449	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.3;
	449	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	826;
	449	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	872.5;
	449	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3123;
	449	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9980.7;
	449	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5298.3;
	449	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9179.1;
	449	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1573.3;
	450	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.9;
	450	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	854.5;
	450	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884.7;
	450	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3120.3;
	450	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10366.1;
	450	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5473.7;
	450	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9301.4;
	450	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1642;
	451	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923;
	451	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	909.1;
	451	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	959;
	451	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3357.6;
	451	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11108.9;
	451	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5977.4;
	451	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9754.3;
	451	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1758.3;
	452	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.2;
	452	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	920.5;
	452	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	986.7;
	452	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3425.6;
	452	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11170.1;
	452	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6101.9;
	452	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9697.3;
	452	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1774.9;
	453	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.5;
	453	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	926.8;
	453	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	998.6;
	453	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3401.7;
	453	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11189.8;
	453	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6008.1;
	453	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9550.6;
	453	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1734.9;
	454	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.5;
	454	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	911.8;
	454	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	981.7;
	454	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3293.1;
	454	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11030.6;
	454	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5737.1;
	454	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9198.4;
	454	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1657.4;
	455	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.3;
	455	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	883.5;
	455	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	943.2;
	455	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3058.7;
	455	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10547.5;
	455	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5312.2;
	455	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8598.5;
	455	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1563.4;
	456	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.6;
	456	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	852.5;
	456	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	899.3;
	456	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2761.8;
	456	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10103;
	456	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4878.4;
	456	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8073.5;
	456	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1457.8;
	457	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896;
	457	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	839.9;
	457	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	893.8;
	457	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2559.1;
	457	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10022.3;
	457	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4607.7;
	457	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7716.9;
	457	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1375;
	458	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.1;
	458	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	835.8;
	458	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.9;
	458	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2453.3;
	458	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10086;
	458	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4490;
	458	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7547;
	458	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1365;
	459	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.2;
	459	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	846.4;
	459	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	911.7;
	459	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2412;
	459	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10302.9;
	459	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4463.7;
	459	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7514.5;
	459	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1378.3;
	460	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910;
	460	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	860.5;
	460	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	936;
	460	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2401.7;
	460	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10609.6;
	460	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4507.5;
	460	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7557;
	460	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1412.5;
	461	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.5;
	461	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	884.5;
	461	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	971.8;
	461	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2456.7;
	461	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11206.8;
	461	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4701.2;
	461	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7838;
	461	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1487.4;
	462	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.5;
	462	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	943.8;
	462	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1039.5;
	462	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2657.3;
	462	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12287.1;
	462	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5220.1;
	462	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8501.5;
	462	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1629.4;
	463	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000.9;
	463	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1027.9;
	463	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1161.2;
	463	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3139.4;
	463	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13856.9;
	463	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6062.6;
	463	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9423.7;
	463	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1882.1;
	464	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1024.9;
	464	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1056;
	464	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1199.1;
	464	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3247.6;
	464	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14335.5;
	464	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6328.4;
	464	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9636.4;
	464	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1902.6;
	465	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1008.3;
	465	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1042.6;
	465	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1153;
	465	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3127.5;
	465	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13876.9;
	465	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6244.2;
	465	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9557.5;
	465	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1892;
	466	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1005;
	466	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1036.9;
	466	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1122.3;
	466	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3144.5;
	466	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13528;
	466	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6267.5;
	466	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9611.4;
	466	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1893.5;
	467	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990.6;
	467	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1008.5;
	467	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1080.5;
	467	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3165.4;
	467	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12988;
	467	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6172.9;
	467	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9633.8;
	467	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1874.2;
	468	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.1;
	468	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	974.5;
	468	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1047.1;
	468	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3192.5;
	468	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12386.6;
	468	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6022.5;
	468	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9596.7;
	468	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1806.1;
	469	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.3;
	469	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	941.5;
	469	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1010.5;
	469	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3225.9;
	469	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11710.9;
	469	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5886;
	469	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9528.2;
	469	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1725.6;
	470	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.4;
	470	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	915.1;
	470	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	970.4;
	470	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3267.3;
	470	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11169.4;
	470	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5730.3;
	470	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9513.5;
	470	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1643.6;
	471	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.3;
	471	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	882.1;
	471	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	929.6;
	471	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3307.8;
	471	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10901.8;
	471	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5598.2;
	471	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9459.4;
	471	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1615.9;
	472	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.3;
	472	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	869.7;
	472	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	909.9;
	472	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3339.4;
	472	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10830.9;
	472	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5564.5;
	472	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9403.2;
	472	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1591.2;
	473	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.8;
	473	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	878.6;
	473	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	912.5;
	473	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3360.8;
	473	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11146.1;
	473	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5669.6;
	473	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9405.2;
	473	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1641.5;
	474	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.1;
	474	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	916.6;
	474	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	928.8;
	474	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3353.4;
	474	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11822.1;
	474	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5963.3;
	474	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9625.1;
	474	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1722;
	475	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.4;
	475	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	967.9;
	475	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	991.9;
	475	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3538.7;
	475	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12663.7;
	475	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6417.9;
	475	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9947.1;
	475	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1854.7;
	476	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.3;
	476	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	972.7;
	476	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1011.8;
	476	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3590.6;
	476	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12622.9;
	476	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6434.7;
	476	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9896.1;
	476	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1817.5;
	477	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954;
	477	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	964;
	477	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1015.7;
	477	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3555.3;
	477	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12328.7;
	477	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6309.1;
	477	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9718.5;
	477	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1778.1;
	478	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951;
	478	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	933.4;
	478	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	992.6;
	478	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3430.8;
	478	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11751.1;
	478	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5937;
	478	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9326.3;
	478	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1658.8;
	479	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.5;
	479	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	879.2;
	479	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	941.9;
	479	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3176;
	479	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10817.3;
	479	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5455;
	479	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8704.1;
	479	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1531.5;
	480	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.9;
	480	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	831.6;
	480	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	887.8;
	480	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2842.6;
	480	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9923.1;
	480	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4997.5;
	480	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8117.4;
	480	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1404.5;
	481	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.9;
	481	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	813.5;
	481	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.9;
	481	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2618.2;
	481	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9380.9;
	481	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4794.6;
	481	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7755.9;
	481	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1343.9;
	482	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.7;
	482	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	806.9;
	482	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	825.5;
	482	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2490.1;
	482	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9102.7;
	482	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4632.8;
	482	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7563.2;
	482	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1307.8;
	483	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.7;
	483	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	798.4;
	483	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	822.3;
	483	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2412.7;
	483	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8931.9;
	483	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4581.6;
	483	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7429.5;
	483	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1280.6;
	484	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.7;
	484	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	801.1;
	484	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	830;
	484	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2395.1;
	484	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8916.3;
	484	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4585.7;
	484	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7438.9;
	484	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1283.3;
	485	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.9;
	485	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	806.3;
	485	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	853;
	485	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2432.6;
	485	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9121.8;
	485	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4747.1;
	485	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7609.2;
	485	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1320.1;
	486	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.7;
	486	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	851;
	486	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	915;
	486	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2616.8;
	486	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9951;
	486	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5226.2;
	486	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8155.6;
	486	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1428.9;
	487	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	988.5;
	487	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	927.1;
	487	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1024.5;
	487	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3074.1;
	487	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11315.2;
	487	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5986.5;
	487	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9015.1;
	487	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1635.4;
	488	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1020.9;
	488	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	961.6;
	488	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1077.7;
	488	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3215;
	488	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11838.5;
	488	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6148.8;
	488	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9286.1;
	488	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1702.3;
	489	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1011.4;
	489	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	961.3;
	489	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1065.2;
	489	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3130.9;
	489	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11721.9;
	489	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5896.2;
	489	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9304.4;
	489	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1686.1;
	490	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1005.9;
	490	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	969.5;
	490	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1063.3;
	490	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3180.1;
	490	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11958;
	490	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5711.1;
	490	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9510.8;
	490	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1710.7;
	491	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000.5;
	491	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	978.4;
	491	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1048.4;
	491	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3231.8;
	491	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12283.3;
	491	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5643;
	491	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9735.3;
	491	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1706.6;
	492	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	998.1;
	492	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	979.9;
	492	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1022.1;
	492	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3264.4;
	492	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12502.1;
	492	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5535.2;
	492	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9856.8;
	492	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1697.4;
	493	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985.6;
	493	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	968.2;
	493	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	988.4;
	493	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3278.4;
	493	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12540.7;
	493	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5447.7;
	493	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9830.1;
	493	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1709.2;
	494	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981.1;
	494	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	965.3;
	494	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	962;
	494	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3312.2;
	494	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12630;
	494	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5353.4;
	494	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9832;
	494	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1735.9;
	495	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	971.3;
	495	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	963.1;
	495	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	944.3;
	495	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3331.7;
	495	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12796.3;
	495	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5271.9;
	495	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9831;
	495	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1737.7;
	496	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.2;
	496	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	961.6;
	496	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	934;
	496	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3307.8;
	496	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12982;
	496	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5260.8;
	496	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9658.7;
	496	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1745.8;
	497	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.5;
	497	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	972.9;
	497	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	933.2;
	497	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3281.2;
	497	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13286.6;
	497	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5316.5;
	497	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9545.6;
	497	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1821;
	498	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.9;
	498	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	997;
	498	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	948.2;
	498	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3240;
	498	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13896.7;
	498	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5492.6;
	498	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9729;
	498	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1939.8;
	499	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999;
	499	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1030;
	499	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1023.1;
	499	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3433.6;
	499	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14504.6;
	499	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6021.8;
	499	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10218.3;
	499	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2047.3;
	500	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1020.7;
	500	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1025.1;
	500	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1051;
	500	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3525.6;
	500	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14501.4;
	500	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6153.4;
	500	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10283.3;
	500	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2058;
	501	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1025.1;
	501	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1013.5;
	501	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1058.7;
	501	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3517.8;
	501	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14397.3;
	501	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6103.4;
	501	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10254.2;
	501	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2054.1;
	502	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1017.1;
	502	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	981.8;
	502	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1035.5;
	502	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3421.7;
	502	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13996.3;
	502	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5922.4;
	502	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10041.9;
	502	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1985.5;
	503	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.7;
	503	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	935.1;
	503	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	989.5;
	503	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3209.7;
	503	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13168.3;
	503	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5593.8;
	503	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9742.6;
	503	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1867.6;
	504	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	976.1;
	504	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	900.2;
	504	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	958.8;
	504	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2956.8;
	504	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12483.2;
	504	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5220;
	504	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9357;
	504	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1784.2;
	505	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.6;
	505	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	897.1;
	505	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	945.5;
	505	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2678.3;
	505	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12017.3;
	505	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5340.6;
	505	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9003.5;
	505	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1864.2;
	506	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.2;
	506	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	889.8;
	506	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	949.6;
	506	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2626.2;
	506	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11826.3;
	506	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5315.9;
	506	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8934.4;
	506	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1860;
	507	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951.7;
	507	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	889.7;
	507	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	961;
	507	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2629.3;
	507	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11780.2;
	507	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5341.7;
	507	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8982.5;
	507	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1844.6;
	508	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.1;
	508	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	893.7;
	508	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	981.8;
	508	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2681.9;
	508	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11891.6;
	508	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5491.8;
	508	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9142.2;
	508	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1834.8;
	509	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	971.6;
	509	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	913.4;
	509	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1016.7;
	509	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2812.8;
	509	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12334.3;
	509	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5785;
	509	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9508.5;
	509	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1865.5;
	510	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1003.1;
	510	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	967.5;
	510	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1089;
	510	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3148.9;
	510	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13357.7;
	510	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6438.1;
	510	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10317.1;
	510	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1970;
	511	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1043.5;
	511	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1045.7;
	511	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1208.6;
	511	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3804.7;
	511	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14842.4;
	511	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7389;
	511	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11343.1;
	511	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2182.1;
	512	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1068.6;
	512	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1077.5;
	512	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1256.9;
	512	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4029.1;
	512	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15311.2;
	512	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7681.7;
	512	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11676.7;
	512	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2234.2;
	513	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1048.6;
	513	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1064;
	513	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1210.8;
	513	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3823.8;
	513	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14703.6;
	513	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7380.5;
	513	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11634.6;
	513	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2205.1;
	514	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1025.7;
	514	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1044.5;
	514	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1162.3;
	514	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3675.8;
	514	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14219.8;
	514	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7070.6;
	514	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11580.7;
	514	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2153.5;
	515	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000.1;
	515	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1024;
	515	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1111.4;
	515	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3530.7;
	515	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13727.6;
	515	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6825.5;
	515	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11365.9;
	515	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2061.6;
	516	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	978.4;
	516	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	992;
	516	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1055.7;
	516	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3376.6;
	516	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13190;
	516	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6509.5;
	516	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11086.1;
	516	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1980.6;
	517	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956;
	517	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	954.1;
	517	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	999.9;
	517	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3239.4;
	517	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12606.5;
	517	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6213.5;
	517	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10776.3;
	517	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1897.6;
	518	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.5;
	518	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	915.7;
	518	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	957;
	518	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3152.6;
	518	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12124.8;
	518	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5958.5;
	518	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10529.5;
	518	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1811.8;
	519	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.6;
	519	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	884.5;
	519	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	930.2;
	519	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3089.5;
	519	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11652.1;
	519	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5778.2;
	519	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10280.6;
	519	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1750.5;
	520	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.4;
	520	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	862.2;
	520	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913;
	520	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3030.1;
	520	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11326.4;
	520	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5680.3;
	520	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10103.3;
	520	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1702.7;
	521	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.7;
	521	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	872.3;
	521	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	917.3;
	521	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3031.9;
	521	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11410.1;
	521	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5690.8;
	521	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10084.5;
	521	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1719.8;
	522	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.4;
	522	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	908.9;
	522	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	941;
	522	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3041.1;
	522	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12037.5;
	522	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5906.4;
	522	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10378.9;
	522	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1811.3;
	523	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.8;
	523	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	964.5;
	523	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1006.1;
	523	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3305.6;
	523	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13064.4;
	523	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6504.6;
	523	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10907;
	523	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1965.5;
	524	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.4;
	524	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	981.8;
	524	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1034.3;
	524	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3419;
	524	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13252.2;
	524	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6679.1;
	524	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10948.4;
	524	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1990.9;
	525	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	968.2;
	525	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	990.7;
	525	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1050.3;
	525	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3463.6;
	525	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13333.7;
	525	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6708.4;
	525	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10963.4;
	525	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2000.5;
	526	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.7;
	526	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	982.2;
	526	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1051.8;
	526	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3488;
	526	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13280.1;
	526	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6675.1;
	526	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10798.9;
	526	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1982;
	527	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.4;
	527	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	961.2;
	527	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1041.2;
	527	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3470.1;
	527	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12951.3;
	527	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6522.7;
	527	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10479.7;
	527	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1917;
	528	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948.5;
	528	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	935.6;
	528	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1019.7;
	528	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3389.5;
	528	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12561.4;
	528	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6302.5;
	528	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10100.6;
	528	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1848.3;
	529	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.5;
	529	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	893;
	529	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1007.1;
	529	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3421.1;
	529	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12138.5;
	529	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6192.6;
	529	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9933.4;
	529	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1836.9;
	530	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.2;
	530	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	890.8;
	530	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1007.3;
	530	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3408.2;
	530	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12077.3;
	530	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6172.8;
	530	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9829.5;
	530	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1836.5;
	531	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.9;
	531	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	893.5;
	531	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1018.7;
	531	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3426;
	531	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12125.4;
	531	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6213.7;
	531	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9805.3;
	531	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1851.8;
	532	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951.3;
	532	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	901.7;
	532	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1034.9;
	532	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3476.5;
	532	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12269.8;
	532	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6344.5;
	532	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9881.1;
	532	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1884.1;
	533	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.7;
	533	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	917.6;
	533	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1057.1;
	533	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3580;
	533	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12490.1;
	533	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6528.1;
	533	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10040.7;
	533	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1933.1;
	534	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.5;
	534	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	948.9;
	534	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1101.2;
	534	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3770.8;
	534	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12940.2;
	534	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6891.7;
	534	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10360.4;
	534	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2022.5;
	535	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986.2;
	535	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	987.4;
	535	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1155.5;
	535	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4059.2;
	535	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13542.6;
	535	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7311;
	535	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10802.4;
	535	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2136.1;
	536	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1001.4;
	536	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1023;
	536	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1212.8;
	536	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4304.9;
	536	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14074.3;
	536	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7719.4;
	536	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11132.4;
	536	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2224.2;
	537	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000.3;
	537	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1042.7;
	537	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1208.4;
	537	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4374.3;
	537	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14386.3;
	537	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7736.2;
	537	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11236.3;
	537	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2222.7;
	538	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	982.9;
	538	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1029.4;
	538	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1150.1;
	538	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4199.1;
	538	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14315.5;
	538	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7275.7;
	538	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11064.2;
	538	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2151.1;
	539	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.5;
	539	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	988;
	539	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1080.4;
	539	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3899.6;
	539	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13664.7;
	539	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6807.4;
	539	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10690.7;
	539	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2027.3;
	540	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.7;
	540	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	935.4;
	540	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1011;
	540	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3586.2;
	540	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12743;
	540	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6325.8;
	540	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10272.2;
	540	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1886.9;
	541	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914;
	541	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	875;
	541	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	950.5;
	541	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3351.8;
	541	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11837.8;
	541	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5890;
	541	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9852.4;
	541	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1754.8;
	542	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.2;
	542	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	829.6;
	542	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897.2;
	542	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3168.7;
	542	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11019;
	542	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5522.3;
	542	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9414.8;
	542	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1637.7;
	543	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.4;
	543	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	797.9;
	543	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.4;
	543	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3043.1;
	543	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10372.4;
	543	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5268.2;
	543	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9129.3;
	543	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1545.1;
	544	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	876.2;
	544	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776.5;
	544	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838.7;
	544	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2992.3;
	544	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10035.2;
	544	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5129.3;
	544	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8984.7;
	544	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1490.5;
	545	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.5;
	545	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	778.6;
	545	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838.6;
	545	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2969.8;
	545	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10013.9;
	545	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5119.5;
	545	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8921.4;
	545	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1500;
	546	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	876;
	546	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	805.5;
	546	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	855.4;
	546	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3006.2;
	546	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10549.7;
	546	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5300.8;
	546	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9178.9;
	546	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1579.6;
	547	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.8;
	547	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	868.5;
	547	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	926.3;
	547	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3300.9;
	547	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11528;
	547	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5850.3;
	547	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9895.3;
	547	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1758.2;
	548	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934;
	548	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	886.5;
	548	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	955.9;
	548	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3446.3;
	548	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11859.9;
	548	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6049.4;
	548	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10116.1;
	548	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1829.9;
	549	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.3;
	549	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	888.1;
	549	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	956.2;
	549	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3490.3;
	549	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11930.8;
	549	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6114.4;
	549	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10127.6;
	549	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1867.7;
	550	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.2;
	550	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	880.7;
	550	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	948.7;
	550	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3520.5;
	550	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11822.8;
	550	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6073.4;
	550	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10076.6;
	550	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1862.2;
	551	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912;
	551	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	862;
	551	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	922.5;
	551	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3516.9;
	551	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11525.8;
	551	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5955.8;
	551	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9901.2;
	551	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1824.5;
	552	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.1;
	552	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	837.8;
	552	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897.3;
	552	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3449.7;
	552	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11118.1;
	552	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5813.6;
	552	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9656.2;
	552	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1772.9;
	553	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.8;
	553	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	830.1;
	553	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884.3;
	553	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3364.7;
	553	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10846.3;
	553	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5677.5;
	553	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9418;
	553	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1748.4;
	554	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.7;
	554	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	824.5;
	554	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.6;
	554	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3332.2;
	554	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10674.2;
	554	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5618.7;
	554	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9316.4;
	554	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1750.6;
	555	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.3;
	555	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	826.5;
	555	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889.3;
	555	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3329.5;
	555	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10627.6;
	555	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5645;
	555	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9282.6;
	555	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1747;
	556	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.1;
	556	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	827.8;
	556	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	898.9;
	556	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3351.6;
	556	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10692.3;
	556	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5732.5;
	556	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9325.2;
	556	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1764.4;
	557	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.4;
	557	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	832.2;
	557	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	915.4;
	557	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3396.2;
	557	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10842.2;
	557	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5895.8;
	557	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9425.2;
	557	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1799.4;
	558	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.3;
	558	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	853.9;
	558	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	951.6;
	558	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3509.3;
	558	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11185.3;
	558	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6158.2;
	558	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9650.8;
	558	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1860.9;
	559	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.9;
	559	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	882.2;
	559	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1003.8;
	559	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3663.6;
	559	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11694.6;
	559	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6523.7;
	559	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9916;
	559	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1940.6;
	560	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.2;
	560	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	915.7;
	560	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1052;
	560	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3774.3;
	560	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12179.3;
	560	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6853.3;
	560	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10067.6;
	560	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2011.7;
	561	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.9;
	561	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	936.9;
	561	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1043.1;
	561	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3796.9;
	561	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12427.8;
	561	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6868.7;
	561	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10128.2;
	561	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2024;
	562	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.7;
	562	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	920.7;
	562	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1007.8;
	562	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3635.8;
	562	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12252.4;
	562	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6523.4;
	562	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9933.7;
	562	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1939.3;
	563	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.1;
	563	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	882.9;
	563	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	953.5;
	563	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3384.8;
	563	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11693.9;
	563	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6096.5;
	563	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9634.5;
	563	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1789.1;
	564	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879;
	564	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	839.7;
	564	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	900.8;
	564	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3187.2;
	564	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11037.9;
	564	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5698.3;
	564	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9292.4;
	564	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1649.1;
	565	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871.7;
	565	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	807.1;
	565	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	822.1;
	565	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3093.7;
	565	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10494.5;
	565	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5393.1;
	565	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9082.1;
	565	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1562.3;
	566	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867.4;
	566	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	778.6;
	566	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	788.4;
	566	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3038.3;
	566	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10029.5;
	566	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5183.6;
	566	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8938.6;
	566	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1483.6;
	567	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	858.1;
	567	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755.6;
	567	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	771.5;
	567	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2998.2;
	567	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9668.8;
	567	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5016.1;
	567	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8783.2;
	567	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1411.4;
	568	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	854.3;
	568	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.1;
	568	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	765.9;
	568	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2979.4;
	568	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9387.7;
	568	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4937.7;
	568	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8699.2;
	568	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1375.7;
	569	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	847.7;
	569	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.4;
	569	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	784.9;
	569	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2981;
	569	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9307.1;
	569	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4979.2;
	569	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8679.7;
	569	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1379.8;
	570	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	843.1;
	570	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	766.4;
	570	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	798.2;
	570	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2996.2;
	570	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9709.9;
	570	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5142.4;
	570	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8869.2;
	570	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1444.9;
	571	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.1;
	571	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	822.6;
	571	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.6;
	571	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3260.2;
	571	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10651.8;
	571	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5650.6;
	571	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9530.4;
	571	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1607.3;
	572	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.1;
	572	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	834;
	572	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897;
	572	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3374;
	572	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10828.2;
	572	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5826.1;
	572	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9617.1;
	572	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1646.2;
	573	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.7;
	573	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	823.9;
	573	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	891.5;
	573	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3352.6;
	573	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10686.2;
	573	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5683;
	573	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9433.1;
	573	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1622.9;
	574	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	876.4;
	574	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	797.6;
	574	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	877.6;
	574	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3258.5;
	574	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10348.8;
	574	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5449.3;
	574	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9081.3;
	574	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1566.9;
	575	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	859.8;
	575	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	754.8;
	575	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	832.5;
	575	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3054.1;
	575	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9688.7;
	575	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4984.4;
	575	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8577.3;
	575	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1463.4;
	576	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	838.9;
	576	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	716.7;
	576	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	788.5;
	576	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2772;
	576	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8969.4;
	576	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4578.1;
	576	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8060.8;
	576	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1363.8;
	577	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	850.3;
	577	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	682.7;
	577	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	764.2;
	577	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2598.8;
	577	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8307.7;
	577	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4299.8;
	577	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7822.8;
	577	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1256.8;
	578	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	842.3;
	578	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	672.4;
	578	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	752.9;
	578	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2491.5;
	578	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8029.2;
	578	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4146.2;
	578	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7657;
	578	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1224.3;
	579	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	842.6;
	579	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	665.8;
	579	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	741.7;
	579	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2434.1;
	579	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7873.7;
	579	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4106.3;
	579	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7562.5;
	579	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1220;
	580	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	848.2;
	580	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	669;
	580	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	747.4;
	580	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2419;
	580	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7890;
	580	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4153;
	580	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7605.8;
	580	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1260.8;
	581	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	859.7;
	581	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	681.7;
	581	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	765.4;
	581	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2475.7;
	581	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8161.3;
	581	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4311.2;
	581	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7865;
	581	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1310.5;
	582	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.6;
	582	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.9;
	582	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	821.8;
	582	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2694.2;
	582	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8947.5;
	582	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4812.2;
	582	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8466.1;
	582	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1430.4;
	583	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934;
	583	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	812.6;
	583	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	929.2;
	583	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3167.5;
	583	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10292.9;
	583	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5581;
	583	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9310.6;
	583	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1608.7;
	584	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.3;
	584	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	847.8;
	584	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	976.7;
	584	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3290.9;
	584	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10764.6;
	584	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5817.4;
	584	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9517.5;
	584	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1615.1;
	585	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.6;
	585	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	829.4;
	585	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	955.6;
	585	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3172.5;
	585	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10333;
	585	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5631.9;
	585	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9403.1;
	585	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1569.1;
	586	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.2;
	586	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	816.9;
	586	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	934.4;
	586	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3156.3;
	586	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10117.4;
	586	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5573.1;
	586	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9473.4;
	586	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1551;
	587	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.4;
	587	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	805.9;
	587	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	905.1;
	587	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3165.7;
	587	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10017.6;
	587	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5474.5;
	587	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9539;
	587	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1524.2;
	588	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.2;
	588	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	794.1;
	588	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	879.4;
	588	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3169.8;
	588	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9890.8;
	588	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5392.6;
	588	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9583.9;
	588	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1547.3;
	589	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.4;
	589	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776.8;
	589	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.2;
	589	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3195.2;
	589	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9765.7;
	589	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5374.6;
	589	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9628.8;
	589	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1549.3;
	590	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.9;
	590	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	762.3;
	590	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	853;
	590	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3234.5;
	590	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9660.9;
	590	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5379.3;
	590	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9648.6;
	590	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1543;
	591	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.4;
	591	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	750.4;
	591	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.2;
	591	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3262.8;
	591	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9493.5;
	591	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5386.8;
	591	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9525.2;
	591	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1557.2;
	592	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.4;
	592	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.1;
	592	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851.4;
	592	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3280.2;
	592	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9347.6;
	592	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5399;
	592	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9448.1;
	592	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1545.2;
	593	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887;
	593	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.7;
	593	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.4;
	593	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3271.5;
	593	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9361.3;
	593	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5432.1;
	593	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9417.7;
	593	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1526.4;
	594	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.2;
	594	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	760;
	594	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.6;
	594	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3266.9;
	594	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9687.7;
	594	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5537.9;
	594	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9563.1;
	594	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1511.8;
	595	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.1;
	595	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	822.3;
	595	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	918.4;
	595	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3487.7;
	595	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10527.8;
	595	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5929.2;
	595	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9923.1;
	595	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1597;
	596	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.3;
	596	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	834;
	596	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	942.1;
	596	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3562.7;
	596	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10664.4;
	596	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5989.7;
	596	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9936;
	596	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1610.9;
	597	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.6;
	597	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	836.1;
	597	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	941.6;
	597	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3508;
	597	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10539.3;
	597	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5815.6;
	597	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9719.4;
	597	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1568.7;
	598	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.5;
	598	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	815.8;
	598	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	919.5;
	598	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3364.2;
	598	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10159.1;
	598	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5489.9;
	598	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9298;
	598	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1499.2;
	599	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904;
	599	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	781.8;
	599	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	876.3;
	599	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3095.4;
	599	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9506.8;
	599	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5031;
	599	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8683.5;
	599	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1443;
	600	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.5;
	600	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755.3;
	600	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.2;
	600	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2787.5;
	600	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8875.7;
	600	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4567.5;
	600	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8107;
	600	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1350.9;
	601	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.7;
	601	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.9;
	601	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	844.7;
	601	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2561.5;
	601	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8602.3;
	601	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4277.8;
	601	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7713.2;
	601	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1327.7;
	602	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.1;
	602	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742;
	602	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.8;
	602	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2441.6;
	602	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8515.5;
	602	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4133.1;
	602	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7514.2;
	602	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1306.2;
	603	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	881.1;
	603	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746;
	603	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	853.1;
	603	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2377.8;
	603	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8562.3;
	603	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4092.5;
	603	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7386.9;
	603	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1298.1;
	604	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.9;
	604	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	762.1;
	604	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	864.2;
	604	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2359.2;
	604	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8780;
	604	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4154.6;
	604	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7377.2;
	604	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1322.5;
	605	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.5;
	605	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	784.4;
	605	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	887;
	605	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2402.1;
	605	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9311.9;
	605	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4352.6;
	605	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7555;
	605	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1400.6;
	606	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.9;
	606	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	844.9;
	606	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	947.4;
	606	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2591.9;
	606	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10447.9;
	606	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4901.6;
	606	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8139;
	606	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1532.8;
	607	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.8;
	607	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	940.9;
	607	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1047.4;
	607	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3045.1;
	607	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12077.5;
	607	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5779.5;
	607	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9025.7;
	607	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1789.9;
	608	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	991.9;
	608	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	987.8;
	608	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1098.8;
	608	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3209.2;
	608	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12787.4;
	608	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6121.3;
	608	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9420.9;
	608	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1826.4;
	609	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	987.2;
	609	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	980.9;
	609	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1090;
	609	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3151.8;
	609	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12442.7;
	609	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6101.3;
	609	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9489.3;
	609	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1859;
	610	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	995.7;
	610	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	976.7;
	610	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1083.9;
	610	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3192.2;
	610	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12300.2;
	610	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6182.2;
	610	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9624.9;
	610	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1857.6;
	611	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1005.2;
	611	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	963;
	611	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1087.9;
	611	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3262.4;
	611	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12178.6;
	611	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6216.7;
	611	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9742.1;
	611	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1817.3;
	612	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1006.5;
	612	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	943.8;
	612	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1083.9;
	612	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3319.7;
	612	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11844.8;
	612	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6246;
	612	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9782;
	612	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1751.8;
	613	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	997.1;
	613	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	904;
	613	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1075.9;
	613	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3357.6;
	613	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11594.6;
	613	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6303.1;
	613	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9740.8;
	613	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1694.4;
	614	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1003.6;
	614	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	880;
	614	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1069.4;
	614	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3336.2;
	614	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11553.1;
	614	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6413.6;
	614	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9764.1;
	614	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1654;
	615	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1007.6;
	615	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	861;
	615	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1064.2;
	615	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3335.8;
	615	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11558.3;
	615	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6452.1;
	615	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9740.7;
	615	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1650.3;
	616	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1012.7;
	616	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	852.6;
	616	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1066.3;
	616	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3372.3;
	616	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11632.7;
	616	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6567.7;
	616	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9814.4;
	616	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1688;
	617	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1019.8;
	617	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	864.8;
	617	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1096.5;
	617	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3487.6;
	617	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11954.5;
	617	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6820.6;
	617	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10004;
	617	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1718.9;
	618	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1022.5;
	618	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	894.2;
	618	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1128.7;
	618	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3638.5;
	618	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12478;
	618	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7134.6;
	618	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10291.8;
	618	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1852.7;
	619	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1049.9;
	619	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	948.9;
	619	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1185.9;
	619	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3954.3;
	619	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13080.2;
	619	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7586.6;
	619	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10561.6;
	619	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1977.9;
	620	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1055.8;
	620	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	959;
	620	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1202.9;
	620	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4076.9;
	620	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13050.4;
	620	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7587.5;
	620	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10567.6;
	620	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2004.8;
	621	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1047.6;
	621	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	951.4;
	621	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1201;
	621	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4098.6;
	621	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12870;
	621	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7443.6;
	621	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10398.4;
	621	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1957.6;
	622	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030;
	622	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	928.5;
	622	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1157.4;
	622	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4022.1;
	622	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12397.1;
	622	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7092.7;
	622	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9994.1;
	622	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1871.4;
	623	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	997.3;
	623	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	880.4;
	623	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1091.4;
	623	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3797.9;
	623	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11575.9;
	623	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6564;
	623	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9404.3;
	623	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1764.5;
	624	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970.8;
	624	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	844.7;
	624	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1027.2;
	624	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3536;
	624	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10886.6;
	624	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6136.1;
	624	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8862.3;
	624	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1668.7;
	625	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948.7;
	625	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	835;
	625	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1002.2;
	625	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3361.5;
	625	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10561.4;
	625	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5944.7;
	625	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8536.7;
	625	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1621.6;
	626	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.8;
	626	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	837.5;
	626	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1007.4;
	626	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3305.2;
	626	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10548.7;
	626	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5891.6;
	626	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8448.5;
	626	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1620.1;
	627	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.2;
	627	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	847.7;
	627	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1015.6;
	627	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3280.6;
	627	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10665.9;
	627	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5888.1;
	627	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8456.3;
	627	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1651.4;
	628	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.1;
	628	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	864.3;
	628	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1031.2;
	628	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3312.8;
	628	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10939.1;
	628	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5952.8;
	628	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8597.5;
	628	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1709.2;
	629	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.3;
	629	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	896.6;
	629	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1061.2;
	629	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3439.2;
	629	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11533.1;
	629	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6185.8;
	629	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8930.9;
	629	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1782.1;
	630	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	987.8;
	630	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	964.8;
	630	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1128.5;
	630	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3753.5;
	630	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12696.3;
	630	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6768.8;
	630	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9691.8;
	630	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1933.8;
	631	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1026.4;
	631	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1056.8;
	631	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1241.8;
	631	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4352.8;
	631	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14291.1;
	631	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7615.6;
	631	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10652.5;
	631	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2177.8;
	632	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1052;
	632	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1087.7;
	632	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1289.3;
	632	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4504.4;
	632	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14784.9;
	632	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7790.5;
	632	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10881.3;
	632	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2201.3;
	633	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1033;
	633	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1050.3;
	633	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1244.5;
	633	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4368.7;
	633	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13986;
	633	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7472.5;
	633	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10871.3;
	633	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2141.6;
	634	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1012.5;
	634	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	995.4;
	634	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1184.5;
	634	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4340.9;
	634	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13032.8;
	634	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7198.8;
	634	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10827.1;
	634	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2052.8;
	635	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989.7;
	635	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	940;
	635	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1128.5;
	635	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4309.6;
	635	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12238;
	635	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6843.7;
	635	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10638.7;
	635	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1993.9;
	636	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.2;
	636	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	886.5;
	636	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1067.6;
	636	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4232;
	636	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11544.5;
	636	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6496.1;
	636	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10419.6;
	636	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1872.7;
	637	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.4;
	637	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	840.8;
	637	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	995.5;
	637	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4118.1;
	637	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10955.8;
	637	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6154.4;
	637	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10150.1;
	637	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1753.4;
	638	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.1;
	638	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	812.2;
	638	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	951.4;
	638	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4017.3;
	638	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10518;
	638	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5888.8;
	638	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9969.8;
	638	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1679.3;
	639	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.5;
	639	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	789.2;
	639	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	915.1;
	639	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3926;
	639	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10137.8;
	639	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5680.4;
	639	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9783.2;
	639	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1656.2;
	640	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912;
	640	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	773.6;
	640	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	905.6;
	640	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3858.5;
	640	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9919.6;
	640	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5584.8;
	640	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9736.7;
	640	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1625.9;
	641	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.6;
	641	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	770.7;
	641	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	910.9;
	641	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3828.2;
	641	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10000.2;
	641	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5629;
	641	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9766.5;
	641	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1666.3;
	642	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.5;
	642	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	801;
	642	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	934.1;
	642	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3916.8;
	642	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10515.2;
	642	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5889.4;
	642	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10006;
	642	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1738;
	643	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.3;
	643	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	873.1;
	643	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1021.2;
	643	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4262.5;
	643	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11635.4;
	643	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6571.2;
	643	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10514.2;
	643	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1934.5;
	644	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	973.5;
	644	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	894.1;
	644	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1066.4;
	644	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4442.8;
	644	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12033.6;
	644	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6884.8;
	644	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10604;
	644	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1967.4;
	645	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989.5;
	645	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	904.3;
	645	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1091.5;
	645	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4518.7;
	645	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12187.2;
	645	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6987.7;
	645	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10543.2;
	645	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1989;
	646	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989.3;
	646	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	884.4;
	646	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1085;
	646	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4492.8;
	646	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11991.6;
	646	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6882.6;
	646	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10271.2;
	646	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1926.2;
	647	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970.5;
	647	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	840.1;
	647	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1050.3;
	647	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4352.8;
	647	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11391.9;
	647	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6540.1;
	647	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9821.5;
	647	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1843.5;
	648	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.4;
	648	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	804.9;
	648	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1016;
	648	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4123.4;
	648	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10792.6;
	648	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6233.3;
	648	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9390.7;
	648	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1765.7;
	649	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.6;
	649	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	819.9;
	649	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	964.2;
	649	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4074.5;
	649	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10641.5;
	649	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5696.1;
	649	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9298.9;
	649	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1780.9;
	650	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.8;
	650	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	817.2;
	650	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	968.1;
	650	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4050.2;
	650	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10597.8;
	650	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5672.4;
	650	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9262.3;
	650	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1797.4;
	651	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.7;
	651	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	824.4;
	651	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	980.4;
	651	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4080.2;
	651	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10693.7;
	651	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5717.6;
	651	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9304.9;
	651	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1812.4;
	652	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970;
	652	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	838.5;
	652	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	996.9;
	652	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4151.5;
	652	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10921.8;
	652	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5871.5;
	652	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9483.9;
	652	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1851.1;
	653	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981.3;
	653	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	868.6;
	653	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1032;
	653	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4306.9;
	653	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11427.8;
	653	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6174.3;
	653	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9872.2;
	653	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1929.4;
	654	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1010.1;
	654	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	911.4;
	654	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1102.9;
	654	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4670.3;
	654	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12486.3;
	654	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6821.5;
	654	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10713.5;
	654	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2081.4;
	655	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1054.3;
	655	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	997;
	655	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1216.3;
	655	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5347;
	655	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14021.7;
	655	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7753.4;
	655	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11707.4;
	655	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2324.8;
	656	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1076.3;
	656	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1023.2;
	656	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1258.1;
	656	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5441.5;
	656	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14385.4;
	656	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7923.6;
	656	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11875.1;
	656	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2365.9;
	657	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1042.9;
	657	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	981.5;
	657	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1199.9;
	657	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5004.4;
	657	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13466.8;
	657	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7371.5;
	657	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11466.3;
	657	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2224.2;
	658	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1016.9;
	658	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	930.7;
	658	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1129.2;
	658	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4546.8;
	658	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12553.7;
	658	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6778.4;
	658	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11035.6;
	658	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2045.6;
	659	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	995.7;
	659	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	886.3;
	659	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1066.3;
	659	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4099.9;
	659	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11758.2;
	659	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6298.9;
	659	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10640.9;
	659	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1883.7;
	660	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970.8;
	660	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	837.8;
	660	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1010.7;
	660	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3699.8;
	660	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11033.6;
	660	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5929.3;
	660	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10265.4;
	660	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1738.1;
	661	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.7;
	661	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	796.4;
	661	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	971;
	661	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3417.4;
	661	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10453.1;
	661	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5641.2;
	661	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9940.8;
	661	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1622.3;
	662	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.1;
	662	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	773;
	662	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	925.7;
	662	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3262.4;
	662	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10041.5;
	662	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5452.6;
	662	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9694;
	662	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1545;
	663	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.6;
	663	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749.4;
	663	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	905.3;
	663	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3163.5;
	663	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9712.2;
	663	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5314.1;
	663	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9487.5;
	663	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1485.5;
	664	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924;
	664	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.1;
	664	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	911.1;
	664	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3107.7;
	664	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9504.6;
	664	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5242.5;
	664	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9338;
	664	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1439.5;
	665	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.4;
	665	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.5;
	665	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	914.8;
	665	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3087.3;
	665	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9449.7;
	665	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5290.3;
	665	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9232.2;
	665	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1427.9;
	666	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.7;
	666	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.8;
	666	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	921.5;
	666	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3085;
	666	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9684.2;
	666	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5416.7;
	666	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9320.9;
	666	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1471.1;
	667	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.1;
	667	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	791.7;
	667	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	972.8;
	667	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3343.8;
	667	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10487.4;
	667	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5852.8;
	667	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9836.3;
	667	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1624.3;
	668	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.4;
	668	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	808;
	668	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	986.3;
	668	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3534.1;
	668	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10701.7;
	668	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6051.3;
	668	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9943.4;
	668	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1678.8;
	669	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.3;
	669	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	807.3;
	669	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	985.7;
	669	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3610;
	669	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10703.7;
	669	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6021.8;
	669	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9903.1;
	669	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1702;
	670	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.3;
	670	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	788.8;
	670	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	963.2;
	670	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3598.5;
	670	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10460.8;
	670	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5836.9;
	670	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9653.1;
	670	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1678.5;
	671	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.2;
	671	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	764.6;
	671	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	923.2;
	671	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3467.3;
	671	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9852.8;
	671	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5454.9;
	671	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9267.2;
	671	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1599.4;
	672	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.7;
	672	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	726.2;
	672	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	893.4;
	672	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3259.9;
	672	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9254;
	672	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5098.8;
	672	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8831;
	672	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1515.9;
	673	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.3;
	673	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.3;
	673	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849;
	673	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3109.5;
	673	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8948.8;
	673	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5036.8;
	673	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8502;
	673	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1570;
	674	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.7;
	674	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	722.8;
	674	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	856.2;
	674	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3100.1;
	674	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8869.1;
	674	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5009.7;
	674	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8421.3;
	674	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1560.7;
	675	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.1;
	675	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723.8;
	675	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	870.6;
	675	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3148.8;
	675	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8951;
	675	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5064.6;
	675	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8440.2;
	675	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1561.2;
	676	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.1;
	676	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	734.2;
	676	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	891.7;
	676	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3245.4;
	676	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9148.3;
	676	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5217.3;
	676	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8566.9;
	676	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1598.8;
	677	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.4;
	677	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	750.6;
	677	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	925.3;
	677	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3402.7;
	677	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9614.8;
	677	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5530.1;
	677	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8866.5;
	677	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1599.7;
	678	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.1;
	678	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	802;
	678	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	993.2;
	678	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3760.6;
	678	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10640;
	678	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6159.4;
	678	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9571.8;
	678	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1731.3;
	679	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	982.9;
	679	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	886.4;
	679	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1110.4;
	679	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4403.7;
	679	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12135.6;
	679	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7064.1;
	679	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10513.9;
	679	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1973.4;
	680	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1007.5;
	680	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	916.7;
	680	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1160.2;
	680	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4556.8;
	680	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12592.3;
	680	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7304.8;
	680	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10745.3;
	680	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2011.9;
	681	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985.8;
	681	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	884.9;
	681	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1104.7;
	681	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4232.2;
	681	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11890.4;
	681	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6871.1;
	681	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10468.9;
	681	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1901.6;
	682	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.5;
	682	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	848.1;
	682	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1032.6;
	682	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3916.2;
	682	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11190.3;
	682	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6357.4;
	682	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10152.1;
	682	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1754.4;
	683	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.1;
	683	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	814.2;
	683	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	965.5;
	683	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3589.1;
	683	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10608.6;
	683	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5940;
	683	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9881.1;
	683	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1624.6;
	684	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.2;
	684	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	783.3;
	684	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	911;
	684	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3333.6;
	684	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10148.3;
	684	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5630;
	684	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9721.3;
	684	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1528.7;
	685	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.6;
	685	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	763.4;
	685	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	871.4;
	685	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3197.1;
	685	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9850.8;
	685	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5435.2;
	685	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9651.7;
	685	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1478.1;
	686	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925;
	686	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.4;
	686	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.3;
	686	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3125.6;
	686	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9702.4;
	686	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5345.9;
	686	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9616.9;
	686	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1452;
	687	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916;
	687	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	739.5;
	687	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838.2;
	687	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3094;
	687	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9525.2;
	687	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5258.1;
	687	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9575.2;
	687	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1424.4;
	688	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.3;
	688	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.3;
	688	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	830.7;
	688	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3076.4;
	688	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9374.6;
	688	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5230.7;
	688	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9590.8;
	688	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1406.7;
	689	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.6;
	689	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	721.6;
	689	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	831.2;
	689	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3068.9;
	689	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9333.2;
	689	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5238.6;
	689	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9497.3;
	689	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1396.4;
	690	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.6;
	690	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720.2;
	690	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	828.3;
	690	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3020.2;
	690	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9382.7;
	690	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5290.8;
	690	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9423.8;
	690	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1390.2;
	691	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.8;
	691	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749.6;
	691	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.6;
	691	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3189.7;
	691	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9886.5;
	691	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5588.1;
	691	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9689.1;
	691	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1470.8;
	692	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.1;
	692	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749;
	692	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	875.3;
	692	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3251.1;
	692	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9810.1;
	692	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5610.2;
	692	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9595.2;
	692	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1469.2;
	693	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.5;
	693	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	739;
	693	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.5;
	693	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3218.8;
	693	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9594.2;
	693	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5520.6;
	693	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9370;
	693	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1440.3;
	694	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.2;
	694	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720.6;
	694	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	853.3;
	694	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3161.8;
	694	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9290.6;
	694	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5352.5;
	694	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9126.8;
	694	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1398.3;
	695	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.8;
	695	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	701.5;
	695	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	826.1;
	695	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3052;
	695	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8818.6;
	695	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5104.7;
	695	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8768.3;
	695	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1331;
	696	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.6;
	696	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	676.3;
	696	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	795;
	696	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2884.1;
	696	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8239;
	696	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4803.7;
	696	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8342.6;
	696	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1250.7;
	697	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	869.1;
	697	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	652.2;
	697	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	774;
	697	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2754.9;
	697	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7697.5;
	697	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4580.6;
	697	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8035.2;
	697	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1202;
	698	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.1;
	698	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	639.1;
	698	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	766.1;
	698	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2664;
	698	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7452.6;
	698	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4471.8;
	698	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7834.3;
	698	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1174;
	699	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867.4;
	699	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	641.6;
	699	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	766.3;
	699	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2632.1;
	699	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7368.5;
	699	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4444.3;
	699	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7710.9;
	699	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1164.1;
	700	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.9;
	700	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	642.3;
	700	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	778.7;
	700	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2630;
	700	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7393.6;
	700	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4487.9;
	700	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7669.2;
	700	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1169.3;
	701	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878;
	701	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	660.1;
	701	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	799.3;
	701	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2691.7;
	701	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7581.1;
	701	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4615;
	701	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7744.6;
	701	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1184.6;
	702	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893;
	702	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	684.4;
	702	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838.2;
	702	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2821.8;
	702	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8042;
	702	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4878.3;
	702	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7951.4;
	702	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1242.9;
	703	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.1;
	703	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	724.5;
	703	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	890.9;
	703	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3018.3;
	703	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8719.5;
	703	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5273.7;
	703	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8328.3;
	703	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1329.1;
	704	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.2;
	704	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	766;
	704	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	950.5;
	704	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3184.4;
	704	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9363.2;
	704	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5687.1;
	704	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8600.9;
	704	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1401.4;
	705	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.5;
	705	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	782.6;
	705	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	959.7;
	705	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3273;
	705	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9711.9;
	705	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5797.1;
	705	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8826.4;
	705	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1426.7;
	706	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.6;
	706	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772.3;
	706	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	926.7;
	706	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3246.5;
	706	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9719.9;
	706	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5664.1;
	706	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8925.1;
	706	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1413.4;
	707	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.5;
	707	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	754.4;
	707	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.1;
	707	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3164.1;
	707	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9564.4;
	707	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5445.2;
	707	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8997.2;
	707	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1384.3;
	708	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.8;
	708	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	731.4;
	708	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	845.2;
	708	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3090.1;
	708	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9321.1;
	708	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5243.8;
	708	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9006.4;
	708	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1348.2;
	709	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.9;
	709	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	706.1;
	709	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	821.8;
	709	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3060.1;
	709	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9043.1;
	709	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5107.5;
	709	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8968.2;
	709	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1316.7;
	710	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	876.2;
	710	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	689.8;
	710	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	808.3;
	710	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3056.8;
	710	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8825.4;
	710	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5000.2;
	710	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8946.6;
	710	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1296.9;
	711	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.7;
	711	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	681.7;
	711	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	803.9;
	711	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3058.6;
	711	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8718.1;
	711	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4966.2;
	711	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8942.6;
	711	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1286;
	712	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.4;
	712	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	677;
	712	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	802.5;
	712	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3072.7;
	712	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8675.5;
	712	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4958.8;
	712	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8965.8;
	712	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1277.9;
	713	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.1;
	713	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	681.4;
	713	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	815.2;
	713	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3072.1;
	713	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8695.5;
	713	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4993.5;
	713	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8933.4;
	713	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1295.8;
	714	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.2;
	714	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	687.2;
	714	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	828.9;
	714	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3042.3;
	714	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8806.4;
	714	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5063;
	714	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8918.5;
	714	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1313.9;
	715	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.6;
	715	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719.5;
	715	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.9;
	715	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3195.8;
	715	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9316.8;
	715	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5334.9;
	715	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9246.3;
	715	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1403.6;
	716	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.1;
	716	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720.6;
	716	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.5;
	716	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3245.4;
	716	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9325.3;
	716	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5396.2;
	716	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9148.4;
	716	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1409.8;
	717	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895;
	717	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	710.1;
	717	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	850.4;
	717	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3171.8;
	717	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9131;
	717	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5260.3;
	717	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9008.5;
	717	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1380.2;
	718	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884;
	718	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	691.8;
	718	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	826.1;
	718	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3074.2;
	718	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8846.5;
	718	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5091.7;
	718	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8791.7;
	718	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1333.9;
	719	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.6;
	719	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	670.5;
	719	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	790.5;
	719	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2939.7;
	719	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8381.8;
	719	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4836.5;
	719	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8458.8;
	719	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1269.4;
	720	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	854.5;
	720	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	638.3;
	720	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	744.1;
	720	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2760.8;
	720	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7821.3;
	720	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4515.6;
	720	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8064.8;
	720	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1188.5;
	721	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	843.5;
	721	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	608.8;
	721	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	722.8;
	721	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2588.5;
	721	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7362.6;
	721	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4255.5;
	721	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7707.5;
	721	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1103.3;
	722	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	837.7;
	722	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	596.3;
	722	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	711.6;
	722	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2464.6;
	722	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7042.9;
	722	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4072.3;
	722	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7462;
	722	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1058.3;
	723	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	837.2;
	723	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	593.3;
	723	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	707.9;
	723	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2395.6;
	723	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6872.8;
	723	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3979.6;
	723	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7302.9;
	723	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1027.9;
	724	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	838.2;
	724	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589.6;
	724	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	709.8;
	724	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2354.6;
	724	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6818.5;
	724	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3914.9;
	724	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7214.5;
	724	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1018.1;
	725	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	841.5;
	725	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	596.1;
	725	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	719.9;
	725	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2348.1;
	725	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6845.2;
	725	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3904.3;
	725	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7201;
	725	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1005.8;
	726	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	846.2;
	726	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	610.6;
	726	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	739.4;
	726	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2387;
	726	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7019.3;
	726	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3981.9;
	726	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7271.5;
	726	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1026.1;
	727	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	856.8;
	727	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	634.8;
	727	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	766.6;
	727	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2485;
	727	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7371.5;
	727	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4145.3;
	727	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7414;
	727	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1071.4;
	728	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	863.8;
	728	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	667.2;
	728	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	807.6;
	728	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2556.3;
	728	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7758.8;
	728	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4352.1;
	728	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7452.6;
	728	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1122.6;
	729	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.7;
	729	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	696.7;
	729	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	832.8;
	729	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2694.9;
	729	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8198;
	729	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4591.3;
	729	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7660.4;
	729	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1193.4;
	730	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.7;
	730	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	707.6;
	730	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	830.6;
	730	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2852.1;
	730	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8519.3;
	730	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4813.4;
	730	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8020.4;
	730	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1241.8;
	731	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	869.1;
	731	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	697.4;
	731	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	808.9;
	731	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2947.2;
	731	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8637.4;
	731	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4917.8;
	731	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8284.4;
	731	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1256.2;
	732	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867.1;
	732	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	686.6;
	732	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	792;
	732	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3018.7;
	732	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8665.3;
	732	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5018.3;
	732	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8470.5;
	732	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1268.7;
	733	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	869.4;
	733	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	680.7;
	733	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	789.2;
	733	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3116.6;
	733	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8704.8;
	733	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5117.2;
	733	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8667.5;
	733	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1299.3;
	734	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.1;
	734	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	674.6;
	734	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	793.7;
	734	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3239.4;
	734	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8710.6;
	734	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5219.7;
	734	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8888.5;
	734	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1321.3;
	735	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.6;
	735	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	670;
	735	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	795;
	735	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3359.9;
	735	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8657.1;
	735	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5333.4;
	735	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9013.4;
	735	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1335.5;
	736	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.3;
	736	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	667.8;
	736	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	810.4;
	736	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3451.3;
	736	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8603.8;
	736	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5453.8;
	736	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9080.7;
	736	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1351.1;
	737	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.8;
	737	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	672.4;
	737	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	797;
	737	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3510.4;
	737	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8640.8;
	737	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5580.4;
	737	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9116.2;
	737	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1377.3;
	738	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871.2;
	738	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	688.6;
	738	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	803;
	738	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3487.8;
	738	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8851.4;
	738	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5675.2;
	738	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9182.6;
	738	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1399.1;
	739	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.1;
	739	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	731.1;
	739	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	837;
	739	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3516.8;
	739	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9525;
	739	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5924.5;
	739	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9606.2;
	739	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1498.4;
	740	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.9;
	740	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	745;
	740	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.9;
	740	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3555;
	740	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9690.6;
	740	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5981.9;
	740	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9653.7;
	740	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1511.6;
	741	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.2;
	741	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.7;
	741	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	842.5;
	741	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3450;
	741	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9485.5;
	741	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5769.1;
	741	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9477.8;
	741	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1471.8;
	742	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.3;
	742	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	715.8;
	742	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	811;
	742	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3320.2;
	742	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9116.7;
	742	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5450.4;
	742	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9167.2;
	742	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1402.7;
	743	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867.9;
	743	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	677.9;
	743	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	765.4;
	743	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3076.1;
	743	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8431.3;
	743	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4962.2;
	743	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8657.1;
	743	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1293.4;
	744	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	846.3;
	744	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	654.9;
	744	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	721.7;
	744	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2780.3;
	744	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7752.9;
	744	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4427;
	744	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8159.4;
	744	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1180.3;
	745	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	828.5;
	745	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	633.7;
	745	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	718.9;
	745	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2518;
	745	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7346.2;
	745	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4108.6;
	745	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7695.9;
	745	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1180.9;
	746	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	825.8;
	746	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	632.1;
	746	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	713.7;
	746	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2388.2;
	746	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7174;
	746	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3925.8;
	746	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7463.9;
	746	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1150.2;
	747	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	831.8;
	747	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	639;
	747	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	724.8;
	747	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2327.9;
	747	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7189.8;
	747	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3852.9;
	747	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7376.9;
	747	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1114.2;
	748	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	837.7;
	748	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	652;
	748	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	738.1;
	748	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2303.5;
	748	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7349.7;
	748	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3882;
	748	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7362.5;
	748	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1166.1;
	749	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	851.9;
	749	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	674.4;
	749	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	768.4;
	749	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2347.2;
	749	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7834.7;
	749	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4050.8;
	749	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7561.4;
	749	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1208.4;
	750	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879;
	750	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.5;
	750	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	839.4;
	750	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2532.5;
	750	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8870.4;
	750	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4510.1;
	750	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8067;
	750	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1275.6;
	751	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.4;
	751	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	835.9;
	751	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	964.8;
	751	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2980.5;
	751	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10493.5;
	751	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5285.7;
	751	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8834.6;
	751	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1490.2;
	752	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951.5;
	752	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	876.8;
	752	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1019.9;
	752	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3121.9;
	752	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11120.4;
	752	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5590.2;
	752	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9043.7;
	752	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1579.6;
	753	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.5;
	753	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	857.6;
	753	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	983;
	753	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3056.7;
	753	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10784.3;
	753	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5510.2;
	753	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9119.8;
	753	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1532.4;
	754	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927;
	754	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	834.5;
	754	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	944;
	754	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3116.6;
	754	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10482.7;
	754	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5518.1;
	754	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9329.2;
	754	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1632.3;
	755	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.8;
	755	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	809.9;
	755	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	907.1;
	755	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3206.8;
	755	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10320.1;
	755	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5500.2;
	755	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9558.1;
	755	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1618.2;
	756	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.5;
	756	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	784.5;
	756	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	875.2;
	756	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3320.9;
	756	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10178.8;
	756	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5455.1;
	756	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9730;
	756	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1615.1;
	757	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.1;
	757	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	762.9;
	757	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.1;
	757	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3451.1;
	757	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10014.8;
	757	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5390.1;
	757	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9953.1;
	757	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1609.5;
	758	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.6;
	758	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.3;
	758	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.2;
	758	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3583.7;
	758	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9782.9;
	758	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5410.2;
	758	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10161.3;
	758	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1617.9;
	759	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.1;
	759	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735.4;
	759	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849.4;
	759	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3708.1;
	759	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9582.1;
	759	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5454.9;
	759	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10315.7;
	759	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1644.3;
	760	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.9;
	760	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.8;
	760	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851;
	760	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3787.4;
	760	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9482.8;
	760	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5562;
	760	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10432.4;
	760	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1620;
	761	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.1;
	761	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723.3;
	761	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.1;
	761	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3786;
	761	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9470;
	761	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5649;
	761	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10314.1;
	761	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1627.5;
	762	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.7;
	762	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735.5;
	762	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	846;
	762	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3662.2;
	762	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9669.2;
	762	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5722.4;
	762	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10216.5;
	762	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1605.7;
	763	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.4;
	763	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	781.5;
	763	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	887.5;
	763	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3716.3;
	763	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10404.4;
	763	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6068.3;
	763	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10527.9;
	763	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1714.2;
	764	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.7;
	764	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	781.3;
	764	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	906.4;
	764	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3758;
	764	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10529.1;
	764	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6152.8;
	764	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10501;
	764	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1732.2;
	765	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948.7;
	765	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	768.8;
	765	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	890.7;
	765	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3692.3;
	765	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10261.5;
	765	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6000;
	765	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10303.5;
	765	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1680.2;
	766	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949;
	766	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	739.1;
	766	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	855.1;
	766	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3530.8;
	766	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9765;
	766	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5619;
	766	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9947.7;
	766	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1520.8;
	767	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.2;
	767	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	694.9;
	767	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	808;
	767	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3244.7;
	767	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8930.4;
	767	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5151.7;
	767	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9315.1;
	767	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1415.2;
	768	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.5;
	768	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	656.8;
	768	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	764;
	768	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2893.9;
	768	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8057.6;
	768	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4645.1;
	768	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8625.9;
	768	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1347.9;
	769	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	876.4;
	769	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	631.6;
	769	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	777.3;
	769	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2619.4;
	769	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7469.2;
	769	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4278.2;
	769	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8053.6;
	769	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1279.7;
	770	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875;
	770	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	621.3;
	770	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	772;
	770	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2465.6;
	770	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7168;
	770	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4068.3;
	770	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7772.7;
	770	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1218.6;
	771	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.4;
	771	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	620.3;
	771	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	780.6;
	771	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2385.8;
	771	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7026;
	771	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3988.7;
	771	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7638.5;
	771	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1165.3;
	772	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.7;
	772	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	629;
	772	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	791.6;
	772	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2346.8;
	772	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7061;
	772	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3999.4;
	772	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7523.3;
	772	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1144.6;
	773	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897;
	773	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	648.6;
	773	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	819.8;
	773	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2368.6;
	773	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7380.9;
	773	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4127;
	773	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7643.2;
	773	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1190;
	774	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.2;
	774	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	695.1;
	774	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889.4;
	774	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2535.6;
	774	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8282.4;
	774	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4592.7;
	774	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8148.5;
	774	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1271.7;
	775	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969.6;
	775	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	784.6;
	775	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1006.5;
	775	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2982.1;
	775	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9787.7;
	775	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5368.8;
	775	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8951.5;
	775	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1464.8;
	776	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.4;
	776	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	822.3;
	776	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1058.4;
	776	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3116.9;
	776	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10407.6;
	776	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5586.5;
	776	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9152.4;
	776	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1519;
	777	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.3;
	777	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	806.7;
	777	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1027.5;
	777	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3035.7;
	777	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10153;
	777	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5451.2;
	777	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9104.3;
	777	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1526.4;
	778	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	968.6;
	778	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	797.2;
	778	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	994.8;
	778	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3077.7;
	778	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10036.4;
	778	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5405.8;
	778	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9217.9;
	778	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1546.5;
	779	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.3;
	779	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	789.2;
	779	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	965.8;
	779	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3106.1;
	779	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9974.7;
	779	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5376.9;
	779	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9352.5;
	779	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1561.1;
	780	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.3;
	780	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772.3;
	780	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	931.5;
	780	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3104.1;
	780	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9810.2;
	780	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5336;
	780	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9366.7;
	780	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1566.9;
	781	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.7;
	781	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755.7;
	781	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	901.4;
	781	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3159.2;
	781	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9651.3;
	781	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5301.5;
	781	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9397.4;
	781	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1564;
	782	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.7;
	782	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	748;
	782	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	880;
	782	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3211.5;
	782	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9557.4;
	782	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5313.5;
	782	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9480.8;
	782	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1536.3;
	783	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.8;
	783	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.1;
	783	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.1;
	783	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3268;
	783	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9435.7;
	783	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5313.3;
	783	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9528.5;
	783	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1537.4;
	784	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.2;
	784	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.9;
	784	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	858.7;
	784	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3325.6;
	784	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9295.2;
	784	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5370.4;
	784	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9549.8;
	784	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1536.9;
	785	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.5;
	785	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	731.7;
	785	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	864.4;
	785	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3348.5;
	785	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9279.2;
	785	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5459;
	785	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9520.5;
	785	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1540.4;
	786	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.9;
	786	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753;
	786	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	873.4;
	786	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3293;
	786	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9497.9;
	786	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5532.1;
	786	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9459.7;
	786	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1532.4;
	787	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.6;
	787	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	814;
	787	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	931.9;
	787	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3383.4;
	787	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10319.4;
	787	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5920.1;
	787	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9682.3;
	787	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1641.5;
	788	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.8;
	788	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	836.8;
	788	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	963.2;
	788	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3476.1;
	788	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10573.9;
	788	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6013.9;
	788	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9686.2;
	788	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1665.7;
	789	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.9;
	789	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	841.9;
	789	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	970.5;
	789	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3426.8;
	789	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10535.9;
	789	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5892.3;
	789	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9452.1;
	789	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1644.8;
	790	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	976.5;
	790	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	826.6;
	790	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	959.6;
	790	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3320.5;
	790	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10247.7;
	790	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5603.5;
	790	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9045.4;
	790	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1592.4;
	791	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.6;
	791	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	792.7;
	791	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	929.2;
	791	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3093.2;
	791	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9696;
	791	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5171.9;
	791	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8469.4;
	791	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1418.8;
	792	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.3;
	792	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	765.7;
	792	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	893.6;
	792	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2807.9;
	792	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9121.4;
	792	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4749.8;
	792	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7918.8;
	792	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1311.7;
	793	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.5;
	793	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744.8;
	793	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	879.3;
	793	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2561.3;
	793	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8694.8;
	793	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4834.7;
	793	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7409.8;
	793	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1313.2;
	794	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.6;
	794	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.9;
	794	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	879.9;
	794	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2459.4;
	794	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8655.5;
	794	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4710.3;
	794	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7227.6;
	794	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1328.7;
	795	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.8;
	795	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	754.7;
	795	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	899.6;
	795	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2433.9;
	795	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8813.8;
	795	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4691.9;
	795	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7201.3;
	795	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1322.7;
	796	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.4;
	796	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	774.7;
	796	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	922.5;
	796	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2452.3;
	796	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9099.1;
	796	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4743.1;
	796	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7265.1;
	796	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1349.6;
	797	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.9;
	797	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	809.1;
	797	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	957.9;
	797	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2564.4;
	797	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9731.5;
	797	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4964.4;
	797	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7561.5;
	797	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1414.8;
	798	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970.4;
	798	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	868.1;
	798	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1033.2;
	798	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2869.8;
	798	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10980.3;
	798	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5517.7;
	798	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8295.4;
	798	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1493.8;
	799	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1027;
	799	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	968.1;
	799	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1156;
	799	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3494.4;
	799	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12753.8;
	799	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6366.9;
	799	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9354.5;
	799	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1761.2;
	800	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1050.1;
	800	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1003.6;
	800	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1209.1;
	800	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3698.5;
	800	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13390.5;
	800	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6693.7;
	800	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9708.1;
	800	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1853.5;
	801	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1024.3;
	801	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	979.3;
	801	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1159.8;
	801	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3547.8;
	801	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12905.6;
	801	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6544;
	801	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9698.6;
	801	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1813.9;
	802	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009.1;
	802	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	956.4;
	802	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1107.4;
	802	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3459.1;
	802	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12495;
	802	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6401.2;
	802	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9661.3;
	802	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1836.9;
	803	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	987.7;
	803	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	925.1;
	803	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1071.5;
	803	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3354.5;
	803	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12098.3;
	803	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6202;
	803	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9573;
	803	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1817.2;
	804	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	968;
	804	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	894.8;
	804	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1021.3;
	804	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3257.9;
	804	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11627.5;
	804	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6016.2;
	804	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9443.5;
	804	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1729.8;
	805	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.3;
	805	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	860.5;
	805	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	976.5;
	805	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3160.5;
	805	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11149.3;
	805	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5803.9;
	805	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9290.1;
	805	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1680.2;
	806	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.8;
	806	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	836.4;
	806	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	935;
	806	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3122.6;
	806	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10853.5;
	806	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5636.9;
	806	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9230.6;
	806	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1654;
	807	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.1;
	807	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	813.7;
	807	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	902.7;
	807	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3084.6;
	807	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10496.8;
	807	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5508.8;
	807	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9056.7;
	807	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1615.1;
	808	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.8;
	808	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	802.3;
	808	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	895;
	808	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3074;
	808	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10292.5;
	808	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5460.7;
	808	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8966.4;
	808	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1573.8;
	809	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.7;
	809	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	803.1;
	809	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	898.2;
	809	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3078.5;
	809	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10381;
	809	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5539;
	809	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8908.5;
	809	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1617.4;
	810	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.2;
	810	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	830.7;
	810	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	916.9;
	810	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3086.6;
	810	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10938.1;
	810	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5808.5;
	810	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9002.9;
	810	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1690.9;
	811	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.7;
	811	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	904.4;
	811	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1006.5;
	811	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3351.8;
	811	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12131.5;
	811	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6376;
	811	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9577.6;
	811	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1886.9;
	812	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990.3;
	812	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	928.2;
	812	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1068.8;
	812	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3557.9;
	812	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12531.2;
	812	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6599.9;
	812	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9778.7;
	812	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1948.4;
	813	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1006.9;
	813	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	936;
	813	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1092.5;
	813	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3631.8;
	813	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12622.2;
	813	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6631.9;
	813	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9773;
	813	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1954.4;
	814	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1003.8;
	814	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	918.9;
	814	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1080;
	814	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3613.6;
	814	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12368.4;
	814	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6496.1;
	814	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9530.8;
	814	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1929.3;
	815	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981;
	815	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	878.4;
	815	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1045.4;
	815	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3468.1;
	815	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11703;
	815	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6156.3;
	815	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9058.9;
	815	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1837;
	816	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.8;
	816	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	852.8;
	816	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1011;
	816	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3244.1;
	816	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11110.9;
	816	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5816.7;
	816	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8587.5;
	816	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1717.5;
	817	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.7;
	817	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	846.5;
	817	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	966.3;
	817	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3126.5;
	817	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10869.5;
	817	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5621.1;
	817	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8356.8;
	817	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1725.2;
	818	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958;
	818	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	850.7;
	818	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	975.8;
	818	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3080.4;
	818	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10861.3;
	818	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5598.8;
	818	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8285.3;
	818	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1730.7;
	819	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.4;
	819	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	866.1;
	819	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	995.2;
	819	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3095.6;
	819	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11027.1;
	819	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5651;
	819	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8292;
	819	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1735;
	820	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970.8;
	820	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	879;
	820	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1021.8;
	820	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3160.4;
	820	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11334.1;
	820	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5797.1;
	820	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8408.5;
	820	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1790.9;
	821	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	979.2;
	821	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	914.6;
	821	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1053.2;
	821	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3306.6;
	821	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11897.7;
	821	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6108;
	821	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8776.8;
	821	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1844.4;
	822	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1008.9;
	822	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	973.3;
	822	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1128.5;
	822	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3654;
	822	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13007.8;
	822	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6747.3;
	822	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9590.5;
	822	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2012.6;
	823	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1055.3;
	823	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1061.4;
	823	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1251.9;
	823	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4331.8;
	823	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14627.4;
	823	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7710.2;
	823	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10592.4;
	823	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2243.6;
	824	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1070.8;
	824	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1088.4;
	824	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1292.8;
	824	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4498.3;
	824	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14998;
	824	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8000.3;
	824	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10767.8;
	824	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2293.3;
	825	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1041.3;
	825	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1039.2;
	825	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1230.7;
	825	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4214.3;
	825	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14055.4;
	825	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7561.7;
	825	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10546.2;
	825	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2148;
	826	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1017.5;
	826	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	979.8;
	826	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1156.2;
	826	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3965.5;
	826	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13062.5;
	826	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7021.5;
	826	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10297;
	826	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2088.4;
	827	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989;
	827	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	925.3;
	827	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1079.6;
	827	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3719;
	827	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12220.5;
	827	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6565.7;
	827	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10093.6;
	827	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1973.5;
	828	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.2;
	828	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	873.4;
	828	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1008.2;
	828	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3506.7;
	828	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11506.3;
	828	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6210.4;
	828	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9870.9;
	828	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1843;
	829	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.7;
	829	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	834;
	829	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	949.5;
	829	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3316;
	829	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10942.3;
	829	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5900.4;
	829	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9608.7;
	829	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1758.2;
	830	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.9;
	830	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	811.4;
	830	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	905.5;
	830	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3203.8;
	830	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10516;
	830	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5653.7;
	830	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9428.5;
	830	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1690.7;
	831	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.1;
	831	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	791.1;
	831	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	880.3;
	831	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3132;
	831	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10136.9;
	831	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5480.8;
	831	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9241.5;
	831	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1643.8;
	832	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.8;
	832	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	774.3;
	832	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.5;
	832	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3096.8;
	832	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9911.9;
	832	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5411;
	832	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9102.4;
	832	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1609.9;
	833	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.4;
	833	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776.4;
	833	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.1;
	833	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3089.9;
	833	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9918;
	833	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5439.7;
	833	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9024.5;
	833	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1624.7;
	834	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.9;
	834	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	794.3;
	834	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	898.6;
	834	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3125.8;
	834	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10308.9;
	834	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5684.6;
	834	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9169;
	834	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1692.6;
	835	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.5;
	835	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	866.9;
	835	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	979.2;
	835	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3459.5;
	835	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11425.5;
	835	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6298;
	835	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9766.4;
	835	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1899;
	836	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	973.7;
	836	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	897;
	836	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1025.1;
	836	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3752.7;
	836	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11918.8;
	836	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6583.1;
	836	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10045.7;
	836	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1986.8;
	837	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989.3;
	837	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	909.9;
	837	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1046.3;
	837	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3881;
	837	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12106.9;
	837	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6678.5;
	837	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10107;
	837	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2017.9;
	838	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989;
	838	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	903;
	838	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1041.9;
	838	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3909.8;
	838	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11927.8;
	838	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6574.3;
	838	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9900.4;
	838	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2004.8;
	839	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	971.5;
	839	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	870.8;
	839	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	999.9;
	839	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3793.5;
	839	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11378.4;
	839	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6235.5;
	839	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9456.4;
	839	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1919;
	840	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.5;
	840	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	843.8;
	840	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	968.9;
	840	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3608.1;
	840	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10820.1;
	840	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5913.5;
	840	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9054.6;
	840	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1836.9;
	841	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.5;
	841	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	829.6;
	841	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	974.9;
	841	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3494.5;
	841	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10460.3;
	841	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5857.7;
	841	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8828.3;
	841	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1818.6;
	842	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.2;
	842	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	829.8;
	842	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	986.4;
	842	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3483.8;
	842	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10398.7;
	842	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5871.4;
	842	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8791;
	842	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1821.6;
	843	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953;
	843	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	832.7;
	843	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1000.3;
	843	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3510.1;
	843	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10483.5;
	843	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5947.8;
	843	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8820.9;
	843	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1828.6;
	844	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.8;
	844	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	845.3;
	844	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1016.3;
	844	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3575.6;
	844	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10681.1;
	844	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6090.2;
	844	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8923.3;
	844	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1863.7;
	845	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.6;
	845	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	863.9;
	845	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1053.5;
	845	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3724.1;
	845	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11218.1;
	845	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6407.1;
	845	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9289.3;
	845	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1865.5;
	846	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990.6;
	846	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	918.7;
	846	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1124.9;
	846	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4065.2;
	846	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12294.4;
	846	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7070.1;
	846	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10074.3;
	846	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2002.9;
	847	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030.6;
	847	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1002.6;
	847	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1242.6;
	847	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4709.3;
	847	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13831.2;
	847	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8036.8;
	847	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11109.3;
	847	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2244.6;
	848	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1050.2;
	848	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1024;
	848	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1283.8;
	848	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4803;
	848	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14182.4;
	848	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8225.8;
	848	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11268.7;
	848	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2273;
	849	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1027.4;
	849	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	993.6;
	849	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1226.9;
	849	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4426.8;
	849	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13380;
	849	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7693.8;
	849	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10932.4;
	849	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2151.8;
	850	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1008.9;
	850	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	950.7;
	850	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1160.4;
	850	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4062.1;
	850	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12723.7;
	850	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7080.4;
	850	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10548;
	850	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1991.3;
	851	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	995.1;
	851	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	909.4;
	851	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1097.9;
	851	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3731;
	851	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12070;
	851	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6560.3;
	851	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10204.3;
	851	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1846.5;
	852	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969;
	852	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	870.6;
	852	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1032;
	852	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3482.9;
	852	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11522.1;
	852	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6152.8;
	852	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9859.8;
	852	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1722.6;
	853	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.6;
	853	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	827.6;
	853	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	974;
	853	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3299.9;
	853	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11010.6;
	853	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5829.4;
	853	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9534.1;
	853	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1616.1;
	854	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.9;
	854	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	803.9;
	854	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	934.3;
	854	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3201.7;
	854	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10515.3;
	854	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5576.5;
	854	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9438.3;
	854	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1548.9;
	855	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.7;
	855	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	781.7;
	855	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897.3;
	855	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3150.4;
	855	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10077.2;
	855	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5425.3;
	855	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9336.8;
	855	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1499.2;
	856	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.3;
	856	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	762.8;
	856	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	877;
	856	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3138.9;
	856	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9802.2;
	856	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5332;
	856	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9228;
	856	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1467.2;
	857	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.7;
	857	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	760.8;
	857	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	877.6;
	857	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3166.8;
	857	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9814.5;
	857	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5361.4;
	857	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9165.1;
	857	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1467.6;
	858	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.3;
	858	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	784.4;
	858	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	888.8;
	858	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3221.6;
	858	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10212.9;
	858	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5576.7;
	858	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9321.2;
	858	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1522.8;
	859	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.8;
	859	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	833;
	859	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	936.2;
	859	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3420.4;
	859	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11079.7;
	859	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6023.4;
	859	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9868.1;
	859	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1682.2;
	860	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	974.6;
	860	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	843.6;
	860	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	967.3;
	860	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3494.8;
	860	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11306;
	860	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6122.6;
	860	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10022;
	860	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1745.5;
	861	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.7;
	861	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	842.7;
	861	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	979.2;
	861	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3468.6;
	861	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11293.9;
	861	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6068.5;
	861	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9932;
	861	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1769.1;
	862	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	977.4;
	862	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	829.4;
	862	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	979.1;
	862	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3398.8;
	862	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11136.1;
	862	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5905.4;
	862	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9702.6;
	862	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1756.5;
	863	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969.6;
	863	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	799.4;
	863	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	958;
	863	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3286.4;
	863	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10650.7;
	863	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5632.1;
	863	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9308.8;
	863	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1693.7;
	864	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.6;
	864	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772.1;
	864	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	925.1;
	864	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3152.8;
	864	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10052.4;
	864	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5325.3;
	864	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8867.9;
	864	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1620.5;
	865	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951.3;
	865	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753.2;
	865	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	907.5;
	865	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3016.1;
	865	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9577.2;
	865	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5102.9;
	865	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8509.5;
	865	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1572.1;
	866	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.4;
	866	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.8;
	866	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	903.7;
	866	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2976.5;
	866	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9389.6;
	866	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4979.1;
	866	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8314.3;
	866	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1535.4;
	867	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.1;
	867	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.8;
	867	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	910.1;
	867	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2956.1;
	867	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9377.7;
	867	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4940.7;
	867	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8212.1;
	867	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1512.5;
	868	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.7;
	868	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	759.9;
	868	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	927.8;
	868	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2969.7;
	868	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9495;
	868	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4999.4;
	868	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8237.5;
	868	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1505.7;
	869	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	968.3;
	869	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772.5;
	869	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	950.5;
	869	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3038.2;
	869	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9768;
	869	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5202.2;
	869	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8379.7;
	869	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1526.2;
	870	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	982.7;
	870	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	798.7;
	870	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	990.6;
	870	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3200.4;
	870	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10328.9;
	870	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5522.4;
	870	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8643.3;
	870	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1581;
	871	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	997.3;
	871	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	833.8;
	871	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1045.3;
	871	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3426.7;
	871	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11045.5;
	871	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5981.5;
	871	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8959.4;
	871	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1664.2;
	872	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1010.5;
	872	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	874.6;
	872	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1099.3;
	872	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3619;
	872	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11696.7;
	872	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6376.4;
	872	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9194.9;
	872	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1735.1;
	873	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1006;
	873	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	902.7;
	873	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1101.9;
	873	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3676.5;
	873	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12077.5;
	873	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6483.1;
	873	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9356.6;
	873	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1757.4;
	874	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	987.2;
	874	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	900.2;
	874	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1065.2;
	874	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3591.1;
	874	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12288.5;
	874	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6369.7;
	874	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9383.6;
	874	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1748.7;
	875	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.2;
	875	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	872.3;
	875	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1019.4;
	875	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3441.7;
	875	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12266.3;
	875	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6167.2;
	875	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9384.4;
	875	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1703.7;
	876	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.8;
	876	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	834.7;
	876	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	967.4;
	876	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3289;
	876	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11885.8;
	876	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5922.8;
	876	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9249.3;
	876	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1645.4;
	877	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.1;
	877	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	786.7;
	877	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	921.4;
	877	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3150.5;
	877	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11219.3;
	877	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5659.5;
	877	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9039.5;
	877	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1569.7;
	878	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.4;
	878	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	750.9;
	878	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	880.8;
	878	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3048.6;
	878	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10593.5;
	878	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5406.3;
	878	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8800.5;
	878	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1508.1;
	879	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.7;
	879	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729;
	879	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851.1;
	879	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2975.6;
	879	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10168.5;
	879	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5223.4;
	879	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8590.2;
	879	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1463.9;
	880	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.4;
	880	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	713.3;
	880	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	833.8;
	880	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2936.2;
	880	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9904.4;
	880	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5126.3;
	880	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8490.1;
	880	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1434;
	881	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.2;
	881	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711.6;
	881	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851.9;
	881	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2910;
	881	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9715.2;
	881	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5126.2;
	881	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8437.5;
	881	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1437.6;
	882	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.2;
	882	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.6;
	882	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.3;
	882	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2922.8;
	882	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9941.9;
	882	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5256.3;
	882	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8590.4;
	882	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1499.4;
	883	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.8;
	883	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	780.8;
	883	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	934.1;
	883	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3131.3;
	883	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10794.4;
	883	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5735.4;
	883	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9179.7;
	883	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1651.4;
	884	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.1;
	884	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	804.1;
	884	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	960.2;
	884	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3283.9;
	884	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11154.1;
	884	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6019.4;
	884	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9415.2;
	884	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1710.1;
	885	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.7;
	885	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	817.5;
	885	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	978;
	885	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3320;
	885	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11239.6;
	885	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6112.6;
	885	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9444.2;
	885	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1732.3;
	886	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.7;
	886	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	816.8;
	886	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	983.3;
	886	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3364.2;
	886	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11216.3;
	886	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6113.9;
	886	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9405.9;
	886	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1737.6;
	887	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929;
	887	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	804;
	887	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	976.6;
	887	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3384;
	887	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10939.3;
	887	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6024.9;
	887	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9222.4;
	887	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1699.9;
	888	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.1;
	888	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	784.8;
	888	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	953.3;
	888	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3347.4;
	888	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10590.3;
	888	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5867.1;
	888	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8999.5;
	888	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1656.2;
	889	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.9;
	889	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	774;
	889	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	942.5;
	889	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3325.3;
	889	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10348.6;
	889	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5776.7;
	889	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8816.1;
	889	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1631.5;
	890	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.7;
	890	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	774.8;
	890	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	950;
	890	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3337.7;
	890	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10248.9;
	890	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5756.2;
	890	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8738.4;
	890	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1632.3;
	891	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.1;
	891	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	781.7;
	891	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	961.2;
	891	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3385.5;
	891	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10269;
	891	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5829.9;
	891	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8740.5;
	891	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1639.4;
	892	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.8;
	892	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	787.6;
	892	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	978.6;
	892	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3452;
	892	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10370.1;
	892	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5958.8;
	892	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8789.5;
	892	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1668.6;
	893	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930;
	893	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	802.7;
	893	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	999.5;
	893	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3553.3;
	893	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10576.5;
	893	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6158.6;
	893	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8946.9;
	893	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1710;
	894	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.2;
	894	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	830.1;
	894	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1034.8;
	894	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3713.8;
	894	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10994.3;
	894	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6465.8;
	894	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9210.7;
	894	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1772.3;
	895	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951.2;
	895	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	863.1;
	895	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1079.6;
	895	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3948;
	895	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11560;
	895	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6873;
	895	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9532.6;
	895	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1870.3;
	896	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.8;
	896	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	900.2;
	896	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1129.8;
	896	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4140.9;
	896	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12073.7;
	896	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7209.2;
	896	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9726.8;
	896	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1955.8;
	897	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.5;
	897	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	913.6;
	897	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1126.9;
	897	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4166.2;
	897	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12114.1;
	897	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7175.4;
	897	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9757.3;
	897	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1944.8;
	898	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.7;
	898	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	882.2;
	898	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1065.2;
	898	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3913.6;
	898	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11641.1;
	898	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6746.5;
	898	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9523.9;
	898	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1809.2;
	899	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.6;
	899	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	830.2;
	899	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	982.7;
	899	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3558;
	899	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10974.3;
	899	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6264.2;
	899	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9215.2;
	899	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1651.2;
	900	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.5;
	900	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	786.7;
	900	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	915.2;
	900	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3273.5;
	900	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10322.6;
	900	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5865.5;
	900	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8912.8;
	900	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1523.1;
	901	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.3;
	901	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	758.8;
	901	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	871;
	901	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3107.1;
	901	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9818.4;
	901	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5557.1;
	901	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8705.7;
	901	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1453.3;
	902	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.3;
	902	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.7;
	902	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	837.4;
	902	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3014.8;
	902	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9388.5;
	902	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5330.3;
	902	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8561.2;
	902	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1388.1;
	903	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867.5;
	903	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714.5;
	903	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	812.5;
	903	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2974.6;
	903	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9061.8;
	903	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5185.9;
	903	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8468.8;
	903	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1343.1;
	904	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	861.6;
	904	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	702.7;
	904	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	799;
	904	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2958.7;
	904	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8873.1;
	904	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5121.4;
	904	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8419.4;
	904	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1326.9;
	905	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	862.5;
	905	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	701.8;
	905	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	799.7;
	905	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2949.9;
	905	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8868.1;
	905	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5116.9;
	905	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8420;
	905	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1344.4;
	906	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	854.1;
	906	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	709;
	906	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	799.6;
	906	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2902.9;
	906	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9023;
	906	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5153.7;
	906	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8433.9;
	906	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1373.2;
	907	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	862.1;
	907	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.5;
	907	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	834.2;
	907	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3027.3;
	907	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9560.3;
	907	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5431.1;
	907	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8810.6;
	907	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1478.1;
	908	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.9;
	908	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	763.8;
	908	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.9;
	908	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3134;
	908	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9783.5;
	908	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5601.4;
	908	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8936.2;
	908	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1527.3;
	909	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.5;
	909	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	770.6;
	909	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	875.2;
	909	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3156;
	909	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9845.5;
	909	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5636.9;
	909	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8932.6;
	909	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1549.7;
	910	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.3;
	910	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	767;
	910	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	874;
	910	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3172.7;
	910	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9847.8;
	910	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5612.8;
	910	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8878;
	910	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1562.2;
	911	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.2;
	911	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.9;
	911	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	867.9;
	911	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3182.8;
	911	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9667.4;
	911	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5505.8;
	911	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8715.7;
	911	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1552.6;
	912	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.8;
	912	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720.4;
	912	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	827.3;
	912	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2946;
	912	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9076.2;
	912	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5111.9;
	912	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8300.9;
	912	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1472.1;
	913	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.3;
	913	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	687.5;
	913	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.7;
	913	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2777.9;
	913	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8650;
	913	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4882.7;
	913	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8050.1;
	913	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1510.7;
	914	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.9;
	914	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	690.9;
	914	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	832.1;
	914	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2704.8;
	914	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8579.5;
	914	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4803.5;
	914	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7988.2;
	914	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1505.8;
	915	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.1;
	915	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	699.5;
	915	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	839.4;
	915	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2691.7;
	915	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8683.7;
	915	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4803;
	915	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8025.7;
	915	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1472.9;
	916	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.9;
	916	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	713;
	916	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.8;
	916	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2721.9;
	916	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8944.2;
	916	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4919;
	916	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8140.5;
	916	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1539.6;
	917	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918;
	917	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.4;
	917	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884;
	917	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2843.6;
	917	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9537.1;
	917	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5159.9;
	917	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8442.7;
	917	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1601.7;
	918	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.6;
	918	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	802.9;
	918	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	956.5;
	918	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3129.3;
	918	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10689.9;
	918	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5775.5;
	918	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9165.1;
	918	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1704.1;
	919	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1001.6;
	919	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	895.8;
	919	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1076.8;
	919	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3635.8;
	919	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12304.7;
	919	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6670.7;
	919	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10125.7;
	919	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1943.2;
	920	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1029.7;
	920	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	932.4;
	920	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1131.9;
	920	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3808.2;
	920	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12874.9;
	920	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6909.2;
	920	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10340.1;
	920	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2009.7;
	921	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1015.5;
	921	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	911.9;
	921	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1094.5;
	921	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3686.2;
	921	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12384.5;
	921	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6635.3;
	921	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10217.4;
	921	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1895.9;
	922	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.5;
	922	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	893;
	922	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1056.4;
	922	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3580.1;
	922	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12009.1;
	922	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6398.4;
	922	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10122.1;
	922	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1936.7;
	923	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	983.4;
	923	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	878.9;
	923	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1030.6;
	923	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3499.3;
	923	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11748.3;
	923	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6212;
	923	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10062.9;
	923	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1875.1;
	924	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.5;
	924	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	856.7;
	924	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	990.8;
	924	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3390.7;
	924	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11478.6;
	924	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6006.5;
	924	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9899.2;
	924	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1820.8;
	925	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.5;
	925	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	834.3;
	925	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	952.9;
	925	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3282.7;
	925	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11162.2;
	925	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5813.9;
	925	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9732.8;
	925	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1762.2;
	926	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.2;
	926	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	817.5;
	926	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	923.9;
	926	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3202.1;
	926	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10871.3;
	926	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5620.2;
	926	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9644.8;
	926	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1713.2;
	927	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914;
	927	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	791.7;
	927	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	893.7;
	927	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3146.9;
	927	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10568.5;
	927	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5477.7;
	927	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9493.2;
	927	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1691.3;
	928	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.4;
	928	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776.4;
	928	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	875.9;
	928	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3114.9;
	928	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10393;
	928	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5420.6;
	928	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9378.1;
	928	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1649.5;
	929	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.5;
	929	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	780.4;
	929	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878;
	929	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3112.2;
	929	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10471.8;
	929	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5446;
	929	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9320.8;
	929	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1684.4;
	930	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.3;
	930	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	808.6;
	930	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	886.7;
	930	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3171.1;
	930	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10959;
	930	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5660.3;
	930	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9526.3;
	930	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1748.4;
	931	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.6;
	931	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	869.2;
	931	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	961.3;
	931	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3465.7;
	931	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12028.1;
	931	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6223.3;
	931	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10131.2;
	931	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1941.1;
	932	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970;
	932	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	885.8;
	932	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1018;
	932	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3677.2;
	932	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12375.8;
	932	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6511;
	932	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10350.6;
	932	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2020.8;
	933	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	977.6;
	933	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	882.8;
	933	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1028.8;
	933	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3747.1;
	933	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12280.6;
	933	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6517.9;
	933	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10271.4;
	933	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2004.7;
	934	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	978.9;
	934	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	861.3;
	934	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1020.1;
	934	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3712.7;
	934	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11926;
	934	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6366.7;
	934	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9952.1;
	934	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1874.2;
	935	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.9;
	935	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	820.3;
	935	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	986.5;
	935	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3532;
	935	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11200.5;
	935	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5993.3;
	935	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9415;
	935	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1773.6;
	936	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.8;
	936	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	790;
	936	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	948.3;
	936	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3298.3;
	936	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10530;
	936	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5655;
	936	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8944.9;
	936	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1737.1;
	937	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919;
	937	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777.3;
	937	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	939.3;
	937	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3166;
	937	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10211.5;
	937	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5480.4;
	937	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8680.1;
	937	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1735;
	938	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.3;
	938	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	773.8;
	938	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	942.9;
	938	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3127.8;
	938	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10161.2;
	938	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5457.1;
	938	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8594.1;
	938	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1724.8;
	939	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.6;
	939	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	784.8;
	939	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	953.2;
	939	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3149.1;
	939	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10257.3;
	939	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5505.5;
	939	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8636.9;
	939	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1717.5;
	940	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.4;
	940	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	789.4;
	940	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	969.7;
	940	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3203.7;
	940	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10483.2;
	940	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5646.5;
	940	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8751.8;
	940	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1737.6;
	941	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.8;
	941	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	817.9;
	941	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1002;
	941	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3334.8;
	941	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11034.9;
	941	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5926.5;
	941	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9110.3;
	941	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1841.9;
	942	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	977.7;
	942	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	878.9;
	942	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1078.6;
	942	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3667.3;
	942	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12152.3;
	942	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6570.3;
	942	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9923.7;
	942	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2009.2;
	943	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1021.8;
	943	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	964.4;
	943	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1196.8;
	943	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4314.7;
	943	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13755.6;
	943	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7458.3;
	943	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10867.9;
	943	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2256;
	944	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1037.8;
	944	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	985.3;
	944	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1224.9;
	944	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4460.9;
	944	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14067.3;
	944	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7597.8;
	944	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10995.4;
	944	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2267.3;
	945	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1010.3;
	945	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	944.8;
	945	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1151.9;
	945	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4155.5;
	945	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13128.6;
	945	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7057.5;
	945	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10703.1;
	945	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2154;
	946	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	987;
	946	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	902.5;
	946	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1073.9;
	946	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3852.4;
	946	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12365.7;
	946	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6527.5;
	946	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10487.2;
	946	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2032.2;
	947	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.2;
	947	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	860.2;
	947	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1007.1;
	947	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3558.6;
	947	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11726.9;
	947	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6099.9;
	947	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10268.6;
	947	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1919.8;
	948	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.3;
	948	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	823.6;
	948	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	953.8;
	948	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3332.4;
	948	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11126;
	948	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5746.9;
	948	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10003.2;
	948	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1824.4;
	949	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.9;
	949	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	789.1;
	949	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	911.4;
	949	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3227.7;
	949	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10621.1;
	949	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5507;
	949	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9704;
	949	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1739.9;
	950	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.1;
	950	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	765.8;
	950	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	883.1;
	950	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3174.3;
	950	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10227.5;
	950	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5364.9;
	950	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9517.9;
	950	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1655.3;
	951	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.3;
	951	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.9;
	951	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865;
	951	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3145.8;
	951	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9859.8;
	951	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5263.4;
	951	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9316;
	951	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1609.1;
	952	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.9;
	952	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729;
	952	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857;
	952	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3158.9;
	952	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9616.9;
	952	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5229.8;
	952	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9195.8;
	952	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1590.1;
	953	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.9;
	953	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729;
	953	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.9;
	953	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3179.3;
	953	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9621.9;
	953	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5257.1;
	953	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9118;
	953	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1594.7;
	954	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.1;
	954	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749.1;
	954	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.8;
	954	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3158.9;
	954	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9928.9;
	954	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5411.1;
	954	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9184.5;
	954	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1622.2;
	955	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.9;
	955	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	809.4;
	955	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	909.6;
	955	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3313.3;
	955	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10809.3;
	955	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5790.3;
	955	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9643.1;
	955	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1780;
	956	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.2;
	956	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	832.5;
	956	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	947.4;
	956	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3485.1;
	956	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11141.1;
	956	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6016.5;
	956	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9821.6;
	956	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1838.7;
	957	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.7;
	957	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	839.6;
	957	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	951.6;
	957	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3502.8;
	957	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11123.3;
	957	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5932;
	957	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9749.9;
	957	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1839.3;
	958	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.5;
	958	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	824.5;
	958	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	932.6;
	958	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3446.7;
	958	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10831.7;
	958	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5740.4;
	958	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9444.1;
	958	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1799.8;
	959	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.3;
	959	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	784.9;
	959	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	905.9;
	959	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3273;
	959	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10187.2;
	959	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5360.8;
	959	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8931.7;
	959	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1636.2;
	960	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.4;
	960	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.7;
	960	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	864.9;
	960	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2995.9;
	960	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9556.1;
	960	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4976.5;
	960	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8484.6;
	960	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1537.4;
	961	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.7;
	961	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	739.4;
	961	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	844;
	961	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2850.3;
	961	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9269.4;
	961	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4760.9;
	961	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8214.3;
	961	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1550.6;
	962	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.2;
	962	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.2;
	962	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	845.9;
	962	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2810.1;
	962	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9259.8;
	962	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4716.1;
	962	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8132;
	962	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1589.8;
	963	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896;
	963	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	757.5;
	963	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	870.6;
	963	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2833.6;
	963	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9446.2;
	963	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4771.8;
	963	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8137.1;
	963	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1604.3;
	964	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.8;
	964	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777;
	964	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	896.4;
	964	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2911.4;
	964	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9747.2;
	964	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4908.1;
	964	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8239;
	964	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1650.7;
	965	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.1;
	965	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	800.7;
	965	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	932.5;
	965	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3075.1;
	965	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10370.4;
	965	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5191.2;
	965	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8598.6;
	965	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1726.1;
	966	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.9;
	966	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	864;
	966	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1007.2;
	966	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3439.1;
	966	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11597.4;
	966	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5855.9;
	966	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9406.5;
	966	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1820.8;
	967	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.9;
	967	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	957.6;
	967	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1125.3;
	967	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4092.3;
	967	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13227.7;
	967	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6804;
	967	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10382.5;
	967	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2074.6;
	968	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1011.2;
	968	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	979.3;
	968	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1167.3;
	968	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4231.7;
	968	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13598.4;
	968	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7020.5;
	968	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10529.6;
	968	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2116.6;
	969	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	982.6;
	969	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	935;
	969	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1096.4;
	969	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3894.8;
	969	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12680.5;
	969	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6581.4;
	969	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10257.3;
	969	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1987.8;
	970	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.8;
	970	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	889.6;
	970	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1012.3;
	970	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3622.2;
	970	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11871.6;
	970	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6131.8;
	970	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9977.3;
	970	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1913.5;
	971	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.4;
	971	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	860.9;
	971	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	961;
	971	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3390.1;
	971	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11215.7;
	971	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5813.4;
	971	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9755.7;
	971	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1827.7;
	972	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.5;
	972	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	826.9;
	972	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	900.9;
	972	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3241.5;
	972	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10643.4;
	972	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5552.9;
	972	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9497.4;
	972	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1703.6;
	973	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906;
	973	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	791;
	973	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.1;
	973	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3180.8;
	973	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10175.9;
	973	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5397.4;
	973	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9357.7;
	973	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1640.1;
	974	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.5;
	974	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	762.8;
	974	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	841.4;
	974	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3176.6;
	974	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9887.7;
	974	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5315.8;
	974	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9260.8;
	974	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1602.1;
	975	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.4;
	975	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744.3;
	975	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	837.3;
	975	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3213.3;
	975	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9589.4;
	975	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5283.8;
	975	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9215.2;
	975	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1568.3;
	976	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.4;
	976	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	732.1;
	976	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.4;
	976	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3249.9;
	976	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9428.7;
	976	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5307.8;
	976	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9160.3;
	976	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1518.8;
	977	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.6;
	977	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	721;
	977	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	845.9;
	977	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3269;
	977	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9409.6;
	977	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5354.9;
	977	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9110.8;
	977	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1536.9;
	978	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.5;
	978	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719.6;
	978	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	841;
	978	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3229.5;
	978	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9535.5;
	978	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5420.9;
	978	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9129.3;
	978	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1559.9;
	979	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890;
	979	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	757.9;
	979	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	872.4;
	979	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3318.2;
	979	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10151.6;
	979	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5698.9;
	979	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9456;
	979	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1678.5;
	980	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.6;
	980	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777.1;
	980	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	906.5;
	980	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3451.8;
	980	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10376.4;
	980	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5814.7;
	980	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9590;
	980	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1718.5;
	981	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.9;
	981	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	774.5;
	981	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	904.3;
	981	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3454.5;
	981	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10279;
	981	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5789.3;
	981	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9497;
	981	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1704.3;
	982	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.7;
	982	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.3;
	982	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.1;
	982	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3386.2;
	982	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9934.5;
	982	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5558.8;
	982	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9212.2;
	982	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1664.2;
	983	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893;
	983	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	709.8;
	983	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	832.9;
	983	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3174.5;
	983	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9200.7;
	983	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5103.6;
	983	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8674.2;
	983	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1560.8;
	984	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	869.9;
	984	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	669.6;
	984	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	788.7;
	984	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2886.6;
	984	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8453.5;
	984	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4682.7;
	984	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8147;
	984	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1423.9;
	985	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	862.6;
	985	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	641.9;
	985	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	753;
	985	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2689.7;
	985	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7948.2;
	985	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4534.5;
	985	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7818.3;
	985	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1397.3;
	986	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	862;
	986	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	634.3;
	986	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	745.3;
	986	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2608.4;
	986	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7760.9;
	986	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4413.4;
	986	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7690.6;
	986	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1370.7;
	987	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	865.5;
	987	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	634.5;
	987	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	745.7;
	987	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2593.4;
	987	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7735;
	987	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4398.8;
	987	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7633.7;
	987	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1344.3;
	988	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.9;
	988	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	643.2;
	988	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	752.5;
	988	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2626.3;
	988	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7863;
	988	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4484;
	988	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7668.8;
	988	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1375.6;
	989	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.3;
	989	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	669.9;
	989	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	772.8;
	989	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2720.4;
	989	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8255.6;
	989	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4703.2;
	989	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7875;
	989	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1409.1;
	990	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.7;
	990	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	722.5;
	990	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	822.9;
	990	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2987.2;
	990	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9148.4;
	990	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5255.3;
	990	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8483.3;
	990	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1544.1;
	991	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.9;
	991	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	806.3;
	991	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	916.4;
	991	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3546.7;
	991	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10620.1;
	991	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6126.1;
	991	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9365.8;
	991	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1754.8;
	992	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.9;
	992	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	834.1;
	992	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	958.3;
	992	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3657.8;
	992	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11074.1;
	992	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6331.9;
	992	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9465.2;
	992	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1792.7;
	993	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.7;
	993	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	808.2;
	993	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	930.9;
	993	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3451.7;
	993	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10600.6;
	993	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6039.5;
	993	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9343.7;
	993	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1677;
	994	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.7;
	994	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	783.7;
	994	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	911.5;
	994	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3318.8;
	994	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10186.2;
	994	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5786.4;
	994	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9302.1;
	994	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1679.9;
	995	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.7;
	995	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	770.9;
	995	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	896.9;
	995	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3228.3;
	995	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9909.8;
	995	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5570.8;
	995	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9338.5;
	995	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1626;
	996	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.1;
	996	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753;
	996	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884;
	996	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3204.8;
	996	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9727.4;
	996	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5455.4;
	996	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9352.4;
	996	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1565;
	997	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.3;
	997	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.9;
	997	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	862.4;
	997	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3211.9;
	997	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9633.3;
	997	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5429.1;
	997	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9381.6;
	997	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1548.5;
	998	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893;
	998	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735;
	998	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.7;
	998	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3279.5;
	998	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9609.4;
	998	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5442;
	998	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9524;
	998	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1548.7;
	999	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.2;
	999	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	731.8;
	999	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	862.4;
	999	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3340.4;
	999	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9562.4;
	999	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5481.7;
	999	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9570.8;
	999	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1560.5;
	1000	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899;
	1000	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	721.4;
	1000	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	862.5;
	1000	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3397.7;
	1000	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9555.8;
	1000	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5537.2;
	1000	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9588.3;
	1000	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1557.2;
	1001	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.1;
	1001	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	724.1;
	1001	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	871;
	1001	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3432.9;
	1001	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9614.2;
	1001	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5532.5;
	1001	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9574.3;
	1001	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1565.5;
	1002	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.1;
	1002	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723.7;
	1002	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.2;
	1002	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3359.7;
	1002	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9632.9;
	1002	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5637.2;
	1002	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9432.4;
	1002	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1550.7;
	1003	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.9;
	1003	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755.8;
	1003	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	888.9;
	1003	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3420.4;
	1003	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10047.2;
	1003	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5914.4;
	1003	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9603.2;
	1003	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1632;
	1004	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.6;
	1004	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	767;
	1004	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	907.2;
	1004	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3525;
	1004	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10184.5;
	1004	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5991.3;
	1004	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9658.2;
	1004	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1650.7;
	1005	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.5;
	1005	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753.1;
	1005	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	890.9;
	1005	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3470.8;
	1005	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9966.6;
	1005	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5928.1;
	1005	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9442.2;
	1005	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1618.3;
	1006	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.3;
	1006	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.2;
	1006	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	862.2;
	1006	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3356.1;
	1006	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9510.4;
	1006	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5676;
	1006	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9064;
	1006	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1560;
	1007	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.4;
	1007	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	692.9;
	1007	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	816.1;
	1007	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3133.4;
	1007	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8803.2;
	1007	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5236.9;
	1007	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8472.7;
	1007	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1441.8;
	1008	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	861;
	1008	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	664.1;
	1008	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	774.8;
	1008	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2822.3;
	1008	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8045.1;
	1008	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4778.1;
	1008	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7848;
	1008	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1316.4;
	1009	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866.6;
	1009	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	649.9;
	1009	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	755.4;
	1009	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2616.4;
	1009	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7602.7;
	1009	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4298.7;
	1009	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7490.1;
	1009	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1273.8;
	1010	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	865.3;
	1010	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	648.4;
	1010	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	756.4;
	1010	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2494.2;
	1010	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7385.5;
	1010	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4133.5;
	1010	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7304.4;
	1010	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1239.1;
	1011	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	869.8;
	1011	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	651.9;
	1011	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	769.1;
	1011	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2438.9;
	1011	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7365.3;
	1011	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4073.6;
	1011	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7214.1;
	1011	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1216.8;
	1012	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.2;
	1012	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	665.9;
	1012	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	793.9;
	1012	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2429;
	1012	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7502.4;
	1012	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4090.6;
	1012	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7216.7;
	1012	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1234.9;
	1013	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.7;
	1013	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	690.1;
	1013	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.7;
	1013	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2483.7;
	1013	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7929.6;
	1013	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4269.3;
	1013	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7417.3;
	1013	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1214.2;
	1014	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920.7;
	1014	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	748.5;
	1014	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	909;
	1014	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2694.6;
	1014	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8884.3;
	1014	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4751.3;
	1014	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8005.4;
	1014	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1329.4;
	1015	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.1;
	1015	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	840.6;
	1015	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1020.2;
	1015	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3152.4;
	1015	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10382.7;
	1015	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5598.8;
	1015	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8825;
	1015	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1548.3;
	1016	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	987.3;
	1016	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	876.6;
	1016	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1071.4;
	1016	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3268;
	1016	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10972.5;
	1016	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5858.2;
	1016	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9039.3;
	1016	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1601.1;
	1017	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	968.9;
	1017	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	851.3;
	1017	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1029.3;
	1017	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3172.5;
	1017	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10628.3;
	1017	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5725.8;
	1017	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9098;
	1017	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1560.3;
	1018	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.1;
	1018	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	818.7;
	1018	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	985.4;
	1018	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3181.1;
	1018	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10360.1;
	1018	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5591.6;
	1018	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9242.5;
	1018	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1530.9;
	1019	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.1;
	1019	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	790.6;
	1019	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	933.5;
	1019	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3213.6;
	1019	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10097.1;
	1019	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5508;
	1019	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9388.9;
	1019	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1497;
	1020	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.4;
	1020	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	758.6;
	1020	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889.3;
	1020	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3265;
	1020	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9831.7;
	1020	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5399.9;
	1020	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9483;
	1020	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1463.5;
	1021	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.4;
	1021	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	731.8;
	1021	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	856.8;
	1021	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3329.2;
	1021	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9674.5;
	1021	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5397.3;
	1021	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9571.9;
	1021	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1446.9;
	1022	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.6;
	1022	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	724.9;
	1022	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851.9;
	1022	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3413.9;
	1022	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9586.6;
	1022	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5475.1;
	1022	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9776.8;
	1022	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1442.1;
	1023	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904;
	1023	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.7;
	1023	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849.7;
	1023	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3528.5;
	1023	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9467.1;
	1023	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5561.1;
	1023	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9995.5;
	1023	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1439.8;
	1024	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.4;
	1024	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	702.3;
	1024	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849;
	1024	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3602.7;
	1024	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9344.7;
	1024	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5659.7;
	1024	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10128.7;
	1024	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1429.8;
	1025	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.4;
	1025	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	694.9;
	1025	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	853.3;
	1025	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3631.1;
	1025	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9304;
	1025	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5734.3;
	1025	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10121.5;
	1025	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1431.8;
	1026	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.3;
	1026	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	695.2;
	1026	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.2;
	1026	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3520.7;
	1026	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9348.2;
	1026	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5698.8;
	1026	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9963.5;
	1026	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1413.3;
	1027	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.6;
	1027	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.2;
	1027	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.2;
	1027	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3477.6;
	1027	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9719.6;
	1027	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5791.2;
	1027	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9943.1;
	1027	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1469.7;
	1028	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.4;
	1028	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.9;
	1028	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861;
	1028	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3466.3;
	1028	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9780.5;
	1028	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5774.3;
	1028	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9831.6;
	1028	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1472;
	1029	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.6;
	1029	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.8;
	1029	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.7;
	1029	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3356;
	1029	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9603.7;
	1029	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5569.9;
	1029	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9552.5;
	1029	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1438.1;
	1030	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.2;
	1030	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	718.8;
	1030	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	829.9;
	1030	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3220;
	1030	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9337.3;
	1030	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5336.7;
	1030	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9182.5;
	1030	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1386.8;
	1031	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.8;
	1031	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	700.1;
	1031	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	797.4;
	1031	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3041.4;
	1031	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8893.7;
	1031	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5005.9;
	1031	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8703;
	1031	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1316.2;
	1032	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.6;
	1032	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	675.3;
	1032	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	759.6;
	1032	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2818;
	1032	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8375;
	1032	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4625.6;
	1032	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8189.1;
	1032	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1242.4;
	1033	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.7;
	1033	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	686.7;
	1033	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	730.2;
	1033	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2617;
	1033	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7966.5;
	1033	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4310.8;
	1033	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7759.6;
	1033	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1193.4;
	1034	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	859.1;
	1034	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	686.1;
	1034	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	721.6;
	1034	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2475.3;
	1034	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7797.2;
	1034	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4119.1;
	1034	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7467.3;
	1034	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1161;
	1035	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	863.4;
	1035	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	697.9;
	1035	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	724;
	1035	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2408.1;
	1035	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7803.7;
	1035	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4002.4;
	1035	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7306.6;
	1035	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1153.3;
	1036	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.8;
	1036	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	709;
	1036	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	737.1;
	1036	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2328.1;
	1036	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7915;
	1036	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3992.3;
	1036	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7232.1;
	1036	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1158.2;
	1037	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.1;
	1037	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.2;
	1037	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	756.2;
	1037	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2324.5;
	1037	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8155.2;
	1037	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4041.3;
	1037	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7248.6;
	1037	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1196.4;
	1038	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.7;
	1038	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	763.3;
	1038	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	789.9;
	1038	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2407.5;
	1038	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8672.1;
	1038	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4237;
	1038	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7406.5;
	1038	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1259.7;
	1039	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.7;
	1039	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	805.8;
	1039	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	840.7;
	1039	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2562.9;
	1039	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9381.5;
	1039	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4562.4;
	1039	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7694.8;
	1039	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1352.6;
	1040	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.2;
	1040	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	845.2;
	1040	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897.5;
	1040	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2698;
	1040	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10054.1;
	1040	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4853.9;
	1040	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7874.8;
	1040	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1439.8;
	1041	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.4;
	1041	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	861.4;
	1041	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913.4;
	1041	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2876.1;
	1041	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10427.6;
	1041	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5122.8;
	1041	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8219.3;
	1041	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1489.6;
	1042	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.1;
	1042	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	851.5;
	1042	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.3;
	1042	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3010.2;
	1042	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10378;
	1042	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5234.8;
	1042	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8487.6;
	1042	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1475.8;
	1043	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.9;
	1043	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	826.7;
	1043	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	858.8;
	1043	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3081.7;
	1043	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10097.6;
	1043	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5203.6;
	1043	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8716.5;
	1043	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1431.3;
	1044	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.1;
	1044	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	788.6;
	1044	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824.9;
	1044	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3132.2;
	1044	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9691.5;
	1044	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5140.8;
	1044	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8808.9;
	1044	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1384.8;
	1045	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.4;
	1045	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	750.5;
	1045	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	806;
	1045	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3181.3;
	1045	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9285.3;
	1045	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5094.4;
	1045	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8809;
	1045	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1340.4;
	1046	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.9;
	1046	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	721.3;
	1046	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	795.4;
	1046	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3245.9;
	1046	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8931.2;
	1046	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5054.6;
	1046	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8774.9;
	1046	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1302.6;
	1047	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.7;
	1047	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	702.5;
	1047	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	792.2;
	1047	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3291.1;
	1047	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8686.9;
	1047	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5052.7;
	1047	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8741.4;
	1047	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1283.4;
	1048	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.2;
	1048	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	690.7;
	1048	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	790.8;
	1048	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3312.7;
	1048	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8554.6;
	1048	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5079.9;
	1048	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8702.8;
	1048	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1276.4;
	1049	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870;
	1049	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	692.6;
	1049	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	802;
	1049	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3308.7;
	1049	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8534.6;
	1049	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5141;
	1049	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8624.9;
	1049	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1287.1;
	1050	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.3;
	1050	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	705.7;
	1050	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	815.4;
	1050	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3243.3;
	1050	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8687.3;
	1050	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5168.2;
	1050	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8605;
	1050	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1319.4;
	1051	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.8;
	1051	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.7;
	1051	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	833.6;
	1051	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3278.8;
	1051	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9227.7;
	1051	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5350.3;
	1051	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8948;
	1051	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1420.8;
	1052	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894;
	1052	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	757;
	1052	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	853;
	1052	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3340.4;
	1052	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9415.5;
	1052	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5462;
	1052	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9020.1;
	1052	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1444.6;
	1053	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.8;
	1053	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	756.2;
	1053	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	841.4;
	1053	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3252.9;
	1053	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9278.7;
	1053	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5336.9;
	1053	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8887.2;
	1053	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1424.3;
	1054	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.7;
	1054	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742.1;
	1054	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	819.4;
	1054	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3137.6;
	1054	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9027;
	1054	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5153.6;
	1054	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8684.5;
	1054	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1388;
	1055	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.2;
	1055	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719;
	1055	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	791.1;
	1055	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3000.5;
	1055	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8613.2;
	1055	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4893.1;
	1055	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8381.2;
	1055	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1329.3;
	1056	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	858.7;
	1056	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	690.5;
	1056	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	748;
	1056	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2822.6;
	1056	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8130.9;
	1056	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4558.2;
	1056	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8018.9;
	1056	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1255.4;
	1057	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	849.6;
	1057	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	667.9;
	1057	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	717.1;
	1057	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2643.6;
	1057	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7698.2;
	1057	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4262.8;
	1057	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7669.4;
	1057	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1187.3;
	1058	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	843.3;
	1058	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	655.2;
	1058	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	703.1;
	1058	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2510.4;
	1058	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7370;
	1058	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4076.6;
	1058	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7440.9;
	1058	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1140.9;
	1059	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	843.5;
	1059	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	648.3;
	1059	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	697.6;
	1059	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2426.3;
	1059	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7180.6;
	1059	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3972.2;
	1059	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7312.8;
	1059	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1109.6;
	1060	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	844.1;
	1060	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	643;
	1060	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	698.6;
	1060	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2384.9;
	1060	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7091.5;
	1060	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3919.1;
	1060	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7233.9;
	1060	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1098.9;
	1061	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	848.1;
	1061	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	649.8;
	1061	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	703.5;
	1061	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2377.4;
	1061	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7103.5;
	1061	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3915.9;
	1061	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7242;
	1061	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1105.4;
	1062	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.2;
	1062	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	656;
	1062	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	713.9;
	1062	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2409.6;
	1062	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7259;
	1062	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4013.2;
	1062	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7325.8;
	1062	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1124.9;
	1063	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866.9;
	1063	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	675.6;
	1063	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	738.5;
	1063	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2491.5;
	1063	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7560.2;
	1063	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4189;
	1063	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7498.9;
	1063	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1171.2;
	1064	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.1;
	1064	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	701.8;
	1064	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	770.7;
	1064	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2547.3;
	1064	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7909.2;
	1064	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4376;
	1064	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7548.5;
	1064	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1219.4;
	1065	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.5;
	1065	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.8;
	1065	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	802.1;
	1065	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2715.3;
	1065	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8381.5;
	1065	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4682.4;
	1065	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7796.4;
	1065	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1280.8;
	1066	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.7;
	1066	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.9;
	1066	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	827.8;
	1066	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2882.8;
	1066	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8767.2;
	1066	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4931.5;
	1066	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8143.8;
	1066	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1305.5;
	1067	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.7;
	1067	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	724.8;
	1067	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	811.6;
	1067	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2981.4;
	1067	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8874.5;
	1067	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5034.4;
	1067	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8405.6;
	1067	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1293;
	1068	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.4;
	1068	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	706.2;
	1068	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	796.5;
	1068	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3036.2;
	1068	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8833.2;
	1068	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5035;
	1068	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8540.5;
	1068	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1276;
	1069	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.3;
	1069	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	699.9;
	1069	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	788.5;
	1069	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3098;
	1069	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8794.8;
	1069	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4999.9;
	1069	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8636.1;
	1069	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1282.8;
	1070	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.9;
	1070	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	696.2;
	1070	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	780.2;
	1070	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3176.4;
	1070	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8723.7;
	1070	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4974.2;
	1070	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8705;
	1070	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1273.2;
	1071	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	863.8;
	1071	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	681.9;
	1071	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	780.7;
	1071	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3246;
	1071	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8598.1;
	1071	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4936.1;
	1071	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8693.8;
	1071	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1264.3;
	1072	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	860.5;
	1072	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	684;
	1072	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	790.3;
	1072	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3295.8;
	1072	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8521.2;
	1072	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4946.9;
	1072	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8672.7;
	1072	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1271.5;
	1073	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	865.4;
	1073	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	689;
	1073	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	797.3;
	1073	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3308.8;
	1073	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8533;
	1073	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4960.3;
	1073	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8693.9;
	1073	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1294.6;
	1074	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867.3;
	1074	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	705.3;
	1074	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	807;
	1074	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3269.6;
	1074	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8744;
	1074	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5103.3;
	1074	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8784.7;
	1074	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1324.5;
	1075	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.8;
	1075	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	734.1;
	1075	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	836.2;
	1075	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3346;
	1075	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9335.5;
	1075	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5442.5;
	1075	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9244.5;
	1075	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1423.1;
	1076	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.7;
	1076	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.4;
	1076	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857;
	1076	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3430.7;
	1076	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9481.1;
	1076	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5584.6;
	1076	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9333.1;
	1076	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1445.5;
	1077	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.9;
	1077	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.9;
	1077	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	841.5;
	1077	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3354.1;
	1077	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9251;
	1077	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5424.9;
	1077	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9171.6;
	1077	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1409.8;
	1078	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894;
	1078	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	715.8;
	1078	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	816.5;
	1078	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3270.8;
	1078	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8924.4;
	1078	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5220.4;
	1078	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8932.8;
	1078	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1359.5;
	1079	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.7;
	1079	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	686.8;
	1079	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	780.1;
	1079	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3083.5;
	1079	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8354;
	1079	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4887.7;
	1079	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8516.1;
	1079	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1276.4;
	1080	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	861;
	1080	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	661.1;
	1080	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	740.6;
	1080	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2813.6;
	1080	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7717.8;
	1080	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4491.8;
	1080	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8010.3;
	1080	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1182.6;
	1081	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	851;
	1081	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	633.6;
	1081	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	747.6;
	1081	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2565;
	1081	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7246.7;
	1081	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4173.6;
	1081	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7589.3;
	1081	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1190;
	1082	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	850.9;
	1082	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	629.5;
	1082	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	744.6;
	1082	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2423.7;
	1082	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6989.2;
	1082	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3999.1;
	1082	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7368.3;
	1082	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1154.8;
	1083	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.4;
	1083	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	635.5;
	1083	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	751.3;
	1083	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2345.1;
	1083	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6903.2;
	1083	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3919.1;
	1083	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7263.3;
	1083	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1094.8;
	1084	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.4;
	1084	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	633.4;
	1084	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	752.9;
	1084	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2317.8;
	1084	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6940.8;
	1084	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3917.7;
	1084	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7260.2;
	1084	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1129.2;
	1085	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	863.3;
	1085	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	645.6;
	1085	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	766;
	1085	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2351.7;
	1085	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7213.6;
	1085	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4014.7;
	1085	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7440.2;
	1085	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1160.9;
	1086	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.4;
	1086	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	680.4;
	1086	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	816.3;
	1086	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2509.8;
	1086	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7924.1;
	1086	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4323.8;
	1086	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7906.1;
	1086	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1199.7;
	1087	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925.5;
	1087	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	748.3;
	1087	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	908.3;
	1087	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2849.9;
	1087	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8984.5;
	1087	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4781.2;
	1087	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8573.7;
	1087	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1352.7;
	1088	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.4;
	1088	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	785.1;
	1088	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	956.9;
	1088	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2960.6;
	1088	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9531.3;
	1088	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5049;
	1088	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8797.2;
	1088	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1434.8;
	1089	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.2;
	1089	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	783.6;
	1089	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	943;
	1089	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2981.4;
	1089	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9589.5;
	1089	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5162.7;
	1089	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9050.6;
	1089	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1427.5;
	1090	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.9;
	1090	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	775.9;
	1090	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	922.7;
	1090	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3077.2;
	1090	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9660.3;
	1090	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5284.4;
	1090	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9357;
	1090	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1554;
	1091	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.6;
	1091	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	768.7;
	1091	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	906.3;
	1091	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3214;
	1091	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9742.8;
	1091	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5386.2;
	1091	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9621.6;
	1091	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1562.2;
	1092	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911;
	1092	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	758.3;
	1092	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	883.5;
	1092	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3348.7;
	1092	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9725.3;
	1092	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5445.8;
	1092	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9794.7;
	1092	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1560.2;
	1093	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.8;
	1093	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.7;
	1093	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	871.5;
	1093	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3507.8;
	1093	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9661;
	1093	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5476.3;
	1093	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9906.4;
	1093	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1547.8;
	1094	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896;
	1094	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.4;
	1094	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	874.2;
	1094	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3683.4;
	1094	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9618.5;
	1094	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5518.9;
	1094	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10088.2;
	1094	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1551.2;
	1095	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.1;
	1095	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	713;
	1095	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	870.8;
	1095	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3846.3;
	1095	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9523.9;
	1095	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5542.1;
	1095	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10190.8;
	1095	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1561.2;
	1096	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	881.5;
	1096	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	709;
	1096	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.7;
	1096	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3951.1;
	1096	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9448.8;
	1096	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5583.7;
	1096	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10232.5;
	1096	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1536.3;
	1097	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871.7;
	1097	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	705.4;
	1097	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	853.9;
	1097	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3978;
	1097	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9442.8;
	1097	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5627.3;
	1097	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10110.5;
	1097	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1546.1;
	1098	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870;
	1098	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711.3;
	1098	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851;
	1098	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3872.7;
	1098	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9544;
	1098	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5674;
	1098	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9932.6;
	1098	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1525.5;
	1099	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.6;
	1099	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.7;
	1099	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	887;
	1099	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3824.2;
	1099	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10100.8;
	1099	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5879;
	1099	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9974.8;
	1099	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1628.3;
	1100	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.8;
	1100	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	760.9;
	1100	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	915.6;
	1100	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3858.7;
	1100	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10277.7;
	1100	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6002.9;
	1100	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9979.2;
	1100	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1669.8;
	1101	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.6;
	1101	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753.9;
	1101	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	905;
	1101	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3749.2;
	1101	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10096;
	1101	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5781;
	1101	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9678.4;
	1101	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1633.9;
	1102	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.8;
	1102	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.1;
	1102	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	879;
	1102	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3571.2;
	1102	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9690.5;
	1102	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5429.7;
	1102	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9257.3;
	1102	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1494.2;
	1103	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.7;
	1103	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	705.9;
	1103	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	840.8;
	1103	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3268.3;
	1103	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9014;
	1103	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4962.8;
	1103	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8624.2;
	1103	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1406;
	1104	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866.9;
	1104	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	672.9;
	1104	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	797.3;
	1104	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2902.9;
	1104	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8278.4;
	1104	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4496.6;
	1104	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8007.4;
	1104	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1371.5;
	1105	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	857.6;
	1105	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	657;
	1105	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	775.3;
	1105	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2634.1;
	1105	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7831.3;
	1105	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4197.7;
	1105	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7594.1;
	1105	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1355.3;
	1106	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	857.1;
	1106	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	652;
	1106	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	772.7;
	1106	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2491.5;
	1106	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7680.5;
	1106	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4062.5;
	1106	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7441.5;
	1106	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1330.2;
	1107	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	859;
	1107	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	651.1;
	1107	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	777.6;
	1107	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2416.3;
	1107	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7653.2;
	1107	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4018.8;
	1107	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7384.7;
	1107	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1312.3;
	1108	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	863.1;
	1108	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	655.4;
	1108	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	790.4;
	1108	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2397.8;
	1108	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7740.3;
	1108	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4058.5;
	1108	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7426.8;
	1108	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1326.1;
	1109	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.7;
	1109	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	676.1;
	1109	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	815.3;
	1109	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2419.9;
	1109	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8089.1;
	1109	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4254.8;
	1109	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7655.7;
	1109	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1402.4;
	1110	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.8;
	1110	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.3;
	1110	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	879.4;
	1110	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2611.7;
	1110	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8943.3;
	1110	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4787.5;
	1110	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8335.8;
	1110	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1545.8;
	1111	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.4;
	1111	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	795.6;
	1111	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	990.2;
	1111	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3061.8;
	1111	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10357.6;
	1111	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5669.1;
	1111	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9303.2;
	1111	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1786.1;
	1112	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948.2;
	1112	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	820.6;
	1112	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1020;
	1112	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3172.5;
	1112	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10706.5;
	1112	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5907.6;
	1112	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9456.3;
	1112	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1806;
	1113	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.6;
	1113	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	800.3;
	1113	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	976.2;
	1113	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3094.6;
	1113	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10285.8;
	1113	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5658.3;
	1113	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9434.8;
	1113	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1750.7;
	1114	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.7;
	1114	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	779.3;
	1114	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	934.5;
	1114	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3117.8;
	1114	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10022.6;
	1114	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5463.8;
	1114	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9437.4;
	1114	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1678.1;
	1115	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.6;
	1115	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	763.4;
	1115	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	901.4;
	1115	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3177.4;
	1115	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9867.5;
	1115	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5341.8;
	1115	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9506.8;
	1115	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1630.5;
	1116	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.8;
	1116	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	748.5;
	1116	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	879.9;
	1116	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3271.8;
	1116	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9714.7;
	1116	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5309.3;
	1116	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9570.8;
	1116	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1600.8;
	1117	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.3;
	1117	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.7;
	1117	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.2;
	1117	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3446.2;
	1117	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9637.4;
	1117	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5317.7;
	1117	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9613.2;
	1117	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1575.4;
	1118	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.5;
	1118	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.2;
	1118	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.5;
	1118	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3640.6;
	1118	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9603.8;
	1118	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5377.5;
	1118	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9716.2;
	1118	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1547.5;
	1119	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.7;
	1119	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.9;
	1119	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.6;
	1119	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3800.4;
	1119	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9513.9;
	1119	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5464.6;
	1119	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9811.4;
	1119	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1562;
	1120	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.6;
	1120	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	715.3;
	1120	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.6;
	1120	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3925.3;
	1120	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9394.1;
	1120	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5580.4;
	1120	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9923.1;
	1120	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1570.6;
	1121	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.9;
	1121	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.6;
	1121	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.2;
	1121	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3976.9;
	1121	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9403.5;
	1121	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5669.9;
	1121	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9951;
	1121	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1575.9;
	1122	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.7;
	1122	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	726.2;
	1122	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	862.3;
	1122	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3870.3;
	1122	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9495.2;
	1122	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5726.4;
	1122	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9833.6;
	1122	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1541.1;
	1123	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.7;
	1123	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	768.6;
	1123	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	891.4;
	1123	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3818.3;
	1123	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10013;
	1123	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5914.2;
	1123	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9945.2;
	1123	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1630.8;
	1124	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.4;
	1124	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	783.3;
	1124	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	917.9;
	1124	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3857.7;
	1124	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10248.6;
	1124	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6032;
	1124	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9962.6;
	1124	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1660.1;
	1125	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922;
	1125	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777;
	1125	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	908.7;
	1125	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3753.5;
	1125	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10063.6;
	1125	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5844.6;
	1125	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9670.6;
	1125	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1622.2;
	1126	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908;
	1126	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	756.1;
	1126	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	875.7;
	1126	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3573.4;
	1126	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9602.9;
	1126	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5470.4;
	1126	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9177.3;
	1126	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1549.2;
	1127	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.7;
	1127	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723.3;
	1127	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	827.4;
	1127	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3292.9;
	1127	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8891.1;
	1127	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4969.1;
	1127	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8557.1;
	1127	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1373.7;
	1128	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	869.9;
	1128	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	693.2;
	1128	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	780.4;
	1128	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2945.5;
	1128	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8160.9;
	1128	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4455.9;
	1128	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7947;
	1128	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1264.6;
	1129	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.2;
	1129	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	681.1;
	1129	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	757.7;
	1129	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2684.7;
	1129	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7798.3;
	1129	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4114.3;
	1129	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7510.2;
	1129	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1268.5;
	1130	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	852.8;
	1130	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	682.1;
	1130	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	752.7;
	1130	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2527.6;
	1130	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7661.5;
	1130	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3958.6;
	1130	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7315.9;
	1130	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1288.3;
	1131	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	856.7;
	1131	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	692.8;
	1131	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	762.8;
	1131	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2449.6;
	1131	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7719.6;
	1131	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3892.2;
	1131	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7235.1;
	1131	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1276.3;
	1132	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	862.6;
	1132	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	705.5;
	1132	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	776.5;
	1132	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2422.1;
	1132	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7893.5;
	1132	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3907.2;
	1132	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7260;
	1132	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1307;
	1133	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	869.6;
	1133	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	734.8;
	1133	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	801.1;
	1133	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2455.5;
	1133	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8386.2;
	1133	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4061.7;
	1133	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7484.8;
	1133	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1352.2;
	1134	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.7;
	1134	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	786.2;
	1134	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	863.3;
	1134	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2638.5;
	1134	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9462.8;
	1134	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4548.1;
	1134	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8127.3;
	1134	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1412.9;
	1135	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.9;
	1135	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	872.1;
	1135	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	974.8;
	1135	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3071.4;
	1135	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11049.3;
	1135	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5390.2;
	1135	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9041.8;
	1135	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1658.3;
	1136	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.2;
	1136	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	895.8;
	1136	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1019.4;
	1136	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3156.2;
	1136	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11447;
	1136	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5659.7;
	1136	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9196.9;
	1136	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1712.3;
	1137	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.6;
	1137	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	858.8;
	1137	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	975.1;
	1137	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3112.9;
	1137	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10925;
	1137	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5481.5;
	1137	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9216.8;
	1137	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1646.8;
	1138	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.8;
	1138	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	824.1;
	1138	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	922.2;
	1138	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3197.8;
	1138	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10474.5;
	1138	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5358.6;
	1138	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9269.9;
	1138	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1646.7;
	1139	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917;
	1139	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	801.3;
	1139	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	902.6;
	1139	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3312;
	1139	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10146.6;
	1139	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5307.9;
	1139	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9351;
	1139	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1634.2;
	1140	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.3;
	1140	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	779.8;
	1140	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.3;
	1140	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3435.1;
	1140	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9898.3;
	1140	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5255.2;
	1140	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9416.5;
	1140	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1568.9;
	1141	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.4;
	1141	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	756;
	1141	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.1;
	1141	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3556;
	1141	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9667.2;
	1141	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5276.4;
	1141	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9456.3;
	1141	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1551.7;
	1142	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.9;
	1142	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.7;
	1142	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	834.3;
	1142	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3675.5;
	1142	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9580.7;
	1142	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5292;
	1142	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9500.1;
	1142	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1547;
	1143	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.4;
	1143	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727;
	1143	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	836.2;
	1143	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3770.8;
	1143	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9413.8;
	1143	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5366.3;
	1143	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9559;
	1143	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1536.5;
	1144	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904;
	1144	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	721.7;
	1144	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	848.2;
	1144	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3850.2;
	1144	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9355.5;
	1144	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5451.3;
	1144	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9626.4;
	1144	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1510.1;
	1145	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.2;
	1145	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723.1;
	1145	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.9;
	1145	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3844.2;
	1145	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9370.4;
	1145	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5505;
	1145	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9528.3;
	1145	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1541.5;
	1146	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.5;
	1146	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723.4;
	1146	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	846.8;
	1146	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3706.9;
	1146	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9428.1;
	1146	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5568.1;
	1146	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9322;
	1146	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1534.7;
	1147	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.4;
	1147	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	761.6;
	1147	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	876;
	1147	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3688.1;
	1147	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10038.7;
	1147	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5774.8;
	1147	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9583;
	1147	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1628.4;
	1148	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915;
	1148	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	784.4;
	1148	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	911.1;
	1148	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3773.2;
	1148	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10311.9;
	1148	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5903.8;
	1148	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9687;
	1148	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1670.3;
	1149	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.7;
	1149	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	779.5;
	1149	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	901.6;
	1149	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3688.6;
	1149	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10165.9;
	1149	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5797.2;
	1149	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9529.4;
	1149	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1636.7;
	1150	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.1;
	1150	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.3;
	1150	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.5;
	1150	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3532.8;
	1150	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9738.8;
	1150	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5445.1;
	1150	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9158.4;
	1150	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1584.4;
	1151	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879;
	1151	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711.8;
	1151	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	809.3;
	1151	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3258.1;
	1151	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8984.7;
	1151	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4991.1;
	1151	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8596.1;
	1151	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1482.4;
	1152	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	848.6;
	1152	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	674.3;
	1152	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	754.9;
	1152	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2900.6;
	1152	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8169.5;
	1152	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4470.1;
	1152	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8003.2;
	1152	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1327.9;
	1153	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	813.7;
	1153	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	649.2;
	1153	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	731.6;
	1153	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2616.7;
	1153	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7740.3;
	1153	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4235.5;
	1153	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7558.8;
	1153	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1293.8;
	1154	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	810.4;
	1154	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	641.6;
	1154	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	721;
	1154	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2489.7;
	1154	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7545;
	1154	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4058.8;
	1154	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7450.4;
	1154	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1268.9;
	1155	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	807.3;
	1155	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	642.7;
	1155	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	716.6;
	1155	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2426.9;
	1155	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7465.4;
	1155	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4001;
	1155	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7416.8;
	1155	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1253.4;
	1156	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	807;
	1156	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	650.2;
	1156	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	717.6;
	1156	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2411.5;
	1156	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7519.9;
	1156	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3997.8;
	1156	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7468.2;
	1156	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1283.7;
	1157	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	812.1;
	1157	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	664.2;
	1157	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	732.3;
	1157	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2441.9;
	1157	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7828.7;
	1157	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4137;
	1157	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7725.7;
	1157	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1326.3;
	1158	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	835.5;
	1158	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	712.7;
	1158	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	778.7;
	1158	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2615.4;
	1158	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8678.2;
	1158	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4556.1;
	1158	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8416.9;
	1158	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1473.6;
	1159	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.8;
	1159	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	792.9;
	1159	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	870.2;
	1159	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3043.5;
	1159	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10070;
	1159	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5307.9;
	1159	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9316.3;
	1159	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1682.2;
	1160	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.6;
	1160	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	814.2;
	1160	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	901.7;
	1160	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3128;
	1160	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10466.9;
	1160	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5524.1;
	1160	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9407.9;
	1160	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1719.7;
	1161	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890;
	1161	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	797.1;
	1161	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.1;
	1161	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3107.1;
	1161	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10247.1;
	1161	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5449;
	1161	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9366.8;
	1161	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1634.2;
	1162	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.7;
	1162	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	779.2;
	1162	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	876.4;
	1162	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3193.1;
	1162	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10099;
	1162	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5462.3;
	1162	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9370.4;
	1162	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1650.4;
	1163	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.8;
	1163	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	766.7;
	1163	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	873.4;
	1163	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3306.8;
	1163	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10012.1;
	1163	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5452.9;
	1163	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9484.4;
	1163	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1622.8;
	1164	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899;
	1164	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755.3;
	1164	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	862.4;
	1164	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3428;
	1164	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9856.3;
	1164	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5443.4;
	1164	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9566.2;
	1164	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1575.1;
	1165	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.6;
	1165	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746.2;
	1165	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	850;
	1165	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3541.7;
	1165	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9745.8;
	1165	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5503.3;
	1165	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9650.8;
	1165	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1578.3;
	1166	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.2;
	1166	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	748.5;
	1166	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.3;
	1166	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3676.9;
	1166	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9716.4;
	1166	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5585.4;
	1166	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9830.9;
	1166	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1585.3;
	1167	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927;
	1167	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	750.5;
	1167	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	855.8;
	1167	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3783.3;
	1167	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9648.6;
	1167	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5663.6;
	1167	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9944.4;
	1167	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1602.1;
	1168	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.1;
	1168	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749.8;
	1168	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861;
	1168	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3846.8;
	1168	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9608.3;
	1168	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5777.3;
	1168	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9977.2;
	1168	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1608.7;
	1169	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.6;
	1169	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	750.3;
	1169	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	863.6;
	1169	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3855.5;
	1169	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9629.9;
	1169	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5874.6;
	1169	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9950.6;
	1169	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1654.7;
	1170	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925.5;
	1170	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.2;
	1170	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	858;
	1170	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3734.3;
	1170	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9654.1;
	1170	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5924.6;
	1170	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9798.4;
	1170	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1635.7;
	1171	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.2;
	1171	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	775.1;
	1171	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	877.5;
	1171	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3702.2;
	1171	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10080.2;
	1171	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6089.9;
	1171	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9934.6;
	1171	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1703.7;
	1172	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.5;
	1172	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	781.9;
	1172	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	906.2;
	1172	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3781.2;
	1172	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10282.8;
	1172	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6150.5;
	1172	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10018.7;
	1172	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1729.3;
	1173	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.9;
	1173	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	766.9;
	1173	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	890.4;
	1173	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3697;
	1173	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10086.1;
	1173	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5995.3;
	1173	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9816;
	1173	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1690.4;
	1174	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.8;
	1174	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.2;
	1174	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	855.8;
	1174	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3537.2;
	1174	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9625;
	1174	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5723.6;
	1174	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9428.5;
	1174	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1624.5;
	1175	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.9;
	1175	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	694;
	1175	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	802.3;
	1175	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3267.9;
	1175	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8885.3;
	1175	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5315.3;
	1175	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8806.4;
	1175	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1497.5;
	1176	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.3;
	1176	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	652.3;
	1176	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	753;
	1176	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2940.3;
	1176	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8062.6;
	1176	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4845;
	1176	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8188.7;
	1176	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1360.5;
	1177	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	840.4;
	1177	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	624.5;
	1177	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	722.3;
	1177	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2698.9;
	1177	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7538.9;
	1177	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4361.1;
	1177	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7832.9;
	1177	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1295.2;
	1178	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	828.1;
	1178	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	610.8;
	1178	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	706.9;
	1178	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2544.3;
	1178	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7174.5;
	1178	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4162.7;
	1178	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7622.9;
	1178	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1243.4;
	1179	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	819;
	1179	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	603.3;
	1179	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	695.3;
	1179	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2452.1;
	1179	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6992.1;
	1179	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4038.7;
	1179	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7495.5;
	1179	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1205.7;
	1180	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	814.9;
	1180	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.2;
	1180	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	695.7;
	1180	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2417;
	1180	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6924.5;
	1180	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4002.9;
	1180	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7469.5;
	1180	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1190.4;
	1181	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	818.7;
	1181	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	602.8;
	1181	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	704.6;
	1181	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2436;
	1181	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7121.5;
	1181	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4065.6;
	1181	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7625.8;
	1181	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1141.4;
	1182	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	834.6;
	1182	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	641.5;
	1182	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	739.8;
	1182	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2587.1;
	1182	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7766.8;
	1182	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4408;
	1182	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8148.1;
	1182	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1219.3;
	1183	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.4;
	1183	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	718.9;
	1183	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	821.4;
	1183	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2979.8;
	1183	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8921.2;
	1183	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5089.4;
	1183	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8960.7;
	1183	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1401.2;
	1184	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.5;
	1184	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.5;
	1184	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.2;
	1184	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3070.1;
	1184	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9364.5;
	1184	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5352.4;
	1184	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9095.9;
	1184	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1440.1;
	1185	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.9;
	1185	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.9;
	1185	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	853.4;
	1185	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3083.9;
	1185	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9350.8;
	1185	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5343.7;
	1185	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9254.2;
	1185	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1442.4;
	1186	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.3;
	1186	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746;
	1186	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857;
	1186	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3207.1;
	1186	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9514.7;
	1186	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5448.4;
	1186	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9517.3;
	1186	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1472.1;
	1187	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.2;
	1187	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746.9;
	1187	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861;
	1187	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3364.9;
	1187	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9692.8;
	1187	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5516;
	1187	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9834.2;
	1187	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1496.4;
	1188	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.5;
	1188	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	745;
	1188	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	855.1;
	1188	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3509.6;
	1188	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9774.6;
	1188	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5608.6;
	1188	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10104;
	1188	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1508.8;
	1189	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.8;
	1189	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.1;
	1189	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.6;
	1189	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3644;
	1189	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9873.6;
	1189	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5736.3;
	1189	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10317.2;
	1189	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1527.8;
	1190	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948.4;
	1190	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755.5;
	1190	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	894.8;
	1190	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3786.5;
	1190	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10059.7;
	1190	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5889.1;
	1190	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10628;
	1190	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1553.6;
	1191	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.5;
	1191	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	758.6;
	1191	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	916.4;
	1191	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3912.2;
	1191	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10129;
	1191	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6018.5;
	1191	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10752.8;
	1191	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1579.3;
	1192	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.9;
	1192	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.4;
	1192	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	924.4;
	1192	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3982.3;
	1192	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10088.4;
	1192	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6144;
	1192	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10717.1;
	1192	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1576.4;
	1193	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.1;
	1193	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744.1;
	1193	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	923.4;
	1193	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3970.4;
	1193	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10097;
	1193	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6224.8;
	1193	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10615;
	1193	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1571.8;
	1194	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.7;
	1194	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.6;
	1194	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	902;
	1194	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3807.3;
	1194	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10070.5;
	1194	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6179;
	1194	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10321.3;
	1194	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1541.5;
	1195	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.5;
	1195	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.3;
	1195	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	887.8;
	1195	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3728.5;
	1195	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10275.4;
	1195	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6214.7;
	1195	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10211.1;
	1195	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1571.9;
	1196	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.2;
	1196	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	750.7;
	1196	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889;
	1196	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3735.1;
	1196	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10252;
	1196	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6222;
	1196	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10138.9;
	1196	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1569.7;
	1197	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.3;
	1197	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735;
	1197	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	864.5;
	1197	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3623.5;
	1197	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9986.7;
	1197	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5999.5;
	1197	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9868.7;
	1197	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1517.2;
	1198	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.3;
	1198	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	715.6;
	1198	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	836.4;
	1198	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3483;
	1198	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9632.9;
	1198	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5717.7;
	1198	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9505.3;
	1198	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1451;
	1199	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.8;
	1199	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	680.2;
	1199	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	797.9;
	1199	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3292;
	1199	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9063.9;
	1199	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5384.2;
	1199	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9022.2;
	1199	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1371.4;
	1200	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	876.7;
	1200	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	642.1;
	1200	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	752.1;
	1200	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3056.9;
	1200	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8360.3;
	1200	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4965.1;
	1200	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8501.1;
	1200	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1274;
	1201	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	859;
	1201	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	616.8;
	1201	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	716.3;
	1201	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2849.7;
	1201	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7729.6;
	1201	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4612.7;
	1201	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8071.4;
	1201	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1194.5;
	1202	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	847.1;
	1202	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	596.4;
	1202	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	692.6;
	1202	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2687.3;
	1202	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7330;
	1202	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4349.2;
	1202	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7808.2;
	1202	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1140;
	1203	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	841.4;
	1203	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586.6;
	1203	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	676.4;
	1203	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2581.1;
	1203	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7082.3;
	1203	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4191.7;
	1203	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7626;
	1203	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1107;
	1204	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	837.1;
	1204	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	582.6;
	1204	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	670.3;
	1204	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2517.2;
	1204	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6948.6;
	1204	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4083.1;
	1204	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7579.2;
	1204	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1092.2;
	1205	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	833.9;
	1205	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.3;
	1205	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	669.9;
	1205	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2503.9;
	1205	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6932;
	1205	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4080.8;
	1205	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7588.8;
	1205	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1089;
	1206	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	830.6;
	1206	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.3;
	1206	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	681.4;
	1206	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2550.5;
	1206	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7144.1;
	1206	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4148.7;
	1206	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7668.5;
	1206	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1121.1;
	1207	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	839.9;
	1207	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	620.7;
	1207	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	708.5;
	1207	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2656.6;
	1207	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7561.5;
	1207	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4394.2;
	1207	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7909.3;
	1207	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1175.4;
	1208	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	844.8;
	1208	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	639.4;
	1208	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	740.4;
	1208	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2738.4;
	1208	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8006.1;
	1208	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4647.1;
	1208	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8052.7;
	1208	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1228.6;
	1209	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	854.4;
	1209	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	669.6;
	1209	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	775.3;
	1209	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2917.9;
	1209	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8547.8;
	1209	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4938.2;
	1209	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8491.7;
	1209	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1294.8;
	1210	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871.1;
	1210	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	698;
	1210	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	805.7;
	1210	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3118.7;
	1210	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9075.9;
	1210	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5215.2;
	1210	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8919.2;
	1210	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1348.8;
	1211	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.6;
	1211	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	716.5;
	1211	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	820.4;
	1211	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3288.6;
	1211	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9387.2;
	1211	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5379.8;
	1211	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9199.8;
	1211	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1367.2;
	1212	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.4;
	1212	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.8;
	1212	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824.2;
	1212	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3442.3;
	1212	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9479.6;
	1212	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5439.1;
	1212	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9387.8;
	1212	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1339.4;
	1213	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.8;
	1213	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	722.8;
	1213	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	828.1;
	1213	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3610.1;
	1213	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9506.1;
	1213	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5469.2;
	1213	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9536.4;
	1213	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1330.9;
	1214	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.8;
	1214	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723.3;
	1214	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	830.6;
	1214	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3735.6;
	1214	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9460.9;
	1214	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5481.2;
	1214	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9598.9;
	1214	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1319.1;
	1215	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.4;
	1215	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.2;
	1215	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.2;
	1215	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3835.4;
	1215	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9440.3;
	1215	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5526.8;
	1215	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9657.9;
	1215	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1310.8;
	1216	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.1;
	1216	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	731.9;
	1216	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	840.8;
	1216	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3878.4;
	1216	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9400.9;
	1216	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5626.5;
	1216	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9705.3;
	1216	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1315;
	1217	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.2;
	1217	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.1;
	1217	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	848.5;
	1217	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3831.9;
	1217	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9337.5;
	1217	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5678.7;
	1217	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9619.7;
	1217	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1323.7;
	1218	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.5;
	1218	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.5;
	1218	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.3;
	1218	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3681.6;
	1218	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9403.1;
	1218	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5695.6;
	1218	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9415.2;
	1218	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1341.2;
	1219	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.7;
	1219	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.5;
	1219	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	871.3;
	1219	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3648.9;
	1219	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9759.1;
	1219	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5874;
	1219	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9565.4;
	1219	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1425.1;
	1220	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.1;
	1220	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.8;
	1220	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.7;
	1220	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3676.2;
	1220	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9807.5;
	1220	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5958.6;
	1220	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9586.8;
	1220	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1429.5;
	1221	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.8;
	1221	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	724.3;
	1221	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.5;
	1221	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3545.7;
	1221	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9586.6;
	1221	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5776.2;
	1221	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9364.1;
	1221	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1399.4;
	1222	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.7;
	1222	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	702.1;
	1222	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	833.8;
	1222	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3393.9;
	1222	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9271.2;
	1222	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5518.5;
	1222	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9083.2;
	1222	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1350.9;
	1223	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	881.5;
	1223	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	672.4;
	1223	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	796.4;
	1223	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3225.6;
	1223	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8784.4;
	1223	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5182.3;
	1223	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8696.1;
	1223	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1282.2;
	1224	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	862.1;
	1224	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	633.8;
	1224	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	747.5;
	1224	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3009.6;
	1224	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8173.7;
	1224	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4789.4;
	1224	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8260.8;
	1224	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1194.1;
	1225	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	836.3;
	1225	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	595.5;
	1225	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	710.4;
	1225	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2814;
	1225	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7558.1;
	1225	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4448.6;
	1225	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7912.6;
	1225	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1130.9;
	1226	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	826.6;
	1226	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	576.4;
	1226	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	687.7;
	1226	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2651.9;
	1226	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7149.2;
	1226	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4201.6;
	1226	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7635.8;
	1226	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1076;
	1227	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	824.5;
	1227	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565.2;
	1227	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	676.1;
	1227	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2552.3;
	1227	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6907.5;
	1227	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4035.5;
	1227	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7484.7;
	1227	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1042.5;
	1228	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	819.5;
	1228	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559.5;
	1228	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	670.1;
	1228	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2491.7;
	1228	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6762.9;
	1228	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3936.1;
	1228	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7369.3;
	1228	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1026.1;
	1229	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	818.4;
	1229	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	558.2;
	1229	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	677.8;
	1229	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2463.5;
	1229	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6705.4;
	1229	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3898.2;
	1229	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7327.9;
	1229	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1014.9;
	1230	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	817.4;
	1230	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	568.7;
	1230	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	695.2;
	1230	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2487.1;
	1230	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6807.2;
	1230	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3940.1;
	1230	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7366;
	1230	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1031.1;
	1231	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	825.9;
	1231	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586.1;
	1231	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	715.1;
	1231	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2554;
	1231	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7068.6;
	1231	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4099.6;
	1231	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7505.6;
	1231	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1071.9;
	1232	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	827.8;
	1232	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	605.7;
	1232	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	732.4;
	1232	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2588.8;
	1232	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7358.9;
	1232	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4306.9;
	1232	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7562.3;
	1232	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1111.6;
	1233	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	838.1;
	1233	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	643.2;
	1233	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	761.6;
	1233	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2777.6;
	1233	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7959.4;
	1233	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4586.5;
	1233	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7958.3;
	1233	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1200.6;
	1234	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	857.6;
	1234	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	670.6;
	1234	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	787.6;
	1234	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3026.3;
	1234	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8500.3;
	1234	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4894.9;
	1234	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8408.4;
	1234	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1271;
	1235	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871.1;
	1235	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	674.7;
	1235	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	793.8;
	1235	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3244.5;
	1235	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8838.3;
	1235	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5091.7;
	1235	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8768.9;
	1235	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1303;
	1236	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.5;
	1236	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	676.1;
	1236	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	798.2;
	1236	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3406.5;
	1236	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9018.4;
	1236	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5199.3;
	1236	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9011;
	1236	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1312.3;
	1237	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.4;
	1237	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	677.8;
	1237	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	801.9;
	1237	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3523.8;
	1237	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9100.8;
	1237	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5250.4;
	1237	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9054.9;
	1237	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1328.6;
	1238	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.8;
	1238	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	673.1;
	1238	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	798.2;
	1238	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3624.8;
	1238	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9077.4;
	1238	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5258.3;
	1238	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9077.7;
	1238	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1328.6;
	1239	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.8;
	1239	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	666;
	1239	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	793.4;
	1239	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3706;
	1239	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9002.2;
	1239	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5248.4;
	1239	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9085.1;
	1239	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1325.2;
	1240	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.5;
	1240	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	668.2;
	1240	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	793.1;
	1240	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3739.7;
	1240	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8962.5;
	1240	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5281.2;
	1240	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9090.3;
	1240	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1330.6;
	1241	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.4;
	1241	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	676.2;
	1241	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	800.7;
	1241	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3682.7;
	1241	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9047.9;
	1241	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5322.7;
	1241	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9104.6;
	1241	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1360.4;
	1242	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	881.1;
	1242	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	693.5;
	1242	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	815.7;
	1242	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3601.1;
	1242	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9318.9;
	1242	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5487.7;
	1242	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9232.7;
	1242	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1409.2;
	1243	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.3;
	1243	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	726.9;
	1243	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	848.7;
	1243	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3651.3;
	1243	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9773.5;
	1243	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5816;
	1243	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9685.1;
	1243	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1488.2;
	1244	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.8;
	1244	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.3;
	1244	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	871.9;
	1244	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3750.4;
	1244	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9843.9;
	1244	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5998.8;
	1244	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9787.9;
	1244	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1502;
	1245	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.5;
	1245	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	722.5;
	1245	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.8;
	1245	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3671.3;
	1245	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9574.7;
	1245	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5859.9;
	1245	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9616.8;
	1245	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1467.1;
	1246	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.2;
	1246	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	696.2;
	1246	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	817.8;
	1246	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3547.1;
	1246	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9111.6;
	1246	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5583.4;
	1246	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9278;
	1246	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1407.2;
	1247	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.9;
	1247	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	654.8;
	1247	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	769.1;
	1247	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3307.9;
	1247	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8372.7;
	1247	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5150.8;
	1247	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8726.1;
	1247	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1312.2;
	1248	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	853.6;
	1248	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	617.5;
	1248	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	720.8;
	1248	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2990;
	1248	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7613.4;
	1248	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4656.4;
	1248	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8114.1;
	1248	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1204.8;
	1249	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	826;
	1249	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.1;
	1249	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	704.8;
	1249	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2703;
	1249	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7205.1;
	1249	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4292.5;
	1249	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7571;
	1249	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1208.5;
	1250	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	818.8;
	1250	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	590.1;
	1250	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	688.9;
	1250	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2548.7;
	1250	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6940.4;
	1250	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4055;
	1250	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7323.5;
	1250	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1167.1;
	1251	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	819;
	1251	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	587;
	1251	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	684.6;
	1251	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2468.5;
	1251	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6836.6;
	1251	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3930.4;
	1251	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7210;
	1251	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1104.4;
	1252	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	816.9;
	1252	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	591.7;
	1252	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	683.8;
	1252	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2436;
	1252	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6862.7;
	1252	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3888;
	1252	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7197.1;
	1252	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1143.2;
	1253	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	823.1;
	1253	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	608.5;
	1253	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	696.6;
	1253	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2455.6;
	1253	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7130.8;
	1253	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3980.7;
	1253	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7383.2;
	1253	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1165.6;
	1254	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	839.8;
	1254	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	656.4;
	1254	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	744.8;
	1254	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2609.7;
	1254	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7909.1;
	1254	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4356.6;
	1254	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7857.3;
	1254	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1216.1;
	1255	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.8;
	1255	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.8;
	1255	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.7;
	1255	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3000.3;
	1255	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9243.4;
	1255	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5028.9;
	1255	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8637.8;
	1255	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1410;
	1256	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.4;
	1256	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776.8;
	1256	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889.1;
	1256	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3115.7;
	1256	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9811;
	1256	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5268.2;
	1256	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8827.8;
	1256	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1493.1;
	1257	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.9;
	1257	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777.6;
	1257	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	883.7;
	1257	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3132.8;
	1257	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9767.3;
	1257	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5259.3;
	1257	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8909.5;
	1257	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1467.6;
	1258	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.7;
	1258	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772.8;
	1258	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.9;
	1258	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3248.7;
	1258	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9793.8;
	1258	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5335.1;
	1258	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9085.2;
	1258	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1588.3;
	1259	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.4;
	1259	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	768.3;
	1259	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884.5;
	1259	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3369.4;
	1259	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9809.2;
	1259	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5390;
	1259	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9252.9;
	1259	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1589.4;
	1260	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.6;
	1260	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755.2;
	1260	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.1;
	1260	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3497.8;
	1260	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9760.3;
	1260	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5401.9;
	1260	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9324.6;
	1260	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1589.6;
	1261	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.9;
	1261	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.6;
	1261	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.7;
	1261	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3631.8;
	1261	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9674.6;
	1261	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5419.1;
	1261	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9338.3;
	1261	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1572.1;
	1262	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.1;
	1262	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735.9;
	1262	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878.3;
	1262	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3737.9;
	1262	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9605.3;
	1262	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5472;
	1262	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9336.3;
	1262	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1573.6;
	1263	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.3;
	1263	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.7;
	1263	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.8;
	1263	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3859.2;
	1263	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9463.7;
	1263	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5498.8;
	1263	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9353.9;
	1263	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1581.6;
	1264	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.2;
	1264	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	708.9;
	1264	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.7;
	1264	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3937;
	1264	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9352.3;
	1264	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5525.3;
	1264	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9361.8;
	1264	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1552.1;
	1265	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.6;
	1265	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	713.7;
	1265	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	844.4;
	1265	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3967.2;
	1265	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9399.8;
	1265	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5537.9;
	1265	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9304.1;
	1265	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1577.5;
	1266	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.2;
	1266	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.6;
	1266	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	844.3;
	1266	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3840.7;
	1266	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9620.7;
	1266	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5599.9;
	1266	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9303;
	1266	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1585.9;
	1267	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.5;
	1267	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	768.4;
	1267	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.3;
	1267	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3802.5;
	1267	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10258.8;
	1267	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5844.3;
	1267	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9558.6;
	1267	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1690.9;
	1268	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.2;
	1268	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	778.9;
	1268	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	916;
	1268	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3883.9;
	1268	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10429.7;
	1268	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6009.7;
	1268	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9650.6;
	1268	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1722.1;
	1269	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.4;
	1269	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	767.9;
	1269	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	898.6;
	1269	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3792.1;
	1269	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10226.5;
	1269	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5849.6;
	1269	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9444.7;
	1269	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1675.1;
	1270	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.2;
	1270	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744.1;
	1270	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.2;
	1270	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3636.1;
	1270	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9790.4;
	1270	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5500.2;
	1270	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9067.2;
	1270	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1517.1;
	1271	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.1;
	1271	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	699.7;
	1271	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	809.5;
	1271	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3347;
	1271	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9024.5;
	1271	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5003.2;
	1271	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8406.7;
	1271	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1402.2;
	1272	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855;
	1272	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	659;
	1272	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	755.9;
	1272	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2980.7;
	1272	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8228.6;
	1272	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4509.3;
	1272	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7747.4;
	1272	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1344.1;
	1273	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	847.7;
	1273	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	635.8;
	1273	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	737.2;
	1273	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2737;
	1273	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7610;
	1273	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4176.3;
	1273	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7362.8;
	1273	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1298.5;
	1274	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	851.3;
	1274	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	633.7;
	1274	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	731.9;
	1274	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2579.5;
	1274	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7387.7;
	1274	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3962.5;
	1274	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7167.7;
	1274	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1255.6;
	1275	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	858.5;
	1275	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	634.9;
	1275	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	731.2;
	1275	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2486;
	1275	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7292.1;
	1275	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3849.1;
	1275	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7045.2;
	1275	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1214.9;
	1276	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867;
	1276	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	639.7;
	1276	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	736.7;
	1276	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2445.6;
	1276	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7355.8;
	1276	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3818.4;
	1276	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7045.6;
	1276	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1204.3;
	1277	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.8;
	1277	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	656.5;
	1277	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	752.5;
	1277	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2460.2;
	1277	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7647.1;
	1277	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3901;
	1277	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7168.1;
	1277	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1270.1;
	1278	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.2;
	1278	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	708.2;
	1278	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	802.6;
	1278	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2609.2;
	1278	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8449.4;
	1278	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4268.6;
	1278	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7635.8;
	1278	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1370.2;
	1279	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.1;
	1279	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	791.7;
	1279	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	895.3;
	1279	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3003.5;
	1279	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9865.2;
	1279	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4981.8;
	1279	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8374.8;
	1279	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1575.4;
	1280	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.4;
	1280	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	835.4;
	1280	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	944.4;
	1280	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3099.9;
	1280	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10491.4;
	1280	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5204.7;
	1280	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8656.3;
	1280	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1630.6;
	1281	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.2;
	1281	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	841.4;
	1281	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	945.7;
	1281	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3101.5;
	1281	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10469.3;
	1281	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5169.2;
	1281	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8772.8;
	1281	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1655.6;
	1282	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.1;
	1282	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	855.5;
	1282	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	977.6;
	1282	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3217.3;
	1282	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10669.4;
	1282	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5248.5;
	1282	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8964.2;
	1282	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1683.4;
	1283	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.7;
	1283	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	872.8;
	1283	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	995.1;
	1283	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3406.6;
	1283	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10872.5;
	1283	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5324.3;
	1283	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9122.9;
	1283	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1705.4;
	1284	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	976.4;
	1284	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	881.9;
	1284	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1001.4;
	1284	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3598.6;
	1284	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11018.9;
	1284	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5346.3;
	1284	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9376.7;
	1284	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1696.5;
	1285	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	977.8;
	1285	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	887.8;
	1285	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	993.5;
	1285	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3788.4;
	1285	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11161.4;
	1285	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5292.1;
	1285	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9548.1;
	1285	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1674.3;
	1286	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.7;
	1286	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	886.3;
	1286	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	987.8;
	1286	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3899.8;
	1286	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11346;
	1286	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5278.9;
	1286	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9555.4;
	1286	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1628.5;
	1287	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.5;
	1287	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	879.2;
	1287	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	986.4;
	1287	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3949.8;
	1287	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11462.5;
	1287	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5272;
	1287	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9423.9;
	1287	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1640.4;
	1288	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.9;
	1288	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	896.1;
	1288	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	994.8;
	1288	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3975.6;
	1288	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11667;
	1288	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5298.7;
	1288	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9348.1;
	1288	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1675.7;
	1289	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.1;
	1289	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	913.8;
	1289	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1017.4;
	1289	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3924.6;
	1289	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12034.7;
	1289	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5394.6;
	1289	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9095.1;
	1289	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1741.2;
	1290	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.4;
	1290	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	930.4;
	1290	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1031.3;
	1290	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3747.9;
	1290	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12585.3;
	1290	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5644.2;
	1290	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8947.8;
	1290	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1802.5;
	1291	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	977.7;
	1291	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	957.1;
	1291	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1068.1;
	1291	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3656.5;
	1291	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13279.8;
	1291	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6139.1;
	1291	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9205.4;
	1291	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1926;
	1292	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1005.6;
	1292	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	952.6;
	1292	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1096;
	1292	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3728.7;
	1292	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13397.9;
	1292	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6478.5;
	1292	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9346.9;
	1292	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1972.3;
	1293	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1007.8;
	1293	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	931.5;
	1293	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1086;
	1293	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3620;
	1293	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13159.6;
	1293	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6509.7;
	1293	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9307.4;
	1293	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1964.5;
	1294	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.6;
	1294	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	894;
	1294	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1050.1;
	1294	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3449.8;
	1294	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12533.2;
	1294	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6247.4;
	1294	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9068.5;
	1294	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1899.3;
	1295	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.1;
	1295	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	839.8;
	1295	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	993.4;
	1295	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3181.6;
	1295	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11528.3;
	1295	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5800;
	1295	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8590.5;
	1295	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1718.4;
	1296	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.2;
	1296	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	805.4;
	1296	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	943;
	1296	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2883.6;
	1296	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10603.6;
	1296	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5367.8;
	1296	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8127.3;
	1296	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1602.1;
	1297	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.1;
	1297	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777.2;
	1297	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	912.9;
	1297	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2684;
	1297	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9975.4;
	1297	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5074.1;
	1297	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7835.4;
	1297	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1596.7;
	1298	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.7;
	1298	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	770;
	1298	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	905.2;
	1298	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2572.6;
	1298	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9734.1;
	1298	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4935.9;
	1298	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7685.9;
	1298	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1603.4;
	1299	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.4;
	1299	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	770;
	1299	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	912.7;
	1299	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2518.8;
	1299	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9684.1;
	1299	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4881.8;
	1299	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7633.6;
	1299	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1576.7;
	1300	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.6;
	1300	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	782.9;
	1300	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	922.4;
	1300	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2518.2;
	1300	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9778;
	1300	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4913.6;
	1300	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7644.4;
	1300	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1587.5;
	1301	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.8;
	1301	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	801.3;
	1301	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	945.9;
	1301	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2600.5;
	1301	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10244.5;
	1301	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5142.2;
	1301	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7884.2;
	1301	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1612.6;
	1302	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.7;
	1302	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	857.3;
	1302	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1009.6;
	1302	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2858.6;
	1302	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11278.6;
	1302	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5765.4;
	1302	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8603.1;
	1302	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1660.6;
	1303	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009.3;
	1303	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	941.8;
	1303	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1121.3;
	1303	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3401.5;
	1303	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12800.7;
	1303	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6669.5;
	1303	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9643.6;
	1303	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1880.9;
	1304	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1021.2;
	1304	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	958.8;
	1304	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1152.8;
	1304	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3545.7;
	1304	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13050.3;
	1304	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6781.4;
	1304	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9743.2;
	1304	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1908.6;
	1305	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990.7;
	1305	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	925.4;
	1305	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1090.1;
	1305	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3415.1;
	1305	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12354.3;
	1305	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6423;
	1305	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9614.1;
	1305	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1830.8;
	1306	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	973;
	1306	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	897.7;
	1306	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1030;
	1306	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3351.3;
	1306	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11873.7;
	1306	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6185.2;
	1306	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9556.8;
	1306	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1833.3;
	1307	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.5;
	1307	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	868.6;
	1307	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	996;
	1307	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3294.1;
	1307	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11488.4;
	1307	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5994.7;
	1307	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9474.9;
	1307	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1806.1;
	1308	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936;
	1308	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	839.6;
	1308	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	942.5;
	1308	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3246.8;
	1308	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11087.2;
	1308	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5774.1;
	1308	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9398.7;
	1308	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1721.3;
	1309	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.3;
	1309	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	809.8;
	1309	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	902;
	1309	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3193.4;
	1309	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10654.7;
	1309	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5566.8;
	1309	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9278.9;
	1309	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1678.1;
	1310	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907;
	1310	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	786.2;
	1310	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	862.6;
	1310	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3149.9;
	1310	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10345.6;
	1310	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5405.1;
	1310	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9156.5;
	1310	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1645.7;
	1311	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.5;
	1311	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	767.6;
	1311	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	844.6;
	1311	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3123.3;
	1311	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9956.6;
	1311	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5279.6;
	1311	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9107.9;
	1311	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1603.7;
	1312	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.6;
	1312	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.8;
	1312	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	837.8;
	1312	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3113.7;
	1312	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9675.9;
	1312	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5187.7;
	1312	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9050.1;
	1312	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1555.9;
	1313	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.2;
	1313	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.2;
	1313	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	837.1;
	1313	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3124.1;
	1313	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9622.8;
	1313	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5237;
	1313	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8983.8;
	1313	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1584.3;
	1314	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.1;
	1314	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.7;
	1314	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	837.2;
	1314	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3121.1;
	1314	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9791.9;
	1314	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5357.8;
	1314	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8988.4;
	1314	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1610.7;
	1315	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.4;
	1315	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	785.7;
	1315	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884.9;
	1315	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3244.7;
	1315	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10508.7;
	1315	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5700.7;
	1315	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9312.3;
	1315	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1757.4;
	1316	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.1;
	1316	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	824.9;
	1316	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	947.5;
	1316	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3445.9;
	1316	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11025.2;
	1316	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6065.2;
	1316	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9561.9;
	1316	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1836.1;
	1317	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948.9;
	1317	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	831.7;
	1317	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	964.2;
	1317	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3481.5;
	1317	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11088.5;
	1317	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6085;
	1317	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9524.5;
	1317	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1844.8;
	1318	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.9;
	1318	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	807.5;
	1318	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	951.6;
	1318	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3432.3;
	1318	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10850.7;
	1318	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5931.2;
	1318	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9292.2;
	1318	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1823.1;
	1319	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925.5;
	1319	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	768.2;
	1319	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	915.5;
	1319	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3267.9;
	1319	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10192;
	1319	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5589.1;
	1319	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8833.2;
	1319	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1725.7;
	1320	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.5;
	1320	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.9;
	1320	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.6;
	1320	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3017.9;
	1320	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9557.4;
	1320	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5255.6;
	1320	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8378.1;
	1320	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1605.4;
	1321	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.5;
	1321	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	721.4;
	1321	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.1;
	1321	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2873.5;
	1321	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9199.8;
	1321	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5069.7;
	1321	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8118.8;
	1321	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1579.3;
	1322	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.1;
	1322	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	715.7;
	1322	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	872;
	1322	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2796;
	1322	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9065.5;
	1322	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5024.7;
	1322	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8023;
	1322	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1572.8;
	1323	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.7;
	1323	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.3;
	1323	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	883.8;
	1323	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2790;
	1323	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9071.4;
	1323	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5058.3;
	1323	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8007.7;
	1323	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1558.6;
	1324	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901;
	1324	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.7;
	1324	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	900.2;
	1324	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2839.3;
	1324	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9188;
	1324	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5178.1;
	1324	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8084.1;
	1324	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1597.4;
	1325	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.1;
	1325	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.9;
	1325	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	934.7;
	1325	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2938.2;
	1325	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9593.8;
	1325	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5457.6;
	1325	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8361.2;
	1325	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1640.1;
	1326	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.5;
	1326	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	802.7;
	1326	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1003.1;
	1326	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3204;
	1326	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10599.2;
	1326	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6113.4;
	1326	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9092.7;
	1326	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1786.1;
	1327	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.1;
	1327	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	886.9;
	1327	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1119.2;
	1327	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3784.2;
	1327	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12091.5;
	1327	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7055.7;
	1327	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10030.4;
	1327	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2005.2;
	1328	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	994.6;
	1328	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	903.9;
	1328	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1148.8;
	1328	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3866.7;
	1328	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12319.2;
	1328	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7139.8;
	1328	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10115.3;
	1328	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2007.7;
	1329	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	974.1;
	1329	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	875.8;
	1329	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1077.1;
	1329	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3581.5;
	1329	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11620.5;
	1329	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6621.2;
	1329	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9858.9;
	1329	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1840.9;
	1330	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.8;
	1330	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	852.7;
	1330	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1014.2;
	1330	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3386;
	1330	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11147.3;
	1330	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6132.7;
	1330	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9630.4;
	1330	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1810.3;
	1331	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.4;
	1331	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	833.6;
	1331	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	962.2;
	1331	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3234.1;
	1331	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10804.6;
	1331	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5795.8;
	1331	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9529.6;
	1331	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1734.8;
	1332	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.3;
	1332	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	809.5;
	1332	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	914.2;
	1332	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3163.1;
	1332	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10470.6;
	1332	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5533.2;
	1332	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9361.5;
	1332	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1641.8;
	1333	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.7;
	1333	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	791.1;
	1333	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	873.6;
	1333	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3116.4;
	1333	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10177.4;
	1333	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5371.5;
	1333	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9251.3;
	1333	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1600.2;
	1334	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.2;
	1334	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	778.1;
	1334	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	855.4;
	1334	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3144.2;
	1334	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9934.2;
	1334	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5305.8;
	1334	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9245.8;
	1334	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1568.5;
	1335	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.8;
	1335	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	761.6;
	1335	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	840.6;
	1335	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3172.6;
	1335	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9693.9;
	1335	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5252.2;
	1335	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9158.2;
	1335	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1549.9;
	1336	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.7;
	1336	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	750.8;
	1336	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	831.2;
	1336	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3215.5;
	1336	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9540.6;
	1336	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5263.6;
	1336	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9067.1;
	1336	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1523.4;
	1337	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.2;
	1337	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753.2;
	1337	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	831.3;
	1337	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3244.2;
	1337	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9560.7;
	1337	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5293;
	1337	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9012.6;
	1337	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1523;
	1338	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.3;
	1338	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	771.4;
	1338	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.3;
	1338	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3203.1;
	1338	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9823.9;
	1338	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5354.8;
	1338	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8944.2;
	1338	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1547.1;
	1339	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.6;
	1339	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	833.7;
	1339	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	890.2;
	1339	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3247;
	1339	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10650.5;
	1339	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5640.9;
	1339	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9136.2;
	1339	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1693.6;
	1340	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.7;
	1340	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	871;
	1340	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	944.3;
	1340	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3408.9;
	1340	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11230.7;
	1340	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5946.1;
	1340	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9358.4;
	1340	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1776.8;
	1341	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.5;
	1341	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	876.4;
	1341	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	951.4;
	1341	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3376.9;
	1341	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11294.8;
	1341	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5903.4;
	1341	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9196.8;
	1341	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1790.2;
	1342	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.7;
	1342	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	854;
	1342	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	935.9;
	1342	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3272.3;
	1342	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11028.6;
	1342	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5692.5;
	1342	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8888.9;
	1342	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1768.9;
	1343	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.4;
	1343	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	813.4;
	1343	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	898.6;
	1343	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3063;
	1343	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10449.3;
	1343	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5335.4;
	1343	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8387.4;
	1343	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1678.1;
	1344	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.9;
	1344	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	787.8;
	1344	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.9;
	1344	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2809.5;
	1344	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9869.1;
	1344	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4951.5;
	1344	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7900.2;
	1344	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1582.5;
	1345	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.5;
	1345	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	774.8;
	1345	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	848.9;
	1345	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2629.3;
	1345	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9563.8;
	1345	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4752.2;
	1345	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7620.7;
	1345	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1571.7;
	1346	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903;
	1346	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	779.1;
	1346	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.2;
	1346	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2557.2;
	1346	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9534.4;
	1346	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4691.8;
	1346	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7537;
	1346	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1560.5;
	1347	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.5;
	1347	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	787.9;
	1347	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.8;
	1347	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2539.2;
	1347	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9677.1;
	1347	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4709.1;
	1347	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7523.1;
	1347	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1567.1;
	1348	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.3;
	1348	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	805.2;
	1348	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	886.7;
	1348	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2559.6;
	1348	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9957.7;
	1348	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4819.1;
	1348	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7626;
	1348	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1608;
	1349	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.6;
	1349	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	834.8;
	1349	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	924.6;
	1349	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2648.3;
	1349	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10543.3;
	1349	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5081.5;
	1349	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7949.2;
	1349	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1606.1;
	1350	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.6;
	1350	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	894.2;
	1350	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	993.2;
	1350	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2900.3;
	1350	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11674.4;
	1350	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5685.5;
	1350	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8713.5;
	1350	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1755.9;
	1351	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.2;
	1351	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	983.2;
	1351	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1105;
	1351	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3400.4;
	1351	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13218.5;
	1351	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6606.1;
	1351	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9664.9;
	1351	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2003.3;
	1352	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1010.6;
	1352	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	997;
	1352	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1143.1;
	1352	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3501.6;
	1352	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13477.1;
	1352	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6783.2;
	1352	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9818;
	1352	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2019.3;
	1353	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985.4;
	1353	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	938;
	1353	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1081.1;
	1353	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3366.5;
	1353	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12475.5;
	1353	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6445.7;
	1353	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9724.1;
	1353	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1883.2;
	1354	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.6;
	1354	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	892.1;
	1354	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1019.4;
	1354	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3288.7;
	1354	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11664.6;
	1354	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6100.9;
	1354	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9617.6;
	1354	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1763;
	1355	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948.8;
	1355	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	851.5;
	1355	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	971.6;
	1355	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3225.5;
	1355	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11059.1;
	1355	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5857.4;
	1355	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9509.2;
	1355	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1660.6;
	1356	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.1;
	1356	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	815.9;
	1356	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	918.2;
	1356	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3167.1;
	1356	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10504.4;
	1356	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5623.2;
	1356	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9356.1;
	1356	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1571.5;
	1357	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.5;
	1357	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	782.8;
	1357	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	874.8;
	1357	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3119.5;
	1357	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10077.2;
	1357	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5416.8;
	1357	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9164.4;
	1357	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1508;
	1358	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.7;
	1358	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	759;
	1358	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	850.8;
	1358	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3090;
	1358	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9769.1;
	1358	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5263.6;
	1358	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9082.5;
	1358	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1466.6;
	1359	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.2;
	1359	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.1;
	1359	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	829.9;
	1359	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3095.5;
	1359	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9488;
	1359	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5158.8;
	1359	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8994.3;
	1359	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1428.8;
	1360	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.4;
	1360	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.9;
	1360	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	816.6;
	1360	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3123;
	1360	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9271.6;
	1360	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5141.1;
	1360	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8913.5;
	1360	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1393.2;
	1361	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.1;
	1361	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	708.8;
	1361	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	816.3;
	1361	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3107;
	1361	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9212.6;
	1361	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5140.1;
	1361	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8802.6;
	1361	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1389.9;
	1362	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.5;
	1362	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	710.4;
	1362	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	815.4;
	1362	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3039.9;
	1362	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9309;
	1362	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5185.4;
	1362	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8735.8;
	1362	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1399.6;
	1363	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.7;
	1363	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753.4;
	1363	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	839.9;
	1363	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3084.5;
	1363	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9844.4;
	1363	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5427.9;
	1363	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8970.2;
	1363	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1512.2;
	1364	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.6;
	1364	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	782.2;
	1364	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	877.2;
	1364	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3210;
	1364	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10194.4;
	1364	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5679.1;
	1364	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9162.4;
	1364	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1583;
	1365	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.4;
	1365	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	778.9;
	1365	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	881.6;
	1365	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3151;
	1365	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10145.2;
	1365	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5630.8;
	1365	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9057.3;
	1365	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1592.8;
	1366	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.4;
	1366	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	771.8;
	1366	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.8;
	1366	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3074;
	1366	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9998.4;
	1366	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5509.1;
	1366	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8854;
	1366	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1588.2;
	1367	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.9;
	1367	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.7;
	1367	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	842.4;
	1367	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2955.7;
	1367	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9612.3;
	1367	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5308.3;
	1367	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8554.9;
	1367	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1537.3;
	1368	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.9;
	1368	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.4;
	1368	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	815.6;
	1368	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2783.8;
	1368	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9159.6;
	1368	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5043.4;
	1368	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8201.2;
	1368	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1488.1;
	1369	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.8;
	1369	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	709.7;
	1369	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	848.8;
	1369	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2634.3;
	1369	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8752.3;
	1369	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4852.8;
	1369	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7918.3;
	1369	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1573.6;
	1370	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.4;
	1370	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	700.2;
	1370	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.6;
	1370	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2531.6;
	1370	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8595.8;
	1370	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4735.2;
	1370	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7781.2;
	1370	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1563.2;
	1371	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.6;
	1371	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	701.4;
	1371	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.9;
	1371	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2481.4;
	1371	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8575.7;
	1371	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4723.4;
	1371	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7731.9;
	1371	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1554.7;
	1372	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.8;
	1372	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	709.5;
	1372	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.4;
	1372	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2475.1;
	1372	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8646.7;
	1372	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4765.9;
	1372	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7761.7;
	1372	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1579.6;
	1373	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.5;
	1373	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719.3;
	1373	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	888.4;
	1373	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2498.3;
	1373	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8844.2;
	1373	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4883.5;
	1373	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7896.3;
	1373	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1617.8;
	1374	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.3;
	1374	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746.3;
	1374	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	921.3;
	1374	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2590.2;
	1374	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9293.7;
	1374	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5141.9;
	1374	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8169.5;
	1374	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1664.2;
	1375	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.2;
	1375	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	784;
	1375	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	967;
	1375	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2734.7;
	1375	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9938.7;
	1375	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5484.7;
	1375	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8548.9;
	1375	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1767.7;
	1376	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925.8;
	1376	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	803.4;
	1376	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1007.9;
	1376	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2825.6;
	1376	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10352.2;
	1376	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5709.4;
	1376	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8739;
	1376	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1809.3;
	1377	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920.9;
	1377	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	795;
	1377	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1010.1;
	1377	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2928;
	1377	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10335.4;
	1377	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5760;
	1377	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8837.8;
	1377	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1749.7;
	1378	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.7;
	1378	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772.3;
	1378	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	976.7;
	1378	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2968.8;
	1378	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10047.6;
	1378	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5575.7;
	1378	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8772.8;
	1378	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1668.9;
	1379	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894;
	1379	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.3;
	1379	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	930.8;
	1379	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2977.9;
	1379	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9687.3;
	1379	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5345;
	1379	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8691.5;
	1379	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1556.5;
	1380	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.3;
	1380	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	710.6;
	1380	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.7;
	1380	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2995.1;
	1380	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9327.4;
	1380	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5180.8;
	1380	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8593;
	1380	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1454.7;
	1381	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.1;
	1381	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	689.1;
	1381	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	864.5;
	1381	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2996.2;
	1381	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9010.7;
	1381	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5041.8;
	1381	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8502.6;
	1381	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1406.8;
	1382	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	865.6;
	1382	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	674.9;
	1382	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	846.8;
	1382	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2994.8;
	1382	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8760.9;
	1382	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4903.1;
	1382	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8443.7;
	1382	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1333.7;
	1383	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.9;
	1383	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	667;
	1383	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	836.1;
	1383	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2996;
	1383	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8624.3;
	1383	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4837.7;
	1383	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8433.3;
	1383	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1314.1;
	1384	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	863.2;
	1384	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	665.5;
	1384	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	832.5;
	1384	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2993.7;
	1384	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8560.5;
	1384	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4880;
	1384	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8437.2;
	1384	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1348.9;
	1385	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	865.7;
	1385	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	669.1;
	1385	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	826.3;
	1385	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2991.6;
	1385	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8575.4;
	1385	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4928.1;
	1385	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8422.7;
	1385	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1402.8;
	1386	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866.5;
	1386	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	671.5;
	1386	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	807.6;
	1386	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2974.4;
	1386	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8645.4;
	1386	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4981.6;
	1386	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8388.2;
	1386	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1412.4;
	1387	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866.7;
	1387	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	693.5;
	1387	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	827.9;
	1387	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3059.9;
	1387	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8994.5;
	1387	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5152;
	1387	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8609.3;
	1387	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1451.6;
	1388	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.2;
	1388	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714.7;
	1388	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.8;
	1388	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3178.3;
	1388	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9293.3;
	1388	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5367.5;
	1388	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8795.9;
	1388	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1503.3;
	1389	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.2;
	1389	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711.1;
	1389	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851.3;
	1389	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3123.8;
	1389	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9131.4;
	1389	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5265.6;
	1389	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8685.2;
	1389	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1504.3;
	1390	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.6;
	1390	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	694.9;
	1390	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	832.5;
	1390	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3042.8;
	1390	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8874.2;
	1390	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5107.3;
	1390	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8508.5;
	1390	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1487.2;
	1391	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	876.2;
	1391	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	671.8;
	1391	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	795.6;
	1391	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2921;
	1391	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8437.7;
	1391	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4847.1;
	1391	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8219.2;
	1391	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1435.4;
	1392	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	862.6;
	1392	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	643.9;
	1392	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	756.9;
	1392	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2754.2;
	1392	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7919.5;
	1392	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4535.6;
	1392	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7887.3;
	1392	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1334.7;
	1393	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	843.5;
	1393	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	630.4;
	1393	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	733.4;
	1393	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2596.3;
	1393	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7504.5;
	1393	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4264.7;
	1393	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7581.4;
	1393	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1298.6;
	1394	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	839.1;
	1394	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	621.1;
	1394	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	721;
	1394	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2480.2;
	1394	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7224.7;
	1394	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4101.5;
	1394	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7441.1;
	1394	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1274.2;
	1395	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	835.8;
	1395	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	614.7;
	1395	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	717.1;
	1395	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2402.2;
	1395	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7079.1;
	1395	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4008.2;
	1395	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7391.3;
	1395	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1252.6;
	1396	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	836.8;
	1396	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	615.1;
	1396	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	715.8;
	1396	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2360;
	1396	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7045.7;
	1396	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3961.8;
	1396	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7354;
	1396	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1260.7;
	1397	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	840.8;
	1397	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	628;
	1397	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	720.1;
	1397	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2345.5;
	1397	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7103.1;
	1397	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3984.9;
	1397	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7357.8;
	1397	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1301.1;
	1398	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	846;
	1398	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	641;
	1398	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	735.3;
	1398	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2372.9;
	1398	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7331.8;
	1398	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4073.5;
	1398	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7462.1;
	1398	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1360.9;
	1399	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	854.8;
	1399	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	667.8;
	1399	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	760.6;
	1399	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2438.2;
	1399	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7691.6;
	1399	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4264;
	1399	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7642.5;
	1399	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1335.3;
	1400	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.7;
	1400	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	690.2;
	1400	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	787.2;
	1400	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2464.6;
	1400	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8038.8;
	1400	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4427.5;
	1400	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7656.4;
	1400	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1383.1;
	1401	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	854.5;
	1401	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717;
	1401	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	813.4;
	1401	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2637.4;
	1401	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8489.9;
	1401	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4698.4;
	1401	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7870.3;
	1401	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1474.7;
	1402	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	858.4;
	1402	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.3;
	1402	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	819.1;
	1402	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2829.1;
	1402	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8816.5;
	1402	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4849.6;
	1402	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8073.9;
	1402	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1463.8;
	1403	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	857.1;
	1403	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711.6;
	1403	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	810.4;
	1403	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2942.7;
	1403	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8870.2;
	1403	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4929.3;
	1403	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8193.3;
	1403	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1450.2;
	1404	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	856.6;
	1404	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	697.3;
	1404	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	799.6;
	1404	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3023.4;
	1404	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8767.1;
	1404	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4996.6;
	1404	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8262.9;
	1404	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1435;
	1405	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	854.7;
	1405	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	692.9;
	1405	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	798.9;
	1405	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3099.8;
	1405	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8695.6;
	1405	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5020.2;
	1405	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8355.3;
	1405	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1417.6;
	1406	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	856;
	1406	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	684.2;
	1406	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	796.8;
	1406	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3155.7;
	1406	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8619.7;
	1406	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5009.8;
	1406	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8412.3;
	1406	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1410.2;
	1407	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	860.2;
	1407	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	672.9;
	1407	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	798.2;
	1407	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3202.2;
	1407	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8549.3;
	1407	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5045.7;
	1407	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8455.4;
	1407	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1392.3;
	1408	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	859.1;
	1408	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	676;
	1408	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	798.4;
	1408	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3248;
	1408	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8543.6;
	1408	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5100.5;
	1408	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8473.9;
	1408	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1388;
	1409	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	861.7;
	1409	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	685.5;
	1409	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	810;
	1409	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3276.4;
	1409	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8639.2;
	1409	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5201.6;
	1409	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8475.9;
	1409	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1416;
	1410	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	863.4;
	1410	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	694.1;
	1410	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	820;
	1410	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3274.6;
	1410	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8829.5;
	1410	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5342.2;
	1410	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8590.9;
	1410	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1425.4;
	1411	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	861.1;
	1411	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.4;
	1411	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	841.1;
	1411	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3337.9;
	1411	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9312.2;
	1411	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5595.5;
	1411	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8978.4;
	1411	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1532.8;
	1412	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.5;
	1412	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.3;
	1412	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	877.5;
	1412	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3508.2;
	1412	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9734.1;
	1412	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5873.1;
	1412	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9219.7;
	1412	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1613.7;
	1413	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.8;
	1413	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737;
	1413	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.8;
	1413	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3456.8;
	1413	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9518.3;
	1413	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5714.5;
	1413	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9071.9;
	1413	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1583.4;
	1414	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.7;
	1414	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	713.8;
	1414	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	827.5;
	1414	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3344;
	1414	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9079.1;
	1414	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5384.6;
	1414	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8763.9;
	1414	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1520.5;
	1415	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	851.3;
	1415	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	671;
	1415	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	779.1;
	1415	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3121.4;
	1415	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8374.4;
	1415	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4979.8;
	1415	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8276.2;
	1415	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1413.9;
	1416	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	824.8;
	1416	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	630.3;
	1416	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	728.7;
	1416	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2833.4;
	1416	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7610.1;
	1416	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4524.2;
	1416	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7717;
	1416	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1307.2;
	1417	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	812;
	1417	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	604.7;
	1417	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	740.7;
	1417	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2581.1;
	1417	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7083.5;
	1417	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4177.3;
	1417	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7318.1;
	1417	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1227.4;
	1418	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	809.4;
	1418	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598;
	1418	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	730.9;
	1418	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2431;
	1418	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6828.2;
	1418	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3961.3;
	1418	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7107.4;
	1418	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1153.1;
	1419	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	810.8;
	1419	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	582.4;
	1419	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	726.1;
	1419	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2362.5;
	1419	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6733.4;
	1419	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3862.5;
	1419	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7029.4;
	1419	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1188.1;
	1420	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	814.1;
	1420	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	590.1;
	1420	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	732.2;
	1420	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2332.9;
	1420	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6799.8;
	1420	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3845.9;
	1420	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7031.9;
	1420	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1168.9;
	1421	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	823.4;
	1421	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	613.9;
	1421	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	752.7;
	1421	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2366.6;
	1421	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7118.5;
	1421	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3942.2;
	1421	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7246.2;
	1421	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1196.1;
	1422	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	843.7;
	1422	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	659.1;
	1422	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	803.1;
	1422	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2537.2;
	1422	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7932.5;
	1422	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4361.2;
	1422	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7809.1;
	1422	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1319;
	1423	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.2;
	1423	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	745;
	1423	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	899.8;
	1423	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2924.1;
	1423	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9292.7;
	1423	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5052.3;
	1423	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8581.8;
	1423	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1507.2;
	1424	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.8;
	1424	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776.7;
	1424	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	938.6;
	1424	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3031.2;
	1424	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9737.6;
	1424	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5285.6;
	1424	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8744.9;
	1424	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1534.1;
	1425	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.1;
	1425	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	763;
	1425	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	921.2;
	1425	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3041.9;
	1425	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9606.8;
	1425	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5292.7;
	1425	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8934.8;
	1425	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1514.8;
	1426	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.4;
	1426	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752;
	1426	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	907.4;
	1426	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3142;
	1426	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9601.7;
	1426	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5373.2;
	1426	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9185.2;
	1426	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1591.5;
	1427	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.9;
	1427	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744.2;
	1427	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	900;
	1427	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3258.2;
	1427	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9653.5;
	1427	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5465.5;
	1427	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9440.2;
	1427	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1600;
	1428	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.6;
	1428	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.5;
	1428	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	894.9;
	1428	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3385.8;
	1428	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9659.6;
	1428	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5537.2;
	1428	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9670.4;
	1428	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1614.2;
	1429	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.7;
	1429	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730;
	1429	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	894.8;
	1429	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3504.4;
	1429	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9695.7;
	1429	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5602.6;
	1429	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9827.5;
	1429	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1609.1;
	1430	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.3;
	1430	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.9;
	1430	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	900.9;
	1430	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3637.5;
	1430	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9767.3;
	1430	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5686;
	1430	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10048.4;
	1430	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1621.1;
	1431	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.2;
	1431	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	731.4;
	1431	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	905.8;
	1431	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3767.4;
	1431	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9835.8;
	1431	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5763.1;
	1431	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10145.7;
	1431	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1632.5;
	1432	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.3;
	1432	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.8;
	1432	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	910.2;
	1432	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3855.2;
	1432	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9838.5;
	1432	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5838.9;
	1432	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10171.9;
	1432	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1634.7;
	1433	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.7;
	1433	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.4;
	1433	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	919.1;
	1433	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3878.6;
	1433	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9837.7;
	1433	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5920.7;
	1433	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10126.7;
	1433	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1634.3;
	1434	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.2;
	1434	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.8;
	1434	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913.3;
	1434	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3776.5;
	1434	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9875.7;
	1434	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5975.6;
	1434	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9994.1;
	1434	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1618.2;
	1435	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.8;
	1435	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	750.8;
	1435	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	931;
	1435	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3740.6;
	1435	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10214.3;
	1435	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6151.8;
	1435	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10058.3;
	1435	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1695.9;
	1436	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.2;
	1436	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	773.4;
	1436	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	957.3;
	1436	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3871.1;
	1436	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10508.5;
	1436	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6350.3;
	1436	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10159.9;
	1436	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1725.1;
	1437	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.1;
	1437	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	757.2;
	1437	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	936.1;
	1437	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3790.6;
	1437	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10272.9;
	1437	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6184.4;
	1437	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9887.5;
	1437	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1674.2;
	1438	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.6;
	1438	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	722.3;
	1438	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	901.8;
	1438	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3628.4;
	1438	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9776.6;
	1438	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5796;
	1438	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9422.4;
	1438	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1609.8;
	1439	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	863.3;
	1439	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	672.6;
	1439	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	844.8;
	1439	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3349.2;
	1439	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8925.9;
	1439	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5245.3;
	1439	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8743.6;
	1439	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1470.8;
	1440	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	829.8;
	1440	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	628.8;
	1440	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	791.3;
	1440	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3000.3;
	1440	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8050.8;
	1440	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4697.4;
	1440	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8072.8;
	1440	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1355.7;
	1441	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	807.9;
	1441	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	615.9;
	1441	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	713.4;
	1441	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2708.4;
	1441	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7475.5;
	1441	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4366.8;
	1441	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7578.3;
	1441	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1273.7;
	1442	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	802.4;
	1442	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	602.3;
	1442	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	696.5;
	1442	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2557.5;
	1442	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7146.1;
	1442	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4131.6;
	1442	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7309.9;
	1442	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1228;
	1443	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	803.5;
	1443	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	593.6;
	1443	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	693.4;
	1443	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2485.9;
	1443	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6971.8;
	1443	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4001.8;
	1443	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7182.8;
	1443	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1171.5;
	1444	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	805.5;
	1444	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	591.2;
	1444	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	690.6;
	1444	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2450;
	1444	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6936.3;
	1444	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3957.7;
	1444	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7162.4;
	1444	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1197.8;
	1445	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	810.4;
	1445	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.1;
	1445	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	698.2;
	1445	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2466.9;
	1445	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7131.4;
	1445	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4024.8;
	1445	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7346.3;
	1445	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1211.1;
	1446	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	832.3;
	1446	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	628.8;
	1446	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	734.8;
	1446	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2626.2;
	1446	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7794.7;
	1446	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4372.2;
	1446	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7905.8;
	1446	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1305.5;
	1447	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.3;
	1447	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	698.1;
	1447	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	817.9;
	1447	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3043.2;
	1447	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9034.4;
	1447	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5067.5;
	1447	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8696.3;
	1447	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1472;
	1448	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.3;
	1448	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	722;
	1448	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	844.2;
	1448	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3129.8;
	1448	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9401.2;
	1448	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5293.8;
	1448	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8873;
	1448	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1452.2;
	1449	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.7;
	1449	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.4;
	1449	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	840.5;
	1449	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3129.3;
	1449	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9434.6;
	1449	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5298;
	1449	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9072.9;
	1449	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1492.7;
	1450	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882;
	1450	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.9;
	1450	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.1;
	1450	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3275.4;
	1450	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9619.7;
	1450	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5391.4;
	1450	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9347.5;
	1450	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1558;
	1451	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.6;
	1451	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.7;
	1451	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	855.6;
	1451	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3426;
	1451	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9723;
	1451	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5516.1;
	1451	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9539.5;
	1451	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1584.3;
	1452	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.3;
	1452	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	734.9;
	1452	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.3;
	1452	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3623;
	1452	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9758.4;
	1452	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5717.4;
	1452	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9708.8;
	1452	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1610.1;
	1453	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.4;
	1453	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.2;
	1453	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.8;
	1453	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3811.3;
	1453	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9737.4;
	1453	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5916.4;
	1453	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9808.2;
	1453	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1620.7;
	1454	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.9;
	1454	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.7;
	1454	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.1;
	1454	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4016.2;
	1454	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9753.2;
	1454	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6110;
	1454	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9981.7;
	1454	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1601.7;
	1455	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.5;
	1455	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	721.2;
	1455	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.2;
	1455	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4180.4;
	1455	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9693.2;
	1455	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6231.2;
	1455	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9987.8;
	1455	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1615;
	1456	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878;
	1456	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720.6;
	1456	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.2;
	1456	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4279.6;
	1456	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9607.9;
	1456	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6211.5;
	1456	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9914;
	1456	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1611.8;
	1457	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880;
	1457	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	724.9;
	1457	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	853.4;
	1457	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4315.9;
	1457	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9603.2;
	1457	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6180.3;
	1457	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9930.3;
	1457	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1603.7;
	1458	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.9;
	1458	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.5;
	1458	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	848.7;
	1458	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4190;
	1458	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9647.1;
	1458	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6134.5;
	1458	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9931;
	1458	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1502.5;
	1459	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.7;
	1459	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	750.5;
	1459	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	862.2;
	1459	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4075.4;
	1459	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9901.1;
	1459	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6143.8;
	1459	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9994.1;
	1459	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1609.2;
	1460	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.5;
	1460	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	767.4;
	1460	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	895.1;
	1460	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4114.7;
	1460	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10234.5;
	1460	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6307.5;
	1460	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10089.9;
	1460	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1683.4;
	1461	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.7;
	1461	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	762.3;
	1461	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	887.5;
	1461	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3987.6;
	1461	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10099.7;
	1461	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6105.9;
	1461	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9936.4;
	1461	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1601.7;
	1462	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.4;
	1462	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	739.9;
	1462	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.6;
	1462	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3792.1;
	1462	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9663.7;
	1462	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5702.8;
	1462	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9523.4;
	1462	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1538.9;
	1463	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	869.5;
	1463	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	700.8;
	1463	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	809.5;
	1463	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3481;
	1463	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8887.6;
	1463	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5126.4;
	1463	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8848;
	1463	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1378.5;
	1464	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	847;
	1464	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	665.9;
	1464	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	759.3;
	1464	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3102.8;
	1464	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8078.5;
	1464	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4571.2;
	1464	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8153;
	1464	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1342.4;
	1465	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	834.2;
	1465	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	655.6;
	1465	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	737.5;
	1465	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2819;
	1465	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7670.9;
	1465	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4204.2;
	1465	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7628.2;
	1465	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1307.8;
	1466	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	831.2;
	1466	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	648.1;
	1466	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	729;
	1466	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2649.3;
	1466	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7472.1;
	1466	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3980.7;
	1466	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7312.9;
	1466	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1298;
	1467	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	835;
	1467	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	653.9;
	1467	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	732.4;
	1467	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2544.5;
	1467	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7461.8;
	1467	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3906.5;
	1467	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7134.4;
	1467	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1302.4;
	1468	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	842.5;
	1468	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	664.1;
	1468	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	741.7;
	1468	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2501.4;
	1468	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7588.9;
	1468	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3883.5;
	1468	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7086.9;
	1468	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1297.5;
	1469	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	852.7;
	1469	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	687.8;
	1469	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	766.8;
	1469	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2510.7;
	1469	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8030.9;
	1469	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4018.4;
	1469	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7228;
	1469	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1345.3;
	1470	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.1;
	1470	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.6;
	1470	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824.2;
	1470	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2660.3;
	1470	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8993.5;
	1470	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4436.4;
	1470	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7770.6;
	1470	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1417.5;
	1471	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.3;
	1471	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	826.3;
	1471	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	935.9;
	1471	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3052.1;
	1471	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10532.5;
	1471	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5246.9;
	1471	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8566.3;
	1471	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1656.5;
	1472	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.1;
	1472	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	847.9;
	1472	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	973.3;
	1472	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3137.7;
	1472	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10936;
	1472	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5427.6;
	1472	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8720.6;
	1472	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1693.1;
	1473	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.8;
	1473	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	829.7;
	1473	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	935.3;
	1473	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3128;
	1473	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10527.6;
	1473	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5327.9;
	1473	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8905.5;
	1473	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1642.6;
	1474	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.2;
	1474	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	807.8;
	1474	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	909.3;
	1474	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3222.2;
	1474	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10268.3;
	1474	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5304.2;
	1474	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9093.1;
	1474	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1600;
	1475	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901;
	1475	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	794.6;
	1475	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	895.9;
	1475	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3350.9;
	1475	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10066.7;
	1475	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5281.1;
	1475	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9218.3;
	1475	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1552.1;
	1476	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.4;
	1476	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777.7;
	1476	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	898.5;
	1476	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3470.6;
	1476	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9863.2;
	1476	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5289.7;
	1476	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9250.9;
	1476	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1503;
	1477	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.7;
	1477	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755.8;
	1477	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	880.1;
	1477	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3604.2;
	1477	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9717.2;
	1477	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5346.1;
	1477	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9276.1;
	1477	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1469.3;
	1478	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.6;
	1478	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	745;
	1478	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884.6;
	1478	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3775.7;
	1478	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9600.3;
	1478	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5450.4;
	1478	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9354.1;
	1478	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1462;
	1479	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.4;
	1479	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735.8;
	1479	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	875.3;
	1479	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3928.3;
	1479	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9496.8;
	1479	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5556.2;
	1479	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9351.8;
	1479	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1454.1;
	1480	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.8;
	1480	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.3;
	1480	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857;
	1480	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4038.4;
	1480	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9462.5;
	1480	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5657.4;
	1480	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9370.1;
	1480	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1447.9;
	1481	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.4;
	1481	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.2;
	1481	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.7;
	1481	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4040.5;
	1481	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9494.7;
	1481	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5743.6;
	1481	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9250;
	1481	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1462.8;
	1482	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.8;
	1482	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.8;
	1482	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.8;
	1482	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3925.4;
	1482	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9579.1;
	1482	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5805.1;
	1482	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9209.8;
	1482	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1460.8;
	1483	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.4;
	1483	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	754.1;
	1483	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	874.4;
	1483	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3875.7;
	1483	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9974.6;
	1483	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5997.9;
	1483	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9460.8;
	1483	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1540.6;
	1484	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.5;
	1484	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	779.7;
	1484	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	904.1;
	1484	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3959.3;
	1484	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10326.9;
	1484	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6239.6;
	1484	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9603.4;
	1484	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1587.1;
	1485	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.6;
	1485	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772.6;
	1485	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	896.3;
	1485	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3871.3;
	1485	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10171.6;
	1485	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6099.1;
	1485	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9418.9;
	1485	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1561.6;
	1486	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.2;
	1486	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	739.3;
	1486	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	862.6;
	1486	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3705.7;
	1486	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9695.6;
	1486	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5778.5;
	1486	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9010.6;
	1486	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1489.1;
	1487	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.7;
	1487	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	704.9;
	1487	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	806.5;
	1487	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3421.4;
	1487	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8892.8;
	1487	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5290.7;
	1487	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8495.2;
	1487	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1368.9;
	1488	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	847.6;
	1488	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	664.4;
	1488	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	752.8;
	1488	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3080.1;
	1488	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8050;
	1488	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4768;
	1488	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7977.9;
	1488	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1251.7;
	1489	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	823;
	1489	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	629.7;
	1489	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	716.6;
	1489	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2818.1;
	1489	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7450.3;
	1489	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4466.2;
	1489	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7573.9;
	1489	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1169.4;
	1490	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	812.3;
	1490	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	614.9;
	1490	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	699.3;
	1490	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2654.4;
	1490	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7133.7;
	1490	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4248.6;
	1490	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7364.8;
	1490	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1126.9;
	1491	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	810.1;
	1491	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	599.1;
	1491	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	689.6;
	1491	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2565;
	1491	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6957.5;
	1491	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4131.1;
	1491	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7247;
	1491	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1085.1;
	1492	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	809;
	1492	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.8;
	1492	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	686.5;
	1492	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2524.8;
	1492	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6944;
	1492	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4106.1;
	1492	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7224.9;
	1492	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1075.5;
	1493	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	817.4;
	1493	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	615.1;
	1493	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	694.9;
	1493	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2529.1;
	1493	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7137.1;
	1493	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4200.6;
	1493	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7418.2;
	1493	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1108.3;
	1494	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	836.9;
	1494	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	660.2;
	1494	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	731.1;
	1494	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2683.2;
	1494	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7822.9;
	1494	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4596;
	1494	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7987.4;
	1494	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1201.4;
	1495	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.9;
	1495	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.8;
	1495	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	814.5;
	1495	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3075.4;
	1495	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9041;
	1495	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5329.3;
	1495	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8832.7;
	1495	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1386.3;
	1496	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.4;
	1496	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	759.9;
	1496	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.2;
	1496	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3155.9;
	1496	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9388.1;
	1496	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5524.3;
	1496	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9029.2;
	1496	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1438.5;
	1497	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903;
	1497	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753;
	1497	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	841.7;
	1497	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3199.8;
	1497	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9440.2;
	1497	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5528.1;
	1497	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9267.3;
	1497	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1465.3;
	1498	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901;
	1498	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	745.6;
	1498	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.1;
	1498	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3369.8;
	1498	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9601.1;
	1498	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5632.5;
	1498	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9527.2;
	1498	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1499.2;
	1499	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.2;
	1499	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744;
	1499	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	856.7;
	1499	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3603;
	1499	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9756.2;
	1499	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5833.2;
	1499	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9719.7;
	1499	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1544.2;
	1500	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.5;
	1500	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.8;
	1500	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.5;
	1500	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3875;
	1500	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9829.9;
	1500	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5973.6;
	1500	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9898.1;
	1500	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1580.6;
	1501	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.4;
	1501	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.3;
	1501	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	862.3;
	1501	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4129.6;
	1501	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9839.6;
	1501	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6088.7;
	1501	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10005.2;
	1501	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1597.4;
	1502	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.5;
	1502	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	721.6;
	1502	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.7;
	1502	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4402.8;
	1502	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9870.1;
	1502	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6206.5;
	1502	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10257.4;
	1502	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1616.9;
	1503	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.6;
	1503	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	722.1;
	1503	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	875.5;
	1503	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4604.8;
	1503	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9877.9;
	1503	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6266;
	1503	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10622.2;
	1503	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1626.1;
	1504	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.3;
	1504	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	713.4;
	1504	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	876.5;
	1504	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4705.4;
	1504	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9796.3;
	1504	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6339.8;
	1504	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10871.1;
	1504	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1621.9;
	1505	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.2;
	1505	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	712.1;
	1505	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	874.1;
	1505	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4705.2;
	1505	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9759.6;
	1505	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6361.9;
	1505	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10914.2;
	1505	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1627.7;
	1506	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.3;
	1506	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719.1;
	1506	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	872.4;
	1506	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4537.2;
	1506	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9684.3;
	1506	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6297;
	1506	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10607.9;
	1506	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1609.8;
	1507	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883;
	1507	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.6;
	1507	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885;
	1507	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4374;
	1507	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9878.8;
	1507	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6272.2;
	1507	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10382.4;
	1507	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1675.8;
	1508	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.6;
	1508	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772.1;
	1508	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	904.9;
	1508	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4339.3;
	1508	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10198.7;
	1508	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6382.7;
	1508	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10279.7;
	1508	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1709.2;
	1509	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.9;
	1509	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	762.5;
	1509	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	887.8;
	1509	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4135.6;
	1509	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10003.8;
	1509	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6152;
	1509	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9888.6;
	1509	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1644.8;
	1510	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.3;
	1510	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.3;
	1510	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	864.3;
	1510	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3885.8;
	1510	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9534.2;
	1510	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5746.5;
	1510	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9408;
	1510	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1531.4;
	1511	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	852.2;
	1511	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	700.8;
	1511	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	811.1;
	1511	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3546.8;
	1511	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8751.6;
	1511	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5216;
	1511	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8697;
	1511	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1429.3;
	1512	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	833.1;
	1512	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	659.2;
	1512	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	754.8;
	1512	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3159.7;
	1512	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7954.8;
	1512	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4670.1;
	1512	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7935.8;
	1512	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1309.1;
	1513	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	829.3;
	1513	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	643.3;
	1513	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	735.3;
	1513	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2838.7;
	1513	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7529.9;
	1513	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4239.3;
	1513	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7455;
	1513	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1239.6;
	1514	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	822.7;
	1514	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	639;
	1514	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	716.9;
	1514	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2631.9;
	1514	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7313.7;
	1514	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4021.7;
	1514	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7165.2;
	1514	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1231;
	1515	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	820.5;
	1515	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	640.3;
	1515	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	712.8;
	1515	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2510;
	1515	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7263.1;
	1515	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3908.8;
	1515	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7009;
	1515	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1236.7;
	1516	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	822.6;
	1516	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	650.4;
	1516	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	723.2;
	1516	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2466.4;
	1516	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7387.1;
	1516	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3882.4;
	1516	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	6980.3;
	1516	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1248.2;
	1517	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	830.2;
	1517	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	673.9;
	1517	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	748.7;
	1517	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2476.8;
	1517	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7786.5;
	1517	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3989.5;
	1517	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7157.7;
	1517	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1267.6;
	1518	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	851.2;
	1518	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.2;
	1518	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	803.3;
	1518	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2615.9;
	1518	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8709.8;
	1518	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4413.4;
	1518	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7719.7;
	1518	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1432.7;
	1519	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887;
	1519	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	805.7;
	1519	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	906.2;
	1519	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2979.8;
	1519	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10165.3;
	1519	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5186.9;
	1519	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8520.4;
	1519	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1654.2;
	1520	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.5;
	1520	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	822.2;
	1520	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	944;
	1520	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3035.7;
	1520	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10532.2;
	1520	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5382.9;
	1520	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8708.9;
	1520	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1719.8;
	1521	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.3;
	1521	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	797.8;
	1521	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913.3;
	1521	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3056.1;
	1521	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10144.2;
	1521	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5322.8;
	1521	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8843;
	1521	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1654.2;
	1522	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.7;
	1522	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	774.3;
	1522	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	890.4;
	1522	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3176.8;
	1522	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9892.1;
	1522	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5286.7;
	1522	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9033.4;
	1522	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1626;
	1523	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.9;
	1523	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	757.6;
	1523	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	867.1;
	1523	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3315.9;
	1523	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9758.7;
	1523	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5308.1;
	1523	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9196.6;
	1523	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1598.8;
	1524	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.1;
	1524	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744.8;
	1524	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.9;
	1524	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3461.8;
	1524	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9665;
	1524	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5327.5;
	1524	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9332.3;
	1524	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1562.5;
	1525	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.5;
	1525	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.6;
	1525	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	833.2;
	1525	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3591.7;
	1525	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9556.5;
	1525	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5361.3;
	1525	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9390.7;
	1525	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1559.7;
	1526	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.1;
	1526	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.3;
	1526	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.4;
	1526	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3763.7;
	1526	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9528.9;
	1526	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5458.8;
	1526	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9570.1;
	1526	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1550.8;
	1527	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.8;
	1527	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	715.5;
	1527	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	836.3;
	1527	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3897.6;
	1527	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9417.4;
	1527	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5536.1;
	1527	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9701.6;
	1527	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1531.7;
	1528	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919;
	1528	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	699;
	1528	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.6;
	1528	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3967.4;
	1528	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9264.1;
	1528	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5649.5;
	1528	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9750.1;
	1528	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1554;
	1529	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916;
	1529	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	691;
	1529	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	830;
	1529	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3950.9;
	1529	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9162.7;
	1529	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5709.8;
	1529	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9684.8;
	1529	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1550.3;
	1530	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.4;
	1530	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	687;
	1530	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	818.8;
	1530	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3798.6;
	1530	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9147.6;
	1530	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5651.8;
	1530	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9468.6;
	1530	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1522.1;
	1531	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.5;
	1531	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	710.9;
	1531	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	827;
	1531	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3635.3;
	1531	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9416.2;
	1531	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5643;
	1531	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9399.5;
	1531	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1564;
	1532	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.9;
	1532	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.4;
	1532	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851.3;
	1532	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3646.9;
	1532	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9622;
	1532	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5748.9;
	1532	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9390.5;
	1532	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1583.2;
	1533	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.4;
	1533	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.1;
	1533	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	842.4;
	1533	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3504.4;
	1533	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9448.6;
	1533	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5561;
	1533	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9127.7;
	1533	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1532.7;
	1534	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.7;
	1534	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723.4;
	1534	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	814.3;
	1534	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3365.1;
	1534	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9158.6;
	1534	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5294.2;
	1534	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8807.8;
	1534	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1486.3;
	1535	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.7;
	1535	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	692.1;
	1535	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	776.7;
	1535	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3169.2;
	1535	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8624.6;
	1535	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4939.8;
	1535	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8329.7;
	1535	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1422.1;
	1536	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	853.3;
	1536	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	659.7;
	1536	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	742.9;
	1536	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2923.4;
	1536	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8057.9;
	1536	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4568.8;
	1536	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7829.1;
	1536	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1323.8;
	1537	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	834.7;
	1537	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	638.9;
	1537	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	746;
	1537	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2706.9;
	1537	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7572.6;
	1537	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4241.8;
	1537	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7404.7;
	1537	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1272.7;
	1538	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	828.2;
	1538	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	629.1;
	1538	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	731.7;
	1538	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2562.7;
	1538	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7303.1;
	1538	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4035.6;
	1538	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7143.8;
	1538	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1239.5;
	1539	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	827.2;
	1539	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	627.1;
	1539	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	725.3;
	1539	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2474.9;
	1539	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7176;
	1539	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3943.3;
	1539	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7004.5;
	1539	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1215.8;
	1540	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	825.8;
	1540	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	631.8;
	1540	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	728.7;
	1540	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2424.9;
	1540	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7166.6;
	1540	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3892.4;
	1540	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	6945.6;
	1540	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1235.5;
	1541	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	825.8;
	1541	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	636.9;
	1541	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	741;
	1541	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2414.9;
	1541	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7304;
	1541	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3894.4;
	1541	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	6982.8;
	1541	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1242.5;
	1542	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	836.1;
	1542	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	660;
	1542	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	765.8;
	1542	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2472.1;
	1542	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7657.6;
	1542	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4007.1;
	1542	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7164.4;
	1542	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1274.4;
	1543	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	850.3;
	1543	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	695.4;
	1543	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	801.6;
	1543	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2582.4;
	1543	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8225.2;
	1543	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4282.6;
	1543	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7442.8;
	1543	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1376.2;
	1544	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	853.4;
	1544	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711.2;
	1544	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.4;
	1544	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2656.8;
	1544	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8640.5;
	1544	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4486.1;
	1544	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7628.2;
	1544	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1426.1;
	1545	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	863.5;
	1545	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.8;
	1545	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	864;
	1545	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2850.7;
	1545	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8934.8;
	1545	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4761.1;
	1545	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7939.3;
	1545	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1452.3;
	1546	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.2;
	1546	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	712.8;
	1546	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	872.3;
	1546	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3021.7;
	1546	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8997.9;
	1546	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4962.8;
	1546	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8219.4;
	1546	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1461.7;
	1547	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871.1;
	1547	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	702.7;
	1547	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866;
	1547	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3182.5;
	1547	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9039.8;
	1547	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5105.3;
	1547	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8487.2;
	1547	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1434.6;
	1548	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.1;
	1548	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	688.1;
	1548	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851.7;
	1548	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3324;
	1548	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9050.1;
	1548	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5237.6;
	1548	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8655.5;
	1548	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1407.8;
	1549	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	869.1;
	1549	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	674.7;
	1549	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.6;
	1549	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3421.5;
	1549	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9029.5;
	1549	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5345;
	1549	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8787;
	1549	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1410.1;
	1550	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.3;
	1550	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	673.6;
	1550	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851;
	1550	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3538.1;
	1550	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9028.4;
	1550	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5442.7;
	1550	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8935.9;
	1550	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1394;
	1551	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.7;
	1551	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	672.7;
	1551	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851.3;
	1551	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3625.2;
	1551	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9047.3;
	1551	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5575.1;
	1551	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9151.2;
	1551	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1403.7;
	1552	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.8;
	1552	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	674.7;
	1552	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	855.9;
	1552	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3645.5;
	1552	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9097;
	1552	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5676.8;
	1552	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9275.8;
	1552	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1455.2;
	1553	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867.1;
	1553	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	675.1;
	1553	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849.8;
	1553	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3621.2;
	1553	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9141.8;
	1553	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5743.6;
	1553	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9258.6;
	1553	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1512.7;
	1554	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	863.9;
	1554	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	674.9;
	1554	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	826;
	1554	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3528.3;
	1554	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9118.9;
	1554	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5716.6;
	1554	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9084.5;
	1554	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1498.2;
	1555	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	865.6;
	1555	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	689;
	1555	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.7;
	1555	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3485.7;
	1555	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9283.3;
	1555	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5706.3;
	1555	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9130.7;
	1555	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1496.5;
	1556	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884;
	1556	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	703.8;
	1556	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.1;
	1556	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3548.8;
	1556	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9454;
	1556	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5816.1;
	1556	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9230.2;
	1556	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1525.1;
	1557	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.7;
	1557	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	693.3;
	1557	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	839.5;
	1557	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3425.6;
	1557	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9185.3;
	1557	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5618.7;
	1557	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8979.7;
	1557	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1506.4;
	1558	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.3;
	1558	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	671.8;
	1558	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	816.6;
	1558	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3297.3;
	1558	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8860.1;
	1558	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5362.5;
	1558	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8724.5;
	1558	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1465.4;
	1559	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	861.7;
	1559	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	643.3;
	1559	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	772;
	1559	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3132.8;
	1559	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8334.7;
	1559	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5023.3;
	1559	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8338.7;
	1559	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1401.3;
	1560	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	847.3;
	1560	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	614.3;
	1560	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	729.4;
	1560	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2933.7;
	1560	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7753.5;
	1560	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4629;
	1560	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7917.4;
	1560	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1285.5;
	1561	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	835.2;
	1561	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.4;
	1561	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	705.1;
	1561	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2751.5;
	1561	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7244.5;
	1561	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4288.8;
	1561	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7550.1;
	1561	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1233.1;
	1562	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	828.2;
	1562	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	582.6;
	1562	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	687;
	1562	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2605.8;
	1562	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6893.3;
	1562	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4065.2;
	1562	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7295.5;
	1562	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1187.2;
	1563	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	822.6;
	1563	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	574.6;
	1563	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	677.4;
	1563	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2500.7;
	1563	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6690.6;
	1563	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3910.4;
	1563	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7149.3;
	1563	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1153.7;
	1564	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	820.4;
	1564	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	575;
	1564	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	669.9;
	1564	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2442.8;
	1564	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6604.7;
	1564	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3833.9;
	1564	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7068.7;
	1564	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1153.8;
	1565	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	821.9;
	1565	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	573.5;
	1565	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	671.9;
	1565	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2420.4;
	1565	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6621.3;
	1565	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3805.6;
	1565	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7069.5;
	1565	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1175.4;
	1566	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	822.9;
	1566	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586.8;
	1566	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	684.7;
	1566	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2440.9;
	1566	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6797.1;
	1566	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3873.5;
	1566	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7163.5;
	1566	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1217.7;
	1567	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	829.3;
	1567	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	614.3;
	1567	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	709.4;
	1567	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2500.5;
	1567	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7162.8;
	1567	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4034.9;
	1567	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7298;
	1567	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1179.5;
	1568	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	829.5;
	1568	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	636.9;
	1568	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	732.9;
	1568	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2542.3;
	1568	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7467;
	1568	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4206.5;
	1568	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7391.9;
	1568	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1226.2;
	1569	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	841.6;
	1569	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	665.6;
	1569	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	771.4;
	1569	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2745.1;
	1569	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7964.6;
	1569	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4513.4;
	1569	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7767;
	1569	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1349.8;
	1570	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	858;
	1570	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	677.3;
	1570	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	793.7;
	1570	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2991.3;
	1570	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8362.1;
	1570	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4797.4;
	1570	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8111.1;
	1570	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1380.8;
	1571	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.3;
	1571	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	672.5;
	1571	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	800.3;
	1571	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3177.5;
	1571	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8529.5;
	1571	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4972.5;
	1571	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8358.7;
	1571	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1405.4;
	1572	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871.7;
	1572	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	666.6;
	1572	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	799.3;
	1572	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3297.3;
	1572	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8593.8;
	1572	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5077.9;
	1572	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8548.7;
	1572	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1424.4;
	1573	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.9;
	1573	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	666.9;
	1573	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	803.5;
	1573	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3392.7;
	1573	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8659.4;
	1573	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5130.4;
	1573	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8733.4;
	1573	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1418.7;
	1574	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.5;
	1574	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	661.1;
	1574	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	802;
	1574	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3489.6;
	1574	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8657.3;
	1574	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5153;
	1574	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8850.8;
	1574	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1436.6;
	1575	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.8;
	1575	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	653.7;
	1575	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	795;
	1575	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3584.1;
	1575	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8654.1;
	1575	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5205.1;
	1575	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8934.8;
	1575	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1436.1;
	1576	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.6;
	1576	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	656.9;
	1576	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	791.4;
	1576	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3644.7;
	1576	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8678.4;
	1576	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5289.2;
	1576	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8981.2;
	1576	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1441.9;
	1577	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.4;
	1577	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	667.6;
	1577	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	800.1;
	1577	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3647;
	1577	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8760.4;
	1577	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5391.7;
	1577	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8990;
	1577	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1468.6;
	1578	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.6;
	1578	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	682.3;
	1578	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	814.2;
	1578	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3590.9;
	1578	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9006.8;
	1578	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5484.6;
	1578	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9031.1;
	1578	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1458.5;
	1579	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.6;
	1579	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711.2;
	1579	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	844.5;
	1579	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3621.8;
	1579	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9428.1;
	1579	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5758.9;
	1579	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9345.4;
	1579	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1539.2;
	1580	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.6;
	1580	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.2;
	1580	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.8;
	1580	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3778.7;
	1580	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9704.2;
	1580	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6046.8;
	1580	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9632.8;
	1580	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1619;
	1581	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.5;
	1581	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	715.3;
	1581	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.8;
	1581	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3704.7;
	1581	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9528.2;
	1581	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5887.9;
	1581	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9413.9;
	1581	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1580.2;
	1582	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.5;
	1582	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	690.6;
	1582	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	837.9;
	1582	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3595.9;
	1582	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9152.6;
	1582	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5589.2;
	1582	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9072.2;
	1582	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1518.9;
	1583	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.3;
	1583	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	650.2;
	1583	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	791.7;
	1583	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3370.6;
	1583	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8516.1;
	1583	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5152.5;
	1583	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8535;
	1583	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1430.2;
	1584	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	843.7;
	1584	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	611.9;
	1584	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	740.4;
	1584	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3071.7;
	1584	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7786.6;
	1584	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4664;
	1584	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7936.5;
	1584	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1325;
	1585	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	824.3;
	1585	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588.4;
	1585	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	737.5;
	1585	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2839.3;
	1585	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7195.4;
	1585	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4301;
	1585	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7571.2;
	1585	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1236;
	1586	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	817.2;
	1586	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	576.4;
	1586	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	721.9;
	1586	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2699;
	1586	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6891.7;
	1586	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4088.3;
	1586	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7343.2;
	1586	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1157.4;
	1587	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	814.6;
	1587	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	571;
	1587	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	715.9;
	1587	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2619.4;
	1587	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6724.6;
	1587	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3978.5;
	1587	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7232.4;
	1587	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1186.9;
	1588	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	812;
	1588	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	573.6;
	1588	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	712.8;
	1588	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2589.3;
	1588	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6735.6;
	1588	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3942;
	1588	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7243.8;
	1588	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1167.6;
	1589	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	815.4;
	1589	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577.5;
	1589	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	721.3;
	1589	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2614.3;
	1589	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6939.7;
	1589	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4039.3;
	1589	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7471.7;
	1589	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1176.6;
	1590	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	829.2;
	1590	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	605.2;
	1590	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	756.6;
	1590	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2772;
	1590	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7546.6;
	1590	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4447.4;
	1590	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8044;
	1590	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1284.7;
	1591	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	853.5;
	1591	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	665.5;
	1591	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.7;
	1591	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3126.4;
	1591	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8611.2;
	1591	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5117.7;
	1591	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8796.3;
	1591	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1434.3;
	1592	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.8;
	1592	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	695.8;
	1592	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	872.5;
	1592	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3224.3;
	1592	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9085;
	1592	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5371;
	1592	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9002.9;
	1592	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1473;
	1593	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.9;
	1593	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714.1;
	1593	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	876.3;
	1593	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3299.6;
	1593	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9275.6;
	1593	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5409.4;
	1593	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9229.3;
	1593	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1479.6;
	1594	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.4;
	1594	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730;
	1594	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	888.3;
	1594	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3471;
	1594	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9493.6;
	1594	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5532.7;
	1594	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9487.9;
	1594	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1575.6;
	1595	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.6;
	1595	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742.8;
	1595	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	900.2;
	1595	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3637.4;
	1595	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9703.9;
	1595	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5646.4;
	1595	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9744.6;
	1595	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1597.2;
	1596	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.5;
	1596	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.4;
	1596	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	904.9;
	1596	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3785.7;
	1596	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9869.4;
	1596	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5727.7;
	1596	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9934.2;
	1596	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1632.6;
	1597	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.5;
	1597	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	745.4;
	1597	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	907.5;
	1597	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3891.4;
	1597	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9853.6;
	1597	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5787.6;
	1597	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10028.6;
	1597	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1634.8;
	1598	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920.2;
	1598	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.2;
	1598	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	916.6;
	1598	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3970;
	1598	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9829.4;
	1598	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5845;
	1598	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10113.8;
	1598	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1643.9;
	1599	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.1;
	1599	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	734.9;
	1599	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	924.9;
	1599	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4010.8;
	1599	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9755.6;
	1599	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5876;
	1599	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10095.6;
	1599	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1642.9;
	1600	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924;
	1600	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.4;
	1600	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	926.7;
	1600	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4010.2;
	1600	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9671.7;
	1600	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5884;
	1600	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9993.2;
	1600	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1631.4;
	1601	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.8;
	1601	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	732.6;
	1601	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	931;
	1601	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3958.4;
	1601	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9702.5;
	1601	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5915.5;
	1601	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9879.8;
	1601	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1626.4;
	1602	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.4;
	1602	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.9;
	1602	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	923.3;
	1602	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3883.3;
	1602	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9869;
	1602	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6028.1;
	1602	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9825.8;
	1602	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1615.5;
	1603	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.6;
	1603	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	758.8;
	1603	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	937.3;
	1603	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3928;
	1603	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10252.8;
	1603	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6279.6;
	1603	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10060.1;
	1603	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1707.2;
	1604	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920.5;
	1604	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	768.9;
	1604	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	960.3;
	1604	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4067.1;
	1604	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10319.1;
	1604	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6407.8;
	1604	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10166.6;
	1604	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1717.5;
	1605	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.1;
	1605	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749;
	1605	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940.3;
	1605	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3998;
	1605	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10063.1;
	1605	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6246.7;
	1605	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9958.3;
	1605	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1676.6;
	1606	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.3;
	1606	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	715.6;
	1606	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	907.6;
	1606	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3857.7;
	1606	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9616.6;
	1606	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5930.6;
	1606	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9623.7;
	1606	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1615.6;
	1607	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867.1;
	1607	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	675.5;
	1607	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851.3;
	1607	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3604.5;
	1607	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8850.3;
	1607	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5422.7;
	1607	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9040.1;
	1607	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1487.7;
	1608	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	838.5;
	1608	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	627;
	1608	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	795.9;
	1608	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3290.4;
	1608	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8077.8;
	1608	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4911;
	1608	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8438.5;
	1608	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1391.3;
	1609	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	818.6;
	1609	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	621.1;
	1609	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	724.7;
	1609	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3007.1;
	1609	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7554.5;
	1609	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4557.2;
	1609	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7991;
	1609	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1311.8;
	1610	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	810.6;
	1610	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	611.2;
	1610	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	708.6;
	1610	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2867;
	1610	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7247.1;
	1610	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4361.6;
	1610	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7694.1;
	1610	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1273.1;
	1611	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	803.3;
	1611	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	601.9;
	1611	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	701.2;
	1611	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2784.7;
	1611	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7063.8;
	1611	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4221.4;
	1611	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7534.6;
	1611	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1221.2;
	1612	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	800.4;
	1612	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.9;
	1612	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	692.4;
	1612	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2737.4;
	1612	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7010.2;
	1612	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4170.2;
	1612	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7480.6;
	1612	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1230.6;
	1613	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	800.2;
	1613	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	600.4;
	1613	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	699.6;
	1613	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2752.3;
	1613	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7171.7;
	1613	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4245.8;
	1613	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7646.9;
	1613	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1242.4;
	1614	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	815.6;
	1614	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	627.9;
	1614	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	729.6;
	1614	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2889.7;
	1614	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7742;
	1614	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4613.2;
	1614	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8193.8;
	1614	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1372.4;
	1615	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	847;
	1615	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	682.4;
	1615	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	802;
	1615	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3261.7;
	1615	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8819.3;
	1615	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5256.2;
	1615	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8939.6;
	1615	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1509.4;
	1616	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	857.6;
	1616	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	712.7;
	1616	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	834.5;
	1616	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3367.9;
	1616	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9339.2;
	1616	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5513.2;
	1616	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9157.1;
	1616	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1498.7;
	1617	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	865.5;
	1617	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.3;
	1617	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	845.3;
	1617	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3374.5;
	1617	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9487.7;
	1617	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5545;
	1617	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9312;
	1617	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1560;
	1618	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871;
	1618	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.3;
	1618	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	877;
	1618	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3471.6;
	1618	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9676.1;
	1618	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5668.7;
	1618	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9529.7;
	1618	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1634.6;
	1619	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	881.7;
	1619	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	750.8;
	1619	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	877.2;
	1619	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3616.6;
	1619	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9893.3;
	1619	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5799.5;
	1619	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9728.8;
	1619	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1665.9;
	1620	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.1;
	1620	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.5;
	1620	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	891.1;
	1620	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3775.9;
	1620	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9960.7;
	1620	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5954.2;
	1620	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9803.8;
	1620	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1679.2;
	1621	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.2;
	1621	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.7;
	1621	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	908.9;
	1621	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3968.5;
	1621	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10022.3;
	1621	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6134.3;
	1621	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9889.5;
	1621	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1686.4;
	1622	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908;
	1622	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.8;
	1622	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	932.2;
	1622	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4158.9;
	1622	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9860.9;
	1622	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6345.6;
	1622	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9925.9;
	1622	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1655.6;
	1623	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.3;
	1623	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.7;
	1623	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	948.2;
	1623	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4268.6;
	1623	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9713.3;
	1623	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6543.8;
	1623	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9874.2;
	1623	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1673.2;
	1624	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.8;
	1624	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	732.4;
	1624	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	941.7;
	1624	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4321.3;
	1624	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9709.6;
	1624	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6732.5;
	1624	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9829.8;
	1624	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1664.1;
	1625	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.4;
	1625	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	731.6;
	1625	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	926.1;
	1625	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4323.7;
	1625	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9737.5;
	1625	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6867.1;
	1625	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9803.5;
	1625	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1657.3;
	1626	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.8;
	1626	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.7;
	1626	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	909.4;
	1626	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4193.3;
	1626	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9827.1;
	1626	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6853.3;
	1626	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9844.3;
	1626	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1582;
	1627	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.3;
	1627	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.1;
	1627	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913.9;
	1627	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4131.4;
	1627	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10160.9;
	1627	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6822.6;
	1627	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10126.7;
	1627	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1702.4;
	1628	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.2;
	1628	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	763.8;
	1628	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	926.5;
	1628	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4204.9;
	1628	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10319.2;
	1628	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6938.5;
	1628	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10219.5;
	1628	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1765.3;
	1629	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.9;
	1629	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.1;
	1629	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	911.2;
	1629	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4112.1;
	1629	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10101.4;
	1629	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6732.1;
	1629	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10086.1;
	1629	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1675.8;
	1630	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.7;
	1630	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719.9;
	1630	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	867.7;
	1630	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3895.3;
	1630	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9612.6;
	1630	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6343.9;
	1630	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9678.3;
	1630	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1609.5;
	1631	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	858.6;
	1631	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	677.4;
	1631	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	801;
	1631	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3593.6;
	1631	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8871.1;
	1631	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5796.5;
	1631	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9099.1;
	1631	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1453.9;
	1632	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	835.4;
	1632	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	631.7;
	1632	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	741.7;
	1632	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3226.3;
	1632	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8021.5;
	1632	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5217.8;
	1632	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8441.3;
	1632	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1402.8;
	1633	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	817.6;
	1633	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	622.3;
	1633	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	709.9;
	1633	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2946.5;
	1633	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7465.5;
	1633	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4698.1;
	1633	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7993.4;
	1633	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1316.7;
	1634	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	807.6;
	1634	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	603.2;
	1634	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	691.8;
	1634	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2754.2;
	1634	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7134.3;
	1634	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4242.1;
	1634	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7695;
	1634	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1280.6;
	1635	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	803.6;
	1635	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	596.1;
	1635	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	685.9;
	1635	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2612.2;
	1635	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6941.6;
	1635	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4040.5;
	1635	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7505.6;
	1635	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1255.2;
	1636	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	806.6;
	1636	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	604.1;
	1636	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	687;
	1636	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2562;
	1636	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6914.2;
	1636	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3960.6;
	1636	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7483.6;
	1636	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1221.8;
	1637	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	813.7;
	1637	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	613.5;
	1637	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	697.2;
	1637	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2563.4;
	1637	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7116.8;
	1637	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4013.8;
	1637	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7611.4;
	1637	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1226.6;
	1638	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	829.2;
	1638	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	638.4;
	1638	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	729.6;
	1638	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2707.8;
	1638	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7749;
	1638	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4313.1;
	1638	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8127.6;
	1638	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1224.8;
	1639	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	851.3;
	1639	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	698.9;
	1639	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	810.2;
	1639	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3040.5;
	1639	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8814.7;
	1639	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4977.9;
	1639	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8878.6;
	1639	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1356.4;
	1640	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	863.7;
	1640	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.3;
	1640	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.1;
	1640	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3132.2;
	1640	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9344.8;
	1640	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5198.6;
	1640	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9120.4;
	1640	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1424.8;
	1641	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.4;
	1641	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.9;
	1641	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852;
	1641	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3166.8;
	1641	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9523.5;
	1641	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5295.2;
	1641	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9376.2;
	1641	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1458.4;
	1642	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.1;
	1642	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749.7;
	1642	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860;
	1642	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3244.8;
	1642	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9725.7;
	1642	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5417.6;
	1642	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9620.8;
	1642	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1493.3;
	1643	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.5;
	1643	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	761;
	1643	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	876.6;
	1643	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3296.5;
	1643	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9878;
	1643	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5495.6;
	1643	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9835.2;
	1643	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1515.6;
	1644	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.7;
	1644	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	758.2;
	1644	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	903.3;
	1644	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3292.1;
	1644	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9939.9;
	1644	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5515.7;
	1644	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9980.7;
	1644	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1525;
	1645	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900;
	1645	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	748;
	1645	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	896.7;
	1645	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3247.9;
	1645	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9947.4;
	1645	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5489.9;
	1645	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9973.1;
	1645	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1527.8;
	1646	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903;
	1646	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749.5;
	1646	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	901.7;
	1646	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3235.3;
	1646	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9910.8;
	1646	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5440.8;
	1646	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9914.3;
	1646	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1526.4;
	1647	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.7;
	1647	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746.9;
	1647	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	881.9;
	1647	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3237.8;
	1647	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9775.1;
	1647	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5439.1;
	1647	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9697.1;
	1647	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1500.7;
	1648	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.7;
	1648	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.8;
	1648	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849.7;
	1648	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3222.6;
	1648	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9679.2;
	1648	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5432.1;
	1648	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9512.2;
	1648	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1483.3;
	1649	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.9;
	1649	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744.1;
	1649	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851;
	1649	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3191.8;
	1649	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9717.6;
	1649	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5479.1;
	1649	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9411.2;
	1649	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1492.3;
	1650	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.5;
	1650	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.4;
	1650	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.9;
	1650	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3218.2;
	1650	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9923.5;
	1650	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5560;
	1650	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9650.2;
	1650	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1515.6;
	1651	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.2;
	1651	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	763.5;
	1651	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.3;
	1651	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3330.2;
	1651	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10230;
	1651	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5859.6;
	1651	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9696.5;
	1651	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1574.2;
	1652	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902;
	1652	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	766.2;
	1652	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	899.9;
	1652	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3405.9;
	1652	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10295.4;
	1652	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5972.8;
	1652	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9617.7;
	1652	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1571.3;
	1653	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.2;
	1653	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	756.4;
	1653	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892;
	1653	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3358.1;
	1653	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10083.8;
	1653	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5824.3;
	1653	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9357.9;
	1653	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1539.7;
	1654	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.9;
	1654	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.7;
	1654	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	863.6;
	1654	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3210.7;
	1654	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9653.8;
	1654	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5499.6;
	1654	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8970;
	1654	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1471.2;
	1655	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868;
	1655	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	687.9;
	1655	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	809.6;
	1655	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2979.1;
	1655	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8931.4;
	1655	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5022.4;
	1655	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8393.6;
	1655	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1364.2;
	1656	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	846.2;
	1656	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	647;
	1656	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	759.6;
	1656	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2715.9;
	1656	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8163.4;
	1656	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4526;
	1656	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7860.3;
	1656	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1259;
	1657	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	830;
	1657	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	618.2;
	1657	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	719.4;
	1657	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2500.4;
	1657	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7500.2;
	1657	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4348.1;
	1657	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7429.9;
	1657	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1169.4;
	1658	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	828.7;
	1658	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	609.4;
	1658	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	708.2;
	1658	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2397.1;
	1658	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7233.6;
	1658	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4159.9;
	1658	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7244.3;
	1658	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1127.2;
	1659	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	830.5;
	1659	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	600.2;
	1659	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	699.2;
	1659	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2333.8;
	1659	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7056.8;
	1659	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4067.6;
	1659	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7100.8;
	1659	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1138.6;
	1660	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	826.8;
	1660	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	601.2;
	1660	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	694.4;
	1660	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2308.5;
	1660	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7049.7;
	1660	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4067.9;
	1660	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7105.4;
	1660	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1136.8;
	1661	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	833.2;
	1661	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	613.4;
	1661	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	706.9;
	1661	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2338.7;
	1661	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7247.6;
	1661	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4168.3;
	1661	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7261.6;
	1661	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1169.8;
	1662	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	846.6;
	1662	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	639.8;
	1662	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	741.7;
	1662	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2480.8;
	1662	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7839.9;
	1662	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4560.5;
	1662	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7694;
	1662	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1242.3;
	1663	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.1;
	1663	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	690.9;
	1663	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	822.3;
	1663	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2833.9;
	1663	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8912.1;
	1663	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5264.2;
	1663	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8425.8;
	1663	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1377.3;
	1664	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.2;
	1664	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	712.9;
	1664	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	853.4;
	1664	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2925;
	1664	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9319.4;
	1664	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5495.2;
	1664	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8559.5;
	1664	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1452.2;
	1665	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.9;
	1665	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	718.7;
	1665	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	848;
	1665	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2946;
	1665	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9394.2;
	1665	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5506.2;
	1665	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8702.8;
	1665	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1475.2;
	1666	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.2;
	1666	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.9;
	1666	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.8;
	1666	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3022.9;
	1666	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9582.8;
	1666	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5553.8;
	1666	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8979.9;
	1666	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1513.8;
	1667	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888;
	1667	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	732.6;
	1667	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.5;
	1667	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3108.9;
	1667	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9775.1;
	1667	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5636.5;
	1667	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9128.8;
	1667	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1543.5;
	1668	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.9;
	1668	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	732.2;
	1668	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	858.7;
	1668	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3201.5;
	1668	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9792;
	1668	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5684.9;
	1668	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9285.5;
	1668	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1544.4;
	1669	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.7;
	1669	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.4;
	1669	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.7;
	1669	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3288.9;
	1669	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9795.4;
	1669	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5678.5;
	1669	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9440;
	1669	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1537.8;
	1670	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.1;
	1670	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	726.3;
	1670	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.9;
	1670	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3389.1;
	1670	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9785;
	1670	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5652.8;
	1670	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9650.1;
	1670	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1543.2;
	1671	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.8;
	1671	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725;
	1671	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854;
	1671	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3461.6;
	1671	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9723;
	1671	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5582.8;
	1671	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9922.6;
	1671	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1547;
	1672	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.3;
	1672	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714;
	1672	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849.4;
	1672	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3495.6;
	1672	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9590.5;
	1672	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5572.3;
	1672	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9935;
	1672	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1541.1;
	1673	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	869.6;
	1673	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	713.1;
	1673	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	850.3;
	1673	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3453.8;
	1673	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9541.7;
	1673	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5614.7;
	1673	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9809.8;
	1673	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1551.2;
	1674	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	859;
	1674	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	713.9;
	1674	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.3;
	1674	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3384.4;
	1674	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9621.7;
	1674	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5692.1;
	1674	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9741.9;
	1674	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1580;
	1675	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	860.9;
	1675	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.3;
	1675	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	883.8;
	1675	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3407.9;
	1675	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9993.2;
	1675	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5879.4;
	1675	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9872.7;
	1675	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1682.7;
	1676	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.7;
	1676	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	750.3;
	1676	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	904.2;
	1676	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3494.4;
	1676	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10131.2;
	1676	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6019.8;
	1676	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9873.7;
	1676	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1710.6;
	1677	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.3;
	1677	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.9;
	1677	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.1;
	1677	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3425.5;
	1677	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9915.1;
	1677	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5960.4;
	1677	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9684.9;
	1677	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1662.3;
	1678	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.4;
	1678	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	718.8;
	1678	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	867.2;
	1678	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3284;
	1678	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9502.7;
	1678	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5692.2;
	1678	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9333.7;
	1678	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1570;
	1679	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	856.3;
	1679	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	678.1;
	1679	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	812.7;
	1679	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3050.6;
	1679	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8781.4;
	1679	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5267.7;
	1679	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8778.4;
	1679	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1474.4;
	1680	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	837.6;
	1680	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	638.8;
	1680	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	761.8;
	1680	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2762;
	1680	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8041.2;
	1680	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4773;
	1680	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8172.7;
	1680	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1351.4;
	1681	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	838;
	1681	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	614.2;
	1681	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	740.2;
	1681	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2556.8;
	1681	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7548.5;
	1681	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4281.5;
	1681	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7745.1;
	1681	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1277;
	1682	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	831.8;
	1682	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	599.7;
	1682	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	715.8;
	1682	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2425.6;
	1682	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7232.9;
	1682	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4102.2;
	1682	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7429.7;
	1682	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1250.7;
	1683	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	834.2;
	1683	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	593.2;
	1683	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	696.8;
	1683	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2362.3;
	1683	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7043.1;
	1683	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3983.4;
	1683	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7229.5;
	1683	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1231.5;
	1684	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	832.8;
	1684	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589.7;
	1684	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	690.8;
	1684	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2327.9;
	1684	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6995;
	1684	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3929.4;
	1684	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7114.7;
	1684	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1198.2;
	1685	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	842;
	1685	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	603;
	1685	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	705.9;
	1685	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2357;
	1685	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7197.7;
	1685	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4022.4;
	1685	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7250.7;
	1685	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1194.3;
	1686	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	858.8;
	1686	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	637.4;
	1686	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	745.1;
	1686	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2499;
	1686	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7786.6;
	1686	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4382.3;
	1686	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7734.8;
	1686	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1290.2;
	1687	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.1;
	1687	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	699.7;
	1687	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	830.3;
	1687	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2848.2;
	1687	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8806.4;
	1687	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5038.9;
	1687	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8490.5;
	1687	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1412.5;
	1688	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901;
	1688	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	718.2;
	1688	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	872.6;
	1688	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2941.2;
	1688	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9257.5;
	1688	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5243;
	1688	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8626.1;
	1688	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1507;
	1689	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.5;
	1689	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	724.8;
	1689	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	875.7;
	1689	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2966.4;
	1689	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9414.8;
	1689	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5310.7;
	1689	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8806;
	1689	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1545.6;
	1690	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.1;
	1690	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.1;
	1690	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	883.3;
	1690	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3043;
	1690	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9674.9;
	1690	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5414.9;
	1690	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9104;
	1690	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1606.6;
	1691	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.3;
	1691	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.2;
	1691	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	890.9;
	1691	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3109.5;
	1691	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9851.5;
	1691	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5491.1;
	1691	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9396.3;
	1691	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1628.3;
	1692	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.7;
	1692	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	739.2;
	1692	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	891.9;
	1692	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3168.4;
	1692	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9925.1;
	1692	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5525.5;
	1692	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9684.6;
	1692	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1616.8;
	1693	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.9;
	1693	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.2;
	1693	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884.8;
	1693	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3199.5;
	1693	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9908.2;
	1693	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5528.6;
	1693	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9887.7;
	1693	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1618.1;
	1694	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889;
	1694	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.7;
	1694	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	881.3;
	1694	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3251.1;
	1694	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9851.3;
	1694	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5575.3;
	1694	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10071.2;
	1694	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1603.5;
	1695	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.4;
	1695	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	724.2;
	1695	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	875.9;
	1695	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3271.3;
	1695	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9732.6;
	1695	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5570.7;
	1695	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10289.3;
	1695	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1576;
	1696	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.2;
	1696	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.5;
	1696	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	869.3;
	1696	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3246.6;
	1696	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9596.1;
	1696	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5511.8;
	1696	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10399.2;
	1696	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1603.1;
	1697	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	881.6;
	1697	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719.7;
	1697	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.2;
	1697	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3188.2;
	1697	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9495.9;
	1697	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5478.3;
	1697	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10269;
	1697	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1596.1;
	1698	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.8;
	1698	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	721.5;
	1698	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	869.7;
	1698	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3115.5;
	1698	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9514.1;
	1698	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5443.7;
	1698	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9928.7;
	1698	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1573.1;
	1699	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.4;
	1699	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.4;
	1699	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897.7;
	1699	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3133.5;
	1699	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9718.7;
	1699	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5526.6;
	1699	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9773.5;
	1699	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1611.7;
	1700	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.6;
	1700	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	739.8;
	1700	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	910.5;
	1700	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3248;
	1700	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9799.6;
	1700	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5621.1;
	1700	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9748.1;
	1700	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1642;
	1701	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.9;
	1701	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723.6;
	1701	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	894.5;
	1701	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3178.8;
	1701	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9579;
	1701	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5512.9;
	1701	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9535.3;
	1701	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1581;
	1702	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.6;
	1702	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	701.3;
	1702	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.5;
	1702	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3099.6;
	1702	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9229.6;
	1702	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5283.4;
	1702	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9167.8;
	1702	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1530.3;
	1703	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	861.8;
	1703	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	669;
	1703	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	830.8;
	1703	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2979.6;
	1703	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8690.8;
	1703	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4970.1;
	1703	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8722.6;
	1703	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1459.2;
	1704	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	847.3;
	1704	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	635.4;
	1704	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	805.9;
	1704	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2784.8;
	1704	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8094.3;
	1704	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4596.1;
	1704	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8208.2;
	1704	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1349.6;
	1705	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	829.8;
	1705	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	613.6;
	1705	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	807;
	1705	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2597.3;
	1705	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7590.1;
	1705	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4268.7;
	1705	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7734.6;
	1705	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1290.5;
	1706	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	823.4;
	1706	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	596.3;
	1706	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	793.4;
	1706	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2466.7;
	1706	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7244.1;
	1706	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4072.2;
	1706	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7411.3;
	1706	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1232.7;
	1707	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	823.2;
	1707	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	593.6;
	1707	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	781.5;
	1707	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2387.1;
	1707	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7043.7;
	1707	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3931.8;
	1707	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7221.1;
	1707	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1191.9;
	1708	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	820.5;
	1708	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586.7;
	1708	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	775.2;
	1708	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2346.1;
	1708	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6928.6;
	1708	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3879.6;
	1708	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7071.6;
	1708	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1191.9;
	1709	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	824.5;
	1709	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	587.9;
	1709	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	777.2;
	1709	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2341;
	1709	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6955.4;
	1709	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3894.8;
	1709	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7080.1;
	1709	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1200;
	1710	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	830.6;
	1710	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	611.9;
	1710	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	795.8;
	1710	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2397.8;
	1710	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7186.6;
	1710	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4010.8;
	1710	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7177.8;
	1710	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1207.5;
	1711	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	836.9;
	1711	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	630.7;
	1711	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	825.6;
	1711	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2486.8;
	1711	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7555.9;
	1711	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4226.2;
	1711	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7338.7;
	1711	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1266.6;
	1712	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	838.7;
	1712	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	645.4;
	1712	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.7;
	1712	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2564.1;
	1712	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7918;
	1712	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4444;
	1712	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7479;
	1712	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1302.5;
	1713	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	847.9;
	1713	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	674.4;
	1713	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.9;
	1713	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2742.4;
	1713	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8389.2;
	1713	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4737.7;
	1713	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7988.6;
	1713	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1371.6;
	1714	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	854.7;
	1714	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	703.1;
	1714	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	898;
	1714	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2893.4;
	1714	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8791.4;
	1714	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4926.4;
	1714	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8422.8;
	1714	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1455.4;
	1715	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	860.2;
	1715	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	704.9;
	1715	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	887.8;
	1715	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3023.6;
	1715	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9037.9;
	1715	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5060.7;
	1715	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8744.1;
	1715	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1472.9;
	1716	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	862.9;
	1716	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	697.7;
	1716	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	867.7;
	1716	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3129.7;
	1716	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9098.4;
	1716	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5120.3;
	1716	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9003.7;
	1716	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1462.3;
	1717	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	856.8;
	1717	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	685.2;
	1717	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	842.8;
	1717	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3236.4;
	1717	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9025.6;
	1717	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5158.9;
	1717	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9144.8;
	1717	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1462.9;
	1718	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	860;
	1718	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	678.3;
	1718	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	830.3;
	1718	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3364.8;
	1718	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8908.5;
	1718	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5157.8;
	1718	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9281.6;
	1718	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1416.6;
	1719	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	861.8;
	1719	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	672.3;
	1719	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	820.8;
	1719	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3477.2;
	1719	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8781.8;
	1719	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5181.5;
	1719	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9429;
	1719	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1407.9;
	1720	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	865.6;
	1720	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	671.2;
	1720	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	817.9;
	1720	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3566;
	1720	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8719.8;
	1720	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5189.5;
	1720	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9504.2;
	1720	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1431.6;
	1721	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	865.1;
	1721	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	669.9;
	1721	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	811.5;
	1721	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3594.2;
	1721	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8705.2;
	1721	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5216.8;
	1721	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9481.9;
	1721	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1469.8;
	1722	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867;
	1722	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	673;
	1722	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	792.2;
	1722	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3554;
	1722	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8752.4;
	1722	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5242.6;
	1722	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9402.9;
	1722	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1473.6;
	1723	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.4;
	1723	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	685.5;
	1723	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	800.7;
	1723	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3465.6;
	1723	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8918.9;
	1723	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5292.2;
	1723	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9345.9;
	1723	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1465.7;
	1724	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	881.3;
	1724	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	708.8;
	1724	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	832.6;
	1724	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3514.9;
	1724	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9233.4;
	1724	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5478.6;
	1724	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9445.4;
	1724	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1508.4;
	1725	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.8;
	1725	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	702.3;
	1725	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	826.2;
	1725	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3393.7;
	1725	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9046;
	1725	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5339.2;
	1725	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9210;
	1725	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1502.5;
	1726	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866.5;
	1726	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	683;
	1726	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	805.7;
	1726	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3243.6;
	1726	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8737.2;
	1726	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5135.4;
	1726	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8898.1;
	1726	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1455.5;
	1727	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	853.4;
	1727	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	654;
	1727	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	767.4;
	1727	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3062.6;
	1727	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8247.3;
	1727	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4823.9;
	1727	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8445.2;
	1727	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1386.6;
	1728	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	837.2;
	1728	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	625.1;
	1728	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	729.3;
	1728	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2863.2;
	1728	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7708.1;
	1728	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4480.4;
	1728	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7962.3;
	1728	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1279.9;
	1729	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	826.8;
	1729	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	600.9;
	1729	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	707.2;
	1729	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2730.8;
	1729	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7238.3;
	1729	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4168.8;
	1729	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7487.2;
	1729	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1228.9;
	1730	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	811.4;
	1730	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	593.8;
	1730	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	681.8;
	1730	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2554;
	1730	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6862.3;
	1730	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3962.7;
	1730	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7275;
	1730	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1177;
	1731	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	809.1;
	1731	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.6;
	1731	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	678.5;
	1731	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2473.8;
	1731	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6661.2;
	1731	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3853.2;
	1731	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7120.1;
	1731	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1138.9;
	1732	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	812.9;
	1732	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	587.1;
	1732	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	678.2;
	1732	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2433.3;
	1732	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6608.5;
	1732	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3812.3;
	1732	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7040.2;
	1732	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1152.7;
	1733	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	816.5;
	1733	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.7;
	1733	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	694.9;
	1733	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2435.2;
	1733	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6691.5;
	1733	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3836.2;
	1733	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7070.4;
	1733	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1181.7;
	1734	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	828.4;
	1734	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	600.7;
	1734	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	723;
	1734	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2486.4;
	1734	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6976.3;
	1734	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3976;
	1734	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7224.4;
	1734	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1134.5;
	1735	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	841.5;
	1735	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	618.7;
	1735	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	756.7;
	1735	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2527.6;
	1735	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7289.2;
	1735	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4121.4;
	1735	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7351.6;
	1735	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1178.1;
	1736	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	838.7;
	1736	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	638.2;
	1736	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	776.8;
	1736	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2575.7;
	1736	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7560.7;
	1736	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4282.2;
	1736	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7446.4;
	1736	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1274;
	1737	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	849.7;
	1737	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	648.1;
	1737	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	783;
	1737	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2793.7;
	1737	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7944.4;
	1737	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4530.8;
	1737	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7832.9;
	1737	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1313.7;
	1738	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	852.2;
	1738	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	650.7;
	1738	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	781.7;
	1738	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3012.7;
	1738	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8251.3;
	1738	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4709;
	1738	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8274.6;
	1738	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1375.5;
	1739	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.1;
	1739	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	652.6;
	1739	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	775.5;
	1739	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3236.3;
	1739	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8474.2;
	1739	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4935.3;
	1739	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8696.1;
	1739	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1404;
	1740	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	853.7;
	1740	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	656.2;
	1740	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	779.5;
	1740	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3475.7;
	1740	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8663.9;
	1740	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5106.8;
	1740	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9096.9;
	1740	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1415.5;
	1741	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	858.2;
	1741	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	654.8;
	1741	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	778.3;
	1741	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3734.5;
	1741	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8775.4;
	1741	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5262.6;
	1741	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9424.8;
	1741	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1452;
	1742	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.6;
	1742	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	650.2;
	1742	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	778.2;
	1742	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3972.8;
	1742	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8862.6;
	1742	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5385.1;
	1742	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9756.2;
	1742	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1469.2;
	1743	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.8;
	1743	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	650.8;
	1743	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	782.3;
	1743	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4167.5;
	1743	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8973.2;
	1743	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5554;
	1743	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10062.1;
	1743	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1491.3;
	1744	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871.4;
	1744	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	657.2;
	1744	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	795.8;
	1744	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4302.3;
	1744	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9095.3;
	1744	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5725.3;
	1744	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10292.9;
	1744	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1537.8;
	1745	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.6;
	1745	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	661.6;
	1745	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	807.8;
	1745	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4346.7;
	1745	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9187.6;
	1745	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5890.1;
	1745	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10398.8;
	1745	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1528;
	1746	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.2;
	1746	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	664;
	1746	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	815.7;
	1746	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4244.8;
	1746	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9174;
	1746	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5941.5;
	1746	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10232.5;
	1746	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1536.9;
	1747	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866.8;
	1747	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	678.8;
	1747	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	820.4;
	1747	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4050.8;
	1747	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9247.6;
	1747	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5872.4;
	1747	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10064.2;
	1747	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1576.3;
	1748	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.3;
	1748	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	710.5;
	1748	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	850.9;
	1748	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4019.7;
	1748	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9587.6;
	1748	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5956.7;
	1748	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10134.7;
	1748	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1621.1;
	1749	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	876.3;
	1749	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	700.5;
	1749	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	829.9;
	1749	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3817.1;
	1749	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9305.6;
	1749	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5700.4;
	1749	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9814.9;
	1749	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1565.3;
	1750	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	856.5;
	1750	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	661.1;
	1750	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	786.2;
	1750	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3569.1;
	1750	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8662.8;
	1750	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5276.2;
	1750	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9373;
	1750	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1463.1;
	1751	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	824.6;
	1751	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	610.9;
	1751	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	734.1;
	1751	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3255.7;
	1751	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7839;
	1751	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4753;
	1751	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8714.2;
	1751	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1344.7;
	1752	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	804.2;
	1752	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	576.5;
	1752	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	724.5;
	1752	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2934.4;
	1752	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7204.2;
	1752	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4306.5;
	1752	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8087.2;
	1752	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1225.7;
	1753	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	794;
	1753	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	563.6;
	1753	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	706.7;
	1753	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2716.5;
	1753	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6812.3;
	1753	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4013;
	1753	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7676.4;
	1753	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1136.6;
	1754	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	790.3;
	1754	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	557.1;
	1754	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	697.2;
	1754	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2579.7;
	1754	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6579.7;
	1754	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3852.9;
	1754	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7411.9;
	1754	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1155.7;
	1755	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	789.3;
	1755	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	551.5;
	1755	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	696.5;
	1755	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2513.9;
	1755	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6552.2;
	1755	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3795.1;
	1755	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7314.7;
	1755	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1127.5;
	1756	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	793.1;
	1756	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.3;
	1756	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	708.3;
	1756	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2505.6;
	1756	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6712.2;
	1756	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3843;
	1756	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7413;
	1756	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1147.2;
	1757	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	807.2;
	1757	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	590.9;
	1757	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	741.9;
	1757	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2601.8;
	1757	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7271.9;
	1757	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4116.8;
	1757	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7781;
	1757	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1244.8;
	1758	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	840.4;
	1758	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	648.2;
	1758	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	810.3;
	1758	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2813.1;
	1758	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8227.5;
	1758	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4561.7;
	1758	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8292.2;
	1758	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1401.4;
	1759	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.1;
	1759	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	693.6;
	1759	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868;
	1759	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2974.3;
	1759	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8883.8;
	1759	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4883.2;
	1759	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8621.3;
	1759	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1453.7;
	1760	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	869.1;
	1760	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	698.7;
	1760	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	873.2;
	1760	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3004.5;
	1760	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8962.1;
	1760	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4962.2;
	1760	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8778.9;
	1760	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1427;
	1761	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866.3;
	1761	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	704;
	1761	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.7;
	1761	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3111.4;
	1761	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9106.5;
	1761	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5110.4;
	1761	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9028.8;
	1761	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1519.3;
	1762	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.3;
	1762	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711.8;
	1762	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	864.5;
	1762	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3300.7;
	1762	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9371.3;
	1762	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5319.7;
	1762	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9426.4;
	1762	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1550.6;
	1763	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.3;
	1763	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.2;
	1763	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	863.4;
	1763	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3544.2;
	1763	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9632.3;
	1763	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5561.3;
	1763	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9915.8;
	1763	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1598.8;
	1764	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.8;
	1764	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	712.7;
	1764	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	869;
	1764	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3819.9;
	1764	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9874.1;
	1764	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5837.3;
	1764	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10409.1;
	1764	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1636.5;
	1765	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.1;
	1765	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	724.5;
	1765	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889;
	1765	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4112.1;
	1765	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10198;
	1765	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6185;
	1765	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11029.7;
	1765	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1707.7;
	1766	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.4;
	1766	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.9;
	1766	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	910.7;
	1766	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4366.2;
	1766	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10502.2;
	1766	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6495.4;
	1766	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11551.4;
	1766	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1765.2;
	1767	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.4;
	1767	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744.1;
	1767	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	936.8;
	1767	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4574.8;
	1767	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10766.7;
	1767	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6824.7;
	1767	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11925;
	1767	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1812.4;
	1768	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.2;
	1768	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753.4;
	1768	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	959.9;
	1768	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4646.6;
	1768	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11026.8;
	1768	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7088.8;
	1768	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12123.7;
	1768	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1863.5;
	1769	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925.9;
	1769	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	756;
	1769	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	962.5;
	1769	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4543;
	1769	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11195.5;
	1769	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7257.3;
	1769	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12056.2;
	1769	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1865;
	1770	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.9;
	1770	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	754.9;
	1770	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	959.6;
	1770	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4362.7;
	1770	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11089.8;
	1770	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7264;
	1770	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11598.7;
	1770	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1850.3;
	1771	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.9;
	1771	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	754.3;
	1771	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	943.3;
	1771	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4196.8;
	1771	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10896.4;
	1771	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7037.8;
	1771	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11180.5;
	1771	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1830.2;
	1772	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.9;
	1772	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	770.9;
	1772	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	949.4;
	1772	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4189.3;
	1772	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11046.4;
	1772	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6969.4;
	1772	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11002.4;
	1772	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1858.6;
	1773	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.6;
	1773	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.9;
	1773	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	920.2;
	1773	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4004.6;
	1773	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10558.8;
	1773	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6593.2;
	1773	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10424;
	1773	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1779.9;
	1774	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.1;
	1774	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	695.7;
	1774	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	864.2;
	1774	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3776.9;
	1774	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9757.9;
	1774	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6096.9;
	1774	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9840.5;
	1774	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1641.7;
	1775	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	847.2;
	1775	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	639.2;
	1775	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	798.2;
	1775	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3426.9;
	1775	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8762.6;
	1775	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5482.5;
	1775	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9089.7;
	1775	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1540.2;
	1776	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	826.7;
	1776	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	613.8;
	1776	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	721.6;
	1776	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3111.9;
	1776	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7938;
	1776	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4978.9;
	1776	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8477;
	1776	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1424.2;
	1777	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	812.5;
	1777	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	593.5;
	1777	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	694.1;
	1777	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2904;
	1777	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7442.3;
	1777	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4625.3;
	1777	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8037.4;
	1777	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1351.4;
	1778	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	807.3;
	1778	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	581.6;
	1778	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	682.2;
	1778	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2784.2;
	1778	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7145.6;
	1778	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4400.6;
	1778	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7821;
	1778	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1283.2;
	1779	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	802.6;
	1779	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	576.9;
	1779	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	673.8;
	1779	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2708.3;
	1779	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7004.5;
	1779	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4263.7;
	1779	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7712.7;
	1779	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1286.4;
	1780	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	806.6;
	1780	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	581.1;
	1780	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	677.4;
	1780	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2687.1;
	1780	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7117.7;
	1780	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4282.5;
	1780	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7818.4;
	1780	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1292;
	1781	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	821.4;
	1781	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	603.6;
	1781	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	702.6;
	1781	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2751.7;
	1781	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7594.9;
	1781	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4477;
	1781	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8173.4;
	1781	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1349.8;
	1782	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	846.2;
	1782	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	650.7;
	1782	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	757.8;
	1782	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2975.1;
	1782	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8502.6;
	1782	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4883.9;
	1782	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8728;
	1782	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1473.6;
	1783	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.8;
	1783	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	688.7;
	1783	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	806.2;
	1783	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3132.1;
	1783	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9077.3;
	1783	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5168.9;
	1783	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9104;
	1783	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1465.7;
	1784	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	869.7;
	1784	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	688.4;
	1784	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	814.6;
	1784	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3130.4;
	1784	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9093.5;
	1784	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5245.8;
	1784	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9214.9;
	1784	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1493.6;
	1785	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867.5;
	1785	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	698.6;
	1785	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	825.2;
	1785	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3273.2;
	1785	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9390.2;
	1785	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5406.7;
	1785	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9579.2;
	1785	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1576.1;
	1786	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.4;
	1786	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	712.8;
	1786	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.8;
	1786	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3487.9;
	1786	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9780.2;
	1786	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5607.9;
	1786	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10061.7;
	1786	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1636.3;
	1787	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.1;
	1787	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723.1;
	1787	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	863.3;
	1787	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3719;
	1787	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10112.8;
	1787	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5878.6;
	1787	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10597.1;
	1787	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1699.1;
	1788	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.5;
	1788	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.2;
	1788	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	870.2;
	1788	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3980.7;
	1788	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10404.5;
	1788	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6242;
	1788	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11063;
	1788	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1759.9;
	1789	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.6;
	1789	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742.7;
	1789	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	891.6;
	1789	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4266.5;
	1789	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10722.2;
	1789	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6658.2;
	1789	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11561.5;
	1789	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1806.5;
	1790	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.4;
	1790	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747;
	1790	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	908.6;
	1790	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4499.3;
	1790	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10935;
	1790	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7042.1;
	1790	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11936.4;
	1790	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1875;
	1791	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.9;
	1791	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	750.5;
	1791	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	922.1;
	1791	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4648;
	1791	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11102.4;
	1791	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7380.4;
	1791	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12097;
	1791	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1927.5;
	1792	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925.8;
	1792	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	756.1;
	1792	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	930.7;
	1792	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4687.6;
	1792	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11239;
	1792	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7613.5;
	1792	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12221.1;
	1792	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1961.9;
	1793	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.3;
	1793	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749.3;
	1793	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	928.4;
	1793	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4573.9;
	1793	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11263.2;
	1793	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7705.7;
	1793	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12117.8;
	1793	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1872.6;
	1794	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.2;
	1794	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.8;
	1794	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	918.4;
	1794	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4351.9;
	1794	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10987.8;
	1794	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7509.4;
	1794	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11650;
	1794	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1889;
	1795	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.7;
	1795	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.6;
	1795	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	902;
	1795	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4185.9;
	1795	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10748.9;
	1795	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7278.9;
	1795	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11353.5;
	1795	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1897.9;
	1796	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920.9;
	1796	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	761.1;
	1796	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	915.8;
	1796	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4203.2;
	1796	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10873.9;
	1796	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7176.7;
	1796	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11278.9;
	1796	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1872.2;
	1797	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.3;
	1797	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	734.3;
	1797	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	888.2;
	1797	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4030.5;
	1797	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10357.8;
	1797	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6730;
	1797	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10806.6;
	1797	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1779.2;
	1798	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.6;
	1798	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	686.7;
	1798	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	825.5;
	1798	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3785.9;
	1798	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9504.7;
	1798	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6096.2;
	1798	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10169;
	1798	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1576;
	1799	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	850.2;
	1799	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	637;
	1799	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	759.5;
	1799	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3436.3;
	1799	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8470;
	1799	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5383.6;
	1799	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9401.7;
	1799	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1483.1;
	1800	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	833.6;
	1800	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	606.3;
	1800	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	720.5;
	1800	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3146.8;
	1800	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7720.8;
	1800	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4819.6;
	1800	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8853;
	1800	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1355.9;
	1801	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	822.3;
	1801	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586.7;
	1801	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	692.6;
	1801	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2942.4;
	1801	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7250;
	1801	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4431.8;
	1801	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8421.7;
	1801	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1291.2;
	1802	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	819.8;
	1802	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579.9;
	1802	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	679.4;
	1802	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2814.6;
	1802	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6978.3;
	1802	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4164.9;
	1802	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8105.3;
	1802	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1247.8;
	1803	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	817.3;
	1803	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577.3;
	1803	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	673.7;
	1803	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2735.8;
	1803	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6844.3;
	1803	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4064.9;
	1803	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7901.5;
	1803	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1196.1;
	1804	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	821.1;
	1804	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588.7;
	1804	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	686.5;
	1804	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2709.4;
	1804	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7007.9;
	1804	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4064;
	1804	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7882;
	1804	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1195.5;
	1805	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	840.4;
	1805	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	625.3;
	1805	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	721.5;
	1805	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2794.6;
	1805	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7554;
	1805	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4269.4;
	1805	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8224.9;
	1805	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1201.4;
	1806	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871.4;
	1806	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	682.4;
	1806	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	782.1;
	1806	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3012.3;
	1806	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8491.1;
	1806	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4663.5;
	1806	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8727.2;
	1806	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1349;
	1807	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.9;
	1807	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.4;
	1807	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835;
	1807	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3203.1;
	1807	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9129;
	1807	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4984.4;
	1807	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9079.8;
	1807	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1429.4;
	1808	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.8;
	1808	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.2;
	1808	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	837.3;
	1808	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3227.8;
	1808	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9148.7;
	1808	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5029.4;
	1808	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9251;
	1808	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1421.9;
	1809	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.1;
	1809	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	726;
	1809	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	839.6;
	1809	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3346.4;
	1809	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9282.3;
	1809	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5192.4;
	1809	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9539.1;
	1809	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1452.3;
	1810	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.1;
	1810	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	722.3;
	1810	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	846.1;
	1810	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3461.5;
	1810	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9442;
	1810	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5399.4;
	1810	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9904.3;
	1810	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1475.5;
	1811	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.2;
	1811	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	722.3;
	1811	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	876.5;
	1811	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3593.2;
	1811	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9588.8;
	1811	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5580;
	1811	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10289.1;
	1811	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1495.6;
	1812	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.1;
	1812	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	718.1;
	1812	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	875.7;
	1812	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3691.3;
	1812	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9694.7;
	1812	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5811.4;
	1812	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10509.1;
	1812	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1515.3;
	1813	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.4;
	1813	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719.2;
	1813	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	891.6;
	1813	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3786.1;
	1813	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9821.7;
	1813	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6104.3;
	1813	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10770;
	1813	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1560.5;
	1814	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920.4;
	1814	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	722.6;
	1814	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	896;
	1814	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3876.4;
	1814	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9960.1;
	1814	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6349.6;
	1814	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10991.9;
	1814	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1605.3;
	1815	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.4;
	1815	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.2;
	1815	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	893;
	1815	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3941.2;
	1815	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10131.2;
	1815	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6555.6;
	1815	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11113.7;
	1815	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1649.6;
	1816	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.5;
	1816	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.7;
	1816	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	912;
	1816	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3965;
	1816	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10364.6;
	1816	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6684.1;
	1816	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11226;
	1816	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1690.5;
	1817	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.3;
	1817	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	732.8;
	1817	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	918.2;
	1817	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3923.1;
	1817	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10471.9;
	1817	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6714.9;
	1817	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11084.9;
	1817	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1697.3;
	1818	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.8;
	1818	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	731.8;
	1818	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	926.2;
	1818	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3841.1;
	1818	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10445.8;
	1818	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6628.6;
	1818	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10800;
	1818	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1684.5;
	1819	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.2;
	1819	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.1;
	1819	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	920.2;
	1819	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3804.2;
	1819	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10340.2;
	1819	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6542.8;
	1819	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10765.8;
	1819	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1680.7;
	1820	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.9;
	1820	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.9;
	1820	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	930.1;
	1820	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3896.4;
	1820	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10545.9;
	1820	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6652.7;
	1820	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10825.5;
	1820	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1720.1;
	1821	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.4;
	1821	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.2;
	1821	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	898.9;
	1821	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3796.4;
	1821	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10133.6;
	1821	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6405.7;
	1821	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10462.2;
	1821	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1639.4;
	1822	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.6;
	1822	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	682.4;
	1822	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.7;
	1822	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3601.9;
	1822	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9326.2;
	1822	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5991.1;
	1822	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9941.1;
	1822	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1511.1;
	1823	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	858.1;
	1823	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	637.6;
	1823	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	784.9;
	1823	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3326.3;
	1823	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8406.9;
	1823	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5422.7;
	1823	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9312.8;
	1823	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1374.2;
	1824	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	838.4;
	1824	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	606.4;
	1824	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	743.7;
	1824	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3066.9;
	1824	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7777.3;
	1824	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4842.1;
	1824	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8719.6;
	1824	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1267.1;
	1825	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	828.9;
	1825	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588.3;
	1825	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	714.5;
	1825	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2878.3;
	1825	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7345.2;
	1825	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4592.7;
	1825	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8308.2;
	1825	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1210.4;
	1826	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	823.5;
	1826	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	580.7;
	1826	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	696.6;
	1826	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2768.1;
	1826	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7082.7;
	1826	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4393.6;
	1826	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8018.1;
	1826	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1168.7;
	1827	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	817.8;
	1827	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579.4;
	1827	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	685.6;
	1827	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2705.5;
	1827	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6989.6;
	1827	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4271.3;
	1827	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7850;
	1827	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1147.7;
	1828	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	817.5;
	1828	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	587.3;
	1828	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	689.1;
	1828	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2695.3;
	1828	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7079.4;
	1828	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4305.8;
	1828	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7920.7;
	1828	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1165.8;
	1829	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	827.1;
	1829	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	616;
	1829	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	711.6;
	1829	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2785.5;
	1829	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7566.5;
	1829	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4555.7;
	1829	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8280.3;
	1829	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1239.7;
	1830	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	852.2;
	1830	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	668.2;
	1830	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	767.3;
	1830	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3006.9;
	1830	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8454;
	1830	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5031.2;
	1830	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8845;
	1830	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1386.1;
	1831	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.8;
	1831	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	710.9;
	1831	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	820.5;
	1831	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3200.4;
	1831	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9049.6;
	1831	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5329;
	1831	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9254.8;
	1831	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1464.2;
	1832	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	881.5;
	1832	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	716.8;
	1832	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	828.8;
	1832	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3207.8;
	1832	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9149.5;
	1832	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5375.1;
	1832	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9403.5;
	1832	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1477.6;
	1833	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.2;
	1833	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.9;
	1833	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	830.2;
	1833	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3333.8;
	1833	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9366.2;
	1833	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5522.9;
	1833	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9702;
	1833	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1498.7;
	1834	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892;
	1834	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.8;
	1834	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	844.7;
	1834	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3512.7;
	1834	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9654.3;
	1834	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5657.3;
	1834	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10178.9;
	1834	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1539.3;
	1835	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.9;
	1835	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742.6;
	1835	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.1;
	1835	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3670.4;
	1835	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9761.7;
	1835	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5759.2;
	1835	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10594;
	1835	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1567.2;
	1836	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.3;
	1836	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	734;
	1836	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861;
	1836	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3738.5;
	1836	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9772.9;
	1836	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5859.5;
	1836	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10813.6;
	1836	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1579.8;
	1837	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.1;
	1837	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	726.1;
	1837	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.2;
	1837	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3854.6;
	1837	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9770.9;
	1837	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6046.6;
	1837	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11035.1;
	1837	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1597.5;
	1838	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.3;
	1838	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720.6;
	1838	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	895.3;
	1838	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3993.1;
	1838	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9760.2;
	1838	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6224.9;
	1838	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11201.3;
	1838	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1602.2;
	1839	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927;
	1839	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	715.5;
	1839	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	908.1;
	1839	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4118.2;
	1839	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9707.7;
	1839	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6378.1;
	1839	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11315.5;
	1839	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1589.4;
	1840	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.3;
	1840	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.2;
	1840	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	921.3;
	1840	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4171.6;
	1840	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9743.7;
	1840	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6523.7;
	1840	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11383.5;
	1840	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1602.7;
	1841	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.1;
	1841	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714.8;
	1841	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	924.6;
	1841	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4116.7;
	1841	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9705.8;
	1841	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6605.4;
	1841	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11251.4;
	1841	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1617.9;
	1842	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931;
	1842	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714.8;
	1842	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	919.2;
	1842	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3993.2;
	1842	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9654.2;
	1842	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6589.9;
	1842	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10978.1;
	1842	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1662.8;
	1843	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.1;
	1843	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723.3;
	1843	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	910;
	1843	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3926.4;
	1843	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9793.5;
	1843	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6528.9;
	1843	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10903.7;
	1843	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1700;
	1844	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.5;
	1844	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744.7;
	1844	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	910.9;
	1844	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4000;
	1844	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10019.5;
	1844	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6616.9;
	1844	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10945.4;
	1844	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1725.3;
	1845	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.9;
	1845	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.5;
	1845	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	879.8;
	1845	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3886.2;
	1845	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9706.5;
	1845	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6322.9;
	1845	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10550.4;
	1845	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1634.9;
	1846	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.2;
	1846	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	686.1;
	1846	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	830.8;
	1846	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3695.5;
	1846	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8993.7;
	1846	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5864.3;
	1846	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10029.4;
	1846	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1527.6;
	1847	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	861.2;
	1847	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	649.7;
	1847	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	774.7;
	1847	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3427.6;
	1847	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8196.8;
	1847	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5321;
	1847	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9392.4;
	1847	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1390.9;
	1848	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	842.3;
	1848	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	614.4;
	1848	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	753.1;
	1848	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3154.2;
	1848	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7599;
	1848	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4918.7;
	1848	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8772.9;
	1848	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1292.3;
	1849	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	826.7;
	1849	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	602.2;
	1849	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	718.6;
	1849	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2952.2;
	1849	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7230.9;
	1849	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4658.3;
	1849	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8359.1;
	1849	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1253.8;
	1850	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	823;
	1850	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	600.7;
	1850	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	694.7;
	1850	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2814.3;
	1850	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6986.7;
	1850	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4513.5;
	1850	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8099.3;
	1850	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1231.5;
	1851	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	818;
	1851	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589.7;
	1851	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	684.1;
	1851	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2728;
	1851	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6904.4;
	1851	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4402.1;
	1851	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7935.2;
	1851	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1201.5;
	1852	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	819.8;
	1852	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	603.5;
	1852	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	693.8;
	1852	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2712.5;
	1852	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7046.1;
	1852	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4431.1;
	1852	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7993.3;
	1852	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1193.2;
	1853	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	831.5;
	1853	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	635;
	1853	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	722.8;
	1853	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2770.8;
	1853	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7553.9;
	1853	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4626.3;
	1853	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8318.9;
	1853	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1294.8;
	1854	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	861.7;
	1854	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	687.9;
	1854	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	780.8;
	1854	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2977.3;
	1854	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8433.6;
	1854	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5001.7;
	1854	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8828.4;
	1854	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1419.8;
	1855	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.7;
	1855	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	721.5;
	1855	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	826.3;
	1855	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3127;
	1855	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9015.5;
	1855	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5313.2;
	1855	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9117.4;
	1855	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1504.6;
	1856	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.9;
	1856	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.7;
	1856	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	829.7;
	1856	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3160.5;
	1856	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9116.2;
	1856	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5451.9;
	1856	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9264.7;
	1856	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1503.7;
	1857	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.5;
	1857	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.7;
	1857	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	836.4;
	1857	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3320.2;
	1857	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9333.5;
	1857	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5625.3;
	1857	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9637.2;
	1857	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1561.7;
	1858	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.1;
	1858	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.1;
	1858	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849.9;
	1858	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3504.9;
	1858	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9531;
	1858	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5818.7;
	1858	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10059.6;
	1858	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1600.8;
	1859	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897;
	1859	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	757;
	1859	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.1;
	1859	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3691.1;
	1859	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9673.6;
	1859	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5943;
	1859	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10511;
	1859	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1613.3;
	1860	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.7;
	1860	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	759.9;
	1860	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.2;
	1860	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3893.4;
	1860	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9752.5;
	1860	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6024.6;
	1860	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11034.6;
	1860	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1648;
	1861	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904;
	1861	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	761.2;
	1861	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	875.3;
	1861	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4137.3;
	1861	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9749.6;
	1861	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6176.5;
	1861	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11551.2;
	1861	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1656.3;
	1862	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.7;
	1862	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	754.6;
	1862	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.1;
	1862	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4371.4;
	1862	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9696.4;
	1862	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6377.3;
	1862	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11972.5;
	1862	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1644.9;
	1863	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.8;
	1863	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.7;
	1863	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	887.5;
	1863	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4546.8;
	1863	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9604.3;
	1863	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6611.7;
	1863	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12222.9;
	1863	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1677.8;
	1864	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.6;
	1864	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744;
	1864	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	880.8;
	1864	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4652.3;
	1864	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9555.9;
	1864	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6817.8;
	1864	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12297.9;
	1864	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1704.7;
	1865	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.8;
	1865	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742.8;
	1865	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	867.7;
	1865	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4589.1;
	1865	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9538.2;
	1865	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6911.7;
	1865	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12088.8;
	1865	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1700.6;
	1866	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.7;
	1866	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746;
	1866	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.3;
	1866	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4360.2;
	1866	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9505;
	1866	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6747.8;
	1866	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11480.7;
	1866	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1693.5;
	1867	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.3;
	1867	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	758.6;
	1867	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866;
	1867	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4214.4;
	1867	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9606.3;
	1867	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6612.7;
	1867	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11066.7;
	1867	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1698.2;
	1868	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.6;
	1868	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	781.3;
	1868	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884.6;
	1868	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4192.3;
	1868	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9712.7;
	1868	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6215.6;
	1868	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10906;
	1868	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1684.3;
	1869	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.5;
	1869	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	773.2;
	1869	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	856.9;
	1869	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3986.3;
	1869	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9439.8;
	1869	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5798.8;
	1869	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10424.2;
	1869	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1627.5;
	1870	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.5;
	1870	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.5;
	1870	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824.4;
	1870	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3728.7;
	1870	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8991.6;
	1870	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5391.6;
	1870	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9900.6;
	1870	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1531.7;
	1871	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.9;
	1871	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	726;
	1871	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	793.9;
	1871	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3434.8;
	1871	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8436;
	1871	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4924.5;
	1871	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9317.1;
	1871	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1393.9;
	1872	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	876.5;
	1872	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	700.9;
	1872	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	790.5;
	1872	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3104.9;
	1872	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7975.5;
	1872	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4489.2;
	1872	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8776.3;
	1872	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1321;
	1873	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.9;
	1873	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	689.7;
	1873	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	772.2;
	1873	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2779.3;
	1873	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7676.3;
	1873	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4158.7;
	1873	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8345.7;
	1873	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1249.4;
	1874	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867.9;
	1874	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	684.3;
	1874	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	761.4;
	1874	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2566.7;
	1874	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7542.1;
	1874	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3963;
	1874	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7987.8;
	1874	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1202.9;
	1875	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866.5;
	1875	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	686.1;
	1875	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	761.3;
	1875	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2444.3;
	1875	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7559.7;
	1875	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3861.2;
	1875	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7701.8;
	1875	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1202.4;
	1876	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.7;
	1876	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	701;
	1876	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	771.4;
	1876	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2394;
	1876	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7707.1;
	1876	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3845;
	1876	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7553.6;
	1876	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1212.4;
	1877	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.6;
	1877	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	726.6;
	1877	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	794.1;
	1877	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2408.5;
	1877	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8102.4;
	1877	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3935.7;
	1877	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7531.9;
	1877	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1224.5;
	1878	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.3;
	1878	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	754.4;
	1878	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	833;
	1878	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2463;
	1878	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8684.7;
	1878	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4164.6;
	1878	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7639.3;
	1878	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1307;
	1879	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.2;
	1879	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	791.7;
	1879	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.2;
	1879	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2542.1;
	1879	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9383.2;
	1879	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4459.8;
	1879	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7795.8;
	1879	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1377.6;
	1880	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.4;
	1880	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	809.1;
	1880	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	924.8;
	1880	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2602.4;
	1880	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9916.1;
	1880	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4735.9;
	1880	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7971.8;
	1880	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1433;
	1881	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.9;
	1881	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	803.3;
	1881	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	945.3;
	1881	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2728.9;
	1881	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10163.6;
	1881	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5010.5;
	1881	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8223.3;
	1881	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1516.1;
	1882	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.9;
	1882	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	786.8;
	1882	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	942.6;
	1882	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2856.8;
	1882	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10142.2;
	1882	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5158.5;
	1882	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8450.6;
	1882	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1529.5;
	1883	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.5;
	1883	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	758.4;
	1883	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	919.2;
	1883	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2925.7;
	1883	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9931;
	1883	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5172.8;
	1883	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8524.7;
	1883	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1477.8;
	1884	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.9;
	1884	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.9;
	1884	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	887.3;
	1884	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2941.3;
	1884	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9615.2;
	1884	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5097.1;
	1884	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8488.3;
	1884	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1443.7;
	1885	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.3;
	1885	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	705.8;
	1885	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.9;
	1885	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2927.4;
	1885	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9255.6;
	1885	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4966.9;
	1885	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8455.1;
	1885	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1371.8;
	1886	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.7;
	1886	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	682.5;
	1886	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	837.6;
	1886	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2894.3;
	1886	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8897.9;
	1886	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4848.2;
	1886	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8398.4;
	1886	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1337.8;
	1887	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866.1;
	1887	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	667.6;
	1887	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824.2;
	1887	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2881.9;
	1887	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8648.7;
	1887	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4763.7;
	1887	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8342.9;
	1887	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1349.1;
	1888	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.7;
	1888	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	657.9;
	1888	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	809.7;
	1888	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2882.1;
	1888	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8535.1;
	1888	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4750;
	1888	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8275.2;
	1888	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1385.7;
	1889	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.1;
	1889	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	656.8;
	1889	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	786.6;
	1889	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2875.2;
	1889	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8483.9;
	1889	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4768.3;
	1889	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8260.4;
	1889	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1391.4;
	1890	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.1;
	1890	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	658.8;
	1890	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	790;
	1890	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2860.5;
	1890	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8495;
	1890	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4812.5;
	1890	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8210.9;
	1890	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1379.3;
	1891	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	859.9;
	1891	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	680.5;
	1891	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	808.4;
	1891	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2923;
	1891	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8803.2;
	1891	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4965.1;
	1891	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8340.5;
	1891	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1443.3;
	1892	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.3;
	1892	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.2;
	1892	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	853.7;
	1892	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3073.7;
	1892	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9334.7;
	1892	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5287.1;
	1892	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8530.4;
	1892	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1546.2;
	1893	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.6;
	1893	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.9;
	1893	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.6;
	1893	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3026.5;
	1893	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9306.9;
	1893	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5211.7;
	1893	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8457.7;
	1893	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1537.4;
	1894	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.8;
	1894	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711.8;
	1894	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	832.5;
	1894	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2942.1;
	1894	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9035;
	1894	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5022.6;
	1894	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8202.3;
	1894	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1494.5;
	1895	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.4;
	1895	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	693.7;
	1895	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	803.9;
	1895	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2791;
	1895	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8642.1;
	1895	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4749;
	1895	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7850.4;
	1895	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1402.4;
	1896	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870;
	1896	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	678.1;
	1896	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	787.5;
	1896	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2635.4;
	1896	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8287.8;
	1896	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4514.6;
	1896	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7487.7;
	1896	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1354;
	1897	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	863.4;
	1897	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	670.7;
	1897	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	783.3;
	1897	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2506.5;
	1897	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8058.2;
	1897	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4342.9;
	1897	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7259.7;
	1897	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1326.8;
	1898	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.1;
	1898	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	679.1;
	1898	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	786.7;
	1898	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2421.1;
	1898	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7972.2;
	1898	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4247.8;
	1898	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7129.8;
	1898	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1304.1;
	1899	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	865.9;
	1899	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	688.1;
	1899	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	791.8;
	1899	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2385.6;
	1899	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7990.7;
	1899	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4229.5;
	1899	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7066.6;
	1899	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1307.5;
	1900	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.9;
	1900	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	701.1;
	1900	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	805.4;
	1900	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2375.5;
	1900	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8117.4;
	1900	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4277;
	1900	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7081.5;
	1900	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1338.8;
	1901	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.4;
	1901	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735.6;
	1901	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	831;
	1901	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2406.6;
	1901	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8429.8;
	1901	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4424.6;
	1901	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7187.7;
	1901	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1389.9;
	1902	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.4;
	1902	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	773.2;
	1902	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	874.2;
	1902	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2498.2;
	1902	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8986.9;
	1902	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4673.6;
	1902	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7424.4;
	1902	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1378.5;
	1903	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.8;
	1903	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	818.7;
	1903	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	925.9;
	1903	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2588.8;
	1903	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9580.3;
	1903	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4988.2;
	1903	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7600.7;
	1903	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1465.3;
	1904	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.8;
	1904	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	832.4;
	1904	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	947.8;
	1904	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2674.2;
	1904	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9882.7;
	1904	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5212.5;
	1904	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7812.3;
	1904	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1583.3;
	1905	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.6;
	1905	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	823.1;
	1905	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	929.8;
	1905	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2812;
	1905	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9968.6;
	1905	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5285;
	1905	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7996.6;
	1905	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1596.5;
	1906	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.2;
	1906	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	798.9;
	1906	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	899.6;
	1906	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2864.6;
	1906	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9895.8;
	1906	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5236.6;
	1906	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8109.3;
	1906	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1594.4;
	1907	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.8;
	1907	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	775;
	1907	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	871.5;
	1907	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2873;
	1907	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9713.7;
	1907	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5099.8;
	1907	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8160.7;
	1907	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1566.9;
	1908	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.4;
	1908	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755.5;
	1908	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.5;
	1908	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2859.1;
	1908	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9512;
	1908	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4989.1;
	1908	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8181.7;
	1908	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1521.3;
	1909	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867.6;
	1909	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.9;
	1909	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	818.9;
	1909	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2846.3;
	1909	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9228.8;
	1909	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4879.3;
	1909	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8141.6;
	1909	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1496.5;
	1910	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	861.2;
	1910	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	697.2;
	1910	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	800.9;
	1910	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2855.4;
	1910	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8949.5;
	1910	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4790.5;
	1910	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8120.7;
	1910	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1458.2;
	1911	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	856.4;
	1911	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	679.4;
	1911	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	788.4;
	1911	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2873.8;
	1911	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8729.4;
	1911	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4759.2;
	1911	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8042;
	1911	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1429.7;
	1912	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855;
	1912	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	673.9;
	1912	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	792.4;
	1912	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2905.9;
	1912	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8643.5;
	1912	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4781;
	1912	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8012.5;
	1912	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1446.9;
	1913	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	856.9;
	1913	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	678.7;
	1913	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	801;
	1913	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2931.9;
	1913	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8701.2;
	1913	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4869;
	1913	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8043.9;
	1913	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1430.3;
	1914	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	853.8;
	1914	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	687.2;
	1914	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	817.9;
	1914	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2927.4;
	1914	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8844.1;
	1914	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4978.7;
	1914	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8102.5;
	1914	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1478.5;
	1915	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	853.8;
	1915	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	712.8;
	1915	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	839.3;
	1915	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2988.9;
	1915	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9296.3;
	1915	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5198;
	1915	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8355.1;
	1915	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1591.5;
	1916	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.9;
	1916	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	766.7;
	1916	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	883.3;
	1916	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3233.6;
	1916	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10034.3;
	1916	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5617.7;
	1916	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8817.6;
	1916	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1702.7;
	1917	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.3;
	1917	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	763;
	1917	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	871.8;
	1917	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3234;
	1917	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9987;
	1917	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5540.1;
	1917	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8704.4;
	1917	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1691.6;
	1918	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.9;
	1918	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	732.9;
	1918	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	836.3;
	1918	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3091.8;
	1918	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9500.1;
	1918	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5176;
	1918	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8352.6;
	1918	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1625.5;
	1919	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	854;
	1919	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	698.4;
	1919	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	794.9;
	1919	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2832.4;
	1919	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8907.6;
	1919	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4732.3;
	1919	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7976.9;
	1919	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1541.1;
	1920	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	845.4;
	1920	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	682.5;
	1920	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	795;
	1920	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2623.9;
	1920	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8514.3;
	1920	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4493.7;
	1920	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7677;
	1920	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1470.8;
	1921	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	842.1;
	1921	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	682.1;
	1921	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	794;
	1921	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2514.2;
	1921	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8422.1;
	1921	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4402.1;
	1921	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7524.9;
	1921	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1425.6;
	1922	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	846.7;
	1922	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	691.4;
	1922	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	800.9;
	1922	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2468.7;
	1922	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8483.4;
	1922	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4404.3;
	1922	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7508.3;
	1922	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1491.3;
	1923	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	852.2;
	1923	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	706.8;
	1923	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	817.1;
	1923	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2467.7;
	1923	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8726.8;
	1923	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4491.1;
	1923	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7603.8;
	1923	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1503.2;
	1924	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	861.6;
	1924	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.6;
	1924	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	842.1;
	1924	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2528.3;
	1924	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9152.3;
	1924	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4727.3;
	1924	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7905.5;
	1924	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1555.4;
	1925	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884;
	1925	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	784.2;
	1925	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	903.7;
	1925	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2743.5;
	1925	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10189.8;
	1925	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5329.4;
	1925	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8620;
	1925	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1720.2;
	1926	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.2;
	1926	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	873.6;
	1926	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1018.7;
	1926	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3243.5;
	1926	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11810.4;
	1926	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6294.9;
	1926	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9622.1;
	1926	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1977;
	1927	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.9;
	1927	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	917.9;
	1927	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1080.6;
	1927	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3474.8;
	1927	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12557.7;
	1927	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6739.2;
	1927	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9960.5;
	1927	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2052.9;
	1928	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.4;
	1928	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	892.6;
	1928	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1049.3;
	1928	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3325.7;
	1928	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12043.6;
	1928	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6514.7;
	1928	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9908.7;
	1928	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1963.6;
	1929	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.4;
	1929	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	861.8;
	1929	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1005.2;
	1929	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3246.2;
	1929	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11452.8;
	1929	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6193.8;
	1929	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9792.8;
	1929	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1922.3;
	1930	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925;
	1930	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	832.7;
	1930	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	971.4;
	1930	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3180.4;
	1930	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11045.7;
	1930	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5931.2;
	1930	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9699.3;
	1930	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1825.1;
	1931	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.9;
	1931	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	799.1;
	1931	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	933.6;
	1931	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3120.6;
	1931	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10605.7;
	1931	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5690.2;
	1931	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9547.7;
	1931	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1748.6;
	1932	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.9;
	1932	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	768.7;
	1932	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	902.2;
	1932	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3088.1;
	1932	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10205.5;
	1932	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5486.1;
	1932	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9389.6;
	1932	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1667;
	1933	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.2;
	1933	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.5;
	1933	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	880.2;
	1933	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3090.5;
	1933	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9857.8;
	1933	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5374;
	1933	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9287.8;
	1933	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1611;
	1934	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889;
	1934	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719.9;
	1934	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.4;
	1934	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3108.1;
	1934	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9558;
	1934	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5249.7;
	1934	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9190.2;
	1934	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1576.9;
	1935	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887;
	1935	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	703.1;
	1935	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	855.6;
	1935	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3152.5;
	1935	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9313.8;
	1935	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5205.5;
	1935	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9112.8;
	1935	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1540.9;
	1936	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.8;
	1936	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	697.3;
	1936	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.4;
	1936	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3219.2;
	1936	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9268.6;
	1936	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5231.8;
	1936	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9047.7;
	1936	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1542.4;
	1937	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.5;
	1937	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	696.5;
	1937	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	856.9;
	1937	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3221.6;
	1937	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9293.3;
	1937	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5266.6;
	1937	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8975;
	1937	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1527.4;
	1938	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.7;
	1938	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	702;
	1938	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.3;
	1938	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3197.3;
	1938	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9330.1;
	1938	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5334.5;
	1938	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8847.9;
	1938	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1564.6;
	1939	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.2;
	1939	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	718.8;
	1939	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	870.6;
	1939	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3235.8;
	1939	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9605.5;
	1939	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5479.4;
	1939	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9015.3;
	1939	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1622;
	1940	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.6;
	1940	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	760.2;
	1940	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	914.7;
	1940	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3447.6;
	1940	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10174.4;
	1940	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5811.7;
	1940	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9336.4;
	1940	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1705.4;
	1941	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.9;
	1941	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.7;
	1941	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	904.3;
	1941	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3405.8;
	1941	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9966.3;
	1941	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5659.4;
	1941	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9092.1;
	1941	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1674.1;
	1942	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	876.6;
	1942	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	705.8;
	1942	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.6;
	1942	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3199.2;
	1942	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9240.1;
	1942	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5237;
	1942	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8557.1;
	1942	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1556.5;
	1943	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	846.5;
	1943	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	659.7;
	1943	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	801.7;
	1943	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2890.4;
	1943	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8458.9;
	1943	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4766.8;
	1943	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7983.5;
	1943	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1461.4;
	1944	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	826.6;
	1944	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	655;
	1944	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	741.3;
	1944	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2631.3;
	1944	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8008.6;
	1944	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4451.9;
	1944	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7657;
	1944	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1408.8;
	1945	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	822.2;
	1945	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	639.9;
	1945	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	733.1;
	1945	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2505;
	1945	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7799.7;
	1945	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4315.1;
	1945	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7486;
	1945	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1394.1;
	1946	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	821.5;
	1946	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	639.4;
	1946	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	733.4;
	1946	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2448.4;
	1946	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7721.9;
	1946	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4257.8;
	1946	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7407.2;
	1946	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1360.5;
	1947	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	819.6;
	1947	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	646.9;
	1947	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	734;
	1947	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2435.5;
	1947	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7792.3;
	1947	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4277.4;
	1947	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7433.3;
	1947	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1390.7;
	1948	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	826;
	1948	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	662.3;
	1948	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	750.6;
	1948	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2487.1;
	1948	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8116.3;
	1948	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4452;
	1948	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7675.3;
	1948	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1430;
	1949	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	846.4;
	1949	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	700.2;
	1949	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	796.5;
	1949	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2659.1;
	1949	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8960.9;
	1949	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4917.3;
	1949	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8269.5;
	1949	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1554.4;
	1950	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.4;
	1950	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	775;
	1950	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.7;
	1950	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3106.2;
	1950	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10369.2;
	1950	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5694;
	1950	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9138.5;
	1950	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1755.6;
	1951	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.8;
	1951	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	810.7;
	1951	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	943.2;
	1951	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3290.7;
	1951	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10993;
	1951	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6002.7;
	1951	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9469.3;
	1951	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1751;
	1952	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.8;
	1952	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	792.8;
	1952	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	917.1;
	1952	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3143.6;
	1952	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10573.2;
	1952	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5765.9;
	1952	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9368.9;
	1952	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1720.7;
	1953	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.1;
	1953	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	774.9;
	1953	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	890.2;
	1953	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3113.7;
	1953	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10265.4;
	1953	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5570.8;
	1953	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9298;
	1953	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1694.9;
	1954	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.1;
	1954	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	761.1;
	1954	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	876.2;
	1954	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3135.8;
	1954	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10085.2;
	1954	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5424.5;
	1954	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9302;
	1954	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1643.1;
	1955	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.6;
	1955	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.9;
	1955	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.1;
	1955	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3178.2;
	1955	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9873.6;
	1955	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5300.9;
	1955	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9322.3;
	1955	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1612.8;
	1956	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.1;
	1956	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729;
	1956	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	840.3;
	1956	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3216.6;
	1956	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9671.7;
	1956	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5240.5;
	1956	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9288.5;
	1956	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1592.8;
	1957	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.8;
	1957	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723;
	1957	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	840.5;
	1957	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3293.2;
	1957	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9597.9;
	1957	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5245.9;
	1957	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9345.8;
	1957	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1547;
	1958	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.4;
	1958	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	718.4;
	1958	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	842.6;
	1958	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3382.3;
	1958	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9509.6;
	1958	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5274.3;
	1958	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9316;
	1958	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1543.3;
	1959	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.2;
	1959	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	716.4;
	1959	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	846.7;
	1959	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3474.3;
	1959	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9514;
	1959	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5353.2;
	1959	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9287.4;
	1959	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1548.3;
	1960	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.6;
	1960	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720.7;
	1960	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	856.8;
	1960	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3553.7;
	1960	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9582.1;
	1960	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5472.2;
	1960	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9269.1;
	1960	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1559.8;
	1961	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.4;
	1961	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	718.5;
	1961	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	864.6;
	1961	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3562.1;
	1961	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9639;
	1961	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5629.5;
	1961	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9203.9;
	1961	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1478.6;
	1962	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.5;
	1962	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	718.4;
	1962	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	864.9;
	1962	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3501.9;
	1962	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9604.1;
	1962	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5635.4;
	1962	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9077.3;
	1962	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1540.7;
	1963	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.9;
	1963	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.4;
	1963	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	863.7;
	1963	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3464.1;
	1963	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9802.5;
	1963	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5724.6;
	1963	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9243.9;
	1963	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1615;
	1964	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925.7;
	1964	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	763.4;
	1964	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	898.4;
	1964	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3657.8;
	1964	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10285.3;
	1964	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6006.9;
	1964	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9482.1;
	1964	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1643.5;
	1965	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.3;
	1965	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.2;
	1965	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	880.7;
	1965	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3590.9;
	1965	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9934.5;
	1965	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5802.4;
	1965	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9222.1;
	1965	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1591.7;
	1966	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.3;
	1966	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	692.7;
	1966	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	821.2;
	1966	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3372.3;
	1966	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9166.4;
	1966	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5335.7;
	1966	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8689.4;
	1966	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1423.1;
	1967	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	860.5;
	1967	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	641.5;
	1967	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	758;
	1967	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3025.5;
	1967	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8246.8;
	1967	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4774.9;
	1967	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8061.4;
	1967	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1371.5;
	1968	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	836.3;
	1968	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	618.4;
	1968	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	716.4;
	1968	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2746.3;
	1968	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7685.1;
	1968	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4372.5;
	1968	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7607.5;
	1968	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1280.9;
	1969	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	825.4;
	1969	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	616.6;
	1969	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	691.5;
	1969	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2586.6;
	1969	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7280.7;
	1969	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4144;
	1969	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7356.6;
	1969	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1252.1;
	1970	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	823.1;
	1970	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	604.5;
	1970	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	679;
	1970	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2501;
	1970	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7082.2;
	1970	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4025.8;
	1970	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7232.3;
	1970	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1226.6;
	1971	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	821.3;
	1971	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	604;
	1971	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	677.4;
	1971	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2470.8;
	1971	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7032;
	1971	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3969.6;
	1971	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7225.2;
	1971	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1190.1;
	1972	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	824.2;
	1972	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	608.9;
	1972	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	688.6;
	1972	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2473.1;
	1972	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7224.5;
	1972	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4045.1;
	1972	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7376.7;
	1972	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1195.4;
	1973	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	841.6;
	1973	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	639.7;
	1973	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	721.3;
	1973	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2613.9;
	1973	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7846.8;
	1973	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4392.5;
	1973	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7909.9;
	1973	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1217;
	1974	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.5;
	1974	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	704.2;
	1974	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	800;
	1974	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2996.8;
	1974	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9007.9;
	1974	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5084.1;
	1974	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8721.3;
	1974	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1392.7;
	1975	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.3;
	1975	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.5;
	1975	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	848.4;
	1975	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3160.7;
	1975	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9609;
	1975	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5412.7;
	1975	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9054.4;
	1975	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1469.7;
	1976	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.4;
	1976	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.1;
	1976	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	834.8;
	1976	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3079.6;
	1976	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9548;
	1976	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5364;
	1976	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9087.9;
	1976	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1452.8;
	1977	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.9;
	1977	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	734.5;
	1977	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	841.2;
	1977	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3169.4;
	1977	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9693.2;
	1977	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5455;
	1977	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9292.1;
	1977	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1480.3;
	1978	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902;
	1978	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742;
	1978	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	855.9;
	1978	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3288.4;
	1978	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9885.3;
	1978	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5549.3;
	1978	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9510.5;
	1978	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1502.3;
	1979	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909;
	1979	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.9;
	1979	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889.8;
	1979	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3404.2;
	1979	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10046.4;
	1979	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5627.4;
	1979	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9671.9;
	1979	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1517.7;
	1980	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.2;
	1980	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749.1;
	1980	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	895;
	1980	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3533.3;
	1980	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10244.5;
	1980	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5710.4;
	1980	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9731.6;
	1980	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1530.3;
	1981	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.6;
	1981	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	761.5;
	1981	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	917.7;
	1981	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3737.9;
	1981	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10510.4;
	1981	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5869.3;
	1981	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9789.8;
	1981	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1557;
	1982	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920.7;
	1982	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	768.4;
	1982	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	922.8;
	1982	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3957.9;
	1982	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10722.4;
	1982	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6049.8;
	1982	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9776.5;
	1982	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1580.9;
	1983	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.5;
	1983	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	773.1;
	1983	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	918.1;
	1983	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4166.5;
	1983	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10922.8;
	1983	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6261.2;
	1983	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9751.9;
	1983	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1589.9;
	1984	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.5;
	1984	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	781.6;
	1984	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	934.8;
	1984	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4329.9;
	1984	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11123.6;
	1984	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6477.5;
	1984	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9652.4;
	1984	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1606.2;
	1985	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.6;
	1985	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776.2;
	1985	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940.6;
	1985	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4331.5;
	1985	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11207.1;
	1985	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6633.5;
	1985	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9548.9;
	1985	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1602.6;
	1986	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.3;
	1986	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	774.6;
	1986	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	932.9;
	1986	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4204.9;
	1986	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11166.2;
	1986	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6660.9;
	1986	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9503.3;
	1986	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1600.9;
	1987	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.6;
	1987	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	773.7;
	1987	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	920.2;
	1987	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4035.7;
	1987	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11099.2;
	1987	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6599.2;
	1987	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9773.1;
	1987	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1638.3;
	1988	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.9;
	1988	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	793.9;
	1988	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	944.2;
	1988	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4096.6;
	1988	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11399.5;
	1988	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6739.2;
	1988	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9950.9;
	1988	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1694.4;
	1989	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.7;
	1989	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	761.1;
	1989	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	912.5;
	1989	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3954.8;
	1989	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10951.9;
	1989	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6471.8;
	1989	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9718.9;
	1989	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1629.8;
	1990	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.4;
	1990	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	706.9;
	1990	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.4;
	1990	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3700.5;
	1990	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9867.8;
	1990	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5957.2;
	1990	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9252.2;
	1990	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1503.4;
	1991	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	854.4;
	1991	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	653.9;
	1991	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	781.3;
	1991	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3355.3;
	1991	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8716.9;
	1991	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5342.1;
	1991	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8654.2;
	1991	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1372.7;
	1992	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	848.4;
	1992	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	625.7;
	1992	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	734.2;
	1992	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3064.3;
	1992	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7883.4;
	1992	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4795.6;
	1992	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8282.6;
	1992	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1281;
	1993	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	842.5;
	1993	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	605.8;
	1993	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	704.8;
	1993	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2835.4;
	1993	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7379.3;
	1993	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4483.1;
	1993	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7968.2;
	1993	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1217.5;
	1994	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	836.4;
	1994	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.3;
	1994	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	685.9;
	1994	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2688;
	1994	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7025.3;
	1994	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4130.9;
	1994	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7765.4;
	1994	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1154.8;
	1995	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	834.9;
	1995	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	593;
	1995	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	679.9;
	1995	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2615.7;
	1995	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6934.8;
	1995	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3978.3;
	1995	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7702.1;
	1995	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1106;
	1996	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	838.8;
	1996	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	610.3;
	1996	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	688.5;
	1996	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2613.6;
	1996	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7099.3;
	1996	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3987.7;
	1996	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7785.9;
	1996	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1100.7;
	1997	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	860.4;
	1997	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	648.1;
	1997	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	727.3;
	1997	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2732.7;
	1997	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7764.6;
	1997	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4252.5;
	1997	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8162.5;
	1997	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1178.2;
	1998	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907;
	1998	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	731.9;
	1998	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	819.2;
	1998	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3070;
	1998	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9100.1;
	1998	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4898;
	1998	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8792.9;
	1998	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1357.8;
	1999	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.6;
	1999	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	783.6;
	1999	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	873.2;
	1999	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3202.1;
	1999	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9853.8;
	1999	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5192.3;
	1999	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9063.4;
	1999	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1441.3;
	2000	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.9;
	2000	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777.5;
	2000	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	855.7;
	2000	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3111.5;
	2000	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9778.4;
	2000	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5173.6;
	2000	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8998.6;
	2000	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1463;
	2001	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.8;
	2001	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	775.2;
	2001	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	856;
	2001	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3131.8;
	2001	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9917.5;
	2001	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5240.6;
	2001	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9140;
	2001	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1489.9;
	2002	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.4;
	2002	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772;
	2002	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849.3;
	2002	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3156.3;
	2002	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9930.7;
	2002	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5307.5;
	2002	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9185.2;
	2002	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1495.6;
	2003	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.9;
	2003	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	761.8;
	2003	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	840.1;
	2003	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3219.1;
	2003	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9853.3;
	2003	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5317.2;
	2003	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9229.1;
	2003	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1494.3;
	2004	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.1;
	2004	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746.2;
	2004	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	825.8;
	2004	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3258.4;
	2004	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9736.3;
	2004	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5293.5;
	2004	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9209.8;
	2004	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1484.4;
	2005	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.9;
	2005	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.7;
	2005	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	821.3;
	2005	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3329.9;
	2005	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9588.3;
	2005	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5300.3;
	2005	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9218.7;
	2005	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1465.1;
	2006	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	876.3;
	2006	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	721.7;
	2006	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	812;
	2006	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3408.2;
	2006	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9422.8;
	2006	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5303.6;
	2006	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9200.2;
	2006	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1459.8;
	2007	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.6;
	2007	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	709.3;
	2007	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	807;
	2007	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3503.3;
	2007	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9266.4;
	2007	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5332.1;
	2007	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9156.9;
	2007	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1449.9;
	2008	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.6;
	2008	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	706.1;
	2008	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	805.8;
	2008	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3580.1;
	2008	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9200.1;
	2008	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5379.5;
	2008	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9120.4;
	2008	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1464.2;
	2009	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867.3;
	2009	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	696.4;
	2009	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	811.1;
	2009	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3559.7;
	2009	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9156.8;
	2009	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5418.5;
	2009	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9026;
	2009	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1476.8;
	2010	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	862.9;
	2010	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	695;
	2010	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	819.7;
	2010	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3452.5;
	2010	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9096;
	2010	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5416.6;
	2010	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8789.9;
	2010	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1512.7;
	2011	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.5;
	2011	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	703;
	2011	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	831.4;
	2011	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3320;
	2011	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9230.4;
	2011	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5438.5;
	2011	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8733;
	2011	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1545.7;
	2012	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	881.5;
	2012	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720.1;
	2012	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	871.3;
	2012	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3422.4;
	2012	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9759.9;
	2012	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5678.9;
	2012	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9005.2;
	2012	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1619;
	2013	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	881.1;
	2013	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	704.4;
	2013	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.1;
	2013	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3338.4;
	2013	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9571.2;
	2013	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5516.1;
	2013	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8855;
	2013	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1566.8;
	2014	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.9;
	2014	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	682.3;
	2014	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	810.1;
	2014	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3163;
	2014	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9042.8;
	2014	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5080;
	2014	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8512.5;
	2014	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1509.3;
	2015	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	850.2;
	2015	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	663.3;
	2015	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	761;
	2015	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2897.9;
	2015	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8364.9;
	2015	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4611.2;
	2015	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7996.2;
	2015	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1406.1;
	2016	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	823.5;
	2016	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	665.1;
	2016	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	734.7;
	2016	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2629;
	2016	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7946.6;
	2016	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4336.2;
	2016	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7505.3;
	2016	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1328.6;
	2017	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	816.8;
	2017	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	662.1;
	2017	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	715.5;
	2017	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2468.3;
	2017	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7757.7;
	2017	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4164.4;
	2017	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7281.9;
	2017	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1322.4;
	2018	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	819.9;
	2018	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	659.5;
	2018	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	704.7;
	2018	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2382.3;
	2018	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7698.3;
	2018	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4087.9;
	2018	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7163.9;
	2018	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1329.3;
	2019	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	821.7;
	2019	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	668.4;
	2019	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	707.4;
	2019	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2343.7;
	2019	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7794.3;
	2019	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4070.2;
	2019	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7150;
	2019	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1334.6;
	2020	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	824.5;
	2020	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	689.2;
	2020	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	727.7;
	2020	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2344.9;
	2020	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8097.1;
	2020	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4196.5;
	2020	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7278.2;
	2020	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1358.6;
	2021	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	833;
	2021	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	726.6;
	2021	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	768;
	2021	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2422.5;
	2021	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8757.9;
	2021	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4513.1;
	2021	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7639.2;
	2021	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1476.3;
	2022	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	848.4;
	2022	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	783.9;
	2022	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	839.8;
	2022	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2595.1;
	2022	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9723.8;
	2022	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5033.5;
	2022	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8158.7;
	2022	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1605.5;
	2023	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866.1;
	2023	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	818.4;
	2023	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	899.5;
	2023	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2712.8;
	2023	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10360.3;
	2023	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5432.6;
	2023	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8476;
	2023	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1713.4;
	2024	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866;
	2024	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	805.2;
	2024	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	900;
	2024	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2787.2;
	2024	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10404.4;
	2024	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5551.8;
	2024	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8672.8;
	2024	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1718.8;
	2025	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	869.1;
	2025	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	779.5;
	2025	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	879.3;
	2025	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2912;
	2025	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10214.6;
	2025	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5538.5;
	2025	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8807;
	2025	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1702.4;
	2026	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.5;
	2026	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755.1;
	2026	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.5;
	2026	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3006.8;
	2026	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9961.8;
	2026	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5436.4;
	2026	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8885.3;
	2026	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1643.9;
	2027	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	865.8;
	2027	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.6;
	2027	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	834.2;
	2027	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3078.4;
	2027	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9689.8;
	2027	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5314.8;
	2027	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8873.8;
	2027	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1573.8;
	2028	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	856.8;
	2028	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	706.3;
	2028	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	811.1;
	2028	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3135.3;
	2028	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9437.6;
	2028	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5218.3;
	2028	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8825.1;
	2028	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1531.5;
	2029	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	853.6;
	2029	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	692.3;
	2029	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	797.7;
	2029	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3204.9;
	2029	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9225.5;
	2029	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5157.7;
	2029	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8821.2;
	2029	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1491.1;
	2030	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	859.3;
	2030	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	682.4;
	2030	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	793.6;
	2030	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3305;
	2030	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9101.3;
	2030	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5147.3;
	2030	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8869.1;
	2030	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1452.9;
	2031	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.4;
	2031	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	671.5;
	2031	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	795.4;
	2031	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3390.1;
	2031	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9026.9;
	2031	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5203.9;
	2031	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8922.1;
	2031	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1481.3;
	2032	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871.8;
	2032	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	673.6;
	2032	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	797.1;
	2032	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3452.1;
	2032	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9020.2;
	2032	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5278.9;
	2032	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8986;
	2032	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1500.9;
	2033	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.6;
	2033	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	670.7;
	2033	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	802.2;
	2033	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3439.9;
	2033	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9035.3;
	2033	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5325.8;
	2033	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9001.8;
	2033	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1506.2;
	2034	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.7;
	2034	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	669.6;
	2034	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	800.3;
	2034	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3328.2;
	2034	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8938.2;
	2034	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5269.6;
	2034	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8762.8;
	2034	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1505.2;
	2035	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	860.6;
	2035	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	675.2;
	2035	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	796.7;
	2035	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3219.4;
	2035	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8903.4;
	2035	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5227.9;
	2035	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8654.4;
	2035	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1504.1;
	2036	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	876.1;
	2036	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	704.5;
	2036	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	826.5;
	2036	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3313.4;
	2036	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9256;
	2036	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5426.5;
	2036	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8827.3;
	2036	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1537.8;
	2037	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.8;
	2037	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	696.2;
	2037	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	810.8;
	2037	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3230.4;
	2037	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9067.4;
	2037	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5278.9;
	2037	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8600.1;
	2037	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1511.6;
	2038	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	862.2;
	2038	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	675.1;
	2038	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	780.4;
	2038	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3092.5;
	2038	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8630.6;
	2038	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5013.1;
	2038	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8277.9;
	2038	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1454.8;
	2039	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	841.1;
	2039	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	647.4;
	2039	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	746.3;
	2039	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2884.6;
	2039	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8035.8;
	2039	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4638.1;
	2039	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7840.2;
	2039	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1348.6;
	2040	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	828.5;
	2040	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	617.5;
	2040	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	745.2;
	2040	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2664.3;
	2040	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7477.9;
	2040	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4294.9;
	2040	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7437.1;
	2040	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1286.4;
	2041	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	818.4;
	2041	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	603.2;
	2041	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	724.5;
	2041	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2505.4;
	2041	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7133.4;
	2041	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4076.5;
	2041	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7167;
	2041	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1233.6;
	2042	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	817.2;
	2042	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	599.9;
	2042	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	713.8;
	2042	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2400;
	2042	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6965.1;
	2042	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3957.1;
	2042	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7024.8;
	2042	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1206.5;
	2043	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	814.8;
	2043	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	599.9;
	2043	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	714.1;
	2043	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2344.4;
	2043	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6902.2;
	2043	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3902.8;
	2043	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	6950.6;
	2043	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1215.6;
	2044	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	814.5;
	2044	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	600.5;
	2044	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	720.1;
	2044	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2332.1;
	2044	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6972;
	2044	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3916.1;
	2044	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	6984.1;
	2044	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1232.3;
	2045	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	821;
	2045	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	620.1;
	2045	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	735.9;
	2045	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2369.9;
	2045	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7229.6;
	2045	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4027;
	2045	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7123.3;
	2045	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1251.1;
	2046	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	830.3;
	2046	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	647.5;
	2046	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	763.9;
	2046	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2452;
	2046	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7697.2;
	2046	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4274.2;
	2046	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7369.5;
	2046	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1338;
	2047	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	842.2;
	2047	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	669.6;
	2047	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	798.1;
	2047	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2523.4;
	2047	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8185.1;
	2047	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4548.7;
	2047	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7539;
	2047	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1390.8;
	2048	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	845.7;
	2048	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	690.9;
	2048	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	825.6;
	2048	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2630;
	2048	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8596.5;
	2048	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4794.2;
	2048	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7797.8;
	2048	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1436.9;
	2049	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	852.9;
	2049	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	698.6;
	2049	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	841.6;
	2049	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2818.2;
	2049	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8858.6;
	2049	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5024.6;
	2049	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8084.9;
	2049	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1476.3;
	2050	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	859.2;
	2050	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	692.4;
	2050	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838.8;
	2050	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2974.6;
	2050	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8943.1;
	2050	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5074;
	2050	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8278.5;
	2050	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1451.9;
	2051	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867.4;
	2051	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	681.2;
	2051	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	837;
	2051	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3098.9;
	2051	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8936;
	2051	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5079.6;
	2051	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8363.7;
	2051	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1412;
	2052	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871.2;
	2052	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	666.8;
	2052	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.3;
	2052	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3214.9;
	2052	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8875.2;
	2052	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5038.9;
	2052	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8458.8;
	2052	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1414.8;
	2053	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.2;
	2053	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	657.5;
	2053	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	850.9;
	2053	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3289.7;
	2053	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8733.9;
	2053	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5014.9;
	2053	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8493.7;
	2053	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1384.9;
	2054	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.7;
	2054	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	656;
	2054	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.8;
	2054	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3374;
	2054	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8737.6;
	2054	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5086.6;
	2054	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8633.3;
	2054	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1406.4;
	2055	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.9;
	2055	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	660.3;
	2055	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.7;
	2055	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3428.7;
	2055	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8807.7;
	2055	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5152.5;
	2055	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8697.5;
	2055	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1463.1;
	2056	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.8;
	2056	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	667.8;
	2056	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.1;
	2056	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3461.6;
	2056	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8969.8;
	2056	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5206.9;
	2056	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8761.6;
	2056	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1536.3;
	2057	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.9;
	2057	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	676.9;
	2057	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.4;
	2057	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3444.2;
	2057	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9067.7;
	2057	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5251.8;
	2057	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8795.1;
	2057	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1538.3;
	2058	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.9;
	2058	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	675.6;
	2058	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851.2;
	2058	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3383;
	2058	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9014;
	2058	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5225.3;
	2058	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8773.8;
	2058	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1496;
	2059	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.7;
	2059	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	675.6;
	2059	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	841.7;
	2059	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3375.8;
	2059	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9011.9;
	2059	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5228.1;
	2059	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8919.5;
	2059	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1506.1;
	2060	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.4;
	2060	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	705.9;
	2060	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.6;
	2060	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3493.3;
	2060	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9358.4;
	2060	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5448.8;
	2060	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9142.7;
	2060	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1574.2;
	2061	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.4;
	2061	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	688.8;
	2061	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.3;
	2061	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3427.2;
	2061	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9150.6;
	2061	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5328.6;
	2061	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8981.5;
	2061	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1543.5;
	2062	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871;
	2062	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	661;
	2062	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	802.8;
	2062	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3302.7;
	2062	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8666.8;
	2062	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5049.1;
	2062	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8668.1;
	2062	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1473.9;
	2063	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	852.8;
	2063	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	624.4;
	2063	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	754.4;
	2063	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3120.5;
	2063	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8046.2;
	2063	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4665.2;
	2063	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8254.8;
	2063	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1352.7;
	2064	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	837.6;
	2064	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	596;
	2064	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	716.8;
	2064	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2923.1;
	2064	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7423.8;
	2064	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4292.4;
	2064	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7840.3;
	2064	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1271.3;
	2065	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	823.3;
	2065	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	578;
	2065	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	689.4;
	2065	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2739.9;
	2065	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6973.4;
	2065	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4032.5;
	2065	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7511.3;
	2065	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1211.1;
	2066	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	817;
	2066	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	568.2;
	2066	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	674;
	2066	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2609.1;
	2066	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6680.4;
	2066	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3876.8;
	2066	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7268;
	2066	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1160.6;
	2067	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	816.3;
	2067	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	563.8;
	2067	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	665.6;
	2067	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2546.2;
	2067	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6535.7;
	2067	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3781.5;
	2067	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7131.9;
	2067	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1142.7;
	2068	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	820.8;
	2068	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.1;
	2068	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	666.7;
	2068	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2514.2;
	2068	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6512.2;
	2068	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3771.9;
	2068	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7080.3;
	2068	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1168.3;
	2069	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	826.3;
	2069	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586.2;
	2069	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	677.4;
	2069	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2522.3;
	2069	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6628.2;
	2069	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3838.3;
	2069	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7138.3;
	2069	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1197.5;
	2070	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	838.6;
	2070	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	614.6;
	2070	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	706.7;
	2070	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2587.4;
	2070	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6967;
	2070	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4003.8;
	2070	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7312.2;
	2070	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1149.9;
	2071	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	849.3;
	2071	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	655.6;
	2071	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	740.8;
	2071	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2645.9;
	2071	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7406.3;
	2071	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4229.5;
	2071	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7379.1;
	2071	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1201.2;
	2072	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	852.7;
	2072	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	693.2;
	2072	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	776;
	2072	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2756;
	2072	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7909.1;
	2072	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4493.1;
	2072	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7601.2;
	2072	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1321;
	2073	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.9;
	2073	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	708.1;
	2073	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	800.1;
	2073	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2988;
	2073	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8401.2;
	2073	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4790.4;
	2073	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7989.9;
	2073	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1378.4;
	2074	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	861.2;
	2074	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	696.3;
	2074	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	798.8;
	2074	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3158.7;
	2074	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8578.9;
	2074	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5016.9;
	2074	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8253.1;
	2074	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1421;
	2075	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	853.3;
	2075	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	678;
	2075	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	783.5;
	2075	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3289.4;
	2075	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8524.9;
	2075	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5060.9;
	2075	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8554.8;
	2075	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1427.6;
	2076	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	847.6;
	2076	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	665.6;
	2076	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	775;
	2076	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3434.9;
	2076	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8458.2;
	2076	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4990.8;
	2076	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8925;
	2076	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1408.7;
	2077	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	842.6;
	2077	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	643.8;
	2077	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	759.3;
	2077	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3519.1;
	2077	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8314.5;
	2077	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4984.8;
	2077	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9143.5;
	2077	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1408.3;
	2078	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	838;
	2078	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	628.1;
	2078	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	749.3;
	2078	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3585.7;
	2078	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8153;
	2078	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4994.9;
	2078	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9186.8;
	2078	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1434.5;
	2079	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	838.5;
	2079	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	623.9;
	2079	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	747.2;
	2079	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3662.4;
	2079	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8069.5;
	2079	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5034.8;
	2079	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9314.7;
	2079	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1450.6;
	2080	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	839.5;
	2080	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	632.2;
	2080	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	757;
	2080	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3719.4;
	2080	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8063.4;
	2080	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5096.4;
	2080	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9491.6;
	2080	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1486.4;
	2081	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	846.1;
	2081	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	649.1;
	2081	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	766.5;
	2081	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3723.3;
	2081	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8142.5;
	2081	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5164.8;
	2081	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9558.3;
	2081	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1467.2;
	2082	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	847.4;
	2082	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	654.8;
	2082	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	787.8;
	2082	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3636.8;
	2082	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8221.9;
	2082	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5192;
	2082	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9428.8;
	2082	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1488.3;
	2083	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	845.2;
	2083	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	669.7;
	2083	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	803.2;
	2083	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3558.7;
	2083	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8459.5;
	2083	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5228.7;
	2083	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9302.9;
	2083	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1546.5;
	2084	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	863.4;
	2084	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	709.3;
	2084	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	833.2;
	2084	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3711.3;
	2084	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9060.8;
	2084	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5517.1;
	2084	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9524.8;
	2084	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1627.7;
	2085	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866.5;
	2085	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	700;
	2085	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	819.2;
	2085	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3699.9;
	2085	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8958.1;
	2085	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5389.1;
	2085	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9270.5;
	2085	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1599;
	2086	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	847.7;
	2086	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	664;
	2086	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	778.8;
	2086	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3464.1;
	2086	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8369.8;
	2086	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4956.9;
	2086	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8659.1;
	2086	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1495.1;
	2087	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	824.8;
	2087	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	631.2;
	2087	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	729.3;
	2087	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3088.1;
	2087	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7645.6;
	2087	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4419.3;
	2087	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7960.5;
	2087	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1381.7;
	2088	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	806.7;
	2088	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	614.7;
	2088	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	725.1;
	2088	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2775;
	2088	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7165.7;
	2088	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4039.9;
	2088	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7420.8;
	2088	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1292.8;
	2089	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	801.1;
	2089	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	611.3;
	2089	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	719.1;
	2089	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2584.5;
	2089	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6990.5;
	2089	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3829.4;
	2089	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7110.6;
	2089	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1254.2;
	2090	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	799;
	2090	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	613.6;
	2090	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	717.6;
	2090	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2472.6;
	2090	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6952.6;
	2090	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3730.1;
	2090	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	6952.1;
	2090	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1299.4;
	2091	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	802.3;
	2091	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	625.5;
	2091	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	725.3;
	2091	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2420.3;
	2091	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7087.2;
	2091	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3727.5;
	2091	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	6909.9;
	2091	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1288.7;
	2092	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	809.2;
	2092	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	651;
	2092	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	743.3;
	2092	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2429.9;
	2092	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7453.3;
	2092	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3852.1;
	2092	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7086.5;
	2092	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1329.7;
	2093	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	828.1;
	2093	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	696;
	2093	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	792.5;
	2093	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2562.2;
	2093	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8334.9;
	2093	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4263.3;
	2093	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7588.3;
	2093	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1466;
	2094	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870;
	2094	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	786.6;
	2094	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897.5;
	2094	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2896.4;
	2094	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9846.2;
	2094	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4981.6;
	2094	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8366.9;
	2094	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1683.8;
	2095	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.2;
	2095	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	831.6;
	2095	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	953.6;
	2095	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3028;
	2095	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10589;
	2095	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5336.5;
	2095	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8645.9;
	2095	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1744.4;
	2096	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.2;
	2096	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	808.8;
	2096	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	932.4;
	2096	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2994.1;
	2096	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10318.5;
	2096	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5286.5;
	2096	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8757.2;
	2096	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1690.3;
	2097	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.6;
	2097	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	784.2;
	2097	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913.3;
	2097	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3075.2;
	2097	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10042.1;
	2097	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5287.8;
	2097	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8943.3;
	2097	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1750.4;
	2098	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.9;
	2098	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	763.9;
	2098	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	894.5;
	2098	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3189;
	2098	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9895.1;
	2098	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5280.2;
	2098	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9167.3;
	2098	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1742.7;
	2099	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.8;
	2099	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.4;
	2099	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	876.6;
	2099	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3283.2;
	2099	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9727.2;
	2099	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5270.7;
	2099	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9331.4;
	2099	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1730.2;
	2100	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.7;
	2100	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.8;
	2100	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	863.5;
	2100	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3374.7;
	2100	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9581.7;
	2100	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5255.9;
	2100	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9434.5;
	2100	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1685.6;
	2101	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.6;
	2101	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	718.5;
	2101	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	862.4;
	2101	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3485.2;
	2101	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9518.6;
	2101	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5296.3;
	2101	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9743.2;
	2101	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1678.7;
	2102	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.9;
	2102	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	712.4;
	2102	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.4;
	2102	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3600.6;
	2102	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9453.1;
	2102	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5345.8;
	2102	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9986.8;
	2102	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1684.3;
	2103	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.8;
	2103	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	705.4;
	2103	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	869.8;
	2103	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3730.5;
	2103	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9415.7;
	2103	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5454;
	2103	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10253.7;
	2103	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1694;
	2104	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.8;
	2104	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	707.1;
	2104	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.6;
	2104	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3822.8;
	2104	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9493;
	2104	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5576.3;
	2104	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10459.9;
	2104	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1722.6;
	2105	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.8;
	2105	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	705.7;
	2105	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.2;
	2105	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3782.5;
	2105	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9567.6;
	2105	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5652.1;
	2105	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10477.8;
	2105	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1710.6;
	2106	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902;
	2106	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	712;
	2106	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	888.9;
	2106	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3709.3;
	2106	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9611.5;
	2106	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5686.7;
	2106	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10230.5;
	2106	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1731.1;
	2107	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.6;
	2107	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720.3;
	2107	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897.3;
	2107	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3702;
	2107	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9715.5;
	2107	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5731.4;
	2107	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10142.1;
	2107	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1740.6;
	2108	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.8;
	2108	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753.6;
	2108	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	923.1;
	2108	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3864.6;
	2108	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10157.7;
	2108	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5990.4;
	2108	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10347.9;
	2108	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1793.7;
	2109	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.9;
	2109	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	734.1;
	2109	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	899.7;
	2109	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3774.6;
	2109	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9811.1;
	2109	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5733.2;
	2109	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9978.7;
	2109	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1718.4;
	2110	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.4;
	2110	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	690.4;
	2110	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.9;
	2110	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3514.4;
	2110	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8995.7;
	2110	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5209.2;
	2110	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9302;
	2110	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1568.9;
	2111	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	853.9;
	2111	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	640.9;
	2111	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	783.4;
	2111	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3145.6;
	2111	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8069.2;
	2111	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4631.8;
	2111	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8517.5;
	2111	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1445.5;
	2112	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	832.8;
	2112	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	623.2;
	2112	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	705;
	2112	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2814.9;
	2112	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7542.2;
	2112	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4253.7;
	2112	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8018.2;
	2112	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1316.9;
	2113	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	820.4;
	2113	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	606.9;
	2113	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	686.2;
	2113	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2624.7;
	2113	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7221.7;
	2113	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4017.9;
	2113	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7689.1;
	2113	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1226.2;
	2114	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	815.7;
	2114	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	606.3;
	2114	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	675.7;
	2114	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2539.8;
	2114	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7055.9;
	2114	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3887.1;
	2114	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7517.8;
	2114	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1211.6;
	2115	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	813.7;
	2115	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	611.2;
	2115	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	668.6;
	2115	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2505.1;
	2115	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7035.1;
	2115	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3829;
	2115	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7443.4;
	2115	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1222.6;
	2116	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	815.6;
	2116	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	618.7;
	2116	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	673.5;
	2116	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2519.9;
	2116	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7299.1;
	2116	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3902.9;
	2116	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7586.5;
	2116	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1261.4;
	2117	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	832.8;
	2117	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	651.9;
	2117	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	706.4;
	2117	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2665.3;
	2117	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8019.1;
	2117	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4256.5;
	2117	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8101.5;
	2117	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1361.7;
	2118	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867.8;
	2118	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	726.5;
	2118	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	787.7;
	2118	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3077;
	2118	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9301.2;
	2118	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4960.2;
	2118	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8921.8;
	2118	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1571;
	2119	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.9;
	2119	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755.3;
	2119	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	831.2;
	2119	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3222.6;
	2119	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9826;
	2119	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5273.4;
	2119	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9187.6;
	2119	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1630.2;
	2120	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.1;
	2120	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	748.2;
	2120	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	829.3;
	2120	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3167.7;
	2120	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9589.5;
	2120	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5231.3;
	2120	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9252.8;
	2120	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1604.8;
	2121	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895;
	2121	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749;
	2121	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	831.7;
	2121	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3246.7;
	2121	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9639.9;
	2121	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5293.2;
	2121	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9445.9;
	2121	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1624.3;
	2122	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902;
	2122	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	745.4;
	2122	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	839.5;
	2122	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3360.4;
	2122	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9746.1;
	2122	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5404.3;
	2122	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9679.4;
	2122	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1606.3;
	2123	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.8;
	2123	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.1;
	2123	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.8;
	2123	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3462.8;
	2123	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9770.2;
	2123	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5463.4;
	2123	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9833;
	2123	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1627.9;
	2124	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921;
	2124	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.6;
	2124	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	844.7;
	2124	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3546.1;
	2124	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9750.1;
	2124	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5479.4;
	2124	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9895.4;
	2124	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1630.8;
	2125	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.7;
	2125	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	726.6;
	2125	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	848.3;
	2125	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3651.7;
	2125	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9795.1;
	2125	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5492.9;
	2125	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9955.6;
	2125	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1632.1;
	2126	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.6;
	2126	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.8;
	2126	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.7;
	2126	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3763.3;
	2126	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9765.5;
	2126	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5482.5;
	2126	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9996.3;
	2126	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1626.2;
	2127	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.4;
	2127	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.2;
	2127	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	862.1;
	2127	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3870.6;
	2127	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9830.3;
	2127	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5494.8;
	2127	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9990.3;
	2127	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1598;
	2128	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	979.2;
	2128	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.6;
	2128	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	875.2;
	2128	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3941;
	2128	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9884.9;
	2128	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5536.2;
	2128	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9967.9;
	2128	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1623.7;
	2129	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.1;
	2129	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	721.7;
	2129	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	880.2;
	2129	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3917.9;
	2129	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9890.5;
	2129	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5622;
	2129	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9918.6;
	2129	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1619.6;
	2130	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.2;
	2130	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.7;
	2130	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	873.9;
	2130	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3859.3;
	2130	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9924.3;
	2130	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5725.2;
	2130	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9872.1;
	2130	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1632.9;
	2131	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.6;
	2131	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737;
	2131	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	879.7;
	2131	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3871.5;
	2131	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10149.3;
	2131	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5902.4;
	2131	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10036.8;
	2131	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1703.4;
	2132	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.9;
	2132	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	766.2;
	2132	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	905.5;
	2132	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4044.8;
	2132	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10493.8;
	2132	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6039.3;
	2132	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10153.2;
	2132	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1747;
	2133	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954;
	2133	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	739.4;
	2133	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889;
	2133	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3952.4;
	2133	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10094.8;
	2133	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5822.8;
	2133	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9881.9;
	2133	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1689;
	2134	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.9;
	2134	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	692.5;
	2134	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	831.2;
	2134	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3696.6;
	2134	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9296.2;
	2134	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5379.8;
	2134	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9343;
	2134	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1567.2;
	2135	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887;
	2135	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	651.8;
	2135	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	772.7;
	2135	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3334.8;
	2135	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8421.6;
	2135	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4853.9;
	2135	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8677.7;
	2135	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1436.4;
	2136	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	862;
	2136	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	626.2;
	2136	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	741;
	2136	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3049.9;
	2136	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7760.8;
	2136	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4507.6;
	2136	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8182.1;
	2136	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1368.2;
	2137	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	852.9;
	2137	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	613;
	2137	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	717.3;
	2137	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2894.3;
	2137	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7440.3;
	2137	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4297.8;
	2137	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7963.5;
	2137	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1298.3;
	2138	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	847.6;
	2138	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	605.2;
	2138	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	706.1;
	2138	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2804.6;
	2138	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7317.6;
	2138	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4173.9;
	2138	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7828.1;
	2138	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1301.3;
	2139	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	840.2;
	2139	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	603.5;
	2139	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	702.3;
	2139	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2760.5;
	2139	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7266.8;
	2139	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4133;
	2139	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7769.8;
	2139	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1290.8;
	2140	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	837.9;
	2140	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	617.6;
	2140	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	709.9;
	2140	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2759;
	2140	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7430.3;
	2140	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4201.9;
	2140	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7930.3;
	2140	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1303.4;
	2141	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	845.6;
	2141	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	631.3;
	2141	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	738;
	2141	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2891.4;
	2141	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8037.9;
	2141	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4542.8;
	2141	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8444.4;
	2141	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1368.9;
	2142	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.8;
	2142	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	694.4;
	2142	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	812.9;
	2142	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3292.3;
	2142	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9253.7;
	2142	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5240.9;
	2142	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9322.1;
	2142	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1558;
	2143	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.6;
	2143	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.9;
	2143	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.3;
	2143	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3456.2;
	2143	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9930.9;
	2143	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5607.1;
	2143	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9719.8;
	2143	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1648.2;
	2144	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893;
	2144	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.2;
	2144	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849;
	2144	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3387.6;
	2144	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9926.9;
	2144	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5600;
	2144	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9703.3;
	2144	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1646.3;
	2145	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.6;
	2145	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743;
	2145	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	858.1;
	2145	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3464.3;
	2145	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10070.7;
	2145	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5677.9;
	2145	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9829.8;
	2145	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1688.5;
	2146	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.4;
	2146	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	756.8;
	2146	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	869.9;
	2146	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3560.9;
	2146	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10274.8;
	2146	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5799.6;
	2146	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10026.6;
	2146	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1696.8;
	2147	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.8;
	2147	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	771.1;
	2147	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	886.7;
	2147	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3631.7;
	2147	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10389.7;
	2147	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5881.6;
	2147	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10211;
	2147	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1733.5;
	2148	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.6;
	2148	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	775.1;
	2148	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897.8;
	2148	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3703.3;
	2148	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10529.7;
	2148	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5883.5;
	2148	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10265.3;
	2148	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1742.6;
	2149	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970.7;
	2149	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	791.1;
	2149	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	920;
	2149	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3794.8;
	2149	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10609.6;
	2149	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5883.8;
	2149	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10304.3;
	2149	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1742.2;
	2150	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986.3;
	2150	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	802;
	2150	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940.9;
	2150	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3898.7;
	2150	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10613.3;
	2150	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5882.6;
	2150	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10313.2;
	2150	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1751.3;
	2151	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	994.7;
	2151	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	807;
	2151	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	959.5;
	2151	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4003.8;
	2151	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10643.5;
	2151	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5946.7;
	2151	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10309.8;
	2151	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1728.5;
	2152	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1001.2;
	2152	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	812.8;
	2152	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	982.4;
	2152	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4080.9;
	2152	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10692.2;
	2152	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6074.4;
	2152	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10270.6;
	2152	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1672.5;
	2153	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990.2;
	2153	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	808.7;
	2153	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	991.2;
	2153	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4037.2;
	2153	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10771.4;
	2153	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6182.1;
	2153	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10219.7;
	2153	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1640.9;
	2154	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975;
	2154	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	799.4;
	2154	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	980.2;
	2154	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3960;
	2154	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10703.6;
	2154	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6249.5;
	2154	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10103.8;
	2154	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1682.6;
	2155	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.3;
	2155	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	792.1;
	2155	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	959;
	2155	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3960.6;
	2155	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10461.8;
	2155	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6289.5;
	2155	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10256.4;
	2155	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1705;
	2156	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.5;
	2156	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	806.2;
	2156	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	970.3;
	2156	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4106.4;
	2156	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10564.4;
	2156	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6513.5;
	2156	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10455.7;
	2156	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1745.4;
	2157	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.4;
	2157	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	775.7;
	2157	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	939.5;
	2157	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4025.2;
	2157	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10138.6;
	2157	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6298.2;
	2157	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10232.7;
	2157	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1736;
	2158	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.7;
	2158	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.4;
	2158	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	871.5;
	2158	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3769.2;
	2158	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9331.3;
	2158	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5789.9;
	2158	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9744.8;
	2158	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1619.5;
	2159	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	857.3;
	2159	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	657.2;
	2159	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	796.7;
	2159	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3425.9;
	2159	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8465.4;
	2159	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5174.2;
	2159	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9095.6;
	2159	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1483.9;
	2160	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	839.9;
	2160	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	623.1;
	2160	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	739.8;
	2160	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3146.8;
	2160	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7843.7;
	2160	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4741.5;
	2160	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8592.9;
	2160	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1380.2;
	2161	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	833.5;
	2161	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	605.8;
	2161	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	713.7;
	2161	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2975.7;
	2161	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7478.1;
	2161	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4487.2;
	2161	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8300.7;
	2161	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1329.3;
	2162	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	829.8;
	2162	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	594.5;
	2162	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	695.3;
	2162	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2880.5;
	2162	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7276.3;
	2162	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4352.5;
	2162	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8155.5;
	2162	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1263.4;
	2163	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	825.8;
	2163	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	593.3;
	2163	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	683.1;
	2163	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2836.1;
	2163	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7232.6;
	2163	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4274.2;
	2163	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8113;
	2163	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1274.1;
	2164	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	825.3;
	2164	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	595.8;
	2164	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	685.7;
	2164	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2835.7;
	2164	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7370.9;
	2164	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4310.8;
	2164	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8259.9;
	2164	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1302.2;
	2165	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	840;
	2165	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	619.8;
	2165	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	718.6;
	2165	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2952.9;
	2165	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7959.3;
	2165	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4618.6;
	2165	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8755.7;
	2165	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1367.5;
	2166	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877;
	2166	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	684.4;
	2166	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	794;
	2166	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3330.1;
	2166	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9118.5;
	2166	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5283.9;
	2166	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9599.2;
	2166	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1545.3;
	2167	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.7;
	2167	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	712.8;
	2167	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	829.9;
	2167	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3462.3;
	2167	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9573.9;
	2167	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5568.5;
	2167	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9916.7;
	2167	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1593.9;
	2168	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.6;
	2168	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711.2;
	2168	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	818.7;
	2168	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3408.3;
	2168	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9416.9;
	2168	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5574.7;
	2168	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9971.4;
	2168	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1633.4;
	2169	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.6;
	2169	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714.9;
	2169	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	827.8;
	2169	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3562.3;
	2169	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9581.6;
	2169	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5814.5;
	2169	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10140;
	2169	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1697.3;
	2170	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.8;
	2170	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	724.4;
	2170	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	831.9;
	2170	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3819.4;
	2170	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9767.4;
	2170	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6051.9;
	2170	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10350.2;
	2170	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1752.8;
	2171	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.7;
	2171	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.8;
	2171	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	836;
	2171	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4137.1;
	2171	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9888.3;
	2171	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6226.1;
	2171	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10602.6;
	2171	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1814;
	2172	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.3;
	2172	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	732.5;
	2172	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	850.3;
	2172	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4451.7;
	2172	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9993;
	2172	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6368.8;
	2172	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10944.9;
	2172	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1888.9;
	2173	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.6;
	2173	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.6;
	2173	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878.8;
	2173	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4781.7;
	2173	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10126.2;
	2173	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6552.8;
	2173	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11440.6;
	2173	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1924.3;
	2174	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.4;
	2174	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.7;
	2174	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	899.1;
	2174	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5054;
	2174	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10207.9;
	2174	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6748.5;
	2174	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11943.6;
	2174	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1942.5;
	2175	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.1;
	2175	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.5;
	2175	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	916.5;
	2175	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5260.9;
	2175	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10378.5;
	2175	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6968.7;
	2175	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12449.6;
	2175	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2003.3;
	2176	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.6;
	2176	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.8;
	2176	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	938.8;
	2176	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5367.7;
	2176	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10522.2;
	2176	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7177.5;
	2176	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12778.3;
	2176	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1980.3;
	2177	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931;
	2177	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735.4;
	2177	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	942.4;
	2177	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5306.5;
	2177	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10584.1;
	2177	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7277.2;
	2177	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12871.1;
	2177	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1968.2;
	2178	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.7;
	2178	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.3;
	2178	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	932.5;
	2178	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5085.3;
	2178	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10421.7;
	2178	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7148.9;
	2178	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12452.7;
	2178	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1844.9;
	2179	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.9;
	2179	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	722.7;
	2179	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	910.3;
	2179	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4831.5;
	2179	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10251.1;
	2179	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6827.9;
	2179	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12070.2;
	2179	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1763.5;
	2180	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.2;
	2180	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	750.2;
	2180	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	916.7;
	2180	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4814.2;
	2180	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10602.3;
	2180	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6789.4;
	2180	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12019.2;
	2180	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1828.6;
	2181	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.4;
	2181	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	732.3;
	2181	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	891.1;
	2181	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4603.2;
	2181	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10218.7;
	2181	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6397.7;
	2181	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11444.7;
	2181	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1766.7;
	2182	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.4;
	2182	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	685.1;
	2182	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	830.8;
	2182	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4241.5;
	2182	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9340.8;
	2182	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5753.7;
	2182	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10604;
	2182	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1604.8;
	2183	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	850.5;
	2183	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	635.7;
	2183	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	761.7;
	2183	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3778.5;
	2183	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8309.3;
	2183	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5041.2;
	2183	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9534.7;
	2183	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1429.9;
	2184	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	826.8;
	2184	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	606.5;
	2184	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	717.5;
	2184	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3400.5;
	2184	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7533.3;
	2184	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4519.9;
	2184	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8667.3;
	2184	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1316.5;
	2185	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	814.1;
	2185	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.9;
	2185	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	687.4;
	2185	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3169.3;
	2185	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7162;
	2185	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4198.3;
	2185	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8127.5;
	2185	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1233.2;
	2186	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	811.4;
	2186	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586.3;
	2186	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	664.6;
	2186	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3009.9;
	2186	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6907.5;
	2186	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4001;
	2186	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7788.9;
	2186	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1210.7;
	2187	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	808.7;
	2187	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586.7;
	2187	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	656.2;
	2187	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2918.2;
	2187	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6849.5;
	2187	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3895.7;
	2187	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7649.2;
	2187	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1174.1;
	2188	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	815.2;
	2188	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	606.2;
	2188	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	667.5;
	2188	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2881.4;
	2188	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7004;
	2188	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3928.5;
	2188	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7673.9;
	2188	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1195.6;
	2189	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	829.4;
	2189	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	634.2;
	2189	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	705.1;
	2189	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2959.3;
	2189	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7603.1;
	2189	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4221.6;
	2189	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8004.4;
	2189	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1287.7;
	2190	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.7;
	2190	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	705.2;
	2190	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	794.1;
	2190	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3291.7;
	2190	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8813.6;
	2190	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4843.6;
	2190	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8665.3;
	2190	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1453.1;
	2191	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.2;
	2191	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740;
	2191	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	848.4;
	2191	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3404.3;
	2191	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9448.7;
	2191	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5133.3;
	2191	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8828.7;
	2191	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1515.2;
	2192	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.6;
	2192	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.5;
	2192	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	844;
	2192	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3348.2;
	2192	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9456.8;
	2192	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5118.2;
	2192	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8871.1;
	2192	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1450.4;
	2193	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.2;
	2193	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742;
	2193	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	848.9;
	2193	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3428.7;
	2193	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9613.6;
	2193	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5233.8;
	2193	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8992.5;
	2193	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1519;
	2194	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.4;
	2194	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.2;
	2194	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	858.9;
	2194	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3559.3;
	2194	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9762.4;
	2194	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5319.7;
	2194	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9176.9;
	2194	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1555.9;
	2195	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.6;
	2195	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	745;
	2195	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	856.8;
	2195	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3677.3;
	2195	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9800.7;
	2195	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5356.4;
	2195	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9265.3;
	2195	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1578.7;
	2196	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929;
	2196	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	734.4;
	2196	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	844.3;
	2196	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3721.5;
	2196	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9772.4;
	2196	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5341.5;
	2196	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9209.1;
	2196	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1569.2;
	2197	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.8;
	2197	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.6;
	2197	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	836.8;
	2197	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3757.7;
	2197	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9625.7;
	2197	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5292.5;
	2197	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9224.7;
	2197	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1566.6;
	2198	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.1;
	2198	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723;
	2198	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	826.6;
	2198	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3765.1;
	2198	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9463.6;
	2198	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5252.2;
	2198	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9136.2;
	2198	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1552.6;
	2199	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.6;
	2199	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	710.3;
	2199	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824.8;
	2199	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3788.1;
	2199	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9328.3;
	2199	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5253.7;
	2199	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9076.4;
	2199	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1515.7;
	2200	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.6;
	2200	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	697.1;
	2200	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	822.4;
	2200	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3763.3;
	2200	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9211.7;
	2200	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5203.2;
	2200	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8980.9;
	2200	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1512;
	2201	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.1;
	2201	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	687.9;
	2201	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	823;
	2201	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3657.7;
	2201	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9146.1;
	2201	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5163.6;
	2201	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8844.5;
	2201	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1523.6;
	2202	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.6;
	2202	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	681.9;
	2202	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	825.4;
	2202	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3467.5;
	2202	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9087.4;
	2202	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5144;
	2202	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8603;
	2202	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1530.6;
	2203	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.7;
	2203	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	689.7;
	2203	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	834.5;
	2203	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3348.4;
	2203	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9235.7;
	2203	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5185.9;
	2203	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8651.7;
	2203	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1545.7;
	2204	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.1;
	2204	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723.1;
	2204	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	864.9;
	2204	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3433.7;
	2204	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9563.6;
	2204	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5440.6;
	2204	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8893.2;
	2204	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1605.9;
	2205	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943;
	2205	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719.3;
	2205	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.7;
	2205	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3357;
	2205	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9354.9;
	2205	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5368.2;
	2205	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8716.9;
	2205	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1597.6;
	2206	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.7;
	2206	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	694.1;
	2206	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	836.5;
	2206	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3202.2;
	2206	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8894.7;
	2206	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5122;
	2206	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8374.9;
	2206	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1532;
	2207	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.2;
	2207	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	662.3;
	2207	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	801.4;
	2207	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2957.5;
	2207	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8264.9;
	2207	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4770;
	2207	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7916.5;
	2207	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1446.8;
	2208	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898;
	2208	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	639.9;
	2208	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	772.4;
	2208	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2702.6;
	2208	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7884.8;
	2208	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4474.6;
	2208	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7496.4;
	2208	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1377.9;
	2209	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.6;
	2209	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	630.9;
	2209	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	761.2;
	2209	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2544.6;
	2209	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7606.3;
	2209	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4292.2;
	2209	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7257.5;
	2209	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1333.8;
	2210	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.8;
	2210	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	630.6;
	2210	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	754.9;
	2210	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2438.9;
	2210	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7491.5;
	2210	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4221.1;
	2210	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7108.9;
	2210	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1308.2;
	2211	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.2;
	2211	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	633.8;
	2211	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	759.3;
	2211	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2385.2;
	2211	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7452.6;
	2211	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4207.9;
	2211	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7031.6;
	2211	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1323.8;
	2212	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.1;
	2212	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	640;
	2212	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	773.6;
	2212	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2377.3;
	2212	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7610.9;
	2212	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4244.1;
	2212	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7077.5;
	2212	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1313;
	2213	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.3;
	2213	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	662.5;
	2213	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	798.2;
	2213	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2419.2;
	2213	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7915;
	2213	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4414.5;
	2213	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7224.7;
	2213	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1366.8;
	2214	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.3;
	2214	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	696.9;
	2214	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	845.5;
	2214	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2546.4;
	2214	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8542.1;
	2214	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4750.3;
	2214	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7564.9;
	2214	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1447.2;
	2215	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.6;
	2215	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.8;
	2215	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	891.3;
	2215	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2628.7;
	2215	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9097.5;
	2215	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5107.3;
	2215	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7772.6;
	2215	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1503.4;
	2216	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.8;
	2216	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742.3;
	2216	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	900.5;
	2216	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2739.9;
	2216	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9416.5;
	2216	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5296;
	2216	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8028.9;
	2216	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1582.4;
	2217	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.6;
	2217	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.1;
	2217	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889.2;
	2217	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2875.3;
	2217	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9494.6;
	2217	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5328.2;
	2217	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8246.1;
	2217	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1554.4;
	2218	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.2;
	2218	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.4;
	2218	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	856.7;
	2218	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2970.2;
	2218	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9374.8;
	2218	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5260;
	2218	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8355.9;
	2218	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1515.4;
	2219	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.8;
	2219	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	691.7;
	2219	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	825.2;
	2219	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2996.1;
	2219	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9162.4;
	2219	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5133.2;
	2219	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8365;
	2219	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1444.6;
	2220	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.6;
	2220	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	668.9;
	2220	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	792.7;
	2220	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2992.3;
	2220	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8929.8;
	2220	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5019.3;
	2220	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8314;
	2220	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1433.5;
	2221	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.7;
	2221	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	655.4;
	2221	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	776.1;
	2221	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2999.6;
	2221	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8668.6;
	2221	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4937.5;
	2221	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8298.7;
	2221	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1398.7;
	2222	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	865.7;
	2222	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	645.9;
	2222	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	763.6;
	2222	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3012.6;
	2222	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8497.8;
	2222	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4886;
	2222	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8307;
	2222	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1385.2;
	2223	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	865.9;
	2223	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	643.7;
	2223	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	762;
	2223	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3045.5;
	2223	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8410;
	2223	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4910.6;
	2223	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8391.4;
	2223	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1373.2;
	2224	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866;
	2224	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	646;
	2224	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	769.6;
	2224	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3082.3;
	2224	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8421.3;
	2224	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4988.8;
	2224	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8481.4;
	2224	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1397.4;
	2225	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.9;
	2225	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	651;
	2225	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	776.6;
	2225	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3094.6;
	2225	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8463.5;
	2225	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5046.1;
	2225	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8526;
	2225	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1410;
	2226	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	861.3;
	2226	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	656.9;
	2226	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	784;
	2226	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3067.2;
	2226	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8433.3;
	2226	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5072.5;
	2226	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8489.4;
	2226	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1420.7;
	2227	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	862.9;
	2227	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	668.7;
	2227	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	801.2;
	2227	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3047;
	2227	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8488.3;
	2227	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5086.4;
	2227	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8497.4;
	2227	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1434;
	2228	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.9;
	2228	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	700;
	2228	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	830;
	2228	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3183;
	2228	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8891.9;
	2228	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5283.8;
	2228	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8684.6;
	2228	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1512;
	2229	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.5;
	2229	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	678.8;
	2229	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	811.6;
	2229	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3132.4;
	2229	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8747.4;
	2229	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5175.5;
	2229	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8535.9;
	2229	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1462.9;
	2230	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	862.4;
	2230	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	648;
	2230	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	772.6;
	2230	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3000.4;
	2230	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8314.1;
	2230	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4898.4;
	2230	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8188.3;
	2230	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1416.6;
	2231	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	847.5;
	2231	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	616;
	2231	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	733.7;
	2231	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2812.9;
	2231	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7776.5;
	2231	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4544.2;
	2231	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7770.5;
	2231	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1345;
	2232	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	836.2;
	2232	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	593;
	2232	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	703.4;
	2232	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2650.1;
	2232	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7300.3;
	2232	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4240.6;
	2232	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7384;
	2232	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1313.9;
	2233	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	828.1;
	2233	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	583.1;
	2233	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	691;
	2233	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2526.5;
	2233	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7049.2;
	2233	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4062.2;
	2233	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7143.6;
	2233	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1298.9;
	2234	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	830.2;
	2234	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	581.6;
	2234	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	686.5;
	2234	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2439.8;
	2234	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6923.6;
	2234	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3970.9;
	2234	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7002.6;
	2234	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1283.8;
	2235	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	832.1;
	2235	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	585.2;
	2235	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	695.7;
	2235	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2404.1;
	2235	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6930.5;
	2235	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3928.2;
	2235	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	6963;
	2235	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1230.6;
	2236	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	832.6;
	2236	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.3;
	2236	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	705.6;
	2236	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2393.4;
	2236	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7016.2;
	2236	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3978.9;
	2236	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	6961.7;
	2236	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1225.9;
	2237	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	841.2;
	2237	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	605.1;
	2237	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	727.1;
	2237	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2428.7;
	2237	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7253;
	2237	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4090.3;
	2237	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7067.3;
	2237	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1276.7;
	2238	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	852.3;
	2238	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	628.5;
	2238	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	768.9;
	2238	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2495.8;
	2238	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7698.3;
	2238	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4337.9;
	2238	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7296.7;
	2238	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1354.2;
	2239	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	859.2;
	2239	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	654.4;
	2239	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	812.1;
	2239	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2535.9;
	2239	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8118.2;
	2239	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4594.4;
	2239	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7404.4;
	2239	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1426.5;
	2240	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	859.9;
	2240	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	674.2;
	2240	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.1;
	2240	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2652.6;
	2240	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8493.8;
	2240	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4829.8;
	2240	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7721.9;
	2240	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1477;
	2241	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.7;
	2241	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	671.1;
	2241	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	821.7;
	2241	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2784.3;
	2241	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8647.4;
	2241	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4931.2;
	2241	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7962.5;
	2241	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1444.6;
	2242	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	859.6;
	2242	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	657.7;
	2242	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	797.4;
	2242	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2868.1;
	2242	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8623.5;
	2242	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4927.6;
	2242	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8158.5;
	2242	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1381.7;
	2243	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	858.5;
	2243	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	647.9;
	2243	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	780.1;
	2243	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2956;
	2243	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8576.9;
	2243	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4935.1;
	2243	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8295.3;
	2243	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1349;
	2244	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	856.3;
	2244	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	646.6;
	2244	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	776.4;
	2244	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3071.4;
	2244	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8559;
	2244	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4931.2;
	2244	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8436.8;
	2244	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1348.6;
	2245	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	854.8;
	2245	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	642.3;
	2245	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	774.5;
	2245	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3253.3;
	2245	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8559.4;
	2245	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4973.2;
	2245	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8547.6;
	2245	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1351.2;
	2246	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	856.8;
	2246	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	643.3;
	2246	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	775.3;
	2246	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3388.8;
	2246	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8605.2;
	2246	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5049.5;
	2246	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8700.8;
	2246	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1352.7;
	2247	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	860.2;
	2247	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	648.7;
	2247	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	782.2;
	2247	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3520;
	2247	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8719.8;
	2247	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5188.8;
	2247	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8920.6;
	2247	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1378.1;
	2248	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	865.5;
	2248	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	657.6;
	2248	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	797.6;
	2248	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3635.1;
	2248	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8915.5;
	2248	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5376.2;
	2248	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9155.5;
	2248	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1406.3;
	2249	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.9;
	2249	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	672.2;
	2249	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	812.6;
	2249	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3674;
	2249	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9156.2;
	2249	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5559.2;
	2249	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9321.5;
	2249	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1435.8;
	2250	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.2;
	2250	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	679.5;
	2250	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	821.6;
	2250	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3610.6;
	2250	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9266.1;
	2250	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5631.7;
	2250	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9315.9;
	2250	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1451.3;
	2251	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	869.6;
	2251	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	681.4;
	2251	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	821.1;
	2251	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3516.6;
	2251	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9294.2;
	2251	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5624.3;
	2251	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9236.7;
	2251	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1467.5;
	2252	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.6;
	2252	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	708.7;
	2252	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.3;
	2252	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3640.2;
	2252	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9693;
	2252	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5834.4;
	2252	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9450.8;
	2252	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1526.2;
	2253	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.1;
	2253	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	690.1;
	2253	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	832.9;
	2253	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3575.3;
	2253	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9418.9;
	2253	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5601.4;
	2253	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9206.5;
	2253	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1473.8;
	2254	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	862.5;
	2254	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	651.5;
	2254	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	783.4;
	2254	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3354.8;
	2254	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8690.4;
	2254	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5132.6;
	2254	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8630.5;
	2254	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1366.6;
	2255	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	836.2;
	2255	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	609.2;
	2255	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	723.1;
	2255	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3019.7;
	2255	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7804.3;
	2255	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4559.8;
	2255	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7953.7;
	2255	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1244.3;
	2256	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	820.3;
	2256	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586.6;
	2256	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	686.2;
	2256	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2735.2;
	2256	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7167.3;
	2256	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4138.3;
	2256	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7499.7;
	2256	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1168;
	2257	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	809.5;
	2257	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	574.1;
	2257	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	666.9;
	2257	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2569.6;
	2257	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6796.7;
	2257	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3908.8;
	2257	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7219.4;
	2257	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1135.5;
	2258	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	808.8;
	2258	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	569.4;
	2258	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	658.4;
	2258	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2482.9;
	2258	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6630.2;
	2258	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3800.1;
	2258	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7077.2;
	2258	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1117.7;
	2259	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	805.5;
	2259	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577.4;
	2259	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	659;
	2259	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2446.1;
	2259	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6617.8;
	2259	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3779;
	2259	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7071.6;
	2259	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1115.9;
	2260	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	810.2;
	2260	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577.2;
	2260	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	671.4;
	2260	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2464.4;
	2260	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6848.1;
	2260	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3888.8;
	2260	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7249;
	2260	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1152.3;
	2261	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	823.7;
	2261	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	610.5;
	2261	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	714.3;
	2261	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2619.1;
	2261	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7504.2;
	2261	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4307.5;
	2261	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7778.3;
	2261	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1256.5;
	2262	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	862.6;
	2262	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	683.5;
	2262	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	806.5;
	2262	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2998.3;
	2262	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8762;
	2262	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5043.4;
	2262	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8581.3;
	2262	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1466.3;
	2263	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.7;
	2263	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714.5;
	2263	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.7;
	2263	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3108.5;
	2263	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9287.7;
	2263	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5324.3;
	2263	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8766.1;
	2263	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1542.4;
	2264	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.7;
	2264	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	712.3;
	2264	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	845.2;
	2264	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3058;
	2264	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9240.3;
	2264	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5269.5;
	2264	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8865.6;
	2264	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1537.4;
	2265	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.2;
	2265	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714.8;
	2265	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	842.2;
	2265	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3137;
	2265	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9341.2;
	2265	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5278.6;
	2265	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9075;
	2265	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1531.8;
	2266	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.4;
	2266	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720.6;
	2266	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.8;
	2266	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3241.3;
	2266	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9512.5;
	2266	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5346.6;
	2266	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9314.1;
	2266	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1548.3;
	2267	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.9;
	2267	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	721.5;
	2267	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	848.4;
	2267	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3371.6;
	2267	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9669.5;
	2267	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5416;
	2267	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9517.4;
	2267	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1563.4;
	2268	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.3;
	2268	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.1;
	2268	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.3;
	2268	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3528;
	2268	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9878;
	2268	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5507.6;
	2268	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9616.3;
	2268	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1579.1;
	2269	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921;
	2269	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749.2;
	2269	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	880;
	2269	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3728.4;
	2269	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10169;
	2269	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5646.9;
	2269	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9804.4;
	2269	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1661.9;
	2270	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.8;
	2270	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	764.4;
	2270	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	902.7;
	2270	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3949.1;
	2270	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10479.8;
	2270	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5833.4;
	2270	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10097.2;
	2270	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1734;
	2271	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.5;
	2271	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	783.7;
	2271	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	926.6;
	2271	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4151.9;
	2271	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10821.7;
	2271	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6057.2;
	2271	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10377.5;
	2271	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1750.7;
	2272	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.1;
	2272	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	801.4;
	2272	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	957;
	2272	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4300.8;
	2272	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11176.8;
	2272	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6296.6;
	2272	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10673.5;
	2272	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1841.1;
	2273	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.6;
	2273	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	812.4;
	2273	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	967.4;
	2273	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4304;
	2273	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11432.3;
	2273	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6503.2;
	2273	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10802;
	2273	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1827.4;
	2274	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.7;
	2274	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	817.1;
	2274	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	968.4;
	2274	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4198.8;
	2274	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11467.8;
	2274	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6568.4;
	2274	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10644.4;
	2274	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1838.4;
	2275	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.6;
	2275	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	798.1;
	2275	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	949;
	2275	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3994.6;
	2275	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11249;
	2275	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6445;
	2275	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10371.7;
	2275	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1805.3;
	2276	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.7;
	2276	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	810.5;
	2276	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	952.9;
	2276	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4041.5;
	2276	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11389.3;
	2276	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6503.9;
	2276	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10444.6;
	2276	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1850.3;
	2277	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.7;
	2277	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	780.8;
	2277	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	920.4;
	2277	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3903.4;
	2277	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10881.7;
	2277	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6168.7;
	2277	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9939.8;
	2277	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1762.9;
	2278	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.6;
	2278	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	718.5;
	2278	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	850.1;
	2278	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3566.2;
	2278	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9833.7;
	2278	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5535.7;
	2278	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9101.5;
	2278	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1608.5;
	2279	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	859.9;
	2279	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	660.4;
	2279	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	776;
	2279	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3165.4;
	2279	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8714.7;
	2279	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4864.8;
	2279	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8259.2;
	2279	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1448.2;
	2280	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	832.6;
	2280	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	624.2;
	2280	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	723;
	2280	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2862;
	2280	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7884.3;
	2280	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4382.8;
	2280	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7664.2;
	2280	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1335.2;
	2281	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	819.9;
	2281	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	599.6;
	2281	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	697.5;
	2281	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2665.9;
	2281	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7401.6;
	2281	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4098.6;
	2281	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7311.1;
	2281	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1250.1;
	2282	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	812.4;
	2282	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588;
	2282	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	676.2;
	2282	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2564.7;
	2282	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7076.4;
	2282	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3928.4;
	2282	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7099;
	2282	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1216.2;
	2283	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	808.8;
	2283	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.3;
	2283	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	665.2;
	2283	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2527.6;
	2283	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6947.8;
	2283	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3866.7;
	2283	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7052.2;
	2283	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1219.5;
	2284	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	810.5;
	2284	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	594.9;
	2284	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	669.9;
	2284	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2534.9;
	2284	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7108;
	2284	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3939.4;
	2284	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7189.3;
	2284	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1250.8;
	2285	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	823;
	2285	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	610.6;
	2285	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	703.7;
	2285	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2668.4;
	2285	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7670.3;
	2285	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4281.1;
	2285	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7664.1;
	2285	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1334.9;
	2286	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	858.8;
	2286	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	676;
	2286	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	787.2;
	2286	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3053.8;
	2286	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8855.6;
	2286	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4964.1;
	2286	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8533.3;
	2286	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1545.3;
	2287	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	881;
	2287	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	701.5;
	2287	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	826.8;
	2287	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3148.1;
	2287	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9300.2;
	2287	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5191.3;
	2287	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8726.7;
	2287	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1606.4;
	2288	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.9;
	2288	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	710.5;
	2288	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	826;
	2288	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3133.4;
	2288	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9275.6;
	2288	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5170.1;
	2288	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8870.6;
	2288	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1608;
	2289	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896;
	2289	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	724.9;
	2289	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	830.7;
	2289	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3239.5;
	2289	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9544.5;
	2289	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5237.2;
	2289	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9135.2;
	2289	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1633.9;
	2290	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.5;
	2290	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.4;
	2290	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.3;
	2290	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3370.5;
	2290	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9922.7;
	2290	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5355.3;
	2290	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9438.8;
	2290	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1639.7;
	2291	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.7;
	2291	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753.4;
	2291	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.8;
	2291	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3542.3;
	2291	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10247.4;
	2291	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5479;
	2291	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9677.7;
	2291	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1695;
	2292	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.5;
	2292	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772.3;
	2292	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	881.4;
	2292	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3721.7;
	2292	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10635.2;
	2292	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5614.6;
	2292	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9981;
	2292	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1745.6;
	2293	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.2;
	2293	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	790.9;
	2293	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	903.5;
	2293	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3924;
	2293	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10997;
	2293	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5796.6;
	2293	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10371.8;
	2293	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1804.2;
	2294	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969.4;
	2294	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	812.4;
	2294	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	928;
	2294	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4110.3;
	2294	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11237.5;
	2294	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5968.5;
	2294	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10607.5;
	2294	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1847.6;
	2295	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	979;
	2295	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	829.1;
	2295	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	945.4;
	2295	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4255.5;
	2295	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11502.2;
	2295	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6181.7;
	2295	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11002.4;
	2295	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1848;
	2296	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	983.9;
	2296	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	835.8;
	2296	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	963.4;
	2296	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4352.4;
	2296	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11689.5;
	2296	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6391.5;
	2296	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11216;
	2296	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1906.7;
	2297	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.2;
	2297	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	823.3;
	2297	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	968.6;
	2297	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4308.2;
	2297	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11688.7;
	2297	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6498.1;
	2297	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11140.4;
	2297	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1907.7;
	2298	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958;
	2298	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	807.8;
	2298	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	953.7;
	2298	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4155.9;
	2298	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11447.6;
	2298	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6450.6;
	2298	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10755.3;
	2298	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1878.3;
	2299	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943;
	2299	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	792.2;
	2299	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	933.2;
	2299	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3969.4;
	2299	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11157.7;
	2299	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6319.4;
	2299	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10479.5;
	2299	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1861.2;
	2300	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950;
	2300	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	807.9;
	2300	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	946.2;
	2300	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4034.5;
	2300	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11390.1;
	2300	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6396.9;
	2300	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10482.8;
	2300	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1876.9;
	2301	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.8;
	2301	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776;
	2301	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	923.6;
	2301	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3907.8;
	2301	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10891.9;
	2301	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6080.8;
	2301	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10012;
	2301	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1799.2;
	2302	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.9;
	2302	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720.9;
	2302	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.6;
	2302	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3598.3;
	2302	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9884.5;
	2302	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5435.5;
	2302	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9239.3;
	2302	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1645.2;
	2303	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	862.6;
	2303	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	662.5;
	2303	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	777.6;
	2303	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3192.3;
	2303	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8761.2;
	2303	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4816.9;
	2303	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8437;
	2303	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1478.9;
	2304	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	845.9;
	2304	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	628;
	2304	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	734.2;
	2304	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2898.8;
	2304	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7915.8;
	2304	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4368.8;
	2304	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7897.9;
	2304	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1368.3;
	2305	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	832.2;
	2305	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	604.3;
	2305	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	705.3;
	2305	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2734.1;
	2305	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7436.9;
	2305	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4123.5;
	2305	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7584.7;
	2305	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1279.7;
	2306	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	818.2;
	2306	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588.5;
	2306	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	683.9;
	2306	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2628.6;
	2306	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7160.6;
	2306	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4001.4;
	2306	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7393.1;
	2306	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1254.3;
	2307	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	810.8;
	2307	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.3;
	2307	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	676.9;
	2307	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2583.9;
	2307	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7030.5;
	2307	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3946.9;
	2307	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7333.4;
	2307	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1236.9;
	2308	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	808.6;
	2308	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	591.1;
	2308	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	681.9;
	2308	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2582.5;
	2308	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7122.1;
	2308	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4031.2;
	2308	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7447.2;
	2308	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1236.7;
	2309	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	824.8;
	2309	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	613.5;
	2309	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	712.9;
	2309	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2727.4;
	2309	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7712.2;
	2309	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4381.8;
	2309	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8052.3;
	2309	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1316.7;
	2310	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	859.8;
	2310	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	672.2;
	2310	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	784.8;
	2310	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3095.1;
	2310	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8861.9;
	2310	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5062.5;
	2310	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8870.6;
	2310	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1504.3;
	2311	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	876.7;
	2311	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	696.6;
	2311	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	815.5;
	2311	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3179.3;
	2311	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9292.6;
	2311	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5338.4;
	2311	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9033.9;
	2311	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1549.8;
	2312	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.9;
	2312	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	698.5;
	2312	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	809.9;
	2312	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3147.6;
	2312	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9292.2;
	2312	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5358;
	2312	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9110.5;
	2312	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1556.5;
	2313	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.2;
	2313	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	708.8;
	2313	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	816.6;
	2313	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3283.5;
	2313	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9456.6;
	2313	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5439.9;
	2313	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9323.4;
	2313	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1599.1;
	2314	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.8;
	2314	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	726.3;
	2314	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	828;
	2314	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3454.4;
	2314	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9711.1;
	2314	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5515.3;
	2314	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9588.8;
	2314	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1615.9;
	2315	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.5;
	2315	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.3;
	2315	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	840.8;
	2315	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3609.2;
	2315	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9856.9;
	2315	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5618.5;
	2315	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9804;
	2315	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1660.9;
	2316	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.3;
	2316	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	732.6;
	2316	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.5;
	2316	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3804.8;
	2316	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9979.1;
	2316	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5770.5;
	2316	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9890.3;
	2316	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1691.2;
	2317	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.8;
	2317	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.3;
	2317	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.7;
	2317	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4004.1;
	2317	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10164.5;
	2317	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5886;
	2317	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10057.1;
	2317	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1725.7;
	2318	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920.6;
	2318	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746.6;
	2318	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878.7;
	2318	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4189.5;
	2318	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10267.4;
	2318	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5991.7;
	2318	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10249.5;
	2318	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1767.1;
	2319	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.2;
	2319	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.3;
	2319	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	896.5;
	2319	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4308.6;
	2319	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10409.5;
	2319	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6161.3;
	2319	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10427.7;
	2319	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1781;
	2320	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948.8;
	2320	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	760.2;
	2320	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	917.6;
	2320	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4423.8;
	2320	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10605.9;
	2320	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6354.1;
	2320	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10651.4;
	2320	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1741.3;
	2321	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.7;
	2321	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	756.9;
	2321	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	923.7;
	2321	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4382.8;
	2321	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10689.5;
	2321	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6453.3;
	2321	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10624.8;
	2321	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1738.3;
	2322	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.8;
	2322	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	748.8;
	2322	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	923.4;
	2322	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4254.1;
	2322	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10614;
	2322	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6423.7;
	2322	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10412.3;
	2322	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1744;
	2323	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.5;
	2323	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.2;
	2323	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	923.8;
	2323	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4132.3;
	2323	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10396.3;
	2323	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6266.4;
	2323	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10179;
	2323	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1694.4;
	2324	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.9;
	2324	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	757.4;
	2324	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	934.8;
	2324	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4221.9;
	2324	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10590.4;
	2324	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6315.7;
	2324	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10272.2;
	2324	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1731.7;
	2325	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.1;
	2325	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	732.5;
	2325	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	898.2;
	2325	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4088.2;
	2325	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10137.6;
	2325	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5983.9;
	2325	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9826.4;
	2325	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1705.9;
	2326	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.4;
	2326	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	676.1;
	2326	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	818.9;
	2326	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3754.2;
	2326	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9186.7;
	2326	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5326.9;
	2326	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9052.1;
	2326	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1559.5;
	2327	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	839.1;
	2327	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	618.7;
	2327	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	743.2;
	2327	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3360.9;
	2327	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8172.4;
	2327	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4691.1;
	2327	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8284.7;
	2327	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1407.9;
	2328	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	823.4;
	2328	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	594.8;
	2328	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	697.8;
	2328	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3047.8;
	2328	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7460.7;
	2328	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4248.6;
	2328	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7735;
	2328	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1297.6;
	2329	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	819.4;
	2329	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	581.7;
	2329	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	678.8;
	2329	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2863.5;
	2329	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7046.9;
	2329	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4007.3;
	2329	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7456.7;
	2329	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1237.3;
	2330	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	815.7;
	2330	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	573.8;
	2330	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	671.8;
	2330	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2747.6;
	2330	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6847.9;
	2330	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3874.9;
	2330	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7259.1;
	2330	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1183;
	2331	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	806.4;
	2331	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.5;
	2331	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	664;
	2331	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2664.7;
	2331	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6760.7;
	2331	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3828.8;
	2331	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7167.7;
	2331	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1198.3;
	2332	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	809.4;
	2332	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	585;
	2332	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	674.6;
	2332	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2657.5;
	2332	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6959;
	2332	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3893.2;
	2332	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7335.2;
	2332	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1245.6;
	2333	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	827.6;
	2333	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	617;
	2333	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	711.4;
	2333	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2787.6;
	2333	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7643.9;
	2333	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4241.2;
	2333	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7873.7;
	2333	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1345;
	2334	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.4;
	2334	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	689.6;
	2334	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	795.7;
	2334	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3172.8;
	2334	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8935.5;
	2334	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4939.1;
	2334	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8745.5;
	2334	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1560.5;
	2335	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.8;
	2335	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	712.8;
	2335	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838.3;
	2335	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3261.5;
	2335	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9367.1;
	2335	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5177.7;
	2335	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8903;
	2335	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1598.3;
	2336	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.6;
	2336	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711.1;
	2336	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835;
	2336	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3242.1;
	2336	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9272.5;
	2336	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5150.9;
	2336	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9034.2;
	2336	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1616.8;
	2337	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.2;
	2337	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	712.1;
	2337	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	833.5;
	2337	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3366.2;
	2337	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9394;
	2337	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5265.7;
	2337	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9267.4;
	2337	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1632.8;
	2338	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.8;
	2338	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	722.1;
	2338	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	828.9;
	2338	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3544.6;
	2338	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9610.6;
	2338	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5354.5;
	2338	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9584.1;
	2338	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1633.8;
	2339	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.4;
	2339	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	732.6;
	2339	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	830.5;
	2339	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3736.4;
	2339	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9821.2;
	2339	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5503;
	2339	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9840.6;
	2339	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1654.4;
	2340	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.7;
	2340	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.4;
	2340	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	837;
	2340	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3910.7;
	2340	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10059.3;
	2340	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5656.7;
	2340	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10085.4;
	2340	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1698.8;
	2341	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.8;
	2341	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	758.3;
	2341	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.1;
	2341	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4084.5;
	2341	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10402.1;
	2341	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5812.3;
	2341	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10477;
	2341	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1729;
	2342	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.8;
	2342	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	771.7;
	2342	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	893.5;
	2342	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4240.4;
	2342	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10754.2;
	2342	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5978.8;
	2342	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10860.8;
	2342	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1773.5;
	2343	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.5;
	2343	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	780.9;
	2343	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	922.3;
	2343	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4369.4;
	2343	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11100.1;
	2343	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6163.9;
	2343	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11269;
	2343	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1852.5;
	2344	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	973.6;
	2344	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	789.5;
	2344	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940.6;
	2344	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4471.7;
	2344	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11472;
	2344	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6326.8;
	2344	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11627.3;
	2344	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1856.1;
	2345	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959;
	2345	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	780.4;
	2345	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	929.6;
	2345	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4419.6;
	2345	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11518.9;
	2345	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6452.1;
	2345	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11691.2;
	2345	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1890.6;
	2346	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.7;
	2346	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	769.1;
	2346	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	925.9;
	2346	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4300;
	2346	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11221.1;
	2346	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6421.9;
	2346	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11288.6;
	2346	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1795.6;
	2347	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.2;
	2347	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	757.5;
	2347	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	910.4;
	2347	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4133.5;
	2347	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10961.4;
	2347	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6264.5;
	2347	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10939;
	2347	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1739.4;
	2348	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928;
	2348	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	775.9;
	2348	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	925;
	2348	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4188.1;
	2348	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11207.9;
	2348	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6343.2;
	2348	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10881.4;
	2348	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1814.1;
	2349	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.3;
	2349	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753.4;
	2349	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	899.6;
	2349	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4054;
	2349	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10708.6;
	2349	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6046.6;
	2349	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10340.3;
	2349	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1771.1;
	2350	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.3;
	2350	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	697.7;
	2350	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.9;
	2350	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3738.5;
	2350	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9716.1;
	2350	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5462.2;
	2350	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9471.6;
	2350	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1610.8;
	2351	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	854.7;
	2351	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	640.7;
	2351	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	765.3;
	2351	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3347.2;
	2351	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8633.5;
	2351	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4828.7;
	2351	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8588;
	2351	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1455.2;
	2352	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	830.3;
	2352	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	604.4;
	2352	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	713.5;
	2352	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3054.8;
	2352	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7814.2;
	2352	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4403;
	2352	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7968.1;
	2352	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1359.7;
	2353	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	815.7;
	2353	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	582.9;
	2353	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	675.5;
	2353	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2869;
	2353	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7338.9;
	2353	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4117.1;
	2353	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7588.1;
	2353	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1285.7;
	2354	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	811.3;
	2354	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.7;
	2354	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	648.5;
	2354	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2754.7;
	2354	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7012.4;
	2354	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3965.3;
	2354	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7365.8;
	2354	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1257.7;
	2355	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	809.2;
	2355	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.7;
	2355	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	640.7;
	2355	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2710.1;
	2355	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6920.1;
	2355	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3888.3;
	2355	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7308.5;
	2355	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1218.5;
	2356	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	812.6;
	2356	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	582.2;
	2356	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	647.3;
	2356	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2711.1;
	2356	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7004.6;
	2356	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3960;
	2356	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7429.9;
	2356	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1237.4;
	2357	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	820.8;
	2357	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	608.4;
	2357	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	679.2;
	2357	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2831.2;
	2357	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7524;
	2357	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4266.5;
	2357	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7862.6;
	2357	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1324.7;
	2358	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	854.3;
	2358	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	671.3;
	2358	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	759.7;
	2358	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3191.5;
	2358	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8627.1;
	2358	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4928;
	2358	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8625.2;
	2358	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1483.1;
	2359	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.9;
	2359	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	696.3;
	2359	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	806.2;
	2359	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3312.8;
	2359	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9094.6;
	2359	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5161.2;
	2359	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8785;
	2359	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1518.6;
	2360	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.6;
	2360	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	705.3;
	2360	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	809;
	2360	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3296.4;
	2360	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9175;
	2360	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5178.4;
	2360	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8941.3;
	2360	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1455.5;
	2361	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.8;
	2361	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	708.5;
	2361	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	818.8;
	2361	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3415.1;
	2361	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9380.2;
	2361	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5279.3;
	2361	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9196.9;
	2361	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1532.8;
	2362	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.4;
	2362	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	712;
	2362	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	832.5;
	2362	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3590.2;
	2362	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9592.5;
	2362	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5408.1;
	2362	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9483.4;
	2362	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1591.6;
	2363	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.2;
	2363	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	710.2;
	2363	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.7;
	2363	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3717.9;
	2363	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9708.1;
	2363	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5495.9;
	2363	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9628.1;
	2363	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1627.7;
	2364	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.1;
	2364	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	709.4;
	2364	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	831.5;
	2364	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3845.9;
	2364	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9847.7;
	2364	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5587.4;
	2364	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9822.8;
	2364	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1653.7;
	2365	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.8;
	2365	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714.9;
	2365	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	837.7;
	2365	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4029;
	2365	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9980.3;
	2365	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5701.9;
	2365	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10183.9;
	2365	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1694.2;
	2366	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.4;
	2366	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	716.1;
	2366	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	837.9;
	2366	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4169.9;
	2366	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10068.2;
	2366	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5791.6;
	2366	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10473.1;
	2366	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1718.7;
	2367	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.7;
	2367	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	712.8;
	2367	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	839;
	2367	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4241.2;
	2367	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10060;
	2367	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5856.1;
	2367	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10750.2;
	2367	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1714.4;
	2368	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.1;
	2368	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714.7;
	2368	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.1;
	2368	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4256;
	2368	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10031;
	2368	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5914;
	2368	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11001.6;
	2368	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1722.7;
	2369	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.7;
	2369	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	706.8;
	2369	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	839.6;
	2369	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4128.8;
	2369	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9885.5;
	2369	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5940.7;
	2369	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10975.7;
	2369	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1705;
	2370	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.8;
	2370	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	696;
	2370	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	830.1;
	2370	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3933.3;
	2370	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9647.4;
	2370	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5910.6;
	2370	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10599.4;
	2370	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1659.6;
	2371	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.2;
	2371	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	691;
	2371	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	820.8;
	2371	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3765.2;
	2371	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9533.2;
	2371	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5768.2;
	2371	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10235.7;
	2371	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1627.7;
	2372	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.6;
	2372	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	715.4;
	2372	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	842.9;
	2372	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3810.4;
	2372	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9779.6;
	2372	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5857;
	2372	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10258.2;
	2372	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1650.8;
	2373	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.4;
	2373	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	700.7;
	2373	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	832.6;
	2373	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3706.7;
	2373	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9495.8;
	2373	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5690;
	2373	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9875.8;
	2373	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1635.4;
	2374	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.2;
	2374	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	664.7;
	2374	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	793.2;
	2374	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3490.6;
	2374	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8884.6;
	2374	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5292.5;
	2374	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9275.6;
	2374	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1555.7;
	2375	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	853.8;
	2375	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	625.9;
	2375	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	745.6;
	2375	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3234.1;
	2375	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8143.7;
	2375	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4839.4;
	2375	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8665.1;
	2375	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1436.3;
	2376	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	841.8;
	2376	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	600;
	2376	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	711.8;
	2376	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2985.5;
	2376	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7507.4;
	2376	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4416.3;
	2376	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8129.6;
	2376	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1326.5;
	2377	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	830.2;
	2377	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.4;
	2377	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	689;
	2377	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2812.3;
	2377	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7115.9;
	2377	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4112.1;
	2377	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7779.5;
	2377	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1269.2;
	2378	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	822;
	2378	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	574.6;
	2378	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	669.5;
	2378	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2704.7;
	2378	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6878.7;
	2378	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3960.4;
	2378	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7538.8;
	2378	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1220.3;
	2379	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	817;
	2379	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.3;
	2379	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	664.7;
	2379	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2634;
	2379	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6734.9;
	2379	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3895.6;
	2379	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7383.6;
	2379	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1219.1;
	2380	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	807.9;
	2380	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	575.9;
	2380	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	666.8;
	2380	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2592.6;
	2380	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6764.4;
	2380	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3825.2;
	2380	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7329.8;
	2380	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1187.4;
	2381	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	803.3;
	2381	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	585.5;
	2381	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	682.9;
	2381	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2613.6;
	2381	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6932.6;
	2381	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3908.4;
	2381	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7361.4;
	2381	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1233.5;
	2382	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	812.3;
	2382	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	610.2;
	2382	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	715.6;
	2382	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2715.2;
	2382	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7361.8;
	2382	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4138.8;
	2382	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7600.1;
	2382	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1283.4;
	2383	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	822.6;
	2383	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	628;
	2383	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	739.1;
	2383	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2781;
	2383	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7756;
	2383	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4358.7;
	2383	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7809.6;
	2383	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1305.9;
	2384	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	830.7;
	2384	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	652.4;
	2384	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	769.3;
	2384	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2933.3;
	2384	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8238.2;
	2384	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4583.6;
	2384	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8133.6;
	2384	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1421.1;
	2385	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	847.3;
	2385	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	672.2;
	2385	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	795.6;
	2385	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3169.3;
	2385	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8677.7;
	2385	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4867.1;
	2385	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8511.4;
	2385	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1460.5;
	2386	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	863.5;
	2386	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	681.8;
	2386	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	807.7;
	2386	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3406.2;
	2386	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8939.7;
	2386	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5050;
	2386	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8849.1;
	2386	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1499.2;
	2387	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.4;
	2387	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	682;
	2387	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	808.3;
	2387	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3601.1;
	2387	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9085.7;
	2387	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5174.4;
	2387	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9123.7;
	2387	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1492.4;
	2388	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.8;
	2388	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	673.1;
	2388	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	799.5;
	2388	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3738;
	2388	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9070.6;
	2388	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5213.3;
	2388	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9190.4;
	2388	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1510.1;
	2389	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.7;
	2389	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	666.9;
	2389	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	792.1;
	2389	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3855.1;
	2389	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8924.9;
	2389	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5266.2;
	2389	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9355.3;
	2389	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1494.2;
	2390	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.8;
	2390	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	658;
	2390	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	781.9;
	2390	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3949.2;
	2390	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8775.6;
	2390	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5290.5;
	2390	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9602.6;
	2390	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1495.9;
	2391	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.1;
	2391	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	656;
	2391	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	777.7;
	2391	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4012.8;
	2391	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8635.7;
	2391	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5407.2;
	2391	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9763.4;
	2391	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1481.6;
	2392	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.1;
	2392	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	658.6;
	2392	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	783.5;
	2392	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4028.5;
	2392	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8575.7;
	2392	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5543.9;
	2392	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9689.1;
	2392	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1503.5;
	2393	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925;
	2393	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	660.9;
	2393	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	791.8;
	2393	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3971.5;
	2393	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8585.9;
	2393	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5595.1;
	2393	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9585.6;
	2393	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1516.2;
	2394	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.3;
	2394	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	660.8;
	2394	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	796.2;
	2394	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3852.3;
	2394	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8598.8;
	2394	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5581.1;
	2394	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9463.6;
	2394	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1535.5;
	2395	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.9;
	2395	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	664;
	2395	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	798.9;
	2395	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3728.3;
	2395	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8694;
	2395	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5578.8;
	2395	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9379.2;
	2395	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1550.1;
	2396	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.5;
	2396	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	686.4;
	2396	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824.8;
	2396	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3794.3;
	2396	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8984.3;
	2396	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5742;
	2396	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9658.3;
	2396	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1599;
	2397	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.6;
	2397	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	674.1;
	2397	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	818.5;
	2397	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3712.4;
	2397	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8817.5;
	2397	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5576.6;
	2397	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9509.7;
	2397	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1538.2;
	2398	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.6;
	2398	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	652;
	2398	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	784.3;
	2398	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3550.2;
	2398	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8405.4;
	2398	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5254.6;
	2398	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9165.6;
	2398	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1481.8;
	2399	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867.2;
	2399	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	623.1;
	2399	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	742.9;
	2399	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3320.7;
	2399	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7862;
	2399	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4864.6;
	2399	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8676.7;
	2399	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1392.5;
	2400	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	848.2;
	2400	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	601.8;
	2400	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	712;
	2400	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3116.3;
	2400	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7355.7;
	2400	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4523.2;
	2400	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8219.5;
	2400	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1298.8;
	2401	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	831.4;
	2401	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	585;
	2401	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	686.3;
	2401	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2932.1;
	2401	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7009.4;
	2401	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4267.9;
	2401	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7892.4;
	2401	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1253.4;
	2402	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	826.3;
	2402	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	578.5;
	2402	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	670.9;
	2402	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2802.2;
	2402	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6785;
	2402	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4091.1;
	2402	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7695.9;
	2402	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1207.6;
	2403	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	822.2;
	2403	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	573.7;
	2403	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	666.7;
	2403	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2728.8;
	2403	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6692.8;
	2403	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3983.4;
	2403	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7599.6;
	2403	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1126.3;
	2404	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	822.7;
	2404	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	575.3;
	2404	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	667.8;
	2404	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2693.1;
	2404	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6695.9;
	2404	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3938.9;
	2404	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7564.4;
	2404	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1102.3;
	2405	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	825.4;
	2405	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.2;
	2405	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	674.9;
	2405	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2698.3;
	2405	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6799.2;
	2405	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3983.3;
	2405	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7601.8;
	2405	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1121;
	2406	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	833;
	2406	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	595.8;
	2406	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	696.3;
	2406	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2749.9;
	2406	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7069.8;
	2406	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4125.7;
	2406	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7753.4;
	2406	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1155.9;
	2407	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	831.3;
	2407	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	605.5;
	2407	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	716.6;
	2407	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2762.3;
	2407	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7358.4;
	2407	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4282.6;
	2407	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7738;
	2407	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1185.5;
	2408	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	834.1;
	2408	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	634.2;
	2408	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	748.7;
	2408	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2923.1;
	2408	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7858.9;
	2408	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4563.1;
	2408	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8131.3;
	2408	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1273.6;
	2409	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.4;
	2409	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	656.3;
	2409	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	772.8;
	2409	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3184.7;
	2409	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8347.3;
	2409	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4893.5;
	2409	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8653.4;
	2409	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1336.4;
	2410	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	869.2;
	2410	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	660.8;
	2410	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	787;
	2410	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3430.7;
	2410	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8654.4;
	2410	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5126.3;
	2410	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9118.3;
	2410	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1374.2;
	2411	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.8;
	2411	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	669.3;
	2411	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	800.7;
	2411	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3668;
	2411	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8825;
	2411	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5261.6;
	2411	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9511.8;
	2411	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1415.5;
	2412	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.1;
	2412	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	683.1;
	2412	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	816.6;
	2412	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3868.9;
	2412	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8982.1;
	2412	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5368.1;
	2412	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9863.3;
	2412	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1458.9;
	2413	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.7;
	2413	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	699.5;
	2413	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	830.7;
	2413	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4046.9;
	2413	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9072.1;
	2413	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5411.6;
	2413	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10202;
	2413	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1506.1;
	2414	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.6;
	2414	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714.7;
	2414	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	845;
	2414	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4173.9;
	2414	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9167.7;
	2414	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5514.8;
	2414	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10453.6;
	2414	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1534;
	2415	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.4;
	2415	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729;
	2415	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.3;
	2415	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4234.4;
	2415	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9254.4;
	2415	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5598.9;
	2415	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10545.4;
	2415	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1571.7;
	2416	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.3;
	2416	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735.8;
	2416	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	890.1;
	2416	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4234;
	2416	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9392.6;
	2416	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5739.7;
	2416	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10567.4;
	2416	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1612.7;
	2417	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.9;
	2417	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.4;
	2417	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	908.9;
	2417	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4194.6;
	2417	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9595.3;
	2417	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5890.8;
	2417	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10461.7;
	2417	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1617;
	2418	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.5;
	2418	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.8;
	2418	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	917.4;
	2418	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4117.4;
	2418	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9715.4;
	2418	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5972.9;
	2418	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10327.8;
	2418	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1620.5;
	2419	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.9;
	2419	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.6;
	2419	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	909.3;
	2419	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4067;
	2419	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9847.7;
	2419	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6093;
	2419	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10478.8;
	2419	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1627.9;
	2420	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.6;
	2420	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	760.3;
	2420	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	921.5;
	2420	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4194.6;
	2420	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10310.9;
	2420	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6337.3;
	2420	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10727.2;
	2420	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1703.8;
	2421	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.9;
	2421	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	739.7;
	2421	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	904.5;
	2421	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4144.5;
	2421	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10155.3;
	2421	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6181.1;
	2421	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10501.7;
	2421	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1657.2;
	2422	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.6;
	2422	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	693;
	2422	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.9;
	2422	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3900.3;
	2422	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9418.1;
	2422	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5738;
	2422	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9928.5;
	2422	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1531.8;
	2423	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.5;
	2423	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	646.8;
	2423	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	779;
	2423	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3567.3;
	2423	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8513;
	2423	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5142.5;
	2423	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9272.8;
	2423	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1396.3;
	2424	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	847.5;
	2424	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	614.1;
	2424	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	723.2;
	2424	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3278.1;
	2424	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7873;
	2424	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4698.8;
	2424	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8721.2;
	2424	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1301.5;
	2425	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	826.7;
	2425	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	596.6;
	2425	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	695.4;
	2425	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3080.8;
	2425	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7471.6;
	2425	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4455.1;
	2425	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8361.3;
	2425	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1253;
	2426	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	814.8;
	2426	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	581.8;
	2426	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	678.9;
	2426	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2976.2;
	2426	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7242.4;
	2426	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4319.9;
	2426	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8132.1;
	2426	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1220.4;
	2427	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	810.5;
	2427	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579.5;
	2427	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	673.5;
	2427	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2910.8;
	2427	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7196.9;
	2427	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4272.3;
	2427	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8120.1;
	2427	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1218.6;
	2428	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	814.8;
	2428	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	585;
	2428	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	680;
	2428	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2923.6;
	2428	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7369.2;
	2428	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4341.9;
	2428	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8354.1;
	2428	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1248.4;
	2429	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	822.1;
	2429	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	611.5;
	2429	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	709.2;
	2429	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3064.9;
	2429	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7952.4;
	2429	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4730.8;
	2429	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8881.7;
	2429	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1331.6;
	2430	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	851.5;
	2430	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	669.7;
	2430	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	786.7;
	2430	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3429.4;
	2430	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9096;
	2430	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5401.1;
	2430	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9658.7;
	2430	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1514.1;
	2431	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.9;
	2431	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	704.8;
	2431	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	828.7;
	2431	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3540.6;
	2431	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9699;
	2431	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5697.2;
	2431	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9940.2;
	2431	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1603.4;
	2432	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.2;
	2432	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	715.9;
	2432	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	837.1;
	2432	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3549;
	2432	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9814.5;
	2432	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5734.3;
	2432	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10011.8;
	2432	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1635.3;
	2433	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893;
	2433	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.5;
	2433	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	864;
	2433	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3702.9;
	2433	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10044.3;
	2433	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5874.5;
	2433	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10283.9;
	2433	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1665.7;
	2434	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.5;
	2434	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	745.8;
	2434	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	890.3;
	2434	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3888.5;
	2434	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10371.3;
	2434	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6011.1;
	2434	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10733.1;
	2434	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1683.5;
	2435	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.6;
	2435	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	757.5;
	2435	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	918.3;
	2435	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4054.3;
	2435	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10775.9;
	2435	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6195.4;
	2435	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11139.9;
	2435	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1708.1;
	2436	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.5;
	2436	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	770.5;
	2436	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	941.6;
	2436	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4193.3;
	2436	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11190.6;
	2436	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6476.1;
	2436	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11282.7;
	2436	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1733.7;
	2437	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.9;
	2437	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777.4;
	2437	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	967.8;
	2437	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4316.8;
	2437	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11656.6;
	2437	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6813.7;
	2437	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11298.9;
	2437	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1799.3;
	2438	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.7;
	2438	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	770.1;
	2438	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	990.3;
	2438	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4456.2;
	2438	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12101.1;
	2438	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7211.8;
	2438	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11245.6;
	2438	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1861.9;
	2439	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.4;
	2439	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	754.8;
	2439	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1014.2;
	2439	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4623.8;
	2439	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12537.8;
	2439	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7601.7;
	2439	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11186.9;
	2439	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1868.8;
	2440	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.4;
	2440	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	745.5;
	2440	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1034.5;
	2440	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4740.8;
	2440	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12705;
	2440	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7940.4;
	2440	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11100.5;
	2440	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1947.6;
	2441	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.8;
	2441	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	739.9;
	2441	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1037;
	2441	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4719.7;
	2441	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12456.2;
	2441	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8099.7;
	2441	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11021.2;
	2441	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1928.5;
	2442	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.7;
	2442	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.7;
	2442	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1025.9;
	2442	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4590.2;
	2442	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11842.1;
	2442	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8045;
	2442	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10848;
	2442	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1924.3;
	2443	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.1;
	2443	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.7;
	2443	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	988.6;
	2443	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4471.3;
	2443	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11361;
	2443	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7765.9;
	2443	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10895.5;
	2443	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1905.5;
	2444	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.2;
	2444	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747;
	2444	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	983.1;
	2444	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4562;
	2444	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11417;
	2444	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7724.6;
	2444	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11119.8;
	2444	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1932.9;
	2445	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.4;
	2445	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.1;
	2445	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	941.6;
	2445	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4417;
	2445	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10932.8;
	2445	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7332.6;
	2445	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10761.2;
	2445	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1844.7;
	2446	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.2;
	2446	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	675.2;
	2446	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.4;
	2446	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4050.8;
	2446	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9868.5;
	2446	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6607.3;
	2446	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10018.4;
	2446	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1688.5;
	2447	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	850.3;
	2447	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	630.1;
	2447	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	783.5;
	2447	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3648.9;
	2447	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8753.6;
	2447	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5807.7;
	2447	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9301.6;
	2447	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1531.5;
	2448	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	823.3;
	2448	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	604.7;
	2448	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	731.8;
	2448	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3333;
	2448	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7951.1;
	2448	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5295;
	2448	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8853.9;
	2448	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1402.2;
	2449	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	813.4;
	2449	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586.2;
	2449	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	706.1;
	2449	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3138;
	2449	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7481.3;
	2449	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4940;
	2449	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8558.3;
	2449	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1319.7;
	2450	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	809.5;
	2450	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	580.4;
	2450	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	689.4;
	2450	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3036.3;
	2450	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7167.6;
	2450	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4687.4;
	2450	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8356;
	2450	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1283.9;
	2451	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	807;
	2451	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	580;
	2451	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	679.7;
	2451	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2979.6;
	2451	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7025.8;
	2451	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4535.7;
	2451	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8257.7;
	2451	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1280.6;
	2452	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	805;
	2452	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584;
	2452	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	684.3;
	2452	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2973.8;
	2452	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7150.6;
	2452	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4524.6;
	2452	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8338.7;
	2452	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1301.6;
	2453	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	822.9;
	2453	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	619.1;
	2453	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	716.2;
	2453	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3107.3;
	2453	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7750.6;
	2453	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4744.7;
	2453	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8831.9;
	2453	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1373;
	2454	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	860.6;
	2454	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	693.6;
	2454	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	791.8;
	2454	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3491.2;
	2454	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8879.8;
	2454	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5328.5;
	2454	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9678.9;
	2454	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1558.6;
	2455	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.9;
	2455	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	718.7;
	2455	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	826;
	2455	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3585;
	2455	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9322.4;
	2455	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5529.4;
	2455	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9860.7;
	2455	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1624.2;
	2456	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	881.1;
	2456	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	726.3;
	2456	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	826.2;
	2456	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3568.6;
	2456	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9300.8;
	2456	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5464.8;
	2456	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9968.7;
	2456	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1629.2;
	2457	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.2;
	2457	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	731.5;
	2457	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	832.4;
	2457	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3634.8;
	2457	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9462.1;
	2457	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5535.2;
	2457	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10157.6;
	2457	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1654.9;
	2458	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889;
	2458	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735.6;
	2458	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	844.2;
	2458	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3751.2;
	2458	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9676.2;
	2458	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5641.4;
	2458	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10419.1;
	2458	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1661.7;
	2459	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891;
	2459	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	731;
	2459	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	855.5;
	2459	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3867.7;
	2459	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9793.3;
	2459	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5705.4;
	2459	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10678.2;
	2459	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1692.5;
	2460	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.8;
	2460	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	724.9;
	2460	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849;
	2460	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3975.5;
	2460	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9856.7;
	2460	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5752.2;
	2460	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11000.1;
	2460	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1698.3;
	2461	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.4;
	2461	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	726.7;
	2461	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	846.1;
	2461	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4093;
	2461	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9924.9;
	2461	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5783.3;
	2461	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11248.2;
	2461	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1708.6;
	2462	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.4;
	2462	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	731.1;
	2462	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	840.1;
	2462	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4187.9;
	2462	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9854.5;
	2462	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5738.7;
	2462	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11325.1;
	2462	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1702.2;
	2463	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.7;
	2463	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.3;
	2463	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	837.5;
	2463	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4288.6;
	2463	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9856.6;
	2463	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5711;
	2463	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11290.5;
	2463	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1650.1;
	2464	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.3;
	2464	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.2;
	2464	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	850.3;
	2464	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4361.9;
	2464	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9848.2;
	2464	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5762.2;
	2464	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11357;
	2464	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1678.3;
	2465	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.9;
	2465	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723.9;
	2465	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	858.9;
	2465	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4315.7;
	2465	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9814.2;
	2465	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5809.7;
	2465	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11302.7;
	2465	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1665.2;
	2466	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.3;
	2466	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	732.8;
	2466	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	863.4;
	2466	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4204.2;
	2466	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9810.7;
	2466	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5875.8;
	2466	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10981.4;
	2466	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1672.6;
	2467	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.5;
	2467	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.8;
	2467	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	876.5;
	2467	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4137.9;
	2467	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9904;
	2467	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6008.5;
	2467	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10900.3;
	2467	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1711.6;
	2468	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.5;
	2468	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753.2;
	2468	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884;
	2468	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4238.7;
	2468	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10195.2;
	2468	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6172.3;
	2468	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11006.2;
	2468	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1739;
	2469	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.4;
	2469	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730;
	2469	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	862.2;
	2469	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4147.4;
	2469	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9810.7;
	2469	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5906.9;
	2469	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10683.2;
	2469	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1686.2;
	2470	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.9;
	2470	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	688.9;
	2470	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	811.5;
	2470	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3857;
	2470	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8994.2;
	2470	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5312.5;
	2470	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10050.2;
	2470	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1566.4;
	2471	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	852.3;
	2471	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	641.8;
	2471	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	762.6;
	2471	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3474.7;
	2471	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8086;
	2471	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4735.8;
	2471	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9153.4;
	2471	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1430.4;
	2472	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	842.8;
	2472	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	620.3;
	2472	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	737.8;
	2472	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3208.1;
	2472	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7450.8;
	2472	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4352.5;
	2472	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8587.2;
	2472	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1354.1;
	2473	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	832.2;
	2473	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	608.3;
	2473	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	727.1;
	2473	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3034.9;
	2473	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7112;
	2473	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4104;
	2473	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8218.4;
	2473	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1274.1;
	2474	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	821.2;
	2474	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	606.1;
	2474	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	715.5;
	2474	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2917.4;
	2474	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6921.3;
	2474	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3964.7;
	2474	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7853.7;
	2474	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1256.8;
	2475	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	815.6;
	2475	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	600.2;
	2475	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	712.7;
	2475	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2855;
	2475	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6849.7;
	2475	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3919.5;
	2475	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7708.3;
	2475	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1245.5;
	2476	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	817.4;
	2476	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	603;
	2476	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	715.1;
	2476	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2854.7;
	2476	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7016.7;
	2476	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3972.3;
	2476	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7768.3;
	2476	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1248.7;
	2477	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	834;
	2477	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	636;
	2477	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	745.8;
	2477	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2970.9;
	2477	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7642.2;
	2477	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4287.6;
	2477	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8195;
	2477	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1319.1;
	2478	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.8;
	2478	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	699.6;
	2478	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	819.6;
	2478	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3332.6;
	2478	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8825.2;
	2478	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4969.9;
	2478	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8937.9;
	2478	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1505.8;
	2479	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.1;
	2479	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.4;
	2479	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	841.4;
	2479	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3443.3;
	2479	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9357.9;
	2479	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5216.1;
	2479	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9235.2;
	2479	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1572.7;
	2480	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.6;
	2480	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	724.4;
	2480	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	829.1;
	2480	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3389.6;
	2480	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9382.5;
	2480	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5216.9;
	2480	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9307.6;
	2480	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1580.2;
	2481	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.9;
	2481	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	732.1;
	2481	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	834.5;
	2481	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3444.8;
	2481	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9506.5;
	2481	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5306.2;
	2481	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9494;
	2481	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1623.9;
	2482	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	881.3;
	2482	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.8;
	2482	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	844;
	2482	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3513;
	2482	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9651.7;
	2482	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5414.4;
	2482	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9693.3;
	2482	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1618.2;
	2483	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.3;
	2483	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.8;
	2483	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851.3;
	2483	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3578.1;
	2483	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9687.3;
	2483	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5454.2;
	2483	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9820.6;
	2483	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1648.7;
	2484	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.8;
	2484	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735.5;
	2484	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	846.8;
	2484	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3621.5;
	2484	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9660.3;
	2484	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5450.4;
	2484	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9883.9;
	2484	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1662.8;
	2485	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.6;
	2485	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.7;
	2485	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	850.9;
	2485	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3694.7;
	2485	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9650.7;
	2485	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5476.7;
	2485	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10045.2;
	2485	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1670.2;
	2486	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.6;
	2486	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.2;
	2486	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.2;
	2486	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3762;
	2486	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9513.2;
	2486	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5491.9;
	2486	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10118.9;
	2486	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1678.7;
	2487	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.5;
	2487	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.8;
	2487	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	871;
	2487	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3852.5;
	2487	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9440.3;
	2487	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5567.2;
	2487	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10130.3;
	2487	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1664.3;
	2488	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.7;
	2488	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743;
	2488	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.2;
	2488	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3976.9;
	2488	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9458.1;
	2488	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5652.2;
	2488	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10145.4;
	2488	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1614.7;
	2489	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.8;
	2489	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743;
	2489	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884.1;
	2489	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3987.9;
	2489	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9485.8;
	2489	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5764.1;
	2489	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10147.9;
	2489	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1607.5;
	2490	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.2;
	2490	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.4;
	2490	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	887.6;
	2490	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3935.6;
	2490	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9516;
	2490	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5836.3;
	2490	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9997.5;
	2490	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1640.7;
	2491	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.8;
	2491	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742.1;
	2491	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	886.1;
	2491	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3882.9;
	2491	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9643.3;
	2491	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5817.9;
	2491	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10051;
	2491	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1647.7;
	2492	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.6;
	2492	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	762.3;
	2492	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897.6;
	2492	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4019.5;
	2492	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9970.1;
	2492	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6015.8;
	2492	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10207.1;
	2492	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1691.1;
	2493	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.2;
	2493	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.8;
	2493	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	876.9;
	2493	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3963.6;
	2493	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9615.7;
	2493	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5826.6;
	2493	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9946.6;
	2493	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1685.4;
	2494	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.4;
	2494	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	685.4;
	2494	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	814.5;
	2494	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3685;
	2494	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8858;
	2494	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5308.8;
	2494	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9317.9;
	2494	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1564;
	2495	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	845.1;
	2495	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	634.9;
	2495	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	744.8;
	2495	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3302.5;
	2495	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7994.5;
	2495	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4707.4;
	2495	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8528.9;
	2495	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1424.5;
	2496	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	818.1;
	2496	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	610.2;
	2496	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	719.9;
	2496	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2992.1;
	2496	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7416.5;
	2496	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4302.1;
	2496	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7984.3;
	2496	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1325.5;
	2497	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	806.4;
	2497	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	596.5;
	2497	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	702.1;
	2497	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2803.3;
	2497	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7055.8;
	2497	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4074.4;
	2497	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7658.7;
	2497	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1271;
	2498	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	802.6;
	2498	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	591.6;
	2498	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	695.6;
	2498	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2698.6;
	2498	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6893.5;
	2498	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3938.1;
	2498	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7505.9;
	2498	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1214.9;
	2499	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	795.4;
	2499	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588.9;
	2499	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	692;
	2499	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2636.8;
	2499	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6850.4;
	2499	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3886;
	2499	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7426.2;
	2499	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1230.4;
	2500	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	796.6;
	2500	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	601.2;
	2500	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	702.3;
	2500	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2633.6;
	2500	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7025.3;
	2500	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3953;
	2500	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7532.6;
	2500	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1258.4;
	2501	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	812.7;
	2501	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	638.6;
	2501	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	749.3;
	2501	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2751.8;
	2501	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7670.2;
	2501	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4289.6;
	2501	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8025.3;
	2501	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1326.1;
	2502	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	852.3;
	2502	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	706.3;
	2502	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838.7;
	2502	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3118.6;
	2502	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8858.3;
	2502	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4973.2;
	2502	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8803.5;
	2502	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1509;
	2503	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.2;
	2503	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735.4;
	2503	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.7;
	2503	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3207.1;
	2503	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9376.6;
	2503	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5191.2;
	2503	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9094.3;
	2503	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1554.2;
	2504	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.8;
	2504	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.8;
	2504	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	870.6;
	2504	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3221.4;
	2504	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9312.8;
	2504	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5171.5;
	2504	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9241.9;
	2504	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1594.9;
	2505	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.1;
	2505	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.4;
	2505	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	863.7;
	2505	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3388.6;
	2505	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9398;
	2505	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5274.6;
	2505	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9484.9;
	2505	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1624.6;
	2506	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.1;
	2506	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.7;
	2506	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	856.6;
	2506	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3676.1;
	2506	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9550;
	2506	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5398.7;
	2506	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9746.2;
	2506	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1630.3;
	2507	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.1;
	2507	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742.8;
	2507	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	842.7;
	2507	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3897.9;
	2507	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9642.8;
	2507	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5531.2;
	2507	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10033.9;
	2507	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1632.6;
	2508	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.1;
	2508	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.4;
	2508	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838.2;
	2508	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4106.3;
	2508	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9723.4;
	2508	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5657.7;
	2508	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10267.8;
	2508	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1652.8;
	2509	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.6;
	2509	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	739.1;
	2509	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.5;
	2509	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4324.3;
	2509	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9820.9;
	2509	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5837;
	2509	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10578.7;
	2509	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1635.7;
	2510	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.2;
	2510	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.4;
	2510	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	869.1;
	2510	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4504.3;
	2510	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9890.4;
	2510	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6031.5;
	2510	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10858.8;
	2510	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1627.9;
	2511	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.9;
	2511	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.1;
	2511	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	887.3;
	2511	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4675.3;
	2511	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9937.7;
	2511	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6234.7;
	2511	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11095.6;
	2511	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1651.7;
	2512	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990.1;
	2512	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.4;
	2512	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	910.1;
	2512	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4800.4;
	2512	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10041.3;
	2512	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6437.9;
	2512	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11305.6;
	2512	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1620.6;
	2513	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	984.4;
	2513	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.3;
	2513	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	907.3;
	2513	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4740.3;
	2513	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10051.5;
	2513	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6512.9;
	2513	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11342;
	2513	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1640.6;
	2514	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.3;
	2514	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.1;
	2514	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	902.3;
	2514	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4561.3;
	2514	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9991.8;
	2514	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6463.4;
	2514	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11046.1;
	2514	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1572.1;
	2515	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.7;
	2515	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725;
	2515	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.8;
	2515	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4322.4;
	2515	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9980;
	2515	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6235.5;
	2515	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10774.7;
	2515	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1559.3;
	2516	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948.3;
	2516	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751;
	2516	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913.3;
	2516	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4307.5;
	2516	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10383.4;
	2516	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6319.1;
	2516	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10856.9;
	2516	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1667.8;
	2517	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.8;
	2517	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.2;
	2517	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	896.7;
	2517	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4185.3;
	2517	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10127.6;
	2517	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6050.7;
	2517	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10551.8;
	2517	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1666.9;
	2518	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.5;
	2518	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	694.3;
	2518	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.8;
	2518	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3839.6;
	2518	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9299.4;
	2518	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5524.4;
	2518	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9774.9;
	2518	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1538.4;
	2519	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	881;
	2519	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	642.7;
	2519	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	767.1;
	2519	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3425.3;
	2519	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8355.2;
	2519	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4917.5;
	2519	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8936.3;
	2519	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1404.9;
	2520	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.4;
	2520	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	615.6;
	2520	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	714.6;
	2520	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3104.6;
	2520	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7675.4;
	2520	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4403.1;
	2520	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8329.4;
	2520	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1323.7;
	2521	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	837.1;
	2521	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.8;
	2521	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	685.8;
	2521	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2884.9;
	2521	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7260.7;
	2521	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4131.3;
	2521	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7904.4;
	2521	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1268.6;
	2522	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	828.5;
	2522	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	590.4;
	2522	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	672.4;
	2522	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2749.6;
	2522	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7014;
	2522	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3962.8;
	2522	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7680.9;
	2522	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1250.7;
	2523	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	825.1;
	2523	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	591;
	2523	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	669.5;
	2523	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2685.8;
	2523	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6988.7;
	2523	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3901.5;
	2523	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7610.8;
	2523	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1219.8;
	2524	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	826.5;
	2524	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	609.9;
	2524	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	675;
	2524	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2671.1;
	2524	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7113.1;
	2524	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3948.2;
	2524	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7708.6;
	2524	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1239;
	2525	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	838.2;
	2525	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	628.7;
	2525	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	707.2;
	2525	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2801.2;
	2525	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7687.1;
	2525	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4258.4;
	2525	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8170;
	2525	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1333.9;
	2526	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867.8;
	2526	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	691.3;
	2526	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	781.1;
	2526	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3154.4;
	2526	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8813.5;
	2526	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4918.6;
	2526	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8888.8;
	2526	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1499.7;
	2527	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.1;
	2527	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711.7;
	2527	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	818.2;
	2527	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3216.1;
	2527	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9248.7;
	2527	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5111;
	2527	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9021.9;
	2527	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1537.9;
	2528	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.5;
	2528	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.9;
	2528	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	818.5;
	2528	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3279.6;
	2528	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9353.3;
	2528	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5138;
	2528	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9108.6;
	2528	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1474.3;
	2529	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.3;
	2529	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	726.1;
	2529	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	828.8;
	2529	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3427.9;
	2529	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9599.1;
	2529	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5299.8;
	2529	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9395.6;
	2529	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1539.2;
	2530	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.9;
	2530	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.9;
	2530	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.3;
	2530	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3623.7;
	2530	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9820.1;
	2530	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5411.1;
	2530	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9748.2;
	2530	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1597.4;
	2531	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.1;
	2531	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	731.5;
	2531	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.9;
	2531	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3825.3;
	2531	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9881.3;
	2531	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5516.9;
	2531	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10076.6;
	2531	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1627.8;
	2532	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.5;
	2532	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.7;
	2532	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851;
	2532	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4033.2;
	2532	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9960.5;
	2532	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5653.4;
	2532	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10430.1;
	2532	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1636.6;
	2533	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.3;
	2533	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.9;
	2533	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.6;
	2533	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4262.3;
	2533	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10046.1;
	2533	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5843.4;
	2533	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10938.2;
	2533	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1653.7;
	2534	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	971.9;
	2534	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.6;
	2534	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889.7;
	2534	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4426.8;
	2534	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10167.8;
	2534	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6102.4;
	2534	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11361.9;
	2534	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1679.5;
	2535	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985;
	2535	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.6;
	2535	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913.2;
	2535	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4503.8;
	2535	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10298;
	2535	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6372.3;
	2535	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11675.9;
	2535	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1687.2;
	2536	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	994.5;
	2536	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.9;
	2536	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	932.2;
	2536	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4499.2;
	2536	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10459.3;
	2536	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6585.8;
	2536	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11926.8;
	2536	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1727;
	2537	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	987;
	2537	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.8;
	2537	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	938.1;
	2537	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4365.1;
	2537	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10455.3;
	2537	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6696.3;
	2537	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11838.4;
	2537	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1746.3;
	2538	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	976.1;
	2538	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727;
	2538	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	919.5;
	2538	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4173.2;
	2538	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10248.8;
	2538	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6578.1;
	2538	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11385.4;
	2538	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1715.6;
	2539	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.5;
	2539	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.4;
	2539	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897.9;
	2539	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4010;
	2539	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10008.7;
	2539	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6309.6;
	2539	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10914.6;
	2539	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1675.3;
	2540	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.3;
	2540	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.3;
	2540	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	905.2;
	2540	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4066.9;
	2540	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10178.3;
	2540	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6331.3;
	2540	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10836.9;
	2540	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1702.4;
	2541	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.2;
	2541	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.6;
	2541	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	890.5;
	2541	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3991.8;
	2541	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9916.1;
	2541	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6107.7;
	2541	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10445.4;
	2541	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1674.8;
	2542	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.1;
	2542	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	695.4;
	2542	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.5;
	2542	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3823.6;
	2542	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9333.1;
	2542	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5699.6;
	2542	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9856.5;
	2542	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1583.2;
	2543	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.6;
	2543	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	658.5;
	2543	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	790.9;
	2543	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3587;
	2543	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8537.2;
	2543	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5190.8;
	2543	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9173.6;
	2543	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1460.1;
	2544	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.5;
	2544	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	622.2;
	2544	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	753.1;
	2544	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3331;
	2544	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7875.3;
	2544	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4744.6;
	2544	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8576.5;
	2544	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1364.2;
	2545	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	851.9;
	2545	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	600.5;
	2545	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	722.3;
	2545	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3150.8;
	2545	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7414.6;
	2545	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4452.6;
	2545	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8135.7;
	2545	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1298.3;
	2546	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	842.4;
	2546	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588.3;
	2546	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	695.6;
	2546	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3035.5;
	2546	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7143;
	2546	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4242.9;
	2546	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7882.3;
	2546	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1243.7;
	2547	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	835.1;
	2547	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	581.4;
	2547	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	684.7;
	2547	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2974.4;
	2547	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6991.5;
	2547	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4116.7;
	2547	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7768.2;
	2547	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1243.2;
	2548	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	826.6;
	2548	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.5;
	2548	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	680.7;
	2548	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2944;
	2548	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7016.3;
	2548	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4066.5;
	2548	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7763;
	2548	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1203.9;
	2549	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	828.1;
	2549	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.1;
	2549	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	691.4;
	2549	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2980.1;
	2549	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7212.9;
	2549	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4153.1;
	2549	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7884.5;
	2549	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1242.9;
	2550	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	838.7;
	2550	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	615.8;
	2550	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	715.9;
	2550	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3083.2;
	2550	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7609.8;
	2550	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4357.5;
	2550	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8163.4;
	2550	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1284.6;
	2551	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	844.5;
	2551	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	633.5;
	2551	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	739.7;
	2551	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3154.6;
	2551	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7993.1;
	2551	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4612.6;
	2551	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8348.9;
	2551	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1298.4;
	2552	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	856.8;
	2552	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	661.1;
	2552	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	769.9;
	2552	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3297.3;
	2552	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8509.5;
	2552	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4902.3;
	2552	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8820;
	2552	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1424.8;
	2553	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	881.2;
	2553	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	690.2;
	2553	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	804.4;
	2553	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3540.6;
	2553	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9051.5;
	2553	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5189.4;
	2553	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9379.2;
	2553	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1496.6;
	2554	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.7;
	2554	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	707.3;
	2554	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	821.8;
	2554	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3730.8;
	2554	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9391.4;
	2554	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5409.3;
	2554	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9914.5;
	2554	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1561.6;
	2555	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.3;
	2555	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	712.5;
	2555	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	831.9;
	2555	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3887.3;
	2555	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9692.6;
	2555	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5550.3;
	2555	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10282.2;
	2555	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1597.9;
	2556	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934;
	2556	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711.2;
	2556	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	833.9;
	2556	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4012.1;
	2556	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9870.5;
	2556	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5612.3;
	2556	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10449.9;
	2556	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1645;
	2557	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.8;
	2557	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	710.7;
	2557	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	839.2;
	2557	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4153.8;
	2557	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9929.4;
	2557	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5645.8;
	2557	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10587.4;
	2557	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1672.1;
	2558	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.5;
	2558	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	705.7;
	2558	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	844.9;
	2558	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4261.2;
	2558	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9997.2;
	2558	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5700.4;
	2558	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10634.6;
	2558	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1693.7;
	2559	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969.1;
	2559	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711.1;
	2559	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	856.5;
	2559	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4340;
	2559	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10106.1;
	2559	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5763.8;
	2559	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10559.4;
	2559	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1696.4;
	2560	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	976.5;
	2560	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711;
	2560	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	874.2;
	2560	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4379.4;
	2560	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10159.1;
	2560	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5845.1;
	2560	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10398.3;
	2560	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1721.9;
	2561	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	977.4;
	2561	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	708.6;
	2561	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884.9;
	2561	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4362.4;
	2561	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10119;
	2561	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5901.6;
	2561	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10259.9;
	2561	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1737.1;
	2562	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.8;
	2562	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	699.2;
	2562	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882;
	2562	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4237.8;
	2562	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9965.8;
	2562	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5905;
	2562	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10098.1;
	2562	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1720.1;
	2563	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.2;
	2563	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	701.9;
	2563	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.9;
	2563	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4075.2;
	2563	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9940.6;
	2563	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5915.7;
	2563	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10062.2;
	2563	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1701.5;
	2564	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.9;
	2564	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	718.3;
	2564	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	877.1;
	2564	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4091.8;
	2564	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10098.4;
	2564	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6059.3;
	2564	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10108.5;
	2564	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1741;
	2565	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.3;
	2565	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	700.8;
	2565	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	870.2;
	2565	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4005.8;
	2565	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9883.2;
	2565	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5954.4;
	2565	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9963.4;
	2565	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1680.2;
	2566	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	881.9;
	2566	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	668.6;
	2566	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	828.3;
	2566	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3820.8;
	2566	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9344.9;
	2566	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5685.1;
	2566	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9634.9;
	2566	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1609.5;
	2567	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	853.2;
	2567	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	630.3;
	2567	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	779.8;
	2567	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3596.3;
	2567	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8654.2;
	2567	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5278.6;
	2567	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9211.2;
	2567	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1502;
	2568	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	835.1;
	2568	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.3;
	2568	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	724.6;
	2568	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3399;
	2568	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7985.3;
	2568	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4892.9;
	2568	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8788.1;
	2568	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1394.8;
	2569	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	821.8;
	2569	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	576.7;
	2569	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	696.9;
	2569	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3237.1;
	2569	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7536;
	2569	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4610.3;
	2569	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8461.7;
	2569	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1334.6;
	2570	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	811.2;
	2570	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	565;
	2570	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	679.4;
	2570	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3108.2;
	2570	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7231.1;
	2570	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4427.1;
	2570	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8246.3;
	2570	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1279.5;
	2571	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	805.5;
	2571	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.5;
	2571	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	666.4;
	2571	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3031.9;
	2571	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7057.9;
	2571	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4291.4;
	2571	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8120;
	2571	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1188.8;
	2572	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	804.3;
	2572	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	559;
	2572	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	664.8;
	2572	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2999.8;
	2572	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7007;
	2572	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4244.8;
	2572	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8088.2;
	2572	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1157.3;
	2573	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	803.8;
	2573	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	561.5;
	2573	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	670.3;
	2573	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3003.2;
	2573	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7056.5;
	2573	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4269;
	2573	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8098.6;
	2573	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1159.8;
	2574	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	811.2;
	2574	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	574.1;
	2574	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	702.9;
	2574	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3059.1;
	2574	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7257.6;
	2574	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4391.3;
	2574	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8224.4;
	2574	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1191.7;
	2575	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	813.5;
	2575	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.9;
	2575	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	716.7;
	2575	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3071.7;
	2575	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7521.5;
	2575	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4554;
	2575	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8278.4;
	2575	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1227.4;
	2576	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	817;
	2576	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	623.2;
	2576	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	755.3;
	2576	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3214.3;
	2576	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8000.2;
	2576	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4837.6;
	2576	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8594.7;
	2576	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1313.3;
	2577	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	838;
	2577	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	650.3;
	2577	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	779.1;
	2577	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3483.9;
	2577	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8517.1;
	2577	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5212.4;
	2577	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9019.6;
	2577	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1381.9;
	2578	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	846.8;
	2578	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	662.8;
	2578	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	803.1;
	2578	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3722.4;
	2578	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8880.4;
	2578	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5568.6;
	2578	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9455.7;
	2578	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1418.9;
	2579	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	854.1;
	2579	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	661;
	2579	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	817.3;
	2579	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3938.8;
	2579	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9051;
	2579	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5824;
	2579	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9878.3;
	2579	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1451.4;
	2580	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866;
	2580	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	666.1;
	2580	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	833.3;
	2580	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4154.5;
	2580	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9111.2;
	2580	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6090.7;
	2580	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10182.6;
	2580	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1484.1;
	2581	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.5;
	2581	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	664.7;
	2581	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	836.9;
	2581	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4352.5;
	2581	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9051.8;
	2581	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6264.3;
	2581	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10259.5;
	2581	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1488.6;
	2582	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.9;
	2582	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	665.2;
	2582	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	833.8;
	2582	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4513.3;
	2582	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8965;
	2582	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6320.7;
	2582	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10340;
	2582	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1484.6;
	2583	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897;
	2583	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	665.9;
	2583	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	828.9;
	2583	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4612.7;
	2583	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8912.6;
	2583	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6290;
	2583	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10358.1;
	2583	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1479.5;
	2584	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.3;
	2584	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	668.2;
	2584	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	842.7;
	2584	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4658.6;
	2584	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8956.3;
	2584	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6128.8;
	2584	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10333.4;
	2584	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1504.9;
	2585	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.9;
	2585	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	680.3;
	2585	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851.3;
	2585	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4637.6;
	2585	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9091.4;
	2585	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6086;
	2585	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10318.3;
	2585	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1535.4;
	2586	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.2;
	2586	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	680.3;
	2586	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	850.3;
	2586	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4531.4;
	2586	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9226.8;
	2586	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6068.6;
	2586	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10333.9;
	2586	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1542.5;
	2587	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904;
	2587	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	692.5;
	2587	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849.8;
	2587	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4424.3;
	2587	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9477.7;
	2587	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6110.2;
	2587	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10480.7;
	2587	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1580;
	2588	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.8;
	2588	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	709.6;
	2588	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.1;
	2588	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4505;
	2588	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9734.5;
	2588	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6179.2;
	2588	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10545.9;
	2588	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1631.5;
	2589	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901;
	2589	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	694.6;
	2589	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.6;
	2589	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4444.7;
	2589	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9506.5;
	2589	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6014.8;
	2589	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10318.9;
	2589	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1586.2;
	2590	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.4;
	2590	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	653.6;
	2590	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	797.3;
	2590	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4170.4;
	2590	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8797.5;
	2590	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5607.4;
	2590	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9820;
	2590	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1476.8;
	2591	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	846.7;
	2591	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	610.5;
	2591	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	737.9;
	2591	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3824.6;
	2591	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7971.7;
	2591	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5062.1;
	2591	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9234.7;
	2591	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1380.4;
	2592	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	830;
	2592	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.9;
	2592	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	701.4;
	2592	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3515.5;
	2592	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7371.7;
	2592	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4657.2;
	2592	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8646.1;
	2592	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1316.8;
	2593	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	821.4;
	2593	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	582.9;
	2593	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	677.9;
	2593	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3336.4;
	2593	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7027.4;
	2593	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4387.3;
	2593	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8282.3;
	2593	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1256.3;
	2594	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	817;
	2594	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	574;
	2594	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	665.1;
	2594	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3224.2;
	2594	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6854.9;
	2594	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4272.7;
	2594	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8129.1;
	2594	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1226.9;
	2595	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	815;
	2595	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.9;
	2595	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	660.5;
	2595	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3151.2;
	2595	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6827.6;
	2595	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4230;
	2595	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8123;
	2595	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1216.2;
	2596	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	819.4;
	2596	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	580.3;
	2596	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	666.6;
	2596	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3119.3;
	2596	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7020.1;
	2596	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4320.1;
	2596	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8207.9;
	2596	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1233.2;
	2597	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	825.5;
	2597	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	613.6;
	2597	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	695.8;
	2597	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3220.7;
	2597	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7615.6;
	2597	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4667.2;
	2597	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8381.4;
	2597	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1308;
	2598	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.1;
	2598	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	677.1;
	2598	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	773.8;
	2598	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3525.9;
	2598	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8699.8;
	2598	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5219.7;
	2598	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8688.7;
	2598	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1458.8;
	2599	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.1;
	2599	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	707.3;
	2599	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	813.2;
	2599	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3613;
	2599	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9174.6;
	2599	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5472.9;
	2599	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8798.8;
	2599	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1546.5;
	2600	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	865.2;
	2600	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	715.8;
	2600	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	816.6;
	2600	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3609.9;
	2600	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9228.1;
	2600	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5542.5;
	2600	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8903;
	2600	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1568.3;
	2601	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.7;
	2601	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.8;
	2601	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	831.2;
	2601	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3746.5;
	2601	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9462.5;
	2601	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5645;
	2601	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9084.2;
	2601	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1606.1;
	2602	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.6;
	2602	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737;
	2602	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.7;
	2602	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3918.9;
	2602	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9688.8;
	2602	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5799.4;
	2602	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9204.4;
	2602	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1615.8;
	2603	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.8;
	2603	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.1;
	2603	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.7;
	2603	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4119;
	2603	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9840.9;
	2603	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5895.9;
	2603	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9255.1;
	2603	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1632.6;
	2604	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.9;
	2604	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.2;
	2604	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	842.3;
	2604	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4315.9;
	2604	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9890.5;
	2604	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5937.3;
	2604	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9256.4;
	2604	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1632.8;
	2605	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.2;
	2605	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.8;
	2605	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	840;
	2605	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4463.8;
	2605	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9954.7;
	2605	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5993.1;
	2605	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9268.1;
	2605	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1682.5;
	2606	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	876;
	2606	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	739;
	2606	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	834.4;
	2606	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4505.4;
	2606	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9988.2;
	2606	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6053.6;
	2606	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9315.4;
	2606	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1723.9;
	2607	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	881.4;
	2607	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	734.2;
	2607	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	831.9;
	2607	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4550.4;
	2607	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10036.9;
	2607	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6135.9;
	2607	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9397.2;
	2607	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1733;
	2608	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.1;
	2608	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.3;
	2608	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.7;
	2608	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4584.8;
	2608	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10099.6;
	2608	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6146.5;
	2608	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9460.9;
	2608	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1807.6;
	2609	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.4;
	2609	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.6;
	2609	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.5;
	2609	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4485.3;
	2609	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10147.8;
	2609	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6169.6;
	2609	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9507.8;
	2609	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1789;
	2610	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.1;
	2610	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740;
	2610	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	839.5;
	2610	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4281.6;
	2610	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10177.7;
	2610	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6138;
	2610	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9461.5;
	2610	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1796;
	2611	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.6;
	2611	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.3;
	2611	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	841;
	2611	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4120;
	2611	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10199.5;
	2611	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6038.9;
	2611	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9534.6;
	2611	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1792.7;
	2612	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.5;
	2612	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	758.9;
	2612	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868;
	2612	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4145.2;
	2612	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10495.8;
	2612	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6218.3;
	2612	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9788.4;
	2612	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1838.6;
	2613	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.3;
	2613	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	745.2;
	2613	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	855.5;
	2613	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3991.1;
	2613	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10189.9;
	2613	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6024.1;
	2613	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9533.3;
	2613	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1772.3;
	2614	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	861.8;
	2614	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	695;
	2614	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	803.3;
	2614	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3643.4;
	2614	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9352.7;
	2614	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5515.4;
	2614	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8979.5;
	2614	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1643.2;
	2615	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	834;
	2615	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	645.5;
	2615	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	743.8;
	2615	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3243.5;
	2615	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8393.3;
	2615	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4919.8;
	2615	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8384.3;
	2615	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1498.6;
	2616	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	814.6;
	2616	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	609.5;
	2616	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	706.7;
	2616	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2969.8;
	2616	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7714.5;
	2616	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4515.3;
	2616	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7963;
	2616	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1386.1;
	2617	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	807.7;
	2617	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	594;
	2617	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	686.6;
	2617	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2826.5;
	2617	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7336.9;
	2617	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4248.6;
	2617	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7677.5;
	2617	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1311.2;
	2618	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	805.2;
	2618	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	587.4;
	2618	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	684.9;
	2618	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2734.7;
	2618	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7101.5;
	2618	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4090.6;
	2618	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7502.3;
	2618	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1270.7;
	2619	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	804.9;
	2619	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.2;
	2619	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	686.2;
	2619	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2672.4;
	2619	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6997.5;
	2619	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4012.7;
	2619	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7437.1;
	2619	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1274.2;
	2620	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	806.3;
	2620	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.8;
	2620	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	693.8;
	2620	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2667.5;
	2620	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7148.4;
	2620	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4055.3;
	2620	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7516.8;
	2620	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1302.2;
	2621	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	820.3;
	2621	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	624.9;
	2621	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	716.1;
	2621	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2810.3;
	2621	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7746.6;
	2621	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4361.4;
	2621	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7905;
	2621	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1376.2;
	2622	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	853.2;
	2622	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	685.5;
	2622	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	791.9;
	2622	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3180.3;
	2622	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8876.7;
	2622	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5023.8;
	2622	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8571.9;
	2622	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1550.4;
	2623	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.3;
	2623	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	712.3;
	2623	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824.7;
	2623	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3282.6;
	2623	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9347.9;
	2623	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5264.3;
	2623	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8860.9;
	2623	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1621.9;
	2624	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.4;
	2624	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	724;
	2624	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	829.5;
	2624	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3299.7;
	2624	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9408;
	2624	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5312.9;
	2624	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9183.3;
	2624	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1656.4;
	2625	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.7;
	2625	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.2;
	2625	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	840.3;
	2625	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3429.3;
	2625	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9657.1;
	2625	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5450.5;
	2625	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9513.9;
	2625	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1697.7;
	2626	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.4;
	2626	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.1;
	2626	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	850.4;
	2626	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3632.1;
	2626	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9943.6;
	2626	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5594.1;
	2626	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9980.3;
	2626	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1723;
	2627	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.6;
	2627	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746.7;
	2627	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.5;
	2627	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3889.8;
	2627	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10122.8;
	2627	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5731.9;
	2627	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10384.6;
	2627	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1799.8;
	2628	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.1;
	2628	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744.7;
	2628	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.4;
	2628	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4153.4;
	2628	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10223.3;
	2628	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5887.4;
	2628	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10769.9;
	2628	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1857.7;
	2629	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.2;
	2629	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	757.9;
	2629	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.9;
	2629	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4409.1;
	2629	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10461.2;
	2629	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6086;
	2629	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11169;
	2629	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1924.7;
	2630	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.5;
	2630	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	764.8;
	2630	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	864.4;
	2630	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4599.8;
	2630	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10580.5;
	2630	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6275;
	2630	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11495.4;
	2630	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1972.5;
	2631	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.2;
	2631	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	773.3;
	2631	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.2;
	2631	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4738.1;
	2631	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10807.8;
	2631	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6443;
	2631	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11670.2;
	2631	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1966.9;
	2632	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.2;
	2632	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	782.2;
	2632	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878.8;
	2632	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4806.8;
	2632	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11055.7;
	2632	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6562.4;
	2632	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11859.9;
	2632	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2005.7;
	2633	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942;
	2633	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	781.2;
	2633	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	887.6;
	2633	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4689.4;
	2633	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11233.3;
	2633	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6600.3;
	2633	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11884.4;
	2633	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1992.4;
	2634	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938;
	2634	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777.2;
	2634	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878.4;
	2634	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4490;
	2634	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11159.4;
	2634	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6613.8;
	2634	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11426;
	2634	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1969.6;
	2635	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925.1;
	2635	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	774;
	2635	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	876.3;
	2635	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4324.9;
	2635	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10972.7;
	2635	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6555.7;
	2635	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11054;
	2635	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1949.5;
	2636	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.4;
	2636	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777.7;
	2636	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	901.1;
	2636	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4384.9;
	2636	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11239.7;
	2636	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6669.1;
	2636	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11076.1;
	2636	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1970.1;
	2637	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.7;
	2637	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755.7;
	2637	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889.4;
	2637	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4258.7;
	2637	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10832.3;
	2637	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6428;
	2637	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10708.8;
	2637	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1905.5;
	2638	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.7;
	2638	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	704.6;
	2638	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	828.5;
	2638	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3923.3;
	2638	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9838.4;
	2638	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5896.4;
	2638	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10024.6;
	2638	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1768.1;
	2639	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	856.4;
	2639	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	653.4;
	2639	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	761.1;
	2639	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3530.7;
	2639	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8803.7;
	2639	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5245.5;
	2639	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9267.1;
	2639	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1612.8;
	2640	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	827.6;
	2640	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	624.6;
	2640	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	724.5;
	2640	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3214.6;
	2640	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7994.9;
	2640	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4780.2;
	2640	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8643.9;
	2640	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1512;
	2641	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	812;
	2641	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	606.2;
	2641	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	701.6;
	2641	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3028.6;
	2641	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7558.8;
	2641	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4455;
	2641	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8253.5;
	2641	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1419.8;
	2642	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	801.8;
	2642	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	596.5;
	2642	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	686.2;
	2642	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2916.1;
	2642	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7269.7;
	2642	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4311.2;
	2642	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7980.4;
	2642	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1388.1;
	2643	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	797.1;
	2643	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589.2;
	2643	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	683.6;
	2643	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2864;
	2643	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7083.8;
	2643	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4240.8;
	2643	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7879.4;
	2643	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1361.7;
	2644	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	799.6;
	2644	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.3;
	2644	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	687.5;
	2644	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2861.5;
	2644	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7198.2;
	2644	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4254.9;
	2644	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7985.9;
	2644	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1355.4;
	2645	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	812.3;
	2645	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	641.4;
	2645	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	713.1;
	2645	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2989.3;
	2645	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7734.3;
	2645	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4571.5;
	2645	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8467.8;
	2645	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1415.9;
	2646	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	850;
	2646	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	698.2;
	2646	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	788.2;
	2646	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3346.8;
	2646	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8797.8;
	2646	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5253.9;
	2646	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9173.5;
	2646	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1572.6;
	2647	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.1;
	2647	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719.1;
	2647	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	821.8;
	2647	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3429.2;
	2647	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9336.5;
	2647	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5537.9;
	2647	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9375.2;
	2647	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1620;
	2648	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	869.4;
	2648	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.7;
	2648	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	823;
	2648	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3486.6;
	2648	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9400.3;
	2648	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5511.2;
	2648	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9526.4;
	2648	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1618.8;
	2649	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.1;
	2649	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.4;
	2649	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	834.2;
	2649	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3686.4;
	2649	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9603.5;
	2649	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5507.7;
	2649	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9925.4;
	2649	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1661.3;
	2650	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.9;
	2650	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746.5;
	2650	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	845.1;
	2650	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3920.6;
	2650	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9848.1;
	2650	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5678.5;
	2650	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10189.6;
	2650	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1656.5;
	2651	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894;
	2651	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753.4;
	2651	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.8;
	2651	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4130.8;
	2651	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9912.6;
	2651	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5828.2;
	2651	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10154.4;
	2651	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1687.9;
	2652	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.3;
	2652	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	756.5;
	2652	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	881;
	2652	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4340.1;
	2652	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9998;
	2652	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5986.5;
	2652	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10048.6;
	2652	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1719.6;
	2653	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.6;
	2653	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753.8;
	2653	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	903.1;
	2653	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4542.2;
	2653	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10123.5;
	2653	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6278.3;
	2653	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10145.4;
	2653	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1762.1;
	2654	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.8;
	2654	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753.4;
	2654	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	918.1;
	2654	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4724.4;
	2654	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10200.6;
	2654	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6591.6;
	2654	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10457.5;
	2654	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1810.9;
	2655	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.4;
	2655	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755.7;
	2655	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	919.4;
	2655	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4863.3;
	2655	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10225.8;
	2655	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6906.7;
	2655	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10886.9;
	2655	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1839.2;
	2656	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970.4;
	2656	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	765;
	2656	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	939.8;
	2656	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4959.4;
	2656	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10396.8;
	2656	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7150.6;
	2656	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11415.3;
	2656	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1824.7;
	2657	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.7;
	2657	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	766.8;
	2657	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	944.8;
	2657	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4879.8;
	2657	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10560.6;
	2657	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7245.6;
	2657	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11638;
	2657	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1827.7;
	2658	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	968.3;
	2658	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	764.6;
	2658	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	942.4;
	2658	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4689.6;
	2658	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10577.5;
	2658	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7194.8;
	2658	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11418.9;
	2658	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1853.8;
	2659	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.8;
	2659	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	750.3;
	2659	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	914.5;
	2659	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4497.9;
	2659	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10426.4;
	2659	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6988.2;
	2659	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11045.6;
	2659	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1801.4;
	2660	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.6;
	2660	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	765.2;
	2660	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	912;
	2660	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4534.8;
	2660	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10641.8;
	2660	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7011.1;
	2660	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11036.3;
	2660	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1831.5;
	2661	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.9;
	2661	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.9;
	2661	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	896.7;
	2661	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4369.8;
	2661	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10309.4;
	2661	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6682;
	2661	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10711.2;
	2661	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1807.6;
	2662	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.4;
	2662	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	699.3;
	2662	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838.3;
	2662	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4035.2;
	2662	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9453.6;
	2662	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6098.3;
	2662	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10048.8;
	2662	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1668.4;
	2663	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867.9;
	2663	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	649.6;
	2663	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	766.9;
	2663	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3644.3;
	2663	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8450.6;
	2663	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5402.6;
	2663	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9238.5;
	2663	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1504.9;
	2664	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	848;
	2664	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	622.2;
	2664	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	720.8;
	2664	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3365.6;
	2664	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7732.2;
	2664	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4829.5;
	2664	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8611.3;
	2664	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1389.1;
	2665	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	834.2;
	2665	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	608.7;
	2665	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	695.1;
	2665	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3173.5;
	2665	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7291.1;
	2665	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4520.7;
	2665	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8222.3;
	2665	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1320.4;
	2666	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	826.2;
	2666	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	595;
	2666	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	679.4;
	2666	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3051.8;
	2666	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7050.6;
	2666	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4328.6;
	2666	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7956.3;
	2666	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1252.9;
	2667	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	817.3;
	2667	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	595;
	2667	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	666.2;
	2667	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2973.9;
	2667	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6961.5;
	2667	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4240.5;
	2667	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7831.7;
	2667	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1259.4;
	2668	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	812.8;
	2668	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.2;
	2668	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	670.9;
	2668	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2954.6;
	2668	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7097;
	2668	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4284.7;
	2668	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7914.4;
	2668	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1285.2;
	2669	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	827.4;
	2669	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	629.2;
	2669	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	704;
	2669	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3081.8;
	2669	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7694.4;
	2669	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4611.5;
	2669	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8404.9;
	2669	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1354.1;
	2670	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.5;
	2670	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	686.8;
	2670	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	774.3;
	2670	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3422.2;
	2670	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8780.7;
	2670	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5298.1;
	2670	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9069.3;
	2670	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1527.7;
	2671	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.3;
	2671	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	704.1;
	2671	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	808.6;
	2671	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3515;
	2671	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9230.8;
	2671	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5528.9;
	2671	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9311.2;
	2671	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1577;
	2672	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883;
	2672	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	715.3;
	2672	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	817.3;
	2672	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3576.8;
	2672	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9254;
	2672	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5538.7;
	2672	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9589.3;
	2672	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1630.3;
	2673	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.5;
	2673	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	722.1;
	2673	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	831.6;
	2673	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3758.6;
	2673	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9444.6;
	2673	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5578;
	2673	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9986.3;
	2673	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1662;
	2674	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.7;
	2674	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736;
	2674	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	833.1;
	2674	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3967;
	2674	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9678.4;
	2674	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5586.9;
	2674	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10173.8;
	2674	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1682.5;
	2675	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.7;
	2675	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.7;
	2675	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	841.3;
	2675	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4165.1;
	2675	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9834.3;
	2675	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5587.4;
	2675	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10002.3;
	2675	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1699.6;
	2676	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.1;
	2676	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.9;
	2676	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	850.8;
	2676	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4364.8;
	2676	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10008.2;
	2676	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5684.2;
	2676	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9791.7;
	2676	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1726.2;
	2677	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940;
	2677	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	768.2;
	2677	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	875.9;
	2677	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4482.5;
	2677	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10337;
	2677	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5915.1;
	2677	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9892.4;
	2677	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1747;
	2678	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.1;
	2678	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	783;
	2678	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897.9;
	2678	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4698.1;
	2678	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10633.6;
	2678	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6246.9;
	2678	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9955.4;
	2678	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1780.3;
	2679	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.9;
	2679	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	794;
	2679	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	918;
	2679	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4792.6;
	2679	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10921.6;
	2679	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6650.2;
	2679	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9976.3;
	2679	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1854.9;
	2680	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	978.2;
	2680	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	803.6;
	2680	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	939.2;
	2680	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4826.8;
	2680	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11182;
	2680	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6989.2;
	2680	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10091.7;
	2680	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1862.7;
	2681	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.7;
	2681	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	800.8;
	2681	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940.8;
	2681	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4773.2;
	2681	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11270.1;
	2681	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7179.8;
	2681	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10243.6;
	2681	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1905.3;
	2682	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.5;
	2682	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	791.6;
	2682	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	934.9;
	2682	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4611.7;
	2682	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11142.7;
	2682	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7100.5;
	2682	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10214.1;
	2682	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1817.7;
	2683	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.1;
	2683	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772.2;
	2683	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	912.2;
	2683	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4378.7;
	2683	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10867.8;
	2683	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6833.1;
	2683	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10045.9;
	2683	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1747.8;
	2684	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.3;
	2684	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	784.2;
	2684	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	919.5;
	2684	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4378.4;
	2684	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11014.6;
	2684	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6816;
	2684	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10184.3;
	2684	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1814;
	2685	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.2;
	2685	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	769.8;
	2685	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	901.3;
	2685	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4252.5;
	2685	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10621.4;
	2685	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6515.8;
	2685	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9950.7;
	2685	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1785.5;
	2686	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.6;
	2686	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714.4;
	2686	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	840;
	2686	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3922.4;
	2686	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9626.5;
	2686	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5928.7;
	2686	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9326.5;
	2686	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1643.5;
	2687	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	858.9;
	2687	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	665.4;
	2687	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	778.5;
	2687	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3520.4;
	2687	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8563.3;
	2687	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5271.1;
	2687	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8583.7;
	2687	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1487.9;
	2688	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	835.2;
	2688	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	628.8;
	2688	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	720.1;
	2688	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3206;
	2688	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7785.7;
	2688	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4781.9;
	2688	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8029.4;
	2688	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1393.2;
	2689	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	822.9;
	2689	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	606.5;
	2689	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	685.1;
	2689	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2987.1;
	2689	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7323;
	2689	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4448.2;
	2689	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7652.9;
	2689	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1314.4;
	2690	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	817.8;
	2690	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	594.9;
	2690	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	665.7;
	2690	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2857.7;
	2690	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7036.6;
	2690	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4222;
	2690	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7446.4;
	2690	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1292.4;
	2691	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	812.7;
	2691	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.5;
	2691	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	659.7;
	2691	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2789.8;
	2691	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6919.2;
	2691	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4059.1;
	2691	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7348.3;
	2691	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1244.7;
	2692	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	816.1;
	2692	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	593.1;
	2692	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	664.7;
	2692	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2786.5;
	2692	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7015.9;
	2692	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4083.3;
	2692	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7442.8;
	2692	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1254.3;
	2693	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	830.5;
	2693	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	621.8;
	2693	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	695.2;
	2693	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2907.7;
	2693	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7552.1;
	2693	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4336.5;
	2693	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7864.8;
	2693	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1343.5;
	2694	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	858.4;
	2694	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	686.5;
	2694	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	765.5;
	2694	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3246.1;
	2694	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8597.2;
	2694	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4818.6;
	2694	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8492.6;
	2694	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1489.7;
	2695	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.5;
	2695	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	712;
	2695	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	806.8;
	2695	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3342.7;
	2695	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9049.1;
	2695	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5011.5;
	2695	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8700.6;
	2695	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1527.6;
	2696	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.8;
	2696	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.5;
	2696	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	827.7;
	2696	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3407.6;
	2696	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9281.4;
	2696	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5245.4;
	2696	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9004.4;
	2696	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1500.3;
	2697	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.9;
	2697	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742.2;
	2697	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	850;
	2697	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3623.2;
	2697	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9616.3;
	2697	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5539.7;
	2697	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9406.2;
	2697	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1595.2;
	2698	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.4;
	2698	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	758.2;
	2698	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	871.1;
	2698	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3899.4;
	2698	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10000.9;
	2698	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5832.4;
	2698	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9950.6;
	2698	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1698.2;
	2699	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.9;
	2699	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	767.4;
	2699	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	888;
	2699	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4168.2;
	2699	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10287.6;
	2699	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6073.3;
	2699	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10433.5;
	2699	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1769.3;
	2700	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.5;
	2700	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776;
	2700	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	903;
	2700	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4392.8;
	2700	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10579.6;
	2700	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6271.1;
	2700	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10904.6;
	2700	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1796.1;
	2701	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.2;
	2701	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	790.4;
	2701	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	921.3;
	2701	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4613.5;
	2701	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10849.4;
	2701	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6519.5;
	2701	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11313.3;
	2701	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1843.7;
	2702	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970.6;
	2702	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	804;
	2702	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940.3;
	2702	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4799.6;
	2702	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11173.7;
	2702	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6754;
	2702	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11667.4;
	2702	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1877.7;
	2703	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	983.2;
	2703	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	817;
	2703	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	963.5;
	2703	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4908.1;
	2703	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11482.7;
	2703	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7006.9;
	2703	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11967.9;
	2703	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1882;
	2704	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	991.4;
	2704	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	824.8;
	2704	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	980.5;
	2704	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4939.8;
	2704	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11720.1;
	2704	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7207.9;
	2704	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12118.4;
	2704	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1913.7;
	2705	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985.9;
	2705	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	825.3;
	2705	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	975.7;
	2705	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4814.4;
	2705	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11760.9;
	2705	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7261.4;
	2705	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11957.7;
	2705	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1916.7;
	2706	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.4;
	2706	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	808.3;
	2706	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	955.4;
	2706	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4546.1;
	2706	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11472.3;
	2706	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7106.3;
	2706	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11522.2;
	2706	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1868.8;
	2707	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.6;
	2707	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777.2;
	2707	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	914.4;
	2707	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4255.7;
	2707	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10963.7;
	2707	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6718.6;
	2707	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10933;
	2707	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1781.4;
	2708	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.9;
	2708	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772.8;
	2708	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	896.1;
	2708	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4185.2;
	2708	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10764;
	2708	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6560.8;
	2708	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10705.5;
	2708	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1766.1;
	2709	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.4;
	2709	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	756.7;
	2709	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	874;
	2709	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4023.5;
	2709	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10369.9;
	2709	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6236.4;
	2709	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10247.7;
	2709	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1716;
	2710	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.6;
	2710	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	718.8;
	2710	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	823;
	2710	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3767.9;
	2710	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9631;
	2710	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5734.8;
	2710	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9571.7;
	2710	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1613.3;
	2711	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.2;
	2711	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	665.9;
	2711	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	762;
	2711	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3468.7;
	2711	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8719.8;
	2711	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5214.5;
	2711	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8817.5;
	2711	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1478.1;
	2712	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.2;
	2712	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	628.5;
	2712	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	725.7;
	2712	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3199.5;
	2712	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7949.3;
	2712	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4713.1;
	2712	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8204.5;
	2712	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1361.1;
	2713	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	837.2;
	2713	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	602;
	2713	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	691.6;
	2713	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2994.6;
	2713	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7402.3;
	2713	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4362.1;
	2713	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7770.7;
	2713	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1282.4;
	2714	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	826.9;
	2714	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	585.9;
	2714	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	670;
	2714	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2849.5;
	2714	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7075.6;
	2714	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4121.4;
	2714	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7476.7;
	2714	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1226.2;
	2715	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	821.7;
	2715	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579;
	2715	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	660.4;
	2715	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2760.4;
	2715	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6875.6;
	2715	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3977.9;
	2715	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7303.6;
	2715	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1227.2;
	2716	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	818.6;
	2716	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579.2;
	2716	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	660.7;
	2716	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2718.6;
	2716	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6884;
	2716	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3913.5;
	2716	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7264.5;
	2716	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1184.8;
	2717	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	821.3;
	2717	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	590.7;
	2717	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	671.1;
	2717	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2728.6;
	2717	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7054.3;
	2717	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3975.2;
	2717	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7357;
	2717	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1227.7;
	2718	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	828.8;
	2718	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	614.3;
	2718	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	693.3;
	2718	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2797.9;
	2718	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7377.4;
	2718	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4117.1;
	2718	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7546.8;
	2718	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1254.2;
	2719	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	832.6;
	2719	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	620.4;
	2719	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	714;
	2719	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2843.2;
	2719	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7726.9;
	2719	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4286.3;
	2719	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7700.2;
	2719	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1263.4;
	2720	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	846.1;
	2720	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	655.5;
	2720	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	749.2;
	2720	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3087.1;
	2720	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8358.9;
	2720	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4642.9;
	2720	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8208.8;
	2720	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1396;
	2721	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	863.8;
	2721	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	684.7;
	2721	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	781.3;
	2721	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3405.4;
	2721	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8965.7;
	2721	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4999.1;
	2721	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8793.8;
	2721	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1469.9;
	2722	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.8;
	2722	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	709.4;
	2722	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	802.8;
	2722	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3711.8;
	2722	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9515.5;
	2722	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5317.5;
	2722	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9406.6;
	2722	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1555.3;
	2723	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.6;
	2723	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	726;
	2723	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	821.7;
	2723	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3974;
	2723	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9948.9;
	2723	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5574.5;
	2723	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9900.9;
	2723	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1604.5;
	2724	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.9;
	2724	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.7;
	2724	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	837;
	2724	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4211.5;
	2724	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10276.8;
	2724	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5830.2;
	2724	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10318.1;
	2724	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1688.9;
	2725	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.3;
	2725	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	758.5;
	2725	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.4;
	2725	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4433.9;
	2725	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10577.8;
	2725	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6072.6;
	2725	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10762.6;
	2725	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1740.4;
	2726	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.5;
	2726	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	775.1;
	2726	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885;
	2726	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4606.4;
	2726	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10912.8;
	2726	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6356.3;
	2726	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11175.3;
	2726	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1797.3;
	2727	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.5;
	2727	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	797.9;
	2727	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	912.3;
	2727	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4669.7;
	2727	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11210.1;
	2727	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6628.8;
	2727	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11527.4;
	2727	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1826.2;
	2728	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.4;
	2728	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	807.6;
	2728	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940.5;
	2728	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4647.2;
	2728	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11448.9;
	2728	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6841;
	2728	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11664;
	2728	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1866.9;
	2729	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.5;
	2729	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	810.1;
	2729	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	948.5;
	2729	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4487;
	2729	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11484.2;
	2729	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6920;
	2729	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11621.2;
	2729	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1877.5;
	2730	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.8;
	2730	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	796.2;
	2730	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	934.1;
	2730	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4260.7;
	2730	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11219.1;
	2730	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6771.8;
	2730	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11266.7;
	2730	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1849.4;
	2731	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.5;
	2731	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	770.7;
	2731	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897.8;
	2731	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4030.2;
	2731	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10701.6;
	2731	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6430.2;
	2731	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10702.9;
	2731	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1776.2;
	2732	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.5;
	2732	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	773.1;
	2732	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	896.9;
	2732	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4004.4;
	2732	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10608.8;
	2732	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6316.2;
	2732	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10548.8;
	2732	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1780.4;
	2733	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.8;
	2733	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	757.6;
	2733	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	881.2;
	2733	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3875.1;
	2733	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10249.2;
	2733	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6055.5;
	2733	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10145.1;
	2733	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1687.9;
	2734	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.3;
	2734	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.6;
	2734	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	827.8;
	2734	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3661.7;
	2734	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9558.5;
	2734	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5601.2;
	2734	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9607.5;
	2734	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1601.1;
	2735	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.6;
	2735	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	667.8;
	2735	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	768.7;
	2735	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3412;
	2735	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8742.8;
	2735	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5148.1;
	2735	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8980.1;
	2735	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1489.1;
	2736	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	850.2;
	2736	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	627.5;
	2736	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	720.5;
	2736	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3165.7;
	2736	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8024.1;
	2736	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4714.8;
	2736	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8409.7;
	2736	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1368.4;
	2737	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	836.3;
	2737	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.6;
	2737	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	689;
	2737	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2969.6;
	2737	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7473.4;
	2737	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4389.6;
	2737	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7981;
	2737	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1305.2;
	2738	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	829.2;
	2738	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588.1;
	2738	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	668;
	2738	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2832.3;
	2738	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7120.2;
	2738	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4166.1;
	2738	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7688.4;
	2738	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1246.9;
	2739	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	823.4;
	2739	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577.3;
	2739	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	660.4;
	2739	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2760;
	2739	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6905.3;
	2739	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4011.9;
	2739	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7524.6;
	2739	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1155.5;
	2740	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	821.4;
	2740	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	572.4;
	2740	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	658;
	2740	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2705.8;
	2740	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6814.2;
	2740	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3939.7;
	2740	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7447.9;
	2740	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1131.5;
	2741	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	822.3;
	2741	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	574.6;
	2741	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	662.9;
	2741	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2696.9;
	2741	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6869.8;
	2741	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3947.1;
	2741	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7449.6;
	2741	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1126.3;
	2742	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	825;
	2742	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	580.5;
	2742	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	674.4;
	2742	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2722.1;
	2742	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7013.6;
	2742	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4070.9;
	2742	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7514.8;
	2742	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1138.4;
	2743	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	823.5;
	2743	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.1;
	2743	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	688.9;
	2743	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2713.4;
	2743	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7225.5;
	2743	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4165.5;
	2743	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7540.1;
	2743	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1163.4;
	2744	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	833.2;
	2744	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	624.4;
	2744	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	728.4;
	2744	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2902.4;
	2744	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7863.5;
	2744	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4480.4;
	2744	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7978.9;
	2744	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1267.5;
	2745	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	860.4;
	2745	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	658.8;
	2745	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	760.5;
	2745	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3156.5;
	2745	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8499.6;
	2745	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4845.5;
	2745	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8486.7;
	2745	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1353.4;
	2746	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.8;
	2746	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	675.7;
	2746	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	783.3;
	2746	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3367.9;
	2746	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8994.1;
	2746	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5135.6;
	2746	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8955.9;
	2746	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1416.6;
	2747	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.6;
	2747	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	690;
	2747	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	803.2;
	2747	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3570;
	2747	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9423.4;
	2747	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5318.5;
	2747	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9357.7;
	2747	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1479.6;
	2748	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.2;
	2748	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	709;
	2748	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	829.6;
	2748	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3760.9;
	2748	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9762.6;
	2748	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5473.3;
	2748	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9647;
	2748	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1538.3;
	2749	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.4;
	2749	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723.2;
	2749	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.5;
	2749	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3954.9;
	2749	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9983.2;
	2749	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5698.3;
	2749	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9695.8;
	2749	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1577.7;
	2750	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.9;
	2750	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.8;
	2750	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.9;
	2750	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4148.2;
	2750	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10109.9;
	2750	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5916.4;
	2750	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9655.9;
	2750	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1607.6;
	2751	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	977.1;
	2751	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	757;
	2751	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	918.4;
	2751	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4327.3;
	2751	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10263.2;
	2751	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6079.9;
	2751	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9591.1;
	2751	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1633.3;
	2752	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	991.4;
	2752	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772.1;
	2752	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	949.6;
	2752	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4428.3;
	2752	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10440;
	2752	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6273.9;
	2752	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9569.8;
	2752	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1653.9;
	2753	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	998.7;
	2753	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	787;
	2753	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	972.7;
	2753	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4444.2;
	2753	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10693.9;
	2753	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6499.6;
	2753	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9656.1;
	2753	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1673.7;
	2754	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.6;
	2754	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	786.1;
	2754	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	975;
	2754	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4396.2;
	2754	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10742.3;
	2754	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6582.8;
	2754	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9671.9;
	2754	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1654.5;
	2755	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	973.5;
	2755	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	771.6;
	2755	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	950.2;
	2755	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4255.4;
	2755	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10575.6;
	2755	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6494.4;
	2755	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9664.3;
	2755	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1639.2;
	2756	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.9;
	2756	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	781.9;
	2756	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	946.9;
	2756	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4308.9;
	2756	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10785.9;
	2756	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6556;
	2756	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9994.5;
	2756	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1694;
	2757	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951.7;
	2757	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776;
	2757	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	937;
	2757	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4256.5;
	2757	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10676.6;
	2757	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6394.6;
	2757	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9917.6;
	2757	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1660;
	2758	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925.8;
	2758	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	724.7;
	2758	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	875.2;
	2758	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3980.8;
	2758	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9859;
	2758	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5889.2;
	2758	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9442.1;
	2758	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1543.4;
	2759	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.8;
	2759	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	670.3;
	2759	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	800.3;
	2759	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3619.6;
	2759	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8825;
	2759	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5274.6;
	2759	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8883.1;
	2759	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1412.7;
	2760	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.5;
	2760	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	638.9;
	2760	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	743.9;
	2760	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3314.1;
	2760	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8109.7;
	2760	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4831.7;
	2760	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8424.4;
	2760	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1304.1;
	2761	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	848.9;
	2761	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	627.6;
	2761	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	713.2;
	2761	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3127.9;
	2761	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7662.4;
	2761	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4560.9;
	2761	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8107.3;
	2761	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1255.2;
	2762	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	838.6;
	2762	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	605;
	2762	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	693.1;
	2762	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3015.4;
	2762	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7426.1;
	2762	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4393.1;
	2762	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7950;
	2762	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1212.8;
	2763	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	832;
	2763	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	595.7;
	2763	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	682.5;
	2763	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2963.4;
	2763	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7369.3;
	2763	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4320.8;
	2763	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7936.1;
	2763	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1203.4;
	2764	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	832.4;
	2764	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.3;
	2764	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	685.8;
	2764	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2978;
	2764	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7550.1;
	2764	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4382.9;
	2764	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8112.6;
	2764	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1215.7;
	2765	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	841;
	2765	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	633;
	2765	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	716.7;
	2765	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3126;
	2765	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8193.7;
	2765	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4758;
	2765	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8632.8;
	2765	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1306.8;
	2766	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.4;
	2766	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	695.1;
	2766	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	793;
	2766	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3493;
	2766	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9347.6;
	2766	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5441.3;
	2766	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9389.9;
	2766	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1485.9;
	2767	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	881.1;
	2767	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.1;
	2767	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	831.7;
	2767	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3591.6;
	2767	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9848;
	2767	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5699.3;
	2767	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9568.2;
	2767	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1568.3;
	2768	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.7;
	2768	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742.9;
	2768	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.2;
	2768	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3649.7;
	2768	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10056.2;
	2768	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5818.8;
	2768	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9855;
	2768	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1614.7;
	2769	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.1;
	2769	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	767.9;
	2769	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	879.3;
	2769	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3859.9;
	2769	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10478.1;
	2769	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6032.2;
	2769	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10297.7;
	2769	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1679.2;
	2770	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.9;
	2770	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	801.7;
	2770	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	912.9;
	2770	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4159.4;
	2770	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11002.8;
	2770	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6328;
	2770	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10873.1;
	2770	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1751.3;
	2771	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.2;
	2771	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	830.9;
	2771	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	953.9;
	2771	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4497.9;
	2771	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11537.1;
	2771	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6724;
	2771	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11440.8;
	2771	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1824.6;
	2772	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986.5;
	2772	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	875.2;
	2772	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	992.5;
	2772	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4821.3;
	2772	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12102.9;
	2772	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7179.5;
	2772	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11947.7;
	2772	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1891.7;
	2773	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1008;
	2773	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	909.2;
	2773	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1040.9;
	2773	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5117.5;
	2773	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12702.3;
	2773	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7656.1;
	2773	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12565.4;
	2773	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2020.7;
	2774	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1032.7;
	2774	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	932.6;
	2774	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1092.1;
	2774	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5323;
	2774	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13253.3;
	2774	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8101.2;
	2774	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13049.5;
	2774	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2134.3;
	2775	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1052.8;
	2775	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	954.6;
	2775	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1134;
	2775	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5449;
	2775	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13685.3;
	2775	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8492.6;
	2775	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13403.7;
	2775	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2177.7;
	2776	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1065.3;
	2776	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	972.1;
	2776	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1165.9;
	2776	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5500.1;
	2776	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14053.9;
	2776	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8791.5;
	2776	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13655.3;
	2776	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2282.6;
	2777	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1064;
	2777	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	971.2;
	2777	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1171.5;
	2777	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5377.2;
	2777	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14163.5;
	2777	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8942.3;
	2777	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13580;
	2777	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2264.3;
	2778	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1053.2;
	2778	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	962.3;
	2778	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1156.3;
	2778	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5136.5;
	2778	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13930.9;
	2778	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8760.3;
	2778	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13069.3;
	2778	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2250.5;
	2779	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1034;
	2779	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	936.9;
	2779	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1117.7;
	2779	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4906;
	2779	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13548.5;
	2779	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8300;
	2779	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12570.3;
	2779	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2177.8;
	2780	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1012.3;
	2780	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	928.5;
	2780	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1096.3;
	2780	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4891.3;
	2780	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13545.8;
	2780	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8152.2;
	2780	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12507.1;
	2780	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2198.5;
	2781	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	998.8;
	2781	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	908.8;
	2781	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1057.5;
	2781	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4739.9;
	2781	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13148;
	2781	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7694.5;
	2781	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12015.9;
	2781	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2126.4;
	2782	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.3;
	2782	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	855.4;
	2782	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	971.3;
	2782	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4373.1;
	2782	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12046.5;
	2782	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6951;
	2782	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11192.4;
	2782	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1962.7;
	2783	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.5;
	2783	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	782.5;
	2783	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	879.9;
	2783	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3948.1;
	2783	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10811.6;
	2783	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6178.4;
	2783	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10338.7;
	2783	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1781.2;
	2784	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.3;
	2784	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	732.8;
	2784	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	822.9;
	2784	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3596.4;
	2784	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9831.2;
	2784	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5585.6;
	2784	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9672;
	2784	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1642.2;
	2785	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866.9;
	2785	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	710.2;
	2785	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	791.5;
	2785	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3361;
	2785	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9298;
	2785	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5200.5;
	2785	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9249.2;
	2785	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1543.3;
	2786	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	859.8;
	2786	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	694.7;
	2786	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	766.7;
	2786	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3231.7;
	2786	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8954.2;
	2786	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4934.7;
	2786	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8994.3;
	2786	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1492.8;
	2787	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	851.5;
	2787	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	692.1;
	2787	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	750.5;
	2787	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3163.7;
	2787	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8714.5;
	2787	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4793.9;
	2787	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8847.4;
	2787	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1485.5;
	2788	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	849.3;
	2788	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	686.9;
	2788	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	750.6;
	2788	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3165.8;
	2788	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8780.7;
	2788	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4788.5;
	2788	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8926.5;
	2788	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1496.6;
	2789	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.7;
	2789	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	694.4;
	2789	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	774.3;
	2789	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3288.1;
	2789	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9295.2;
	2789	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5073.5;
	2789	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9363.8;
	2789	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1566.7;
	2790	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.6;
	2790	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749.8;
	2790	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.3;
	2790	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3648.3;
	2790	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10335;
	2790	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5716.4;
	2790	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10108;
	2790	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1745.2;
	2791	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.3;
	2791	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	773;
	2791	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	875.3;
	2791	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3718.7;
	2791	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10675;
	2791	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5904;
	2791	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10301.5;
	2791	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1807.1;
	2792	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.5;
	2792	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	795.7;
	2792	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	888.7;
	2792	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3765.7;
	2792	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10811.1;
	2792	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5976.4;
	2792	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10470;
	2792	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1832.3;
	2793	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.2;
	2793	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	822.2;
	2793	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	908.6;
	2793	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3925.2;
	2793	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11190;
	2793	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6176.4;
	2793	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10815.9;
	2793	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1891.5;
	2794	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951.5;
	2794	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	849.6;
	2794	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	942.3;
	2794	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4103.1;
	2794	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11700.7;
	2794	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6450.7;
	2794	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11253.5;
	2794	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1932.4;
	2795	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	977;
	2795	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	870.7;
	2795	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	982.2;
	2795	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4310.2;
	2795	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12179;
	2795	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6765.7;
	2795	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11678.9;
	2795	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2023.8;
	2796	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	991.3;
	2796	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	887.8;
	2796	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1015.8;
	2796	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4498.6;
	2796	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12654.9;
	2796	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7088.9;
	2796	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12048.7;
	2796	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2096.5;
	2797	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1010.5;
	2797	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	917.1;
	2797	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1049.6;
	2797	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4718.6;
	2797	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13146;
	2797	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7368.5;
	2797	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12533.3;
	2797	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2166.8;
	2798	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1032.1;
	2798	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	938.5;
	2798	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1085.3;
	2798	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4942.9;
	2798	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13582.8;
	2798	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7619.3;
	2798	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13039.1;
	2798	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2236.7;
	2799	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1032.2;
	2799	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	953.5;
	2799	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1102.6;
	2799	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5108.6;
	2799	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14012.7;
	2799	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7797.7;
	2799	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13367.1;
	2799	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2260.5;
	2800	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1029.8;
	2800	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	964.8;
	2800	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1117.1;
	2800	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5210.3;
	2800	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14287.5;
	2800	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7934.6;
	2800	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13443.7;
	2800	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2325.2;
	2801	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009.7;
	2801	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	955.1;
	2801	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1100;
	2801	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5151.1;
	2801	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14151.7;
	2801	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8009.5;
	2801	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13184.9;
	2801	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2312.9;
	2802	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	991.1;
	2802	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	925;
	2802	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1067;
	2802	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4983;
	2802	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13678.2;
	2802	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7902.5;
	2802	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12628.2;
	2802	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2254.9;
	2803	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969.8;
	2803	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	895;
	2803	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1021.2;
	2803	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4797.1;
	2803	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13344.4;
	2803	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7734.7;
	2803	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12250.9;
	2803	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2206;
	2804	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.1;
	2804	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	859;
	2804	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1017.4;
	2804	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4819;
	2804	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13327.7;
	2804	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7842.5;
	2804	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12255.2;
	2804	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2206.2;
	2805	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.1;
	2805	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	808.4;
	2805	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	985;
	2805	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4696.7;
	2805	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12578.3;
	2805	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7575.9;
	2805	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11941.4;
	2805	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2128.2;
	2806	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.3;
	2806	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	732.5;
	2806	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	900.3;
	2806	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4350.2;
	2806	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10819.5;
	2806	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6943.7;
	2806	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11128;
	2806	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1958.3;
	2807	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871.3;
	2807	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	684.5;
	2807	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	821.1;
	2807	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3925.1;
	2807	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9357.3;
	2807	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6237.1;
	2807	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10326.3;
	2807	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1786.8;
	2808	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	847.2;
	2808	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	638.9;
	2808	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	767.5;
	2808	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3640.9;
	2808	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8376.6;
	2808	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5729.4;
	2808	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9800.5;
	2808	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1654.7;
	2809	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	828.6;
	2809	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	614.5;
	2809	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	731.4;
	2809	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3433.8;
	2809	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7858;
	2809	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5391.6;
	2809	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9421.2;
	2809	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1518.9;
	2810	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	818.4;
	2810	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	608.1;
	2810	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	699.6;
	2810	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3282.2;
	2810	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7544.8;
	2810	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5025.4;
	2810	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9170.5;
	2810	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1456.8;
	2811	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	811.9;
	2811	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	604;
	2811	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	686.3;
	2811	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3193.1;
	2811	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7379.5;
	2811	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4455.6;
	2811	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9114.3;
	2811	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1400.5;
	2812	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	815;
	2812	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	604.9;
	2812	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	683.9;
	2812	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3174.3;
	2812	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7486.9;
	2812	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4295.3;
	2812	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9196.1;
	2812	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1350;
	2813	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	816.5;
	2813	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	615.5;
	2813	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	703.6;
	2813	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3279.2;
	2813	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7956.5;
	2813	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4517.3;
	2813	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9157.8;
	2813	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1382.4;
	2814	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	847.5;
	2814	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	668.1;
	2814	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	768.4;
	2814	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3613.6;
	2814	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9007.8;
	2814	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5104.8;
	2814	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9383.8;
	2814	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1538.3;
	2815	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	856.8;
	2815	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	683.7;
	2815	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	796;
	2815	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3634.4;
	2815	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9404.7;
	2815	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5277.2;
	2815	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9300.4;
	2815	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1596.8;
	2816	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866.4;
	2816	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	705.4;
	2816	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	807.2;
	2816	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3642.6;
	2816	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9572;
	2816	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5387.2;
	2816	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9470.2;
	2816	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1631.1;
	2817	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.2;
	2817	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.4;
	2817	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	823.3;
	2817	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3832.9;
	2817	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10004.9;
	2817	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5684.4;
	2817	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9886.6;
	2817	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1733.1;
	2818	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.2;
	2818	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746.1;
	2818	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	845.8;
	2818	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4097.6;
	2818	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10537.3;
	2818	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6055.6;
	2818	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10485;
	2818	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1800.7;
	2819	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.5;
	2819	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	766.9;
	2819	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	872.1;
	2819	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4407.5;
	2819	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10957.9;
	2819	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6428.1;
	2819	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11081.5;
	2819	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1909.9;
	2820	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.7;
	2820	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	783.6;
	2820	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	899.8;
	2820	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4745.8;
	2820	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11368.1;
	2820	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6812.5;
	2820	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11681.8;
	2820	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1988.4;
	2821	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.1;
	2821	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	807.3;
	2821	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	927.5;
	2821	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5037.6;
	2821	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11795;
	2821	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7169.6;
	2821	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12399.5;
	2821	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2079.6;
	2822	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	974;
	2822	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	826.6;
	2822	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	962.6;
	2822	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5274;
	2822	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12149.7;
	2822	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7494.4;
	2822	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13000.7;
	2822	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2181.1;
	2823	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	991.9;
	2823	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	845.7;
	2823	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1004.6;
	2823	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5458.8;
	2823	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12478.3;
	2823	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7844.6;
	2823	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13577;
	2823	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2223.5;
	2824	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1003.1;
	2824	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	864.9;
	2824	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1042.1;
	2824	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5559.9;
	2824	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12824.8;
	2824	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8148.9;
	2824	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13955.1;
	2824	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2199;
	2825	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1010.2;
	2825	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	867.4;
	2825	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1053.7;
	2825	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5490.9;
	2825	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12955.5;
	2825	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8296.7;
	2825	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13945.1;
	2825	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2192.1;
	2826	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1002.6;
	2826	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	857.7;
	2826	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1037.2;
	2826	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5296.1;
	2826	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12727.1;
	2826	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8184.1;
	2826	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13516.7;
	2826	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2173.2;
	2827	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	983.6;
	2827	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	831.3;
	2827	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1003.3;
	2827	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5012.2;
	2827	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12178.8;
	2827	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7756.6;
	2827	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12918.3;
	2827	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2059.8;
	2828	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969.6;
	2828	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	821.7;
	2828	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	977.5;
	2828	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4906.6;
	2828	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11930.2;
	2828	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7437.8;
	2828	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12679;
	2828	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1982.5;
	2829	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.2;
	2829	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	799;
	2829	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	945.1;
	2829	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4711.6;
	2829	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11358.1;
	2829	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6966.6;
	2829	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12164.6;
	2829	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1931.9;
	2830	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.2;
	2830	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742.4;
	2830	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	864.3;
	2830	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4304.4;
	2830	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10217.1;
	2830	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6183.4;
	2830	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11217.6;
	2830	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1760.6;
	2831	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871.3;
	2831	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	674;
	2831	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	782.1;
	2831	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3865.2;
	2831	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9027.5;
	2831	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5392.7;
	2831	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10325.9;
	2831	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1567.4;
	2832	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	851.5;
	2832	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	635.6;
	2832	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	720.5;
	2832	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3526.4;
	2832	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8182;
	2832	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4880.5;
	2832	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9542.4;
	2832	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1432.1;
	2833	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	840.1;
	2833	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	618.7;
	2833	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	692.7;
	2833	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3316.7;
	2833	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7640.7;
	2833	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4540.3;
	2833	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9065.2;
	2833	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1350.2;
	2834	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	832;
	2834	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	609.9;
	2834	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	674.9;
	2834	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3189.6;
	2834	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7320.1;
	2834	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4334.7;
	2834	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8727.3;
	2834	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1265;
	2835	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	827.1;
	2835	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	608;
	2835	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	663.3;
	2835	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3126.5;
	2835	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7164.3;
	2835	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4240.3;
	2835	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8511.6;
	2835	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1267.5;
	2836	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	827.1;
	2836	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	606.4;
	2836	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	665.4;
	2836	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3121.8;
	2836	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7263;
	2836	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4297.9;
	2836	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8521.8;
	2836	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1281;
	2837	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	838.5;
	2837	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	623.3;
	2837	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	695.3;
	2837	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3255.2;
	2837	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7840.6;
	2837	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4658.1;
	2837	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8995.6;
	2837	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1355.1;
	2838	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	863.4;
	2838	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	680.2;
	2838	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	766.9;
	2838	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3620.5;
	2838	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8895.4;
	2838	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5348.7;
	2838	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9742.2;
	2838	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1526.5;
	2839	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.7;
	2839	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	694.5;
	2839	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	795.8;
	2839	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3726.8;
	2839	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9307.9;
	2839	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5548.8;
	2839	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9981.1;
	2839	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1572.1;
	2840	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896;
	2840	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.3;
	2840	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	810.4;
	2840	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3771.3;
	2840	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9533.7;
	2840	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5684;
	2840	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10411.2;
	2840	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1655.1;
	2841	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.7;
	2841	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.7;
	2841	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	833.9;
	2841	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3951.6;
	2841	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9961.7;
	2841	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5889.3;
	2841	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11085.6;
	2841	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1755.3;
	2842	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.7;
	2842	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	761.3;
	2842	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.7;
	2842	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4197.1;
	2842	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10486.4;
	2842	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6231.1;
	2842	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11851.4;
	2842	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1838.8;
	2843	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.9;
	2843	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	784.7;
	2843	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884.1;
	2843	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4428;
	2843	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10932.6;
	2843	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6548;
	2843	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12523.1;
	2843	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1944.9;
	2844	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981.4;
	2844	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	803.2;
	2844	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	908.9;
	2844	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4636.2;
	2844	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11428.4;
	2844	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6841.3;
	2844	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13079.3;
	2844	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2063.8;
	2845	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1003.8;
	2845	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	830.6;
	2845	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	944.4;
	2845	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4827;
	2845	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12041.4;
	2845	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7153;
	2845	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13540.7;
	2845	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2148.1;
	2846	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1031.9;
	2846	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	859.1;
	2846	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	981.8;
	2846	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5025.1;
	2846	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12603.9;
	2846	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7303;
	2846	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13691.3;
	2846	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2226;
	2847	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1050.2;
	2847	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	882.1;
	2847	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1012.8;
	2847	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5174.2;
	2847	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13125.6;
	2847	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7504.2;
	2847	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13518.8;
	2847	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2315.3;
	2848	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1066.3;
	2848	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	899.7;
	2848	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1041.2;
	2848	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5279.1;
	2848	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13521.3;
	2848	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7657.8;
	2848	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13467.5;
	2848	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2321.7;
	2849	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1060.6;
	2849	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	896.1;
	2849	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1040.9;
	2849	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5209.3;
	2849	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13629.2;
	2849	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7712.2;
	2849	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13259.5;
	2849	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2336.8;
	2850	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1049.6;
	2850	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	879;
	2850	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1030.7;
	2850	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5029.6;
	2850	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13370.6;
	2850	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7622;
	2850	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12870.9;
	2850	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2216;
	2851	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030.4;
	2851	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	849.2;
	2851	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1006.4;
	2851	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4853.7;
	2851	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12889.3;
	2851	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7449.3;
	2851	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12681.6;
	2851	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2148.5;
	2852	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009.9;
	2852	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	846.2;
	2852	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	995.1;
	2852	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4865.6;
	2852	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12778.3;
	2852	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7548.6;
	2852	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12692.1;
	2852	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2214.3;
	2853	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.6;
	2853	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	827.6;
	2853	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	978.1;
	2853	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4763;
	2853	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12301.7;
	2853	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7322.9;
	2853	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12395.8;
	2853	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2179.2;
	2854	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	968.4;
	2854	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	779.2;
	2854	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	918.7;
	2854	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4467.6;
	2854	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11294.2;
	2854	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6720.9;
	2854	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11733.6;
	2854	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2028.2;
	2855	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.5;
	2855	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	713.4;
	2855	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.5;
	2855	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4080.3;
	2855	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10170;
	2855	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5991.9;
	2855	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10972.7;
	2855	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1859;
	2856	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.8;
	2856	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	659.1;
	2856	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	819.5;
	2856	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3698.5;
	2856	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9184.3;
	2856	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5649.9;
	2856	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10313.1;
	2856	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1720.7;
	2857	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	876;
	2857	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	632;
	2857	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	790.1;
	2857	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3504;
	2857	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8594;
	2857	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5298;
	2857	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9885.8;
	2857	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1638.3;
	2858	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866.9;
	2858	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	620.2;
	2858	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	771.1;
	2858	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3381.2;
	2858	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8211.2;
	2858	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5069.5;
	2858	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9639.7;
	2858	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1603.1;
	2859	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.9;
	2859	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	621.4;
	2859	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	766.9;
	2859	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3331;
	2859	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8046.8;
	2859	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4952.3;
	2859	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9544.4;
	2859	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1570.5;
	2860	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867.6;
	2860	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	628.2;
	2860	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	769;
	2860	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3336.9;
	2860	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8213.5;
	2860	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4985.4;
	2860	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9601.8;
	2860	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1589.6;
	2861	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	876.7;
	2861	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	654.4;
	2861	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	794.7;
	2861	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3486.9;
	2861	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8842.3;
	2861	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5311;
	2861	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9990.8;
	2861	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1671.9;
	2862	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.9;
	2862	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	708.3;
	2862	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.6;
	2862	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3865.6;
	2862	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10013.6;
	2862	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6013.8;
	2862	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10632.4;
	2862	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1800.2;
	2863	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.2;
	2863	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.1;
	2863	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	894.5;
	2863	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3967;
	2863	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10581;
	2863	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6319.6;
	2863	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10777.1;
	2863	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1862.6;
	2864	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.8;
	2864	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	759.6;
	2864	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	914.1;
	2864	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4009.4;
	2864	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10861.2;
	2864	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6453;
	2864	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11099.2;
	2864	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1821.6;
	2865	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.9;
	2865	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	779;
	2865	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	936.8;
	2865	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4147.4;
	2865	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11222.9;
	2865	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6674.6;
	2865	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11674;
	2865	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1923.3;
	2866	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.9;
	2866	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	798.7;
	2866	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	963.8;
	2866	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4301.8;
	2866	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11530.2;
	2866	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6926.7;
	2866	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12382.7;
	2866	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2025.8;
	2867	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	977.6;
	2867	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	821.6;
	2867	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	991.2;
	2867	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4470.1;
	2867	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11929.7;
	2867	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7159.5;
	2867	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13056.6;
	2867	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2118.6;
	2868	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986.1;
	2868	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	842.6;
	2868	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1013.8;
	2868	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4581.3;
	2868	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12257.8;
	2868	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7321.5;
	2868	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13633.9;
	2868	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2163.9;
	2869	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1003.2;
	2869	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	867.9;
	2869	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1053.2;
	2869	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4708.9;
	2869	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12722.5;
	2869	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7445.8;
	2869	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14092;
	2869	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2185.5;
	2870	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1015.8;
	2870	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	887.4;
	2870	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1086.1;
	2870	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4825.1;
	2870	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12791.7;
	2870	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7616.1;
	2870	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14111.6;
	2870	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2166;
	2871	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1020.9;
	2871	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	889.4;
	2871	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1112.1;
	2871	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4901.1;
	2871	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12735.9;
	2871	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7853.4;
	2871	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13910.6;
	2871	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2078.1;
	2872	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1021.8;
	2872	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	868.7;
	2872	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1132.5;
	2872	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4947.2;
	2872	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12543.1;
	2872	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7934.7;
	2872	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13554.5;
	2872	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2060.8;
	2873	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009;
	2873	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	848.2;
	2873	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1125.8;
	2873	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4843.2;
	2873	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12102.1;
	2873	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7948.6;
	2873	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13173.1;
	2873	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2032.6;
	2874	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	987.9;
	2874	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	830.8;
	2874	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1103.5;
	2874	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4663.6;
	2874	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11742.9;
	2874	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7839.4;
	2874	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12695.3;
	2874	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1976.3;
	2875	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.5;
	2875	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	804.3;
	2875	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1057.4;
	2875	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4539.8;
	2875	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11466.4;
	2875	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7650.3;
	2875	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12379;
	2875	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1948.5;
	2876	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.7;
	2876	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	798.1;
	2876	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1026.8;
	2876	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4587.8;
	2876	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11490.4;
	2876	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7622.6;
	2876	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12502.1;
	2876	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1946.7;
	2877	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.8;
	2877	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	775.3;
	2877	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	990.2;
	2877	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4500.1;
	2877	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11176.2;
	2877	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7195.2;
	2877	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12211.9;
	2877	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1928.7;
	2878	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.5;
	2878	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.9;
	2878	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	920.7;
	2878	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4283.9;
	2878	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10408;
	2878	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6669.5;
	2878	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11624.8;
	2878	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1880.6;
	2879	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.6;
	2879	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	672.6;
	2879	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	844.5;
	2879	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3997.4;
	2879	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9484;
	2879	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6084.6;
	2879	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10975.5;
	2879	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1729.4;
	2880	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	865.5;
	2880	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	635.5;
	2880	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	786.9;
	2880	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3720.6;
	2880	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8663.8;
	2880	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5565.8;
	2880	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10523;
	2880	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1587.8;
	2881	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	847.9;
	2881	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	611.5;
	2881	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	746.4;
	2881	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3499.1;
	2881	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8060.9;
	2881	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5189.6;
	2881	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10078.9;
	2881	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1501.7;
	2882	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	837.1;
	2882	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.1;
	2882	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	720.6;
	2882	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3347.8;
	2882	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7601.8;
	2882	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4957.6;
	2882	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9769.2;
	2882	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1438.2;
	2883	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	827.2;
	2883	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589.6;
	2883	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	701;
	2883	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3245;
	2883	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7357.7;
	2883	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4793.1;
	2883	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9562.2;
	2883	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1414.8;
	2884	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	819.2;
	2884	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	590.2;
	2884	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	693;
	2884	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3194.6;
	2884	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7242.9;
	2884	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4484.8;
	2884	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9475;
	2884	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1442.1;
	2885	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	818;
	2885	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	601;
	2885	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	695.2;
	2885	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3204.8;
	2885	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7384.5;
	2885	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4338.9;
	2885	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9509.1;
	2885	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1417.5;
	2886	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	827.6;
	2886	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	615.8;
	2886	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	711.4;
	2886	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3276.2;
	2886	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7688.4;
	2886	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4418.9;
	2886	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9650.6;
	2886	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1385.8;
	2887	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	830;
	2887	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	621.5;
	2887	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	728.6;
	2887	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3317.5;
	2887	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8028.1;
	2887	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4523.9;
	2887	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9707.9;
	2887	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1500.3;
	2888	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	842.9;
	2888	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	650.9;
	2888	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	766.9;
	2888	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3498.8;
	2888	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8493.3;
	2888	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4939.4;
	2888	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9944.7;
	2888	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1547.6;
	2889	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	863;
	2889	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	674.8;
	2889	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	794.4;
	2889	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3724.3;
	2889	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8976.1;
	2889	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5394.5;
	2889	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10247.2;
	2889	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1572.9;
	2890	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.9;
	2890	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	691.1;
	2890	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	820.3;
	2890	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4014.6;
	2890	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9243.8;
	2890	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5804.1;
	2890	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10751;
	2890	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1664.6;
	2891	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.6;
	2891	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	705.7;
	2891	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.1;
	2891	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4322.5;
	2891	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9338.7;
	2891	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6146.3;
	2891	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11350.1;
	2891	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1810.6;
	2892	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.5;
	2892	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	715.2;
	2892	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885;
	2892	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4636.7;
	2892	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9443.6;
	2892	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6413.6;
	2892	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11917.2;
	2892	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1903.1;
	2893	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.7;
	2893	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714.8;
	2893	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	917.5;
	2893	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4920.9;
	2893	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9628.7;
	2893	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6708.6;
	2893	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12519.5;
	2893	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1983.9;
	2894	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.2;
	2894	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	724.4;
	2894	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	951.3;
	2894	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5120;
	2894	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9951.7;
	2894	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7049.5;
	2894	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13159.4;
	2894	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2074;
	2895	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.1;
	2895	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	739.3;
	2895	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	978.6;
	2895	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5254.4;
	2895	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10359;
	2895	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7412;
	2895	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13631.3;
	2895	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2142.5;
	2896	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	968.1;
	2896	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749.2;
	2896	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1000.5;
	2896	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5302.4;
	2896	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10773.3;
	2896	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7684.3;
	2896	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13755.2;
	2896	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2158.9;
	2897	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	974.6;
	2897	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	756;
	2897	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1006.2;
	2897	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5242.8;
	2897	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11052.4;
	2897	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7815.1;
	2897	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13666.6;
	2897	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2164.9;
	2898	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.8;
	2898	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	757.9;
	2898	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	964.6;
	2898	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5043;
	2898	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11008.7;
	2898	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7712.2;
	2898	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13337.3;
	2898	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2089.7;
	2899	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.7;
	2899	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	745.7;
	2899	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	928.6;
	2899	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4778.3;
	2899	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10646.9;
	2899	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7370.7;
	2899	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12928.9;
	2899	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1995.6;
	2900	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.3;
	2900	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.5;
	2900	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	949.9;
	2900	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4679.2;
	2900	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10531.4;
	2900	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7096.9;
	2900	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12793.8;
	2900	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1973.1;
	2901	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.4;
	2901	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735.4;
	2901	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	922.5;
	2901	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4494.2;
	2901	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10264.5;
	2901	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6757;
	2901	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12415;
	2901	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1878.3;
	2902	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.1;
	2902	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	697.3;
	2902	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.3;
	2902	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4226.8;
	2902	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9587.3;
	2902	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6247.4;
	2902	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11821.6;
	2902	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1774.3;
	2903	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.3;
	2903	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	657.1;
	2903	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	805.6;
	2903	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3939.7;
	2903	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8787.1;
	2903	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5668;
	2903	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11126.7;
	2903	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1652.7;
	2904	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	850.9;
	2904	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	623.4;
	2904	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	747;
	2904	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3664.5;
	2904	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8044;
	2904	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5147.7;
	2904	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10446.6;
	2904	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1461.5;
	2905	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	834.2;
	2905	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.8;
	2905	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	711.8;
	2905	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3440.3;
	2905	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7474.5;
	2905	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4773.3;
	2905	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9940.4;
	2905	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1361.7;
	2906	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	824.4;
	2906	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	582.8;
	2906	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	688.9;
	2906	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3281.8;
	2906	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7098.6;
	2906	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4523.7;
	2906	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9585;
	2906	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1343.3;
	2907	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	817.6;
	2907	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	576.1;
	2907	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	675.5;
	2907	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3204.4;
	2907	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6893.9;
	2907	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4352.4;
	2907	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9374.7;
	2907	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1352.7;
	2908	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	814.6;
	2908	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	574.1;
	2908	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	669.4;
	2908	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3151.4;
	2908	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6792.4;
	2908	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4244.1;
	2908	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9244;
	2908	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1320.6;
	2909	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	813.4;
	2909	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579.4;
	2909	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	670.5;
	2909	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3139.9;
	2909	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6822.3;
	2909	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4230.1;
	2909	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9154.4;
	2909	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1310.1;
	2910	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	820.4;
	2910	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	595.5;
	2910	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	683.4;
	2910	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3154.3;
	2910	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6942.6;
	2910	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4310.3;
	2910	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9163.7;
	2910	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1312.4;
	2911	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	820.1;
	2911	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	599.4;
	2911	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	697.8;
	2911	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3135.2;
	2911	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7149.2;
	2911	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4421.5;
	2911	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9145.6;
	2911	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1340.7;
	2912	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	835.6;
	2912	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	635.6;
	2912	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	739.3;
	2912	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3377.7;
	2912	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7791.2;
	2912	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4775.1;
	2912	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9560.3;
	2912	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1466.3;
	2913	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	854.2;
	2913	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	658.3;
	2913	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	772.8;
	2913	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3733.5;
	2913	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8350.5;
	2913	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5113.4;
	2913	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10141.5;
	2913	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1561.7;
	2914	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867.2;
	2914	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	659.5;
	2914	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	785.5;
	2914	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4083.4;
	2914	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8722.2;
	2914	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5423.7;
	2914	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10790.4;
	2914	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1637.2;
	2915	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.1;
	2915	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	663.6;
	2915	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	796.9;
	2915	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4364.3;
	2915	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8974.9;
	2915	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5631.7;
	2915	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11380.6;
	2915	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1704.7;
	2916	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.4;
	2916	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	671.5;
	2916	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	808.5;
	2916	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4607.2;
	2916	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9183.1;
	2916	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5848.8;
	2916	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11829.2;
	2916	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1813.2;
	2917	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	876.4;
	2917	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	677.1;
	2917	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	813;
	2917	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4781.4;
	2917	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9368.2;
	2917	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6047.8;
	2917	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12123.9;
	2917	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1900;
	2918	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	876.2;
	2918	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	681.2;
	2918	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	818.7;
	2918	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4951.4;
	2918	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9588.3;
	2918	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6240.3;
	2918	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12510.4;
	2918	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1959.7;
	2919	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.8;
	2919	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	694.3;
	2919	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	825.3;
	2919	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5086;
	2919	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9853.6;
	2919	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6389.2;
	2919	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12777;
	2919	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2003.8;
	2920	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.4;
	2920	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	708.8;
	2920	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838;
	2920	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5154.2;
	2920	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10134.9;
	2920	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6516.7;
	2920	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12917.3;
	2920	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1971.4;
	2921	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.9;
	2921	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	718.1;
	2921	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	841.9;
	2921	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5078.3;
	2921	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10312.4;
	2921	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6605.7;
	2921	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12739;
	2921	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2016.5;
	2922	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866.1;
	2922	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719.1;
	2922	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	837.9;
	2922	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4912.7;
	2922	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10284.3;
	2922	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6574.8;
	2922	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12261.4;
	2922	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1999;
	2923	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.5;
	2923	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	716.5;
	2923	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	833.2;
	2923	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4723.1;
	2923	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10124.5;
	2923	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6419.7;
	2923	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12099.2;
	2923	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1956.7;
	2924	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.1;
	2924	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.3;
	2924	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	842;
	2924	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4674.1;
	2924	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10237;
	2924	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6479.5;
	2924	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11879.7;
	2924	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1949.5;
	2925	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.2;
	2925	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	713;
	2925	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.4;
	2925	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4550.8;
	2925	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10048.3;
	2925	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6342.4;
	2925	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11482.1;
	2925	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1922.8;
	2926	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	861.6;
	2926	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	667.9;
	2926	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	792.2;
	2926	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4255.4;
	2926	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9240.6;
	2926	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5813.5;
	2926	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10846.7;
	2926	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1736.2;
	2927	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	837.9;
	2927	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	617.2;
	2927	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	736.7;
	2927	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3874.5;
	2927	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8240.1;
	2927	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5195.8;
	2927	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10099.8;
	2927	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1563.9;
	2928	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	825.9;
	2928	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	591.3;
	2928	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	708.2;
	2928	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3559.1;
	2928	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7488.6;
	2928	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4701.7;
	2928	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9510.4;
	2928	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1437.5;
	2929	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	816.6;
	2929	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579.5;
	2929	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	685;
	2929	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3347.9;
	2929	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7049.1;
	2929	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4382.4;
	2929	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9092.8;
	2929	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1350.4;
	2930	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	813;
	2930	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.4;
	2930	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	671.7;
	2930	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3204.6;
	2930	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6822.2;
	2930	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4205.1;
	2930	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8860.6;
	2930	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1298.3;
	2931	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	813.8;
	2931	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	571.3;
	2931	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	669.1;
	2931	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3108.9;
	2931	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6786.4;
	2931	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4115.8;
	2931	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8793.8;
	2931	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1248;
	2932	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	819.1;
	2932	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	583.6;
	2932	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	674.5;
	2932	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3085.9;
	2932	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6939.4;
	2932	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4153;
	2932	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8884.5;
	2932	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1277.6;
	2933	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	838.4;
	2933	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	599.9;
	2933	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	709.1;
	2933	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3212.1;
	2933	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7516;
	2933	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4476.4;
	2933	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9322.7;
	2933	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1349.3;
	2934	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.1;
	2934	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	666;
	2934	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	789.9;
	2934	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3513.6;
	2934	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8599.6;
	2934	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5053.7;
	2934	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9977.3;
	2934	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1515.7;
	2935	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.3;
	2935	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	697.9;
	2935	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824;
	2935	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3557.8;
	2935	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9147.5;
	2935	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5203.9;
	2935	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10096.2;
	2935	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1571.2;
	2936	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.6;
	2936	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	712.6;
	2936	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	815.9;
	2936	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3528.1;
	2936	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9255.6;
	2936	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5221.5;
	2936	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10067.6;
	2936	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1603.4;
	2937	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.4;
	2937	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	718.8;
	2937	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	820;
	2937	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3597.2;
	2937	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9423.8;
	2937	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5302.6;
	2937	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10043.8;
	2937	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1634.2;
	2938	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.3;
	2938	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.6;
	2938	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	826.7;
	2938	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3705;
	2938	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9599.9;
	2938	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5398.8;
	2938	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10005.6;
	2938	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1647.8;
	2939	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.7;
	2939	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.3;
	2939	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	842.7;
	2939	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3755.2;
	2939	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9690.4;
	2939	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5444.6;
	2939	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9958.3;
	2939	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1669.1;
	2940	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.9;
	2940	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	726.1;
	2940	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838;
	2940	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3726.3;
	2940	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9700.9;
	2940	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5445.8;
	2940	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9898.7;
	2940	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1686.3;
	2941	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.6;
	2941	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.1;
	2941	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	834.6;
	2941	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3726.3;
	2941	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9742.9;
	2941	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5474.8;
	2941	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9999.7;
	2941	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1686.2;
	2942	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.3;
	2942	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	726.7;
	2942	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	833.1;
	2942	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3721.9;
	2942	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9681.7;
	2942	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5465.4;
	2942	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10158.6;
	2942	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1713.2;
	2943	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.3;
	2943	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	726.2;
	2943	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	819.4;
	2943	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3734.8;
	2943	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9637.7;
	2943	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5423.9;
	2943	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10366.7;
	2943	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1686.1;
	2944	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.2;
	2944	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.1;
	2944	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	825.7;
	2944	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3721.9;
	2944	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9616;
	2944	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5400.9;
	2944	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10466.7;
	2944	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1692.7;
	2945	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.2;
	2945	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.4;
	2945	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	839.7;
	2945	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3626.2;
	2945	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9629.1;
	2945	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5420.4;
	2945	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10478.4;
	2945	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1671.2;
	2946	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.4;
	2946	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	732;
	2946	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	827.3;
	2946	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3539;
	2946	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9654.8;
	2946	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5461.6;
	2946	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10312.6;
	2946	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1643.1;
	2947	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.8;
	2947	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728;
	2947	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	823.9;
	2947	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3493.1;
	2947	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9622.8;
	2947	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5494.2;
	2947	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10083.9;
	2947	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1659.9;
	2948	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.1;
	2948	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753;
	2948	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	862.3;
	2948	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3645.8;
	2948	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9873.9;
	2948	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5685.8;
	2948	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10301.1;
	2948	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1703.4;
	2949	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904;
	2949	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	748.4;
	2949	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	871.5;
	2949	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3598.7;
	2949	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9737.3;
	2949	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5528.1;
	2949	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10060.6;
	2949	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1666.9;
	2950	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.3;
	2950	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	699.7;
	2950	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	820.4;
	2950	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3343.4;
	2950	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8893.1;
	2950	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5078.1;
	2950	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9364.9;
	2950	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1538.6;
	2951	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	840.4;
	2951	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	647.5;
	2951	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	755.5;
	2951	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3020.2;
	2951	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7983.7;
	2951	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4521;
	2951	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8643.8;
	2951	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1384.8;
	2952	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	815.4;
	2952	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	613.5;
	2952	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	712.5;
	2952	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2768.2;
	2952	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7365.3;
	2952	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4149;
	2952	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8119.3;
	2952	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1300.9;
	2953	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	801.3;
	2953	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.2;
	2953	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	690.9;
	2953	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2608.3;
	2953	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7062.9;
	2953	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3938;
	2953	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7785.6;
	2953	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1229.1;
	2954	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	796.4;
	2954	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	583.9;
	2954	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	681.1;
	2954	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2512.8;
	2954	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6889.7;
	2954	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3865.3;
	2954	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7594.9;
	2954	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1222.3;
	2955	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	797.8;
	2955	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	583.6;
	2955	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	679.1;
	2955	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2472.6;
	2955	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6896;
	2955	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3829;
	2955	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7539.7;
	2955	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1146.7;
	2956	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	799.3;
	2956	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	593.9;
	2956	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	687.4;
	2956	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2471;
	2956	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7088.4;
	2956	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3904.9;
	2956	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7661.5;
	2956	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1242.3;
	2957	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	813.3;
	2957	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	621.3;
	2957	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	720;
	2957	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2591.4;
	2957	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7729.8;
	2957	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4228.6;
	2957	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8094.2;
	2957	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1326.4;
	2958	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	843.9;
	2958	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	677;
	2958	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	799.3;
	2958	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2928.3;
	2958	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8874;
	2958	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4893.8;
	2958	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8766.5;
	2958	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1476.7;
	2959	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	853.8;
	2959	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	700.5;
	2959	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824.4;
	2959	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2989.4;
	2959	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9250.3;
	2959	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5067.5;
	2959	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8943.1;
	2959	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1538.9;
	2960	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	854.9;
	2960	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	703.7;
	2960	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	821.2;
	2960	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2962.9;
	2960	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9238.3;
	2960	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5087.5;
	2960	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9096.1;
	2960	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1553.6;
	2961	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.6;
	2961	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	710.1;
	2961	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	819.8;
	2961	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3024.5;
	2961	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9393.1;
	2961	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5173.1;
	2961	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9340.4;
	2961	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1581;
	2962	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	863.7;
	2962	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714.1;
	2962	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	830.2;
	2962	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3120.3;
	2962	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9579.2;
	2962	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5276.8;
	2962	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9662.4;
	2962	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1625.1;
	2963	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.1;
	2963	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	721.6;
	2963	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.4;
	2963	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3225.2;
	2963	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9715.6;
	2963	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5399.6;
	2963	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9943.6;
	2963	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1653.7;
	2964	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.3;
	2964	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.3;
	2964	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.7;
	2964	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3312;
	2964	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9782.5;
	2964	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5503.6;
	2964	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10172.5;
	2964	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1672.8;
	2965	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.3;
	2965	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.2;
	2965	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	883.9;
	2965	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3447.2;
	2965	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9888.2;
	2965	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5662.7;
	2965	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10501.5;
	2965	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1664.8;
	2966	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.7;
	2966	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.8;
	2966	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	901.4;
	2966	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3555.1;
	2966	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10058.7;
	2966	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5828.3;
	2966	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10809.9;
	2966	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1684.3;
	2967	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.6;
	2967	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744.4;
	2967	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	907.3;
	2967	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3636.6;
	2967	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10181.9;
	2967	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6022.9;
	2967	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11104.8;
	2967	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1756.9;
	2968	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.5;
	2968	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.8;
	2968	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	922.3;
	2968	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3733.2;
	2968	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10397;
	2968	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6263.3;
	2968	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11314;
	2968	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1789.6;
	2969	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.2;
	2969	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746.8;
	2969	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	924.6;
	2969	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3742.6;
	2969	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10578.9;
	2969	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6414.1;
	2969	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11347.1;
	2969	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1779.1;
	2970	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.4;
	2970	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.3;
	2970	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	914.6;
	2970	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3696.1;
	2970	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10543.2;
	2970	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6420.1;
	2970	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11079.3;
	2970	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1768.1;
	2971	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.4;
	2971	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	724.7;
	2971	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	890.3;
	2971	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3649.7;
	2971	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10366.3;
	2971	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6274;
	2971	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10702.5;
	2971	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1727.1;
	2972	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.6;
	2972	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735.6;
	2972	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897.1;
	2972	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3777.8;
	2972	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10573.5;
	2972	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6309.2;
	2972	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10714.2;
	2972	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1749.1;
	2973	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.2;
	2973	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	721.2;
	2973	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	881;
	2973	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3734.5;
	2973	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10339.8;
	2973	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6105.6;
	2973	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10359.9;
	2973	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1693.9;
	2974	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.6;
	2974	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	667;
	2974	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	818.8;
	2974	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3473.5;
	2974	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9337.1;
	2974	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5508.7;
	2974	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9635.8;
	2974	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1549;
	2975	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	856;
	2975	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	620;
	2975	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	742.6;
	2975	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3110.5;
	2975	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8263.2;
	2975	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4847.6;
	2975	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8818.1;
	2975	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1432.5;
	2976	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	826;
	2976	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584;
	2976	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	687.5;
	2976	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2836.8;
	2976	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7555;
	2976	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4373.7;
	2976	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8229.3;
	2976	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1338.9;
	2977	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	814.3;
	2977	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	574.2;
	2977	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	680.6;
	2977	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2676.4;
	2977	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7133.3;
	2977	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4108.4;
	2977	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7870.5;
	2977	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1240.7;
	2978	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	808.6;
	2978	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	563.7;
	2978	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	643.1;
	2978	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2577.7;
	2978	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6907.4;
	2978	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3937.2;
	2978	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7655.4;
	2978	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1190.6;
	2979	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	800.7;
	2979	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	552.5;
	2979	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	659.1;
	2979	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2509;
	2979	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6806;
	2979	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3845.6;
	2979	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7576.1;
	2979	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1200.6;
	2980	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	798.9;
	2980	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	561;
	2980	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	663.5;
	2980	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2490.8;
	2980	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6962.6;
	2980	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3918.1;
	2980	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7682.3;
	2980	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1214.2;
	2981	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	813.6;
	2981	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.5;
	2981	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	694.9;
	2981	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2616.3;
	2981	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7600.5;
	2981	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4228.2;
	2981	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8132;
	2981	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1200.7;
	2982	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	847.2;
	2982	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	656.5;
	2982	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	772.2;
	2982	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2954.8;
	2982	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8709.8;
	2982	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4865.8;
	2982	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8767;
	2982	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1363.3;
	2983	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.4;
	2983	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	676.9;
	2983	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	798.4;
	2983	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3016.5;
	2983	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9097.5;
	2983	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5047.7;
	2983	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8972.1;
	2983	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1421;
	2984	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.8;
	2984	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	688.3;
	2984	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	805.1;
	2984	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3084.6;
	2984	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9295.7;
	2984	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5121;
	2984	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9253.9;
	2984	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1444.5;
	2985	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.1;
	2985	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	698.3;
	2985	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	814.9;
	2985	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3267;
	2985	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9587.4;
	2985	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5293.6;
	2985	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9609.7;
	2985	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1505.5;
	2986	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.4;
	2986	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	669.9;
	2986	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	834.9;
	2986	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3481.5;
	2986	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9899.7;
	2986	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5508.9;
	2986	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9962.2;
	2986	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1554.2;
	2987	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.5;
	2987	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	694.8;
	2987	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849.8;
	2987	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3697.5;
	2987	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10213;
	2987	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5752.8;
	2987	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10223.7;
	2987	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1587.4;
	2988	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.5;
	2988	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	716.8;
	2988	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	871;
	2988	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3934.8;
	2988	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10481.3;
	2988	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6005.5;
	2988	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10555.9;
	2988	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1642.6;
	2989	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.4;
	2989	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738;
	2989	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	896.9;
	2989	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4209.5;
	2989	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10782.1;
	2989	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6297.8;
	2989	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11023;
	2989	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1720.2;
	2990	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.9;
	2990	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	763.8;
	2990	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	923.2;
	2990	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4467.2;
	2990	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11062;
	2990	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6604;
	2990	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11458.9;
	2990	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1782.7;
	2991	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989.4;
	2991	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	785.2;
	2991	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	957.3;
	2991	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4668.8;
	2991	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11418.2;
	2991	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6963.6;
	2991	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11852.6;
	2991	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1837;
	2992	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000.8;
	2992	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	815.1;
	2992	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	994;
	2992	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4812.2;
	2992	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11767.7;
	2992	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7305.9;
	2992	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12079.8;
	2992	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1891.9;
	2993	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1012.9;
	2993	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	827.7;
	2993	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1010.3;
	2993	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4789.9;
	2993	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12006.8;
	2993	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7508;
	2993	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12155;
	2993	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1891.9;
	2994	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.7;
	2994	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	836.3;
	2994	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1014.1;
	2994	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4655.7;
	2994	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12036.7;
	2994	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7535.9;
	2994	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11901.5;
	2994	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1878.8;
	2995	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.9;
	2995	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	820.6;
	2995	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	997;
	2995	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4387.8;
	2995	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11741.5;
	2995	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7285.8;
	2995	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11506.4;
	2995	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1825.6;
	2996	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980;
	2996	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	818.2;
	2996	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	979.9;
	2996	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4291.9;
	2996	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11627.7;
	2996	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7100.8;
	2996	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11420.3;
	2996	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1816.1;
	2997	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.6;
	2997	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	794.4;
	2997	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	944.5;
	2997	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4130.1;
	2997	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11245.9;
	2997	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6738.6;
	2997	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10980;
	2997	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1750.6;
	2998	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.2;
	2998	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.5;
	2998	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.9;
	2998	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3768.1;
	2998	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10135.1;
	2998	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6004.6;
	2998	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10123;
	2998	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1580.9;
	2999	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.1;
	2999	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	671.6;
	2999	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	789.4;
	2999	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3317.4;
	2999	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8944.6;
	2999	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5223.2;
	2999	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9187.1;
	2999	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1402.7;
	3000	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	852.3;
	3000	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	641.8;
	3000	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	732.8;
	3000	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2980.6;
	3000	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8044.9;
	3000	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4649.5;
	3000	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8480;
	3000	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1287.8;
	3001	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	837.5;
	3001	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	616.3;
	3001	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	701.3;
	3001	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2768;
	3001	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7524.9;
	3001	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4285.4;
	3001	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8062.1;
	3001	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1214.9;
	3002	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	829.1;
	3002	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	602.9;
	3002	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	683.4;
	3002	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2633.1;
	3002	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7198.7;
	3002	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4088.6;
	3002	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7813.2;
	3002	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1172.7;
	3003	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	822.4;
	3003	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	603.7;
	3003	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	676.9;
	3003	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2557.2;
	3003	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7055.5;
	3003	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3978;
	3003	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7667.5;
	3003	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1173;
	3004	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	826.2;
	3004	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	602.5;
	3004	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	683.1;
	3004	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2539.9;
	3004	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7141.8;
	3004	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3999.8;
	3004	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7752.6;
	3004	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1192.6;
	3005	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	834.9;
	3005	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	619.5;
	3005	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	711;
	3005	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2642.5;
	3005	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7697.7;
	3005	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4305.8;
	3005	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8201;
	3005	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1285.1;
	3006	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	857;
	3006	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	668.2;
	3006	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	782.8;
	3006	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2953.9;
	3006	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8680.1;
	3006	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4903.2;
	3006	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8830.1;
	3006	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1459.6;
	3007	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.1;
	3007	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	690;
	3007	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	813.2;
	3007	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3035.7;
	3007	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9094.6;
	3007	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5068.6;
	3007	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9076.7;
	3007	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1513.4;
	3008	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.4;
	3008	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	708.7;
	3008	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	826;
	3008	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3140.1;
	3008	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9372.7;
	3008	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5243.8;
	3008	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9436;
	3008	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1565.1;
	3009	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.5;
	3009	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	732.7;
	3009	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	864.5;
	3009	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3369.2;
	3009	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9789.7;
	3009	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5509.7;
	3009	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9951;
	3009	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1625.6;
	3010	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.8;
	3010	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.4;
	3010	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897;
	3010	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3638.5;
	3010	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10205.2;
	3010	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5824.9;
	3010	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10411.4;
	3010	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1700.6;
	3011	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942;
	3011	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	767.5;
	3011	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	925.3;
	3011	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3986.9;
	3011	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10578.8;
	3011	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6151.7;
	3011	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10846;
	3011	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1754.8;
	3012	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.1;
	3012	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	785.8;
	3012	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	951.4;
	3012	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4339.6;
	3012	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10942.8;
	3012	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6495.5;
	3012	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11343.5;
	3012	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1846.1;
	3013	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986.6;
	3013	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	813.6;
	3013	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	983.8;
	3013	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4670.1;
	3013	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11411.7;
	3013	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6896.5;
	3013	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11907.8;
	3013	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1855.3;
	3014	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.9;
	3014	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	835.6;
	3014	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1019.2;
	3014	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4934.2;
	3014	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11835.6;
	3014	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7298.8;
	3014	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12372.8;
	3014	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1891;
	3015	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1034.1;
	3015	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	857.6;
	3015	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1058.2;
	3015	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5146.3;
	3015	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12276.8;
	3015	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7650.5;
	3015	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12808;
	3015	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1930.5;
	3016	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1042.2;
	3016	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	877.8;
	3016	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1084.3;
	3016	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5286;
	3016	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12667.4;
	3016	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7912.6;
	3016	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13051.5;
	3016	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1971.7;
	3017	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1035.3;
	3017	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	879.4;
	3017	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1083.6;
	3017	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5232.6;
	3017	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12755.2;
	3017	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8038.3;
	3017	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12907.7;
	3017	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2001.4;
	3018	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1018.4;
	3018	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	865.9;
	3018	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1064.5;
	3018	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5031.5;
	3018	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12500.2;
	3018	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7888.9;
	3018	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12382.6;
	3018	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1950.8;
	3019	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	994.2;
	3019	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	837.5;
	3019	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1031.6;
	3019	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4734.7;
	3019	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11950.2;
	3019	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7525.1;
	3019	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11812.8;
	3019	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1871.1;
	3020	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	984.2;
	3020	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	831.8;
	3020	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1016.5;
	3020	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4619.8;
	3020	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11699.1;
	3020	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7370.6;
	3020	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11697.9;
	3020	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1848.8;
	3021	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970.8;
	3021	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	809.9;
	3021	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	988.3;
	3021	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4415;
	3021	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11263.5;
	3021	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6994.1;
	3021	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11137.9;
	3021	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1780.9;
	3022	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.1;
	3022	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	757;
	3022	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913.9;
	3022	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4007.4;
	3022	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10218.5;
	3022	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6227;
	3022	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10204.1;
	3022	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1625.6;
	3023	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.6;
	3023	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	682.4;
	3023	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	827.8;
	3023	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3523.8;
	3023	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9034.7;
	3023	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5429;
	3023	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9241.9;
	3023	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1444.5;
	3024	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	861.2;
	3024	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	625.1;
	3024	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	772.8;
	3024	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3172.1;
	3024	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8145.2;
	3024	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4851.2;
	3024	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8566;
	3024	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1340.6;
	3025	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	844.7;
	3025	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	601;
	3025	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	740.4;
	3025	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2937.5;
	3025	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7595.7;
	3025	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4485.8;
	3025	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8124.5;
	3025	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1262.6;
	3026	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	834.7;
	3026	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589.3;
	3026	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	719.6;
	3026	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2789.1;
	3026	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7267.1;
	3026	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4247.6;
	3026	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7859.5;
	3026	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1224.5;
	3027	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	828.2;
	3027	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	591.3;
	3027	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	708.8;
	3027	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2687.7;
	3027	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7085.5;
	3027	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4104.5;
	3027	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7721.3;
	3027	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1222.3;
	3028	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	826.9;
	3028	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	591.9;
	3028	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	708.3;
	3028	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2648.4;
	3028	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7185.2;
	3028	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4111.7;
	3028	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7781.4;
	3028	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1219;
	3029	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	833.4;
	3029	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	602.8;
	3029	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	731.1;
	3029	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2739.9;
	3029	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7701.7;
	3029	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4358;
	3029	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8173.2;
	3029	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1293.5;
	3030	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.8;
	3030	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	652.9;
	3030	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	794.9;
	3030	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3032.4;
	3030	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8683.9;
	3030	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4943.9;
	3030	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8747.2;
	3030	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1421.9;
	3031	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867.3;
	3031	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	677;
	3031	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	819.3;
	3031	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3107.1;
	3031	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9106;
	3031	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5116.3;
	3031	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8934.7;
	3031	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1494.2;
	3032	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.1;
	3032	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	689.3;
	3032	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	836.6;
	3032	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3245.3;
	3032	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9382.8;
	3032	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5322.2;
	3032	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9329.8;
	3032	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1536;
	3033	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.7;
	3033	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711.9;
	3033	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	862;
	3033	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3522.5;
	3033	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9856.2;
	3033	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5635.7;
	3033	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9869.1;
	3033	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1606.9;
	3034	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.6;
	3034	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	734.7;
	3034	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	890.5;
	3034	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3844.4;
	3034	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10306.5;
	3034	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5941.6;
	3034	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10415.8;
	3034	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1675.6;
	3035	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.2;
	3035	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.1;
	3035	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	922.1;
	3035	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4127.1;
	3035	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10718.9;
	3035	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6197.2;
	3035	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10855.4;
	3035	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1714.6;
	3036	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.2;
	3036	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776.1;
	3036	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	952.4;
	3036	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4406.6;
	3036	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11127.6;
	3036	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6502.6;
	3036	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11313.5;
	3036	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1802;
	3037	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	998.7;
	3037	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	801.1;
	3037	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	995.6;
	3037	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4673.3;
	3037	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11680.1;
	3037	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6857.8;
	3037	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11864.7;
	3037	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1884.6;
	3038	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1017.6;
	3038	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	825.9;
	3038	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1029;
	3038	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4885.3;
	3038	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12182.4;
	3038	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7221.1;
	3038	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12351.2;
	3038	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1909.2;
	3039	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1036.7;
	3039	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	846.9;
	3039	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1063.2;
	3039	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5056.4;
	3039	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12680.3;
	3039	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7571.9;
	3039	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12823.3;
	3039	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1879.7;
	3040	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1044;
	3040	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	861.3;
	3040	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1088.6;
	3040	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5145.7;
	3040	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13035.7;
	3040	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7855.5;
	3040	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13089.5;
	3040	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1940.2;
	3041	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1043.2;
	3041	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	866.7;
	3041	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1090.1;
	3041	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5072.7;
	3041	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13153;
	3041	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7922.8;
	3041	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13065;
	3041	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1930.7;
	3042	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030.8;
	3042	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	851;
	3042	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1067.2;
	3042	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4844.1;
	3042	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12878.8;
	3042	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7754.7;
	3042	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12583.6;
	3042	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1881;
	3043	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	998.6;
	3043	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	810.1;
	3043	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1020.9;
	3043	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4524.1;
	3043	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12188.8;
	3043	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7327.7;
	3043	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11864.6;
	3043	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1877.8;
	3044	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	978.1;
	3044	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	788.4;
	3044	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	988;
	3044	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4366.1;
	3044	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11781.1;
	3044	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7008.1;
	3044	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11501.8;
	3044	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1849.4;
	3045	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	971.8;
	3045	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	771.2;
	3045	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	961.2;
	3045	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4182.1;
	3045	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11310;
	3045	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6670.4;
	3045	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10994;
	3045	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1758.7;
	3046	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.1;
	3046	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744.8;
	3046	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	904.4;
	3046	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3906.2;
	3046	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10492.3;
	3046	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6151.7;
	3046	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10322.3;
	3046	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1556.6;
	3047	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922;
	3047	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	688;
	3047	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	836.1;
	3047	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3568.5;
	3047	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9437.3;
	3047	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5539.9;
	3047	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9528.6;
	3047	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1433.4;
	3048	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.6;
	3048	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	639.9;
	3048	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	783.3;
	3048	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3259.5;
	3048	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8549.7;
	3048	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4998.7;
	3048	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8832.4;
	3048	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1357.1;
	3049	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.4;
	3049	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	611.3;
	3049	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	744.6;
	3049	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3020.4;
	3049	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7889.4;
	3049	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4597.5;
	3049	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8339.9;
	3049	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1278.2;
	3050	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	861.6;
	3050	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.8;
	3050	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	718.8;
	3050	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2852;
	3050	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7435.6;
	3050	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4331.8;
	3050	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7990.4;
	3050	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1226.7;
	3051	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	848.5;
	3051	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589.8;
	3051	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	703.9;
	3051	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2736.2;
	3051	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7205.4;
	3051	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4150.9;
	3051	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7779;
	3051	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1203.3;
	3052	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	836.4;
	3052	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586.5;
	3052	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	694;
	3052	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2675.5;
	3052	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7091.6;
	3052	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4087.6;
	3052	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7677.8;
	3052	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1187.8;
	3053	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	839.9;
	3053	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.6;
	3053	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	700.9;
	3053	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2693.5;
	3053	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7270;
	3053	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4134.1;
	3053	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7784.8;
	3053	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1181.7;
	3054	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	849.7;
	3054	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	605.1;
	3054	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	719;
	3054	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2757.3;
	3054	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7548.7;
	3054	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4279.7;
	3054	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7960;
	3054	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1143.5;
	3055	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	847.3;
	3055	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	620;
	3055	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	734;
	3055	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2813.9;
	3055	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7903.1;
	3055	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4459;
	3055	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8139.7;
	3055	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1277.8;
	3056	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.1;
	3056	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	659.5;
	3056	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	777.8;
	3056	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3117.7;
	3056	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8635.7;
	3056	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4861.4;
	3056	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8763.9;
	3056	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1363.2;
	3057	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.6;
	3057	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	689;
	3057	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	821.2;
	3057	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3493.2;
	3057	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9474.9;
	3057	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5271.7;
	3057	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9470.6;
	3057	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1427.4;
	3058	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.4;
	3058	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	715.6;
	3058	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	856;
	3058	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3809.8;
	3058	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10045;
	3058	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5649.8;
	3058	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10125.9;
	3058	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1525.9;
	3059	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.6;
	3059	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	734.5;
	3059	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	872.5;
	3059	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4065.6;
	3059	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10478.4;
	3059	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5969;
	3059	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10659;
	3059	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1645.2;
	3060	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	983.2;
	3060	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	758;
	3060	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	894.5;
	3060	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4269.9;
	3060	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10907.1;
	3060	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6296.7;
	3060	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11126.2;
	3060	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1723;
	3061	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1006.9;
	3061	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	780.3;
	3061	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	917.1;
	3061	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4447;
	3061	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11281.8;
	3061	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6579.8;
	3061	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11532.3;
	3061	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1789.1;
	3062	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1027.7;
	3062	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	800;
	3062	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	948.2;
	3062	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4544.7;
	3062	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11645.8;
	3062	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6849.2;
	3062	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11841;
	3062	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1857;
	3063	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1048.6;
	3063	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	824.2;
	3063	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	981.5;
	3063	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4605.7;
	3063	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12008.9;
	3063	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7050.6;
	3063	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12119.8;
	3063	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1914.7;
	3064	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1062;
	3064	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	844;
	3064	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1005.1;
	3064	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4616;
	3064	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12267;
	3064	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7264.6;
	3064	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12265.9;
	3064	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1930.3;
	3065	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1058.8;
	3065	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	848.9;
	3065	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1010.4;
	3065	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4529.6;
	3065	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12246.4;
	3065	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7320.5;
	3065	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12159.2;
	3065	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1936.2;
	3066	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1037.1;
	3066	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	844.8;
	3066	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	992.1;
	3066	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4349.1;
	3066	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11958.8;
	3066	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7128;
	3066	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11762.3;
	3066	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1903.9;
	3067	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1007;
	3067	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	821.3;
	3067	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	962.7;
	3067	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4131.7;
	3067	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11457.2;
	3067	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6828.5;
	3067	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11192.3;
	3067	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1837.8;
	3068	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	979.9;
	3068	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	803.7;
	3068	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	950.5;
	3068	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4100.3;
	3068	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11229.2;
	3068	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6750.8;
	3068	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10990.4;
	3068	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1827.3;
	3069	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.8;
	3069	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	792.9;
	3069	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	943.4;
	3069	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4017.4;
	3069	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10941.5;
	3069	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6559.1;
	3069	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10659.6;
	3069	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1757;
	3070	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.8;
	3070	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749.2;
	3070	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	896.9;
	3070	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3813;
	3070	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10245.3;
	3070	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6141.6;
	3070	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10112.8;
	3070	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1656.8;
	3071	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.5;
	3071	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	696.6;
	3071	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.3;
	3071	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3576.7;
	3071	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9402.2;
	3071	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5635.8;
	3071	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9491.7;
	3071	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1528.2;
	3072	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.1;
	3072	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	658.3;
	3072	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	793.8;
	3072	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3341.3;
	3072	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8705.7;
	3072	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5173.9;
	3072	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8929;
	3072	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1323.1;
	3073	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.4;
	3073	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	629.9;
	3073	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	759.5;
	3073	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3148.8;
	3073	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8107;
	3073	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4819.2;
	3073	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8519.1;
	3073	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1235.8;
	3074	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	861.5;
	3074	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	610.5;
	3074	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	734.3;
	3074	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2995.3;
	3074	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7663.6;
	3074	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4561;
	3074	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8234.1;
	3074	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1218.8;
	3075	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	849.6;
	3075	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	599.2;
	3075	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	718.7;
	3075	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2888.8;
	3075	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7421.8;
	3075	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4376.1;
	3075	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8036.9;
	3075	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1237;
	3076	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	842.6;
	3076	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	593.7;
	3076	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	709.7;
	3076	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2835.4;
	3076	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7310.6;
	3076	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4305.2;
	3076	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7949.7;
	3076	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1209.4;
	3077	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	841;
	3077	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	601.1;
	3077	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	710.4;
	3077	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2834.5;
	3077	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7362.4;
	3077	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4343.4;
	3077	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7950.8;
	3077	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1213;
	3078	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	844.2;
	3078	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	613.2;
	3078	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	722.6;
	3078	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2862.3;
	3078	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7517.5;
	3078	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4453.6;
	3078	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7991.1;
	3078	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1227.1;
	3079	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	841;
	3079	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	618;
	3079	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	738.4;
	3079	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2865.1;
	3079	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7773.7;
	3079	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4579.2;
	3079	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8067.1;
	3079	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1263;
	3080	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	859.8;
	3080	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	655.6;
	3080	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	781.3;
	3080	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3058.9;
	3080	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8384.1;
	3080	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4889.7;
	3080	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8618;
	3080	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1369.8;
	3081	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.7;
	3081	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	689.6;
	3081	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	819.9;
	3081	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3296.3;
	3081	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9014.9;
	3081	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5247.5;
	3081	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9239.8;
	3081	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1446;
	3082	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920.7;
	3082	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	696.9;
	3082	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.8;
	3082	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3498.7;
	3082	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9361.6;
	3082	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5477.6;
	3082	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9681.7;
	3082	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1489.6;
	3083	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.5;
	3083	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	703.1;
	3083	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	874;
	3083	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3620.9;
	3083	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9460.9;
	3083	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5600.8;
	3083	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9990.4;
	3083	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1515;
	3084	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.1;
	3084	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.8;
	3084	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	907.1;
	3084	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3681.6;
	3084	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9522.1;
	3084	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5638.4;
	3084	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10213.6;
	3084	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1533.7;
	3085	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	991.6;
	3085	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	734.5;
	3085	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940.5;
	3085	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3789.3;
	3085	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9569.9;
	3085	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5621.3;
	3085	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10295.3;
	3085	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1509.7;
	3086	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1011.2;
	3086	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.4;
	3086	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	965.2;
	3086	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3941.3;
	3086	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9585;
	3086	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5703.4;
	3086	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10361.2;
	3086	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1532.9;
	3087	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1023.5;
	3087	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	774.6;
	3087	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	975.4;
	3087	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4029.6;
	3087	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9604.6;
	3087	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5737.8;
	3087	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10289.6;
	3087	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1543;
	3088	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030.6;
	3088	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	791.2;
	3088	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	965.1;
	3088	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4058;
	3088	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9668.8;
	3088	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5768;
	3088	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10212.3;
	3088	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1489.1;
	3089	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1019.1;
	3089	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	791.2;
	3089	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	941.4;
	3089	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4037.8;
	3089	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9759.1;
	3089	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5791.9;
	3089	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10169.2;
	3089	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1540.1;
	3090	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	994;
	3090	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	780.7;
	3090	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	915.6;
	3090	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3965.3;
	3090	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9799.2;
	3090	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5838.7;
	3090	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10113.5;
	3090	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1559.7;
	3091	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	968.8;
	3091	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	773.1;
	3091	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897.6;
	3091	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3876.5;
	3091	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9976.3;
	3091	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5914;
	3091	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10168.4;
	3091	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1583.4;
	3092	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.5;
	3092	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	784.4;
	3092	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	904.6;
	3092	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4031.6;
	3092	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10394.1;
	3092	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6144.7;
	3092	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10473.4;
	3092	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1639.8;
	3093	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948;
	3093	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	773.2;
	3093	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	905;
	3093	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4024.1;
	3093	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10248;
	3093	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6128.7;
	3093	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10394.5;
	3093	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1654.9;
	3094	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.1;
	3094	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.9;
	3094	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.3;
	3094	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3790.6;
	3094	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9502.1;
	3094	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5772.3;
	3094	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9916.5;
	3094	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1563.9;
	3095	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.8;
	3095	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	667.4;
	3095	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	793.8;
	3095	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3442.5;
	3095	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8592.1;
	3095	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5216.9;
	3095	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9223.5;
	3095	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1428.4;
	3096	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866.1;
	3096	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	634.9;
	3096	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	751.2;
	3096	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3165.9;
	3096	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7879.3;
	3096	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4796.2;
	3096	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8753.2;
	3096	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1336.2;
	3097	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	848.8;
	3097	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	626.7;
	3097	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	723.5;
	3097	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2982;
	3097	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7499;
	3097	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4545.1;
	3097	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8465.3;
	3097	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1278.7;
	3098	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	834.6;
	3098	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	603;
	3098	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	707.4;
	3098	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2866;
	3098	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7252.8;
	3098	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4358.6;
	3098	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8334.3;
	3098	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1240.4;
	3099	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	825.3;
	3099	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	599.7;
	3099	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	700.3;
	3099	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2799.4;
	3099	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7192.9;
	3099	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4297.6;
	3099	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8299.3;
	3099	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1210.8;
	3100	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	825.2;
	3100	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	608.4;
	3100	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	700.6;
	3100	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2810.5;
	3100	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7357.8;
	3100	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4359.8;
	3100	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8461.7;
	3100	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1247.1;
	3101	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	836;
	3101	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	625.4;
	3101	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	728.4;
	3101	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2967.4;
	3101	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7948.6;
	3101	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4728.7;
	3101	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8994.8;
	3101	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1323.6;
	3102	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.7;
	3102	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	687.3;
	3102	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	802.2;
	3102	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3321.7;
	3102	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9099.8;
	3102	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5380.4;
	3102	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9749;
	3102	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1502.4;
	3103	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.7;
	3103	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720.1;
	3103	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	830.2;
	3103	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3421.4;
	3103	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9627.9;
	3103	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5595;
	3103	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9964.1;
	3103	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1567.6;
	3104	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892;
	3104	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.9;
	3104	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838.6;
	3104	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3504.9;
	3104	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9816.9;
	3104	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5751.2;
	3104	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10153.7;
	3104	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1613.7;
	3105	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.7;
	3105	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755;
	3105	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	869.7;
	3105	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3725.1;
	3105	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10097.8;
	3105	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5996.6;
	3105	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10464.4;
	3105	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1692.8;
	3106	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.9;
	3106	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	781.7;
	3106	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	910.4;
	3106	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3977;
	3106	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10624.7;
	3106	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6238.4;
	3106	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11021.1;
	3106	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1762.2;
	3107	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.5;
	3107	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	799.4;
	3107	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	960;
	3107	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4188;
	3107	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11081.8;
	3107	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6499.8;
	3107	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11488.4;
	3107	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1838.4;
	3108	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.3;
	3108	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	818.7;
	3108	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	990.9;
	3108	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4395.6;
	3108	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11495.9;
	3108	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6780.4;
	3108	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12098.8;
	3108	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1909.9;
	3109	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	983.6;
	3109	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	844.3;
	3109	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1029.4;
	3109	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4633.6;
	3109	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12091.5;
	3109	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7175.9;
	3109	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12367.5;
	3109	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1955.3;
	3110	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000.7;
	3110	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	863.5;
	3110	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1064.4;
	3110	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4816.4;
	3110	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12630.9;
	3110	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7553;
	3110	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12411.8;
	3110	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1995.8;
	3111	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1014.8;
	3111	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	886.1;
	3111	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1078.4;
	3111	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4976.6;
	3111	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13206.9;
	3111	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7931.3;
	3111	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12635.7;
	3111	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2023.3;
	3112	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1023.7;
	3112	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	906.5;
	3112	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1106;
	3112	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5104.7;
	3112	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13658.3;
	3112	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8272;
	3112	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12842.8;
	3112	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2089.2;
	3113	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1028.8;
	3113	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	914.2;
	3113	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1124;
	3113	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5032.5;
	3113	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13923.8;
	3113	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8515.8;
	3113	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12919.5;
	3113	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2103.1;
	3114	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1022.7;
	3114	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	910.9;
	3114	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1100.6;
	3114	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4890.8;
	3114	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13853.7;
	3114	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8508.9;
	3114	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12597.1;
	3114	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2069;
	3115	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1004.2;
	3115	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	880.8;
	3115	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1065.3;
	3115	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4717.7;
	3115	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13369.5;
	3115	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8257.2;
	3115	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12120.9;
	3115	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2042.8;
	3116	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985.1;
	3116	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	863.1;
	3116	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1050.5;
	3116	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4740.6;
	3116	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13027.9;
	3116	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8106.1;
	3116	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12146.3;
	3116	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2057.8;
	3117	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.8;
	3117	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	842.7;
	3117	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1031.4;
	3117	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4635;
	3117	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12618;
	3117	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7795.2;
	3117	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11782.9;
	3117	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2002.8;
	3118	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.3;
	3118	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	791.5;
	3118	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	954.2;
	3118	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4303;
	3118	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11523.5;
	3118	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7062.2;
	3118	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11004.7;
	3118	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1846;
	3119	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.6;
	3119	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.5;
	3119	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	864.3;
	3119	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3907;
	3119	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10342.2;
	3119	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6277;
	3119	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10190;
	3119	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1661.3;
	3120	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.9;
	3120	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	680.6;
	3120	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	810.8;
	3120	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3595.7;
	3120	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9409.9;
	3120	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5710;
	3120	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9603.5;
	3120	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1544.7;
	3121	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	854.4;
	3121	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	656.2;
	3121	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	775.1;
	3121	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3393.5;
	3121	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8944.4;
	3121	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5370.9;
	3121	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9238.3;
	3121	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1450.5;
	3122	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	839.6;
	3122	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	640;
	3122	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	748.2;
	3122	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3239.4;
	3122	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8557.2;
	3122	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5132.4;
	3122	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8941.2;
	3122	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1424.4;
	3123	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	832.6;
	3123	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	637.5;
	3123	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	734.7;
	3123	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3153.3;
	3123	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8381.1;
	3123	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5015.7;
	3123	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8794;
	3123	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1346.4;
	3124	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	825.8;
	3124	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	624.8;
	3124	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	735.2;
	3124	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3122.2;
	3124	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8449.5;
	3124	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5046.7;
	3124	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8863.3;
	3124	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1418.3;
	3125	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	834.9;
	3125	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	631.1;
	3125	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	758.8;
	3125	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3234.5;
	3125	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9002.8;
	3125	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5372.1;
	3125	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9335.8;
	3125	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1503.3;
	3126	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	865.9;
	3126	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	697.4;
	3126	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	828.8;
	3126	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3586.4;
	3126	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10107.2;
	3126	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6023.7;
	3126	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10120.5;
	3126	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1668;
	3127	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.7;
	3127	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	734.8;
	3127	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.5;
	3127	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3651.7;
	3127	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10598.9;
	3127	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6238;
	3127	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10375.1;
	3127	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1743.6;
	3128	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.5;
	3128	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	767.4;
	3128	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889.4;
	3128	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3774.6;
	3128	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11040.6;
	3128	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6323.8;
	3128	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10775.7;
	3128	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1788.6;
	3129	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.6;
	3129	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	807.9;
	3129	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	930.8;
	3129	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4043.3;
	3129	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11771.4;
	3129	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6680.3;
	3129	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11365.8;
	3129	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1858.1;
	3130	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.4;
	3130	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	848.4;
	3130	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	969.1;
	3130	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4367.2;
	3130	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12673.2;
	3130	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7262.1;
	3130	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12115.3;
	3130	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1965.6;
	3131	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	971.9;
	3131	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	895.8;
	3131	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1054.7;
	3131	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4651.6;
	3131	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13503.7;
	3131	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7838.6;
	3131	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12842.7;
	3131	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2075.1;
	3132	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000.5;
	3132	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	942.2;
	3132	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1115;
	3132	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4906.2;
	3132	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14355.5;
	3132	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8372.1;
	3132	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13498.5;
	3132	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2194.9;
	3133	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1033.3;
	3133	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	995.5;
	3133	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1182.6;
	3133	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5196.3;
	3133	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15261.9;
	3133	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8847;
	3133	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14072.6;
	3133	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2292.8;
	3134	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1065.4;
	3134	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1036;
	3134	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1229.9;
	3134	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5418.6;
	3134	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16082.7;
	3134	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9233.1;
	3134	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14479;
	3134	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2387.4;
	3135	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1094.8;
	3135	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1069.8;
	3135	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1264.4;
	3135	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5569.2;
	3135	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16714.2;
	3135	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9541.2;
	3135	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14744.4;
	3135	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2506.7;
	3136	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1110.5;
	3136	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1098.5;
	3136	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1288.8;
	3136	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5625.4;
	3136	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17114.3;
	3136	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9677.8;
	3136	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14867.5;
	3136	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2546.9;
	3137	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1106.2;
	3137	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1103.3;
	3137	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1274.7;
	3137	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5491.2;
	3137	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17093.9;
	3137	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9599.5;
	3137	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14686;
	3137	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2510.5;
	3138	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1095;
	3138	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1091.6;
	3138	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1240;
	3138	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5240.5;
	3138	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16411.8;
	3138	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9317;
	3138	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14009.6;
	3138	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2431.6;
	3139	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1074.7;
	3139	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1054;
	3139	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1170;
	3139	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4959;
	3139	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15517.1;
	3139	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8907.2;
	3139	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13221.8;
	3139	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2331.7;
	3140	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1044.9;
	3140	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1020.7;
	3140	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1129.8;
	3140	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4914.6;
	3140	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14312.3;
	3140	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8680.7;
	3140	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12988.5;
	3140	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2282.9;
	3141	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1021.5;
	3141	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	976.6;
	3141	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1080.8;
	3141	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4764.1;
	3141	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13254.8;
	3141	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8271.1;
	3141	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12553;
	3141	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2155.7;
	3142	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.7;
	3142	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	879.8;
	3142	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	996.1;
	3142	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4449.2;
	3142	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11821.6;
	3142	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7477.1;
	3142	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11760.7;
	3142	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1961.5;
	3143	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.8;
	3143	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	800.1;
	3143	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	896.6;
	3143	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4046.5;
	3143	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10396.3;
	3143	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6514;
	3143	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10924.9;
	3143	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1776.1;
	3144	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.1;
	3144	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	754;
	3144	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	800.6;
	3144	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3768.5;
	3144	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9439;
	3144	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5562.2;
	3144	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10347.2;
	3144	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1617.4;
	3145	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871;
	3145	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	716.5;
	3145	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	779.3;
	3145	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3560.5;
	3145	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8814.7;
	3145	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5161.2;
	3145	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9842;
	3145	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1466.9;
	3146	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.9;
	3146	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	692.6;
	3146	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	734;
	3146	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3432.1;
	3146	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8454.7;
	3146	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4793.9;
	3146	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9495.9;
	3146	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1396.9;
	3147	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	856.3;
	3147	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	678.1;
	3147	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	745.5;
	3147	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3336.3;
	3147	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8211.1;
	3147	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4583.9;
	3147	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9303.8;
	3147	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1382.9;
	3148	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	852.3;
	3148	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	675.2;
	3148	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	742.4;
	3148	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3296.6;
	3148	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8191;
	3148	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4535.6;
	3148	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9320.3;
	3148	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1370.7;
	3149	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866.9;
	3149	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	685.9;
	3149	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	765.6;
	3149	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3418.8;
	3149	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8723.3;
	3149	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4745.9;
	3149	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9741.2;
	3149	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1341.1;
	3150	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.2;
	3150	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.8;
	3150	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	833.2;
	3150	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3749.2;
	3150	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9735.9;
	3150	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5363.1;
	3150	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10282.1;
	3150	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1492.9;
	3151	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.1;
	3151	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.8;
	3151	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.9;
	3151	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3802.8;
	3151	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10143.1;
	3151	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5600.3;
	3151	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10372.8;
	3151	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1551.5;
	3152	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.6;
	3152	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	782.5;
	3152	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	880.5;
	3152	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3890.6;
	3152	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10583.6;
	3152	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5737.4;
	3152	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10637.7;
	3152	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1589.9;
	3153	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.2;
	3153	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	811.9;
	3153	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	910.3;
	3153	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4128.1;
	3153	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11238.1;
	3153	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6056.1;
	3153	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11010.3;
	3153	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1666.6;
	3154	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000;
	3154	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	862;
	3154	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	947.9;
	3154	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4399.8;
	3154	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12028.2;
	3154	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6465;
	3154	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11516.4;
	3154	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1781.9;
	3155	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1036.4;
	3155	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	906.2;
	3155	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	999.6;
	3155	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4623.7;
	3155	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12814.1;
	3155	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6890.6;
	3155	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12063.8;
	3155	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1896.6;
	3156	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1062.6;
	3156	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	947.9;
	3156	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1058.2;
	3156	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4815.6;
	3156	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13517.6;
	3156	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7319.4;
	3156	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12776.5;
	3156	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2012.3;
	3157	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1090.3;
	3157	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	995.3;
	3157	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1114.7;
	3157	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5019.3;
	3157	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14282.7;
	3157	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7681.7;
	3157	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13431.4;
	3157	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2147.9;
	3158	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1122;
	3158	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1036.6;
	3158	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1169.9;
	3158	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5215;
	3158	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15002.5;
	3158	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7966.7;
	3158	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13952.3;
	3158	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2226.6;
	3159	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1138.4;
	3159	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1047.2;
	3159	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1222;
	3159	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5382.7;
	3159	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15587.7;
	3159	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8280.7;
	3159	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14285;
	3159	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2281.6;
	3160	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1135.5;
	3160	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1055.5;
	3160	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1259.7;
	3160	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5475;
	3160	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15929.5;
	3160	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8564.7;
	3160	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14348.6;
	3160	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2296;
	3161	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1141.9;
	3161	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1066.5;
	3161	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1262;
	3161	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5409.4;
	3161	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15982.2;
	3161	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8697.2;
	3161	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14220.7;
	3161	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2281.7;
	3162	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1124.4;
	3162	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1046.1;
	3162	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1216.3;
	3162	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5225.6;
	3162	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15510.3;
	3162	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8660.4;
	3162	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13665.1;
	3162	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2231.6;
	3163	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1098.8;
	3163	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1004.7;
	3163	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1162.5;
	3163	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4991.6;
	3163	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14787.5;
	3163	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8363.7;
	3163	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13086.9;
	3163	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2136.1;
	3164	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1074;
	3164	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	984.3;
	3164	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1136.2;
	3164	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4974.3;
	3164	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14558.9;
	3164	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8233.4;
	3164	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13025;
	3164	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2110.9;
	3165	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1052.5;
	3165	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	955.3;
	3165	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1088.5;
	3165	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4834.6;
	3165	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13992.4;
	3165	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7888.6;
	3165	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12542.7;
	3165	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2022.9;
	3166	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1012.1;
	3166	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	890.6;
	3166	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1004.7;
	3166	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4482.9;
	3166	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12715.6;
	3166	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7121.1;
	3166	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11695.7;
	3166	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1838.2;
	3167	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.3;
	3167	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	812.1;
	3167	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	914.4;
	3167	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4049.3;
	3167	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10963.2;
	3167	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6303.2;
	3167	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10738.9;
	3167	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1644.7;
	3168	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.6;
	3168	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.9;
	3168	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.4;
	3168	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3748.4;
	3168	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9493.2;
	3168	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5697;
	3168	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10117.5;
	3168	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1511.6;
	3169	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918;
	3169	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711.8;
	3169	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	804.9;
	3169	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3532.8;
	3169	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8815.9;
	3169	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5325.2;
	3169	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9663.4;
	3169	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1421.4;
	3170	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.1;
	3170	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	685.1;
	3170	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	777.8;
	3170	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3384.9;
	3170	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8351.5;
	3170	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5088.6;
	3170	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9324.2;
	3170	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1360.9;
	3171	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.8;
	3171	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	676.5;
	3171	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	762.1;
	3171	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3301.7;
	3171	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8110.6;
	3171	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4946;
	3171	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9109.8;
	3171	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1329.3;
	3172	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.5;
	3172	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	666.1;
	3172	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	759.9;
	3172	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3272.6;
	3172	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8115.8;
	3172	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4952.4;
	3172	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9152.9;
	3172	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1347.7;
	3173	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.8;
	3173	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	679.1;
	3173	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	777.1;
	3173	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3394.7;
	3173	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8703.2;
	3173	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5248.9;
	3173	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9648.1;
	3173	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1435.8;
	3174	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.2;
	3174	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	722.1;
	3174	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	839.1;
	3174	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3731.8;
	3174	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9713.1;
	3174	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5861.7;
	3174	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10274.3;
	3174	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1605.8;
	3175	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931;
	3175	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744.6;
	3175	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.9;
	3175	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3800.1;
	3175	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10129.1;
	3175	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5980.4;
	3175	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10501.1;
	3175	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1655.1;
	3176	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.5;
	3176	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	760.5;
	3176	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	875.4;
	3176	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3869;
	3176	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10307.4;
	3176	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6132.3;
	3176	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10962;
	3176	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1717.8;
	3177	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.3;
	3177	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	778.2;
	3177	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	895.2;
	3177	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4070.3;
	3177	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10535.9;
	3177	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6396.5;
	3177	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11618.8;
	3177	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1800;
	3178	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.8;
	3178	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	806;
	3178	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	923.8;
	3178	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4308.6;
	3178	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10857.8;
	3178	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6668;
	3178	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12419.8;
	3178	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1891.5;
	3179	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.7;
	3179	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	831.9;
	3179	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940.7;
	3179	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4500.3;
	3179	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11108.3;
	3179	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6769.4;
	3179	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13105.4;
	3179	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1954;
	3180	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.6;
	3180	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	857.4;
	3180	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	954.5;
	3180	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4651.9;
	3180	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11457.2;
	3180	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6796.6;
	3180	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13673.5;
	3180	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2016.5;
	3181	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	984.1;
	3181	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	892.4;
	3181	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	979.9;
	3181	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4840.2;
	3181	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12060.6;
	3181	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7065.9;
	3181	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14184.3;
	3181	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2040;
	3182	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1011.6;
	3182	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	916.3;
	3182	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1011.1;
	3182	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5005.7;
	3182	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12630;
	3182	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7391.5;
	3182	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14455.2;
	3182	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2100.2;
	3183	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1028.9;
	3183	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	937.8;
	3183	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1047.9;
	3183	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5149.8;
	3183	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13139.7;
	3183	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7690.6;
	3183	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14745.7;
	3183	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2154.8;
	3184	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1038.5;
	3184	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	952.6;
	3184	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1071.7;
	3184	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5225;
	3184	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13554.4;
	3184	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8006.7;
	3184	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15053.1;
	3184	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2219.5;
	3185	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1036.5;
	3185	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	942.9;
	3185	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1074.1;
	3185	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5193.7;
	3185	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13658.5;
	3185	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8204;
	3185	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14995.2;
	3185	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2259.5;
	3186	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1021.4;
	3186	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	918.7;
	3186	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1055.4;
	3186	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4999.8;
	3186	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13373.2;
	3186	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8114.1;
	3186	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14377.8;
	3186	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2217.7;
	3187	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1005;
	3187	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	883.8;
	3187	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1032;
	3187	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4780.2;
	3187	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12824.6;
	3187	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7848.3;
	3187	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13827.4;
	3187	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2150.9;
	3188	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	993;
	3188	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	865.6;
	3188	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1016.1;
	3188	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4779.7;
	3188	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12514.3;
	3188	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7842.1;
	3188	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13592.3;
	3188	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2118.9;
	3189	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	984.7;
	3189	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	840.4;
	3189	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1000.1;
	3189	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4634.1;
	3189	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12093.1;
	3189	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7524.4;
	3189	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12856.8;
	3189	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2040.5;
	3190	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.6;
	3190	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	768.8;
	3190	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	941.5;
	3190	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4293.2;
	3190	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10985.3;
	3190	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6831;
	3190	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11891.6;
	3190	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1856.3;
	3191	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.7;
	3191	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	703.3;
	3191	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.8;
	3191	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3870.6;
	3191	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9712.4;
	3191	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6092.6;
	3191	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10904.6;
	3191	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1653.7;
	3192	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.7;
	3192	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	667.9;
	3192	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	814.6;
	3192	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3532.9;
	3192	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8700.3;
	3192	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5558.4;
	3192	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10112;
	3192	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1526.2;
	3193	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.3;
	3193	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	639.1;
	3193	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	781.7;
	3193	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3313.3;
	3193	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8085;
	3193	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5182.6;
	3193	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9547;
	3193	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1424.8;
	3194	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	857.8;
	3194	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	623.3;
	3194	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	760.5;
	3194	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3164.8;
	3194	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7687;
	3194	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4941;
	3194	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9157.2;
	3194	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1368.7;
	3195	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	850.4;
	3195	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	623.3;
	3195	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	749.5;
	3195	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3081;
	3195	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7446.9;
	3195	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4810.6;
	3195	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8948.4;
	3195	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1344.7;
	3196	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	850.7;
	3196	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	619.4;
	3196	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	748.8;
	3196	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3059.3;
	3196	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7540.2;
	3196	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4793.6;
	3196	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8972.9;
	3196	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1341.7;
	3197	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	860.4;
	3197	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	631.1;
	3197	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	771.7;
	3197	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3183;
	3197	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8044.8;
	3197	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5064.8;
	3197	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9332.2;
	3197	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1399.2;
	3198	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.8;
	3198	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	682.3;
	3198	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.8;
	3198	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3528;
	3198	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9019.8;
	3198	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5653.2;
	3198	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9931.6;
	3198	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1521.2;
	3199	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.6;
	3199	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	698.6;
	3199	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.9;
	3199	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3610.2;
	3199	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9481.2;
	3199	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5840.6;
	3199	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10159.3;
	3199	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1602.7;
	3200	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.5;
	3200	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.1;
	3200	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	891.4;
	3200	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3769.4;
	3200	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9887.3;
	3200	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6066.3;
	3200	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10704.7;
	3200	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1668.2;
	3201	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.7;
	3201	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	754.2;
	3201	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	922.2;
	3201	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4043.5;
	3201	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10408;
	3201	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6422.3;
	3201	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11447.7;
	3201	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1758.4;
	3202	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	978.3;
	3202	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	786.7;
	3202	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	965.3;
	3202	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4379.3;
	3202	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10982.2;
	3202	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6866.3;
	3202	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12310.2;
	3202	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1858.1;
	3203	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1003.7;
	3203	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	813.1;
	3203	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1005.4;
	3203	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4656.1;
	3203	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11530;
	3203	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7313.1;
	3203	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13106.4;
	3203	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1931.4;
	3204	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030.5;
	3204	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	844.2;
	3204	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1042.3;
	3204	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4915.5;
	3204	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12059.2;
	3204	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7772.3;
	3204	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13808.3;
	3204	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2043.6;
	3205	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1053.6;
	3205	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	874.5;
	3205	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1090.3;
	3205	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5127.4;
	3205	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12693.3;
	3205	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8221.9;
	3205	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14227.8;
	3205	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2145.6;
	3206	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1077;
	3206	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	906;
	3206	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1126.5;
	3206	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5307.7;
	3206	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13252.2;
	3206	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8557.5;
	3206	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14416.2;
	3206	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2161;
	3207	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1095.9;
	3207	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	932.3;
	3207	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1162.8;
	3207	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5407.5;
	3207	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13768.6;
	3207	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8844.6;
	3207	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14528;
	3207	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2120.1;
	3208	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1104;
	3208	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	949.7;
	3208	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1185.6;
	3208	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5431.5;
	3208	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14137.5;
	3208	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9045;
	3208	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14275.2;
	3208	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2162.6;
	3209	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1097.2;
	3209	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	946.4;
	3209	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1179.6;
	3209	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5318.1;
	3209	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14176;
	3209	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9078.2;
	3209	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13429.3;
	3209	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2140.5;
	3210	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1076.2;
	3210	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	925.4;
	3210	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1151.4;
	3210	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5102.7;
	3210	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13799.9;
	3210	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8827.1;
	3210	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12557.4;
	3210	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2073.1;
	3211	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1047.7;
	3211	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	878.5;
	3211	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1105.1;
	3211	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4826.1;
	3211	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13057.8;
	3211	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8398.6;
	3211	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11998.7;
	3211	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2049.5;
	3212	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1023.5;
	3212	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	855.6;
	3212	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1073.5;
	3212	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4734.9;
	3212	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12611.2;
	3212	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8175.9;
	3212	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11920.2;
	3212	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2007.5;
	3213	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1019.2;
	3213	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	836.4;
	3213	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1048.3;
	3213	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4597.5;
	3213	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12183.7;
	3213	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7828.8;
	3213	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11617.1;
	3213	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1910.1;
	3214	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	988.5;
	3214	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	804;
	3214	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	984.5;
	3214	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4333.3;
	3214	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11310.2;
	3214	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7293.7;
	3214	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10993.6;
	3214	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1694.1;
	3215	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.9;
	3215	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.3;
	3215	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	903.7;
	3215	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4008.6;
	3215	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10245.2;
	3215	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6590.5;
	3215	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10246.8;
	3215	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1563.7;
	3216	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.2;
	3216	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	696.3;
	3216	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	817;
	3216	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3721.7;
	3216	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9405.1;
	3216	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5689.3;
	3216	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9691.9;
	3216	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1510.4;
	3217	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.2;
	3217	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	666.9;
	3217	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	776.6;
	3217	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3491.2;
	3217	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8790.3;
	3217	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5253.5;
	3217	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9182.9;
	3217	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1417.9;
	3218	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871.9;
	3218	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	647.4;
	3218	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	747.1;
	3218	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3336.3;
	3218	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8348.8;
	3218	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4948.2;
	3218	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8847.6;
	3218	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1365;
	3219	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.8;
	3219	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	628.7;
	3219	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	729.6;
	3219	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3215.7;
	3219	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8067.8;
	3219	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4752.8;
	3219	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8635;
	3219	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1336.9;
	3220	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	847.6;
	3220	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	619.2;
	3220	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	717.9;
	3220	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3149.7;
	3220	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7899.9;
	3220	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4672.3;
	3220	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8533.6;
	3220	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1303;
	3221	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	850.3;
	3221	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	629;
	3221	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	718.9;
	3221	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3157.2;
	3221	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8061.8;
	3221	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4705.8;
	3221	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8613.1;
	3221	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1291.1;
	3222	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	856.5;
	3222	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	626.2;
	3222	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	732.5;
	3222	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3201.8;
	3222	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8343;
	3222	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4862.4;
	3222	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8762.1;
	3222	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1249.2;
	3223	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	849.1;
	3223	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	630.7;
	3223	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	747.4;
	3223	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3242.5;
	3223	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8581.9;
	3223	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5046.2;
	3223	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8966.8;
	3223	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1374.8;
	3224	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.7;
	3224	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	658.7;
	3224	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	787.4;
	3224	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3518.9;
	3224	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9063.4;
	3224	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5425.9;
	3224	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9681.7;
	3224	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1481.3;
	3225	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.5;
	3225	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	677.7;
	3225	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	820.5;
	3225	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3858.9;
	3225	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9553.7;
	3225	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5867;
	3225	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10572.2;
	3225	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1568.2;
	3226	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905;
	3226	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	691.6;
	3226	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	833.8;
	3226	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4134.8;
	3226	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9810.4;
	3226	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6310.5;
	3226	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11531.7;
	3226	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1663.1;
	3227	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.2;
	3227	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	698.7;
	3227	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838.5;
	3227	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4298.5;
	3227	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9869.8;
	3227	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6535.9;
	3227	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12335.1;
	3227	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1779.2;
	3228	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.6;
	3228	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	697.4;
	3228	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.2;
	3228	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4437.4;
	3228	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9899.7;
	3228	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6612.5;
	3228	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12962.5;
	3228	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1838.3;
	3229	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.2;
	3229	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	694.8;
	3229	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	863.9;
	3229	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4552.9;
	3229	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9901.7;
	3229	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6509.5;
	3229	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13399.7;
	3229	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1870.8;
	3230	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.3;
	3230	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	697.6;
	3230	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.2;
	3230	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4571.7;
	3230	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9977;
	3230	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6336.2;
	3230	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13657.8;
	3230	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1892.4;
	3231	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.5;
	3231	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	704.1;
	3231	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859;
	3231	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4533.1;
	3231	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10099.8;
	3231	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6177.1;
	3231	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13161.2;
	3231	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1897.9;
	3232	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.8;
	3232	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711.2;
	3232	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.3;
	3232	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4418;
	3232	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10296.2;
	3232	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6108.8;
	3232	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12170.3;
	3232	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1855.7;
	3233	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.3;
	3233	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.2;
	3233	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.2;
	3233	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4238.4;
	3233	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10417.4;
	3233	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6122.7;
	3233	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11339.8;
	3233	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1815.5;
	3234	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.8;
	3234	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	716.2;
	3234	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.8;
	3234	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4010.7;
	3234	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10271.9;
	3234	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6145.2;
	3234	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10868.9;
	3234	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1764.5;
	3235	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.7;
	3235	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	703.2;
	3235	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	833.1;
	3235	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3874;
	3235	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9908.6;
	3235	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6105;
	3235	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10543.8;
	3235	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1708.4;
	3236	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.1;
	3236	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	702.5;
	3236	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	837.6;
	3236	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3862.8;
	3236	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9790.9;
	3236	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6110.7;
	3236	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10395.3;
	3236	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1718.7;
	3237	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.8;
	3237	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	703.1;
	3237	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	836.3;
	3237	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3800;
	3237	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9651.2;
	3237	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5955.9;
	3237	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10152.1;
	3237	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1671.5;
	3238	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	863.6;
	3238	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	673.2;
	3238	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	796.8;
	3238	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3633.1;
	3238	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9065.7;
	3238	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5586.8;
	3238	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9731.1;
	3238	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1597.1;
	3239	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	842.7;
	3239	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	633.8;
	3239	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	753.7;
	3239	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3421.4;
	3239	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8307.5;
	3239	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5110;
	3239	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9213.2;
	3239	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1489.7;
	3240	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	828.8;
	3240	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	610;
	3240	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	726.4;
	3240	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3197.3;
	3240	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7662.7;
	3240	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4729.9;
	3240	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8715.3;
	3240	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1286.8;
	3241	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	817.9;
	3241	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.4;
	3241	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	698.8;
	3241	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3030;
	3241	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7174;
	3241	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4427.4;
	3241	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8349.3;
	3241	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1209.6;
	3242	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	810.1;
	3242	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	581.4;
	3242	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	680.3;
	3242	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2899.9;
	3242	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6848.3;
	3242	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4212.9;
	3242	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8093.4;
	3242	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1202.9;
	3243	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	803.2;
	3243	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	575.7;
	3243	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	669.2;
	3243	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2821.6;
	3243	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6668.4;
	3243	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4085.4;
	3243	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7953.1;
	3243	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1214.5;
	3244	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	800.5;
	3244	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	574.6;
	3244	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	664.6;
	3244	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2782.5;
	3244	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6603.9;
	3244	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4031.2;
	3244	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7878.3;
	3244	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1195.4;
	3245	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	803.5;
	3245	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	585.2;
	3245	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	665.9;
	3245	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2782.2;
	3245	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6668.5;
	3245	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4066.2;
	3245	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7874.1;
	3245	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1202.9;
	3246	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	808;
	3246	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	600.6;
	3246	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	677.4;
	3246	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2800.4;
	3246	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6797.8;
	3246	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4168.1;
	3246	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7880.5;
	3246	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1215.1;
	3247	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	810.8;
	3247	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	608.9;
	3247	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	693.3;
	3247	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2817.5;
	3247	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7023.6;
	3247	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4282.4;
	3247	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8023;
	3247	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1251.1;
	3248	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	819.9;
	3248	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	639.1;
	3248	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	730.6;
	3248	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3059.4;
	3248	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7569.5;
	3248	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4571;
	3248	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8545.6;
	3248	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1353.9;
	3249	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	835.2;
	3249	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	666.2;
	3249	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	760.7;
	3249	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3351.1;
	3249	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8082.9;
	3249	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4899.2;
	3249	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9104.7;
	3249	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1421;
	3250	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	843.4;
	3250	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	668.5;
	3250	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	774.5;
	3250	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3572.1;
	3250	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8440.1;
	3250	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5131.5;
	3250	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9645.9;
	3250	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1477.1;
	3251	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	852.3;
	3251	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	677.9;
	3251	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	783;
	3251	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3631.8;
	3251	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8710.5;
	3251	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5279.2;
	3251	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10137.7;
	3251	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1522.8;
	3252	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	861.5;
	3252	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	688.3;
	3252	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	796.6;
	3252	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3645.5;
	3252	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8990.8;
	3252	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5359.4;
	3252	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10686.1;
	3252	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1591.6;
	3253	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.6;
	3253	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	695;
	3253	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	806.3;
	3253	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3738;
	3253	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9270.4;
	3253	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5389.7;
	3253	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11100.3;
	3253	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1622.7;
	3254	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	876.5;
	3254	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	698.3;
	3254	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	816;
	3254	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3703.1;
	3254	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9495.6;
	3254	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5422.5;
	3254	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11290.9;
	3254	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1678.8;
	3255	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.3;
	3255	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	707.3;
	3255	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	826.9;
	3255	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3613;
	3255	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9745.2;
	3255	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5485.4;
	3255	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11001.9;
	3255	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1698;
	3256	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.6;
	3256	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.9;
	3256	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	834.4;
	3256	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3586.3;
	3256	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9981.3;
	3256	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5575.1;
	3256	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10652.8;
	3256	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1663.6;
	3257	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.4;
	3257	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720.7;
	3257	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	841.8;
	3257	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3585.5;
	3257	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10138.5;
	3257	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5694.5;
	3257	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10392.3;
	3257	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1699.6;
	3258	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.4;
	3258	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723.3;
	3258	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849;
	3258	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3608.7;
	3258	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10210.4;
	3258	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5767.8;
	3258	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10268.2;
	3258	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1688.7;
	3259	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.6;
	3259	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723.7;
	3259	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	850.3;
	3259	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3623.8;
	3259	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10170.6;
	3259	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5857.9;
	3259	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10225.4;
	3259	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1672.3;
	3260	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.3;
	3260	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744.4;
	3260	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.1;
	3260	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3778.5;
	3260	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10435.2;
	3260	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6043.6;
	3260	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10352.1;
	3260	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1685.6;
	3261	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.8;
	3261	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	739.4;
	3261	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.1;
	3261	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3811.9;
	3261	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10329.7;
	3261	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5937.8;
	3261	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10213.9;
	3261	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1674.6;
	3262	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.5;
	3262	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	694.8;
	3262	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	814.8;
	3262	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3644.2;
	3262	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9541.4;
	3262	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5525.7;
	3262	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9724.8;
	3262	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1597.7;
	3263	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	848.6;
	3263	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	650.9;
	3263	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	757.2;
	3263	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3363.5;
	3263	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8542.4;
	3263	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4985.3;
	3263	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9088.5;
	3263	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1436.6;
	3264	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	832.1;
	3264	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	619.4;
	3264	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	720.8;
	3264	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3112.2;
	3264	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7803.1;
	3264	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4557.4;
	3264	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8563.7;
	3264	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1339.3;
	3265	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	823.7;
	3265	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	612.4;
	3265	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	697.4;
	3265	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2939.9;
	3265	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7362.4;
	3265	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4318.4;
	3265	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8256.7;
	3265	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1280.6;
	3266	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	817.7;
	3266	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	601.9;
	3266	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	685.8;
	3266	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2829.7;
	3266	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7121.4;
	3266	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4189.3;
	3266	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8099.2;
	3266	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1242.5;
	3267	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	813.3;
	3267	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	599.2;
	3267	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	680.1;
	3267	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2767.9;
	3267	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7075.7;
	3267	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4157.3;
	3267	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8086.9;
	3267	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1212.8;
	3268	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	812.8;
	3268	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	608.7;
	3268	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	685.2;
	3268	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2763.8;
	3268	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7229;
	3268	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4240.3;
	3268	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8263.7;
	3268	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1252.6;
	3269	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	823.2;
	3269	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	625.9;
	3269	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	714.4;
	3269	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2903;
	3269	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7812.1;
	3269	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4615.3;
	3269	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8768.9;
	3269	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1334.9;
	3270	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	852;
	3270	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	678.9;
	3270	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	785.5;
	3270	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3225.6;
	3270	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8861.9;
	3270	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5249.8;
	3270	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9520.9;
	3270	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1506.5;
	3271	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866.5;
	3271	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	707.6;
	3271	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	814.4;
	3271	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3304;
	3271	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9348.3;
	3271	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5483.1;
	3271	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9778;
	3271	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1564.4;
	3272	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	881.1;
	3272	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	731.9;
	3272	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	814.5;
	3272	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3414.5;
	3272	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9553.3;
	3272	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5583.3;
	3272	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9809;
	3272	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1597.6;
	3273	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.7;
	3273	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723.3;
	3273	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	832.6;
	3273	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3619.1;
	3273	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9855.4;
	3273	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5735.5;
	3273	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9891.9;
	3273	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1634.7;
	3274	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.4;
	3274	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	739.1;
	3274	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857;
	3274	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3878.1;
	3274	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10195.8;
	3274	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5930.4;
	3274	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10049.6;
	3274	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1667.1;
	3275	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965;
	3275	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	763.1;
	3275	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	901.6;
	3275	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4206.3;
	3275	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10600.4;
	3275	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6114.3;
	3275	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10177.9;
	3275	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1687.2;
	3276	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	984.2;
	3276	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	781.3;
	3276	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	932.5;
	3276	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4498.5;
	3276	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11024.8;
	3276	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6400.1;
	3276	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10256.2;
	3276	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1712.9;
	3277	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1017.4;
	3277	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	793.9;
	3277	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	968.9;
	3277	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4797.8;
	3277	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11447.6;
	3277	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6683.1;
	3277	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10343.6;
	3277	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1728.8;
	3278	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1043.2;
	3278	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	809.6;
	3278	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1007.2;
	3278	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5026.1;
	3278	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11732.8;
	3278	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6923.4;
	3278	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10341.1;
	3278	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1763.3;
	3279	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1063.3;
	3279	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	821.5;
	3279	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1025.5;
	3279	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5174.2;
	3279	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11902.9;
	3279	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7092.3;
	3279	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10399.3;
	3279	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1760.1;
	3280	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1076.2;
	3280	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	836.9;
	3280	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1060.3;
	3280	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5279.3;
	3280	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12012;
	3280	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7212.1;
	3280	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10484.5;
	3280	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1791.4;
	3281	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1078.6;
	3281	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	839;
	3281	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1081.5;
	3281	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5253.4;
	3281	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11986.5;
	3281	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7270.3;
	3281	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10605.6;
	3281	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1788.7;
	3282	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1070;
	3282	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	840.2;
	3282	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1061.1;
	3282	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5120.4;
	3282	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11839.3;
	3282	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7239.1;
	3282	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10566.5;
	3282	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1759.8;
	3283	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1050.8;
	3283	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	823.6;
	3283	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1032.6;
	3283	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4883.5;
	3283	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11667.4;
	3283	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7064;
	3283	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10432.4;
	3283	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1765.4;
	3284	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1032.8;
	3284	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	831;
	3284	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1029.9;
	3284	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4824.7;
	3284	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11767.8;
	3284	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7027.6;
	3284	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10695;
	3284	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1809.4;
	3285	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1027;
	3285	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	825.9;
	3285	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1024.4;
	3285	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4687.1;
	3285	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11590.7;
	3285	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6860.9;
	3285	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10571.6;
	3285	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1765.3;
	3286	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	988;
	3286	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777;
	3286	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	953.4;
	3286	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4286.2;
	3286	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10648.8;
	3286	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6198.8;
	3286	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9883.5;
	3286	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1631.9;
	3287	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.6;
	3287	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	722.6;
	3287	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.5;
	3287	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3836.2;
	3287	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9528.2;
	3287	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5504.7;
	3287	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9127.3;
	3287	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1472;
	3288	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.8;
	3288	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	682.4;
	3288	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	807.2;
	3288	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3489.1;
	3288	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8603.2;
	3288	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4977.3;
	3288	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8543.3;
	3288	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1377.4;
	3289	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.9;
	3289	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	656;
	3289	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	774;
	3289	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3249.6;
	3289	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8130.2;
	3289	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4660.7;
	3289	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8207;
	3289	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1291.6;
	3290	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.2;
	3290	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	630.4;
	3290	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	751.9;
	3290	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3101.6;
	3290	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7771.7;
	3290	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4471;
	3290	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7982.2;
	3290	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1279.1;
	3291	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	862;
	3291	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	617.4;
	3291	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	735.9;
	3291	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3030;
	3291	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7635.7;
	3291	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4381.3;
	3291	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7912.2;
	3291	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1196.6;
	3292	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	847;
	3292	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	617.2;
	3292	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	729.8;
	3292	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3007.2;
	3292	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7721.2;
	3292	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4436.6;
	3292	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8020.9;
	3292	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1271.9;
	3293	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	852.4;
	3293	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	632.9;
	3293	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	754.7;
	3293	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3126;
	3293	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8281.5;
	3293	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4734.6;
	3293	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8528.7;
	3293	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1356.8;
	3294	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	881.6;
	3294	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	686;
	3294	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	823.7;
	3294	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3489.6;
	3294	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9302.1;
	3294	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5344;
	3294	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9300.2;
	3294	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1511.6;
	3295	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.5;
	3295	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719.9;
	3295	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849.5;
	3295	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3595.2;
	3295	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9713.4;
	3295	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5571.5;
	3295	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9617;
	3295	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1599.4;
	3296	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.7;
	3296	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.3;
	3296	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.7;
	3296	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3706.7;
	3296	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9875.5;
	3296	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5659.3;
	3296	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9886.5;
	3296	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1631.1;
	3297	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.2;
	3297	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	748.6;
	3297	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	871.3;
	3297	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3924.7;
	3297	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10285.7;
	3297	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5800.6;
	3297	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10311.1;
	3297	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1683.8;
	3298	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.4;
	3298	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	757.2;
	3298	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.6;
	3298	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4192.8;
	3298	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10796.2;
	3298	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5922.8;
	3298	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10821.2;
	3298	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1761.7;
	3299	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.8;
	3299	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	767.8;
	3299	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	910.1;
	3299	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4478.7;
	3299	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11277.3;
	3299	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6035.9;
	3299	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11112.4;
	3299	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1841.5;
	3300	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.5;
	3300	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	779.8;
	3300	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	923.3;
	3300	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4781.6;
	3300	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11812;
	3300	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6156.3;
	3300	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11408;
	3300	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1920.6;
	3301	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.9;
	3301	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	791.4;
	3301	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	946.5;
	3301	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5117.2;
	3301	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12400.3;
	3301	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6278.4;
	3301	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11852.2;
	3301	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1997.1;
	3302	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.2;
	3302	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	791.8;
	3302	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	958.9;
	3302	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5375.7;
	3302	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12870.5;
	3302	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6471.8;
	3302	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12385.6;
	3302	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2064.9;
	3303	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.3;
	3303	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	794.1;
	3303	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	961.6;
	3303	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5546.3;
	3303	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12936.9;
	3303	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6784.8;
	3303	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13033.3;
	3303	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2176.1;
	3304	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.4;
	3304	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	791;
	3304	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	961.7;
	3304	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5621;
	3304	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12644;
	3304	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7217.6;
	3304	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13509.7;
	3304	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2209.9;
	3305	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.8;
	3305	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	780.4;
	3305	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	947.5;
	3305	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5513.6;
	3305	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12238.2;
	3305	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7421.5;
	3305	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13665.7;
	3305	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2177.5;
	3306	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907;
	3306	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	768;
	3306	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	930.3;
	3306	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5270.6;
	3306	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11701.5;
	3306	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7406.2;
	3306	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13483.5;
	3306	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2126.9;
	3307	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.4;
	3307	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.2;
	3307	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	908.7;
	3307	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5008.8;
	3307	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11312.1;
	3307	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7231.2;
	3307	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12969.2;
	3307	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2037.8;
	3308	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.9;
	3308	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753.8;
	3308	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913.8;
	3308	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4931.2;
	3308	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11306.3;
	3308	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7286;
	3308	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12781.5;
	3308	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2003.8;
	3309	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903;
	3309	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.5;
	3309	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	896.8;
	3309	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4742.9;
	3309	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11000;
	3309	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7089.1;
	3309	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12296.5;
	3309	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1922.8;
	3310	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879;
	3310	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	691.7;
	3310	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	836.7;
	3310	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4350.9;
	3310	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9958.4;
	3310	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6453.3;
	3310	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11402.4;
	3310	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1760.2;
	3311	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	845.2;
	3311	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	645.4;
	3311	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	759.4;
	3311	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3904.4;
	3311	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8800.9;
	3311	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5583.4;
	3311	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10435.1;
	3311	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1611.8;
	3312	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	818;
	3312	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	616.6;
	3312	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	703.7;
	3312	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3551.8;
	3312	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8011.6;
	3312	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4837.8;
	3312	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9711.9;
	3312	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1467.9;
	3313	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	804.8;
	3313	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.5;
	3313	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	691.4;
	3313	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3334.3;
	3313	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7508.1;
	3313	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4458.4;
	3313	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9240.9;
	3313	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1342.6;
	3314	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	804.8;
	3314	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588;
	3314	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	652.4;
	3314	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3195.1;
	3314	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7247.5;
	3314	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4236;
	3314	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8950.3;
	3314	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1273.8;
	3315	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	799.1;
	3315	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	583.8;
	3315	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	668.2;
	3315	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3104;
	3315	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7078.8;
	3315	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4188.1;
	3315	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8742.8;
	3315	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1271.5;
	3316	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	795.5;
	3316	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.3;
	3316	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	671.5;
	3316	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3075.9;
	3316	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7175.7;
	3316	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4212.4;
	3316	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8728.3;
	3316	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1265.7;
	3317	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	811.7;
	3317	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	617.7;
	3317	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	698.3;
	3317	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3180.8;
	3317	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7758;
	3317	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4494;
	3317	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9106.7;
	3317	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1239.2;
	3318	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	847.4;
	3318	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	673.3;
	3318	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	769.6;
	3318	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3480.8;
	3318	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8827.1;
	3318	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5114.3;
	3318	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9709.6;
	3318	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1378.7;
	3319	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	865.4;
	3319	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	704.6;
	3319	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	799;
	3319	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3532.4;
	3319	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9216.6;
	3319	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5310.4;
	3319	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9856.9;
	3319	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1432.9;
	3320	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873;
	3320	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	706.5;
	3320	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	808.4;
	3320	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3564.6;
	3320	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9304.8;
	3320	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5335.9;
	3320	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10104.9;
	3320	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1444.7;
	3321	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.6;
	3321	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719.1;
	3321	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	822.3;
	3321	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3691;
	3321	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9557.9;
	3321	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5447.6;
	3321	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10557.9;
	3321	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1488.3;
	3322	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.5;
	3322	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736;
	3322	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	823.9;
	3322	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3861.6;
	3322	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9802.6;
	3322	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5573.1;
	3322	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10986.2;
	3322	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1522.2;
	3323	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.6;
	3323	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.5;
	3323	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	831;
	3323	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4094.7;
	3323	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9920.9;
	3323	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5653.9;
	3323	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11500.5;
	3323	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1534.5;
	3324	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.5;
	3324	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.4;
	3324	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.1;
	3324	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4304.4;
	3324	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9942.8;
	3324	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5775.2;
	3324	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11880.3;
	3324	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1552.8;
	3325	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.8;
	3325	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.5;
	3325	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.3;
	3325	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4500.4;
	3325	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9965.1;
	3325	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5950.4;
	3325	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12274.1;
	3325	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1584.7;
	3326	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.9;
	3326	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.2;
	3326	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.7;
	3326	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4635.6;
	3326	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9899.5;
	3326	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6104.7;
	3326	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12613.8;
	3326	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1594.9;
	3327	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.5;
	3327	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730;
	3327	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849.8;
	3327	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4712.8;
	3327	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9843;
	3327	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6247.8;
	3327	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12914.7;
	3327	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1596;
	3328	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.7;
	3328	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.8;
	3328	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859;
	3328	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4718.4;
	3328	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9796.9;
	3328	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6399.8;
	3328	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12944.1;
	3328	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1595.3;
	3329	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.2;
	3329	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.2;
	3329	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.5;
	3329	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4588;
	3329	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9796.1;
	3329	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6492;
	3329	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12637.9;
	3329	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1576.3;
	3330	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.8;
	3330	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723.5;
	3330	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.8;
	3330	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4451.3;
	3330	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9785.7;
	3330	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6464;
	3330	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12103.3;
	3330	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1563.1;
	3331	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903;
	3331	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720.6;
	3331	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.3;
	3331	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4307.4;
	3331	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9774;
	3331	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6351.1;
	3331	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11819.7;
	3331	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1554.5;
	3332	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.7;
	3332	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.5;
	3332	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.8;
	3332	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4322.4;
	3332	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9905.2;
	3332	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6360.8;
	3332	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11801.4;
	3332	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1582;
	3333	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.1;
	3333	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.2;
	3333	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	858.8;
	3333	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4280.3;
	3333	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9738.9;
	3333	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6159.2;
	3333	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11446.7;
	3333	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1551.7;
	3334	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.8;
	3334	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	688.2;
	3334	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	808.3;
	3334	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4001.1;
	3334	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8974.6;
	3334	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5578.8;
	3334	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10667.4;
	3334	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1429.7;
	3335	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	850.6;
	3335	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	647.6;
	3335	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	752.4;
	3335	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3607.9;
	3335	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8121.5;
	3335	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4950.8;
	3335	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9807.3;
	3335	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1298;
	3336	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	827.4;
	3336	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	617.4;
	3336	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	711.8;
	3336	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3257.2;
	3336	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7531.1;
	3336	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4524.3;
	3336	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9074;
	3336	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1193.6;
	3337	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	821.3;
	3337	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	594.8;
	3337	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	696.7;
	3337	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3072;
	3337	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7179.3;
	3337	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4239.3;
	3337	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8654.1;
	3337	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1160;
	3338	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	814.1;
	3338	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586.6;
	3338	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	686.5;
	3338	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2950.1;
	3338	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6963.1;
	3338	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4093.5;
	3338	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8344;
	3338	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1148.7;
	3339	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	818.3;
	3339	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.4;
	3339	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	680.6;
	3339	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2900.3;
	3339	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6908.1;
	3339	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4027.5;
	3339	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8199.7;
	3339	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1145.3;
	3340	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	822.4;
	3340	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.4;
	3340	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	688.5;
	3340	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2899.9;
	3340	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7048.7;
	3340	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4078.7;
	3340	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8292.7;
	3340	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1186.2;
	3341	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	833.9;
	3341	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	613.9;
	3341	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	714.3;
	3341	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3011.6;
	3341	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7630.3;
	3341	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4398.8;
	3341	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8728;
	3341	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1278.6;
	3342	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864;
	3342	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	668.7;
	3342	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	783.2;
	3342	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3362.3;
	3342	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8701.6;
	3342	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5036.4;
	3342	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9419.2;
	3342	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1441.8;
	3343	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.9;
	3343	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	698.8;
	3343	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	821.1;
	3343	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3475.7;
	3343	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9209.2;
	3343	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5347.5;
	3343	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9606.1;
	3343	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1490.1;
	3344	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.8;
	3344	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.2;
	3344	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.4;
	3344	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3553.9;
	3344	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9345.3;
	3344	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5396.4;
	3344	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9785.2;
	3344	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1523.7;
	3345	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.1;
	3345	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	734.9;
	3345	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.3;
	3345	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3754;
	3345	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9592.1;
	3345	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5493.2;
	3345	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10125.7;
	3345	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1554.3;
	3346	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.2;
	3346	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.5;
	3346	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.4;
	3346	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3922.2;
	3346	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9744.9;
	3346	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5566.5;
	3346	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10309.3;
	3346	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1581.9;
	3347	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.8;
	3347	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749.5;
	3347	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.6;
	3347	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4007.3;
	3347	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9833.7;
	3347	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5588.9;
	3347	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10367.2;
	3347	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1605.8;
	3348	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.5;
	3348	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.4;
	3348	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	881.5;
	3348	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4095.5;
	3348	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9808.2;
	3348	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5601.8;
	3348	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10294.3;
	3348	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1623.8;
	3349	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.8;
	3349	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.5;
	3349	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	872.5;
	3349	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4229.2;
	3349	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9810.6;
	3349	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5611;
	3349	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10166.3;
	3349	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1559;
	3350	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.3;
	3350	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736;
	3350	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	858.6;
	3350	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4299.6;
	3350	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9729.1;
	3350	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5677.3;
	3350	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10021;
	3350	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1514.8;
	3351	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.3;
	3351	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.4;
	3351	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.1;
	3351	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4395.4;
	3351	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9690.9;
	3351	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5836.1;
	3351	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9880.8;
	3351	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1489.2;
	3352	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.8;
	3352	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719.1;
	3352	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	879;
	3352	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4409.2;
	3352	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9774.5;
	3352	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6058.9;
	3352	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9769.7;
	3352	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1500.7;
	3353	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.3;
	3353	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.2;
	3353	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	881;
	3353	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4304.6;
	3353	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9774.8;
	3353	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6219.3;
	3353	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9656.1;
	3353	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1546.4;
	3354	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.2;
	3354	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719.7;
	3354	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	886.5;
	3354	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4155.8;
	3354	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9726.3;
	3354	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6226.5;
	3354	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9399.9;
	3354	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1559.8;
	3355	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.1;
	3355	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714.3;
	3355	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	887.2;
	3355	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3999.2;
	3355	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9666.3;
	3355	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6150.2;
	3355	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9317.8;
	3355	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1579;
	3356	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.7;
	3356	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719.2;
	3356	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	894.8;
	3356	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4037;
	3356	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9844.2;
	3356	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6250.6;
	3356	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9671.7;
	3356	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1613.1;
	3357	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.5;
	3357	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	718.8;
	3357	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	888.6;
	3357	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4055.1;
	3357	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9817.4;
	3357	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6121.2;
	3357	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9648.3;
	3357	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1602.1;
	3358	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	876.7;
	3358	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	688.2;
	3358	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	834.4;
	3358	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3781.9;
	3358	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9087.9;
	3358	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5600.7;
	3358	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9132.1;
	3358	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1493.7;
	3359	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	838.8;
	3359	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	639.7;
	3359	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	774.8;
	3359	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3406.8;
	3359	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8192.6;
	3359	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4979.4;
	3359	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8475;
	3359	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1357.4;
	3360	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	821.6;
	3360	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	610.1;
	3360	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	746.1;
	3360	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3116.2;
	3360	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7530.9;
	3360	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4510.5;
	3360	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7983.7;
	3360	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1284.5;
	3361	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	813.1;
	3361	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	593.8;
	3361	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	721.8;
	3361	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2942.5;
	3361	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7163.7;
	3361	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4227.9;
	3361	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7708.6;
	3361	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1229.4;
	3362	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	807.8;
	3362	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	581.2;
	3362	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	700.6;
	3362	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2835.4;
	3362	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6944;
	3362	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4056.5;
	3362	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7536.3;
	3362	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1203.2;
	3363	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	804.6;
	3363	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	581.9;
	3363	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	694.3;
	3363	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2772.9;
	3363	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6855;
	3363	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3986.9;
	3363	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7465.8;
	3363	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1203.8;
	3364	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	806.4;
	3364	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589.2;
	3364	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	696.8;
	3364	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2761;
	3364	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7007.3;
	3364	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4040.7;
	3364	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7566.5;
	3364	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1214.3;
	3365	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	821.1;
	3365	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	608.9;
	3365	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	723;
	3365	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2875.5;
	3365	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7574;
	3365	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4340.2;
	3365	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8008.5;
	3365	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1283.1;
	3366	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	849.4;
	3366	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	655.7;
	3366	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	786.8;
	3366	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3188.5;
	3366	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8574.4;
	3366	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4953;
	3366	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8630.1;
	3366	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1411.4;
	3367	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	860;
	3367	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	681.1;
	3367	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	816.6;
	3367	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3273.8;
	3367	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9034.8;
	3367	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5199.5;
	3367	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8899.7;
	3367	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1486.9;
	3368	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.1;
	3368	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	705;
	3368	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	829.8;
	3368	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3330.3;
	3368	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9286.7;
	3368	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5314.7;
	3368	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9308.6;
	3368	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1525.1;
	3369	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.8;
	3369	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.5;
	3369	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	845;
	3369	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3459.2;
	3369	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9613.5;
	3369	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5470.1;
	3369	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9826.3;
	3369	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1566;
	3370	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.8;
	3370	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733;
	3370	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	863;
	3370	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3642.1;
	3370	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9950.4;
	3370	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5626.9;
	3370	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10431.5;
	3370	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1606.3;
	3371	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.1;
	3371	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.1;
	3371	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	879;
	3371	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3846;
	3371	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10309.8;
	3371	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5787.1;
	3371	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10963.3;
	3371	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1631.4;
	3372	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.2;
	3372	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.2;
	3372	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	899.6;
	3372	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4024.2;
	3372	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10551.6;
	3372	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6050;
	3372	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11478.3;
	3372	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1696.9;
	3373	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	968.3;
	3373	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	767.1;
	3373	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	941.7;
	3373	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4192.7;
	3373	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10967.4;
	3373	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6382.3;
	3373	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12132;
	3373	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1763.1;
	3374	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999;
	3374	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	780.6;
	3374	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	976.2;
	3374	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4383.4;
	3374	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11429.3;
	3374	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6735.4;
	3374	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12682;
	3374	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1790.4;
	3375	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1021.5;
	3375	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	793.7;
	3375	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1015.7;
	3375	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4469.1;
	3375	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11950.3;
	3375	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7138.1;
	3375	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13141.7;
	3375	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1764.3;
	3376	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1039;
	3376	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	808;
	3376	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1053.5;
	3376	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4488.7;
	3376	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12424.3;
	3376	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7443.3;
	3376	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13396;
	3376	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1852.1;
	3377	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1042.5;
	3377	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	814.7;
	3377	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1067.9;
	3377	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4441.4;
	3377	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12654.8;
	3377	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7613.9;
	3377	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13353.4;
	3377	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1861.9;
	3378	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1028.7;
	3378	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	813;
	3378	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1052.4;
	3378	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4304.2;
	3378	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12503.3;
	3378	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7472.6;
	3378	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12860.4;
	3378	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1826.8;
	3379	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1007.5;
	3379	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	792.2;
	3379	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1016.8;
	3379	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4174.4;
	3379	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11984;
	3379	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7097.3;
	3379	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12186.2;
	3379	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1839.1;
	3380	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	983.2;
	3380	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776.8;
	3380	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	992.5;
	3380	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4206.6;
	3380	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11587.1;
	3380	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6963.1;
	3380	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11840.5;
	3380	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1820.3;
	3381	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980;
	3381	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	775.5;
	3381	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	980.5;
	3381	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4178.9;
	3381	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11323;
	3381	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6802.1;
	3381	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11487.3;
	3381	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1769.1;
	3382	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.1;
	3382	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.4;
	3382	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	926.9;
	3382	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4000.1;
	3382	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10527.1;
	3382	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6380.1;
	3382	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10820.2;
	3382	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1580.1;
	3383	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.2;
	3383	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	704.2;
	3383	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	856.9;
	3383	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3736.4;
	3383	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9531.4;
	3383	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5808.5;
	3383	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10046.7;
	3383	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1464.4;
	3384	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.2;
	3384	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	658.9;
	3384	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	802;
	3384	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3486.9;
	3384	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8614.8;
	3384	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5305.6;
	3384	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9316.6;
	3384	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1384.7;
	3385	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	881.1;
	3385	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	632.9;
	3385	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	765.7;
	3385	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3316.5;
	3385	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8009.5;
	3385	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4951.4;
	3385	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8823.2;
	3385	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1299.7;
	3386	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867.8;
	3386	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	615.7;
	3386	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	739.9;
	3386	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3199.9;
	3386	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7566.4;
	3386	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4700;
	3386	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8493.4;
	3386	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1254.8;
	3387	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	857.1;
	3387	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	609.5;
	3387	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	723.9;
	3387	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3113.6;
	3387	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7317.9;
	3387	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4587.8;
	3387	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8284.7;
	3387	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1232.3;
	3388	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	852.8;
	3388	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	601.3;
	3388	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	715.8;
	3388	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3084.5;
	3388	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7226.9;
	3388	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4527.1;
	3388	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8205.8;
	3388	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1213.9;
	3389	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	853.3;
	3389	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	606.1;
	3389	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	720.5;
	3389	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3115.1;
	3389	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7391;
	3389	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4566.9;
	3389	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8244.9;
	3389	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1207;
	3390	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	860.8;
	3390	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	612.9;
	3390	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	736.5;
	3390	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3171.6;
	3390	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7620.8;
	3390	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4742;
	3390	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8384.9;
	3390	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1166.6;
	3391	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	860.5;
	3391	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	613.5;
	3391	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	755.8;
	3391	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3230.5;
	3391	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8043.2;
	3391	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4970.4;
	3391	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8650.4;
	3391	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1301.5;
	3392	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.7;
	3392	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	662.2;
	3392	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	804.8;
	3392	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3494.8;
	3392	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8843.8;
	3392	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5342.4;
	3392	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9316.3;
	3392	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1394.8;
	3393	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.5;
	3393	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	696.4;
	3393	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.5;
	3393	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3827.1;
	3393	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9716.8;
	3393	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5737.9;
	3393	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10035.1;
	3393	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1479.1;
	3394	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.5;
	3394	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735.8;
	3394	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.2;
	3394	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4181.9;
	3394	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10484.9;
	3394	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6029.9;
	3394	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10825.6;
	3394	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1611.2;
	3395	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.9;
	3395	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	768.2;
	3395	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	919.5;
	3395	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4488.2;
	3395	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11133.1;
	3395	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6244.3;
	3395	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11390.3;
	3395	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1765.8;
	3396	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1003.2;
	3396	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	788.4;
	3396	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	956.7;
	3396	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4705.3;
	3396	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11682.9;
	3396	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6507.6;
	3396	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11509.1;
	3396	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1863.5;
	3397	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1026.1;
	3397	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	809.7;
	3397	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	998.1;
	3397	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4828.1;
	3397	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12125.1;
	3397	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6811.6;
	3397	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11856;
	3397	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1897.4;
	3398	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1051.5;
	3398	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	833.6;
	3398	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1039.4;
	3398	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4958.2;
	3398	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12453.8;
	3398	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7169.3;
	3398	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12417.4;
	3398	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1941.4;
	3399	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1069;
	3399	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	856.9;
	3399	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1077.1;
	3399	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5093.1;
	3399	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12651.6;
	3399	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7449.1;
	3399	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12836.4;
	3399	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1982.7;
	3400	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1076.8;
	3400	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	880.1;
	3400	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1106.9;
	3400	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5170.9;
	3400	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12711.1;
	3400	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7670.3;
	3400	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12808;
	3400	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1990.3;
	3401	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1073.8;
	3401	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	883.9;
	3401	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1121;
	3401	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5145.1;
	3401	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12593.7;
	3401	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7790.7;
	3401	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12015.9;
	3401	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2028.9;
	3402	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1048.2;
	3402	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	873.6;
	3402	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1107.3;
	3402	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5023;
	3402	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12261.2;
	3402	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7756.9;
	3402	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11369.2;
	3402	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2028.5;
	3403	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1018.1;
	3403	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	852.1;
	3403	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1075.3;
	3403	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4810.3;
	3403	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11877.5;
	3403	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7576.7;
	3403	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11024.8;
	3403	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1962.3;
	3404	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	994.7;
	3404	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	831.4;
	3404	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1045.2;
	3404	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4698.4;
	3404	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11661.4;
	3404	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7438.8;
	3404	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11013;
	3404	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1937.1;
	3405	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	991.5;
	3405	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	823;
	3405	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1031.3;
	3405	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4600.8;
	3405	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11488.7;
	3405	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7294.2;
	3405	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10934.5;
	3405	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1867.8;
	3406	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.8;
	3406	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	793;
	3406	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	977;
	3406	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4340.2;
	3406	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10759.6;
	3406	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6834.4;
	3406	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10498.2;
	3406	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1763.7;
	3407	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.3;
	3407	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.2;
	3407	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	918.2;
	3407	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4053.4;
	3407	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9885;
	3407	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6299.4;
	3407	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9958.7;
	3407	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1635.3;
	3408	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.8;
	3408	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	694.9;
	3408	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.7;
	3408	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3773.6;
	3408	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9063;
	3408	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5797.5;
	3408	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9473;
	3408	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1424.9;
	3409	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.2;
	3409	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	666.4;
	3409	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824.3;
	3409	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3558.9;
	3409	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8451.4;
	3409	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5423;
	3409	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9126.1;
	3409	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1332.4;
	3410	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.3;
	3410	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	641.6;
	3410	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	798.9;
	3410	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3405.8;
	3410	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8000.8;
	3410	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5124.9;
	3410	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8886.7;
	3410	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1320.6;
	3411	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.2;
	3411	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	630.2;
	3411	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	779;
	3411	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3301.1;
	3411	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7689.4;
	3411	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4939.1;
	3411	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8725;
	3411	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1326;
	3412	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.6;
	3412	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	617.7;
	3412	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	764.9;
	3412	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3220.8;
	3412	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7522.4;
	3412	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4831.6;
	3412	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8648.9;
	3412	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1302.4;
	3413	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878;
	3413	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	617.4;
	3413	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	760.7;
	3413	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3185.4;
	3413	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7535.2;
	3413	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4811.2;
	3413	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8650.7;
	3413	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1310.8;
	3414	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	881.1;
	3414	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	624.8;
	3414	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	769;
	3414	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3166.9;
	3414	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7641.5;
	3414	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4865.4;
	3414	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8713.9;
	3414	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1327;
	3415	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.9;
	3415	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	635.3;
	3415	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	781.2;
	3415	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3187.2;
	3415	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7932;
	3415	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4978.4;
	3415	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8831.1;
	3415	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1372.4;
	3416	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.7;
	3416	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	681;
	3416	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	827.4;
	3416	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3469.5;
	3416	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8618;
	3416	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5317.9;
	3416	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9321.6;
	3416	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1500.6;
	3417	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.2;
	3417	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	710;
	3417	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.4;
	3417	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3825.6;
	3417	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9382.8;
	3417	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5753.8;
	3417	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9901.2;
	3417	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1602.2;
	3418	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.4;
	3418	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.3;
	3418	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	902.4;
	3418	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4184;
	3418	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10148.9;
	3418	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6229.5;
	3418	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10440.5;
	3418	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1690.5;
	3419	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.5;
	3419	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	771.5;
	3419	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	939.9;
	3419	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4533;
	3419	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10860.6;
	3419	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6662.9;
	3419	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11092.7;
	3419	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1768.9;
	3420	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1024.1;
	3420	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	802.6;
	3420	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	983.8;
	3420	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4797.3;
	3420	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11390.3;
	3420	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7091.7;
	3420	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11805.8;
	3420	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1888.7;
	3421	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1054.9;
	3421	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	828.8;
	3421	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1025.8;
	3421	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4976.7;
	3421	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11886.9;
	3421	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7416.7;
	3421	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12512.4;
	3421	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1973.3;
	3422	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1078.6;
	3422	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	854.8;
	3422	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1058.3;
	3422	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5098.6;
	3422	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12253.7;
	3422	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7686.8;
	3422	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13044.2;
	3422	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2098;
	3423	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1090.6;
	3423	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	883.5;
	3423	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1084.1;
	3423	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5171.2;
	3423	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12760.5;
	3423	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7933.8;
	3423	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13402.4;
	3423	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2175.6;
	3424	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1100.5;
	3424	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	915.3;
	3424	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1103.3;
	3424	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5229.1;
	3424	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13339.2;
	3424	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8174.5;
	3424	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13591.8;
	3424	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2175.9;
	3425	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1104.5;
	3425	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	932;
	3425	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1111.5;
	3425	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5160.5;
	3425	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13619.8;
	3425	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8383.6;
	3425	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13507.9;
	3425	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2224.1;
	3426	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1095.5;
	3426	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	920;
	3426	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1096.8;
	3426	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5011.6;
	3426	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13560.3;
	3426	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8389.1;
	3426	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13207.5;
	3426	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2191.8;
	3427	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1064.8;
	3427	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	895.4;
	3427	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1065.8;
	3427	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4852.9;
	3427	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13224;
	3427	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8192.3;
	3427	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12749.1;
	3427	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2125.6;
	3428	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030;
	3428	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	895.9;
	3428	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1039.5;
	3428	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4839;
	3428	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13085.3;
	3428	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8119.1;
	3428	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12512.5;
	3428	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2080.6;
	3429	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1008.1;
	3429	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	876.6;
	3429	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1030.7;
	3429	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4751.5;
	3429	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12789;
	3429	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7878.9;
	3429	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12152.3;
	3429	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2029;
	3430	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	982.2;
	3430	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	830.3;
	3430	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	975.6;
	3430	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4431.7;
	3430	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11813.4;
	3430	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7184.3;
	3430	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11412.5;
	3430	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1880.6;
	3431	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.7;
	3431	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	758.5;
	3431	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	901.7;
	3431	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4021.6;
	3431	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10579.7;
	3431	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6354.2;
	3431	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10409.5;
	3431	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1677;
	3432	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927;
	3432	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	704;
	3432	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851.6;
	3432	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3692.4;
	3432	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9618.1;
	3432	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5699.1;
	3432	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9658.6;
	3432	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1529.9;
	3433	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.2;
	3433	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	678.9;
	3433	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	814.7;
	3433	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3486;
	3433	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9005.5;
	3433	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5255;
	3433	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9187.2;
	3433	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1431.1;
	3434	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.8;
	3434	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	656.7;
	3434	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	790.6;
	3434	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3350.6;
	3434	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8637.3;
	3434	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4977;
	3434	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8876.6;
	3434	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1374.2;
	3435	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893;
	3435	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	648.1;
	3435	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	777.2;
	3435	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3269.8;
	3435	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8458.8;
	3435	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4835;
	3435	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8749.9;
	3435	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1332.9;
	3436	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.8;
	3436	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	641.5;
	3436	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	775.5;
	3436	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3248.6;
	3436	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8494.9;
	3436	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4874;
	3436	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8857.1;
	3436	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1360;
	3437	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.6;
	3437	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	647.6;
	3437	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	800.5;
	3437	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3386.8;
	3437	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9001.9;
	3437	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5216.3;
	3437	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9363.3;
	3437	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1433.4;
	3438	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.4;
	3438	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	697;
	3438	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	862.7;
	3438	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3700.6;
	3438	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10005;
	3438	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5844.2;
	3438	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10025.6;
	3438	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1600;
	3439	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944;
	3439	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.4;
	3439	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	893.4;
	3439	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3807.6;
	3439	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10476.3;
	3439	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6080;
	3439	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10344.3;
	3439	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1666.3;
	3440	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961;
	3440	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	750.8;
	3440	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	903;
	3440	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3979.9;
	3440	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10558.8;
	3440	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6154.3;
	3440	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10804.3;
	3440	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1724.5;
	3441	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989.4;
	3441	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	764;
	3441	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	924.9;
	3441	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4238.1;
	3441	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10672.5;
	3441	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6350.3;
	3441	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11340.4;
	3441	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1791.1;
	3442	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1021;
	3442	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	793.6;
	3442	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	957;
	3442	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4501.8;
	3442	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11128.1;
	3442	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6594.4;
	3442	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11896.2;
	3442	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1852;
	3443	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1056.1;
	3443	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	825.9;
	3443	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1002.8;
	3443	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4751.5;
	3443	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11644.7;
	3443	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6882.2;
	3443	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12514.4;
	3443	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1919;
	3444	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1080.2;
	3444	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	855.9;
	3444	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1034.1;
	3444	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5009.1;
	3444	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12080.7;
	3444	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7207.8;
	3444	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13031.7;
	3444	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1991.2;
	3445	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1111.9;
	3445	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	884.1;
	3445	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1073.9;
	3445	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5211.3;
	3445	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12505.4;
	3445	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7516.1;
	3445	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13476.7;
	3445	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2071.2;
	3446	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1124.8;
	3446	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	898.1;
	3446	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1117;
	3446	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5334.2;
	3446	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12850.2;
	3446	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7818.3;
	3446	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13890.8;
	3446	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2149;
	3447	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1138.9;
	3447	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	908.7;
	3447	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1139.3;
	3447	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5397.9;
	3447	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12982.1;
	3447	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8102.1;
	3447	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13876.4;
	3447	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2172.4;
	3448	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1156.2;
	3448	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	922.2;
	3448	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1171.3;
	3448	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5388;
	3448	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13125.7;
	3448	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8360.2;
	3448	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13705.9;
	3448	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2231.7;
	3449	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1153;
	3449	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	921.5;
	3449	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1180.9;
	3449	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5260.1;
	3449	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13226.9;
	3449	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8522.6;
	3449	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13515;
	3449	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2222;
	3450	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1133.2;
	3450	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	911;
	3450	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1140.6;
	3450	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5072.3;
	3450	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13126;
	3450	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8375.5;
	3450	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13107;
	3450	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2167.3;
	3451	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1106.3;
	3451	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	895.4;
	3451	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1112;
	3451	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4897.9;
	3451	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13002.1;
	3451	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8108.1;
	3451	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12733.9;
	3451	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2128.1;
	3452	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1079.2;
	3452	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	892.3;
	3452	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1109.4;
	3452	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4910.1;
	3452	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13070.4;
	3452	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8067;
	3452	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12707.5;
	3452	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2124.4;
	3453	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1067.8;
	3453	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	880.5;
	3453	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1102.5;
	3453	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4812.9;
	3453	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12924.4;
	3453	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7796.4;
	3453	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12328.3;
	3453	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2059.4;
	3454	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1024;
	3454	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	845.3;
	3454	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1033.9;
	3454	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4489;
	3454	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11971.9;
	3454	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7170.3;
	3454	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11480.2;
	3454	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1894.9;
	3455	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	973.1;
	3455	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	783.3;
	3455	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	945.9;
	3455	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4085.9;
	3455	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10784.2;
	3455	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6370.3;
	3455	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10583;
	3455	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1690.7;
	3456	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.6;
	3456	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.9;
	3456	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884.5;
	3456	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3770.6;
	3456	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9816.4;
	3456	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5804.5;
	3456	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9973.1;
	3456	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1556.7;
	3457	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.1;
	3457	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	707.7;
	3457	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849.5;
	3457	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3562;
	3457	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9312.9;
	3457	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5479.8;
	3457	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9551.3;
	3457	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1457.4;
	3458	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.6;
	3458	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	693.7;
	3458	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	828.7;
	3458	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3425.3;
	3458	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8965.1;
	3458	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5251.1;
	3458	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9275;
	3458	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1430;
	3459	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.6;
	3459	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	688.5;
	3459	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	811.2;
	3459	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3350.1;
	3459	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8812;
	3459	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5134;
	3459	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9142.5;
	3459	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1324.5;
	3460	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.6;
	3460	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	678.4;
	3460	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	806.7;
	3460	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3333.1;
	3460	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8863.2;
	3460	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5141.1;
	3460	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9244.4;
	3460	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1406.5;
	3461	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.7;
	3461	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	691.9;
	3461	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824.9;
	3461	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3462.6;
	3461	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9377.3;
	3461	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5419.9;
	3461	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9766.1;
	3461	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1479.6;
	3462	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925.9;
	3462	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742.4;
	3462	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	883.6;
	3462	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3805.9;
	3462	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10390.6;
	3462	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6081;
	3462	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10445.7;
	3462	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1618.1;
	3463	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940;
	3463	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	771.6;
	3463	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	917;
	3463	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3903.7;
	3463	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10842;
	3463	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6322.4;
	3463	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10742;
	3463	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1696.5;
	3464	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.6;
	3464	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	808.9;
	3464	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	939.5;
	3464	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4046.1;
	3464	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10994.3;
	3464	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6491.2;
	3464	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11232.3;
	3464	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1760.3;
	3465	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	988.7;
	3465	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	842.9;
	3465	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	965.9;
	3465	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4240.5;
	3465	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11408.5;
	3465	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6771.5;
	3465	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11857.9;
	3465	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1845.5;
	3466	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1019.7;
	3466	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	892.7;
	3466	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	977.7;
	3466	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4462.6;
	3466	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11901.5;
	3466	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7138.5;
	3466	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12526.8;
	3466	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1956.5;
	3467	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1046.1;
	3467	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	936;
	3467	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1027;
	3467	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4686.7;
	3467	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12288.5;
	3467	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7505.6;
	3467	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13036.4;
	3467	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2040.1;
	3468	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1075;
	3468	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	959.9;
	3468	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1044;
	3468	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4869.9;
	3468	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12740.6;
	3468	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7742.3;
	3468	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13594.1;
	3468	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2126;
	3469	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1109;
	3469	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	989.8;
	3469	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1072.9;
	3469	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5083.2;
	3469	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13255.5;
	3469	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7895.8;
	3469	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14250.9;
	3469	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2206.7;
	3470	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1133.4;
	3470	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1011.2;
	3470	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1097;
	3470	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5233.9;
	3470	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13631.3;
	3470	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8109.1;
	3470	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14680;
	3470	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2279;
	3471	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1149.1;
	3471	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1017.2;
	3471	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1115.3;
	3471	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5390.5;
	3471	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13968.1;
	3471	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8257;
	3471	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14923.5;
	3471	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2397.8;
	3472	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1157.7;
	3472	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1028.3;
	3472	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1131.4;
	3472	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5496.6;
	3472	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14361.4;
	3472	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8446.3;
	3472	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14989.1;
	3472	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2450.2;
	3473	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1146.3;
	3473	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1022.3;
	3473	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1124.9;
	3473	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5449.1;
	3473	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14603.3;
	3473	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8558.7;
	3473	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14841.1;
	3473	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2436.7;
	3474	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1127.7;
	3474	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1007.7;
	3474	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1116.5;
	3474	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5273.2;
	3474	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14400.1;
	3474	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8479.7;
	3474	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14330.7;
	3474	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2382.8;
	3475	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1098.6;
	3475	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	982.9;
	3475	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1086.5;
	3475	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5062;
	3475	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14089.7;
	3475	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8338.6;
	3475	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13777.8;
	3475	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2306.8;
	3476	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1082.7;
	3476	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	969.3;
	3476	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1080.3;
	3476	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5051.4;
	3476	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14097.7;
	3476	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8297.8;
	3476	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13658.7;
	3476	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2285.8;
	3477	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1072.6;
	3477	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	949.3;
	3477	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1066.4;
	3477	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4959.5;
	3477	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13924;
	3477	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8048.3;
	3477	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13217.3;
	3477	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2205.5;
	3478	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1035.3;
	3478	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	890.6;
	3478	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1009.6;
	3478	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4640.8;
	3478	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12929.9;
	3478	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7415.6;
	3478	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12409.2;
	3478	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2033.9;
	3479	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	988.7;
	3479	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	834.1;
	3479	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	922.8;
	3479	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4232.1;
	3479	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11699;
	3479	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6624.5;
	3479	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11561.1;
	3479	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1872.2;
	3480	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.5;
	3480	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	775;
	3480	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	856.6;
	3480	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3902.6;
	3480	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10701.9;
	3480	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6048;
	3480	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10904.5;
	3480	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1741.2;
	3481	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.9;
	3481	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751;
	3481	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	836.2;
	3481	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3702.4;
	3481	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10110.3;
	3481	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5674.5;
	3481	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10495.1;
	3481	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1625.5;
	3482	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.1;
	3482	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735;
	3482	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	792.8;
	3482	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3569.5;
	3482	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9729.9;
	3482	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5471.8;
	3482	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10223.2;
	3482	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1561.7;
	3483	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.7;
	3483	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723.1;
	3483	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	805.1;
	3483	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3491.3;
	3483	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9447.8;
	3483	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5360.6;
	3483	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10091.6;
	3483	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1555.6;
	3484	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.2;
	3484	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	724.6;
	3484	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	806.7;
	3484	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3469;
	3484	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9510.9;
	3484	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5403.3;
	3484	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10204.3;
	3484	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1568.5;
	3485	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.3;
	3485	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	745.2;
	3485	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	831.8;
	3485	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3602.2;
	3485	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10076.9;
	3485	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5699.1;
	3485	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10691.2;
	3485	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1552.8;
	3486	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.2;
	3486	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	800.3;
	3486	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	901.7;
	3486	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3940.2;
	3486	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11154;
	3486	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6298.2;
	3486	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11271.4;
	3486	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1706;
	3487	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.2;
	3487	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	826.5;
	3487	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	942.5;
	3487	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4030.6;
	3487	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11596.4;
	3487	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6549.6;
	3487	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11545.2;
	3487	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1770.7;
	3488	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	983.2;
	3488	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	852;
	3488	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	965;
	3488	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4166.1;
	3488	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11868.7;
	3488	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6721.8;
	3488	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12047;
	3488	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1836.9;
	3489	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1015.5;
	3489	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	884;
	3489	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	996.7;
	3489	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4429.2;
	3489	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12256.4;
	3489	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6997.2;
	3489	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12761.2;
	3489	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1943.2;
	3490	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1045.2;
	3490	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	920;
	3490	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1031.7;
	3490	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4748.2;
	3490	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12714.1;
	3490	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7304.4;
	3490	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13568.8;
	3490	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2050.6;
	3491	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1075.1;
	3491	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	953.2;
	3491	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1078.1;
	3491	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4968.1;
	3491	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13280.8;
	3491	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7519.4;
	3491	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14196.5;
	3491	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2135.5;
	3492	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1098.1;
	3492	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	993;
	3492	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1126;
	3492	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5184.7;
	3492	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13621;
	3492	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7734.8;
	3492	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14801.6;
	3492	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2221.5;
	3493	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1132.1;
	3493	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1037.3;
	3493	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1169.1;
	3493	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5391.5;
	3493	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13877.2;
	3493	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7867.9;
	3493	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15383.3;
	3493	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2315.7;
	3494	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1159.2;
	3494	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1075.2;
	3494	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1200.9;
	3494	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5587;
	3494	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14197.4;
	3494	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8038.1;
	3494	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15790.2;
	3494	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2347.5;
	3495	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1179.4;
	3495	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1098.5;
	3495	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1234.3;
	3495	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5709.4;
	3495	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14409.1;
	3495	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8204.7;
	3495	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16009.1;
	3495	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2376.2;
	3496	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1197.3;
	3496	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1112.2;
	3496	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1263.2;
	3496	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5719.4;
	3496	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14635;
	3496	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8415.3;
	3496	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16024.6;
	3496	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2378.5;
	3497	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1192.6;
	3497	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1106.5;
	3497	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1261.3;
	3497	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5567.9;
	3497	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14729.8;
	3497	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8537.3;
	3497	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15578.9;
	3497	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2331.9;
	3498	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1179.8;
	3498	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1096.7;
	3498	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1239.2;
	3498	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5312.4;
	3498	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14622.1;
	3498	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8452.7;
	3498	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14898.6;
	3498	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2273;
	3499	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1154.7;
	3499	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1064.1;
	3499	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1202.9;
	3499	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5129.8;
	3499	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14373.2;
	3499	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8293;
	3499	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14396.5;
	3499	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2206.2;
	3500	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1127.6;
	3500	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1037.2;
	3500	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1177.3;
	3500	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5150.2;
	3500	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14360.9;
	3500	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8310.6;
	3500	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14263.9;
	3500	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2196.3;
	3501	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1113.9;
	3501	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1020.2;
	3501	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1158.8;
	3501	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5024.7;
	3501	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14213.4;
	3501	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8169.5;
	3501	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13858.9;
	3501	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2146.1;
	3502	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1072;
	3502	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	948.8;
	3502	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1085.5;
	3502	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4683.1;
	3502	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13196.5;
	3502	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7563.8;
	3502	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12977.2;
	3502	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1989.6;
	3503	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1023.7;
	3503	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	879.2;
	3503	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1000.9;
	3503	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4233.1;
	3503	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11968;
	3503	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6869.4;
	3503	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12090.4;
	3503	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1811.7;
	3504	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009.4;
	3504	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	822.8;
	3504	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	951.2;
	3504	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3965.7;
	3504	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10635.1;
	3504	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6312.7;
	3504	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11593.5;
	3504	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1660.6;
	3505	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	977.7;
	3505	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	787.7;
	3505	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	910;
	3505	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3719.6;
	3505	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9938.8;
	3505	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5929.5;
	3505	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11071.1;
	3505	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1562.6;
	3506	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.8;
	3506	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	764.2;
	3506	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.7;
	3506	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3569.2;
	3506	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9482.6;
	3506	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5710.3;
	3506	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10703;
	3506	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1508.4;
	3507	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.1;
	3507	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	766.4;
	3507	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.9;
	3507	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3506.1;
	3507	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9273.4;
	3507	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5621.3;
	3507	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10534.5;
	3507	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1493.3;
	3508	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.9;
	3508	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776.6;
	3508	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.1;
	3508	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3497.9;
	3508	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9356.7;
	3508	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5660.9;
	3508	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10642;
	3508	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1519.9;
	3509	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.6;
	3509	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	795.2;
	3509	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	863.7;
	3509	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3610;
	3509	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9966.4;
	3509	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5998.7;
	3509	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11087.8;
	3509	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1612.8;
	3510	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.2;
	3510	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	845.5;
	3510	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	919.5;
	3510	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3937.8;
	3510	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11030.8;
	3510	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6620.5;
	3510	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11669.8;
	3510	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1791.9;
	3511	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	977.9;
	3511	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	872.8;
	3511	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	949.6;
	3511	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4041.7;
	3511	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11509.6;
	3511	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6824.2;
	3511	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11881.1;
	3511	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1856.4;
	3512	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	996.3;
	3512	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	877.5;
	3512	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	961.4;
	3512	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4149.2;
	3512	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11712.3;
	3512	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6945.1;
	3512	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12133.8;
	3512	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1903.7;
	3513	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1033;
	3513	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	900.6;
	3513	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	990.1;
	3513	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4409.9;
	3513	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11929.7;
	3513	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7155.3;
	3513	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12579.3;
	3513	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1930.5;
	3514	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1061;
	3514	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	929;
	3514	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1037.6;
	3514	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4692.8;
	3514	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12085.2;
	3514	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7416.5;
	3514	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13030.5;
	3514	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1950.5;
	3515	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1088.2;
	3515	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	946;
	3515	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1083.6;
	3515	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4978;
	3515	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12168.3;
	3515	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7643;
	3515	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13346.7;
	3515	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1963.6;
	3516	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1110;
	3516	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	958.1;
	3516	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1135.4;
	3516	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5220.7;
	3516	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12167.6;
	3516	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7893.9;
	3516	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13571.3;
	3516	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1956.9;
	3517	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1135.9;
	3517	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	970.9;
	3517	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1193;
	3517	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5456;
	3517	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12214.8;
	3517	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8291.7;
	3517	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13523.7;
	3517	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1867.2;
	3518	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1161.4;
	3518	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	977.1;
	3518	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1250.7;
	3518	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5643.2;
	3518	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12211.2;
	3518	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8586.4;
	3518	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13460.3;
	3518	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1807.2;
	3519	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1176.8;
	3519	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	994;
	3519	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1298.7;
	3519	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5773;
	3519	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12198.4;
	3519	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8899.3;
	3519	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13339;
	3519	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1793.7;
	3520	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1159.8;
	3520	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	993.5;
	3520	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1310.7;
	3520	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5820.8;
	3520	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12370.4;
	3520	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9135.8;
	3520	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13105;
	3520	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1802.8;
	3521	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1148.8;
	3521	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	982.6;
	3521	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1271;
	3521	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5679.7;
	3521	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12393.5;
	3521	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9082.2;
	3521	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12929.3;
	3521	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1837.7;
	3522	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1141;
	3522	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	962.8;
	3522	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1213;
	3522	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5421;
	3522	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12420.6;
	3522	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8773.4;
	3522	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12654.2;
	3522	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1835.1;
	3523	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1116.8;
	3523	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	928.3;
	3523	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1145.9;
	3523	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5178.6;
	3523	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12093.6;
	3523	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8416.3;
	3523	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12367.2;
	3523	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1825.5;
	3524	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1090.7;
	3524	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	910.5;
	3524	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1095.9;
	3524	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5187.8;
	3524	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11993.6;
	3524	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8327.5;
	3524	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12199;
	3524	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1803.8;
	3525	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1073.4;
	3525	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	893.5;
	3525	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1066.7;
	3525	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5074.5;
	3525	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11852.5;
	3525	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8104;
	3525	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11801.8;
	3525	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1755.7;
	3526	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1035.3;
	3526	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	829.2;
	3526	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	999.5;
	3526	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4761.6;
	3526	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11010.1;
	3526	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7422.1;
	3526	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11104.7;
	3526	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1644;
	3527	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.7;
	3527	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	760.6;
	3527	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	912.7;
	3527	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4352.9;
	3527	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9760.1;
	3527	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6624.3;
	3527	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10272.6;
	3527	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1502.8;
	3528	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.5;
	3528	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720.8;
	3528	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824.7;
	3528	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4015;
	3528	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8893.2;
	3528	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5855.8;
	3528	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9613;
	3528	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1408.4;
	3529	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.4;
	3529	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	684.2;
	3529	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	790;
	3529	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3805.4;
	3529	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8308.6;
	3529	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5459.7;
	3529	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9175;
	3529	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1332.5;
	3530	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.5;
	3530	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	665.6;
	3530	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	766.8;
	3530	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3666.5;
	3530	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7977.2;
	3530	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5203.2;
	3530	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8968.7;
	3530	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1288.9;
	3531	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.5;
	3531	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	657.6;
	3531	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	754.7;
	3531	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3562.7;
	3531	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7818.5;
	3531	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5040.2;
	3531	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8904.4;
	3531	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1283.3;
	3532	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	862.8;
	3532	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	655.4;
	3532	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	751.6;
	3532	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3534.3;
	3532	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7926.1;
	3532	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5001.2;
	3532	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9058.8;
	3532	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1285.5;
	3533	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	862.7;
	3533	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	672.5;
	3533	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	772.4;
	3533	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3628.6;
	3533	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8362;
	3533	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5271.1;
	3533	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9439.1;
	3533	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1348.1;
	3534	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.9;
	3534	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	712.4;
	3534	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	822.9;
	3534	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3903.6;
	3534	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9158.6;
	3534	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5886.8;
	3534	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9872.8;
	3534	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1456.8;
	3535	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.1;
	3535	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	739.5;
	3535	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	853.1;
	3535	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3990.5;
	3535	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9643.1;
	3535	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6167.1;
	3535	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10104.2;
	3535	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1554.4;
	3536	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909;
	3536	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753.9;
	3536	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882;
	3536	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4180;
	3536	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9925.1;
	3536	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6385.8;
	3536	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10600.2;
	3536	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1602.3;
	3537	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.2;
	3537	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	774.5;
	3537	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	924.1;
	3537	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4518.1;
	3537	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10292.1;
	3537	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6697.9;
	3537	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11308.8;
	3537	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1654.2;
	3538	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	971.5;
	3538	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	803.9;
	3538	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	969.8;
	3538	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4876.8;
	3538	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10579.3;
	3538	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6942.5;
	3538	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12072.8;
	3538	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1693.9;
	3539	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	996.8;
	3539	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	810.4;
	3539	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1011.5;
	3539	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5174.7;
	3539	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10738.6;
	3539	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7125;
	3539	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12729.4;
	3539	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1693.9;
	3540	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013;
	3540	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	823.3;
	3540	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1045.3;
	3540	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5408.2;
	3540	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10690.3;
	3540	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7363.8;
	3540	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13004.9;
	3540	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1698.2;
	3541	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030.5;
	3541	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	850.9;
	3541	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1091.9;
	3541	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5624.4;
	3541	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10978.1;
	3541	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7650.7;
	3541	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12951.5;
	3541	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1690;
	3542	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1049.8;
	3542	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	880.2;
	3542	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1129.5;
	3542	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5791.1;
	3542	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11369;
	3542	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7898;
	3542	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12701.1;
	3542	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1632.4;
	3543	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1060.1;
	3543	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	904.2;
	3543	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1155.2;
	3543	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5899.3;
	3543	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11783.8;
	3543	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8131.3;
	3543	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12242.8;
	3543	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1519.9;
	3544	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1062.5;
	3544	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	911.8;
	3544	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1169.1;
	3544	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5821.5;
	3544	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12077;
	3544	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8262.6;
	3544	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11689.5;
	3544	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1516.6;
	3545	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1061.3;
	3545	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	908.5;
	3545	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1172.2;
	3545	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5586;
	3545	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12251.3;
	3545	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8201.6;
	3545	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11015.7;
	3545	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1493.2;
	3546	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1048.9;
	3546	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	897.8;
	3546	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1153.1;
	3546	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5398.3;
	3546	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12100.5;
	3546	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7936.5;
	3546	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10470.7;
	3546	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1452;
	3547	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1024.9;
	3547	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	872.9;
	3547	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1112.7;
	3547	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5142;
	3547	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11625.3;
	3547	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7552.3;
	3547	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10143.3;
	3547	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1519.4;
	3548	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986.7;
	3548	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	839.3;
	3548	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1058.7;
	3548	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4966.3;
	3548	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11143.9;
	3548	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7331.2;
	3548	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9924.2;
	3548	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1545.1;
	3549	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.6;
	3549	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	829.8;
	3549	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1028.4;
	3549	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4812.7;
	3549	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10947;
	3549	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7088.1;
	3549	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9717.5;
	3549	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1534.2;
	3550	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.3;
	3550	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	790.8;
	3550	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	958.2;
	3550	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4493.3;
	3550	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10221;
	3550	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6547.3;
	3550	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9327.2;
	3550	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1391.3;
	3551	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.3;
	3551	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.6;
	3551	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	879.5;
	3551	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4157;
	3551	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9391.8;
	3551	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5889.1;
	3551	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8821.3;
	3551	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1325.7;
	3552	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.9;
	3552	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	689.3;
	3552	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	823.3;
	3552	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3841.9;
	3552	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8623;
	3552	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5238;
	3552	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8296.9;
	3552	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1290;
	3553	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.5;
	3553	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	656.7;
	3553	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	778.9;
	3553	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3612.9;
	3553	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8067.6;
	3553	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4863.5;
	3553	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7963.4;
	3553	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1231;
	3554	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	851.2;
	3554	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	636.4;
	3554	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	747.3;
	3554	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3463.9;
	3554	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7683.2;
	3554	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4663.1;
	3554	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7781.5;
	3554	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1200.4;
	3555	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	840.5;
	3555	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	625.1;
	3555	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	729;
	3555	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3389.5;
	3555	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7486.9;
	3555	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4551.3;
	3555	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7655.8;
	3555	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1189.1;
	3556	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	835.2;
	3556	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	622.3;
	3556	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	721.3;
	3556	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3374.6;
	3556	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7421.3;
	3556	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4476.6;
	3556	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7660.4;
	3556	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1181.4;
	3557	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	838.4;
	3557	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	629.4;
	3557	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	725.9;
	3557	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3436.4;
	3557	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7579.2;
	3557	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4527.6;
	3557	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7763.3;
	3557	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1171.3;
	3558	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	843.8;
	3558	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	631.7;
	3558	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	742.4;
	3558	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3513.5;
	3558	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7799.3;
	3558	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4686.7;
	3558	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7895.5;
	3558	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1136.1;
	3559	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	834.4;
	3559	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	637.7;
	3559	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	761.4;
	3559	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3571.1;
	3559	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8083.2;
	3559	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4881.9;
	3559	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8191.9;
	3559	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1263.4;
	3560	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	858.8;
	3560	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	678;
	3560	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	811.2;
	3560	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3865.7;
	3560	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8717;
	3560	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5298.1;
	3560	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8987.8;
	3560	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1354;
	3561	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.9;
	3561	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720.2;
	3561	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.2;
	3561	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4215.6;
	3561	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9473.1;
	3561	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5762.6;
	3561	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9822.8;
	3561	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1428.4;
	3562	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.6;
	3562	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	768.2;
	3562	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	905.5;
	3562	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4543.4;
	3562	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10117.3;
	3562	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6262.5;
	3562	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10728.8;
	3562	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1538.9;
	3563	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	971.4;
	3563	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	814.9;
	3563	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	961.1;
	3563	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4841.3;
	3563	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10601.4;
	3563	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6811.6;
	3563	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11646.9;
	3563	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1684;
	3564	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1004.2;
	3564	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	861.4;
	3564	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1023.1;
	3564	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5066.9;
	3564	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11098;
	3564	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7384.9;
	3564	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12459.3;
	3564	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1784;
	3565	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1033.2;
	3565	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	905.5;
	3565	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1078.3;
	3565	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5250.9;
	3565	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11672.2;
	3565	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7909.8;
	3565	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13111.9;
	3565	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1862.6;
	3566	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1062.6;
	3566	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	944.9;
	3566	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1129.9;
	3566	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5358.5;
	3566	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12416.4;
	3566	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8366.4;
	3566	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13656.1;
	3566	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1948.9;
	3567	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1086.9;
	3567	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	977.5;
	3567	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1174.3;
	3567	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5397.4;
	3567	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13205.7;
	3567	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8729.7;
	3567	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14005.4;
	3567	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2041.4;
	3568	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1099.9;
	3568	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1010.3;
	3568	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1201.6;
	3568	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5431.6;
	3568	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13902.7;
	3568	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8972.7;
	3568	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14158.9;
	3568	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2097.3;
	3569	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1101.8;
	3569	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1022.3;
	3569	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1204.4;
	3569	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5378.2;
	3569	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14244;
	3569	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9047.1;
	3569	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14104.3;
	3569	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2140;
	3570	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1089.4;
	3570	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1013.6;
	3570	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1174.4;
	3570	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5219.8;
	3570	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14100.3;
	3570	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8836;
	3570	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13697.4;
	3570	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2143;
	3571	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1066.8;
	3571	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	979.8;
	3571	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1124.5;
	3571	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4971.1;
	3571	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13577.8;
	3571	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8414.9;
	3571	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13126.3;
	3571	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2078.1;
	3572	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1039.2;
	3572	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	941;
	3572	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1083.3;
	3572	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4851.6;
	3572	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13127.6;
	3572	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8176.6;
	3572	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12907.3;
	3572	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2031.9;
	3573	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1032.3;
	3573	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	920.9;
	3573	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1050.4;
	3573	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4741.3;
	3573	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12812.9;
	3573	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7905.3;
	3573	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12491.8;
	3573	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1952.7;
	3574	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000.4;
	3574	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	868.8;
	3574	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	986;
	3574	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4505.3;
	3574	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11989.2;
	3574	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7343.2;
	3574	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11772.6;
	3574	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1834.3;
	3575	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.7;
	3575	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	797.2;
	3575	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	922.4;
	3575	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4213.6;
	3575	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11029.3;
	3575	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6729.9;
	3575	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11000.9;
	3575	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1696.9;
	3576	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.5;
	3576	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	739.7;
	3576	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	780.7;
	3576	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3944.3;
	3576	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10198.8;
	3576	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6150.9;
	3576	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10160.2;
	3576	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1558.1;
	3577	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.9;
	3577	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	704.9;
	3577	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	741.5;
	3577	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3714;
	3577	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9499.6;
	3577	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5764.1;
	3577	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9502;
	3577	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1463;
	3578	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.5;
	3578	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	683.4;
	3578	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	723.6;
	3578	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3538.9;
	3578	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8986.2;
	3578	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5447.5;
	3578	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9106.5;
	3578	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1382;
	3579	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.5;
	3579	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	665.6;
	3579	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	712.1;
	3579	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3419.3;
	3579	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8650.1;
	3579	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5203.9;
	3579	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8851;
	3579	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1367.8;
	3580	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.8;
	3580	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	657.6;
	3580	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	700.9;
	3580	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3353.4;
	3580	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8418.8;
	3580	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4932.2;
	3580	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8701.3;
	3580	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1332.9;
	3581	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.8;
	3581	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	652.2;
	3581	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	706;
	3581	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3316.8;
	3581	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8189.5;
	3581	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4798.7;
	3581	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8592.8;
	3581	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1312.3;
	3582	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.7;
	3582	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	649.8;
	3582	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	710.1;
	3582	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3294.7;
	3582	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8035.5;
	3582	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4781.1;
	3582	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8541.3;
	3582	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1330.6;
	3583	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	865.2;
	3583	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	662.4;
	3583	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	722;
	3583	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3291.4;
	3583	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8038.3;
	3583	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4823.7;
	3583	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8627.1;
	3583	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1333;
	3584	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.1;
	3584	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	701.2;
	3584	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	776.3;
	3584	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3528.7;
	3584	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8533.2;
	3584	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5067;
	3584	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9237.5;
	3584	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1480.7;
	3585	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.1;
	3585	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746.7;
	3585	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	830.5;
	3585	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3876.7;
	3585	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9241.2;
	3585	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5505.4;
	3585	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10152.1;
	3585	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1582;
	3586	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.8;
	3586	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	781.6;
	3586	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	876.2;
	3586	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4216.6;
	3586	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10042.5;
	3586	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6011.9;
	3586	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11115.9;
	3586	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1667.3;
	3587	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.2;
	3587	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	816.4;
	3587	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	925.6;
	3587	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4568;
	3587	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10844.9;
	3587	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6523.7;
	3587	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12000.7;
	3587	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1774.6;
	3588	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.1;
	3588	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	849.9;
	3588	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	978.5;
	3588	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4815.2;
	3588	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11688.3;
	3588	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7020.9;
	3588	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12617.5;
	3588	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1910.6;
	3589	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986.9;
	3589	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	885.1;
	3589	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1035.5;
	3589	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5011.8;
	3589	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12571.6;
	3589	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7449.1;
	3589	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13106.1;
	3589	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2000.6;
	3590	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	996;
	3590	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	924;
	3590	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1085.3;
	3590	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5152.1;
	3590	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13236.2;
	3590	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7820.5;
	3590	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13593.4;
	3590	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2059.9;
	3591	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1002.8;
	3591	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	958.3;
	3591	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1125;
	3591	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5205.4;
	3591	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13666.4;
	3591	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8135.9;
	3591	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13930;
	3591	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2143.1;
	3592	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1011.5;
	3592	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	983.2;
	3592	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1151.9;
	3592	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5209.5;
	3592	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13998.7;
	3592	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8357.2;
	3592	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14101.1;
	3592	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2166;
	3593	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1008.5;
	3593	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	991.6;
	3593	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1163;
	3593	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5099.2;
	3593	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13987.7;
	3593	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8403.6;
	3593	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14041.8;
	3593	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2116.8;
	3594	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990.9;
	3594	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	960.6;
	3594	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1151;
	3594	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4898.3;
	3594	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13535.8;
	3594	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8251;
	3594	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13592.1;
	3594	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2118.4;
	3595	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.2;
	3595	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	917.9;
	3595	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1111.4;
	3595	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4693.4;
	3595	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12739.9;
	3595	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7970.5;
	3595	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12940.1;
	3595	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2008.8;
	3596	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.5;
	3596	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	895.6;
	3596	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1069.8;
	3596	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4643.9;
	3596	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12241.3;
	3596	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7672;
	3596	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12503.9;
	3596	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2024.8;
	3597	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.9;
	3597	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	858.9;
	3597	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1042.6;
	3597	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4578.7;
	3597	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11916;
	3597	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7490;
	3597	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12146.8;
	3597	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1958.2;
	3598	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.1;
	3598	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	788.9;
	3598	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	973.7;
	3598	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4337.6;
	3598	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11109.8;
	3598	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7007.6;
	3598	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11443.5;
	3598	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1818.2;
	3599	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.7;
	3599	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.2;
	3599	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	896.8;
	3599	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4023.4;
	3599	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10156.7;
	3599	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6424.6;
	3599	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10639.8;
	3599	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1690.5;
	3600	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.8;
	3600	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	684.2;
	3600	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	841.5;
	3600	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3747.4;
	3600	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9262.9;
	3600	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5882.6;
	3600	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9861.5;
	3600	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1572.6;
	3601	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	865.7;
	3601	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	655.9;
	3601	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	785.2;
	3601	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3524.5;
	3601	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8587.2;
	3601	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5493.4;
	3601	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9278.5;
	3601	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1468.1;
	3602	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	853.4;
	3602	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	633;
	3602	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	754;
	3602	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3380.6;
	3602	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8144.5;
	3602	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5221.9;
	3602	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8862.6;
	3602	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1385.8;
	3603	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	843.8;
	3603	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	620.9;
	3603	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	735.6;
	3603	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3293.5;
	3603	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7854.3;
	3603	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5014.1;
	3603	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8617.9;
	3603	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1340.7;
	3604	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	840.1;
	3604	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	618;
	3604	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	727.2;
	3604	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3262.4;
	3604	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7686.1;
	3604	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4948.5;
	3604	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8509.4;
	3604	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1346.8;
	3605	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	839.1;
	3605	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	630;
	3605	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	736.7;
	3605	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3327.4;
	3605	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7725.9;
	3605	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5010.3;
	3605	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8526.9;
	3605	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1359;
	3606	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	846.5;
	3606	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	619.4;
	3606	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	752.1;
	3606	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3451.6;
	3606	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7852.2;
	3606	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5076.5;
	3606	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8511;
	3606	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1384.8;
	3607	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	842.7;
	3607	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	624.1;
	3607	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	759.5;
	3607	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3503.6;
	3607	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7936.2;
	3607	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4927.3;
	3607	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8619.3;
	3607	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1396.9;
	3608	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	856.3;
	3608	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	664.8;
	3608	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	791.6;
	3608	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3739;
	3608	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8393.1;
	3608	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5020.1;
	3608	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9304.8;
	3608	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1451.1;
	3609	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.2;
	3609	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	712.2;
	3609	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838.3;
	3609	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4065.3;
	3609	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9115;
	3609	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5282.4;
	3609	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10041.9;
	3609	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1573.2;
	3610	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.7;
	3610	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746.9;
	3610	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	890.8;
	3610	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4350.3;
	3610	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9907.4;
	3610	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5667.1;
	3610	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10747.5;
	3610	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1660.4;
	3611	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910;
	3611	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	784.6;
	3611	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	934.1;
	3611	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4521.6;
	3611	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10655.5;
	3611	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6212.6;
	3611	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11061.6;
	3611	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1773.4;
	3612	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.8;
	3612	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	824.3;
	3612	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	979.5;
	3612	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4685.1;
	3612	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11285.4;
	3612	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6749.7;
	3612	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11132.3;
	3612	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1853.6;
	3613	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	977.4;
	3613	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	861.8;
	3613	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1035.6;
	3613	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4894.1;
	3613	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12027.4;
	3613	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7255.2;
	3613	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11481.3;
	3613	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1960;
	3614	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009.6;
	3614	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	901.8;
	3614	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1083.5;
	3614	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4993.8;
	3614	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12754.1;
	3614	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7720.7;
	3614	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12056.1;
	3614	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2025.7;
	3615	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1038.3;
	3615	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	935.2;
	3615	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1126;
	3615	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5052.6;
	3615	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13384.3;
	3615	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8146;
	3615	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12738;
	3615	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2072.6;
	3616	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1060.8;
	3616	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	963;
	3616	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1157;
	3616	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5099.2;
	3616	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13868;
	3616	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8501;
	3616	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13291.7;
	3616	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2095.3;
	3617	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1063.7;
	3617	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	977.2;
	3617	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1164.2;
	3617	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5080.6;
	3617	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14122.2;
	3617	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8724.1;
	3617	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13485.3;
	3617	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2123.1;
	3618	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1033.6;
	3618	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	965.2;
	3618	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1141.3;
	3618	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4921.9;
	3618	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13986.4;
	3618	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8633;
	3618	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13229.5;
	3618	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2163.9;
	3619	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1001.4;
	3619	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	937.8;
	3619	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1096.2;
	3619	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4735.8;
	3619	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13560.9;
	3619	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8289;
	3619	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12802.8;
	3619	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2106.6;
	3620	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.8;
	3620	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	903.4;
	3620	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1059.5;
	3620	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4748.4;
	3620	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13122.1;
	3620	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8080.8;
	3620	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12583.1;
	3620	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2057.1;
	3621	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.4;
	3621	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	880.6;
	3621	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1037.8;
	3621	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4686.3;
	3621	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12776.6;
	3621	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7803.3;
	3621	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12217.1;
	3621	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2000.8;
	3622	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.3;
	3622	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	813;
	3622	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	973.7;
	3622	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4393.6;
	3622	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11679.6;
	3622	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7121.1;
	3622	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11368;
	3622	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1847.5;
	3623	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.8;
	3623	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.7;
	3623	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	893.4;
	3623	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4014.1;
	3623	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10395.8;
	3623	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6378.1;
	3623	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10386.4;
	3623	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1678.3;
	3624	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	844.5;
	3624	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	706;
	3624	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	837;
	3624	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3681.8;
	3624	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9436;
	3624	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5787.5;
	3624	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9601.4;
	3624	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1535.1;
	3625	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	831.4;
	3625	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	674.9;
	3625	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	795.2;
	3625	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3489.3;
	3625	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8837.8;
	3625	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5411.5;
	3625	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9077;
	3625	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1431.8;
	3626	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	825.4;
	3626	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	654.4;
	3626	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	768.7;
	3626	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3354.7;
	3626	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8432;
	3626	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5171.4;
	3626	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8713.1;
	3626	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1298.4;
	3627	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	821.4;
	3627	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	639.3;
	3627	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	756.8;
	3627	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3278.6;
	3627	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8238.8;
	3627	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5067.8;
	3627	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8537.1;
	3627	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1308.2;
	3628	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	819.7;
	3628	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	644.5;
	3628	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	755.7;
	3628	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3256.9;
	3628	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8336.3;
	3628	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5107.6;
	3628	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8576;
	3628	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1326.8;
	3629	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	830.6;
	3629	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	668.2;
	3629	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	777.3;
	3629	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3358.7;
	3629	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8911.3;
	3629	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5435.3;
	3629	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8986.4;
	3629	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1337.8;
	3630	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855;
	3630	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	710.6;
	3630	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	827.1;
	3630	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3643.9;
	3630	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9884.2;
	3630	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6053.4;
	3630	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9546.5;
	3630	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1523.1;
	3631	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.7;
	3631	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.9;
	3631	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	863.5;
	3631	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3791.2;
	3631	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10492.6;
	3631	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6321.8;
	3631	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10009.9;
	3631	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1606.6;
	3632	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.8;
	3632	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	786.1;
	3632	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	904.5;
	3632	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4032.6;
	3632	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11074.9;
	3632	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6474.3;
	3632	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10766.3;
	3632	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1730.9;
	3633	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.4;
	3633	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	822.1;
	3633	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	954.4;
	3633	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4344.2;
	3633	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11788.5;
	3633	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6733.5;
	3633	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11588;
	3633	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1859.5;
	3634	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.8;
	3634	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	863.6;
	3634	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	982.4;
	3634	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4668.3;
	3634	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12636.4;
	3634	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7055.3;
	3634	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12459;
	3634	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2001.3;
	3635	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	973.6;
	3635	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	897.3;
	3635	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1051.5;
	3635	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4942.7;
	3635	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13567;
	3635	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7456.5;
	3635	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13108.5;
	3635	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2107;
	3636	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	983.7;
	3636	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	928.1;
	3636	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1075.2;
	3636	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5230.4;
	3636	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14274.6;
	3636	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7795.6;
	3636	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13610.4;
	3636	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2241.8;
	3637	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000;
	3637	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	953.4;
	3637	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1097.3;
	3637	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5463.3;
	3637	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14377.2;
	3637	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7819.2;
	3637	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14209;
	3637	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2358.9;
	3638	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1023.2;
	3638	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	947.9;
	3638	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1100.8;
	3638	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5601.2;
	3638	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13926.9;
	3638	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7475.8;
	3638	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14603.8;
	3638	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2399.7;
	3639	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1049.4;
	3639	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	921.3;
	3639	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1083.8;
	3639	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5650.6;
	3639	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13285.6;
	3639	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7047.9;
	3639	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14718.7;
	3639	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2392.1;
	3640	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1069.8;
	3640	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	892.2;
	3640	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1066.3;
	3640	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5568.7;
	3640	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12758.2;
	3640	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6740.4;
	3640	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14556.8;
	3640	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2287.1;
	3641	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1068;
	3641	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	866.8;
	3641	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1041.3;
	3641	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5345.6;
	3641	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12198.8;
	3641	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6590;
	3641	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14023.7;
	3641	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2271.4;
	3642	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1041.9;
	3642	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	845.8;
	3642	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1004.2;
	3642	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5078.5;
	3642	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11699.8;
	3642	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6614.5;
	3642	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13301.2;
	3642	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2140.8;
	3643	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1011.1;
	3643	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	822.2;
	3643	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	956.2;
	3643	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4833.4;
	3643	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11324.2;
	3643	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6549.7;
	3643	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12927.1;
	3643	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2070.8;
	3644	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	994.7;
	3644	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	812.2;
	3644	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	942.5;
	3644	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4715.9;
	3644	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11230.2;
	3644	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6623.1;
	3644	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12819.6;
	3644	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2029;
	3645	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	984.3;
	3645	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	801;
	3645	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	941;
	3645	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4588;
	3645	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10970.4;
	3645	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6554.1;
	3645	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12297.7;
	3645	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1915.1;
	3646	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.9;
	3646	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	758.1;
	3646	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	898.8;
	3646	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4269.5;
	3646	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10204.6;
	3646	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6062.5;
	3646	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11481.7;
	3646	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1763.1;
	3647	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883;
	3647	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	703.1;
	3647	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	828.5;
	3647	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3789.3;
	3647	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9194.1;
	3647	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5436.8;
	3647	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10630.1;
	3647	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1589.8;
	3648	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	856.2;
	3648	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	667.5;
	3648	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	798.5;
	3648	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3256.2;
	3648	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8487.9;
	3648	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4916.3;
	3648	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9944.8;
	3648	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1427.9;
	3649	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	840.3;
	3649	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	648.4;
	3649	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	765.4;
	3649	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3043.1;
	3649	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8041.8;
	3649	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4605.5;
	3649	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9494.1;
	3649	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1316.8;
	3650	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	828;
	3650	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	632.3;
	3650	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	746.7;
	3650	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2899.6;
	3650	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7778.8;
	3650	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4422.2;
	3650	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9163.5;
	3650	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1261.6;
	3651	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	822.6;
	3651	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	627.5;
	3651	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	736.2;
	3651	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2814.4;
	3651	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7678.6;
	3651	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4327.7;
	3651	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8977.3;
	3651	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1237.2;
	3652	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	823.3;
	3652	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	628;
	3652	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	739.7;
	3652	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2786.5;
	3652	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7793;
	3652	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4362.2;
	3652	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9043.9;
	3652	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1314.8;
	3653	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	828.9;
	3653	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	640.4;
	3653	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	762;
	3653	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2887.6;
	3653	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8327.9;
	3653	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4667.8;
	3653	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9445.7;
	3653	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1348.5;
	3654	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	851.5;
	3654	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	676.5;
	3654	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	818.3;
	3654	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3136.8;
	3654	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9273.8;
	3654	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5252.2;
	3654	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9965;
	3654	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1506.9;
	3655	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867.6;
	3655	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	708.7;
	3655	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.6;
	3655	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3235.1;
	3655	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9822.6;
	3655	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5526.9;
	3655	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10251.9;
	3655	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1574.9;
	3656	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.9;
	3656	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.4;
	3656	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	886.7;
	3656	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3341.2;
	3656	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10095.1;
	3656	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5773.4;
	3656	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10768.1;
	3656	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1648.1;
	3657	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.4;
	3657	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	754;
	3657	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	918.7;
	3657	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3546.1;
	3657	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10470.5;
	3657	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6146.1;
	3657	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11518.5;
	3657	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1741.5;
	3658	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.8;
	3658	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	785.7;
	3658	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	943.8;
	3658	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3826;
	3658	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10953.4;
	3658	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6604.2;
	3658	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12159.1;
	3658	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1827.3;
	3659	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.3;
	3659	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	810.3;
	3659	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	974.8;
	3659	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4117.6;
	3659	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11335.5;
	3659	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7061.9;
	3659	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12618.1;
	3659	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1871.4;
	3660	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	977.5;
	3660	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	835.5;
	3660	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1015;
	3660	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4383.1;
	3660	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11677.6;
	3660	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7475.9;
	3660	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12947.6;
	3660	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1906.4;
	3661	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	997.2;
	3661	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	864.2;
	3661	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1059.2;
	3661	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4682.6;
	3661	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12141.9;
	3661	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7925.9;
	3661	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13337.1;
	3661	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1998.8;
	3662	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1019.9;
	3662	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	888.4;
	3662	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1097.1;
	3662	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4927.8;
	3662	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12588.8;
	3662	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8225.6;
	3662	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13707.6;
	3662	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2052.8;
	3663	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1037.5;
	3663	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	907.9;
	3663	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1118.8;
	3663	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5115.4;
	3663	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12943.8;
	3663	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8490.9;
	3663	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13909.2;
	3663	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2076;
	3664	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1048.6;
	3664	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	919.6;
	3664	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1122.6;
	3664	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5227.8;
	3664	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13258.2;
	3664	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8690;
	3664	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13352.8;
	3664	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2066.3;
	3665	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1046.5;
	3665	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	907.9;
	3665	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1096.1;
	3665	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5189.6;
	3665	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13341;
	3665	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8731.3;
	3665	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12833.5;
	3665	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2078;
	3666	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1032.6;
	3666	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	901.1;
	3666	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1079.7;
	3666	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5053.9;
	3666	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13256.6;
	3666	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8514.2;
	3666	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12309.6;
	3666	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2002.2;
	3667	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1004.9;
	3667	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	881;
	3667	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1045.1;
	3667	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4840.8;
	3667	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12860.4;
	3667	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8112.8;
	3667	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11955.8;
	3667	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1905.3;
	3668	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.8;
	3668	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	868.7;
	3668	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1016.6;
	3668	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4747.1;
	3668	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12618.4;
	3668	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7789.9;
	3668	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11875.6;
	3668	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1932.9;
	3669	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.4;
	3669	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	859.2;
	3669	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	997.7;
	3669	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4625.3;
	3669	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12380.5;
	3669	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7485.9;
	3669	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11494;
	3669	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1883.4;
	3670	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.1;
	3670	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	806.3;
	3670	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	934.8;
	3670	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4300.8;
	3670	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11410.3;
	3670	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6839.7;
	3670	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10746.9;
	3670	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1702.7;
	3671	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.8;
	3671	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744.7;
	3671	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	858.1;
	3671	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3896;
	3671	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10209.1;
	3671	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6110.6;
	3671	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9916.5;
	3671	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1568.2;
	3672	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.8;
	3672	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	704.5;
	3672	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	821.6;
	3672	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3577.9;
	3672	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9312;
	3672	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5455.7;
	3672	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9297.3;
	3672	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1475.7;
	3673	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	835.9;
	3673	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	679.3;
	3673	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	791.9;
	3673	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3313.4;
	3673	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8755;
	3673	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5115.6;
	3673	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8893.9;
	3673	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1412.5;
	3674	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	824.2;
	3674	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	663.4;
	3674	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	769.7;
	3674	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3143;
	3674	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8401.4;
	3674	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4888.3;
	3674	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8645.1;
	3674	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1364.7;
	3675	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	824.5;
	3675	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	652;
	3675	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	750.8;
	3675	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3022.2;
	3675	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8247;
	3675	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4718.2;
	3675	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8541.6;
	3675	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1366;
	3676	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	823.7;
	3676	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	653.6;
	3676	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	753.1;
	3676	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2975.6;
	3676	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8328.3;
	3676	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4692.6;
	3676	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8701.2;
	3676	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1364.1;
	3677	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	835.1;
	3677	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	674;
	3677	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	773;
	3677	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3066.9;
	3677	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8839.3;
	3677	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4954.3;
	3677	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9215.5;
	3677	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1438.5;
	3678	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	857.5;
	3678	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.4;
	3678	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	813.1;
	3678	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3288.2;
	3678	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9788;
	3678	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5488.6;
	3678	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9845.5;
	3678	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1548.9;
	3679	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871.4;
	3679	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.2;
	3679	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	840.3;
	3679	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3369.1;
	3679	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10294.6;
	3679	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5694.5;
	3679	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10176.1;
	3679	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1630.6;
	3680	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.3;
	3680	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772.2;
	3680	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.7;
	3680	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3519.7;
	3680	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10533;
	3680	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5838.3;
	3680	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10525.1;
	3680	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1696.8;
	3681	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.6;
	3681	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	786.9;
	3681	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.9;
	3681	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3755.5;
	3681	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10889;
	3681	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6106.4;
	3681	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10714.5;
	3681	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1745.5;
	3682	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916;
	3682	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	814.5;
	3682	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	912.8;
	3682	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3990.1;
	3682	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11181.1;
	3682	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6504.6;
	3682	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10801.7;
	3682	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1776.1;
	3683	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.8;
	3683	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	833;
	3683	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	941.1;
	3683	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4235.3;
	3683	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11359.5;
	3683	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6889.5;
	3683	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11044.3;
	3683	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1789;
	3684	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.3;
	3684	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	840.4;
	3684	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	973.5;
	3684	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4485.6;
	3684	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11331.6;
	3684	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7254.8;
	3684	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11320.6;
	3684	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1822.6;
	3685	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.5;
	3685	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	848.3;
	3685	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1000.7;
	3685	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4699.8;
	3685	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11266.7;
	3685	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7600.7;
	3685	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11508.5;
	3685	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1839.3;
	3686	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960;
	3686	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	845.2;
	3686	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1031.6;
	3686	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4847.3;
	3686	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11138.1;
	3686	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7771.9;
	3686	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11674.8;
	3686	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1846.2;
	3687	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.2;
	3687	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	849.2;
	3687	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1066.6;
	3687	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4968.1;
	3687	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11111.4;
	3687	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7834.6;
	3687	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11937;
	3687	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1915;
	3688	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.5;
	3688	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	843.2;
	3688	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1076.6;
	3688	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4976.4;
	3688	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11233.2;
	3688	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7975.2;
	3688	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11953.4;
	3688	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1931.9;
	3689	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951.9;
	3689	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	833.3;
	3689	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1065;
	3689	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4847.5;
	3689	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11175.4;
	3689	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8090.3;
	3689	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11620.7;
	3689	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1885.1;
	3690	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.8;
	3690	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	823.7;
	3690	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1039.5;
	3690	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4596.3;
	3690	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11068.7;
	3690	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7936.2;
	3690	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11174.5;
	3690	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1866.3;
	3691	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.3;
	3691	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	810.6;
	3691	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1008.4;
	3691	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4370.2;
	3691	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10902.4;
	3691	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7449.9;
	3691	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10899.3;
	3691	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1860.9;
	3692	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.8;
	3692	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	812.6;
	3692	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	971;
	3692	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4282.4;
	3692	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10959.4;
	3692	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7148.2;
	3692	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10753.2;
	3692	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1851;
	3693	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.2;
	3693	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	807.2;
	3693	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	957.7;
	3693	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4199.3;
	3693	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10891;
	3693	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6903.2;
	3693	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10491.6;
	3693	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1808.4;
	3694	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.7;
	3694	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	767.3;
	3694	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	903;
	3694	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3925.9;
	3694	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10246.9;
	3694	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6343.9;
	3694	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9895.1;
	3694	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1726.8;
	3695	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.3;
	3695	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720.8;
	3695	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	830.3;
	3695	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3578.1;
	3695	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9334.3;
	3695	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5689.8;
	3695	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9221.9;
	3695	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1585.6;
	3696	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	856.4;
	3696	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	684.2;
	3696	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	763.7;
	3696	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3290;
	3696	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8641;
	3696	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5040.9;
	3696	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8668.3;
	3696	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1468.3;
	3697	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	842.6;
	3697	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	663.5;
	3697	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	734.2;
	3697	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3085.7;
	3697	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8165.3;
	3697	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4695;
	3697	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8329.1;
	3697	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1378.9;
	3698	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	832.6;
	3698	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	651.5;
	3698	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	714.7;
	3698	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2951.3;
	3698	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7890.3;
	3698	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4490;
	3698	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8112.6;
	3698	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1349.3;
	3699	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	829.3;
	3699	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	642.6;
	3699	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	705.5;
	3699	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2872.5;
	3699	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7731.8;
	3699	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4376.9;
	3699	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8056.7;
	3699	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1265.3;
	3700	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	828.5;
	3700	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	639.6;
	3700	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	702.1;
	3700	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2848.4;
	3700	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7812.4;
	3700	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4377.6;
	3700	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8156.5;
	3700	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1265;
	3701	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	838.5;
	3701	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	663.5;
	3701	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	722.3;
	3701	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2917.8;
	3701	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8261.8;
	3701	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4601.8;
	3701	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8548.3;
	3701	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1380.3;
	3702	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	861.6;
	3702	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	701.6;
	3702	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	767.9;
	3702	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3112.7;
	3702	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9031.4;
	3702	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5073.2;
	3702	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9109.4;
	3702	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1455.7;
	3703	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.1;
	3703	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.4;
	3703	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	798.2;
	3703	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3235.1;
	3703	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9506.1;
	3703	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5339.5;
	3703	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9473;
	3703	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1566.7;
	3704	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.5;
	3704	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	748.3;
	3704	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824.2;
	3704	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3491.4;
	3704	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9950.2;
	3704	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5628;
	3704	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9755.3;
	3704	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1653.1;
	3705	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.2;
	3705	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	780.2;
	3705	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.3;
	3705	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3839.6;
	3705	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10433.3;
	3705	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6016.7;
	3705	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10138.5;
	3705	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1737.4;
	3706	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.3;
	3706	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	814.8;
	3706	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.8;
	3706	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4238.7;
	3706	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11091.1;
	3706	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6447.7;
	3706	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10573.3;
	3706	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1813.2;
	3707	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	968.7;
	3707	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	843.1;
	3707	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	916.4;
	3707	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4619.2;
	3707	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11729.4;
	3707	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6827.1;
	3707	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10954.6;
	3707	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1890.1;
	3708	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990.5;
	3708	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	856;
	3708	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	935.6;
	3708	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4932;
	3708	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12275;
	3708	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7144.7;
	3708	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11399.6;
	3708	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1948.7;
	3709	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1014;
	3709	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	873.8;
	3709	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	960.3;
	3709	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5128.1;
	3709	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12864.9;
	3709	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7400;
	3709	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11960.4;
	3709	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1972.3;
	3710	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030.5;
	3710	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	884.6;
	3710	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	991.8;
	3710	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5130;
	3710	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13332.7;
	3710	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7563.3;
	3710	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12257.7;
	3710	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1980.4;
	3711	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1037.6;
	3711	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	891;
	3711	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1011.9;
	3711	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4920.2;
	3711	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13622.9;
	3711	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7619.1;
	3711	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12118.5;
	3711	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1959.3;
	3712	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1047.7;
	3712	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	892.8;
	3712	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1026.5;
	3712	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4779.4;
	3712	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13692.9;
	3712	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7600;
	3712	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11908.2;
	3712	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1890.1;
	3713	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1038;
	3713	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	874.3;
	3713	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1018.8;
	3713	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4585.8;
	3713	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13317.3;
	3713	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7500.6;
	3713	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11663.1;
	3713	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1843.2;
	3714	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1024.4;
	3714	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	860.3;
	3714	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	999.4;
	3714	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4341.2;
	3714	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12655.2;
	3714	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7292.5;
	3714	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11367.9;
	3714	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1838;
	3715	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1006.9;
	3715	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	843.8;
	3715	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	972.6;
	3715	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4139.6;
	3715	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12017.9;
	3715	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7011;
	3715	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11030.6;
	3715	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1787.8;
	3716	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	982.2;
	3716	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	825.4;
	3716	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	948.3;
	3716	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4099.7;
	3716	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11710.7;
	3716	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6846.2;
	3716	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10966.1;
	3716	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1789.7;
	3717	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970.9;
	3717	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	823.9;
	3717	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	946.7;
	3717	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4051.7;
	3717	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11533.8;
	3717	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6658.7;
	3717	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10767.8;
	3717	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1797.1;
	3718	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.8;
	3718	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	791.4;
	3718	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	904.2;
	3718	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3864;
	3718	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10835.9;
	3718	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6218.1;
	3718	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10254.8;
	3718	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1716.5;
	3719	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.2;
	3719	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.9;
	3719	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.8;
	3719	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3595.4;
	3719	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9951;
	3719	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5685.4;
	3719	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9607.5;
	3719	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1606.6;
	3720	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.3;
	3720	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	700.9;
	3720	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	800.9;
	3720	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3344.4;
	3720	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9121.9;
	3720	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5172;
	3720	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8969.6;
	3720	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1471.8;
	3721	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	857.7;
	3721	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	673.4;
	3721	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	763.3;
	3721	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3145;
	3721	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8510.4;
	3721	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4850.7;
	3721	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8566.4;
	3721	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1400.8;
	3722	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	843.1;
	3722	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	658.7;
	3722	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	746.7;
	3722	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3001.4;
	3722	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8103.7;
	3722	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4579.8;
	3722	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8267.8;
	3722	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1346.4;
	3723	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	835.9;
	3723	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	651.8;
	3723	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	744.1;
	3723	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2910.2;
	3723	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7859.2;
	3723	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4428.3;
	3723	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8041.2;
	3723	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1317.4;
	3724	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	828.7;
	3724	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	637.7;
	3724	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	738.7;
	3724	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2869.1;
	3724	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7795;
	3724	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4354;
	3724	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7909.1;
	3724	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1284.1;
	3725	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	825.8;
	3725	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	641.2;
	3725	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	725.3;
	3725	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2906.3;
	3725	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7919.4;
	3725	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4394.3;
	3725	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7947.7;
	3725	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1312.4;
	3726	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	830.7;
	3726	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	650.8;
	3726	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	727.8;
	3726	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2958.7;
	3726	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8160.4;
	3726	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4548;
	3726	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8085.6;
	3726	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1324.5;
	3727	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	830.6;
	3727	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	659;
	3727	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	746;
	3727	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2986.2;
	3727	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8478.5;
	3727	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4736.3;
	3727	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8261.6;
	3727	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1353.5;
	3728	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	852.2;
	3728	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	693.1;
	3728	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	791.7;
	3728	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3155.5;
	3728	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9074;
	3728	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5133;
	3728	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8606.4;
	3728	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1445.9;
	3729	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.1;
	3729	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733;
	3729	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838.1;
	3729	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3376.9;
	3729	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9741.9;
	3729	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5615.3;
	3729	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9075.2;
	3729	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1548.8;
	3730	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.8;
	3730	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777.2;
	3730	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	879.7;
	3730	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3575.9;
	3730	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10506.8;
	3730	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6006.5;
	3730	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9483.1;
	3730	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1625;
	3731	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.2;
	3731	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	825.1;
	3731	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	924.8;
	3731	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3695;
	3731	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11191.7;
	3731	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6365.5;
	3731	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9791.7;
	3731	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1703.1;
	3732	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.6;
	3732	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	867;
	3732	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	977;
	3732	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3842.8;
	3732	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11779.3;
	3732	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6728.8;
	3732	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10094.3;
	3732	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1781.4;
	3733	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009.7;
	3733	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	903.4;
	3733	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1022;
	3733	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3992.6;
	3733	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12292.9;
	3733	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6998.9;
	3733	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10284.4;
	3733	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1814.9;
	3734	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1036.2;
	3734	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	934.6;
	3734	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1059.9;
	3734	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4102;
	3734	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12734;
	3734	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7204.4;
	3734	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10496.2;
	3734	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1841.4;
	3735	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1050.9;
	3735	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	956.8;
	3735	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1091.1;
	3735	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4155.2;
	3735	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13056.6;
	3735	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7426.2;
	3735	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10667.1;
	3735	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1882.3;
	3736	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1055.7;
	3736	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	942.3;
	3736	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1106.1;
	3736	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4238.8;
	3736	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13397.3;
	3736	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7552.5;
	3736	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10726.6;
	3736	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1900.7;
	3737	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1056.4;
	3737	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	940.8;
	3737	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1111.4;
	3737	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4286.3;
	3737	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13634.3;
	3737	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7438.4;
	3737	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10585.7;
	3737	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1876.6;
	3738	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1048.7;
	3738	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	932;
	3738	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1077;
	3738	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4188.5;
	3738	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13404.1;
	3738	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7249.2;
	3738	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10389.9;
	3738	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1839.2;
	3739	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1029.2;
	3739	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	911.4;
	3739	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1028.1;
	3739	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3961.7;
	3739	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12871.9;
	3739	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6909.4;
	3739	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10154.3;
	3739	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1742.2;
	3740	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1002.5;
	3740	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	880.1;
	3740	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	983.1;
	3740	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3897.3;
	3740	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12298.6;
	3740	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6698.9;
	3740	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10212.4;
	3740	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1714.3;
	3741	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	988.6;
	3741	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	861.8;
	3741	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	970.4;
	3741	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3837;
	3741	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11935.7;
	3741	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6574.4;
	3741	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10053.2;
	3741	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1761.1;
	3742	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.1;
	3742	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	818;
	3742	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889.5;
	3742	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3650.3;
	3742	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11147.7;
	3742	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6184;
	3742	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9636.3;
	3742	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1675.3;
	3743	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.6;
	3743	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	756.9;
	3743	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	779.8;
	3743	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3413.7;
	3743	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10195.4;
	3743	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5711.5;
	3743	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9148.5;
	3743	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1561.3;
	3744	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.1;
	3744	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	705.1;
	3744	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	731.2;
	3744	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3178.6;
	3744	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9328.5;
	3744	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5217.3;
	3744	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8625.4;
	3744	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1470.9;
	3745	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.7;
	3745	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	673.5;
	3745	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	710.1;
	3745	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2995.4;
	3745	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8620.9;
	3745	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4865.9;
	3745	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8261.2;
	3745	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1392.7;
	3746	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.7;
	3746	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	657;
	3746	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	698.8;
	3746	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2864;
	3746	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8118.6;
	3746	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4616.2;
	3746	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7990.9;
	3746	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1319.7;
	3747	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	848.2;
	3747	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	649.1;
	3747	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	694.6;
	3747	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2779.1;
	3747	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7778.6;
	3747	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4418.7;
	3747	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7815.3;
	3747	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1308.6;
	3748	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	840;
	3748	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	639.9;
	3748	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	685.8;
	3748	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2729.9;
	3748	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7609;
	3748	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4314.1;
	3748	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7762.7;
	3748	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1278.1;
	3749	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	842;
	3749	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	636.6;
	3749	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	694.8;
	3749	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2726.6;
	3749	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7577.4;
	3749	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4335.2;
	3749	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7783;
	3749	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1267.8;
	3750	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	841.5;
	3750	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	637.4;
	3750	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	698.3;
	3750	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2718.9;
	3750	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7551.5;
	3750	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4374;
	3750	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7740.6;
	3750	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1286.2;
	3751	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	843.5;
	3751	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	657.5;
	3751	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	715.6;
	3751	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2754.2;
	3751	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7976.9;
	3751	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4549.4;
	3751	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7906.2;
	3751	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1300.9;
	3752	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.4;
	3752	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	708.5;
	3752	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	774.6;
	3752	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3022.5;
	3752	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8886.8;
	3752	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5053;
	3752	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8511.1;
	3752	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1477.6;
	3753	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.9;
	3753	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	758.4;
	3753	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	834.9;
	3753	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3366.1;
	3753	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9904;
	3753	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5648.4;
	3753	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9249.1;
	3753	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1602.6;
	3754	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.5;
	3754	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	804;
	3754	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	888.3;
	3754	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3695.6;
	3754	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10825.2;
	3754	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6206.1;
	3754	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9907.4;
	3754	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1711.4;
	3755	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.8;
	3755	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	832;
	3755	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	936.7;
	3755	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3954.6;
	3755	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11628.2;
	3755	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6645.7;
	3755	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10503.7;
	3755	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1807;
	3756	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	987.2;
	3756	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	867;
	3756	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	973.2;
	3756	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4134.6;
	3756	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12270.7;
	3756	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6925.7;
	3756	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10987.9;
	3756	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1908.5;
	3757	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1012.9;
	3757	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	897.7;
	3757	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1015.2;
	3757	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4281.8;
	3757	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12818.8;
	3757	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7142.7;
	3757	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11382.2;
	3757	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1976.2;
	3758	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1033.2;
	3758	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	927.9;
	3758	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1051.7;
	3758	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4385.5;
	3758	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13327.6;
	3758	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7367;
	3758	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11657.9;
	3758	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2022;
	3759	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1043.8;
	3759	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	953.5;
	3759	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1081.4;
	3759	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4366.5;
	3759	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13738.8;
	3759	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7593.5;
	3759	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11760.8;
	3759	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2089.8;
	3760	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1058.4;
	3760	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	977.2;
	3760	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1106.2;
	3760	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4352.6;
	3760	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14085.1;
	3760	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7807.8;
	3760	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11788.3;
	3760	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2090.1;
	3761	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1067.7;
	3761	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	992.2;
	3761	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1116.9;
	3761	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4291.5;
	3761	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14238.2;
	3761	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7904.7;
	3761	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11665.3;
	3761	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2043.4;
	3762	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1059.2;
	3762	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	977.9;
	3762	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1108;
	3762	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4202.2;
	3762	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13976.9;
	3762	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7821.5;
	3762	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11278.5;
	3762	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2046.3;
	3763	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1038.9;
	3763	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	945.4;
	3763	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1084.7;
	3763	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4078;
	3763	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13447.7;
	3763	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7539.8;
	3763	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10964.3;
	3763	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1950.4;
	3764	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	996.5;
	3764	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	902.3;
	3764	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1028.3;
	3764	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4054.8;
	3764	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12769.3;
	3764	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7211.1;
	3764	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10880.7;
	3764	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1976.5;
	3765	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	979.4;
	3765	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	876.8;
	3765	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1002.1;
	3765	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4035.1;
	3765	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12425.6;
	3765	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7016.7;
	3765	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10800.4;
	3765	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1936;
	3766	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947;
	3766	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	814.4;
	3766	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	930.6;
	3766	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3819.4;
	3766	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11387.6;
	3766	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6436.2;
	3766	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10285;
	3766	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1788.2;
	3767	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.7;
	3767	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	748.2;
	3767	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.4;
	3767	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3507.3;
	3767	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10100;
	3767	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5718.6;
	3767	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9568.9;
	3767	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1640.4;
	3768	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.4;
	3768	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	683.2;
	3768	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	784.6;
	3768	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3200.6;
	3768	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9005.6;
	3768	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5094.6;
	3768	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8975.1;
	3768	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1499.2;
	3769	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	851.2;
	3769	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	646;
	3769	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	741.5;
	3769	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2990.3;
	3769	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8273.9;
	3769	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4693.5;
	3769	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8484.5;
	3769	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1415.5;
	3770	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	837.4;
	3770	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	626.3;
	3770	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	714.7;
	3770	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2853.3;
	3770	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7851.3;
	3770	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4454;
	3770	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8225.2;
	3770	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1348.6;
	3771	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	829.1;
	3771	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	613;
	3771	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	698.6;
	3771	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2771.4;
	3771	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7638.3;
	3771	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4328.6;
	3771	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8157.8;
	3771	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1313;
	3772	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	825.8;
	3772	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	614.1;
	3772	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	695.4;
	3772	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2755.2;
	3772	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7689.3;
	3772	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4328.6;
	3772	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8328.4;
	3772	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1305.4;
	3773	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	833.5;
	3773	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	636;
	3773	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	719;
	3773	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2853;
	3773	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8143.5;
	3773	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4601.1;
	3773	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8770.1;
	3773	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1374.6;
	3774	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	852.1;
	3774	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	666.6;
	3774	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	786.5;
	3774	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3037.8;
	3774	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8801.1;
	3774	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4977.3;
	3774	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9254.6;
	3774	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1455.9;
	3775	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.7;
	3775	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	710.2;
	3775	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824.6;
	3775	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3154.6;
	3775	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9399.9;
	3775	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5276.2;
	3775	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9597.5;
	3775	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1562.1;
	3776	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.7;
	3776	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	756.4;
	3776	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.7;
	3776	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3420.5;
	3776	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10132;
	3776	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5684.6;
	3776	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10088.4;
	3776	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1671.9;
	3777	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.9;
	3777	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	803.7;
	3777	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	918.5;
	3777	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3765.8;
	3777	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10970.6;
	3777	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6174.4;
	3777	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10630.2;
	3777	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1811.5;
	3778	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.2;
	3778	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	866.3;
	3778	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	967.5;
	3778	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4142.1;
	3778	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11917.5;
	3778	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6708.2;
	3778	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11137.2;
	3778	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1962;
	3779	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1001.2;
	3779	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	922.2;
	3779	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1008.4;
	3779	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4502.2;
	3779	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12814;
	3779	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7153.7;
	3779	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11748.4;
	3779	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2041.1;
	3780	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1026.8;
	3780	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	962.1;
	3780	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1050.7;
	3780	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4780.4;
	3780	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13594.9;
	3780	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7573.9;
	3780	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12446.9;
	3780	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2167.7;
	3781	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1053.9;
	3781	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	999;
	3781	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1094.5;
	3781	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4958.8;
	3781	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14361.6;
	3781	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7986.7;
	3781	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13145.6;
	3781	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2271.9;
	3782	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1067;
	3782	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1032.1;
	3782	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1140.7;
	3782	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5076.4;
	3782	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14986.6;
	3782	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8362.3;
	3782	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13768.9;
	3782	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2321.1;
	3783	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1083.5;
	3783	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1059.2;
	3783	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1186.2;
	3783	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5157.5;
	3783	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15485.3;
	3783	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8700.2;
	3783	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14301.4;
	3783	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2368.1;
	3784	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1102.7;
	3784	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1070.1;
	3784	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1218.8;
	3784	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5163.9;
	3784	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15760.6;
	3784	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8935.5;
	3784	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14542.9;
	3784	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2396.1;
	3785	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1100.6;
	3785	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1076.9;
	3785	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1227.4;
	3785	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5123.7;
	3785	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15859;
	3785	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9028.2;
	3785	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14574.1;
	3785	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2400.1;
	3786	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1098.6;
	3786	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1064.3;
	3786	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1218.2;
	3786	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4988.7;
	3786	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15582.2;
	3786	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8974.8;
	3786	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14132.7;
	3786	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2338.6;
	3787	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1081.4;
	3787	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1030;
	3787	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1178.5;
	3787	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4786.9;
	3787	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14927.4;
	3787	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8635.6;
	3787	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13559.2;
	3787	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2257.7;
	3788	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1046.3;
	3788	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	982.9;
	3788	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1116.7;
	3788	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4652.5;
	3788	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14210;
	3788	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8192.1;
	3788	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13194.2;
	3788	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2184.8;
	3789	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030.7;
	3789	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	950.3;
	3789	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1083.1;
	3789	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4556.9;
	3789	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13664.9;
	3789	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7823.2;
	3789	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12800.2;
	3789	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2120;
	3790	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990.7;
	3790	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	880.7;
	3790	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1005.6;
	3790	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4230.5;
	3790	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12422.4;
	3790	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7084.6;
	3790	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11890.6;
	3790	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1922.9;
	3791	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937;
	3791	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	804.6;
	3791	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	909.4;
	3791	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3822.1;
	3791	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10967.4;
	3791	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6247.5;
	3791	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10894.5;
	3791	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1757.8;
	3792	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.6;
	3792	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.9;
	3792	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824.6;
	3792	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3552.3;
	3792	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9658.2;
	3792	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5578.6;
	3792	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10163.3;
	3792	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1606.7;
	3793	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	863;
	3793	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	699.7;
	3793	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	777.5;
	3793	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3309;
	3793	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8894.7;
	3793	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5121.2;
	3793	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9547.3;
	3793	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1481.3;
	3794	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	853.1;
	3794	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	663.7;
	3794	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	750.4;
	3794	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3152.5;
	3794	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8414.2;
	3794	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4839.5;
	3794	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9119.8;
	3794	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1348.6;
	3795	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	837.7;
	3795	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	638.2;
	3795	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	727.1;
	3795	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3028.3;
	3795	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8102.1;
	3795	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4669.4;
	3795	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8845.6;
	3795	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1343.4;
	3796	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	830.3;
	3796	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	640.2;
	3796	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	720.9;
	3796	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2997;
	3796	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8096.7;
	3796	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4648.6;
	3796	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8880.5;
	3796	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1353.1;
	3797	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	842.8;
	3797	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	666.5;
	3797	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	740.1;
	3797	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3073.6;
	3797	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8540.5;
	3797	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4866.4;
	3797	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9269.2;
	3797	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1351.4;
	3798	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	852.9;
	3798	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	692.7;
	3798	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	789;
	3798	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3251.1;
	3798	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9135.9;
	3798	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5186;
	3798	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9617.5;
	3798	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1512.1;
	3799	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.3;
	3799	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.5;
	3799	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824.7;
	3799	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3413.1;
	3799	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9730.7;
	3799	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5479.7;
	3799	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10120.9;
	3799	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1602.7;
	3800	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.7;
	3800	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777.9;
	3800	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	873.9;
	3800	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3717.4;
	3800	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10459.5;
	3800	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5901.1;
	3800	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10915.2;
	3800	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1739.7;
	3801	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.6;
	3801	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	827.6;
	3801	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	930.4;
	3801	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4084.9;
	3801	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11383.2;
	3801	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6509;
	3801	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11825.7;
	3801	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1895.2;
	3802	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	979.8;
	3802	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	888.3;
	3802	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	969;
	3802	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4505.2;
	3802	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12530.7;
	3802	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7163.4;
	3802	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12825.4;
	3802	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2063.3;
	3803	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1014.9;
	3803	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	943;
	3803	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1044.6;
	3803	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4880.1;
	3803	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13556.3;
	3803	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7730.5;
	3803	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13646.3;
	3803	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2181.4;
	3804	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1040.5;
	3804	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	989.2;
	3804	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1091.7;
	3804	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5184;
	3804	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14457.2;
	3804	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8186.9;
	3804	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14174.4;
	3804	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2317.5;
	3805	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1069.5;
	3805	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1034.2;
	3805	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1143.5;
	3805	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5412.4;
	3805	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15331.1;
	3805	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8623.2;
	3805	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14644.8;
	3805	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2464.4;
	3806	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1088.4;
	3806	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1072.4;
	3806	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1189.3;
	3806	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5553.7;
	3806	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16072.2;
	3806	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8984.2;
	3806	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15065.1;
	3806	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2535;
	3807	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1108.6;
	3807	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1097.3;
	3807	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1225.7;
	3807	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5604.5;
	3807	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16630;
	3807	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9287.4;
	3807	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15279.8;
	3807	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2585.7;
	3808	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1120.5;
	3808	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1099;
	3808	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1246.1;
	3808	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5570.4;
	3808	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16959.2;
	3808	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9539.1;
	3808	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15222.8;
	3808	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2564.7;
	3809	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1114.8;
	3809	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1089.3;
	3809	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1241.7;
	3809	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5439;
	3809	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16995.6;
	3809	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9618.2;
	3809	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14926.6;
	3809	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2622;
	3810	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1105.2;
	3810	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1073.3;
	3810	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1220.7;
	3810	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5223.9;
	3810	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16692;
	3810	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9489.8;
	3810	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14389.5;
	3810	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2527.8;
	3811	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1082.3;
	3811	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1041;
	3811	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1164.8;
	3811	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4985.5;
	3811	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16018.6;
	3811	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9122.7;
	3811	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13871.1;
	3811	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2436;
	3812	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1050;
	3812	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	995.2;
	3812	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1109.5;
	3812	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4872.6;
	3812	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15150.9;
	3812	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8744.2;
	3812	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13499.6;
	3812	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2359.9;
	3813	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030.8;
	3813	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	959.8;
	3813	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1080.8;
	3813	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4758.7;
	3813	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14423.2;
	3813	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8450.8;
	3813	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12997.4;
	3813	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2227.6;
	3814	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	982.3;
	3814	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	902.2;
	3814	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1006.6;
	3814	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4423.9;
	3814	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13099;
	3814	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7703.4;
	3814	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12040.4;
	3814	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2017.1;
	3815	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.3;
	3815	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	820.2;
	3815	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913.8;
	3815	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4027.9;
	3815	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11562.3;
	3815	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6838.6;
	3815	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10987.2;
	3815	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1791.6;
	3816	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.3;
	3816	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	768.3;
	3816	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.8;
	3816	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3584.9;
	3816	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10337.3;
	3816	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6165;
	3816	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10167.9;
	3816	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1574.7;
	3817	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.9;
	3817	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.3;
	3817	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	812.8;
	3817	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3347.7;
	3817	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9519.5;
	3817	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5658.4;
	3817	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9568.8;
	3817	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1428.1;
	3818	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	858.2;
	3818	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	700.7;
	3818	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	784.2;
	3818	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3193.8;
	3818	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8991.1;
	3818	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5323.4;
	3818	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9183;
	3818	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1348;
	3819	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	850.8;
	3819	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	691.7;
	3819	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	766.4;
	3819	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3098.5;
	3819	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8680.4;
	3819	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5133;
	3819	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8987.3;
	3819	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1307.7;
	3820	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	847.1;
	3820	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	681.3;
	3820	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	760;
	3820	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3051.7;
	3820	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8628.4;
	3820	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5093.8;
	3820	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9039;
	3820	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1366.1;
	3821	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	852.2;
	3821	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	689.4;
	3821	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	776.3;
	3821	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3129.2;
	3821	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9013.2;
	3821	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5298.9;
	3821	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9470.3;
	3821	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1385.4;
	3822	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.5;
	3822	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719.1;
	3822	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	816.2;
	3822	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3297.9;
	3822	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9595.4;
	3822	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5696;
	3822	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9852.1;
	3822	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1512.4;
	3823	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878;
	3823	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746;
	3823	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	855.7;
	3823	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3417.7;
	3823	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10236.9;
	3823	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5989.6;
	3823	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10302.4;
	3823	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1607.5;
	3824	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.9;
	3824	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	801.1;
	3824	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	906.3;
	3824	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3712.4;
	3824	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11071;
	3824	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6463.5;
	3824	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11075.6;
	3824	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1747.5;
	3825	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.4;
	3825	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	854.7;
	3825	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	969.7;
	3825	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4036.4;
	3825	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12063.1;
	3825	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7046.9;
	3825	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12022.3;
	3825	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1910.9;
	3826	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985.3;
	3826	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	928.7;
	3826	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1037.3;
	3826	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4411.5;
	3826	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13240.7;
	3826	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7670.1;
	3826	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12999.9;
	3826	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2077.4;
	3827	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1022.5;
	3827	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	996.5;
	3827	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1100.2;
	3827	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4732;
	3827	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14300.6;
	3827	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8222.5;
	3827	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13836.8;
	3827	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2214.5;
	3828	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1049.1;
	3828	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1054.5;
	3828	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1156.5;
	3828	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4980.4;
	3828	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15205.1;
	3828	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8659.8;
	3828	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14474;
	3828	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2335.5;
	3829	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1077.7;
	3829	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1105.7;
	3829	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1206.2;
	3829	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5148.1;
	3829	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16032.2;
	3829	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9030.1;
	3829	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14937.4;
	3829	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2481.9;
	3830	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1103.6;
	3830	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1148.2;
	3830	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1254.1;
	3830	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5298.3;
	3830	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16701.8;
	3830	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9337.1;
	3830	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15285.6;
	3830	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2556.6;
	3831	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1127.8;
	3831	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1183.8;
	3831	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1287.4;
	3831	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5403.6;
	3831	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17194.6;
	3831	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9589.7;
	3831	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15541.5;
	3831	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2584.9;
	3832	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1142.4;
	3832	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1197.1;
	3832	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1310.7;
	3832	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5359.7;
	3832	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17516.9;
	3832	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9770.2;
	3832	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15597.9;
	3832	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2591.4;
	3833	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1138.1;
	3833	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1187.3;
	3833	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1303.8;
	3833	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5242.9;
	3833	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17502.2;
	3833	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9765.7;
	3833	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15407.8;
	3833	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2567.3;
	3834	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1121.7;
	3834	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1157.5;
	3834	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1283.2;
	3834	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5085.1;
	3834	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17097.4;
	3834	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9552.1;
	3834	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14802.7;
	3834	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2469.6;
	3835	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1098.3;
	3835	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1110.1;
	3835	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1236.7;
	3835	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4898.3;
	3835	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16335.7;
	3835	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9075.9;
	3835	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14120.4;
	3835	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2335.8;
	3836	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1063.3;
	3836	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1052.5;
	3836	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1175.5;
	3836	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4767.7;
	3836	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15472.8;
	3836	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8651.8;
	3836	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13706.1;
	3836	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2287.8;
	3837	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1045.8;
	3837	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1016.9;
	3837	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1145.2;
	3837	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4676.5;
	3837	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14855.1;
	3837	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8326.1;
	3837	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13172.8;
	3837	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2194.8;
	3838	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1012.3;
	3838	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	957.8;
	3838	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1067.2;
	3838	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4392.9;
	3838	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13554.8;
	3838	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7612.4;
	3838	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12307.9;
	3838	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1967.8;
	3839	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.6;
	3839	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	870.6;
	3839	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	974.8;
	3839	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3984.9;
	3839	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12055.6;
	3839	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6788.5;
	3839	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11350.1;
	3839	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1806.3;
	3840	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.7;
	3840	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	801.5;
	3840	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.9;
	3840	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3615.8;
	3840	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10905.2;
	3840	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6106.2;
	3840	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10484.5;
	3840	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1677.2;
	3841	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.8;
	3841	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	757.5;
	3841	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.1;
	3841	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3372.4;
	3841	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10068.5;
	3841	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5639.3;
	3841	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9914.5;
	3841	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1589.8;
	3842	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.1;
	3842	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.7;
	3842	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	813.5;
	3842	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3210.6;
	3842	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9496.3;
	3842	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5329.5;
	3842	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9518;
	3842	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1516.8;
	3843	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.9;
	3843	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711.9;
	3843	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	790.2;
	3843	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3061.2;
	3843	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9161.5;
	3843	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5117.7;
	3843	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9268.2;
	3843	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1485.1;
	3844	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.5;
	3844	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	693.8;
	3844	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	784.2;
	3844	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3010.6;
	3844	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9141.3;
	3844	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5063.7;
	3844	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9321.4;
	3844	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1475.7;
	3845	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.3;
	3845	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	713.1;
	3845	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	793.2;
	3845	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3082;
	3845	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9515.1;
	3845	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5254.5;
	3845	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9697.3;
	3845	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1527.6;
	3846	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.2;
	3846	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.8;
	3846	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	827;
	3846	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3268.7;
	3846	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10134.9;
	3846	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5645.5;
	3846	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10093.9;
	3846	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1606.4;
	3847	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.3;
	3847	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777.2;
	3847	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	867.8;
	3847	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3416.8;
	3847	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10749.8;
	3847	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5935;
	3847	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10545.3;
	3847	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1699.9;
	3848	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.4;
	3848	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	839;
	3848	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	923.4;
	3848	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3747.1;
	3848	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11633.8;
	3848	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6390.9;
	3848	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11419;
	3848	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1855;
	3849	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	977.7;
	3849	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	898.9;
	3849	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	981.5;
	3849	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4138.3;
	3849	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12762.1;
	3849	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6929.9;
	3849	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12446;
	3849	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2035.1;
	3850	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1012;
	3850	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	970;
	3850	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1045.3;
	3850	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4528.1;
	3850	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13906.1;
	3850	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7531.4;
	3850	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13419.9;
	3850	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2200.8;
	3851	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1050.1;
	3851	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1032.5;
	3851	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1109.9;
	3851	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4914.6;
	3851	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14911.3;
	3851	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8093.7;
	3851	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14262.6;
	3851	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2334.7;
	3852	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1083.4;
	3852	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1088;
	3852	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1172.4;
	3852	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5229.2;
	3852	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15744.1;
	3852	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8553;
	3852	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14922.4;
	3852	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2476.5;
	3853	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1115.9;
	3853	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1135.2;
	3853	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1231.4;
	3853	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5508.1;
	3853	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16537.7;
	3853	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8934.4;
	3853	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15395.6;
	3853	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2580.5;
	3854	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1138.4;
	3854	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1168.6;
	3854	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1279.8;
	3854	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5709.4;
	3854	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17148.6;
	3854	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9214.1;
	3854	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15379.2;
	3854	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2641.6;
	3855	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1158.2;
	3855	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1191;
	3855	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1320.2;
	3855	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5820.5;
	3855	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17546.3;
	3855	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9479.3;
	3855	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14918.8;
	3855	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2700.7;
	3856	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1167;
	3856	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1195.8;
	3856	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1337.9;
	3856	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5836.9;
	3856	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17770.1;
	3856	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9634;
	3856	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14551.4;
	3856	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2691.1;
	3857	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1156.9;
	3857	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1186.3;
	3857	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1327;
	3857	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5736.2;
	3857	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17636.1;
	3857	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9591;
	3857	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14154.4;
	3857	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2614;
	3858	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1141.4;
	3858	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1162.3;
	3858	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1294.2;
	3858	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5527.2;
	3858	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17171.5;
	3858	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9298;
	3858	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13632.6;
	3858	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2520.6;
	3859	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1121.9;
	3859	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1121.7;
	3859	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1246.8;
	3859	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5290.4;
	3859	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16427.5;
	3859	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8899.7;
	3859	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13334.7;
	3859	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2452.7;
	3860	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1086.1;
	3860	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1077.2;
	3860	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1178.8;
	3860	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5117.7;
	3860	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15728;
	3860	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8475;
	3860	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13040;
	3860	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2357.1;
	3861	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1072.9;
	3861	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1045.6;
	3861	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1154.1;
	3861	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5014.2;
	3861	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15263.9;
	3861	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8174.3;
	3861	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12659.1;
	3861	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2273.4;
	3862	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1032;
	3862	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	975.2;
	3862	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1084.5;
	3862	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4682.3;
	3862	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13977.7;
	3862	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7512.4;
	3862	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11779.7;
	3862	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2114.8;
	3863	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.8;
	3863	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	893.4;
	3863	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	996.5;
	3863	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4284.6;
	3863	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12563.9;
	3863	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6760.1;
	3863	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10880.8;
	3863	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1920.4;
	3864	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.7;
	3864	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	829.2;
	3864	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	921;
	3864	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3946.5;
	3864	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11357.2;
	3864	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6008.6;
	3864	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10194;
	3864	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1744.5;
	3865	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.9;
	3865	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	781.4;
	3865	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	871.3;
	3865	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3682.9;
	3865	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10489.4;
	3865	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5559.7;
	3865	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9647.2;
	3865	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1608.5;
	3866	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.6;
	3866	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753.2;
	3866	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	836.8;
	3866	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3492.6;
	3866	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9920.1;
	3866	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5253.7;
	3866	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9279.2;
	3866	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1542;
	3867	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.1;
	3867	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742.4;
	3867	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	822;
	3867	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3359.3;
	3867	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9573.5;
	3867	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5060.8;
	3867	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9126;
	3867	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1441;
	3868	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.7;
	3868	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	731.1;
	3868	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	812.5;
	3868	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3293.1;
	3868	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9469.9;
	3868	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5013;
	3868	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9166.2;
	3868	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1416.6;
	3869	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.5;
	3869	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746.2;
	3869	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	828;
	3869	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3347.9;
	3869	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9768;
	3869	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5203.6;
	3869	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9513.7;
	3869	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1529.7;
	3870	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.1;
	3870	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	767.1;
	3870	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.6;
	3870	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3476.2;
	3870	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10206.2;
	3870	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5566.9;
	3870	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9862.4;
	3870	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1577.4;
	3871	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.3;
	3871	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	796.1;
	3871	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	895.8;
	3871	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3612.3;
	3871	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10764.6;
	3871	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5831.3;
	3871	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10368.1;
	3871	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1689.1;
	3872	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.4;
	3872	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	853.6;
	3872	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	935.9;
	3872	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3922.2;
	3872	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11638.7;
	3872	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6218.8;
	3872	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11205.8;
	3872	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1820;
	3873	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.3;
	3873	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	911.8;
	3873	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	982.1;
	3873	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4309;
	3873	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12668.3;
	3873	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6698.5;
	3873	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12125.9;
	3873	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1983.5;
	3874	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.7;
	3874	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	982;
	3874	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1035.5;
	3874	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4728.1;
	3874	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13850.4;
	3874	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7215.5;
	3874	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13109.8;
	3874	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2144.3;
	3875	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1038.4;
	3875	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1045.8;
	3875	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1088.8;
	3875	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5102.8;
	3875	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14864.5;
	3875	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7713.1;
	3875	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13877.8;
	3875	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2302.7;
	3876	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1070.2;
	3876	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1091;
	3876	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1134.8;
	3876	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5386.5;
	3876	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15679.5;
	3876	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8176.2;
	3876	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14483.2;
	3876	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2425.6;
	3877	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1104.3;
	3877	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1138.6;
	3877	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1188.6;
	3877	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5573.8;
	3877	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16469.3;
	3877	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8597.1;
	3877	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15047.1;
	3877	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2518.6;
	3878	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1131.3;
	3878	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1173.2;
	3878	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1239.2;
	3878	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5693.4;
	3878	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17080.2;
	3878	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8944.7;
	3878	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15491;
	3878	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2594.7;
	3879	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1149;
	3879	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1198.7;
	3879	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1289.3;
	3879	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5762.4;
	3879	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17548.9;
	3879	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9238.5;
	3879	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15720.9;
	3879	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2644.3;
	3880	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1160.8;
	3880	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1207.9;
	3880	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1319;
	3880	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5766.7;
	3880	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17764;
	3880	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9436.9;
	3880	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15626.8;
	3880	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2587.1;
	3881	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1149.5;
	3881	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1194.6;
	3881	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1317.8;
	3881	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5637.7;
	3881	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17751.9;
	3881	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9456.7;
	3881	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15253.9;
	3881	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2529.4;
	3882	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1134.4;
	3882	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1170.4;
	3882	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1296.8;
	3882	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5423.2;
	3882	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17324.1;
	3882	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9267.1;
	3882	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14686.5;
	3882	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2496.4;
	3883	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1112.1;
	3883	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1128.2;
	3883	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1250.5;
	3883	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5182.7;
	3883	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16533.2;
	3883	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8941.6;
	3883	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14052.5;
	3883	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2394.8;
	3884	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1078.9;
	3884	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1076.4;
	3884	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1196.6;
	3884	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5066.8;
	3884	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15746.3;
	3884	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8585.4;
	3884	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13649.5;
	3884	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2315.1;
	3885	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1065.1;
	3885	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1047;
	3885	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1171.9;
	3885	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4949.9;
	3885	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15216.8;
	3885	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8306;
	3885	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13180.8;
	3885	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2267.6;
	3886	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1032.3;
	3886	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	989.2;
	3886	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1098.8;
	3886	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4675.1;
	3886	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14070.2;
	3886	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7745.6;
	3886	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12471.5;
	3886	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2116;
	3887	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	993.8;
	3887	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	915.8;
	3887	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1012.2;
	3887	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4372.5;
	3887	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12880.9;
	3887	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7138.2;
	3887	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11661.3;
	3887	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1955;
	3888	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.1;
	3888	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	846.4;
	3888	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	935.9;
	3888	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4061.5;
	3888	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11748.3;
	3888	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6563.9;
	3888	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10885.3;
	3888	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1797.8;
	3889	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.1;
	3889	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	799.3;
	3889	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	890.8;
	3889	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3832;
	3889	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10915.3;
	3889	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6101.7;
	3889	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10318.3;
	3889	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1695;
	3890	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.4;
	3890	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	771.9;
	3890	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.9;
	3890	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3668.6;
	3890	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10283.7;
	3890	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5724.8;
	3890	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9903.6;
	3890	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1611.5;
	3891	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.4;
	3891	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.7;
	3891	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	837.2;
	3891	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3550.3;
	3891	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9861.7;
	3891	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5514.6;
	3891	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9612.6;
	3891	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1559.9;
	3892	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.5;
	3892	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	739.7;
	3892	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	829.9;
	3892	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3492.8;
	3892	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9663.3;
	3892	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5417.8;
	3892	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9487;
	3892	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1520.8;
	3893	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.4;
	3893	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741;
	3893	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	829.1;
	3893	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3501.7;
	3893	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9686.5;
	3893	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5458.3;
	3893	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9490.6;
	3893	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1539.8;
	3894	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890;
	3894	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	739.3;
	3894	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	833.1;
	3894	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3517.2;
	3894	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9790.6;
	3894	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5550.9;
	3894	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9494.4;
	3894	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1549.7;
	3895	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.6;
	3895	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776.8;
	3895	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.5;
	3895	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3568.7;
	3895	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10251.3;
	3895	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5726.9;
	3895	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9837.5;
	3895	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1604.9;
	3896	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.4;
	3896	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	833.6;
	3896	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	912.8;
	3896	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3851.7;
	3896	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11170.2;
	3896	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6185.1;
	3896	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10761.8;
	3896	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1751.9;
	3897	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.3;
	3897	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	894.1;
	3897	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	974.5;
	3897	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4219;
	3897	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12108.9;
	3897	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6730.8;
	3897	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11771.4;
	3897	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1901.6;
	3898	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1001.1;
	3898	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	965.2;
	3898	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1035.4;
	3898	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4607.2;
	3898	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13218.6;
	3898	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7300.9;
	3898	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12809;
	3898	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2026.3;
	3899	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1044.4;
	3899	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1022.8;
	3899	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1080.1;
	3899	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4938.6;
	3899	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14235.1;
	3899	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7861;
	3899	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13638.4;
	3899	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2165.9;
	3900	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1077.2;
	3900	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1064.6;
	3900	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1109.5;
	3900	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5198.4;
	3900	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15070.1;
	3900	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8321.4;
	3900	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14240.8;
	3900	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2315.8;
	3901	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1105.8;
	3901	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1088.9;
	3901	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1141.9;
	3901	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5382;
	3901	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15433.4;
	3901	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8679.6;
	3901	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14702.5;
	3901	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2425.8;
	3902	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1126;
	3902	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1082;
	3902	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1164.1;
	3902	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5495.8;
	3902	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15341.5;
	3902	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8976.5;
	3902	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15103;
	3902	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2478.2;
	3903	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1138.7;
	3903	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1051.9;
	3903	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1195.2;
	3903	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5567.5;
	3903	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15299.7;
	3903	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9242.8;
	3903	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15371.2;
	3903	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2509.7;
	3904	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1142.1;
	3904	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1018.6;
	3904	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1215.3;
	3904	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5557.8;
	3904	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15239.1;
	3904	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9396.1;
	3904	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15465.6;
	3904	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2522.8;
	3905	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1142.2;
	3905	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	971.1;
	3905	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1219.3;
	3905	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5465.2;
	3905	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15039.4;
	3905	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9408.3;
	3905	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15347;
	3905	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2509.2;
	3906	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1128.5;
	3906	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	939.9;
	3906	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1185.8;
	3906	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5275;
	3906	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14759.8;
	3906	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9249.6;
	3906	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14890.1;
	3906	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2471.4;
	3907	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1107.5;
	3907	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	926.5;
	3907	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1149.1;
	3907	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5091.2;
	3907	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14347.8;
	3907	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8922.1;
	3907	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14424.9;
	3907	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2341.8;
	3908	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1076.2;
	3908	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	906.9;
	3908	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1105.6;
	3908	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5037;
	3908	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14030.5;
	3908	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8577.1;
	3908	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14073.7;
	3908	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2255;
	3909	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1067.3;
	3909	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	902.4;
	3909	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1097.4;
	3909	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4964.9;
	3909	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13811.3;
	3909	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8323.7;
	3909	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13741.6;
	3909	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2259.2;
	3910	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1042;
	3910	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	866.9;
	3910	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1012.9;
	3910	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4752.5;
	3910	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13077.9;
	3910	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7857.4;
	3910	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13076.7;
	3910	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2133.2;
	3911	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1005.9;
	3911	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	816;
	3911	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889.7;
	3911	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4485.5;
	3911	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12155.2;
	3911	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7356.1;
	3911	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12367.6;
	3911	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1986.2;
	3912	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	968;
	3912	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	775.9;
	3912	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	836.3;
	3912	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4172.4;
	3912	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11242.5;
	3912	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6940;
	3912	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11555.3;
	3912	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1833.1;
	3913	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922;
	3913	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746;
	3913	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	802.8;
	3913	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3947.4;
	3913	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10565.1;
	3913	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6498.1;
	3913	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11035.5;
	3913	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1736.2;
	3914	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.6;
	3914	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.2;
	3914	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	790.9;
	3914	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3771.2;
	3914	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10092.2;
	3914	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6181.8;
	3914	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10609.1;
	3914	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1651.8;
	3915	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.8;
	3915	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714.3;
	3915	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	786.1;
	3915	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3652;
	3915	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9779.7;
	3915	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5925.8;
	3915	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10293.1;
	3915	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1631.5;
	3916	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.4;
	3916	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711;
	3916	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	769.9;
	3916	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3566.1;
	3916	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9600;
	3916	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5774.5;
	3916	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10084.6;
	3916	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1588.7;
	3917	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.4;
	3917	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720.3;
	3917	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	782.9;
	3917	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3539.9;
	3917	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9576.1;
	3917	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5755.2;
	3917	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10059.8;
	3917	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1575.9;
	3918	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867.2;
	3918	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723.4;
	3918	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	790.2;
	3918	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3527.7;
	3918	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9576.8;
	3918	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5750.1;
	3918	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9981;
	3918	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1587.5;
	3919	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	865.7;
	3919	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.8;
	3919	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	804.3;
	3919	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3535.2;
	3919	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9853.5;
	3919	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5823.4;
	3919	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10111.5;
	3919	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1608.1;
	3920	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.4;
	3920	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	800.2;
	3920	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	839.6;
	3920	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3774.9;
	3920	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10706.7;
	3920	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6289.4;
	3920	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10643.8;
	3920	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1799.1;
	3921	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.9;
	3921	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	842.7;
	3921	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884.2;
	3921	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4123.7;
	3921	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11717.3;
	3921	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6899.8;
	3921	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11393;
	3921	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1943.4;
	3922	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.8;
	3922	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	874.4;
	3922	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	933;
	3922	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4480.6;
	3922	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12609.1;
	3922	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7485.2;
	3922	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12065.8;
	3922	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2065.8;
	3923	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985.6;
	3923	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	912.8;
	3923	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	990.6;
	3923	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4851;
	3923	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13476.3;
	3923	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8066.5;
	3923	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12534.9;
	3923	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2182.6;
	3924	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1026.9;
	3924	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	928.7;
	3924	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1053.7;
	3924	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5162.6;
	3924	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14464.4;
	3924	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8589;
	3924	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13048.3;
	3924	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2302.9;
	3925	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1065.7;
	3925	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	947.5;
	3925	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1116.7;
	3925	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5400.8;
	3925	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15234.2;
	3925	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9056.1;
	3925	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12991.8;
	3925	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2328.6;
	3926	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1094.5;
	3926	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	984.1;
	3926	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1137.4;
	3926	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5573.8;
	3926	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15827.2;
	3926	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9418;
	3926	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12696.3;
	3926	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2363;
	3927	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1108.6;
	3927	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1014.1;
	3927	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1166;
	3927	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5650.7;
	3927	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15907.8;
	3927	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9671.5;
	3927	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12111.4;
	3927	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2401.6;
	3928	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1115.1;
	3928	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1028.6;
	3928	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1188.6;
	3928	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5644;
	3928	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15291;
	3928	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9854;
	3928	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11505.2;
	3928	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2323.2;
	3929	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1114.1;
	3929	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1030.4;
	3929	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1205.4;
	3929	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5570.7;
	3929	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14397.3;
	3929	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9892.1;
	3929	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11195.4;
	3929	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2226.1;
	3930	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1106.8;
	3930	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1006.1;
	3930	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1204.6;
	3930	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5445.6;
	3930	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13622.8;
	3930	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9770.2;
	3930	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11005.6;
	3930	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2196.6;
	3931	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1091.5;
	3931	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	970.9;
	3931	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1179.2;
	3931	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5260.9;
	3931	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13017.3;
	3931	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9475.5;
	3931	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10989.5;
	3931	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2091.1;
	3932	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1064.2;
	3932	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	934.7;
	3932	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1134.1;
	3932	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5146.2;
	3932	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12597.3;
	3932	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9156.6;
	3932	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11168.4;
	3932	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2125;
	3933	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1059;
	3933	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	915.6;
	3933	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1118.4;
	3933	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5082.3;
	3933	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12534.9;
	3933	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8978.2;
	3933	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11277.4;
	3933	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2072.7;
	3934	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1029.6;
	3934	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	865.8;
	3934	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1052.4;
	3934	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4796.3;
	3934	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11803;
	3934	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8393.9;
	3934	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10914.1;
	3934	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1926.7;
	3935	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985;
	3935	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	806.3;
	3935	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	966.8;
	3935	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4422.9;
	3935	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10797.9;
	3935	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7650.2;
	3935	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10293.9;
	3935	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1802.9;
	3936	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.7;
	3936	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753.1;
	3936	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	951.3;
	3936	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4141.8;
	3936	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9824.1;
	3936	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6880.7;
	3936	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9924.9;
	3936	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1691.4;
	3937	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.5;
	3937	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723;
	3937	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	904.5;
	3937	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3910.5;
	3937	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9253.8;
	3937	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6390.9;
	3937	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9550.6;
	3937	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1626.8;
	3938	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.5;
	3938	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	707.1;
	3938	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	876.4;
	3938	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3769.1;
	3938	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8940.7;
	3938	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6054.7;
	3938	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9377.4;
	3938	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1571.3;
	3939	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.9;
	3939	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	693.8;
	3939	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	858;
	3939	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3672.4;
	3939	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8816.4;
	3939	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5847.5;
	3939	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9390;
	3939	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1541.4;
	3940	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.6;
	3940	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	694.1;
	3940	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851.2;
	3940	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3631.8;
	3940	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8969.6;
	3940	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5792.6;
	3940	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9635.3;
	3940	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1542.4;
	3941	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.4;
	3941	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719.9;
	3941	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.5;
	3941	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3712.2;
	3941	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9531.2;
	3941	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6037.5;
	3941	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10127.3;
	3941	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1616.9;
	3942	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.9;
	3942	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	763;
	3942	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	906.8;
	3942	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3880.6;
	3942	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10329.5;
	3942	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6421.1;
	3942	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10583.2;
	3942	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1717.6;
	3943	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.6;
	3943	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	793.3;
	3943	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	939.1;
	3943	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3978;
	3943	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10834.3;
	3943	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6629.1;
	3943	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10926.5;
	3943	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1813.1;
	3944	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	979.7;
	3944	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	838.7;
	3944	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	998.9;
	3944	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4278.3;
	3944	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11489.1;
	3944	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6941.7;
	3944	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11576.3;
	3944	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1882.5;
	3945	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1028.3;
	3945	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	871.6;
	3945	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1069.1;
	3945	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4642.4;
	3945	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12376.8;
	3945	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7412.8;
	3945	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12454.9;
	3945	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1953;
	3946	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1070.5;
	3946	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	913.9;
	3946	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1140;
	3946	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5067.2;
	3946	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13507.1;
	3946	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7927.4;
	3946	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13370.2;
	3946	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2047.2;
	3947	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1124.7;
	3947	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	930.2;
	3947	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1212.4;
	3947	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5441.1;
	3947	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14617.9;
	3947	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8431.9;
	3947	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14258.5;
	3947	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2123.5;
	3948	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1159.3;
	3948	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	931.1;
	3948	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1278;
	3948	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5722.5;
	3948	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15564.8;
	3948	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8936.5;
	3948	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14956.9;
	3948	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2268.2;
	3949	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1191.7;
	3949	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	947.6;
	3949	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1336;
	3949	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5953;
	3949	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16351;
	3949	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9446.1;
	3949	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15510.3;
	3949	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2402.9;
	3950	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1205.5;
	3950	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	972.4;
	3950	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1369.5;
	3950	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6098.9;
	3950	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15801.8;
	3950	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9840.3;
	3950	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15932.5;
	3950	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2459.2;
	3951	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1214.6;
	3951	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1006.8;
	3951	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1400.2;
	3951	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6178.3;
	3951	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14824.7;
	3951	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10159.6;
	3951	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16167.3;
	3951	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2529.3;
	3952	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1230.7;
	3952	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1035.5;
	3952	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1427.7;
	3952	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6176.6;
	3952	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14404.4;
	3952	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10349.8;
	3952	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16288.3;
	3952	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2553.2;
	3953	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1218.8;
	3953	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1054.7;
	3953	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1423.3;
	3953	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6051.9;
	3953	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14147.9;
	3953	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10371.8;
	3953	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16062.7;
	3953	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2497.7;
	3954	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1206.5;
	3954	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1053.5;
	3954	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1404.5;
	3954	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5867.8;
	3954	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14018;
	3954	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10234.6;
	3954	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15490.3;
	3954	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2411.4;
	3955	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1182.9;
	3955	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1035.7;
	3955	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1355.9;
	3955	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5631;
	3955	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13909.7;
	3955	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9887.6;
	3955	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14872.9;
	3955	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2335.3;
	3956	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1150.2;
	3956	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1004.1;
	3956	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1294.1;
	3956	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5477.4;
	3956	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13726.4;
	3956	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9519.8;
	3956	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14475.5;
	3956	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2294;
	3957	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1133.6;
	3957	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	986.9;
	3957	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1267.1;
	3957	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5371;
	3957	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13624.4;
	3957	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9230.4;
	3957	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14009.4;
	3957	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2242.3;
	3958	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1098.1;
	3958	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	935.8;
	3958	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1191.6;
	3958	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5017.1;
	3958	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12700.1;
	3958	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8537.1;
	3958	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13049.5;
	3958	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2062.2;
	3959	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1049.6;
	3959	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	867.1;
	3959	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1101.2;
	3959	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4603.5;
	3959	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11507.2;
	3959	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7736.4;
	3959	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12069.7;
	3959	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1926.4;
	3960	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1003.8;
	3960	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	800.7;
	3960	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1017.5;
	3960	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4295.5;
	3960	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10430.8;
	3960	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7012.9;
	3960	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11304.4;
	3960	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1778.4;
	3961	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981.9;
	3961	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	768.7;
	3961	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	969.1;
	3961	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4048.7;
	3961	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9769.5;
	3961	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6550.5;
	3961	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10728.8;
	3961	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1667.9;
	3962	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.2;
	3962	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	732.5;
	3962	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	934.4;
	3962	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3856.6;
	3962	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9327;
	3962	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6235.3;
	3962	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10266.6;
	3962	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1542.5;
	3963	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946;
	3963	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	709.7;
	3963	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913.9;
	3963	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3745.9;
	3963	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9130.7;
	3963	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6040.3;
	3963	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10016.5;
	3963	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1556.3;
	3964	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.4;
	3964	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	704.5;
	3964	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	903.5;
	3964	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3696.1;
	3964	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9198;
	3964	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6005.3;
	3964	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10003;
	3964	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1583.3;
	3965	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.5;
	3965	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	722.7;
	3965	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913.1;
	3965	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3760.3;
	3965	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9715.2;
	3965	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6191.5;
	3965	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10373.6;
	3965	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1594;
	3966	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.5;
	3966	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	759.5;
	3966	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	948.3;
	3966	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3917.1;
	3966	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10397;
	3966	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6485.6;
	3966	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10705.1;
	3966	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1760.2;
	3967	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.3;
	3967	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	787.4;
	3967	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	977.8;
	3967	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4038.4;
	3967	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10984.8;
	3967	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6679.2;
	3967	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11103.6;
	3967	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1846.1;
	3968	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986;
	3968	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	835.6;
	3968	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1022.7;
	3968	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4321.6;
	3968	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11564.1;
	3968	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6947.8;
	3968	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11924.7;
	3968	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1959.3;
	3969	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1021.1;
	3969	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	881.2;
	3969	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1084.6;
	3969	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4676.9;
	3969	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12295.3;
	3969	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7305.1;
	3969	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12879.9;
	3969	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2096.7;
	3970	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1063.8;
	3970	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	939.5;
	3970	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1150.8;
	3970	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5064.3;
	3970	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13273.3;
	3970	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7684.5;
	3970	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13863;
	3970	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2246.2;
	3971	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1110.5;
	3971	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	990.5;
	3971	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1215.7;
	3971	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5406.2;
	3971	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14125.5;
	3971	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8102.8;
	3971	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14729.9;
	3971	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2361;
	3972	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1147.5;
	3972	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1056.9;
	3972	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1272.5;
	3972	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5687.5;
	3972	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14830.3;
	3972	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8583.2;
	3972	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15460.7;
	3972	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2482.9;
	3973	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1182.9;
	3973	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1117;
	3973	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1327.9;
	3973	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5928.7;
	3973	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15654.5;
	3973	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9046.1;
	3973	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16098.8;
	3973	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2623.9;
	3974	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1212.4;
	3974	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1170;
	3974	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1379.4;
	3974	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6125.9;
	3974	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16369.8;
	3974	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9532;
	3974	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16532.6;
	3974	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2697.1;
	3975	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1233.4;
	3975	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1204.3;
	3975	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1414.4;
	3975	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6237.4;
	3975	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17033.9;
	3975	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9956.9;
	3975	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16744.4;
	3975	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2741.1;
	3976	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1236.9;
	3976	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1221.1;
	3976	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1427.2;
	3976	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6238.7;
	3976	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17537.4;
	3976	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10251.8;
	3976	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16690.2;
	3976	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2720.5;
	3977	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1224.5;
	3977	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1225.5;
	3977	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1416;
	3977	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6111.3;
	3977	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17706.6;
	3977	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10320.7;
	3977	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16524.9;
	3977	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2775.6;
	3978	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1216;
	3978	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1205.4;
	3978	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1392.3;
	3978	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5912.2;
	3978	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17561.5;
	3978	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10163.8;
	3978	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16075.4;
	3978	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2677.9;
	3979	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1197.9;
	3979	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1169.2;
	3979	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1340.5;
	3979	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5674.4;
	3979	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17008.9;
	3979	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9784.1;
	3979	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15490;
	3979	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2593.2;
	3980	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1169.3;
	3980	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1120.9;
	3980	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1285.2;
	3980	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5510.4;
	3980	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16283.3;
	3980	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9420.7;
	3980	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14979.8;
	3980	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2536.8;
	3981	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1161.7;
	3981	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1086.2;
	3981	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1258.6;
	3981	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5402.1;
	3981	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15726.3;
	3981	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9126.7;
	3981	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14477.8;
	3981	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2444.5;
	3982	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1116.6;
	3982	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1033.8;
	3982	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1187;
	3982	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5081.5;
	3982	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14484.4;
	3982	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8394.7;
	3982	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13543.4;
	3982	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2264.4;
	3983	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1057.4;
	3983	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	950.4;
	3983	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1093.3;
	3983	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4687.2;
	3983	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13034.7;
	3983	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7509.8;
	3983	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12479.7;
	3983	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2074;
	3984	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1017.1;
	3984	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	886.1;
	3984	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1034.9;
	3984	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4207.7;
	3984	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11891.7;
	3984	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7038.8;
	3984	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11529.8;
	3984	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1859.4;
	3985	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	991.1;
	3985	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	843.7;
	3985	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	983.1;
	3985	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3963.4;
	3985	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11194;
	3985	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6574.5;
	3985	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10923.5;
	3985	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1716.4;
	3986	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.2;
	3986	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	807.3;
	3986	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	945.8;
	3986	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3793.7;
	3986	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10729.6;
	3986	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6294.4;
	3986	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10495.7;
	3986	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1635.6;
	3987	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.7;
	3987	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	789.3;
	3987	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	918;
	3987	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3689;
	3987	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10495.7;
	3987	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6138.9;
	3987	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10280.2;
	3987	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1588.9;
	3988	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937;
	3988	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776.9;
	3988	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	900.1;
	3988	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3627.7;
	3988	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10420;
	3988	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6105.6;
	3988	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10311.6;
	3988	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1630.1;
	3989	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.1;
	3989	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	787.3;
	3989	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	908;
	3989	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3694;
	3989	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10777.3;
	3989	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6307.6;
	3989	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10756.1;
	3989	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1643.6;
	3990	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.1;
	3990	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	808.3;
	3990	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940.4;
	3990	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3844.8;
	3990	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11316.8;
	3990	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6710.7;
	3990	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11184.5;
	3990	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1769.3;
	3991	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.3;
	3991	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	860.8;
	3991	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	974;
	3991	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3951.6;
	3991	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11830.1;
	3991	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6877.2;
	3991	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11601.2;
	3991	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1855.4;
	3992	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986.8;
	3992	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	906.2;
	3992	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1029.4;
	3992	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4248.7;
	3992	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12515;
	3992	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7208.2;
	3992	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12395.4;
	3992	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1971.6;
	3993	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1034.5;
	3993	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	975;
	3993	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1096.8;
	3993	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4643.1;
	3993	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13473.1;
	3993	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7761.6;
	3993	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13397.5;
	3993	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2113.4;
	3994	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1073.8;
	3994	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1048.2;
	3994	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1166.2;
	3994	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5047.3;
	3994	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14591.1;
	3994	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8352.2;
	3994	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14502.9;
	3994	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2254.4;
	3995	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1115.1;
	3995	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1116.4;
	3995	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1239.7;
	3995	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5397.4;
	3995	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15727.3;
	3995	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8929.4;
	3995	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15425;
	3995	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2395.5;
	3996	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1149.1;
	3996	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1176.5;
	3996	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1306.5;
	3996	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5706.6;
	3996	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16741.5;
	3996	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9471.4;
	3996	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16116.7;
	3996	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2519.6;
	3997	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1183.5;
	3997	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1233;
	3997	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1367.8;
	3997	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5951;
	3997	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17648.8;
	3997	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9976.1;
	3997	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16640.4;
	3997	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2669.8;
	3998	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1210.5;
	3998	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1280.5;
	3998	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1423.8;
	3998	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6149.2;
	3998	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18413;
	3998	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10340.5;
	3998	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17142.1;
	3998	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2750;
	3999	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1226.9;
	3999	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1311.6;
	3999	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1462;
	3999	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6248.5;
	3999	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18914;
	3999	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10599;
	3999	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17348.3;
	3999	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2781.2;
	4000	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1235.6;
	4000	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1317.5;
	4000	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1483.6;
	4000	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6243.9;
	4000	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19298.9;
	4000	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10755.5;
	4000	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17303.9;
	4000	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2799.6;
	4001	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1226.6;
	4001	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1308.1;
	4001	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1473.6;
	4001	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6159.3;
	4001	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19357.1;
	4001	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10833;
	4001	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17081.8;
	4001	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2809.2;
	4002	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1197.4;
	4002	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1279.5;
	4002	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1447.8;
	4002	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5991.6;
	4002	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19016.7;
	4002	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10797.2;
	4002	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16534.2;
	4002	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2721.1;
	4003	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1154;
	4003	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1236.3;
	4003	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1397.9;
	4003	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5737.7;
	4003	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18264;
	4003	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10377.3;
	4003	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15865.3;
	4003	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2577.2;
	4004	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1077.5;
	4004	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1176.8;
	4004	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1322.6;
	4004	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5543.9;
	4004	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17293.1;
	4004	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9966.8;
	4004	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15253.2;
	4004	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2509.9;
	4005	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1038.8;
	4005	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1142.4;
	4005	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1284.3;
	4005	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5431.2;
	4005	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16650.8;
	4005	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9553.4;
	4005	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14679.8;
	4005	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2422.9;
	4006	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	988.6;
	4006	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1064.6;
	4006	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1200.3;
	4006	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5087.7;
	4006	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15296.7;
	4006	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8788.9;
	4006	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13684.4;
	4006	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2194.4;
	4007	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.6;
	4007	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	975.2;
	4007	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1104.1;
	4007	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4676.1;
	4007	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13795.8;
	4007	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7902.5;
	4007	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12653.1;
	4007	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2022.4;
	4008	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.4;
	4008	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	911;
	4008	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1004.2;
	4008	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4290.3;
	4008	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12590.9;
	4008	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7146.8;
	4008	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11792.6;
	4008	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1878.4;
	4009	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.6;
	4009	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	861.9;
	4009	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	949.5;
	4009	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4023.9;
	4009	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11699.4;
	4009	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6608.2;
	4009	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11137.4;
	4009	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1787.4;
	4010	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871.5;
	4010	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	831.4;
	4010	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	906.2;
	4010	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3832.7;
	4010	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11052.3;
	4010	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6232.2;
	4010	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10646.6;
	4010	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1709.1;
	4011	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.1;
	4011	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	805.6;
	4011	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	879.4;
	4011	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3670.1;
	4011	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10705.3;
	4011	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5991.6;
	4011	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10403.7;
	4011	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1684.4;
	4012	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	869.7;
	4012	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	794.1;
	4012	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	870.6;
	4012	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3608.8;
	4012	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10633;
	4012	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5930.7;
	4012	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10437.6;
	4012	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1668.3;
	4013	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.8;
	4013	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	815.7;
	4013	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	879.9;
	4013	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3667.8;
	4013	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10975.5;
	4013	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6124.8;
	4013	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10811.2;
	4013	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1721;
	4014	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892;
	4014	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	845.8;
	4014	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	910.6;
	4014	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3834.7;
	4014	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11558.4;
	4014	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6498.7;
	4014	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11162.3;
	4014	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1801.7;
	4015	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.3;
	4015	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	872;
	4015	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	946.8;
	4015	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3955.7;
	4015	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12168;
	4015	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6710.4;
	4015	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11582.5;
	4015	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1885;
	4016	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.5;
	4016	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	930.3;
	4016	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1008.1;
	4016	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4239.1;
	4016	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12982.7;
	4016	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6999.5;
	4016	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12437.6;
	4016	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2017.6;
	4017	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1002.7;
	4017	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	990.7;
	4017	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1081.9;
	4017	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4626.1;
	4017	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14088.7;
	4017	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7504.3;
	4017	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13456.2;
	4017	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2186.4;
	4018	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1049.2;
	4018	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1055.9;
	4018	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1163.1;
	4018	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5054;
	4018	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15225.8;
	4018	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8111.2;
	4018	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14559.2;
	4018	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2350.2;
	4019	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1093.8;
	4019	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1119.3;
	4019	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1233.4;
	4019	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5430.7;
	4019	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16307.4;
	4019	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8703.3;
	4019	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15546.6;
	4019	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2490.2;
	4020	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1126.2;
	4020	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1165.9;
	4020	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1298.8;
	4020	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5742.6;
	4020	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17242.2;
	4020	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9247.6;
	4020	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16239.5;
	4020	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2643.8;
	4021	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1161.2;
	4021	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1222.8;
	4021	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1357.3;
	4021	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5994.9;
	4021	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18134.2;
	4021	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9727.1;
	4021	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16859.9;
	4021	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2754.4;
	4022	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1189.7;
	4022	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1278.8;
	4022	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1409.5;
	4022	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6184.5;
	4022	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18822.4;
	4022	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10080;
	4022	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17336.5;
	4022	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2836.3;
	4023	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1208.6;
	4023	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1313.4;
	4023	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1447.3;
	4023	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6252.4;
	4023	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19240.2;
	4023	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10338.3;
	4023	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17419.4;
	4023	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2902.7;
	4024	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1218;
	4024	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1321.5;
	4024	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1463.6;
	4024	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6222.4;
	4024	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19438;
	4024	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10461;
	4024	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17315.3;
	4024	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2877.6;
	4025	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1217.8;
	4025	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1315;
	4025	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1453.8;
	4025	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6150.8;
	4025	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19428.2;
	4025	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10533.7;
	4025	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17135.8;
	4025	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2835;
	4026	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1206.2;
	4026	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1293.6;
	4026	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1421.2;
	4026	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5981.4;
	4026	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19033.3;
	4026	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10547.9;
	4026	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16627;
	4026	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2799;
	4027	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1191.4;
	4027	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1250.9;
	4027	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1373.1;
	4027	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5750.4;
	4027	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18232.2;
	4027	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10160.7;
	4027	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16114.6;
	4027	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2728.9;
	4028	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1154.5;
	4028	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1195.6;
	4028	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1301.3;
	4028	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5548.3;
	4028	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17368.2;
	4028	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9669.7;
	4028	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15565.6;
	4028	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2598.4;
	4029	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1135.8;
	4029	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1158.3;
	4029	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1271.9;
	4029	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5415.1;
	4029	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16767.4;
	4029	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9309.6;
	4029	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15033.3;
	4029	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2506.8;
	4030	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1097.7;
	4030	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1075.4;
	4030	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1195.3;
	4030	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5066.2;
	4030	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15469.8;
	4030	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8577.8;
	4030	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13954.4;
	4030	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2336.3;
	4031	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1046;
	4031	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	993.7;
	4031	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1109.5;
	4031	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4680.9;
	4031	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14103.7;
	4031	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7795.3;
	4031	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12912.4;
	4031	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2141.6;
	4032	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1012.8;
	4032	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	919;
	4032	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1050.5;
	4032	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4295.5;
	4032	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12878.9;
	4032	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7043.9;
	4032	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11952.1;
	4032	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1950.8;
	4033	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986.6;
	4033	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	872.5;
	4033	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1000.1;
	4033	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4022.8;
	4033	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11995.9;
	4033	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6563.5;
	4033	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11301.4;
	4033	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1815.8;
	4034	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.7;
	4034	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	837.4;
	4034	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	952.3;
	4034	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3818.6;
	4034	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11348.5;
	4034	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6236.9;
	4034	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10806;
	4034	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1732.5;
	4035	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.1;
	4035	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	812.3;
	4035	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	925.3;
	4035	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3682.2;
	4035	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10908.3;
	4035	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6031.5;
	4035	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10569.5;
	4035	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1619.1;
	4036	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.4;
	4036	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	799.2;
	4036	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	907.5;
	4036	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3601.4;
	4036	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10713.3;
	4036	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5964.4;
	4036	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10555.5;
	4036	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1607.8;
	4037	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.8;
	4037	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	814;
	4037	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913.4;
	4037	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3640.4;
	4037	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10874.7;
	4037	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6114.9;
	4037	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10783.7;
	4037	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1716;
	4038	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.6;
	4038	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	824.5;
	4038	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	938.2;
	4038	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3761.1;
	4038	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11262.2;
	4038	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6427.7;
	4038	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11003.9;
	4038	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1751;
	4039	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.5;
	4039	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	862.3;
	4039	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	967.4;
	4039	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3870.1;
	4039	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11707.8;
	4039	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6610.6;
	4039	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11430.8;
	4039	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1842.8;
	4040	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	995;
	4040	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	920.6;
	4040	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1028.2;
	4040	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4202.2;
	4040	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12608.5;
	4040	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6984.5;
	4040	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12267.8;
	4040	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1971.9;
	4041	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1037.8;
	4041	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	989.8;
	4041	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1096.8;
	4041	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4570.6;
	4041	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13601.4;
	4041	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7494.7;
	4041	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13204.1;
	4041	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2113.1;
	4042	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1078.6;
	4042	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1064;
	4042	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1168;
	4042	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4996.8;
	4042	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14766.9;
	4042	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8065.3;
	4042	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14204.8;
	4042	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2275.7;
	4043	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1124.3;
	4043	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1139.3;
	4043	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1240.6;
	4043	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5397.3;
	4043	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15954.8;
	4043	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8617.7;
	4043	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15138;
	4043	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2411.6;
	4044	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1163.9;
	4044	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1197.9;
	4044	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1310.2;
	4044	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5741.3;
	4044	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16981;
	4044	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9129.6;
	4044	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15917.8;
	4044	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2554.5;
	4045	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1204.1;
	4045	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1262.1;
	4045	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1376.1;
	4045	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6038.4;
	4045	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17977.2;
	4045	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9618.9;
	4045	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16660.3;
	4045	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2685.7;
	4046	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1234.5;
	4046	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1312.6;
	4046	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1439.2;
	4046	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6255.9;
	4046	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18763.4;
	4046	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10040.2;
	4046	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17080.9;
	4046	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2783.9;
	4047	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1248.4;
	4047	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1335.5;
	4047	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1488.4;
	4047	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6333.5;
	4047	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19276;
	4047	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10381.8;
	4047	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17171.7;
	4047	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2828.9;
	4048	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1255.6;
	4048	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1343;
	4048	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1511.1;
	4048	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6312.7;
	4048	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19479.5;
	4048	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10564.5;
	4048	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17144;
	4048	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2758.7;
	4049	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1248.1;
	4049	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1329.1;
	4049	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1505.2;
	4049	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6237.4;
	4049	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19518.4;
	4049	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10626.5;
	4049	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16908.6;
	4049	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2739.3;
	4050	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1239.2;
	4050	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1306.9;
	4050	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1476.5;
	4050	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6081.2;
	4050	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19086.5;
	4050	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10497.3;
	4050	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16507.1;
	4050	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2722.3;
	4051	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1215.9;
	4051	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1262.3;
	4051	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1432.5;
	4051	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5828.1;
	4051	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18255.1;
	4051	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10109.9;
	4051	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15949.1;
	4051	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2619.9;
	4052	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1177.2;
	4052	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1203;
	4052	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1364.2;
	4052	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5620.3;
	4052	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17268.2;
	4052	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9660;
	4052	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15390.1;
	4052	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2528.3;
	4053	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1155.7;
	4053	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1164.4;
	4053	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1319.4;
	4053	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5481.9;
	4053	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16568.2;
	4053	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9306.7;
	4053	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14874.8;
	4053	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2459.4;
	4054	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1119.7;
	4054	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1103.3;
	4054	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1230.9;
	4054	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5181.5;
	4054	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15313.5;
	4054	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8660.3;
	4054	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14037;
	4054	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2289.6;
	4055	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1074.5;
	4055	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1017.8;
	4055	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1137.6;
	4055	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4820.4;
	4055	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13999.6;
	4055	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7957.1;
	4055	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13091.7;
	4055	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2109.9;
	4056	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1027.7;
	4056	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	931.5;
	4056	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1054.7;
	4056	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4480.4;
	4056	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12685.6;
	4056	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7431.1;
	4056	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12226.9;
	4056	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1936.5;
	4057	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	995.6;
	4057	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	878.3;
	4057	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	996.1;
	4057	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4206.9;
	4057	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11720.1;
	4057	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6895;
	4057	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11510.1;
	4057	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1827.4;
	4058	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969.7;
	4058	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	844;
	4058	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	953.3;
	4058	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3995.6;
	4058	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11023.4;
	4058	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6483.1;
	4058	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10948;
	4058	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1735.5;
	4059	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.1;
	4059	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	819.3;
	4059	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	922.8;
	4059	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3843.2;
	4059	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10566.4;
	4059	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6190;
	4059	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10555.5;
	4059	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1672.3;
	4060	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.1;
	4060	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	798.7;
	4060	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	904.1;
	4060	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3737.9;
	4060	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10305.3;
	4060	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6028.8;
	4060	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10342.3;
	4060	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1626;
	4061	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.5;
	4061	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	792.7;
	4061	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	894.8;
	4061	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3691;
	4061	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10279.8;
	4061	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6030.6;
	4061	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10264.9;
	4061	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1641.6;
	4062	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.8;
	4062	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	788.1;
	4062	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	893.5;
	4062	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3657;
	4062	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10291;
	4062	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6126.8;
	4062	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10167;
	4062	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1644.4;
	4063	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.4;
	4063	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	809.7;
	4063	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	914.5;
	4063	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3698.4;
	4063	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10788.5;
	4063	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6267.1;
	4063	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10500;
	4063	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1696.2;
	4064	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.2;
	4064	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	866.1;
	4064	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	988.1;
	4064	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4073.5;
	4064	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11987.2;
	4064	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6742.7;
	4064	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11496.2;
	4064	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1845.5;
	4065	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009.9;
	4065	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	942.1;
	4065	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1066;
	4065	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4480.7;
	4065	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13366.4;
	4065	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7331.6;
	4065	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12525.7;
	4065	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2022.9;
	4066	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1056.8;
	4066	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1012.5;
	4066	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1155.3;
	4066	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4911.5;
	4066	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14755.4;
	4066	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7958.3;
	4066	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13529.6;
	4066	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2171.5;
	4067	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1102.7;
	4067	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1078.4;
	4067	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1237.7;
	4067	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5317.1;
	4067	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16047.1;
	4067	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8524;
	4067	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14487.3;
	4067	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2316.2;
	4068	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1144.7;
	4068	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1134.7;
	4068	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1307;
	4068	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5642.7;
	4068	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17112.6;
	4068	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9054.8;
	4068	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15164.7;
	4068	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2439.7;
	4069	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1187.4;
	4069	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1182.2;
	4069	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1362.3;
	4069	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5875.8;
	4069	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17765.7;
	4069	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9545.5;
	4069	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15718.7;
	4069	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2548.6;
	4070	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1213.8;
	4070	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1219.8;
	4070	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1408.5;
	4070	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6040.5;
	4070	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18226.7;
	4070	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9946.5;
	4070	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16143.2;
	4070	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2623.6;
	4071	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1227.5;
	4071	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1248.7;
	4071	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1443.5;
	4071	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6128.3;
	4071	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18545.9;
	4071	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10317.3;
	4071	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16378.7;
	4071	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2651.1;
	4072	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1232.9;
	4072	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1259.4;
	4072	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1458.4;
	4072	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6147.6;
	4072	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18694.7;
	4072	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10562.9;
	4072	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16348.1;
	4072	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2622.1;
	4073	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1228.7;
	4073	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1254.8;
	4073	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1451.1;
	4073	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6085.9;
	4073	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18596.6;
	4073	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10621.7;
	4073	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15812.9;
	4073	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2526.9;
	4074	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1216.4;
	4074	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1228.8;
	4074	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1404.9;
	4074	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5900.4;
	4074	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18099.8;
	4074	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10418.5;
	4074	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14884.5;
	4074	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2457.3;
	4075	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1193.2;
	4075	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1188.6;
	4075	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1351.3;
	4075	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5650.2;
	4075	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17384.1;
	4075	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10026.1;
	4075	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13959.6;
	4075	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2315.1;
	4076	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1157.6;
	4076	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1143.3;
	4076	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1295.1;
	4076	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5459.5;
	4076	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16637.5;
	4076	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9593.3;
	4076	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13395.1;
	4076	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2210;
	4077	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1131.3;
	4077	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1116.6;
	4077	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1268.2;
	4077	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5326.9;
	4077	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16116.6;
	4077	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9254.2;
	4077	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12918.5;
	4077	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2211.5;
	4078	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1095.5;
	4078	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1064.4;
	4078	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1167.8;
	4078	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5048.2;
	4078	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15130;
	4078	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8582.2;
	4078	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12200.1;
	4078	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2081.4;
	4079	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1056.3;
	4079	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	992.7;
	4079	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1036.3;
	4079	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4734.4;
	4079	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13942.8;
	4079	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7916.1;
	4079	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11549.4;
	4079	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1922.8;
	4080	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1012;
	4080	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	927.7;
	4080	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	959.6;
	4080	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4402.8;
	4080	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12741;
	4080	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7246.9;
	4080	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10823.4;
	4080	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1772.9;
	4081	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	973;
	4081	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	880.2;
	4081	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	907.7;
	4081	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4104.9;
	4081	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11746.3;
	4081	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6716.3;
	4081	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10244.5;
	4081	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1666.9;
	4082	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.6;
	4082	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	840.2;
	4082	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878.5;
	4082	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3864.3;
	4082	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10990.2;
	4082	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6313.4;
	4082	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9792.1;
	4082	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1566.9;
	4083	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.9;
	4083	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	815.5;
	4083	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.7;
	4083	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3697.2;
	4083	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10433.7;
	4083	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6021.4;
	4083	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9493.1;
	4083	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1536.4;
	4084	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.8;
	4084	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	789;
	4084	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	837.3;
	4084	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3575.7;
	4084	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10036.6;
	4084	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5836;
	4084	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9267;
	4084	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1486.4;
	4085	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.3;
	4085	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772.5;
	4085	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	839.8;
	4085	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3521.1;
	4085	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9798.4;
	4085	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5778.7;
	4085	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9155.2;
	4085	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1464.9;
	4086	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.5;
	4086	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	761.8;
	4086	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	836.3;
	4086	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3468.2;
	4086	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9569.5;
	4086	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5800.9;
	4086	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9028.4;
	4086	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1463.4;
	4087	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.5;
	4087	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	783.4;
	4087	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	853.2;
	4087	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3482.5;
	4087	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9872.9;
	4087	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5938.1;
	4087	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9262.9;
	4087	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1484.8;
	4088	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.7;
	4088	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	848.9;
	4088	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	911.2;
	4088	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3780.7;
	4088	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10795.9;
	4088	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6323.3;
	4088	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10054.3;
	4088	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1678.9;
	4089	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.7;
	4089	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	912.3;
	4089	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	975.6;
	4089	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4125;
	4089	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12001.8;
	4089	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6820;
	4089	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11097.4;
	4089	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1843;
	4090	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951.2;
	4090	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	970.9;
	4090	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1036;
	4090	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4456.8;
	4090	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13217.6;
	4090	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7387.1;
	4090	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12158.1;
	4090	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1990.5;
	4091	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.2;
	4091	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1026.6;
	4091	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1088.3;
	4091	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4770.7;
	4091	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14340.5;
	4091	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7975;
	4091	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13046.1;
	4091	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2147.9;
	4092	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	977.7;
	4092	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1082.1;
	4092	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1160.4;
	4092	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5032.1;
	4092	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15396.3;
	4092	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8491.1;
	4092	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13695.5;
	4092	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2279.5;
	4093	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000.3;
	4093	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1123;
	4093	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1217.8;
	4093	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5237.2;
	4093	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16251;
	4093	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8857.9;
	4093	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13993.1;
	4093	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2375.4;
	4094	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1028.1;
	4094	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1155.1;
	4094	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1263.9;
	4094	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5430.3;
	4094	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16887.6;
	4094	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9162;
	4094	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14162.9;
	4094	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2439.6;
	4095	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1055.9;
	4095	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1180.2;
	4095	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1307.6;
	4095	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5531.7;
	4095	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17301.1;
	4095	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9427;
	4095	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14177.5;
	4095	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2523.2;
	4096	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1080.9;
	4096	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1200.8;
	4096	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1323.6;
	4096	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5545.5;
	4096	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17544.1;
	4096	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9611.5;
	4096	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14243.8;
	4096	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2498.9;
	4097	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1099;
	4097	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1202.1;
	4097	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1327;
	4097	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5517.8;
	4097	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17529;
	4097	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9673.3;
	4097	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14292.3;
	4097	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2430.7;
	4098	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1101.4;
	4098	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1176.3;
	4098	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1313.7;
	4098	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5339.1;
	4098	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17165.6;
	4098	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9597.9;
	4098	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14207.5;
	4098	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2421.2;
	4099	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1081.5;
	4099	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1137;
	4099	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1280.7;
	4099	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5117.9;
	4099	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16548.5;
	4099	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9293.5;
	4099	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13925.9;
	4099	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2305.6;
	4100	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1051.8;
	4100	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1086.4;
	4100	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1223.8;
	4100	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4955.5;
	4100	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15773.7;
	4100	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8952.9;
	4100	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13589.9;
	4100	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2294.1;
	4101	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1044.8;
	4101	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1057;
	4101	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1191.7;
	4101	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4874.4;
	4101	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15292.8;
	4101	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8678.1;
	4101	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13313.8;
	4101	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2231.2;
	4102	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1016.9;
	4102	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	994.8;
	4102	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1119;
	4102	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4644.5;
	4102	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14186.1;
	4102	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8109.5;
	4102	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12683.3;
	4102	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2078.6;
	4103	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	974.9;
	4103	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	914.2;
	4103	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1028.1;
	4103	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4294.5;
	4103	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12798.3;
	4103	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7318.5;
	4103	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11805.8;
	4103	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1924.1;
	4104	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.5;
	4104	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	844.8;
	4104	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	946.6;
	4104	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3915.5;
	4104	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11662.8;
	4104	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6604.4;
	4104	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11002.4;
	4104	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1761.5;
	4105	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.4;
	4105	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	801.6;
	4105	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.1;
	4105	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3667.2;
	4105	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10858.2;
	4105	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6141.2;
	4105	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10409;
	4105	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1676.2;
	4106	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.2;
	4106	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	770.9;
	4106	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860;
	4106	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3492.8;
	4106	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10345.7;
	4106	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5815.8;
	4106	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10046.1;
	4106	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1602.7;
	4107	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.8;
	4107	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752;
	4107	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	836.5;
	4107	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3383.2;
	4107	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10036.5;
	4107	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5626.7;
	4107	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9876.6;
	4107	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1562.8;
	4108	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882;
	4108	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740;
	4108	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	825.4;
	4108	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3343.1;
	4108	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9992;
	4108	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5600.6;
	4108	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9945.3;
	4108	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1547.9;
	4109	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.6;
	4109	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	757.6;
	4109	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	840.1;
	4109	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3428;
	4109	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10374.3;
	4109	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5829.9;
	4109	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10310.7;
	4109	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1608.5;
	4110	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.1;
	4110	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	784.7;
	4110	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	876.2;
	4110	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3576.6;
	4110	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10905.4;
	4110	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6206.4;
	4110	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10656;
	4110	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1672.5;
	4111	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.3;
	4111	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	836.5;
	4111	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913.1;
	4111	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3735.8;
	4111	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11576;
	4111	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6499.4;
	4111	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11151.5;
	4111	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1806.7;
	4112	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.9;
	4112	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	895.2;
	4112	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	978.9;
	4112	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4036.8;
	4112	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12529.7;
	4112	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7010.9;
	4112	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12038.1;
	4112	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1960.5;
	4113	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	998.7;
	4113	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	967.8;
	4113	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1050.7;
	4113	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4423.4;
	4113	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13666.7;
	4113	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7612;
	4113	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13071.7;
	4113	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2112.7;
	4114	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1037.4;
	4114	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1046.3;
	4114	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1115.6;
	4114	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4832.3;
	4114	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14928.7;
	4114	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8206.3;
	4114	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14026;
	4114	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2269.3;
	4115	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1077.8;
	4115	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1117;
	4115	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1185.3;
	4115	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5185.8;
	4115	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16056;
	4115	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8688.2;
	4115	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14831.1;
	4115	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2359.7;
	4116	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1101.8;
	4116	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1171.7;
	4116	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1245;
	4116	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5446.5;
	4116	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16963.9;
	4116	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9218.7;
	4116	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15453;
	4116	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2516.4;
	4117	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1125.9;
	4117	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1217.4;
	4117	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1297.1;
	4117	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5643.9;
	4117	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17746.5;
	4117	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9679.9;
	4117	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15959.4;
	4117	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2670.9;
	4118	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1136.6;
	4118	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1250.9;
	4118	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1343.1;
	4118	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5795.7;
	4118	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18316.2;
	4118	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10055.8;
	4118	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16265.8;
	4118	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2752.6;
	4119	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1143.6;
	4119	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1274.8;
	4119	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1380.4;
	4119	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5843.5;
	4119	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18720.1;
	4119	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10353.4;
	4119	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16273.8;
	4119	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2799.5;
	4120	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1163.1;
	4120	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1282.8;
	4120	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1399.9;
	4120	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5756.7;
	4120	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18983.8;
	4120	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10524.3;
	4120	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15871.5;
	4120	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2815.1;
	4121	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1156.3;
	4121	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1276.7;
	4121	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1391.2;
	4121	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5525.1;
	4121	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18993.2;
	4121	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10521.2;
	4121	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15407.8;
	4121	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2782.5;
	4122	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1144.8;
	4122	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1253.1;
	4122	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1365.5;
	4122	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5273.2;
	4122	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18611.3;
	4122	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10360.1;
	4122	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14685.3;
	4122	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2693.2;
	4123	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1126.4;
	4123	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1209.2;
	4123	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1322.2;
	4123	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5068.5;
	4123	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17955.2;
	4123	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9996.2;
	4123	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14131.1;
	4123	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2583.6;
	4124	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1093.8;
	4124	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1151.9;
	4124	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1259;
	4124	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4962.9;
	4124	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17136.9;
	4124	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9574;
	4124	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13866;
	4124	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2502.1;
	4125	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1079.1;
	4125	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1110.1;
	4125	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1229.5;
	4125	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4896.3;
	4125	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16546.7;
	4125	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9245;
	4125	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13533.5;
	4125	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2432.5;
	4126	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1048.8;
	4126	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1027.2;
	4126	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1154;
	4126	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4594;
	4126	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15264.6;
	4126	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8500.8;
	4126	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12684.8;
	4126	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2228.8;
	4127	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.1;
	4127	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	943.1;
	4127	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1055.9;
	4127	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4208.5;
	4127	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13723.9;
	4127	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7658.1;
	4127	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11755.4;
	4127	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2057.6;
	4128	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.3;
	4128	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	879.2;
	4128	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	972.5;
	4128	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3933.5;
	4128	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12405;
	4128	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6931.8;
	4128	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11028.1;
	4128	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1889.3;
	4129	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.2;
	4129	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	832;
	4129	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	917.6;
	4129	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3700.6;
	4129	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11508.6;
	4129	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6435.6;
	4129	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10473.1;
	4129	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1751.9;
	4130	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.2;
	4130	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	803.8;
	4130	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.9;
	4130	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3543.8;
	4130	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10929;
	4130	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6084.6;
	4130	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10125.7;
	4130	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1606.2;
	4131	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.6;
	4131	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	785.9;
	4131	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.2;
	4131	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3425.6;
	4131	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10570.5;
	4131	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5859.3;
	4131	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9899.2;
	4131	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1599.9;
	4132	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.5;
	4132	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	770.2;
	4132	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849.6;
	4132	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3375.4;
	4132	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10519.8;
	4132	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5787.5;
	4132	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9935;
	4132	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1608.6;
	4133	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.9;
	4133	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	780.6;
	4133	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.8;
	4133	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3438.8;
	4133	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10911.5;
	4133	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5966.9;
	4133	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10319.3;
	4133	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1597.2;
	4134	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.8;
	4134	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	802.6;
	4134	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	891.4;
	4134	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3577.9;
	4134	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11387.5;
	4134	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6260.2;
	4134	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10668.3;
	4134	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1736.9;
	4135	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.3;
	4135	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	843.7;
	4135	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	928.5;
	4135	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3718;
	4135	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12010.2;
	4135	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6560.6;
	4135	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11140.5;
	4135	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1835.9;
	4136	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.1;
	4136	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	912;
	4136	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	991.1;
	4136	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4041.8;
	4136	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12982.6;
	4136	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7067;
	4136	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11952.4;
	4136	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1988.5;
	4137	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.6;
	4137	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	981.1;
	4137	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1066;
	4137	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4439.5;
	4137	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14159.5;
	4137	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7633.6;
	4137	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12896.5;
	4137	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2170.1;
	4138	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1041.5;
	4138	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1063.3;
	4138	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1146.1;
	4138	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4861.8;
	4138	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15499;
	4138	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8243.5;
	4138	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13886.1;
	4138	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2343.3;
	4139	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1077.6;
	4139	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1139.5;
	4139	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1216;
	4139	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5278.1;
	4139	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16727.5;
	4139	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8851.3;
	4139	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14512.4;
	4139	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2487.9;
	4140	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1100.5;
	4140	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1200.6;
	4140	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1280.7;
	4140	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5566.8;
	4140	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17746;
	4140	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9405.6;
	4140	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14915;
	4140	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2618.6;
	4141	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1132;
	4141	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1254.9;
	4141	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1341.2;
	4141	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5845.5;
	4141	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18576.4;
	4141	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9889.8;
	4141	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15304.1;
	4141	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2752.6;
	4142	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1154.5;
	4142	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1292.3;
	4142	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1385.8;
	4142	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6044;
	4142	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19116.5;
	4142	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10295;
	4142	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15454.9;
	4142	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2801.8;
	4143	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1170.3;
	4143	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1312.9;
	4143	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1419.5;
	4143	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6154.9;
	4143	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19491.8;
	4143	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10600.1;
	4143	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15271;
	4143	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2819.1;
	4144	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1177.5;
	4144	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1322;
	4144	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1437.3;
	4144	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6151.2;
	4144	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19686.9;
	4144	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10793.2;
	4144	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14873.1;
	4144	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2788.6;
	4145	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1169.5;
	4145	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1318.1;
	4145	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1428.5;
	4145	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5936.9;
	4145	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19576;
	4145	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10790.9;
	4145	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14521.6;
	4145	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2824.4;
	4146	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1161;
	4146	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1295.5;
	4146	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1408;
	4146	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5730.8;
	4146	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19241.9;
	4146	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10607;
	4146	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14221.7;
	4146	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2724;
	4147	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1141.8;
	4147	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1251.5;
	4147	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1355.4;
	4147	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5511.5;
	4147	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18542.5;
	4147	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10282.2;
	4147	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13913.3;
	4147	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2630.8;
	4148	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1111.2;
	4148	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1191.5;
	4148	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1289.4;
	4148	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5338.1;
	4148	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17672.9;
	4148	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9783.8;
	4148	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13648.5;
	4148	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2557.7;
	4149	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1098.6;
	4149	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1155.9;
	4149	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1249.8;
	4149	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5241.9;
	4149	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17083.8;
	4149	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9451.9;
	4149	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13394.6;
	4149	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2465.3;
	4150	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1055.2;
	4150	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1083.1;
	4150	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1168;
	4150	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4913.1;
	4150	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15808.1;
	4150	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8714.1;
	4150	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12657.7;
	4150	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2295.9;
	4151	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000.2;
	4151	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	996.3;
	4151	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1063.6;
	4151	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4497;
	4151	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14321.3;
	4151	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7862.4;
	4151	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11747;
	4151	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2106.6;
	4152	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.3;
	4152	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	939.5;
	4152	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	986.4;
	4152	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4048.9;
	4152	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13214.9;
	4152	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7146.2;
	4152	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10977.8;
	4152	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1891.1;
	4153	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.7;
	4153	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	897.6;
	4153	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	937.4;
	4153	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3797.1;
	4153	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12401.8;
	4153	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6634.3;
	4153	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10444.3;
	4153	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1749.3;
	4154	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.6;
	4154	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	861.1;
	4154	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	905.6;
	4154	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3615.5;
	4154	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11818;
	4154	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6263.9;
	4154	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10091.5;
	4154	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1666.4;
	4155	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925.3;
	4155	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	840;
	4155	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.5;
	4155	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3488.1;
	4155	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11458;
	4155	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6015.5;
	4155	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9916.2;
	4155	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1617.4;
	4156	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.1;
	4156	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	835.5;
	4156	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	871.2;
	4156	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3425.6;
	4156	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11364;
	4156	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5912.5;
	4156	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9981;
	4156	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1655.4;
	4157	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.9;
	4157	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	849.7;
	4157	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.9;
	4157	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3471.4;
	4157	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11714.2;
	4157	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6071.2;
	4157	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10428.8;
	4157	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1668.8;
	4158	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.4;
	4158	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	867.9;
	4158	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	921.2;
	4158	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3603.2;
	4158	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12169.2;
	4158	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6388.8;
	4158	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10858.9;
	4158	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1783.3;
	4159	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.3;
	4159	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	900.2;
	4159	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	957.2;
	4159	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3696.4;
	4159	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12692.6;
	4159	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6655.4;
	4159	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11232.8;
	4159	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1865.9;
	4160	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	977.6;
	4160	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	966.4;
	4160	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1014.3;
	4160	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4014.9;
	4160	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13562.2;
	4160	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7114.2;
	4160	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11893.4;
	4160	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2007.4;
	4161	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1024.6;
	4161	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1033.1;
	4161	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1079.4;
	4161	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4431.7;
	4161	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14620.1;
	4161	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7678;
	4161	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12789.9;
	4161	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2167.8;
	4162	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1057.9;
	4162	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1108.6;
	4162	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1144.2;
	4162	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4863.1;
	4162	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15834.8;
	4162	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8317.1;
	4162	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13637.1;
	4162	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2323.9;
	4163	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1100.5;
	4163	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1176;
	4163	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1215.3;
	4163	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5271.7;
	4163	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16975.9;
	4163	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8934.9;
	4163	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14227.6;
	4163	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2458.9;
	4164	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1135.1;
	4164	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1234.7;
	4164	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1277.3;
	4164	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5606.1;
	4164	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17931;
	4164	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9487.6;
	4164	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14653.5;
	4164	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2572.2;
	4165	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1168.9;
	4165	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1287.4;
	4165	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1332.6;
	4165	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5864.6;
	4165	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18750.2;
	4165	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9950.8;
	4165	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15145.3;
	4165	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2725;
	4166	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1192.5;
	4166	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1323.8;
	4166	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1380.1;
	4166	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6038.6;
	4166	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19302.8;
	4166	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10285.4;
	4166	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15693.5;
	4166	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2777.3;
	4167	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1214.7;
	4167	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1351.2;
	4167	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1420.1;
	4167	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6169.2;
	4167	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19603.4;
	4167	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10573.1;
	4167	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16193.8;
	4167	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2807.9;
	4168	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1228.8;
	4168	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1361;
	4168	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1446.9;
	4168	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6212.9;
	4168	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19746.4;
	4168	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10756.8;
	4168	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16350.3;
	4168	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2818.3;
	4169	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1227.1;
	4169	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1347.1;
	4169	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1442.8;
	4169	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6119.9;
	4169	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19622;
	4169	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10767.7;
	4169	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16096.1;
	4169	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2795.4;
	4170	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1212.5;
	4170	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1317.5;
	4170	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1420.5;
	4170	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5928.4;
	4170	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19214.8;
	4170	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10574;
	4170	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15544.2;
	4170	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2698;
	4171	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1191.1;
	4171	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1277.9;
	4171	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1377.2;
	4171	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5664.3;
	4171	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18498.2;
	4171	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10225.2;
	4171	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14960.5;
	4171	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2572.1;
	4172	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1143.8;
	4172	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1213.4;
	4172	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1308.4;
	4172	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5446.8;
	4172	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17600.4;
	4172	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9847.6;
	4172	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14485.8;
	4172	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2510.6;
	4173	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1114.9;
	4173	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1182;
	4173	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1268.9;
	4173	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5326.4;
	4173	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17073.7;
	4173	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9570.9;
	4173	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14009.4;
	4173	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2436;
	4174	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1067.4;
	4174	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1105.4;
	4174	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1184.5;
	4174	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4965.3;
	4174	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15785.3;
	4174	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8854.1;
	4174	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13071.7;
	4174	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2220;
	4175	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1015.8;
	4175	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1022.4;
	4175	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1089.5;
	4175	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4539.3;
	4175	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14410.1;
	4175	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7989.4;
	4175	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12133.1;
	4175	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2056.9;
	4176	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	978.2;
	4176	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	951;
	4176	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1008.3;
	4176	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4174.5;
	4176	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13333.7;
	4176	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7189.4;
	4176	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11326.8;
	4176	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1908.6;
	4177	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.7;
	4177	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	906.9;
	4177	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	955.1;
	4177	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3883.1;
	4177	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12526.3;
	4177	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6703.7;
	4177	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10721.6;
	4177	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1813.9;
	4178	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920.4;
	4178	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	874.1;
	4178	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	916;
	4178	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3672.5;
	4178	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11899.7;
	4178	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6326.3;
	4178	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10306;
	4178	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1737.6;
	4179	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.8;
	4179	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	845.7;
	4179	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	887.8;
	4179	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3475.7;
	4179	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11519.9;
	4179	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6042.3;
	4179	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10018;
	4179	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1696.4;
	4180	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906;
	4180	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	834.1;
	4180	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	877.9;
	4180	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3393.6;
	4180	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11444.6;
	4180	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5916.8;
	4180	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10023.7;
	4180	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1660.2;
	4181	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.7;
	4181	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	847;
	4181	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	880.7;
	4181	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3422.4;
	4181	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11740.4;
	4181	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6036.2;
	4181	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10347.9;
	4181	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1708.7;
	4182	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.8;
	4182	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	873.9;
	4182	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	904.9;
	4182	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3538.1;
	4182	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12191.4;
	4182	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6359;
	4182	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10649;
	4182	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1770.6;
	4183	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.8;
	4183	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	906;
	4183	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940.7;
	4183	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3651.3;
	4183	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12647.4;
	4183	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6603.1;
	4183	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11085.6;
	4183	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1850.1;
	4184	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972;
	4184	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	971.9;
	4184	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	997.7;
	4184	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3978.4;
	4184	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13449.3;
	4184	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7015.8;
	4184	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11953.1;
	4184	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1984.4;
	4185	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.1;
	4185	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1029.3;
	4185	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1057;
	4185	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4353.9;
	4185	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14454.9;
	4185	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7527.8;
	4185	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12885.9;
	4185	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2144.2;
	4186	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1056.6;
	4186	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1101.2;
	4186	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1129.8;
	4186	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4764.1;
	4186	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15501.2;
	4186	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8110.9;
	4186	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13866.9;
	4186	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2300.6;
	4187	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1098.9;
	4187	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1162.4;
	4187	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1202.2;
	4187	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5134.5;
	4187	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16457.6;
	4187	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8679.7;
	4187	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14709.9;
	4187	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2410.5;
	4188	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1132.7;
	4188	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1222.1;
	4188	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1268.9;
	4188	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5470.1;
	4188	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17345.9;
	4188	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9220.8;
	4188	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15419;
	4188	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2546.8;
	4189	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1166.5;
	4189	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1274;
	4189	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1329.9;
	4189	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5761.6;
	4189	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18186.2;
	4189	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9690.7;
	4189	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16053.4;
	4189	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2642.3;
	4190	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1193.5;
	4190	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1313.5;
	4190	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1384.4;
	4190	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5991.9;
	4190	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18869.1;
	4190	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10039.3;
	4190	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16471.6;
	4190	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2725.9;
	4191	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1212.6;
	4191	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1337.9;
	4191	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1424.2;
	4191	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6086;
	4191	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19274.8;
	4191	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10322.6;
	4191	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16591.5;
	4191	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2793.7;
	4192	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1222.9;
	4192	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1342.9;
	4192	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1443.5;
	4192	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6095.1;
	4192	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19496.3;
	4192	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10506.3;
	4192	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16586.6;
	4192	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2789.4;
	4193	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1211.3;
	4193	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1333.6;
	4193	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1433.9;
	4193	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5948.4;
	4193	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19408.6;
	4193	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10521.7;
	4193	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16253.7;
	4193	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2741.5;
	4194	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1190.7;
	4194	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1306.5;
	4194	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1396;
	4194	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5752.5;
	4194	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18930.1;
	4194	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10366.4;
	4194	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15442;
	4194	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2681.7;
	4195	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1172.7;
	4195	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1258.9;
	4195	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1344.4;
	4195	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5496.1;
	4195	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18113.7;
	4195	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9961.9;
	4195	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14692;
	4195	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2613.3;
	4196	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1136.2;
	4196	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1201.8;
	4196	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1268.8;
	4196	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5315.3;
	4196	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17263.6;
	4196	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9527.9;
	4196	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14182.7;
	4196	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2498.6;
	4197	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1117.2;
	4197	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1160.4;
	4197	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1231.4;
	4197	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5237.3;
	4197	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16726.6;
	4197	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9247.2;
	4197	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13771.6;
	4197	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2414.8;
	4198	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1077.1;
	4198	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1076.4;
	4198	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1156.1;
	4198	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4937.6;
	4198	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15426.3;
	4198	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8509.2;
	4198	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12843.9;
	4198	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2259.2;
	4199	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1024;
	4199	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	994.1;
	4199	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1068.5;
	4199	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4546.8;
	4199	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13985.9;
	4199	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7668.2;
	4199	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11848.7;
	4199	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2059.9;
	4200	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	988.7;
	4200	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	927.7;
	4200	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	990.9;
	4200	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4206.4;
	4200	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12713.1;
	4200	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6882.1;
	4200	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11017.9;
	4200	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1885.1;
	4201	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.4;
	4201	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	882;
	4201	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940.5;
	4201	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3953.8;
	4201	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11789.9;
	4201	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6373.3;
	4201	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10414.8;
	4201	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1755.6;
	4202	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.9;
	4202	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	845.3;
	4202	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	899.9;
	4202	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3760.1;
	4202	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11184.4;
	4202	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6041.4;
	4202	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9958.9;
	4202	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1683;
	4203	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926;
	4203	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	820;
	4203	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	876.5;
	4203	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3624.4;
	4203	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10802.6;
	4203	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5839.1;
	4203	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9718.4;
	4203	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1570.9;
	4204	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.5;
	4204	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	807;
	4204	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	858.6;
	4204	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3548.2;
	4204	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10692.3;
	4204	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5786;
	4204	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9682.5;
	4204	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1552.6;
	4205	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916;
	4205	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	822.3;
	4205	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.4;
	4205	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3577.4;
	4205	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10967.5;
	4205	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5932.3;
	4205	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9938;
	4205	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1654.5;
	4206	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925.1;
	4206	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	842.5;
	4206	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	894.3;
	4206	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3701.2;
	4206	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11362.8;
	4206	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6292.7;
	4206	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10222.7;
	4206	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1690.4;
	4207	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.5;
	4207	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	878.1;
	4207	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	929.2;
	4207	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3797;
	4207	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11923.2;
	4207	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6520.6;
	4207	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10735.9;
	4207	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1797.3;
	4208	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	976.6;
	4208	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	935.1;
	4208	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	985.3;
	4208	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4073.8;
	4208	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12852.8;
	4208	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6954;
	4208	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11670.4;
	4208	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1928.9;
	4209	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1010.3;
	4209	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1002.3;
	4209	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1047;
	4209	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4464.1;
	4209	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13849.6;
	4209	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7495.6;
	4209	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12730.9;
	4209	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2090.5;
	4210	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1053.1;
	4210	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1074.8;
	4210	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1115.2;
	4210	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4888.3;
	4210	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15010.9;
	4210	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8065.9;
	4210	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13799;
	4210	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2245.1;
	4211	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1094.7;
	4211	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1143.6;
	4211	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1183.2;
	4211	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5245.4;
	4211	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16073.9;
	4211	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8572.5;
	4211	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14628.9;
	4211	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2392.1;
	4212	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1120.6;
	4212	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1193.3;
	4212	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1242.2;
	4212	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5531.8;
	4212	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16955.2;
	4212	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9039.1;
	4212	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15307.6;
	4212	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2511.4;
	4213	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1148.8;
	4213	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1241.9;
	4213	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1293.8;
	4213	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5750.4;
	4213	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17694;
	4213	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9431.4;
	4213	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15862.4;
	4213	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2609.6;
	4214	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1170.3;
	4214	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1274.5;
	4214	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1337.7;
	4214	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5908.6;
	4214	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18198.2;
	4214	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9812.5;
	4214	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15724.3;
	4214	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2675.8;
	4215	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1184.3;
	4215	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1293.4;
	4215	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1374.2;
	4215	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6015.2;
	4215	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18550;
	4215	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10044.6;
	4215	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15489.2;
	4215	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2708.9;
	4216	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1193.9;
	4216	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1300.4;
	4216	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1391.9;
	4216	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6038.6;
	4216	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18681.1;
	4216	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10170.2;
	4216	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15235.6;
	4216	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2646.5;
	4217	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1184.3;
	4217	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1281.3;
	4217	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1384.1;
	4217	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5937.9;
	4217	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18622.2;
	4217	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10025.1;
	4217	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15089.7;
	4217	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2583.5;
	4218	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1170.1;
	4218	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1249.7;
	4218	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1351.3;
	4218	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5735.7;
	4218	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18099;
	4218	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9742.3;
	4218	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14856.4;
	4218	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2563.3;
	4219	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1147.9;
	4219	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1203.8;
	4219	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1309.8;
	4219	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5497.3;
	4219	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17270.5;
	4219	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9348.5;
	4219	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14415.4;
	4219	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2457.1;
	4220	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1114.1;
	4220	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1145.9;
	4220	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1248.1;
	4220	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5319.9;
	4220	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16382.7;
	4220	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8937.8;
	4220	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14003;
	4220	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2368;
	4221	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1095.1;
	4221	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1110.3;
	4221	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1218.2;
	4221	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5227;
	4221	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15856.5;
	4221	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8722.6;
	4221	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13636.7;
	4221	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2329.6;
	4222	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1063.7;
	4222	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1042.3;
	4222	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1148.4;
	4222	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4967.1;
	4222	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14786.3;
	4222	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8189.5;
	4222	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12921.2;
	4222	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2195.6;
	4223	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1021.3;
	4223	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	971.5;
	4223	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1063.1;
	4223	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4632.8;
	4223	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13577.1;
	4223	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7559.2;
	4223	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12100.1;
	4223	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2032.8;
	4224	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	979.3;
	4224	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	910;
	4224	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1012.3;
	4224	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4335.2;
	4224	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12422.6;
	4224	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6893.5;
	4224	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11347.3;
	4224	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1919.3;
	4225	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.1;
	4225	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	867.3;
	4225	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	962.9;
	4225	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4101.8;
	4225	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11601.5;
	4225	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6418.9;
	4225	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10787.5;
	4225	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1808.4;
	4226	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.4;
	4226	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	824.2;
	4226	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	925.5;
	4226	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3927;
	4226	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11001.3;
	4226	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6103.5;
	4226	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10377.7;
	4226	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1723.6;
	4227	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.6;
	4227	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	799.1;
	4227	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	899.9;
	4227	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3816.7;
	4227	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10655;
	4227	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5907.4;
	4227	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10089.8;
	4227	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1670.5;
	4228	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.9;
	4228	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	787;
	4228	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	888.4;
	4228	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3749.1;
	4228	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10471.4;
	4228	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5853.2;
	4228	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9931;
	4228	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1624.1;
	4229	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908;
	4229	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	791.4;
	4229	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	886.9;
	4229	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3750.1;
	4229	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10498.3;
	4229	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5848.5;
	4229	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9915.9;
	4229	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1643.9;
	4230	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.4;
	4230	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	795.5;
	4230	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	890.1;
	4230	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3765.5;
	4230	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10554.9;
	4230	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5938.9;
	4230	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9888.1;
	4230	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1644;
	4231	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911;
	4231	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	817.6;
	4231	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	907.5;
	4231	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3774.2;
	4231	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11029.5;
	4231	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6093.8;
	4231	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10293.7;
	4231	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1700;
	4232	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.8;
	4232	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	886.9;
	4232	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	960.1;
	4232	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3987.6;
	4232	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12093.6;
	4232	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6513.3;
	4232	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11347.9;
	4232	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1878.4;
	4233	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	984.3;
	4233	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	968.3;
	4233	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1014.3;
	4233	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4328.1;
	4233	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13359.6;
	4233	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6990.2;
	4233	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12539.4;
	4233	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2066.2;
	4234	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1024.3;
	4234	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1036.8;
	4234	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1082.2;
	4234	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4589.4;
	4234	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14586.3;
	4234	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7415.1;
	4234	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13596.6;
	4234	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2223.2;
	4235	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1061.9;
	4235	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1098.5;
	4235	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1136.5;
	4235	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4780.9;
	4235	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15654.2;
	4235	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7826.5;
	4235	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14266.7;
	4235	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2378.2;
	4236	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1094.8;
	4236	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1153.3;
	4236	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1179;
	4236	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4941.1;
	4236	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16445;
	4236	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7935.4;
	4236	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14677;
	4236	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2496.3;
	4237	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1123;
	4237	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1194.7;
	4237	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1220.8;
	4237	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5149.7;
	4237	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16958.4;
	4237	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7926.3;
	4237	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14688.7;
	4237	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2573.3;
	4238	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1144.3;
	4238	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1224.9;
	4238	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1246.9;
	4238	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5203.2;
	4238	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17344.6;
	4238	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8068.1;
	4238	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14932.3;
	4238	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2628;
	4239	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1161.9;
	4239	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1248.6;
	4239	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1268.3;
	4239	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5190.1;
	4239	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17509.8;
	4239	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8089.9;
	4239	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15444.1;
	4239	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2657.1;
	4240	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1166.4;
	4240	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1258.2;
	4240	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1270.8;
	4240	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5219.1;
	4240	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17360.4;
	4240	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7925.1;
	4240	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15570.8;
	4240	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2670.7;
	4241	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1163;
	4241	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1249.3;
	4241	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1252.8;
	4241	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5209.2;
	4241	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16954.8;
	4241	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7718.7;
	4241	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15216.1;
	4241	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2610.6;
	4242	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1149.2;
	4242	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1219.6;
	4242	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1209.9;
	4242	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5119;
	4242	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16398.1;
	4242	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7486;
	4242	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14789.7;
	4242	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2548.7;
	4243	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1134.5;
	4243	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1173;
	4243	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1171.7;
	4243	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4979.8;
	4243	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15784.8;
	4243	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7332.9;
	4243	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14228.6;
	4243	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2397.8;
	4244	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1106.9;
	4244	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1131;
	4244	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1129.8;
	4244	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4895.5;
	4244	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15301.9;
	4244	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7316.9;
	4244	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13900.3;
	4244	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2315.7;
	4245	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1089.7;
	4245	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1108.2;
	4245	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1121.7;
	4245	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4828.5;
	4245	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14938.8;
	4245	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7265.2;
	4245	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13495.8;
	4245	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2309.1;
	4246	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1061.9;
	4246	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1057.7;
	4246	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1044.9;
	4246	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4606.5;
	4246	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14079.8;
	4246	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6944.8;
	4246	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12827.3;
	4246	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2185.1;
	4247	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1024.2;
	4247	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	981.4;
	4247	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	931.7;
	4247	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4355.7;
	4247	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13079.2;
	4247	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6514.1;
	4247	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12080;
	4247	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2040.8;
	4248	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	983.7;
	4248	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	913.3;
	4248	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.9;
	4248	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4099.2;
	4248	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12145.5;
	4248	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6137.5;
	4248	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11351.6;
	4248	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1893.6;
	4249	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.2;
	4249	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	869.5;
	4249	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824.1;
	4249	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3875.2;
	4249	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11423.1;
	4249	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5851.1;
	4249	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10825.3;
	4249	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1801.5;
	4250	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.7;
	4250	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	840.4;
	4250	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	812.7;
	4250	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3704.3;
	4250	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10878.1;
	4250	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5660.3;
	4250	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10394.1;
	4250	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1710.9;
	4251	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919;
	4251	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	824.6;
	4251	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	802.6;
	4251	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3585.5;
	4251	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10464.9;
	4251	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5499.3;
	4251	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10084.8;
	4251	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1679.2;
	4252	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.9;
	4252	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	800.9;
	4252	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	796.8;
	4252	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3511.9;
	4252	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10198;
	4252	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5384.2;
	4252	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9921.3;
	4252	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1625.6;
	4253	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.2;
	4253	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	796.8;
	4253	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	807.7;
	4253	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3498.5;
	4253	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10128.5;
	4253	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5376.1;
	4253	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9860.9;
	4253	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1597.4;
	4254	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.8;
	4254	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	793.4;
	4254	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	810.2;
	4254	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3472.3;
	4254	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10040.2;
	4254	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5429.3;
	4254	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9740.1;
	4254	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1590.5;
	4255	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.8;
	4255	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	830.9;
	4255	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	828.5;
	4255	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3478.2;
	4255	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10443;
	4255	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5530.7;
	4255	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10049.7;
	4255	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1626.2;
	4256	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.5;
	4256	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	898.2;
	4256	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	876;
	4256	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3761;
	4256	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11576.7;
	4256	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5962.2;
	4256	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11006.8;
	4256	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1839.2;
	4257	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.8;
	4257	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	967.2;
	4257	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	927.7;
	4257	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4160.2;
	4257	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12819.6;
	4257	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6606.8;
	4257	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12112.5;
	4257	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2010.2;
	4258	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965;
	4258	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1032;
	4258	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	977;
	4258	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4552;
	4258	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14041.8;
	4258	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7267;
	4258	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13141.1;
	4258	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2159.3;
	4259	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	982.3;
	4259	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1091.6;
	4259	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1026.5;
	4259	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4918.5;
	4259	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15198.4;
	4259	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7884.8;
	4259	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13999.4;
	4259	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2305.5;
	4260	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985.6;
	4260	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1147.5;
	4260	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1080.4;
	4260	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5215.5;
	4260	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16184.6;
	4260	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8357.3;
	4260	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14641;
	4260	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2441.5;
	4261	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986.1;
	4261	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1190.9;
	4261	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1124.5;
	4261	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5440.2;
	4261	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17014.3;
	4261	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8806.8;
	4261	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15019.5;
	4261	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2539.5;
	4262	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.9;
	4262	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1224.2;
	4262	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1156.2;
	4262	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5579.6;
	4262	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17579.9;
	4262	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9143;
	4262	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15219.2;
	4262	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2593.1;
	4263	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1012.8;
	4263	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1240.3;
	4263	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1176.3;
	4263	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5658.5;
	4263	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17935.5;
	4263	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9375.1;
	4263	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15255.6;
	4263	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2669.5;
	4264	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1038.4;
	4264	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1261.4;
	4264	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1177.5;
	4264	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5682.8;
	4264	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18222.3;
	4264	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9439.2;
	4264	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15281.9;
	4264	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2667;
	4265	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1046.8;
	4265	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1265.7;
	4265	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1178.1;
	4265	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5651.5;
	4265	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18322;
	4265	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9343.4;
	4265	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15216.1;
	4265	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2616;
	4266	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1038.9;
	4266	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1242.3;
	4266	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1173.4;
	4266	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5508.8;
	4266	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18007.7;
	4266	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9164.3;
	4266	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14823.1;
	4266	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2613.8;
	4267	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1032.5;
	4267	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1194.6;
	4267	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1147.4;
	4267	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5287.6;
	4267	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17403.2;
	4267	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8932.2;
	4267	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14338.9;
	4267	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2501.6;
	4268	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1002.9;
	4268	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1141.4;
	4268	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1112;
	4268	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5119.9;
	4268	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16605.7;
	4268	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8686.7;
	4268	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13998.4;
	4268	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2482.1;
	4269	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	996.1;
	4269	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1104.9;
	4269	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1095.7;
	4269	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5016.9;
	4269	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16039.4;
	4269	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8502.5;
	4269	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13646.6;
	4269	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2398.1;
	4270	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	974.3;
	4270	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1026.7;
	4270	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1032.6;
	4270	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4747.1;
	4270	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14874.5;
	4270	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7955.7;
	4270	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12890.4;
	4270	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2219.1;
	4271	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.2;
	4271	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	957.2;
	4271	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	955.7;
	4271	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4388.6;
	4271	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13521.5;
	4271	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7211;
	4271	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12032.5;
	4271	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2053;
	4272	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.7;
	4272	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	887.9;
	4272	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	911.1;
	4272	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4053.3;
	4272	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12374.9;
	4272	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6564.6;
	4272	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11265.9;
	4272	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1883.5;
	4273	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.8;
	4273	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	841.6;
	4273	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	864;
	4273	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3792.5;
	4273	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11549.7;
	4273	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6084.3;
	4273	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10663;
	4273	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1787;
	4274	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.4;
	4274	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	804.3;
	4274	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	839.5;
	4274	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3605.8;
	4274	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11018.7;
	4274	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5767.9;
	4274	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10272.1;
	4274	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1701.9;
	4275	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	863.2;
	4275	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	785.4;
	4275	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	820.6;
	4275	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3493.9;
	4275	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10693.1;
	4275	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5583.5;
	4275	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10087.3;
	4275	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1653.2;
	4276	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	863;
	4276	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	771.2;
	4276	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	815.9;
	4276	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3439.8;
	4276	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10644.7;
	4276	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5563.6;
	4276	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10150.6;
	4276	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1629.3;
	4277	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.8;
	4277	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	792.6;
	4277	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	833.3;
	4277	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3499.3;
	4277	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10957.4;
	4277	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5785;
	4277	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10478.9;
	4277	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1693.5;
	4278	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.3;
	4278	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	808.9;
	4278	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	870.7;
	4278	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3640.7;
	4278	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11467.8;
	4278	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6148.1;
	4278	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10821.3;
	4278	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1758.9;
	4279	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.3;
	4279	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	859.8;
	4279	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	902.5;
	4279	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3792.6;
	4279	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12140.3;
	4279	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6431.7;
	4279	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11324.4;
	4279	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1889.9;
	4280	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.2;
	4280	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	926.1;
	4280	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	958.6;
	4280	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4133.8;
	4280	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13216.1;
	4280	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6832.7;
	4280	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12256.5;
	4280	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2061.2;
	4281	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.5;
	4281	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	995.1;
	4281	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1017;
	4281	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4516.8;
	4281	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14417.3;
	4281	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7391.5;
	4281	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13262.3;
	4281	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2245.5;
	4282	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985.3;
	4282	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1084.3;
	4282	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1078.4;
	4282	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4921.9;
	4282	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15786.7;
	4282	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8001.5;
	4282	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14426.8;
	4282	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2422.9;
	4283	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1004.3;
	4283	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1165;
	4283	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1147.9;
	4283	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5320.3;
	4283	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17000.1;
	4283	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8601.7;
	4283	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15447.7;
	4283	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2533.4;
	4284	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1007.4;
	4284	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1227.4;
	4284	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1210.5;
	4284	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5642.7;
	4284	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17927.7;
	4284	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9153.3;
	4284	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16110.5;
	4284	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2682.4;
	4285	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1015.8;
	4285	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1272.7;
	4285	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1251.1;
	4285	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5847.2;
	4285	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18718.4;
	4285	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9657.7;
	4285	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16607.7;
	4285	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2816.1;
	4286	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1019.1;
	4286	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1296.6;
	4286	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1293.6;
	4286	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5984.6;
	4286	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19336;
	4286	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10071.4;
	4286	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16565.8;
	4286	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2879;
	4287	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1020;
	4287	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1290.3;
	4287	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1328.6;
	4287	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5977.8;
	4287	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19682.4;
	4287	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10391.9;
	4287	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16039.1;
	4287	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2897.3;
	4288	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1041.3;
	4288	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1249.5;
	4288	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1352.8;
	4288	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5877.6;
	4288	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19782.1;
	4288	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10597.7;
	4288	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15674.6;
	4288	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2877;
	4289	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1040.7;
	4289	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1182.4;
	4289	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1361.5;
	4289	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5767.6;
	4289	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19306.2;
	4289	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10650.6;
	4289	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15508;
	4289	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2819.1;
	4290	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1041.1;
	4290	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1141.1;
	4290	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1360.5;
	4290	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5642.7;
	4290	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18290.9;
	4290	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10508;
	4290	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15243.5;
	4290	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2735.5;
	4291	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1037.7;
	4291	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1107;
	4291	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1325.6;
	4291	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5455.6;
	4291	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17109.8;
	4291	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10176;
	4291	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14869.4;
	4291	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2622.7;
	4292	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1014.1;
	4292	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1074.5;
	4292	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1263;
	4292	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5300.6;
	4292	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16281.6;
	4292	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9714.6;
	4292	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14519.2;
	4292	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2533.6;
	4293	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1008.9;
	4293	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1056.5;
	4293	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1235.7;
	4293	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5197.2;
	4293	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15680.2;
	4293	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9320.9;
	4293	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14158.5;
	4293	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2466.7;
	4294	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986.6;
	4294	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	998.7;
	4294	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1161.6;
	4294	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4839;
	4294	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14444.1;
	4294	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8491.3;
	4294	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13202.1;
	4294	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2252.8;
	4295	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948.4;
	4295	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	927.2;
	4295	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1065.8;
	4295	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4444.6;
	4295	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13108;
	4295	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7603.4;
	4295	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12231.5;
	4295	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2079;
	4296	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.7;
	4296	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	879.7;
	4296	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	964.7;
	4296	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4116.1;
	4296	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11943.3;
	4296	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6855.8;
	4296	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11434.4;
	4296	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1934.4;
	4297	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.6;
	4297	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	834.3;
	4297	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	910.8;
	4297	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3858.9;
	4297	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11137.5;
	4297	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6348.1;
	4297	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10810.5;
	4297	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1812.6;
	4298	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.4;
	4298	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	798.8;
	4298	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	875.4;
	4298	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3666.5;
	4298	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10611.1;
	4298	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6003.1;
	4298	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10379.1;
	4298	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1727;
	4299	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889;
	4299	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	784.5;
	4299	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854;
	4299	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3538.6;
	4299	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10316.8;
	4299	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5764.4;
	4299	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10167.7;
	4299	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1687.2;
	4300	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.5;
	4300	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772.5;
	4300	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	841.6;
	4300	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3489.7;
	4300	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10275.5;
	4300	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5675.8;
	4300	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10171.3;
	4300	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1665;
	4301	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.3;
	4301	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	791.4;
	4301	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	853.8;
	4301	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3548.1;
	4301	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10680;
	4301	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5800.6;
	4301	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10522.8;
	4301	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1687.1;
	4302	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.9;
	4302	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	819;
	4302	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	896.2;
	4302	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3675.9;
	4302	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11274.2;
	4302	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6143.4;
	4302	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10890.1;
	4302	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1783.7;
	4303	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.2;
	4303	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	868;
	4303	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	922.1;
	4303	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3799.1;
	4303	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11930.2;
	4303	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6431.8;
	4303	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11324.6;
	4303	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1841.4;
	4304	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.6;
	4304	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	925.9;
	4304	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	997.3;
	4304	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4146.7;
	4304	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12933.1;
	4304	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6963.2;
	4304	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12172.2;
	4304	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2052.3;
	4305	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	968;
	4305	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	998.9;
	4305	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1072.1;
	4305	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4531.8;
	4305	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14166.1;
	4305	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7625;
	4305	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13198.4;
	4305	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2155.8;
	4306	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1004.7;
	4306	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1079.3;
	4306	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1164.8;
	4306	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4955.3;
	4306	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15585.7;
	4306	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8382.3;
	4306	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14310.3;
	4306	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2332.3;
	4307	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1042.2;
	4307	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1163;
	4307	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1243.2;
	4307	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5367;
	4307	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16858.8;
	4307	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9119.4;
	4307	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15319.6;
	4307	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2576.6;
	4308	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1072.4;
	4308	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1235.8;
	4308	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1307.7;
	4308	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5701.1;
	4308	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17767.8;
	4308	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9738.7;
	4308	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16099.6;
	4308	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2654.6;
	4309	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1104.3;
	4309	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1272.9;
	4309	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1334.8;
	4309	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5952.3;
	4309	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18108.1;
	4309	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10235.8;
	4309	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16438.8;
	4309	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2600.4;
	4310	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1126.5;
	4310	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1298;
	4310	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1318.3;
	4310	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6143.3;
	4310	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18232.7;
	4310	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10601.2;
	4310	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15513.2;
	4310	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2565.7;
	4311	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1134.9;
	4311	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1307.9;
	4311	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1302.3;
	4311	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6248.2;
	4311	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18485.9;
	4311	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10668;
	4311	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14201.3;
	4311	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2536.7;
	4312	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1142.5;
	4312	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1311.7;
	4312	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1304.3;
	4312	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6277.5;
	4312	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18761.9;
	4312	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10154;
	4312	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13558.8;
	4312	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2470.7;
	4313	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1131.5;
	4313	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1301.9;
	4313	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1280.4;
	4313	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6193.9;
	4313	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18615.9;
	4313	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9464.6;
	4313	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12987.9;
	4313	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2434.3;
	4314	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1114.1;
	4314	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1273;
	4314	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1248.1;
	4314	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6019.5;
	4314	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17981.3;
	4314	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8779.8;
	4314	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12492.7;
	4314	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2391.4;
	4315	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1085.8;
	4315	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1223.1;
	4315	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1194.7;
	4315	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5765.5;
	4315	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17101.7;
	4315	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8329.4;
	4315	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12166.2;
	4315	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2306;
	4316	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1053.6;
	4316	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1161.5;
	4316	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1137.8;
	4316	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5557.1;
	4316	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16152.9;
	4316	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8027.9;
	4316	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12165.5;
	4316	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2249.7;
	4317	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1041.3;
	4317	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1121.1;
	4317	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1113.9;
	4317	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5409.9;
	4317	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15424.9;
	4317	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7796.8;
	4317	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11991.2;
	4317	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2195.7;
	4318	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009.4;
	4318	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1050.9;
	4318	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1044.9;
	4318	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5045.5;
	4318	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14146.4;
	4318	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7202.3;
	4318	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11378.9;
	4318	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2052.4;
	4319	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.8;
	4319	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	959.6;
	4319	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	962.4;
	4319	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4544.3;
	4319	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12636.2;
	4319	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6486.7;
	4319	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10564.6;
	4319	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1867.1;
	4320	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.6;
	4320	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	894.9;
	4320	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	910;
	4320	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4142.9;
	4320	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11432.4;
	4320	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5959.7;
	4320	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9908.2;
	4320	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1719.7;
	4321	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.3;
	4321	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	854.4;
	4321	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	858.6;
	4321	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3872.5;
	4321	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10614.1;
	4321	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5559.4;
	4321	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9448.9;
	4321	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1641.8;
	4322	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.7;
	4322	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	820.6;
	4322	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	836.8;
	4322	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3658.5;
	4322	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10111.8;
	4322	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5329.6;
	4322	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9175.1;
	4322	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1587.3;
	4323	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.8;
	4323	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	786.9;
	4323	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	826;
	4323	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3515.9;
	4323	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9801;
	4323	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5175.8;
	4323	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9043.4;
	4323	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1555.6;
	4324	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.4;
	4324	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	766.3;
	4324	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	814.2;
	4324	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3459.4;
	4324	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9790.7;
	4324	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5167.4;
	4324	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9123.3;
	4324	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1493.1;
	4325	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880;
	4325	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	782.9;
	4325	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.8;
	4325	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3501.3;
	4325	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10117;
	4325	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5394.1;
	4325	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9541.9;
	4325	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1513.3;
	4326	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.3;
	4326	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	795.3;
	4326	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	883;
	4326	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3638;
	4326	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10744.6;
	4326	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5785;
	4326	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9976.2;
	4326	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1595.8;
	4327	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.9;
	4327	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	827.4;
	4327	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913.5;
	4327	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3731.7;
	4327	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11330.5;
	4327	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6067.1;
	4327	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10369;
	4327	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1689.1;
	4328	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.3;
	4328	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	888.4;
	4328	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	974;
	4328	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4004.6;
	4328	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12282.5;
	4328	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6506.2;
	4328	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10929.1;
	4328	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1833.1;
	4329	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.8;
	4329	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	958.4;
	4329	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1044.1;
	4329	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4361.8;
	4329	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13427;
	4329	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7083.7;
	4329	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11722.2;
	4329	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2001.7;
	4330	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1018.3;
	4330	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1040.9;
	4330	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1116.1;
	4330	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4795.6;
	4330	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14657;
	4330	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7723.3;
	4330	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12808.5;
	4330	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2180;
	4331	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1061.9;
	4331	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1120.3;
	4331	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1194;
	4331	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5234.7;
	4331	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15706.3;
	4331	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8311.1;
	4331	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13870.7;
	4331	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2343.1;
	4332	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1096.7;
	4332	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1194.7;
	4332	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1251.5;
	4332	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5587.1;
	4332	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16765.3;
	4332	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8917.5;
	4332	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14732.8;
	4332	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2454.4;
	4333	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1132;
	4333	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1239.6;
	4333	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1321.3;
	4333	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5874.8;
	4333	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17699;
	4333	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9459.8;
	4333	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15440.3;
	4333	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2572.2;
	4334	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1151.6;
	4334	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1270.1;
	4334	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1373.7;
	4334	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6079.8;
	4334	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18352.9;
	4334	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9889.2;
	4334	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15960.9;
	4334	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2649.8;
	4335	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1166.1;
	4335	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1293.4;
	4335	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1412.3;
	4335	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6158.1;
	4335	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18792.4;
	4335	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10216.9;
	4335	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16296;
	4335	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2701.3;
	4336	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1176.8;
	4336	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1303.4;
	4336	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1435.4;
	4336	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6083.1;
	4336	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19032.6;
	4336	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10451.8;
	4336	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16292.4;
	4336	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2740.2;
	4337	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1170.6;
	4337	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1282.3;
	4337	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1425.8;
	4337	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5922.6;
	4337	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18928.8;
	4337	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10506.8;
	4337	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16119;
	4337	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2704;
	4338	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1156.1;
	4338	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1243.9;
	4338	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1393.9;
	4338	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5709.6;
	4338	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18518.3;
	4338	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10357.8;
	4338	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15599.5;
	4338	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2650.1;
	4339	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1136.1;
	4339	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1199.8;
	4339	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1343;
	4339	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5521.5;
	4339	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17752.2;
	4339	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9963.5;
	4339	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14965;
	4339	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2547.1;
	4340	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1094.8;
	4340	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1140.8;
	4340	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1275.9;
	4340	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5375.5;
	4340	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16787.9;
	4340	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9497.8;
	4340	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14499.8;
	4340	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2428.1;
	4341	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1072.1;
	4341	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1093.5;
	4341	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1231.2;
	4341	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5265.1;
	4341	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16086.4;
	4341	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9165.4;
	4341	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14004.9;
	4341	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2339.6;
	4342	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1031.2;
	4342	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1015.7;
	4342	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1145.2;
	4342	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4934.3;
	4342	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14711.5;
	4342	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8413;
	4342	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13067.3;
	4342	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2142.8;
	4343	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	978.6;
	4343	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	943.6;
	4343	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1041.5;
	4343	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4533.6;
	4343	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13226.1;
	4343	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7567.1;
	4343	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12074.8;
	4343	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1933.4;
	4344	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.4;
	4344	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	870.9;
	4344	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	952;
	4344	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4152.5;
	4344	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12111.5;
	4344	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6855.5;
	4344	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11184.7;
	4344	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1759.1;
	4345	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.8;
	4345	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	825.9;
	4345	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	891.5;
	4345	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3896.9;
	4345	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11190.9;
	4345	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6349.2;
	4345	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10554.7;
	4345	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1642.2;
	4346	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.4;
	4346	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	794.1;
	4346	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.4;
	4346	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3718.3;
	4346	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10543.5;
	4346	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6015.3;
	4346	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10145;
	4346	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1551.9;
	4347	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.6;
	4347	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777.1;
	4347	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	818.5;
	4347	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3597.7;
	4347	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10184;
	4347	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5803.5;
	4347	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9948.2;
	4347	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1497;
	4348	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.7;
	4348	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	760.8;
	4348	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	810.9;
	4348	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3547.2;
	4348	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10146.2;
	4348	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5724.7;
	4348	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9998.2;
	4348	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1487.4;
	4349	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.3;
	4349	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776.2;
	4349	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824.2;
	4349	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3591.1;
	4349	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10458;
	4349	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5904.8;
	4349	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10357.8;
	4349	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1526.5;
	4350	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.5;
	4350	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	795.5;
	4350	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	855.4;
	4350	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3703;
	4350	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10969.4;
	4350	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6243.4;
	4350	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10693;
	4350	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1605.4;
	4351	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.4;
	4351	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	830.4;
	4351	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	890.8;
	4351	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3804.7;
	4351	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11529.8;
	4351	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6516.6;
	4351	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11047.8;
	4351	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1695.5;
	4352	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.6;
	4352	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	898.7;
	4352	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	959.6;
	4352	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4123.5;
	4352	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12537.5;
	4352	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7027.3;
	4352	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11794.7;
	4352	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1869.7;
	4353	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989;
	4353	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	955.5;
	4353	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1040.6;
	4353	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4558.9;
	4353	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13724.8;
	4353	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7702.7;
	4353	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12761;
	4353	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2100.3;
	4354	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1036.1;
	4354	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1034.7;
	4354	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1125.1;
	4354	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5013.3;
	4354	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15039.2;
	4354	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8391.1;
	4354	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13827.6;
	4354	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2243.9;
	4355	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1080;
	4355	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1106.1;
	4355	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1207.9;
	4355	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5438.7;
	4355	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16159.2;
	4355	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8974.7;
	4355	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14686;
	4355	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2477.8;
	4356	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1108.5;
	4356	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1166.8;
	4356	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1276.8;
	4356	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5755.6;
	4356	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17097.3;
	4356	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9481.1;
	4356	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15355.2;
	4356	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2579.8;
	4357	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1134.7;
	4357	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1224.2;
	4357	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1335.8;
	4357	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6010;
	4357	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17992.1;
	4357	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9957.4;
	4357	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16031.3;
	4357	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2727.7;
	4358	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1156.3;
	4358	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1272;
	4358	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1376.3;
	4358	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6165.5;
	4358	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18679.4;
	4358	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10343.7;
	4358	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16416.8;
	4358	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2770.6;
	4359	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1177.8;
	4359	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1301.3;
	4359	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1413.7;
	4359	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6274.6;
	4359	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19111.7;
	4359	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10573.6;
	4359	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16569.7;
	4359	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2822.8;
	4360	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1185.4;
	4360	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1308.8;
	4360	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1434.3;
	4360	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6299.5;
	4360	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19365.1;
	4360	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10686.5;
	4360	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16642.1;
	4360	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2850.5;
	4361	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1174.1;
	4361	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1295.4;
	4361	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1427.3;
	4361	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6179.7;
	4361	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19299.6;
	4361	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10717.9;
	4361	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16374.4;
	4361	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2809;
	4362	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1160.3;
	4362	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1264.2;
	4362	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1391.3;
	4362	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5956.1;
	4362	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18849.3;
	4362	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10545.5;
	4362	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15896.8;
	4362	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2764.9;
	4363	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1142.5;
	4363	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1217.9;
	4363	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1333.9;
	4363	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5694.5;
	4363	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18125.2;
	4363	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10157.4;
	4363	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15371.5;
	4363	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2650.3;
	4364	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1107.7;
	4364	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1161.2;
	4364	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1267;
	4364	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5522.2;
	4364	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17168.4;
	4364	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9697.5;
	4364	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14851.3;
	4364	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2552.8;
	4365	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1089.4;
	4365	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1125.4;
	4365	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1233.6;
	4365	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5407.2;
	4365	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16446.3;
	4365	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9348;
	4365	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14385;
	4365	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2467.7;
	4366	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1050.3;
	4366	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1050.6;
	4366	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1151.4;
	4366	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5071.1;
	4366	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15088.6;
	4366	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8641.8;
	4366	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13469.1;
	4366	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2261.8;
	4367	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.2;
	4367	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	980.1;
	4367	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1059.3;
	4367	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4668.1;
	4367	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13575.6;
	4367	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7782.1;
	4367	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12437.9;
	4367	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2073.9;
	4368	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.6;
	4368	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	912;
	4368	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	986.9;
	4368	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4331.2;
	4368	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12147.6;
	4368	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7097.9;
	4368	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11480.7;
	4368	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1883;
	4369	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.7;
	4369	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	868.6;
	4369	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	931.8;
	4369	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4071.6;
	4369	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11258.1;
	4369	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6555.6;
	4369	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10865.2;
	4369	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1764.5;
	4370	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.6;
	4370	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	825.2;
	4370	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889.6;
	4370	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3867.6;
	4370	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10586.4;
	4370	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6195.3;
	4370	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10363.4;
	4370	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1672.4;
	4371	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894;
	4371	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	801.2;
	4371	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.5;
	4371	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3729.5;
	4371	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10164.1;
	4371	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5935;
	4371	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10059.5;
	4371	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1641.3;
	4372	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.3;
	4372	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	781.4;
	4372	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851.8;
	4372	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3654.1;
	4372	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10011.4;
	4372	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5827.3;
	4372	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10015.6;
	4372	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1610.7;
	4373	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.7;
	4373	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	795.3;
	4373	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.2;
	4373	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3693.5;
	4373	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10254.2;
	4373	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5923.8;
	4373	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10251.1;
	4373	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1642.2;
	4374	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.3;
	4374	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	815.9;
	4374	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889.3;
	4374	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3789.4;
	4374	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10668.7;
	4374	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6216.1;
	4374	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10483.6;
	4374	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1702.3;
	4375	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.5;
	4375	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	837.8;
	4375	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	924.1;
	4375	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3878.6;
	4375	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11084.3;
	4375	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6454.5;
	4375	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10930.9;
	4375	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1801;
	4376	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.7;
	4376	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	895.8;
	4376	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	994;
	4376	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4211.5;
	4376	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12111.9;
	4376	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6970.4;
	4376	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11869.7;
	4376	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1991.6;
	4377	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999;
	4377	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	976.7;
	4377	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1068.2;
	4377	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4640;
	4377	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13463.6;
	4377	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7566.7;
	4377	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12932.3;
	4377	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2071.7;
	4378	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1041.5;
	4378	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1065.4;
	4378	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1141.2;
	4378	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5055.8;
	4378	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14919.4;
	4378	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8209.3;
	4378	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13920.4;
	4378	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2277;
	4379	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1081.3;
	4379	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1145.2;
	4379	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1215.7;
	4379	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5443.5;
	4379	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16179.8;
	4379	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8780.6;
	4379	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14735.4;
	4379	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2472.7;
	4380	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1112.2;
	4380	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1212.4;
	4380	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1278.5;
	4380	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5776.8;
	4380	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17277.7;
	4380	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9304.5;
	4380	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15429.2;
	4380	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2576.2;
	4381	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1139.9;
	4381	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1264.6;
	4381	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1330.4;
	4381	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6044.9;
	4381	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18211.5;
	4381	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9783.6;
	4381	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16016.2;
	4381	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2663;
	4382	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1163.5;
	4382	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1295.5;
	4382	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1373;
	4382	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6232.8;
	4382	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18877.2;
	4382	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10176.3;
	4382	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16383.5;
	4382	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2723.8;
	4383	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1175.4;
	4383	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1311.1;
	4383	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1404.8;
	4383	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6323.5;
	4383	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19241.3;
	4383	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10456;
	4383	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16603.4;
	4383	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2814.8;
	4384	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1179.6;
	4384	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1317.9;
	4384	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1413.9;
	4384	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6334.9;
	4384	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19445.2;
	4384	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10635.9;
	4384	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16734;
	4384	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2840.9;
	4385	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1168.8;
	4385	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1304.9;
	4385	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1398.2;
	4385	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6204.7;
	4385	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19304.6;
	4385	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10642;
	4385	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16573.8;
	4385	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2823.7;
	4386	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1154.9;
	4386	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1274;
	4386	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1366.1;
	4386	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5993.5;
	4386	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18773.5;
	4386	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10427.5;
	4386	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15989.2;
	4386	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2736.4;
	4387	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1136.2;
	4387	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1227.3;
	4387	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1324.5;
	4387	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5733.6;
	4387	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17871.6;
	4387	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10035.7;
	4387	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15276.5;
	4387	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2615.2;
	4388	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1107.3;
	4388	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1169.5;
	4388	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1265.1;
	4388	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5546.7;
	4388	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16923.7;
	4388	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9565.3;
	4388	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14748.9;
	4388	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2501.2;
	4389	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1094;
	4389	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1133.9;
	4389	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1234.2;
	4389	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5443.2;
	4389	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16318.5;
	4389	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9311.2;
	4389	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14318.5;
	4389	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2395.1;
	4390	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1061.1;
	4390	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1078.4;
	4390	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1159.8;
	4390	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5148.3;
	4390	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15161.8;
	4390	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8708.8;
	4390	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13493.7;
	4390	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2211.5;
	4391	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1020.5;
	4391	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1003.4;
	4391	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1075.4;
	4391	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4814.1;
	4391	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13869.5;
	4391	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7992.1;
	4391	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12601.4;
	4391	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2047.4;
	4392	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.8;
	4392	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	929.4;
	4392	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1019.9;
	4392	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4488.9;
	4392	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12686.1;
	4392	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7378.4;
	4392	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11715.4;
	4392	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1908.2;
	4393	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.4;
	4393	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	882.8;
	4393	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	968.8;
	4393	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4230.9;
	4393	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11845.6;
	4393	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6825.5;
	4393	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11076.7;
	4393	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1758.9;
	4394	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.9;
	4394	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	847.1;
	4394	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	928.8;
	4394	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4029.6;
	4394	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11243.7;
	4394	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6404;
	4394	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10593.9;
	4394	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1649.1;
	4395	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.2;
	4395	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	823.1;
	4395	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	898.6;
	4395	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3882.6;
	4395	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10824.6;
	4395	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6108.6;
	4395	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10234.4;
	4395	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1646.4;
	4396	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916;
	4396	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	800.1;
	4396	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	880.2;
	4396	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3786.6;
	4396	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10597.9;
	4396	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5940.1;
	4396	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10046.8;
	4396	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1633.7;
	4397	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.2;
	4397	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	797.4;
	4397	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	877;
	4397	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3764.5;
	4397	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10595.3;
	4397	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5888.7;
	4397	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10024.3;
	4397	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1632.5;
	4398	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.7;
	4398	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	795.7;
	4398	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	877.6;
	4398	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3741.1;
	4398	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10564.4;
	4398	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5914;
	4398	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9953.1;
	4398	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1621.9;
	4399	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.5;
	4399	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	806.8;
	4399	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	893.9;
	4399	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3736.4;
	4399	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10929.8;
	4399	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6055.9;
	4399	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10224.7;
	4399	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1659.5;
	4400	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.8;
	4400	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	879.1;
	4400	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	960.6;
	4400	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4065.7;
	4400	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12042.1;
	4400	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6517.4;
	4400	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11182.3;
	4400	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1828.2;
	4401	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.1;
	4401	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	963;
	4401	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1040;
	4401	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4487.6;
	4401	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13362;
	4401	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7149.4;
	4401	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12304.3;
	4401	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2037.7;
	4402	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1002.6;
	4402	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1036.3;
	4402	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1123.7;
	4402	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4900.9;
	4402	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14587.4;
	4402	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7868.2;
	4402	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13370.8;
	4402	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2209.1;
	4403	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1047.9;
	4403	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1102.2;
	4403	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1203.2;
	4403	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5264.2;
	4403	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15689.9;
	4403	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8526.7;
	4403	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14274.8;
	4403	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2360.8;
	4404	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1084.8;
	4404	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1162.5;
	4404	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1277.4;
	4404	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5552.8;
	4404	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16627.8;
	4404	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9115.5;
	4404	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14932.8;
	4404	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2478.3;
	4405	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1117.2;
	4405	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1210.1;
	4405	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1339.2;
	4405	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5760.1;
	4405	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17337.1;
	4405	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9587;
	4405	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15457.4;
	4405	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2594.9;
	4406	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1143.1;
	4406	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1245;
	4406	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1388.2;
	4406	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5916.8;
	4406	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17921.2;
	4406	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9977.2;
	4406	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15869.4;
	4406	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2643;
	4407	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1162.4;
	4407	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1268.2;
	4407	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1427.2;
	4407	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5990.3;
	4407	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18286.3;
	4407	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10255.4;
	4407	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16143.8;
	4407	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2703.1;
	4408	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1167.9;
	4408	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1283.5;
	4408	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1440.4;
	4408	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6016.3;
	4408	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18475.1;
	4408	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10439.5;
	4408	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16268;
	4408	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2726.3;
	4409	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1155.1;
	4409	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1281.6;
	4409	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1448.1;
	4409	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5957.7;
	4409	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18408.6;
	4409	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10486.7;
	4409	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16189.5;
	4409	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2725.7;
	4410	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1144.6;
	4410	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1255.1;
	4410	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1432.6;
	4410	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5816.3;
	4410	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18059.7;
	4410	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10313.8;
	4410	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15824.5;
	4410	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2663.1;
	4411	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1127.4;
	4411	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1200;
	4411	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1383.8;
	4411	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5577.2;
	4411	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17380.3;
	4411	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9992.9;
	4411	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15258.8;
	4411	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2579.5;
	4412	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1092.4;
	4412	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1146.3;
	4412	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1321;
	4412	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5371.8;
	4412	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16603.6;
	4412	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9609.3;
	4412	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14730.8;
	4412	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2486.6;
	4413	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1066.8;
	4413	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1110.7;
	4413	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1283.8;
	4413	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5278;
	4413	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16063.3;
	4413	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9315;
	4413	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14281.3;
	4413	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2394.4;
	4414	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1037.4;
	4414	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1054.2;
	4414	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1218.6;
	4414	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5036.2;
	4414	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15138.1;
	4414	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8720.1;
	4414	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13542.2;
	4414	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2277.5;
	4415	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1002.5;
	4415	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	989.8;
	4415	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1131.5;
	4415	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4745.7;
	4415	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14081.3;
	4415	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8055.5;
	4415	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12722.6;
	4415	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2122.4;
	4416	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.5;
	4416	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	937.9;
	4416	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1075.3;
	4416	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4491.3;
	4416	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13115.5;
	4416	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7406.7;
	4416	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11915.8;
	4416	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1958.8;
	4417	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.2;
	4417	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	905.9;
	4417	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1019;
	4417	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4256.4;
	4417	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12404.1;
	4417	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6891;
	4417	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11290.5;
	4417	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1879.2;
	4418	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932;
	4418	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	885.6;
	4418	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	981;
	4418	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4085;
	4418	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11873.3;
	4418	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6512.5;
	4418	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10796.3;
	4418	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1828.2;
	4419	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.7;
	4419	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	867;
	4419	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	951.9;
	4419	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3964.7;
	4419	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11460;
	4419	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6237.4;
	4419	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10487.3;
	4419	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1770.2;
	4420	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912;
	4420	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	846.7;
	4420	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	935.9;
	4420	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3882.6;
	4420	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11161;
	4420	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6091.4;
	4420	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10355.3;
	4420	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1707.8;
	4421	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.5;
	4421	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	840.6;
	4421	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	930.8;
	4421	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3855.4;
	4421	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11003.7;
	4421	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6052.5;
	4421	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10275.5;
	4421	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1709.3;
	4422	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.2;
	4422	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	841.9;
	4422	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	925.9;
	4422	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3828.5;
	4422	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10844.5;
	4422	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6101;
	4422	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10143.2;
	4422	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1692.7;
	4423	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.9;
	4423	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	860.9;
	4423	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	937.2;
	4423	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3828.5;
	4423	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11042.3;
	4423	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6189.1;
	4423	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10342.3;
	4423	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1716.9;
	4424	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.9;
	4424	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	910;
	4424	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	994.5;
	4424	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4122.4;
	4424	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11842.1;
	4424	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6520.9;
	4424	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11285;
	4424	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1863.6;
	4425	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.1;
	4425	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	978.6;
	4425	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1058.1;
	4425	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4546.5;
	4425	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12948.9;
	4425	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7036.8;
	4425	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12389.2;
	4425	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1960.1;
	4426	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1015.3;
	4426	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1025.4;
	4426	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1118.9;
	4426	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4896.1;
	4426	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14041;
	4426	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7626.2;
	4426	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13376;
	4426	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2123.2;
	4427	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1058.7;
	4427	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1075.7;
	4427	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1187.3;
	4427	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5204.1;
	4427	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15060.3;
	4427	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8256.3;
	4427	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14236.2;
	4427	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2313;
	4428	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1100.8;
	4428	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1127.9;
	4428	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1260.2;
	4428	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5478.8;
	4428	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16027.4;
	4428	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8841.4;
	4428	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14892.3;
	4428	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2446.6;
	4429	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1135.4;
	4429	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1182.9;
	4429	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1327;
	4429	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5700.8;
	4429	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16894.4;
	4429	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9361.4;
	4429	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15422.3;
	4429	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2556.5;
	4430	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1165.8;
	4430	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1225.2;
	4430	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1389.6;
	4430	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5867.2;
	4430	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17616.2;
	4430	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9805.2;
	4430	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15819.2;
	4430	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2616.6;
	4431	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1183.8;
	4431	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1256.7;
	4431	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1437.8;
	4431	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5973.6;
	4431	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18140.4;
	4431	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10164;
	4431	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15991;
	4431	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2689.1;
	4432	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1197.4;
	4432	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1272.5;
	4432	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1481;
	4432	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6018.7;
	4432	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18450.4;
	4432	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10461.9;
	4432	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16030.3;
	4432	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2726.4;
	4433	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1201.4;
	4433	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1258.5;
	4433	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1510.3;
	4433	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5989.9;
	4433	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18473.7;
	4433	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10564.8;
	4433	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15890.1;
	4433	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2710.8;
	4434	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1196.3;
	4434	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1225.2;
	4434	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1502.4;
	4434	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5838.9;
	4434	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18127.3;
	4434	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10474.8;
	4434	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15478.8;
	4434	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2666.3;
	4435	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1179.5;
	4435	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1182.8;
	4435	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1438.7;
	4435	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5614.3;
	4435	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17490.4;
	4435	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10133;
	4435	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14891.9;
	4435	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2573.6;
	4436	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1144.6;
	4436	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1121.2;
	4436	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1357.3;
	4436	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5429.8;
	4436	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16777.8;
	4436	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9723.3;
	4436	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14452;
	4436	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2476.9;
	4437	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1115.2;
	4437	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1096.4;
	4437	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1314.9;
	4437	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5347.6;
	4437	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16273.9;
	4437	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9459.3;
	4437	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14210;
	4437	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2337.6;
	4438	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1076.6;
	4438	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1052.3;
	4438	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1251.8;
	4438	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5114.8;
	4438	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15379.8;
	4438	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8901.7;
	4438	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13581.4;
	4438	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2217.3;
	4439	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1029.8;
	4439	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	992.2;
	4439	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1173.4;
	4439	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4809.8;
	4439	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14140.6;
	4439	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8245.9;
	4439	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12803.4;
	4439	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2150.7;
	4440	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1011;
	4440	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	934.1;
	4440	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1128;
	4440	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4526.6;
	4440	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12877.2;
	4440	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7628.4;
	4440	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12162.7;
	4440	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1967.1;
	4441	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	984.1;
	4441	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	887.4;
	4441	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1070.1;
	4441	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4281.9;
	4441	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12119.4;
	4441	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7133.3;
	4441	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11614.2;
	4441	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1888.3;
	4442	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.8;
	4442	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	850.3;
	4442	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1026.2;
	4442	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4089;
	4442	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11499.2;
	4442	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6784;
	4442	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11189;
	4442	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1835.5;
	4443	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951.3;
	4443	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	837.9;
	4443	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	989.5;
	4443	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3960.3;
	4443	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10967.2;
	4443	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6493.1;
	4443	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10984.6;
	4443	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1786.4;
	4444	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.3;
	4444	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	818.9;
	4444	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	972.2;
	4444	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3884.7;
	4444	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10538.6;
	4444	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6379.5;
	4444	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10935.8;
	4444	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1752.9;
	4445	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944;
	4445	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	815.3;
	4445	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	972.1;
	4445	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3879.5;
	4445	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10314.2;
	4445	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6395.9;
	4445	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10967.7;
	4445	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1787;
	4446	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.9;
	4446	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	827.7;
	4446	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	959.2;
	4446	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3878.6;
	4446	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10012.8;
	4446	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6416.2;
	4446	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10897.8;
	4446	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1789.3;
	4447	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.4;
	4447	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	817.2;
	4447	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	956.8;
	4447	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3876.8;
	4447	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9963.8;
	4447	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6424.3;
	4447	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11020.1;
	4447	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1775.8;
	4448	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.8;
	4448	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	841;
	4448	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1013;
	4448	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4168.8;
	4448	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10186.5;
	4448	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6814.6;
	4448	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11789;
	4448	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1859.8;
	4449	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	998.1;
	4449	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	879.5;
	4449	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1090.2;
	4449	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4602;
	4449	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10674.7;
	4449	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7447.2;
	4449	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12812.5;
	4449	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1904.8;
	4450	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1038.6;
	4450	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	939.7;
	4450	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1174.4;
	4450	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5007.4;
	4450	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11314.4;
	4450	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8196.1;
	4450	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13843.3;
	4450	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2020.3;
	4451	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1084;
	4451	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1000.5;
	4451	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1269.6;
	4451	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5354.6;
	4451	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12160.2;
	4451	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8867.4;
	4451	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14765.4;
	4451	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2141.5;
	4452	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1129.5;
	4452	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1057.6;
	4452	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1354.4;
	4452	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5640.7;
	4452	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13128.5;
	4452	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9430.6;
	4452	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15402.6;
	4452	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2201.4;
	4453	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1174;
	4453	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1126.1;
	4453	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1420.3;
	4453	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5875.3;
	4453	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14263.5;
	4453	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9943.9;
	4453	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15902;
	4453	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2247.3;
	4454	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1207.1;
	4454	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1171.1;
	4454	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1474.1;
	4454	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6044.8;
	4454	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15239;
	4454	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10334.6;
	4454	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16166.2;
	4454	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2333.3;
	4455	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1229.6;
	4455	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1210.7;
	4455	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1515.7;
	4455	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6166.6;
	4455	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16014.7;
	4455	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10630.8;
	4455	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16278.2;
	4455	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2462.7;
	4456	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1244;
	4456	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1240.4;
	4456	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1548.5;
	4456	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6196.5;
	4456	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16576.1;
	4456	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10789.9;
	4456	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16294.5;
	4456	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2553.2;
	4457	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1240.7;
	4457	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1234.2;
	4457	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1568.3;
	4457	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6135.1;
	4457	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16919.1;
	4457	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10846.6;
	4457	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16102.3;
	4457	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2562.3;
	4458	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1220.2;
	4458	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1196.5;
	4458	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1536;
	4458	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5950.3;
	4458	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16811.3;
	4458	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10665.4;
	4458	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15638.2;
	4458	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2534.3;
	4459	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1188.4;
	4459	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1140.4;
	4459	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1451.1;
	4459	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5716.4;
	4459	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16311.7;
	4459	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10236.7;
	4459	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14995.9;
	4459	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2468.7;
	4460	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1144.6;
	4460	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1091.3;
	4460	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1366.8;
	4460	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5479.8;
	4460	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15601.9;
	4460	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9765.5;
	4460	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14480.1;
	4460	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2362;
	4461	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1124.4;
	4461	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1062.7;
	4461	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1302;
	4461	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5323.7;
	4461	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15031;
	4461	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9377.7;
	4461	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13993.1;
	4461	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2183.5;
	4462	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1093.6;
	4462	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1027.2;
	4462	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1231.1;
	4462	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5107.9;
	4462	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14282.8;
	4462	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8852.3;
	4462	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13428.7;
	4462	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2100.6;
	4463	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1055;
	4463	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	980.5;
	4463	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1160.3;
	4463	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4833.6;
	4463	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13414.6;
	4463	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8207.8;
	4463	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12785;
	4463	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2062.7;
	4464	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1020.1;
	4464	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	937.9;
	4464	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1063.7;
	4464	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4513.1;
	4464	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12263.3;
	4464	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7498;
	4464	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12059.8;
	4464	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1942.3;
	4465	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	994.7;
	4465	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	888.3;
	4465	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1002.3;
	4465	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4279.6;
	4465	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11075.7;
	4465	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6968;
	4465	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11538.3;
	4465	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1825;
	4466	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	973.3;
	4466	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	852.6;
	4466	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	964.8;
	4466	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4115.3;
	4466	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10559.5;
	4466	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6636.8;
	4466	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11233.4;
	4466	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1751;
	4467	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.9;
	4467	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	832;
	4467	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	939;
	4467	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3980.1;
	4467	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10366.5;
	4467	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6433.9;
	4467	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11077.6;
	4467	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1723;
	4468	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.3;
	4468	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	814.3;
	4468	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	927.5;
	4468	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3914.9;
	4468	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10381.8;
	4468	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6347.1;
	4468	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11120.5;
	4468	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1709.6;
	4469	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.9;
	4469	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	825.1;
	4469	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	939.4;
	4469	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3947.4;
	4469	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10772.6;
	4469	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6490.1;
	4469	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11502.2;
	4469	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1747.2;
	4470	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.7;
	4470	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	841.4;
	4470	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	968.5;
	4470	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4026.1;
	4470	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11311.9;
	4470	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6819;
	4470	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11827.2;
	4470	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1854.5;
	4471	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.6;
	4471	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	880.4;
	4471	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	996;
	4471	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4095.5;
	4471	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11867.3;
	4471	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6998;
	4471	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12106.9;
	4471	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1895.8;
	4472	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.5;
	4472	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	915.5;
	4472	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1044.8;
	4472	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4407.5;
	4472	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12636.8;
	4472	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7318.2;
	4472	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12801.2;
	4472	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2074.3;
	4473	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1038.3;
	4473	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	945.7;
	4473	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1109.5;
	4473	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4814.4;
	4473	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13430.7;
	4473	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7908.7;
	4473	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13738.9;
	4473	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2125.5;
	4474	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1087.6;
	4474	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	981.6;
	4474	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1182.3;
	4474	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5198.4;
	4474	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14354.2;
	4474	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8567.4;
	4474	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14832.6;
	4474	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2241.8;
	4475	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1131.9;
	4475	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1031.6;
	4475	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1248.2;
	4475	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5557.6;
	4475	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15173.4;
	4475	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9203.6;
	4475	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15722.4;
	4475	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2420;
	4476	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1167;
	4476	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1086.8;
	4476	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1311.8;
	4476	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5841.1;
	4476	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15970.8;
	4476	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9812.8;
	4476	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16353.6;
	4476	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2506.5;
	4477	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1209.5;
	4477	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1151.9;
	4477	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1374.5;
	4477	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6117.7;
	4477	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16711.4;
	4477	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10391.1;
	4477	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16900.1;
	4477	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2592.4;
	4478	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1239.7;
	4478	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1203.1;
	4478	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1423.4;
	4478	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6311.1;
	4478	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17371.4;
	4478	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10891.7;
	4478	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17273.1;
	4478	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2736.6;
	4479	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1255.2;
	4479	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1231.4;
	4479	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1451.3;
	4479	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6420.4;
	4479	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17675.1;
	4479	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11270;
	4479	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17381.4;
	4479	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2825.6;
	4480	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1254.6;
	4480	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1252;
	4480	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1467.7;
	4480	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6436.4;
	4480	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17814.6;
	4480	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11481.2;
	4480	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17229.6;
	4480	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2850;
	4481	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1223.1;
	4481	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1265.5;
	4481	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1453;
	4481	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6339.2;
	4481	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18168.2;
	4481	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11528.1;
	4481	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17030.8;
	4481	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2850.1;
	4482	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1166.6;
	4482	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1256;
	4482	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1434.3;
	4482	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6155.2;
	4482	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18142.3;
	4482	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11336.7;
	4482	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16523.9;
	4482	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2803.8;
	4483	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1123.8;
	4483	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1217.4;
	4483	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1385.5;
	4483	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5912.5;
	4483	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17720.4;
	4483	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10928.1;
	4483	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15978.3;
	4483	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2709.5;
	4484	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1095.7;
	4484	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1165;
	4484	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1327.4;
	4484	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5722.8;
	4484	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17056.3;
	4484	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10463.9;
	4484	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15511.2;
	4484	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2611.1;
	4485	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1094;
	4485	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1142.7;
	4485	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1296.2;
	4485	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5645.6;
	4485	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16615.8;
	4485	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10095.8;
	4485	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15129.8;
	4485	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2541.3;
	4486	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1062.8;
	4486	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1091.9;
	4486	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1216.5;
	4486	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5331.6;
	4486	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15533.3;
	4486	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9246.5;
	4486	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14232.9;
	4486	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2382.6;
	4487	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1018.4;
	4487	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1015.1;
	4487	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1124;
	4487	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4951.9;
	4487	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14231.1;
	4487	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8378.3;
	4487	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13267.9;
	4487	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2176.5;
	4488	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989.8;
	4488	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	958.1;
	4488	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1070.4;
	4488	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4622.2;
	4488	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13235.6;
	4488	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7598.1;
	4488	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12415.5;
	4488	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2049.6;
	4489	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	974.9;
	4489	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	925.8;
	4489	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1028.7;
	4489	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4402.8;
	4489	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12539.9;
	4489	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7102.6;
	4489	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11908.1;
	4489	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1954.7;
	4490	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.1;
	4490	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	897.1;
	4490	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	992.3;
	4490	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4207.6;
	4490	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11936.7;
	4490	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6789.5;
	4490	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11542.4;
	4490	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1884.8;
	4491	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.1;
	4491	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	882.5;
	4491	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	971;
	4491	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4079.3;
	4491	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11545.3;
	4491	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6596.3;
	4491	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11362.6;
	4491	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1856.4;
	4492	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.3;
	4492	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	862.2;
	4492	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	960.2;
	4492	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4018.2;
	4492	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11438;
	4492	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6546.9;
	4492	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11361.9;
	4492	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1778.1;
	4493	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.1;
	4493	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	873.4;
	4493	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	973.5;
	4493	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4046.7;
	4493	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11683;
	4493	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6714.2;
	4493	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11671.1;
	4493	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1795.4;
	4494	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.6;
	4494	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	894.1;
	4494	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1004.4;
	4494	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4137.2;
	4494	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12192.9;
	4494	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7060.8;
	4494	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11899.3;
	4494	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1864.2;
	4495	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970.9;
	4495	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	919.4;
	4495	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1029.4;
	4495	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4180.3;
	4495	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12554.6;
	4495	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7169.5;
	4495	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12187.5;
	4495	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1927.8;
	4496	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009.5;
	4496	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	980.4;
	4496	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1079.6;
	4496	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4456.9;
	4496	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13347.8;
	4496	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7543.9;
	4496	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12958.5;
	4496	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2041.8;
	4497	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1047.5;
	4497	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1045.1;
	4497	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1144.7;
	4497	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4835.9;
	4497	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14378.8;
	4497	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8083.6;
	4497	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13910.9;
	4497	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2192.7;
	4498	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1086.6;
	4498	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1119.8;
	4498	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1219.9;
	4498	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5203.9;
	4498	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15589.1;
	4498	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8736.3;
	4498	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14922.7;
	4498	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2357.9;
	4499	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1131.6;
	4499	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1186.5;
	4499	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1304.5;
	4499	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5546.9;
	4499	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16636;
	4499	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9348.8;
	4499	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15786.8;
	4499	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2514.7;
	4500	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1166;
	4500	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1250.5;
	4500	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1376;
	4500	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5821.3;
	4500	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17683.1;
	4500	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9920.1;
	4500	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16381.4;
	4500	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2649.4;
	4501	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1204.3;
	4501	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1298.4;
	4501	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1441.6;
	4501	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6048.1;
	4501	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18580.4;
	4501	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10429;
	4501	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16800.6;
	4501	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2780.6;
	4502	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1235.7;
	4502	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1341.3;
	4502	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1496.4;
	4502	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6248.5;
	4502	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19336.4;
	4502	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10819.4;
	4502	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17101.2;
	4502	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2863.5;
	4503	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1254.3;
	4503	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1366.3;
	4503	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1539.4;
	4503	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6328.5;
	4503	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19727;
	4503	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11121.4;
	4503	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16998.5;
	4503	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2901.5;
	4504	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1262;
	4504	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1376.9;
	4504	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1566.2;
	4504	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6299.4;
	4504	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19989.9;
	4504	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11221.8;
	4504	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16936.2;
	4504	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2889.9;
	4505	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1261.2;
	4505	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1373.3;
	4505	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1568.5;
	4505	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6230;
	4505	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19955.7;
	4505	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11290.5;
	4505	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16771;
	4505	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2865.8;
	4506	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1248.2;
	4506	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1350.7;
	4506	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1546.9;
	4506	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6080.1;
	4506	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19642;
	4506	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11204.4;
	4506	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16357;
	4506	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2821.7;
	4507	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1209.5;
	4507	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1306.5;
	4507	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1493.6;
	4507	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5837.5;
	4507	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18906.3;
	4507	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10791.9;
	4507	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15840;
	4507	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2705.2;
	4508	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1128.4;
	4508	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1245.9;
	4508	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1420.6;
	4508	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5649.3;
	4508	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17983.6;
	4508	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10250.6;
	4508	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15383.1;
	4508	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2585.1;
	4509	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1089;
	4509	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1200.6;
	4509	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1375.7;
	4509	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5560.2;
	4509	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17439.2;
	4509	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9934.6;
	4509	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14896.2;
	4509	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2505.4;
	4510	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1060.7;
	4510	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1142.8;
	4510	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1286.2;
	4510	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5263.4;
	4510	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16179.2;
	4510	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9164.8;
	4510	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13988.4;
	4510	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2324.7;
	4511	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1015;
	4511	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1050.1;
	4511	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1177.2;
	4511	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4876.2;
	4511	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14773.5;
	4511	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8284.6;
	4511	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13052.7;
	4511	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2129;
	4512	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985.2;
	4512	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	984.2;
	4512	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1101.7;
	4512	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4531.9;
	4512	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13532.1;
	4512	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7541.6;
	4512	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12302.1;
	4512	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1960.5;
	4513	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	968.9;
	4513	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	939.2;
	4513	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1043.6;
	4513	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4310;
	4513	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12619.1;
	4513	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7040.6;
	4513	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11732.6;
	4513	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1851.3;
	4514	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.2;
	4514	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	917;
	4514	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	996.9;
	4514	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4147.9;
	4514	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11981;
	4514	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6689;
	4514	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11345.7;
	4514	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1772.2;
	4515	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946;
	4515	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	902.6;
	4515	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	965.3;
	4515	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4021.2;
	4515	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11626.1;
	4515	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6479.8;
	4515	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11111.8;
	4515	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1720.6;
	4516	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.8;
	4516	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	884.9;
	4516	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	953.8;
	4516	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3968.5;
	4516	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11586.3;
	4516	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6416.2;
	4516	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11132.9;
	4516	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1714.8;
	4517	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.6;
	4517	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	892.6;
	4517	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	962.3;
	4517	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4018.5;
	4517	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11861;
	4517	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6580.4;
	4517	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11511.6;
	4517	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1765.4;
	4518	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.4;
	4518	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	906.1;
	4518	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	994.3;
	4518	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4137.2;
	4518	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12286.1;
	4518	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6915.2;
	4518	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11841.1;
	4518	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1850.2;
	4519	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.1;
	4519	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	933;
	4519	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1019.5;
	4519	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4205.2;
	4519	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12640.6;
	4519	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7091.5;
	4519	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12166.1;
	4519	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1909.9;
	4520	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	994;
	4520	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	998.1;
	4520	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1069.2;
	4520	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4463.9;
	4520	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13414.7;
	4520	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7419.7;
	4520	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12904.4;
	4520	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2048.7;
	4521	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1031.4;
	4521	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1048.9;
	4521	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1132.3;
	4521	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4838.8;
	4521	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14378.6;
	4521	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7976.4;
	4521	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13801.6;
	4521	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2237.4;
	4522	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1076.5;
	4522	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1115.8;
	4522	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1212.3;
	4522	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5218;
	4522	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15490.3;
	4522	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8618.1;
	4522	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14783.3;
	4522	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2372.1;
	4523	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1122.1;
	4523	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1170.7;
	4523	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1291.2;
	4523	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5583.1;
	4523	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16542.9;
	4523	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9247.3;
	4523	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15687.8;
	4523	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2596.3;
	4524	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1162.3;
	4524	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1241.2;
	4524	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1357.1;
	4524	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5858.7;
	4524	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17464.4;
	4524	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9801.7;
	4524	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16372.2;
	4524	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2714.6;
	4525	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1199;
	4525	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1322.7;
	4525	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1427.4;
	4525	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6086.2;
	4525	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18347.4;
	4525	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10361.7;
	4525	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16860.2;
	4525	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2862.4;
	4526	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1236.9;
	4526	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1375.5;
	4526	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1489.4;
	4526	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6269.5;
	4526	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19124.1;
	4526	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10846;
	4526	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17287.6;
	4526	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2903.7;
	4527	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1255.3;
	4527	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1396.2;
	4527	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1533.1;
	4527	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6322;
	4527	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19546;
	4527	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11182.9;
	4527	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17225.2;
	4527	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2941.1;
	4528	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1260.4;
	4528	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1402.6;
	4528	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1555.4;
	4528	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6284.1;
	4528	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19747.4;
	4528	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11385.7;
	4528	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17089.8;
	4528	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2946;
	4529	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1252.9;
	4529	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1399.9;
	4529	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1551.6;
	4529	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6195.5;
	4529	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19802.3;
	4529	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11390.2;
	4529	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16855;
	4529	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2924.3;
	4530	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1235.1;
	4530	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1378;
	4530	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1525;
	4530	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6024.5;
	4530	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19434.7;
	4530	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11174.2;
	4530	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16335.2;
	4530	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2892.3;
	4531	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1207.4;
	4531	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1328.8;
	4531	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1472.4;
	4531	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5792.7;
	4531	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18765.1;
	4531	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10751.1;
	4531	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15891;
	4531	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2777.8;
	4532	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1155.7;
	4532	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1252.9;
	4532	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1401.3;
	4532	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5623;
	4532	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17925;
	4532	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10244.2;
	4532	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15446.5;
	4532	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2685.1;
	4533	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1118.2;
	4533	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1217.2;
	4533	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1358.5;
	4533	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5529.9;
	4533	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17392.7;
	4533	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9869.8;
	4533	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15013;
	4533	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2610.8;
	4534	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1087.3;
	4534	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1150.8;
	4534	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1278.6;
	4534	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5217;
	4534	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16155.1;
	4534	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9076.1;
	4534	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14103.5;
	4534	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2426.3;
	4535	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1039.6;
	4535	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1074.5;
	4535	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1184.2;
	4535	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4864.9;
	4535	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14840;
	4535	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8163.2;
	4535	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13167.3;
	4535	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2252.4;
	4536	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1005.7;
	4536	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1013.6;
	4536	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1106.8;
	4536	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4593.4;
	4536	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13653.5;
	4536	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7482;
	4536	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12341.3;
	4536	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2082.1;
	4537	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	977.6;
	4537	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	968.6;
	4537	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1043.8;
	4537	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4361;
	4537	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12786.8;
	4537	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6987;
	4537	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11724.6;
	4537	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1960.4;
	4538	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.6;
	4538	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	925;
	4538	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	998.9;
	4538	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4182;
	4538	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12103.5;
	4538	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6678.2;
	4538	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11242.4;
	4538	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1861.7;
	4539	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.1;
	4539	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	896.7;
	4539	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	968.2;
	4539	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4060.6;
	4539	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11593.3;
	4539	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6472.9;
	4539	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10936.7;
	4539	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1823.5;
	4540	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933;
	4540	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	872.8;
	4540	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	951.7;
	4540	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4003.2;
	4540	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11352.9;
	4540	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6430.9;
	4540	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10902.6;
	4540	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1786.8;
	4541	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934;
	4541	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	881.4;
	4541	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	958.3;
	4541	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4032;
	4541	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11527.9;
	4541	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6580.9;
	4541	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11166.1;
	4541	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1810.9;
	4542	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.2;
	4542	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	895.1;
	4542	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	981.9;
	4542	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4100.5;
	4542	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11868;
	4542	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6900.3;
	4542	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11459.8;
	4542	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1871;
	4543	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948;
	4543	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	919.2;
	4543	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1007.2;
	4543	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4150.2;
	4543	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12202;
	4543	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7052.3;
	4543	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11847.4;
	4543	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1943;
	4544	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.5;
	4544	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	984.4;
	4544	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1067.6;
	4544	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4454.1;
	4544	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12973.2;
	4544	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7386.3;
	4544	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12729.8;
	4544	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2088.6;
	4545	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1008.1;
	4545	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1053.7;
	4545	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1129.7;
	4545	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4830.2;
	4545	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14022.8;
	4545	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7987.6;
	4545	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13734;
	4545	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2130.4;
	4546	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1055.3;
	4546	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1125.8;
	4546	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1195.9;
	4546	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5211.1;
	4546	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15185.4;
	4546	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8687.2;
	4546	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14760.5;
	4546	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2333.1;
	4547	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1107.1;
	4547	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1188.5;
	4547	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1274.9;
	4547	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5535.5;
	4547	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16229.5;
	4547	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9327.9;
	4547	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15594.7;
	4547	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2529.5;
	4548	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1148.5;
	4548	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1245.5;
	4548	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1348.7;
	4548	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5797.9;
	4548	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17237.8;
	4548	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9884.1;
	4548	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16324.2;
	4548	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2632.8;
	4549	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1190.3;
	4549	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1294.3;
	4549	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1417.1;
	4549	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6029;
	4549	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18152.4;
	4549	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10401.9;
	4549	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17060.5;
	4549	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2728.4;
	4550	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1224.1;
	4550	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1328.2;
	4550	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1475.8;
	4550	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6213.3;
	4550	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18894.4;
	4550	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10835.6;
	4550	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17479.6;
	4550	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2798;
	4551	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1242.6;
	4551	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1352.4;
	4551	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1519.8;
	4551	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6300.7;
	4551	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19374.2;
	4551	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11134.2;
	4551	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17485.8;
	4551	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2902.9;
	4552	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1243.4;
	4552	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1360;
	4552	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1536.4;
	4552	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6298.8;
	4552	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19598.1;
	4552	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11254.3;
	4552	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17349.2;
	4552	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2909.7;
	4553	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1235.3;
	4553	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1340;
	4553	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1525;
	4553	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6186.4;
	4553	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19563.1;
	4553	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11215.5;
	4553	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17042.6;
	4553	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2909.7;
	4554	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1214.2;
	4554	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1295.7;
	4554	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1487.5;
	4554	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5984.4;
	4554	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19122.6;
	4554	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10934.5;
	4554	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16541.4;
	4554	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2857.7;
	4555	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1176.3;
	4555	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1232.9;
	4555	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1435.9;
	4555	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5736.5;
	4555	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18317.1;
	4555	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10442.3;
	4555	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15858.1;
	4555	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2763.7;
	4556	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1118.5;
	4556	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1179.4;
	4556	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1365;
	4556	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5548.4;
	4556	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17401.3;
	4556	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9896.4;
	4556	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15187.8;
	4556	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2648.8;
	4557	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1109.4;
	4557	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1131.1;
	4557	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1322.5;
	4557	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5448.2;
	4557	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16743.1;
	4557	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9514.4;
	4557	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14723.9;
	4557	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2532.5;
	4558	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1080.3;
	4558	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1064.5;
	4558	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1243.9;
	4558	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5195.1;
	4558	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15577.1;
	4558	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8843;
	4558	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14007.9;
	4558	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2331.3;
	4559	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1037.4;
	4559	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	985.9;
	4559	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1152.6;
	4559	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4892.3;
	4559	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14308.6;
	4559	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8059.2;
	4559	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13079.3;
	4559	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2170.1;
	4560	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	993.7;
	4560	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	919.4;
	4560	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1090.7;
	4560	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4602.2;
	4560	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13140;
	4560	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7363.5;
	4560	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12180.3;
	4560	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2028.6;
	4561	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.5;
	4561	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	872.5;
	4561	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1027.5;
	4561	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4360.7;
	4561	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12370.5;
	4561	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6865.8;
	4561	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11497.5;
	4561	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1888.4;
	4562	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.8;
	4562	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	844.5;
	4562	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	988.1;
	4562	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4167.8;
	4562	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11804.7;
	4562	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6536.5;
	4562	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11000.4;
	4562	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1780.3;
	4563	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.6;
	4563	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	831;
	4563	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	953.1;
	4563	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4027.1;
	4563	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11360.3;
	4563	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6315.9;
	4563	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10656.8;
	4563	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1774.3;
	4564	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.4;
	4564	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	813.8;
	4564	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	931.3;
	4564	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3946;
	4564	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11053.8;
	4564	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6188;
	4564	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10487.2;
	4564	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1751.3;
	4565	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.6;
	4565	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	812.2;
	4565	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	936.1;
	4565	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3923.6;
	4565	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10950.2;
	4565	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6186;
	4565	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10465.9;
	4565	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1744.2;
	4566	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.8;
	4566	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	813.3;
	4566	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	938.7;
	4566	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3918.6;
	4566	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10887.4;
	4566	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6245.8;
	4566	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10480.6;
	4566	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1725.6;
	4567	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.1;
	4567	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	825.8;
	4567	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	944.4;
	4567	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3901.4;
	4567	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11151.9;
	4567	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6372.4;
	4567	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10750.9;
	4567	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1754.2;
	4568	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.6;
	4568	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	882.4;
	4568	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	990.4;
	4568	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4177.9;
	4568	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12087.8;
	4568	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6799.4;
	4568	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11735.6;
	4568	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1896.7;
	4569	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	997;
	4569	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	949.4;
	4569	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1049.1;
	4569	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4569.6;
	4569	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13238.3;
	4569	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7423.5;
	4569	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12889.9;
	4569	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2097.3;
	4570	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1041.1;
	4570	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1026;
	4570	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1114.2;
	4570	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4983.9;
	4570	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14457.8;
	4570	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8128.5;
	4570	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14029.5;
	4570	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2277.2;
	4571	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1078.4;
	4571	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1089.4;
	4571	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1179.8;
	4571	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5357.8;
	4571	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15512.1;
	4571	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8773.7;
	4571	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14995.5;
	4571	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2436.1;
	4572	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1106.9;
	4572	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1148.6;
	4572	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1243.7;
	4572	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5664.4;
	4572	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16414.5;
	4572	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9387;
	4572	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15749.3;
	4572	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2556.1;
	4573	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1138.4;
	4573	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1173.6;
	4573	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1301.2;
	4573	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5891.2;
	4573	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17156.9;
	4573	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9837.9;
	4573	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16237.6;
	4573	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2686;
	4574	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1162.6;
	4574	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1111.5;
	4574	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1359.8;
	4574	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6042.1;
	4574	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17781.2;
	4574	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10223.5;
	4574	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16564.8;
	4574	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2735.5;
	4575	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1185.8;
	4575	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1094.5;
	4575	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1412.9;
	4575	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6114.4;
	4575	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18027.2;
	4575	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10481.8;
	4575	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16788.1;
	4575	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2793.6;
	4576	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1191;
	4576	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1084.4;
	4576	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1437.2;
	4576	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6108.2;
	4576	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16988;
	4576	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10709.3;
	4576	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16745.2;
	4576	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2788.8;
	4577	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1186.1;
	4577	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1060.9;
	4577	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1439;
	4577	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6013.9;
	4577	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15375.6;
	4577	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10785;
	4577	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16464.4;
	4577	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2779.4;
	4578	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1178.7;
	4578	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1024.7;
	4578	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1422;
	4578	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5848.4;
	4578	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14437.9;
	4578	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10592.6;
	4578	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16034.2;
	4578	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2693.3;
	4579	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1157.4;
	4579	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	983.5;
	4579	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1380.3;
	4579	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5610.9;
	4579	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13625.4;
	4579	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10206.2;
	4579	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15381;
	4579	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2608.9;
	4580	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1127.3;
	4580	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	944.3;
	4580	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1314.5;
	4580	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5433;
	4580	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13205.4;
	4580	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9805.6;
	4580	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14904.4;
	4580	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2507.6;
	4581	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1109.6;
	4581	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	933;
	4581	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1263.2;
	4581	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5338.9;
	4581	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12936.6;
	4581	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9523.7;
	4581	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14484.4;
	4581	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2399.3;
	4582	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1075.6;
	4582	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	892.9;
	4582	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1192;
	4582	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5087.2;
	4582	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12182.1;
	4582	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8896.5;
	4582	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13670.4;
	4582	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2252.8;
	4583	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1036.6;
	4583	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	839;
	4583	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1110.3;
	4583	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4808.6;
	4583	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11269.7;
	4583	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8218.5;
	4583	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12920.4;
	4583	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2093.4;
	4584	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.8;
	4584	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	795.1;
	4584	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1026.3;
	4584	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4551.2;
	4584	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10481.5;
	4584	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7540.1;
	4584	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12197.2;
	4584	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1921.1;
	4585	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.6;
	4585	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	759.4;
	4585	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	973.9;
	4585	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4311.6;
	4585	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9856.8;
	4585	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6993.8;
	4585	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11565.3;
	4585	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1845.6;
	4586	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.7;
	4586	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.9;
	4586	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	934.2;
	4586	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4133.8;
	4586	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9368.2;
	4586	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6602.9;
	4586	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11085.6;
	4586	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1798.7;
	4587	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.3;
	4587	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	724.9;
	4587	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	906.2;
	4587	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4005.8;
	4587	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9030.2;
	4587	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6353;
	4587	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10789.1;
	4587	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1749.5;
	4588	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.6;
	4588	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	710.1;
	4588	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889;
	4588	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3906.7;
	4588	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8835.7;
	4588	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6224.5;
	4588	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10657.7;
	4588	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1701.5;
	4589	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.9;
	4589	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711;
	4589	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	883.6;
	4589	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3857.3;
	4589	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8833.4;
	4589	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6210.7;
	4589	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10644;
	4589	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1714.3;
	4590	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916;
	4590	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711.8;
	4590	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.2;
	4590	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3820.2;
	4590	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8887.1;
	4590	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6228.9;
	4590	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10670;
	4590	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1716.4;
	4591	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.7;
	4591	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.3;
	4591	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	900.3;
	4591	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3791.2;
	4591	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9230.9;
	4591	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6272.2;
	4591	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10753;
	4591	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1713.2;
	4592	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.5;
	4592	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	800.1;
	4592	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	962.2;
	4592	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4048.7;
	4592	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10258.2;
	4592	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6584.5;
	4592	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11273.9;
	4592	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1827.1;
	4593	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	995.5;
	4593	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	868.5;
	4593	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1027.5;
	4593	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4424.3;
	4593	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11512.3;
	4593	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7070.3;
	4593	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12089;
	4593	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1895.8;
	4594	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1029.2;
	4594	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	931.3;
	4594	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1095.7;
	4594	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4737.3;
	4594	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12735.5;
	4594	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7646.4;
	4594	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12788.1;
	4594	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2037.6;
	4595	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1064.9;
	4595	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	995.9;
	4595	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1171.1;
	4595	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5020.1;
	4595	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13968.9;
	4595	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8322.5;
	4595	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13420.7;
	4595	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2209.6;
	4596	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1103.5;
	4596	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1069.7;
	4596	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1252.6;
	4596	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5282.8;
	4596	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15204.7;
	4596	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8908;
	4596	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14209.8;
	4596	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2357.2;
	4597	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1142.3;
	4597	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1138.9;
	4597	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1329.3;
	4597	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5496.5;
	4597	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16219.2;
	4597	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9400.8;
	4597	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14935.5;
	4597	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2504;
	4598	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1172.7;
	4598	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1189.3;
	4598	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1391.8;
	4598	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5657.4;
	4598	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17046.6;
	4598	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9853.3;
	4598	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15424.8;
	4598	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2591.9;
	4599	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1193.6;
	4599	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1228.4;
	4599	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1436.5;
	4599	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5757.8;
	4599	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17665.6;
	4599	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10211.1;
	4599	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15745.9;
	4599	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2646;
	4600	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1205.6;
	4600	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1252.1;
	4600	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1472.4;
	4600	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5787.5;
	4600	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18057.6;
	4600	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10500.6;
	4600	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15875.1;
	4600	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2728.4;
	4601	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1215.4;
	4601	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1262.5;
	4601	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1498.5;
	4601	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5730.5;
	4601	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18247.7;
	4601	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10586.8;
	4601	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15880.6;
	4601	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2738.5;
	4602	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1206;
	4602	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1239.1;
	4602	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1486.8;
	4602	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5616.1;
	4602	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18010.4;
	4602	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10471;
	4602	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15554.9;
	4602	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2688.2;
	4603	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1186.8;
	4603	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1191.2;
	4603	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1417.7;
	4603	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5410.8;
	4603	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17385.8;
	4603	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10124.4;
	4603	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14960.1;
	4603	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2584.1;
	4604	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1158.9;
	4604	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1149.4;
	4604	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1341.4;
	4604	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5286.2;
	4604	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16656.6;
	4604	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9786.9;
	4604	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14590.8;
	4604	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2483.5;
	4605	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1146.9;
	4605	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1132.4;
	4605	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1309;
	4605	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5269;
	4605	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16220.6;
	4605	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9559.4;
	4605	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14334.5;
	4605	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2356.1;
	4606	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1112.4;
	4606	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1061.5;
	4606	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1233.7;
	4606	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5035.6;
	4606	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15137;
	4606	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8937.2;
	4606	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13590.4;
	4606	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2210;
	4607	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1065.5;
	4607	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	990.2;
	4607	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1137.9;
	4607	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4723.8;
	4607	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13842.2;
	4607	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8158.7;
	4607	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12681.6;
	4607	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2115.4;
	4608	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1031.6;
	4608	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	927.1;
	4608	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1067.5;
	4608	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4464.8;
	4608	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12827.9;
	4608	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7327.7;
	4608	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11994.7;
	4608	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1975.4;
	4609	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1006.7;
	4609	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	883.9;
	4609	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1015.6;
	4609	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4258.5;
	4609	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12050.4;
	4609	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6813;
	4609	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11523.8;
	4609	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1874.9;
	4610	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	982.5;
	4610	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	854.4;
	4610	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	975.3;
	4610	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4100.9;
	4610	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11478.9;
	4610	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6507.2;
	4610	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11199.8;
	4610	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1804.4;
	4611	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969.6;
	4611	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	834.9;
	4611	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	947.5;
	4611	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4015;
	4611	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11172.7;
	4611	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6402.3;
	4611	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11114.6;
	4611	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1754.6;
	4612	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.8;
	4612	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	830.5;
	4612	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	935.2;
	4612	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3972.3;
	4612	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11144.8;
	4612	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6373.4;
	4612	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11225.7;
	4612	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1763.4;
	4613	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.4;
	4613	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	850.4;
	4613	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	946.2;
	4613	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4026.2;
	4613	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11503.7;
	4613	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6613.2;
	4613	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11582.5;
	4613	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1825.3;
	4614	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.4;
	4614	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	872.3;
	4614	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	978.9;
	4614	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4129.7;
	4614	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11994.9;
	4614	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6989.6;
	4614	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11863;
	4614	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1896.1;
	4615	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969.2;
	4615	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	900.2;
	4615	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1000.6;
	4615	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4167.7;
	4615	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12394.7;
	4615	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7154.6;
	4615	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12014.7;
	4615	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1977.7;
	4616	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000.5;
	4616	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	964.9;
	4616	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1060.2;
	4616	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4418.8;
	4616	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13060.8;
	4616	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7500.1;
	4616	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12692.7;
	4616	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2114.8;
	4617	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1041.7;
	4617	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1024.5;
	4617	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1130.2;
	4617	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4786.9;
	4617	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13830.1;
	4617	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8026.6;
	4617	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13651.8;
	4617	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2274;
	4618	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1088.1;
	4618	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1082.5;
	4618	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1200.3;
	4618	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5137.1;
	4618	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14777;
	4618	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8685.6;
	4618	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14655.2;
	4618	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2466.9;
	4619	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1134.4;
	4619	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1140.8;
	4619	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1279.1;
	4619	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5481.8;
	4619	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15991.7;
	4619	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9278.3;
	4619	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15434.2;
	4619	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2595;
	4620	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1177.2;
	4620	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1204.4;
	4620	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1349.5;
	4620	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5780.7;
	4620	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17061.3;
	4620	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9831.5;
	4620	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16106.7;
	4620	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2700.4;
	4621	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1217.1;
	4621	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1263.4;
	4621	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1413.8;
	4621	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6046.8;
	4621	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18053.8;
	4621	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10306.9;
	4621	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16738.7;
	4621	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2724;
	4622	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1251;
	4622	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1300;
	4622	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1465.8;
	4622	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6221.3;
	4622	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18766.3;
	4622	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10720.2;
	4622	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17082.7;
	4622	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2828.8;
	4623	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1275;
	4623	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1332.9;
	4623	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1508.1;
	4623	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6329.3;
	4623	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19243.2;
	4623	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11060.8;
	4623	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17276.2;
	4623	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2885.4;
	4624	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1290.3;
	4624	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1340;
	4624	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1533.4;
	4624	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6348.2;
	4624	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19502.3;
	4624	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11283.1;
	4624	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17182.7;
	4624	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2862.1;
	4625	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1286.6;
	4625	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1332.8;
	4625	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1529.1;
	4625	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6236.2;
	4625	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19477.5;
	4625	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11234.2;
	4625	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16909.3;
	4625	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2864.2;
	4626	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1273.7;
	4626	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1312.5;
	4626	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1508.2;
	4626	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6027.1;
	4626	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19079.5;
	4626	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10982.2;
	4626	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16315.4;
	4626	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2836.4;
	4627	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1245.6;
	4627	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1268.4;
	4627	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1458.6;
	4627	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5772.6;
	4627	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18325.9;
	4627	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10484.2;
	4627	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15611.9;
	4627	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2725.6;
	4628	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1211.7;
	4628	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1218.7;
	4628	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1390.2;
	4628	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5592.2;
	4628	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17437;
	4628	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9976.5;
	4628	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15094.2;
	4628	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2625.7;
	4629	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1187;
	4629	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1185.9;
	4629	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1348.7;
	4629	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5495.1;
	4629	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16942.1;
	4629	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9622.2;
	4629	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14618.8;
	4629	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2564.3;
	4630	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1145.7;
	4630	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1113.8;
	4630	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1265.5;
	4630	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5166.6;
	4630	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15682.6;
	4630	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8878.6;
	4630	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13698.6;
	4630	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2382;
	4631	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1098.8;
	4631	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1041.7;
	4631	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1166.8;
	4631	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4785.5;
	4631	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14350.4;
	4631	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8012.2;
	4631	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12772.4;
	4631	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2171.9;
	4632	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1065.3;
	4632	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	982.1;
	4632	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1091.4;
	4632	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4458.2;
	4632	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13157.4;
	4632	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7312.5;
	4632	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11994.4;
	4632	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2026.6;
	4633	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1040.5;
	4633	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	935.5;
	4633	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1035;
	4633	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4212.5;
	4633	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12329.3;
	4633	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6820;
	4633	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11406.9;
	4633	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1907.8;
	4634	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1015.6;
	4634	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	896;
	4634	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1000.2;
	4634	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4023;
	4634	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11746.3;
	4634	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6501.3;
	4634	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10959.2;
	4634	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1813;
	4635	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	997.8;
	4635	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	871;
	4635	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	972.3;
	4635	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3889.2;
	4635	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11363.8;
	4635	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6325.7;
	4635	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10719.3;
	4635	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1775.9;
	4636	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985;
	4636	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	859.8;
	4636	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	955.9;
	4636	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3830.1;
	4636	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11240.3;
	4636	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6255.5;
	4636	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10724.7;
	4636	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1755.9;
	4637	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	982.6;
	4637	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	875.9;
	4637	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	965;
	4637	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3869;
	4637	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11524.2;
	4637	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6448.8;
	4637	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11049.8;
	4637	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1787;
	4638	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990;
	4638	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	886.5;
	4638	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	986.7;
	4638	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3987.4;
	4638	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12008.1;
	4638	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6820.8;
	4638	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11358.9;
	4638	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1890.2;
	4639	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	994.9;
	4639	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	914.6;
	4639	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1003.3;
	4639	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4073.5;
	4639	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12391.7;
	4639	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6984.4;
	4639	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11719.3;
	4639	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1915.9;
	4640	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1025.3;
	4640	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	970.3;
	4640	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1051;
	4640	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4367.8;
	4640	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13065.6;
	4640	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7282.9;
	4640	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12560.4;
	4640	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2102.9;
	4641	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1065.5;
	4641	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1040.6;
	4641	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1109.3;
	4641	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4760;
	4641	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13970.4;
	4641	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7782.3;
	4641	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13492.2;
	4641	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2176.4;
	4642	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1111.5;
	4642	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1096.8;
	4642	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1185.5;
	4642	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5129.3;
	4642	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14971.9;
	4642	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8398.4;
	4642	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14497.6;
	4642	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2308.3;
	4643	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1153.6;
	4643	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1145.5;
	4643	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1258.7;
	4643	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5489.9;
	4643	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16034.3;
	4643	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9005.3;
	4643	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15378.8;
	4643	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2517.3;
	4644	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1192.7;
	4644	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1188.7;
	4644	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1335.8;
	4644	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5778.9;
	4644	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16996.7;
	4644	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9561.7;
	4644	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16090;
	4644	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2651.4;
	4645	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1232.5;
	4645	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1247.2;
	4645	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1401.8;
	4645	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6025.8;
	4645	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17876.2;
	4645	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10077.2;
	4645	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16726;
	4645	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2747.2;
	4646	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1267.1;
	4646	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1297.9;
	4646	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1452;
	4646	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6234;
	4646	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18567.6;
	4646	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10532.2;
	4646	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17116.1;
	4646	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2872.9;
	4647	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1293.2;
	4647	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1330;
	4647	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1486.8;
	4647	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6329.7;
	4647	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19149.8;
	4647	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10899.2;
	4647	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17409.9;
	4647	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2938.2;
	4648	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1306.7;
	4648	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1325.6;
	4648	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1511;
	4648	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6325.7;
	4648	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19501.8;
	4648	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11150.8;
	4648	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17435.6;
	4648	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2933.5;
	4649	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1290.4;
	4649	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1318;
	4649	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1499.7;
	4649	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6222.6;
	4649	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19520.4;
	4649	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11204.5;
	4649	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17189.4;
	4649	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2920.3;
	4650	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1261.2;
	4650	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1308.8;
	4650	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1475.4;
	4650	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6039.8;
	4650	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19149.9;
	4650	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11006.5;
	4650	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16620.9;
	4650	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2883.6;
	4651	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1237.4;
	4651	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1270.8;
	4651	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1429.9;
	4651	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5801.5;
	4651	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18373.4;
	4651	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10600.2;
	4651	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15982.6;
	4651	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2773.7;
	4652	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1201;
	4652	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1215.4;
	4652	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1369.5;
	4652	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5634.2;
	4652	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17527.6;
	4652	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10163;
	4652	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15526.4;
	4652	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2665;
	4653	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1182.6;
	4653	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1186.7;
	4653	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1337.3;
	4653	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5557.2;
	4653	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17018.4;
	4653	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9860.6;
	4653	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15080.5;
	4653	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2590.6;
	4654	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1143.1;
	4654	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1117;
	4654	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1259.8;
	4654	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5253.5;
	4654	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15792.8;
	4654	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9081.7;
	4654	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14129;
	4654	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2421;
	4655	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1093.3;
	4655	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1036.2;
	4655	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1164.3;
	4655	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4884.2;
	4655	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14441.1;
	4655	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8209.4;
	4655	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13155;
	4655	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2219.6;
	4656	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1050.8;
	4656	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	972.4;
	4656	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1086.8;
	4656	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4542.4;
	4656	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13367.2;
	4656	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7439.1;
	4656	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12310.7;
	4656	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2068;
	4657	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1023.7;
	4657	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	937;
	4657	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1032.2;
	4657	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4303.9;
	4657	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12638.6;
	4657	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6942.5;
	4657	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11722.6;
	4657	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1960.1;
	4658	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1005.6;
	4658	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	907.2;
	4658	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	993.6;
	4658	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4118.1;
	4658	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12065.4;
	4658	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6691.6;
	4658	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11327.3;
	4658	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1883.8;
	4659	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	993.8;
	4659	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	885.3;
	4659	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	963.1;
	4659	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3985.6;
	4659	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11625.5;
	4659	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6553.3;
	4659	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11088.2;
	4659	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1831.9;
	4660	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986.6;
	4660	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	871.5;
	4660	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	948.8;
	4660	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3933.1;
	4660	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11489.6;
	4660	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6535;
	4660	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11124.7;
	4660	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1762.1;
	4661	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990.5;
	4661	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	886.1;
	4661	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	960.8;
	4661	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3999.4;
	4661	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11728.4;
	4661	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6771.3;
	4661	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11489.7;
	4661	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1779;
	4662	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.6;
	4662	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	907.9;
	4662	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	989.9;
	4662	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4118.1;
	4662	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12279.6;
	4662	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7118.7;
	4662	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11813;
	4662	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1860.5;
	4663	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	998.6;
	4663	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	932.4;
	4663	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1014;
	4663	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4174.3;
	4663	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12672.3;
	4663	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7284;
	4663	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12216.6;
	4663	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1931;
	4664	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1037.8;
	4664	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	992.4;
	4664	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1069.1;
	4664	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4454.8;
	4664	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13549.1;
	4664	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7612.9;
	4664	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13052.6;
	4664	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2051.6;
	4665	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1082.4;
	4665	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1064.6;
	4665	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1140.1;
	4665	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4848.2;
	4665	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14606.3;
	4665	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8189.5;
	4665	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14067.3;
	4665	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2213.3;
	4666	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1121.6;
	4666	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1134.8;
	4666	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1216.2;
	4666	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5248.9;
	4666	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15800.3;
	4666	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8874.3;
	4666	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15134.7;
	4666	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2366.4;
	4667	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1168;
	4667	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1197.5;
	4667	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1298.5;
	4667	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5600.3;
	4667	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16825.6;
	4667	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9543.6;
	4667	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16061.6;
	4667	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2522.1;
	4668	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1200.6;
	4668	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1255.8;
	4668	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1367.2;
	4668	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5880.7;
	4668	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17794.5;
	4668	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10111.2;
	4668	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16727.1;
	4668	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2663.6;
	4669	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1238.7;
	4669	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1312.4;
	4669	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1434.7;
	4669	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6126.4;
	4669	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18721.2;
	4669	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10630.1;
	4669	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17253.4;
	4669	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2796.7;
	4670	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1269.6;
	4670	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1360;
	4670	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1490.8;
	4670	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6317.2;
	4670	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19507;
	4670	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11059.5;
	4670	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17543.9;
	4670	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2904.7;
	4671	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1281.5;
	4671	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1387.2;
	4671	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1537;
	4671	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6386.5;
	4671	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19971.9;
	4671	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11293.9;
	4671	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17543.9;
	4671	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2948.7;
	4672	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1285.9;
	4672	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1395.7;
	4672	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1564;
	4672	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6357.8;
	4672	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20284.2;
	4672	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11360.1;
	4672	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17552.2;
	4672	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2948.4;
	4673	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1287.1;
	4673	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1389.5;
	4673	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1565;
	4673	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6270;
	4673	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20330.4;
	4673	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11426.2;
	4673	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17366.4;
	4673	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2921.7;
	4674	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1278.9;
	4674	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1374;
	4674	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1544.4;
	4674	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6121.2;
	4674	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20103.8;
	4674	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11370.4;
	4674	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16871.8;
	4674	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2888.5;
	4675	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1246.8;
	4675	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1335.5;
	4675	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1496.5;
	4675	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5896.2;
	4675	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19433.9;
	4675	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10952;
	4675	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16277;
	4675	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2789.9;
	4676	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1176.9;
	4676	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1273.5;
	4676	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1425.2;
	4676	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5707.2;
	4676	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18523.6;
	4676	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10433.2;
	4676	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15726.4;
	4676	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2674.5;
	4677	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1150.4;
	4677	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1223.6;
	4677	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1376.8;
	4677	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5607.2;
	4677	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17950.5;
	4677	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10043;
	4677	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15203.3;
	4677	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2575.2;
	4678	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1117.3;
	4678	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1152.9;
	4678	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1288.8;
	4678	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5305.8;
	4678	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16692;
	4678	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9228.5;
	4678	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14264.6;
	4678	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2390;
	4679	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1071.7;
	4679	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1075.3;
	4679	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1187.1;
	4679	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4941.5;
	4679	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15386.6;
	4679	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8314.2;
	4679	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13275.9;
	4679	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2190.8;
	4680	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1038.5;
	4680	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1014.3;
	4680	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1106.3;
	4680	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4585.4;
	4680	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14213;
	4680	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7597.5;
	4680	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12432.1;
	4680	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2009.2;
	4681	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1012.3;
	4681	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	976.3;
	4681	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1048.8;
	4681	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4331.5;
	4681	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13311.3;
	4681	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7066.3;
	4681	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11804.3;
	4681	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1887.8;
	4682	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989.9;
	4682	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	944.3;
	4682	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1004;
	4682	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4144;
	4682	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12584.9;
	4682	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6716.3;
	4682	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11376.8;
	4682	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1800.3;
	4683	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	971.9;
	4683	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	922.8;
	4683	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	969.7;
	4683	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4023.1;
	4683	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12106.5;
	4683	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6498.9;
	4683	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11120.1;
	4683	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1740.8;
	4684	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958;
	4684	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	906.4;
	4684	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	949.7;
	4684	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3963.3;
	4684	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11928.7;
	4684	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6419.7;
	4684	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11040.9;
	4684	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1719.8;
	4685	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.1;
	4685	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	917.9;
	4685	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	957.2;
	4685	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4027;
	4685	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12142.7;
	4685	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6565.4;
	4685	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11378.1;
	4685	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1758.2;
	4686	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.2;
	4686	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	928.4;
	4686	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	987.1;
	4686	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4144.5;
	4686	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12494.3;
	4686	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6897.8;
	4686	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11710.5;
	4686	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1821.4;
	4687	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970.4;
	4687	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	950.4;
	4687	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1010.2;
	4687	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4191.4;
	4687	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12778.6;
	4687	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7080.8;
	4687	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12024;
	4687	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1883.8;
	4688	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1001.1;
	4688	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	990.7;
	4688	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1062.4;
	4688	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4474.7;
	4688	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13538.6;
	4688	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7410;
	4688	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12769.2;
	4688	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2014.2;
	4689	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1044;
	4689	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1037.1;
	4689	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1130.4;
	4689	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4845.1;
	4689	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14564.2;
	4689	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7980.6;
	4689	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13725.8;
	4689	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2217.4;
	4690	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1095.3;
	4690	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1106.5;
	4690	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1211.6;
	4690	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5232.2;
	4690	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15760;
	4690	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8656.9;
	4690	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14807.7;
	4690	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2328.6;
	4691	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1140.7;
	4691	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1179.9;
	4691	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1295.5;
	4691	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5593.5;
	4691	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16919.9;
	4691	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9302.9;
	4691	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15766.8;
	4691	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2567;
	4692	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1183.7;
	4692	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1250.6;
	4692	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1371.4;
	4692	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5896.5;
	4692	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17910.9;
	4692	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9916.8;
	4692	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16466.2;
	4692	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2704.9;
	4693	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1221.3;
	4693	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1320;
	4693	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1447.5;
	4693	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6166.2;
	4693	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18877.7;
	4693	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10487;
	4693	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17055.8;
	4693	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2861.6;
	4694	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1254.1;
	4694	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1368.6;
	4694	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1505.3;
	4694	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6360.3;
	4694	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19700.5;
	4694	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10984.6;
	4694	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17442.4;
	4694	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2921.7;
	4695	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1272.3;
	4695	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1405;
	4695	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1545.4;
	4695	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6433.5;
	4695	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20157.2;
	4695	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11234.9;
	4695	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17539.6;
	4695	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2979.1;
	4696	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1276;
	4696	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1394.1;
	4696	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1568;
	4696	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6408.4;
	4696	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20375.5;
	4696	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11318.3;
	4696	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17515.2;
	4696	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2987.8;
	4697	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1269.6;
	4697	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1350.7;
	4697	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1562.2;
	4697	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6315;
	4697	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20352.2;
	4697	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11443.1;
	4697	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17290.6;
	4697	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2951.7;
	4698	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1266.3;
	4698	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1315.3;
	4698	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1537.9;
	4698	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6157.8;
	4698	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19991.7;
	4698	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11417.1;
	4698	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16736;
	4698	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2917.1;
	4699	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1229.1;
	4699	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1264.5;
	4699	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1478.6;
	4699	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5892.5;
	4699	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19254.6;
	4699	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10996.1;
	4699	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16118;
	4699	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2803.1;
	4700	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1162.3;
	4700	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1199.9;
	4700	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1400.3;
	4700	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5698.5;
	4700	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18370.9;
	4700	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10515.2;
	4700	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15549.9;
	4700	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2714;
	4701	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1118.6;
	4701	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1161.6;
	4701	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1349.1;
	4701	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5590.8;
	4701	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17787.9;
	4701	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10156.2;
	4701	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15053.2;
	4701	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2624.1;
	4702	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1082;
	4702	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1092;
	4702	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1261.5;
	4702	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5264.4;
	4702	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16416.2;
	4702	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9355.3;
	4702	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14058.9;
	4702	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2432.8;
	4703	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1031.4;
	4703	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1008.1;
	4703	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1167.4;
	4703	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4889.6;
	4703	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14928.6;
	4703	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8457.5;
	4703	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13067.9;
	4703	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2234;
	4704	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.7;
	4704	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	919.3;
	4704	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1110.8;
	4704	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4587.1;
	4704	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13538.9;
	4704	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7706.1;
	4704	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12283.4;
	4704	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2055.6;
	4705	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969.9;
	4705	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	870.4;
	4705	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1051;
	4705	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4351.4;
	4705	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12673.5;
	4705	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7160;
	4705	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11692.4;
	4705	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1930.8;
	4706	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.1;
	4706	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	839;
	4706	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1007.8;
	4706	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4174.9;
	4706	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12059.9;
	4706	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6756;
	4706	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11224.5;
	4706	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1838.9;
	4707	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.9;
	4707	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	823.1;
	4707	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	981.3;
	4707	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4053.4;
	4707	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11657.1;
	4707	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6496.9;
	4707	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10964.9;
	4707	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1811.2;
	4708	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.2;
	4708	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	809.8;
	4708	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	960.4;
	4708	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3996.5;
	4708	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11509.7;
	4708	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6422.1;
	4708	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10936.5;
	4708	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1780;
	4709	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943;
	4709	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	818.5;
	4709	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	966.5;
	4709	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4035.6;
	4709	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11753.4;
	4709	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6583.5;
	4709	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11170.1;
	4709	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1807.2;
	4710	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.1;
	4710	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	838.7;
	4710	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	993.6;
	4710	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4123.2;
	4710	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12135;
	4710	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6878.6;
	4710	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11420.6;
	4710	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1879.7;
	4711	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.4;
	4711	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	848.1;
	4711	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1021.3;
	4711	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4143.7;
	4711	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12364.2;
	4711	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7019.1;
	4711	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11684.3;
	4711	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1947.5;
	4712	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981.9;
	4712	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	893.2;
	4712	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1078.8;
	4712	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4386.3;
	4712	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12871.1;
	4712	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7385.3;
	4712	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12435.3;
	4712	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2087.4;
	4713	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.9;
	4713	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	933;
	4713	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1141.1;
	4713	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4744.3;
	4713	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13202.9;
	4713	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7947;
	4713	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13333.4;
	4713	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2122.2;
	4714	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1042;
	4714	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	978.5;
	4714	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1205.1;
	4714	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5079.7;
	4714	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13404.9;
	4714	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8590.5;
	4714	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14262.6;
	4714	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2285.8;
	4715	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1086.7;
	4715	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1017.2;
	4715	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1285.7;
	4715	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5406.7;
	4715	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13793.1;
	4715	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9194.3;
	4715	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15173.3;
	4715	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2448;
	4716	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1130;
	4716	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1074;
	4716	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1362.6;
	4716	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5732.9;
	4716	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14577.3;
	4716	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9766.4;
	4716	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16000.8;
	4716	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2563.9;
	4717	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1168.2;
	4717	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1136;
	4717	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1435.5;
	4717	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6023.4;
	4717	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15691.3;
	4717	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10305.7;
	4717	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16734.9;
	4717	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2662.1;
	4718	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1205.7;
	4718	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1185;
	4718	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1499.8;
	4718	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6226.1;
	4718	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16723.4;
	4718	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10780.4;
	4718	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17276.9;
	4718	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2744;
	4719	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1226.8;
	4719	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1221.9;
	4719	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1541.2;
	4719	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6352.9;
	4719	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17626.6;
	4719	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11155.2;
	4719	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17580.6;
	4719	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2858.6;
	4720	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1238.6;
	4720	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1246.1;
	4720	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1557.4;
	4720	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6379.2;
	4720	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18280.6;
	4720	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11405.1;
	4720	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17678.1;
	4720	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2881.3;
	4721	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1233.8;
	4721	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1249.6;
	4721	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1546.4;
	4721	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6266.5;
	4721	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18509.8;
	4721	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11424.9;
	4721	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17496.1;
	4721	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2848.5;
	4722	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1220.9;
	4722	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1228.6;
	4722	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1511;
	4722	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6061.9;
	4722	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18208.5;
	4722	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11157.5;
	4722	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16840.9;
	4722	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2751.7;
	4723	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1201;
	4723	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1176.4;
	4723	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1458.3;
	4723	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5796.2;
	4723	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17468.1;
	4723	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10642.6;
	4723	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16077;
	4723	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2642.2;
	4724	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1161.8;
	4724	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1131.2;
	4724	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1391.7;
	4724	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5593.2;
	4724	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16626;
	4724	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10167.8;
	4724	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15539;
	4724	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2536.1;
	4725	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1139.2;
	4725	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1101.4;
	4725	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1343.1;
	4725	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5477.7;
	4725	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16069.5;
	4725	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9776.7;
	4725	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14995.3;
	4725	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2420.6;
	4726	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1103.5;
	4726	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1043.1;
	4726	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1258.4;
	4726	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5190.6;
	4726	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14923.4;
	4726	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9090.1;
	4726	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14136.8;
	4726	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2225.3;
	4727	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1055.2;
	4727	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	970.9;
	4727	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1171.5;
	4727	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4857.6;
	4727	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13693.5;
	4727	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8329.8;
	4727	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13201.8;
	4727	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2079.2;
	4728	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1017.3;
	4728	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	895.1;
	4728	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1095.9;
	4728	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4560.2;
	4728	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12530.8;
	4728	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7595.6;
	4728	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12373.7;
	4728	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1957.7;
	4729	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	988.7;
	4729	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	842.5;
	4729	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1031.2;
	4729	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4289.2;
	4729	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11693.8;
	4729	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7002.2;
	4729	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11709;
	4729	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1814.8;
	4730	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	971.7;
	4730	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	806.3;
	4730	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	983.8;
	4730	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4059.8;
	4730	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11133.6;
	4730	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6572.9;
	4730	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11219.5;
	4730	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1708.2;
	4731	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.4;
	4731	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	788.8;
	4731	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	939.2;
	4731	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3895.3;
	4731	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10724.9;
	4731	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6296.1;
	4731	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10815.5;
	4731	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1700.1;
	4732	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.6;
	4732	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	766.1;
	4732	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	907.4;
	4732	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3782;
	4732	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10460.1;
	4732	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6077.7;
	4732	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10545;
	4732	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1679.7;
	4733	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.6;
	4733	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	763.5;
	4733	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	893.3;
	4733	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3729;
	4733	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10360.8;
	4733	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6012.5;
	4733	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10384.5;
	4733	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1673.7;
	4734	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.9;
	4734	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	771.7;
	4734	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	890.3;
	4734	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3703.9;
	4734	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10344.8;
	4734	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6061.5;
	4734	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10318.3;
	4734	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1667.7;
	4735	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.2;
	4735	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	773;
	4735	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	891.6;
	4735	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3686.2;
	4735	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10480.4;
	4735	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6124;
	4735	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10475.3;
	4735	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1675.6;
	4736	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.5;
	4736	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	829.1;
	4736	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	933.1;
	4736	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3989.8;
	4736	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11232.9;
	4736	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6488.1;
	4736	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11347.9;
	4736	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1793.1;
	4737	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	973;
	4737	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	890.7;
	4737	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	991.3;
	4737	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4384.1;
	4737	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12349.1;
	4737	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6990;
	4737	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12455.6;
	4737	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1980.9;
	4738	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1019.4;
	4738	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	961.1;
	4738	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1063.7;
	4738	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4831.9;
	4738	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13696.4;
	4738	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7625;
	4738	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13572.9;
	4738	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2170.2;
	4739	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1069.2;
	4739	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1031.5;
	4739	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1139.3;
	4739	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5245.3;
	4739	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15016.8;
	4739	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8315.8;
	4739	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14572.4;
	4739	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2336.3;
	4740	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1111.9;
	4740	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1095.4;
	4740	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1213.3;
	4740	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5580.1;
	4740	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16169.2;
	4740	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8994.4;
	4740	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15378.1;
	4740	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2469;
	4741	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1148.6;
	4741	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1146.9;
	4741	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1280.9;
	4741	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5841.8;
	4741	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17082.2;
	4741	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9580;
	4741	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15953.5;
	4741	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2612.9;
	4742	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1177;
	4742	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1189.5;
	4742	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1341.5;
	4742	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6024.7;
	4742	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17869.6;
	4742	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10056.2;
	4742	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16395.2;
	4742	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2690.2;
	4743	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1196.8;
	4743	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1227.5;
	4743	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1397.8;
	4743	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6124.9;
	4743	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18432.7;
	4743	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10391.2;
	4743	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16687.7;
	4743	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2760.3;
	4744	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1213.2;
	4744	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1259.7;
	4744	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1436.9;
	4744	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6166.9;
	4744	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18789.8;
	4744	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10648.8;
	4744	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16811.1;
	4744	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2773.9;
	4745	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1215;
	4745	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1271.5;
	4745	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1451.3;
	4745	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6094.2;
	4745	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18758.2;
	4745	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10713.6;
	4745	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16602.1;
	4745	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2782.5;
	4746	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1203.8;
	4746	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1254.7;
	4746	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1434.2;
	4746	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5921.8;
	4746	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18325.2;
	4746	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10540;
	4746	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16129.7;
	4746	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2698.4;
	4747	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1181.2;
	4747	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1215.2;
	4747	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1389.9;
	4747	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5669.1;
	4747	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17544.9;
	4747	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10187.9;
	4747	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15423.1;
	4747	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2611.4;
	4748	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1144.7;
	4748	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1159.5;
	4748	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1326.8;
	4748	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5464.2;
	4748	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16650.9;
	4748	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9734.1;
	4748	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14856.7;
	4748	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2517.5;
	4749	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1125.6;
	4749	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1130.3;
	4749	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1287.7;
	4749	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5372.2;
	4749	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16132.9;
	4749	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9430;
	4749	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14407.6;
	4749	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2422;
	4750	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1091.1;
	4750	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1072.4;
	4750	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1219.5;
	4750	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5097.1;
	4750	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15077.4;
	4750	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8763.9;
	4750	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13640.1;
	4750	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2271;
	4751	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1052.7;
	4751	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1003.8;
	4751	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1140.4;
	4751	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4781.4;
	4751	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13891.5;
	4751	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8043.4;
	4751	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12771.9;
	4751	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2101.4;
	4752	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1015.6;
	4752	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	935.9;
	4752	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1069.9;
	4752	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4456.9;
	4752	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12884;
	4752	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7351.6;
	4752	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11965.7;
	4752	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1930.5;
	4753	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989.9;
	4753	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	887.4;
	4753	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1017.2;
	4753	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4191.8;
	4753	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12042.4;
	4753	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6782.9;
	4753	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11329.9;
	4753	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1836.1;
	4754	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.3;
	4754	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	850.5;
	4754	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	974.2;
	4754	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3970.1;
	4754	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11374.7;
	4754	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6377.3;
	4754	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10852.2;
	4754	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1763.5;
	4755	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948.4;
	4755	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	830.7;
	4755	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	939.4;
	4755	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3806.6;
	4755	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10929.6;
	4755	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6086.1;
	4755	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10505.6;
	4755	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1698;
	4756	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.2;
	4756	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	809.9;
	4756	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	914.2;
	4756	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3690.3;
	4756	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10603.7;
	4756	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5900.4;
	4756	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10282.4;
	4756	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1634.4;
	4757	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.4;
	4757	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	801.4;
	4757	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	904.5;
	4757	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3633.7;
	4757	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10449.9;
	4757	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5812.5;
	4757	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10190.3;
	4757	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1633.1;
	4758	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.7;
	4758	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	793.7;
	4758	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897.8;
	4758	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3587.1;
	4758	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10287.5;
	4758	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5821.1;
	4758	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10036;
	4758	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1614.1;
	4759	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.5;
	4759	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	816.1;
	4759	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	907.5;
	4759	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3578.8;
	4759	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10515.2;
	4759	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5858.8;
	4759	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10214.2;
	4759	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1625.4;
	4760	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954;
	4760	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	883.5;
	4760	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	970.3;
	4760	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3919.9;
	4760	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11508.9;
	4760	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6340.7;
	4760	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11147;
	4760	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1795.7;
	4761	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1001;
	4761	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	956.4;
	4761	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1037.4;
	4761	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4346.2;
	4761	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12743.8;
	4761	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7005.5;
	4761	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12198.9;
	4761	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1925.6;
	4762	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1032.7;
	4762	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1020.9;
	4762	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1103.7;
	4762	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4758.6;
	4762	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13900;
	4762	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7719.2;
	4762	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13139.6;
	4762	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2123.1;
	4763	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1067.6;
	4763	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1082.8;
	4763	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1175.8;
	4763	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5135.1;
	4763	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15023.6;
	4763	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8388.3;
	4763	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13920;
	4763	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2345.5;
	4764	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1103.9;
	4764	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1142.6;
	4764	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1249.5;
	4764	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5458.4;
	4764	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16098.7;
	4764	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9014.9;
	4764	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14696.2;
	4764	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2507.7;
	4765	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1135.5;
	4765	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1202.4;
	4765	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1318.7;
	4765	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5699.2;
	4765	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16983.5;
	4765	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9540.6;
	4765	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15397.3;
	4765	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2618.5;
	4766	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1166.1;
	4766	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1240.3;
	4766	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1377.6;
	4766	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5862.8;
	4766	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17706.3;
	4766	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10041.6;
	4766	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15709;
	4766	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2709.5;
	4767	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1188.6;
	4767	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1269.1;
	4767	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1419;
	4767	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5969.1;
	4767	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18230.1;
	4767	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10356.1;
	4767	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15519.5;
	4767	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2778.2;
	4768	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1200.7;
	4768	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1287.1;
	4768	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1452.3;
	4768	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6009.9;
	4768	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18544.5;
	4768	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10582.7;
	4768	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14933.4;
	4768	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2822.4;
	4769	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1207.3;
	4769	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1290.5;
	4769	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1479.8;
	4769	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5978.3;
	4769	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18642;
	4769	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10700.9;
	4769	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14550.3;
	4769	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2807;
	4770	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1198.2;
	4770	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1272.9;
	4770	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1470.9;
	4770	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5856.4;
	4770	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18384.2;
	4770	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10582.5;
	4770	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14461.4;
	4770	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2748.4;
	4771	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1177.3;
	4771	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1227;
	4771	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1404.7;
	4771	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5654.6;
	4771	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17754.2;
	4771	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10288.4;
	4771	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14222.3;
	4771	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2655.3;
	4772	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1146;
	4772	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1170.5;
	4772	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1326.7;
	4772	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5480.4;
	4772	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16954;
	4772	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9919.4;
	4772	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13942.9;
	4772	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2558.4;
	4773	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1131.8;
	4773	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1143.9;
	4773	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1292.1;
	4773	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5425.9;
	4773	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16456.8;
	4773	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9639.7;
	4773	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13741.2;
	4773	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2408;
	4774	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1100.1;
	4774	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1079.9;
	4774	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1221.2;
	4774	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5157.5;
	4774	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15341.9;
	4774	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8971.7;
	4774	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13023.2;
	4774	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2248.3;
	4775	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1050.7;
	4775	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	997.7;
	4775	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1129.5;
	4775	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4789;
	4775	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13987.7;
	4775	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8158.8;
	4775	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12124.2;
	4775	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2155;
	4776	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1011.7;
	4776	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	915.1;
	4776	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1059.9;
	4776	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4446.7;
	4776	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12949;
	4776	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7426.2;
	4776	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11339.7;
	4776	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1997.9;
	4777	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	987.2;
	4777	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	872.8;
	4777	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1009.4;
	4777	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4189.4;
	4777	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12191.8;
	4777	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6882.8;
	4777	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10837.9;
	4777	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1882.5;
	4778	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.2;
	4778	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	833.3;
	4778	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	970.4;
	4778	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3992.9;
	4778	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11599.4;
	4778	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6510.3;
	4778	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10489.7;
	4778	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1799.8;
	4779	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.7;
	4779	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	809.2;
	4779	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940.6;
	4779	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3873.5;
	4779	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11206.5;
	4779	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6269.9;
	4779	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10344.6;
	4779	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1757;
	4780	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940;
	4780	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	802.6;
	4780	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	924;
	4780	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3826.2;
	4780	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11129.9;
	4780	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6193;
	4780	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10422.7;
	4780	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1746.3;
	4781	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.3;
	4781	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	816.5;
	4781	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	926.9;
	4781	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3865.7;
	4781	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11432.5;
	4781	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6382.1;
	4781	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10744.9;
	4781	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1780.1;
	4782	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.5;
	4782	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	840.2;
	4782	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	954.6;
	4782	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3972.3;
	4782	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11873.7;
	4782	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6696.2;
	4782	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11089;
	4782	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1832.9;
	4783	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.4;
	4783	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	875.1;
	4783	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	983.6;
	4783	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4028.9;
	4783	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12289.2;
	4783	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6881.1;
	4783	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11386.3;
	4783	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1928.5;
	4784	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.7;
	4784	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	948.1;
	4784	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1048.5;
	4784	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4359.4;
	4784	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13221.5;
	4784	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7258.2;
	4784	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12232.8;
	4784	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2109.7;
	4785	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1034.9;
	4785	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1023.9;
	4785	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1120.8;
	4785	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4753.5;
	4785	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14372;
	4785	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7819;
	4785	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13267.7;
	4785	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2302.8;
	4786	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1078.9;
	4786	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1106.6;
	4786	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1195.9;
	4786	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5161.2;
	4786	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15584.9;
	4786	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8475.3;
	4786	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14302.9;
	4786	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2475.7;
	4787	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1123.4;
	4787	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1182.6;
	4787	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1279.3;
	4787	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5578.9;
	4787	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16780.1;
	4787	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9130.1;
	4787	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15284.9;
	4787	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2636;
	4788	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1159.9;
	4788	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1249.7;
	4788	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1352.8;
	4788	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5901.4;
	4788	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17786;
	4788	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9719.3;
	4788	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15909.2;
	4788	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2771.8;
	4789	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1194.9;
	4789	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1306.4;
	4789	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1420.8;
	4789	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6173.1;
	4789	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18716.8;
	4789	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10259.6;
	4789	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16371;
	4789	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2799.1;
	4790	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1220.8;
	4790	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1339.4;
	4790	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1472.9;
	4790	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6349;
	4790	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19362.4;
	4790	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10698.1;
	4790	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15975.8;
	4790	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2898.2;
	4791	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1235.8;
	4791	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1366.6;
	4791	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1511.9;
	4791	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6444.1;
	4791	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19751.8;
	4791	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11040.7;
	4791	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15651.9;
	4791	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2939.6;
	4792	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1248;
	4792	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1377.7;
	4792	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1531.5;
	4792	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6461.1;
	4792	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19950.2;
	4792	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11263.1;
	4792	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15542.7;
	4792	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2909.2;
	4793	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1238.9;
	4793	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1367.2;
	4793	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1519.3;
	4793	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6339.6;
	4793	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19884.9;
	4793	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11307.7;
	4793	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15267.8;
	4793	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2885.7;
	4794	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1224.7;
	4794	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1338;
	4794	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1493.5;
	4794	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6145.1;
	4794	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19555.6;
	4794	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11159.4;
	4794	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14940.7;
	4794	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2853.6;
	4795	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1202.7;
	4795	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1292.2;
	4795	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1443;
	4795	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5903.4;
	4795	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18852.3;
	4795	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10745.3;
	4795	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14469.7;
	4795	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2759.7;
	4796	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1170.2;
	4796	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1236.7;
	4796	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1372.1;
	4796	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5718;
	4796	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17978.5;
	4796	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10266.9;
	4796	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14295;
	4796	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2663.2;
	4797	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1140.6;
	4797	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1188.3;
	4797	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1324.2;
	4797	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5586.1;
	4797	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17305.1;
	4797	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9957.5;
	4797	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13982.1;
	4797	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2582.1;
	4798	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1103.3;
	4798	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1122.8;
	4798	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1244;
	4798	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5264.1;
	4798	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15928.6;
	4798	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9202.3;
	4798	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13206.2;
	4798	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2401;
	4799	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1050;
	4799	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1029.6;
	4799	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1143.7;
	4799	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4853.1;
	4799	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14472.2;
	4799	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8351;
	4799	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12280.2;
	4799	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2181.6;
	4800	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1012.8;
	4800	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	951.7;
	4800	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1067.5;
	4800	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4484.1;
	4800	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13268.6;
	4800	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7635.7;
	4800	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11505.6;
	4800	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2042.9;
	4801	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	988.5;
	4801	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	902.9;
	4801	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1009.5;
	4801	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4215.1;
	4801	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12465;
	4801	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7094.8;
	4801	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10950.9;
	4801	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1924.3;
	4802	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.8;
	4802	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	865.5;
	4802	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	973.3;
	4802	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3994.2;
	4802	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11876.2;
	4802	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6695.1;
	4802	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10565.3;
	4802	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1821.8;
	4803	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.5;
	4803	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	847.2;
	4803	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	945.8;
	4803	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3844.3;
	4803	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11486.3;
	4803	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6448.6;
	4803	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10370.7;
	4803	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1776.6;
	4804	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.6;
	4804	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	832;
	4804	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	932.8;
	4804	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3780.1;
	4804	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11384.4;
	4804	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6355.7;
	4804	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10390.1;
	4804	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1746.3;
	4805	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.7;
	4805	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	842.3;
	4805	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	941.4;
	4805	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3813.2;
	4805	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11656;
	4805	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6478.4;
	4805	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10745;
	4805	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1764.7;
	4806	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.9;
	4806	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	866.3;
	4806	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	964.6;
	4806	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3929;
	4806	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12113.3;
	4806	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6764.1;
	4806	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11087.7;
	4806	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1868.3;
	4807	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	968.6;
	4807	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	891.8;
	4807	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	982.4;
	4807	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3981.6;
	4807	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12533.4;
	4807	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6914;
	4807	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11452.3;
	4807	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1901.2;
	4808	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1005.3;
	4808	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	959.5;
	4808	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1040.8;
	4808	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4276.4;
	4808	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13446.5;
	4808	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7347.7;
	4808	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12302.1;
	4808	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2113.6;
	4809	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1047;
	4809	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1043.5;
	4809	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1115.2;
	4809	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4629.2;
	4809	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14697.3;
	4809	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7876.7;
	4809	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13357.1;
	4809	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2229.6;
	4810	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1091.1;
	4810	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1128.4;
	4810	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1197.1;
	4810	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5006.3;
	4810	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16029.7;
	4810	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8556.3;
	4810	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14237.5;
	4810	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2395.6;
	4811	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1131.8;
	4811	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1205.5;
	4811	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1272.5;
	4811	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5371.8;
	4811	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17259.1;
	4811	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9205.5;
	4811	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14572.8;
	4811	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2617.2;
	4812	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1159.1;
	4812	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1263.9;
	4812	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1345.2;
	4812	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5657.7;
	4812	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18307.6;
	4812	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9769;
	4812	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14251.7;
	4812	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2749.9;
	4813	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1188.8;
	4813	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1309;
	4813	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1412.8;
	4813	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5887.3;
	4813	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19157.5;
	4813	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10320.1;
	4813	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14236.1;
	4813	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2844.1;
	4814	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1214.8;
	4814	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1348.4;
	4814	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1463.8;
	4814	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6025.6;
	4814	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19786.5;
	4814	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10715.8;
	4814	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14702.8;
	4814	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2977.2;
	4815	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1230.6;
	4815	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1371.1;
	4815	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1487.2;
	4815	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6113;
	4815	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20123;
	4815	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11035.2;
	4815	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15341.3;
	4815	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3033;
	4816	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1241.3;
	4816	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1377.7;
	4816	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1501.1;
	4816	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6203.4;
	4816	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20341;
	4816	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11214.5;
	4816	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15566.8;
	4816	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3027.5;
	4817	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1226.5;
	4817	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1364;
	4817	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1484.5;
	4817	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6108.2;
	4817	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20304.2;
	4817	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11082.2;
	4817	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15365.4;
	4817	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3004.9;
	4818	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1206.9;
	4818	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1338.8;
	4818	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1459.7;
	4818	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5924.9;
	4818	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19905.3;
	4818	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10645.5;
	4818	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14883.6;
	4818	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2960.7;
	4819	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1187.3;
	4819	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1298.9;
	4819	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1415.3;
	4819	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5690.6;
	4819	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19157.7;
	4819	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10168.4;
	4819	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14410.4;
	4819	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2848.7;
	4820	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1150.3;
	4820	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1244;
	4820	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1355.6;
	4820	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5525.4;
	4820	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18258.7;
	4820	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9770;
	4820	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14144.2;
	4820	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2740.8;
	4821	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1134.8;
	4821	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1207;
	4821	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1319;
	4821	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5413.8;
	4821	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17591.2;
	4821	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9503.7;
	4821	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13800;
	4821	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2642.4;
	4822	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1095.3;
	4822	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1131.3;
	4822	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1235.8;
	4822	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5095.2;
	4822	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16262.5;
	4822	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8817.9;
	4822	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13030.8;
	4822	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2449.9;
	4823	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1051.3;
	4823	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1048.6;
	4823	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1140.7;
	4823	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4719.6;
	4823	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14849.9;
	4823	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8006.8;
	4823	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12184.8;
	4823	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2233;
	4824	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1015.9;
	4824	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	977.3;
	4824	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1065.1;
	4824	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4395.6;
	4824	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13618.8;
	4824	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7351.6;
	4824	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11432.5;
	4824	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2072.7;
	4825	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	991.7;
	4825	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	938.2;
	4825	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1016.6;
	4825	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4136.4;
	4825	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12835.5;
	4825	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6844.1;
	4825	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10895.6;
	4825	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1957.4;
	4826	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.5;
	4826	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	907.5;
	4826	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	977;
	4826	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3930.4;
	4826	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12261.2;
	4826	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6462.1;
	4826	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10493.9;
	4826	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1876;
	4827	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.9;
	4827	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	889.7;
	4827	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	948.7;
	4827	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3777.4;
	4827	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11864.6;
	4827	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6206.4;
	4827	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10277.4;
	4827	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1821.2;
	4828	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961;
	4828	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	876.4;
	4828	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	931.8;
	4828	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3703.3;
	4828	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11744.6;
	4828	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6117.5;
	4828	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10289.9;
	4828	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1734.4;
	4829	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967;
	4829	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	880.8;
	4829	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	941.1;
	4829	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3752.1;
	4829	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11974.5;
	4829	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6275.5;
	4829	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10686.3;
	4829	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1732.6;
	4830	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.6;
	4830	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	901.3;
	4830	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	967.2;
	4830	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3858.8;
	4830	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12427.6;
	4830	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6613.8;
	4830	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11023.2;
	4830	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1797.4;
	4831	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.1;
	4831	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	920.7;
	4831	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	994.3;
	4831	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3917.9;
	4831	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12799.8;
	4831	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6754;
	4831	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11353.5;
	4831	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1868.6;
	4832	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1012.6;
	4832	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	984.4;
	4832	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1052;
	4832	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4245.3;
	4832	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13697.7;
	4832	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7191.3;
	4832	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12157.1;
	4832	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2016;
	4833	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1056.2;
	4833	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1063.2;
	4833	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1126.6;
	4833	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4672.6;
	4833	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14852.5;
	4833	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7767.6;
	4833	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13219.7;
	4833	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2201.7;
	4834	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1094;
	4834	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1144.2;
	4834	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1198.5;
	4834	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5073.8;
	4834	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16183;
	4834	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8434.3;
	4834	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14215;
	4834	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2388.8;
	4835	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1137;
	4835	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1220.1;
	4835	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1283.4;
	4835	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5445.2;
	4835	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17348;
	4835	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9065.8;
	4835	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14916.1;
	4835	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2563.2;
	4836	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1164.6;
	4836	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1288.3;
	4836	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1353;
	4836	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5757.4;
	4836	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18413.4;
	4836	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9646.9;
	4836	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15543;
	4836	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2717.6;
	4837	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1199;
	4837	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1341.7;
	4837	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1414.9;
	4837	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6016.7;
	4837	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19322.6;
	4837	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10194;
	4837	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16271;
	4837	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2846.4;
	4838	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1226.7;
	4838	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1381.5;
	4838	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1465.7;
	4838	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6206.1;
	4838	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19952.4;
	4838	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10650.9;
	4838	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16678.8;
	4838	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2947.3;
	4839	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1241.5;
	4839	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1401.7;
	4839	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1511.8;
	4839	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6320.3;
	4839	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20321.5;
	4839	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11008.9;
	4839	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17017.6;
	4839	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2986.9;
	4840	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1249.3;
	4840	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1407;
	4840	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1538.8;
	4840	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6333.5;
	4840	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20546.7;
	4840	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11253.5;
	4840	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17182.9;
	4840	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2994.2;
	4841	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1244.5;
	4841	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1393;
	4841	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1533.3;
	4841	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6247.8;
	4841	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20385.6;
	4841	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11307.8;
	4841	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17064.9;
	4841	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2962.5;
	4842	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1230.2;
	4842	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1369.4;
	4842	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1507;
	4842	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6078.4;
	4842	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20010.6;
	4842	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11117.3;
	4842	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16530.8;
	4842	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2913.6;
	4843	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1204.3;
	4843	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1328.8;
	4843	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1459.6;
	4843	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5825.7;
	4843	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19272.4;
	4843	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10650.1;
	4843	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15894.8;
	4843	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2805.4;
	4844	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1151.5;
	4844	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1266.1;
	4844	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1392.7;
	4844	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5634;
	4844	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18345.5;
	4844	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10209.6;
	4844	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15368.1;
	4844	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2691.7;
	4845	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1132;
	4845	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1216.6;
	4845	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1347.9;
	4845	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5515.4;
	4845	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17677;
	4845	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9832.9;
	4845	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14833.3;
	4845	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2582.1;
	4846	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1100.8;
	4846	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1134.9;
	4846	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1256.5;
	4846	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5194.5;
	4846	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16283.2;
	4846	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9107.6;
	4846	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13917.2;
	4846	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2386.1;
	4847	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1048.6;
	4847	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1046.5;
	4847	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1151.4;
	4847	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4797.4;
	4847	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14833.3;
	4847	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8255.9;
	4847	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12896.3;
	4847	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2184;
	4848	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009.4;
	4848	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	976.3;
	4848	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1066.9;
	4848	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4412.9;
	4848	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13635.3;
	4848	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7442;
	4848	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12031.6;
	4848	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1998.1;
	4849	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981;
	4849	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	927.2;
	4849	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1006;
	4849	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4130.7;
	4849	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12740;
	4849	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6887.3;
	4849	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11320.7;
	4849	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1874.1;
	4850	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.7;
	4850	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	895.2;
	4850	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	962.1;
	4850	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3916.9;
	4850	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12125.5;
	4850	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6489;
	4850	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10859.2;
	4850	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1790.1;
	4851	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.5;
	4851	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	868.9;
	4851	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	929.4;
	4851	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3764.5;
	4851	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11743.3;
	4851	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6220.7;
	4851	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10589.9;
	4851	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1722.2;
	4852	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.7;
	4852	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	859.2;
	4852	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	909.7;
	4852	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3677.7;
	4852	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11654.7;
	4852	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6138.9;
	4852	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10556;
	4852	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1689.3;
	4853	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.8;
	4853	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	870.3;
	4853	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	917.4;
	4853	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3727.1;
	4853	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11936.6;
	4853	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6283.3;
	4853	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10883.4;
	4853	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1728;
	4854	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.1;
	4854	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	887.5;
	4854	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	949.4;
	4854	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3842.9;
	4854	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12368.1;
	4854	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6598.6;
	4854	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11201.6;
	4854	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1801.1;
	4855	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.2;
	4855	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	915;
	4855	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	978.8;
	4855	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3902.5;
	4855	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12740.1;
	4855	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6763.6;
	4855	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11514.9;
	4855	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1873.8;
	4856	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	994.1;
	4856	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	984.3;
	4856	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1043.1;
	4856	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4230.6;
	4856	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13661.7;
	4856	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7191.3;
	4856	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12363.8;
	4856	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2048.4;
	4857	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1042.2;
	4857	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1056.5;
	4857	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1119;
	4857	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4629.6;
	4857	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14835.2;
	4857	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7802.4;
	4857	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13421.9;
	4857	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2281.7;
	4858	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1087.7;
	4858	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1144;
	4858	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1201.6;
	4858	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5057.3;
	4858	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16191.2;
	4858	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8482.5;
	4858	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14519.2;
	4858	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2446.8;
	4859	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1128.5;
	4859	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1225.5;
	4859	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1282.8;
	4859	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5439.1;
	4859	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17433.5;
	4859	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9136.7;
	4859	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15495.9;
	4859	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2667.6;
	4860	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1163.7;
	4860	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1292.7;
	4860	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1357.8;
	4860	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5758.6;
	4860	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18581.4;
	4860	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9732.9;
	4860	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16348.1;
	4860	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2815;
	4861	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1194;
	4861	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1349.3;
	4861	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1430.3;
	4861	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5963.6;
	4861	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19532;
	4861	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10311.4;
	4861	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16983.5;
	4861	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2971.5;
	4862	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1219.9;
	4862	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1391.9;
	4862	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1485.3;
	4862	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6084.7;
	4862	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20248.6;
	4862	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10775.4;
	4862	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17424.2;
	4862	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3012.8;
	4863	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1234.2;
	4863	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1412;
	4863	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1521;
	4863	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6153.1;
	4863	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20561.2;
	4863	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11066.9;
	4863	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17538.4;
	4863	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3038.9;
	4864	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1237.4;
	4864	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1417.8;
	4864	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1539;
	4864	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6118.2;
	4864	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20670.3;
	4864	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11140.3;
	4864	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17451.5;
	4864	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3045.9;
	4865	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1232.6;
	4865	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1409.5;
	4865	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1531.2;
	4865	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5981.3;
	4865	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20635.4;
	4865	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11214.9;
	4865	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17299.3;
	4865	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3007.7;
	4866	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1229;
	4866	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1381.6;
	4866	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1508.1;
	4866	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5842.5;
	4866	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20257;
	4866	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11154.5;
	4866	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16795;
	4866	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2977.8;
	4867	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1212.2;
	4867	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1329.7;
	4867	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1456.1;
	4867	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5653.2;
	4867	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19566.8;
	4867	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10736.8;
	4867	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16262.1;
	4867	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2879.4;
	4868	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1176.4;
	4868	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1268.6;
	4868	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1382.9;
	4868	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5499.5;
	4868	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18676.2;
	4868	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10284.5;
	4868	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15725.7;
	4868	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2781.6;
	4869	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1153.3;
	4869	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1224;
	4869	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1334.6;
	4869	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5376.9;
	4869	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17949.7;
	4869	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9873.7;
	4869	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15151.7;
	4869	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2665;
	4870	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1116.4;
	4870	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1146.5;
	4870	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1248.6;
	4870	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5072.8;
	4870	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16596.9;
	4870	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9179.6;
	4870	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14212;
	4870	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2461.4;
	4871	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1062.2;
	4871	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1073.1;
	4871	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1152.4;
	4871	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4715.2;
	4871	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15114.8;
	4871	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8360.3;
	4871	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13200.7;
	4871	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2274.9;
	4872	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1011.1;
	4872	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	988.1;
	4872	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1074.8;
	4872	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4385.1;
	4872	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13733.9;
	4872	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7644.3;
	4872	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12271;
	4872	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2120.5;
	4873	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	976.5;
	4873	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	931.8;
	4873	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1013;
	4873	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4115.6;
	4873	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12791.6;
	4873	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7078.8;
	4873	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11539.1;
	4873	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1987.1;
	4874	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.5;
	4874	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	889.6;
	4874	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	969.1;
	4874	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3908.8;
	4874	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12115.8;
	4874	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6641.6;
	4874	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10991.9;
	4874	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1897.5;
	4875	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.7;
	4875	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	866.7;
	4875	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	934.4;
	4875	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3757.7;
	4875	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11629.9;
	4875	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6345.1;
	4875	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10655.5;
	4875	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1858.2;
	4876	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.6;
	4876	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	842.9;
	4876	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	917.3;
	4876	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3692.8;
	4876	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11484.3;
	4876	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6218.9;
	4876	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10621;
	4876	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1817.3;
	4877	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.8;
	4877	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	849.8;
	4877	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	924.6;
	4877	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3716.2;
	4877	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11736.5;
	4877	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6308.6;
	4877	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10854.7;
	4877	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1839.6;
	4878	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.7;
	4878	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	861.9;
	4878	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	948.6;
	4878	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3809.2;
	4878	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12083.9;
	4878	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6598.3;
	4878	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11090;
	4878	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1898.8;
	4879	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.3;
	4879	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	889.2;
	4879	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	977.1;
	4879	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3874.3;
	4879	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12376.1;
	4879	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6774;
	4879	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11411;
	4879	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1973.5;
	4880	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	979.7;
	4880	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	959.6;
	4880	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1046.9;
	4880	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4183;
	4880	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13314.1;
	4880	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7301;
	4880	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12315.9;
	4880	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2145.8;
	4881	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1033;
	4881	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1050.6;
	4881	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1132.8;
	4881	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4635.3;
	4881	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14620;
	4881	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7933;
	4881	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13426.6;
	4881	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2245.4;
	4882	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1079.4;
	4882	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1144.7;
	4882	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1219.9;
	4882	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5097.3;
	4882	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16107.7;
	4882	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8663.2;
	4882	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14565;
	4882	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2469.1;
	4883	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1125.6;
	4883	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1225.6;
	4883	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1309;
	4883	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5493.9;
	4883	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17421.6;
	4883	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9371.3;
	4883	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15549.6;
	4883	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2693.4;
	4884	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1158.9;
	4884	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1294.2;
	4884	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1380.1;
	4884	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5789.3;
	4884	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18553;
	4884	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9974;
	4884	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16278.4;
	4884	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2830;
	4885	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1192.3;
	4885	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1355.8;
	4885	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1447.8;
	4885	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6016.7;
	4885	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19524.6;
	4885	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10528;
	4885	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17017.8;
	4885	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2935.4;
	4886	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1217.3;
	4886	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1392.6;
	4886	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1505;
	4886	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6215.8;
	4886	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20161.9;
	4886	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11003.9;
	4886	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17415.4;
	4886	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3000.6;
	4887	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1222.1;
	4887	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1410.1;
	4887	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1541.8;
	4887	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6260.8;
	4887	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20576.7;
	4887	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11272;
	4887	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17409.5;
	4887	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3069.2;
	4888	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1231.3;
	4888	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1410;
	4888	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1556.7;
	4888	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6219.2;
	4888	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20720.3;
	4888	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11283;
	4888	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17288.3;
	4888	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3054.6;
	4889	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1234.4;
	4889	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1400.4;
	4889	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1551.2;
	4889	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6169.7;
	4889	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20740.9;
	4889	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11376.8;
	4889	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17140.7;
	4889	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3028.4;
	4890	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1228.5;
	4890	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1374.8;
	4890	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1526.8;
	4890	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6048.7;
	4890	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20331.6;
	4890	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11309.8;
	4890	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16662.6;
	4890	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2963.1;
	4891	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1209.3;
	4891	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1328;
	4891	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1482.6;
	4891	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5829.9;
	4891	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19488.1;
	4891	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10863.9;
	4891	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16131.6;
	4891	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2847.8;
	4892	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1170.3;
	4892	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1264.3;
	4892	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1411.3;
	4892	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5623.4;
	4892	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18504.5;
	4892	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10377;
	4892	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15696.3;
	4892	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2739.4;
	4893	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1142.9;
	4893	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1222;
	4893	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1364.7;
	4893	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5496.8;
	4893	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17781.3;
	4893	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10024.7;
	4893	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15240;
	4893	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2607.8;
	4894	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1111.7;
	4894	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1142;
	4894	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1276.1;
	4894	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5198.7;
	4894	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16479.3;
	4894	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9383;
	4894	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14380.3;
	4894	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2414.3;
	4895	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1066.5;
	4895	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1064.3;
	4895	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1185.8;
	4895	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4862.4;
	4895	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15180.4;
	4895	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8659.6;
	4895	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13500;
	4895	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2258.3;
	4896	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1022.1;
	4896	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1010.4;
	4896	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1112.8;
	4896	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4536.3;
	4896	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13972;
	4896	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7929.6;
	4896	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12665;
	4896	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2127.7;
	4897	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.9;
	4897	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	952.4;
	4897	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1052.9;
	4897	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4267.3;
	4897	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13062.3;
	4897	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7329.1;
	4897	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11972.5;
	4897	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1978.8;
	4898	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970.5;
	4898	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	908.7;
	4898	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1003.8;
	4898	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4039.9;
	4898	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12360.3;
	4898	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6867.1;
	4898	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11417.9;
	4898	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1860.5;
	4899	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.1;
	4899	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	884.3;
	4899	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	970.1;
	4899	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3891.4;
	4899	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11858.2;
	4899	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6560.7;
	4899	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11043.2;
	4899	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1855.7;
	4900	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.1;
	4900	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	855.3;
	4900	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940;
	4900	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3777.7;
	4900	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11495.3;
	4900	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6298.2;
	4900	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10788.2;
	4900	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1830.9;
	4901	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.7;
	4901	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	844.9;
	4901	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	931.8;
	4901	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3740.3;
	4901	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11438;
	4901	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6239.1;
	4901	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10734.4;
	4901	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1824.4;
	4902	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.6;
	4902	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	842.7;
	4902	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	933.5;
	4902	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3731.2;
	4902	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11426.5;
	4902	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6336.5;
	4902	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10724.1;
	4902	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1815.5;
	4903	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.4;
	4903	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	862.1;
	4903	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	943.2;
	4903	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3703;
	4903	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11669.2;
	4903	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6423.5;
	4903	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10933.1;
	4903	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1825.8;
	4904	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.8;
	4904	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	924.3;
	4904	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1010.3;
	4904	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4052.1;
	4904	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12750.8;
	4904	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6941.8;
	4904	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11948.1;
	4904	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1971.7;
	4905	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1012.9;
	4905	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1018.2;
	4905	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1097.8;
	4905	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4477;
	4905	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14182.4;
	4905	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7666.4;
	4905	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13138.7;
	4905	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2185.7;
	4906	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1062.3;
	4906	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1111.3;
	4906	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1195.4;
	4906	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4956.2;
	4906	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15720.2;
	4906	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8388.9;
	4906	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14276.2;
	4906	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2387.2;
	4907	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1112.2;
	4907	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1192.7;
	4907	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1286.6;
	4907	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5386.6;
	4907	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17099.8;
	4907	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9076.2;
	4907	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15285.4;
	4907	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2571.2;
	4908	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1148.9;
	4908	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1260.6;
	4908	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1360.8;
	4908	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5742.5;
	4908	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18222;
	4908	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9671.3;
	4908	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16042;
	4908	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2709;
	4909	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1175.7;
	4909	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1306.5;
	4909	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1421.4;
	4909	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6006.9;
	4909	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19036.9;
	4909	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10169;
	4909	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16600.3;
	4909	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2837.6;
	4910	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1196;
	4910	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1338.8;
	4910	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1466.7;
	4910	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6180.7;
	4910	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19655.9;
	4910	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10596.6;
	4910	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16983;
	4910	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2909.9;
	4911	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1214.9;
	4911	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1368.7;
	4911	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1501.3;
	4911	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6268.6;
	4911	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20011.3;
	4911	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10931.3;
	4911	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17210.4;
	4911	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2971.7;
	4912	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1227.6;
	4912	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1389.8;
	4912	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1522.2;
	4912	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6287.2;
	4912	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20255;
	4912	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11142.1;
	4912	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17240.9;
	4912	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2982.4;
	4913	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1230.1;
	4913	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1391.2;
	4913	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1527.3;
	4913	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6240;
	4913	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20211.2;
	4913	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11216.6;
	4913	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17041.9;
	4913	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2979;
	4914	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1223.8;
	4914	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1366.9;
	4914	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1508.4;
	4914	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6094.7;
	4914	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19820.6;
	4914	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11066.2;
	4914	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16599.3;
	4914	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2906.5;
	4915	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1206.7;
	4915	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1312.2;
	4915	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1466;
	4915	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5859;
	4915	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19096.7;
	4915	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10719.7;
	4915	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16009.5;
	4915	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2823.9;
	4916	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1175.4;
	4916	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1246.8;
	4916	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1403.9;
	4916	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5659.1;
	4916	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18265.6;
	4916	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10303.9;
	4916	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15537.1;
	4916	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2716.9;
	4917	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1154.7;
	4917	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1210.2;
	4917	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1355.5;
	4917	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5536.4;
	4917	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17645.1;
	4917	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9980.4;
	4917	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15096.1;
	4917	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2596.5;
	4918	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1116.6;
	4918	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1134.7;
	4918	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1276;
	4918	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5266.2;
	4918	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16457.3;
	4918	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9334;
	4918	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14340.4;
	4918	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2444.4;
	4919	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1076.5;
	4919	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1065.3;
	4919	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1191.6;
	4919	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4966.7;
	4919	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15233.8;
	4919	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8654.2;
	4919	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13493.5;
	4919	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2267.4;
	4920	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1040.3;
	4920	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	997.2;
	4920	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1124.7;
	4920	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4678.8;
	4920	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14081;
	4920	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7945.8;
	4920	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12703.1;
	4920	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2059;
	4921	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1010.4;
	4921	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	947.7;
	4921	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1070.4;
	4921	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4423;
	4921	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13168.5;
	4921	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7407.9;
	4921	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12064.3;
	4921	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1959.9;
	4922	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986;
	4922	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	907.3;
	4922	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1024.7;
	4922	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4202.4;
	4922	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12425.1;
	4922	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6981.4;
	4922	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11538.1;
	4922	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1886.5;
	4923	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.5;
	4923	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	884;
	4923	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	987.2;
	4923	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4017.6;
	4923	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11863.9;
	4923	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6654.2;
	4923	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11148.7;
	4923	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1817.7;
	4924	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.9;
	4924	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	853.7;
	4924	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	959.3;
	4924	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3884.2;
	4924	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11501.9;
	4924	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6380.6;
	4924	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10897.3;
	4924	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1755.1;
	4925	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948.3;
	4925	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	838.7;
	4925	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	945.7;
	4925	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3810.7;
	4925	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11326.5;
	4925	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6284.1;
	4925	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10739.1;
	4925	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1752.7;
	4926	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.4;
	4926	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	833.5;
	4926	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	938.4;
	4926	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3771.1;
	4926	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11181.7;
	4926	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6269.5;
	4926	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10580.5;
	4926	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1742.6;
	4927	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.9;
	4927	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	844.2;
	4927	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	944.2;
	4927	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3732.4;
	4927	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11345.2;
	4927	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6269.9;
	4927	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10664.4;
	4927	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1751.5;
	4928	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.3;
	4928	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	926.5;
	4928	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1012.4;
	4928	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4066.9;
	4928	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12418.3;
	4928	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6737.8;
	4928	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11561.9;
	4928	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1924.3;
	4929	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1015.2;
	4929	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1009.7;
	4929	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1090.3;
	4929	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4543.4;
	4929	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13811.5;
	4929	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7475.7;
	4929	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12733.1;
	4929	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2045.4;
	4930	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1060.1;
	4930	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1090.7;
	4930	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1169.8;
	4930	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4964.4;
	4930	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15196.1;
	4930	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8215.4;
	4930	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13840.6;
	4930	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2242.7;
	4931	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1103.6;
	4931	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1169.6;
	4931	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1254.8;
	4931	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5369.1;
	4931	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16583.4;
	4931	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8943.3;
	4931	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14822.5;
	4931	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2484.4;
	4932	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1145;
	4932	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1241.7;
	4932	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1340.2;
	4932	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5700.2;
	4932	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17821;
	4932	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9620.2;
	4932	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15645.7;
	4932	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2652.8;
	4933	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1177.3;
	4933	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1302;
	4933	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1403.5;
	4933	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5966.1;
	4933	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18796.5;
	4933	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10185.5;
	4933	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16328.1;
	4933	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2779;
	4934	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1199.1;
	4934	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1339.6;
	4934	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1457.2;
	4934	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6137.6;
	4934	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19524.3;
	4934	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10641.8;
	4934	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16741.3;
	4934	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2863.9;
	4935	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1218.7;
	4935	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1365.3;
	4935	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1490.6;
	4935	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6231;
	4935	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19984.3;
	4935	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10992.9;
	4935	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16959.4;
	4935	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2907.9;
	4936	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1231.3;
	4936	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1381.9;
	4936	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1519.9;
	4936	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6268.3;
	4936	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20283.5;
	4936	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11203.8;
	4936	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16982.7;
	4936	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2903.5;
	4937	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1238.7;
	4937	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1388.4;
	4937	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1547.8;
	4937	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6235.3;
	4937	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20394.2;
	4937	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11315.9;
	4937	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16769.9;
	4937	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2819.2;
	4938	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1228.7;
	4938	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1369.9;
	4938	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1540.8;
	4938	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6107;
	4938	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20093.3;
	4938	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11190.5;
	4938	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16310.3;
	4938	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2748.2;
	4939	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1213.6;
	4939	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1326.8;
	4939	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1478.4;
	4939	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5896.8;
	4939	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19461.9;
	4939	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10887.5;
	4939	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15848.3;
	4939	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2671.2;
	4940	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1184.1;
	4940	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1270;
	4940	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1400.4;
	4940	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5742.7;
	4940	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18688.3;
	4940	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10492.2;
	4940	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15489;
	4940	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2571.3;
	4941	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1170.5;
	4941	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1238.3;
	4941	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1362.8;
	4941	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5671;
	4941	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18101.5;
	4941	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10205.2;
	4941	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15192.4;
	4941	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2413.2;
	4942	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1131.3;
	4942	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1157.7;
	4942	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1277.5;
	4942	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5391.9;
	4942	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16845.5;
	4942	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9495.7;
	4942	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14367.4;
	4942	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2257.9;
	4943	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1080.3;
	4943	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1086.9;
	4943	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1173;
	4943	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5011.9;
	4943	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15376.7;
	4943	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8662.9;
	4943	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13283.7;
	4943	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2157.1;
	4944	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1041.1;
	4944	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1004.7;
	4944	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1115.3;
	4944	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4660.8;
	4944	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14109.2;
	4944	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7923;
	4944	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12320.8;
	4944	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2020.3;
	4945	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.5;
	4945	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	948;
	4945	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1057.1;
	4945	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4383.3;
	4945	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13178.8;
	4945	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7356.3;
	4945	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11704.1;
	4945	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1914;
	4946	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	987.5;
	4946	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	908.3;
	4946	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1011.6;
	4946	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4166.8;
	4946	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12509.4;
	4946	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6935.3;
	4946	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11298.3;
	4946	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1835.9;
	4947	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	971.9;
	4947	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	887.1;
	4947	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	977.5;
	4947	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4024.3;
	4947	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12065.3;
	4947	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6664.1;
	4947	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11108.5;
	4947	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1798.2;
	4948	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.6;
	4948	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	869.1;
	4948	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	955.9;
	4948	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3946.9;
	4948	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11910.7;
	4948	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6546.6;
	4948	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11095.2;
	4948	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1796.8;
	4949	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960;
	4949	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	875.2;
	4949	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	953.8;
	4949	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3975;
	4949	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12155.2;
	4949	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6701.1;
	4949	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11361.2;
	4949	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1839.3;
	4950	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	968.9;
	4950	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	903.7;
	4950	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	985.9;
	4950	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4102.3;
	4950	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12637.6;
	4950	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7035.4;
	4950	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11668;
	4950	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1919;
	4951	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	976.6;
	4951	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	940.2;
	4951	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1011;
	4951	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4147.7;
	4951	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12908.2;
	4951	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7163.6;
	4951	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11974.2;
	4951	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1989.4;
	4952	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1005.9;
	4952	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	991.5;
	4952	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1071.3;
	4952	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4445.1;
	4952	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13477.3;
	4952	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7430.5;
	4952	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12755.8;
	4952	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2097.6;
	4953	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1043.1;
	4953	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1070;
	4953	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1155.8;
	4953	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4845.1;
	4953	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14382.6;
	4953	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7952.5;
	4953	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13737.5;
	4953	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2248.3;
	4954	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1099.5;
	4954	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1145.2;
	4954	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1243.5;
	4954	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5257;
	4954	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15459.4;
	4954	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8748.5;
	4954	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14721.7;
	4954	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2407.1;
	4955	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1159.3;
	4955	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1227.2;
	4955	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1362.1;
	4955	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5663;
	4955	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16526.6;
	4955	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9476.2;
	4955	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15554.8;
	4955	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2589.1;
	4956	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1202.8;
	4956	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1312.2;
	4956	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1451.9;
	4956	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6022.4;
	4956	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17877.5;
	4956	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10197.3;
	4956	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16207.8;
	4956	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2763.2;
	4957	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1237.5;
	4957	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1377.1;
	4957	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1514.4;
	4957	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6313.5;
	4957	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19214.1;
	4957	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10800.6;
	4957	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16588.5;
	4957	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2818.9;
	4958	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1258.8;
	4958	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1389.7;
	4958	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1552.5;
	4958	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6509;
	4958	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20109.4;
	4958	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11299.8;
	4958	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16371.9;
	4958	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2836.1;
	4959	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1272.4;
	4959	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1394.3;
	4959	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1585.6;
	4959	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6600;
	4959	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20326.4;
	4959	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11662.4;
	4959	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15510.2;
	4959	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2715.5;
	4960	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1278.1;
	4960	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1390.9;
	4960	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1598.7;
	4960	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6562.3;
	4960	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19730.6;
	4960	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11756.2;
	4960	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14713.2;
	4960	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2574.9;
	4961	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1259.1;
	4961	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1360.6;
	4961	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1577.4;
	4961	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6378.6;
	4961	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18888.2;
	4961	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11407.5;
	4961	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14336.5;
	4961	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2533.1;
	4962	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1246.8;
	4962	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1304.2;
	4962	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1543.4;
	4962	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6157.9;
	4962	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17938.2;
	4962	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10774.7;
	4962	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13832.4;
	4962	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2464.9;
	4963	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1226;
	4963	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1262.8;
	4963	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1483;
	4963	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5926;
	4963	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17009.5;
	4963	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10160.1;
	4963	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13442.2;
	4963	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2378.6;
	4964	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1197.5;
	4964	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1220.9;
	4964	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1412.5;
	4964	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5795.3;
	4964	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16354.4;
	4964	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9707.4;
	4964	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13390.8;
	4964	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2345.9;
	4965	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1173.3;
	4965	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1183.6;
	4965	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1356.6;
	4965	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5659.7;
	4965	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15842.1;
	4965	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9027;
	4965	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13016.6;
	4965	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2290.8;
	4966	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1138.9;
	4966	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1121.4;
	4966	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1267;
	4966	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5342.9;
	4966	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14655.5;
	4966	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8181.8;
	4966	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12337.1;
	4966	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2143.5;
	4967	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1079.3;
	4967	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1037.4;
	4967	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1154.6;
	4967	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4921.4;
	4967	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13344;
	4967	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7376.4;
	4967	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11499.8;
	4967	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1943;
	4968	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1048.3;
	4968	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	956.9;
	4968	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1065.9;
	4968	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4573;
	4968	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12171.2;
	4968	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6720.9;
	4968	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10924.5;
	4968	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1823.9;
	4969	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1020.1;
	4969	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	902.9;
	4969	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1002.8;
	4969	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4312.5;
	4969	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11360.4;
	4969	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6267.2;
	4969	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10427.5;
	4969	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1717;
	4970	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000.5;
	4970	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	865.8;
	4970	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	960.5;
	4970	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4117.2;
	4970	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10837.4;
	4970	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5967.2;
	4970	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10157.4;
	4970	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1648.6;
	4971	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	982.4;
	4971	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	849.5;
	4971	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	932.1;
	4971	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3983.5;
	4971	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10528.9;
	4971	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5803.7;
	4971	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10039.8;
	4971	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1627;
	4972	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	976.5;
	4972	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	839.1;
	4972	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	919.6;
	4972	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3924.9;
	4972	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10536.9;
	4972	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5778.7;
	4972	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10167.6;
	4972	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1617.3;
	4973	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	974.8;
	4973	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	844.7;
	4973	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	930.5;
	4973	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3968;
	4973	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10884.3;
	4973	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6015.4;
	4973	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10540.3;
	4973	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1645.4;
	4974	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	983;
	4974	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	870.5;
	4974	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	965.3;
	4974	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4103.2;
	4974	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11450.5;
	4974	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6395.5;
	4974	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10885.6;
	4974	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1758.2;
	4975	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	982.5;
	4975	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	891.2;
	4975	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	983;
	4975	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4132.7;
	4975	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11921.1;
	4975	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6578.3;
	4975	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11054.5;
	4975	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1790.5;
	4976	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1010.1;
	4976	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	943.6;
	4976	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1035.8;
	4976	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4333.2;
	4976	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12784.7;
	4976	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6848.1;
	4976	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11432.6;
	4976	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1986.6;
	4977	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1046.4;
	4977	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1018;
	4977	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1108;
	4977	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4657.5;
	4977	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13967.1;
	4977	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7272.1;
	4977	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12083.8;
	4977	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2068.4;
	4978	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1085.8;
	4978	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1099.6;
	4978	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1184.9;
	4978	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5040.2;
	4978	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15291.6;
	4978	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7762.8;
	4978	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12955.1;
	4978	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2223.3;
	4979	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1128;
	4979	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1179.9;
	4979	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1268.4;
	4979	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5388;
	4979	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16617.4;
	4979	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8328.5;
	4979	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13765.5;
	4979	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2444.2;
	4980	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1166.6;
	4980	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1234.2;
	4980	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1341.3;
	4980	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5660.9;
	4980	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17800.6;
	4980	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8906.6;
	4980	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14150.3;
	4980	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2607;
	4981	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1203.6;
	4981	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1277.7;
	4981	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1409.8;
	4981	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5856.8;
	4981	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18778.9;
	4981	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9369.9;
	4981	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14111.9;
	4981	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2707.1;
	4982	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1228.9;
	4982	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1319.3;
	4982	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1460.1;
	4982	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5909.9;
	4982	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19475.3;
	4982	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9292.2;
	4982	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13703.5;
	4982	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2838.2;
	4983	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1242.3;
	4983	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1344;
	4983	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1474.8;
	4983	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5936.2;
	4983	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19920.9;
	4983	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9152.9;
	4983	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13405.7;
	4983	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2911.1;
	4984	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1236.2;
	4984	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1328.9;
	4984	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1467.1;
	4984	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5861;
	4984	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19999.3;
	4984	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8896.4;
	4984	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13206.7;
	4984	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2888.2;
	4985	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1197.2;
	4985	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1273.2;
	4985	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1428.4;
	4985	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5711.2;
	4985	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19791.1;
	4985	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8755.7;
	4985	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13158.4;
	4985	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2795.8;
	4986	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1138.5;
	4986	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1222.6;
	4986	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1387.2;
	4986	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5523.2;
	4986	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19108.4;
	4986	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8751.5;
	4986	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13056.9;
	4986	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2721.1;
	4987	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1086.9;
	4987	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1180.1;
	4987	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1336.9;
	4987	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5313.4;
	4987	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18051;
	4987	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8583.6;
	4987	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12798.8;
	4987	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2601.6;
	4988	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1056.4;
	4988	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1146;
	4988	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1286.1;
	4988	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5250.8;
	4988	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17266.6;
	4988	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8397.3;
	4988	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12764.3;
	4988	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2523.5;
	4989	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1043.5;
	4989	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1117.5;
	4989	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1258.4;
	4989	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5165.8;
	4989	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16635.3;
	4989	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8227.2;
	4989	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12518.2;
	4989	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2438.9;
	4990	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1011.7;
	4990	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1063.3;
	4990	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1179.7;
	4990	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4889.7;
	4990	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15371.7;
	4990	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7672.4;
	4990	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11843.4;
	4990	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2269.3;
	4991	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	974.2;
	4991	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	981.4;
	4991	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1083;
	4991	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4546.7;
	4991	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13963.5;
	4991	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6968.1;
	4991	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11049.9;
	4991	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2066.8;
	4992	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.3;
	4992	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	918.2;
	4992	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1010.5;
	4992	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4268.6;
	4992	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12837.9;
	4992	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6396;
	4992	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10341.1;
	4992	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1908.5;
	4993	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.6;
	4993	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	879.2;
	4993	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	960.7;
	4993	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4060.3;
	4993	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12065.2;
	4993	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5992.1;
	4993	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9881.8;
	4993	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1811.6;
	4994	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.4;
	4994	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	846;
	4994	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	920.5;
	4994	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3909.5;
	4994	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11500.4;
	4994	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5716.2;
	4994	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9588.6;
	4994	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1747.9;
	4995	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920.2;
	4995	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	835.6;
	4995	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	898.2;
	4995	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3826.9;
	4995	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11157.2;
	4995	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5584.6;
	4995	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9476.4;
	4995	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1712.8;
	4996	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.8;
	4996	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	821;
	4996	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	888.7;
	4996	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3811.6;
	4996	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11127.4;
	4996	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5631.4;
	4996	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9637.5;
	4996	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1642.7;
	4997	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.6;
	4997	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	840.9;
	4997	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	901.6;
	4997	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3903.5;
	4997	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11491.2;
	4997	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5890.5;
	4997	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10152;
	4997	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1659.2;
	4998	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.8;
	4998	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	870.5;
	4998	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	943.4;
	4998	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4050;
	4998	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12187.9;
	4998	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6339.8;
	4998	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10652.6;
	4998	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1741.9;
	4999	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946;
	4999	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	898.8;
	4999	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	982.7;
	4999	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4082.6;
	4999	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12531;
	4999	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6546.6;
	4999	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10900.5;
	4999	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1807.6;
	5000	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	978.2;
	5000	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	944.1;
	5000	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1045.7;
	5000	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4309.8;
	5000	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13284.3;
	5000	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6743;
	5000	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11357;
	5000	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1943.7;
	5001	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1024.4;
	5001	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1013;
	5001	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1123;
	5001	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4596.4;
	5001	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14370.1;
	5001	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7013.8;
	5001	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11875.8;
	5001	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2119.4;
	5002	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1065;
	5002	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1087.3;
	5002	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1205.5;
	5002	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4921.2;
	5002	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15759.6;
	5002	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7391.8;
	5002	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12113.1;
	5002	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2314.7;
	5003	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1116.4;
	5003	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1162.2;
	5003	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1291.5;
	5003	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5243.2;
	5003	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16998.6;
	5003	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7822.7;
	5003	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12356.6;
	5003	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2473;
	5004	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1145.4;
	5004	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1209.9;
	5004	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1353.3;
	5004	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5519.5;
	5004	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17979.3;
	5004	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8218;
	5004	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12640.6;
	5004	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2527.4;
	5005	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1182.4;
	5005	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1233.1;
	5005	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1413.1;
	5005	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5737.8;
	5005	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18771.5;
	5005	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8579.4;
	5005	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13133.7;
	5005	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2602.5;
	5006	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1199.7;
	5006	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1244.6;
	5006	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1432.5;
	5006	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5888.2;
	5006	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19143.3;
	5006	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8582.2;
	5006	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13400.6;
	5006	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2617.9;
	5007	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1211.5;
	5007	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1257.2;
	5007	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1441.1;
	5007	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5959.1;
	5007	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18955.3;
	5007	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8553.2;
	5007	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13801.2;
	5007	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2647.9;
	5008	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1223.3;
	5008	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1261.3;
	5008	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1438.8;
	5008	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5908.2;
	5008	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18308.2;
	5008	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8613.3;
	5008	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14114.2;
	5008	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2650.2;
	5009	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1221.5;
	5009	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1246.6;
	5009	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1426.8;
	5009	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5782.4;
	5009	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17501.5;
	5009	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8491.1;
	5009	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14193.3;
	5009	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2626.3;
	5010	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1210.1;
	5010	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1227.9;
	5010	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1389;
	5010	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5627.1;
	5010	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16984.8;
	5010	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8408.1;
	5010	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13944.7;
	5010	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2582.8;
	5011	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1190.7;
	5011	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1200;
	5011	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1335.8;
	5011	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5421.5;
	5011	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16365;
	5011	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8188.6;
	5011	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13692.8;
	5011	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2471.3;
	5012	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1144.5;
	5012	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1159.6;
	5012	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1275;
	5012	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5321.5;
	5012	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15996.9;
	5012	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8044.4;
	5012	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13694.2;
	5012	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2367.3;
	5013	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1123.5;
	5013	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1116.3;
	5013	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1242.2;
	5013	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5239.7;
	5013	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15637.6;
	5013	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7850.4;
	5013	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13385.4;
	5013	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2260.6;
	5014	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1086.1;
	5014	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1054.1;
	5014	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1163.7;
	5014	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4966;
	5014	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14486.6;
	5014	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7315.6;
	5014	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12649.4;
	5014	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2092.4;
	5015	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1033.4;
	5015	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	969.4;
	5015	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1070.2;
	5015	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4628.8;
	5015	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13298.2;
	5015	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6724.7;
	5015	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11873.1;
	5015	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1921.6;
	5016	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	987.6;
	5016	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	894.4;
	5016	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	988.8;
	5016	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4260;
	5016	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12215.2;
	5016	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6384.7;
	5016	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11116.7;
	5016	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1752.3;
	5017	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.6;
	5017	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	855.2;
	5017	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	935.2;
	5017	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4069.7;
	5017	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11517.2;
	5017	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5957.3;
	5017	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10680.6;
	5017	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1682.4;
	5018	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948.8;
	5018	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	836.5;
	5018	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	899;
	5018	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3931.7;
	5018	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11001.3;
	5018	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5697.8;
	5018	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10412.8;
	5018	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1625.2;
	5019	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.7;
	5019	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	821;
	5019	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.9;
	5019	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3839.8;
	5019	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10714.3;
	5019	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5551.1;
	5019	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10307.3;
	5019	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1595.1;
	5020	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.8;
	5020	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	814.1;
	5020	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	856.6;
	5020	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3818;
	5020	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10763.3;
	5020	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5562;
	5020	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10443.2;
	5020	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1594.7;
	5021	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.6;
	5021	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	837.5;
	5021	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	873.1;
	5021	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3912.3;
	5021	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11172.7;
	5021	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5821.7;
	5021	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10894.1;
	5021	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1666.9;
	5022	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.4;
	5022	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	854.5;
	5022	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913.5;
	5022	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4079.5;
	5022	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11824.5;
	5022	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6251.7;
	5022	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11344.3;
	5022	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1779.6;
	5023	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.6;
	5023	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	869.8;
	5023	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	943.3;
	5023	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4125.8;
	5023	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12150.5;
	5023	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6505.8;
	5023	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11563.7;
	5023	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1835.6;
	5024	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	979.2;
	5024	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	925.2;
	5024	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1003.4;
	5024	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4377.2;
	5024	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12679.6;
	5024	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6878.7;
	5024	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12137.3;
	5024	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1955.9;
	5025	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1011.9;
	5025	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	986.9;
	5025	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1066.9;
	5025	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4739.6;
	5025	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13451.4;
	5025	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7414.9;
	5025	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12983.2;
	5025	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2140;
	5026	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1054.9;
	5026	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1059.6;
	5026	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1140;
	5026	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5109;
	5026	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14601.9;
	5026	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8101.1;
	5026	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13933.6;
	5026	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2255.5;
	5027	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1093.9;
	5027	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1147.1;
	5027	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1228.9;
	5027	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5466.6;
	5027	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15802.1;
	5027	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8781.3;
	5027	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14426.5;
	5027	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2445.6;
	5028	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1123.6;
	5028	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1222.1;
	5028	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1312.6;
	5028	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5752.8;
	5028	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16934.9;
	5028	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9376.8;
	5028	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13676.4;
	5028	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2541.4;
	5029	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1152.9;
	5029	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1285.9;
	5029	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1378.3;
	5029	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5995.7;
	5029	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18116.4;
	5029	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9908.4;
	5029	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13249.5;
	5029	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2664.4;
	5030	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1184.7;
	5030	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1333.9;
	5030	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1416.9;
	5030	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6134.9;
	5030	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19043.6;
	5030	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10348.1;
	5030	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13094.8;
	5030	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2670.8;
	5031	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1205;
	5031	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1360.3;
	5031	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1457.4;
	5031	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6191.2;
	5031	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19607.9;
	5031	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10476.8;
	5031	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13211.2;
	5031	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2587.2;
	5032	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1211.3;
	5032	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1361.7;
	5032	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1472.5;
	5032	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6100.3;
	5032	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19798.6;
	5032	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9963.5;
	5032	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13295.8;
	5032	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2495.3;
	5033	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1206.7;
	5033	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1344.7;
	5033	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1451.7;
	5033	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5925.7;
	5033	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19278.7;
	5033	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9347.4;
	5033	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13405;
	5033	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2388.4;
	5034	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1201.5;
	5034	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1315.7;
	5034	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1400.4;
	5034	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5624.9;
	5034	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18221.2;
	5034	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8839.1;
	5034	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13447.2;
	5034	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2313.7;
	5035	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1176;
	5035	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1261;
	5035	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1331.4;
	5035	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5325.5;
	5035	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17086.8;
	5035	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8457.8;
	5035	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13196.2;
	5035	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2219.7;
	5036	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1137;
	5036	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1209.6;
	5036	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1270.2;
	5036	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5227.7;
	5036	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16282.7;
	5036	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8208.9;
	5036	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13176.9;
	5036	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2205.1;
	5037	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1109;
	5037	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1165.5;
	5037	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1229.6;
	5037	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5103;
	5037	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15680.2;
	5037	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7997.9;
	5037	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12908.8;
	5037	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2150.6;
	5038	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1071.6;
	5038	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1097.5;
	5038	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1148.3;
	5038	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4834.9;
	5038	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14544.1;
	5038	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7436;
	5038	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12202.5;
	5038	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2019.7;
	5039	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1023.3;
	5039	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1004.7;
	5039	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1059.3;
	5039	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4522.1;
	5039	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13286.5;
	5039	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6765.3;
	5039	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11433.5;
	5039	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1876.4;
	5040	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	982;
	5040	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	938.7;
	5040	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1009.1;
	5040	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4232.2;
	5040	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12210.3;
	5040	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6152.1;
	5040	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10703.9;
	5040	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1713;
	5041	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.4;
	5041	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	890.5;
	5041	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	958.3;
	5041	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4015.5;
	5041	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11491.8;
	5041	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5781.3;
	5041	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10239.1;
	5041	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1643;
	5042	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.7;
	5042	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	857.8;
	5042	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	922.6;
	5042	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3860.6;
	5042	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11023.5;
	5042	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5527.8;
	5042	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9914.6;
	5042	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1615.1;
	5043	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.6;
	5043	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	834.6;
	5043	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	895.4;
	5043	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3747.8;
	5043	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10662.7;
	5043	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5391.2;
	5043	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9766.1;
	5043	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1580.2;
	5044	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921;
	5044	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	820.8;
	5044	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.4;
	5044	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3697.1;
	5044	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10663;
	5044	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5393.7;
	5044	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9813.7;
	5044	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1520.2;
	5045	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.2;
	5045	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	830.3;
	5045	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	895.4;
	5045	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3739.3;
	5045	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11023.7;
	5045	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5651.4;
	5045	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10149.5;
	5045	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1626.6;
	5046	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938;
	5046	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	854.4;
	5046	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	928.2;
	5046	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3860;
	5046	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11505.2;
	5046	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6088;
	5046	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10530.2;
	5046	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1685.6;
	5047	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.1;
	5047	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	870.5;
	5047	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	951.4;
	5047	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3910.3;
	5047	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11820.2;
	5047	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6353.1;
	5047	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10823.8;
	5047	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1726.2;
	5048	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.1;
	5048	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	926.4;
	5048	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1007.2;
	5048	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4207;
	5048	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12428.5;
	5048	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6732.4;
	5048	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11637.9;
	5048	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1847.8;
	5049	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1004.8;
	5049	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	983.6;
	5049	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1075.4;
	5049	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4623.3;
	5049	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13353;
	5049	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7262.8;
	5049	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12736.8;
	5049	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1976.6;
	5050	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1051.3;
	5050	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1046.5;
	5050	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1128.7;
	5050	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5037.7;
	5050	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14632.2;
	5050	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7797.3;
	5050	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13825.7;
	5050	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2103.9;
	5051	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1090.7;
	5051	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1102.2;
	5051	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1237.7;
	5051	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5399.9;
	5051	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15682.4;
	5051	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8247.2;
	5051	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14683.9;
	5051	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2246.9;
	5052	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1124.1;
	5052	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1160.2;
	5052	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1307;
	5052	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5700.1;
	5052	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16423.6;
	5052	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8682.8;
	5052	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15448.2;
	5052	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2382.6;
	5053	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1155.1;
	5053	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1219.3;
	5053	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1360.9;
	5053	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5927.3;
	5053	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17244.7;
	5053	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9164.1;
	5053	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16089.8;
	5053	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2560;
	5054	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1181.9;
	5054	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1270.9;
	5054	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1402.4;
	5054	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6064.3;
	5054	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18115.7;
	5054	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9654.4;
	5054	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16425.4;
	5054	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2612.2;
	5055	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1197.5;
	5055	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1309.2;
	5055	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1437.9;
	5055	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6064.3;
	5055	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18771.4;
	5055	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10091.8;
	5055	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16535.6;
	5055	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2658.8;
	5056	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1208.5;
	5056	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1324.2;
	5056	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1452.3;
	5056	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6034.6;
	5056	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19177;
	5056	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10436.1;
	5056	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16282.8;
	5056	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2667.6;
	5057	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1203.1;
	5057	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1317.7;
	5057	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1451.2;
	5057	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5945.9;
	5057	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19068.5;
	5057	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10547;
	5057	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15921.7;
	5057	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2652.9;
	5058	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1191;
	5058	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1287.9;
	5058	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1425.3;
	5058	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5717.4;
	5058	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18557.7;
	5058	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10354.8;
	5058	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15155.9;
	5058	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2598.1;
	5059	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1167.4;
	5059	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1236.1;
	5059	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1381.4;
	5059	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5445.1;
	5059	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17771.4;
	5059	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9888.7;
	5059	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14464.4;
	5059	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2517.3;
	5060	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1133.6;
	5060	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1188.4;
	5060	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1321;
	5060	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5310.6;
	5060	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16938.7;
	5060	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9405.8;
	5060	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14091;
	5060	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2396.5;
	5061	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1117.1;
	5061	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1149.9;
	5061	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1280.2;
	5061	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5201.8;
	5061	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16303.1;
	5061	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9015;
	5061	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13703.1;
	5061	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2322.1;
	5062	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1084.4;
	5062	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1085.2;
	5062	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1203;
	5062	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4922.2;
	5062	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15164.5;
	5062	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8397.7;
	5062	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12989.5;
	5062	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2168.1;
	5063	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1042.5;
	5063	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1003.7;
	5063	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1117.9;
	5063	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4598.7;
	5063	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13970.6;
	5063	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7713.2;
	5063	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12224;
	5063	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1962.7;
	5064	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	988;
	5064	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	946.3;
	5064	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1044.6;
	5064	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4302.5;
	5064	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12956.2;
	5064	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7151.8;
	5064	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11475.8;
	5064	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1811.1;
	5065	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.9;
	5065	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	895.5;
	5065	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	988.2;
	5065	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4053.1;
	5065	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12166.5;
	5065	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6666.5;
	5065	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10845.8;
	5065	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1713.4;
	5066	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.6;
	5066	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	862.9;
	5066	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	948.8;
	5066	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3847.2;
	5066	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11584.5;
	5066	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6276.9;
	5066	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10361.7;
	5066	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1725.8;
	5067	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.1;
	5067	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	845.9;
	5067	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	922.5;
	5067	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3698.8;
	5067	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11234.3;
	5067	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5980.6;
	5067	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10031.8;
	5067	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1604.3;
	5068	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.7;
	5068	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	820.4;
	5068	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	902.1;
	5068	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3604.2;
	5068	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10966.3;
	5068	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5815.1;
	5068	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9858.9;
	5068	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1674.8;
	5069	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.4;
	5069	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	817.9;
	5069	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	899.1;
	5069	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3582.7;
	5069	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10952.5;
	5069	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5814.6;
	5069	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9853;
	5069	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1715;
	5070	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920.5;
	5070	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	820.5;
	5070	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	900.8;
	5070	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3594;
	5070	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11048.5;
	5070	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5904.3;
	5070	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9874.5;
	5070	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1726.6;
	5071	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.9;
	5071	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	824.1;
	5071	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	907;
	5071	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3581.8;
	5071	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11257.3;
	5071	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6022.5;
	5071	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10074.3;
	5071	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1758.7;
	5072	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.7;
	5072	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	883.6;
	5072	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	969.9;
	5072	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3927.1;
	5072	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12284.1;
	5072	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6504;
	5072	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11051.3;
	5072	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1921.6;
	5073	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	982.7;
	5073	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	967.5;
	5073	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1052.9;
	5073	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4367;
	5073	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13703;
	5073	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7132.8;
	5073	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12273.3;
	5073	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2120;
	5074	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1027.9;
	5074	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1051.1;
	5074	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1140.2;
	5074	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4780;
	5074	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15122.9;
	5074	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7846;
	5074	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13428.1;
	5074	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2311.6;
	5075	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1067.4;
	5075	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1127.8;
	5075	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1225.2;
	5075	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5167.9;
	5075	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16459.3;
	5075	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8477;
	5075	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14326.2;
	5075	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2435.2;
	5076	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1100.3;
	5076	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1197.9;
	5076	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1302.7;
	5076	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5463.2;
	5076	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17548.2;
	5076	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9019;
	5076	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15037.5;
	5076	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2574;
	5077	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1129;
	5077	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1246.5;
	5077	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1371.3;
	5077	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5708.6;
	5077	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18371;
	5077	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9493.9;
	5077	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15616.4;
	5077	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2707.5;
	5078	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1150.2;
	5078	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1276.9;
	5078	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1422.7;
	5078	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5879.2;
	5078	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18999.7;
	5078	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9878.6;
	5078	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16100.5;
	5078	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2804.9;
	5079	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1172.4;
	5079	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1307.4;
	5079	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1460.6;
	5079	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5956.3;
	5079	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19375.6;
	5079	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10185.7;
	5079	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16355.1;
	5079	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2863.3;
	5080	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1179.7;
	5080	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1319.2;
	5080	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1482;
	5080	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5992.4;
	5080	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19538.4;
	5080	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10350.4;
	5080	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16095.7;
	5080	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2816.9;
	5081	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1179.8;
	5081	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1316.2;
	5081	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1480.7;
	5081	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5925.1;
	5081	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19442.2;
	5081	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10395;
	5081	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15243.8;
	5081	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2818.3;
	5082	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1170.9;
	5082	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1286.4;
	5082	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1456.1;
	5082	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5767.3;
	5082	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19043.3;
	5082	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10262.5;
	5082	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14570.8;
	5082	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2757.4;
	5083	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1151.7;
	5083	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1237.5;
	5083	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1403.8;
	5083	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5522.7;
	5083	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18239.2;
	5083	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9950.6;
	5083	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14110.9;
	5083	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2648.8;
	5084	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1121.2;
	5084	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1175.4;
	5084	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1338.8;
	5084	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5384.5;
	5084	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17381.5;
	5084	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9527.6;
	5084	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13874;
	5084	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2520.9;
	5085	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1105.8;
	5085	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1136.1;
	5085	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1298.2;
	5085	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5272.9;
	5085	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16728.8;
	5085	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9179.2;
	5085	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13582.3;
	5085	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2378.6;
	5086	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1073;
	5086	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1077.3;
	5086	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1218.3;
	5086	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4999;
	5086	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15576.9;
	5086	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8613.6;
	5086	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12911.3;
	5086	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2221.9;
	5087	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030.6;
	5087	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1002.8;
	5087	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1136.4;
	5087	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4701.3;
	5087	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14429.5;
	5087	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7970.5;
	5087	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12146.6;
	5087	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2061.1;
	5088	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	996.9;
	5088	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	941;
	5088	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1066.1;
	5088	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4415.5;
	5088	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13386.3;
	5088	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7325.9;
	5088	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11387.8;
	5088	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1950.8;
	5089	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.5;
	5089	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	897.6;
	5089	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1017.2;
	5089	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4186.4;
	5089	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12580.7;
	5089	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6822;
	5089	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10816.3;
	5089	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1840.1;
	5090	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.4;
	5090	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	862.9;
	5090	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	976.3;
	5090	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4001.7;
	5090	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11946.9;
	5090	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6405.6;
	5090	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10371.6;
	5090	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1740.5;
	5091	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.9;
	5091	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	840;
	5091	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	944.9;
	5091	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3849.4;
	5091	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11485.1;
	5091	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6124.9;
	5091	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10084.7;
	5091	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1701.8;
	5092	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.4;
	5092	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	821.1;
	5092	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	924.5;
	5092	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3744.3;
	5092	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11189;
	5092	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5998.4;
	5092	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9917.6;
	5092	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1676.4;
	5093	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.3;
	5093	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	815.1;
	5093	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	919.3;
	5093	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3699.9;
	5093	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11054.5;
	5093	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5973.8;
	5093	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9882.7;
	5093	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1681.8;
	5094	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920.7;
	5094	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	817.8;
	5094	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	918.2;
	5094	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3680.2;
	5094	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10973.9;
	5094	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6003.6;
	5094	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9862.4;
	5094	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1671.1;
	5095	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917;
	5095	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	834.7;
	5095	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	926.5;
	5095	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3622.2;
	5095	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11124.3;
	5095	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6024.3;
	5095	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9899.2;
	5095	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1691.2;
	5096	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.4;
	5096	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	895.9;
	5096	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	992.5;
	5096	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3885.4;
	5096	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12185.7;
	5096	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6491.8;
	5096	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10753.6;
	5096	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1819.6;
	5097	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989.6;
	5097	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	980.2;
	5097	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1068.2;
	5097	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4302.5;
	5097	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13494.4;
	5097	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7191.2;
	5097	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11852.5;
	5097	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2024.1;
	5098	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030.2;
	5098	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1059.4;
	5098	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1137.4;
	5098	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4665.4;
	5098	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14768.4;
	5098	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7886.2;
	5098	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12845.7;
	5098	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2217.4;
	5099	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1070.4;
	5099	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1131.9;
	5099	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1210.8;
	5099	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5007.7;
	5099	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16014.4;
	5099	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8518.2;
	5099	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13732.9;
	5099	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2387.5;
	5100	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1110.4;
	5100	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1199.2;
	5100	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1283.7;
	5100	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5326;
	5100	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17170;
	5100	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9067.7;
	5100	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14375.9;
	5100	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2546.8;
	5101	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1140.5;
	5101	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1257.1;
	5101	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1345.5;
	5101	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5602.1;
	5101	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18091.7;
	5101	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9510.5;
	5101	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14871.6;
	5101	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2681.4;
	5102	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1164.3;
	5102	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1296;
	5102	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1400.9;
	5102	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5802.6;
	5102	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18814;
	5102	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9892.8;
	5102	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15114.1;
	5102	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2753.5;
	5103	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1184.8;
	5103	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1322;
	5103	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1439;
	5103	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5934.1;
	5103	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19318.9;
	5103	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10217;
	5103	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15135.3;
	5103	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2797.2;
	5104	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1193.7;
	5104	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1339.2;
	5104	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1459;
	5104	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5992.8;
	5104	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19619.4;
	5104	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10479.3;
	5104	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14965.5;
	5104	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2832.9;
	5105	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1193.7;
	5105	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1341.3;
	5105	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1465.5;
	5105	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5950.8;
	5105	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19695;
	5105	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10638.9;
	5105	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14914.7;
	5105	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2825.3;
	5106	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1176.6;
	5106	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1322.9;
	5106	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1453.8;
	5106	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5812.4;
	5106	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19380.4;
	5106	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10530;
	5106	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14915.2;
	5106	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2792.6;
	5107	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1145.7;
	5107	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1281.1;
	5107	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1412;
	5107	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5576.1;
	5107	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18679.2;
	5107	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10178.3;
	5107	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14566.5;
	5107	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2662.6;
	5108	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1118.4;
	5108	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1226.8;
	5108	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1348.2;
	5108	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5434.6;
	5108	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17905.6;
	5108	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9815.4;
	5108	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14191.7;
	5108	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2525.1;
	5109	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1105;
	5109	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1191.4;
	5109	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1307.4;
	5109	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5339.1;
	5109	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17318.3;
	5109	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9512;
	5109	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13782;
	5109	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2494.9;
	5110	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1066.4;
	5110	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1124;
	5110	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1223.4;
	5110	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5055;
	5110	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16045.8;
	5110	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8819.3;
	5110	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12933.6;
	5110	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2365.9;
	5111	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1021.8;
	5111	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1038;
	5111	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1133;
	5111	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4707.5;
	5111	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14716.6;
	5111	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7968.9;
	5111	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12027.4;
	5111	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2204.2;
	5112	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	982.3;
	5112	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	972.2;
	5112	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1086.1;
	5112	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4368.2;
	5112	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13556.3;
	5112	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7262.6;
	5112	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11238.8;
	5112	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2021.5;
	5113	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.4;
	5113	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	931;
	5113	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1032.8;
	5113	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4115.6;
	5113	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12767.7;
	5113	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6727.3;
	5113	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10689.7;
	5113	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1886.1;
	5114	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944;
	5114	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	896.2;
	5114	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	993.3;
	5114	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3931.8;
	5114	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12203.5;
	5114	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6337.6;
	5114	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10328.8;
	5114	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1767.6;
	5115	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.3;
	5115	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	874.3;
	5115	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	963.4;
	5115	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3800.9;
	5115	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11812.8;
	5115	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6101.8;
	5115	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10127.6;
	5115	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1770.9;
	5116	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.8;
	5116	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	862.4;
	5116	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	945.1;
	5116	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3736.4;
	5116	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11736.9;
	5116	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5995.5;
	5116	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10153.9;
	5116	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1781.8;
	5117	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.2;
	5117	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	877.5;
	5117	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	951.6;
	5117	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3766;
	5117	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12000.3;
	5117	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6164.2;
	5117	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10448.9;
	5117	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1820.7;
	5118	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940;
	5118	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	900.2;
	5118	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	979.9;
	5118	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3884;
	5118	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12417.1;
	5118	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6456.8;
	5118	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10779.1;
	5118	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1913.2;
	5119	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.4;
	5119	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	917.6;
	5119	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1000.2;
	5119	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3922.7;
	5119	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12674.1;
	5119	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6608;
	5119	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11033;
	5119	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1973.4;
	5120	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.4;
	5120	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	983.2;
	5120	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1062.1;
	5120	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4220;
	5120	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13440.6;
	5120	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6956;
	5120	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11874;
	5120	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2093;
	5121	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1022.4;
	5121	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1058.9;
	5121	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1135.8;
	5121	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4644.2;
	5121	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14568.5;
	5121	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7532.7;
	5121	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12960.5;
	5121	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2278.5;
	5122	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1069.3;
	5122	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1138.2;
	5122	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1212.2;
	5122	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5064.2;
	5122	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15848.2;
	5122	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8224.6;
	5122	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14040.4;
	5122	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2448.8;
	5123	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1112.7;
	5123	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1214.8;
	5123	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1294.4;
	5123	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5447.5;
	5123	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17060;
	5123	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8895.3;
	5123	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14996.4;
	5123	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2605.5;
	5124	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1149.9;
	5124	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1282.8;
	5124	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1365.3;
	5124	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5783.4;
	5124	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18178.5;
	5124	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9481.8;
	5124	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15821.1;
	5124	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2752.5;
	5125	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1185.3;
	5125	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1347.8;
	5125	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1432.6;
	5125	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6056.1;
	5125	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19174.9;
	5125	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10003.6;
	5125	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16517.3;
	5125	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2858.9;
	5126	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1213.8;
	5126	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1392.5;
	5126	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1486.6;
	5126	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6267.2;
	5126	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19998.4;
	5126	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10441.1;
	5126	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17013.8;
	5126	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2964.5;
	5127	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1235.6;
	5127	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1421.8;
	5127	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1529.5;
	5127	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6384.3;
	5127	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20553.7;
	5127	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10746.8;
	5127	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17269.3;
	5127	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3020.1;
	5128	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1245.8;
	5128	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1426.6;
	5128	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1553.2;
	5128	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6389.9;
	5128	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20809.3;
	5128	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10965.5;
	5128	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17259.7;
	5128	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3030.5;
	5129	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1233.4;
	5129	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1411.6;
	5129	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1544;
	5129	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6264.7;
	5129	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20739.7;
	5129	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11020.1;
	5129	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17087.4;
	5129	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2996.3;
	5130	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1219.6;
	5130	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1383.4;
	5130	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1520.2;
	5130	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6058;
	5130	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20214.2;
	5130	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10880;
	5130	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16688.3;
	5130	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2938.3;
	5131	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1196.7;
	5131	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1329.3;
	5131	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1461.4;
	5131	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5792.3;
	5131	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19301.4;
	5131	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10446.2;
	5131	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16072.9;
	5131	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2822;
	5132	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1157.9;
	5132	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1266.5;
	5132	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1382;
	5132	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5643.7;
	5132	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18306.2;
	5132	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9923.1;
	5132	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15657.2;
	5132	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2695.8;
	5133	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1136.7;
	5133	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1211.4;
	5133	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1332.2;
	5133	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5526.7;
	5133	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17497.6;
	5133	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9540.1;
	5133	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15115.3;
	5133	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2597.4;
	5134	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1088.7;
	5134	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1126.2;
	5134	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1238;
	5134	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5180.6;
	5134	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16076.2;
	5134	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8829.9;
	5134	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14078.6;
	5134	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2405.2;
	5135	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1038.3;
	5135	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1038.7;
	5135	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1137.4;
	5135	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4773.9;
	5135	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14505.5;
	5135	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7996.3;
	5135	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13040.7;
	5135	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2208.5;
	5136	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1016.8;
	5136	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	965.7;
	5136	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1071.1;
	5136	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4399.7;
	5136	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13313.4;
	5136	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7284.8;
	5136	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12052.8;
	5136	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2036;
	5137	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	991.8;
	5137	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	918.3;
	5137	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1019.3;
	5137	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4140.3;
	5137	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12454.4;
	5137	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6729.6;
	5137	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11367.8;
	5137	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1847.4;
	5138	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	973.8;
	5138	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	883;
	5138	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	981.2;
	5138	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3928.7;
	5138	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11867.6;
	5138	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6329.1;
	5138	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10882.1;
	5138	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1761.1;
	5139	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.3;
	5139	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	855.5;
	5139	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	954;
	5139	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3775.6;
	5139	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11441.9;
	5139	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6114.5;
	5139	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10604.3;
	5139	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1758.7;
	5140	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948.4;
	5140	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	836.7;
	5140	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	936.7;
	5140	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3691.5;
	5140	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11372.2;
	5140	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6029.1;
	5140	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10559.2;
	5140	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1738.3;
	5141	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.7;
	5141	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	847.1;
	5141	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	946.9;
	5141	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3739.6;
	5141	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11623.3;
	5141	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6219.2;
	5141	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10859.4;
	5141	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1727.2;
	5142	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970.7;
	5142	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	873.1;
	5142	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	979.9;
	5142	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3902.8;
	5142	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12069.5;
	5142	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6554.9;
	5142	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11206.9;
	5142	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1854;
	5143	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	976.6;
	5143	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	892.9;
	5143	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	995.1;
	5143	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3963;
	5143	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12360.3;
	5143	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6712;
	5143	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11455;
	5143	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1925;
	5144	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009.7;
	5144	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	947;
	5144	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1045.9;
	5144	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4267.4;
	5144	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13179.6;
	5144	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7061.9;
	5144	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12307.1;
	5144	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2092.8;
	5145	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1046;
	5145	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1026.3;
	5145	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1115.2;
	5145	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4677.2;
	5145	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14343.3;
	5145	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7607.7;
	5145	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13390.7;
	5145	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2244.1;
	5146	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1087.1;
	5146	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1109;
	5146	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1190.4;
	5146	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5095.4;
	5146	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15549.3;
	5146	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8239.1;
	5146	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14422;
	5146	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2458.7;
	5147	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1130.4;
	5147	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1188.9;
	5147	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1273.8;
	5147	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5513.1;
	5147	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16779.3;
	5147	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8894.3;
	5147	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15368.7;
	5147	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2622.5;
	5148	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1171.8;
	5148	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1264.4;
	5148	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1354.4;
	5148	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5872.8;
	5148	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17973.2;
	5148	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9479.9;
	5148	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16117.2;
	5148	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2780.8;
	5149	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1210.8;
	5149	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1331.1;
	5149	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1426.9;
	5149	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6154.4;
	5149	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19034.6;
	5149	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10100.8;
	5149	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16806.1;
	5149	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2942.7;
	5150	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1237.9;
	5150	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1383.8;
	5150	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1483.2;
	5150	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6377.3;
	5150	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19948.2;
	5150	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10509.1;
	5150	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17222.8;
	5150	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3028.9;
	5151	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1255.1;
	5151	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1416.7;
	5151	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1521.4;
	5151	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6499.7;
	5151	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20529;
	5151	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10840.2;
	5151	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17328.3;
	5151	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3077.5;
	5152	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1256.8;
	5152	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1421.1;
	5152	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1547.7;
	5152	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6401.4;
	5152	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20807.6;
	5152	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11000.4;
	5152	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17381.8;
	5152	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3066.6;
	5153	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1236.9;
	5153	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1414.6;
	5153	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1541.7;
	5153	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6313.5;
	5153	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20831.6;
	5153	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11118.5;
	5153	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17175.3;
	5153	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3084.2;
	5154	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1199.8;
	5154	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1390.7;
	5154	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1520.8;
	5154	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6165.4;
	5154	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20539;
	5154	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11031.6;
	5154	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16688.5;
	5154	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2975;
	5155	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1147.6;
	5155	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1335.6;
	5155	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1469.6;
	5155	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5922.9;
	5155	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19677.8;
	5155	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10625.7;
	5155	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16157.2;
	5155	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2864.8;
	5156	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1119.7;
	5156	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1271.2;
	5156	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1402.7;
	5156	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5749.3;
	5156	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18692.3;
	5156	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10214.4;
	5156	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15765.3;
	5156	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2813;
	5157	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1102.1;
	5157	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1217;
	5157	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1349.9;
	5157	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5596.6;
	5157	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17882.4;
	5157	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9830.5;
	5157	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15171.2;
	5157	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2673;
	5158	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1053.4;
	5158	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1124.6;
	5158	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1251.8;
	5158	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5233;
	5158	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16461.6;
	5158	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9076;
	5158	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14195.4;
	5158	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2471.9;
	5159	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1011.9;
	5159	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1037.3;
	5159	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1153.3;
	5159	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4829.8;
	5159	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14968.5;
	5159	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8216;
	5159	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13143.8;
	5159	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2264.7;
	5160	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981.3;
	5160	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	957.1;
	5160	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1090.4;
	5160	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4459.1;
	5160	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13744.1;
	5160	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7516.5;
	5160	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12184.1;
	5160	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2087.7;
	5161	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.2;
	5161	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	910;
	5161	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1036.1;
	5161	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4174.6;
	5161	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12850.6;
	5161	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6950.7;
	5161	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11513;
	5161	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1979.4;
	5162	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.6;
	5162	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	874.4;
	5162	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	995;
	5162	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3968.4;
	5162	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12211.2;
	5162	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6532;
	5162	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11036.4;
	5162	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1915.6;
	5163	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.1;
	5163	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	850.8;
	5163	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	965.3;
	5163	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3818.7;
	5163	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11860;
	5163	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6256.4;
	5163	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10726.9;
	5163	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1823;
	5164	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.7;
	5164	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	834.2;
	5164	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	949.6;
	5164	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3736.9;
	5164	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11721.1;
	5164	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6162.3;
	5164	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10672.9;
	5164	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1824.2;
	5165	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.8;
	5165	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	843.2;
	5165	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	958.8;
	5165	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3764.7;
	5165	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12021.5;
	5165	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6315.4;
	5165	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10948.3;
	5165	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1822.7;
	5166	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.7;
	5166	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	877.6;
	5166	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	991.9;
	5166	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3930.9;
	5166	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12518.5;
	5166	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6677;
	5166	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11317.3;
	5166	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1929.1;
	5167	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.7;
	5167	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	912.7;
	5167	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1016.8;
	5167	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3983.4;
	5167	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12827.8;
	5167	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6838.1;
	5167	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11597;
	5167	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1935.2;
	5168	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	995.3;
	5168	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	976;
	5168	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1072.3;
	5168	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4287;
	5168	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13800.4;
	5168	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7225.6;
	5168	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12417.9;
	5168	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2077.2;
	5169	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1039.4;
	5169	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1066;
	5169	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1151.8;
	5169	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4680.5;
	5169	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15099.7;
	5169	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7774.6;
	5169	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13381;
	5169	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2332.4;
	5170	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1084;
	5170	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1152.9;
	5170	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1237.6;
	5170	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5099.3;
	5170	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16562.5;
	5170	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8406.4;
	5170	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14374.3;
	5170	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2545.7;
	5171	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1135;
	5171	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1236.4;
	5171	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1322.5;
	5171	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5529.9;
	5171	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18049.9;
	5171	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9039.2;
	5171	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15391.2;
	5171	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2721.3;
	5172	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1173.5;
	5172	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1319.6;
	5172	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1399.2;
	5172	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5901.6;
	5172	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19224.4;
	5172	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9623.5;
	5172	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16261.8;
	5172	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2876.4;
	5173	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1211.4;
	5173	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1382;
	5173	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1470.8;
	5173	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6193.5;
	5173	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20250.4;
	5173	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10183.2;
	5173	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17035.1;
	5173	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3011.9;
	5174	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1243.8;
	5174	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1433.2;
	5174	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1535.6;
	5174	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6403.8;
	5174	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20970.1;
	5174	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10667.5;
	5174	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17624.3;
	5174	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3058.8;
	5175	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1255.1;
	5175	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1455.8;
	5175	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1579.1;
	5175	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6463.7;
	5175	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21401.1;
	5175	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10954.9;
	5175	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17709.9;
	5175	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3107.7;
	5176	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1257;
	5176	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1458.9;
	5176	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1598.3;
	5176	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6417.5;
	5176	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21504.8;
	5176	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11096.4;
	5176	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17624.9;
	5176	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3118.9;
	5177	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1258;
	5177	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1447.5;
	5177	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1597.2;
	5177	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6352.8;
	5177	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21464;
	5177	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11168;
	5177	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17485.4;
	5177	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3107.9;
	5178	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1254.1;
	5178	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1423.1;
	5178	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1575.7;
	5178	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6215.5;
	5178	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21166.2;
	5178	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11150.9;
	5178	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17051.7;
	5178	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3047.1;
	5179	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1231.4;
	5179	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1374.9;
	5179	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1523;
	5179	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5970;
	5179	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20324.1;
	5179	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10719.3;
	5179	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16593.6;
	5179	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2951.7;
	5180	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1186.9;
	5180	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1310.8;
	5180	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1452.4;
	5180	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5816.2;
	5180	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19445;
	5180	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10297.9;
	5180	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16143;
	5180	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2832.3;
	5181	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1158.9;
	5181	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1255.6;
	5181	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1400.2;
	5181	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5649.2;
	5181	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18602.1;
	5181	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9895.9;
	5181	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15538.9;
	5181	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2696.2;
	5182	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1117.9;
	5182	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1167.4;
	5182	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1303.4;
	5182	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5301;
	5182	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17131;
	5182	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9171.7;
	5182	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14577;
	5182	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2455.3;
	5183	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1060.9;
	5183	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1076.4;
	5183	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1195.7;
	5183	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4873.1;
	5183	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15591.7;
	5183	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8354.9;
	5183	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13463.3;
	5183	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2254.6;
	5184	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1022.9;
	5184	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1014.4;
	5184	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1114.2;
	5184	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4472.3;
	5184	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14346.7;
	5184	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7563.7;
	5184	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12338.9;
	5184	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2062.9;
	5185	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	993.1;
	5185	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	962.1;
	5185	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1056.1;
	5185	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4194.9;
	5185	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13448;
	5185	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6990.7;
	5185	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11460;
	5185	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1990.9;
	5186	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	968.7;
	5186	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	921.4;
	5186	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1011.8;
	5186	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3980.1;
	5186	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12765.6;
	5186	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6584.5;
	5186	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10919.5;
	5186	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1902;
	5187	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956;
	5187	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	896.7;
	5187	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	978.8;
	5187	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3834.1;
	5187	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12278.2;
	5187	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6304.8;
	5187	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10602.4;
	5187	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1841;
	5188	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.4;
	5188	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	880.4;
	5188	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	958.5;
	5188	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3756.7;
	5188	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12136.8;
	5188	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6180.6;
	5188	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10543.1;
	5188	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1844.9;
	5189	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.7;
	5189	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	889.9;
	5189	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	963.6;
	5189	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3805.1;
	5189	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12393.9;
	5189	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6336.3;
	5189	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10837.2;
	5189	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1866.8;
	5190	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.2;
	5190	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	907.8;
	5190	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	994.5;
	5190	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3953.2;
	5190	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12849.4;
	5190	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6685.3;
	5190	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11202.6;
	5190	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1890.3;
	5191	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.2;
	5191	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	932.4;
	5191	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1014.4;
	5191	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4001.9;
	5191	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13091.8;
	5191	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6863.2;
	5191	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11541.5;
	5191	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1976.2;
	5192	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.1;
	5192	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1008.3;
	5192	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1067.8;
	5192	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4306.2;
	5192	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13975.9;
	5192	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7258.9;
	5192	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12298.7;
	5192	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2130.9;
	5193	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1045.3;
	5193	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1101;
	5193	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1147.9;
	5193	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4748.5;
	5193	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15251.4;
	5193	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7824.5;
	5193	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13356.2;
	5193	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2316.6;
	5194	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1095.6;
	5194	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1195;
	5194	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1236.3;
	5194	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5167.2;
	5194	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16664.6;
	5194	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8487.6;
	5194	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14474.9;
	5194	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2519.8;
	5195	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1140.6;
	5195	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1278;
	5195	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1318.5;
	5195	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5594;
	5195	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18023.9;
	5195	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9113;
	5195	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15507.4;
	5195	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2695.1;
	5196	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1182.1;
	5196	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1347.8;
	5196	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1399.2;
	5196	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5954.1;
	5196	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19174.5;
	5196	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9704.5;
	5196	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16338.3;
	5196	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2898.7;
	5197	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1221.5;
	5197	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1409.3;
	5197	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1474.5;
	5197	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6228.9;
	5197	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20242.7;
	5197	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10282.5;
	5197	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17062.8;
	5197	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3007.6;
	5198	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1248;
	5198	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1448;
	5198	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1534.4;
	5198	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6402.7;
	5198	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20961.2;
	5198	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10740.1;
	5198	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17526.1;
	5198	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3099.2;
	5199	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1254.8;
	5199	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1473.6;
	5199	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1582.1;
	5199	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6473.1;
	5199	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21409.9;
	5199	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11079.1;
	5199	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17643.6;
	5199	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3043.6;
	5200	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1253.9;
	5200	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1472.5;
	5200	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1603.2;
	5200	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6429.1;
	5200	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21521.5;
	5200	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11119.2;
	5200	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17491.6;
	5200	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3028.3;
	5201	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1249.5;
	5201	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1460.5;
	5201	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1594.2;
	5201	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6335.8;
	5201	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21485.2;
	5201	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11161.6;
	5201	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17166.9;
	5201	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3046.2;
	5202	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1249.7;
	5202	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1435.5;
	5202	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1568.9;
	5202	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6208.7;
	5202	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21058;
	5202	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11066.6;
	5202	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16777.2;
	5202	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3014.1;
	5203	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1237.1;
	5203	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1380.9;
	5203	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1512.8;
	5203	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5975.2;
	5203	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20177;
	5203	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10698.8;
	5203	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16223.6;
	5203	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2874.9;
	5204	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1202.9;
	5204	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1325.5;
	5204	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1438.6;
	5204	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5819.3;
	5204	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19301.9;
	5204	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10307.5;
	5204	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15806.1;
	5204	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2815.5;
	5205	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1175.8;
	5205	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1278.8;
	5205	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1393;
	5205	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5675.2;
	5205	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18481.3;
	5205	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9972.1;
	5205	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15345.5;
	5205	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2662.9;
	5206	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1129.1;
	5206	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1184.5;
	5206	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1298.4;
	5206	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5337.6;
	5206	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17101.5;
	5206	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9232.7;
	5206	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14384;
	5206	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2469.6;
	5207	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1076;
	5207	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1089.5;
	5207	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1200;
	5207	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4930.9;
	5207	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15604.3;
	5207	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8361.8;
	5207	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13363.4;
	5207	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2305.1;
	5208	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1035.7;
	5208	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1017.1;
	5208	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1115.9;
	5208	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4587.1;
	5208	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14296.2;
	5208	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7780.2;
	5208	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12461.3;
	5208	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2110.6;
	5209	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1006.6;
	5209	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	970.6;
	5209	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1057.8;
	5209	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4318.1;
	5209	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13389.3;
	5209	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7194.6;
	5209	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11769.6;
	5209	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1983;
	5210	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	982;
	5210	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	934.2;
	5210	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1013.4;
	5210	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4102.3;
	5210	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12720.2;
	5210	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6745.4;
	5210	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11237.6;
	5210	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1911.8;
	5211	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970.7;
	5211	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	902.1;
	5211	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	982.4;
	5211	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3977.5;
	5211	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12242;
	5211	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6458.8;
	5211	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10951.7;
	5211	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1843.8;
	5212	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.6;
	5212	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	890.8;
	5212	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	963.1;
	5212	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3900.7;
	5212	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12167.5;
	5212	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6369.1;
	5212	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10918.1;
	5212	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1773.1;
	5213	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.6;
	5213	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	905.2;
	5213	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	969;
	5213	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3912.4;
	5213	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12407.3;
	5213	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6496.6;
	5213	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11169;
	5213	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1846.2;
	5214	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	977.8;
	5214	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	934.7;
	5214	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	998.3;
	5214	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4061;
	5214	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12819.7;
	5214	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6797.8;
	5214	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11507.4;
	5214	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1882.9;
	5215	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	984.3;
	5215	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	944.7;
	5215	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1015.4;
	5215	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4107;
	5215	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13047.5;
	5215	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6971.4;
	5215	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11754.9;
	5215	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1919.8;
	5216	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1001;
	5216	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	996.7;
	5216	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1077.3;
	5216	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4416.8;
	5216	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13753.7;
	5216	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7386.4;
	5216	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12643.1;
	5216	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2077.8;
	5217	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1045.6;
	5217	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1066.1;
	5217	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1155.1;
	5217	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4823.4;
	5217	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14898.2;
	5217	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7972.8;
	5217	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13718.1;
	5217	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2273;
	5218	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1092.5;
	5218	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1156.4;
	5218	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1216.2;
	5218	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5235.2;
	5218	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16329.5;
	5218	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8586.7;
	5218	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14770.6;
	5218	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2464.9;
	5219	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1135.5;
	5219	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1250.1;
	5219	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1324.9;
	5219	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5626.4;
	5219	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17772;
	5219	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9220.3;
	5219	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15624.2;
	5219	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2618.3;
	5220	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1168.8;
	5220	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1327.5;
	5220	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1406.7;
	5220	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5972.9;
	5220	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18957.7;
	5220	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9814.4;
	5220	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16334.1;
	5220	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2785.3;
	5221	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1204;
	5221	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1389.7;
	5221	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1478.5;
	5221	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6257;
	5221	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19999.8;
	5221	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10355.8;
	5221	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16977.1;
	5221	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2988.7;
	5222	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1237.1;
	5222	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1428;
	5222	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1531.3;
	5222	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6444.2;
	5222	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20783.7;
	5222	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10810;
	5222	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17430.6;
	5222	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3051.2;
	5223	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1244.2;
	5223	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1449.1;
	5223	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1575;
	5223	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6523;
	5223	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21218.7;
	5223	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11143.9;
	5223	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17624.3;
	5223	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3064.7;
	5224	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1248.5;
	5224	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1456;
	5224	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1587.6;
	5224	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6482;
	5224	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21392.5;
	5224	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11285.9;
	5224	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17500;
	5224	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3057.1;
	5225	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1245.8;
	5225	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1440.8;
	5225	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1574.5;
	5225	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6371.2;
	5225	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21260.3;
	5225	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11354.5;
	5225	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17292.9;
	5225	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3050.8;
	5226	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1233.5;
	5226	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1407.5;
	5226	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1539.2;
	5226	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6163;
	5226	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20731;
	5226	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11155.3;
	5226	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16727.3;
	5226	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2985.6;
	5227	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1211.8;
	5227	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1354.1;
	5227	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1486.2;
	5227	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5903.5;
	5227	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19803.4;
	5227	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10730.3;
	5227	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16101.4;
	5227	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2888.6;
	5228	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1181.2;
	5228	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1299.2;
	5228	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1421.1;
	5228	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5762;
	5228	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18920.9;
	5228	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10331.3;
	5228	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15701.9;
	5228	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2747.5;
	5229	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1160.8;
	5229	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1254;
	5229	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1373.4;
	5229	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5625.9;
	5229	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18101.2;
	5229	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9959.2;
	5229	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15192.6;
	5229	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2632.6;
	5230	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1120;
	5230	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1188.5;
	5230	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1284.8;
	5230	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5336.2;
	5230	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16813.2;
	5230	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9282.5;
	5230	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14345.4;
	5230	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2450.4;
	5231	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1073;
	5231	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1103.7;
	5231	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1196.6;
	5231	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5019.4;
	5231	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15495.6;
	5231	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8531.1;
	5231	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13488.8;
	5231	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2243;
	5232	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1036.4;
	5232	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1038.5;
	5232	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1117.6;
	5232	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4684.5;
	5232	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14290.8;
	5232	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7820.8;
	5232	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12643.6;
	5232	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2070.7;
	5233	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1005.5;
	5233	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	983.5;
	5233	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1056.8;
	5233	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4413.5;
	5233	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13367.2;
	5233	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7218.3;
	5233	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11979.1;
	5233	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1949.5;
	5234	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985.3;
	5234	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	943.5;
	5234	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1008.8;
	5234	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4205.1;
	5234	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12723;
	5234	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6797.8;
	5234	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11451.5;
	5234	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1932.4;
	5235	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970.3;
	5235	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	924.3;
	5235	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	974.3;
	5235	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4042.1;
	5235	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12308.5;
	5235	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6456.5;
	5235	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11054.8;
	5235	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1783;
	5236	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.5;
	5236	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	896.2;
	5236	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	945.2;
	5236	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3938.2;
	5236	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11986.7;
	5236	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6227.2;
	5236	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10791.9;
	5236	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1788.6;
	5237	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.5;
	5237	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	888.1;
	5237	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	932.2;
	5237	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3898.2;
	5237	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11884.3;
	5237	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6195.3;
	5237	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10725.4;
	5237	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1804.7;
	5238	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942;
	5238	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	889.1;
	5238	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	931.1;
	5238	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3890.6;
	5238	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11895.2;
	5238	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6293.8;
	5238	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10694.2;
	5238	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1802.1;
	5239	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.2;
	5239	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	893.5;
	5239	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940.6;
	5239	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3860.9;
	5239	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12068.2;
	5239	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6381.8;
	5239	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10789;
	5239	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1823;
	5240	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.3;
	5240	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	957.8;
	5240	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1009.1;
	5240	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4182.1;
	5240	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13098.4;
	5240	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6841.4;
	5240	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11740.2;
	5240	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1998.7;
	5241	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1018.5;
	5241	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1057;
	5241	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1098.2;
	5241	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4627.2;
	5241	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14633.1;
	5241	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7489.7;
	5241	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12938.3;
	5241	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2212;
	5242	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1069.2;
	5242	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1145.2;
	5242	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1187.2;
	5242	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5067.4;
	5242	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16160.9;
	5242	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8199;
	5242	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14044.4;
	5242	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2417.6;
	5243	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1116.4;
	5243	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1222.2;
	5243	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1276.7;
	5243	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5440.4;
	5243	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17584.1;
	5243	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8845.9;
	5243	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15040.7;
	5243	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2553.2;
	5244	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1153.4;
	5244	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1292;
	5244	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1358.1;
	5244	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5764.3;
	5244	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18650.2;
	5244	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9457.9;
	5244	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15744.4;
	5244	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2685.6;
	5245	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1184.1;
	5245	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1340.3;
	5245	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1424.3;
	5245	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6015.3;
	5245	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19465.6;
	5245	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9949.1;
	5245	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16300.5;
	5245	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2815.7;
	5246	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1202.7;
	5246	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1372.7;
	5246	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1472.3;
	5246	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6165.8;
	5246	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19972.2;
	5246	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10312;
	5246	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16646.6;
	5246	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2905.2;
	5247	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1219.8;
	5247	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1397.6;
	5247	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1510;
	5247	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6261.1;
	5247	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20325.6;
	5247	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10642.8;
	5247	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16894.6;
	5247	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2961.3;
	5248	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1226.4;
	5248	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1404.7;
	5248	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1531;
	5248	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6292.6;
	5248	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20564.4;
	5248	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10855.6;
	5248	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17017.1;
	5248	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2946.1;
	5249	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1229.2;
	5249	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1393;
	5249	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1537.6;
	5249	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6240.2;
	5249	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20585;
	5249	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10951.3;
	5249	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16953.7;
	5249	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2979.9;
	5250	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1222.8;
	5250	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1367;
	5250	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1514.1;
	5250	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6073.9;
	5250	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20182;
	5250	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10792.9;
	5250	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16581;
	5250	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2906.6;
	5251	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1197.9;
	5251	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1320.4;
	5251	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1461.5;
	5251	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5822.5;
	5251	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19355.7;
	5251	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10394.9;
	5251	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15992.5;
	5251	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2776.5;
	5252	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1165.8;
	5252	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1268.7;
	5252	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1401.4;
	5252	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5683.6;
	5252	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18519.6;
	5252	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9989.9;
	5252	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15580.8;
	5252	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2654;
	5253	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1145.9;
	5253	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1225.5;
	5253	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1357.2;
	5253	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5550.7;
	5253	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17792.2;
	5253	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9633.4;
	5253	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15097.6;
	5253	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2499.3;
	5254	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1107.1;
	5254	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1160.3;
	5254	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1268.2;
	5254	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5275.7;
	5254	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16549.5;
	5254	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9019.8;
	5254	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14294.1;
	5254	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2321.8;
	5255	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1064.8;
	5255	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1076.4;
	5255	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1182.2;
	5255	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4978;
	5255	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15332.7;
	5255	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8386.5;
	5255	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13493.9;
	5255	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2160.3;
	5256	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030.2;
	5256	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1007.8;
	5256	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1114.2;
	5256	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4685.6;
	5256	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14207.7;
	5256	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7728.3;
	5256	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12731.2;
	5256	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2037.1;
	5257	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1001.6;
	5257	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	958.6;
	5257	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1060.7;
	5257	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4430.2;
	5257	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13301.4;
	5257	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7177.4;
	5257	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12075.4;
	5257	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1903;
	5258	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.1;
	5258	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	922.8;
	5258	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1017;
	5258	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4219.3;
	5258	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12578.2;
	5258	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6737.5;
	5258	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11552.6;
	5258	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1798.2;
	5259	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.3;
	5259	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	901.8;
	5259	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	980.4;
	5259	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4046.2;
	5259	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12034.3;
	5259	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6438.9;
	5259	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11152.5;
	5259	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1748.6;
	5260	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951.5;
	5260	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	872.7;
	5260	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	954.1;
	5260	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3921.6;
	5260	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11623.1;
	5260	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6263.1;
	5260	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10844.8;
	5260	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1721.5;
	5261	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.3;
	5261	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	858.8;
	5261	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	943.4;
	5261	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3859.5;
	5261	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11448;
	5261	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6189.8;
	5261	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10686.7;
	5261	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1727.9;
	5262	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.6;
	5262	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	850.9;
	5262	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	942.4;
	5262	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3826.7;
	5262	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11325.2;
	5262	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6177;
	5262	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10587;
	5262	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1732;
	5263	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.4;
	5263	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	860.8;
	5263	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	949.4;
	5263	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3757.3;
	5263	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11369.1;
	5263	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6210.1;
	5263	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10483.3;
	5263	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1745.6;
	5264	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.2;
	5264	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	920.1;
	5264	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1013.1;
	5264	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4055.5;
	5264	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12394.1;
	5264	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6675.4;
	5264	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11268;
	5264	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1891;
	5265	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1011.1;
	5265	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1012.2;
	5265	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1100.2;
	5265	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4530.9;
	5265	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13814.3;
	5265	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7410.7;
	5265	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12432.8;
	5265	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2108.1;
	5266	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1058.2;
	5266	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1103.3;
	5266	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1185.9;
	5266	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4944.3;
	5266	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15320.4;
	5266	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8166.9;
	5266	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13673.2;
	5266	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2319.9;
	5267	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1100.6;
	5267	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1184.6;
	5267	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1269.7;
	5267	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5325.5;
	5267	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16789.4;
	5267	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8876;
	5267	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14690.8;
	5267	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2505.5;
	5268	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1142.7;
	5268	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1259.1;
	5268	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1349;
	5268	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5647.6;
	5268	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18053.6;
	5268	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9518.2;
	5268	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15525.8;
	5268	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2668.7;
	5269	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1174.2;
	5269	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1321.4;
	5269	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1423.3;
	5269	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5906.7;
	5269	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19081.5;
	5269	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10059.3;
	5269	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16212;
	5269	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2804.7;
	5270	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1200.2;
	5270	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1365;
	5270	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1485.6;
	5270	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6099.8;
	5270	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19813;
	5270	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10509.5;
	5270	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16705.5;
	5270	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2896.6;
	5271	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1221.5;
	5271	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1387.3;
	5271	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1523;
	5271	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6209.7;
	5271	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20271.1;
	5271	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10836.1;
	5271	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17004.9;
	5271	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2947.3;
	5272	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1234.3;
	5272	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1405;
	5272	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1544.7;
	5272	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6233.9;
	5272	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20543.2;
	5272	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11075.2;
	5272	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17160.6;
	5272	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2977;
	5273	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1238.5;
	5273	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1410;
	5273	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1556.7;
	5273	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6189.1;
	5273	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20645.8;
	5273	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11214.7;
	5273	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17104.9;
	5273	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2973.2;
	5274	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1230.2;
	5274	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1383.1;
	5274	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1541.7;
	5274	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6062.7;
	5274	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20335.1;
	5274	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11101.3;
	5274	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16767.1;
	5274	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2935.1;
	5275	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1211.9;
	5275	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1332.1;
	5275	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1498.9;
	5275	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5847.5;
	5275	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19610.9;
	5275	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10734;
	5275	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16199.7;
	5275	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2804.1;
	5276	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1184.1;
	5276	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1280.3;
	5276	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1437.9;
	5276	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5740.8;
	5276	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18889.5;
	5276	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10356.6;
	5276	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15844.1;
	5276	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2660.7;
	5277	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1164.2;
	5277	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1231.5;
	5277	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1382.3;
	5277	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5608.5;
	5277	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18161.2;
	5277	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9998;
	5277	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15286;
	5277	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2598.2;
	5278	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1120.6;
	5278	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1165.3;
	5278	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1288.1;
	5278	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5285.7;
	5278	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16842.9;
	5278	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9299.6;
	5278	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14367.6;
	5278	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2460.7;
	5279	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1071.6;
	5279	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1073.2;
	5279	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1190.5;
	5279	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4940.7;
	5279	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15476.7;
	5279	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8520.1;
	5279	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13385;
	5279	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2309.2;
	5280	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1032.2;
	5280	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1000.7;
	5280	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1141.7;
	5280	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4595.7;
	5280	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14238.3;
	5280	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7786.7;
	5280	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12535.4;
	5280	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2099.4;
	5281	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1006.8;
	5281	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	951.6;
	5281	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1087.3;
	5281	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4325.7;
	5281	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13355.1;
	5281	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7216;
	5281	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11863.7;
	5281	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1960.1;
	5282	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989.8;
	5282	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	911.9;
	5282	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1043.2;
	5282	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4131.8;
	5282	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12750.1;
	5282	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6808.2;
	5282	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11415.8;
	5282	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1834;
	5283	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	977.5;
	5283	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	893.8;
	5283	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1009.7;
	5283	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3988.4;
	5283	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12334.6;
	5283	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6531.3;
	5283	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11146.7;
	5283	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1820.5;
	5284	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.4;
	5284	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	877.7;
	5284	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	991.1;
	5284	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3911.2;
	5284	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12224.5;
	5284	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6481.4;
	5284	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11152.3;
	5284	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1819.9;
	5285	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	976.8;
	5285	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	889.7;
	5285	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	994.7;
	5285	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3949.6;
	5285	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12515.5;
	5285	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6691.8;
	5285	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11452.6;
	5285	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1843.5;
	5286	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	993.4;
	5286	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	918.1;
	5286	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1026.6;
	5286	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4116.8;
	5286	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13046.6;
	5286	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7079.4;
	5286	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11802.7;
	5286	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1945.7;
	5287	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1003;
	5287	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	935;
	5287	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1046;
	5287	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4169.7;
	5287	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13278.4;
	5287	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7184.2;
	5287	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11971.6;
	5287	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1994;
	5288	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1034.5;
	5288	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	985.3;
	5288	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1101.4;
	5288	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4492.9;
	5288	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14137.7;
	5288	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7550.7;
	5288	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12755.2;
	5288	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2112.3;
	5289	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1082.3;
	5289	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1044.3;
	5289	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1181.7;
	5289	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4934.1;
	5289	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15387.3;
	5289	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8141.1;
	5289	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13731.8;
	5289	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2326.3;
	5290	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1134.3;
	5290	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1111.7;
	5290	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1270.4;
	5290	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5376.2;
	5290	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16852.7;
	5290	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8871;
	5290	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14776;
	5290	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2527.1;
	5291	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1179;
	5291	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1189.4;
	5291	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1367.4;
	5291	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5816.5;
	5291	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18315.8;
	5291	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9576.7;
	5291	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15695.8;
	5291	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2722.3;
	5292	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1220.7;
	5292	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1250.3;
	5292	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1448.7;
	5292	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6172.4;
	5292	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19620.4;
	5292	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10226.1;
	5292	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16495.7;
	5292	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2880.5;
	5293	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1254.9;
	5293	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1300.7;
	5293	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1522.9;
	5293	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6468.5;
	5293	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20647.2;
	5293	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10781.3;
	5293	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17296.8;
	5293	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3006.2;
	5294	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1271.4;
	5294	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1344.3;
	5294	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1584.1;
	5294	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6691.3;
	5294	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21419.8;
	5294	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11184.4;
	5294	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17706.1;
	5294	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3115;
	5295	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1282.9;
	5295	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1369.7;
	5295	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1620.7;
	5295	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6754.4;
	5295	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21777.3;
	5295	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11425.8;
	5295	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17833;
	5295	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3150.7;
	5296	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1281;
	5296	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1362.4;
	5296	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1635.8;
	5296	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6702.4;
	5296	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21790.4;
	5296	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11550;
	5296	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17886.2;
	5296	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3108.3;
	5297	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1270.8;
	5297	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1341.2;
	5297	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1624.9;
	5297	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6572.5;
	5297	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21629;
	5297	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11653.4;
	5297	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17751.5;
	5297	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3023.4;
	5298	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1263.3;
	5298	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1286.7;
	5298	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1596.9;
	5298	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6408.9;
	5298	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21096.5;
	5298	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11598;
	5298	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17235;
	5298	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2965.7;
	5299	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1242.9;
	5299	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1218.2;
	5299	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1529.9;
	5299	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6174.3;
	5299	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20221.2;
	5299	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11203.4;
	5299	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16817.3;
	5299	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2864.8;
	5300	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1213.3;
	5300	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1175;
	5300	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1456.6;
	5300	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6045.1;
	5300	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19407.7;
	5300	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10781.6;
	5300	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16516.2;
	5300	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2781;
	5301	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1186.9;
	5301	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1124.7;
	5301	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1393.8;
	5301	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5871.6;
	5301	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18349.7;
	5301	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10318.8;
	5301	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15863.8;
	5301	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2674.2;
	5302	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1137;
	5302	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1059.3;
	5302	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1287.3;
	5302	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5504.6;
	5302	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16813.2;
	5302	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9512.9;
	5302	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14753.9;
	5302	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2473.4;
	5303	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1087.5;
	5303	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	982.2;
	5303	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1187.3;
	5303	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5101.2;
	5303	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15279.9;
	5303	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8629.9;
	5303	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13698.9;
	5303	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2287.5;
	5304	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1049;
	5304	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	912.7;
	5304	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1114.1;
	5304	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4707.3;
	5304	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14196.7;
	5304	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7869.9;
	5304	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12746.2;
	5304	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2118;
	5305	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1021.9;
	5305	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	876.1;
	5305	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1062;
	5305	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4459.9;
	5305	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13437.2;
	5305	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7295.2;
	5305	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12107.5;
	5305	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1934.3;
	5306	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1008.5;
	5306	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	854.6;
	5306	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1027.1;
	5306	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4276.2;
	5306	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12867.9;
	5306	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6873.2;
	5306	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11650.6;
	5306	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1854;
	5307	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	998.2;
	5307	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	846.5;
	5307	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1004.1;
	5307	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4134.5;
	5307	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12424.6;
	5307	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6630.9;
	5307	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11343.5;
	5307	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1849.3;
	5308	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	988.8;
	5308	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	835.2;
	5308	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	988;
	5308	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4043.5;
	5308	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12336;
	5308	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6579.3;
	5308	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11247.1;
	5308	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1839.2;
	5309	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	997;
	5309	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	853.8;
	5309	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1002.1;
	5309	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4109.9;
	5309	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12679.4;
	5309	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6786.6;
	5309	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11549.1;
	5309	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1829;
	5310	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1012.5;
	5310	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	897.3;
	5310	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1040.2;
	5310	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4285.5;
	5310	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13194.8;
	5310	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7182.5;
	5310	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11920.3;
	5310	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1950.1;
	5311	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1014;
	5311	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	919.1;
	5311	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1058.9;
	5311	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4313.2;
	5311	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13438.7;
	5311	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7306.7;
	5311	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12094.8;
	5311	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2017.8;
	5312	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1036.7;
	5312	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	954.1;
	5312	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1098.1;
	5312	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4583.2;
	5312	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14120.8;
	5312	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7609.5;
	5312	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12891.9;
	5312	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2194.4;
	5313	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1078.9;
	5313	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1008.8;
	5313	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1147.7;
	5313	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4970.1;
	5313	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15312.9;
	5313	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8196.4;
	5313	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13977.3;
	5313	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2354.2;
	5314	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1122.3;
	5314	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1073.4;
	5314	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1212.2;
	5314	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5408.5;
	5314	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16637.5;
	5314	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8886.9;
	5314	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15118.5;
	5314	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2591.2;
	5315	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1169.8;
	5315	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1125.7;
	5315	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1274.9;
	5315	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5817.6;
	5315	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17926.9;
	5315	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9552.7;
	5315	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16098.5;
	5315	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2759;
	5316	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1209.5;
	5316	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1172.7;
	5316	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1334.8;
	5316	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6175.9;
	5316	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19146.5;
	5316	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10192.1;
	5316	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16976.6;
	5316	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2932.3;
	5317	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1245.5;
	5317	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1217.8;
	5317	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1399.8;
	5317	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6472.4;
	5317	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20149.1;
	5317	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10759.3;
	5317	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17757.3;
	5317	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3082;
	5318	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1263.3;
	5318	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1256.9;
	5318	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1454.4;
	5318	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6652.6;
	5318	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20895.1;
	5318	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11255.3;
	5318	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	18123.6;
	5318	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3164.2;
	5319	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1244.7;
	5319	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1287;
	5319	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1492.5;
	5319	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6702.9;
	5319	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21252.7;
	5319	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11559.5;
	5319	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	18189.6;
	5319	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3187.6;
	5320	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1189.8;
	5320	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1310.3;
	5320	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1528.7;
	5320	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6697.7;
	5320	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21456.3;
	5320	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11711.2;
	5320	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	18184.2;
	5320	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3150.1;
	5321	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1127.3;
	5321	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1308.5;
	5321	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1525.3;
	5321	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6571.7;
	5321	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21408.4;
	5321	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11771.1;
	5321	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17972.9;
	5321	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3124.4;
	5322	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1094.5;
	5322	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1295;
	5322	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1499.4;
	5322	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6388.9;
	5322	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21020.5;
	5322	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11611;
	5322	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17622.1;
	5322	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2992.9;
	5323	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1065;
	5323	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1258.6;
	5323	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1439.2;
	5323	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6148.8;
	5323	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20175.8;
	5323	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11211.6;
	5323	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17098.5;
	5323	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2876.6;
	5324	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1052.7;
	5324	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1216.7;
	5324	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1384.5;
	5324	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6035.8;
	5324	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19319.3;
	5324	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10819;
	5324	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16719;
	5324	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2850.5;
	5325	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1043.6;
	5325	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1177.6;
	5325	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1338.3;
	5325	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5876.1;
	5325	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18531;
	5325	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10392.2;
	5325	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16097.7;
	5325	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2709.1;
	5326	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009.1;
	5326	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1107.7;
	5326	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1248.3;
	5326	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5512.8;
	5326	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17166.1;
	5326	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9635.5;
	5326	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15049.5;
	5326	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2522.5;
	5327	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981;
	5327	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1021.7;
	5327	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1162.4;
	5327	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5136.3;
	5327	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15737.8;
	5327	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8840.7;
	5327	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13950;
	5327	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2331.2;
	5328	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.8;
	5328	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	957.7;
	5328	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1091.9;
	5328	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4767.4;
	5328	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14542.6;
	5328	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8092.6;
	5328	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12944.1;
	5328	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2140.6;
	5329	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.1;
	5329	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	913.2;
	5329	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1043.6;
	5329	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4497.6;
	5329	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13693.1;
	5329	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7557.3;
	5329	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12253.7;
	5329	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2044.7;
	5330	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.7;
	5330	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	884.1;
	5330	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1002.8;
	5330	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4303.5;
	5330	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13100.4;
	5330	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7189.9;
	5330	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11773.2;
	5330	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1976.3;
	5331	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.2;
	5331	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	863.6;
	5331	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	973.2;
	5331	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4149;
	5331	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12753.2;
	5331	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6954.7;
	5331	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11465;
	5331	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1888.9;
	5332	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.7;
	5332	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	849.6;
	5332	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	956.7;
	5332	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4063.3;
	5332	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12607.6;
	5332	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6873.7;
	5332	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11433.8;
	5332	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1894.4;
	5333	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.3;
	5333	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	862.5;
	5333	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	967.9;
	5333	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4101.5;
	5333	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12925.4;
	5333	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7045.3;
	5333	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11764.2;
	5333	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1885.6;
	5334	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948.3;
	5334	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	894.8;
	5334	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1010.8;
	5334	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4269.7;
	5334	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13479.8;
	5334	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7396.2;
	5334	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12159.9;
	5334	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2000.6;
	5335	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.9;
	5335	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	913.7;
	5335	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1036.7;
	5335	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4314.2;
	5335	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13669.6;
	5335	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7469.9;
	5335	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12322.4;
	5335	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2004.1;
	5336	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	988.4;
	5336	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	975.7;
	5336	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1088.2;
	5336	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4594.1;
	5336	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14551.5;
	5336	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7780.8;
	5336	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13035.8;
	5336	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2126.8;
	5337	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1022;
	5337	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1062.3;
	5337	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1168;
	5337	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4991.9;
	5337	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15793.5;
	5337	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8333.5;
	5337	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14091.6;
	5337	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2390.1;
	5338	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1063.4;
	5338	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1154.3;
	5338	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1256.9;
	5338	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5448;
	5338	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17246.2;
	5338	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9010.4;
	5338	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15142.7;
	5338	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2600.5;
	5339	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1109.8;
	5339	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1235.7;
	5339	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1341.4;
	5339	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5874.1;
	5339	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18615.4;
	5339	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9687.9;
	5339	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16131;
	5339	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2758;
	5340	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1146.6;
	5340	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1318.7;
	5340	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1419.4;
	5340	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6243.8;
	5340	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19803.8;
	5340	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10344.2;
	5340	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16970.7;
	5340	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2938.1;
	5341	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1179;
	5341	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1380.7;
	5341	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1496.4;
	5341	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6527.5;
	5341	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20850.1;
	5341	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10967.4;
	5341	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17645.7;
	5341	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3074.8;
	5342	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1201.5;
	5342	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1426;
	5342	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1561.6;
	5342	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6711.1;
	5342	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21536.5;
	5342	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11428.1;
	5342	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	18093;
	5342	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3141.2;
	5343	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1211.9;
	5343	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1443.7;
	5343	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1594.8;
	5343	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6724.7;
	5343	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21857.5;
	5343	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11651.4;
	5343	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	18098.6;
	5343	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3185;
	5344	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1213.5;
	5344	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1435.7;
	5344	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1608;
	5344	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6669.4;
	5344	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21898.3;
	5344	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11737.4;
	5344	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	18056.7;
	5344	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3187.4;
	5345	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1203.3;
	5345	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1416.7;
	5345	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1606.9;
	5345	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6559.2;
	5345	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21765.2;
	5345	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11889.4;
	5345	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17853.7;
	5345	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3147.6;
	5346	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1186;
	5346	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1384.3;
	5346	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1578.3;
	5346	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6406.1;
	5346	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21385.1;
	5346	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11791.8;
	5346	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17492.4;
	5346	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3076.6;
	5347	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1169.3;
	5347	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1329.1;
	5347	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1519.5;
	5347	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6171.8;
	5347	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20568.3;
	5347	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11381.3;
	5347	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17006.5;
	5347	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2990.5;
	5348	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1147.3;
	5348	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1274.8;
	5348	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1453.1;
	5348	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6071.8;
	5348	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19769.8;
	5348	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11041.2;
	5348	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16706.7;
	5348	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2886.8;
	5349	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1130.2;
	5349	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1224.2;
	5349	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1400.2;
	5349	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5898.2;
	5349	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18893.9;
	5349	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10555.9;
	5349	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16053.6;
	5349	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2752.7;
	5350	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1097.8;
	5350	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1152.7;
	5350	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1302.5;
	5350	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5555.5;
	5350	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17451.2;
	5350	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9737.5;
	5350	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15127.4;
	5350	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2528.5;
	5351	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1052.9;
	5351	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1070.1;
	5351	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1210.5;
	5351	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5192.2;
	5351	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15997.3;
	5351	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8908.3;
	5351	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14157.5;
	5351	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2334.9;
	5352	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1012.6;
	5352	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	992.2;
	5352	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1124.6;
	5352	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4828.6;
	5352	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14897.8;
	5352	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8070.3;
	5352	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13274.7;
	5352	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2161.7;
	5353	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	983.5;
	5353	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	943.8;
	5353	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1064.2;
	5353	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4559.9;
	5353	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13983.6;
	5353	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7553.3;
	5353	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12587.4;
	5353	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2093.7;
	5354	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.6;
	5354	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	903.5;
	5354	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1020.9;
	5354	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4371.4;
	5354	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13319.2;
	5354	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7119.7;
	5354	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12111.6;
	5354	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2009.3;
	5355	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.6;
	5355	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	884.6;
	5355	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	992;
	5355	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4248.6;
	5355	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12861.7;
	5355	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6863.8;
	5355	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11827.9;
	5355	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1941.4;
	5356	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.9;
	5356	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	866.8;
	5356	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	974.8;
	5356	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4181.4;
	5356	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12726.6;
	5356	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6783.5;
	5356	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11764.5;
	5356	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1951.9;
	5357	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.9;
	5357	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	877.9;
	5357	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	985.3;
	5357	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4244.2;
	5357	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13033.9;
	5357	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6988.5;
	5357	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12060.4;
	5357	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1995.3;
	5358	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969.2;
	5358	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	907.6;
	5358	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1026.7;
	5358	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4434.1;
	5358	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13590.4;
	5358	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7318.5;
	5358	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12432.3;
	5358	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2025.7;
	5359	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	979.8;
	5359	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	929.5;
	5359	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1046.6;
	5359	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4472.7;
	5359	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13751;
	5359	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7410;
	5359	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12551.3;
	5359	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2103.1;
	5360	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1007.6;
	5360	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1003.2;
	5360	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1093.3;
	5360	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4706.2;
	5360	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14543.9;
	5360	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7707.7;
	5360	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13167.6;
	5360	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2244.6;
	5361	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1054.2;
	5361	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1096.3;
	5361	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1176.6;
	5361	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5095.7;
	5361	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15801.4;
	5361	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8241.9;
	5361	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14188.7;
	5361	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2420;
	5362	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1103.3;
	5362	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1189.5;
	5362	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1269.4;
	5362	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5527;
	5362	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17186;
	5362	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8863.4;
	5362	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15262.5;
	5362	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2599.5;
	5363	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1148.3;
	5363	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1281;
	5363	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1364.1;
	5363	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5925.2;
	5363	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18610.1;
	5363	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9509;
	5363	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16280.3;
	5363	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2770.4;
	5364	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1184.9;
	5364	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1356.4;
	5364	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1449;
	5364	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6266.8;
	5364	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19825.5;
	5364	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10167.9;
	5364	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16969.4;
	5364	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2975.5;
	5365	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1226.4;
	5365	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1422;
	5365	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1529;
	5365	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6557.1;
	5365	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20929.3;
	5365	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10801.1;
	5365	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17658.5;
	5365	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3098.7;
	5366	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1255;
	5366	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1442.8;
	5366	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1588.2;
	5366	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6748.8;
	5366	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21722.7;
	5366	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11362.6;
	5366	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	18169.7;
	5366	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3183.7;
	5367	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1257.6;
	5367	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1435.9;
	5367	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1626.6;
	5367	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6786.1;
	5367	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	22141.7;
	5367	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11663.6;
	5367	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	18152.3;
	5367	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3124.9;
	5368	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1255.6;
	5368	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1441.1;
	5368	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1640.8;
	5368	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6724.7;
	5368	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	22264;
	5368	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11758.8;
	5368	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	18058.2;
	5368	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3132.5;
	5369	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1244.9;
	5369	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1443.9;
	5369	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1630;
	5369	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6586.1;
	5369	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	22217.2;
	5369	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11903.4;
	5369	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17801.5;
	5369	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3133;
	5370	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1238.3;
	5370	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1419;
	5370	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1600.5;
	5370	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6388.2;
	5370	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21682.9;
	5370	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11855.9;
	5370	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17430.6;
	5370	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3083.3;
	5371	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1219.4;
	5371	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1370.2;
	5371	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1534.7;
	5371	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6157.8;
	5371	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20788.8;
	5371	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11441;
	5371	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17047;
	5371	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2957.9;
	5372	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1179.5;
	5372	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1319.8;
	5372	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1465;
	5372	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6044.2;
	5372	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19919.9;
	5372	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11041.6;
	5372	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16633.5;
	5372	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2905.5;
	5373	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1146.6;
	5373	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1270.9;
	5373	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1408.4;
	5373	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5850.3;
	5373	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18993.4;
	5373	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10573.9;
	5373	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16059.3;
	5373	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2747;
	5374	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1096.6;
	5374	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1190;
	5374	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1309.2;
	5374	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5487.2;
	5374	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17524.1;
	5374	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9769;
	5374	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15087.6;
	5374	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2553.6;
	5375	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1049.8;
	5375	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1099.1;
	5375	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1213.5;
	5375	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5105.6;
	5375	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16026.5;
	5375	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8923.4;
	5375	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14116.4;
	5375	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2392.7;
	5376	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009.4;
	5376	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1031.3;
	5376	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1131.5;
	5376	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4747.6;
	5376	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14692.2;
	5376	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8228.9;
	5376	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13241.3;
	5376	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2186.4;
	5377	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	983.2;
	5377	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	976;
	5377	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1069.5;
	5377	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4482.3;
	5377	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13768.3;
	5377	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7643.8;
	5377	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12545.8;
	5377	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2077.4;
	5378	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.9;
	5378	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	942.3;
	5378	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1030.8;
	5378	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4295.3;
	5378	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13105.5;
	5378	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7193.1;
	5378	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11994.2;
	5378	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2013.2;
	5379	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.3;
	5379	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	928.1;
	5379	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1003.4;
	5379	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4184.1;
	5379	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12665.5;
	5379	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6906.7;
	5379	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11691.3;
	5379	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1958.6;
	5380	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950;
	5380	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	913;
	5380	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	971.6;
	5380	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4106.4;
	5380	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12583.5;
	5380	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6781.7;
	5380	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11552.3;
	5380	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1878.9;
	5381	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.4;
	5381	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	905.2;
	5381	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	974.4;
	5381	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4144.7;
	5381	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12942.9;
	5381	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6925.8;
	5381	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11772.1;
	5381	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1958.1;
	5382	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.9;
	5382	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	932.6;
	5382	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1015.6;
	5382	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4310.2;
	5382	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13504.4;
	5382	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7301.4;
	5382	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12162.9;
	5382	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2011.2;
	5383	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989.4;
	5383	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	950.5;
	5383	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1030.5;
	5383	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4323.7;
	5383	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13729.8;
	5383	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7353.8;
	5383	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12232.2;
	5383	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2018.8;
	5384	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1011.3;
	5384	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1014.9;
	5384	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1086.8;
	5384	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4547.2;
	5384	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14577.1;
	5384	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7595.1;
	5384	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12847.2;
	5384	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2153.1;
	5385	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1053.9;
	5385	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1098.6;
	5385	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1178.1;
	5385	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4923.3;
	5385	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15859.3;
	5385	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8072.3;
	5385	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13733.3;
	5385	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2321.5;
	5386	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1101.4;
	5386	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1190.3;
	5386	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1262.1;
	5386	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5357.6;
	5386	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17390.5;
	5386	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8748.9;
	5386	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14725.1;
	5386	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2507.6;
	5387	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1146.1;
	5387	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1277.6;
	5387	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1386.6;
	5387	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5766.2;
	5387	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18886.4;
	5387	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9497;
	5387	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15583.5;
	5387	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2678.2;
	5388	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1183.2;
	5388	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1353.8;
	5388	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1484.3;
	5388	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6151.4;
	5388	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20258.5;
	5388	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10275.3;
	5388	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16375.9;
	5388	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2858.6;
	5389	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1205.2;
	5389	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1414.1;
	5389	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1555.5;
	5389	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6463.7;
	5389	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21382.2;
	5389	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11050.1;
	5389	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17067.6;
	5389	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3076.7;
	5390	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1184.4;
	5390	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1435.5;
	5390	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1567.6;
	5390	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6660.6;
	5390	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	22017.6;
	5390	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11727.3;
	5390	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17487.7;
	5390	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3134;
	5391	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1145.8;
	5391	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1425.9;
	5391	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1534.6;
	5391	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6734.5;
	5391	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21962.8;
	5391	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	12104.4;
	5391	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17628.9;
	5391	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3165.3;
	5392	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1115.8;
	5392	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1355.4;
	5392	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1484.4;
	5392	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6665.8;
	5392	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	21122.8;
	5392	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	12261;
	5392	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17615.9;
	5392	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3148.8;
	5393	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1084.8;
	5393	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1287.7;
	5393	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1425.9;
	5393	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6471.2;
	5393	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19600.9;
	5393	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	12227.1;
	5393	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17344.3;
	5393	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	3096.6;
	5394	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1060.6;
	5394	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1217.9;
	5394	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1356.8;
	5394	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6209.7;
	5394	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18202.1;
	5394	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11952.4;
	5394	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16784.3;
	5394	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2998.4;
	5395	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1042.4;
	5395	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1157.4;
	5395	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1276.5;
	5395	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5931.8;
	5395	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17012.6;
	5395	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11379.6;
	5395	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16021;
	5395	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2868.7;
	5396	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1036.2;
	5396	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1124.1;
	5396	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1241.9;
	5396	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5818.9;
	5396	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16399.1;
	5396	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10844.4;
	5396	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15703.3;
	5396	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2747.7;
	5397	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030.5;
	5397	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1094;
	5397	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1215.1;
	5397	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5628.8;
	5397	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15676.9;
	5397	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10242.2;
	5397	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15066.7;
	5397	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2617;
	5398	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009.3;
	5398	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1041.6;
	5398	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1146.8;
	5398	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5338.2;
	5398	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14631.2;
	5398	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9384.6;
	5398	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14189;
	5398	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2436;
	5399	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	982.3;
	5399	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	969.9;
	5399	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1074.3;
	5399	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5011.7;
	5399	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13565.1;
	5399	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8569.9;
	5399	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13289.4;
	5399	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2216.3;
	5400	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.2;
	5400	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	922.1;
	5400	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1015.5;
	5400	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4678.1;
	5400	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12637.9;
	5400	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7849.8;
	5400	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12452;
	5400	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2032.8;
	5401	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.7;
	5401	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	884.8;
	5401	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	961.7;
	5401	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4427.9;
	5401	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11934.2;
	5401	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7297.3;
	5401	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11896.7;
	5401	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1921.9;
	5402	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925.3;
	5402	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	853.9;
	5402	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	931;
	5402	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4224.6;
	5402	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11391.8;
	5402	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6886.8;
	5402	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11406.8;
	5402	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1908.5;
	5403	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.3;
	5403	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	845.9;
	5403	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	914.5;
	5403	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4079.4;
	5403	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11053.7;
	5403	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6591.7;
	5403	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11127.6;
	5403	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1760.2;
	5404	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.2;
	5404	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	827.1;
	5404	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	910.9;
	5404	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3992;
	5404	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10849.8;
	5404	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6421.1;
	5404	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10956.7;
	5404	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1784;
	5405	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.4;
	5405	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	833.9;
	5405	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	915.7;
	5405	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3978.4;
	5405	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10937.9;
	5405	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6359.7;
	5405	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11020.1;
	5405	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1822.4;
	5406	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921;
	5406	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	853.5;
	5406	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	917.9;
	5406	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4019.6;
	5406	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11207.8;
	5406	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6464.1;
	5406	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11165.6;
	5406	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1831.5;
	5407	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.9;
	5407	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	854.1;
	5407	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	915;
	5407	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3986.2;
	5407	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11399.1;
	5407	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6541.3;
	5407	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11206.3;
	5407	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1836.3;
	5408	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937;
	5408	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	888.3;
	5408	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	950.7;
	5408	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4190.7;
	5408	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11877;
	5408	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6829.2;
	5408	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11516.3;
	5408	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1932.6;
	5409	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.8;
	5409	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	933.3;
	5409	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	995.7;
	5409	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4492.8;
	5409	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12549.6;
	5409	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7180.6;
	5409	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12115.9;
	5409	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2061.7;
	5410	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1004;
	5410	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	985.1;
	5410	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1043.3;
	5410	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4857.3;
	5410	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13366.3;
	5410	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7493.6;
	5410	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12949.3;
	5410	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2200.9;
	5411	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1037.7;
	5411	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1035.5;
	5411	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1092.1;
	5411	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5191.7;
	5411	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14195.8;
	5411	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7875.8;
	5411	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13840.4;
	5411	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2310.1;
	5412	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1065.5;
	5412	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1093.4;
	5412	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1138.8;
	5412	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5492.3;
	5412	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15031.7;
	5412	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8177.8;
	5412	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14806.1;
	5412	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2454.5;
	5413	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1086;
	5413	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1141.9;
	5413	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1177.7;
	5413	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5736.2;
	5413	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15914.9;
	5413	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8376.1;
	5413	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15598.6;
	5413	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2576.2;
	5414	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1100.1;
	5414	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1185.3;
	5414	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1209.6;
	5414	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5916.7;
	5414	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16674;
	5414	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8542.5;
	5414	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16086.2;
	5414	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2623.6;
	5415	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1099.9;
	5415	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1218.4;
	5415	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1230.7;
	5415	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6004.4;
	5415	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17108.1;
	5415	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8641.5;
	5415	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16272.8;
	5415	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2641.9;
	5416	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1087.4;
	5416	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1233.3;
	5416	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1234.1;
	5416	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6047.8;
	5416	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17412.1;
	5416	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8625.4;
	5416	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16274.5;
	5416	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2585.5;
	5417	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1076.7;
	5417	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1230.2;
	5417	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1227.9;
	5417	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5987.7;
	5417	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17478.7;
	5417	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8667.8;
	5417	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16015.5;
	5417	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2524.4;
	5418	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1058.4;
	5418	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1196.6;
	5418	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1206;
	5418	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5774.8;
	5418	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17106.8;
	5418	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8540.9;
	5418	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14990.2;
	5418	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2385.2;
	5419	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1041.9;
	5419	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1143.4;
	5419	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1170.2;
	5419	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5530.7;
	5419	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16431.4;
	5419	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8280.6;
	5419	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14058.4;
	5419	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2312.9;
	5420	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1029.6;
	5420	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1097.8;
	5420	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1139.1;
	5420	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5456.8;
	5420	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15824.3;
	5420	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8208;
	5420	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13391.5;
	5420	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2236.2;
	5421	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1025.6;
	5421	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1056.9;
	5421	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1112.2;
	5421	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5297.9;
	5421	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15208.6;
	5421	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8009.5;
	5421	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12739.8;
	5421	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2101.5;
	5422	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000.6;
	5422	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1007.2;
	5422	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1057.5;
	5422	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5052.5;
	5422	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14261;
	5422	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7589.3;
	5422	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12114.3;
	5422	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1986.6;
	5423	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	973.7;
	5423	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	936.3;
	5423	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1000.7;
	5423	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4768.7;
	5423	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13238.4;
	5423	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7041.3;
	5423	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11431.2;
	5423	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1865.8;
	5424	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948.7;
	5424	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	868.5;
	5424	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	957.7;
	5424	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4497.1;
	5424	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12306.2;
	5424	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6565.2;
	5424	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10820.3;
	5424	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1769.9;
	5425	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.6;
	5425	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	824.8;
	5425	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	914;
	5425	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4281.6;
	5425	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11538.1;
	5425	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6196;
	5425	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10384.2;
	5425	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1681.6;
	5426	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.3;
	5426	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	793.3;
	5426	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	883.2;
	5426	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4111.8;
	5426	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10958.3;
	5426	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5933.2;
	5426	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10043.4;
	5426	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1612.1;
	5427	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893;
	5427	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	773.1;
	5427	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	856;
	5427	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3972.9;
	5427	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10516.9;
	5427	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5755.7;
	5427	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9806.4;
	5427	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1595.1;
	5428	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.1;
	5428	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.6;
	5428	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	845;
	5428	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3865.9;
	5428	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10207.4;
	5428	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5666.2;
	5428	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9676.4;
	5428	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1592;
	5429	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.3;
	5429	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.3;
	5429	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.3;
	5429	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3815.8;
	5429	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10067;
	5429	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5648.1;
	5429	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9671.7;
	5429	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1620.7;
	5430	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.5;
	5430	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	756.2;
	5430	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851.1;
	5430	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3821.7;
	5430	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10023.1;
	5430	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5703.6;
	5430	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9728.6;
	5430	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1650;
	5431	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.8;
	5431	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	757.7;
	5431	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	858.6;
	5431	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3751.1;
	5431	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10038;
	5431	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5772.6;
	5431	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9701.5;
	5431	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1660;
	5432	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.1;
	5432	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	805;
	5432	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	907;
	5432	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3838.4;
	5432	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10887.5;
	5432	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6083.4;
	5432	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10001.6;
	5432	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1715.1;
	5433	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.2;
	5433	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	860.8;
	5433	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	956.5;
	5433	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4065.6;
	5433	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12025.1;
	5433	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6445.5;
	5433	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10407.7;
	5433	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1807.4;
	5434	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.4;
	5434	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	900.7;
	5434	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1014.4;
	5434	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4353.3;
	5434	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13131.3;
	5434	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6918.6;
	5434	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10996.1;
	5434	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1884.8;
	5435	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.4;
	5435	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	942.7;
	5435	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1073.7;
	5435	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4652.5;
	5435	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14023.4;
	5435	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7397;
	5435	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11651.9;
	5435	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1943.1;
	5436	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985.7;
	5436	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	981.1;
	5436	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1121.5;
	5436	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4892.4;
	5436	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14635;
	5436	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7681.3;
	5436	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12385.2;
	5436	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1999;
	5437	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1003.9;
	5437	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1014.4;
	5437	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1163.7;
	5437	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5049.6;
	5437	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14968.8;
	5437	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7690.8;
	5437	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12966.7;
	5437	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2056.2;
	5438	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1017.2;
	5438	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1033.2;
	5438	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1196.5;
	5438	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5057.2;
	5438	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15294.2;
	5438	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7640.3;
	5438	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13241.7;
	5438	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2133.3;
	5439	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1035.5;
	5439	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1056.5;
	5439	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1214.2;
	5439	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4976.5;
	5439	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15747;
	5439	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7568.4;
	5439	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13428.2;
	5439	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2177.4;
	5440	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1041.7;
	5440	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1074.8;
	5440	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1222.6;
	5440	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4872.8;
	5440	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16174.8;
	5440	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7405.9;
	5440	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13248.4;
	5440	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2171.5;
	5441	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1032.7;
	5441	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1079.4;
	5441	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1214.3;
	5441	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4751.1;
	5441	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16344;
	5441	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7311.9;
	5441	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12885;
	5441	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2145.3;
	5442	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1017.2;
	5442	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1055.4;
	5442	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1184;
	5442	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4593.7;
	5442	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16098.4;
	5442	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7225.6;
	5442	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12409.3;
	5442	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2101.1;
	5443	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1002.2;
	5443	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1022.1;
	5443	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1142.7;
	5443	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4477.5;
	5443	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15542.9;
	5443	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7195.1;
	5443	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12078;
	5443	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2020.4;
	5444	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	997.7;
	5444	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1009;
	5444	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1124;
	5444	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4536.4;
	5444	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15238.3;
	5444	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7230.6;
	5444	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12113.9;
	5444	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1960.7;
	5445	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	998.3;
	5445	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	985.2;
	5445	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1100.3;
	5445	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4472.3;
	5445	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14758.8;
	5445	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7027.4;
	5445	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11841.4;
	5445	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1946.9;
	5446	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	973.5;
	5446	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	946.1;
	5446	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1042;
	5446	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4258.7;
	5446	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13710.2;
	5446	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6590.4;
	5446	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11276.4;
	5446	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1860.4;
	5447	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.8;
	5447	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	874.1;
	5447	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	968.3;
	5447	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3960.7;
	5447	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12539.1;
	5447	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6050;
	5447	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10618.6;
	5447	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1747.2;
	5448	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909;
	5448	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	812.6;
	5448	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	927.4;
	5448	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3695.8;
	5448	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11558.8;
	5448	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5643;
	5448	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10100.4;
	5448	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1625.2;
	5449	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892;
	5449	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	783.2;
	5449	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	890.5;
	5449	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3530.8;
	5449	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10892.7;
	5449	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5345.4;
	5449	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9774.6;
	5449	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1542.4;
	5450	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	881.5;
	5450	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	765.8;
	5450	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.9;
	5450	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3423.3;
	5450	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10484.6;
	5450	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5159.2;
	5450	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9636.8;
	5450	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1467.2;
	5451	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.3;
	5451	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	761;
	5451	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.8;
	5451	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3344.7;
	5451	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10243;
	5451	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5069.9;
	5451	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9566.9;
	5451	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1500.5;
	5452	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872;
	5452	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	754.9;
	5452	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	848.4;
	5452	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3356.6;
	5452	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10292.5;
	5452	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5144.2;
	5452	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9754.4;
	5452	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1532.9;
	5453	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.8;
	5453	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	781.6;
	5453	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	864.1;
	5453	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3454.7;
	5453	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10738.3;
	5453	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5438.8;
	5453	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10191;
	5453	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1591.5;
	5454	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.5;
	5454	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	816.1;
	5454	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	909.5;
	5454	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3680.8;
	5454	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11463.6;
	5454	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5899.6;
	5454	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10727.2;
	5454	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1728.9;
	5455	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.7;
	5455	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	845.2;
	5455	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940.5;
	5455	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3744.4;
	5455	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11768.8;
	5455	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6110.5;
	5455	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10915.2;
	5455	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1797.3;
	5456	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.6;
	5456	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	867.6;
	5456	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	967.3;
	5456	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3898.8;
	5456	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12009.1;
	5456	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6226.8;
	5456	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11388.9;
	5456	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1831.8;
	5457	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.8;
	5457	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	894.1;
	5457	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1020.6;
	5457	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4205;
	5457	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12399.8;
	5457	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6377.9;
	5457	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12029.7;
	5457	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1893.6;
	5458	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1006;
	5458	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	929.7;
	5458	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1066.7;
	5458	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4550.5;
	5458	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12812.9;
	5458	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6578.7;
	5458	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12387.1;
	5458	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1957.2;
	5459	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1034.1;
	5459	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	966.4;
	5459	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1110.2;
	5459	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4918.5;
	5459	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13052.7;
	5459	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6706.6;
	5459	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12348.1;
	5459	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2007;
	5460	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1061.9;
	5460	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1001.9;
	5460	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1148.4;
	5460	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5160.9;
	5460	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13181.8;
	5460	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6781.5;
	5460	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12179;
	5460	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2018.1;
	5461	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1089.5;
	5461	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1033.9;
	5461	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1191.9;
	5461	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5334.9;
	5461	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13239.6;
	5461	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6873.8;
	5461	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12175.8;
	5461	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2031.2;
	5462	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1104.2;
	5462	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1063.5;
	5462	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1230.3;
	5462	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5520.3;
	5462	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13310.2;
	5462	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6920.2;
	5462	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12418.3;
	5462	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2050.4;
	5463	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1126.9;
	5463	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1089.3;
	5463	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1258.8;
	5463	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5603.3;
	5463	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13318.5;
	5463	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6965.7;
	5463	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12972.5;
	5463	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2055.7;
	5464	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1142.5;
	5464	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1087.9;
	5464	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1273.3;
	5464	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5572.5;
	5464	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13279.8;
	5464	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7023.8;
	5464	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13645;
	5464	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2058.3;
	5465	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1140.9;
	5465	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1081.4;
	5465	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1261.9;
	5465	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5425.5;
	5465	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13214.2;
	5465	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7066.5;
	5465	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14002.7;
	5465	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2018.3;
	5466	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1135.3;
	5466	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1071;
	5466	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1238.2;
	5466	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5225.9;
	5466	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12956;
	5466	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7036.9;
	5466	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13791.4;
	5466	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1994.1;
	5467	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1113.5;
	5467	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1036.3;
	5467	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1186.4;
	5467	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5039;
	5467	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12710.8;
	5467	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6924;
	5467	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13334.4;
	5467	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1976;
	5468	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1090.9;
	5468	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1013.4;
	5468	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1142.1;
	5468	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5045.9;
	5468	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12746.6;
	5468	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7040.3;
	5468	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13216;
	5468	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2005.6;
	5469	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1072.1;
	5469	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	982.2;
	5469	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1112.2;
	5469	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4906.4;
	5469	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12493.2;
	5469	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6946;
	5469	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12724.9;
	5469	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1970.6;
	5470	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030;
	5470	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	913.1;
	5470	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1036.7;
	5470	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4602.2;
	5470	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11628.6;
	5470	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6542.9;
	5470	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11895.8;
	5470	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1857.3;
	5471	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986.9;
	5471	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	847;
	5471	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	953.4;
	5471	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4266.1;
	5471	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10575.8;
	5471	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5985.4;
	5471	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11126.9;
	5471	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1736.7;
	5472	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.9;
	5472	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	783.9;
	5472	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	890.2;
	5472	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4003.9;
	5472	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9717.1;
	5472	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5517.8;
	5472	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10496.4;
	5472	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1656.5;
	5473	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.1;
	5473	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746.9;
	5473	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	848;
	5473	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3794.1;
	5473	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9269.7;
	5473	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5230;
	5473	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10078.7;
	5473	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1517.2;
	5474	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.2;
	5474	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720.9;
	5474	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	821;
	5474	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3627.1;
	5474	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8992.8;
	5474	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5055.9;
	5474	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9810.7;
	5474	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1472.7;
	5475	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908;
	5475	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	700.4;
	5475	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	810.7;
	5475	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3527.4;
	5475	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8830.6;
	5475	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4995.1;
	5475	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9679.3;
	5475	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1506.1;
	5476	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.4;
	5476	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	691.4;
	5476	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	814.4;
	5476	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3481.7;
	5476	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8986.6;
	5476	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5056.7;
	5476	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9716.1;
	5476	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1531.1;
	5477	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.7;
	5477	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	715;
	5477	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.3;
	5477	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3580.9;
	5477	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9548;
	5477	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5386;
	5477	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10130.3;
	5477	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1550.4;
	5478	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.8;
	5478	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	759.2;
	5478	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	898.1;
	5478	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3811.3;
	5478	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10394.9;
	5478	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5925.1;
	5478	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10672.4;
	5478	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1714.7;
	5479	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.8;
	5479	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	784.9;
	5479	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	919;
	5479	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3845;
	5479	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10826.9;
	5479	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6187.1;
	5479	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10828.6;
	5479	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1793.2;
	5480	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957;
	5480	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	811.3;
	5480	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	942.4;
	5480	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3913.2;
	5480	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11136.8;
	5480	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6327.6;
	5480	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11187.5;
	5480	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1869.7;
	5481	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989.4;
	5481	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	858;
	5481	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	975.5;
	5481	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4030.5;
	5481	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11664.1;
	5481	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6560.6;
	5481	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11593.9;
	5481	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1902.1;
	5482	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1021;
	5482	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	916.7;
	5482	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1010;
	5482	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4189.5;
	5482	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12155.2;
	5482	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6891.7;
	5482	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11811.4;
	5482	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2008.2;
	5483	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1054.8;
	5483	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	973;
	5483	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1049.8;
	5483	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4354.8;
	5483	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12625;
	5483	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7313.8;
	5483	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11858.4;
	5483	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2057.5;
	5484	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1084.5;
	5484	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1009.9;
	5484	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1095.5;
	5484	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4524.2;
	5484	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13052.4;
	5484	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7673.8;
	5484	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11781.2;
	5484	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2101;
	5485	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1114.4;
	5485	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1056;
	5485	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1146.1;
	5485	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4662.4;
	5485	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13474.2;
	5485	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7947.8;
	5485	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11914.2;
	5485	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2157;
	5486	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1137.5;
	5486	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1098.5;
	5486	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1194;
	5486	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4826.4;
	5486	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13950.7;
	5486	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7937.9;
	5486	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12179.8;
	5486	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2144.7;
	5487	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1159;
	5487	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1127.4;
	5487	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1235.6;
	5487	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4946.6;
	5487	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14215.7;
	5487	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7741.9;
	5487	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12490.8;
	5487	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2139.2;
	5488	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1167.8;
	5488	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1149.6;
	5488	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1264;
	5488	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5035;
	5488	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14352.1;
	5488	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7519.1;
	5488	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12837.2;
	5488	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2088.1;
	5489	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1150.7;
	5489	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1139.3;
	5489	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1256.5;
	5489	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4973.7;
	5489	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14331.5;
	5489	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7353.1;
	5489	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12918.2;
	5489	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2086.7;
	5490	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1140.4;
	5490	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1114.2;
	5490	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1229.3;
	5490	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4847.4;
	5490	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14188.2;
	5490	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7232.3;
	5490	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12651.1;
	5490	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1995.6;
	5491	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1112.4;
	5491	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1078.8;
	5491	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1191.4;
	5491	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4710.7;
	5491	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13851.4;
	5491	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7124.3;
	5491	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12476.7;
	5491	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1960.6;
	5492	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1088.5;
	5492	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1052.2;
	5492	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1161.7;
	5492	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4754.8;
	5492	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13757.1;
	5492	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7199.7;
	5492	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12572.3;
	5492	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2047.3;
	5493	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1066.2;
	5493	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1014.4;
	5493	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1133.1;
	5493	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4636.1;
	5493	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13494.5;
	5493	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7087.3;
	5493	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12281.5;
	5493	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1983.4;
	5494	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1020;
	5494	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	945.2;
	5494	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1059.2;
	5494	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4366.6;
	5494	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12623.4;
	5494	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6702.7;
	5494	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11666.9;
	5494	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1875.7;
	5495	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	984.5;
	5495	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	876.6;
	5495	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	981.2;
	5495	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4068.2;
	5495	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11576.2;
	5495	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6161.5;
	5495	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10985.9;
	5495	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1753.7;
	5496	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.2;
	5496	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	812.2;
	5496	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	921.5;
	5496	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3838;
	5496	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10654.3;
	5496	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5744.6;
	5496	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10439.9;
	5496	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1635.8;
	5497	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.3;
	5497	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	774.9;
	5497	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	881.4;
	5497	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3653.4;
	5497	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10150.7;
	5497	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5471.7;
	5497	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10091.5;
	5497	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1589.3;
	5498	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.4;
	5498	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	758.5;
	5498	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	858.2;
	5498	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3522;
	5498	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9821.1;
	5498	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5321.7;
	5498	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9851.1;
	5498	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1562.6;
	5499	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.3;
	5499	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746.6;
	5499	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.1;
	5499	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3434.8;
	5499	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9716.1;
	5499	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5265;
	5499	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9740;
	5499	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1510.3;
	5500	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.1;
	5500	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.7;
	5500	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851.6;
	5500	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3409;
	5500	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9780.3;
	5500	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5327.7;
	5500	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9834.2;
	5500	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1546.9;
	5501	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.8;
	5501	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	765.4;
	5501	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	880.6;
	5501	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3503.5;
	5501	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10336.7;
	5501	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5615.4;
	5501	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10315.1;
	5501	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1577.8;
	5502	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.8;
	5502	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	811.5;
	5502	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	941.2;
	5502	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3745.8;
	5502	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11204.6;
	5502	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6146.1;
	5502	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10872.4;
	5502	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1728.9;
	5503	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944;
	5503	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	836.2;
	5503	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	978.1;
	5503	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3799.8;
	5503	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11558.8;
	5503	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6379.2;
	5503	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10987;
	5503	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1742.1;
	5504	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.5;
	5504	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	854.5;
	5504	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1000.2;
	5504	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3922.5;
	5504	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11856.5;
	5504	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6564.2;
	5504	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11098.2;
	5504	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1780.4;
	5505	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	974.8;
	5505	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	896.5;
	5505	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1034.7;
	5505	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4213;
	5505	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12196.2;
	5505	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6857.5;
	5505	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11518;
	5505	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1910.9;
	5506	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009.7;
	5506	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	947;
	5506	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1077.5;
	5506	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4598.3;
	5506	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12687.5;
	5506	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7262.4;
	5506	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12174.6;
	5506	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2002.7;
	5507	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1040.3;
	5507	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	994.2;
	5507	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1123.8;
	5507	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4942.5;
	5507	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13088.6;
	5507	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7666;
	5507	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12654.7;
	5507	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2045.4;
	5508	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1066.3;
	5508	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1037;
	5508	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1164.6;
	5508	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5221.3;
	5508	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13296.3;
	5508	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7898.2;
	5508	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12929.3;
	5508	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2114.9;
	5509	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1095;
	5509	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1066.2;
	5509	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1206.8;
	5509	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5336.8;
	5509	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13504.4;
	5509	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7909;
	5509	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13081.6;
	5509	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2183.3;
	5510	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1129;
	5510	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1098.3;
	5510	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1245.5;
	5510	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5415;
	5510	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13576.8;
	5510	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8019.4;
	5510	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13192.8;
	5510	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2134.5;
	5511	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1146.9;
	5511	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1122.2;
	5511	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1248.8;
	5511	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5470.6;
	5511	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13776.4;
	5511	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8243.6;
	5511	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13141.5;
	5511	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2128.7;
	5512	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1148.7;
	5512	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1133.4;
	5512	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1248.6;
	5512	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5432.1;
	5512	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13949.4;
	5512	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8265.7;
	5512	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12980.6;
	5512	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2099.6;
	5513	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1132.7;
	5513	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1122.5;
	5513	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1223.5;
	5513	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5136.5;
	5513	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13884.8;
	5513	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7985.1;
	5513	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12458;
	5513	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2042.8;
	5514	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1121.2;
	5514	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1103.3;
	5514	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1192.1;
	5514	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4889.5;
	5514	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13744.5;
	5514	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7745.1;
	5514	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11887.1;
	5514	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1989.3;
	5515	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1099.3;
	5515	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1064.1;
	5515	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1149.3;
	5515	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4703.3;
	5515	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13469.7;
	5515	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7550.1;
	5515	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11707;
	5515	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1970;
	5516	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1079.8;
	5516	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1039;
	5516	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1129.3;
	5516	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4729.7;
	5516	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13385.2;
	5516	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7529.5;
	5516	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11762.7;
	5516	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1961.1;
	5517	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1064.7;
	5517	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1007.1;
	5517	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1106;
	5517	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4622.6;
	5517	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13028;
	5517	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7322.7;
	5517	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11466.4;
	5517	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1914.4;
	5518	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1031.2;
	5518	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	942.2;
	5518	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1042.4;
	5518	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4352.2;
	5518	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12104.8;
	5518	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6800.4;
	5518	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10871.6;
	5518	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1766.7;
	5519	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989.4;
	5519	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	877.6;
	5519	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	970.4;
	5519	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4045.3;
	5519	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11073.7;
	5519	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6285.5;
	5519	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10220.9;
	5519	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1637.5;
	5520	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.9;
	5520	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	830.3;
	5520	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	902.2;
	5520	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3755.6;
	5520	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10269.9;
	5520	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5944.4;
	5520	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9628.6;
	5520	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1515.6;
	5521	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.3;
	5521	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	792.8;
	5521	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.8;
	5521	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3582.2;
	5521	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9789.9;
	5521	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5594.5;
	5521	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9322.9;
	5521	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1510.6;
	5522	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.6;
	5522	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	766.3;
	5522	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.2;
	5522	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3453;
	5522	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9533;
	5522	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5360;
	5522	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9166.5;
	5522	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1477;
	5523	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.6;
	5523	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	756.2;
	5523	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	829.2;
	5523	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3368.2;
	5523	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9413.7;
	5523	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5289.3;
	5523	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9122.8;
	5523	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1451.5;
	5524	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.8;
	5524	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.7;
	5524	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	825.5;
	5524	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3331.9;
	5524	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9573.1;
	5524	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5323;
	5524	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9278.4;
	5524	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1487.1;
	5525	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.8;
	5525	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777.8;
	5525	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849.1;
	5525	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3426.2;
	5525	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10141.7;
	5525	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5599.8;
	5525	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9834.3;
	5525	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1573.3;
	5526	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.2;
	5526	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	826.5;
	5526	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	900.5;
	5526	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3673.5;
	5526	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11041.9;
	5526	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6139;
	5526	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10509.4;
	5526	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1653.2;
	5527	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.5;
	5527	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	851.6;
	5527	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	927.2;
	5527	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3723;
	5527	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11370.4;
	5527	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6356.5;
	5527	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10795.6;
	5527	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1745;
	5528	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.6;
	5528	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	887.3;
	5528	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	951.7;
	5528	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3937.2;
	5528	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11798.1;
	5528	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6538.9;
	5528	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11308;
	5528	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1821.6;
	5529	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981;
	5529	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	914.1;
	5529	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	990.3;
	5529	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4322.2;
	5529	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12440.1;
	5529	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6837.6;
	5529	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12093.2;
	5529	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1904.8;
	5530	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1016.6;
	5530	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	957.8;
	5530	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1036.1;
	5530	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4741;
	5530	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13179.5;
	5530	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7239.9;
	5530	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12971.6;
	5530	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2010.1;
	5531	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1045.6;
	5531	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	981.2;
	5531	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1075.7;
	5531	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5112.7;
	5531	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13945.8;
	5531	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7641.7;
	5531	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13481.6;
	5531	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2092.1;
	5532	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1073.1;
	5532	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	996.8;
	5532	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1106.5;
	5532	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5441.6;
	5532	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14675;
	5532	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8092.9;
	5532	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13322.3;
	5532	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2244.5;
	5533	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1095.2;
	5533	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1011.9;
	5533	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1145.3;
	5533	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5703.1;
	5533	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15419.9;
	5533	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8602.8;
	5533	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12821.5;
	5533	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2245.8;
	5534	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1115.7;
	5534	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1013.4;
	5534	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1187;
	5534	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5893.6;
	5534	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15654.4;
	5534	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9072;
	5534	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12507.4;
	5534	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2209.7;
	5535	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1123.5;
	5535	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1017.9;
	5535	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1223;
	5535	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5987.2;
	5535	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15370.7;
	5535	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9372.6;
	5535	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12560.4;
	5535	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2112.2;
	5536	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1127.8;
	5536	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1008.9;
	5536	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1230.8;
	5536	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5992.8;
	5536	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15106.9;
	5536	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9490.7;
	5536	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12727.2;
	5536	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2110;
	5537	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1112.2;
	5537	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	976;
	5537	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1205.1;
	5537	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5813.2;
	5537	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14683.8;
	5537	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9487.6;
	5537	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12697;
	5537	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2101;
	5538	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1093.2;
	5538	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	950;
	5538	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1172.6;
	5538	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5596.6;
	5538	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14062.5;
	5538	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9222.5;
	5538	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12563.8;
	5538	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2089;
	5539	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1076.7;
	5539	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	934.2;
	5539	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1132.9;
	5539	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5367.4;
	5539	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13618.4;
	5539	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8654.3;
	5539	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12465;
	5539	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2024.1;
	5540	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1063.1;
	5540	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	936;
	5540	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1116.5;
	5540	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5302.5;
	5540	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13566.6;
	5540	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8456.3;
	5540	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12594.7;
	5540	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2084.2;
	5541	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1051.2;
	5541	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	928.7;
	5541	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1100.9;
	5541	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5152.3;
	5541	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13211.2;
	5541	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8157.2;
	5541	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12308.9;
	5541	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1997.8;
	5542	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1015.2;
	5542	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	882.9;
	5542	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1033.2;
	5542	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4818;
	5542	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12353.4;
	5542	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7581.8;
	5542	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11579.1;
	5542	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1876.4;
	5543	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.2;
	5543	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	826.3;
	5543	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	960.9;
	5543	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4447.8;
	5543	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11316.3;
	5543	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6934;
	5543	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10793.3;
	5543	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1767.6;
	5544	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.7;
	5544	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776.6;
	5544	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	921.4;
	5544	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4172.2;
	5544	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10543.8;
	5544	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6162.2;
	5544	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10254.7;
	5544	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1653;
	5545	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.9;
	5545	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	745.7;
	5545	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884;
	5545	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3936;
	5545	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9989.1;
	5545	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5831;
	5545	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9807.7;
	5545	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1576;
	5546	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.1;
	5546	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	731;
	5546	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	863.2;
	5546	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3764.4;
	5546	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9657.8;
	5546	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5638.9;
	5546	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9563.3;
	5546	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1543.3;
	5547	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.1;
	5547	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	721.6;
	5547	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	850.6;
	5547	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3653;
	5547	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9464.1;
	5547	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5538.2;
	5547	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9489.7;
	5547	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1524;
	5548	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.6;
	5548	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	722.5;
	5548	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.2;
	5548	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3606.7;
	5548	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9597.8;
	5548	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5626;
	5548	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9587.2;
	5548	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1482.7;
	5549	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.6;
	5549	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744.9;
	5549	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.8;
	5549	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3671.7;
	5549	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10107.4;
	5549	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5893.7;
	5549	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10004.8;
	5549	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1589.6;
	5550	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.2;
	5550	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	787.3;
	5550	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	916.1;
	5550	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3907.8;
	5550	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10946;
	5550	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6422.9;
	5550	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10571.4;
	5550	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1681;
	5551	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.5;
	5551	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	815.1;
	5551	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	942.9;
	5551	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3963.6;
	5551	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11400.3;
	5551	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6644.8;
	5551	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10848.9;
	5551	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1712.9;
	5552	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.3;
	5552	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	852.9;
	5552	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	978.6;
	5552	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4188.5;
	5552	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11830.6;
	5552	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6885.7;
	5552	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11606;
	5552	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1781.9;
	5553	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	994.6;
	5553	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	881.3;
	5553	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1026.7;
	5553	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4568;
	5553	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12437.3;
	5553	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7341.4;
	5553	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12652.5;
	5553	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1875.7;
	5554	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1028;
	5554	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	914.7;
	5554	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1053.6;
	5554	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4937.8;
	5554	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13101;
	5554	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7898.9;
	5554	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13531.6;
	5554	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1995.9;
	5555	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1057.5;
	5555	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	934.9;
	5555	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1112.4;
	5555	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5276;
	5555	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13612.2;
	5555	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8392.5;
	5555	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14328.6;
	5555	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2124.2;
	5556	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1077.7;
	5556	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	945.8;
	5556	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1144.3;
	5556	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5537.9;
	5556	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13975.3;
	5556	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8849.1;
	5556	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14772.5;
	5556	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2264.1;
	5557	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1096;
	5557	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	949.5;
	5557	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1190.4;
	5557	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5768;
	5557	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14418;
	5557	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9281.4;
	5557	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14683.6;
	5557	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2459.1;
	5558	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1122.5;
	5558	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	944.7;
	5558	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1223.8;
	5558	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5947.1;
	5558	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14867.6;
	5558	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9621.5;
	5558	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14577.6;
	5558	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2482.3;
	5559	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1127.1;
	5559	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	941.8;
	5559	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1255.6;
	5559	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6012.1;
	5559	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14802.9;
	5559	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9914.7;
	5559	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14303.2;
	5559	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2542.7;
	5560	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1114.4;
	5560	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	926.6;
	5560	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1269.4;
	5560	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5950.9;
	5560	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14374.3;
	5560	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10032.7;
	5560	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13954.9;
	5560	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2540.3;
	5561	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1094.6;
	5561	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	905.3;
	5561	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1255.4;
	5561	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5804.4;
	5561	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13592.7;
	5561	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10007.8;
	5561	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13774.2;
	5561	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2469.3;
	5562	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1081;
	5562	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	891.9;
	5562	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1232.8;
	5562	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5593.9;
	5562	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12893.9;
	5562	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9799.7;
	5562	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13591.8;
	5562	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2344.1;
	5563	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1070;
	5563	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	869.2;
	5563	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1190.1;
	5563	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5330.3;
	5563	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12408.1;
	5563	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9349.1;
	5563	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13222.1;
	5563	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2219.5;
	5564	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1058.2;
	5564	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	870.4;
	5564	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1162.2;
	5564	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5252.2;
	5564	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12431.6;
	5564	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9087.1;
	5564	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13173.2;
	5564	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2150;
	5565	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1045.6;
	5565	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	865.7;
	5565	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1138.3;
	5565	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5097.3;
	5565	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12226.1;
	5565	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8705.8;
	5565	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12787.3;
	5565	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2083.6;
	5566	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1016.9;
	5566	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	830.2;
	5566	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1081.5;
	5566	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4847.8;
	5566	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11618;
	5566	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8107.8;
	5566	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12152.4;
	5566	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1957.9;
	5567	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981.8;
	5567	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	786.6;
	5567	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1018.3;
	5567	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4567.1;
	5567	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10820.7;
	5567	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7420.7;
	5567	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11439;
	5567	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1793.3;
	5568	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.2;
	5568	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	764.2;
	5568	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	961.9;
	5568	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4297.5;
	5568	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10055;
	5568	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6811.5;
	5568	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10760.9;
	5568	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1666.6;
	5569	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.6;
	5569	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735;
	5569	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	915.7;
	5569	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4078.1;
	5569	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9561.2;
	5569	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6367.1;
	5569	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10241.3;
	5569	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1581.8;
	5570	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.8;
	5570	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	716.2;
	5570	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.5;
	5570	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3904.7;
	5570	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9219.3;
	5570	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6066.9;
	5570	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9837.1;
	5570	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1607.6;
	5571	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.7;
	5571	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711.8;
	5571	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.7;
	5571	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3787.8;
	5571	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9146.2;
	5571	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5923.1;
	5571	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9597;
	5571	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1500.9;
	5572	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.8;
	5572	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	710.4;
	5572	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.3;
	5572	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3731.2;
	5572	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9183.5;
	5572	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5841.5;
	5572	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9492.7;
	5572	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1544.6;
	5573	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.8;
	5573	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	724;
	5573	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.3;
	5573	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3735.6;
	5573	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9396.8;
	5573	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5889.6;
	5573	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9531.4;
	5573	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1594.7;
	5574	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.6;
	5574	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.4;
	5574	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.9;
	5574	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3788.2;
	5574	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9812;
	5574	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6045.4;
	5574	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9694.9;
	5574	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1625.2;
	5575	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.6;
	5575	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	748.2;
	5575	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	874.4;
	5575	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3811.7;
	5575	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10191.4;
	5575	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6153;
	5575	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9845.3;
	5575	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1662;
	5576	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.4;
	5576	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776.6;
	5576	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	907.1;
	5576	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4068.5;
	5576	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10548.4;
	5576	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6509;
	5576	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10759;
	5576	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1781.7;
	5577	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.2;
	5577	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	801.7;
	5577	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	938.2;
	5577	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4416.8;
	5577	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11034.3;
	5577	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6808.9;
	5577	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11853.6;
	5577	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1912.8;
	5578	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	996;
	5578	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	834.6;
	5578	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	972.2;
	5578	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4747.3;
	5578	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11505.3;
	5578	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7110.8;
	5578	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12921.3;
	5578	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2037.1;
	5579	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1025.2;
	5579	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	855.9;
	5579	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	995.7;
	5579	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5025.3;
	5579	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11788.3;
	5579	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7294;
	5579	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13777.2;
	5579	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2125.8;
	5580	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1054.5;
	5580	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	880.5;
	5580	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1009.4;
	5580	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5269.2;
	5580	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11852.3;
	5580	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7356.8;
	5580	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14293.5;
	5580	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2214.1;
	5581	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1071.6;
	5581	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	895.3;
	5581	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1020.6;
	5581	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5456.7;
	5581	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11816.5;
	5581	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7402.2;
	5581	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14266.8;
	5581	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2268.4;
	5582	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1078.7;
	5582	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	918;
	5582	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1036.3;
	5582	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5584.8;
	5582	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11945.2;
	5582	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7423.9;
	5582	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14365.3;
	5582	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2294.4;
	5583	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1078.1;
	5583	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	953;
	5583	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1058.1;
	5583	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5675.5;
	5583	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12197.5;
	5583	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7505.1;
	5583	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14691.1;
	5583	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2316.7;
	5584	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1068.4;
	5584	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	974;
	5584	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1080;
	5584	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5686.9;
	5584	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12516.6;
	5584	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7524;
	5584	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14778.9;
	5584	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2249.6;
	5585	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1061.3;
	5585	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	984.9;
	5585	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1091.3;
	5585	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5578.2;
	5585	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12737.3;
	5585	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7393.1;
	5585	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14592.4;
	5585	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2216.6;
	5586	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1044.1;
	5586	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	976.6;
	5586	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1097.4;
	5586	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5383.9;
	5586	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12662.2;
	5586	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7265.3;
	5586	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14247.3;
	5586	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2116.2;
	5587	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1019.5;
	5587	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	954;
	5587	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1078.4;
	5587	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5156.5;
	5587	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12447.1;
	5587	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7137.6;
	5587	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13823;
	5587	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2067.4;
	5588	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1014;
	5588	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	944.2;
	5588	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1069.7;
	5588	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5125.9;
	5588	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12459.2;
	5588	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7222.7;
	5588	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13651.9;
	5588	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2016.2;
	5589	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1010.8;
	5589	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	912.3;
	5589	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1041.5;
	5589	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4956.8;
	5589	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12238.6;
	5589	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7044.3;
	5589	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13174.7;
	5589	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1898.5;
	5590	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985.6;
	5590	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	861.6;
	5590	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	986.9;
	5590	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4689.4;
	5590	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11550.1;
	5590	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6668.4;
	5590	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12517.1;
	5590	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1793.2;
	5591	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.2;
	5591	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	806.3;
	5591	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	931.4;
	5591	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4380.9;
	5591	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10705.9;
	5591	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6211.4;
	5591	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11840.5;
	5591	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1676.9;
	5592	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.1;
	5592	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749.9;
	5592	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878.2;
	5592	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4137.3;
	5592	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9913.9;
	5592	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5790.4;
	5592	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11251.2;
	5592	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1589.3;
	5593	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.6;
	5593	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	716.6;
	5593	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	840.9;
	5593	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3935.3;
	5593	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9320.4;
	5593	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5457.4;
	5593	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10925.3;
	5593	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1508.5;
	5594	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.5;
	5594	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	692.9;
	5594	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	815.4;
	5594	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3788.2;
	5594	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8927.8;
	5594	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5213.2;
	5594	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10553.7;
	5594	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1441;
	5595	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.8;
	5595	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	675.5;
	5595	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	793.8;
	5595	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3672.5;
	5595	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8677.7;
	5595	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5032.2;
	5595	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10319.2;
	5595	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1424.3;
	5596	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.1;
	5596	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	666;
	5596	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	785.1;
	5596	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3606;
	5596	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8558.5;
	5596	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4932.4;
	5596	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10195.4;
	5596	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1423;
	5597	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.6;
	5597	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	665.7;
	5597	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	787.1;
	5597	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3584.4;
	5597	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8619.5;
	5597	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4961.1;
	5597	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10103.2;
	5597	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1451;
	5598	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.5;
	5598	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	671.4;
	5598	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	799.1;
	5598	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3599;
	5598	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8804.7;
	5598	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5108.1;
	5598	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10062.9;
	5598	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1480.5;
	5599	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.6;
	5599	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	675.9;
	5599	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	810.9;
	5599	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3553.8;
	5599	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8935.4;
	5599	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5195.4;
	5599	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9909.2;
	5599	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1497.7;
	5600	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.2;
	5600	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	726.1;
	5600	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.1;
	5600	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3706.5;
	5600	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9648.3;
	5600	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5417.9;
	5600	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10192.8;
	5600	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1537.9;
	5601	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.6;
	5601	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772.6;
	5601	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	895;
	5601	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3993.7;
	5601	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10546.8;
	5601	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5792.4;
	5601	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10716.4;
	5601	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1631.2;
	5602	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.1;
	5602	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	823;
	5602	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	929.6;
	5602	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4297.5;
	5602	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11455;
	5602	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6231.5;
	5602	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11207.8;
	5602	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1703.6;
	5603	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985.9;
	5603	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	869.8;
	5603	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	966;
	5603	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4573.8;
	5603	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12382.1;
	5603	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6640;
	5603	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11764.9;
	5603	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1751;
	5604	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1016.5;
	5604	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	915.6;
	5604	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1000.2;
	5604	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4742.2;
	5604	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13201.8;
	5604	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6991.7;
	5604	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12119.8;
	5604	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1810.7;
	5605	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1035.4;
	5605	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	957.9;
	5605	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1030.9;
	5605	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4899.9;
	5605	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13869.6;
	5605	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7223.2;
	5605	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12142.1;
	5605	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1860;
	5606	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1055.4;
	5606	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	993.8;
	5606	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1068.1;
	5606	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4982.6;
	5606	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14466.9;
	5606	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7336.6;
	5606	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12134.1;
	5606	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1900.3;
	5607	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1074.3;
	5607	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1028.2;
	5607	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1096.8;
	5607	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4979.7;
	5607	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14970.9;
	5607	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7387.5;
	5607	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12592.5;
	5607	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1940;
	5608	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1086.8;
	5608	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1052.9;
	5608	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1118.3;
	5608	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4857;
	5608	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15304.1;
	5608	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7423.9;
	5608	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13087.8;
	5608	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1978.6;
	5609	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1092.7;
	5609	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1061.8;
	5609	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1136.1;
	5609	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4773.5;
	5609	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15455.9;
	5609	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7456.6;
	5609	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13160;
	5609	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1978.1;
	5610	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1087.5;
	5610	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1042.3;
	5610	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1130.7;
	5610	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4693.3;
	5610	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15276.2;
	5610	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7454.1;
	5610	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12915.2;
	5610	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1968;
	5611	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1073.8;
	5611	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	999.9;
	5611	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1105.3;
	5611	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4630.9;
	5611	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14692.3;
	5611	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7435.1;
	5611	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12656.3;
	5611	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1925.9;
	5612	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1063.7;
	5612	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	971.6;
	5612	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1092.6;
	5612	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4722;
	5612	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14375.7;
	5612	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7546;
	5612	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12770.5;
	5612	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1901.2;
	5613	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1054;
	5613	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	922.9;
	5613	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1064.2;
	5613	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4602.8;
	5613	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13748.7;
	5613	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7324.5;
	5613	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12374.6;
	5613	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1890.6;
	5614	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1018.4;
	5614	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	847.9;
	5614	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	989.5;
	5614	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4306.9;
	5614	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12487.9;
	5614	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6789.5;
	5614	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11590.9;
	5614	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1803.7;
	5615	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	973;
	5615	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	773.9;
	5615	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	908.4;
	5615	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3973.2;
	5615	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11175.4;
	5615	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6169.6;
	5615	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10739.1;
	5615	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1681.5;
	5616	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.8;
	5616	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723.1;
	5616	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	872.9;
	5616	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3719.2;
	5616	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10224.9;
	5616	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5724.7;
	5616	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10117.1;
	5616	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1568.9;
	5617	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.5;
	5617	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	690.6;
	5617	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.7;
	5617	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3560;
	5617	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9606.8;
	5617	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5439.3;
	5617	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9729.4;
	5617	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1487.8;
	5618	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.9;
	5618	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	673.5;
	5618	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	813.8;
	5618	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3457.3;
	5618	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9278.5;
	5618	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5243.6;
	5618	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9520.6;
	5618	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1416;
	5619	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.9;
	5619	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	662.9;
	5619	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	803.8;
	5619	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3399.5;
	5619	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9135.5;
	5619	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5188.3;
	5619	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9456.1;
	5619	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1461.9;
	5620	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.9;
	5620	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	667.5;
	5620	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	801.5;
	5620	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3402.9;
	5620	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9296.7;
	5620	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5271.1;
	5620	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9609.7;
	5620	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1497.1;
	5621	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.9;
	5621	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	701.9;
	5621	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	827.4;
	5621	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3552.1;
	5621	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9906.1;
	5621	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5625.3;
	5621	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10143.1;
	5621	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1568.2;
	5622	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.3;
	5622	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	760.4;
	5622	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	898.8;
	5622	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3884.9;
	5622	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11001.1;
	5622	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6266;
	5622	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10820.1;
	5622	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1754.3;
	5623	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.3;
	5623	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777.4;
	5623	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	927.8;
	5623	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3868.2;
	5623	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11314.6;
	5623	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6386.4;
	5623	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10932.3;
	5623	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1800.1;
	5624	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	983.7;
	5624	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	808.5;
	5624	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	955.4;
	5624	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4029.1;
	5624	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11618.2;
	5624	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6355.1;
	5624	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11463.8;
	5624	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1847;
	5625	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1014.9;
	5625	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	843.4;
	5625	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	993.3;
	5625	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4391.1;
	5625	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12293.6;
	5625	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6542.2;
	5625	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12438.2;
	5625	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1978.1;
	5626	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1046.4;
	5626	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	893.8;
	5626	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1026.5;
	5626	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4764.1;
	5626	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12941.1;
	5626	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6881.1;
	5626	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13175.4;
	5626	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2097.3;
	5627	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1066.5;
	5627	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	935.7;
	5627	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1077.8;
	5627	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5087.4;
	5627	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13485.5;
	5627	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7383.6;
	5627	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12925.2;
	5627	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2226.6;
	5628	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1086;
	5628	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	966.7;
	5628	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1126.1;
	5628	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5366.8;
	5628	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14175.9;
	5628	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7930.1;
	5628	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12439.9;
	5628	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2347.4;
	5629	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1113;
	5629	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1010.9;
	5629	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1182.5;
	5629	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5620.9;
	5629	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15040.3;
	5629	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8526.2;
	5629	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12300.4;
	5629	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2396.9;
	5630	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1122.9;
	5630	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1052.2;
	5630	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1230.5;
	5630	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5821.8;
	5630	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15808.5;
	5630	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9004.4;
	5630	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12326.7;
	5630	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2306.5;
	5631	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1127.9;
	5631	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1070.7;
	5631	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1258.6;
	5631	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5932.1;
	5631	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15859.1;
	5631	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9343;
	5631	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12588.5;
	5631	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2263.4;
	5632	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1126.1;
	5632	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1087.7;
	5632	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1262.7;
	5632	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5977.6;
	5632	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15437.4;
	5632	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9663.7;
	5632	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13276.7;
	5632	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2256;
	5633	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1113.6;
	5633	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1080.1;
	5633	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1257.5;
	5633	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5908.4;
	5633	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15005;
	5633	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9794.6;
	5633	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13819.9;
	5633	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2268;
	5634	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1103.5;
	5634	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1044.8;
	5634	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1235.3;
	5634	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5736.2;
	5634	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14519.4;
	5634	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9699.2;
	5634	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13797.2;
	5634	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2280.9;
	5635	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1088.2;
	5635	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1008.5;
	5635	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1181.1;
	5635	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5513.6;
	5635	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14161.9;
	5635	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9335.1;
	5635	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13478.9;
	5635	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2240;
	5636	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1082.9;
	5636	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1004.3;
	5636	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1147;
	5636	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5513.1;
	5636	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14179.6;
	5636	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9124.7;
	5636	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13497.3;
	5636	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2247.2;
	5637	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1071.6;
	5637	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	970.3;
	5637	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1111.5;
	5637	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5318;
	5637	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13770.1;
	5637	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8634.4;
	5637	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13000;
	5637	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2166.8;
	5638	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1035.6;
	5638	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	909.8;
	5638	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1037.1;
	5638	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4931.6;
	5638	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12799.3;
	5638	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7825.6;
	5638	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12107;
	5638	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2012.5;
	5639	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999;
	5639	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	855.7;
	5639	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	956.6;
	5639	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4508.3;
	5639	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11640.1;
	5639	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7009.2;
	5639	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11195.1;
	5639	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1852.1;
	5640	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	968.4;
	5640	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	812.5;
	5640	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	890.3;
	5640	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4142.4;
	5640	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10921.3;
	5640	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6439.7;
	5640	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10461.8;
	5640	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1715.6;
	5641	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.1;
	5641	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	791.1;
	5641	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.4;
	5641	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3930.6;
	5641	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10475.3;
	5641	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6121.3;
	5641	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10091.5;
	5641	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1563.7;
	5642	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.6;
	5642	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776;
	5642	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	846.3;
	5642	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3773.3;
	5642	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10191.8;
	5642	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5908.5;
	5642	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9835.5;
	5642	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1516.3;
	5643	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.2;
	5643	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	769.1;
	5643	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	842.5;
	5643	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3666.7;
	5643	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10038.6;
	5643	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5827.6;
	5643	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9741.7;
	5643	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1546.4;
	5644	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.1;
	5644	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	773.1;
	5644	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	842.3;
	5644	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3619.3;
	5644	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10227.4;
	5644	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5838.2;
	5644	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9854.9;
	5644	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1561.6;
	5645	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.5;
	5645	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	800.5;
	5645	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	875.4;
	5645	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3754.8;
	5645	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10884.6;
	5645	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6130.1;
	5645	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10436.9;
	5645	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1588.9;
	5646	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990;
	5646	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	860;
	5646	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	942.5;
	5646	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4081.4;
	5646	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11896.1;
	5646	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6774.7;
	5646	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11127.9;
	5646	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1787.9;
	5647	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999;
	5647	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	876.1;
	5647	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	964.8;
	5647	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4042;
	5647	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12128.3;
	5647	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6868.4;
	5647	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11216.4;
	5647	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1854.5;
	5648	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1010.2;
	5648	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	903.4;
	5648	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	990.1;
	5648	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4211.3;
	5648	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12566.3;
	5648	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6998.1;
	5648	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11872.3;
	5648	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1962.2;
	5649	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1037.1;
	5649	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	950.2;
	5649	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1035.5;
	5649	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4558.9;
	5649	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13394.4;
	5649	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7391.8;
	5649	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12855.5;
	5649	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2054.2;
	5650	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1060.7;
	5650	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	998;
	5650	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1098.4;
	5650	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4927.5;
	5650	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14395.7;
	5650	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7930;
	5650	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13850.8;
	5650	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2242.5;
	5651	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1078.9;
	5651	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1055.9;
	5651	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1164.1;
	5651	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5259.1;
	5651	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15340;
	5651	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8453.4;
	5651	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14769;
	5651	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2371;
	5652	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1098.7;
	5652	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1101.7;
	5652	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1221.5;
	5652	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5559.3;
	5652	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16253.9;
	5652	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8916.4;
	5652	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15572.8;
	5652	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2510.8;
	5653	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1124.4;
	5653	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1150.2;
	5653	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1280.1;
	5653	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5821;
	5653	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17091.5;
	5653	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9379.2;
	5653	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16283;
	5653	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2671.6;
	5654	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1135.2;
	5654	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1191;
	5654	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1320.7;
	5654	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5982.2;
	5654	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17774.9;
	5654	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9754.1;
	5654	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16600.9;
	5654	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2749;
	5655	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1141.8;
	5655	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1218.1;
	5655	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1354.1;
	5655	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6079.2;
	5655	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18224.1;
	5655	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10052.9;
	5655	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16713.7;
	5655	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2786.2;
	5656	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1141;
	5656	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1241.3;
	5656	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1385.8;
	5656	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6133.7;
	5656	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18533.1;
	5656	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10272.1;
	5656	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16772.6;
	5656	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2758.3;
	5657	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1115.5;
	5657	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1224.8;
	5657	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1383.3;
	5657	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6039.6;
	5657	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18491.9;
	5657	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10314.7;
	5657	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16518.6;
	5657	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2759.2;
	5658	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1098.6;
	5658	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1195.5;
	5658	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1352.3;
	5658	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5834.6;
	5658	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18072.9;
	5658	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10119.9;
	5658	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15564.6;
	5658	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2636.8;
	5659	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1076.9;
	5659	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1151.4;
	5659	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1293.6;
	5659	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5586.4;
	5659	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17258.7;
	5659	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9686.6;
	5659	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14964.4;
	5659	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2521.2;
	5660	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1078.9;
	5660	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1132.4;
	5660	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1250.2;
	5660	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5553.1;
	5660	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16801.8;
	5660	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9402.5;
	5660	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14931;
	5660	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2532.6;
	5661	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1061.9;
	5661	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1080.6;
	5661	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1199;
	5661	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5342.3;
	5661	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15962;
	5661	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8891.9;
	5661	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14317.5;
	5661	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2384.7;
	5662	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1019;
	5662	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	999.7;
	5662	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1104.3;
	5662	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4915.8;
	5662	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14528.6;
	5662	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8017.8;
	5662	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13271;
	5662	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2191.6;
	5663	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.6;
	5663	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	921.3;
	5663	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1013.8;
	5663	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4475.2;
	5663	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13062.8;
	5663	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7132.7;
	5663	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12271.3;
	5663	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2001.4;
	5664	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.2;
	5664	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	862.4;
	5664	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	950.7;
	5664	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4135.3;
	5664	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12055.3;
	5664	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6452.6;
	5664	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11534.8;
	5664	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1849.3;
	5665	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.8;
	5665	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	826.6;
	5665	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	908.5;
	5665	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3899.4;
	5665	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11333.3;
	5665	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5993.2;
	5665	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10980;
	5665	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1775.4;
	5666	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.2;
	5666	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	800.8;
	5666	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878.6;
	5666	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3738.7;
	5666	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10827;
	5666	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5719.2;
	5666	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10588.6;
	5666	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1727.6;
	5667	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.1;
	5667	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	781.3;
	5667	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.1;
	5667	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3625.6;
	5667	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10573.1;
	5667	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5609.8;
	5667	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10342.1;
	5667	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1656.4;
	5668	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.3;
	5668	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777;
	5668	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851.4;
	5668	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3578.2;
	5668	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10510.8;
	5668	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5675.3;
	5668	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10356.5;
	5668	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1665.4;
	5669	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.8;
	5669	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	800.3;
	5669	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	873.8;
	5669	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3674.1;
	5669	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10963.5;
	5669	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6018.9;
	5669	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10803.1;
	5669	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1689.8;
	5670	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.5;
	5670	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	849.5;
	5670	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	929.5;
	5670	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3987.1;
	5670	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11812.2;
	5670	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6641.9;
	5670	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11406.4;
	5670	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1857.1;
	5671	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.5;
	5671	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	863.4;
	5671	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	954.6;
	5671	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3974.2;
	5671	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11965.5;
	5671	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6772.7;
	5671	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11463.4;
	5671	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1825.5;
	5672	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.3;
	5672	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	906.7;
	5672	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	984.3;
	5672	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4117.9;
	5672	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12453.6;
	5672	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6876.7;
	5672	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11986.2;
	5672	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1889.8;
	5673	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	976.1;
	5673	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	957.9;
	5673	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1037.7;
	5673	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4464.9;
	5673	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13291.8;
	5673	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7281.4;
	5673	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12897.5;
	5673	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2080.9;
	5674	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1014.3;
	5674	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1030.8;
	5674	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1096.9;
	5674	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4859.2;
	5674	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14342;
	5674	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7839.5;
	5674	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13898.2;
	5674	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2251.2;
	5675	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1045.3;
	5675	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1094.3;
	5675	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1162.5;
	5675	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5222.2;
	5675	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15459.1;
	5675	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8408;
	5675	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14866.6;
	5675	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2383.7;
	5676	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1077.1;
	5676	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1162.2;
	5676	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1226.7;
	5676	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5553.4;
	5676	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16537.7;
	5676	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8920.1;
	5676	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15765.7;
	5676	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2540.3;
	5677	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1107.9;
	5677	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1215.1;
	5677	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1294.5;
	5677	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5808.1;
	5677	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17588.3;
	5677	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9409.6;
	5677	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16371.2;
	5677	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2690.5;
	5678	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1138.4;
	5678	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1265.2;
	5678	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1348.3;
	5678	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6019.3;
	5678	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18345.1;
	5678	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9786.3;
	5678	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16717.2;
	5678	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2746.2;
	5679	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1147.2;
	5679	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1292;
	5679	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1386;
	5679	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6132.1;
	5679	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18821.4;
	5679	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10065.1;
	5679	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16620;
	5679	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2829.7;
	5680	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1127.6;
	5680	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1308.5;
	5680	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1415.1;
	5680	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6190.7;
	5680	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19097.9;
	5680	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10269.1;
	5680	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16483.5;
	5680	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2879.4;
	5681	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1089.3;
	5681	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1290.8;
	5681	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1411.2;
	5681	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6094.8;
	5681	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18978.7;
	5681	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10314.4;
	5681	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16136.8;
	5681	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2834.4;
	5682	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1065.1;
	5682	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1249.2;
	5682	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1381.5;
	5682	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5886.9;
	5682	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18497.5;
	5682	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10145;
	5682	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15576.1;
	5682	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2746.3;
	5683	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1047;
	5683	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1189.7;
	5683	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1315.6;
	5683	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5610.8;
	5683	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17553.1;
	5683	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9749.3;
	5683	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14964.5;
	5683	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2644.3;
	5684	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1041.8;
	5684	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1148.9;
	5684	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1276.8;
	5684	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5537.6;
	5684	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16967.9;
	5684	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9485.3;
	5684	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14706.6;
	5684	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2566;
	5685	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030.4;
	5685	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1092.4;
	5685	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1219.1;
	5685	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5303.6;
	5685	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16066.1;
	5685	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8946.5;
	5685	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14000.2;
	5685	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2422.7;
	5686	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	993.1;
	5686	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1001.6;
	5686	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1120;
	5686	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4879.2;
	5686	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14566;
	5686	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8160.3;
	5686	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13003.1;
	5686	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2175.4;
	5687	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.3;
	5687	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	923.7;
	5687	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1022.5;
	5687	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4428.6;
	5687	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13101.5;
	5687	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7335.3;
	5687	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11986.2;
	5687	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1976.9;
	5688	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.2;
	5688	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	865.9;
	5688	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	945.1;
	5688	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4070.5;
	5688	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11990.3;
	5688	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6700.6;
	5688	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11098.6;
	5688	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1821;
	5689	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912;
	5689	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	820.4;
	5689	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	900.6;
	5689	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3828.2;
	5689	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11233.6;
	5689	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6245.7;
	5689	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10482;
	5689	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1768.1;
	5690	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.7;
	5690	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	787.3;
	5690	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	870.4;
	5690	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3650.6;
	5690	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10726.6;
	5690	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5931.9;
	5690	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10109.8;
	5690	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1704.9;
	5691	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.7;
	5691	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	773.6;
	5691	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.5;
	5691	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3534.5;
	5691	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10459.9;
	5691	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5766.1;
	5691	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9953.1;
	5691	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1656.7;
	5692	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.6;
	5692	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	767.2;
	5692	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	845.8;
	5692	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3464.6;
	5692	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10455.7;
	5692	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5730.1;
	5692	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9998.4;
	5692	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1670.9;
	5693	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.3;
	5693	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	794.2;
	5693	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.7;
	5693	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3572.2;
	5693	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10941.3;
	5693	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5971.4;
	5693	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10503.9;
	5693	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1748.6;
	5694	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.2;
	5694	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	860.6;
	5694	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	930.3;
	5694	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3904.8;
	5694	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11886;
	5694	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6498.2;
	5694	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11201.9;
	5694	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1840.7;
	5695	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.6;
	5695	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	862.2;
	5695	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	948.3;
	5695	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3881.9;
	5695	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12029.6;
	5695	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6625.9;
	5695	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11264.3;
	5695	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1888.8;
	5696	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	971.2;
	5696	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	903.8;
	5696	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	973.5;
	5696	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4024.3;
	5696	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12504.3;
	5696	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6827.3;
	5696	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11658.5;
	5696	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1979;
	5697	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000.3;
	5697	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	971.4;
	5697	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1032.2;
	5697	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4361;
	5697	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13475.5;
	5697	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7253.1;
	5697	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12605.4;
	5697	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2132.2;
	5698	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1037.4;
	5698	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1052.4;
	5698	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1104.9;
	5698	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4762.4;
	5698	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14626.4;
	5698	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7804;
	5698	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13622.2;
	5698	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2316.3;
	5699	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1068.6;
	5699	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1122.3;
	5699	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1174.9;
	5699	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5143.2;
	5699	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15793.2;
	5699	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8329.4;
	5699	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14413.7;
	5699	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2456.6;
	5700	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1099.4;
	5700	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1186.1;
	5700	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1240.7;
	5700	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5502.3;
	5700	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16829.8;
	5700	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8885.1;
	5700	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15083.3;
	5700	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2616.1;
	5701	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1133.7;
	5701	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1256.5;
	5701	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1308.8;
	5701	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5798.1;
	5701	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17914.8;
	5701	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9341.6;
	5701	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15419.6;
	5701	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2711.2;
	5702	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1153.7;
	5702	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1295.7;
	5702	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1363.4;
	5702	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6003.1;
	5702	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18668.3;
	5702	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9740.7;
	5702	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15131.3;
	5702	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2775;
	5703	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1164.5;
	5703	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1309.2;
	5703	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1409.7;
	5703	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6141;
	5703	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19202.7;
	5703	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10066.1;
	5703	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14831.4;
	5703	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2734.8;
	5704	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1162.7;
	5704	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1312.7;
	5704	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1432.2;
	5704	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6208.1;
	5704	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19483.4;
	5704	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10238.7;
	5704	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14526.8;
	5704	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2764.6;
	5705	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1130.8;
	5705	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1294;
	5705	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1416.4;
	5705	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6078.7;
	5705	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19388;
	5705	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10149.1;
	5705	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14076.6;
	5705	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2750.7;
	5706	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1097;
	5706	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1256.9;
	5706	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1380.7;
	5706	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5853.6;
	5706	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18798.7;
	5706	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9867.7;
	5706	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13663.1;
	5706	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2685.1;
	5707	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1077.3;
	5707	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1203.5;
	5707	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1318.7;
	5707	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5602.4;
	5707	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17848.3;
	5707	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9446.2;
	5707	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13426.2;
	5707	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2541.7;
	5708	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1062.8;
	5708	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1166.5;
	5708	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1273.5;
	5708	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5566.5;
	5708	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17314.8;
	5708	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9198;
	5708	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13467.3;
	5708	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2523.9;
	5709	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1037.8;
	5709	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1112.7;
	5709	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1213;
	5709	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5343.4;
	5709	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16302.9;
	5709	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8828.8;
	5709	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12948.7;
	5709	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2336.7;
	5710	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	996.3;
	5710	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1024.5;
	5710	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1113.3;
	5710	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4932.9;
	5710	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14804.6;
	5710	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8084.2;
	5710	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12023;
	5710	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2131;
	5711	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.6;
	5711	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	940.1;
	5711	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1019.8;
	5711	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4503.1;
	5711	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13358.4;
	5711	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7303.2;
	5711	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11130.2;
	5711	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1972;
	5712	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.7;
	5712	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	877;
	5712	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	950.3;
	5712	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4174.6;
	5712	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12150.3;
	5712	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6638.7;
	5712	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10522.6;
	5712	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1795.2;
	5713	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.6;
	5713	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	834.4;
	5713	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	903.6;
	5713	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3922.6;
	5713	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11407.1;
	5713	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6213.8;
	5713	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10019.8;
	5713	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1682.7;
	5714	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897;
	5714	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	801;
	5714	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	870.6;
	5714	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3725.8;
	5714	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10888.3;
	5714	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5903.5;
	5714	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9651.4;
	5714	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1624.3;
	5715	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.1;
	5715	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	784.7;
	5715	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851.5;
	5715	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3601.2;
	5715	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10517.5;
	5715	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5700;
	5715	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9524.9;
	5715	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1581.3;
	5716	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894;
	5716	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	773.3;
	5716	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	842.1;
	5716	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3548.9;
	5716	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10518.9;
	5716	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5672;
	5716	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9601.9;
	5716	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1527.6;
	5717	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.8;
	5717	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	796.8;
	5717	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	856.7;
	5717	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3648.7;
	5717	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10939.7;
	5717	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5939.6;
	5717	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10059.7;
	5717	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1644;
	5718	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.9;
	5718	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	866;
	5718	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	923.5;
	5718	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3971.9;
	5718	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11845.1;
	5718	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6570.9;
	5718	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10750;
	5718	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1759.2;
	5719	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.5;
	5719	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	881.1;
	5719	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	948.3;
	5719	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3972.7;
	5719	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12101;
	5719	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6741.2;
	5719	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10874;
	5719	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1766.1;
	5720	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.7;
	5720	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	907.2;
	5720	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	981.6;
	5720	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4115.9;
	5720	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12475.8;
	5720	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6839.2;
	5720	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11083.6;
	5720	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1814.7;
	5721	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	979.2;
	5721	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	955.3;
	5721	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1035.2;
	5721	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4467;
	5721	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13242.6;
	5721	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7260;
	5721	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11558.2;
	5721	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1907.8;
	5722	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.2;
	5722	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1017.6;
	5722	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1080.9;
	5722	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4829.2;
	5722	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14330.9;
	5722	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7858.2;
	5722	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12187;
	5722	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2013.4;
	5723	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1039.8;
	5723	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1072.7;
	5723	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1178.3;
	5723	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5214.6;
	5723	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15415.4;
	5723	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8414.6;
	5723	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12861.3;
	5723	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2101.2;
	5724	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1056.1;
	5724	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1129.6;
	5724	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1254;
	5724	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5536.5;
	5724	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16236.2;
	5724	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8937.1;
	5724	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13310.8;
	5724	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2231.5;
	5725	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1073.6;
	5725	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1172.3;
	5725	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1317.5;
	5725	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5802.4;
	5725	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16777.4;
	5725	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9426.7;
	5725	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13816.6;
	5725	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2427.3;
	5726	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1081.3;
	5726	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1162.3;
	5726	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1353.6;
	5726	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6016.3;
	5726	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16878.4;
	5726	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9816.4;
	5726	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14271.3;
	5726	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2470.1;
	5727	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1069.8;
	5727	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1167.6;
	5727	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1385.5;
	5727	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6101.7;
	5727	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16682;
	5727	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10059;
	5727	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14524.3;
	5727	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2438;
	5728	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1057.9;
	5728	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1162.4;
	5728	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1397.3;
	5728	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6111.7;
	5728	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16610.7;
	5728	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10220.1;
	5728	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14622.5;
	5728	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2369;
	5729	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1051;
	5729	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1132.8;
	5729	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1365.6;
	5729	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5953.8;
	5729	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16248.3;
	5729	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10114.2;
	5729	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14267.9;
	5729	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2301.4;
	5730	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1037.9;
	5730	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1095.7;
	5730	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1310.7;
	5730	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5679.2;
	5730	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15450.5;
	5730	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9743.1;
	5730	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13396.5;
	5730	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2198.7;
	5731	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1026.5;
	5731	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1046.9;
	5731	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1254.3;
	5731	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5381.8;
	5731	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14596.6;
	5731	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9220;
	5731	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12813.8;
	5731	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2132.3;
	5732	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1024.1;
	5732	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1025.9;
	5732	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1210.4;
	5732	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5291.5;
	5732	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14267.3;
	5732	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8895.8;
	5732	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12579.2;
	5732	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2058.4;
	5733	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1014.2;
	5733	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	991.4;
	5733	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1161;
	5733	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5067.5;
	5733	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13652.9;
	5733	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8396;
	5733	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12130.3;
	5733	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1977.6;
	5734	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990.3;
	5734	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	933.1;
	5734	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1072.5;
	5734	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4754.9;
	5734	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12744.3;
	5734	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7744.8;
	5734	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11489.9;
	5734	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1857.9;
	5735	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.1;
	5735	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	877.4;
	5735	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	998;
	5735	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4440.1;
	5735	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11820;
	5735	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7056;
	5735	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10826.5;
	5735	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1715;
	5736	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.3;
	5736	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	821.2;
	5736	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	929.9;
	5736	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4104.2;
	5736	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10948.5;
	5736	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6440.4;
	5736	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10169.7;
	5736	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1590.3;
	5737	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.1;
	5737	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	782.7;
	5737	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.5;
	5737	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3852.5;
	5737	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10249.7;
	5737	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5982.7;
	5737	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9677.3;
	5737	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1514.9;
	5738	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.4;
	5738	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.4;
	5738	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	855.3;
	5738	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3661.2;
	5738	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9713.4;
	5738	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5692.8;
	5738	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9313.4;
	5738	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1531.1;
	5739	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.6;
	5739	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735.5;
	5739	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	839;
	5739	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3543.7;
	5739	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9457.6;
	5739	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5470.4;
	5739	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9155.4;
	5739	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1423.4;
	5740	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.1;
	5740	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.6;
	5740	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	823.3;
	5740	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3450.9;
	5740	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9272.1;
	5740	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5331.8;
	5740	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9100.2;
	5740	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1459.2;
	5741	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.3;
	5741	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.5;
	5741	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	823.1;
	5741	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3428.9;
	5741	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9332.5;
	5741	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5355;
	5741	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9171.7;
	5741	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1508.8;
	5742	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.1;
	5742	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.6;
	5742	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	834.6;
	5742	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3477.7;
	5742	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9610.1;
	5742	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5501.9;
	5742	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9365;
	5742	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1533.3;
	5743	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.4;
	5743	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742.9;
	5743	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	846.4;
	5743	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3464.6;
	5743	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9842.1;
	5743	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5592.3;
	5743	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9541.5;
	5743	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1558;
	5744	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.6;
	5744	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	790.9;
	5744	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897.3;
	5744	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3750.1;
	5744	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10609.8;
	5744	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6131.3;
	5744	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9984.7;
	5744	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1671.1;
	5745	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953;
	5745	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	857.8;
	5745	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	965.6;
	5745	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4198.7;
	5745	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11757.2;
	5745	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6799.7;
	5745	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10680.8;
	5745	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1808.5;
	5746	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	995.4;
	5746	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	924.3;
	5746	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1040.4;
	5746	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4639.1;
	5746	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12942.8;
	5746	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7473.9;
	5746	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11357.2;
	5746	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1965.9;
	5747	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1032.9;
	5747	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	984.3;
	5747	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1121.8;
	5747	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5043.2;
	5747	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14147.6;
	5747	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8061;
	5747	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12007.8;
	5747	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2080.9;
	5748	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1069;
	5748	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1048.4;
	5748	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1214;
	5748	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5370.8;
	5748	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15261.8;
	5748	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8584.6;
	5748	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12677.6;
	5748	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2213.1;
	5749	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1095.8;
	5749	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1101.1;
	5749	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1280.7;
	5749	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5632.9;
	5749	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16153.3;
	5749	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9031.6;
	5749	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13263.1;
	5749	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2333.1;
	5750	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1122.5;
	5750	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1148.3;
	5750	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1317.8;
	5750	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5796.1;
	5750	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16835.2;
	5750	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9386.3;
	5750	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13729.8;
	5750	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2423.1;
	5751	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1138.7;
	5751	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1176.3;
	5751	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1312.8;
	5751	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5898.8;
	5751	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17218.2;
	5751	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9682.5;
	5751	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14140.2;
	5751	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2492.7;
	5752	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1145.2;
	5752	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1173.8;
	5752	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1299.2;
	5752	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5944.9;
	5752	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17478.4;
	5752	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9844.6;
	5752	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14334.1;
	5752	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2457.2;
	5753	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1144.1;
	5753	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1135.7;
	5753	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1283.3;
	5753	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5864.2;
	5753	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17411.5;
	5753	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9846.4;
	5753	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14113.4;
	5753	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2437.4;
	5754	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1129.8;
	5754	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1086.7;
	5754	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1249;
	5754	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5673.2;
	5754	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16872;
	5754	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9595;
	5754	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13315.2;
	5754	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2335.6;
	5755	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1104.5;
	5755	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1029.9;
	5755	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1195.2;
	5755	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5397.3;
	5755	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15981;
	5755	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9132.8;
	5755	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12670;
	5755	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2246.4;
	5756	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1085.2;
	5756	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1008.2;
	5756	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1161.3;
	5756	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5295.7;
	5756	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15478.4;
	5756	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8870.9;
	5756	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12550.4;
	5756	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2166;
	5757	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1069.1;
	5757	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	966.2;
	5757	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1113.9;
	5757	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5065.2;
	5757	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14699.3;
	5757	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8431.5;
	5757	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12132.3;
	5757	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2006.4;
	5758	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1038.5;
	5758	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	913;
	5758	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1048.3;
	5758	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4788.6;
	5758	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13668.8;
	5758	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7812.9;
	5758	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11552.1;
	5758	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1871.5;
	5759	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1002.4;
	5759	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	848.5;
	5759	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	979.2;
	5759	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4471.2;
	5759	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12576.1;
	5759	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7154.6;
	5759	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10888.3;
	5759	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1730.3;
	5760	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969.2;
	5760	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	800.2;
	5760	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	921.8;
	5760	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4165.8;
	5760	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11606.8;
	5760	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6557.5;
	5760	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10287.9;
	5760	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1645.4;
	5761	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943;
	5761	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	766;
	5761	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	881.1;
	5761	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3911.4;
	5761	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10847;
	5761	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6085;
	5761	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9846.4;
	5761	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1543.8;
	5762	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.8;
	5762	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.9;
	5762	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849.4;
	5762	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3728.4;
	5762	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10289.2;
	5762	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5745.5;
	5762	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9537.9;
	5762	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1467.4;
	5763	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910;
	5763	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	721;
	5763	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	826;
	5763	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3589.1;
	5763	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9879.4;
	5763	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5477;
	5763	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9360.1;
	5763	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1441.7;
	5764	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.9;
	5764	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	713.3;
	5764	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	812.6;
	5764	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3476.4;
	5764	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9591.7;
	5764	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5307.1;
	5764	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9238.7;
	5764	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1436.3;
	5765	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.2;
	5765	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	715.2;
	5765	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	810.5;
	5765	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3426.3;
	5765	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9473.1;
	5765	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5240.8;
	5765	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9228.7;
	5765	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1463.9;
	5766	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.2;
	5766	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	726.5;
	5766	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	823.3;
	5766	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3435.6;
	5766	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9522.9;
	5766	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5295.3;
	5766	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9311.4;
	5766	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1498.7;
	5767	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.9;
	5767	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.9;
	5767	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	832.7;
	5767	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3392.1;
	5767	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9611.9;
	5767	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5344;
	5767	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9324.9;
	5767	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1518.6;
	5768	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.3;
	5768	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	795.4;
	5768	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889.5;
	5768	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3648;
	5768	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10526.3;
	5768	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5794.5;
	5768	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9649.9;
	5768	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1598.1;
	5769	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.7;
	5769	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	865.5;
	5769	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	963.4;
	5769	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4097.6;
	5769	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11737.9;
	5769	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6377.5;
	5769	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10443.7;
	5769	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1733.9;
	5770	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.2;
	5770	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	930.5;
	5770	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1039.6;
	5770	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4546.8;
	5770	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12911.9;
	5770	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7039.7;
	5770	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11065.1;
	5770	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1858.9;
	5771	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1032.3;
	5771	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	994.9;
	5771	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1122.2;
	5771	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4973.3;
	5771	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14130.6;
	5771	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7709.9;
	5771	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11694.7;
	5771	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1997.4;
	5772	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1068.3;
	5772	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1043.8;
	5772	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1188.3;
	5772	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5315.3;
	5772	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15193;
	5772	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8268.3;
	5772	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12465.1;
	5772	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2130.6;
	5773	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1098.6;
	5773	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1092.2;
	5773	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1245.7;
	5773	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5579.6;
	5773	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15988.7;
	5773	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8707.9;
	5773	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13159.4;
	5773	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2259.7;
	5774	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1122;
	5774	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1121.9;
	5774	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1260.7;
	5774	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5757.4;
	5774	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16564.5;
	5774	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9080.7;
	5774	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13573.3;
	5774	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2370.5;
	5775	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1139.2;
	5775	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1141;
	5775	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1265.6;
	5775	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5854.5;
	5775	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16960.2;
	5775	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9408.8;
	5775	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13757.9;
	5775	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2418.5;
	5776	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1143.5;
	5776	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1162.8;
	5776	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1269.7;
	5776	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5883.2;
	5776	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17204.4;
	5776	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9542.8;
	5776	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13745.9;
	5776	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2396.7;
	5777	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1137.5;
	5777	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1163.5;
	5777	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1281.4;
	5777	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5853.7;
	5777	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17225.4;
	5777	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9599.1;
	5777	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13128.5;
	5777	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2308.8;
	5778	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1112.8;
	5778	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1134.4;
	5778	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1261.9;
	5778	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5696.2;
	5778	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16791.5;
	5778	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9446.1;
	5778	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12465.8;
	5778	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2232.6;
	5779	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1080.7;
	5779	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1084;
	5779	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1215.6;
	5779	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5488.3;
	5779	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16056.1;
	5779	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9030;
	5779	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12156.7;
	5779	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2132.2;
	5780	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1066.5;
	5780	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1064.8;
	5780	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1191.6;
	5780	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5464.6;
	5780	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15706.2;
	5780	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8804.2;
	5780	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12159;
	5780	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2056.4;
	5781	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1057.3;
	5781	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1012.3;
	5781	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1142.1;
	5781	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5239.2;
	5781	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14901;
	5781	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8263.8;
	5781	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11806.9;
	5781	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2007.8;
	5782	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1021.5;
	5782	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	932.4;
	5782	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1058;
	5782	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4857.6;
	5782	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13531.6;
	5782	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7501.5;
	5782	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11130.4;
	5782	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1895.3;
	5783	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	978.8;
	5783	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	853.7;
	5783	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	969;
	5783	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4433.7;
	5783	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12147.9;
	5783	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6715.9;
	5783	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10352.3;
	5783	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1749.7;
	5784	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946;
	5784	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	806.6;
	5784	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	904.9;
	5784	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4067.1;
	5784	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11091.9;
	5784	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6099.7;
	5784	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9798;
	5784	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1603.2;
	5785	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.2;
	5785	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	769.4;
	5785	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.8;
	5785	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3825.1;
	5785	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10365.2;
	5785	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5690.5;
	5785	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9444.2;
	5785	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1522.6;
	5786	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.9;
	5786	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744.9;
	5786	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	833.7;
	5786	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3646.8;
	5786	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9905.2;
	5786	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5452.2;
	5786	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9275.9;
	5786	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1483;
	5787	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901;
	5787	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.8;
	5787	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	814.1;
	5787	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3544.9;
	5787	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9643.9;
	5787	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5308.7;
	5787	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9263.3;
	5787	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1467.5;
	5788	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.9;
	5788	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.2;
	5788	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	811.4;
	5788	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3511.8;
	5788	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9728.8;
	5788	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5333.3;
	5788	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9437;
	5788	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1483.9;
	5789	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.6;
	5789	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	764;
	5789	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	845.2;
	5789	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3630.7;
	5789	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10287.3;
	5789	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5719.4;
	5789	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10006.3;
	5789	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1560.4;
	5790	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.9;
	5790	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	827.2;
	5790	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	915.6;
	5790	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3953;
	5790	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11312.5;
	5790	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6369.5;
	5790	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10738.9;
	5790	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1712.5;
	5791	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.5;
	5791	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	853.6;
	5791	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	934.5;
	5791	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3974.3;
	5791	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11636.1;
	5791	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6529.1;
	5791	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10885.8;
	5791	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1708.2;
	5792	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969.9;
	5792	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	893;
	5792	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	964.2;
	5792	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4073.2;
	5792	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12018.6;
	5792	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6615.7;
	5792	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11109.5;
	5792	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1822.3;
	5793	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	988.8;
	5793	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	945.6;
	5793	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1006.5;
	5793	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4370.1;
	5793	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12862.7;
	5793	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6915;
	5793	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11531.8;
	5793	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1966.6;
	5794	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1019;
	5794	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1011;
	5794	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1054.8;
	5794	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4740.1;
	5794	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14029.3;
	5794	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7214.2;
	5794	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12043.7;
	5794	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2126.1;
	5795	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1040.5;
	5795	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1073.4;
	5795	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1111.8;
	5795	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5102.2;
	5795	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15251;
	5795	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7550;
	5795	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12559.7;
	5795	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2295.4;
	5796	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1055.4;
	5796	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1127.5;
	5796	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1170.6;
	5796	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5378.6;
	5796	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16199.2;
	5796	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7870.2;
	5796	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12863.5;
	5796	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2391.6;
	5797	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1068.7;
	5797	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1180.4;
	5797	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1222.5;
	5797	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5629.3;
	5797	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16848.3;
	5797	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8222.8;
	5797	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12968.9;
	5797	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2492.4;
	5798	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1067.8;
	5798	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1222.9;
	5798	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1261.3;
	5798	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5797.5;
	5798	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16682.2;
	5798	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8640.2;
	5798	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12701.3;
	5798	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2500.7;
	5799	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1063.6;
	5799	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1245.7;
	5799	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1287.4;
	5799	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5852.9;
	5799	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16112.2;
	5799	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9035.9;
	5799	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12573.5;
	5799	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2543.7;
	5800	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1052.5;
	5800	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1258.4;
	5800	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1303.8;
	5800	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5908.1;
	5800	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15832.3;
	5800	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9374.1;
	5800	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12874.4;
	5800	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2502.7;
	5801	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1037.5;
	5801	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1244.6;
	5801	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1294.4;
	5801	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5828.6;
	5801	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15555.8;
	5801	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9508.7;
	5801	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13022;
	5801	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2405.3;
	5802	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1024.1;
	5802	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1210.5;
	5802	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1270.4;
	5802	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5638.7;
	5802	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15184.8;
	5802	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9126.2;
	5802	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12789.4;
	5802	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2337.6;
	5803	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.3;
	5803	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1162.9;
	5803	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1215.8;
	5803	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5418.1;
	5803	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14776.3;
	5803	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8532.1;
	5803	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12605.3;
	5803	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2266.6;
	5804	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1018.8;
	5804	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1144.1;
	5804	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1188.9;
	5804	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5411.8;
	5804	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14760.9;
	5804	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8389.9;
	5804	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12712.2;
	5804	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2238.4;
	5805	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1016.1;
	5805	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1076.4;
	5805	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1145.3;
	5805	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5190.1;
	5805	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14129.5;
	5805	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7939;
	5805	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12270.4;
	5805	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2117.1;
	5806	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981.6;
	5806	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	989.4;
	5806	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1059.9;
	5806	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4785;
	5806	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12884.5;
	5806	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7196.1;
	5806	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11499.5;
	5806	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1942.2;
	5807	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.5;
	5807	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	906.6;
	5807	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	966.9;
	5807	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4335.8;
	5807	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11642;
	5807	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6475.1;
	5807	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10622.2;
	5807	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1759.6;
	5808	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.6;
	5808	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	845.3;
	5808	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	911.3;
	5808	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3987.5;
	5808	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10734.8;
	5808	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5939.2;
	5808	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9940;
	5808	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1655.4;
	5809	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.7;
	5809	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	813.9;
	5809	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	871.7;
	5809	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3782.4;
	5809	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10130.8;
	5809	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5577;
	5809	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9545.2;
	5809	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1582.4;
	5810	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.9;
	5810	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	788.2;
	5810	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	844.4;
	5810	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3628.5;
	5810	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9697.1;
	5810	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5363.9;
	5810	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9279;
	5810	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1536.1;
	5811	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907;
	5811	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	771.2;
	5811	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	829.7;
	5811	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3533.3;
	5811	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9449.8;
	5811	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5246.6;
	5811	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9177.9;
	5811	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1504.4;
	5812	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.6;
	5812	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	765.7;
	5812	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	828;
	5812	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3503.3;
	5812	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9511.5;
	5812	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5278.2;
	5812	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9266.5;
	5812	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1523.6;
	5813	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.6;
	5813	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	789.3;
	5813	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	855.3;
	5813	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3623.8;
	5813	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10070.9;
	5813	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5580.9;
	5813	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9838.2;
	5813	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1588.9;
	5814	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.3;
	5814	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	845;
	5814	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	919.9;
	5814	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3956.5;
	5814	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11123.9;
	5814	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6254.4;
	5814	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10569;
	5814	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1737.1;
	5815	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.9;
	5815	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	866.9;
	5815	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	949;
	5815	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3992.5;
	5815	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11512.5;
	5815	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6461;
	5815	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10661.7;
	5815	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1795.1;
	5816	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	974.9;
	5816	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	905;
	5816	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	970.4;
	5816	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4007.6;
	5816	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11949.6;
	5816	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6533.1;
	5816	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10987.5;
	5816	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1892.6;
	5817	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1002.6;
	5817	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	954.5;
	5817	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1012.5;
	5817	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4190.8;
	5817	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12874.7;
	5817	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6826;
	5817	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11597.5;
	5817	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2030.7;
	5818	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1028.4;
	5818	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1026.3;
	5818	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1049.8;
	5818	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4373.8;
	5818	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14063.5;
	5818	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7196;
	5818	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12419.7;
	5818	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2209.4;
	5819	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1051;
	5819	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1099.8;
	5819	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1101;
	5819	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4565.8;
	5819	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15276.2;
	5819	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7621.2;
	5819	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13286.4;
	5819	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2361.8;
	5820	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1067.9;
	5820	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1158.3;
	5820	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1147.6;
	5820	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4765.4;
	5820	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16432.6;
	5820	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8046.7;
	5820	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14179.9;
	5820	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2533.4;
	5821	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1087.1;
	5821	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1203.1;
	5821	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1189.5;
	5821	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4928.6;
	5821	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17186.5;
	5821	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8412.6;
	5821	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14921.7;
	5821	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2674.2;
	5822	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1100.8;
	5822	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1235.9;
	5822	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1228.1;
	5822	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5079.9;
	5822	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17600;
	5822	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8690.6;
	5822	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15408.7;
	5822	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2745.6;
	5823	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1105.7;
	5823	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1241.8;
	5823	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1262.9;
	5823	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5083.9;
	5823	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17698.7;
	5823	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8785.4;
	5823	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15740.2;
	5823	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2796.7;
	5824	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1104.8;
	5824	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1226;
	5824	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1275.5;
	5824	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5064.9;
	5824	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17787;
	5824	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8871.9;
	5824	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15749.1;
	5824	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2767;
	5825	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1082.7;
	5825	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1204.5;
	5825	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1263.4;
	5825	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5013.5;
	5825	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17665.2;
	5825	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8928.5;
	5825	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15197.9;
	5825	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2744.8;
	5826	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1064.1;
	5826	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1172.6;
	5826	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1237.6;
	5826	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4903.9;
	5826	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17044.1;
	5826	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8818;
	5826	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14567.6;
	5826	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2675.9;
	5827	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1048.8;
	5827	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1133.4;
	5827	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1194.9;
	5827	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4777.5;
	5827	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16353.5;
	5827	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8600.2;
	5827	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14120.4;
	5827	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2560.1;
	5828	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1051;
	5828	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1110.7;
	5828	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1181.4;
	5828	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4855.5;
	5828	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16067;
	5828	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8578.8;
	5828	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13870.6;
	5828	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2510.5;
	5829	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1036.7;
	5829	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1049.8;
	5829	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1138.7;
	5829	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4716.6;
	5829	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15166.5;
	5829	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8146.6;
	5829	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13009.4;
	5829	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2339.9;
	5830	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.6;
	5830	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	963.8;
	5830	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1059.5;
	5830	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4359.7;
	5830	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13761.5;
	5830	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7417.4;
	5830	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12048.8;
	5830	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2143.4;
	5831	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.5;
	5831	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	889.4;
	5831	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	973.6;
	5831	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3982.4;
	5831	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12434.5;
	5831	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6624.2;
	5831	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11137;
	5831	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1945.8;
	5832	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.6;
	5832	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	823.7;
	5832	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	907.4;
	5832	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3690.7;
	5832	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11412.9;
	5832	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6042;
	5832	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10351;
	5832	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1827.6;
	5833	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.3;
	5833	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	789.6;
	5833	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.8;
	5833	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3513.1;
	5833	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10726.6;
	5833	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5652.8;
	5833	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9856.2;
	5833	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1740.6;
	5834	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.9;
	5834	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	770.5;
	5834	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	846.9;
	5834	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3392.6;
	5834	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10271.1;
	5834	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5405.3;
	5834	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9577.5;
	5834	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1671.6;
	5835	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.6;
	5835	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	758.9;
	5835	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	822.9;
	5835	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3324.6;
	5835	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9994.5;
	5835	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5313.4;
	5835	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9475.9;
	5835	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1542.4;
	5836	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899;
	5836	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	756.8;
	5836	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	815.5;
	5836	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3310.6;
	5836	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10026.7;
	5836	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5323.7;
	5836	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9557.1;
	5836	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1529.9;
	5837	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913;
	5837	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	785.4;
	5837	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	839.8;
	5837	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3447.6;
	5837	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10578.4;
	5837	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5654.3;
	5837	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10074.3;
	5837	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1600;
	5838	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.5;
	5838	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	846.4;
	5838	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	909.7;
	5838	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3808.1;
	5838	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11611.3;
	5838	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6286.5;
	5838	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10784.2;
	5838	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1833.2;
	5839	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	971.3;
	5839	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	865.2;
	5839	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	943.3;
	5839	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3829.9;
	5839	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11901.9;
	5839	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6432.9;
	5839	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10859.1;
	5839	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1876.1;
	5840	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	971.3;
	5840	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	898.6;
	5840	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	968.8;
	5840	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3878.5;
	5840	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12385.4;
	5840	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6608.1;
	5840	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11361.3;
	5840	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1957;
	5841	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	993.6;
	5841	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	966;
	5841	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1033.2;
	5841	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4059.6;
	5841	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13366.4;
	5841	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7136.2;
	5841	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12311;
	5841	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2081.8;
	5842	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1022.2;
	5842	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1046.4;
	5842	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1106;
	5842	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4316.2;
	5842	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14643.6;
	5842	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7803.5;
	5842	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13399.9;
	5842	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2244.5;
	5843	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1047.5;
	5843	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1121.6;
	5843	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1178.8;
	5843	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4652.3;
	5843	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15844.4;
	5843	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8458.6;
	5843	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14434.3;
	5843	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2423.3;
	5844	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1068.4;
	5844	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1180.8;
	5844	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1231.7;
	5844	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5028.6;
	5844	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16848.1;
	5844	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9019.8;
	5844	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15146;
	5844	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2586.4;
	5845	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1096.6;
	5845	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1226.8;
	5845	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1275.6;
	5845	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5362.2;
	5845	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17803.8;
	5845	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9475;
	5845	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15737.2;
	5845	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2760.3;
	5846	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1123.3;
	5846	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1260.3;
	5846	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1302.1;
	5846	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5597.8;
	5846	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18512.5;
	5846	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9726.9;
	5846	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16098;
	5846	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2831.5;
	5847	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1132.6;
	5847	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1270.9;
	5847	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1317.9;
	5847	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5691.3;
	5847	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18947;
	5847	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9951;
	5847	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16263.1;
	5847	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2856.5;
	5848	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1126.3;
	5848	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1265.5;
	5848	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1338.5;
	5848	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5722;
	5848	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19166.9;
	5848	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9993;
	5848	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16079.3;
	5848	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2850.6;
	5849	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1102.9;
	5849	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1245.9;
	5849	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1329.7;
	5849	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5634;
	5849	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19027.9;
	5849	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9934.2;
	5849	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15566;
	5849	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2810.6;
	5850	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1081.4;
	5850	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1193.4;
	5850	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1309;
	5850	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5446.2;
	5850	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18459.5;
	5850	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9711.9;
	5850	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14899.2;
	5850	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2723.7;
	5851	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1054.8;
	5851	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1128;
	5851	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1266.1;
	5851	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5240.5;
	5851	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17615.2;
	5851	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9314.9;
	5851	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14365.2;
	5851	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2606.9;
	5852	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1046.4;
	5852	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1101.7;
	5852	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1225.5;
	5852	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5227.4;
	5852	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17143.7;
	5852	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9142.1;
	5852	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14159.1;
	5852	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2542.3;
	5853	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030.5;
	5853	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1044.2;
	5853	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1164;
	5853	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5019.8;
	5853	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16072.7;
	5853	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8634;
	5853	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13449.2;
	5853	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2349.4;
	5854	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	998;
	5854	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	964.1;
	5854	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1074.1;
	5854	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4613.2;
	5854	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14622.6;
	5854	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7853.9;
	5854	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12476.1;
	5854	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2140;
	5855	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.4;
	5855	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	891.4;
	5855	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	987.6;
	5855	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4175.7;
	5855	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13108;
	5855	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7043.6;
	5855	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11486;
	5855	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1933.3;
	5856	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.8;
	5856	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	839.1;
	5856	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	916.8;
	5856	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3860.5;
	5856	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11916.2;
	5856	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6417.8;
	5856	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10679.6;
	5856	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1868.8;
	5857	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.9;
	5857	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	811.7;
	5857	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878.2;
	5857	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3669.4;
	5857	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11166.7;
	5857	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6003.8;
	5857	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10170.2;
	5857	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1699.1;
	5858	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.8;
	5858	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	793.9;
	5858	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	841.5;
	5858	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3499.2;
	5858	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10567.5;
	5858	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5718.8;
	5858	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9776.2;
	5858	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1684.9;
	5859	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890;
	5859	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777.7;
	5859	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	822;
	5859	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3405.9;
	5859	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10235.2;
	5859	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5541.5;
	5859	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9587.4;
	5859	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1645.7;
	5860	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.6;
	5860	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	778.2;
	5860	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	816.1;
	5860	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3373.7;
	5860	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10230.3;
	5860	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5526.9;
	5860	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9660.1;
	5860	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1638.6;
	5861	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.1;
	5861	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	794.7;
	5861	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	839.1;
	5861	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3489.7;
	5861	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10779.3;
	5861	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5807.3;
	5861	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10150.2;
	5861	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1706.4;
	5862	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.5;
	5862	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	848.8;
	5862	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	910.5;
	5862	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3831.9;
	5862	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11746.6;
	5862	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6427.8;
	5862	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10839.9;
	5862	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1858.2;
	5863	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.8;
	5863	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	861.5;
	5863	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	938.2;
	5863	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3833.3;
	5863	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11937.4;
	5863	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6584.6;
	5863	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10923.7;
	5863	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1883.2;
	5864	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.7;
	5864	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	886.7;
	5864	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	954.9;
	5864	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3978.2;
	5864	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12345.2;
	5864	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6778;
	5864	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11432.4;
	5864	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1984;
	5865	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	988.5;
	5865	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	935;
	5865	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1000.4;
	5865	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4343.1;
	5865	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13250.5;
	5865	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7396.9;
	5865	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12414.6;
	5865	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2127.6;
	5866	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009.5;
	5866	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	988.7;
	5866	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1055.1;
	5866	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4778.7;
	5866	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14326.1;
	5866	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8095.9;
	5866	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13502.1;
	5866	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2335.8;
	5867	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1026.1;
	5867	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1026.4;
	5867	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1114.6;
	5867	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5189;
	5867	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15283.1;
	5867	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8695.3;
	5867	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14479.1;
	5867	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2525.8;
	5868	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1035.6;
	5868	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1059.7;
	5868	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1173.5;
	5868	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5520.2;
	5868	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16048.5;
	5868	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9208.5;
	5868	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15198.9;
	5868	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2641.9;
	5869	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1037.3;
	5869	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1096.9;
	5869	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1210.5;
	5869	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5776.8;
	5869	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16648.8;
	5869	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9647.5;
	5869	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15772.2;
	5869	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2741.5;
	5870	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1039.4;
	5870	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1125.1;
	5870	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1222.8;
	5870	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5951.8;
	5870	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17154.5;
	5870	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10013.2;
	5870	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16203.5;
	5870	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2826.2;
	5871	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1048.9;
	5871	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1130.8;
	5871	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1233.4;
	5871	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5934.6;
	5871	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17517.3;
	5871	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10173.5;
	5871	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16435.7;
	5871	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2816.2;
	5872	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1054.6;
	5872	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1092.7;
	5872	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1227.1;
	5872	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5938.9;
	5872	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17513.2;
	5872	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10267.7;
	5872	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16512.7;
	5872	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2788.9;
	5873	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1056.3;
	5873	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1052.1;
	5873	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1189.8;
	5873	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5799.3;
	5873	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16855.4;
	5873	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10232.6;
	5873	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16329.5;
	5873	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2690.6;
	5874	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1049.8;
	5874	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1025.1;
	5874	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1147;
	5874	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5624;
	5874	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15806.9;
	5874	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9970.5;
	5874	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15747;
	5874	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2605.5;
	5875	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1039.3;
	5875	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	996.6;
	5875	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1112.9;
	5875	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5442.3;
	5875	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14971.3;
	5875	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9566.9;
	5875	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15137.5;
	5875	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2513.7;
	5876	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1039.5;
	5876	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	988.1;
	5876	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1107.3;
	5876	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5448.6;
	5876	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14682.1;
	5876	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9393.5;
	5876	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14818.2;
	5876	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2461.2;
	5877	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1024.6;
	5877	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	954.5;
	5877	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1075.2;
	5877	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5222.8;
	5877	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13929;
	5877	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8848.5;
	5877	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14051;
	5877	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2285.7;
	5878	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	993.7;
	5878	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	894.3;
	5878	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1009.7;
	5878	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4830.5;
	5878	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12758;
	5878	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8066.8;
	5878	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12993.8;
	5878	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2051.5;
	5879	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.9;
	5879	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	825.6;
	5879	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	949.5;
	5879	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4395.9;
	5879	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11547.3;
	5879	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7232.8;
	5879	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11976.6;
	5879	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1873.6;
	5880	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.5;
	5880	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	797.7;
	5880	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	905.3;
	5880	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4086.2;
	5880	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10716.9;
	5880	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6228.5;
	5880	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11324.6;
	5880	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1795.7;
	5881	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.5;
	5881	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	758.3;
	5881	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.4;
	5881	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3840.9;
	5881	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10049.1;
	5881	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5835.1;
	5881	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10707.5;
	5881	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1720.3;
	5882	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.2;
	5882	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742.6;
	5882	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	840.5;
	5882	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3666.3;
	5882	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9613.1;
	5882	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5562.9;
	5882	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10267.8;
	5882	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1648.6;
	5883	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.8;
	5883	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.5;
	5883	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	818.3;
	5883	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3567.2;
	5883	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9397.8;
	5883	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5417.2;
	5883	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10038.8;
	5883	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1599.8;
	5884	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.8;
	5884	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742.7;
	5884	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	808.4;
	5884	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3509.3;
	5884	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9459.6;
	5884	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5446.5;
	5884	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10048;
	5884	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1589.8;
	5885	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.6;
	5885	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	775.7;
	5885	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	827.5;
	5885	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3608.9;
	5885	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9988.2;
	5885	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5752.4;
	5885	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10518.4;
	5885	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1676.6;
	5886	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.2;
	5886	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	837;
	5886	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	886.5;
	5886	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3934.7;
	5886	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11041.2;
	5886	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6374.3;
	5886	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11085.5;
	5886	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1832.2;
	5887	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.9;
	5887	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	849.5;
	5887	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	912.2;
	5887	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3932.9;
	5887	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11389.2;
	5887	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6540.1;
	5887	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11071.4;
	5887	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1862.8;
	5888	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.6;
	5888	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	870.9;
	5888	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	927;
	5888	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4076;
	5888	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11657.8;
	5888	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6633.4;
	5888	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11549.5;
	5888	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1938;
	5889	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986.6;
	5889	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	902.8;
	5889	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	973;
	5889	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4444.9;
	5889	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12312.3;
	5889	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6969.6;
	5889	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12432.2;
	5889	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2038.6;
	5890	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1014.4;
	5890	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	938.5;
	5890	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1018.5;
	5890	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4871.4;
	5890	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13113.2;
	5890	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7469.7;
	5890	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13398.1;
	5890	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2151.1;
	5891	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1045.3;
	5891	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	969.5;
	5891	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1065.4;
	5891	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5290.6;
	5891	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13946.7;
	5891	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7925.6;
	5891	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14367.7;
	5891	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2300.9;
	5892	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1069.5;
	5892	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1013.6;
	5892	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1106.4;
	5892	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5613.9;
	5892	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14831.8;
	5892	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8348.6;
	5892	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15297.9;
	5892	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2452.2;
	5893	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1095.6;
	5893	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1053.4;
	5893	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1153.3;
	5893	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5891.9;
	5893	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15653.8;
	5893	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8786.2;
	5893	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15934;
	5893	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2586.9;
	5894	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1111.1;
	5894	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1087;
	5894	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1195.9;
	5894	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6067.2;
	5894	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16121.3;
	5894	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8949.3;
	5894	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16332.3;
	5894	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2654.8;
	5895	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1122.9;
	5895	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1102.5;
	5895	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1224.4;
	5895	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6110.7;
	5895	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16342.1;
	5895	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8778.4;
	5895	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16494.6;
	5895	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2657.1;
	5896	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1132.2;
	5896	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1108.7;
	5896	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1232.6;
	5896	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6035.8;
	5896	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16565.7;
	5896	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8594.8;
	5896	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16235.8;
	5896	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2652.5;
	5897	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1125.1;
	5897	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1101.3;
	5897	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1216.2;
	5897	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5864.6;
	5897	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16411.7;
	5897	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8501.5;
	5897	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15406.7;
	5897	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2574.7;
	5898	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1099;
	5898	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1069.3;
	5898	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1174.9;
	5898	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5614.7;
	5898	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15688.4;
	5898	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8422.9;
	5898	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14389.3;
	5898	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2481.6;
	5899	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1071.5;
	5899	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1023.8;
	5899	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1126.5;
	5899	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5343.4;
	5899	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14866.2;
	5899	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8109.8;
	5899	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13815;
	5899	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2350.5;
	5900	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1060.1;
	5900	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1006.3;
	5900	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1104.3;
	5900	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5281.5;
	5900	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14523.5;
	5900	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7930.5;
	5900	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13574.6;
	5900	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2261.9;
	5901	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1040.7;
	5901	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	959.5;
	5901	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1058.8;
	5901	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5047.7;
	5901	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13794.8;
	5901	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7542.6;
	5901	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12975.1;
	5901	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2191.8;
	5902	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.2;
	5902	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	904.1;
	5902	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	992;
	5902	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4740.9;
	5902	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12804.4;
	5902	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7026.5;
	5902	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12257.6;
	5902	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2020.2;
	5903	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.3;
	5903	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	847.8;
	5903	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	924.8;
	5903	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4411;
	5903	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11774.7;
	5903	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6438.4;
	5903	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11480.1;
	5903	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1868.5;
	5904	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941;
	5904	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	786.9;
	5904	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	887.1;
	5904	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4059.5;
	5904	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10672.3;
	5904	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6135.8;
	5904	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10638.6;
	5904	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1759.1;
	5905	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917;
	5905	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742.7;
	5905	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	844.3;
	5905	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3814.9;
	5905	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9983.3;
	5905	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5742.2;
	5905	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10097.5;
	5905	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1656.3;
	5906	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.2;
	5906	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	715.9;
	5906	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	814.7;
	5906	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3645.3;
	5906	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9479.4;
	5906	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5454.9;
	5906	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9710.2;
	5906	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1580;
	5907	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.5;
	5907	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	696.6;
	5907	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	790.4;
	5907	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3516.8;
	5907	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9167.2;
	5907	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5263.1;
	5907	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9398.6;
	5907	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1538.9;
	5908	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.6;
	5908	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	685.9;
	5908	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	777.4;
	5908	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3432.2;
	5908	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8981.1;
	5908	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5153.2;
	5908	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9240.8;
	5908	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1489.9;
	5909	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.3;
	5909	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	688.2;
	5909	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	782.4;
	5909	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3423.3;
	5909	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8995.5;
	5909	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5186.2;
	5909	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9271.5;
	5909	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1478.7;
	5910	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.5;
	5910	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	699.9;
	5910	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	796.7;
	5910	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3467.9;
	5910	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9189.7;
	5910	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5336.8;
	5910	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9385;
	5910	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1502.9;
	5911	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.5;
	5911	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	705.8;
	5911	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	808.8;
	5911	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3453.1;
	5911	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9331.7;
	5911	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5453.7;
	5911	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9409.3;
	5911	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1529.1;
	5912	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.7;
	5912	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	748.2;
	5912	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.6;
	5912	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3712.5;
	5912	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10039.8;
	5912	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5900.4;
	5912	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10043.6;
	5912	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1657.9;
	5913	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931;
	5913	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	799.7;
	5913	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	917.6;
	5913	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4158.9;
	5913	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11019;
	5913	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6565.3;
	5913	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10946.5;
	5913	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1798.9;
	5914	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.7;
	5914	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	859.9;
	5914	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	981.3;
	5914	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4602.9;
	5914	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12026.8;
	5914	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7256.1;
	5914	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11922.8;
	5914	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1944.3;
	5915	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	984.1;
	5915	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	915.6;
	5915	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1048.5;
	5915	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5005.1;
	5915	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13049.5;
	5915	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7899.5;
	5915	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12870.7;
	5915	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2091.5;
	5916	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1011.5;
	5916	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	971.1;
	5916	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1112.7;
	5916	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5318.4;
	5916	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14019.9;
	5916	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8403.2;
	5916	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13755;
	5916	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2214;
	5917	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1042.9;
	5917	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1019.9;
	5917	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1169.5;
	5917	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5568.8;
	5917	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14847.9;
	5917	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8780.1;
	5917	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14393.3;
	5917	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2317.2;
	5918	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1068.9;
	5918	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1055.3;
	5918	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1231.2;
	5918	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5698.2;
	5918	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15462.2;
	5918	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9108.2;
	5918	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14725.9;
	5918	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2357.5;
	5919	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1084.6;
	5919	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1077.3;
	5919	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1296.1;
	5919	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5577.4;
	5919	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15850.5;
	5919	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9356.7;
	5919	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14930.4;
	5919	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2427.3;
	5920	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1094.1;
	5920	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1092.8;
	5920	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1314;
	5920	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5376.3;
	5920	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16045.4;
	5920	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9533.2;
	5920	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15008.6;
	5920	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2469;
	5921	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1095;
	5921	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1090.1;
	5921	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1325.6;
	5921	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5227.6;
	5921	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15979.5;
	5921	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9516.6;
	5921	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14799.4;
	5921	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2451.6;
	5922	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1085.5;
	5922	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1060.2;
	5922	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1300.3;
	5922	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5054.4;
	5922	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15430.3;
	5922	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9241.9;
	5922	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14253;
	5922	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2386.3;
	5923	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1065.5;
	5923	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1012.2;
	5923	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1244.6;
	5923	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4890.3;
	5923	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14581.6;
	5923	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8772;
	5923	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13724.2;
	5923	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2247.2;
	5924	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1053.5;
	5924	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	985;
	5924	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1212.8;
	5924	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4898.3;
	5924	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14149.5;
	5924	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8507.8;
	5924	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13436.2;
	5924	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2210.9;
	5925	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1035.8;
	5925	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	939.6;
	5925	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1159.6;
	5925	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4701.3;
	5925	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13396.4;
	5925	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8061;
	5925	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12875.1;
	5925	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2067;
	5926	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1007.1;
	5926	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	888.4;
	5926	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1090.1;
	5926	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4425.8;
	5926	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12406.5;
	5926	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7479;
	5926	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12182;
	5926	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1950.8;
	5927	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.3;
	5927	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	829.4;
	5927	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	993.1;
	5927	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4137.5;
	5927	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11411.5;
	5927	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6872.6;
	5927	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11427.8;
	5927	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1797.8;
	5928	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.9;
	5928	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776;
	5928	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	933.1;
	5928	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3852.8;
	5928	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10475.4;
	5928	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6314.4;
	5928	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10712.8;
	5928	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1674.1;
	5929	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.3;
	5929	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741;
	5929	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	886.7;
	5929	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3652;
	5929	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9753.8;
	5929	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5904.3;
	5929	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10168.8;
	5929	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1556.8;
	5930	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920.5;
	5930	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714.8;
	5930	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.6;
	5930	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3509.7;
	5930	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9244.9;
	5930	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5605.6;
	5930	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9776;
	5930	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1482.8;
	5931	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907;
	5931	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	694.7;
	5931	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851.1;
	5931	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3404.2;
	5931	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8881.1;
	5931	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5417.3;
	5931	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9469.1;
	5931	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1447.9;
	5932	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.4;
	5932	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	685.9;
	5932	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	853.3;
	5932	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3339.1;
	5932	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8626.6;
	5932	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5290.1;
	5932	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9287.1;
	5932	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1423.7;
	5933	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.3;
	5933	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	683.2;
	5933	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.7;
	5933	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3312.2;
	5933	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8601.2;
	5933	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5288;
	5933	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9217.9;
	5933	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1420.1;
	5934	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.3;
	5934	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	688.7;
	5934	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	856.8;
	5934	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3328.4;
	5934	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8733.2;
	5934	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5397.7;
	5934	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9195.1;
	5934	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1425.7;
	5935	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.1;
	5935	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	701.9;
	5935	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	864.4;
	5935	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3302.4;
	5935	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8836.8;
	5935	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5467.2;
	5935	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9153.3;
	5935	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1447.6;
	5936	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.8;
	5936	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742.9;
	5936	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	904.5;
	5936	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3572.6;
	5936	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9637.4;
	5936	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5882.4;
	5936	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9794.6;
	5936	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1602.7;
	5937	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.6;
	5937	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	808;
	5937	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	965.3;
	5937	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4028.8;
	5937	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10775.2;
	5937	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6519.5;
	5937	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10861.6;
	5937	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1799.8;
	5938	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.2;
	5938	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	863;
	5938	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1025.7;
	5938	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4450.6;
	5938	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11938.7;
	5938	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7159.1;
	5938	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12053.8;
	5938	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1955.5;
	5939	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1006.2;
	5939	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	925;
	5939	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1086.6;
	5939	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4826.9;
	5939	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13108.6;
	5939	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7749.8;
	5939	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13091.9;
	5939	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2156.8;
	5940	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1034.3;
	5940	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	987.2;
	5940	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1144.1;
	5940	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5128;
	5940	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14231.4;
	5940	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8278.9;
	5940	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13914.9;
	5940	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2328.4;
	5941	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1061.2;
	5941	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1047.3;
	5941	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1199;
	5941	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5174.5;
	5941	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15185.3;
	5941	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8751.6;
	5941	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14459.2;
	5941	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2460.7;
	5942	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1088;
	5942	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1084.3;
	5942	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1225.7;
	5942	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5041.2;
	5942	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15888.8;
	5942	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9137.8;
	5942	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14668.8;
	5942	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2533.6;
	5943	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1106;
	5943	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1111.8;
	5943	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1265.5;
	5943	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4993.3;
	5943	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16414;
	5943	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9390.6;
	5943	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14482;
	5943	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2551.8;
	5944	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1118.8;
	5944	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1134.1;
	5944	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1285.4;
	5944	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4921.7;
	5944	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16678.9;
	5944	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9541.3;
	5944	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14322.1;
	5944	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2580.9;
	5945	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1118.3;
	5945	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1132.9;
	5945	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1293.9;
	5945	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4773.9;
	5945	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16646.4;
	5945	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9549.8;
	5945	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13916.9;
	5945	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2635.3;
	5946	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1107.4;
	5946	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1105.7;
	5946	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1268.3;
	5946	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4624.9;
	5946	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16201.9;
	5946	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9008.1;
	5946	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13320.5;
	5946	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2474.7;
	5947	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1080.2;
	5947	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1051.9;
	5947	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1206;
	5947	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4458.4;
	5947	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15423;
	5947	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8410.8;
	5947	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12856.1;
	5947	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2427.5;
	5948	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1063.1;
	5948	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1032.5;
	5948	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1174.6;
	5948	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4537.1;
	5948	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14996.2;
	5948	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8110.8;
	5948	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12638.2;
	5948	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2366.6;
	5949	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1044;
	5949	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	982.2;
	5949	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1125.1;
	5949	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4411.2;
	5949	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14304.2;
	5949	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7683.3;
	5949	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12166.7;
	5949	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2249.9;
	5950	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.1;
	5950	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	928.1;
	5950	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1055.7;
	5950	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4190.8;
	5950	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13352.1;
	5950	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7138.4;
	5950	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11583.8;
	5950	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2122.7;
	5951	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.3;
	5951	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	878.8;
	5951	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	987.3;
	5951	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3916.5;
	5951	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12345.4;
	5951	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6595.9;
	5951	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10972.6;
	5951	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1966.9;
	5952	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954;
	5952	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	833.5;
	5952	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	926.5;
	5952	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3666.9;
	5952	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11474.2;
	5952	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6056.2;
	5952	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10315.6;
	5952	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1833.3;
	5953	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.3;
	5953	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	793.4;
	5953	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	881.4;
	5953	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3487.2;
	5953	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10749.5;
	5953	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5651.4;
	5953	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9932.5;
	5953	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1718.4;
	5954	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920.3;
	5954	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	765.2;
	5954	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	850.6;
	5954	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3373.2;
	5954	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10282.1;
	5954	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5370.2;
	5954	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9730.4;
	5954	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1645.4;
	5955	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.1;
	5955	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749.4;
	5955	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851.5;
	5955	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3308.7;
	5955	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9956.1;
	5955	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5211.2;
	5955	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9579.6;
	5955	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1619.3;
	5956	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.9;
	5956	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.6;
	5956	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.5;
	5956	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3308.5;
	5956	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9801.9;
	5956	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5144.4;
	5956	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9557;
	5956	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1617.7;
	5957	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.6;
	5957	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744.8;
	5957	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.7;
	5957	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3400.7;
	5957	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9880.6;
	5957	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5266;
	5957	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9659.6;
	5957	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1633.7;
	5958	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.5;
	5958	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	756.7;
	5958	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	874.4;
	5958	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3574.8;
	5958	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10068.2;
	5958	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5488.8;
	5958	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9731.9;
	5958	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1664.5;
	5959	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.2;
	5959	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	766.8;
	5959	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	875.8;
	5959	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3578.6;
	5959	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10052;
	5959	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5581.6;
	5959	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9687.7;
	5959	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1669.8;
	5960	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.2;
	5960	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	810.3;
	5960	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	910.7;
	5960	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3795;
	5960	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10743.1;
	5960	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5874.6;
	5960	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10404.2;
	5960	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1805.6;
	5961	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.5;
	5961	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	886.4;
	5961	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	974.5;
	5961	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4245.7;
	5961	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12018.3;
	5961	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6439.3;
	5961	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11627.9;
	5961	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1979;
	5962	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	996.3;
	5962	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	963;
	5962	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1046;
	5962	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4654.6;
	5962	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13404.9;
	5962	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7122.2;
	5962	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12781.9;
	5962	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2149.4;
	5963	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030.2;
	5963	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1031.1;
	5963	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1117.1;
	5963	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5009.5;
	5963	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14630.7;
	5963	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7813.4;
	5963	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13485.4;
	5963	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2332.2;
	5964	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1058.3;
	5964	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1092.9;
	5964	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1180.6;
	5964	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5329;
	5964	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15702.2;
	5964	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8399.1;
	5964	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13887.2;
	5964	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2489.7;
	5965	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1083;
	5965	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1148;
	5965	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1226.7;
	5965	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5581;
	5965	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16516.5;
	5965	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8859.9;
	5965	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14101.1;
	5965	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2608.2;
	5966	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1105.7;
	5966	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1187.3;
	5966	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1244.6;
	5966	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5722.7;
	5966	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17108.1;
	5966	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9167.3;
	5966	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14015.4;
	5966	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2623.4;
	5967	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1121.6;
	5967	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1216.5;
	5967	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1282.6;
	5967	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5795.6;
	5967	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17495.5;
	5967	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9258.8;
	5967	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14109.3;
	5967	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2592;
	5968	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1128;
	5968	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1229.6;
	5968	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1299.3;
	5968	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5817.4;
	5968	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17698.3;
	5968	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9239.5;
	5968	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14353.1;
	5968	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2554.7;
	5969	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1125.2;
	5969	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1221.2;
	5969	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1302.1;
	5969	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5694.2;
	5969	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17614.7;
	5969	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9280.1;
	5969	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14475;
	5969	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2552.6;
	5970	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1112.9;
	5970	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1188.4;
	5970	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1279.8;
	5970	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5517.9;
	5970	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17196;
	5970	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9168.6;
	5970	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14228.3;
	5970	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2386.9;
	5971	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1091.3;
	5971	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1135;
	5971	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1229;
	5971	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5318.4;
	5971	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16513.9;
	5971	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8928.4;
	5971	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13850.1;
	5971	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2367.5;
	5972	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1084.6;
	5972	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1113.4;
	5972	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1207.3;
	5972	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5367.5;
	5972	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16224.9;
	5972	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8866;
	5972	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13794.7;
	5972	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2323;
	5973	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1066.6;
	5973	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1048.5;
	5973	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1154.3;
	5973	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5143.5;
	5973	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15345.9;
	5973	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8403.7;
	5973	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13231.4;
	5973	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2214;
	5974	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1026.3;
	5974	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	975.8;
	5974	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1073.1;
	5974	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4773.1;
	5974	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13906.1;
	5974	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7667.9;
	5974	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12279.8;
	5974	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2053.4;
	5975	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	984;
	5975	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	914.8;
	5975	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	994.5;
	5975	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4365.7;
	5975	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12568.7;
	5975	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6934.4;
	5975	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11354.1;
	5975	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1879.5;
	5976	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956;
	5976	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	859.9;
	5976	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	949.5;
	5976	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4053.6;
	5976	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11475.4;
	5976	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6358.3;
	5976	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10616.4;
	5976	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1773.9;
	5977	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940;
	5977	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	828.4;
	5977	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	914.4;
	5977	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3857.5;
	5977	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10794.2;
	5977	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5990.3;
	5977	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10171.8;
	5977	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1696.2;
	5978	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.6;
	5978	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	805.8;
	5978	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	894.2;
	5978	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3713.5;
	5978	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10369.2;
	5978	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5775.9;
	5978	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9906.7;
	5978	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1651;
	5979	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.9;
	5979	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	793.3;
	5979	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.7;
	5979	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3608.7;
	5979	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10179.1;
	5979	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5676;
	5979	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9791.3;
	5979	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1621;
	5980	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.7;
	5980	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	788.9;
	5980	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	880.5;
	5980	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3578;
	5980	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10280.3;
	5980	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5742.6;
	5980	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9866.1;
	5980	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1642.5;
	5981	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.7;
	5981	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	815.6;
	5981	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	906.9;
	5981	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3712.1;
	5981	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10870.9;
	5981	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6128;
	5981	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10396.6;
	5981	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1715;
	5982	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	973.8;
	5982	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	872.5;
	5982	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	972.8;
	5982	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4039.2;
	5982	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11946.2;
	5982	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6782.2;
	5982	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11033.2;
	5982	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1878.1;
	5983	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.3;
	5983	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	893.1;
	5983	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	998.9;
	5983	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4067.9;
	5983	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12315.3;
	5983	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6989.5;
	5983	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11110.1;
	5983	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1944.8;
	5984	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.3;
	5984	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	915.3;
	5984	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1016.7;
	5984	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4217.4;
	5984	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12576;
	5984	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7071.1;
	5984	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11680.1;
	5984	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2029.8;
	5985	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1028.2;
	5985	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	973.9;
	5985	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1059.8;
	5985	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4610.4;
	5985	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13280.1;
	5985	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7422.8;
	5985	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12659.2;
	5985	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2164.2;
	5986	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1059.5;
	5986	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1041.6;
	5986	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1112.7;
	5986	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4976.7;
	5986	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14189.7;
	5986	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7941;
	5986	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13684.4;
	5986	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2321.7;
	5987	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1092.5;
	5987	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1095.5;
	5987	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1177.3;
	5987	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5310.7;
	5987	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15153.1;
	5987	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8488.7;
	5987	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14689.4;
	5987	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2457.6;
	5988	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1116.4;
	5988	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1150.6;
	5988	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1234.7;
	5988	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5593.2;
	5988	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16127.2;
	5988	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9010.1;
	5988	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15518.8;
	5988	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2604.8;
	5989	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1138;
	5989	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1204.1;
	5989	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1291.2;
	5989	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5810.6;
	5989	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17097.9;
	5989	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9478.8;
	5989	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16168;
	5989	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2744.4;
	5990	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1154.5;
	5990	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1243.9;
	5990	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1335.4;
	5990	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6000.5;
	5990	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17848.7;
	5990	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9817.2;
	5990	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16456.3;
	5990	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2826.5;
	5991	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1162.6;
	5991	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1263;
	5991	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1368.3;
	5991	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6083.6;
	5991	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18344.3;
	5991	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10009.7;
	5991	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16567.8;
	5991	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2885.8;
	5992	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1167.6;
	5992	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1271.2;
	5992	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1381.7;
	5992	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6106.6;
	5992	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18644.8;
	5992	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10019.7;
	5992	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16600;
	5992	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2877.1;
	5993	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1161.6;
	5993	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1261.7;
	5993	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1369.3;
	5993	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6003.7;
	5993	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18595.6;
	5993	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9992;
	5993	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16307.9;
	5993	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2855.5;
	5994	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1151.8;
	5994	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1229.1;
	5994	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1336.1;
	5994	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5810.4;
	5994	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18051.8;
	5994	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9833.2;
	5994	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15619.3;
	5994	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2791.4;
	5995	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1125.3;
	5995	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1182.2;
	5995	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1283.8;
	5995	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5618;
	5995	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17279;
	5995	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9460.6;
	5995	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15055.6;
	5995	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2686.3;
	5996	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1116.6;
	5996	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1157.9;
	5996	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1267.3;
	5996	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5634.4;
	5996	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16911.9;
	5996	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9284.7;
	5996	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14725;
	5996	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2637.2;
	5997	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1091.7;
	5997	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1095.9;
	5997	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1213;
	5997	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5416.4;
	5997	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15973.8;
	5997	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8786.9;
	5997	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13886.7;
	5997	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2464.5;
	5998	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1048.2;
	5998	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1019.2;
	5998	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1124.1;
	5998	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5024;
	5998	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14584.7;
	5998	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8032.5;
	5998	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12816.6;
	5998	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2274.5;
	5999	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1003.1;
	5999	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	944.6;
	5999	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1040.6;
	5999	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4606.9;
	5999	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13222.3;
	5999	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7266.8;
	5999	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11805;
	5999	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2075.5;
	6000	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	973.5;
	6000	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	885.5;
	6000	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	981;
	6000	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4276.8;
	6000	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12178.6;
	6000	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6698.9;
	6000	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10952.7;
	6000	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1936.1;
	6001	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.9;
	6001	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	851.2;
	6001	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	944.5;
	6001	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4056.7;
	6001	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11507.3;
	6001	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6331.9;
	6001	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10395.7;
	6001	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1836.9;
	6002	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.6;
	6002	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	833.7;
	6002	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	918.2;
	6002	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3889.6;
	6002	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11058.6;
	6002	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6137.1;
	6002	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10044.6;
	6002	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1762.2;
	6003	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.8;
	6003	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	817.9;
	6003	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	901.1;
	6003	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3795;
	6003	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10819.4;
	6003	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6030.2;
	6003	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9884.8;
	6003	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1625.2;
	6004	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941;
	6004	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	814.2;
	6004	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.7;
	6004	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3750;
	6004	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10822.9;
	6004	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6074.6;
	6004	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9950.4;
	6004	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1606.9;
	6005	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.6;
	6005	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	842.4;
	6005	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	915.1;
	6005	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3841.5;
	6005	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11313.5;
	6005	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6405.1;
	6005	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10480.2;
	6005	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1664.5;
	6006	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	983.7;
	6006	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	898.2;
	6006	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	983.1;
	6006	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4166.3;
	6006	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12343.7;
	6006	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7039;
	6006	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11161.4;
	6006	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1888.3;
	6007	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1004.1;
	6007	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	918.4;
	6007	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1016.9;
	6007	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4170.2;
	6007	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12609.3;
	6007	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7236.6;
	6007	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11239.3;
	6007	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1938.9;
	6008	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.1;
	6008	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	946.7;
	6008	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1033.5;
	6008	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4279.5;
	6008	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12963.8;
	6008	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7257.5;
	6008	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11810.3;
	6008	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2007.9;
	6009	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1042.8;
	6009	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	992.8;
	6009	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1083.1;
	6009	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4636.3;
	6009	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13908.8;
	6009	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7655.2;
	6009	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12871.8;
	6009	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2140.1;
	6010	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1073.7;
	6010	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1058.9;
	6010	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1140;
	6010	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5023;
	6010	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15037.3;
	6010	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8254.2;
	6010	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13914.2;
	6010	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2280.4;
	6011	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1102.3;
	6011	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1123;
	6011	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1195.7;
	6011	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5373;
	6011	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16062.7;
	6011	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8807;
	6011	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14810.6;
	6011	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2432.4;
	6012	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1134;
	6012	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1183.4;
	6012	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1250.3;
	6012	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5678.2;
	6012	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17054.6;
	6012	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9297.4;
	6012	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15578.2;
	6012	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2602.3;
	6013	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1167.2;
	6013	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1238.6;
	6013	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1308.8;
	6013	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5944.6;
	6013	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17973.6;
	6013	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9791.4;
	6013	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16165.3;
	6013	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2796.6;
	6014	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1192.4;
	6014	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1274.9;
	6014	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1353.7;
	6014	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6137.7;
	6014	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18616.5;
	6014	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10163.6;
	6014	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16146.4;
	6014	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2868.5;
	6015	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1194.7;
	6015	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1293.9;
	6015	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1383;
	6015	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6209.1;
	6015	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19019.2;
	6015	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10384.4;
	6015	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15421.5;
	6015	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2894.5;
	6016	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1194.1;
	6016	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1301.3;
	6016	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1414.4;
	6016	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6215.1;
	6016	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19250;
	6016	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10490;
	6016	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14950.7;
	6016	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2903.2;
	6017	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1180;
	6017	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1281.7;
	6017	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1412.7;
	6017	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6124;
	6017	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19111.6;
	6017	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10584.8;
	6017	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14789.8;
	6017	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2895;
	6018	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1157.7;
	6018	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1253;
	6018	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1386.1;
	6018	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5923;
	6018	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18544.8;
	6018	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10442.8;
	6018	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14499.6;
	6018	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2823.7;
	6019	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1109.9;
	6019	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1198.8;
	6019	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1326.2;
	6019	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5695.5;
	6019	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17684.9;
	6019	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9991.8;
	6019	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14227.2;
	6019	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2702.5;
	6020	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1081.6;
	6020	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1174.3;
	6020	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1300.7;
	6020	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5687;
	6020	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17284.6;
	6020	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9812.5;
	6020	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14169.5;
	6020	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2635.9;
	6021	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1060.1;
	6021	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1113.2;
	6021	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1231.1;
	6021	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5441;
	6021	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16291.4;
	6021	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9237.1;
	6021	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13666.7;
	6021	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2445.8;
	6022	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1020.8;
	6022	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1030.4;
	6022	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1135.1;
	6022	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4995.4;
	6022	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14899.1;
	6022	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8356.9;
	6022	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12763.4;
	6022	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2229.4;
	6023	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	983.9;
	6023	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	958.8;
	6023	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1044;
	6023	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4548.8;
	6023	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13549.1;
	6023	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7514.8;
	6023	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11834.8;
	6023	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2024;
	6024	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953;
	6024	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	909.7;
	6024	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	963.3;
	6024	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4226.7;
	6024	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12588.5;
	6024	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6679.7;
	6024	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11110.9;
	6024	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1984.5;
	6025	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.6;
	6025	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	874.7;
	6025	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	920.8;
	6025	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4001.1;
	6025	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11911.7;
	6025	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6231.6;
	6025	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10651.2;
	6025	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1819.1;
	6026	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.5;
	6026	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	852;
	6026	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	888.5;
	6026	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3810.9;
	6026	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11411.4;
	6026	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5950.6;
	6026	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10293.8;
	6026	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1812.2;
	6027	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.5;
	6027	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	834.5;
	6027	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.2;
	6027	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3691.8;
	6027	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11136.2;
	6027	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5808.1;
	6027	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10096.6;
	6027	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1775.7;
	6028	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.8;
	6028	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	834.9;
	6028	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.4;
	6028	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3651.6;
	6028	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11142.3;
	6028	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5860.1;
	6028	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10190.1;
	6028	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1777.1;
	6029	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.7;
	6029	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	850.7;
	6029	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	887.4;
	6029	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3756.8;
	6029	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11690.7;
	6029	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6152.6;
	6029	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10728.9;
	6029	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1840.5;
	6030	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.1;
	6030	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	914;
	6030	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	954.7;
	6030	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4078.2;
	6030	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12695;
	6030	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6743.2;
	6030	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11405.7;
	6030	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2000.8;
	6031	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	974.2;
	6031	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	923.2;
	6031	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	978.2;
	6031	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4067.3;
	6031	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12839;
	6031	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6906.9;
	6031	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11453.4;
	6031	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2028.2;
	6032	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985.2;
	6032	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	963.3;
	6032	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	998.9;
	6032	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4200.5;
	6032	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13174.7;
	6032	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6937.2;
	6032	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12035.7;
	6032	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2102.9;
	6033	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1015.5;
	6033	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1015.6;
	6033	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1052.3;
	6033	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4543;
	6033	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14054.4;
	6033	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7313.3;
	6033	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12997.9;
	6033	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2184.2;
	6034	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1053.3;
	6034	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1085.6;
	6034	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1117.6;
	6034	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4941.2;
	6034	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15112.7;
	6034	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7790.3;
	6034	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13887.5;
	6034	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2317.9;
	6035	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1097;
	6035	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1147.8;
	6035	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1192.3;
	6035	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5317.9;
	6035	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16257.5;
	6035	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8315.9;
	6035	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14759.8;
	6035	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2483.9;
	6036	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1131.7;
	6036	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1206.8;
	6036	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1261.6;
	6036	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5642.2;
	6036	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17343.1;
	6036	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8917.2;
	6036	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15517.9;
	6036	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2625.4;
	6037	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1167.3;
	6037	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1267;
	6037	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1329.3;
	6037	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5916.2;
	6037	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18376.1;
	6037	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9470.8;
	6037	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16195.6;
	6037	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2758.5;
	6038	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1197.2;
	6038	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1310.9;
	6038	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1388.3;
	6038	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6130;
	6038	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19101.9;
	6038	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9895.9;
	6038	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16541.7;
	6038	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2857.9;
	6039	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1206.4;
	6039	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1328.8;
	6039	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1428;
	6039	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6173.2;
	6039	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19383;
	6039	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10135;
	6039	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16411.8;
	6039	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2862;
	6040	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1209.8;
	6040	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1328.5;
	6040	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1449.6;
	6040	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6200.1;
	6040	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19490.7;
	6040	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10263.8;
	6040	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16123.7;
	6040	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2880.2;
	6041	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1203.8;
	6041	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1299.9;
	6041	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1436.6;
	6041	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6100.3;
	6041	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19367;
	6041	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10346.8;
	6041	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15560.6;
	6041	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2871;
	6042	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1179.1;
	6042	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1266.3;
	6042	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1399.4;
	6042	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5882.8;
	6042	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18653.1;
	6042	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10177.9;
	6042	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15111.9;
	6042	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2810.7;
	6043	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1152;
	6043	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1221.5;
	6043	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1343.3;
	6043	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5633.8;
	6043	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17721.8;
	6043	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9725.3;
	6043	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14641;
	6043	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2711.4;
	6044	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1136.6;
	6044	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1190.7;
	6044	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1312.6;
	6044	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5615.3;
	6044	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17225.7;
	6044	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9487.9;
	6044	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14401.6;
	6044	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2633.7;
	6045	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1109.7;
	6045	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1129.2;
	6045	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1246.5;
	6045	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5353.3;
	6045	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16230.4;
	6045	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8942.4;
	6045	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13825.8;
	6045	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2450.6;
	6046	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1070.4;
	6046	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1050.3;
	6046	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1153.9;
	6046	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4948;
	6046	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14804.1;
	6046	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8146.8;
	6046	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12796.4;
	6046	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2219.1;
	6047	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1029.1;
	6047	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	972.5;
	6047	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1071.5;
	6047	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4498.3;
	6047	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13414.1;
	6047	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7328.8;
	6047	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11749.1;
	6047	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2054.3;
	6048	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	998;
	6048	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	912.6;
	6048	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	997.6;
	6048	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4135;
	6048	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12298.2;
	6048	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6683.1;
	6048	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10921.1;
	6048	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1945.9;
	6049	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	971.3;
	6049	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	868.1;
	6049	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	948.9;
	6049	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3892.1;
	6049	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11551.8;
	6049	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6214.4;
	6049	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10357.9;
	6049	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1878.2;
	6050	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.1;
	6050	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	843.3;
	6050	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913.8;
	6050	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3725.5;
	6050	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11082;
	6050	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5960.5;
	6050	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9970.6;
	6050	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1801.1;
	6051	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.2;
	6051	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	826.4;
	6051	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	890;
	6051	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3618.9;
	6051	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10799.9;
	6051	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5832.8;
	6051	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9799.1;
	6051	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1754.6;
	6052	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.8;
	6052	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	825.4;
	6052	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	881;
	6052	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3570.8;
	6052	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10798.1;
	6052	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5872.2;
	6052	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9862;
	6052	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1734.4;
	6053	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.8;
	6053	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	851.2;
	6053	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897.3;
	6053	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3661.9;
	6053	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11232.6;
	6053	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6182.9;
	6053	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10389.8;
	6053	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1803.4;
	6054	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.4;
	6054	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	902.3;
	6054	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	952.6;
	6054	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3968.7;
	6054	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12199.2;
	6054	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6750.6;
	6054	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11038.9;
	6054	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1939.1;
	6055	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981.6;
	6055	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	911.3;
	6055	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	973.5;
	6055	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3978.3;
	6055	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12422.8;
	6055	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6891.3;
	6055	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11104.8;
	6055	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1955.4;
	6056	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	996.9;
	6056	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	947.9;
	6056	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	996.9;
	6056	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4113.2;
	6056	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12813.1;
	6056	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7026.1;
	6056	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11676.5;
	6056	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2042.2;
	6057	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1033.2;
	6057	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	994.6;
	6057	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1058.4;
	6057	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4470.1;
	6057	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13822.8;
	6057	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7466;
	6057	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12645.9;
	6057	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2194.4;
	6058	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1067.6;
	6058	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1054.1;
	6058	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1130.5;
	6058	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4855.1;
	6058	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14916.9;
	6058	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8002.4;
	6058	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13586.9;
	6058	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2337.2;
	6059	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1107.3;
	6059	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1119.8;
	6059	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1200.7;
	6059	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5235.3;
	6059	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15956.7;
	6059	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8527.8;
	6059	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14339.5;
	6059	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2491.5;
	6060	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1140.7;
	6060	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1186.1;
	6060	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1269.7;
	6060	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5554.5;
	6060	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16959.6;
	6060	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9041.5;
	6060	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14716.5;
	6060	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2616.8;
	6061	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1178.2;
	6061	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1246.5;
	6061	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1338.7;
	6061	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5849.6;
	6061	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17918.7;
	6061	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9547.1;
	6061	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14937.1;
	6061	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2721.4;
	6062	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1197.5;
	6062	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1288.6;
	6062	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1387.6;
	6062	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6088.8;
	6062	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18590.5;
	6062	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9951.5;
	6062	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14730.9;
	6062	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2787.4;
	6063	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1206;
	6063	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1307;
	6063	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1415.4;
	6063	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6205.7;
	6063	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19018.1;
	6063	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10254.4;
	6063	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14392.2;
	6063	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2768.3;
	6064	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1211;
	6064	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1305.5;
	6064	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1433.5;
	6064	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6231;
	6064	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19207.4;
	6064	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10429.2;
	6064	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14155.3;
	6064	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2758.9;
	6065	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1207.9;
	6065	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1279.5;
	6065	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1413.4;
	6065	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6094.9;
	6065	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18939.9;
	6065	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10399.1;
	6065	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13823.7;
	6065	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2672.1;
	6066	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1187.5;
	6066	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1234.9;
	6066	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1364.8;
	6066	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5853.3;
	6066	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18124.9;
	6066	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10102.9;
	6066	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13265.2;
	6066	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2570.3;
	6067	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1156.9;
	6067	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1178.3;
	6067	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1291.4;
	6067	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5565.1;
	6067	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17106.4;
	6067	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9598.5;
	6067	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12937.5;
	6067	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2435;
	6068	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1139.7;
	6068	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1142.4;
	6068	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1251;
	6068	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5473;
	6068	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16543.3;
	6068	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9304.2;
	6068	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12827.1;
	6068	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2328.9;
	6069	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1115.7;
	6069	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1093.8;
	6069	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1188.2;
	6069	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5231.9;
	6069	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15654;
	6069	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8811.7;
	6069	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12434.5;
	6069	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2234.7;
	6070	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1079.3;
	6070	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1039.6;
	6070	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1114.7;
	6070	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4931.6;
	6070	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14587.6;
	6070	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8196;
	6070	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11819.4;
	6070	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2067.1;
	6071	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1039.6;
	6071	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	972.3;
	6071	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1040.9;
	6071	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4588.9;
	6071	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13475.2;
	6071	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7503.1;
	6071	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11129;
	6071	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1919.8;
	6072	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	991.4;
	6072	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	928.3;
	6072	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	969.9;
	6072	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4275.1;
	6072	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12396.7;
	6072	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6856.6;
	6072	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10482.2;
	6072	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1814.3;
	6073	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.2;
	6073	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	877.5;
	6073	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	923.3;
	6073	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4040.1;
	6073	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11664.6;
	6073	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6394.8;
	6073	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10026.5;
	6073	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1726.1;
	6074	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948;
	6074	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	840.6;
	6074	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889;
	6074	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3867;
	6074	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11011.9;
	6074	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6039.9;
	6074	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9684.5;
	6074	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1645.9;
	6075	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.3;
	6075	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	795.9;
	6075	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	856.5;
	6075	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3720.8;
	6075	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10572.8;
	6075	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5806.7;
	6075	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9442.3;
	6075	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1605.9;
	6076	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920.9;
	6076	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	763;
	6076	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	837.7;
	6076	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3615;
	6076	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10234.8;
	6076	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5689.3;
	6076	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9320.8;
	6076	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1565.5;
	6077	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924;
	6077	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753;
	6077	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838.7;
	6077	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3594.4;
	6077	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9891.6;
	6077	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5715.8;
	6077	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9393.6;
	6077	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1551.3;
	6078	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925.5;
	6078	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	762.3;
	6078	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	853.7;
	6078	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3622.2;
	6078	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9932.5;
	6078	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5853.6;
	6078	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9593.8;
	6078	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1589.4;
	6079	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.3;
	6079	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	768.1;
	6079	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.3;
	6079	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3603.8;
	6079	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10021.5;
	6079	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5962.9;
	6079	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9694.4;
	6079	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1626.1;
	6080	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.5;
	6080	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	790.5;
	6080	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.9;
	6080	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3870;
	6080	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10318.6;
	6080	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6329.6;
	6080	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10231.1;
	6080	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1729.9;
	6081	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.6;
	6081	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	809.2;
	6081	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	927.1;
	6081	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4320;
	6081	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10801.1;
	6081	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6857.4;
	6081	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11201.3;
	6081	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1869.7;
	6082	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.7;
	6082	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	831.2;
	6082	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	957.4;
	6082	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4741.3;
	6082	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11313.4;
	6082	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7265.3;
	6082	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12159;
	6082	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1976.2;
	6083	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.1;
	6083	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	846.4;
	6083	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	979.9;
	6083	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5127.4;
	6083	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11680.4;
	6083	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7690.8;
	6083	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12830.8;
	6083	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2075.7;
	6084	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.8;
	6084	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	874.9;
	6084	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1008.1;
	6084	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5451.9;
	6084	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12184.5;
	6084	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7905.6;
	6084	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13108.5;
	6084	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2192.1;
	6085	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.9;
	6085	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	901.2;
	6085	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1037.4;
	6085	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5643.5;
	6085	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12777.9;
	6085	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8265.8;
	6085	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12906.2;
	6085	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2305.1;
	6086	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953;
	6086	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	925;
	6086	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1067.3;
	6086	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5714.8;
	6086	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13359.4;
	6086	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8727.7;
	6086	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12358.2;
	6086	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2351.9;
	6087	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.4;
	6087	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	953.3;
	6087	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1111.6;
	6087	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5706.7;
	6087	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13813.8;
	6087	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9127.7;
	6087	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12003;
	6087	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2413.7;
	6088	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.4;
	6088	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	979.4;
	6088	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1116.5;
	6088	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5581.3;
	6088	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14086.3;
	6088	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9335.6;
	6088	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12021.2;
	6088	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2414.4;
	6089	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.7;
	6089	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	987.7;
	6089	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1124.6;
	6089	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5383.3;
	6089	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14115.5;
	6089	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9310.2;
	6089	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12138.1;
	6089	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2364.9;
	6090	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966;
	6090	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	965.7;
	6090	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1105.1;
	6090	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5157;
	6090	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13707.4;
	6090	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9031.7;
	6090	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12072.9;
	6090	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2304.5;
	6091	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.4;
	6091	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	919.4;
	6091	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1081;
	6091	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4992.3;
	6091	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13056.2;
	6091	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8557.9;
	6091	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11915.7;
	6091	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2184.4;
	6092	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.6;
	6092	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	887.2;
	6092	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1076.1;
	6092	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4985.7;
	6092	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12770.8;
	6092	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8262.5;
	6092	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11863.8;
	6092	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2153.9;
	6093	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.1;
	6093	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	843.3;
	6093	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1041.8;
	6093	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4789.8;
	6093	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12113.5;
	6093	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7779.4;
	6093	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11567;
	6093	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2016.7;
	6094	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.9;
	6094	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	802.3;
	6094	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	982.1;
	6094	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4522.2;
	6094	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11235;
	6094	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7213.9;
	6094	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11081.2;
	6094	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1920.6;
	6095	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920.2;
	6095	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	750.1;
	6095	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	894.7;
	6095	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4230.9;
	6095	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10292.6;
	6095	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6611.9;
	6095	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10483.2;
	6095	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1771.2;
	6096	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.1;
	6096	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	702.8;
	6096	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838.2;
	6096	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3935.8;
	6096	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9425.6;
	6096	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6056.3;
	6096	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9854.2;
	6096	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1646.6;
	6097	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.2;
	6097	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	671.8;
	6097	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	802.5;
	6097	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3725.3;
	6097	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8744.6;
	6097	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5624.5;
	6097	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9384.1;
	6097	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1534.9;
	6098	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	865.8;
	6098	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	648.3;
	6098	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	779.8;
	6098	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3557.3;
	6098	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8264.1;
	6098	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5293.3;
	6098	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9023.9;
	6098	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1468.9;
	6099	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.1;
	6099	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	632.7;
	6099	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	788.3;
	6099	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3421.8;
	6099	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7936.6;
	6099	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5076.5;
	6099	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8756.7;
	6099	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1439.6;
	6100	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	851.4;
	6100	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	625.1;
	6100	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	790.7;
	6100	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3330.5;
	6100	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7713.7;
	6100	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4918;
	6100	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8608.5;
	6100	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1414.2;
	6101	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	849.4;
	6101	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	628.1;
	6101	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	777.2;
	6101	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3295.8;
	6101	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7691;
	6101	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4888.8;
	6101	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8601.4;
	6101	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1395.7;
	6102	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	854.3;
	6102	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	633.1;
	6102	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	784.7;
	6102	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3298.9;
	6102	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7840.2;
	6102	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4964.5;
	6102	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8675.6;
	6102	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1399;
	6103	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.4;
	6103	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	638.6;
	6103	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	794.1;
	6103	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3254.5;
	6103	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7933.9;
	6103	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5024;
	6103	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8665.6;
	6103	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1412.4;
	6104	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	862.8;
	6104	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	672.1;
	6104	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	827.9;
	6104	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3498.2;
	6104	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8622.3;
	6104	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5464.5;
	6104	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9277.1;
	6104	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1536.9;
	6105	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.2;
	6105	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719.3;
	6105	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	883.3;
	6105	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3952.8;
	6105	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9552.1;
	6105	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6114.5;
	6105	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10199.6;
	6105	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1667.5;
	6106	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.7;
	6106	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	761.9;
	6106	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	942.1;
	6106	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4390.4;
	6106	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10496.4;
	6106	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6759.4;
	6106	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11228.8;
	6106	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1793.9;
	6107	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.8;
	6107	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	799.9;
	6107	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1000.6;
	6107	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4808.6;
	6107	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11480.5;
	6107	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7391;
	6107	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12299.3;
	6107	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1976;
	6108	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	987.1;
	6108	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	850.4;
	6108	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1064.5;
	6108	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5146.1;
	6108	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12464.6;
	6108	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8011.8;
	6108	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13267.3;
	6108	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2141.6;
	6109	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1017.5;
	6109	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	908.1;
	6109	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1123.9;
	6109	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5387.9;
	6109	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13434.9;
	6109	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8588.9;
	6109	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13998.3;
	6109	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2281.2;
	6110	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1047.7;
	6110	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	957.5;
	6110	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1162;
	6110	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5556.1;
	6110	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14305.9;
	6110	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9036.6;
	6110	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14477.8;
	6110	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2362.2;
	6111	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1074.1;
	6111	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1005.7;
	6111	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1213;
	6111	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5642.2;
	6111	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15033.4;
	6111	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9275.3;
	6111	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14809.3;
	6111	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2382.6;
	6112	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1094.7;
	6112	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1041.4;
	6112	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1246;
	6112	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5655.6;
	6112	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15492.9;
	6112	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9430.4;
	6112	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14966.4;
	6112	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2408.5;
	6113	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1098.9;
	6113	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1051.7;
	6113	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1256.6;
	6113	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5570.8;
	6113	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15592;
	6113	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9492.1;
	6113	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14842.1;
	6113	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2450.3;
	6114	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1095;
	6114	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1033.1;
	6114	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1232.8;
	6114	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5422.4;
	6114	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15262.7;
	6114	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9301.1;
	6114	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14412.6;
	6114	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2310.5;
	6115	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1074.3;
	6115	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	985.3;
	6115	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1182.1;
	6115	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5257.6;
	6115	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14577.4;
	6115	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8935.2;
	6115	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14033.9;
	6115	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2302;
	6116	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1065.6;
	6116	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	967.2;
	6116	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1155.7;
	6116	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5263.9;
	6116	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14119.4;
	6116	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8736.4;
	6116	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13797.5;
	6116	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2250.1;
	6117	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1041.9;
	6117	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	909.9;
	6117	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1093;
	6117	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5029.6;
	6117	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13244.2;
	6117	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8199.9;
	6117	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13049.6;
	6117	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2123.8;
	6118	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.4;
	6118	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	838.7;
	6118	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1005.3;
	6118	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4659.5;
	6118	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11976.1;
	6118	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7423.9;
	6118	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12096.7;
	6118	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1950;
	6119	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958;
	6119	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	778.4;
	6119	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	920.5;
	6119	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4246.8;
	6119	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10748.5;
	6119	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6608.7;
	6119	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11144.3;
	6119	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1762.6;
	6120	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.5;
	6120	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.8;
	6120	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.8;
	6120	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3930.6;
	6120	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9827.3;
	6120	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5990.9;
	6120	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10383.2;
	6120	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1643;
	6121	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.6;
	6121	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	698.7;
	6121	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	821;
	6121	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3715.1;
	6121	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9184.2;
	6121	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5564.3;
	6121	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9861.2;
	6121	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1558.1;
	6122	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.7;
	6122	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	679.7;
	6122	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	794.5;
	6122	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3555;
	6122	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8805.9;
	6122	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5287;
	6122	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9544.7;
	6122	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1500.3;
	6123	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.1;
	6123	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	665.9;
	6123	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	776.1;
	6123	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3437.1;
	6123	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8593.6;
	6123	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5182.6;
	6123	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9395.8;
	6123	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1465.8;
	6124	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.2;
	6124	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	668.9;
	6124	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	774.2;
	6124	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3395.5;
	6124	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8689.6;
	6124	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5193.1;
	6124	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9464.8;
	6124	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1471.5;
	6125	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.6;
	6125	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	702.7;
	6125	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	802.4;
	6125	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3532.4;
	6125	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9277.5;
	6125	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5505.2;
	6125	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9938.2;
	6125	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1543.3;
	6126	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927;
	6126	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	756.7;
	6126	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	870.4;
	6126	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3857;
	6126	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10336.4;
	6126	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6106.6;
	6126	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10593.1;
	6126	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1690.3;
	6127	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951.4;
	6127	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	775.8;
	6127	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	894.9;
	6127	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3882;
	6127	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10685.5;
	6127	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6259.8;
	6127	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10703.6;
	6127	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1679.5;
	6128	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.8;
	6128	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	806.2;
	6128	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	917.2;
	6128	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4015.9;
	6128	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11104.1;
	6128	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6474.7;
	6128	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11074.5;
	6128	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1798.2;
	6129	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.2;
	6129	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	857.7;
	6129	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	972.9;
	6129	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4382.3;
	6129	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12009.8;
	6129	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7015.1;
	6129	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11933.6;
	6129	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1960.6;
	6130	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1014.6;
	6130	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	915.4;
	6130	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1042;
	6130	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4797.6;
	6130	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13067.8;
	6130	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7652;
	6130	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12899.6;
	6130	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2118.2;
	6131	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1051.1;
	6131	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	976.4;
	6131	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1108.5;
	6131	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5179.3;
	6131	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14108.8;
	6131	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8219;
	6131	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13719.4;
	6131	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2287.1;
	6132	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1081.3;
	6132	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1039.3;
	6132	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1181.8;
	6132	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5515.1;
	6132	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15101.7;
	6132	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8756.9;
	6132	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14364.3;
	6132	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2392.1;
	6133	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1115.8;
	6133	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1101.6;
	6133	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1249.3;
	6133	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5817;
	6133	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16116;
	6133	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9256.6;
	6133	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15002.5;
	6133	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2530.2;
	6134	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1141.2;
	6134	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1147.3;
	6134	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1299.1;
	6134	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5996.9;
	6134	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16848.2;
	6134	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9643.8;
	6134	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15404.2;
	6134	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2566;
	6135	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1155.2;
	6135	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1173.4;
	6135	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1334.2;
	6135	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6127.3;
	6135	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17326.7;
	6135	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9955;
	6135	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15558.7;
	6135	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2657.9;
	6136	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1146.8;
	6136	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1187.8;
	6136	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1352;
	6136	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6161.4;
	6136	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17623.7;
	6136	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10168.2;
	6136	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15571.4;
	6136	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2714.6;
	6137	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1129.4;
	6137	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1174.1;
	6137	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1331.5;
	6137	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6042.4;
	6137	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17516;
	6137	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10197.4;
	6137	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15036;
	6137	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2697.5;
	6138	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1115.3;
	6138	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1137.1;
	6138	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1283.9;
	6138	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5809.4;
	6138	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16922.5;
	6138	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9950.6;
	6138	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14366.3;
	6138	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2624.8;
	6139	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1087.1;
	6139	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1088.9;
	6139	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1234.5;
	6139	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5605.3;
	6139	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16116.6;
	6139	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9536;
	6139	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13994.1;
	6139	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2524.3;
	6140	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1081;
	6140	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1064.4;
	6140	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1214.5;
	6140	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5569.7;
	6140	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15664.6;
	6140	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9307.4;
	6140	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13806.2;
	6140	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2454.8;
	6141	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1056.5;
	6141	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	999.4;
	6141	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1150.9;
	6141	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5286;
	6141	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14681.9;
	6141	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8688.6;
	6141	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13116.8;
	6141	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2300.9;
	6142	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1010;
	6142	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	924.9;
	6142	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1063.6;
	6142	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4862.6;
	6142	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13286.2;
	6142	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7862.8;
	6142	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12129.1;
	6142	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2110.9;
	6143	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.9;
	6143	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	853.6;
	6143	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	976.3;
	6143	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4427.1;
	6143	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11949.9;
	6143	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7043.3;
	6143	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11184.5;
	6143	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1917.5;
	6144	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.6;
	6144	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	802;
	6144	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	918.3;
	6144	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4109.4;
	6144	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10944;
	6144	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6438.4;
	6144	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10396;
	6144	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1782.3;
	6145	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.8;
	6145	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776.5;
	6145	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	880.4;
	6145	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3862.3;
	6145	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10290.7;
	6145	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6012.8;
	6145	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9895.2;
	6145	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1694;
	6146	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.2;
	6146	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	756.9;
	6146	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.5;
	6146	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3690;
	6146	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9871.9;
	6146	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5691.5;
	6146	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9556.7;
	6146	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1630.8;
	6147	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.8;
	6147	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744;
	6147	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838.3;
	6147	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3583.1;
	6147	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9681.2;
	6147	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5540.8;
	6147	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9443.9;
	6147	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1588.5;
	6148	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.3;
	6148	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.9;
	6148	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	831.9;
	6148	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3533.1;
	6148	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9743.3;
	6148	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5600.5;
	6148	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9547.5;
	6148	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1599.8;
	6149	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.2;
	6149	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	767.8;
	6149	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.1;
	6149	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3644.8;
	6149	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10310.1;
	6149	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5905.5;
	6149	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10094.3;
	6149	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1665.7;
	6150	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.9;
	6150	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	826.4;
	6150	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	922.1;
	6150	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3965.1;
	6150	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11367.1;
	6150	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6482.3;
	6150	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10725.2;
	6150	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1815;
	6151	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.7;
	6151	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	843.9;
	6151	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	947.2;
	6151	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3986.9;
	6151	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11612;
	6151	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6610.6;
	6151	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10773;
	6151	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1857.5;
	6152	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.1;
	6152	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	870.5;
	6152	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	960.4;
	6152	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4117.6;
	6152	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11985.7;
	6152	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6721.2;
	6152	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10990.6;
	6152	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1955.3;
	6153	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	983.2;
	6153	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	918.4;
	6153	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	995.9;
	6153	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4455.5;
	6153	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12827.2;
	6153	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7018.5;
	6153	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11577.1;
	6153	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2101.6;
	6154	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.8;
	6154	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	986.5;
	6154	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1033.7;
	6154	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4820.4;
	6154	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13871.6;
	6154	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7427.9;
	6154	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12424.9;
	6154	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2279.9;
	6155	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1050;
	6155	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1053.1;
	6155	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1079.6;
	6155	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5234.7;
	6155	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14905.4;
	6155	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7849.9;
	6155	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13389.8;
	6155	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2412.6;
	6156	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1079.3;
	6156	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1115.8;
	6156	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1123.6;
	6156	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5580.1;
	6156	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15872.5;
	6156	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8260.6;
	6156	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14237.5;
	6156	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2543.2;
	6157	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1116.1;
	6157	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1165.3;
	6157	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1173.4;
	6157	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5867;
	6157	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16755.6;
	6157	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8747.6;
	6157	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14838.8;
	6157	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2645.9;
	6158	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1142.5;
	6158	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1196.6;
	6158	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1220.4;
	6158	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6075.6;
	6158	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17380.6;
	6158	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9202.2;
	6158	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14846.8;
	6158	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2700.3;
	6159	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1157.7;
	6159	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1214.4;
	6159	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1255.2;
	6159	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6183.4;
	6159	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17758.3;
	6159	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9553.8;
	6159	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14536.5;
	6159	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2733.8;
	6160	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1164.9;
	6160	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1212.3;
	6160	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1274;
	6160	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6177.3;
	6160	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17945.9;
	6160	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9815.7;
	6160	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14001.3;
	6160	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2710.4;
	6161	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1156.5;
	6161	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1189.9;
	6161	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1270.2;
	6161	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5987.2;
	6161	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17725.5;
	6161	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9897.1;
	6161	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13498.6;
	6161	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2664.3;
	6162	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1143.7;
	6162	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1144.3;
	6162	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1246.3;
	6162	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5759.6;
	6162	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16975.1;
	6162	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9595.1;
	6162	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13154.9;
	6162	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2600.6;
	6163	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1123.4;
	6163	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1106.4;
	6163	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1201.5;
	6163	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5592.1;
	6163	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16418;
	6163	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9336.5;
	6163	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13101.4;
	6163	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2541;
	6164	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1113.7;
	6164	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1081.7;
	6164	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1183.8;
	6164	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5535.8;
	6164	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16095.5;
	6164	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9132.1;
	6164	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13050.4;
	6164	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2496.8;
	6165	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1087.4;
	6165	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1017.7;
	6165	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1129.8;
	6165	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5270;
	6165	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15168.4;
	6165	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8579.5;
	6165	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12574;
	6165	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2325.8;
	6166	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1039.8;
	6166	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	934.6;
	6166	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1040.2;
	6166	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4848;
	6166	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13823.9;
	6166	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7822.4;
	6166	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11730.2;
	6166	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2147.5;
	6167	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.8;
	6167	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	865.3;
	6167	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	955.4;
	6167	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4427;
	6167	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12540.7;
	6167	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7018.2;
	6167	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10916.4;
	6167	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1963.3;
	6168	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.7;
	6168	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	798;
	6168	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	894.8;
	6168	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4087.5;
	6168	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11514.3;
	6168	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6429.9;
	6168	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10256.3;
	6168	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1833.2;
	6169	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932;
	6169	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	762.4;
	6169	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860;
	6169	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3856.2;
	6169	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10844.7;
	6169	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6038.4;
	6169	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9820;
	6169	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1741.8;
	6170	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.9;
	6170	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744.7;
	6170	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835;
	6170	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3699;
	6170	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10378.2;
	6170	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5743.3;
	6170	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9537.7;
	6170	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1671.9;
	6171	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.4;
	6171	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	734.8;
	6171	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	820.5;
	6171	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3601.1;
	6171	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10144.8;
	6171	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5619.7;
	6171	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9444.9;
	6171	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1543.7;
	6172	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.9;
	6172	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.5;
	6172	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	810.8;
	6172	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3558.4;
	6172	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10190;
	6172	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5652.2;
	6172	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9541.8;
	6172	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1530.4;
	6173	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.9;
	6173	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	766.9;
	6173	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	834.2;
	6173	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3680.5;
	6173	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10778.2;
	6173	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5944.8;
	6173	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10130.8;
	6173	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1599.4;
	6174	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.3;
	6174	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	825.3;
	6174	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	904.6;
	6174	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4004.6;
	6174	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11770.5;
	6174	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6533.3;
	6174	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10775.2;
	6174	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1819.5;
	6175	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.5;
	6175	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	843.7;
	6175	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	939.9;
	6175	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4032.5;
	6175	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12036.6;
	6175	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6716.5;
	6175	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10856.1;
	6175	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1858.5;
	6176	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	971.5;
	6176	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	864.1;
	6176	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	964;
	6176	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4152.1;
	6176	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12394.8;
	6176	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6833.6;
	6176	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11274.5;
	6176	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1928.5;
	6177	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	998.5;
	6177	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	903.1;
	6177	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1018.4;
	6177	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4494.4;
	6177	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13306.6;
	6177	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7336.2;
	6177	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12126.3;
	6177	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2064.2;
	6178	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1022.3;
	6178	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	951;
	6178	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1085.6;
	6178	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4913.2;
	6178	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14444.2;
	6178	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7876.8;
	6178	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13012.8;
	6178	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2217.8;
	6179	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1046.3;
	6179	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	995.6;
	6179	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1150.2;
	6179	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5268.7;
	6179	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15475.3;
	6179	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8424.1;
	6179	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13801.3;
	6179	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2379.7;
	6180	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1068.2;
	6180	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1027.6;
	6180	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1197.1;
	6180	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5577.6;
	6180	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16252.5;
	6180	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8906.8;
	6180	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14498.5;
	6180	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2523.7;
	6181	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1090.3;
	6181	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1064.7;
	6181	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1249.7;
	6181	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5815.2;
	6181	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16894.6;
	6181	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9323.6;
	6181	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15149.7;
	6181	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2687.7;
	6182	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1108.8;
	6182	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1090.3;
	6182	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1272.9;
	6182	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5959.2;
	6182	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17125.1;
	6182	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9666.8;
	6182	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15609;
	6182	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2756;
	6183	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1112.7;
	6183	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1113.5;
	6183	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1273.8;
	6183	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6000.9;
	6183	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17159.3;
	6183	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9980.7;
	6183	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15889.1;
	6183	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2779.2;
	6184	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1128.2;
	6184	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1129.7;
	6184	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1284.6;
	6184	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6007.1;
	6184	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16984.2;
	6184	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10203.2;
	6184	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15950.2;
	6184	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2774.4;
	6185	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1120.3;
	6185	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1118.1;
	6185	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1262.3;
	6185	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5824.7;
	6185	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16509.4;
	6185	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10221.7;
	6185	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15656.8;
	6185	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2738.3;
	6186	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1100.8;
	6186	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1091.1;
	6186	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1233.2;
	6186	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5574.1;
	6186	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16022.5;
	6186	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9956.8;
	6186	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15002.6;
	6186	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2652.5;
	6187	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1065;
	6187	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1051.8;
	6187	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1196.8;
	6187	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5451.6;
	6187	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15472.4;
	6187	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9576.1;
	6187	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14590.9;
	6187	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2555.1;
	6188	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1054.9;
	6188	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1029.7;
	6188	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1179.5;
	6188	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5429.3;
	6188	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15133.9;
	6188	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9346.3;
	6188	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14258.7;
	6188	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2482.6;
	6189	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1033.5;
	6189	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	975;
	6189	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1115.6;
	6189	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5188.4;
	6189	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14248.2;
	6189	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8770.1;
	6189	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13516.4;
	6189	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2288.1;
	6190	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000.4;
	6190	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	902.9;
	6190	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1032.1;
	6190	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4778.1;
	6190	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13040.1;
	6190	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7903.7;
	6190	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12524.6;
	6190	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2072.5;
	6191	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.7;
	6191	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	837;
	6191	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	950.7;
	6191	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4364;
	6191	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11807.5;
	6191	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7014.9;
	6191	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11536.4;
	6191	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1879.3;
	6192	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931;
	6192	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	797.4;
	6192	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884.9;
	6192	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4039.5;
	6192	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10886.1;
	6192	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6368.4;
	6192	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10739.8;
	6192	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1831.4;
	6193	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.2;
	6193	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	768.1;
	6193	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	848.4;
	6193	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3828.3;
	6193	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10284.7;
	6193	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5878.9;
	6193	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10231.6;
	6193	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1669.4;
	6194	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.2;
	6194	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.1;
	6194	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	820.4;
	6194	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3676;
	6194	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9874.2;
	6194	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5563.7;
	6194	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9838;
	6194	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1660.8;
	6195	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.5;
	6195	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	734.3;
	6195	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	802.4;
	6195	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3603.1;
	6195	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9677.9;
	6195	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5421.1;
	6195	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9656.9;
	6195	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1630.1;
	6196	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.3;
	6196	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	732;
	6196	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	796.7;
	6196	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3549;
	6196	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9688;
	6196	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5453.5;
	6196	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9661.4;
	6196	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1616.1;
	6197	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.8;
	6197	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753.9;
	6197	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	818.7;
	6197	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3638.2;
	6197	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10264.8;
	6197	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5739.6;
	6197	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10133;
	6197	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1677.5;
	6198	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.8;
	6198	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	819.9;
	6198	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	890.7;
	6198	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3983.6;
	6198	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11346.9;
	6198	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6342.6;
	6198	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10837.8;
	6198	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1839.8;
	6199	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951.7;
	6199	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	836.7;
	6199	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	920.1;
	6199	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3991.3;
	6199	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11591.8;
	6199	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6486.1;
	6199	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10903.3;
	6199	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1868.6;
	6200	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.7;
	6200	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	865.3;
	6200	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	935.6;
	6200	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4101.1;
	6200	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12010.8;
	6200	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6687.8;
	6200	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11370.5;
	6200	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1964.8;
	6201	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	987.4;
	6201	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	916.4;
	6201	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	991.3;
	6201	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4447;
	6201	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13044.4;
	6201	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7244.1;
	6201	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12326.7;
	6201	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2113;
	6202	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1017.1;
	6202	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	981.1;
	6202	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1052.9;
	6202	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4842.5;
	6202	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14239.1;
	6202	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7882.7;
	6202	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13315.5;
	6202	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2313.7;
	6203	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1047.7;
	6203	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1030;
	6203	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1131.9;
	6203	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5179;
	6203	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15281.7;
	6203	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8409.3;
	6203	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14150.2;
	6203	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2485.7;
	6204	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1074.8;
	6204	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1084.8;
	6204	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1200.8;
	6204	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5489.7;
	6204	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16277.3;
	6204	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8920.6;
	6204	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14789.6;
	6204	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2600;
	6205	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1106.5;
	6205	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1134.4;
	6205	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1244.4;
	6205	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5717.1;
	6205	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17200.9;
	6205	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9398.8;
	6205	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15411.7;
	6205	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2678.7;
	6206	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1136.4;
	6206	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1128.3;
	6206	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1287.8;
	6206	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5859;
	6206	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17852.6;
	6206	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9779.4;
	6206	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15825.9;
	6206	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2744.2;
	6207	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1156.4;
	6207	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1110;
	6207	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1302.6;
	6207	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5868;
	6207	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18280.4;
	6207	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10068.6;
	6207	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16050.1;
	6207	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2765.2;
	6208	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1169.2;
	6208	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1103.1;
	6208	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1317.2;
	6208	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5857;
	6208	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18554.8;
	6208	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10266.8;
	6208	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16163.6;
	6208	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2801.4;
	6209	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1153.8;
	6209	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1066.7;
	6209	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1279.1;
	6209	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5690;
	6209	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18251.6;
	6209	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10299.4;
	6209	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15785.3;
	6209	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2741.9;
	6210	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1108.3;
	6210	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1029.5;
	6210	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1227.8;
	6210	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5490.5;
	6210	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17336.7;
	6210	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10016.2;
	6210	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15084.4;
	6210	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2654.2;
	6211	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1086.9;
	6211	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1013.9;
	6211	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1185.7;
	6211	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5380.2;
	6211	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16625.7;
	6211	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9640.2;
	6211	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14727.2;
	6211	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2583.4;
	6212	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1086.6;
	6212	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1004;
	6212	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1167.5;
	6212	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5410.9;
	6212	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16212.7;
	6212	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9415.5;
	6212	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14472.9;
	6212	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2511.3;
	6213	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1061.7;
	6213	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	959.1;
	6213	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1108.3;
	6213	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5157.2;
	6213	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15262.3;
	6213	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8828.8;
	6213	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13745.1;
	6213	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2334.4;
	6214	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1028.7;
	6214	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	896.1;
	6214	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1027.1;
	6214	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4801.9;
	6214	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13962.7;
	6214	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8015.1;
	6214	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12778.8;
	6214	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2108.9;
	6215	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	991.8;
	6215	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	832.2;
	6215	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	954.8;
	6215	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4382.9;
	6215	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12628.2;
	6215	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7188.4;
	6215	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11833.9;
	6215	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1942.7;
	6216	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.9;
	6216	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	790.7;
	6216	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	898.3;
	6216	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4040.9;
	6216	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11566.7;
	6216	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6527.9;
	6216	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11044.6;
	6216	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1842;
	6217	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.8;
	6217	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	745.4;
	6217	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	858.2;
	6217	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3821.1;
	6217	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10877.1;
	6217	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6096.8;
	6217	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10501.8;
	6217	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1764.5;
	6218	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.3;
	6218	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	722.1;
	6218	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	829.3;
	6218	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3674.8;
	6218	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10376.5;
	6218	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5816.8;
	6218	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10115.3;
	6218	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1687.3;
	6219	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.1;
	6219	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719.2;
	6219	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	811.5;
	6219	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3566.2;
	6219	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10081.2;
	6219	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5655.6;
	6219	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9952.8;
	6219	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1642.4;
	6220	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.6;
	6220	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	731.4;
	6220	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	811.1;
	6220	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3518.6;
	6220	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10074.1;
	6220	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5655.8;
	6220	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10031.9;
	6220	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1621.8;
	6221	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.4;
	6221	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755.5;
	6221	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	828.1;
	6221	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3627.5;
	6221	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10557.1;
	6221	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5867.5;
	6221	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10551;
	6221	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1701.3;
	6222	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.3;
	6222	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	809.5;
	6222	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	896;
	6222	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3958.5;
	6222	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11600.9;
	6222	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6443.3;
	6222	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11186.6;
	6222	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1860.7;
	6223	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960;
	6223	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	832.2;
	6223	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	917.7;
	6223	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4008.1;
	6223	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11963.5;
	6223	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6592.8;
	6223	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11285.6;
	6223	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1896.2;
	6224	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.1;
	6224	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	849.6;
	6224	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	926;
	6224	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4168.1;
	6224	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12163.9;
	6224	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6788.9;
	6224	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11644;
	6224	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1996.8;
	6225	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	976.6;
	6225	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	883.8;
	6225	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	974.1;
	6225	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4506;
	6225	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12876.5;
	6225	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7251.6;
	6225	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12286.3;
	6225	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2141;
	6226	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	995.5;
	6226	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	925.8;
	6226	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1026.1;
	6226	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4891;
	6226	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13940.8;
	6226	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7900.8;
	6226	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13058.3;
	6226	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2287.2;
	6227	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030.3;
	6227	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	979.3;
	6227	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1081.5;
	6227	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5291.3;
	6227	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15091.6;
	6227	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8487.8;
	6227	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13891.9;
	6227	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2422;
	6228	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1061.4;
	6228	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1046.9;
	6228	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1145.1;
	6228	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5642.2;
	6228	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16246.6;
	6228	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9066.7;
	6228	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14569.8;
	6228	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2549.2;
	6229	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1095.9;
	6229	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1103.2;
	6229	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1218.5;
	6229	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5871.6;
	6229	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17261.3;
	6229	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9610.6;
	6229	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14979.9;
	6229	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2677.9;
	6230	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1120.7;
	6230	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1147;
	6230	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1284;
	6230	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6069.9;
	6230	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18028;
	6230	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10073.9;
	6230	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15344.9;
	6230	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2774.3;
	6231	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1135.5;
	6231	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1173.8;
	6231	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1326.6;
	6231	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6173.1;
	6231	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18481.6;
	6231	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10360;
	6231	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15695.5;
	6231	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2819.8;
	6232	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1145.6;
	6232	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1179.6;
	6232	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1353.9;
	6232	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6191.1;
	6232	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18553.8;
	6232	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10570.7;
	6232	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15820.7;
	6232	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2850.6;
	6233	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1135.8;
	6233	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1152.9;
	6233	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1316.5;
	6233	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6048.2;
	6233	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18206.4;
	6233	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10539;
	6233	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15479.6;
	6233	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2778.2;
	6234	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1095.9;
	6234	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1105.9;
	6234	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1248;
	6234	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5811.5;
	6234	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17351.5;
	6234	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10138.6;
	6234	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14740.9;
	6234	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2672.4;
	6235	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1052.1;
	6235	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1052;
	6235	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1178.2;
	6235	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5573.6;
	6235	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16416.4;
	6235	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9620.6;
	6235	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14350.5;
	6235	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2558.3;
	6236	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1032.1;
	6236	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1022.7;
	6236	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1135.2;
	6236	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5452.2;
	6236	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15750.1;
	6236	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9266.3;
	6236	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14021.4;
	6236	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2432.8;
	6237	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1003.2;
	6237	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	967.6;
	6237	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1079.8;
	6237	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5167.2;
	6237	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14668.6;
	6237	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8675.1;
	6237	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13386.3;
	6237	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2320.9;
	6238	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.9;
	6238	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	912.7;
	6238	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1010.4;
	6238	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4876.8;
	6238	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13579.3;
	6238	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7997.7;
	6238	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12608.4;
	6238	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2138.7;
	6239	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949;
	6239	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	854.2;
	6239	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	942.1;
	6239	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4550.6;
	6239	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12452;
	6239	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7341.7;
	6239	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11798;
	6239	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1999;
	6240	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.8;
	6240	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	805.8;
	6240	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.6;
	6240	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4261.6;
	6240	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11483;
	6240	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6744.3;
	6240	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11143.8;
	6240	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1904.3;
	6241	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.5;
	6241	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772.3;
	6241	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.1;
	6241	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4011.4;
	6241	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10743;
	6241	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6280.9;
	6241	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10569;
	6241	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1795.7;
	6242	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.2;
	6242	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	754;
	6242	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	815.4;
	6242	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3845.5;
	6242	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10257.8;
	6242	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5962.3;
	6242	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10180.6;
	6242	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1717.5;
	6243	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.4;
	6243	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.3;
	6243	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	789;
	6243	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3705.8;
	6243	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9942.5;
	6243	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5709.8;
	6243	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9886.4;
	6243	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1672.1;
	6244	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.1;
	6244	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	715;
	6244	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	778.1;
	6244	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3623;
	6244	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9765.9;
	6244	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5586.4;
	6244	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9694.5;
	6244	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1626.6;
	6245	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	881.4;
	6245	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714;
	6245	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	787.6;
	6245	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3629;
	6245	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9826.1;
	6245	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5565.1;
	6245	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9730.8;
	6245	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1627;
	6246	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.9;
	6246	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	709.9;
	6246	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	803.8;
	6246	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3679.3;
	6246	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10072.1;
	6246	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5715.2;
	6246	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9893.8;
	6246	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1660.4;
	6247	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.7;
	6247	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719.4;
	6247	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	815.7;
	6247	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3676.1;
	6247	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10281.2;
	6247	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5839.5;
	6247	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9991.2;
	6247	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1690.5;
	6248	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.8;
	6248	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751;
	6248	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	864.3;
	6248	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3938.1;
	6248	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11032.4;
	6248	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6317.6;
	6248	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10705;
	6248	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1801.9;
	6249	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.5;
	6249	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	820.5;
	6249	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	941.4;
	6249	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4419;
	6249	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12269.9;
	6249	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7077.9;
	6249	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11898.1;
	6249	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1989;
	6250	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981.3;
	6250	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	894.1;
	6250	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1023.5;
	6250	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4901.3;
	6250	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13624.2;
	6250	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7886.2;
	6250	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13026.1;
	6250	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2197.1;
	6251	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.6;
	6251	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	967.9;
	6251	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1085.6;
	6251	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5328.8;
	6251	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14919.6;
	6251	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8616.7;
	6251	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13912.7;
	6251	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2374.1;
	6252	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1039.9;
	6252	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1021.9;
	6252	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1146.9;
	6252	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5668.3;
	6252	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16125.3;
	6252	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9204.6;
	6252	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14705.2;
	6252	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2496.8;
	6253	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1067.2;
	6253	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1076.4;
	6253	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1215.4;
	6253	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5922.2;
	6253	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17015.2;
	6253	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9699.9;
	6253	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15307.8;
	6253	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2578.5;
	6254	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1087.2;
	6254	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1114;
	6254	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1281;
	6254	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6071.7;
	6254	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17614.9;
	6254	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10031.5;
	6254	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15648.9;
	6254	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2591.9;
	6255	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1104.7;
	6255	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1140.3;
	6255	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1342.6;
	6255	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6149.5;
	6255	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18036.5;
	6255	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10333.4;
	6255	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15835.7;
	6255	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2646.6;
	6256	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1113;
	6256	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1155.8;
	6256	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1345.9;
	6256	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6129.4;
	6256	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18293.5;
	6256	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10489.1;
	6256	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15881.2;
	6256	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2656.3;
	6257	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1103.9;
	6257	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1156.9;
	6257	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1345.6;
	6257	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6013.9;
	6257	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18196.5;
	6257	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10494.5;
	6257	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15697.9;
	6257	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2624.6;
	6258	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1067.6;
	6258	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1122.2;
	6258	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1308.1;
	6258	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5782.4;
	6258	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17486.7;
	6258	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10192;
	6258	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15114.7;
	6258	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2555.1;
	6259	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1037;
	6259	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1069.8;
	6259	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1249.1;
	6259	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5602.1;
	6259	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16603.3;
	6259	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9739.6;
	6259	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14538.8;
	6259	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2429.1;
	6260	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1014.2;
	6260	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1031.2;
	6260	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1209;
	6260	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5508;
	6260	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15826.7;
	6260	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9378.8;
	6260	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14105;
	6260	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2365.8;
	6261	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990;
	6261	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	973.3;
	6261	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1148.9;
	6261	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5230.8;
	6261	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14869.4;
	6261	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8817.5;
	6261	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13491.4;
	6261	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2209.9;
	6262	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.1;
	6262	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	911.8;
	6262	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1081.6;
	6262	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4913.9;
	6262	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13715.6;
	6262	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8168.7;
	6262	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12677.9;
	6262	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2101.6;
	6263	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.2;
	6263	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	854.3;
	6263	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	987.9;
	6263	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4607.1;
	6263	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12649.9;
	6263	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7518.9;
	6263	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11937.8;
	6263	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1960;
	6264	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.7;
	6264	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	794.4;
	6264	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	934.1;
	6264	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4311.5;
	6264	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11717.1;
	6264	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6948.5;
	6264	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11163;
	6264	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1843.7;
	6265	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.4;
	6265	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755;
	6265	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.2;
	6265	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4067.1;
	6265	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10987.6;
	6265	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6478.8;
	6265	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10596.3;
	6265	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1730.7;
	6266	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.7;
	6266	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.5;
	6266	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.6;
	6266	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3869.3;
	6266	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10474.9;
	6266	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6134;
	6266	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10155.6;
	6266	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1652.4;
	6267	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867.3;
	6267	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	703.7;
	6267	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	850.8;
	6267	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3724;
	6267	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10110.9;
	6267	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5852.1;
	6267	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9844.3;
	6267	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1617;
	6268	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867;
	6268	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	685.6;
	6268	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849.8;
	6268	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3629.5;
	6268	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9941.6;
	6268	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5707.3;
	6268	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9702.4;
	6268	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1593.5;
	6269	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868;
	6269	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	685.3;
	6269	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843;
	6269	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3592.2;
	6269	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9992.8;
	6269	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5705.6;
	6269	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9677.5;
	6269	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1590.2;
	6270	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.9;
	6270	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	692.3;
	6270	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.3;
	6270	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3583;
	6270	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10046.6;
	6270	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5820.8;
	6270	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9696.3;
	6270	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1591.1;
	6271	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.1;
	6271	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	699.6;
	6271	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.6;
	6271	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3537.6;
	6271	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9911.9;
	6271	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5905.9;
	6271	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9682.6;
	6271	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1610;
	6272	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.6;
	6272	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.6;
	6272	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913.4;
	6272	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3789.9;
	6272	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10329.9;
	6272	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6320;
	6272	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10380.9;
	6272	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1743.8;
	6273	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925.3;
	6273	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	775.6;
	6273	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	998.2;
	6273	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4279.6;
	6273	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11280.6;
	6273	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7055.7;
	6273	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11472.7;
	6273	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1883.7;
	6274	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.1;
	6274	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	818.7;
	6274	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1073;
	6274	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4731;
	6274	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12364.4;
	6274	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7826.7;
	6274	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12490.4;
	6274	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2001.4;
	6275	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1006.8;
	6275	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	880.3;
	6275	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1136.2;
	6275	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5178.4;
	6275	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13617.7;
	6275	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8581.8;
	6275	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13549.9;
	6275	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2181.9;
	6276	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1043.7;
	6276	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	951.2;
	6276	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1209.9;
	6276	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5554.5;
	6276	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14953.2;
	6276	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9307.1;
	6276	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14488.9;
	6276	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2367.2;
	6277	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1084.6;
	6277	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1021.8;
	6277	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1278;
	6277	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5852.7;
	6277	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16236.2;
	6277	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9901.9;
	6277	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15223.1;
	6277	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2533;
	6278	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1120.9;
	6278	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1083.9;
	6278	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1308.3;
	6278	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6069.9;
	6278	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17266.7;
	6278	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10355.5;
	6278	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15799.9;
	6278	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2629.4;
	6279	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1146;
	6279	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1134.4;
	6279	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1347.3;
	6279	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6179.9;
	6279	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18000.1;
	6279	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10645.6;
	6279	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16142.9;
	6279	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2657;
	6280	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1161.2;
	6280	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1168;
	6280	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1374.7;
	6280	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6183.6;
	6280	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18370.6;
	6280	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10784.8;
	6280	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16321.1;
	6280	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2675.9;
	6281	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1167.8;
	6281	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1181.5;
	6281	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1389;
	6281	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6133.6;
	6281	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18416.8;
	6281	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10840.6;
	6281	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16267.3;
	6281	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2713.1;
	6282	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1158.1;
	6282	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1157.9;
	6282	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1361.1;
	6282	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5972.8;
	6282	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17919.8;
	6282	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10613.8;
	6282	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15814.5;
	6282	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2567.8;
	6283	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1131.1;
	6283	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1104.3;
	6283	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1292.9;
	6283	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5792.2;
	6283	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17234.6;
	6283	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10262.9;
	6283	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15411.1;
	6283	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2574.3;
	6284	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1113.8;
	6284	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1070.5;
	6284	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1252.8;
	6284	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5745.1;
	6284	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16645.9;
	6284	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9931;
	6284	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15102.2;
	6284	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2515.9;
	6285	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1079.3;
	6285	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1005.5;
	6285	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1184.1;
	6285	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5465.9;
	6285	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15666.8;
	6285	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9287.9;
	6285	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14476.2;
	6285	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2378.3;
	6286	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1034.5;
	6286	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	924.5;
	6286	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1092.2;
	6286	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5092.5;
	6286	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14300.2;
	6286	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8488.8;
	6286	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13548.1;
	6286	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2201.2;
	6287	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985.4;
	6287	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	857;
	6287	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	999.9;
	6287	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4677.1;
	6287	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12996.3;
	6287	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7668.2;
	6287	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12522.2;
	6287	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2012.8;
	6288	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.5;
	6288	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	803.8;
	6288	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	932.9;
	6288	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4325.6;
	6288	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11968;
	6288	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7021.4;
	6288	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11687;
	6288	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1885.7;
	6289	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.8;
	6289	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	764;
	6289	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	888.1;
	6289	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4082.1;
	6289	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11226.5;
	6289	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6552;
	6289	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11111.2;
	6289	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1783.2;
	6290	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.6;
	6290	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.4;
	6290	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.1;
	6290	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3906;
	6290	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10747;
	6290	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6221.2;
	6290	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10757.3;
	6290	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1716.9;
	6291	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.8;
	6291	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.8;
	6291	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	836.7;
	6291	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3796.9;
	6291	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10492.3;
	6291	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6092.9;
	6291	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10586.8;
	6291	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1689.8;
	6292	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893;
	6292	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727;
	6292	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.5;
	6292	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3744.5;
	6292	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10552.9;
	6292	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6110.8;
	6292	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10648.8;
	6292	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1696.9;
	6293	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.2;
	6293	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755.8;
	6293	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	862.9;
	6293	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3846.8;
	6293	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11101.6;
	6293	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6432.9;
	6293	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11088.4;
	6293	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1761;
	6294	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.1;
	6294	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	818.9;
	6294	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	929.7;
	6294	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4135.7;
	6294	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12108.5;
	6294	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7007.4;
	6294	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11662.2;
	6294	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1904.6;
	6295	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.7;
	6295	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	844.8;
	6295	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	958.2;
	6295	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4154.1;
	6295	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12434.8;
	6295	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7120.1;
	6295	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11726.6;
	6295	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1891;
	6296	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.4;
	6296	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	880.9;
	6296	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	983.1;
	6296	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4290;
	6296	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12885;
	6296	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7260.7;
	6296	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12225.5;
	6296	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2041.3;
	6297	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1014.8;
	6297	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	945.8;
	6297	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1047;
	6297	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4668.1;
	6297	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14050.9;
	6297	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7819.2;
	6297	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13218.5;
	6297	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2238.4;
	6298	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1056.7;
	6298	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1029.7;
	6298	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1128;
	6298	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5123;
	6298	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15500.8;
	6298	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8513.1;
	6298	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14312.2;
	6298	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2414.4;
	6299	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1098.3;
	6299	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1108.5;
	6299	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1207.1;
	6299	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5576.7;
	6299	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16927.3;
	6299	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9208.1;
	6299	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15241.8;
	6299	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2610.5;
	6300	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1136.8;
	6300	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1187.3;
	6300	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1296.2;
	6300	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5953.1;
	6300	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18253.7;
	6300	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9893.3;
	6300	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16071.6;
	6300	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2738.9;
	6301	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1173.6;
	6301	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1263.9;
	6301	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1378.1;
	6301	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6262.9;
	6301	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19400.5;
	6301	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10488.4;
	6301	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16817.4;
	6301	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2869.9;
	6302	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1200.8;
	6302	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1317.8;
	6302	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1435.2;
	6302	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6470.7;
	6302	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20094.2;
	6302	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10875.2;
	6302	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17294.1;
	6302	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2898.3;
	6303	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1210.9;
	6303	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1342.8;
	6303	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1472.2;
	6303	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6544.3;
	6303	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20321.6;
	6303	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11111.1;
	6303	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17407.2;
	6303	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2939.8;
	6304	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1208;
	6304	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1345.9;
	6304	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1486.3;
	6304	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6537.8;
	6304	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20325.9;
	6304	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11191.7;
	6304	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17331.7;
	6304	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2954.3;
	6305	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1198.6;
	6305	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1330.1;
	6305	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1472.2;
	6305	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6402.4;
	6305	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20076.4;
	6305	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11312.6;
	6305	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	17007;
	6305	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2932;
	6306	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1183;
	6306	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1291.5;
	6306	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1427.8;
	6306	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6164.4;
	6306	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19402.2;
	6306	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11122.1;
	6306	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16320.6;
	6306	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2857.9;
	6307	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1153.3;
	6307	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1241.5;
	6307	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1371.1;
	6307	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5999.1;
	6307	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18677.7;
	6307	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10694.5;
	6307	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15892.9;
	6307	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2765.1;
	6308	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1135.1;
	6308	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1202;
	6308	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1329.3;
	6308	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5946.2;
	6308	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18104.7;
	6308	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10336.1;
	6308	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15480.3;
	6308	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2682.8;
	6309	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1099;
	6309	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1127.4;
	6309	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1248.6;
	6309	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5617.7;
	6309	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16947.2;
	6309	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9604.2;
	6309	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14632.3;
	6309	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2509.9;
	6310	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1048;
	6310	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1039.2;
	6310	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1148.9;
	6310	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5164.6;
	6310	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15452.9;
	6310	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8673.8;
	6310	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13531.9;
	6310	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2305;
	6311	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000.6;
	6311	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	953.7;
	6311	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1052.2;
	6311	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4698.7;
	6311	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14018.1;
	6311	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7809.2;
	6311	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12430.3;
	6311	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2097.8;
	6312	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	973.5;
	6312	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	889.9;
	6312	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	985.1;
	6312	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4343.5;
	6312	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12875;
	6312	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7135.9;
	6312	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11594;
	6312	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1954.5;
	6313	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951.2;
	6313	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	853.2;
	6313	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	935.4;
	6313	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4079.4;
	6313	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12040.1;
	6313	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6660.5;
	6313	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11037.8;
	6313	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1847.1;
	6314	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937;
	6314	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	825.8;
	6314	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	902;
	6314	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3897.3;
	6314	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11457.4;
	6314	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6329.8;
	6314	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10665.8;
	6314	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1774.6;
	6315	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.5;
	6315	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	807.7;
	6315	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	879;
	6315	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3769.5;
	6315	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11073.3;
	6315	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6094.4;
	6315	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10446.6;
	6315	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1724;
	6316	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.7;
	6316	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	796.8;
	6316	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.8;
	6316	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3696.5;
	6316	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10956.9;
	6316	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6053;
	6316	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10427.5;
	6316	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1725.3;
	6317	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.2;
	6317	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	820.9;
	6317	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	886.6;
	6317	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3792.3;
	6317	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11398.4;
	6317	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6289.9;
	6317	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10882.5;
	6317	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1782.6;
	6318	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.2;
	6318	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	877.6;
	6318	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	951.1;
	6318	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4111;
	6318	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12370.5;
	6318	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6890;
	6318	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11508.1;
	6318	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1931.4;
	6319	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	979;
	6319	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	888;
	6319	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	977.5;
	6319	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4127.6;
	6319	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12602;
	6319	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7044.1;
	6319	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11535.3;
	6319	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1977.6;
	6320	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	984.9;
	6320	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	918.9;
	6320	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	995.1;
	6320	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4248.2;
	6320	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13017.2;
	6320	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7175.3;
	6320	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11969;
	6320	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2064.4;
	6321	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1018;
	6321	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	983.4;
	6321	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1054.6;
	6321	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4612.3;
	6321	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14111.8;
	6321	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7655.7;
	6321	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12873.6;
	6321	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2223.5;
	6322	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1057.7;
	6322	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1066.5;
	6322	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1120.2;
	6322	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5019.3;
	6322	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15555.6;
	6322	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8281.7;
	6322	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13884.4;
	6322	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2425.2;
	6323	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1093.7;
	6323	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1145.3;
	6323	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1198.7;
	6323	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5426;
	6323	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16906.1;
	6323	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8953;
	6323	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14937.5;
	6323	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2588.5;
	6324	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1124.5;
	6324	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1216.2;
	6324	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1279;
	6324	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5774.6;
	6324	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18122.3;
	6324	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9612.1;
	6324	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15721.1;
	6324	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2730.2;
	6325	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1160.7;
	6325	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1275.8;
	6325	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1354.1;
	6325	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6067.9;
	6325	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19091.6;
	6325	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10190;
	6325	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16237.6;
	6325	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2847.3;
	6326	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1185.4;
	6326	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1308.7;
	6326	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1404.7;
	6326	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6261.3;
	6326	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19727.9;
	6326	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10600.5;
	6326	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16530.4;
	6326	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2916.1;
	6327	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1195.1;
	6327	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1322.8;
	6327	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1438;
	6327	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6290.2;
	6327	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20048.3;
	6327	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10909.7;
	6327	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16703.5;
	6327	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2960.4;
	6328	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1203.5;
	6328	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1325.3;
	6328	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1444.4;
	6328	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6309.1;
	6328	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	20164.5;
	6328	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	11053.9;
	6328	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16794.3;
	6328	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2944.8;
	6329	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1191.3;
	6329	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1300.8;
	6329	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1416.1;
	6329	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6185.3;
	6329	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19871.3;
	6329	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10999;
	6329	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16131.4;
	6329	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2886.3;
	6330	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1172.1;
	6330	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1257.3;
	6330	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1367.7;
	6330	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5956.1;
	6330	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	19082;
	6330	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10676.9;
	6330	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15198.9;
	6330	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2731.1;
	6331	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1144.4;
	6331	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1210.3;
	6331	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1310.1;
	6331	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5768.3;
	6331	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18445.6;
	6331	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10243.8;
	6331	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14698.3;
	6331	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2655.7;
	6332	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1129.6;
	6332	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1170.7;
	6332	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1273.5;
	6332	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5679.7;
	6332	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17846.9;
	6332	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9865.9;
	6332	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14226.8;
	6332	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2583.4;
	6333	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1095.7;
	6333	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1101.6;
	6333	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1196.3;
	6333	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5389.1;
	6333	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16788.5;
	6333	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9158.7;
	6333	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13507;
	6333	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2391.1;
	6334	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1046.8;
	6334	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1013.6;
	6334	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1097.3;
	6334	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4941;
	6334	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15328.5;
	6334	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8306;
	6334	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12493.5;
	6334	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2187.1;
	6335	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	994.2;
	6335	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	932.1;
	6335	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1001;
	6335	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4488.4;
	6335	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13870.8;
	6335	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7442.3;
	6335	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11464.5;
	6335	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1971.6;
	6336	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.6;
	6336	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	858.6;
	6336	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940.5;
	6336	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4172.3;
	6336	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12580.6;
	6336	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6795.1;
	6336	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10771;
	6336	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1836.3;
	6337	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939;
	6337	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	816.4;
	6337	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	896.1;
	6337	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3940.7;
	6337	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11774.1;
	6337	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6361.5;
	6337	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10274.4;
	6337	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1724.7;
	6338	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.5;
	6338	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	790.4;
	6338	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.4;
	6338	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3755.3;
	6338	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11123.5;
	6338	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6044;
	6338	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9929;
	6338	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1642.6;
	6339	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.4;
	6339	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	768.9;
	6339	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.8;
	6339	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3630.7;
	6339	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10701.1;
	6339	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5846.9;
	6339	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9776.7;
	6339	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1499.7;
	6340	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.2;
	6340	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	764.2;
	6340	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	823.8;
	6340	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3582.8;
	6340	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10584.2;
	6340	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5818.5;
	6340	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9865.8;
	6340	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1483.2;
	6341	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.7;
	6341	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	786.9;
	6341	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	839.6;
	6341	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3684.7;
	6341	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10980.4;
	6341	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6087.4;
	6341	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10361.9;
	6341	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1544.1;
	6342	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.4;
	6342	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	843;
	6342	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	905;
	6342	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3997.1;
	6342	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11864.6;
	6342	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6693.7;
	6342	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10997;
	6342	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1761.2;
	6343	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.9;
	6343	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	853.6;
	6343	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	929.1;
	6343	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4004.6;
	6343	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11993.4;
	6343	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6849.7;
	6343	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11004.8;
	6343	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1786.1;
	6344	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.8;
	6344	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	875.5;
	6344	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	943.1;
	6344	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4090.2;
	6344	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12270.8;
	6344	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6916;
	6344	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11406.3;
	6344	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1836.5;
	6345	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989.8;
	6345	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	930;
	6345	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1001.8;
	6345	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4431.8;
	6345	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13162.4;
	6345	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7309.1;
	6345	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12291.1;
	6345	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1959.9;
	6346	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1016.9;
	6346	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	993.3;
	6346	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1078.4;
	6346	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4848.1;
	6346	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14260.7;
	6346	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7871.8;
	6346	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13331.3;
	6346	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2113.1;
	6347	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1047.9;
	6347	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1054.5;
	6347	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1157.3;
	6347	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5210.7;
	6347	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15387.6;
	6347	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8433.5;
	6347	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14324.8;
	6347	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2288.2;
	6348	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1077.5;
	6348	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1116.9;
	6348	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1226.3;
	6348	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5535.5;
	6348	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16448;
	6348	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8977.7;
	6348	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15123.8;
	6348	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2471.8;
	6349	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1102.2;
	6349	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1174.3;
	6349	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1301.5;
	6349	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5803.5;
	6349	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17434.4;
	6349	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9502.1;
	6349	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15758.9;
	6349	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2663.2;
	6350	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1126.2;
	6350	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1217.8;
	6350	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1349.9;
	6350	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6006.5;
	6350	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18187.7;
	6350	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9933.1;
	6350	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16154.9;
	6350	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2767.2;
	6351	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1133.2;
	6351	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1244.6;
	6351	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1380.9;
	6351	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6113.8;
	6351	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18686;
	6351	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10287.5;
	6351	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16273;
	6351	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2794.5;
	6352	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1142.1;
	6352	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1253.6;
	6352	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1406.1;
	6352	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6132.4;
	6352	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18898.3;
	6352	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10464.4;
	6352	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16181.8;
	6352	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2799.6;
	6353	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1127.9;
	6353	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1220.4;
	6353	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1388.4;
	6353	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6012.5;
	6353	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18623.6;
	6353	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10403.6;
	6353	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15756;
	6353	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2758;
	6354	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1105.3;
	6354	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1168.8;
	6354	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1342;
	6354	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5752.5;
	6354	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17728.9;
	6354	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10037.5;
	6354	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15063.4;
	6354	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2641.4;
	6355	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1075.6;
	6355	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1125.6;
	6355	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1287.4;
	6355	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5576.5;
	6355	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16948.7;
	6355	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9547.7;
	6355	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14773.7;
	6355	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2523.7;
	6356	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1064.9;
	6356	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1092.6;
	6356	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1249.7;
	6356	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5490.1;
	6356	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16282.2;
	6356	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9184.5;
	6356	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14382.3;
	6356	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2407.1;
	6357	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1041.5;
	6357	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1027.1;
	6357	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1170;
	6357	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5200.4;
	6357	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15185.6;
	6357	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8510.7;
	6357	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13625.1;
	6357	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2213.7;
	6358	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000.5;
	6358	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	944.3;
	6358	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1070.4;
	6358	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4768.8;
	6358	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13758.8;
	6358	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7675.4;
	6358	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12568.9;
	6358	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1994;
	6359	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.4;
	6359	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	870.5;
	6359	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	978.7;
	6359	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4359;
	6359	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12409.6;
	6359	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6830.2;
	6359	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11588.4;
	6359	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1782.1;
	6360	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.7;
	6360	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	827.3;
	6360	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	903.7;
	6360	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4040.7;
	6360	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11308.9;
	6360	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6123.6;
	6360	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10806.1;
	6360	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1733.1;
	6361	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.7;
	6361	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	785.4;
	6361	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.8;
	6361	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3846;
	6361	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10598;
	6361	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5721.9;
	6361	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10334.4;
	6361	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1575.2;
	6362	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.5;
	6362	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	758.1;
	6362	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	826.4;
	6362	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3687.7;
	6362	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10014.9;
	6362	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5409.5;
	6362	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9979.8;
	6362	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1571.7;
	6363	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.9;
	6363	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.4;
	6363	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	804.9;
	6363	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3576.1;
	6363	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9676.5;
	6363	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5229.5;
	6363	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9818;
	6363	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1542.9;
	6364	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.8;
	6364	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.7;
	6364	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	800.7;
	6364	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3549.9;
	6364	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9644.5;
	6364	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5202.2;
	6364	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9932.5;
	6364	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1537.9;
	6365	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905;
	6365	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.8;
	6365	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	816.9;
	6365	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3644.6;
	6365	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10113.4;
	6365	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5498.6;
	6365	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10419.8;
	6365	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1587;
	6366	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.8;
	6366	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	811.7;
	6366	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884.8;
	6366	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3967.8;
	6366	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11073.3;
	6366	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6099.8;
	6366	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11147;
	6366	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1736.4;
	6367	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.9;
	6367	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	827.2;
	6367	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913.3;
	6367	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3989.7;
	6367	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11337.7;
	6367	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6239;
	6367	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11281.5;
	6367	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1772;
	6368	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.5;
	6368	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	846;
	6368	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	925.7;
	6368	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4031.9;
	6368	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11668.9;
	6368	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6397.9;
	6368	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11664.9;
	6368	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1847.5;
	6369	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986.6;
	6369	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	899.1;
	6369	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	975.4;
	6369	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4288.5;
	6369	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12533.4;
	6369	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6974.9;
	6369	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12505;
	6369	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1978.4;
	6370	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1024.7;
	6370	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	969;
	6370	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1044.5;
	6370	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4682.4;
	6370	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13698.3;
	6370	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7613.4;
	6370	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13547.8;
	6370	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2184.6;
	6371	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1064;
	6371	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1031.8;
	6371	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1117;
	6371	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5055.6;
	6371	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14889.6;
	6371	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8233.1;
	6371	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14390.9;
	6371	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2375.5;
	6372	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1094.1;
	6372	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1095.3;
	6372	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1182.9;
	6372	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5398.8;
	6372	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15980.8;
	6372	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8791.5;
	6372	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15069;
	6372	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2525.6;
	6373	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1125.7;
	6373	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1152.9;
	6373	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1245.3;
	6373	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5701.3;
	6373	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16970.2;
	6373	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9282.1;
	6373	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15606.1;
	6373	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2656.2;
	6374	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1153.1;
	6374	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1196.9;
	6374	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1301.4;
	6374	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5913.3;
	6374	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17747.1;
	6374	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9635.7;
	6374	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16004.5;
	6374	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2765.9;
	6375	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1170.2;
	6375	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1223;
	6375	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1340.1;
	6375	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5983;
	6375	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18252.9;
	6375	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9971.8;
	6375	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16202.2;
	6375	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2805.4;
	6376	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1178.3;
	6376	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1232.9;
	6376	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1358.8;
	6376	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5989;
	6376	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18464.8;
	6376	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10196.4;
	6376	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	16201;
	6376	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2824.9;
	6377	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1163.5;
	6377	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1203.6;
	6377	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1337.3;
	6377	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5808.2;
	6377	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18203.4;
	6377	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10112.2;
	6377	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15740.1;
	6377	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2753.7;
	6378	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1131.7;
	6378	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1159.9;
	6378	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1296.9;
	6378	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5583.8;
	6378	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17414.9;
	6378	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9786.5;
	6378	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14958.7;
	6378	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2648;
	6379	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1106.6;
	6379	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1115.4;
	6379	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1241.9;
	6379	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5450.4;
	6379	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16681.3;
	6379	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9491.4;
	6379	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14589.6;
	6379	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2563.4;
	6380	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1097.4;
	6380	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1081.3;
	6380	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1208;
	6380	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5443.4;
	6380	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16170.6;
	6380	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9273.6;
	6380	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14267.9;
	6380	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2470.1;
	6381	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1070.1;
	6381	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1018.8;
	6381	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1138.5;
	6381	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5201.7;
	6381	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15208;
	6381	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8735.4;
	6381	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13587.1;
	6381	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2290.7;
	6382	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1031;
	6382	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	940.3;
	6382	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1050.3;
	6382	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4840.7;
	6382	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13845.6;
	6382	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7997.8;
	6382	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12624.8;
	6382	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2081.4;
	6383	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	991.2;
	6383	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	868.4;
	6383	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	977.3;
	6383	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4442.9;
	6383	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12541.5;
	6383	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7221.5;
	6383	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11669.2;
	6383	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1929.3;
	6384	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.8;
	6384	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	803.2;
	6384	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	922.6;
	6384	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4102.9;
	6384	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11507.5;
	6384	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6775.4;
	6384	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10814.1;
	6384	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1817.5;
	6385	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920;
	6385	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753.9;
	6385	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	880.3;
	6385	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3887;
	6385	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10833.5;
	6385	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6375.6;
	6385	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10299.1;
	6385	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1752.3;
	6386	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.6;
	6386	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	726.9;
	6386	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.5;
	6386	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3741.1;
	6386	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10423.8;
	6386	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6102.1;
	6386	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9950;
	6386	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1684.9;
	6387	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.2;
	6387	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719.3;
	6387	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	836.6;
	6387	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3627.9;
	6387	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10139.7;
	6387	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5916.6;
	6387	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9808.3;
	6387	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1646.4;
	6388	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.1;
	6388	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.6;
	6388	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	832;
	6388	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3571.1;
	6388	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10180.3;
	6388	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5902.7;
	6388	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9902.7;
	6388	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1621.3;
	6389	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.8;
	6389	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	762.1;
	6389	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	853.3;
	6389	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3671.7;
	6389	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10723.2;
	6389	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6115.4;
	6389	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10454.5;
	6389	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1700.5;
	6390	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.5;
	6390	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	818.8;
	6390	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	916.4;
	6390	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3999.2;
	6390	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11812.3;
	6390	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6670.6;
	6390	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11121.1;
	6390	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1866.5;
	6391	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.1;
	6391	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	838.7;
	6391	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	943;
	6391	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4023;
	6391	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12186.6;
	6391	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6838.7;
	6391	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11212.9;
	6391	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1905.9;
	6392	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.2;
	6392	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	873.2;
	6392	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	959.8;
	6392	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4141.4;
	6392	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12481.4;
	6392	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6943.2;
	6392	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11642.5;
	6392	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1975.3;
	6393	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	984.7;
	6393	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	934.7;
	6393	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1018.9;
	6393	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4488;
	6393	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13343.7;
	6393	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7325.7;
	6393	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12514.9;
	6393	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2119.1;
	6394	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.1;
	6394	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	990.3;
	6394	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1088.4;
	6394	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4894;
	6394	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14357.2;
	6394	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7823.7;
	6394	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13231.2;
	6394	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2280.7;
	6395	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1051.7;
	6395	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1051.5;
	6395	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1159.2;
	6395	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5271.9;
	6395	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15307.8;
	6395	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8335.7;
	6395	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13747.6;
	6395	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2424.5;
	6396	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1083.8;
	6396	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1101.2;
	6396	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1215.2;
	6396	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5587.5;
	6396	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16174.2;
	6396	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8848.8;
	6396	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14024.2;
	6396	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2530.1;
	6397	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1121.3;
	6397	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1150;
	6397	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1271.9;
	6397	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5879.6;
	6397	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17050.6;
	6397	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9304.7;
	6397	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14536.3;
	6397	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2647.4;
	6398	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1145.1;
	6398	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1183.3;
	6398	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1312.1;
	6398	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6082;
	6398	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17743.1;
	6398	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9584.2;
	6398	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14990.9;
	6398	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2719.9;
	6399	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1153.4;
	6399	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1192.8;
	6399	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1326.6;
	6399	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6202.7;
	6399	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18152.7;
	6399	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9871;
	6399	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15086.7;
	6399	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2727.7;
	6400	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1161;
	6400	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1196.9;
	6400	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1340.5;
	6400	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6201.3;
	6400	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18298.8;
	6400	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10071.3;
	6400	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15033.6;
	6400	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2757.2;
	6401	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1156.1;
	6401	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1169.5;
	6401	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1315.7;
	6401	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6053.3;
	6401	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18035.9;
	6401	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9994.7;
	6401	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14700.4;
	6401	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2660;
	6402	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1133.1;
	6402	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1125.1;
	6402	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1269;
	6402	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5803.7;
	6402	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17235.4;
	6402	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9674.6;
	6402	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14042.4;
	6402	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2564.6;
	6403	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1102.3;
	6403	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1079;
	6403	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1210.3;
	6403	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5583.6;
	6403	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16464;
	6403	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9239.2;
	6403	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13746.8;
	6403	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2448.3;
	6404	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1090.7;
	6404	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1056.5;
	6404	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1184;
	6404	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5479.6;
	6404	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16036.4;
	6404	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8993.4;
	6404	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13534.6;
	6404	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2321.3;
	6405	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1066;
	6405	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1007.7;
	6405	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1129.9;
	6405	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5233.4;
	6405	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15266.6;
	6405	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8543.6;
	6405	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13034.1;
	6405	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2227.9;
	6406	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1034.9;
	6406	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	955.4;
	6406	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1070;
	6406	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4960;
	6406	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14271.8;
	6406	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8011;
	6406	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12394.8;
	6406	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2073.9;
	6407	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009.2;
	6407	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	909;
	6407	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1010.2;
	6407	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4668.3;
	6407	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13280.7;
	6407	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7407.6;
	6407	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11729;
	6407	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1939.9;
	6408	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981.2;
	6408	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	868.3;
	6408	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	961.4;
	6408	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4367.8;
	6408	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12285.2;
	6408	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6834.4;
	6408	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11035.4;
	6408	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1840.6;
	6409	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.3;
	6409	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	829.1;
	6409	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	920.2;
	6409	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4126.9;
	6409	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11518;
	6409	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6431.5;
	6409	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10520.4;
	6409	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1741;
	6410	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945;
	6410	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	806.8;
	6410	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	893.5;
	6410	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3941.6;
	6410	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10925.9;
	6410	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6124;
	6410	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10139.3;
	6410	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1666;
	6411	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.8;
	6411	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	783;
	6411	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	869.8;
	6411	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3802.3;
	6411	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10573.3;
	6411	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5958.9;
	6411	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9912.2;
	6411	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1633;
	6412	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.5;
	6412	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	771.8;
	6412	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	855.6;
	6412	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3735.7;
	6412	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10409;
	6412	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5874.9;
	6412	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9812.3;
	6412	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1594.3;
	6413	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.8;
	6413	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	769.6;
	6413	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	862.4;
	6413	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3742.9;
	6413	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10466.1;
	6413	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5925.4;
	6413	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9856.5;
	6413	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1583.3;
	6414	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.6;
	6414	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	784.2;
	6414	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	883.1;
	6414	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3807.2;
	6414	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10809.6;
	6414	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6090;
	6414	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10054.5;
	6414	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1623.4;
	6415	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.7;
	6415	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	800.3;
	6415	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	901.1;
	6415	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3817.7;
	6415	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11110.9;
	6415	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6254.4;
	6415	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10215.7;
	6415	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1665.7;
	6416	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.5;
	6416	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	840.2;
	6416	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	939.1;
	6416	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4040.6;
	6416	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11594.4;
	6416	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6563.3;
	6416	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10933.9;
	6416	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1765.6;
	6417	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	977.9;
	6417	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	887.9;
	6417	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	994.4;
	6417	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4441.8;
	6417	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12421.6;
	6417	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7049.2;
	6417	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11933.3;
	6417	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1923.9;
	6418	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1003.9;
	6418	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	945.4;
	6418	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1055.1;
	6418	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4767.1;
	6418	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13279.7;
	6418	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7663.8;
	6418	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12797.6;
	6418	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2083.7;
	6419	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1024;
	6419	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1005.5;
	6419	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1118.4;
	6419	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5062.2;
	6419	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14190.9;
	6419	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8272.9;
	6419	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13538.9;
	6419	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2253.1;
	6420	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1042.7;
	6420	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1051.9;
	6420	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1175;
	6420	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5324.3;
	6420	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15102.5;
	6420	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8795.8;
	6420	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14197.4;
	6420	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2386.3;
	6421	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1062.9;
	6421	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1093.5;
	6421	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1224.8;
	6421	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5581.4;
	6421	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15494.9;
	6421	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9162.1;
	6421	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14528.9;
	6421	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2490;
	6422	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1076.1;
	6422	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1121.8;
	6422	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1270.3;
	6422	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5779.8;
	6422	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15737.5;
	6422	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9494.2;
	6422	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14099.8;
	6422	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2531.1;
	6423	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1076.5;
	6423	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1130.4;
	6423	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1309.1;
	6423	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5878.9;
	6423	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15822.3;
	6423	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9613.1;
	6423	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13304.7;
	6423	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2586.8;
	6424	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1069;
	6424	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1110.4;
	6424	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1287.6;
	6424	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5880.1;
	6424	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15750.9;
	6424	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9252.2;
	6424	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12977.4;
	6424	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2573.1;
	6425	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1046.3;
	6425	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1070.9;
	6425	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1268.6;
	6425	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5794;
	6425	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15874.4;
	6425	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9002;
	6425	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12875.9;
	6425	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2482.4;
	6426	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1028.3;
	6426	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1033.9;
	6426	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1231;
	6426	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5557.1;
	6426	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15540.1;
	6426	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8768.1;
	6426	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12717.2;
	6426	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2385.2;
	6427	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1021.6;
	6427	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1010.8;
	6427	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1190.7;
	6427	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5422.4;
	6427	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15126.5;
	6427	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8537.4;
	6427	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12668.4;
	6427	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2292.6;
	6428	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1020.1;
	6428	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	978.1;
	6428	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1150.5;
	6428	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5348.7;
	6428	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14571.8;
	6428	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8378.4;
	6428	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12451.7;
	6428	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2231.9;
	6429	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1002;
	6429	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	928.1;
	6429	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1096.1;
	6429	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5105.5;
	6429	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13650.7;
	6429	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8017.5;
	6429	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12067;
	6429	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2080.9;
	6430	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969.9;
	6430	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	874.4;
	6430	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1033.2;
	6430	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4818.3;
	6430	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12723.3;
	6430	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7485.6;
	6430	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11522.9;
	6430	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1990.7;
	6431	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.8;
	6431	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	823.3;
	6431	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	948.8;
	6431	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4531.9;
	6431	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11838.6;
	6431	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6942;
	6431	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10924.6;
	6431	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1860.5;
	6432	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.8;
	6432	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	780.2;
	6432	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.1;
	6432	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4282.8;
	6432	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10953.1;
	6432	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6429.5;
	6432	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10443.4;
	6432	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1754.6;
	6433	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.5;
	6433	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749.9;
	6433	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	856.5;
	6433	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4074;
	6433	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10307.5;
	6433	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6054.8;
	6433	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10008.5;
	6433	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1649.1;
	6434	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.3;
	6434	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.4;
	6434	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	831.4;
	6434	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3894.5;
	6434	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9853.3;
	6434	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5812.2;
	6434	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9697;
	6434	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1582;
	6435	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867.2;
	6435	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	706.3;
	6435	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	822.2;
	6435	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3762.1;
	6435	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9543;
	6435	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5691.8;
	6435	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9473.9;
	6435	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1562.6;
	6436	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.9;
	6436	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	700.6;
	6436	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	826.9;
	6436	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3671.1;
	6436	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9354.9;
	6436	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5601.8;
	6436	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9388.6;
	6436	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1541;
	6437	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	865.3;
	6437	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	691.6;
	6437	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	827.6;
	6437	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3629.9;
	6437	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9366.4;
	6437	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5621.6;
	6437	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9423.6;
	6437	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1533.8;
	6438	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871.4;
	6438	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	706.2;
	6438	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838.1;
	6438	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3621.1;
	6438	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9537.9;
	6438	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5717.9;
	6438	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9521.5;
	6438	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1542;
	6439	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.8;
	6439	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	713.5;
	6439	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	856.8;
	6439	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3591.7;
	6439	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9710.1;
	6439	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5851.3;
	6439	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9556.8;
	6439	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1564.4;
	6440	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.3;
	6440	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744.4;
	6440	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	874.9;
	6440	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3738.4;
	6440	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10116;
	6440	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6082.1;
	6440	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10016.4;
	6440	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1682.7;
	6441	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.2;
	6441	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	781.2;
	6441	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913.2;
	6441	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4127.7;
	6441	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10722.4;
	6441	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6503.7;
	6441	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10639.7;
	6441	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1790.9;
	6442	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.6;
	6442	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	806.9;
	6442	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	946.9;
	6442	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4508.6;
	6442	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11235.1;
	6442	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6918.7;
	6442	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11153.5;
	6442	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1905.7;
	6443	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.3;
	6443	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	830.3;
	6443	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	966.8;
	6443	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4820.5;
	6443	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11628.4;
	6443	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7261.2;
	6443	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11742.8;
	6443	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2075.1;
	6444	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.5;
	6444	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	854.8;
	6444	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	987.9;
	6444	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5049.5;
	6444	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11894.5;
	6444	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7627.7;
	6444	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12687.9;
	6444	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2232.7;
	6445	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.4;
	6445	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	862.2;
	6445	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	998.4;
	6445	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5256;
	6445	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11951.9;
	6445	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7833.3;
	6445	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13451.9;
	6445	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2355.3;
	6446	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.6;
	6446	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	867;
	6446	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	965.1;
	6446	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5395.5;
	6446	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11951.6;
	6446	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7973.5;
	6446	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13416.3;
	6446	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2337.8;
	6447	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.5;
	6447	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	870.3;
	6447	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	953.2;
	6447	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5476;
	6447	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11982.8;
	6447	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7752.3;
	6447	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12835.3;
	6447	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2217.7;
	6448	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.5;
	6448	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	865.4;
	6448	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	953.6;
	6448	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5445.5;
	6448	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12055.1;
	6448	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7508;
	6448	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12331.3;
	6448	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2145.7;
	6449	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927;
	6449	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	858.1;
	6449	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	962.7;
	6449	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5351.1;
	6449	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12144.9;
	6449	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7241.3;
	6449	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12051.7;
	6449	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2128.6;
	6450	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.6;
	6450	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	846.3;
	6450	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	959.2;
	6450	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5162.9;
	6450	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12151.4;
	6450	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7093.4;
	6450	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11838.6;
	6450	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1996.3;
	6451	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.5;
	6451	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	844.6;
	6451	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	963.4;
	6451	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5096.4;
	6451	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12414.3;
	6451	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7141.4;
	6451	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12028.9;
	6451	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2053.8;
	6452	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931;
	6452	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	839.4;
	6452	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	964.1;
	6452	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5059.4;
	6452	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12321.5;
	6452	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7035.5;
	6452	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11942.6;
	6452	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2014.1;
	6453	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.2;
	6453	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	807.1;
	6453	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	934.1;
	6453	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4831.8;
	6453	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11904;
	6453	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6757.6;
	6453	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11513.5;
	6453	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1934.8;
	6454	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.7;
	6454	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755.4;
	6454	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	881.6;
	6454	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4515.8;
	6454	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11084.9;
	6454	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6304;
	6454	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10854.1;
	6454	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1814.3;
	6455	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871;
	6455	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	713.5;
	6455	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	825.3;
	6455	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4158.1;
	6455	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10175.5;
	6455	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5754.8;
	6455	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10143.8;
	6455	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1677.8;
	6456	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	847.7;
	6456	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	676.1;
	6456	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	763.8;
	6456	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3810.9;
	6456	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9407;
	6456	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5330.5;
	6456	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9568.2;
	6456	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1583.3;
	6457	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	833.4;
	6457	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	650.4;
	6457	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	739;
	6457	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3592.3;
	6457	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8883.8;
	6457	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5082.3;
	6457	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9212.8;
	6457	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1522.8;
	6458	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	821.2;
	6458	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	635.1;
	6458	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	721.3;
	6458	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3436.2;
	6458	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8552.4;
	6458	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4944.6;
	6458	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9062.5;
	6458	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1491.9;
	6459	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	817.4;
	6459	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	630.7;
	6459	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	717.6;
	6459	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3330.9;
	6459	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8381.2;
	6459	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4936;
	6459	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9065.8;
	6459	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1471.6;
	6460	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	822.6;
	6460	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	633.5;
	6460	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	725.1;
	6460	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3298.6;
	6460	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8470;
	6460	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5013.3;
	6460	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9272.9;
	6460	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1483.5;
	6461	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	836.3;
	6461	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	662.7;
	6461	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	756.9;
	6461	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3414;
	6461	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8972.8;
	6461	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5383.7;
	6461	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9819.3;
	6461	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1554.8;
	6462	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871.8;
	6462	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.3;
	6462	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	827;
	6462	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3723.5;
	6462	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9961.3;
	6462	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5952.5;
	6462	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10538;
	6462	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1710.7;
	6463	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.4;
	6463	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	754.2;
	6463	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.9;
	6463	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3790.9;
	6463	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10407.9;
	6463	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6169.7;
	6463	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10660;
	6463	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1723;
	6464	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.2;
	6464	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	761.3;
	6464	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	858.3;
	6464	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3764.4;
	6464	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10385.5;
	6464	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6213.3;
	6464	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10803;
	6464	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1795.2;
	6465	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908;
	6465	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	775.7;
	6465	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	869;
	6465	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3863.4;
	6465	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10611.8;
	6465	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6341.7;
	6465	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11286.2;
	6465	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1869.4;
	6466	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.6;
	6466	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	797.9;
	6466	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	883.3;
	6466	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4087.5;
	6466	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10858.7;
	6466	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6492.9;
	6466	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12000.4;
	6466	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1912.6;
	6467	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.1;
	6467	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	812.1;
	6467	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	891;
	6467	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4309.6;
	6467	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10993.9;
	6467	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6565.6;
	6467	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12761.2;
	6467	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1951.2;
	6468	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.5;
	6468	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	828.4;
	6468	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	901.9;
	6468	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4554.5;
	6468	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11018;
	6468	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6697.9;
	6468	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13406.6;
	6468	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1940;
	6469	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.9;
	6469	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	835.5;
	6469	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	911.3;
	6469	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4733.1;
	6469	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11001.8;
	6469	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6821.5;
	6469	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13901.1;
	6469	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1975.2;
	6470	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.7;
	6470	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	840.1;
	6470	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	915;
	6470	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4772.2;
	6470	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10863.3;
	6470	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6837.6;
	6470	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14214.7;
	6470	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1931.2;
	6471	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.5;
	6471	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	843;
	6471	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	916.3;
	6471	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4721.9;
	6471	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10668.6;
	6471	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6832.4;
	6471	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14401.4;
	6471	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1969.3;
	6472	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.8;
	6472	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	845.4;
	6472	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	916.9;
	6472	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4630.1;
	6472	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10611.6;
	6472	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6759.2;
	6472	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14196.4;
	6472	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1986.2;
	6473	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.6;
	6473	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	838.7;
	6473	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	910;
	6473	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4465.2;
	6473	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10573.1;
	6473	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6680.3;
	6473	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13727.5;
	6473	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1954.9;
	6474	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.2;
	6474	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	830.7;
	6474	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	906.2;
	6474	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4353;
	6474	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10576.4;
	6474	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6639.3;
	6474	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13185.8;
	6474	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1948.5;
	6475	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.4;
	6475	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	840.6;
	6475	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	918.7;
	6475	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4431.1;
	6475	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10844.7;
	6475	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6692.8;
	6475	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13163.2;
	6475	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1980.6;
	6476	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.4;
	6476	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	831.6;
	6476	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	920.3;
	6476	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4449.7;
	6476	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10820.6;
	6476	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6534.3;
	6476	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12864.7;
	6476	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1953;
	6477	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.9;
	6477	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	798.3;
	6477	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889;
	6477	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4260.5;
	6477	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10418.2;
	6477	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6259;
	6477	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12197.9;
	6477	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1862.8;
	6478	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.8;
	6478	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.7;
	6478	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843;
	6478	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3976;
	6478	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9728.6;
	6478	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5815.2;
	6478	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11310.4;
	6478	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1740.8;
	6479	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	861.3;
	6479	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	695.8;
	6479	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	785.3;
	6479	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3635.7;
	6479	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8843.3;
	6479	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5209.4;
	6479	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10434.3;
	6479	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1574.2;
	6480	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	853.9;
	6480	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	651.9;
	6480	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	756.9;
	6480	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3394.7;
	6480	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8220.9;
	6480	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4768.6;
	6480	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9784.2;
	6480	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1467.5;
	6481	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	850.1;
	6481	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	637.7;
	6481	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	735.5;
	6481	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3232.4;
	6481	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7863.4;
	6481	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4484.9;
	6481	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9436.3;
	6481	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1403.7;
	6482	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	841.8;
	6482	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	620.2;
	6482	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	724.8;
	6482	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3113.5;
	6482	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7629.7;
	6482	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4348.4;
	6482	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9201.8;
	6482	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1364.6;
	6483	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	840.8;
	6483	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	614.2;
	6483	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	723.9;
	6483	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3052.1;
	6483	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7567;
	6483	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4297.1;
	6483	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9142.7;
	6483	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1342;
	6484	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	838.5;
	6484	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	615.9;
	6484	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	730.3;
	6484	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3044.5;
	6484	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7748.6;
	6484	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4372.6;
	6484	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9284.6;
	6484	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1352.8;
	6485	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	848.4;
	6485	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	640.6;
	6485	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	761.4;
	6485	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3153;
	6485	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8317.3;
	6485	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4698.3;
	6485	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9782;
	6485	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1416;
	6486	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.1;
	6486	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	701.2;
	6486	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	831;
	6486	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3496.8;
	6486	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9407.4;
	6486	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5331.6;
	6486	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10430.8;
	6486	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1578.2;
	6487	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.1;
	6487	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	722.2;
	6487	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.7;
	6487	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3564.8;
	6487	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9777.5;
	6487	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5591.3;
	6487	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10493.3;
	6487	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1640.7;
	6488	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.9;
	6488	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.8;
	6488	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.4;
	6488	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3511.2;
	6488	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9841.5;
	6488	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5600.1;
	6488	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10600;
	6488	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1677.9;
	6489	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.7;
	6489	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741;
	6489	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	879.3;
	6489	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3557;
	6489	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10185.5;
	6489	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5741.9;
	6489	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10974.9;
	6489	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1733;
	6490	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.1;
	6490	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	763.7;
	6490	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	899.2;
	6490	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3627.7;
	6490	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10612.9;
	6490	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5925.4;
	6490	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11413.3;
	6490	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1816.4;
	6491	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.1;
	6491	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	790.3;
	6491	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	919.5;
	6491	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3709.4;
	6491	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10988.5;
	6491	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6089.2;
	6491	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11910.5;
	6491	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1880.2;
	6492	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.1;
	6492	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	814.9;
	6492	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	938;
	6492	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3814.2;
	6492	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11333.6;
	6492	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6238.4;
	6492	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12548.9;
	6492	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1946.8;
	6493	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.3;
	6493	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	835;
	6493	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	960.5;
	6493	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3948.8;
	6493	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11660;
	6493	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6412.9;
	6493	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13364;
	6493	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2012.5;
	6494	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.4;
	6494	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	845.8;
	6494	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	979.9;
	6494	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4027.5;
	6494	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11913.7;
	6494	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6502.7;
	6494	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13806.9;
	6494	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2043.7;
	6495	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981.3;
	6495	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	867.5;
	6495	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1003.4;
	6495	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4088;
	6495	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12028.8;
	6495	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6564.4;
	6495	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13875.2;
	6495	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2064.8;
	6496	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	998.3;
	6496	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	892.6;
	6496	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1020.3;
	6496	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4138.9;
	6496	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12032.4;
	6496	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6686.5;
	6496	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13731.9;
	6496	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2042.2;
	6497	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	997.9;
	6497	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	895.8;
	6497	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1019.4;
	6497	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4091.8;
	6497	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11891.6;
	6497	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6767.3;
	6497	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13362.1;
	6497	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2033;
	6498	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989.7;
	6498	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	876.2;
	6498	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1006.6;
	6498	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4040;
	6498	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11653.7;
	6498	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6790.2;
	6498	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12885.8;
	6498	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2010;
	6499	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.1;
	6499	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	861.9;
	6499	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	999.3;
	6499	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4123.5;
	6499	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11609.5;
	6499	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6891.5;
	6499	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12826;
	6499	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2014.2;
	6500	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981.9;
	6500	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	843;
	6500	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	998.8;
	6500	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4198.8;
	6500	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11445.7;
	6500	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6874.5;
	6500	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12589.3;
	6500	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2000.1;
	6501	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.7;
	6501	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	797.8;
	6501	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	948.9;
	6501	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4058.1;
	6501	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10738.6;
	6501	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6539;
	6501	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11893.4;
	6501	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1868.3;
	6502	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.5;
	6502	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.8;
	6502	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	870.2;
	6502	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3747.4;
	6502	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9730.3;
	6502	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5970.1;
	6502	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10972.1;
	6502	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1712.7;
	6503	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.4;
	6503	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	680.6;
	6503	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	797.4;
	6503	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3424.2;
	6503	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8718;
	6503	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5367.7;
	6503	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10045.7;
	6503	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1541.1;
	6504	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	852.1;
	6504	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	651.2;
	6504	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	750.1;
	6504	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3174.5;
	6504	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7998.9;
	6504	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4904.1;
	6504	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9390.5;
	6504	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1432.6;
	6505	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	840.4;
	6505	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	630.7;
	6505	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	725.2;
	6505	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3020.1;
	6505	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7584;
	6505	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4605.1;
	6505	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8960.6;
	6505	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1371.1;
	6506	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	829.4;
	6506	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	622.7;
	6506	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	707.5;
	6506	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2899.7;
	6506	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7309.2;
	6506	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4409.5;
	6506	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8652.1;
	6506	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1317.4;
	6507	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	829.1;
	6507	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	617.6;
	6507	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	698;
	6507	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2841.1;
	6507	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7223.5;
	6507	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4291.3;
	6507	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8535.4;
	6507	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1203;
	6508	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	831.5;
	6508	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	621.3;
	6508	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	698.5;
	6508	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2843.5;
	6508	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7364.8;
	6508	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4383.6;
	6508	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8643;
	6508	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1189.6;
	6509	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	843.8;
	6509	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	648.1;
	6509	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	722.9;
	6509	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2973.5;
	6509	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7940.2;
	6509	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4681.5;
	6509	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9129.3;
	6509	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1264.9;
	6510	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.4;
	6510	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	704.3;
	6510	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	792.5;
	6510	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3309.2;
	6510	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8989.2;
	6510	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5303.2;
	6510	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9811.4;
	6510	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1479.3;
	6511	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.2;
	6511	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	721.2;
	6511	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	816.6;
	6511	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3380.5;
	6511	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9328.2;
	6511	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5503.7;
	6511	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9906;
	6511	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1521.3;
	6512	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.7;
	6512	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	721.3;
	6512	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	818.6;
	6512	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3381.3;
	6512	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9472.1;
	6512	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5591.5;
	6512	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10214.5;
	6512	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1540.9;
	6513	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.8;
	6513	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	750.6;
	6513	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	853.4;
	6513	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3568.9;
	6513	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9920.1;
	6513	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5924.8;
	6513	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10873.7;
	6513	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1596.5;
	6514	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.7;
	6514	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	779.8;
	6514	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	891.8;
	6514	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3794.6;
	6514	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10423.1;
	6514	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6331.9;
	6514	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11600.3;
	6514	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1681.6;
	6515	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.8;
	6515	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	821.2;
	6515	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	943.4;
	6515	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4043.1;
	6515	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10948.2;
	6515	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6733.5;
	6515	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12334.3;
	6515	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1790.9;
	6516	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969.5;
	6516	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	873.9;
	6516	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	993.9;
	6516	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4292.4;
	6516	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11530.9;
	6516	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7188.7;
	6516	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13008.4;
	6516	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1915.9;
	6517	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990.8;
	6517	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	918.2;
	6517	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1050.7;
	6517	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4536.6;
	6517	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12291.1;
	6517	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7629.7;
	6517	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13733.4;
	6517	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2070.7;
	6518	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1017.2;
	6518	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	968.1;
	6518	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1096.2;
	6518	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4705.2;
	6518	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13087;
	6518	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8068.3;
	6518	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14334.5;
	6518	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2164.1;
	6519	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1034.8;
	6519	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1017;
	6519	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1139.6;
	6519	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4845.2;
	6519	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13906.2;
	6519	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8465.7;
	6519	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14801.8;
	6519	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2239.1;
	6520	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1049.1;
	6520	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1048;
	6520	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1182.4;
	6520	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4980.5;
	6520	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14561.2;
	6520	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8779.9;
	6520	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15034.4;
	6520	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2267.1;
	6521	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1051.6;
	6521	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1049.8;
	6521	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1182.8;
	6521	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5012;
	6521	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14738.7;
	6521	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8869.6;
	6521	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14797.1;
	6521	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2256.8;
	6522	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1038.9;
	6522	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1022.5;
	6522	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1155.2;
	6522	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4869.6;
	6522	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14305.8;
	6522	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8615.4;
	6522	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14079.4;
	6522	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2169.6;
	6523	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009.3;
	6523	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	981.9;
	6523	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1111.1;
	6523	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4779.7;
	6523	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13722;
	6523	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8271.4;
	6523	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13587.3;
	6523	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2083.8;
	6524	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	997.3;
	6524	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	951.2;
	6524	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1079.7;
	6524	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4748.9;
	6524	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13223.1;
	6524	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7994;
	6524	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13149.5;
	6524	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2001.7;
	6525	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.4;
	6525	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	897.2;
	6525	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1011.5;
	6525	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4536.8;
	6525	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12271.5;
	6525	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7370.8;
	6525	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12357.7;
	6525	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1836.6;
	6526	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.4;
	6526	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	826.8;
	6526	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	923.4;
	6526	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4177.6;
	6526	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11087.6;
	6526	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6614.4;
	6526	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11373.6;
	6526	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1654.5;
	6527	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.2;
	6527	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	757.2;
	6527	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	844.5;
	6527	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3796.3;
	6527	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9921.7;
	6527	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5836.2;
	6527	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10392;
	6527	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1468.9;
	6528	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	859.4;
	6528	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	705;
	6528	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	762.9;
	6528	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3514.6;
	6528	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9078.7;
	6528	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5230;
	6528	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9549.6;
	6528	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1448.2;
	6529	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	840.6;
	6529	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	668.8;
	6529	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	734;
	6529	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3337.8;
	6529	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8486.8;
	6529	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4918.9;
	6529	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8999.3;
	6529	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1301.8;
	6530	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	832.2;
	6530	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	646;
	6530	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	713.6;
	6530	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3215.5;
	6530	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8084.3;
	6530	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4676.2;
	6530	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8635.1;
	6530	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1313.1;
	6531	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	823.9;
	6531	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	632.6;
	6531	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	700;
	6531	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3135.2;
	6531	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7841;
	6531	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4555.4;
	6531	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8445.9;
	6531	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1294.7;
	6532	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	828.5;
	6532	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	631;
	6532	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	704.5;
	6532	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3123.9;
	6532	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7852.9;
	6532	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4564.4;
	6532	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8499.1;
	6532	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1294.4;
	6533	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	834.9;
	6533	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	646.8;
	6533	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	728.3;
	6533	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3222.9;
	6533	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8336.8;
	6533	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4859.8;
	6533	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8897;
	6533	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1359.4;
	6534	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	862;
	6534	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	705.7;
	6534	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	801.6;
	6534	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3551.5;
	6534	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9300.7;
	6534	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5447.2;
	6534	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9568.3;
	6534	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1509.3;
	6535	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.9;
	6535	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719.8;
	6535	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	827.7;
	6535	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3584.4;
	6535	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9548.3;
	6535	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5629.7;
	6535	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9657.8;
	6535	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1542.8;
	6536	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.4;
	6536	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	731.5;
	6536	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824.2;
	6536	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3638.2;
	6536	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9693.2;
	6536	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5700.1;
	6536	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9953.5;
	6536	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1584.6;
	6537	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.7;
	6537	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755;
	6537	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	842.6;
	6537	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3890.6;
	6537	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10096.3;
	6537	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6017.6;
	6537	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10511.4;
	6537	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1625.1;
	6538	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.6;
	6538	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	781;
	6538	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.6;
	6538	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4256.4;
	6538	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10548.7;
	6538	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6371.7;
	6538	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11128.8;
	6538	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1711.9;
	6539	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.7;
	6539	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	789.7;
	6539	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897.6;
	6539	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4569.9;
	6539	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10897.2;
	6539	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6749.3;
	6539	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11695.2;
	6539	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1783.3;
	6540	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.5;
	6540	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	813.8;
	6540	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	926.8;
	6540	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4870.6;
	6540	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11358.3;
	6540	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7155.8;
	6540	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12429.8;
	6540	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1859.6;
	6541	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	978.2;
	6541	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	838.4;
	6541	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	959.4;
	6541	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5108.1;
	6541	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11814;
	6541	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7593.5;
	6541	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13128.1;
	6541	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1933.9;
	6542	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1001.8;
	6542	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	857.1;
	6542	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	990.9;
	6542	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5260.2;
	6542	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12165.2;
	6542	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7988.9;
	6542	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13656.7;
	6542	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2000.8;
	6543	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1021.1;
	6543	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	869.2;
	6543	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1013.9;
	6543	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5323;
	6543	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12427.9;
	6543	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8325.8;
	6543	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13943.7;
	6543	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2027.9;
	6544	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1031.6;
	6544	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	880.2;
	6544	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1029.1;
	6544	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5376.6;
	6544	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12592.5;
	6544	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8527.1;
	6544	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13927.1;
	6544	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2048.3;
	6545	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1020.4;
	6545	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	866.6;
	6545	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1020.6;
	6545	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5283.1;
	6545	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12483.2;
	6545	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8486.7;
	6545	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13551.7;
	6545	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2007.9;
	6546	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990.6;
	6546	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	843.8;
	6546	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1007;
	6546	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5045.5;
	6546	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12004.7;
	6546	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8149.8;
	6546	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12772;
	6546	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1940.3;
	6547	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	974.8;
	6547	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	836.7;
	6547	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	984.4;
	6547	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4886;
	6547	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11754;
	6547	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7831.5;
	6547	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12309.4;
	6547	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1909.7;
	6548	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	976.6;
	6548	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	828.3;
	6548	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	966.5;
	6548	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4816.9;
	6548	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11554.7;
	6548	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7598.3;
	6548	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11901.6;
	6548	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1844.4;
	6549	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.6;
	6549	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	788.1;
	6549	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	914.9;
	6549	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4526.4;
	6549	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10858.2;
	6549	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7038;
	6549	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11161.6;
	6549	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1700.4;
	6550	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925.2;
	6550	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.6;
	6550	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	850.3;
	6550	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4152.2;
	6550	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9852.8;
	6550	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6314.1;
	6550	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10218;
	6550	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1520.3;
	6551	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.8;
	6551	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	680.1;
	6551	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	791.6;
	6551	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3721.4;
	6551	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8813.9;
	6551	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5561.4;
	6551	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9239;
	6551	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1389.4;
	6552	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871.3;
	6552	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	638.2;
	6552	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	734.1;
	6552	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3394.8;
	6552	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8157.8;
	6552	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4815.2;
	6552	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8574.5;
	6552	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1320.4;
	6553	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	853.7;
	6553	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	617.1;
	6553	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	703.8;
	6553	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3159.9;
	6553	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7694.5;
	6553	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4472.6;
	6553	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8123.3;
	6553	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1281.9;
	6554	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	841.4;
	6554	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	606;
	6554	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	683;
	6554	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2996.9;
	6554	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7404.1;
	6554	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4278.3;
	6554	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7801.7;
	6554	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1227.8;
	6555	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	838.6;
	6555	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	600.1;
	6555	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	675.1;
	6555	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2905.3;
	6555	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7296.1;
	6555	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4158;
	6555	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7674.2;
	6555	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1202.7;
	6556	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	838.7;
	6556	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	612.2;
	6556	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	675.5;
	6556	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2869.6;
	6556	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7423.8;
	6556	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4164.3;
	6556	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7763.4;
	6556	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1200.1;
	6557	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	842.4;
	6557	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	631.2;
	6557	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	703.8;
	6557	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2975;
	6557	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7950.9;
	6557	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4463.9;
	6557	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8226.6;
	6557	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1291.9;
	6558	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	860.1;
	6558	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	688.5;
	6558	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	768.7;
	6558	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3293.2;
	6558	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8909.9;
	6558	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5029.7;
	6558	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8866.4;
	6558	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1451.3;
	6559	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	876.5;
	6559	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	708.4;
	6559	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	786.6;
	6559	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3304.4;
	6559	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9279.4;
	6559	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5223.8;
	6559	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9012.4;
	6559	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1474.4;
	6560	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.8;
	6560	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	721.3;
	6560	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	786.9;
	6560	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3333.4;
	6560	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9393.9;
	6560	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5281.1;
	6560	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9316.8;
	6560	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1499.4;
	6561	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.8;
	6561	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.1;
	6561	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	819.9;
	6561	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3543.3;
	6561	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9791.5;
	6561	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5508.6;
	6561	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9777.7;
	6561	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1570.8;
	6562	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.3;
	6562	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	768.6;
	6562	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	850.9;
	6562	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3814.9;
	6562	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10155.1;
	6562	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5758.6;
	6562	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10275.2;
	6562	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1635.7;
	6563	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.9;
	6563	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	784.5;
	6563	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	870;
	6563	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4090.7;
	6563	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10577.6;
	6563	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5988.8;
	6563	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10677.6;
	6563	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1706.3;
	6564	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.2;
	6564	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	799.8;
	6564	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	893.5;
	6564	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4348;
	6564	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11024.9;
	6564	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6222.2;
	6564	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11026.4;
	6564	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1766;
	6565	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	974.1;
	6565	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	825.7;
	6565	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	930.6;
	6565	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4592.9;
	6565	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11547.2;
	6565	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6490.2;
	6565	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11559.1;
	6565	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1842.9;
	6566	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992;
	6566	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	848;
	6566	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	962.6;
	6566	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4770.5;
	6566	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12010.8;
	6566	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6797.8;
	6566	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12038.5;
	6566	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1913.1;
	6567	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1003.6;
	6567	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	864.9;
	6567	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	983.8;
	6567	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4891;
	6567	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12439.8;
	6567	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7051.9;
	6567	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12435;
	6567	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1944.6;
	6568	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1011.4;
	6568	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	879;
	6568	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1000.4;
	6568	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4919.6;
	6568	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12733.6;
	6568	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7298.4;
	6568	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12643.8;
	6568	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1995.8;
	6569	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.4;
	6569	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	873.1;
	6569	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	997.8;
	6569	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4780.8;
	6569	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12709;
	6569	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7324.2;
	6569	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12404.6;
	6569	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1953;
	6570	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	979.6;
	6570	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	844;
	6570	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	970.6;
	6570	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4504.4;
	6570	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12169.3;
	6570	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6994.7;
	6570	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11750.2;
	6570	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1884.1;
	6571	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.3;
	6571	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	822.7;
	6571	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	930.6;
	6571	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4335.2;
	6571	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11655.5;
	6571	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6719;
	6571	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11349.6;
	6571	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1816.8;
	6572	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.2;
	6572	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	808.6;
	6572	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	914.7;
	6572	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4226.9;
	6572	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11317.2;
	6572	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6513.9;
	6572	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10993.4;
	6572	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1715.7;
	6573	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946;
	6573	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	770.6;
	6573	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878.1;
	6573	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4009.4;
	6573	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10603.9;
	6573	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6153.4;
	6573	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10427.8;
	6573	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1651.7;
	6574	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.4;
	6574	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.9;
	6574	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	829.9;
	6574	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3773.9;
	6574	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9830.3;
	6574	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5684;
	6574	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9794.9;
	6574	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1512.1;
	6575	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895;
	6575	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	689.6;
	6575	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	775.1;
	6575	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3483.3;
	6575	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8996.3;
	6575	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5172;
	6575	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9104.1;
	6575	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1389.4;
	6576	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.3;
	6576	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	656.7;
	6576	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	762.2;
	6576	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3210.6;
	6576	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8307.5;
	6576	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4732;
	6576	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8454.9;
	6576	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1355.3;
	6577	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	844.7;
	6577	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	630.7;
	6577	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	726.2;
	6577	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3001.2;
	6577	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7818.6;
	6577	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4423.9;
	6577	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8034.3;
	6577	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1289.8;
	6578	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	833.1;
	6578	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	617.4;
	6578	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	703.5;
	6578	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2871.1;
	6578	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7516.6;
	6578	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4231.6;
	6578	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7761.1;
	6578	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1239.6;
	6579	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	813.2;
	6579	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	608.3;
	6579	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	689.5;
	6579	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2799.3;
	6579	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7349.3;
	6579	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4112.5;
	6579	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7634.6;
	6579	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1198.4;
	6580	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	807.4;
	6580	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	607.6;
	6580	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	683.8;
	6580	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2751.7;
	6580	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7304.1;
	6580	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4073.1;
	6580	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7574;
	6580	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1205.3;
	6581	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	821.1;
	6581	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	615.3;
	6581	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	693.1;
	6581	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2763.1;
	6581	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7448.3;
	6581	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4131.7;
	6581	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7664.7;
	6581	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1219.9;
	6582	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	835.2;
	6582	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	632.8;
	6582	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	715.3;
	6582	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2835.1;
	6582	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7819.3;
	6582	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4281.2;
	6582	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7907.3;
	6582	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1250.3;
	6583	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	839;
	6583	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	651.1;
	6583	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	741.2;
	6583	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2856.3;
	6583	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8081.2;
	6583	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4480.3;
	6583	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8003.8;
	6583	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1296;
	6584	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	846.7;
	6584	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	664.8;
	6584	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	765.6;
	6584	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3001;
	6584	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8553.1;
	6584	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4694.6;
	6584	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8442.4;
	6584	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1358.8;
	6585	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871.3;
	6585	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	699.2;
	6585	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	800.1;
	6585	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3278.4;
	6585	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9211.3;
	6585	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5015.3;
	6585	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9019.9;
	6585	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1425.7;
	6586	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900;
	6586	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.1;
	6586	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	831.2;
	6586	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3620.8;
	6586	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9816;
	6586	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5289;
	6586	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9606.4;
	6586	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1527.3;
	6587	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922;
	6587	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	748.6;
	6587	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.1;
	6587	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3924.3;
	6587	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10383.8;
	6587	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5471.4;
	6587	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10081.4;
	6587	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1623.1;
	6588	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.5;
	6588	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	764.1;
	6588	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	893;
	6588	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4216.5;
	6588	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10931.7;
	6588	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5648;
	6588	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10538.4;
	6588	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1661.1;
	6589	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.4;
	6589	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	786.9;
	6589	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	931.9;
	6589	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4475.5;
	6589	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11451.4;
	6589	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5860.3;
	6589	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10961.9;
	6589	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1719.9;
	6590	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1003.1;
	6590	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	823.9;
	6590	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	969.4;
	6590	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4641.9;
	6590	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11927.1;
	6590	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6075.3;
	6590	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11299.8;
	6590	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1756;
	6591	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1025.9;
	6591	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	846.8;
	6591	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	997.5;
	6591	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4678.8;
	6591	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12384.8;
	6591	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6245.2;
	6591	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11377.9;
	6591	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1769.9;
	6592	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1036;
	6592	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	864.4;
	6592	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1014.8;
	6592	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4672.2;
	6592	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12646.2;
	6592	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6318.7;
	6592	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11257.8;
	6592	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1783.5;
	6593	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1028.6;
	6593	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	868.7;
	6593	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1012.7;
	6593	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4571.1;
	6593	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12597.4;
	6593	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6284.3;
	6593	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11060.7;
	6593	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1783.6;
	6594	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1016.3;
	6594	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	848.3;
	6594	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1003.7;
	6594	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4366.5;
	6594	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12107.6;
	6594	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6159.6;
	6594	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10864.2;
	6594	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1716.7;
	6595	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	993.2;
	6595	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	825.1;
	6595	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	984.3;
	6595	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4288.6;
	6595	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11733.6;
	6595	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6187.7;
	6595	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10984.1;
	6595	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1730.2;
	6596	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989.1;
	6596	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	812.7;
	6596	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	972.3;
	6596	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4227.2;
	6596	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11409.3;
	6596	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6149.6;
	6596	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10861.6;
	6596	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1706.6;
	6597	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969;
	6597	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	778;
	6597	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	929.4;
	6597	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4043.9;
	6597	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10806.8;
	6597	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5900.1;
	6597	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10528;
	6597	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1555.1;
	6598	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.9;
	6598	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.1;
	6598	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	864;
	6598	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3817.3;
	6598	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10031.6;
	6598	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5471.7;
	6598	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10048;
	6598	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1501.2;
	6599	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.5;
	6599	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	693.5;
	6599	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	802.4;
	6599	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3566.7;
	6599	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9207.5;
	6599	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5043.6;
	6599	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9482;
	6599	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1421.9;
	6600	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877;
	6600	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	656.3;
	6600	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	755.3;
	6600	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3313.2;
	6600	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8457.7;
	6600	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4643.2;
	6600	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8928.4;
	6600	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1348.1;
	6601	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	858.5;
	6601	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	629.2;
	6601	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	720.8;
	6601	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3140.4;
	6601	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7877.9;
	6601	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4350.2;
	6601	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8511.3;
	6601	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1271.3;
	6602	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	845.4;
	6602	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	610.9;
	6602	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	698;
	6602	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3013.4;
	6602	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7489.7;
	6602	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4145.2;
	6602	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8193.9;
	6602	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1210.7;
	6603	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	837.5;
	6603	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.6;
	6603	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	685.2;
	6603	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2925.6;
	6603	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7255;
	6603	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4018.5;
	6603	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8021.4;
	6603	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1200.2;
	6604	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	833.9;
	6604	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	593.2;
	6604	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	678.4;
	6604	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2877.1;
	6604	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7149.2;
	6604	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3965.1;
	6604	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7937.6;
	6604	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1170.3;
	6605	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	832.2;
	6605	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	594.8;
	6605	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	679.3;
	6605	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2869.3;
	6605	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7175.3;
	6605	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3993.4;
	6605	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7954.4;
	6605	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1191;
	6606	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	835.1;
	6606	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	607.6;
	6606	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	693.4;
	6606	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2899.5;
	6606	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7351.1;
	6606	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4108.4;
	6606	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8039.5;
	6606	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1220.2;
	6607	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	840.3;
	6607	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	615.2;
	6607	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	712.5;
	6607	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2895.9;
	6607	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7539.1;
	6607	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4238.8;
	6607	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8057.3;
	6607	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1233.1;
	6608	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	848.1;
	6608	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	651.3;
	6608	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	744.1;
	6608	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3026.9;
	6608	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8068;
	6608	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4507.9;
	6608	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8458.4;
	6608	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1291.2;
	6609	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.5;
	6609	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	689.3;
	6609	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	786.8;
	6609	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3294.4;
	6609	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8802.4;
	6609	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4917.3;
	6609	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9131.1;
	6609	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1400.4;
	6610	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.1;
	6610	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714.9;
	6610	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	823.3;
	6610	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3538.7;
	6610	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9497.3;
	6610	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5310;
	6610	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9888.1;
	6610	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1509.2;
	6611	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.8;
	6611	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.5;
	6611	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.1;
	6611	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3773.3;
	6611	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10164.3;
	6611	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5709;
	6611	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10638.6;
	6611	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1619;
	6612	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969.1;
	6612	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	794.7;
	6612	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	905.9;
	6612	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4028.4;
	6612	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10876.7;
	6612	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6117.3;
	6612	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11376;
	6612	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1734;
	6613	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.6;
	6613	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	838.1;
	6613	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	957;
	6613	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4361.1;
	6613	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11635.7;
	6613	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6541.9;
	6613	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12107.5;
	6613	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1837.9;
	6614	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030.6;
	6614	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	879.4;
	6614	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1004.8;
	6614	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4553.9;
	6614	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12339;
	6614	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6926.5;
	6614	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12695.9;
	6614	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1929.6;
	6615	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1051.4;
	6615	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	916.3;
	6615	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1044;
	6615	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4730.1;
	6615	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12926.6;
	6615	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7285.2;
	6615	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13056;
	6615	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2021;
	6616	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1068.8;
	6616	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	935.5;
	6616	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1072.5;
	6616	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4812;
	6616	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13296.4;
	6616	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7549.6;
	6616	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13203;
	6616	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2052;
	6617	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1070.9;
	6617	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	939.8;
	6617	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1083.1;
	6617	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4769.5;
	6617	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13323.1;
	6617	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7678.7;
	6617	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13073.8;
	6617	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2044.2;
	6618	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1055.9;
	6618	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	912.5;
	6618	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1058.3;
	6618	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4632.7;
	6618	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12883.3;
	6618	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7476.9;
	6618	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12644.9;
	6618	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1934.5;
	6619	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1037.2;
	6619	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	896.1;
	6619	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1033.1;
	6619	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4615.4;
	6619	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12631.6;
	6619	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7292.7;
	6619	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12458;
	6619	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1936.6;
	6620	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030.1;
	6620	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	876.9;
	6620	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1015.7;
	6620	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4596.6;
	6620	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12321.7;
	6620	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7097.1;
	6620	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12184.4;
	6620	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1885.2;
	6621	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1005.1;
	6621	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	827.1;
	6621	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	961.4;
	6621	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4403.4;
	6621	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11572.9;
	6621	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6619.3;
	6621	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11558.9;
	6621	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1755.2;
	6622	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.2;
	6622	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	761.2;
	6622	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	890.6;
	6622	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4096.8;
	6622	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10499.9;
	6622	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5968.8;
	6622	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10697.9;
	6622	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1601.7;
	6623	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.4;
	6623	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	699.7;
	6623	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	816;
	6623	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3743.6;
	6623	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9388.6;
	6623	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5303.8;
	6623	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9850.9;
	6623	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1479.5;
	6624	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.8;
	6624	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	665.2;
	6624	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	764.7;
	6624	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3510.9;
	6624	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8577.1;
	6624	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4800.5;
	6624	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9222.6;
	6624	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1394.3;
	6625	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.4;
	6625	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	639.3;
	6625	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	730.2;
	6625	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3327.1;
	6625	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8048.3;
	6625	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4485.1;
	6625	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8801.1;
	6625	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1333.9;
	6626	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871.4;
	6626	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	623.1;
	6626	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	707.3;
	6626	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3189.1;
	6626	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7750.9;
	6626	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4307.3;
	6626	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8549.1;
	6626	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1276.5;
	6627	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	858.1;
	6627	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	616.6;
	6627	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	695.6;
	6627	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3097.9;
	6627	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7586.4;
	6627	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4242.5;
	6627	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8423.5;
	6627	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1270;
	6628	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	853.8;
	6628	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	618.8;
	6628	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	695.6;
	6628	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3071.6;
	6628	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7683.9;
	6628	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4279.1;
	6628	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8541.6;
	6628	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1298.3;
	6629	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	865;
	6629	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	646.6;
	6629	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	722.8;
	6629	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3205.1;
	6629	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8221.2;
	6629	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4610.7;
	6629	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9022.7;
	6629	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1369.8;
	6630	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.8;
	6630	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711.3;
	6630	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	791.2;
	6630	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3529;
	6630	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9269.3;
	6630	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5202.3;
	6630	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9697.9;
	6630	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1502.5;
	6631	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.6;
	6631	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.7;
	6631	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	823.9;
	6631	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3597.8;
	6631	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9683.1;
	6631	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5420.2;
	6631	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9794.3;
	6631	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1502.3;
	6632	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.5;
	6632	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744.2;
	6632	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	825.2;
	6632	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3651.9;
	6632	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9835.3;
	6632	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5500.2;
	6632	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10024.2;
	6632	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1608.1;
	6633	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.6;
	6633	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	778.1;
	6633	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	858;
	6633	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3948.9;
	6633	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10390.3;
	6633	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5834.2;
	6633	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10672;
	6633	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1719.4;
	6634	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.6;
	6634	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	822;
	6634	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	898.8;
	6634	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4339.3;
	6634	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11098.5;
	6634	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6254.8;
	6634	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11442.9;
	6634	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1816.3;
	6635	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1005.1;
	6635	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	852.4;
	6635	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	939.7;
	6635	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4757.1;
	6635	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11787.2;
	6635	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6664.9;
	6635	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12150.1;
	6635	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1913.8;
	6636	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1037.3;
	6636	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	889.2;
	6636	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	989.1;
	6636	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5064.9;
	6636	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12432;
	6636	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7027;
	6636	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12824.9;
	6636	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2016;
	6637	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1069.4;
	6637	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	935.5;
	6637	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1043.8;
	6637	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5303.8;
	6637	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13149.7;
	6637	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7456.9;
	6637	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13516.3;
	6637	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2120.7;
	6638	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1093.8;
	6638	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	971;
	6638	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1092.4;
	6638	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5486.9;
	6638	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13766.9;
	6638	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7831.1;
	6638	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14089.7;
	6638	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2196.2;
	6639	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1109.7;
	6639	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	997.8;
	6639	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1136;
	6639	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5647.5;
	6639	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14233;
	6639	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8169.5;
	6639	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14494.3;
	6639	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2233.5;
	6640	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1118.2;
	6640	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1012.5;
	6640	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1162.4;
	6640	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5710.3;
	6640	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14533.8;
	6640	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8404.4;
	6640	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14605.4;
	6640	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2276.8;
	6641	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1109.8;
	6641	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	995.5;
	6641	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1151.9;
	6641	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5618.1;
	6641	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14472.3;
	6641	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8488.5;
	6641	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14257.4;
	6641	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2247.1;
	6642	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1091.8;
	6642	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	966.9;
	6642	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1118.7;
	6642	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5357.3;
	6642	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13981.8;
	6642	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8243.7;
	6642	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13539;
	6642	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2171;
	6643	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1070.6;
	6643	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	953.9;
	6643	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1091.3;
	6643	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5234.7;
	6643	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13701.8;
	6643	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7982.8;
	6643	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13302;
	6643	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2125.8;
	6644	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1055.7;
	6644	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	930.9;
	6644	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1068.2;
	6644	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5156.8;
	6644	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13341;
	6644	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7720;
	6644	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12918.8;
	6644	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2057.9;
	6645	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1027.4;
	6645	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	882.1;
	6645	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1008;
	6645	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4870.3;
	6645	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12491.6;
	6645	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7206;
	6645	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12173.9;
	6645	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1916.8;
	6646	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981;
	6646	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	816;
	6646	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	930.2;
	6646	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4462.4;
	6646	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11292.1;
	6646	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6541.3;
	6646	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11168.6;
	6646	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1747.5;
	6647	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.3;
	6647	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	754.2;
	6647	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.8;
	6647	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4044;
	6647	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10072.1;
	6647	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5856.6;
	6647	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10213.1;
	6647	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1579.8;
	6648	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.8;
	6648	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	713.3;
	6648	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	801.4;
	6648	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3731.8;
	6648	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9241.4;
	6648	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5346.3;
	6648	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9503.5;
	6648	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1453.6;
	6649	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.2;
	6649	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	685.9;
	6649	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	769.9;
	6649	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3531.1;
	6649	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8740.4;
	6649	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5018.6;
	6649	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9057.3;
	6649	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1359.4;
	6650	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.8;
	6650	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	671.3;
	6650	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	756.2;
	6650	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3404.1;
	6650	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8404.9;
	6650	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4813.3;
	6650	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8810.5;
	6650	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1245.9;
	6651	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.8;
	6651	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	657.5;
	6651	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	747.4;
	6651	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3308.6;
	6651	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8249.3;
	6651	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4728.3;
	6651	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8674.2;
	6651	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1225;
	6652	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.7;
	6652	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	661.2;
	6652	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	745.3;
	6652	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3285.4;
	6652	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8348.6;
	6652	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4800.4;
	6652	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8790.8;
	6652	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1285.1;
	6653	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.3;
	6653	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	686.4;
	6653	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	771.5;
	6653	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3383.3;
	6653	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8887.7;
	6653	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5127.4;
	6653	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9321;
	6653	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1363.8;
	6654	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.4;
	6654	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.9;
	6654	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.8;
	6654	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3728.4;
	6654	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10037.2;
	6654	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5763.8;
	6654	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10033.3;
	6654	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1464.9;
	6655	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.2;
	6655	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	764.9;
	6655	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.3;
	6655	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3796.7;
	6655	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10480.3;
	6655	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6060.6;
	6655	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10191.2;
	6655	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1519.9;
	6656	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918;
	6656	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777.9;
	6656	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.7;
	6656	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3837.3;
	6656	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10622;
	6656	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6076.6;
	6656	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10438.6;
	6656	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1636.3;
	6657	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.6;
	6657	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	820;
	6657	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	914.5;
	6657	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4157.3;
	6657	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11194.6;
	6657	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6312.4;
	6657	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11243.1;
	6657	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1674.4;
	6658	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	974.2;
	6658	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	867.6;
	6658	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	961.4;
	6658	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4515.4;
	6658	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11914.1;
	6658	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6685.1;
	6658	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12174.7;
	6658	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1771.8;
	6659	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	998.4;
	6659	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	901.6;
	6659	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1010.7;
	6659	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4884.2;
	6659	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12713.7;
	6659	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7151.4;
	6659	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12990.1;
	6659	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1935.3;
	6660	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1018.3;
	6660	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	945.7;
	6660	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1067.4;
	6660	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5249.3;
	6660	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13519.4;
	6660	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7652.7;
	6660	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13701.5;
	6660	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2059.6;
	6661	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1035.3;
	6661	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	990.3;
	6661	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1125.4;
	6661	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5567.9;
	6661	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14326.8;
	6661	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8164.3;
	6661	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14401.5;
	6661	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2182.3;
	6662	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1056;
	6662	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1020.5;
	6662	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1179.3;
	6662	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5801.5;
	6662	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14889;
	6662	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8606.2;
	6662	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14942.3;
	6662	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2260.1;
	6663	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1073.7;
	6663	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1045.3;
	6663	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1216.7;
	6663	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5943.7;
	6663	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15368.1;
	6663	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8972.2;
	6663	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15281.8;
	6663	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2360.6;
	6664	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1086.1;
	6664	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1067.8;
	6664	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1239.1;
	6664	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5996.7;
	6664	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15696.4;
	6664	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9233.2;
	6664	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15383.4;
	6664	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2390.4;
	6665	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1081.2;
	6665	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1061.5;
	6665	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1225.5;
	6665	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5854.6;
	6665	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15593.3;
	6665	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9268;
	6665	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15013.9;
	6665	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2349.1;
	6666	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1059.2;
	6666	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1028.2;
	6666	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1186;
	6666	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5580.9;
	6666	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15103.3;
	6666	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8970.7;
	6666	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14253.7;
	6666	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2290.4;
	6667	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1040.3;
	6667	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1008.4;
	6667	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1146.8;
	6667	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5449.1;
	6667	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14896;
	6667	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8738.6;
	6667	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13979.3;
	6667	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2255.5;
	6668	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1032.2;
	6668	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	991.7;
	6668	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1128.2;
	6668	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5339;
	6668	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14593;
	6668	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8494.2;
	6668	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13601;
	6668	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2190.4;
	6669	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1003.5;
	6669	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	951.1;
	6669	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1065.6;
	6669	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5056.7;
	6669	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13805.9;
	6669	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7994.1;
	6669	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13031.3;
	6669	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2064.9;
	6670	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962;
	6670	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	886.2;
	6670	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	988.3;
	6670	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4663.5;
	6670	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12705.2;
	6670	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7310.1;
	6670	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12144.1;
	6670	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1911.3;
	6671	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.5;
	6671	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	829.2;
	6671	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	908.3;
	6671	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4246.8;
	6671	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11590.9;
	6671	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6500.1;
	6671	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11243.2;
	6671	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1686.6;
	6672	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892;
	6672	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	782.8;
	6672	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.2;
	6672	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3913;
	6672	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10702.2;
	6672	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5935;
	6672	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10519.2;
	6672	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1575.5;
	6673	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.7;
	6673	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.1;
	6673	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	812;
	6673	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3684.7;
	6673	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10139.7;
	6673	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5608.5;
	6673	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10030.6;
	6673	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1507.5;
	6674	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	869.3;
	6674	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723.5;
	6674	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	788.2;
	6674	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3527.9;
	6674	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9736.2;
	6674	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5365.1;
	6674	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9762.3;
	6674	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1422;
	6675	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867.4;
	6675	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	713.2;
	6675	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	772.2;
	6675	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3431;
	6675	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9471.1;
	6675	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5247.3;
	6675	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9628.6;
	6675	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1428.6;
	6676	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	865.6;
	6676	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.8;
	6676	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	770.8;
	6676	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3388;
	6676	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9538.2;
	6676	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5325.7;
	6676	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9699.8;
	6676	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1415.3;
	6677	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.9;
	6677	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.8;
	6677	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	796.4;
	6677	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3508.9;
	6677	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10081.4;
	6677	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5686.1;
	6677	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10241.2;
	6677	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1508.5;
	6678	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.7;
	6678	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	801;
	6678	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.6;
	6678	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3857;
	6678	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11195.7;
	6678	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6320.9;
	6678	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10969.5;
	6678	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1697.3;
	6679	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.7;
	6679	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	824.6;
	6679	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	893.6;
	6679	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3917.8;
	6679	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11608.4;
	6679	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6575.8;
	6679	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11121.8;
	6679	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1761;
	6680	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.3;
	6680	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	838;
	6680	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897.4;
	6680	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3964.3;
	6680	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11655.6;
	6680	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6564.3;
	6680	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11351.6;
	6680	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1781.5;
	6681	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.4;
	6681	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	871.2;
	6681	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	941.7;
	6681	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4324.5;
	6681	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12188.8;
	6681	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6847.1;
	6681	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12185.6;
	6681	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1869.6;
	6682	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	979.8;
	6682	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	930.6;
	6682	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1000.2;
	6682	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4688.5;
	6682	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13037.8;
	6682	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7237.3;
	6682	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13149.3;
	6682	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1972.3;
	6683	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009.3;
	6683	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	981.8;
	6683	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1059.5;
	6683	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5065.1;
	6683	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14058;
	6683	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7735.9;
	6683	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13954;
	6683	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2084.5;
	6684	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1033.6;
	6684	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1026.4;
	6684	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1121.2;
	6684	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5431.5;
	6684	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15074.7;
	6684	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8278.4;
	6684	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14604.3;
	6684	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2201.6;
	6685	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1063.5;
	6685	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1079.1;
	6685	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1190.9;
	6685	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5757.8;
	6685	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16092.6;
	6685	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8805.3;
	6685	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15241;
	6685	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2325.3;
	6686	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1096.5;
	6686	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1129.6;
	6686	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1247.8;
	6686	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5984.2;
	6686	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16909.9;
	6686	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9318.6;
	6686	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15744.3;
	6686	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2429;
	6687	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1116.4;
	6687	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1166.6;
	6687	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1294.5;
	6687	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6117.1;
	6687	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17562.9;
	6687	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9695.9;
	6687	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15913.5;
	6687	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2514.1;
	6688	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1131.8;
	6688	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1183.4;
	6688	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1320;
	6688	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6176.6;
	6688	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17803.7;
	6688	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9953.2;
	6688	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15724.4;
	6688	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2555.4;
	6689	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1122.2;
	6689	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1163.3;
	6689	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1307.2;
	6689	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6037;
	6689	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17504.3;
	6689	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9929.6;
	6689	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15185.6;
	6689	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2524.7;
	6690	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1101.5;
	6690	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1122.4;
	6690	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1261.5;
	6690	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5761.2;
	6690	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16716.7;
	6690	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9568;
	6690	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14479.8;
	6690	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2430;
	6691	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1069.8;
	6691	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1094;
	6691	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1218.9;
	6691	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5623.4;
	6691	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16152.5;
	6691	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9251.1;
	6691	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14287.2;
	6691	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2370.5;
	6692	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1055.7;
	6692	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1058.7;
	6692	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1188.8;
	6692	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5510.6;
	6692	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15540;
	6692	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8892.4;
	6692	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14004.2;
	6692	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2293.8;
	6693	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1024.1;
	6693	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1005.8;
	6693	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1118.9;
	6693	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5214;
	6693	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14497.4;
	6693	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8283.7;
	6693	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13331.4;
	6693	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2147.6;
	6694	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986.6;
	6694	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	949;
	6694	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1028.3;
	6694	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4818.7;
	6694	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13247.7;
	6694	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7571.2;
	6694	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12412.1;
	6694	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1965.2;
	6695	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.1;
	6695	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	879;
	6695	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	944.4;
	6695	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4399.8;
	6695	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12006.4;
	6695	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6821.2;
	6695	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11471.7;
	6695	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1785.6;
	6696	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.1;
	6696	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	844.3;
	6696	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.8;
	6696	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4117.9;
	6696	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11083.9;
	6696	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6121.2;
	6696	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10826.6;
	6696	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1652.7;
	6697	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920.8;
	6697	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	814.1;
	6697	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	863.2;
	6697	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3915.1;
	6697	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10474.6;
	6697	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5736.2;
	6697	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10401.4;
	6697	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1550.7;
	6698	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.5;
	6698	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	790.5;
	6698	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838.7;
	6698	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3737;
	6698	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10067.4;
	6698	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5491.1;
	6698	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10123.2;
	6698	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1482.8;
	6699	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.7;
	6699	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	783;
	6699	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	823.9;
	6699	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3627.1;
	6699	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9827;
	6699	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5383.9;
	6699	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9986.3;
	6699	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1449.2;
	6700	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.1;
	6700	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776.9;
	6700	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	820.4;
	6700	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3580.8;
	6700	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9843.2;
	6700	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5445.5;
	6700	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10087.4;
	6700	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1462.4;
	6701	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.4;
	6701	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	788.3;
	6701	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	841.5;
	6701	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3696.9;
	6701	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10396.2;
	6701	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5758.7;
	6701	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10632.2;
	6701	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1541.9;
	6702	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.9;
	6702	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	851.4;
	6702	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	915.1;
	6702	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4080.7;
	6702	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11563.8;
	6702	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6366.6;
	6702	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11406.6;
	6702	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1728.4;
	6703	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981.3;
	6703	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	879.3;
	6703	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	948.6;
	6703	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4140.2;
	6703	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11991.9;
	6703	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6612.2;
	6703	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11524.6;
	6703	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1788.6;
	6704	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	976.5;
	6704	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	872.7;
	6704	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940.4;
	6704	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4133.9;
	6704	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12035.7;
	6704	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6633.7;
	6704	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11732.7;
	6704	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1870.6;
	6705	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	998.1;
	6705	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	905.1;
	6705	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	973.5;
	6705	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4455.3;
	6705	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12450;
	6705	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6905.2;
	6705	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12431.5;
	6705	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1976.7;
	6706	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1027.9;
	6706	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	951.7;
	6706	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1021.8;
	6706	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4828.8;
	6706	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13211.3;
	6706	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7334.7;
	6706	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13162.1;
	6706	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2113.7;
	6707	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1057.6;
	6707	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	997.6;
	6707	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1077.8;
	6707	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5184.7;
	6707	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13975.8;
	6707	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7815.6;
	6707	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13858.2;
	6707	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2188.9;
	6708	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1080.1;
	6708	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1033.6;
	6708	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1124.5;
	6708	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5474.1;
	6708	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14760.1;
	6708	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8317.3;
	6708	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14412.8;
	6708	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2377.2;
	6709	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1109.2;
	6709	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1082;
	6709	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1180;
	6709	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5733.1;
	6709	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15587;
	6709	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8793.4;
	6709	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14901;
	6709	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2454.6;
	6710	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1131.8;
	6710	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1121.2;
	6710	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1230.2;
	6710	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5899.6;
	6710	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16191.1;
	6710	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9196.2;
	6710	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14985;
	6710	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2539.6;
	6711	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1153.1;
	6711	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1147.5;
	6711	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1267.6;
	6711	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5984.1;
	6711	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16687.9;
	6711	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9531.5;
	6711	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14924.3;
	6711	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2579;
	6712	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1165.3;
	6712	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1149.9;
	6712	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1287.4;
	6712	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	6039.1;
	6712	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16917.2;
	6712	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9739.8;
	6712	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14634.1;
	6712	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2559.6;
	6713	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1151.6;
	6713	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1118.4;
	6713	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1255;
	6713	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5896.5;
	6713	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16759.9;
	6713	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9725.6;
	6713	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14259.2;
	6713	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2452.7;
	6714	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1125.3;
	6714	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1070.5;
	6714	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1210.1;
	6714	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5613.6;
	6714	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16074.8;
	6714	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9405.7;
	6714	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13763;
	6714	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2356.9;
	6715	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1109.7;
	6715	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1057.9;
	6715	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1178;
	6715	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5535.8;
	6715	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15755.8;
	6715	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9092.4;
	6715	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13721.9;
	6715	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2328.6;
	6716	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1095.5;
	6716	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1015.7;
	6716	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1156.2;
	6716	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5450.6;
	6716	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15277.2;
	6716	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8799.1;
	6716	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13416.7;
	6716	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2271;
	6717	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1064.7;
	6717	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	939.4;
	6717	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1085.7;
	6717	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5189.7;
	6717	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14390;
	6717	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8211.6;
	6717	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12782.7;
	6717	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2139.4;
	6718	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1024.5;
	6718	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	882.3;
	6718	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	998.6;
	6718	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4829.8;
	6718	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13166.4;
	6718	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7446.3;
	6718	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11887.9;
	6718	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1967.5;
	6719	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	979.4;
	6719	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	814.2;
	6719	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	920.3;
	6719	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4420.5;
	6719	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11884.7;
	6719	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6675.9;
	6719	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10969;
	6719	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1787.9;
	6720	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948.5;
	6720	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.9;
	6720	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	862.9;
	6720	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4120.6;
	6720	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10775.5;
	6720	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6177.2;
	6720	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10281.4;
	6720	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1639.2;
	6721	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924;
	6721	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	715;
	6721	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	822.8;
	6721	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3883;
	6721	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10066.4;
	6721	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5819.2;
	6721	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9754.4;
	6721	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1559.1;
	6722	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.6;
	6722	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	693.8;
	6722	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	801.7;
	6722	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3727.8;
	6722	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9651.3;
	6722	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5588.4;
	6722	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9419.1;
	6722	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1401.3;
	6723	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.7;
	6723	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	672.3;
	6723	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	781.7;
	6723	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3625.9;
	6723	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9386.3;
	6723	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5451.8;
	6723	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9260.7;
	6723	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1374.8;
	6724	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.2;
	6724	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	672.2;
	6724	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	770.9;
	6724	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3591.5;
	6724	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9350.3;
	6724	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5494.5;
	6724	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9320.4;
	6724	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1363.2;
	6725	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.8;
	6725	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	696.7;
	6725	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	788.3;
	6725	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3669.1;
	6725	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9678.7;
	6725	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5812.3;
	6725	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9805.7;
	6725	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1485;
	6726	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.2;
	6726	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.3;
	6726	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	846.3;
	6726	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3990.2;
	6726	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10506.6;
	6726	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6401.5;
	6726	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10512.3;
	6726	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1649.8;
	6727	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.3;
	6727	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	779;
	6727	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.3;
	6727	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4036.5;
	6727	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10891.6;
	6727	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6642.6;
	6727	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10676.9;
	6727	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1673.2;
	6728	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.9;
	6728	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	781.9;
	6728	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	862.4;
	6728	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4096.1;
	6728	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10900.4;
	6728	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6655.1;
	6728	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10978.7;
	6728	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1732;
	6729	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927;
	6729	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	787;
	6729	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	883.4;
	6729	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4439.1;
	6729	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11042;
	6729	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6938.2;
	6729	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11752.5;
	6729	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1799.4;
	6730	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925.5;
	6730	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	796.1;
	6730	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	899.2;
	6730	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4774.6;
	6730	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11077.5;
	6730	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7279.8;
	6730	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12580.4;
	6730	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1910.5;
	6731	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.7;
	6731	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	798.4;
	6731	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	902.2;
	6731	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5052.3;
	6731	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10998.4;
	6731	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7578.4;
	6731	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13273.7;
	6731	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1935.1;
	6732	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.6;
	6732	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	784.8;
	6732	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	905.7;
	6732	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5272.3;
	6732	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10904.7;
	6732	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7805.1;
	6732	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13818;
	6732	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2031.8;
	6733	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.6;
	6733	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	785.8;
	6733	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	917.6;
	6733	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5469.7;
	6733	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10874.5;
	6733	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8023.9;
	6733	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14380.6;
	6733	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2153.5;
	6734	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.7;
	6734	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	782.3;
	6734	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913.7;
	6734	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5550.5;
	6734	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10767.3;
	6734	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8098.4;
	6734	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14772.1;
	6734	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2291;
	6735	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.8;
	6735	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	778;
	6735	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	899.1;
	6735	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5444.3;
	6735	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10696.5;
	6735	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8149.7;
	6735	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15027.3;
	6735	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2311.6;
	6736	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.5;
	6736	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	774.9;
	6736	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	891;
	6736	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5271;
	6736	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10630.3;
	6736	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8221.1;
	6736	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15130.7;
	6736	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2337.8;
	6737	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921;
	6737	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	765.1;
	6737	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	874.8;
	6737	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5000.9;
	6737	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10561.4;
	6737	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7942.4;
	6737	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14786.3;
	6737	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2277.5;
	6738	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.1;
	6738	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	748;
	6738	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.3;
	6738	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4718.3;
	6738	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10320.4;
	6738	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7403;
	6738	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13900.8;
	6738	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2145.2;
	6739	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.6;
	6739	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	759.6;
	6739	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.4;
	6739	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4688.2;
	6739	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10399.4;
	6739	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7019.3;
	6739	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13522.6;
	6739	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2052.4;
	6740	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.1;
	6740	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.8;
	6740	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	858.1;
	6740	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4613.8;
	6740	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10305.7;
	6740	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6682;
	6740	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13052.3;
	6740	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1941.7;
	6741	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.1;
	6741	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	732.4;
	6741	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	830.2;
	6741	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4460.8;
	6741	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9889.7;
	6741	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6245.1;
	6741	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12468.4;
	6741	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1855;
	6742	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.6;
	6742	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	703;
	6742	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	795.8;
	6742	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4228.9;
	6742	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9322.8;
	6742	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5790.2;
	6742	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11792;
	6742	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1721;
	6743	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.2;
	6743	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	668.8;
	6743	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	752.5;
	6743	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3965.7;
	6743	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8650.5;
	6743	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5307.3;
	6743	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11029.9;
	6743	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1568.2;
	6744	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.7;
	6744	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	634.5;
	6744	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	738.3;
	6744	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3675.3;
	6744	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8026.9;
	6744	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4872.2;
	6744	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10166.6;
	6744	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1496.8;
	6745	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	849.7;
	6745	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	613.2;
	6745	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	716.7;
	6745	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3462.9;
	6745	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7582.5;
	6745	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4575.7;
	6745	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9555.8;
	6745	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1415.3;
	6746	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	838.4;
	6746	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	599.7;
	6746	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	699.7;
	6746	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3283.3;
	6746	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7270.4;
	6746	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4368;
	6746	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9065.6;
	6746	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1342.6;
	6747	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	819.2;
	6747	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592;
	6747	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	692.8;
	6747	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3171.2;
	6747	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7115;
	6747	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4236;
	6747	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8791.7;
	6747	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1297.8;
	6748	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	816;
	6748	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	591.5;
	6748	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	685.2;
	6748	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3111.9;
	6748	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7078.9;
	6748	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4191.4;
	6748	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8610.5;
	6748	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1276;
	6749	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	828.5;
	6749	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	604.7;
	6749	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	692.6;
	6749	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3113.7;
	6749	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7213.7;
	6749	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4245.3;
	6749	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8603.5;
	6749	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1290.7;
	6750	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	837.9;
	6750	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	627;
	6750	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	716.5;
	6750	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3150.5;
	6750	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7586.9;
	6750	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4448.6;
	6750	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8737.2;
	6750	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1323.7;
	6751	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	848.2;
	6751	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	645.4;
	6751	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	745.6;
	6751	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3157.4;
	6751	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7898.6;
	6751	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4643.1;
	6751	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8793.9;
	6751	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1359.4;
	6752	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	853.8;
	6752	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	667;
	6752	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	769.4;
	6752	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3259.8;
	6752	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8333;
	6752	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4879.9;
	6752	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9145;
	6752	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1426.8;
	6753	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.8;
	6753	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	693;
	6753	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	804.2;
	6753	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3501.5;
	6753	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8890.6;
	6753	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5228.1;
	6753	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9813.9;
	6753	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1494.7;
	6754	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	881.5;
	6754	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	708.8;
	6754	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	825.1;
	6754	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3722.6;
	6754	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9267.1;
	6754	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5525.8;
	6754	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10499.4;
	6754	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1581.3;
	6755	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.6;
	6755	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	716.3;
	6755	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	841.7;
	6755	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3895.8;
	6755	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9621.2;
	6755	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5861.7;
	6755	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11164.1;
	6755	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1680.3;
	6756	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.5;
	6756	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719.5;
	6756	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	870;
	6756	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4121.3;
	6756	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9884.3;
	6756	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6205.5;
	6756	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11871.5;
	6756	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1742.4;
	6757	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.8;
	6757	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.5;
	6757	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	898.8;
	6757	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4420.3;
	6757	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10137;
	6757	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6561.6;
	6757	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12538;
	6757	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1834.2;
	6758	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.7;
	6758	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744.6;
	6758	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	932.1;
	6758	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4660;
	6758	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10457.2;
	6758	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6863.6;
	6758	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13066.1;
	6758	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1932;
	6759	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.5;
	6759	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755.9;
	6759	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	957.7;
	6759	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4782.6;
	6759	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10826;
	6759	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7181.1;
	6759	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13337.6;
	6759	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1973.4;
	6760	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.5;
	6760	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	767.1;
	6760	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	977.2;
	6760	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4827;
	6760	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11050.7;
	6760	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7409.2;
	6760	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13410.3;
	6760	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1991.9;
	6761	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.4;
	6761	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	768.8;
	6761	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	971.8;
	6761	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4725.2;
	6761	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11039.7;
	6761	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7437.4;
	6761	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13106.3;
	6761	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1965;
	6762	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.8;
	6762	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753.4;
	6762	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	961;
	6762	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4511.8;
	6762	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10697.7;
	6762	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7205.9;
	6762	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12459;
	6762	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1854.3;
	6763	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.4;
	6763	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	754.2;
	6763	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	961;
	6763	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4457;
	6763	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10613.4;
	6763	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7034.3;
	6763	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12165.8;
	6763	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1827.2;
	6764	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905;
	6764	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.4;
	6764	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	944.6;
	6764	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4354.9;
	6764	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10313.5;
	6764	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6820.8;
	6764	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11779.1;
	6764	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1776.2;
	6765	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.9;
	6765	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714.7;
	6765	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	900.4;
	6765	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4153.2;
	6765	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9797;
	6765	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6454.4;
	6765	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11295.5;
	6765	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1615.5;
	6766	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.3;
	6766	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	682.5;
	6766	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	839.6;
	6766	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3922.2;
	6766	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9158;
	6766	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6013.1;
	6766	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10703.3;
	6766	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1545.6;
	6767	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	857;
	6767	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	647.2;
	6767	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	782.9;
	6767	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3666.2;
	6767	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8427;
	6767	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5527;
	6767	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10031.6;
	6767	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1452.5;
	6768	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	843.8;
	6768	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	621.8;
	6768	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	746.9;
	6768	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3407.6;
	6768	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7796.8;
	6768	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5056.5;
	6768	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9395.2;
	6768	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1354.8;
	6769	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	830.9;
	6769	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	599.9;
	6769	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	715.9;
	6769	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3192;
	6769	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7313;
	6769	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4691.9;
	6769	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8844.1;
	6769	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1267.3;
	6770	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	821;
	6770	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	587.2;
	6770	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	695.9;
	6770	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3039.6;
	6770	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7003.2;
	6770	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4443.7;
	6770	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8416.4;
	6770	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1201.5;
	6771	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	819.1;
	6771	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	577.9;
	6771	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	687.7;
	6771	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2931.4;
	6771	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6818.6;
	6771	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4278.6;
	6771	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8107.7;
	6771	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1191;
	6772	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	817.1;
	6772	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	573.8;
	6772	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	678.1;
	6772	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2873.3;
	6772	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6766;
	6772	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4182.5;
	6772	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7884;
	6772	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1148.3;
	6773	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	814.4;
	6773	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	580.8;
	6773	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	671.8;
	6773	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2855.6;
	6773	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6821.5;
	6773	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4161.1;
	6773	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7786.7;
	6773	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1167.9;
	6774	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	817.6;
	6774	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	603.9;
	6774	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	684.2;
	6774	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2882.3;
	6774	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7020.2;
	6774	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4255.1;
	6774	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7845.5;
	6774	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1200.6;
	6775	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	826.2;
	6775	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	613.8;
	6775	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	705.7;
	6775	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2874.5;
	6775	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7242.5;
	6775	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4321.3;
	6775	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7910;
	6775	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1217.7;
	6776	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	835.2;
	6776	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	636.2;
	6776	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	733.1;
	6776	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3015.6;
	6776	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7686.4;
	6776	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4591;
	6776	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8282.5;
	6776	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1256.1;
	6777	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	850.1;
	6777	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	661.3;
	6777	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	771.1;
	6777	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3331.7;
	6777	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8239.6;
	6777	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4953.3;
	6777	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8749.5;
	6777	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1335.3;
	6778	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	859.9;
	6778	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	668.8;
	6778	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	794.3;
	6778	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3645;
	6778	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8622;
	6778	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5247.3;
	6778	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9207.9;
	6778	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1398.1;
	6779	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875;
	6779	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	681.1;
	6779	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	815.5;
	6779	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3927.8;
	6779	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8932.4;
	6779	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5514.8;
	6779	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9649.9;
	6779	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1448.8;
	6780	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.3;
	6780	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	699.7;
	6780	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.9;
	6780	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4142.4;
	6780	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9241.6;
	6780	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5795.7;
	6780	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10153.5;
	6780	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1514.7;
	6781	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.3;
	6781	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.9;
	6781	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	877.4;
	6781	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4324.7;
	6781	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9538.7;
	6781	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6076.7;
	6781	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10663.8;
	6781	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1612.2;
	6782	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.6;
	6782	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.4;
	6782	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	909.2;
	6782	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4477;
	6782	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9776.2;
	6782	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6357.1;
	6782	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11147.7;
	6782	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1669.8;
	6783	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.6;
	6783	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.5;
	6783	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	926.4;
	6783	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4615.1;
	6783	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9961.2;
	6783	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6687.9;
	6783	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11521.3;
	6783	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1736;
	6784	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.7;
	6784	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	754.1;
	6784	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	947.9;
	6784	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4635.7;
	6784	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10045.8;
	6784	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6884.3;
	6784	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11698.5;
	6784	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1767.5;
	6785	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.7;
	6785	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	748.8;
	6785	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	954.2;
	6785	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4510.3;
	6785	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9966.5;
	6785	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6914;
	6785	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11379.1;
	6785	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1766.6;
	6786	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.2;
	6786	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.8;
	6786	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	937.6;
	6786	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4321.3;
	6786	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9938.1;
	6786	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6700.1;
	6786	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10981.5;
	6786	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1693.1;
	6787	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.3;
	6787	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	766.5;
	6787	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	931.1;
	6787	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4287.4;
	6787	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10265;
	6787	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6579;
	6787	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11073.6;
	6787	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1746;
	6788	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.5;
	6788	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751;
	6788	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	912.5;
	6788	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4153.5;
	6788	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10030.1;
	6788	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6286.4;
	6788	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10711.9;
	6788	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1694;
	6789	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.4;
	6789	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	721.3;
	6789	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	873.1;
	6789	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3937.6;
	6789	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9611.4;
	6789	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5869.5;
	6789	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10208.5;
	6789	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1584.6;
	6790	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.7;
	6790	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	680.4;
	6790	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	825.5;
	6790	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3692.1;
	6790	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9044.4;
	6790	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5421.1;
	6790	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9559.8;
	6790	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1482.7;
	6791	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.1;
	6791	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	635.7;
	6791	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	762.7;
	6791	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3365;
	6791	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8200.6;
	6791	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4822.5;
	6791	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8767.4;
	6791	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1388.2;
	6792	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	846.3;
	6792	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	608.4;
	6792	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	719.4;
	6792	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3082.3;
	6792	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7602.1;
	6792	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4394.1;
	6792	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8197.3;
	6792	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1300.8;
	6793	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	831.8;
	6793	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588.1;
	6793	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	689;
	6793	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2892.8;
	6793	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7191.7;
	6793	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4118.8;
	6793	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7882.5;
	6793	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1251.9;
	6794	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	826;
	6794	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	578.2;
	6794	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	671.4;
	6794	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2753.6;
	6794	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6990.7;
	6794	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3968.2;
	6794	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7722.1;
	6794	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1201.1;
	6795	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	822.2;
	6795	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	576.5;
	6795	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	666.7;
	6795	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2679.3;
	6795	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6936.8;
	6795	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3930.3;
	6795	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7707.4;
	6795	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1209.4;
	6796	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	822.6;
	6796	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	582.4;
	6796	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	668.6;
	6796	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2645.6;
	6796	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7072.6;
	6796	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3980.1;
	6796	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7852.9;
	6796	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1240.2;
	6797	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	830.6;
	6797	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	608.8;
	6797	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	693.7;
	6797	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2733.3;
	6797	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7556;
	6797	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4263.1;
	6797	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8280.9;
	6797	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1305.3;
	6798	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	852.2;
	6798	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	666.3;
	6798	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	749.8;
	6798	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2962.4;
	6798	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8390.3;
	6798	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4710.8;
	6798	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8830.9;
	6798	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1434.6;
	6799	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.6;
	6799	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	690.9;
	6799	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	787.2;
	6799	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3047;
	6799	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8887.6;
	6799	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4974.1;
	6799	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9007.8;
	6799	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1441.2;
	6800	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.7;
	6800	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	700.4;
	6800	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	788;
	6800	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3101.3;
	6800	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9067;
	6800	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5050.8;
	6800	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9178.5;
	6800	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1528;
	6801	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883;
	6801	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	718.1;
	6801	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	810.9;
	6801	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3340.5;
	6801	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9481.2;
	6801	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5276.3;
	6801	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9615.2;
	6801	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1592.9;
	6802	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.3;
	6802	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.4;
	6802	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	834.3;
	6802	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3643.9;
	6802	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9933.7;
	6802	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5525.8;
	6802	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10077.8;
	6802	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1649.5;
	6803	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926;
	6803	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753.4;
	6803	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	853.6;
	6803	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3936.8;
	6803	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10308.9;
	6803	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5753.6;
	6803	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10557.7;
	6803	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1704.2;
	6804	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951.4;
	6804	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	769.6;
	6804	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	880.1;
	6804	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4257;
	6804	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10669.4;
	6804	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5980.5;
	6804	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11091;
	6804	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1776.2;
	6805	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981.4;
	6805	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	795.8;
	6805	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	919.1;
	6805	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4530.3;
	6805	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11077;
	6805	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6248.5;
	6805	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11707.9;
	6805	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1864.6;
	6806	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1008;
	6806	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	812.9;
	6806	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	951.3;
	6806	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4750.8;
	6806	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11429.4;
	6806	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6534.6;
	6806	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12335.8;
	6806	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1960.5;
	6807	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1032.6;
	6807	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	833.9;
	6807	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	994.8;
	6807	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4907.4;
	6807	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11779.5;
	6807	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6857.6;
	6807	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12830.9;
	6807	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2007.5;
	6808	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1046.7;
	6808	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	847.6;
	6808	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1023.1;
	6808	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4971.8;
	6808	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12021.1;
	6808	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7119.1;
	6808	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13121.7;
	6808	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2073.2;
	6809	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1043.4;
	6809	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	842.5;
	6809	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1022.2;
	6809	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4873.6;
	6809	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12064.8;
	6809	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7260.4;
	6809	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13005.5;
	6809	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2046.1;
	6810	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030.8;
	6810	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	830.5;
	6810	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1003.1;
	6810	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4628.9;
	6810	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11845.5;
	6810	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7075.7;
	6810	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12437.1;
	6810	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1995.6;
	6811	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1019;
	6811	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	836.4;
	6811	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	991.1;
	6811	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4526.4;
	6811	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11814.6;
	6811	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6892.8;
	6811	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12275.6;
	6811	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1976.1;
	6812	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1004.7;
	6812	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	820.1;
	6812	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	974.2;
	6812	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4421.1;
	6812	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11534.4;
	6812	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6643;
	6812	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11918.4;
	6812	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1914;
	6813	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981;
	6813	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	780.8;
	6813	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	923.2;
	6813	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4142.6;
	6813	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10834.3;
	6813	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6146.7;
	6813	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11278.4;
	6813	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1795.1;
	6814	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.1;
	6814	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	721.9;
	6814	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	853.8;
	6814	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3759.6;
	6814	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9837.7;
	6814	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5500.3;
	6814	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10383.8;
	6814	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1648.9;
	6815	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.8;
	6815	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	673.4;
	6815	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	784.4;
	6815	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3376.3;
	6815	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8863;
	6815	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4867.1;
	6815	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9566.9;
	6815	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1504.8;
	6816	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.2;
	6816	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	647.9;
	6816	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	738.7;
	6816	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3094.4;
	6816	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8085.7;
	6816	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4430.8;
	6816	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8989.7;
	6816	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1408.8;
	6817	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.6;
	6817	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	626.6;
	6817	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	709.6;
	6817	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2914;
	6817	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7677.1;
	6817	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4162;
	6817	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8629.4;
	6817	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1327.3;
	6818	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	858.9;
	6818	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	616.3;
	6818	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	697;
	6818	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2818.4;
	6818	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7410.2;
	6818	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4008.3;
	6818	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8433.8;
	6818	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1215.7;
	6819	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	850.5;
	6819	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	608;
	6819	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	687;
	6819	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2741.7;
	6819	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7318.4;
	6819	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3939.2;
	6819	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8387.7;
	6819	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1204.8;
	6820	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	846;
	6820	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	610.6;
	6820	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	686.6;
	6820	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2719.1;
	6820	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7444;
	6820	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4004.2;
	6820	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8577.6;
	6820	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1262.1;
	6821	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	856.1;
	6821	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	639.7;
	6821	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	715.9;
	6821	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2836.7;
	6821	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8005.8;
	6821	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4313.5;
	6821	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9216.3;
	6821	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1338.4;
	6822	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885;
	6822	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	700.7;
	6822	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	787.3;
	6822	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3188.3;
	6822	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9130.6;
	6822	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4932.4;
	6822	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10002.9;
	6822	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1448.4;
	6823	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.3;
	6823	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.4;
	6823	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	820.7;
	6823	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3248.2;
	6823	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9551.3;
	6823	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5146.7;
	6823	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10210.5;
	6823	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1510.2;
	6824	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.3;
	6824	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	732.2;
	6824	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	810.5;
	6824	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3220.4;
	6824	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9557.2;
	6824	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5145.9;
	6824	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10268;
	6824	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1608.9;
	6825	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.8;
	6825	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749.1;
	6825	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824.9;
	6825	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3421.2;
	6825	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9922.7;
	6825	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5339.9;
	6825	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10671.2;
	6825	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1614.4;
	6826	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.7;
	6826	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776.8;
	6826	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	853.2;
	6826	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3693.7;
	6826	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10402.9;
	6826	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5625.3;
	6826	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11279.5;
	6826	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1668.7;
	6827	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.8;
	6827	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	808.1;
	6827	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.3;
	6827	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4004.1;
	6827	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10896.5;
	6827	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5902.5;
	6827	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11992.3;
	6827	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1797.4;
	6828	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985.6;
	6828	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	838.4;
	6828	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	923.5;
	6828	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4343.1;
	6828	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11405.5;
	6828	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6211.4;
	6828	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12659.9;
	6828	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1880.5;
	6829	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1021.8;
	6829	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	875.4;
	6829	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	973.8;
	6829	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4677.1;
	6829	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12063.4;
	6829	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6582.1;
	6829	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13153.3;
	6829	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1978.1;
	6830	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1055.5;
	6830	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	908.8;
	6830	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1024.1;
	6830	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4946;
	6830	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12658.4;
	6830	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6985.7;
	6830	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13139.2;
	6830	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2013.5;
	6831	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1082.2;
	6831	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	931.1;
	6831	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1075.3;
	6831	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5129.8;
	6831	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13262.5;
	6831	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7419.9;
	6831	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13039.8;
	6831	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2086.4;
	6832	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1100.6;
	6832	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	951.4;
	6832	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1111.2;
	6832	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5239.3;
	6832	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13758.7;
	6832	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7746.8;
	6832	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13068;
	6832	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2116.9;
	6833	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1093.2;
	6833	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	947.2;
	6833	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1112;
	6833	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5140.2;
	6833	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13840.1;
	6833	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7851.3;
	6833	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12935.3;
	6833	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2090.5;
	6834	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1071.6;
	6834	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	922.7;
	6834	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1082.7;
	6834	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4896.7;
	6834	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13453.8;
	6834	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7659.4;
	6834	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12573.7;
	6834	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2066.7;
	6835	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1055.7;
	6835	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	915.4;
	6835	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1057.1;
	6835	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4810.3;
	6835	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13300.9;
	6835	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7490.7;
	6835	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12780.5;
	6835	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2090.1;
	6836	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1044.1;
	6836	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	895.7;
	6836	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1037.7;
	6836	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4713.2;
	6836	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12961.1;
	6836	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7239.4;
	6836	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12591.7;
	6836	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2052.1;
	6837	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1020.2;
	6837	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	854.2;
	6837	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	979.2;
	6837	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4442;
	6837	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12227.5;
	6837	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6690.7;
	6837	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12121;
	6837	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1959.1;
	6838	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	979.2;
	6838	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	797.6;
	6838	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	905;
	6838	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4071.9;
	6838	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11193.8;
	6838	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5986.1;
	6838	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11359.8;
	6838	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1832;
	6839	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.1;
	6839	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742.2;
	6839	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	829.3;
	6839	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3692.7;
	6839	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10171.4;
	6839	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5328.2;
	6839	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10516.2;
	6839	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1621.6;
	6840	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903;
	6840	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	697.5;
	6840	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	782.2;
	6840	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3410.4;
	6840	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9399.2;
	6840	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4867.8;
	6840	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9786.3;
	6840	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1546.7;
	6841	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.5;
	6841	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	672.8;
	6841	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	751.8;
	6841	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3221.9;
	6841	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9006.3;
	6841	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4522.2;
	6841	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9362.9;
	6841	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1489.9;
	6842	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.2;
	6842	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	663.8;
	6842	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	731.7;
	6842	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3084.7;
	6842	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8796.5;
	6842	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4377;
	6842	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9116.7;
	6842	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1419.4;
	6843	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.1;
	6843	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	663.2;
	6843	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	722;
	6843	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3017.7;
	6843	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8732.1;
	6843	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4290.2;
	6843	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9003.6;
	6843	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1426.6;
	6844	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	876.2;
	6844	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	663.1;
	6844	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	722.3;
	6844	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3005.6;
	6844	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8915.5;
	6844	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4350.9;
	6844	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9119.7;
	6844	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1413.6;
	6845	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.2;
	6845	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	692.8;
	6845	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	746.6;
	6845	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3126.3;
	6845	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9453.1;
	6845	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4655.4;
	6845	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9657.5;
	6845	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1496.6;
	6846	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.5;
	6846	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	758.6;
	6846	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	812.3;
	6846	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3483.6;
	6846	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10599.8;
	6846	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5267.7;
	6846	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10369;
	6846	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1671;
	6847	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.3;
	6847	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	793.6;
	6847	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	842.5;
	6847	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3580.6;
	6847	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11028.7;
	6847	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5506.2;
	6847	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10553.1;
	6847	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1716.9;
	6848	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925.3;
	6848	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	787.7;
	6848	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838.2;
	6848	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3583.7;
	6848	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11115.5;
	6848	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5556.3;
	6848	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10807.2;
	6848	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1740.3;
	6849	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.9;
	6849	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	814.7;
	6849	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	867.8;
	6849	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3832.2;
	6849	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11678.8;
	6849	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5833.3;
	6849	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11555.1;
	6849	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1835.4;
	6850	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970.5;
	6850	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	853.5;
	6850	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	909.4;
	6850	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4174.9;
	6850	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12496.1;
	6850	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6157.3;
	6850	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12436.1;
	6850	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1956.1;
	6851	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	996.7;
	6851	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	885.6;
	6851	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	953.2;
	6851	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4565.1;
	6851	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13309.6;
	6851	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6529.2;
	6851	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13169;
	6851	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2064.2;
	6852	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1023.6;
	6852	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	921.4;
	6852	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1001.9;
	6852	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4912.1;
	6852	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14065.2;
	6852	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6961;
	6852	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13708.5;
	6852	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2161.9;
	6853	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1062;
	6853	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	956.5;
	6853	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1065.2;
	6853	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5229.3;
	6853	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14823.4;
	6853	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7483.4;
	6853	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14233.9;
	6853	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2268.7;
	6854	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1097.7;
	6854	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	980.8;
	6854	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1122.5;
	6854	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5468.9;
	6854	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15521.1;
	6854	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8011.7;
	6854	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14648.8;
	6854	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2372.3;
	6855	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1116.7;
	6855	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	988.9;
	6855	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1154.7;
	6855	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5629;
	6855	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16087.4;
	6855	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8478.4;
	6855	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14940;
	6855	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2434.4;
	6856	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1125.6;
	6856	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	991.1;
	6856	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1170.5;
	6856	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5717.8;
	6856	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16403.7;
	6856	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8798.7;
	6856	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14997.9;
	6856	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2471;
	6857	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1100.2;
	6857	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	967.9;
	6857	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1148.2;
	6857	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5613.4;
	6857	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16264.6;
	6857	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8881.1;
	6857	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14667.9;
	6857	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2433.7;
	6858	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1058.3;
	6858	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	932.3;
	6858	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1094;
	6858	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5359.9;
	6858	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15557.2;
	6858	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8636.3;
	6858	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13816.3;
	6858	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2337.9;
	6859	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030.6;
	6859	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	923;
	6859	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1053.1;
	6859	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5259.7;
	6859	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15037.8;
	6859	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8407.1;
	6859	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13581.8;
	6859	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2291.4;
	6860	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.4;
	6860	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	883.2;
	6860	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1020.5;
	6860	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5119;
	6860	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14298.6;
	6860	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8062.9;
	6860	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13070.9;
	6860	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2213.4;
	6861	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	973.5;
	6861	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	828.2;
	6861	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	959.7;
	6861	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4837.4;
	6861	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13195.9;
	6861	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7518;
	6861	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12412.9;
	6861	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2080.5;
	6862	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.5;
	6862	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	760.1;
	6862	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	880;
	6862	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4460.7;
	6862	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11775.6;
	6862	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6888.4;
	6862	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11580.3;
	6862	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1908.2;
	6863	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.3;
	6863	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	696.5;
	6863	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	804.8;
	6863	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4049.4;
	6863	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10356.7;
	6863	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6236.4;
	6863	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10615.1;
	6863	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1733.6;
	6864	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871.5;
	6864	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	660;
	6864	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	760.2;
	6864	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3845;
	6864	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9185.3;
	6864	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5575;
	6864	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10209.9;
	6864	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1622.8;
	6865	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	861.3;
	6865	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	639.6;
	6865	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	739;
	6865	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3623;
	6865	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8528.9;
	6865	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5281.7;
	6865	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9731.7;
	6865	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1531.7;
	6866	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	853.2;
	6866	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	627.4;
	6866	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	724.7;
	6866	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3450.9;
	6866	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8129.6;
	6866	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5090.1;
	6866	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9422.7;
	6866	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1458.2;
	6867	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	846.6;
	6867	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	621.1;
	6867	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	717.9;
	6867	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3327.7;
	6867	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7880.4;
	6867	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5037.5;
	6867	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9231.1;
	6867	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1425.6;
	6868	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	845.6;
	6868	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	630.8;
	6868	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	721.2;
	6868	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3266.7;
	6868	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7920.4;
	6868	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5110.5;
	6868	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9266.8;
	6868	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1426.8;
	6869	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	860.1;
	6869	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	656.2;
	6869	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	753.9;
	6869	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3375;
	6869	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8436.5;
	6869	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5405.7;
	6869	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9737.5;
	6869	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1504.8;
	6870	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.8;
	6870	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719.8;
	6870	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	825.9;
	6870	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3722;
	6870	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9465.1;
	6870	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6049;
	6870	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10402;
	6870	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1668.6;
	6871	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915;
	6871	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.7;
	6871	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	864.3;
	6871	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3797.9;
	6871	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9909.1;
	6871	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6270.3;
	6871	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10530.7;
	6871	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1729.3;
	6872	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.8;
	6872	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	745.5;
	6872	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849.2;
	6872	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3805.4;
	6872	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9853.4;
	6872	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6307.6;
	6872	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10745.3;
	6872	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1805.8;
	6873	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.6;
	6873	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	764.6;
	6873	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	856.6;
	6873	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4115.2;
	6873	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10097.9;
	6873	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6523;
	6873	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11516.2;
	6873	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1898.5;
	6874	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.9;
	6874	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	786.1;
	6874	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	867.4;
	6874	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4468;
	6874	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10431.1;
	6874	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6846.4;
	6874	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12362.1;
	6874	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2007.9;
	6875	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.5;
	6875	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	794.7;
	6875	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	879.1;
	6875	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4821.4;
	6875	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10647.6;
	6875	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7185.8;
	6875	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13178.3;
	6875	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2048.2;
	6876	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.2;
	6876	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	795.3;
	6876	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	883.5;
	6876	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5133.9;
	6876	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10911.5;
	6876	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7524.9;
	6876	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13855.3;
	6876	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2197;
	6877	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.7;
	6877	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	808.2;
	6877	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	893.9;
	6877	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5422.2;
	6877	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11330.3;
	6877	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7931.8;
	6877	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14525.4;
	6877	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2291.2;
	6878	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948.7;
	6878	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	809.7;
	6878	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	903.3;
	6878	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5571.6;
	6878	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11670.2;
	6878	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8320.7;
	6878	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14911.1;
	6878	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2365.4;
	6879	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.5;
	6879	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	797.8;
	6879	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	912.5;
	6879	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5659.1;
	6879	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11771.2;
	6879	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8596.3;
	6879	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15090.8;
	6879	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2421.9;
	6880	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.5;
	6880	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	799.9;
	6880	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	926.4;
	6880	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5696.5;
	6880	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11769.5;
	6880	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8687;
	6880	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15140;
	6880	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2443.3;
	6881	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.2;
	6881	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	788.2;
	6881	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	926;
	6881	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5520.8;
	6881	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11704.3;
	6881	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8647.8;
	6881	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14644;
	6881	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2351.9;
	6882	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.8;
	6882	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	782.6;
	6882	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	927.7;
	6882	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5214.6;
	6882	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11433.8;
	6882	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8447.5;
	6882	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13766.6;
	6882	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2233.5;
	6883	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.2;
	6883	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	798.5;
	6883	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	932.2;
	6883	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5160.4;
	6883	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11601.2;
	6883	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8303.4;
	6883	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13551.3;
	6883	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2213.3;
	6884	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.5;
	6884	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	781.3;
	6884	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	925.5;
	6884	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5076.5;
	6884	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11376;
	6884	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7971.1;
	6884	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13088.8;
	6884	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2152.9;
	6885	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939;
	6885	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	734.9;
	6885	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	881.8;
	6885	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4814.3;
	6885	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10758;
	6885	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7416.6;
	6885	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12397.1;
	6885	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2044.3;
	6886	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.7;
	6886	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	692.4;
	6886	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	816.6;
	6886	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4437.9;
	6886	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9878.8;
	6886	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6708.7;
	6886	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11486;
	6886	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1899.4;
	6887	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.1;
	6887	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	654;
	6887	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	761.9;
	6887	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4031.1;
	6887	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8958.7;
	6887	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5962.5;
	6887	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10609.9;
	6887	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1738.2;
	6888	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	865.9;
	6888	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	622;
	6888	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	742.4;
	6888	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3688.9;
	6888	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8201.6;
	6888	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5653.1;
	6888	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9832.2;
	6888	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1605.8;
	6889	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	854.8;
	6889	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	607.6;
	6889	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	722.3;
	6889	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3468;
	6889	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7792.2;
	6889	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5261.2;
	6889	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9368.2;
	6889	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1543.7;
	6890	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	846.1;
	6890	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	600.3;
	6890	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	707.2;
	6890	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3311.8;
	6890	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7567.7;
	6890	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5025.8;
	6890	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9031.9;
	6890	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1388.8;
	6891	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	849.3;
	6891	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.9;
	6891	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	704.2;
	6891	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3219;
	6891	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7523.4;
	6891	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4920.3;
	6891	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8937.8;
	6891	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1364.3;
	6892	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	851.2;
	6892	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	614.7;
	6892	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	706.6;
	6892	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3174.8;
	6892	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7679.1;
	6892	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4974;
	6892	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9043;
	6892	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1354.4;
	6893	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	858.2;
	6893	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	637;
	6893	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	734.6;
	6893	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3275.1;
	6893	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8218.5;
	6893	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5272.1;
	6893	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9548.2;
	6893	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1489.7;
	6894	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.7;
	6894	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	698.8;
	6894	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	805.2;
	6894	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3621;
	6894	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9196.7;
	6894	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5878.7;
	6894	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10224.6;
	6894	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1659.9;
	6895	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.5;
	6895	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735.9;
	6895	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.8;
	6895	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3705.2;
	6895	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9804.9;
	6895	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6180;
	6895	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10462;
	6895	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1697.7;
	6896	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.7;
	6896	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.5;
	6896	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	833.9;
	6896	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3735.8;
	6896	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9914;
	6896	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6192.8;
	6896	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10597.7;
	6896	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1749.7;
	6897	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.1;
	6897	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755.6;
	6897	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	858.3;
	6897	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4069.6;
	6897	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10093.3;
	6897	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6415.7;
	6897	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11249.3;
	6897	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1784.9;
	6898	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.1;
	6898	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772.1;
	6898	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	880.2;
	6898	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4431.2;
	6898	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10291;
	6898	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6693.4;
	6898	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12084.3;
	6898	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1849.8;
	6899	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.7;
	6899	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	780.6;
	6899	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.7;
	6899	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4761.4;
	6899	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10508.6;
	6899	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6936;
	6899	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12758.3;
	6899	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1800.6;
	6900	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.4;
	6900	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	781.1;
	6900	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	906.9;
	6900	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5044.3;
	6900	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10648.7;
	6900	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7251.7;
	6900	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13246.4;
	6900	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1810.8;
	6901	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960;
	6901	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	788.8;
	6901	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	935.4;
	6901	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5311.8;
	6901	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10885.8;
	6901	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7683.8;
	6901	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13697.9;
	6901	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1848.7;
	6902	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	974.8;
	6902	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	797.9;
	6902	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	963.8;
	6902	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5498.9;
	6902	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11098.3;
	6902	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8057.1;
	6902	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13981.7;
	6902	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1956.9;
	6903	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	993.3;
	6903	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	799.4;
	6903	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	987.5;
	6903	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5595.8;
	6903	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11389.9;
	6903	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8421.4;
	6903	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14084.2;
	6903	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1986.2;
	6904	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1008.5;
	6904	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	803.1;
	6904	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1009;
	6904	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5612;
	6904	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11626.1;
	6904	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8649.6;
	6904	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13969.5;
	6904	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2075.7;
	6905	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1001.4;
	6905	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	795.9;
	6905	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1005.5;
	6905	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5411.7;
	6905	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11659.2;
	6905	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8635.8;
	6905	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13338.4;
	6905	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2062.9;
	6906	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985.1;
	6906	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	788.3;
	6906	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	982.2;
	6906	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5111.6;
	6906	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11424.9;
	6906	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8253.4;
	6906	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12642.8;
	6906	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1993;
	6907	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	979.2;
	6907	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	799.7;
	6907	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	971.9;
	6907	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4994.6;
	6907	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11389.4;
	6907	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7980;
	6907	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12489.3;
	6907	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1960.6;
	6908	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	978.6;
	6908	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	791;
	6908	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	955.6;
	6908	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4834.3;
	6908	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11174.7;
	6908	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7657.7;
	6908	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12102.9;
	6908	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1893.9;
	6909	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	968;
	6909	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	767.9;
	6909	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	917.7;
	6909	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4608.5;
	6909	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10690.8;
	6909	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7268.3;
	6909	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11618;
	6909	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1848.7;
	6910	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.8;
	6910	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.4;
	6910	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	877.3;
	6910	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4359.7;
	6910	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10112.1;
	6910	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6807.3;
	6910	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10979.4;
	6910	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1745.4;
	6911	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.7;
	6911	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	704.5;
	6911	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	823.3;
	6911	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4072.7;
	6911	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9413.5;
	6911	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6320.7;
	6911	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10259.4;
	6911	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1612.1;
	6912	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.3;
	6912	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	672.7;
	6912	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	813.5;
	6912	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3749.8;
	6912	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8851.9;
	6912	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5827.4;
	6912	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9496.4;
	6912	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1555.3;
	6913	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.1;
	6913	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	654.6;
	6913	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	779.5;
	6913	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3528.1;
	6913	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8416.8;
	6913	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5450.6;
	6913	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9047.3;
	6913	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1487.3;
	6914	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.2;
	6914	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	637.9;
	6914	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	757.5;
	6914	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3373.3;
	6914	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8148.2;
	6914	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5203;
	6914	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8739.8;
	6914	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1433.2;
	6915	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	859;
	6915	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	628.4;
	6915	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	745.3;
	6915	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3263.9;
	6915	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8016.8;
	6915	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5066.9;
	6915	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8565.1;
	6915	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1384;
	6916	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	857.2;
	6916	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	627.5;
	6916	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	740.9;
	6916	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3201;
	6916	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8037.5;
	6916	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4990.8;
	6916	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8481.5;
	6916	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1381.9;
	6917	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864;
	6917	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	646.1;
	6917	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	749;
	6917	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3206.5;
	6917	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8233.3;
	6917	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5056.3;
	6917	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8536.3;
	6917	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1395.8;
	6918	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.4;
	6918	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	663.5;
	6918	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	772.4;
	6918	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3280.2;
	6918	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8704.3;
	6918	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5228.1;
	6918	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8755.7;
	6918	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1436.3;
	6919	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.5;
	6919	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	684.2;
	6919	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	802.1;
	6919	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3322.6;
	6919	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9132.1;
	6919	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5447.3;
	6919	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8919.9;
	6919	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1487.9;
	6920	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.1;
	6920	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714;
	6920	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	829.5;
	6920	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3494.9;
	6920	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9629.7;
	6920	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5682.9;
	6920	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9379.5;
	6920	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1560.5;
	6921	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.5;
	6921	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746.6;
	6921	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	877.6;
	6921	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3922.4;
	6921	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10363.6;
	6921	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6115.3;
	6921	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10291.5;
	6921	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1663.3;
	6922	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.8;
	6922	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	784;
	6922	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	923.3;
	6922	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4334.7;
	6922	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11101.6;
	6922	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6638.2;
	6922	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11270.1;
	6922	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1798.4;
	6923	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.3;
	6923	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	819.9;
	6923	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	970.4;
	6923	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4678.2;
	6923	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11927;
	6923	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7109;
	6923	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12081.8;
	6923	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1940.6;
	6924	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	997.9;
	6924	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	854.6;
	6924	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1020;
	6924	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4976.5;
	6924	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12623.4;
	6924	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7547.3;
	6924	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12815.8;
	6924	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2037.7;
	6925	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1032.1;
	6925	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	890.1;
	6925	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1072.4;
	6925	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5219.6;
	6925	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13174.2;
	6925	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7978.5;
	6925	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13490.7;
	6925	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2151.5;
	6926	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1061.7;
	6926	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	935.3;
	6926	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1119.4;
	6926	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5377.8;
	6926	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13646.8;
	6926	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8307.7;
	6926	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13967.3;
	6926	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2250.4;
	6927	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1077.4;
	6927	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	962.5;
	6927	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1158;
	6927	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5501.3;
	6927	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13937.8;
	6927	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8565.9;
	6927	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14201.8;
	6927	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2292.6;
	6928	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1082.4;
	6928	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	975.7;
	6928	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1182.8;
	6928	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5518.3;
	6928	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14085.3;
	6928	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8727.7;
	6928	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14233;
	6928	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2310.7;
	6929	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1071.6;
	6929	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	971.5;
	6929	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1170.6;
	6929	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5389.8;
	6929	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13959.1;
	6929	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8691.5;
	6929	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13914.4;
	6929	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2276.1;
	6930	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1051.3;
	6930	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	940.5;
	6930	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1142;
	6930	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5117.6;
	6930	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13410.9;
	6930	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8356.7;
	6930	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13238.7;
	6930	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2161.6;
	6931	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1034.9;
	6931	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	917.7;
	6931	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1117.4;
	6931	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5030;
	6931	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13211.6;
	6931	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8093.2;
	6931	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13053;
	6931	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2123.5;
	6932	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1022.3;
	6932	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	890.9;
	6932	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1082;
	6932	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4868.3;
	6932	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12711.1;
	6932	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7740.7;
	6932	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12625.4;
	6932	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2042;
	6933	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1003.9;
	6933	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	854.6;
	6933	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1036.8;
	6933	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4637.4;
	6933	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12129.3;
	6933	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7394.8;
	6933	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12095.7;
	6933	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1885.3;
	6934	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	973.6;
	6934	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	820.8;
	6934	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	967.5;
	6934	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4365.1;
	6934	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11384.7;
	6934	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6948.2;
	6934	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11455;
	6934	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1814.7;
	6935	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.3;
	6935	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777.2;
	6935	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	898.8;
	6935	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4094.3;
	6935	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10570.3;
	6935	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6430.2;
	6935	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10778.2;
	6935	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1738.3;
	6936	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.8;
	6936	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.5;
	6936	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	833.5;
	6936	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3822.1;
	6936	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9804;
	6936	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5932.4;
	6936	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10114.2;
	6936	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1650.3;
	6937	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.3;
	6937	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	700.8;
	6937	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	794.2;
	6937	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3609.6;
	6937	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9222.1;
	6937	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5560.8;
	6937	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9601.9;
	6937	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1566.7;
	6938	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873;
	6938	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	676.9;
	6938	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	765;
	6938	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3437.7;
	6938	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8790.8;
	6938	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5325.2;
	6938	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9204.8;
	6938	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1501.9;
	6939	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.1;
	6939	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	664.9;
	6939	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	748.2;
	6939	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3304.7;
	6939	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8547.2;
	6939	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5172.1;
	6939	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8945.2;
	6939	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1492.3;
	6940	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	861.6;
	6940	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	653.9;
	6940	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	739.3;
	6940	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3228.3;
	6940	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8458.2;
	6940	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5108.6;
	6940	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8808.2;
	6940	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1452.8;
	6941	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	857.1;
	6941	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	654.2;
	6941	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	736.8;
	6941	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3198.7;
	6941	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8519;
	6941	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5157.1;
	6941	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8788.2;
	6941	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1474.8;
	6942	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	862.7;
	6942	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	670.3;
	6942	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	752.5;
	6942	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3219.1;
	6942	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8780.4;
	6942	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5265.6;
	6942	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8884;
	6942	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1513.5;
	6943	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	869.3;
	6943	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	689.9;
	6943	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	775.5;
	6943	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3212.3;
	6943	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9076.1;
	6943	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5415.9;
	6943	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8903.8;
	6943	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1522.5;
	6944	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871.5;
	6944	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	713.2;
	6944	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	807.1;
	6944	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3398;
	6944	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9576.5;
	6944	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5653.6;
	6944	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9364.1;
	6944	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1577.4;
	6945	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.1;
	6945	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	768.4;
	6945	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	862.2;
	6945	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3847.1;
	6945	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10481.6;
	6945	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6189.4;
	6945	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10343.2;
	6945	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1706.4;
	6946	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.7;
	6946	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	809.6;
	6946	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	915.7;
	6946	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4260.7;
	6946	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11411.7;
	6946	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6788.7;
	6946	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11336.5;
	6946	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1840.7;
	6947	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.4;
	6947	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	857.8;
	6947	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	971.8;
	6947	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4635;
	6947	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12319.7;
	6947	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7343.3;
	6947	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12261.5;
	6947	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1953.1;
	6948	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1001.7;
	6948	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	911.4;
	6948	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1032.6;
	6948	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4978.9;
	6948	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13241.5;
	6948	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7889.1;
	6948	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13099.3;
	6948	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2089;
	6949	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1025;
	6949	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	963.8;
	6949	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1095.8;
	6949	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5268.4;
	6949	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14060.3;
	6949	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8382.7;
	6949	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13808;
	6949	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2201.3;
	6950	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1058.4;
	6950	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1010.4;
	6950	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1148.9;
	6950	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5485.2;
	6950	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14783.6;
	6950	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8780.2;
	6950	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14312.8;
	6950	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2288.6;
	6951	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1074.6;
	6951	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1044.7;
	6951	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1183.4;
	6951	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5599.7;
	6951	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15245.2;
	6951	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9058.7;
	6951	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14573.2;
	6951	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2362.1;
	6952	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1090.6;
	6952	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1060.3;
	6952	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1205.4;
	6952	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5594.8;
	6952	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15431.4;
	6952	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9193.1;
	6952	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14639;
	6952	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2386.4;
	6953	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1095.4;
	6953	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1056.6;
	6953	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1204.1;
	6953	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5491.3;
	6953	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15283.2;
	6953	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9127.2;
	6953	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14414.3;
	6953	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2367.7;
	6954	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1077.4;
	6954	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1020.6;
	6954	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1164.9;
	6954	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5247.3;
	6954	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14667.7;
	6954	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8782.7;
	6954	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13822.3;
	6954	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2249.6;
	6955	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1066.3;
	6955	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1008.7;
	6955	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1139.5;
	6955	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5196.6;
	6955	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14569.5;
	6955	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8577.8;
	6955	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13713.7;
	6955	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2259.5;
	6956	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1049.2;
	6956	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	972.5;
	6956	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1103.6;
	6956	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5066.2;
	6956	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14083.3;
	6956	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8215.8;
	6956	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13243.7;
	6956	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2183.4;
	6957	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1019.8;
	6957	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	920.4;
	6957	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1040.7;
	6957	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4798.9;
	6957	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13220.9;
	6957	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7708.1;
	6957	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12548.2;
	6957	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2045.5;
	6958	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	987.5;
	6958	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	862.8;
	6958	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	966.3;
	6958	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4467.8;
	6958	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12132.8;
	6958	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7058.4;
	6958	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11731.7;
	6958	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1890.6;
	6959	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.7;
	6959	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	792.1;
	6959	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.7;
	6959	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4107.7;
	6959	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10978.4;
	6959	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6345.3;
	6959	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10909.2;
	6959	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1779.6;
	6960	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920.5;
	6960	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.9;
	6960	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	832.7;
	6960	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3808.3;
	6960	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10043.9;
	6960	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5807.5;
	6960	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10147.8;
	6960	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1659.7;
	6961	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.5;
	6961	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719.8;
	6961	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	796.5;
	6961	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3607.5;
	6961	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9484.8;
	6961	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5482;
	6961	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9666.6;
	6961	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1595.6;
	6962	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.4;
	6962	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	697.2;
	6962	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	772.4;
	6962	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3455.1;
	6962	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9142.4;
	6962	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5303;
	6962	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9365.3;
	6962	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1534.2;
	6963	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885;
	6963	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	689.9;
	6963	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	762.4;
	6963	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3354.6;
	6963	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8955.3;
	6963	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5286.5;
	6963	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9261.9;
	6963	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1534.7;
	6964	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.2;
	6964	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	692.2;
	6964	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	763.4;
	6964	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3320;
	6964	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9059.9;
	6964	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5394.3;
	6964	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9427.1;
	6964	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1565.1;
	6965	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893;
	6965	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720.8;
	6965	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	793.1;
	6965	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3448.6;
	6965	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9682.7;
	6965	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5832.3;
	6965	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9956.5;
	6965	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1645.7;
	6966	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.9;
	6966	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	789;
	6966	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.7;
	6966	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3785.9;
	6966	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10796.3;
	6966	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6470.9;
	6966	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10609.4;
	6966	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1806.3;
	6967	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951.8;
	6967	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	826.1;
	6967	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	907.7;
	6967	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3858.6;
	6967	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11357.6;
	6967	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6713.2;
	6967	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10720.6;
	6967	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1817.7;
	6968	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.5;
	6968	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	824.7;
	6968	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	909.7;
	6968	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3883.8;
	6968	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11440.1;
	6968	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6695.6;
	6968	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10982.5;
	6968	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1907.2;
	6969	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.1;
	6969	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	857.7;
	6969	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	950.9;
	6969	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4218.3;
	6969	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11997.8;
	6969	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6987.4;
	6969	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11849.9;
	6969	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2008;
	6970	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981.5;
	6970	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	900.7;
	6970	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	998;
	6970	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4564.7;
	6970	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12701.1;
	6970	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7435;
	6970	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12697.4;
	6970	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2103.4;
	6971	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1012.4;
	6971	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	944.4;
	6971	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1045.9;
	6971	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4913.2;
	6971	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13399.8;
	6971	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7911.6;
	6971	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13485.2;
	6971	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2209.9;
	6972	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1041.6;
	6972	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	999.7;
	6972	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1103.5;
	6972	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5238.4;
	6972	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14229.5;
	6972	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8398.1;
	6972	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14176.1;
	6972	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2328.6;
	6973	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1073.9;
	6973	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1056.4;
	6973	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1172.3;
	6973	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5517.1;
	6973	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15107.7;
	6973	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8891.8;
	6973	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14862.8;
	6973	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2448.9;
	6974	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1103.1;
	6974	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1093.5;
	6974	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1222.4;
	6974	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5736.1;
	6974	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15824.6;
	6974	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9294.1;
	6974	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15346.8;
	6974	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2537.3;
	6975	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1124.2;
	6975	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1124.6;
	6975	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1272.3;
	6975	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5867;
	6975	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16343.3;
	6975	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9616.5;
	6975	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15625.8;
	6975	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2552.8;
	6976	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1133.9;
	6976	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1135.2;
	6976	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1297.2;
	6976	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5884.5;
	6976	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16637.2;
	6976	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9810.5;
	6976	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15609.1;
	6976	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2575.7;
	6977	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1122.4;
	6977	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1114.6;
	6977	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1279.2;
	6977	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5728.4;
	6977	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16450.3;
	6977	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9746.8;
	6977	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15175;
	6977	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2519.3;
	6978	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1096.9;
	6978	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1074.3;
	6978	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1233.4;
	6978	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5435.6;
	6978	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15817.1;
	6978	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9356.6;
	6978	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14423.1;
	6978	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2445.1;
	6979	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1078.2;
	6979	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1062.1;
	6979	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1199.2;
	6979	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5391.3;
	6979	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15556.9;
	6979	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9075.2;
	6979	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14243.7;
	6979	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2421.7;
	6980	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1053.6;
	6980	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1027.4;
	6980	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1153.8;
	6980	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5264.6;
	6980	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14993;
	6980	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8623.3;
	6980	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13695.1;
	6980	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2339.4;
	6981	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1020.5;
	6981	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	971.7;
	6981	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1084.1;
	6981	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4977.2;
	6981	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14061.3;
	6981	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8007;
	6981	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12912.8;
	6981	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2205;
	6982	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981.3;
	6982	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	901.4;
	6982	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1003.4;
	6982	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4587.1;
	6982	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12818.7;
	6982	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7293.3;
	6982	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11981.6;
	6982	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2037.6;
	6983	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.5;
	6983	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	836.2;
	6983	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	921.4;
	6983	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4192.6;
	6983	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11594.7;
	6983	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6564;
	6983	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11072.6;
	6983	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1878.9;
	6984	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920.8;
	6984	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	788.2;
	6984	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	867.5;
	6984	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3913.7;
	6984	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10589.2;
	6984	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6018.9;
	6984	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10412.2;
	6984	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1753.6;
	6985	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.8;
	6985	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755;
	6985	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	831.8;
	6985	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3717.4;
	6985	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10000.9;
	6985	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5695.7;
	6985	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9928.7;
	6985	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1656.4;
	6986	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.8;
	6986	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	731.6;
	6986	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	806.9;
	6986	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3582.7;
	6986	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9549.4;
	6986	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5527.3;
	6986	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9617.9;
	6986	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1531.7;
	6987	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.5;
	6987	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	713.9;
	6987	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	789.2;
	6987	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3481.1;
	6987	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9308.5;
	6987	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5453.6;
	6987	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9440.7;
	6987	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1507.1;
	6988	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.3;
	6988	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	716.6;
	6988	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	781.1;
	6988	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3438.4;
	6988	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9349.8;
	6988	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5488.2;
	6988	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9499.6;
	6988	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1566.8;
	6989	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.4;
	6989	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.7;
	6989	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	803.7;
	6989	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3537.9;
	6989	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9874.7;
	6989	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5797.2;
	6989	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9996;
	6989	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1653.6;
	6990	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.5;
	6990	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	785.3;
	6990	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	872.5;
	6990	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3876.5;
	6990	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11073;
	6990	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6389.8;
	6990	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10674.5;
	6990	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1777.8;
	6991	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.2;
	6991	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	809.2;
	6991	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	907;
	6991	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3941.2;
	6991	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11495.9;
	6991	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6626.4;
	6991	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10800.5;
	6991	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1848.3;
	6992	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.3;
	6992	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	812.7;
	6992	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	903.2;
	6992	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3954.9;
	6992	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11542.5;
	6992	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6607.3;
	6992	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11043.1;
	6992	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1943.9;
	6993	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.6;
	6993	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	836.8;
	6993	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	929.7;
	6993	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4268.9;
	6993	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12205;
	6993	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6858;
	6993	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11859.5;
	6993	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1971.6;
	6994	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951.4;
	6994	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	878.6;
	6994	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	970.9;
	6994	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4612.2;
	6994	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13045.8;
	6994	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7217.9;
	6994	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12723.2;
	6994	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2057.3;
	6995	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	971.5;
	6995	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	919.2;
	6995	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1018.6;
	6995	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4914.4;
	6995	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13812.9;
	6995	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7657;
	6995	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13546.7;
	6995	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2214.6;
	6996	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990.1;
	6996	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	958.5;
	6996	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1082.8;
	6996	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5235.3;
	6996	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14477.2;
	6996	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8144.3;
	6996	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14309.1;
	6996	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2321.7;
	6997	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1016.3;
	6997	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1004.5;
	6997	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1150.1;
	6997	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5523.2;
	6997	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15314.2;
	6997	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8679.1;
	6997	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15078.1;
	6997	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2424.2;
	6998	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1046;
	6998	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1043.7;
	6998	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1208.6;
	6998	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5733.9;
	6998	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16134.9;
	6998	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9117.9;
	6998	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15653.8;
	6998	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2481.4;
	6999	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1071.7;
	6999	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1063.4;
	6999	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1241.8;
	6999	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5850.4;
	6999	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16648.8;
	6999	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9372.8;
	6999	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15892.6;
	6999	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2564.6;
	7000	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1090.5;
	7000	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1070.9;
	7000	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1255.1;
	7000	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5884.4;
	7000	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16860.3;
	7000	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9486.3;
	7000	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15924.9;
	7000	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2565.6;
	7001	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1081;
	7001	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1052.5;
	7001	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1231.6;
	7001	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5737;
	7001	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16558.9;
	7001	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9433.6;
	7001	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15477;
	7001	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2507.3;
	7002	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1057.3;
	7002	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1009.2;
	7002	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1180.1;
	7002	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5450.4;
	7002	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15875.5;
	7002	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9074.3;
	7002	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14558;
	7002	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2405.9;
	7003	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1040.8;
	7003	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	992.2;
	7003	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1147.6;
	7003	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5399.4;
	7003	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15570.2;
	7003	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8856.3;
	7003	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14273.5;
	7003	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2371.9;
	7004	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1023;
	7004	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	961.4;
	7004	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1108.8;
	7004	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5248.2;
	7004	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14913;
	7004	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8464.5;
	7004	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13713.2;
	7004	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2275.3;
	7005	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	988.9;
	7005	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	905.4;
	7005	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1037.5;
	7005	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4979.3;
	7005	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13938.4;
	7005	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7949;
	7005	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12960.5;
	7005	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2143.7;
	7006	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.1;
	7006	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	839.4;
	7006	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	957.8;
	7006	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4612.9;
	7006	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12712.2;
	7006	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7310.4;
	7006	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12050.9;
	7006	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1999.9;
	7007	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.8;
	7007	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	774.3;
	7007	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	880.3;
	7007	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4221.1;
	7007	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11469;
	7007	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6602.7;
	7007	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11110.1;
	7007	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1769.7;
	7008	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	881.1;
	7008	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	724.9;
	7008	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	832.9;
	7008	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3935.9;
	7008	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10428.9;
	7008	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6102.6;
	7008	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10399.6;
	7008	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1632;
	7009	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867.5;
	7009	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	694.2;
	7009	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	799.8;
	7009	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3710.1;
	7009	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9836.9;
	7009	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5808;
	7009	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9945.8;
	7009	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1569.4;
	7010	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	858.7;
	7010	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	675.4;
	7010	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	780.1;
	7010	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3562.3;
	7010	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9430.5;
	7010	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5647;
	7010	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9666;
	7010	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1499.7;
	7011	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.8;
	7011	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	668.1;
	7011	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	773.5;
	7011	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3464.1;
	7011	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9214.3;
	7011	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5563.5;
	7011	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9485.3;
	7011	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1515.4;
	7012	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	856.1;
	7012	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	669.8;
	7012	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	775.3;
	7012	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3419;
	7012	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9346.1;
	7012	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5608.7;
	7012	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9526.9;
	7012	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1505.3;
	7013	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.5;
	7013	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	700.8;
	7013	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	800.8;
	7013	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3533.3;
	7013	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9898.3;
	7013	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5958;
	7013	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10050.9;
	7013	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1603.9;
	7014	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.4;
	7014	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	764.5;
	7014	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.9;
	7014	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3885.6;
	7014	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11100.8;
	7014	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6591;
	7014	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10783.3;
	7014	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1803;
	7015	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.2;
	7015	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	797.2;
	7015	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	900.5;
	7015	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3967.2;
	7015	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11604;
	7015	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6854.3;
	7015	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10978.5;
	7015	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1861.7;
	7016	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.8;
	7016	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	804.8;
	7016	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	903.1;
	7016	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3978.3;
	7016	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11586.4;
	7016	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6828.9;
	7016	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11166.1;
	7016	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1880.6;
	7017	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944;
	7017	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	846.9;
	7017	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940.9;
	7017	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4307.8;
	7017	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12214.6;
	7017	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7066.3;
	7017	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11894.9;
	7017	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1956;
	7018	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.7;
	7018	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	898.6;
	7018	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	984.9;
	7018	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4650.9;
	7018	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12975.3;
	7018	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7461.8;
	7018	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12678.5;
	7018	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2051.6;
	7019	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	983.5;
	7019	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	945.3;
	7019	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1032;
	7019	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4980;
	7019	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13711.1;
	7019	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7901.9;
	7019	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13364.6;
	7019	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2154.8;
	7020	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1002.6;
	7020	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	986.7;
	7020	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1082.1;
	7020	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5279.4;
	7020	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14481.6;
	7020	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8369.7;
	7020	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14023.6;
	7020	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2251.5;
	7021	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1040.7;
	7021	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1035;
	7021	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1139.1;
	7021	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5553.4;
	7021	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15262.4;
	7021	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8773;
	7021	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14570.8;
	7021	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2331.6;
	7022	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1079.3;
	7022	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1078.4;
	7022	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1188.8;
	7022	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5746.8;
	7022	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15880.7;
	7022	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9118.4;
	7022	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14882.2;
	7022	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2407.1;
	7023	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1101.6;
	7023	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1101.5;
	7023	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1230.5;
	7023	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5839.3;
	7023	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16308.9;
	7023	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9374.7;
	7023	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	15006.2;
	7023	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2424.6;
	7024	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1122.7;
	7024	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1115.1;
	7024	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1258;
	7024	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5865.6;
	7024	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16437.3;
	7024	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9532;
	7024	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14904.9;
	7024	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2429.1;
	7025	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1118.9;
	7025	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1100.3;
	7025	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1239.3;
	7025	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5697.9;
	7025	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16195.3;
	7025	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9475.9;
	7025	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	14449;
	7025	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2361.2;
	7026	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1095.1;
	7026	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1060.5;
	7026	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1196.8;
	7026	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5421.2;
	7026	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15524.9;
	7026	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9166.1;
	7026	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13805.2;
	7026	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2294.2;
	7027	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1069.7;
	7027	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1043.6;
	7027	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1168.3;
	7027	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5342.3;
	7027	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15194.4;
	7027	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8944.5;
	7027	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13708.2;
	7027	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2286.3;
	7028	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1047.7;
	7028	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	995.5;
	7028	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1126;
	7028	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5153.1;
	7028	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14404.4;
	7028	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8536.2;
	7028	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13188.1;
	7028	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2190.1;
	7029	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.5;
	7029	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	939;
	7029	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1063.1;
	7029	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4882.6;
	7029	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13452.9;
	7029	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8005.7;
	7029	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12511.7;
	7029	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2062.4;
	7030	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.7;
	7030	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	879.4;
	7030	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	986.2;
	7030	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4526.8;
	7030	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12354.9;
	7030	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7352.6;
	7030	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11722.6;
	7030	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1914.1;
	7031	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.1;
	7031	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	797.2;
	7031	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	902.5;
	7031	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4109.3;
	7031	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11089.2;
	7031	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6549.5;
	7031	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10807.9;
	7031	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1746.7;
	7032	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.8;
	7032	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.5;
	7032	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851.2;
	7032	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3775.5;
	7032	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10125.3;
	7032	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6015.1;
	7032	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10073.8;
	7032	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1621.1;
	7033	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.9;
	7033	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714.2;
	7033	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	822.2;
	7033	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3568.1;
	7033	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9521.6;
	7033	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5650;
	7033	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9573.9;
	7033	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1535.1;
	7034	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874;
	7034	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	691;
	7034	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	796;
	7034	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3404.5;
	7034	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9153;
	7034	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5429.8;
	7034	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9250.8;
	7034	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1465.4;
	7035	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.2;
	7035	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	677.4;
	7035	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	780.2;
	7035	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3303.6;
	7035	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8942.5;
	7035	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5310.1;
	7035	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9097.9;
	7035	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1433.6;
	7036	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871.5;
	7036	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	679.8;
	7036	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	778.3;
	7036	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3265.2;
	7036	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8979.9;
	7036	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5303.5;
	7036	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9203.6;
	7036	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1449.3;
	7037	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.8;
	7037	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	697.1;
	7037	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	799.6;
	7037	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3394.6;
	7037	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9476.2;
	7037	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5606.4;
	7037	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9710.8;
	7037	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1520.7;
	7038	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.2;
	7038	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751;
	7038	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.4;
	7038	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3753.6;
	7038	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10488.7;
	7038	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6236.1;
	7038	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10423.4;
	7038	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1697.6;
	7039	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.8;
	7039	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772.9;
	7039	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	904.4;
	7039	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3833.1;
	7039	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10880.1;
	7039	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6466.6;
	7039	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10582.4;
	7039	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1772.7;
	7040	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.9;
	7040	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	763.4;
	7040	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	887.8;
	7040	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3810.3;
	7040	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10691.3;
	7040	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6422;
	7040	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10592.7;
	7040	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1838.2;
	7041	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934;
	7041	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776.5;
	7041	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	890.8;
	7041	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4095.1;
	7041	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10746.1;
	7041	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6763.8;
	7041	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11003;
	7041	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1889.6;
	7042	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.2;
	7042	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	788.9;
	7042	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	905;
	7042	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4457.5;
	7042	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11104.6;
	7042	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7123.5;
	7042	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11625.7;
	7042	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1945.3;
	7043	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.7;
	7043	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	796.1;
	7043	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	923.7;
	7043	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4807.6;
	7043	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11341.5;
	7043	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7270.9;
	7043	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12155.5;
	7043	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1907.3;
	7044	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952;
	7044	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	800.1;
	7044	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	927.5;
	7044	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5072.1;
	7044	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11521.9;
	7044	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7275.7;
	7044	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12561.1;
	7044	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1962.5;
	7045	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.1;
	7045	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	813.7;
	7045	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	936.2;
	7045	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5262.3;
	7045	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11727.8;
	7045	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7252.9;
	7045	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13011.4;
	7045	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1923.8;
	7046	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.4;
	7046	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	821.4;
	7046	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	945.7;
	7046	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5273.1;
	7046	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11830.6;
	7046	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7273.9;
	7046	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13056.9;
	7046	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1925.6;
	7047	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969.5;
	7047	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	822.8;
	7047	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	953.9;
	7047	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5218.8;
	7047	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11890.2;
	7047	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7377.8;
	7047	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12816.2;
	7047	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1935.2;
	7048	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	976.5;
	7048	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	816;
	7048	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	964.8;
	7048	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5060.2;
	7048	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11851.5;
	7048	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7399.5;
	7048	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12514.3;
	7048	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1956.8;
	7049	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.6;
	7049	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	794.9;
	7049	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	948;
	7049	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4815;
	7049	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11602.7;
	7049	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7271.7;
	7049	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12033;
	7049	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1899.5;
	7050	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.1;
	7050	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776.8;
	7050	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	929.8;
	7050	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4568.1;
	7050	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11213.8;
	7050	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6967.7;
	7050	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11574.4;
	7050	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1850.4;
	7051	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.1;
	7051	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	791.7;
	7051	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	942.8;
	7051	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4496.5;
	7051	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11302.6;
	7051	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6923.9;
	7051	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11561.8;
	7051	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1878.2;
	7052	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.4;
	7052	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776.6;
	7052	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	938;
	7052	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4366.5;
	7052	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11001.7;
	7052	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6705.1;
	7052	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11260.2;
	7052	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1837.7;
	7053	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.7;
	7053	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740;
	7053	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	891.6;
	7053	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4168.4;
	7053	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10399.9;
	7053	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6317.5;
	7053	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10769.8;
	7053	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1746.8;
	7054	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.5;
	7054	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	696.3;
	7054	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824.2;
	7054	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3856.5;
	7054	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9484.3;
	7054	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5776.9;
	7054	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9970.2;
	7054	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1606.9;
	7055	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.1;
	7055	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	645.8;
	7055	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	761.8;
	7055	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3498.7;
	7055	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8495.1;
	7055	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5170.3;
	7055	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9089;
	7055	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1455.5;
	7056	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.5;
	7056	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	617.9;
	7056	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	711.5;
	7056	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3263.5;
	7056	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7812.6;
	7056	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4618.7;
	7056	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8438.3;
	7056	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1341.5;
	7057	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	842.2;
	7057	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.5;
	7057	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	690;
	7057	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3074;
	7057	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7383.6;
	7057	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4336.9;
	7057	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7991.4;
	7057	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1296.7;
	7058	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	840.7;
	7058	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.2;
	7058	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	677.4;
	7058	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2944.8;
	7058	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7178.6;
	7058	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4158.1;
	7058	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7741.9;
	7058	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1165.7;
	7059	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	837.7;
	7059	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586.1;
	7059	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	672.2;
	7059	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2837;
	7059	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7090.8;
	7059	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4068.1;
	7059	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7626.1;
	7059	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1153.6;
	7060	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	835.8;
	7060	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.5;
	7060	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	671;
	7060	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2792.7;
	7060	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7221.7;
	7060	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4105.1;
	7060	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7698.3;
	7060	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1161.4;
	7061	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	841;
	7061	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	617;
	7061	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	701.8;
	7061	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2876.4;
	7061	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7719.6;
	7061	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4372.1;
	7061	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8105.3;
	7061	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1280.5;
	7062	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866.4;
	7062	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	677.5;
	7062	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	776;
	7062	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3182.9;
	7062	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8724.6;
	7062	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4963;
	7062	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8794.1;
	7062	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1447.9;
	7063	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.4;
	7063	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711.5;
	7063	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	807.8;
	7063	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3253.3;
	7063	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9250.8;
	7063	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5218.4;
	7063	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8989;
	7063	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1476.3;
	7064	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.7;
	7064	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	716.9;
	7064	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	794.7;
	7064	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3183.4;
	7064	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9263.9;
	7064	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5197.3;
	7064	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9042.4;
	7064	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1518.4;
	7065	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.9;
	7065	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.9;
	7065	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	810.1;
	7065	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3279.3;
	7065	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9521;
	7065	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5304.9;
	7065	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9310.1;
	7065	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1539.8;
	7066	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.2;
	7066	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	745.2;
	7066	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	833.8;
	7066	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3405.9;
	7066	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9747.5;
	7066	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5445.4;
	7066	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9661.3;
	7066	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1595.7;
	7067	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.1;
	7067	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.7;
	7067	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	836.3;
	7067	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3559.1;
	7067	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9958.6;
	7067	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5551;
	7067	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9903.5;
	7067	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1551;
	7068	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.1;
	7068	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.2;
	7068	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.9;
	7068	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3699.3;
	7068	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10119.3;
	7068	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5663;
	7068	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10064.5;
	7068	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1563.9;
	7069	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.6;
	7069	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	760.8;
	7069	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.6;
	7069	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3867.2;
	7069	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10310.9;
	7069	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5787.9;
	7069	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10309.7;
	7069	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1590.8;
	7070	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.5;
	7070	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	766.9;
	7070	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	864;
	7070	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4013.9;
	7070	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10467.3;
	7070	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5960.8;
	7070	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10512.1;
	7070	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1668.7;
	7071	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.5;
	7071	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	767.1;
	7071	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	872.9;
	7071	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4118.8;
	7071	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10643.5;
	7071	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6164.4;
	7071	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10695.7;
	7071	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1679.5;
	7072	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.5;
	7072	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	766.3;
	7072	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.7;
	7072	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4177.2;
	7072	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10755.5;
	7072	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6302.6;
	7072	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10786.2;
	7072	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1730.7;
	7073	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.6;
	7073	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	761;
	7073	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	875.1;
	7073	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4067;
	7073	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10697.2;
	7073	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6289.1;
	7073	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10557;
	7073	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1700;
	7074	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.3;
	7074	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.3;
	7074	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	855.7;
	7074	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3839.7;
	7074	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10398;
	7074	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6052.7;
	7074	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10068.1;
	7074	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1655.3;
	7075	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.2;
	7075	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	760.2;
	7075	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	863.8;
	7075	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3832.4;
	7075	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10430.5;
	7075	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5771;
	7075	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10107.8;
	7075	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1645.8;
	7076	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.8;
	7076	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.2;
	7076	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	848.2;
	7076	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3697.2;
	7076	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10094.6;
	7076	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5578.2;
	7076	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9764.4;
	7076	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1587.8;
	7077	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.9;
	7077	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	713.3;
	7077	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	818.6;
	7077	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3519.5;
	7077	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9559.4;
	7077	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5311.9;
	7077	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9381.9;
	7077	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1558.8;
	7078	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901;
	7078	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	680.4;
	7078	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	779.8;
	7078	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3305.8;
	7078	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8922.7;
	7078	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4926.5;
	7078	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8839.6;
	7078	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1460;
	7079	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.4;
	7079	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	647.8;
	7079	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	733.9;
	7079	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3085.6;
	7079	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8235.5;
	7079	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4544.6;
	7079	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8255.4;
	7079	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1352.5;
	7080	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	860.5;
	7080	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	626.7;
	7080	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	738.8;
	7080	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2810.9;
	7080	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7755;
	7080	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4166.7;
	7080	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7627.4;
	7080	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1304.9;
	7081	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	841.9;
	7081	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	600.7;
	7081	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	708.4;
	7081	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2623.5;
	7081	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7298.8;
	7081	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4065.3;
	7081	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7234.2;
	7081	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1247.9;
	7082	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	837;
	7082	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	591.6;
	7082	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	694;
	7082	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2518.9;
	7082	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7064.3;
	7082	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3966.2;
	7082	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7035.4;
	7082	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1220.9;
	7083	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	821.6;
	7083	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	585.1;
	7083	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	688.8;
	7083	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2473.1;
	7083	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6972.4;
	7083	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3894.3;
	7083	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	6980.6;
	7083	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1189.9;
	7084	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	817.5;
	7084	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586.7;
	7084	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	687.1;
	7084	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2446.7;
	7084	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6970.6;
	7084	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3851.4;
	7084	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	6962.5;
	7084	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1202;
	7085	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	829.3;
	7085	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	601.5;
	7085	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	697.6;
	7085	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2475.2;
	7085	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7141.7;
	7085	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3927.4;
	7085	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7067.1;
	7085	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1233.3;
	7086	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	840.8;
	7086	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	621.6;
	7086	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	727.2;
	7086	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2567.4;
	7086	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7569;
	7086	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4141.8;
	7086	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7326.8;
	7086	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1284.5;
	7087	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	856.8;
	7087	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	641.3;
	7087	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	763.5;
	7087	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2637.1;
	7087	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7953.6;
	7087	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4432.4;
	7087	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7548;
	7087	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1341.1;
	7088	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	860.3;
	7088	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	656.6;
	7088	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	786.7;
	7088	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2724.3;
	7088	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8350.3;
	7088	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4634.4;
	7088	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7820.1;
	7088	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1398.2;
	7089	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871.6;
	7089	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	674.8;
	7089	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	804.8;
	7089	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2900.5;
	7089	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8818.4;
	7089	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4889;
	7089	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8198.5;
	7089	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1430.9;
	7090	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.3;
	7090	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	690.4;
	7090	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	815.8;
	7090	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3070.1;
	7090	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9126.1;
	7090	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5023.2;
	7090	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8477.9;
	7090	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1474.5;
	7091	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.1;
	7091	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	692.9;
	7091	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824;
	7091	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3230.6;
	7091	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9345.9;
	7091	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5089.9;
	7091	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8630.7;
	7091	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1503.9;
	7092	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.8;
	7092	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	699.1;
	7092	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	834;
	7092	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3420.1;
	7092	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9503.2;
	7092	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5177.7;
	7092	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8814.7;
	7092	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1498.8;
	7093	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.9;
	7093	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	706.3;
	7093	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.3;
	7093	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3638;
	7093	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9612.8;
	7093	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5283.9;
	7093	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9062;
	7093	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1533.7;
	7094	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.9;
	7094	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.6;
	7094	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878;
	7094	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3837;
	7094	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9779.7;
	7094	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5459.5;
	7094	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9334.4;
	7094	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1575.5;
	7095	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.3;
	7095	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.3;
	7095	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	898.8;
	7095	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3965.8;
	7095	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10036.7;
	7095	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5654.1;
	7095	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9605.8;
	7095	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1593;
	7096	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	968.6;
	7096	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755.6;
	7096	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	916.3;
	7096	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4006.2;
	7096	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10249.6;
	7096	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5791.5;
	7096	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9809.8;
	7096	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1625.7;
	7097	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.4;
	7097	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	760.7;
	7097	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	916.3;
	7097	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3907.5;
	7097	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10252.9;
	7097	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5834.7;
	7097	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9742.9;
	7097	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1629.7;
	7098	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.1;
	7098	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	748.6;
	7098	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	909.6;
	7098	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3741.8;
	7098	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10037.1;
	7098	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5717.2;
	7098	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9500.5;
	7098	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1582.1;
	7099	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.5;
	7099	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	748.1;
	7099	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	915.6;
	7099	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3773.2;
	7099	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10137.6;
	7099	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5744.4;
	7099	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9581.1;
	7099	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1617.2;
	7100	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.3;
	7100	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.1;
	7100	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.9;
	7100	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3676.6;
	7100	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9830.8;
	7100	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5561.9;
	7100	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9324.8;
	7100	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1573.3;
	7101	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.2;
	7101	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	709.5;
	7101	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.4;
	7101	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3527.4;
	7101	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9437.1;
	7101	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5326;
	7101	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8997.7;
	7101	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1451.8;
	7102	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.1;
	7102	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	682.2;
	7102	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	810.7;
	7102	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3339.2;
	7102	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8921.2;
	7102	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4972.1;
	7102	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8582;
	7102	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1413.7;
	7103	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.2;
	7103	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	648.4;
	7103	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	754.9;
	7103	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3132.4;
	7103	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8249.4;
	7103	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4596;
	7103	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8091.7;
	7103	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1356.9;
	7104	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	857.2;
	7104	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	620.1;
	7104	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	715.2;
	7104	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2932.5;
	7104	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7648.8;
	7104	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4280.6;
	7104	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7667.9;
	7104	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1299.3;
	7105	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	845.8;
	7105	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	596.8;
	7105	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	688.2;
	7105	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2789.8;
	7105	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7241.9;
	7105	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4089.7;
	7105	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7374.7;
	7105	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1247.3;
	7106	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	837.7;
	7106	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	583.1;
	7106	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	671.8;
	7106	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2687.9;
	7106	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6968.4;
	7106	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3938.5;
	7106	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7182.8;
	7106	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1203.8;
	7107	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	833.2;
	7107	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579.1;
	7107	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	662;
	7107	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2613.6;
	7107	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6815.3;
	7107	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3855.4;
	7107	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7076.8;
	7107	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1198.9;
	7108	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	833.1;
	7108	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	578.8;
	7108	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	660.3;
	7108	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2582.6;
	7108	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6799.9;
	7108	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3829.6;
	7108	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7056.2;
	7108	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1184.6;
	7109	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	834;
	7109	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.5;
	7109	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	663.5;
	7109	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2582.7;
	7109	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6883.5;
	7109	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3871.8;
	7109	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7124.1;
	7109	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1212.9;
	7110	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	837;
	7110	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	600.1;
	7110	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	681.9;
	7110	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2625.2;
	7110	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7099.7;
	7110	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4018.6;
	7110	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7268.2;
	7110	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1259.9;
	7111	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	845;
	7111	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	618.7;
	7111	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	709;
	7111	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2647.6;
	7111	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7388.6;
	7111	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4192.5;
	7111	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7376.1;
	7111	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1288.8;
	7112	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	847.6;
	7112	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	644;
	7112	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	732.1;
	7112	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2737;
	7112	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7768.3;
	7112	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4395.6;
	7112	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7610.6;
	7112	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1327.5;
	7113	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	862.5;
	7113	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	677.8;
	7113	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	762.6;
	7113	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3017.1;
	7113	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8341.6;
	7113	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4696.8;
	7113	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8045.9;
	7113	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1401.3;
	7114	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	876.8;
	7114	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	674.2;
	7114	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	778.5;
	7114	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3308.7;
	7114	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8753.8;
	7114	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4904.1;
	7114	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8442.5;
	7114	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1454.1;
	7115	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.5;
	7115	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	684.7;
	7115	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	791.2;
	7115	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3550.2;
	7115	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9075.7;
	7115	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5118.9;
	7115	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8835;
	7115	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1501.5;
	7116	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.6;
	7116	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711.1;
	7116	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	808.5;
	7116	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3782.7;
	7116	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9337.7;
	7116	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5331.9;
	7116	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9240.7;
	7116	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1556.8;
	7117	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.5;
	7117	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	732.5;
	7117	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	826;
	7117	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3983.8;
	7117	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9535.4;
	7117	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5584.2;
	7117	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9595.2;
	7117	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1591.2;
	7118	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.7;
	7118	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746;
	7118	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	840.8;
	7118	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4100.1;
	7118	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9730.7;
	7118	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5835.4;
	7118	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9897.6;
	7118	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1638.5;
	7119	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	978.2;
	7119	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	768.8;
	7119	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	855.3;
	7119	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4177.4;
	7119	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9901.9;
	7119	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6081.9;
	7119	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10108.6;
	7119	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1687.1;
	7120	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	997.5;
	7120	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777.1;
	7120	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878.9;
	7120	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4205.8;
	7120	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10041;
	7120	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6305.1;
	7120	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10297;
	7120	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1701.3;
	7121	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1006.8;
	7121	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776.4;
	7121	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	893.3;
	7121	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4145.6;
	7121	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10144.8;
	7121	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6410.1;
	7121	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10290.7;
	7121	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1709.2;
	7122	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	996;
	7122	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	782.4;
	7122	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	891;
	7122	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4078.4;
	7122	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10333;
	7122	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6387.3;
	7122	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10258.4;
	7122	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1697.3;
	7123	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.5;
	7123	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	805.4;
	7123	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913.2;
	7123	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4203.6;
	7123	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10770.4;
	7123	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6656.1;
	7123	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10570.4;
	7123	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1781.4;
	7124	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985.8;
	7124	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	788.1;
	7124	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	902.6;
	7124	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4118.2;
	7124	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10547.7;
	7124	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6503.6;
	7124	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10374.8;
	7124	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1737.1;
	7125	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.7;
	7125	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	764;
	7125	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	870.8;
	7125	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3956;
	7125	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10116.6;
	7125	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6212.9;
	7125	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9974.2;
	7125	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1637;
	7126	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.5;
	7126	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720;
	7126	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	817.3;
	7126	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3702.9;
	7126	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9357.9;
	7126	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5732.6;
	7126	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9317.4;
	7126	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1504.6;
	7127	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.1;
	7127	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	675;
	7127	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	760.1;
	7127	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3392.5;
	7127	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8502.6;
	7127	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5202.6;
	7127	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8608.6;
	7127	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1411.3;
	7128	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.2;
	7128	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	633;
	7128	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	719.1;
	7128	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3152.1;
	7128	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7847.8;
	7128	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4780;
	7128	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8091.1;
	7128	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1315.9;
	7129	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.7;
	7129	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	611.1;
	7129	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	692.6;
	7129	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2990.8;
	7129	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7462.9;
	7129	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4514.5;
	7129	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7783.9;
	7129	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1274.5;
	7130	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	857;
	7130	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	602.3;
	7130	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	680.1;
	7130	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2873.8;
	7130	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7263;
	7130	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4340.8;
	7130	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7646.4;
	7130	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1214.9;
	7131	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	853.4;
	7131	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	610.6;
	7131	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	674.8;
	7131	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2827.2;
	7131	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7235;
	7131	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4300.3;
	7131	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7658.9;
	7131	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1220;
	7132	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	852.2;
	7132	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	621.1;
	7132	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	677.1;
	7132	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2824.1;
	7132	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7419.5;
	7132	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4354.8;
	7132	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7838.2;
	7132	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1260.3;
	7133	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	860.8;
	7133	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	642.2;
	7133	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	705.1;
	7133	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2983.5;
	7133	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8045.5;
	7133	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4676.9;
	7133	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8359.4;
	7133	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1337.1;
	7134	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884;
	7134	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	704;
	7134	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	780.8;
	7134	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3333.4;
	7134	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9182.2;
	7134	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5309.5;
	7134	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9091.9;
	7134	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1487.8;
	7135	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.2;
	7135	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742.4;
	7135	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	823.5;
	7135	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3466.7;
	7135	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9808.9;
	7135	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5599.8;
	7135	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9341.4;
	7135	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1511.1;
	7136	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.4;
	7136	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.7;
	7136	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	816.6;
	7136	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3412.5;
	7136	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9793.8;
	7136	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5573.7;
	7136	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9396.8;
	7136	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1584.3;
	7137	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.3;
	7137	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	756;
	7137	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	839.2;
	7137	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3623.7;
	7137	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10107.2;
	7137	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5752.3;
	7137	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9864.1;
	7137	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1649.5;
	7138	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.2;
	7138	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	801.3;
	7138	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.9;
	7138	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3943.5;
	7138	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10635.2;
	7138	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5988.5;
	7138	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10419.5;
	7138	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1706.6;
	7139	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.5;
	7139	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	830.1;
	7139	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	893.5;
	7139	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4251.9;
	7139	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11152.3;
	7139	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6264.2;
	7139	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10981.1;
	7139	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1775.2;
	7140	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.1;
	7140	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	858.7;
	7140	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	925;
	7140	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4520.9;
	7140	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11727.6;
	7140	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6562.3;
	7140	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11490.8;
	7140	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1822.6;
	7141	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1003.5;
	7141	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	889.1;
	7141	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	964;
	7141	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4756.8;
	7141	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12305;
	7141	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6893.4;
	7141	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11919.6;
	7141	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1876.3;
	7142	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1027.8;
	7142	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	909.4;
	7142	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	992.1;
	7142	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4935.9;
	7142	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12776.2;
	7142	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7197.9;
	7142	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12279.7;
	7142	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1922.3;
	7143	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1046.4;
	7143	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	935.5;
	7143	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1022.4;
	7143	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5033;
	7143	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13141.3;
	7143	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7477.3;
	7143	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12559.2;
	7143	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1916.9;
	7144	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1058.7;
	7144	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	949.8;
	7144	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1042.5;
	7144	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5047.5;
	7144	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13403.6;
	7144	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7642.7;
	7144	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12644.2;
	7144	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1947.8;
	7145	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1053;
	7145	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	935.2;
	7145	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1031.5;
	7145	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4894.5;
	7145	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13335.6;
	7145	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7657.5;
	7145	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12380.8;
	7145	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1913.3;
	7146	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030.7;
	7146	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	906.9;
	7146	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1008.1;
	7146	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4658.6;
	7146	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13005.5;
	7146	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7537.6;
	7146	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11976.2;
	7146	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1919.8;
	7147	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1024.5;
	7147	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	914.3;
	7147	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1008.8;
	7147	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4678.4;
	7147	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13001.5;
	7147	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7585.3;
	7147	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11984.8;
	7147	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1960;
	7148	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1010.7;
	7148	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	885.7;
	7148	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	983.8;
	7148	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4562;
	7148	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12559.2;
	7148	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7337.9;
	7148	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11556.3;
	7148	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1908;
	7149	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990.7;
	7149	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	844;
	7149	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	933.1;
	7149	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4317.8;
	7149	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11783.9;
	7149	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6886;
	7149	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10966.1;
	7149	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1802.4;
	7150	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958;
	7150	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	786.7;
	7150	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.5;
	7150	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3961.9;
	7150	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10672.3;
	7150	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6249.1;
	7150	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10132.2;
	7150	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1671.2;
	7151	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924;
	7151	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723.3;
	7151	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	798.9;
	7151	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3553.3;
	7151	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9526.4;
	7151	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5554.1;
	7151	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9269.1;
	7151	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1517;
	7152	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.1;
	7152	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	692.4;
	7152	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	756.3;
	7152	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3263.3;
	7152	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8683.6;
	7152	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5060.4;
	7152	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8661.1;
	7152	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1404.9;
	7153	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.3;
	7153	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	670.3;
	7153	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	730.6;
	7153	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3064.9;
	7153	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8214.8;
	7153	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4730.3;
	7153	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8284.9;
	7153	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1318.8;
	7154	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.2;
	7154	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	632.2;
	7154	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	718.5;
	7154	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2940.4;
	7154	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7860.2;
	7154	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4513.8;
	7154	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8056.3;
	7154	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1194.6;
	7155	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.9;
	7155	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	619.1;
	7155	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	710.5;
	7155	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2868.3;
	7155	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7706.6;
	7155	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4386.3;
	7155	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7941.8;
	7155	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1182.2;
	7156	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	861.8;
	7156	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	628.8;
	7156	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	707;
	7156	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2839.8;
	7156	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7772.9;
	7156	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4406.1;
	7156	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8032.9;
	7156	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1241.3;
	7157	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.5;
	7157	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	646.1;
	7157	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	733.5;
	7157	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2952.1;
	7157	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8272.8;
	7157	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4713.2;
	7157	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8537.2;
	7157	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1309.9;
	7158	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.8;
	7158	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	708.6;
	7158	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	803.4;
	7158	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3303.4;
	7158	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9418;
	7158	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5377.3;
	7158	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9257.1;
	7158	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1421.4;
	7159	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.8;
	7159	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.3;
	7159	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	842.5;
	7159	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3405.9;
	7159	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9901.5;
	7159	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5689.3;
	7159	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9463.4;
	7159	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1487.6;
	7160	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.4;
	7160	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	739;
	7160	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.8;
	7160	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3357.2;
	7160	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9805.7;
	7160	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5665.2;
	7160	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9521.9;
	7160	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1577.4;
	7161	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925.7;
	7161	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	748.7;
	7161	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843;
	7161	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3564.6;
	7161	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10139.5;
	7161	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5810.7;
	7161	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10001.8;
	7161	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1582.3;
	7162	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.5;
	7162	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	775.1;
	7162	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868;
	7162	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3827.2;
	7162	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10629.6;
	7162	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6079.8;
	7162	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10575.8;
	7162	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1633.8;
	7163	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954;
	7163	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	800.9;
	7163	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	888.5;
	7163	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4101.7;
	7163	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11020.9;
	7163	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6347.7;
	7163	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11124.6;
	7163	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1744.9;
	7164	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.9;
	7164	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	827.3;
	7164	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	906;
	7164	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4335.1;
	7164	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11450.5;
	7164	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6604.4;
	7164	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11632.4;
	7164	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1798.7;
	7165	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	984.9;
	7165	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	855.9;
	7165	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	930.7;
	7165	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4511.8;
	7165	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11966;
	7165	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6881.5;
	7165	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12095.9;
	7165	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1871.8;
	7166	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1010.5;
	7166	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	873.5;
	7166	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	948.8;
	7166	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4644.8;
	7166	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12307.3;
	7166	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7114.2;
	7166	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12453;
	7166	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1884.6;
	7167	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1032.5;
	7167	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	876.2;
	7167	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	959.1;
	7167	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4633.6;
	7167	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12529.7;
	7167	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7273.9;
	7167	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12692.5;
	7167	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1927.6;
	7168	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1050.7;
	7168	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	879.9;
	7168	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	972.8;
	7168	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4635.5;
	7168	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12660.5;
	7168	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7368.8;
	7168	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12777;
	7168	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1922.5;
	7169	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1037.3;
	7169	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	863.9;
	7169	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	966.5;
	7169	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4515.9;
	7169	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12399.8;
	7169	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7329;
	7169	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12459.2;
	7169	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1879.1;
	7170	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1022.1;
	7170	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	851.7;
	7170	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	958.5;
	7170	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4370.1;
	7170	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12274;
	7170	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7233.5;
	7170	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12038.8;
	7170	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1881.8;
	7171	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1021.2;
	7171	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	867.1;
	7171	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	974.3;
	7171	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4467.9;
	7171	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12440.7;
	7171	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7315.1;
	7171	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12114.8;
	7171	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1907.2;
	7172	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.8;
	7172	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	849.2;
	7172	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	963.3;
	7172	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4380.7;
	7172	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12117.3;
	7172	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7095.3;
	7172	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11763.7;
	7172	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1852.9;
	7173	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989.6;
	7173	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	809.8;
	7173	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	918.7;
	7173	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4161.3;
	7173	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11444.7;
	7173	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6667.7;
	7173	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11221.2;
	7173	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1755.6;
	7174	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.1;
	7174	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	757.7;
	7174	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	856;
	7174	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3844.8;
	7174	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10433.5;
	7174	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6084.7;
	7174	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10425.3;
	7174	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1640.8;
	7175	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.8;
	7175	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	702.7;
	7175	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	792.7;
	7175	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3471.3;
	7175	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9371.8;
	7175	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5433.4;
	7175	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9607.6;
	7175	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1434.8;
	7176	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.9;
	7176	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	660.8;
	7176	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	752.5;
	7176	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3195.1;
	7176	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8532.9;
	7176	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4949.1;
	7176	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8988.6;
	7176	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1348;
	7177	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.8;
	7177	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	634.4;
	7177	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	726.9;
	7177	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2997;
	7177	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8069.8;
	7177	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4617.2;
	7177	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8549.8;
	7177	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1295.1;
	7178	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866.4;
	7178	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	619.3;
	7178	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	707.9;
	7178	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2883.3;
	7178	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7729.9;
	7178	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4413.8;
	7178	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8282.6;
	7178	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1231.9;
	7179	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	862.4;
	7179	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	614.2;
	7179	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	700.4;
	7179	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2830.3;
	7179	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7553;
	7179	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4298.2;
	7179	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8124.2;
	7179	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1252.5;
	7180	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.1;
	7180	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	618.8;
	7180	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	699.3;
	7180	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2824.4;
	7180	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7674.9;
	7180	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4325.4;
	7180	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8140.5;
	7180	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1240.4;
	7181	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.1;
	7181	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	642;
	7181	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	724.5;
	7181	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2963.4;
	7181	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8206.6;
	7181	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4619.7;
	7181	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8615.6;
	7181	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1328.2;
	7182	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893;
	7182	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	703.7;
	7182	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	791.3;
	7182	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3332.3;
	7182	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9353.6;
	7182	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5265.1;
	7182	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9344.7;
	7182	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1511.5;
	7183	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.1;
	7183	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	732.9;
	7183	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824.7;
	7183	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3435.5;
	7183	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9842.9;
	7183	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5544.8;
	7183	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9553.7;
	7183	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1571.8;
	7184	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.4;
	7184	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	731.7;
	7184	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	818.4;
	7184	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3401.4;
	7184	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9828.7;
	7184	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5486.6;
	7184	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9644.3;
	7184	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1581;
	7185	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.7;
	7185	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	756.7;
	7185	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	842.8;
	7185	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3614.8;
	7185	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10199;
	7185	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5713.5;
	7185	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10129.2;
	7185	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1632.9;
	7186	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.3;
	7186	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	787.4;
	7186	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	874.4;
	7186	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3887.3;
	7186	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10687.9;
	7186	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6054.8;
	7186	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10683.7;
	7186	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1700.9;
	7187	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.7;
	7187	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	819;
	7187	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	907.3;
	7187	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4172.1;
	7187	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11280.5;
	7187	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6439.1;
	7187	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11231.2;
	7187	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1769.3;
	7188	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	979.3;
	7188	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	852.6;
	7188	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	944.5;
	7188	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4442.6;
	7188	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11897.5;
	7188	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6772.5;
	7188	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11792.5;
	7188	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1837.8;
	7189	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.3;
	7189	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	885;
	7189	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	991.3;
	7189	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4714.9;
	7189	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12553.5;
	7189	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7145.8;
	7189	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12398.4;
	7189	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1911.8;
	7190	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1051.2;
	7190	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	921.5;
	7190	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1036.5;
	7190	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4881.1;
	7190	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13127;
	7190	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7488.9;
	7190	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12854.7;
	7190	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1991.9;
	7191	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1071;
	7191	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	945;
	7191	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1084.7;
	7191	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4988.8;
	7191	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13596.3;
	7191	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7774.5;
	7191	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13139.6;
	7191	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2037.4;
	7192	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1081.5;
	7192	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	960.2;
	7192	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1117.6;
	7192	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5040.4;
	7192	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13835;
	7192	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7976.2;
	7192	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13221.1;
	7192	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2066.5;
	7193	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1071.9;
	7193	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	943.7;
	7193	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1111.8;
	7193	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4905.2;
	7193	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13689.1;
	7193	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7973.5;
	7193	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12870.7;
	7193	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2020.6;
	7194	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1046.2;
	7194	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	921;
	7194	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1079;
	7194	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4669.6;
	7194	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13304.4;
	7194	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7744.2;
	7194	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12187.3;
	7194	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1943.4;
	7195	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1031.6;
	7195	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	926.2;
	7195	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1055.8;
	7195	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4648.9;
	7195	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13278.8;
	7195	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7656.2;
	7195	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12123.3;
	7195	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1931.6;
	7196	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1018.2;
	7196	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	894.1;
	7196	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1026.3;
	7196	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4520.2;
	7196	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12852.8;
	7196	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7289.6;
	7196	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11723.1;
	7196	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1863.1;
	7197	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	987.2;
	7197	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	846.4;
	7197	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	967.2;
	7197	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4273.3;
	7197	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12120.3;
	7197	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6815;
	7197	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11093.7;
	7197	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1754.6;
	7198	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952;
	7198	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	785.9;
	7198	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	895.9;
	7198	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3939;
	7198	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11076.2;
	7198	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6212;
	7198	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10257.2;
	7198	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1612.3;
	7199	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.3;
	7199	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	718.2;
	7199	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	821.5;
	7199	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3537.7;
	7199	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9863.7;
	7199	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5555.3;
	7199	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9326.6;
	7199	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1458;
	7200	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.4;
	7200	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	673.5;
	7200	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	776.4;
	7200	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3239.3;
	7200	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8914.9;
	7200	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5071;
	7200	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8646.5;
	7200	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1352.8;
	7201	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	863.3;
	7201	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	648.6;
	7201	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	747;
	7201	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3060;
	7201	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8322.6;
	7201	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4755.3;
	7201	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8228.7;
	7201	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1277.6;
	7202	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	851.2;
	7202	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	628.8;
	7202	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	727.9;
	7202	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2917.4;
	7202	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7950.5;
	7202	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4546.3;
	7202	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7938.9;
	7202	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1223;
	7203	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	846.6;
	7203	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	618;
	7203	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	715.5;
	7203	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2840.6;
	7203	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7736.9;
	7203	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4432.1;
	7203	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7777.6;
	7203	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1206.9;
	7204	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	846.8;
	7204	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	624.1;
	7204	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	713.6;
	7204	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2806.3;
	7204	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7797.8;
	7204	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4458.5;
	7204	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7945.5;
	7204	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1221.8;
	7205	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	860.2;
	7205	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	657.8;
	7205	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	742.6;
	7205	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2935.5;
	7205	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8336.2;
	7205	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4742.8;
	7205	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8439.4;
	7205	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1295.8;
	7206	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.7;
	7206	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720.9;
	7206	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	810.8;
	7206	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3312.1;
	7206	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9448.6;
	7206	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5401.4;
	7206	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9171;
	7206	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1463;
	7207	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.3;
	7207	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.8;
	7207	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	853.5;
	7207	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3430.9;
	7207	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9993.9;
	7207	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5671;
	7207	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9428;
	7207	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1533.2;
	7208	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.6;
	7208	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.7;
	7208	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	846.1;
	7208	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3393.9;
	7208	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9963.6;
	7208	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5639.4;
	7208	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9527.7;
	7208	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1610;
	7209	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.5;
	7209	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	778.3;
	7209	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.2;
	7209	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3636.7;
	7209	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10375.1;
	7209	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5890.5;
	7209	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10019;
	7209	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1693.6;
	7210	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.9;
	7210	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	818.1;
	7210	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	898.6;
	7210	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3934.7;
	7210	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11032.4;
	7210	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6278.5;
	7210	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10567.6;
	7210	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1789.3;
	7211	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	976;
	7211	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	848.5;
	7211	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940.8;
	7211	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4247.9;
	7211	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11729;
	7211	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6572.5;
	7211	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11109.6;
	7211	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1825.3;
	7212	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	996.2;
	7212	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	869.7;
	7212	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	988.3;
	7212	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4501.8;
	7212	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12511.1;
	7212	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6908.8;
	7212	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11673.3;
	7212	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1988.1;
	7213	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1024.6;
	7213	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	915.9;
	7213	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1034.4;
	7213	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4726.7;
	7213	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13352.7;
	7213	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7259.2;
	7213	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12247.8;
	7213	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2083.5;
	7214	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1053.3;
	7214	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	958.1;
	7214	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1067.5;
	7214	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4896.1;
	7214	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13984.4;
	7214	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7551.4;
	7214	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12701.3;
	7214	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2165.9;
	7215	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1070.5;
	7215	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	975.6;
	7215	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1100.1;
	7215	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4997.4;
	7215	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14369.2;
	7215	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7834.7;
	7215	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13026.6;
	7215	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2214.7;
	7216	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1084.5;
	7216	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	982.5;
	7216	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1121.5;
	7216	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5036;
	7216	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14533.6;
	7216	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8016.1;
	7216	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13171.7;
	7216	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2232.5;
	7217	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1070.4;
	7217	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	958.7;
	7217	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1097.6;
	7217	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4867.6;
	7217	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14319.5;
	7217	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7975.4;
	7217	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12815.4;
	7217	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2166.2;
	7218	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1046.3;
	7218	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	920.4;
	7218	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1053.3;
	7218	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4607.8;
	7218	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13737.6;
	7218	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7696.6;
	7218	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12291.9;
	7218	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2080.3;
	7219	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1041.9;
	7219	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	923.7;
	7219	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1042.7;
	7219	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4592.3;
	7219	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13577.8;
	7219	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7598.3;
	7219	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12174;
	7219	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2061.3;
	7220	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030.4;
	7220	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	894.9;
	7220	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1019.1;
	7220	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4472.8;
	7220	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13048.9;
	7220	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7259.5;
	7220	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11757.1;
	7220	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1994.4;
	7221	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1003.7;
	7221	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	845.8;
	7221	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	955.7;
	7221	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4245.3;
	7221	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12245.1;
	7221	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6769.4;
	7221	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11154.2;
	7221	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1874.7;
	7222	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969.1;
	7222	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	787.1;
	7222	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	880.8;
	7222	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3918.6;
	7222	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11158.5;
	7222	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6159.1;
	7222	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10312.6;
	7222	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1718;
	7223	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.7;
	7223	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.4;
	7223	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	810.7;
	7223	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3540;
	7223	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10005.5;
	7223	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5479.9;
	7223	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9499.1;
	7223	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1553.3;
	7224	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.7;
	7224	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	685.6;
	7224	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	765.3;
	7224	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3233.2;
	7224	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9087.9;
	7224	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4976.1;
	7224	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8866.4;
	7224	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1428.6;
	7225	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.6;
	7225	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	655.4;
	7225	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	733.4;
	7225	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3032.6;
	7225	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8474.4;
	7225	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4632.9;
	7225	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8460.3;
	7225	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1369.1;
	7226	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.6;
	7226	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	642.3;
	7226	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	712.6;
	7226	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2921.4;
	7226	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8115.8;
	7226	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4404.9;
	7226	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8258.7;
	7226	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1230.4;
	7227	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.6;
	7227	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	632.1;
	7227	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	702.6;
	7227	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2847.3;
	7227	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7915.9;
	7227	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4293.2;
	7227	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8170.1;
	7227	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1210.3;
	7228	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.3;
	7228	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	640.6;
	7228	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	698.4;
	7228	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2823.2;
	7228	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7966.8;
	7228	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4315.1;
	7228	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8277.4;
	7228	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1209;
	7229	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.5;
	7229	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	657.5;
	7229	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	722.6;
	7229	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2928.3;
	7229	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8425.6;
	7229	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4596;
	7229	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8769.9;
	7229	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1330.6;
	7230	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.8;
	7230	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.5;
	7230	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	792.2;
	7230	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3286.3;
	7230	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9426.5;
	7230	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5210.4;
	7230	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9526.1;
	7230	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1495.2;
	7231	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.8;
	7231	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749.2;
	7231	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	821.1;
	7231	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3398.9;
	7231	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9938.4;
	7231	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5472.1;
	7231	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9787.6;
	7231	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1527.7;
	7232	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.4;
	7232	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	750.7;
	7232	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	813.5;
	7232	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3349.1;
	7232	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9898.3;
	7232	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5456.3;
	7232	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9872;
	7232	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1575;
	7233	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.8;
	7233	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772.3;
	7233	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	845.9;
	7233	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3545.5;
	7233	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10367.1;
	7233	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5685.2;
	7233	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10414.7;
	7233	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1642.2;
	7234	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	968.8;
	7234	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	813;
	7234	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884.2;
	7234	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3858.2;
	7234	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10948.2;
	7234	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5990.6;
	7234	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11121.1;
	7234	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1752.3;
	7235	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	988.3;
	7235	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	849.2;
	7235	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	917.4;
	7235	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4158.1;
	7235	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11581;
	7235	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6318.5;
	7235	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11709.8;
	7235	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1780.7;
	7236	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1005.2;
	7236	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	880.3;
	7236	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	951.9;
	7236	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4413.9;
	7236	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12180;
	7236	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6623.7;
	7236	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12220.4;
	7236	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1865.7;
	7237	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1028.4;
	7237	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	920.8;
	7237	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	992.8;
	7237	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4668;
	7237	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12811.9;
	7237	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6910.4;
	7237	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12801.6;
	7237	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1938.8;
	7238	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1044.5;
	7238	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	947.5;
	7238	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1021.2;
	7238	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4814.4;
	7238	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13289.9;
	7238	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7244.5;
	7238	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13185.9;
	7238	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2061.6;
	7239	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1057.1;
	7239	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	957.8;
	7239	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1043.8;
	7239	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4855.3;
	7239	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13684.1;
	7239	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7523.4;
	7239	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13476;
	7239	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2099.7;
	7240	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1057.3;
	7240	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	958.7;
	7240	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1059.3;
	7240	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4909.7;
	7240	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13829.7;
	7240	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7726.3;
	7240	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13497.2;
	7240	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2160.1;
	7241	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030.8;
	7241	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	931.4;
	7241	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1037.7;
	7241	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4786;
	7241	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13541.1;
	7241	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7697.3;
	7241	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12987.8;
	7241	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2104.2;
	7242	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	998;
	7242	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	890.7;
	7242	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	992.9;
	7242	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4520.1;
	7242	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12852.3;
	7242	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7361.8;
	7242	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12246.1;
	7242	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2005;
	7243	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	997.9;
	7243	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	873.9;
	7243	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	970.6;
	7243	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4477;
	7243	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12458;
	7243	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7153.4;
	7243	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11982.6;
	7243	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1942.1;
	7244	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	982.7;
	7244	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	834.6;
	7244	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	931.5;
	7244	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4317;
	7244	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11866.9;
	7244	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6823.9;
	7244	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11424.7;
	7244	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1843.8;
	7245	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969.7;
	7245	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	796.1;
	7245	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	888.1;
	7245	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4124.9;
	7245	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11198.4;
	7245	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6457.3;
	7245	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10917.5;
	7245	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1784.1;
	7246	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.8;
	7246	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.6;
	7246	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	842.2;
	7246	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3887.6;
	7246	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10434.8;
	7246	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6044.7;
	7246	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10277;
	7246	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1667.5;
	7247	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.2;
	7247	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	710.2;
	7247	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	789;
	7247	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3618.7;
	7247	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9592.5;
	7247	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5589;
	7247	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9600.7;
	7247	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1525.8;
	7248	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.8;
	7248	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	667.8;
	7248	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	774.1;
	7248	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3331.7;
	7248	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8724.8;
	7248	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5144.4;
	7248	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8986.1;
	7248	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1433;
	7249	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.4;
	7249	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	640.8;
	7249	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	741.6;
	7249	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3124.3;
	7249	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8168.3;
	7249	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4804.9;
	7249	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8595.8;
	7249	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1360;
	7250	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896;
	7250	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	622.9;
	7250	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	723.1;
	7250	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2973.7;
	7250	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7807;
	7250	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4577.3;
	7250	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8331.3;
	7250	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1251.1;
	7251	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.2;
	7251	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	609.4;
	7251	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	711.4;
	7251	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2873.5;
	7251	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7590.7;
	7251	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4409.6;
	7251	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8178;
	7251	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1202.8;
	7252	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.4;
	7252	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	607.2;
	7252	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	706.7;
	7252	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2836.1;
	7252	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7543.8;
	7252	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4352.5;
	7252	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8146.2;
	7252	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1178.5;
	7253	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.1;
	7253	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	614.8;
	7253	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	712.9;
	7253	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2861.5;
	7253	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7691.7;
	7253	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4420.9;
	7253	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8262;
	7253	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1249.1;
	7254	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.2;
	7254	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	633.4;
	7254	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	735.1;
	7254	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2950.5;
	7254	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8036.6;
	7254	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4637.8;
	7254	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8525.6;
	7254	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1285.9;
	7255	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.5;
	7255	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	650.8;
	7255	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	764.2;
	7255	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3027.3;
	7255	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8415;
	7255	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4934.9;
	7255	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8766.1;
	7255	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1340.1;
	7256	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.4;
	7256	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	665.6;
	7256	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	785.5;
	7256	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3127.3;
	7256	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8821.6;
	7256	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5188.1;
	7256	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9077.5;
	7256	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1445.6;
	7257	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.9;
	7257	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	702.4;
	7257	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824.3;
	7257	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3417.5;
	7257	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9522.7;
	7257	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5463;
	7257	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9736;
	7257	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1513.3;
	7258	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.6;
	7258	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.1;
	7258	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.1;
	7258	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3777.6;
	7258	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10241.7;
	7258	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5772;
	7258	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10507.7;
	7258	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1616.1;
	7259	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.5;
	7259	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	771.3;
	7259	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	900.2;
	7259	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4126.9;
	7259	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10918.7;
	7259	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6102.2;
	7259	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11169.4;
	7259	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1708.2;
	7260	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	987.1;
	7260	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	800.8;
	7260	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940;
	7260	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4422.7;
	7260	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11554.4;
	7260	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6456.7;
	7260	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11706;
	7260	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1796.1;
	7261	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1010.2;
	7261	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	834.5;
	7261	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	983.4;
	7261	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4667;
	7261	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12097.6;
	7261	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6814.6;
	7261	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12199.8;
	7261	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1889;
	7262	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1029;
	7262	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	863;
	7262	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1016.1;
	7262	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4828.4;
	7262	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12573.5;
	7262	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7143.2;
	7262	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12592.3;
	7262	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1994.2;
	7263	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1041.4;
	7263	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	883.5;
	7263	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1047;
	7263	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4914.7;
	7263	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12909;
	7263	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7405.2;
	7263	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12851.2;
	7263	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2032.8;
	7264	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1050.2;
	7264	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	890.7;
	7264	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1066.9;
	7264	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4915.5;
	7264	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12998.6;
	7264	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7586.5;
	7264	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12852.8;
	7264	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2057.4;
	7265	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1047.4;
	7265	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	873.4;
	7265	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1053.2;
	7265	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4762.7;
	7265	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12684.8;
	7265	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7493.1;
	7265	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12465.1;
	7265	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2029.6;
	7266	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1026.2;
	7266	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	844.4;
	7266	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1017.1;
	7266	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4517.5;
	7266	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12189.6;
	7266	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7172;
	7266	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11897.5;
	7266	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1960.2;
	7267	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1016.3;
	7267	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	836.4;
	7267	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1000.2;
	7267	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4470.5;
	7267	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12078.3;
	7267	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7029.2;
	7267	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11748.7;
	7267	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1923.9;
	7268	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.3;
	7268	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	807.5;
	7268	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	966.5;
	7268	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4285.3;
	7268	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11574.7;
	7268	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6703.6;
	7268	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11285.4;
	7268	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1799.6;
	7269	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	983.5;
	7269	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	774.3;
	7269	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	925.2;
	7269	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4064.4;
	7269	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11004.4;
	7269	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6341.2;
	7269	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10769.6;
	7269	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1751.6;
	7270	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965;
	7270	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735.3;
	7270	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	875.2;
	7270	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3822.5;
	7270	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10291.4;
	7270	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5913.8;
	7270	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10214.4;
	7270	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1693.6;
	7271	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.4;
	7271	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	693.3;
	7271	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	818.4;
	7271	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3550.6;
	7271	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9475.7;
	7271	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5493.3;
	7271	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9568.3;
	7271	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1584.3;
	7272	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.2;
	7272	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	655.9;
	7272	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	771.6;
	7272	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3306.7;
	7272	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8739.9;
	7272	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5090;
	7272	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9013.1;
	7272	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1469.4;
	7273	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.4;
	7273	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	628.7;
	7273	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	739.1;
	7273	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3120.3;
	7273	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8174.9;
	7273	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4778.1;
	7273	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8605.6;
	7273	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1387.6;
	7274	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.9;
	7274	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	615.9;
	7274	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	717.8;
	7274	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2978.3;
	7274	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7790.7;
	7274	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4570.8;
	7274	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8303.3;
	7274	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1343.1;
	7275	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.3;
	7275	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	601.8;
	7275	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	705.1;
	7275	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2878.2;
	7275	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7533.3;
	7275	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4413.5;
	7275	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8095.4;
	7275	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1321.8;
	7276	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.9;
	7276	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	595.5;
	7276	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	698;
	7276	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2816.8;
	7276	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7393.6;
	7276	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4333;
	7276	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7995.6;
	7276	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1281.9;
	7277	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.6;
	7277	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	599.7;
	7277	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	699.1;
	7277	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2797.4;
	7277	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7409.4;
	7277	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4338;
	7277	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8014.1;
	7277	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1275.5;
	7278	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.6;
	7278	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	610.2;
	7278	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	717.3;
	7278	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2818.3;
	7278	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7580.8;
	7278	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4453.1;
	7278	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8098.1;
	7278	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1234.2;
	7279	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.9;
	7279	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	636.8;
	7279	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	744.2;
	7279	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2850.8;
	7279	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7873;
	7279	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4605.9;
	7279	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8203.5;
	7279	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1377.1;
	7280	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.4;
	7280	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	650.1;
	7280	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	766.3;
	7280	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2937.8;
	7280	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8299.4;
	7280	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4844.3;
	7280	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8458;
	7280	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1413;
	7281	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.3;
	7281	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	692.7;
	7281	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	805.7;
	7281	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3256.8;
	7281	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9089.4;
	7281	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5275.9;
	7281	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9012;
	7281	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1508.5;
	7282	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.2;
	7282	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.8;
	7282	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	845.2;
	7282	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3595.4;
	7282	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9944.8;
	7282	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5726;
	7282	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9670.6;
	7282	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1583;
	7283	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948.7;
	7283	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	769.9;
	7283	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.2;
	7283	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3941.6;
	7283	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10826.6;
	7283	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6146.8;
	7283	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10431;
	7283	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1675.8;
	7284	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981;
	7284	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	813.9;
	7284	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	945.5;
	7284	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4254.1;
	7284	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11731.8;
	7284	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6655.3;
	7284	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11114.6;
	7284	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1797.8;
	7285	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1010.2;
	7285	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	860.1;
	7285	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	999.6;
	7285	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4512.4;
	7285	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12609.2;
	7285	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7073.5;
	7285	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11777.9;
	7285	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1916.4;
	7286	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1036.7;
	7286	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	900.6;
	7286	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1042.6;
	7286	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4694.8;
	7286	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13307.6;
	7286	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7478.8;
	7286	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12316.4;
	7286	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2010.3;
	7287	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1056.1;
	7287	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	932.7;
	7287	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1070.2;
	7287	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4796.4;
	7287	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13832.4;
	7287	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7772.3;
	7287	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12692.5;
	7287	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2057;
	7288	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1067.2;
	7288	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	945.4;
	7288	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1087.7;
	7288	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4809.6;
	7288	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14066.3;
	7288	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7894.5;
	7288	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12775.9;
	7288	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2107.2;
	7289	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1063.3;
	7289	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	932.7;
	7289	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1081.9;
	7289	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4696.2;
	7289	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13908.3;
	7289	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7844.9;
	7289	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12499.3;
	7289	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2083.2;
	7290	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1044.2;
	7290	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	898.3;
	7290	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1047;
	7290	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4506.4;
	7290	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13361.7;
	7290	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7562.1;
	7290	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12009;
	7290	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1979;
	7291	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1037.9;
	7291	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	889.3;
	7291	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1033.9;
	7291	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4514.4;
	7291	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13217.9;
	7291	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7449.5;
	7291	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11873.7;
	7291	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1971.6;
	7292	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1017.8;
	7292	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	850;
	7292	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	992.3;
	7292	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4375.4;
	7292	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12579.9;
	7292	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7024.6;
	7292	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11367.1;
	7292	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1854.7;
	7293	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.7;
	7293	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	807.2;
	7293	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	945.4;
	7293	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4185;
	7293	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11871.7;
	7293	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6572.5;
	7293	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10765.5;
	7293	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1730.8;
	7294	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.5;
	7294	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749.4;
	7294	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878.2;
	7294	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3895.7;
	7294	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10824.1;
	7294	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5963.7;
	7294	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9963;
	7294	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1555.8;
	7295	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.6;
	7295	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	697.2;
	7295	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	818.9;
	7295	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3595;
	7295	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9791.4;
	7295	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5379.1;
	7295	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9194;
	7295	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1483;
	7296	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.2;
	7296	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	651.1;
	7296	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	764;
	7296	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3303.3;
	7296	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8877.7;
	7296	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4844.5;
	7296	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8566.6;
	7296	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1381.1;
	7297	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.9;
	7297	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	625;
	7297	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	729.1;
	7297	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3110.3;
	7297	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8267.7;
	7297	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4509.8;
	7297	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8185.8;
	7297	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1309;
	7298	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.1;
	7298	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	613.6;
	7298	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	712.6;
	7298	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2980.1;
	7298	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7921.8;
	7298	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4308.9;
	7298	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7999.2;
	7298	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1261.1;
	7299	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884;
	7299	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	609.2;
	7299	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	705.9;
	7299	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2911.5;
	7299	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7756.3;
	7299	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4217.4;
	7299	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7965.4;
	7299	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1259.4;
	7300	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.4;
	7300	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	615.8;
	7300	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	708.8;
	7300	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2902.9;
	7300	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7857.7;
	7300	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4289.1;
	7300	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8139.1;
	7300	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1265.3;
	7301	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.7;
	7301	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	645.9;
	7301	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	736.1;
	7301	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3040.5;
	7301	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8396.4;
	7301	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4647;
	7301	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8618.2;
	7301	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1343.4;
	7302	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.8;
	7302	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	703.8;
	7302	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	807.9;
	7302	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3389.5;
	7302	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9472;
	7302	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5299.5;
	7302	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9314.1;
	7302	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1520.8;
	7303	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.5;
	7303	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.7;
	7303	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	853.8;
	7303	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3500.1;
	7303	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9987.4;
	7303	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5617.7;
	7303	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9570.1;
	7303	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1585.7;
	7304	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.5;
	7304	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.7;
	7304	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	844.8;
	7304	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3431.6;
	7304	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9940.7;
	7304	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5590.3;
	7304	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9593;
	7304	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1585.7;
	7305	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.2;
	7305	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	767.8;
	7305	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866;
	7305	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3597.2;
	7305	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10393.5;
	7305	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5767;
	7305	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9962.1;
	7305	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1625.9;
	7306	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.2;
	7306	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	806.5;
	7306	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	899.6;
	7306	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3871.6;
	7306	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11096.6;
	7306	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6014.9;
	7306	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10489.8;
	7306	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1728.8;
	7307	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	984.8;
	7307	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	844.2;
	7307	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	933.7;
	7307	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4165;
	7307	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11751.6;
	7307	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6288.1;
	7307	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11104.5;
	7307	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1793.7;
	7308	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1001.1;
	7308	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	879.3;
	7308	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	965.1;
	7308	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4452.1;
	7308	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12398.6;
	7308	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6560.1;
	7308	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11675.6;
	7308	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1879.3;
	7309	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030.1;
	7309	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	916.5;
	7309	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1008.2;
	7309	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4746.8;
	7309	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13057.7;
	7309	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6871.8;
	7309	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12289.3;
	7309	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1966.8;
	7310	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1056.6;
	7310	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	937.4;
	7310	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1037.1;
	7310	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4979.7;
	7310	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13559.1;
	7310	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7139.8;
	7310	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12836.5;
	7310	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2070;
	7311	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1072.1;
	7311	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	953.7;
	7311	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1075.5;
	7311	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5132.6;
	7311	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13875.1;
	7311	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7461.2;
	7311	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13190.5;
	7311	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2111.3;
	7312	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1075.7;
	7312	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	961.2;
	7312	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1096.7;
	7312	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5203.5;
	7312	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14015.1;
	7312	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7697.7;
	7312	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13296.9;
	7312	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2120.4;
	7313	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1062.4;
	7313	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	929.2;
	7313	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1076.4;
	7313	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5075.3;
	7313	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13693.2;
	7313	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7705;
	7313	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12928;
	7313	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2084.7;
	7314	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1032.8;
	7314	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	883.1;
	7314	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1022;
	7314	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4762.5;
	7314	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12902.6;
	7314	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7405.5;
	7314	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12299.6;
	7314	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2000.8;
	7315	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1020.4;
	7315	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	863.3;
	7315	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	998.1;
	7315	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4632.1;
	7315	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12397.4;
	7315	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7188.4;
	7315	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12030.5;
	7315	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1867.1;
	7316	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1008.1;
	7316	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	842.8;
	7316	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	972.4;
	7316	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4510.3;
	7316	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12024.4;
	7316	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7026.6;
	7316	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11682.4;
	7316	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1810.4;
	7317	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	998.7;
	7317	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	817.3;
	7317	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	947.7;
	7317	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4400.5;
	7317	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11696.5;
	7317	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6859.1;
	7317	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11337.5;
	7317	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1829;
	7318	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.4;
	7318	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	765.9;
	7318	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889.8;
	7318	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4141.8;
	7318	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10829.5;
	7318	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6337.9;
	7318	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10673.6;
	7318	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1671;
	7319	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.5;
	7319	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	706.7;
	7319	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	819.8;
	7319	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3779.8;
	7319	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9830.5;
	7319	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5767.3;
	7319	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9869.1;
	7319	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1540.9;
	7320	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.3;
	7320	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	681.1;
	7320	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	778.2;
	7320	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3469.5;
	7320	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9033.4;
	7320	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5325.3;
	7320	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9270;
	7320	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1437.8;
	7321	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.2;
	7321	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	664.9;
	7321	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	751.8;
	7321	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3291.5;
	7321	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8614.6;
	7321	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5057.4;
	7321	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8934;
	7321	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1367.3;
	7322	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.8;
	7322	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	653.6;
	7322	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	743.3;
	7322	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3188.6;
	7322	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8372.2;
	7322	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4919.1;
	7322	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8751.9;
	7322	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1241.3;
	7323	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.1;
	7323	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	644.6;
	7323	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	735.6;
	7323	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3116.9;
	7323	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8245.7;
	7323	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4833.4;
	7323	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8658;
	7323	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1210.8;
	7324	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.1;
	7324	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	650.1;
	7324	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	734.2;
	7324	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3090.4;
	7324	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8345.9;
	7324	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4884.1;
	7324	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8735;
	7324	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1239.7;
	7325	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.7;
	7325	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	667.9;
	7325	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	757.2;
	7325	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3210;
	7325	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8961.6;
	7325	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5233.7;
	7325	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9221.8;
	7325	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1353.4;
	7326	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.5;
	7326	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.1;
	7326	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	827.8;
	7326	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3576.4;
	7326	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10125.9;
	7326	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5904.4;
	7326	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10018.1;
	7326	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1533.1;
	7327	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.1;
	7327	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	766.8;
	7327	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	871.6;
	7327	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3694;
	7327	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10701.6;
	7327	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6242.8;
	7327	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10336;
	7327	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1619.3;
	7328	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.9;
	7328	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	766.3;
	7328	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.4;
	7328	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3659.1;
	7328	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10676;
	7328	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6188.7;
	7328	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10451.8;
	7328	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1636.7;
	7329	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.9;
	7329	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	794.4;
	7329	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889;
	7329	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3907.7;
	7329	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11007.2;
	7329	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6357.8;
	7329	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11020.1;
	7329	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1668.7;
	7330	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.3;
	7330	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	825.2;
	7330	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	927.5;
	7330	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4204;
	7330	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11475.5;
	7330	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6648.9;
	7330	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11722.9;
	7330	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1772.5;
	7331	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.7;
	7331	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	855.5;
	7331	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	967.2;
	7331	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4508;
	7331	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11943.5;
	7331	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7023.7;
	7331	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12295.3;
	7331	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1871.9;
	7332	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981.4;
	7332	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	887.7;
	7332	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1007.5;
	7332	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4808.9;
	7332	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12454.8;
	7332	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7437.1;
	7332	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12442;
	7332	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1939.8;
	7333	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1021.4;
	7333	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	924.6;
	7333	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1057.7;
	7333	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5084.9;
	7333	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13050.9;
	7333	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7821;
	7333	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12400.7;
	7333	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2019.7;
	7334	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1052.4;
	7334	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	957.4;
	7334	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1100.8;
	7334	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5283.2;
	7334	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13688;
	7334	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8173.8;
	7334	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12370.1;
	7334	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2072.2;
	7335	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1076.5;
	7335	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	977.4;
	7335	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1139.3;
	7335	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5408.2;
	7335	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14173;
	7335	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8447.2;
	7335	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12439.5;
	7335	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2016.7;
	7336	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1086.1;
	7336	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	993.2;
	7336	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1162.3;
	7336	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5442.5;
	7336	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14445.4;
	7336	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8613.4;
	7336	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12508.6;
	7336	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2074.4;
	7337	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1072.9;
	7337	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	979.8;
	7337	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1150.8;
	7337	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5320.7;
	7337	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14360.3;
	7337	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8609;
	7337	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12400.6;
	7337	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2005.4;
	7338	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1054.9;
	7338	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	960.1;
	7338	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1111.2;
	7338	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5109.5;
	7338	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14025.7;
	7338	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8395.5;
	7338	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12365;
	7338	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2026.3;
	7339	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1047.3;
	7339	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	956.9;
	7339	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1095.8;
	7339	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5118.8;
	7339	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14032.9;
	7339	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8306.7;
	7339	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12507.3;
	7339	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2039.5;
	7340	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1034;
	7340	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	925;
	7340	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1060.6;
	7340	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4976.6;
	7340	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13624.7;
	7340	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8029.3;
	7340	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12286.9;
	7340	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1976.9;
	7341	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1005.3;
	7341	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	878.9;
	7341	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1004.9;
	7341	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4732.5;
	7341	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12862.1;
	7341	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7539.3;
	7341	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11771.4;
	7341	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1902.6;
	7342	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	977.4;
	7342	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	829.1;
	7342	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	939.8;
	7342	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4411;
	7342	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11794.5;
	7342	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6947.9;
	7342	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11027.6;
	7342	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1755.9;
	7343	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.3;
	7343	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	774.4;
	7343	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	871.9;
	7343	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4049.3;
	7343	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10654.7;
	7343	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6287.8;
	7343	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10153.1;
	7343	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1602.5;
	7344	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.6;
	7344	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735.3;
	7344	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.4;
	7344	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3751.2;
	7344	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9841.6;
	7344	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5765;
	7344	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9467.1;
	7344	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1469.8;
	7345	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.7;
	7345	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	708.6;
	7345	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	805.8;
	7345	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3537.2;
	7345	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9343.7;
	7345	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5479.3;
	7345	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9034.5;
	7345	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1434;
	7346	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.4;
	7346	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	694.2;
	7346	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	785.4;
	7346	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3411.2;
	7346	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9075.8;
	7346	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5274.4;
	7346	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8786.1;
	7346	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1362.3;
	7347	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.9;
	7347	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	688.3;
	7347	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	775.4;
	7347	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3326.1;
	7347	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8940.3;
	7347	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5146.5;
	7347	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8660.6;
	7347	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1322.8;
	7348	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.3;
	7348	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	693.3;
	7348	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	775.1;
	7348	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3295;
	7348	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9084;
	7348	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5214.3;
	7348	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8758.5;
	7348	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1302.7;
	7349	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.7;
	7349	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719.8;
	7349	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	799.3;
	7349	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3403.5;
	7349	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9672.1;
	7349	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5521.1;
	7349	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9243;
	7349	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1367.7;
	7350	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.5;
	7350	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	785.9;
	7350	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	871.8;
	7350	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3762.8;
	7350	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10798.2;
	7350	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6201;
	7350	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10002.9;
	7350	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1543.5;
	7351	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954;
	7351	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	818.4;
	7351	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	916.2;
	7351	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3876.1;
	7351	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11338.5;
	7351	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6545.2;
	7351	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10308.7;
	7351	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1616;
	7352	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.8;
	7352	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	819.2;
	7352	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	911.1;
	7352	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3818.8;
	7352	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11225.7;
	7352	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6487.4;
	7352	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10355.3;
	7352	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1611.2;
	7353	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.6;
	7353	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	847.1;
	7353	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	938;
	7353	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4075.4;
	7353	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11527.7;
	7353	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6616.8;
	7353	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10990.8;
	7353	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1667.3;
	7354	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	977;
	7354	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	878.8;
	7354	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	967.6;
	7354	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4352.9;
	7354	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11962.2;
	7354	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6873;
	7354	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11725.3;
	7354	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1752.9;
	7355	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	993.1;
	7355	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	898.2;
	7355	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	999.9;
	7355	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4593.3;
	7355	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12352.7;
	7355	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7149.9;
	7355	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12389.2;
	7355	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1862.6;
	7356	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1005.2;
	7356	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	920.6;
	7356	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1030.7;
	7356	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4811.9;
	7356	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12797.8;
	7356	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7429.3;
	7356	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12939;
	7356	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1962.7;
	7357	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1025.3;
	7357	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	951.9;
	7357	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1064.6;
	7357	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5061.5;
	7357	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13390.4;
	7357	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7767.5;
	7357	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13446.2;
	7357	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2049.5;
	7358	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1042.9;
	7358	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	970.3;
	7358	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1089.4;
	7358	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5245.1;
	7358	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13876.7;
	7358	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8080.5;
	7358	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13706.3;
	7358	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2101.7;
	7359	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1044.5;
	7359	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	981.8;
	7359	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1112.8;
	7359	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5337.3;
	7359	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14034.8;
	7359	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8281.9;
	7359	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13692.5;
	7359	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2133.2;
	7360	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1045.2;
	7360	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	981.3;
	7360	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1126.6;
	7360	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5381.4;
	7360	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13977.8;
	7360	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8390.5;
	7360	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13554.1;
	7360	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2156.2;
	7361	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030.6;
	7361	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	962;
	7361	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1107.4;
	7361	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5248.3;
	7361	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13681.9;
	7361	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8406.8;
	7361	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13148.6;
	7361	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2117.7;
	7362	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1021.9;
	7362	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	940.6;
	7362	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1085.7;
	7362	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5086.4;
	7362	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13526.3;
	7362	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8280.1;
	7362	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12848.1;
	7362	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2087.2;
	7363	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1031.4;
	7363	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	936.7;
	7363	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1085.3;
	7363	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5125.3;
	7363	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13520.7;
	7363	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8287.9;
	7363	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12973.4;
	7363	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2060.1;
	7364	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1020.1;
	7364	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	903.1;
	7364	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1058;
	7364	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4995.5;
	7364	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13063.3;
	7364	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8036.2;
	7364	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12655.3;
	7364	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1985.5;
	7365	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	996.1;
	7365	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	856.9;
	7365	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1010.8;
	7365	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4780.2;
	7365	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12406.3;
	7365	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7610.9;
	7365	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12141;
	7365	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1877.9;
	7366	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.8;
	7366	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	795;
	7366	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	946;
	7366	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4452.2;
	7366	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11448.1;
	7366	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6995.7;
	7366	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11363.3;
	7366	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1739.2;
	7367	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.8;
	7367	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.1;
	7367	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.8;
	7367	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4091.5;
	7367	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10412.5;
	7367	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6370.8;
	7367	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10548.2;
	7367	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1577.7;
	7368	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.3;
	7368	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	690.6;
	7368	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	790.3;
	7368	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3798.4;
	7368	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9513.3;
	7368	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5846.7;
	7368	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9910.6;
	7368	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1472.7;
	7369	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.6;
	7369	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	663.3;
	7369	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	756.2;
	7369	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3565.7;
	7369	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8890;
	7369	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5484.6;
	7369	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9400.1;
	7369	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1401.8;
	7370	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.4;
	7370	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	643.9;
	7370	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	737.2;
	7370	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3400.3;
	7370	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8436.3;
	7370	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5274.2;
	7370	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9073.5;
	7370	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1349.8;
	7371	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.8;
	7371	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	629.1;
	7371	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	726.4;
	7371	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3276.4;
	7371	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8197.3;
	7371	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5147.1;
	7371	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8902.8;
	7371	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1326.6;
	7372	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	876;
	7372	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	634.4;
	7372	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	728.7;
	7372	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3225.5;
	7372	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8244.2;
	7372	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5118.6;
	7372	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8994;
	7372	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1330.7;
	7373	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.9;
	7373	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	664.4;
	7373	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	762.5;
	7373	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3342.3;
	7373	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8829.2;
	7373	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5411.7;
	7373	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9497.3;
	7373	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1414.1;
	7374	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.1;
	7374	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.3;
	7374	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	828.6;
	7374	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3712.6;
	7374	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9929.9;
	7374	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6071.8;
	7374	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10237.6;
	7374	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1590.5;
	7375	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.9;
	7375	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	760.1;
	7375	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.6;
	7375	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3840.5;
	7375	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10514.5;
	7375	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6368.7;
	7375	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10499.7;
	7375	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1659;
	7376	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946;
	7376	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	765.5;
	7376	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.3;
	7376	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3779.7;
	7376	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10490.1;
	7376	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6369.9;
	7376	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10553;
	7376	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1669.1;
	7377	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.8;
	7377	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	775.9;
	7377	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	862.8;
	7377	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4004.1;
	7377	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10651.1;
	7377	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6570.6;
	7377	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11065.6;
	7377	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1705.7;
	7378	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.5;
	7378	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	788.4;
	7378	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	875.1;
	7378	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4279.2;
	7378	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10866.2;
	7378	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6818.1;
	7378	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11724.4;
	7378	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1765.7;
	7379	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.2;
	7379	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	792.6;
	7379	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	881.5;
	7379	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4547.5;
	7379	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10993.4;
	7379	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7011.4;
	7379	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12263.7;
	7379	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1837.3;
	7380	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953;
	7380	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	788.4;
	7380	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889.4;
	7380	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4804.4;
	7380	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11069.5;
	7380	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7177;
	7380	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12764.5;
	7380	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1913.5;
	7381	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.1;
	7381	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	796;
	7381	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897.9;
	7381	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5030;
	7381	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11079.9;
	7381	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7324.5;
	7381	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13213.5;
	7381	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1978;
	7382	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961;
	7382	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	796.6;
	7382	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	908.6;
	7382	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5121.9;
	7382	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11014.8;
	7382	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7515.6;
	7382	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13443.1;
	7382	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2014.6;
	7383	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.8;
	7383	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	793.7;
	7383	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	916.8;
	7383	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5162.9;
	7383	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10922.6;
	7383	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7613.7;
	7383	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13390.4;
	7383	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2001.8;
	7384	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960;
	7384	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	795.8;
	7384	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	923.2;
	7384	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5167;
	7384	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10901.6;
	7384	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7627;
	7384	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13278.3;
	7384	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1971;
	7385	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951;
	7385	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	785.3;
	7385	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	920.5;
	7385	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5004.5;
	7385	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10860.4;
	7385	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7416.6;
	7385	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12898.2;
	7385	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1900.5;
	7386	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.5;
	7386	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	799.2;
	7386	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	932.2;
	7386	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4848.1;
	7386	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11042.4;
	7386	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7224;
	7386	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12739.4;
	7386	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1910.4;
	7387	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969.4;
	7387	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	814.6;
	7387	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	938;
	7387	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4854.9;
	7387	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11296.7;
	7387	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7233.7;
	7387	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12642.4;
	7387	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1908.5;
	7388	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.9;
	7388	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	800.5;
	7388	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	927.2;
	7388	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4724.7;
	7388	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11126.1;
	7388	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7009.5;
	7388	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12312.2;
	7388	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1853.4;
	7389	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.2;
	7389	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772.7;
	7389	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	888.2;
	7389	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4492.3;
	7389	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10674.5;
	7389	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6602.3;
	7389	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11758.5;
	7389	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1754.8;
	7390	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933;
	7390	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	726.7;
	7390	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.3;
	7390	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4202.9;
	7390	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9895.6;
	7390	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6048.6;
	7390	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10974.9;
	7390	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1648.5;
	7391	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.4;
	7391	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	674.2;
	7391	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	782;
	7391	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3847.1;
	7391	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8971.4;
	7391	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5441.1;
	7391	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10136;
	7391	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1528.7;
	7392	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.9;
	7392	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	643.8;
	7392	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	757.7;
	7392	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3523.9;
	7392	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8273.7;
	7392	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5044.1;
	7392	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9505.6;
	7392	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1424;
	7393	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.8;
	7393	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	623.7;
	7393	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	731.3;
	7393	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3330.5;
	7393	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7843.6;
	7393	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4739.3;
	7393	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9079.8;
	7393	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1303.6;
	7394	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875;
	7394	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	611.7;
	7394	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	714.3;
	7394	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3200.8;
	7394	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7564.7;
	7394	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4607.8;
	7394	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8813.3;
	7394	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1283.6;
	7395	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.4;
	7395	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	606.1;
	7395	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	705.9;
	7395	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3121.7;
	7395	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7425.3;
	7395	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4528.8;
	7395	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8682.5;
	7395	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1296.5;
	7396	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.4;
	7396	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	613.2;
	7396	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	706.1;
	7396	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3084.5;
	7396	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7550.2;
	7396	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4555.4;
	7396	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8771.7;
	7396	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1291;
	7397	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.5;
	7397	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	642.5;
	7397	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	731.2;
	7397	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3193.6;
	7397	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8124.2;
	7397	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4868.1;
	7397	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9253.3;
	7397	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1378;
	7398	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.4;
	7398	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	701.1;
	7398	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	806.3;
	7398	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3548.5;
	7398	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9200.3;
	7398	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5536;
	7398	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9981.2;
	7398	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1538.1;
	7399	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.4;
	7399	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.4;
	7399	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.8;
	7399	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3690;
	7399	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9916.1;
	7399	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5869.7;
	7399	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10341.6;
	7399	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1622.8;
	7400	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.8;
	7400	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.7;
	7400	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	856.9;
	7400	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3626.4;
	7400	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9950.2;
	7400	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5847.4;
	7400	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10370.8;
	7400	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1653.6;
	7401	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.9;
	7401	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.1;
	7401	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	870.1;
	7401	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3789.6;
	7401	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10138.6;
	7401	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5956.4;
	7401	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10784.3;
	7401	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1701.5;
	7402	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.2;
	7402	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	773.2;
	7402	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878.9;
	7402	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4047.3;
	7402	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10417;
	7402	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6091.5;
	7402	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11390.4;
	7402	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1722;
	7403	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.2;
	7403	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	778.3;
	7403	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	883.9;
	7403	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4275;
	7403	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10590.3;
	7403	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6159.4;
	7403	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11967;
	7403	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1742.9;
	7404	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.6;
	7404	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	770.5;
	7404	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889.8;
	7404	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4445.6;
	7404	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10712.6;
	7404	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6222.7;
	7404	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12478.9;
	7404	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1762.2;
	7405	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.3;
	7405	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	774.4;
	7405	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.4;
	7405	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4604.9;
	7405	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10810.1;
	7405	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6258.5;
	7405	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12929.2;
	7405	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1727.4;
	7406	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.1;
	7406	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772;
	7406	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	905.3;
	7406	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4751;
	7406	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10816.2;
	7406	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6227.3;
	7406	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13096.9;
	7406	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1732.3;
	7407	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.2;
	7407	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	761.4;
	7407	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897.8;
	7407	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4782.2;
	7407	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10731.6;
	7407	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6202.7;
	7407	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	13090.1;
	7407	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1797.4;
	7408	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.8;
	7408	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	758.5;
	7408	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.8;
	7408	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4713.6;
	7408	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10707.7;
	7408	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6244.2;
	7408	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12936.1;
	7408	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1817.5;
	7409	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.5;
	7409	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	750.9;
	7409	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	876.8;
	7409	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4510.3;
	7409	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10700.5;
	7409	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6246;
	7409	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12536.5;
	7409	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1760.5;
	7410	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.1;
	7410	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755.6;
	7410	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	877.5;
	7410	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4353.1;
	7410	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10830.8;
	7410	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6323.9;
	7410	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12265.9;
	7410	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1811.2;
	7411	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951.8;
	7411	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	762;
	7411	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897.6;
	7411	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4368.4;
	7411	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10913.3;
	7411	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6380.2;
	7411	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12143.3;
	7411	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1841.5;
	7412	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.2;
	7412	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	750.8;
	7412	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.9;
	7412	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4246.3;
	7412	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10688.8;
	7412	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6241.8;
	7412	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11788.4;
	7412	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1802.6;
	7413	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.2;
	7413	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730;
	7413	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	856.6;
	7413	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4079.6;
	7413	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10338.2;
	7413	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6025.8;
	7413	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11368.8;
	7413	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1705.7;
	7414	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925.2;
	7414	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	703.8;
	7414	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	827.7;
	7414	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3869.4;
	7414	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9794;
	7414	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5721.9;
	7414	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10792.2;
	7414	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1618.8;
	7415	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906;
	7415	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	670.3;
	7415	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	785.5;
	7415	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3629.2;
	7415	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9078;
	7415	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5311;
	7415	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10153.6;
	7415	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1505.3;
	7416	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897;
	7416	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	637;
	7416	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	757.7;
	7416	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3373.1;
	7416	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8457.6;
	7416	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4924.8;
	7416	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9487.8;
	7416	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1381.6;
	7417	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.3;
	7417	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	615.2;
	7417	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	740.8;
	7417	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3192.4;
	7417	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7970.1;
	7417	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4655.4;
	7417	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9071;
	7417	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1308.7;
	7418	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.7;
	7418	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	601.5;
	7418	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	733.1;
	7418	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3074.1;
	7418	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7641.4;
	7418	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4481.4;
	7418	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8789.5;
	7418	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1202;
	7419	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	876.9;
	7419	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	607.2;
	7419	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	728.6;
	7419	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2994.8;
	7419	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7429.4;
	7419	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4370.6;
	7419	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8587.8;
	7419	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1155;
	7420	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.5;
	7420	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	610.6;
	7420	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	727.7;
	7420	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2940.6;
	7420	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7337.1;
	7420	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4353;
	7420	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8475.8;
	7420	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1141.7;
	7421	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.5;
	7421	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	619.4;
	7421	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	737.3;
	7421	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2953.4;
	7421	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7458.2;
	7421	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4419.9;
	7421	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8512.9;
	7421	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1210.4;
	7422	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.5;
	7422	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	639.3;
	7422	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	761.2;
	7422	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3043.2;
	7422	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7798.8;
	7422	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4641.1;
	7422	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8688.7;
	7422	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1248.7;
	7423	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898;
	7423	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	658.7;
	7423	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	783.6;
	7423	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3132.6;
	7423	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8197.7;
	7423	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4934.2;
	7423	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8822;
	7423	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1305.7;
	7424	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.1;
	7424	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	678.3;
	7424	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	798.8;
	7424	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3197.1;
	7424	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8563.7;
	7424	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5159.7;
	7424	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8903.3;
	7424	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1401.5;
	7425	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.8;
	7425	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	700.8;
	7425	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	830.3;
	7425	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3402.5;
	7425	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9019.1;
	7425	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5463.7;
	7425	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9333.8;
	7425	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1446.5;
	7426	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.1;
	7426	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	716.6;
	7426	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	848.4;
	7426	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3636.8;
	7426	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9351.3;
	7426	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5734.4;
	7426	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9753.5;
	7426	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1499.8;
	7427	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.3;
	7427	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	705.5;
	7427	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.1;
	7427	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3894.6;
	7427	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9542.4;
	7427	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5897.5;
	7427	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10028.6;
	7427	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1500;
	7428	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.2;
	7428	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	696.4;
	7428	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.2;
	7428	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4141;
	7428	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9663.3;
	7428	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5992;
	7428	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10347.7;
	7428	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1489.6;
	7429	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.1;
	7429	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	700.1;
	7429	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.9;
	7429	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4334;
	7429	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9700.8;
	7429	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6087.8;
	7429	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10745.5;
	7429	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1476.2;
	7430	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904;
	7430	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	700.1;
	7430	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851;
	7430	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4415.2;
	7430	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9719;
	7430	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6143.2;
	7430	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11064.6;
	7430	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1472.4;
	7431	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.9;
	7431	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	702.2;
	7431	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	845.8;
	7431	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4397.3;
	7431	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9736.8;
	7431	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6168.6;
	7431	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11168.9;
	7431	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1456.8;
	7432	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.8;
	7432	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	702;
	7432	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.2;
	7432	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4331.5;
	7432	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9747.8;
	7432	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6197.2;
	7432	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11009.8;
	7432	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1458.4;
	7433	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897.7;
	7433	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	701.8;
	7433	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.2;
	7433	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4211.5;
	7433	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9673.1;
	7433	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6169.9;
	7433	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10773.7;
	7433	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1473.9;
	7434	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.2;
	7434	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	706.4;
	7434	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	863.1;
	7434	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4122;
	7434	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9733.3;
	7434	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6172.1;
	7434	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10649.9;
	7434	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1528.3;
	7435	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.5;
	7435	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	716.4;
	7435	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878.5;
	7435	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4175.3;
	7435	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9901.9;
	7435	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6285;
	7435	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10670.8;
	7435	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1545.5;
	7436	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.3;
	7436	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	704.1;
	7436	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	864.4;
	7436	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4073.7;
	7436	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9669.9;
	7436	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6115.7;
	7436	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10391.9;
	7436	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1465.4;
	7437	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.3;
	7437	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	684.6;
	7437	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838.2;
	7437	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3908.4;
	7437	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9310.9;
	7437	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5837.1;
	7437	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10010.5;
	7437	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1442.3;
	7438	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.8;
	7438	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	657;
	7438	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	804.6;
	7438	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3721.6;
	7438	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8819.8;
	7438	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5502.7;
	7438	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9555.6;
	7438	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1411.9;
	7439	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.6;
	7439	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	625;
	7439	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	765.1;
	7439	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3519.7;
	7439	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8200.1;
	7439	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5098.3;
	7439	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9012.2;
	7439	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1316.8;
	7440	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	863.5;
	7440	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	604.5;
	7440	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	732.8;
	7440	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3333.4;
	7440	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7560.8;
	7440	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4719.9;
	7440	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8563.2;
	7440	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1240.9;
	7441	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	854.4;
	7441	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	582.4;
	7441	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	706.1;
	7441	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3169.8;
	7441	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7140.6;
	7441	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4426.8;
	7441	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8202.7;
	7441	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1181.1;
	7442	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	835.7;
	7442	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	567.2;
	7442	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	686.5;
	7442	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3025.4;
	7442	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6867.3;
	7442	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4205.2;
	7442	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7878;
	7442	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1142.2;
	7443	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	829.6;
	7443	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	562.3;
	7443	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	676.2;
	7443	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2933.8;
	7443	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6711.7;
	7443	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4100.8;
	7443	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7726.6;
	7443	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1133.8;
	7444	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	824.5;
	7444	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	560.3;
	7444	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	673.5;
	7444	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2860.3;
	7444	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6631.9;
	7444	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4014.2;
	7444	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7629.2;
	7444	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1133;
	7445	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	821.3;
	7445	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	564.9;
	7445	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	674.8;
	7445	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2836.6;
	7445	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6651.5;
	7445	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4002.4;
	7445	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7613.5;
	7445	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1124.7;
	7446	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	825;
	7446	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	568.5;
	7446	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	683.3;
	7446	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2855.6;
	7446	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6796.4;
	7446	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4071.9;
	7446	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7686.8;
	7446	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1134.7;
	7447	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	835.1;
	7447	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588.6;
	7447	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	711.8;
	7447	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2902.4;
	7447	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7150.3;
	7447	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4253.8;
	7447	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7866.9;
	7447	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1110.5;
	7448	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	842.4;
	7448	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	615.2;
	7448	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	745;
	7448	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2973.7;
	7448	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7550.3;
	7448	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4483.7;
	7448	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8007.5;
	7448	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1262.2;
	7449	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	858.7;
	7449	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	653.7;
	7449	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	789.1;
	7449	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3277.5;
	7449	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8232.4;
	7449	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4822.4;
	7449	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8486.5;
	7449	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1342.2;
	7450	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.5;
	7450	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	680.5;
	7450	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	814;
	7450	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3576.9;
	7450	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8763.7;
	7450	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5089.3;
	7450	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8908.4;
	7450	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1434;
	7451	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.6;
	7451	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	694.8;
	7451	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	823.2;
	7451	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3787.5;
	7451	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9084.5;
	7451	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5314.1;
	7451	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9327.7;
	7451	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1493.3;
	7452	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.4;
	7452	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	697.5;
	7452	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	827.5;
	7452	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3974.8;
	7452	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9309.1;
	7452	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5504.2;
	7452	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9822;
	7452	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1560;
	7453	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.1;
	7453	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	694.9;
	7453	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.6;
	7453	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4150.6;
	7453	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9402.1;
	7453	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5706.4;
	7453	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10327.8;
	7453	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1608.5;
	7454	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.8;
	7454	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	694.7;
	7454	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	844.3;
	7454	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4255.1;
	7454	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9368.2;
	7454	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5864.6;
	7454	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10636.8;
	7454	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1648.5;
	7455	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.7;
	7455	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	691;
	7455	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	848.8;
	7455	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4259.8;
	7455	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9263.4;
	7455	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5904.3;
	7455	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10705.3;
	7455	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1646.6;
	7456	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.5;
	7456	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	695.1;
	7456	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	855;
	7456	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4230;
	7456	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9213.5;
	7456	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5942;
	7456	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10555.5;
	7456	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1631;
	7457	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.5;
	7457	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	701.4;
	7457	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	858;
	7457	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4147.5;
	7457	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9258.7;
	7457	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5958.8;
	7457	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10428.2;
	7457	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1657.6;
	7458	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.2;
	7458	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728;
	7458	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.2;
	7458	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4113.4;
	7458	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9629.4;
	7458	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5945.8;
	7458	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10613.3;
	7458	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1707.3;
	7459	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952;
	7459	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746.7;
	7459	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	898.6;
	7459	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4255;
	7459	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9949.9;
	7459	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6135.1;
	7459	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10842.2;
	7459	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1711.2;
	7460	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.4;
	7460	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735.3;
	7460	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.4;
	7460	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4192.3;
	7460	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9839.8;
	7460	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6041.7;
	7460	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10697.4;
	7460	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1685.8;
	7461	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.4;
	7461	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714.8;
	7461	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.1;
	7461	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4048.2;
	7461	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9535.6;
	7461	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5777.2;
	7461	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10390.3;
	7461	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1619.7;
	7462	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.7;
	7462	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	685.6;
	7462	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	832.3;
	7462	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3869.1;
	7462	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9105.2;
	7462	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5482.4;
	7462	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9951.6;
	7462	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1520.7;
	7463	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.2;
	7463	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	645;
	7463	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	783.6;
	7463	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3593.6;
	7463	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8442.3;
	7463	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5024.3;
	7463	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9301.6;
	7463	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1374.3;
	7464	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866.8;
	7464	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	618;
	7464	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	751.6;
	7464	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3338.4;
	7464	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7847.6;
	7464	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4680.9;
	7464	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8761.7;
	7464	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1313;
	7465	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	846.5;
	7465	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	591;
	7465	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	719.6;
	7465	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3083.1;
	7465	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7252.9;
	7465	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4337.5;
	7465	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8221.9;
	7465	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1251.6;
	7466	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	836.9;
	7466	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	579.8;
	7466	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	698.8;
	7466	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2945.9;
	7466	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6978;
	7466	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4148.1;
	7466	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7999.3;
	7466	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1210.7;
	7467	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	829.3;
	7467	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	575.4;
	7467	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	685.8;
	7467	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2853.5;
	7467	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6829.8;
	7467	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4036.2;
	7467	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7892.4;
	7467	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1175.7;
	7468	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	829.6;
	7468	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586.5;
	7468	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	683.9;
	7468	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2821.9;
	7468	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6827.8;
	7468	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4020.7;
	7468	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7912;
	7468	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1186.2;
	7469	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	835.5;
	7469	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589;
	7469	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	691.7;
	7469	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2849.5;
	7469	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7047;
	7469	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4139.1;
	7469	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8125.4;
	7469	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1217;
	7470	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	854.2;
	7470	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	630.4;
	7470	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	728.8;
	7470	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3037.5;
	7470	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7734.9;
	7470	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4515.8;
	7470	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8687.6;
	7470	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1307.2;
	7471	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.2;
	7471	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	700.5;
	7471	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	803.9;
	7471	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3391.6;
	7471	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8926.4;
	7471	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5141;
	7471	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9388.9;
	7471	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1493.8;
	7472	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.8;
	7472	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.9;
	7472	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	837.4;
	7472	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3456.2;
	7472	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9416.3;
	7472	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5389.6;
	7472	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9527.4;
	7472	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1539.5;
	7473	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915;
	7473	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.2;
	7473	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	855;
	7473	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3616.7;
	7473	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9631.8;
	7473	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5535.9;
	7473	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9813.4;
	7473	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1588.3;
	7474	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928;
	7474	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	770.7;
	7474	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878.2;
	7474	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3908.9;
	7474	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9933.8;
	7474	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5694.1;
	7474	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10117.2;
	7474	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1643.4;
	7475	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.8;
	7475	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	778.3;
	7475	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	894.5;
	7475	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4188.9;
	7475	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10165.3;
	7475	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5829.1;
	7475	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10360.7;
	7475	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1723.5;
	7476	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.3;
	7476	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	781.1;
	7476	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	910.6;
	7476	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4427;
	7476	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10298.9;
	7476	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5958.2;
	7476	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10685.1;
	7476	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1738.6;
	7477	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951;
	7477	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	778.4;
	7477	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	925.4;
	7477	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4586.4;
	7477	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10369.4;
	7477	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6036.4;
	7477	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10903.4;
	7477	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1759.6;
	7478	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.3;
	7478	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	780.7;
	7478	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	963.6;
	7478	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4699.3;
	7478	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10430.6;
	7478	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6094.6;
	7478	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11183.8;
	7478	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1773.7;
	7479	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.9;
	7479	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	775.1;
	7479	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	977.3;
	7479	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4679;
	7479	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10400.5;
	7479	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6136.7;
	7479	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11355.7;
	7479	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1796.1;
	7480	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.5;
	7480	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772.9;
	7480	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	970.8;
	7480	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4585;
	7480	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10301.5;
	7480	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6243.5;
	7480	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11382.4;
	7480	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1772.6;
	7481	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975;
	7481	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	775.1;
	7481	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	961.4;
	7481	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4482.7;
	7481	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10382.3;
	7481	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6336.6;
	7481	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11314.3;
	7481	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1759.4;
	7482	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.3;
	7482	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	783.3;
	7482	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	953.2;
	7482	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4389.5;
	7482	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10643.1;
	7482	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6493.3;
	7482	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11185.1;
	7482	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1810.8;
	7483	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.6;
	7483	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	799.3;
	7483	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	975.7;
	7483	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4469.1;
	7483	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10856.7;
	7483	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6691.7;
	7483	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11266.9;
	7483	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1858.4;
	7484	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.7;
	7484	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	794;
	7484	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	963.1;
	7484	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4385.2;
	7484	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10743.9;
	7484	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6578.7;
	7484	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11082.1;
	7484	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1730.2;
	7485	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.3;
	7485	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777.2;
	7485	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	935;
	7485	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4227.4;
	7485	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10433.4;
	7485	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6381;
	7485	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10733.8;
	7485	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1688.6;
	7486	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.6;
	7486	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.7;
	7486	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.2;
	7486	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3980.1;
	7486	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9849.5;
	7486	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6058.7;
	7486	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10108.9;
	7486	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1667.9;
	7487	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.1;
	7487	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	686.9;
	7487	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	830.3;
	7487	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3640.1;
	7487	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9024.2;
	7487	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5529.1;
	7487	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9316.7;
	7487	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1498.7;
	7488	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	860.6;
	7488	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	641.5;
	7488	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	771.3;
	7488	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3308.5;
	7488	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8236.1;
	7488	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5069.6;
	7488	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8572.8;
	7488	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1396.8;
	7489	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	846.5;
	7489	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	628;
	7489	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	739.5;
	7489	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3114.8;
	7489	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7623.1;
	7489	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4684.4;
	7489	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8151;
	7489	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1319.7;
	7490	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	839.5;
	7490	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	614.8;
	7490	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	719.1;
	7490	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2969.1;
	7490	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7323.5;
	7490	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4461.5;
	7490	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7866.6;
	7490	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1262;
	7491	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	838.7;
	7491	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	604.3;
	7491	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	710.5;
	7491	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2877.6;
	7491	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7163.7;
	7491	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4329.8;
	7491	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7722;
	7491	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1138;
	7492	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	834.6;
	7492	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	599.7;
	7492	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	708;
	7492	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2835.9;
	7492	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7128.7;
	7492	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4202.7;
	7492	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7689.8;
	7492	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1120.6;
	7493	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	837.8;
	7493	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	608;
	7493	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	710.5;
	7493	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2851.4;
	7493	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7302.8;
	7493	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4230.8;
	7493	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7850.4;
	7493	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1161.1;
	7494	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	853.4;
	7494	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	639.8;
	7494	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	739.1;
	7494	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3010.2;
	7494	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7984;
	7494	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4563.7;
	7494	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8445.7;
	7494	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1293.4;
	7495	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.8;
	7495	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	705.7;
	7495	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	812.8;
	7495	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3377.1;
	7495	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9189.7;
	7495	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5185.8;
	7495	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9251.8;
	7495	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1461.6;
	7496	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.3;
	7496	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	726.5;
	7496	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838.9;
	7496	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3397;
	7496	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9623.6;
	7496	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5391.7;
	7496	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9430.4;
	7496	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1512.5;
	7497	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.1;
	7497	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	734;
	7497	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.6;
	7497	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3385.9;
	7497	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9741.4;
	7497	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5510.7;
	7497	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9639.8;
	7497	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1558;
	7498	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.4;
	7498	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746.6;
	7498	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.4;
	7498	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3435.8;
	7498	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9938.9;
	7498	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5684.9;
	7498	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9901;
	7498	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1570;
	7499	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.6;
	7499	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.5;
	7499	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	873.5;
	7499	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3488.6;
	7499	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10061.3;
	7499	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5836.2;
	7499	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10056.5;
	7499	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1617.2;
	7500	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.7;
	7500	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	748.7;
	7500	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.7;
	7500	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3529.9;
	7500	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10104.7;
	7500	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5917.3;
	7500	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10133.6;
	7500	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1630.8;
	7501	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.7;
	7501	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	748.4;
	7501	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897.2;
	7501	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3538.5;
	7501	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10095.9;
	7501	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5965.3;
	7501	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10173.8;
	7501	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1645.2;
	7502	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.4;
	7502	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	745.2;
	7502	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	909.1;
	7502	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3506.7;
	7502	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10025.2;
	7502	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5957;
	7502	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10186.9;
	7502	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1644.5;
	7503	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.6;
	7503	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	739.9;
	7503	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	915.7;
	7503	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3473.5;
	7503	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9944.6;
	7503	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5955;
	7503	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10179.4;
	7503	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1626;
	7504	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.9;
	7504	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	731.2;
	7504	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	921.9;
	7504	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3443.1;
	7504	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9855.4;
	7504	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5960.2;
	7504	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10091.6;
	7504	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1555.9;
	7505	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.2;
	7505	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.6;
	7505	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	929.7;
	7505	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3448.3;
	7505	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9936.2;
	7505	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6024.4;
	7505	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10097.2;
	7505	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1645.3;
	7506	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.8;
	7506	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	763.9;
	7506	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940;
	7506	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3512.4;
	7506	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10329.9;
	7506	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6222;
	7506	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10384.7;
	7506	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1652.2;
	7507	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.7;
	7507	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	783.9;
	7507	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	949.7;
	7507	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3671.3;
	7507	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10614.8;
	7507	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6393.2;
	7507	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10462.9;
	7507	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1728;
	7508	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.8;
	7508	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	769.9;
	7508	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940.3;
	7508	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3650;
	7508	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10443;
	7508	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6241.7;
	7508	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10330.7;
	7508	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1684.5;
	7509	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.1;
	7509	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749.7;
	7509	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	914.6;
	7509	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3577.3;
	7509	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10115.4;
	7509	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6070.4;
	7509	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10093.6;
	7509	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1626.8;
	7510	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.2;
	7510	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	716.9;
	7510	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.7;
	7510	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3415.3;
	7510	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9545.1;
	7510	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5733.3;
	7510	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9651.9;
	7510	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1570.1;
	7511	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.6;
	7511	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	681.2;
	7511	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	815.4;
	7511	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3171.8;
	7511	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8813.2;
	7511	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5324.1;
	7511	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9092.9;
	7511	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1450.4;
	7512	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.7;
	7512	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	650.1;
	7512	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	767.7;
	7512	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2955.7;
	7512	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8161.1;
	7512	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4902.5;
	7512	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8580.1;
	7512	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1345.9;
	7513	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.7;
	7513	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	631.9;
	7513	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	783.6;
	7513	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2785.9;
	7513	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7655.6;
	7513	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4463.5;
	7513	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8180.8;
	7513	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1246.8;
	7514	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	844.8;
	7514	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	612.6;
	7514	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	764.6;
	7514	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2646.3;
	7514	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7285.7;
	7514	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4261.7;
	7514	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7875.5;
	7514	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1235.5;
	7515	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	838.7;
	7515	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	601.2;
	7515	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	753.1;
	7515	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2579.8;
	7515	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7074.4;
	7515	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4124.2;
	7515	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7693.9;
	7515	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1174.8;
	7516	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	836.7;
	7516	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.6;
	7516	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	753.1;
	7516	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2533;
	7516	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6995.9;
	7516	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4048.9;
	7516	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7604.1;
	7516	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1142.6;
	7517	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	841.5;
	7517	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	606.9;
	7517	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	763.1;
	7517	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2543.8;
	7517	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7172.6;
	7517	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4140.1;
	7517	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7733.1;
	7517	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1139.5;
	7518	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	856.5;
	7518	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	629.4;
	7518	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	801.2;
	7518	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2702.1;
	7518	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7810.1;
	7518	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4453.7;
	7518	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8281.8;
	7518	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1211.3;
	7519	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889;
	7519	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	689.7;
	7519	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	881.4;
	7519	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3056.5;
	7519	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8934.2;
	7519	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5108.2;
	7519	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9004.4;
	7519	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1378.7;
	7520	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.3;
	7520	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	708.9;
	7520	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	908.5;
	7520	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3105.6;
	7520	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9283.3;
	7520	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5340.9;
	7520	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9184.2;
	7520	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1428;
	7521	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.4;
	7521	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723.6;
	7521	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	908.3;
	7521	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3135;
	7521	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9450.7;
	7521	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5378.8;
	7521	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9382.7;
	7521	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1463.8;
	7522	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918;
	7522	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.9;
	7522	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	912.5;
	7522	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3206.1;
	7522	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9648.1;
	7522	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5481.9;
	7522	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9601.9;
	7522	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1498.3;
	7523	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924;
	7523	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.4;
	7523	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	908.6;
	7523	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3272.5;
	7523	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9801.9;
	7523	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5549.2;
	7523	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9820.6;
	7523	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1508.8;
	7524	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.1;
	7524	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	754.4;
	7524	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	904.9;
	7524	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3324.3;
	7524	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9897.8;
	7524	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5596;
	7524	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9993.7;
	7524	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1530.8;
	7525	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.4;
	7525	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	756.2;
	7525	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.9;
	7525	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3342.4;
	7525	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9942.8;
	7525	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5603.9;
	7525	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10078.8;
	7525	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1546.6;
	7526	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.9;
	7526	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	759.9;
	7526	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889.1;
	7526	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3379.7;
	7526	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9924.3;
	7526	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5602;
	7526	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10161.8;
	7526	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1563.1;
	7527	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933;
	7527	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	757;
	7527	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	881.1;
	7527	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3391.4;
	7527	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9879.4;
	7527	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5598.8;
	7527	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10173.4;
	7527	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1561.3;
	7528	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.2;
	7528	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753.1;
	7528	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.7;
	7528	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3387.1;
	7528	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9792.6;
	7528	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5603.1;
	7528	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10129.3;
	7528	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1545.6;
	7529	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.1;
	7529	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751;
	7529	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	898.6;
	7529	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3410.5;
	7529	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9837.8;
	7529	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5677.4;
	7529	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10109.9;
	7529	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1552.6;
	7530	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.2;
	7530	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	759;
	7530	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	927.5;
	7530	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3466.6;
	7530	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10198;
	7530	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5910.8;
	7530	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10308.2;
	7530	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1595.6;
	7531	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970.6;
	7531	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	773.1;
	7531	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	961.5;
	7531	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3635.7;
	7531	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10548.1;
	7531	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6034.4;
	7531	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10474.5;
	7531	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1649.1;
	7532	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.4;
	7532	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	765.8;
	7532	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	956.6;
	7532	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3596.6;
	7532	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10360.1;
	7532	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5943.4;
	7532	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10294.8;
	7532	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1609.6;
	7533	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.1;
	7533	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746.6;
	7533	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	943;
	7533	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3515.9;
	7533	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10021.8;
	7533	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5790.2;
	7533	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9969.2;
	7533	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1561.3;
	7534	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.4;
	7534	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	708.8;
	7534	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	912.5;
	7534	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3364;
	7534	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9447.3;
	7534	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5517.2;
	7534	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9459.3;
	7534	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1472.9;
	7535	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.8;
	7535	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	663.2;
	7535	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.5;
	7535	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3135.4;
	7535	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8639.6;
	7535	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5106.8;
	7535	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8796.6;
	7535	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1351.1;
	7536	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	876.4;
	7536	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	622;
	7536	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	820.4;
	7536	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2873.1;
	7536	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7851.6;
	7536	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4671.1;
	7536	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8133.8;
	7536	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1220.5;
	7537	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.4;
	7537	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	609.2;
	7537	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	750;
	7537	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2655.2;
	7537	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7343.7;
	7537	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4322.5;
	7537	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7716.9;
	7537	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1145.4;
	7538	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.6;
	7538	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	596.4;
	7538	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	732.8;
	7538	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2559;
	7538	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7076.4;
	7538	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4129.1;
	7538	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7468.3;
	7538	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1106.1;
	7539	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871.3;
	7539	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	593.4;
	7539	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	726.6;
	7539	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2502;
	7539	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6920.8;
	7539	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4025.8;
	7539	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7324.3;
	7539	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1086.2;
	7540	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868;
	7540	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.9;
	7540	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	720.4;
	7540	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2475.9;
	7540	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6900.1;
	7540	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3996.1;
	7540	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7293;
	7540	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1083.3;
	7541	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.2;
	7541	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	604.4;
	7541	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	727.6;
	7541	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2486.1;
	7541	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7077.1;
	7541	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4067.5;
	7541	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7414.9;
	7541	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1102.2;
	7542	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.7;
	7542	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	641.1;
	7542	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	763.3;
	7542	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2642.2;
	7542	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7752.8;
	7542	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4428.4;
	7542	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7942.3;
	7542	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1196.8;
	7543	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.7;
	7543	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	710;
	7543	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.3;
	7543	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3014.1;
	7543	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8907;
	7543	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5085.4;
	7543	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8628.2;
	7543	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1371.8;
	7544	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.7;
	7544	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725;
	7544	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	855.4;
	7544	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3059.3;
	7544	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9226.7;
	7544	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5274.9;
	7544	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8744.7;
	7544	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1402.7;
	7545	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.5;
	7545	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.7;
	7545	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.3;
	7545	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3086.1;
	7545	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9347;
	7545	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5304.6;
	7545	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9020.3;
	7545	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1417.1;
	7546	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.4;
	7546	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	734.3;
	7546	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	867.4;
	7546	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3173.2;
	7546	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9504.3;
	7546	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5387.6;
	7546	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9316.5;
	7546	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1429.4;
	7547	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.8;
	7547	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737.6;
	7547	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	872.5;
	7547	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3254.9;
	7547	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9646.2;
	7547	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5465.6;
	7547	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9557.3;
	7547	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1452;
	7548	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.3;
	7548	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.7;
	7548	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	876.6;
	7548	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3320;
	7548	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9773.2;
	7548	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5506.8;
	7548	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9785.5;
	7548	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1473.4;
	7549	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935;
	7549	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.5;
	7549	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	871.5;
	7549	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3361;
	7549	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9874.7;
	7549	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5572.7;
	7549	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9864.9;
	7549	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1484.6;
	7550	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.6;
	7550	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	739.2;
	7550	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	870.3;
	7550	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3404.1;
	7550	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9996;
	7550	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5596.3;
	7550	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9976.7;
	7550	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1502;
	7551	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.1;
	7551	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.2;
	7551	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878.9;
	7551	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3399.1;
	7551	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10010.5;
	7551	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5586;
	7551	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9926.7;
	7551	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1503.5;
	7552	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939;
	7552	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.9;
	7552	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	879.5;
	7552	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3375.9;
	7552	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9977.4;
	7552	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5578.1;
	7552	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9807.6;
	7552	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1490.9;
	7553	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.3;
	7553	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.2;
	7553	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	880.9;
	7553	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3367.2;
	7553	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9926.3;
	7553	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5554.8;
	7553	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9763.5;
	7553	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1490.6;
	7554	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.8;
	7554	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743;
	7554	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897.7;
	7554	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3423.2;
	7554	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10183.9;
	7554	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5650.8;
	7554	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9982.2;
	7554	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1540.5;
	7555	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975;
	7555	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	764.1;
	7555	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	930.2;
	7555	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3578.4;
	7555	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10473.5;
	7555	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5947.4;
	7555	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10032.1;
	7555	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1591.2;
	7556	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970.7;
	7556	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	754;
	7556	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	922.7;
	7556	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3549.6;
	7556	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10252.7;
	7556	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5854;
	7556	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9852.1;
	7556	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1561.1;
	7557	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.8;
	7557	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	739.7;
	7557	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	899.3;
	7557	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3491.8;
	7557	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9955.1;
	7557	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5701;
	7557	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9593.9;
	7557	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1518.6;
	7558	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.1;
	7558	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	707.1;
	7558	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.2;
	7558	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3356.9;
	7558	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9428;
	7558	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5402.1;
	7558	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9187.3;
	7558	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1440.4;
	7559	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922;
	7559	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	666.9;
	7559	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	814.8;
	7559	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3126.4;
	7559	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8637;
	7559	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4995.9;
	7559	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8596.2;
	7559	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1346.4;
	7560	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.4;
	7560	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	629.8;
	7560	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	763.3;
	7560	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2865.1;
	7560	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7883.1;
	7560	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4565.1;
	7560	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8029;
	7560	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1257.9;
	7561	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.8;
	7561	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	607.6;
	7561	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	734.6;
	7561	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2669;
	7561	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7372.1;
	7561	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4202.3;
	7561	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7613.7;
	7561	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1205.8;
	7562	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.1;
	7562	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	595.9;
	7562	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	717;
	7562	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2554.3;
	7562	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7086.8;
	7562	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4002.4;
	7562	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7379;
	7562	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1115.4;
	7563	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868.4;
	7563	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588.1;
	7563	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	700.7;
	7563	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2479.3;
	7563	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6900.1;
	7563	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3916.6;
	7563	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7243.8;
	7563	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1112.9;
	7564	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867.4;
	7564	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588;
	7564	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	698.2;
	7564	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2442.3;
	7564	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6860.4;
	7564	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3862;
	7564	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7225.6;
	7564	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1152.7;
	7565	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	872.3;
	7565	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	602.1;
	7565	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	703.1;
	7565	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2455.1;
	7565	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7053;
	7565	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3931.7;
	7565	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7388;
	7565	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1165.6;
	7566	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.9;
	7566	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	625.8;
	7566	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	736.4;
	7566	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2590.5;
	7566	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7652.3;
	7566	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4267.5;
	7566	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7899.6;
	7566	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1268.9;
	7567	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.3;
	7567	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	686.2;
	7567	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	812;
	7567	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2905.2;
	7567	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8722.3;
	7567	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4894.9;
	7567	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8597.7;
	7567	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1424.3;
	7568	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925.3;
	7568	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	703.8;
	7568	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.9;
	7568	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2964.5;
	7568	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9091;
	7568	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5115.8;
	7568	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8798.2;
	7568	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1457.7;
	7569	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.3;
	7569	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	712.8;
	7569	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849.6;
	7569	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3007.8;
	7569	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9271.9;
	7569	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5218;
	7569	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9019.1;
	7569	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1498.9;
	7570	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.2;
	7570	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	716.7;
	7570	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	862.6;
	7570	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3108;
	7570	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9499.5;
	7570	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5320.7;
	7570	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9272.8;
	7570	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1530;
	7571	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.4;
	7571	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.2;
	7571	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.3;
	7571	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3215.1;
	7571	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9695.8;
	7571	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5399.7;
	7571	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9502.5;
	7571	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1519.8;
	7572	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.9;
	7572	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735.8;
	7572	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	863;
	7572	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3303.1;
	7572	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9813.8;
	7572	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5448.2;
	7572	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9680.1;
	7572	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1529.7;
	7573	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.4;
	7573	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738;
	7573	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	877;
	7573	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3349.5;
	7573	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9921.6;
	7573	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5477;
	7573	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9801.7;
	7573	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1543.4;
	7574	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.7;
	7574	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	748.8;
	7574	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.7;
	7574	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3412.2;
	7574	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10076.4;
	7574	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5554;
	7574	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9993.1;
	7574	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1512.5;
	7575	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.4;
	7575	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.3;
	7575	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	887.2;
	7575	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3451.1;
	7575	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10162.4;
	7575	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5623.7;
	7575	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10047.7;
	7575	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1516.6;
	7576	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941;
	7576	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.4;
	7576	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.7;
	7576	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3441.7;
	7576	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10174.2;
	7576	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5702.5;
	7576	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9990.2;
	7576	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1583;
	7577	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.5;
	7577	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740;
	7577	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	883.8;
	7577	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3411.8;
	7577	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10141.7;
	7577	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5680.4;
	7577	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9860.9;
	7577	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1589.3;
	7578	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.1;
	7578	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.9;
	7578	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	874.7;
	7578	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3418.7;
	7578	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10210.2;
	7578	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5766.4;
	7578	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9886.8;
	7578	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1587;
	7579	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.5;
	7579	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749.9;
	7579	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	894;
	7579	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3537;
	7579	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10353.3;
	7579	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5925.9;
	7579	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9913.1;
	7579	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1631.2;
	7580	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948.5;
	7580	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.4;
	7580	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	872.7;
	7580	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3434.1;
	7580	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9938.1;
	7580	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5739.9;
	7580	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9629.1;
	7580	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1596;
	7581	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.7;
	7581	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714.1;
	7581	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	858.6;
	7581	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3352.5;
	7581	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9612.4;
	7581	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5555.7;
	7581	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9372.1;
	7581	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1560.9;
	7582	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.8;
	7582	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	692.1;
	7582	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	831.9;
	7582	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3239.8;
	7582	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9213;
	7582	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5304.5;
	7582	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9046.3;
	7582	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1468.4;
	7583	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.7;
	7583	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	665.2;
	7583	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	795.9;
	7583	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3101;
	7583	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8650.7;
	7583	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5011.5;
	7583	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8677.4;
	7583	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1378.9;
	7584	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.8;
	7584	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	632.7;
	7584	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	749.9;
	7584	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2894.8;
	7584	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7953;
	7584	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4637.4;
	7584	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8160.5;
	7584	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1292.8;
	7585	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.3;
	7585	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	607.2;
	7585	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	723.3;
	7585	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2719.5;
	7585	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7452.9;
	7585	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4330.9;
	7585	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7760;
	7585	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1225.9;
	7586	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.3;
	7586	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.3;
	7586	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	704.2;
	7586	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2584.4;
	7586	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7082;
	7586	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4105;
	7586	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7498;
	7586	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1168.5;
	7587	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.1;
	7587	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586.8;
	7587	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	698.1;
	7587	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2512.9;
	7587	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6898.6;
	7587	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3968;
	7587	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7380.7;
	7587	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1087.6;
	7588	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.2;
	7588	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.4;
	7588	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	695.2;
	7588	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2464.2;
	7588	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6795.1;
	7588	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3902.7;
	7588	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7307.7;
	7588	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1054.3;
	7589	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.4;
	7589	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	591.9;
	7589	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	697.7;
	7589	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2449.6;
	7589	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6809.5;
	7589	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3894.6;
	7589	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7294.5;
	7589	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1053.7;
	7590	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.5;
	7590	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	602;
	7590	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	712.2;
	7590	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2493.6;
	7590	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7029.9;
	7590	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3993;
	7590	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7408.6;
	7590	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1132.4;
	7591	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.1;
	7591	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	631.5;
	7591	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	740.6;
	7591	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2584.4;
	7591	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7467.7;
	7591	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4218.6;
	7591	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7658.3;
	7591	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1175.3;
	7592	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902;
	7592	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	648.4;
	7592	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	770;
	7592	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2641.5;
	7592	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7837.4;
	7592	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4421.9;
	7592	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7845.2;
	7592	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1222.7;
	7593	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.8;
	7593	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	671.1;
	7593	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	802.5;
	7593	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2816.9;
	7593	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8423.3;
	7593	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4766.3;
	7593	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8304.1;
	7593	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1355.1;
	7594	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927;
	7594	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	692.8;
	7594	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	827.8;
	7594	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2986.6;
	7594	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8845.5;
	7594	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5011.3;
	7594	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8719.6;
	7594	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1385.2;
	7595	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.6;
	7595	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	699.3;
	7595	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	839.6;
	7595	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3122.2;
	7595	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9082.6;
	7595	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5208.8;
	7595	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9113.5;
	7595	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1451.3;
	7596	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.1;
	7596	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	700.8;
	7596	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	839.3;
	7596	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3216.6;
	7596	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9182.7;
	7596	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5340.4;
	7596	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9478.4;
	7596	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1486.5;
	7597	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.6;
	7597	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	698.3;
	7597	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.3;
	7597	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3272.3;
	7597	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9217.7;
	7597	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5450.7;
	7597	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9771.3;
	7597	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1514.4;
	7598	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.8;
	7598	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	693.6;
	7598	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	842.7;
	7598	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3314.8;
	7598	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9208.2;
	7598	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5541.7;
	7598	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9964.9;
	7598	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1540.9;
	7599	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.3;
	7599	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	700.1;
	7599	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838.9;
	7599	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3328;
	7599	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9212.7;
	7599	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5563.9;
	7599	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9983.3;
	7599	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1556.7;
	7600	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.8;
	7600	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	699.5;
	7600	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	841.7;
	7600	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3309.1;
	7600	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9197.8;
	7600	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5502.7;
	7600	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9851.4;
	7600	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1523.4;
	7601	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.5;
	7601	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	692.7;
	7601	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849.5;
	7601	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3277.3;
	7601	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9166.2;
	7601	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5487.8;
	7601	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9689.2;
	7601	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1507.2;
	7602	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.4;
	7602	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	704.9;
	7602	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	862.8;
	7602	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3321;
	7602	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9396.7;
	7602	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5595.8;
	7602	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9803;
	7602	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1551.1;
	7603	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.6;
	7603	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.6;
	7603	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	894.2;
	7603	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3446.2;
	7603	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9612.4;
	7603	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5779.4;
	7603	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9882.3;
	7603	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1574.4;
	7604	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.9;
	7604	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	701.1;
	7604	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	877.3;
	7604	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3385.9;
	7604	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9352.3;
	7604	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5626.1;
	7604	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9666.8;
	7604	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1526.5;
	7605	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.1;
	7605	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	686.9;
	7605	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	858;
	7605	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3302.7;
	7605	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9073.6;
	7605	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5460.4;
	7605	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9496.2;
	7605	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1427.7;
	7606	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920.2;
	7606	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	668.6;
	7606	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	834.6;
	7606	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3191.2;
	7606	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8700.3;
	7606	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5202.7;
	7606	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9179.6;
	7606	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1398.1;
	7607	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.2;
	7607	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	647.4;
	7607	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	804.1;
	7607	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3052.1;
	7607	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8233.5;
	7607	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4905.9;
	7607	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8793.2;
	7607	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1361.5;
	7608	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	895.7;
	7608	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	619.2;
	7608	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	770.4;
	7608	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2890.7;
	7608	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7703.4;
	7608	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4558.4;
	7608	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8352.7;
	7608	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1279;
	7609	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.7;
	7609	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	603.8;
	7609	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	744.3;
	7609	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2709.2;
	7609	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7224.5;
	7609	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4246.9;
	7609	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7954.1;
	7609	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1207.4;
	7610	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.4;
	7610	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.4;
	7610	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	718.4;
	7610	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2591.2;
	7610	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6920.2;
	7610	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4039.8;
	7610	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7672.2;
	7610	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1163.5;
	7611	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.4;
	7611	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	588.5;
	7611	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	698.6;
	7611	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2511.3;
	7611	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6731.3;
	7611	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3885.3;
	7611	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7464.7;
	7611	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1150.8;
	7612	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.5;
	7612	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	584.9;
	7612	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	688.8;
	7612	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2458.8;
	7612	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6625;
	7612	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3809.2;
	7612	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7322.4;
	7612	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1140.3;
	7613	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	875.6;
	7613	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	587.3;
	7613	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	689.4;
	7613	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2438.5;
	7613	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6624.9;
	7613	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3799.3;
	7613	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7272.3;
	7613	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1132.7;
	7614	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.7;
	7614	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	602.8;
	7614	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	701;
	7614	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2462.5;
	7614	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6784.8;
	7614	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3843.5;
	7614	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7324.4;
	7614	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1151.3;
	7615	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.6;
	7615	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	628.3;
	7615	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	729.5;
	7615	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2523.1;
	7615	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7103.4;
	7615	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3977.1;
	7615	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7451.6;
	7615	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1128.4;
	7616	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.3;
	7616	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	639.7;
	7616	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	755.6;
	7616	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2537.7;
	7616	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7407.2;
	7616	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4154.7;
	7616	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7484.7;
	7616	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1267.8;
	7617	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.4;
	7617	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	668.4;
	7617	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	787.8;
	7617	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2712.4;
	7617	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7968.5;
	7617	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4436.9;
	7617	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7862.5;
	7617	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1333.3;
	7618	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.9;
	7618	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	677.6;
	7618	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	802.8;
	7618	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2889.8;
	7618	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8378.1;
	7618	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4731.5;
	7618	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8188.1;
	7618	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1388.1;
	7619	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.4;
	7619	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	672.6;
	7619	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	803.9;
	7619	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3013.2;
	7619	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8608;
	7619	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4894.8;
	7619	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8435.4;
	7619	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1396.4;
	7620	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.4;
	7620	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	660.9;
	7620	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	805.5;
	7620	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3082.9;
	7620	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8761.1;
	7620	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5004.7;
	7620	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8575.3;
	7620	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1406.4;
	7621	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.4;
	7621	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	663.3;
	7621	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	815.9;
	7621	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3109.5;
	7621	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8922.6;
	7621	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5145.2;
	7621	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8634.6;
	7621	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1428;
	7622	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.7;
	7622	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	667.6;
	7622	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	825.7;
	7622	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3142.1;
	7622	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9056.2;
	7622	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5323;
	7622	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8666.3;
	7622	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1453.2;
	7623	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.8;
	7623	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	670.4;
	7623	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	831.7;
	7623	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3145.1;
	7623	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9166.2;
	7623	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5466.6;
	7623	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8692.5;
	7623	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1463.4;
	7624	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.8;
	7624	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	673;
	7624	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838.5;
	7624	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3147.6;
	7624	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9253.1;
	7624	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5570.3;
	7624	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8716.9;
	7624	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1459.6;
	7625	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.5;
	7625	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	676.7;
	7625	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	842.1;
	7625	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3145.7;
	7625	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9276.7;
	7625	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5612.4;
	7625	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8751.7;
	7625	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1495.5;
	7626	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.2;
	7626	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	706.8;
	7626	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	858;
	7626	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3231.8;
	7626	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9603;
	7626	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5757.5;
	7626	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9045.1;
	7626	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1564.9;
	7627	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.8;
	7627	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.6;
	7627	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.3;
	7627	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3415.8;
	7627	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9923.2;
	7627	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5994.6;
	7627	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9375.7;
	7627	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1591.2;
	7628	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.3;
	7628	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.2;
	7628	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884.1;
	7628	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3450;
	7628	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9913.2;
	7628	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5944.9;
	7628	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9316.3;
	7628	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1598.1;
	7629	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.6;
	7629	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	716.8;
	7629	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.5;
	7629	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3379.9;
	7629	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9591.3;
	7629	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5685.1;
	7629	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9084.3;
	7629	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1533.6;
	7630	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.2;
	7630	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	689.9;
	7630	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	828.4;
	7630	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3267.4;
	7630	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9103.4;
	7630	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5376.8;
	7630	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8735;
	7630	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1441.5;
	7631	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.2;
	7631	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	651.3;
	7631	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	781.7;
	7631	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3057.4;
	7631	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8386.4;
	7631	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4922;
	7631	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8253;
	7631	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1300.9;
	7632	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.3;
	7632	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	613.9;
	7632	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	737.2;
	7632	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2797.7;
	7632	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7648.3;
	7632	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4466.8;
	7632	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7741.1;
	7632	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1260.8;
	7633	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	862.9;
	7633	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586.9;
	7633	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	703.5;
	7633	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2611.8;
	7633	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7178.2;
	7633	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4155.4;
	7633	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7349;
	7633	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1218.4;
	7634	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855;
	7634	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	570.3;
	7634	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	685.3;
	7634	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2494.8;
	7634	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6897.4;
	7634	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3952.4;
	7634	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7176.2;
	7634	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1184.2;
	7635	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	852.8;
	7635	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	568.9;
	7635	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	676.9;
	7635	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2426;
	7635	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6762.3;
	7635	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3861.7;
	7635	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7120.6;
	7635	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1158.7;
	7636	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	853.2;
	7636	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	578.3;
	7636	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	675;
	7636	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2403.9;
	7636	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6757.2;
	7636	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3783.8;
	7636	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7157.9;
	7636	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1170.1;
	7637	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	854.7;
	7637	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	580.7;
	7637	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	683.9;
	7637	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2419.7;
	7637	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6964.4;
	7637	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3880.4;
	7637	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7365.6;
	7637	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1201.8;
	7638	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	869.6;
	7638	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	607.6;
	7638	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	719.6;
	7638	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2592.4;
	7638	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7615.6;
	7638	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4269.1;
	7638	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7873.3;
	7638	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1296.2;
	7639	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.7;
	7639	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	676.7;
	7639	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	799.4;
	7639	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2943.3;
	7639	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8763.3;
	7639	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4908.4;
	7639	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8564.6;
	7639	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1484.9;
	7640	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.3;
	7640	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	698.3;
	7640	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	826.9;
	7640	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3000;
	7640	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9123.8;
	7640	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5089.5;
	7640	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8685.6;
	7640	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1520;
	7641	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921;
	7641	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	702.9;
	7641	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	825.1;
	7641	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3040.5;
	7641	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9280.1;
	7641	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5150;
	7641	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8913.3;
	7641	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1535.4;
	7642	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.4;
	7642	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	712.7;
	7642	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	842.9;
	7642	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3155;
	7642	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9594.6;
	7642	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5294.2;
	7642	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9242.1;
	7642	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1552;
	7643	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.4;
	7643	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720.3;
	7643	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859;
	7643	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3299.8;
	7643	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9918.5;
	7643	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5500.7;
	7643	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9579.3;
	7643	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1598.5;
	7644	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.6;
	7644	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	732.5;
	7644	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	870.6;
	7644	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3457.3;
	7644	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10181.1;
	7644	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5684;
	7644	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9901.1;
	7644	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1622.7;
	7645	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.3;
	7645	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.2;
	7645	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	879.5;
	7645	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3552.2;
	7645	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10446.6;
	7645	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5862.1;
	7645	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10151.3;
	7645	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1652.4;
	7646	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.8;
	7646	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	748.8;
	7646	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	901.5;
	7646	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3650.7;
	7646	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10735.9;
	7646	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6047.1;
	7646	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10414.6;
	7646	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1684.7;
	7647	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951.5;
	7647	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.4;
	7647	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	920.1;
	7647	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3702.4;
	7647	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10919.4;
	7647	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6219.3;
	7647	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10624.2;
	7647	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1729;
	7648	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.9;
	7648	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	757.7;
	7648	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	943.7;
	7648	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3730.5;
	7648	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10982.2;
	7648	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6365.3;
	7648	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10730;
	7648	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1728.8;
	7649	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.4;
	7649	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	759.2;
	7649	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	949.7;
	7649	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3724.8;
	7649	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10956.8;
	7649	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6433.8;
	7649	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10680.4;
	7649	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1709.3;
	7650	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.3;
	7650	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	778.6;
	7650	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	947.2;
	7650	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3693.2;
	7650	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11020.5;
	7650	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6437.7;
	7650	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10598;
	7650	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1741.9;
	7651	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.2;
	7651	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	799.2;
	7651	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	962.9;
	7651	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3837.7;
	7651	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11201.6;
	7651	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6603.1;
	7651	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10623.3;
	7651	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1788.1;
	7652	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.9;
	7652	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	785.2;
	7652	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	949.9;
	7652	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3800.4;
	7652	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10895.9;
	7652	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6435.7;
	7652	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10426.1;
	7652	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1662;
	7653	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.1;
	7653	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	754.5;
	7653	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	918.1;
	7653	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3704.3;
	7653	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10453.2;
	7653	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6163.8;
	7653	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10087;
	7653	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1630;
	7654	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.2;
	7654	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.2;
	7654	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	871;
	7654	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3521.7;
	7654	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9776.7;
	7654	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5752.6;
	7654	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9603;
	7654	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1612.1;
	7655	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.5;
	7655	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	671.1;
	7655	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	804.3;
	7655	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3226.9;
	7655	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8862.8;
	7655	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5223.3;
	7655	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8966.2;
	7655	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1451.5;
	7656	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.2;
	7656	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	632.9;
	7656	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	743.2;
	7656	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2913.4;
	7656	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8025;
	7656	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4685.9;
	7656	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8309.7;
	7656	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1354.6;
	7657	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	858.1;
	7657	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	619.2;
	7657	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	711;
	7657	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2693.3;
	7657	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7412.6;
	7657	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4316.6;
	7657	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7945.3;
	7657	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1270.7;
	7658	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	853.4;
	7658	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	607.7;
	7658	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	695.9;
	7658	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2562.5;
	7658	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7106.1;
	7658	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4064.9;
	7658	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7702.7;
	7658	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1221.4;
	7659	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	857.5;
	7659	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	604.4;
	7659	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	692.6;
	7659	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2474.8;
	7659	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6961.3;
	7659	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3941.8;
	7659	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7577.2;
	7659	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1104.1;
	7660	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	854.5;
	7660	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	601;
	7660	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	693.9;
	7660	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2432.1;
	7660	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6933.7;
	7660	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3910.4;
	7660	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7544.7;
	7660	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1096.9;
	7661	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	858.9;
	7661	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	612.9;
	7661	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	701.6;
	7661	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2460.2;
	7661	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7130.6;
	7661	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4019.9;
	7661	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7694.8;
	7661	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1138.6;
	7662	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.9;
	7662	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	636.9;
	7662	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	737.4;
	7662	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2608.2;
	7662	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7800.8;
	7662	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4374.4;
	7662	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8242;
	7662	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1270.1;
	7663	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.4;
	7663	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	703.5;
	7663	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	816.5;
	7663	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2962.1;
	7663	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8953.1;
	7663	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5014.2;
	7663	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8939.1;
	7663	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1436.4;
	7664	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.2;
	7664	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720.5;
	7664	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.2;
	7664	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3006.8;
	7664	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9318.2;
	7664	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5166;
	7664	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9081.6;
	7664	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1480.1;
	7665	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.9;
	7665	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723.4;
	7665	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.7;
	7665	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3065.5;
	7665	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9411.5;
	7665	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5249.6;
	7665	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9351.5;
	7665	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1514.9;
	7666	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.7;
	7666	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.6;
	7666	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	836;
	7666	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3239.8;
	7666	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9635;
	7666	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5448.9;
	7666	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9705.3;
	7666	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1535.1;
	7667	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.6;
	7667	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	745.9;
	7667	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	840.3;
	7667	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3431.4;
	7667	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9899.7;
	7667	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5663.5;
	7667	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10090.5;
	7667	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1598.9;
	7668	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.3;
	7668	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	761.3;
	7668	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.9;
	7668	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3663.7;
	7668	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10182.4;
	7668	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5883.5;
	7668	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10490.5;
	7668	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1645.7;
	7669	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.9;
	7669	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772.4;
	7669	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	874.4;
	7669	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3890;
	7669	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10487.8;
	7669	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6158.4;
	7669	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10844;
	7669	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1693.2;
	7670	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.9;
	7670	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	780.3;
	7670	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	898.2;
	7670	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4106.5;
	7670	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10802.9;
	7670	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6431.9;
	7670	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11244.4;
	7670	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1756.7;
	7671	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.8;
	7671	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	786.5;
	7671	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	918.2;
	7671	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4251.7;
	7671	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11077.4;
	7671	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6653.5;
	7671	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11526.2;
	7671	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1786.3;
	7672	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.6;
	7672	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	787;
	7672	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	933.6;
	7672	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4310.2;
	7672	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11222.3;
	7672	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6788.6;
	7672	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11605.8;
	7672	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1746.3;
	7673	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.7;
	7673	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	789.7;
	7673	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940.2;
	7673	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4254.3;
	7673	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11176.9;
	7673	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6781;
	7673	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11387.3;
	7673	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1810.2;
	7674	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.7;
	7674	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	785.9;
	7674	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	930.3;
	7674	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4082.1;
	7674	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11162.3;
	7674	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6673.7;
	7674	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11091.2;
	7674	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1741.4;
	7675	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.2;
	7675	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	796.6;
	7675	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	938.9;
	7675	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4082.6;
	7675	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11271.9;
	7675	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6750.9;
	7675	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10902.8;
	7675	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1777.6;
	7676	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.8;
	7676	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777;
	7676	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	917.4;
	7676	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3932.8;
	7676	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10949.7;
	7676	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6484.7;
	7676	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10502;
	7676	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1701;
	7677	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.8;
	7677	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	750.7;
	7677	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889.5;
	7677	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3766.3;
	7677	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10507.9;
	7677	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6196.6;
	7677	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10096;
	7677	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1619.6;
	7678	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917;
	7678	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	715.4;
	7678	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	845.9;
	7678	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3557.3;
	7678	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9850.6;
	7678	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5768;
	7678	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9572.7;
	7678	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1552;
	7679	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.2;
	7679	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	672.6;
	7679	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	789.9;
	7679	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3269.2;
	7679	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8973.8;
	7679	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5212.8;
	7679	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8890.9;
	7679	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1420.5;
	7680	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	869.3;
	7680	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	629.9;
	7680	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	738.2;
	7680	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2958.1;
	7680	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8104.8;
	7680	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4671.8;
	7680	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8245.5;
	7680	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1295.7;
	7681	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	857.6;
	7681	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	605.7;
	7681	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	707.8;
	7681	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2740.6;
	7681	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7481;
	7681	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4299.4;
	7681	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7774.5;
	7681	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1188.2;
	7682	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	851.3;
	7682	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.9;
	7682	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	693.4;
	7682	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2611.7;
	7682	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7138.9;
	7682	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4089.5;
	7682	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7537.9;
	7682	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1191.1;
	7683	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	848.7;
	7683	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	585;
	7683	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	684.5;
	7683	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2525.5;
	7683	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6948.2;
	7683	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3967.3;
	7683	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7398.5;
	7683	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1138.6;
	7684	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	848.8;
	7684	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	583.8;
	7684	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	685.6;
	7684	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2472;
	7684	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6896.1;
	7684	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3933.9;
	7684	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7369.7;
	7684	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1118;
	7685	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	860;
	7685	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	599.4;
	7685	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	697.4;
	7685	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2497.8;
	7685	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7119.4;
	7685	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4007.5;
	7685	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7567.1;
	7685	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1125.7;
	7686	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.2;
	7686	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	624.8;
	7686	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	733.4;
	7686	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2650;
	7686	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7779.4;
	7686	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4369.5;
	7686	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8109;
	7686	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1201.6;
	7687	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.8;
	7687	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	692;
	7687	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	815.2;
	7687	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3003.1;
	7687	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8920.7;
	7687	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5030.1;
	7687	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8822.5;
	7687	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1384.8;
	7688	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.2;
	7688	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714.5;
	7688	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	845;
	7688	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3042.2;
	7688	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9269.5;
	7688	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5154.9;
	7688	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8980.6;
	7688	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1418.7;
	7689	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.6;
	7689	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	724.9;
	7689	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	840.8;
	7689	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3095.2;
	7689	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9423.4;
	7689	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5221.4;
	7689	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9242.6;
	7689	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1438.2;
	7690	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.5;
	7690	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	732.6;
	7690	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849.7;
	7690	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3265;
	7690	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9738.7;
	7690	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5409.4;
	7690	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9602.8;
	7690	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1484.6;
	7691	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.5;
	7691	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.9;
	7691	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	864;
	7691	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3470.2;
	7691	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10104.2;
	7691	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5639.4;
	7691	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9975.8;
	7691	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1526.5;
	7692	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.2;
	7692	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	769.2;
	7692	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	881.6;
	7692	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3704.9;
	7692	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10476.1;
	7692	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5864.9;
	7692	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10349.8;
	7692	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1590.6;
	7693	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.7;
	7693	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	780.6;
	7693	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	894;
	7693	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3926.6;
	7693	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10880.8;
	7693	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6088;
	7693	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10759.2;
	7693	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1660.4;
	7694	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.1;
	7694	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	803.8;
	7694	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	917;
	7694	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4131.8;
	7694	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11297.7;
	7694	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6297.3;
	7694	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11207.3;
	7694	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1728.6;
	7695	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.5;
	7695	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	814.6;
	7695	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	934.3;
	7695	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4276.2;
	7695	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11598.8;
	7695	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6507;
	7695	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11526.1;
	7695	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1770.2;
	7696	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	979.4;
	7696	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	815.9;
	7696	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	948.4;
	7696	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4300;
	7696	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11709.5;
	7696	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6628.9;
	7696	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11685.8;
	7696	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1774.9;
	7697	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	982.3;
	7697	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	814.2;
	7697	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	949.6;
	7697	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4244.4;
	7697	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11642.3;
	7697	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6620.2;
	7697	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11496.5;
	7697	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1767.4;
	7698	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.3;
	7698	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	811.2;
	7698	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940.1;
	7698	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4079.2;
	7698	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11605.5;
	7698	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6541.3;
	7698	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11290.2;
	7698	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1752;
	7699	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	987;
	7699	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	822.1;
	7699	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	955.6;
	7699	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4119;
	7699	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11741.6;
	7699	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6660.1;
	7699	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11243.6;
	7699	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1764.8;
	7700	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	976.8;
	7700	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	800.9;
	7700	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	936;
	7700	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3987.4;
	7700	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11423;
	7700	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6435.8;
	7700	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10916.5;
	7700	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1699.9;
	7701	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.7;
	7701	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	779.2;
	7701	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	912.8;
	7701	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3841.5;
	7701	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10993.7;
	7701	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6224.2;
	7701	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10543.9;
	7701	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1642.4;
	7702	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.2;
	7702	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	739.6;
	7702	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	873.8;
	7702	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3650;
	7702	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10354.9;
	7702	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5815.7;
	7702	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10019.7;
	7702	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1553.5;
	7703	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912;
	7703	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	690.6;
	7703	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	816.5;
	7703	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3368.3;
	7703	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9424.6;
	7703	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5283.2;
	7703	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9305.4;
	7703	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1422.8;
	7704	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.7;
	7704	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	650.8;
	7704	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	760.7;
	7704	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3044.8;
	7704	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8499.7;
	7704	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4717.2;
	7704	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8607.9;
	7704	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1289.2;
	7705	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.1;
	7705	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	622;
	7705	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	738.1;
	7705	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2794.9;
	7705	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7804.2;
	7705	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4398.7;
	7705	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8037.7;
	7705	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1196.5;
	7706	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	865.7;
	7706	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	606.3;
	7706	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	716.3;
	7706	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2667.5;
	7706	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7445.2;
	7706	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4160.9;
	7706	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7776.3;
	7706	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1140.9;
	7707	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	860.3;
	7707	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	598.3;
	7707	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	702.2;
	7707	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2587.5;
	7707	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7193;
	7707	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4032;
	7707	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7597.2;
	7707	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1108.2;
	7708	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	859.2;
	7708	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	596.4;
	7708	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	700.8;
	7708	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2549.4;
	7708	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7113.3;
	7708	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3996.9;
	7708	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7559.2;
	7708	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1100.4;
	7709	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866.2;
	7709	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	607.9;
	7709	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	705.5;
	7709	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2564.8;
	7709	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7277.7;
	7709	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4061.6;
	7709	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7738.1;
	7709	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1123.5;
	7710	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.1;
	7710	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	633.4;
	7710	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	740.3;
	7710	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2728.8;
	7710	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7919.4;
	7710	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4409.2;
	7710	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8314.3;
	7710	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1213.8;
	7711	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.4;
	7711	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	697.7;
	7711	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	811.1;
	7711	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3102.3;
	7711	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9037.4;
	7711	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5098.5;
	7711	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9062.6;
	7711	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1388.6;
	7712	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.4;
	7712	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	712.1;
	7712	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	833.1;
	7712	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3169.3;
	7712	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9409.9;
	7712	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5297.5;
	7712	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9230.5;
	7712	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1422.5;
	7713	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.7;
	7713	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730;
	7713	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	840.8;
	7713	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3270.3;
	7713	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9597.1;
	7713	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5379.3;
	7713	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9596.2;
	7713	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1457.1;
	7714	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.9;
	7714	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747;
	7714	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.7;
	7714	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3470.7;
	7714	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9863.9;
	7714	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5539.3;
	7714	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10083.3;
	7714	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1511.8;
	7715	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.5;
	7715	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	760.6;
	7715	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	869.1;
	7715	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3703.4;
	7715	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10145.8;
	7715	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5714.5;
	7715	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10552.6;
	7715	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1562.6;
	7716	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.4;
	7716	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	767.3;
	7716	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	880.1;
	7716	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3896.4;
	7716	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10401;
	7716	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5943.5;
	7716	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10886.2;
	7716	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1613.1;
	7717	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.2;
	7717	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	766.5;
	7717	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.8;
	7717	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4064.2;
	7717	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10691.5;
	7717	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6137.7;
	7717	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11255.6;
	7717	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1665.2;
	7718	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.9;
	7718	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	783.9;
	7718	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	903.6;
	7718	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4225.5;
	7718	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10995.4;
	7718	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6378.6;
	7718	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11632.1;
	7718	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1711.9;
	7719	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.1;
	7719	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	791.6;
	7719	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	919.2;
	7719	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4327.7;
	7719	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11175.4;
	7719	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6559.6;
	7719	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11750.9;
	7719	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1729.1;
	7720	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.5;
	7720	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	790.4;
	7720	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	918.6;
	7720	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4346.2;
	7720	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11170.9;
	7720	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6726;
	7720	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11698;
	7720	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1724.5;
	7721	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.7;
	7721	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	781.9;
	7721	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913.9;
	7721	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4286.7;
	7721	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11069.8;
	7721	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6778.2;
	7721	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11536.8;
	7721	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1713;
	7722	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.3;
	7722	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	793.2;
	7722	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	920.1;
	7722	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4159.2;
	7722	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11181.4;
	7722	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6833.5;
	7722	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11420.8;
	7722	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1719.6;
	7723	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	983.2;
	7723	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	816;
	7723	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	954.5;
	7723	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4276.4;
	7723	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11500.7;
	7723	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6925.6;
	7723	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11485.6;
	7723	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1764.3;
	7724	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	979.1;
	7724	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	808.1;
	7724	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	943.6;
	7724	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4205.1;
	7724	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11337.5;
	7724	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6803;
	7724	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11295.2;
	7724	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1730.7;
	7725	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970.9;
	7725	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	789.9;
	7725	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	924.3;
	7725	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4099.3;
	7725	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11040;
	7725	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6632.9;
	7725	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10991.8;
	7725	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1688.6;
	7726	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.5;
	7726	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	758.3;
	7726	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.8;
	7726	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3927.2;
	7726	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10532.9;
	7726	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6329.6;
	7726	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10564.6;
	7726	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1614.4;
	7727	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.8;
	7727	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711;
	7727	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	840.8;
	7727	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3655.6;
	7727	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9643.7;
	7727	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5837.6;
	7727	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9887.1;
	7727	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1502.8;
	7728	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.6;
	7728	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	664.8;
	7728	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	789.3;
	7728	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3347.1;
	7728	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8801.1;
	7728	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5296.4;
	7728	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9265.1;
	7728	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1397.8;
	7729	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.4;
	7729	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	639.9;
	7729	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	753.9;
	7729	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3106.3;
	7729	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8222.9;
	7729	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4832;
	7729	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8787.1;
	7729	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1328.9;
	7730	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.6;
	7730	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	622.3;
	7730	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	728.3;
	7730	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2953.5;
	7730	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7842.4;
	7730	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4604.7;
	7730	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8476.4;
	7730	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1224.9;
	7731	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.4;
	7731	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	611.7;
	7731	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	713;
	7731	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2861.6;
	7731	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7634.4;
	7731	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4474.7;
	7731	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8277;
	7731	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1213.6;
	7732	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	881.7;
	7732	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	609.2;
	7732	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	706.7;
	7732	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2798.1;
	7732	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7552.4;
	7732	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4421.4;
	7732	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8163.5;
	7732	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1246.2;
	7733	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.6;
	7733	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	622.7;
	7733	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	708.8;
	7733	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2799.3;
	7733	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7698.9;
	7733	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4471.4;
	7733	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8268.2;
	7733	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1256.8;
	7734	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.7;
	7734	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	644.5;
	7734	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	740.7;
	7734	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2949.4;
	7734	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8308.6;
	7734	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4812;
	7734	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8771.6;
	7734	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1342.7;
	7735	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.2;
	7735	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	706.7;
	7735	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	816.2;
	7735	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3307.2;
	7735	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9311.3;
	7735	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5454;
	7735	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9454.1;
	7735	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1482.7;
	7736	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.8;
	7736	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	734.5;
	7736	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	845;
	7736	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3361.5;
	7736	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9566.5;
	7736	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5691.8;
	7736	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9615.4;
	7736	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1528;
	7737	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.2;
	7737	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.8;
	7737	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.4;
	7737	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3473.8;
	7737	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9613.7;
	7737	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5798.6;
	7737	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9966.6;
	7737	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1594.4;
	7738	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.5;
	7738	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	745.6;
	7738	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	874.2;
	7738	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3703.7;
	7738	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9762.6;
	7738	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6047.6;
	7738	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10422.4;
	7738	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1660.8;
	7739	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.9;
	7739	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.9;
	7739	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	881.3;
	7739	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3905.9;
	7739	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9893.5;
	7739	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6351.2;
	7739	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10798.6;
	7739	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1673.2;
	7740	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.2;
	7740	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749;
	7740	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	880.8;
	7740	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4100.1;
	7740	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9928.2;
	7740	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6374.9;
	7740	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11000.5;
	7740	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1666.1;
	7741	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.7;
	7741	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.4;
	7741	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884;
	7741	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4270.8;
	7741	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9898.7;
	7741	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6370.3;
	7741	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11271.1;
	7741	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1615.8;
	7742	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.4;
	7742	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746.8;
	7742	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.2;
	7742	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4426.6;
	7742	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9918.2;
	7742	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6154.9;
	7742	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11462.4;
	7742	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1543.3;
	7743	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.7;
	7743	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.9;
	7743	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	888.1;
	7743	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4531;
	7743	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9817.8;
	7743	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5978.5;
	7743	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11238.2;
	7743	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1505.1;
	7744	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.8;
	7744	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728;
	7744	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	875.9;
	7744	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4479.2;
	7744	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9640.9;
	7744	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5805.5;
	7744	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10725.9;
	7744	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1539.1;
	7745	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.3;
	7745	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	722;
	7745	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	871.4;
	7745	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4310.3;
	7745	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9510.5;
	7745	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5633.3;
	7745	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10255.4;
	7745	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1534.1;
	7746	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.3;
	7746	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	739.2;
	7746	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	876.4;
	7746	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4127.5;
	7746	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9764.7;
	7746	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5603.6;
	7746	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10135;
	7746	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1550.3;
	7747	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.8;
	7747	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	760.5;
	7747	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	903.6;
	7747	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4081.8;
	7747	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10022.3;
	7747	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5781.2;
	7747	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9929;
	7747	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1593.5;
	7748	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.7;
	7748	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755.5;
	7748	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	891.5;
	7748	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3853;
	7748	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9821.1;
	7748	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5616.8;
	7748	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9599.5;
	7748	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1573.7;
	7749	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.3;
	7749	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	750.9;
	7749	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878.9;
	7749	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3674.8;
	7749	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9620.4;
	7749	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5436;
	7749	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9310.4;
	7749	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1552;
	7750	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960;
	7750	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.5;
	7750	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	858.2;
	7750	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3465.9;
	7750	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9304.1;
	7750	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5205.4;
	7750	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8951.2;
	7750	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1477.5;
	7751	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.6;
	7751	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.3;
	7751	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	834.1;
	7751	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3191.6;
	7751	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8832.9;
	7751	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4930.5;
	7751	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8517.1;
	7751	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1409.6;
	7752	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.6;
	7752	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	689.6;
	7752	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	800.4;
	7752	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2950.4;
	7752	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8299.4;
	7752	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4571;
	7752	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8044.8;
	7752	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1345.3;
	7753	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.1;
	7753	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	674.7;
	7753	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	803.5;
	7753	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2714.2;
	7753	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7910.5;
	7753	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4314.4;
	7753	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7626.7;
	7753	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1283.2;
	7754	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924;
	7754	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	669.6;
	7754	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	791.1;
	7754	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2569.9;
	7754	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7658;
	7754	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4138.5;
	7754	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7383.5;
	7754	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1242;
	7755	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.5;
	7755	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	671;
	7755	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	792.3;
	7755	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2478.9;
	7755	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7554.5;
	7755	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4040.2;
	7755	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7253.5;
	7755	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1167.2;
	7756	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.5;
	7756	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	685.4;
	7756	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	799.2;
	7756	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2434.4;
	7756	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7554.4;
	7756	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4007;
	7756	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7214.9;
	7756	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1141.7;
	7757	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.7;
	7757	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	698.9;
	7757	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	817.2;
	7757	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2426.6;
	7757	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7696.7;
	7757	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4057.2;
	7757	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7273.5;
	7757	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1164.6;
	7758	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.2;
	7758	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.1;
	7758	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849.5;
	7758	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2477.6;
	7758	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8078;
	7758	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4224.6;
	7758	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7448.8;
	7758	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1264.6;
	7759	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.9;
	7759	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	768.2;
	7759	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897.9;
	7759	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2580;
	7759	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8707.5;
	7759	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4520.6;
	7759	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7697.3;
	7759	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1342.5;
	7760	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.3;
	7760	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	785.9;
	7760	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	949.2;
	7760	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2655.1;
	7760	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9317.7;
	7760	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4818.2;
	7760	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7856.2;
	7760	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1428.9;
	7761	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981.3;
	7761	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	811.8;
	7761	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	976.4;
	7761	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2824.8;
	7761	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9825.8;
	7761	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5115.9;
	7761	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8287.5;
	7761	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1552.9;
	7762	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.6;
	7762	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	801.3;
	7762	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	985.6;
	7762	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2968.5;
	7762	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10038.5;
	7762	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5274.6;
	7762	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8605;
	7762	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1553.5;
	7763	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.6;
	7763	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777.9;
	7763	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	956.5;
	7763	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3029.9;
	7763	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9937;
	7763	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5259.4;
	7763	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8717.3;
	7763	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1549.2;
	7764	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948.3;
	7764	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.7;
	7764	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	925.3;
	7764	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3043.3;
	7764	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9691.3;
	7764	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5203.3;
	7764	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8703.8;
	7764	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1504.2;
	7765	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.8;
	7765	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.4;
	7765	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	899.3;
	7765	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3029.5;
	7765	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9408.2;
	7765	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5080.6;
	7765	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8624.3;
	7765	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1454.6;
	7766	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.5;
	7766	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	706.8;
	7766	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878;
	7766	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3026.2;
	7766	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9147.8;
	7766	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5020.4;
	7766	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8528.7;
	7766	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1427.7;
	7767	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.4;
	7767	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	692;
	7767	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	858.8;
	7767	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3026.3;
	7767	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8931.8;
	7767	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4906.2;
	7767	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8443.4;
	7767	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1417.6;
	7768	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.6;
	7768	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	691.1;
	7768	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.3;
	7768	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3026.7;
	7768	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8842.1;
	7768	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4874;
	7768	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8412.4;
	7768	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1389.5;
	7769	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.1;
	7769	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	700.1;
	7769	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.7;
	7769	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3011.8;
	7769	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8895.4;
	7769	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4923.2;
	7769	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8389.2;
	7769	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1412.1;
	7770	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.7;
	7770	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738;
	7770	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.2;
	7770	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3051.9;
	7770	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9413;
	7770	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5148.7;
	7770	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8675;
	7770	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1525.1;
	7771	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.8;
	7771	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	774.2;
	7771	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	911.1;
	7771	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3226.4;
	7771	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9931.6;
	7771	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5508.2;
	7771	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8965.8;
	7771	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1618.2;
	7772	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.8;
	7772	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777.5;
	7772	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	918.9;
	7772	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3199.5;
	7772	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9949.5;
	7772	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5545.8;
	7772	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8912.7;
	7772	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1618.6;
	7773	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.5;
	7773	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	782.5;
	7773	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	920.8;
	7773	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3157.4;
	7773	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9925.6;
	7773	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5513.2;
	7773	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8846.1;
	7773	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1565.5;
	7774	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.1;
	7774	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776.1;
	7774	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	910.4;
	7774	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3090;
	7774	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9781;
	7774	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5404.4;
	7774	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8644.5;
	7774	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1579.1;
	7775	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.8;
	7775	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	756.2;
	7775	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889.5;
	7775	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2991.7;
	7775	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9495.1;
	7775	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5205;
	7775	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8394.7;
	7775	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1568.1;
	7776	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.1;
	7776	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	737;
	7776	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	864.9;
	7776	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2866.7;
	7776	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9146.4;
	7776	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4995.1;
	7776	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8133.9;
	7776	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1507.9;
	7777	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.7;
	7777	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714.6;
	7777	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	834.4;
	7777	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2763.5;
	7777	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8784.9;
	7777	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4828.6;
	7777	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7909.8;
	7777	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1454.6;
	7778	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.8;
	7778	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711.2;
	7778	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824.7;
	7778	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2687.7;
	7778	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8637.6;
	7778	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4727.1;
	7778	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7792.2;
	7778	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1443.2;
	7779	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.9;
	7779	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714.3;
	7779	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	822.6;
	7779	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2639.2;
	7779	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8582.5;
	7779	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4704.6;
	7779	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7705.1;
	7779	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1453.4;
	7780	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926;
	7780	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720.2;
	7780	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	827.7;
	7780	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2613.9;
	7780	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8610.6;
	7780	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4721.6;
	7780	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7676.2;
	7780	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1470.9;
	7781	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.2;
	7781	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	724.2;
	7781	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838.2;
	7781	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2644.3;
	7781	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8767.7;
	7781	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4796.5;
	7781	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7742.8;
	7781	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1490.9;
	7782	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.4;
	7782	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746;
	7782	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.9;
	7782	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2729.4;
	7782	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9063.3;
	7782	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5011.3;
	7782	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7885.9;
	7782	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1540.5;
	7783	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.1;
	7783	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	773.9;
	7783	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	902.3;
	7783	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2852.9;
	7783	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9605.2;
	7783	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5326.8;
	7783	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8145.2;
	7783	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1557.7;
	7784	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.7;
	7784	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	801.4;
	7784	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	941.1;
	7784	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2902.4;
	7784	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10071;
	7784	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5622.7;
	7784	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8303.1;
	7784	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1734.9;
	7785	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.5;
	7785	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	818.2;
	7785	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	954.7;
	7785	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3047.3;
	7785	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10367;
	7785	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5742.4;
	7785	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8621.6;
	7785	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1748.3;
	7786	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.3;
	7786	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	796;
	7786	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	935.2;
	7786	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3107.5;
	7786	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10281.2;
	7786	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5643.4;
	7786	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8731.9;
	7786	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1710.1;
	7787	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.1;
	7787	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	764.8;
	7787	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	898.5;
	7787	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3076.3;
	7787	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9982;
	7787	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5443.2;
	7787	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8662.3;
	7787	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1612.8;
	7788	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.2;
	7788	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.4;
	7788	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	867.9;
	7788	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3034.4;
	7788	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9645.2;
	7788	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5230.5;
	7788	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8535;
	7788	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1537.8;
	7789	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.7;
	7789	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	713.4;
	7789	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.4;
	7789	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2988.8;
	7789	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9328.6;
	7789	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5083.7;
	7789	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8410.1;
	7789	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1482.2;
	7790	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.4;
	7790	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	694.2;
	7790	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	840.5;
	7790	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2984.6;
	7790	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9071.7;
	7790	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4965.7;
	7790	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8322.7;
	7790	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1472.1;
	7791	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.7;
	7791	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	682.8;
	7791	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	831.6;
	7791	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2980;
	7791	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8865.4;
	7791	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4905;
	7791	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8214.9;
	7791	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1457.5;
	7792	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.8;
	7792	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	683.8;
	7792	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	830.5;
	7792	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2998.5;
	7792	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8810.9;
	7792	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4898.4;
	7792	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8208.8;
	7792	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1435.4;
	7793	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.2;
	7793	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	697.1;
	7793	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	837.5;
	7793	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2982.6;
	7793	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8863.1;
	7793	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4963.6;
	7793	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8238.2;
	7793	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1479.9;
	7794	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894;
	7794	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	732.9;
	7794	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	862.8;
	7794	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3039.2;
	7794	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9394.5;
	7794	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5248.1;
	7794	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8591.2;
	7794	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1591.9;
	7795	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.9;
	7795	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	765.7;
	7795	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	906.9;
	7795	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3259;
	7795	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9926.4;
	7795	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5631.2;
	7795	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8995.2;
	7795	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1668.8;
	7796	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.8;
	7796	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	759.4;
	7796	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	907.8;
	7796	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3253.7;
	7796	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9902.8;
	7796	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5668.1;
	7796	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8959.9;
	7796	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1693.2;
	7797	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.6;
	7797	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	750.3;
	7797	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	891.2;
	7797	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3188.1;
	7797	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9765.4;
	7797	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5572.7;
	7797	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8853.9;
	7797	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1670.5;
	7798	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.6;
	7798	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730;
	7798	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	867.4;
	7798	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3085.6;
	7798	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9527;
	7798	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5432.7;
	7798	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8647.7;
	7798	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1616.8;
	7799	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.5;
	7799	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	697.7;
	7799	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	829.6;
	7799	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2950.7;
	7799	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9006.4;
	7799	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5136;
	7799	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8271.1;
	7799	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1506.4;
	7800	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.5;
	7800	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	670.2;
	7800	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	796.6;
	7800	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2786.1;
	7800	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8418.2;
	7800	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4813.5;
	7800	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7855.8;
	7800	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1488.8;
	7801	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.1;
	7801	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	660.9;
	7801	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	776;
	7801	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2611.7;
	7801	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8016.5;
	7801	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4579.7;
	7801	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7566;
	7801	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1450;
	7802	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.9;
	7802	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	647.9;
	7802	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	766.8;
	7802	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2522.9;
	7802	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7826.1;
	7802	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4453.3;
	7802	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7450.3;
	7802	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1432.6;
	7803	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878;
	7803	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	649.5;
	7803	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	762.8;
	7803	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2474.5;
	7803	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7792.6;
	7803	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4429.1;
	7803	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7423;
	7803	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1414.4;
	7804	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	880.7;
	7804	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	659.4;
	7804	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	767.2;
	7804	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2464.7;
	7804	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7880.4;
	7804	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4468.2;
	7804	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7481.3;
	7804	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1438.1;
	7805	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	883.8;
	7805	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	664.2;
	7805	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	785.3;
	7805	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2508.1;
	7805	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8221.3;
	7805	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4619.9;
	7805	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7729.2;
	7805	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1478.9;
	7806	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.1;
	7806	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	702.8;
	7806	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	829.8;
	7806	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2664;
	7806	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8966.4;
	7806	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5052.4;
	7806	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8258.2;
	7806	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1587.9;
	7807	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.8;
	7807	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	766.5;
	7807	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	910.7;
	7807	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2905.6;
	7807	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9912.6;
	7807	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5595.9;
	7807	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8869.8;
	7807	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1749.6;
	7808	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942;
	7808	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	796.2;
	7808	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	945.8;
	7808	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3002.7;
	7808	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10375.3;
	7808	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5842.8;
	7808	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9072.7;
	7808	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1808;
	7809	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.5;
	7809	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	796.1;
	7809	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	924.3;
	7809	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3074.5;
	7809	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10434.7;
	7809	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5812.6;
	7809	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9233.7;
	7809	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1796.6;
	7810	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.6;
	7810	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	779;
	7810	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	903.8;
	7810	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3139.7;
	7810	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10333.7;
	7810	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5684.5;
	7810	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9322.3;
	7810	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1737.1;
	7811	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.2;
	7811	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	763.1;
	7811	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884.1;
	7811	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3212.6;
	7811	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10202.7;
	7811	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5563.4;
	7811	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9406.3;
	7811	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1707.4;
	7812	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.8;
	7812	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	750.4;
	7812	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	867.3;
	7812	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3307.7;
	7812	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10015.8;
	7812	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5442.3;
	7812	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9503;
	7812	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1660.1;
	7813	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.5;
	7813	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	734.4;
	7813	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.7;
	7813	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3385.4;
	7813	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9875.1;
	7813	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5449.1;
	7813	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9531.1;
	7813	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1628;
	7814	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910;
	7814	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.2;
	7814	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	856.7;
	7814	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3491.9;
	7814	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9821.7;
	7814	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5487.8;
	7814	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9622.5;
	7814	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1612.6;
	7815	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916;
	7815	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	726.4;
	7815	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	853.2;
	7815	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3586.7;
	7815	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9759.2;
	7815	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5525.2;
	7815	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9702.3;
	7815	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1633.9;
	7816	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	915.6;
	7816	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	722.9;
	7816	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	855;
	7816	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3635.3;
	7816	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9681.3;
	7816	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5574.4;
	7816	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9702.2;
	7816	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1619.3;
	7817	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.7;
	7817	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	724.4;
	7817	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.5;
	7817	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3620.1;
	7817	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9699.9;
	7817	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5470.1;
	7817	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9694.4;
	7817	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1610.9;
	7818	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.9;
	7818	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	748.8;
	7818	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	873.2;
	7818	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3585.1;
	7818	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10063.1;
	7818	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5739;
	7818	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9884.8;
	7818	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1680.3;
	7819	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.9;
	7819	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776.4;
	7819	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	905.1;
	7819	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3748.7;
	7819	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10396.5;
	7819	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5971.8;
	7819	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10065.5;
	7819	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1743.3;
	7820	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.3;
	7820	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	773.5;
	7820	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	907.2;
	7820	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3687.4;
	7820	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10213.7;
	7820	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5875.9;
	7820	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9884.4;
	7820	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1640.1;
	7821	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.9;
	7821	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	759.8;
	7821	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	888.4;
	7821	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3580.1;
	7821	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9939.3;
	7821	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5715.4;
	7821	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9663.3;
	7821	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1620.2;
	7822	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.1;
	7822	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735.1;
	7822	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.9;
	7822	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3437.6;
	7822	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9494.3;
	7822	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5438.3;
	7822	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9301.8;
	7822	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1624.5;
	7823	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.8;
	7823	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	694.3;
	7823	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	809;
	7823	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3229.3;
	7823	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8834.9;
	7823	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5062.8;
	7823	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8777.1;
	7823	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1490.2;
	7824	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.3;
	7824	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	654.4;
	7824	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	759.2;
	7824	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2968.2;
	7824	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8124.9;
	7824	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4637.7;
	7824	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8176;
	7824	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1415.6;
	7825	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	860.2;
	7825	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	631.5;
	7825	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	720.5;
	7825	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3052;
	7825	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7486.5;
	7825	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4328.7;
	7825	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7692.6;
	7825	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1320.8;
	7826	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	854.5;
	7826	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	618.4;
	7826	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	701.4;
	7826	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2886.3;
	7826	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7178.3;
	7826	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4140.7;
	7826	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7484.2;
	7826	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1275.9;
	7827	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	856.1;
	7827	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	611;
	7827	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	695.2;
	7827	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2785.5;
	7827	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7029.5;
	7827	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4010.8;
	7827	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7397.6;
	7827	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1161.6;
	7828	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.4;
	7828	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	604.5;
	7828	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	691.2;
	7828	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2714.5;
	7828	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6995.3;
	7828	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3942.4;
	7828	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7388;
	7828	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1154.3;
	7829	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	857.6;
	7829	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	613.3;
	7829	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	693;
	7829	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2705.3;
	7829	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7182;
	7829	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4028.3;
	7829	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7542.5;
	7829	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1196.5;
	7830	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870;
	7830	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	632.1;
	7830	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	719.2;
	7830	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2812.9;
	7830	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7680.7;
	7830	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4302;
	7830	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7977;
	7830	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1307.7;
	7831	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.7;
	7831	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	680.4;
	7831	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	783.5;
	7831	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3104.1;
	7831	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8445.8;
	7831	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4770;
	7831	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8553.5;
	7831	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1423.9;
	7832	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.2;
	7832	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	706.9;
	7832	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	820;
	7832	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3193.6;
	7832	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8946.9;
	7832	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5059.8;
	7832	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8795.6;
	7832	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1493.6;
	7833	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.6;
	7833	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	724.8;
	7833	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	827.5;
	7833	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3262.9;
	7833	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9360.4;
	7833	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5281;
	7833	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9229.8;
	7833	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1567.3;
	7834	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.6;
	7834	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742.4;
	7834	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	837.1;
	7834	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3499.5;
	7834	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9709.5;
	7834	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5485.1;
	7834	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9685.3;
	7834	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1587.5;
	7835	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900;
	7835	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.6;
	7835	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.7;
	7835	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3769.9;
	7835	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9928.7;
	7835	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5631.5;
	7835	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10073.7;
	7835	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1636.2;
	7836	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.7;
	7836	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749.1;
	7836	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.2;
	7836	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4011.3;
	7836	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9999.6;
	7836	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5739.4;
	7836	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10305;
	7836	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1643.9;
	7837	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.4;
	7837	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.5;
	7837	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	867.4;
	7837	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4198.6;
	7837	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10043.7;
	7837	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5801.5;
	7837	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10529.7;
	7837	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1654.1;
	7838	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.5;
	7838	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746.7;
	7838	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878.7;
	7838	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4338.7;
	7838	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10046.8;
	7838	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5851.6;
	7838	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10758.9;
	7838	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1672.9;
	7839	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920.1;
	7839	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744.4;
	7839	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	890.6;
	7839	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4434.4;
	7839	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10006;
	7839	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5847.5;
	7839	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10836.3;
	7839	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1666;
	7840	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.7;
	7840	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746.8;
	7840	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	910;
	7840	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4430.2;
	7840	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9954.5;
	7840	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5854.2;
	7840	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10766.7;
	7840	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1599.5;
	7841	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920;
	7841	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742.9;
	7841	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	928.3;
	7841	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4382.2;
	7841	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9866.8;
	7841	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5887.9;
	7841	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10558.3;
	7841	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1649.3;
	7842	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.1;
	7842	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	756.9;
	7842	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	941.3;
	7842	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4316.3;
	7842	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10261.4;
	7842	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6088.4;
	7842	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10740.3;
	7842	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1644.5;
	7843	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.7;
	7843	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777.3;
	7843	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	949;
	7843	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4295;
	7843	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10587.9;
	7843	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6281.4;
	7843	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10866.6;
	7843	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1713.5;
	7844	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.2;
	7844	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	769.8;
	7844	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	935.4;
	7844	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4291.7;
	7844	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10402.5;
	7844	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6215.6;
	7844	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10664.9;
	7844	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1664.1;
	7845	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.3;
	7845	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749.7;
	7845	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	924.3;
	7845	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4183.1;
	7845	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10118.4;
	7845	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6085.5;
	7845	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10453.8;
	7845	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1603.5;
	7846	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.1;
	7846	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723;
	7846	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	888.1;
	7846	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3997;
	7846	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9739.6;
	7846	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5859.2;
	7846	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10195.7;
	7846	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1579;
	7847	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910;
	7847	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	689.7;
	7847	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	830.6;
	7847	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3723.5;
	7847	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9143;
	7847	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5488.5;
	7847	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9706.8;
	7847	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1474.9;
	7848	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.7;
	7848	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	649;
	7848	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	773.1;
	7848	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3392.1;
	7848	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8396;
	7848	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5063.6;
	7848	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9119.6;
	7848	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1354.1;
	7849	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	882.4;
	7849	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	631.8;
	7849	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	731.2;
	7849	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3199.1;
	7849	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7731.6;
	7849	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4712.9;
	7849	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8576.1;
	7849	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1242.5;
	7850	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.6;
	7850	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	620.2;
	7850	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	713.3;
	7850	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3015.1;
	7850	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7296.7;
	7850	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4450.8;
	7850	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8240.5;
	7850	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1231.9;
	7851	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.3;
	7851	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	615.7;
	7851	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	706.2;
	7851	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2885.7;
	7851	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7090.8;
	7851	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4291.6;
	7851	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8060;
	7851	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1168.8;
	7852	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.5;
	7852	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	619.7;
	7852	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	709.3;
	7852	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2785.1;
	7852	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7062.4;
	7852	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4171.3;
	7852	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8016.8;
	7852	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1145.3;
	7853	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.1;
	7853	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	638;
	7853	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	719.1;
	7853	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2749;
	7853	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7250;
	7853	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4165.2;
	7853	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8101;
	7853	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1131.3;
	7854	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.2;
	7854	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	654.7;
	7854	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	751.2;
	7854	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2832.8;
	7854	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7825;
	7854	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4383.7;
	7854	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8471.9;
	7854	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1165.1;
	7855	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920.8;
	7855	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	702.8;
	7855	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	815.7;
	7855	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3097.6;
	7855	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8623.2;
	7855	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4727;
	7855	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8923.3;
	7855	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1266.1;
	7856	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.9;
	7856	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.7;
	7856	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	853.5;
	7856	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3153.7;
	7856	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9150.6;
	7856	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4885.9;
	7856	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9161.9;
	7856	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1333;
	7857	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.3;
	7857	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	761.5;
	7857	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.1;
	7857	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3157.6;
	7857	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9599.6;
	7857	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5106.2;
	7857	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9547.1;
	7857	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1402.8;
	7858	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.6;
	7858	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776.8;
	7858	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	875.1;
	7858	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3253;
	7858	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9908.4;
	7858	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5341.5;
	7858	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9750.6;
	7858	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1460.8;
	7859	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.1;
	7859	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	780.9;
	7859	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	877.4;
	7859	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3367.5;
	7859	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10058.3;
	7859	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5479.2;
	7859	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9878;
	7859	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1481.6;
	7860	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.9;
	7860	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772.7;
	7860	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	883.3;
	7860	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3522.9;
	7860	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10056.8;
	7860	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5555.4;
	7860	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10050.4;
	7860	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1500.5;
	7861	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.6;
	7861	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	759.7;
	7861	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	876.2;
	7861	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3665.8;
	7861	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9992.6;
	7861	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5587.5;
	7861	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10141.4;
	7861	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1503.4;
	7862	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925.7;
	7862	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	761.6;
	7862	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	876.4;
	7862	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3835.3;
	7862	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9916.3;
	7862	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5656.7;
	7862	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10236.5;
	7862	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1506.4;
	7863	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.1;
	7863	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.7;
	7863	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	876.8;
	7863	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3971.6;
	7863	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9846.4;
	7863	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5724.2;
	7863	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10316.4;
	7863	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1498.4;
	7864	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.6;
	7864	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744.3;
	7864	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	877.3;
	7864	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4031.6;
	7864	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9737;
	7864	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5772.5;
	7864	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10276.2;
	7864	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1489.6;
	7865	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911;
	7865	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.4;
	7865	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	875.5;
	7865	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4066.3;
	7865	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9621.7;
	7865	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5780.1;
	7865	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10010.4;
	7865	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1476.3;
	7866	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.8;
	7866	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.2;
	7866	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	890.3;
	7866	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3999.1;
	7866	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9934.8;
	7866	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5826;
	7866	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9901.5;
	7866	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1526.5;
	7867	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.3;
	7867	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	775;
	7867	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	922.8;
	7867	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3872.4;
	7867	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10261.3;
	7867	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5963.5;
	7867	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9915.7;
	7867	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1570.1;
	7868	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.8;
	7868	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	761.9;
	7868	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	906.8;
	7868	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3762.9;
	7868	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10078.7;
	7868	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5786.8;
	7868	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9665.2;
	7868	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1540.7;
	7869	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.1;
	7869	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	759;
	7869	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.2;
	7869	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3622.7;
	7869	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9890.9;
	7869	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5627.8;
	7869	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9416.2;
	7869	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1514.9;
	7870	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.2;
	7870	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744.8;
	7870	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.5;
	7870	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3449.1;
	7870	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9600.9;
	7870	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5383.8;
	7870	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9090.4;
	7870	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1468.1;
	7871	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.8;
	7871	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	721.2;
	7871	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	818.8;
	7871	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3256.9;
	7871	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9137.6;
	7871	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5079.6;
	7871	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8681.4;
	7871	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1397.7;
	7872	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.4;
	7872	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	689.4;
	7872	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	778.7;
	7872	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3012.7;
	7872	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8530.2;
	7872	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4718.6;
	7872	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8159.3;
	7872	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1298.9;
	7873	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.8;
	7873	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	659.1;
	7873	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	738.5;
	7873	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2948.2;
	7873	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7867.4;
	7873	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4389.3;
	7873	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7689.8;
	7873	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1292.3;
	7874	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.2;
	7874	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	645.9;
	7874	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	717.7;
	7874	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2777.5;
	7874	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7541.7;
	7874	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4179;
	7874	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7414.7;
	7874	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1244.1;
	7875	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.3;
	7875	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	643.8;
	7875	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	707.4;
	7875	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2659.9;
	7875	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7356.1;
	7875	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4058.7;
	7875	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7214.1;
	7875	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1235.2;
	7876	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	897;
	7876	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	644.8;
	7876	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	708.5;
	7876	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2589;
	7876	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7339.5;
	7876	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3997.6;
	7876	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7166.9;
	7876	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1238.6;
	7877	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903;
	7877	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	656.6;
	7877	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	715.1;
	7877	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2569.3;
	7877	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7467.3;
	7877	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4009.1;
	7877	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7230;
	7877	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1247.6;
	7878	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.8;
	7878	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	683.3;
	7878	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	734;
	7878	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2612.6;
	7878	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7825.4;
	7878	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4180.6;
	7878	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7393.7;
	7878	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1292.5;
	7879	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.3;
	7879	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	710.3;
	7879	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	770.7;
	7879	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2698.9;
	7879	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8367;
	7879	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4445.2;
	7879	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7646.6;
	7879	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1304.7;
	7880	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.7;
	7880	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744.7;
	7880	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	801.4;
	7880	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2740.6;
	7880	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8833.4;
	7880	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4668.7;
	7880	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7772.6;
	7880	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1468.6;
	7881	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.8;
	7881	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	769.6;
	7881	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	830.7;
	7881	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2947.2;
	7881	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9377.1;
	7881	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4941.2;
	7881	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8202.8;
	7881	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1518.4;
	7882	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.7;
	7882	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	773.6;
	7882	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	839;
	7882	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3250.7;
	7882	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9679.3;
	7882	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5160.5;
	7882	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8566.6;
	7882	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1571.3;
	7883	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.7;
	7883	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	764.3;
	7883	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	835.9;
	7883	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3566.9;
	7883	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9811;
	7883	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5283.8;
	7883	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8883.4;
	7883	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1571.2;
	7884	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.3;
	7884	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	739.6;
	7884	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	828.4;
	7884	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3823.2;
	7884	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9744.9;
	7884	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5336.1;
	7884	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9029.9;
	7884	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1540.2;
	7885	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.5;
	7885	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	699.5;
	7885	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	821.1;
	7885	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3930.4;
	7885	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9413;
	7885	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5334.5;
	7885	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9099.1;
	7885	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1473.5;
	7886	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.1;
	7886	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	670;
	7886	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	816;
	7886	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3968.7;
	7886	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9055.6;
	7886	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5274.6;
	7886	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9115.1;
	7886	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1423.3;
	7887	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889;
	7887	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	654.8;
	7887	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	813.9;
	7887	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3988.4;
	7887	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8756.5;
	7887	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5250.5;
	7887	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9094.3;
	7887	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1393;
	7888	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	881;
	7888	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	646.5;
	7888	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	815.3;
	7888	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3958.7;
	7888	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8497.7;
	7888	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5246;
	7888	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8975.1;
	7888	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1355.8;
	7889	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.8;
	7889	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	631.8;
	7889	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	823.9;
	7889	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3848.3;
	7889	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8194;
	7889	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5228;
	7889	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8676.5;
	7889	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1347.9;
	7890	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	858.5;
	7890	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	649.8;
	7890	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	846.4;
	7890	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3733.6;
	7890	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8278.2;
	7890	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5238.8;
	7890	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8599.8;
	7890	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1372.5;
	7891	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.9;
	7891	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	665.8;
	7891	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	869.5;
	7891	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3725.1;
	7891	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8400.9;
	7891	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5282.8;
	7891	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8644.9;
	7891	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1374.6;
	7892	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866.8;
	7892	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	666;
	7892	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.2;
	7892	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3654.1;
	7892	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8308;
	7892	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5258.5;
	7892	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8428;
	7892	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1379.6;
	7893	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866.4;
	7893	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	665.5;
	7893	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	832;
	7893	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3575.4;
	7893	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8231.7;
	7893	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5137.1;
	7893	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8305.6;
	7893	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1354.2;
	7894	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	866.2;
	7894	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	651.6;
	7894	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	803.1;
	7894	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3459.4;
	7894	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8091.8;
	7894	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4981.4;
	7894	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8159.7;
	7894	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1312.4;
	7895	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	862.8;
	7895	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	635.6;
	7895	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	767.2;
	7895	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3290.5;
	7895	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7813.9;
	7895	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4745.8;
	7895	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7945.6;
	7895	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1239.8;
	7896	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.3;
	7896	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	614.5;
	7896	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	729.6;
	7896	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3068.9;
	7896	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7410.3;
	7896	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4466.2;
	7896	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7617.4;
	7896	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1244.3;
	7897	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	850;
	7897	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	619.4;
	7897	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	713.7;
	7897	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2794.7;
	7897	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7197;
	7897	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4083.5;
	7897	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7415.9;
	7897	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1182.2;
	7898	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	850.3;
	7898	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	610.9;
	7898	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	693.3;
	7898	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2666.1;
	7898	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6967.5;
	7898	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3912.5;
	7898	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7240;
	7898	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1097.8;
	7899	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	850.1;
	7899	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	606.7;
	7899	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	681.6;
	7899	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2573;
	7899	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6816.3;
	7899	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3815.1;
	7899	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7121.3;
	7899	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1093.8;
	7900	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	851.1;
	7900	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	610.8;
	7900	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	677.1;
	7900	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2515.3;
	7900	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6765;
	7900	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3774.5;
	7900	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7074.4;
	7900	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1132.4;
	7901	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	855.2;
	7901	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	628;
	7901	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	678.8;
	7901	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2509.1;
	7901	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6901.5;
	7901	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3827.8;
	7901	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7160.5;
	7901	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1135.5;
	7902	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	852.5;
	7902	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	638.7;
	7902	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	699.9;
	7902	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2578.1;
	7902	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7183.8;
	7902	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3978.5;
	7902	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7371.2;
	7902	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1180.4;
	7903	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	850.4;
	7903	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	665.4;
	7903	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	753.3;
	7903	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2742.8;
	7903	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7570.7;
	7903	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4208.3;
	7903	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7662.8;
	7903	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1218.2;
	7904	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	853;
	7904	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	690.3;
	7904	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	781.2;
	7904	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2785.6;
	7904	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7849.4;
	7904	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4378.3;
	7904	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7733.1;
	7904	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1240.2;
	7905	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	863.8;
	7905	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	709.7;
	7905	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	795.7;
	7905	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2948.4;
	7905	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8314.2;
	7905	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4602.9;
	7905	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8056.8;
	7905	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1318.7;
	7906	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871.6;
	7906	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720.2;
	7906	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	814.7;
	7906	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3158.7;
	7906	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8751.8;
	7906	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4840;
	7906	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8403.7;
	7906	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1380.4;
	7907	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877;
	7907	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.6;
	7907	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824.9;
	7907	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3367.5;
	7907	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9043.1;
	7907	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5035.3;
	7907	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8696.8;
	7907	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1388.9;
	7908	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.6;
	7908	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	718.6;
	7908	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	826.9;
	7908	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3542.8;
	7908	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9103.1;
	7908	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5132.2;
	7908	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8844.1;
	7908	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1397.2;
	7909	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.8;
	7909	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	706.3;
	7909	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	828.1;
	7909	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3680.9;
	7909	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9016.8;
	7909	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5175.1;
	7909	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8891.3;
	7909	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1397.3;
	7910	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	864.4;
	7910	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	699.9;
	7910	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	828.5;
	7910	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3797.7;
	7910	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8895.5;
	7910	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5167.9;
	7910	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8950.1;
	7910	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1341.7;
	7911	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	862;
	7911	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	693.8;
	7911	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	832;
	7911	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3857.8;
	7911	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8772.9;
	7911	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5119.7;
	7911	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8961.3;
	7911	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1327.1;
	7912	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	863.4;
	7912	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	689.9;
	7912	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	829.5;
	7912	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3865.7;
	7912	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8696.6;
	7912	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5105.1;
	7912	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8977.8;
	7912	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1379.9;
	7913	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	871.3;
	7913	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	686.8;
	7913	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	844.6;
	7913	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3798.8;
	7913	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8711;
	7913	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5127.3;
	7913	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8930.7;
	7913	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1406.3;
	7914	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.8;
	7914	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	716.4;
	7914	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	874.9;
	7914	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3726.7;
	7914	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9215.8;
	7914	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5313.3;
	7914	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9157.6;
	7914	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1451.1;
	7915	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.8;
	7915	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744;
	7915	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	901.9;
	7915	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3796.2;
	7915	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9546;
	7915	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5506.4;
	7915	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9262.9;
	7915	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1509.5;
	7916	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.5;
	7916	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	745.4;
	7916	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	886.1;
	7916	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3680.9;
	7916	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9436.5;
	7916	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5381.1;
	7916	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9129.6;
	7916	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1501.9;
	7917	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.7;
	7917	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.4;
	7917	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	871.8;
	7917	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3551.7;
	7917	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9303.1;
	7917	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5283.4;
	7917	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8930.8;
	7917	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1490.9;
	7918	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.3;
	7918	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.7;
	7918	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.3;
	7918	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3412.7;
	7918	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9079.6;
	7918	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5121.9;
	7918	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8679.8;
	7918	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1424.3;
	7919	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921;
	7919	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711.3;
	7919	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	809.9;
	7919	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3231;
	7919	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8705.3;
	7919	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4845.5;
	7919	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8371.4;
	7919	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1363.8;
	7920	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.7;
	7920	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	682.1;
	7920	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	767.2;
	7920	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2996.7;
	7920	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8223.5;
	7920	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4497;
	7920	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7962.6;
	7920	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1297.1;
	7921	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.7;
	7921	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	660.3;
	7921	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	756.7;
	7921	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2779.6;
	7921	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7772.8;
	7921	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4210.9;
	7921	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7601.3;
	7921	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1259.5;
	7922	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	890.8;
	7922	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	652.3;
	7922	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	746.9;
	7922	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2651.6;
	7922	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7470.8;
	7922	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4051.9;
	7922	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7352.4;
	7922	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1214.7;
	7923	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.6;
	7923	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	645.9;
	7923	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	727.7;
	7923	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2563.5;
	7923	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7307.7;
	7923	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3893.4;
	7923	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7182.4;
	7923	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1129.7;
	7924	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.3;
	7924	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	647.7;
	7924	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	732.7;
	7924	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2501.1;
	7924	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7237.9;
	7924	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3873.7;
	7924	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7108;
	7924	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1103;
	7925	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.4;
	7925	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	660.8;
	7925	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	738.4;
	7925	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2479.5;
	7925	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7301.4;
	7925	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3876.4;
	7925	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7112.6;
	7925	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1110.2;
	7926	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.4;
	7926	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	683.7;
	7926	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	753.2;
	7926	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2511.9;
	7926	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7525.8;
	7926	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3986.6;
	7926	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7211.6;
	7926	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1190;
	7927	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.6;
	7927	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	706.1;
	7927	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	778.6;
	7927	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2574.7;
	7927	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7923.8;
	7927	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4187.4;
	7927	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7412.4;
	7927	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1232.9;
	7928	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.8;
	7928	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.6;
	7928	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	810.3;
	7928	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2608;
	7928	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8289.6;
	7928	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4365.7;
	7928	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7505.3;
	7928	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1282.5;
	7929	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.5;
	7929	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755.2;
	7929	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	844.7;
	7929	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2768;
	7929	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8745.3;
	7929	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4622.3;
	7929	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7869;
	7929	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1395.4;
	7930	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.6;
	7930	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	766.5;
	7930	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.6;
	7930	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2967.2;
	7930	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9145.9;
	7930	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4858.5;
	7930	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8251.4;
	7930	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1425.7;
	7931	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.5;
	7931	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	762.1;
	7931	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	869.7;
	7931	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3136.1;
	7931	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9249;
	7931	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4983.8;
	7931	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8529;
	7931	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1462.8;
	7932	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.2;
	7932	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	748.1;
	7932	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	870.3;
	7932	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3303;
	7932	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9186.4;
	7932	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5028.2;
	7932	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8720.6;
	7932	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1456.4;
	7933	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.2;
	7933	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.9;
	7933	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	863;
	7933	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3449;
	7933	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9064.1;
	7933	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5036.1;
	7933	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8812.5;
	7933	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1434.4;
	7934	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.9;
	7934	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711.1;
	7934	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.4;
	7934	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3502.9;
	7934	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8879.4;
	7934	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5016;
	7934	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8827.4;
	7934	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1433.4;
	7935	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.2;
	7935	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	699.3;
	7935	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	845.1;
	7935	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3470;
	7935	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8723.7;
	7935	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5010.9;
	7935	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8783;
	7935	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1431.2;
	7936	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.7;
	7936	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	692.7;
	7936	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	839;
	7936	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3413.8;
	7936	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8645.9;
	7936	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4997.3;
	7936	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8757.3;
	7936	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1404.4;
	7937	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.9;
	7937	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	700.1;
	7937	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	844.9;
	7937	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3352.6;
	7937	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8687;
	7937	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5037.7;
	7937	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8703.2;
	7937	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1417.6;
	7938	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.4;
	7938	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.9;
	7938	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	870.1;
	7938	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3387.1;
	7938	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9228.8;
	7938	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5279.3;
	7938	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8965.1;
	7938	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1514.4;
	7939	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.5;
	7939	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	761.1;
	7939	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	894.2;
	7939	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3521.6;
	7939	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9530.3;
	7939	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5520.4;
	7939	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9203.2;
	7939	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1570;
	7940	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939;
	7940	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	754.4;
	7940	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	883.8;
	7940	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3465.7;
	7940	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9405;
	7940	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5433.1;
	7940	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9056;
	7940	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1544.2;
	7941	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.7;
	7941	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	745.3;
	7941	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.1;
	7941	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3383.5;
	7941	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9221.9;
	7941	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5336.1;
	7941	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8897.3;
	7941	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1461.2;
	7942	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.8;
	7942	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.4;
	7942	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	841.2;
	7942	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3276.1;
	7942	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8955.9;
	7942	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5168.2;
	7942	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8661.6;
	7942	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1442.4;
	7943	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916;
	7943	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	698.9;
	7943	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	806.7;
	7943	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3128.8;
	7943	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8515.2;
	7943	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4889.1;
	7943	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8340.3;
	7943	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1418.2;
	7944	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903;
	7944	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	668;
	7944	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	767.8;
	7944	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2975.6;
	7944	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8002.9;
	7944	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4579.8;
	7944	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7988.7;
	7944	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1338.1;
	7945	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.7;
	7945	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	642.6;
	7945	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	740.4;
	7945	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2811.9;
	7945	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7543.3;
	7945	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4268.9;
	7945	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7619.1;
	7945	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1274.2;
	7946	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	885.8;
	7946	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	628.4;
	7946	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	718.8;
	7946	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2672.7;
	7946	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7231.5;
	7946	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4078.2;
	7946	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7389.6;
	7946	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1235.4;
	7947	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	876.6;
	7947	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	624.7;
	7947	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	706.1;
	7947	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2569.1;
	7947	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7046.8;
	7947	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3935.6;
	7947	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7223.7;
	7947	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1226.6;
	7948	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	873.5;
	7948	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	621.7;
	7948	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	702.2;
	7948	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2510.7;
	7948	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6956.9;
	7948	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3858.1;
	7948	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7129.6;
	7948	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1221.8;
	7949	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	874.2;
	7949	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	624.2;
	7949	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	701.4;
	7949	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2478.9;
	7949	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6972.4;
	7949	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3862.6;
	7949	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7130.5;
	7949	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1216.5;
	7950	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	879.3;
	7950	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	641.2;
	7950	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	710.5;
	7950	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2497.1;
	7950	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7138.2;
	7950	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3911;
	7950	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7224;
	7950	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1235.8;
	7951	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	889.9;
	7951	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	658;
	7951	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	733.7;
	7951	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2556.2;
	7951	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7458.8;
	7951	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4081.7;
	7951	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7396.6;
	7951	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1214.8;
	7952	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	893.5;
	7952	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	679.2;
	7952	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	757.6;
	7952	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2553.6;
	7952	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7794.1;
	7952	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4213.2;
	7952	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7420;
	7952	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1359.6;
	7953	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	901.6;
	7953	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711.3;
	7953	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	792.2;
	7953	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2760.1;
	7953	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8289.2;
	7953	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4476.1;
	7953	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7766.4;
	7953	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1408.1;
	7954	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.4;
	7954	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.4;
	7954	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	821.3;
	7954	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3035.8;
	7954	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8720;
	7954	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4776.8;
	7954	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8099.4;
	7954	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1467.1;
	7955	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.4;
	7955	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.3;
	7955	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	832.1;
	7955	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3271.5;
	7955	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9006.9;
	7955	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4950.3;
	7955	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8358.8;
	7955	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1460;
	7956	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.8;
	7956	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.7;
	7956	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	837.2;
	7956	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3453.2;
	7956	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9174.5;
	7956	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5047.1;
	7956	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8577.7;
	7956	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1464.3;
	7957	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910;
	7957	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	731.4;
	7957	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	844.2;
	7957	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3579.7;
	7957	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9273.3;
	7957	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5143.1;
	7957	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8734.7;
	7957	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1464.8;
	7958	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907;
	7958	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.3;
	7958	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	848.4;
	7958	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3677.5;
	7958	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9317.8;
	7958	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5253.9;
	7958	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8873.9;
	7958	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1461.7;
	7959	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	907.2;
	7959	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	731.9;
	7959	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849.8;
	7959	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3760.3;
	7959	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9309;
	7959	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5300.8;
	7959	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8907.8;
	7959	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1450.9;
	7960	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.6;
	7960	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	722.1;
	7960	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	853.9;
	7960	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3790.3;
	7960	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9335.1;
	7960	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5357.2;
	7960	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8921.3;
	7960	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1437.5;
	7961	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.2;
	7961	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.3;
	7961	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	869.5;
	7961	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3751.9;
	7961	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9467.2;
	7961	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5459.3;
	7961	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8972;
	7961	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1488.6;
	7962	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.7;
	7962	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	767.1;
	7962	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	900.4;
	7962	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3754.8;
	7962	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10018.1;
	7962	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5807.7;
	7962	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9465.1;
	7962	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1602.6;
	7963	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.9;
	7963	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	782;
	7963	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	928.2;
	7963	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3940.3;
	7963	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10290.1;
	7963	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6189.8;
	7963	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9876.1;
	7963	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1652.1;
	7964	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.2;
	7964	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772.8;
	7964	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	919.4;
	7964	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3920.6;
	7964	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10186;
	7964	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6176.8;
	7964	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9881.3;
	7964	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1652.6;
	7965	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.6;
	7965	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	750.8;
	7965	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	893.4;
	7965	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3826;
	7965	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9883.4;
	7965	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5981.8;
	7965	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9672;
	7965	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1597.1;
	7966	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.8;
	7966	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.1;
	7966	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.3;
	7966	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3699.8;
	7966	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9430.1;
	7966	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5716.8;
	7966	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9380.5;
	7966	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1511.1;
	7967	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	912.1;
	7967	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	675.5;
	7967	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	811.3;
	7967	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3489.4;
	7967	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8717.7;
	7967	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5284.4;
	7967	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8923.4;
	7967	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1367.2;
	7968	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.8;
	7968	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	633.9;
	7968	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	762.1;
	7968	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3229;
	7968	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7957.7;
	7968	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4845.9;
	7968	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8440.2;
	7968	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1320.6;
	7969	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.3;
	7969	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	609.6;
	7969	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	717.7;
	7969	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3042.5;
	7969	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7397.8;
	7969	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4460.5;
	7969	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8135.4;
	7969	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1179.1;
	7970	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	870.8;
	7970	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	592.7;
	7970	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	698.3;
	7970	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2908.7;
	7970	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7104.4;
	7970	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4289.5;
	7970	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7923.5;
	7970	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1225;
	7971	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	869.6;
	7971	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	589.2;
	7971	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	692.6;
	7971	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2817.3;
	7971	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6975.2;
	7971	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4204.7;
	7971	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7842.2;
	7971	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1199.7;
	7972	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	877.6;
	7972	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	590.8;
	7972	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	693.5;
	7972	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2787.1;
	7972	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6959.6;
	7972	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4173;
	7972	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7894.1;
	7972	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1184.5;
	7973	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	888.5;
	7973	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	600.1;
	7973	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	700.7;
	7973	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2796.6;
	7973	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7179.9;
	7973	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4280.6;
	7973	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8112;
	7973	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1223.6;
	7974	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	908.3;
	7974	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	641.7;
	7974	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	733.7;
	7974	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2952.8;
	7974	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7854.6;
	7974	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4640.5;
	7974	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8664.2;
	7974	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1311;
	7975	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.4;
	7975	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	710.9;
	7975	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	812.2;
	7975	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3287.7;
	7975	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8977.3;
	7975	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5312.5;
	7975	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9446.3;
	7975	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1465.1;
	7976	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	968.3;
	7976	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	748.6;
	7976	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.8;
	7976	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3333.6;
	7976	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9486.3;
	7976	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5525.6;
	7976	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9786.9;
	7976	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1559.8;
	7977	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.5;
	7977	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	759.6;
	7977	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.9;
	7977	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3416.7;
	7977	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9566.6;
	7977	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5616.5;
	7977	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10022.4;
	7977	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1592.4;
	7978	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.7;
	7978	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	763.9;
	7978	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	869.4;
	7978	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3615.6;
	7978	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9784.9;
	7978	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5783.2;
	7978	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10277.7;
	7978	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1640.4;
	7979	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.4;
	7979	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	769.8;
	7979	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	879.4;
	7979	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3845.2;
	7979	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9955.6;
	7979	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5909.9;
	7979	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10546.7;
	7979	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1662.9;
	7980	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956;
	7980	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	764.6;
	7980	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878.9;
	7980	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4074.4;
	7980	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10036.5;
	7980	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5972.2;
	7980	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10890.4;
	7980	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1656.7;
	7981	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.6;
	7981	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	754.4;
	7981	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878.8;
	7981	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4313.6;
	7981	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10060.8;
	7981	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6045.4;
	7981	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11199;
	7981	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1674.4;
	7982	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.8;
	7982	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.5;
	7982	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884.8;
	7982	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4508.1;
	7982	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10082.1;
	7982	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6127.1;
	7982	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11375.7;
	7982	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1681.1;
	7983	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.2;
	7983	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.4;
	7983	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	883.2;
	7983	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4636.4;
	7983	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10044.6;
	7983	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6182.6;
	7983	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11421.1;
	7983	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1679;
	7984	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.3;
	7984	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.1;
	7984	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.8;
	7984	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4673;
	7984	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9942.5;
	7984	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6118.3;
	7984	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11430.8;
	7984	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1660.5;
	7985	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.4;
	7985	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.2;
	7985	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.5;
	7985	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4572.9;
	7985	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9911.1;
	7985	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6105.9;
	7985	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11220.2;
	7985	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1647.2;
	7986	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.3;
	7986	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	766.4;
	7986	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	900.6;
	7986	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4390.2;
	7986	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10301.3;
	7986	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6143.4;
	7986	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11181.2;
	7986	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1672.9;
	7987	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	983.8;
	7987	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	801.3;
	7987	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	939.1;
	7987	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4431.5;
	7987	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10757.4;
	7987	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6388.6;
	7987	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11193.4;
	7987	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1732;
	7988	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986.7;
	7988	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	794.1;
	7988	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	932.2;
	7988	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4259.7;
	7988	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10610.4;
	7988	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6197.8;
	7988	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10884.5;
	7988	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1725.1;
	7989	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	979.8;
	7989	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	778.4;
	7989	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	917.2;
	7989	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4081.9;
	7989	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10345;
	7989	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5947.6;
	7989	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10548.3;
	7989	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1678.8;
	7990	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.3;
	7990	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	761.6;
	7990	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	890.6;
	7990	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3874.4;
	7990	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9892.4;
	7990	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5540.7;
	7990	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10031.1;
	7990	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1617.8;
	7991	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.6;
	7991	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	717.4;
	7991	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838.6;
	7991	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3594.3;
	7991	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9112.4;
	7991	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5073.5;
	7991	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9341.7;
	7991	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1493.7;
	7992	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920.1;
	7992	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	683.4;
	7992	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	788.5;
	7992	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3283.6;
	7992	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8310.9;
	7992	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4601.7;
	7992	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8649.3;
	7992	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1340.3;
	7993	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.5;
	7993	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	664.2;
	7993	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	756.6;
	7993	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3070.1;
	7993	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7781.2;
	7993	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4248.6;
	7993	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8216.1;
	7993	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1282.7;
	7994	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.6;
	7994	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	652.3;
	7994	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	739.1;
	7994	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2919;
	7994	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7489.5;
	7994	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4079.2;
	7994	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7964.1;
	7994	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1284.6;
	7995	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.5;
	7995	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	645.4;
	7995	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	735.1;
	7995	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2827.6;
	7995	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7321.3;
	7995	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	3995.6;
	7995	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7810.8;
	7995	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1191.4;
	7996	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	896.7;
	7996	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	643;
	7996	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	743;
	7996	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2780.7;
	7996	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7330.2;
	7996	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4001.8;
	7996	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7804.7;
	7996	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1184.2;
	7997	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.5;
	7997	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	652.8;
	7997	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	756.1;
	7997	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2793.4;
	7997	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7550.9;
	7997	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4118.5;
	7997	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7997.8;
	7997	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1271.5;
	7998	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921.5;
	7998	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	686;
	7998	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	796.9;
	7998	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2939.3;
	7998	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8302.8;
	7998	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4472.2;
	7998	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8584.6;
	7998	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1388.4;
	7999	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959;
	7999	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	758.5;
	7999	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884.9;
	7999	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3314;
	7999	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9487.9;
	7999	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5192.2;
	7999	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9345;
	7999	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1536;
	8000	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.1;
	8000	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	785.4;
	8000	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	921.8;
	8000	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3384.2;
	8000	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9894.3;
	8000	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5428.9;
	8000	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9552.2;
	8000	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1619.2;
	8001	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.4;
	8001	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777.3;
	8001	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	900.5;
	8001	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3424.1;
	8001	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9774.8;
	8001	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5327.4;
	8001	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9726;
	8001	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1639.7;
	8002	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.2;
	8002	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	775.1;
	8002	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	881.3;
	8002	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3605.5;
	8002	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9794.2;
	8002	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5298.6;
	8002	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10078.4;
	8002	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1618.8;
	8003	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.6;
	8003	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777.4;
	8003	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.6;
	8003	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3847;
	8003	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9869.4;
	8003	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5338.4;
	8003	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10566;
	8003	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1577.3;
	8004	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.5;
	8004	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	770.7;
	8004	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	856.4;
	8004	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4057.2;
	8004	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9896.9;
	8004	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5411.9;
	8004	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10997.4;
	8004	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1551;
	8005	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951;
	8005	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	754.5;
	8005	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	850;
	8005	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4295.1;
	8005	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9917.8;
	8005	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5501.9;
	8005	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11321.1;
	8005	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1553.9;
	8006	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.4;
	8006	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.4;
	8006	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849.2;
	8006	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4493.6;
	8006	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9962.8;
	8006	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5623.9;
	8006	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11586.1;
	8006	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1614.7;
	8007	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.3;
	8007	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.1;
	8007	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.5;
	8007	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4625.6;
	8007	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9919.4;
	8007	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5710.3;
	8007	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11645.1;
	8007	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1575.5;
	8008	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.1;
	8008	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	744.4;
	8008	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.1;
	8008	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4649.7;
	8008	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9899.3;
	8008	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5799.6;
	8008	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11619.6;
	8008	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1550.7;
	8009	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955;
	8009	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	748.6;
	8009	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.3;
	8009	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4585.8;
	8009	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9919;
	8009	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5868.5;
	8009	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11516.8;
	8009	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1573.5;
	8010	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.1;
	8010	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	773.8;
	8010	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	875.3;
	8010	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4405.9;
	8010	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10288.8;
	8010	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5954;
	8010	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11666.7;
	8010	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1632.8;
	8011	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	995.8;
	8011	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	808.6;
	8011	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	911.1;
	8011	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4435.6;
	8011	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10761.3;
	8011	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6255.6;
	8011	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11699.6;
	8011	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1724.7;
	8012	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.4;
	8012	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	808.7;
	8012	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	908.6;
	8012	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4291.7;
	8012	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10678.9;
	8012	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6128.2;
	8012	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11420.3;
	8012	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1658.1;
	8013	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	997.6;
	8013	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	799.3;
	8013	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	900.3;
	8013	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4131.4;
	8013	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10460.2;
	8013	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6003.3;
	8013	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11060.5;
	8013	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1618.6;
	8014	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	988.6;
	8014	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	775.1;
	8014	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	871.9;
	8014	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3923.2;
	8014	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9969.4;
	8014	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5663.2;
	8014	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10555.3;
	8014	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1541.4;
	8015	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.6;
	8015	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	734.3;
	8015	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	820.2;
	8015	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3622.8;
	8015	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9165.8;
	8015	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5172.9;
	8015	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9813.3;
	8015	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1443.5;
	8016	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	939.7;
	8016	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	694.7;
	8016	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	775.6;
	8016	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3276.2;
	8016	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8375.6;
	8016	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4686.8;
	8016	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9006.7;
	8016	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1392;
	8017	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.9;
	8017	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	675.2;
	8017	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	751.7;
	8017	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2976.9;
	8017	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7851.8;
	8017	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4352.5;
	8017	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8398.7;
	8017	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1338.9;
	8018	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.8;
	8018	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	668.6;
	8018	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	738.5;
	8018	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2812.9;
	8018	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7604;
	8018	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4150.3;
	8018	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8030.2;
	8018	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1282.7;
	8019	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.5;
	8019	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	665.1;
	8019	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	734.4;
	8019	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2713.1;
	8019	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7507.9;
	8019	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4051.3;
	8019	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7792.1;
	8019	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1293.4;
	8020	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941;
	8020	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	672;
	8020	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	743.2;
	8020	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2638.5;
	8020	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7493.2;
	8020	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4028.9;
	8020	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7679.9;
	8020	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1298.5;
	8021	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.6;
	8021	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	693;
	8021	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	760.6;
	8021	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2624.6;
	8021	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7831.4;
	8021	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4154.3;
	8021	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7767.9;
	8021	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1341.8;
	8022	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	974.8;
	8022	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.3;
	8022	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	807.1;
	8022	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2765;
	8022	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8658.3;
	8022	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4548.2;
	8022	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8221.2;
	8022	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1447.2;
	8023	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1018;
	8023	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	823;
	8023	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	905.2;
	8023	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3108.8;
	8023	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10018.2;
	8023	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5287.4;
	8023	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8905.1;
	8023	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1640.3;
	8024	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1033.3;
	8024	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	852.4;
	8024	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	953.3;
	8024	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3112.5;
	8024	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10516;
	8024	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5473.7;
	8024	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9086.3;
	8024	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1696.7;
	8025	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1019.7;
	8025	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	839;
	8025	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	941.3;
	8025	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3068.7;
	8025	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10436.7;
	8025	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5423.9;
	8025	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9157.2;
	8025	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1690.9;
	8026	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1004.4;
	8026	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	835.2;
	8026	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	924.8;
	8026	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3106.7;
	8026	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10378.5;
	8026	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5415.3;
	8026	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9272.4;
	8026	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1687.7;
	8027	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	987.9;
	8027	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	831.4;
	8027	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	915.7;
	8027	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3192.9;
	8027	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10324.4;
	8027	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5394.3;
	8027	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9449.4;
	8027	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1672.6;
	8028	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	979;
	8028	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	809.5;
	8028	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	904.3;
	8028	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3261.1;
	8028	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10205.4;
	8028	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5360.6;
	8028	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9568.6;
	8028	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1645.7;
	8029	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966;
	8029	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	779.4;
	8029	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	893.1;
	8029	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3324.8;
	8029	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10047.8;
	8029	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5333.6;
	8029	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9621.1;
	8029	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1627.4;
	8030	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.2;
	8030	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	768.4;
	8030	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.1;
	8030	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3406.2;
	8030	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9945.8;
	8030	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5326.9;
	8030	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9730.1;
	8030	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1611.2;
	8031	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.7;
	8031	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753;
	8031	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	895.1;
	8031	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3479;
	8031	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9814.5;
	8031	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5302.7;
	8031	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9770.8;
	8031	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1574.9;
	8032	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948.3;
	8032	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746.8;
	8032	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	898.3;
	8032	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3500.1;
	8032	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9724.7;
	8032	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5308.3;
	8032	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9708.1;
	8032	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1590.6;
	8033	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951.3;
	8033	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	757.9;
	8033	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	901.6;
	8033	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3503.9;
	8033	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9795.7;
	8033	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5408.5;
	8033	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9581.1;
	8033	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1596.6;
	8034	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.9;
	8034	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	808.8;
	8034	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	912.7;
	8034	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3479.3;
	8034	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10476.8;
	8034	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5673.5;
	8034	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9761.7;
	8034	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1717.1;
	8035	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1001.8;
	8035	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	860;
	8035	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	964.8;
	8035	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3629.7;
	8035	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11244.9;
	8035	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6084.9;
	8035	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9996.2;
	8035	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1805.2;
	8036	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1011.7;
	8036	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	866.6;
	8036	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	970.9;
	8036	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3577.5;
	8036	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11299.3;
	8036	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6116.2;
	8036	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9870.7;
	8036	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1837.8;
	8037	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1019.6;
	8037	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	866.7;
	8037	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	972.9;
	8037	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3520;
	8037	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11245.7;
	8037	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6059.7;
	8037	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9662.9;
	8037	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1853.4;
	8038	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1015.2;
	8038	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	846.4;
	8038	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	951.2;
	8038	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3389.5;
	8038	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10961;
	8038	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5815;
	8038	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9302.8;
	8038	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1819.8;
	8039	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	996.3;
	8039	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	803.6;
	8039	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	911.7;
	8039	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3170.5;
	8039	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10322.7;
	8039	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5436.7;
	8039	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8755.1;
	8039	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1714.8;
	8040	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	976.6;
	8040	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772;
	8040	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	869.5;
	8040	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2878;
	8040	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9587.5;
	8040	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5051.5;
	8040	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8186.9;
	8040	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1540.1;
	8041	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951;
	8041	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.8;
	8041	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	846.2;
	8041	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2685.9;
	8041	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9174.2;
	8041	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4773.9;
	8041	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7717.2;
	8041	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1578.4;
	8042	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.6;
	8042	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.4;
	8042	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.6;
	8042	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2579.2;
	8042	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9037.8;
	8042	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4692.2;
	8042	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7512.1;
	8042	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1540.4;
	8043	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951.5;
	8043	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749.4;
	8043	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	850.3;
	8043	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2529;
	8043	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9072.7;
	8043	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4672.8;
	8043	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7439;
	8043	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1488.7;
	8044	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.4;
	8044	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	761.2;
	8044	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.1;
	8044	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2504.9;
	8044	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9200.8;
	8044	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4735.8;
	8044	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7522.7;
	8044	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1536.6;
	8045	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.9;
	8045	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	778;
	8045	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	886.7;
	8045	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2536;
	8045	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9587.4;
	8045	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4899.3;
	8045	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7747.2;
	8045	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1609.1;
	8046	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	974.3;
	8046	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	817.5;
	8046	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	947.9;
	8046	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2707;
	8046	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10521.7;
	8046	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5448.1;
	8046	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8400.8;
	8046	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1733;
	8047	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1008.5;
	8047	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	900.3;
	8047	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1056.8;
	8047	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3116.3;
	8047	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11996.6;
	8047	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6312.9;
	8047	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9350.8;
	8047	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1982.9;
	8048	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1021.2;
	8048	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	916.1;
	8048	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1087.1;
	8048	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3176.3;
	8048	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12355.4;
	8048	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6479.7;
	8048	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9509;
	8048	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2029;
	8049	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1008.4;
	8049	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	890.7;
	8049	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1040.8;
	8049	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3137.5;
	8049	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11881.5;
	8049	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6174.8;
	8049	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9553.5;
	8049	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1952.7;
	8050	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	993.4;
	8050	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	853.9;
	8050	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	994.5;
	8050	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3158;
	8050	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11331.2;
	8050	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5864.1;
	8050	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9582.8;
	8050	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1858.7;
	8051	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	979.2;
	8051	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	824.6;
	8051	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	947.2;
	8051	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3184.8;
	8051	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10926.7;
	8051	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5637.5;
	8051	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9574;
	8051	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1744.1;
	8052	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.9;
	8052	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	795.7;
	8052	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	915.1;
	8052	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3224.1;
	8052	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10534.2;
	8052	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5450.1;
	8052	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9486.9;
	8052	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1693.1;
	8053	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.9;
	8053	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	767.7;
	8053	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	880.6;
	8053	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3242;
	8053	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10208.5;
	8053	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5334;
	8053	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9340.7;
	8053	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1636.6;
	8054	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.9;
	8054	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	750.4;
	8054	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.1;
	8054	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3272;
	8054	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9978.3;
	8054	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5294.8;
	8054	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9302.9;
	8054	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1592.6;
	8055	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.7;
	8055	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.1;
	8055	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	855.6;
	8055	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3298.9;
	8055	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9781.6;
	8055	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5260.8;
	8055	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9256.9;
	8055	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1565.8;
	8056	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.4;
	8056	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.6;
	8056	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.5;
	8056	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3322.1;
	8056	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9697.4;
	8056	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5280.7;
	8056	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9190.3;
	8056	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1498;
	8057	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.4;
	8057	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.5;
	8057	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	855;
	8057	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3339.4;
	8057	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9762.1;
	8057	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5316.5;
	8057	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9156.9;
	8057	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1562.3;
	8058	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.9;
	8058	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	786.8;
	8058	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	886.5;
	8058	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3440.7;
	8058	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10439.4;
	8058	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5545;
	8058	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9588.6;
	8058	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1627.5;
	8059	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	977.9;
	8059	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	829.5;
	8059	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	949.8;
	8059	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3642.2;
	8059	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11056.1;
	8059	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5922.3;
	8059	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9822.3;
	8059	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1727.9;
	8060	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981.6;
	8060	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	827.5;
	8060	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	953.5;
	8060	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3638.7;
	8060	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11071.1;
	8060	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5988.8;
	8060	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9778;
	8060	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1742.4;
	8061	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	988.7;
	8061	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	821.3;
	8061	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	948.4;
	8061	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3589.1;
	8061	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10926.4;
	8061	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5897.5;
	8061	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9668.9;
	8061	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1805.6;
	8062	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.8;
	8062	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	803.7;
	8062	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	923.3;
	8062	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3465.5;
	8062	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10600.9;
	8062	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5666.6;
	8062	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9376.3;
	8062	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1783.1;
	8063	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.5;
	8063	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	766.7;
	8063	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878.9;
	8063	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3259.9;
	8063	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9927.1;
	8063	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5300.5;
	8063	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8841.9;
	8063	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1664.5;
	8064	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.7;
	8064	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.4;
	8064	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	836;
	8064	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2989.2;
	8064	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9226.5;
	8064	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4873.3;
	8064	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8300.9;
	8064	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1555.4;
	8065	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.6;
	8065	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	708.2;
	8065	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	829.8;
	8065	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2788.2;
	8065	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8647.7;
	8065	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4442.2;
	8065	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7932.5;
	8065	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1421.6;
	8066	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.8;
	8066	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	698.5;
	8066	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	817.8;
	8066	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2652.4;
	8066	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8381.2;
	8066	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4256.2;
	8066	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7696.3;
	8066	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1396.2;
	8067	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.8;
	8067	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	699.9;
	8067	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	815.2;
	8067	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2582.6;
	8067	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8274.3;
	8067	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4174.8;
	8067	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7580;
	8067	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1396.3;
	8068	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.5;
	8068	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	707.7;
	8068	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	821;
	8068	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2555.3;
	8068	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8311.8;
	8068	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4166;
	8068	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7568.8;
	8068	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1411;
	8069	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.6;
	8069	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.4;
	8069	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	833.2;
	8069	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2579.3;
	8069	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8620.6;
	8069	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4281.6;
	8069	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7749.4;
	8069	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1466.9;
	8070	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.9;
	8070	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	767.1;
	8070	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878.1;
	8070	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2745.8;
	8070	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9415.3;
	8070	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4666.3;
	8070	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8276.6;
	8070	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1562;
	8071	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.2;
	8071	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	842.9;
	8071	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	965.6;
	8071	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3142.4;
	8071	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10684.4;
	8071	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5336.8;
	8071	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9053.2;
	8071	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1773.6;
	8072	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1010.8;
	8072	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	867.3;
	8072	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	999.4;
	8072	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3228.3;
	8072	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11034.9;
	8072	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5573.9;
	8072	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9173.1;
	8072	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1797;
	8073	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1010.4;
	8073	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	859.5;
	8073	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	987.5;
	8073	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3256.4;
	8073	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10871.6;
	8073	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5503.4;
	8073	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9273.2;
	8073	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1770.7;
	8074	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1011.1;
	8074	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	847.2;
	8074	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	978.6;
	8074	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3383.2;
	8074	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10782.2;
	8074	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5466.7;
	8074	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9415.8;
	8074	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1721.2;
	8075	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1006.7;
	8075	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	825;
	8075	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	958.8;
	8075	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3502.6;
	8075	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10573.7;
	8075	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5463.1;
	8075	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9475.2;
	8075	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1668.9;
	8076	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000;
	8076	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	802.6;
	8076	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	939.2;
	8076	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3613.4;
	8076	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10271;
	8076	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5443.7;
	8076	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9484;
	8076	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1603.4;
	8077	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989.3;
	8077	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	784.4;
	8077	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	921.9;
	8077	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3660.4;
	8077	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9970.6;
	8077	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5451.1;
	8077	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9454.9;
	8077	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1544.8;
	8078	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992;
	8078	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772.3;
	8078	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913.2;
	8078	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3715.3;
	8078	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9837.2;
	8078	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5419.1;
	8078	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9477.3;
	8078	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1516.6;
	8079	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990.3;
	8079	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	763.1;
	8079	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	908.1;
	8079	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3748.2;
	8079	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9743.2;
	8079	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5390.2;
	8079	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9380.9;
	8079	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1489.9;
	8080	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990.2;
	8080	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755.9;
	8080	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	904.1;
	8080	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3785.5;
	8080	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9693.1;
	8080	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5388;
	8080	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9332.3;
	8080	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1477.6;
	8081	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	994.7;
	8081	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	770.5;
	8081	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	914.6;
	8081	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3792.8;
	8081	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9867.5;
	8081	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5474.1;
	8081	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9413.3;
	8081	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1510.4;
	8082	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1006;
	8082	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	808.6;
	8082	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940.2;
	8082	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3829.4;
	8082	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10589.5;
	8082	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5797.9;
	8082	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9723.4;
	8082	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1611.3;
	8083	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1019.4;
	8083	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	818;
	8083	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	962.5;
	8083	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3892.3;
	8083	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10766.2;
	8083	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6057.2;
	8083	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9711.9;
	8083	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1647.6;
	8084	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1018.8;
	8084	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	808;
	8084	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	953.5;
	8084	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3801.8;
	8084	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10558.4;
	8084	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5993.5;
	8084	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9539.5;
	8084	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1627.4;
	8085	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1017.7;
	8085	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	800.3;
	8085	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940.5;
	8085	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3705.9;
	8085	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10324.8;
	8085	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5940.6;
	8085	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9383;
	8085	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1601.8;
	8086	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009.1;
	8086	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	782.5;
	8086	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	919.3;
	8086	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3619.6;
	8086	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10006.5;
	8086	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5771.5;
	8086	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9157;
	8086	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1554.8;
	8087	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	996.8;
	8087	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753.7;
	8087	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889;
	8087	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3483.8;
	8087	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9481.9;
	8087	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5456.9;
	8087	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8790.5;
	8087	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1469.5;
	8088	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.8;
	8088	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	718.3;
	8088	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.6;
	8088	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3280.7;
	8088	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8843;
	8088	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5104.9;
	8088	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8339.2;
	8088	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1381.2;
	8089	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.9;
	8089	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	686.7;
	8089	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	828.7;
	8089	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3062.3;
	8089	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8277;
	8089	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4874.1;
	8089	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7952.6;
	8089	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1341.1;
	8090	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951.1;
	8090	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	670.4;
	8090	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	814.2;
	8090	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2902.6;
	8090	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7946.2;
	8090	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4747.7;
	8090	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7695.4;
	8090	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1302.2;
	8091	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948.6;
	8091	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	666.5;
	8091	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	810.2;
	8091	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2807.8;
	8091	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7835.5;
	8091	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4621.4;
	8091	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7574.6;
	8091	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1290.2;
	8092	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.7;
	8092	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	668.8;
	8092	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	816.6;
	8092	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2747.7;
	8092	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7824.7;
	8092	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4634.9;
	8092	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7520.5;
	8092	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1284.3;
	8093	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.1;
	8093	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	678.6;
	8093	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	831.9;
	8093	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2726.7;
	8093	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7958.2;
	8093	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4668.6;
	8093	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7569.2;
	8093	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1298.8;
	8094	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947;
	8094	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	705.3;
	8094	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.2;
	8094	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2770.6;
	8094	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8293.7;
	8094	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4838.2;
	8094	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7735.1;
	8094	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1336;
	8095	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.3;
	8095	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.4;
	8095	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	903;
	8095	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2866;
	8095	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8813.8;
	8095	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5097.9;
	8095	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8041.1;
	8095	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1405.8;
	8096	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	988.7;
	8096	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	779;
	8096	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	954.8;
	8096	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2940.9;
	8096	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9420.5;
	8096	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5389.8;
	8096	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8355.7;
	8096	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1495.7;
	8097	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1001;
	8097	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	809.1;
	8097	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	993.9;
	8097	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3118.9;
	8097	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10018.1;
	8097	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5685;
	8097	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8721.1;
	8097	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1618.1;
	8098	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1012.2;
	8098	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	838.7;
	8098	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1011;
	8098	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3328.5;
	8098	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10514.2;
	8098	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5988.1;
	8098	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9081.8;
	8098	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1687.2;
	8099	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1023.1;
	8099	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	860.9;
	8099	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1016;
	8099	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3593.7;
	8099	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10825.3;
	8099	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6158.9;
	8099	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9286.8;
	8099	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1722.1;
	8100	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1026.3;
	8100	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	861.8;
	8100	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1019.1;
	8100	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3704.7;
	8100	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10877.2;
	8100	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6203.5;
	8100	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9272.6;
	8100	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1723;
	8101	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1023.9;
	8101	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	853.3;
	8101	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1011;
	8101	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3765.4;
	8101	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10789.6;
	8101	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6173;
	8101	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9209.6;
	8101	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1707.4;
	8102	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1022.1;
	8102	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	857.6;
	8102	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	999.2;
	8102	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3815.4;
	8102	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10625.4;
	8102	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6109.1;
	8102	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9091.6;
	8102	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1693.6;
	8103	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1016.8;
	8103	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	853.6;
	8103	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	984.5;
	8103	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3806.6;
	8103	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10462.1;
	8103	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6088.8;
	8103	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9036.2;
	8103	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1668;
	8104	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1017.2;
	8104	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	847;
	8104	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	984.9;
	8104	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3752.9;
	8104	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10455.7;
	8104	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6065.7;
	8104	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9030.3;
	8104	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1666.4;
	8105	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1017.4;
	8105	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	855.1;
	8105	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	995.1;
	8105	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3692.9;
	8105	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10602.7;
	8105	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6165.5;
	8105	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9145.3;
	8105	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1681.6;
	8106	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1029.1;
	8106	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	882;
	8106	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1028.2;
	8106	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3625.5;
	8106	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11076.2;
	8106	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6443;
	8106	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9472.7;
	8106	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1759.1;
	8107	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1035.5;
	8107	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	878.4;
	8107	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1059.1;
	8107	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3652.8;
	8107	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11135.8;
	8107	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6565.6;
	8107	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9547.3;
	8107	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1769.9;
	8108	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1027.7;
	8108	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	868.6;
	8108	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1051.6;
	8108	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3535.5;
	8108	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10918.6;
	8108	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6420.9;
	8108	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9428.9;
	8108	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1740.4;
	8109	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1023.5;
	8109	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	865.6;
	8109	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1043.9;
	8109	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3419.3;
	8109	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10674.1;
	8109	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6301.5;
	8109	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9262.4;
	8109	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1677.2;
	8110	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1014.4;
	8110	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	837.6;
	8110	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1024.2;
	8110	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3296.9;
	8110	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10347.4;
	8110	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6104.9;
	8110	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9024.1;
	8110	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1623.2;
	8111	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.2;
	8111	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	813.7;
	8111	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	994.4;
	8111	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3146.9;
	8111	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9870;
	8111	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5820.3;
	8111	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8685.8;
	8111	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1542;
	8112	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	983.1;
	8112	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	783.1;
	8112	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	962.7;
	8112	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2955;
	8112	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9267.9;
	8112	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5483.9;
	8112	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8252.7;
	8112	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1453.7;
	8113	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	978.1;
	8113	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	762;
	8113	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884.6;
	8113	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2799.1;
	8113	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8825.8;
	8113	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4916.4;
	8113	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7953.4;
	8113	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1385.2;
	8114	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969;
	8114	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.4;
	8114	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.9;
	8114	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2677.6;
	8114	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8466.2;
	8114	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4714.6;
	8114	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7726.4;
	8114	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1346.4;
	8115	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.7;
	8115	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720.2;
	8115	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.1;
	8115	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2588.3;
	8115	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8249.5;
	8115	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4590;
	8115	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7568;
	8115	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1317.9;
	8116	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.4;
	8116	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	710.2;
	8116	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	844.9;
	8116	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2540.5;
	8116	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8176.8;
	8116	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4525.3;
	8116	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7483.2;
	8116	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1301.6;
	8117	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.1;
	8117	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714.3;
	8117	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	848.8;
	8117	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2520;
	8117	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8225.4;
	8117	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4561.8;
	8117	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7486.2;
	8117	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1303.2;
	8118	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.1;
	8118	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	731.6;
	8118	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.2;
	8118	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2545;
	8118	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8454.8;
	8118	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4660;
	8118	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7600.5;
	8118	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1337.2;
	8119	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.3;
	8119	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	757.9;
	8119	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.9;
	8119	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2614.1;
	8119	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8827;
	8119	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4855.5;
	8119	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7803.7;
	8119	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1389.5;
	8120	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.4;
	8120	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	776.4;
	8120	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	929.9;
	8120	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2668.6;
	8120	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9267.3;
	8120	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5140.7;
	8120	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8018;
	8120	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1460.2;
	8121	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.2;
	8121	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	804.8;
	8121	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	964;
	8121	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2810.2;
	8121	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9739;
	8121	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5446.8;
	8121	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8314.4;
	8121	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1532.8;
	8122	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.4;
	8122	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	820.5;
	8122	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	986;
	8122	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2998.1;
	8122	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10177;
	8122	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5732;
	8122	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8701.4;
	8122	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1585.2;
	8123	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.7;
	8123	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	811.8;
	8123	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	986;
	8123	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3116.9;
	8123	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10348.2;
	8123	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5944.7;
	8123	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8927.9;
	8123	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1587.9;
	8124	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1004.4;
	8124	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	804.6;
	8124	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	981;
	8124	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3184.9;
	8124	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10393.3;
	8124	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6023.8;
	8124	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9125.1;
	8124	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1585.6;
	8125	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1006.5;
	8125	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	796.4;
	8125	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	979.6;
	8125	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3203.8;
	8125	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10382.2;
	8125	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6007.4;
	8125	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9203.8;
	8125	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1588.8;
	8126	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1006;
	8126	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	789.2;
	8126	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	977.4;
	8126	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3222.9;
	8126	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10301.9;
	8126	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5978.9;
	8126	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9210;
	8126	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1580.9;
	8127	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1003.8;
	8127	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	790.2;
	8127	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	979;
	8127	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3234.3;
	8127	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10232.8;
	8127	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5940.1;
	8127	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9175.1;
	8127	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1573.3;
	8128	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	997.3;
	8128	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	792.1;
	8128	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	979.5;
	8128	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3259.7;
	8128	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10221;
	8128	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5975.9;
	8128	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9199;
	8128	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1576.7;
	8129	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.2;
	8129	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	802.4;
	8129	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	998.3;
	8129	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3316.2;
	8129	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10446.6;
	8129	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6078.2;
	8129	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9336.8;
	8129	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1612.2;
	8130	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009;
	8130	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	856.8;
	8130	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1041.5;
	8130	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3492.6;
	8130	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11154.1;
	8130	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6499.9;
	8130	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9788.3;
	8130	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1712.8;
	8131	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1038.7;
	8131	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	887.1;
	8131	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1077;
	8131	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3668.3;
	8131	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11468.9;
	8131	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6749;
	8131	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9963.5;
	8131	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1760.2;
	8132	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1044.3;
	8132	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	886.5;
	8132	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1073.3;
	8132	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3707.4;
	8132	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11411.9;
	8132	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6707.1;
	8132	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9921.1;
	8132	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1753.3;
	8133	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1045;
	8133	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	871.7;
	8133	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1062;
	8133	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3671.1;
	8133	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11213.3;
	8133	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6574.6;
	8133	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9781.8;
	8133	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1715.6;
	8134	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1033.4;
	8134	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	847.2;
	8134	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1042.2;
	8134	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3571.1;
	8134	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10738.7;
	8134	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6261.6;
	8134	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9510;
	8134	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1695;
	8135	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1011.2;
	8135	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	805.9;
	8135	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	983;
	8135	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3376.3;
	8135	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10016.1;
	8135	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5833.5;
	8135	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8994;
	8135	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1575.9;
	8136	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989.6;
	8136	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	766.2;
	8136	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	925.1;
	8136	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3105.5;
	8136	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9233.7;
	8136	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5340.6;
	8136	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8439.6;
	8136	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1458.3;
	8137	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.8;
	8137	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.7;
	8137	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	893.1;
	8137	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2886.4;
	8137	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8716.9;
	8137	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4986;
	8137	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8088.6;
	8137	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1374.8;
	8138	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.8;
	8138	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	723.6;
	8138	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.2;
	8138	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2753.8;
	8138	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8419.8;
	8138	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4830.1;
	8138	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7869.2;
	8138	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1424.3;
	8139	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.2;
	8139	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	707.1;
	8139	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857;
	8139	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2690.5;
	8139	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8339.9;
	8139	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4798.3;
	8139	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7797.7;
	8139	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1398;
	8140	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.2;
	8140	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	715.8;
	8140	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	856.7;
	8140	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2670.3;
	8140	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8415.1;
	8140	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4816.3;
	8140	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7847.4;
	8140	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1396.8;
	8141	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.3;
	8141	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	739.4;
	8141	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	873.6;
	8141	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2724;
	8141	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8786.2;
	8141	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4971.8;
	8141	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8108.1;
	8141	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1452.8;
	8142	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.2;
	8142	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	794.8;
	8142	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	932.9;
	8142	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2928.1;
	8142	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9662.5;
	8142	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5479.4;
	8142	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8756.8;
	8142	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1575.2;
	8143	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1003.9;
	8143	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	878.8;
	8143	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1036.7;
	8143	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3369.4;
	8143	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10954.1;
	8143	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6220.8;
	8143	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9594.3;
	8143	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1759.3;
	8144	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1023.8;
	8144	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	909.8;
	8144	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1072.7;
	8144	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3484;
	8144	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11447;
	8144	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6479.2;
	8144	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9902.8;
	8144	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1846.7;
	8145	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.5;
	8145	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	899.4;
	8145	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1060.7;
	8145	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3429.2;
	8145	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11331.5;
	8145	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6395.7;
	8145	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9962;
	8145	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1854.1;
	8146	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1005.3;
	8146	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	886.4;
	8146	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1043;
	8146	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3431.7;
	8146	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11351.8;
	8146	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6414.6;
	8146	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10053.1;
	8146	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1879.1;
	8147	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990;
	8147	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	868.3;
	8147	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1025.2;
	8147	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3430.2;
	8147	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11337.1;
	8147	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6370.3;
	8147	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10122.3;
	8147	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1861.6;
	8148	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969.6;
	8148	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	860.3;
	8148	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1007.8;
	8148	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3392.8;
	8148	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11203.1;
	8148	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6289.4;
	8148	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10140.8;
	8148	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1829.2;
	8149	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.6;
	8149	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	842.7;
	8149	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	995.2;
	8149	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3306.1;
	8149	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10975.9;
	8149	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6181;
	8149	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10075.8;
	8149	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1847.5;
	8150	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.2;
	8150	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	823.8;
	8150	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	983.8;
	8150	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3258.5;
	8150	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10850.5;
	8150	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6083.4;
	8150	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10042.8;
	8150	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1839.6;
	8151	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.9;
	8151	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	805.3;
	8151	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	969.4;
	8151	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3218;
	8151	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10778.1;
	8151	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6017.4;
	8151	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9992.6;
	8151	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1828.8;
	8152	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.6;
	8152	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	804.9;
	8152	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	959.4;
	8152	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3208.2;
	8152	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10861.8;
	8152	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6027.2;
	8152	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9975.1;
	8152	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1824.4;
	8153	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.3;
	8153	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	816.6;
	8153	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	963.8;
	8153	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3222.4;
	8153	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11102.5;
	8153	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6167.2;
	8153	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10026.5;
	8153	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1864.5;
	8154	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.4;
	8154	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	858;
	8154	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1000.6;
	8154	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3331.4;
	8154	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11832.6;
	8154	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6554.9;
	8154	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10412.9;
	8154	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1951.3;
	8155	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990;
	8155	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	900.4;
	8155	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1055.5;
	8155	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3636.3;
	8155	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12238.6;
	8155	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6900.1;
	8155	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10654.2;
	8155	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2003.8;
	8156	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1004.7;
	8156	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	895.6;
	8156	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1059.6;
	8156	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3702.4;
	8156	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12097.3;
	8156	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6840.8;
	8156	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10566.8;
	8156	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1995.9;
	8157	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1011.8;
	8157	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	880.4;
	8157	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1056.6;
	8157	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3713.2;
	8157	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11859.5;
	8157	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6714.5;
	8157	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10384.8;
	8157	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1952.8;
	8158	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	995.5;
	8158	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	853.2;
	8158	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1027.2;
	8158	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3594.5;
	8158	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11290.9;
	8158	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6387.4;
	8158	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9909.5;
	8158	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1868.5;
	8159	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	979.2;
	8159	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	807.1;
	8159	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	970.7;
	8159	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3394.7;
	8159	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10407.4;
	8159	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5955.7;
	8159	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9272.7;
	8159	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1730.7;
	8160	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.5;
	8160	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	759.2;
	8160	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	917.6;
	8160	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3107;
	8160	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9541.5;
	8160	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5502;
	8160	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8677.4;
	8160	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1563.7;
	8161	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.6;
	8161	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.5;
	8161	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878.1;
	8161	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2906.5;
	8161	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8979.3;
	8161	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5196.9;
	8161	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8279.8;
	8161	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1461.1;
	8162	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.7;
	8162	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714;
	8162	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	867.9;
	8162	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2809.7;
	8162	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8701.8;
	8162	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5044.2;
	8162	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8066.8;
	8162	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1457.4;
	8163	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.8;
	8163	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	710.5;
	8163	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	871.2;
	8163	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2768.4;
	8163	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8554.3;
	8163	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5015.3;
	8163	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7974.8;
	8163	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1364.7;
	8164	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.9;
	8164	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	706.5;
	8164	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878.9;
	8164	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2759.4;
	8164	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8529.4;
	8164	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5055;
	8164	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7985.1;
	8164	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1346.2;
	8165	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.1;
	8165	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	719;
	8165	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	899.5;
	8165	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2829.3;
	8165	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8761.4;
	8165	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5232.9;
	8165	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8224.1;
	8165	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1427.6;
	8166	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	966.6;
	8166	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	761.2;
	8166	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	955.2;
	8166	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3051.7;
	8166	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9570.4;
	8166	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5727.1;
	8166	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8866.1;
	8166	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1545.8;
	8167	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1015.8;
	8167	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	840.3;
	8167	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1060.7;
	8167	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3544.3;
	8167	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10875.2;
	8167	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6501.6;
	8167	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9786.2;
	8167	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1697.8;
	8168	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1039.5;
	8168	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	866.8;
	8168	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1101.2;
	8168	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3600.1;
	8168	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11316.5;
	8168	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6560.8;
	8168	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9908.7;
	8168	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1764.7;
	8169	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1028.1;
	8169	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	866.9;
	8169	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1075.1;
	8169	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3415.3;
	8169	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11198.9;
	8169	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6201.8;
	8169	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9826.2;
	8169	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1782.3;
	8170	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1023.2;
	8170	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	868.8;
	8170	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1054.8;
	8170	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3311;
	8170	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11204.9;
	8170	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5935.5;
	8170	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9777;
	8170	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1764.4;
	8171	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1014.8;
	8171	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	864.8;
	8171	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1037.7;
	8171	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3240.4;
	8171	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11193.3;
	8171	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5747.2;
	8171	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9692;
	8171	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1705.1;
	8172	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000.7;
	8172	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	850.9;
	8172	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1024.4;
	8172	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3204.6;
	8172	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11194.2;
	8172	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5656.3;
	8172	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9638;
	8172	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1677.8;
	8173	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985.6;
	8173	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	832.6;
	8173	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1008.3;
	8173	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3244.8;
	8173	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11181.5;
	8173	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5600.4;
	8173	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9569.8;
	8173	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1678.3;
	8174	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.2;
	8174	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	824.3;
	8174	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	994.2;
	8174	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3305;
	8174	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11170.2;
	8174	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5589.7;
	8174	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9559.4;
	8174	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1733.1;
	8175	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	968.7;
	8175	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	818;
	8175	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	986.2;
	8175	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3388.3;
	8175	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11066.9;
	8175	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5557.4;
	8175	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9561;
	8175	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1699;
	8176	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.4;
	8176	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	814.5;
	8176	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	984.1;
	8176	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3454.7;
	8176	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11035.9;
	8176	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5583.1;
	8176	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9486.3;
	8176	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1682;
	8177	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	973.6;
	8177	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	831.8;
	8177	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1009.8;
	8177	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3480.2;
	8177	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11306.2;
	8177	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5745.4;
	8177	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9418.9;
	8177	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1742.8;
	8178	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	993.4;
	8178	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	879.6;
	8178	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1049.5;
	8178	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3492.5;
	8178	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12102.2;
	8178	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6200.6;
	8178	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9725.6;
	8178	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1845.1;
	8179	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1038.3;
	8179	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	915;
	8179	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1098;
	8179	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3687;
	8179	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12538.2;
	8179	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6618.9;
	8179	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9960.5;
	8179	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1924.2;
	8180	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1044.6;
	8180	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	917.3;
	8180	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1101.9;
	8180	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3676.6;
	8180	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12461.2;
	8180	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6651.3;
	8180	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9872.6;
	8180	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1854.8;
	8181	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1048.3;
	8181	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	913.6;
	8181	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1101.7;
	8181	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3640.1;
	8181	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12284.8;
	8181	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6626.5;
	8181	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9710.7;
	8181	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1824.6;
	8182	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1044.8;
	8182	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	888.2;
	8182	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1067.5;
	8182	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3508.8;
	8182	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11752.9;
	8182	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6367.5;
	8182	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9393.7;
	8182	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1752.8;
	8183	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1024.2;
	8183	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	843.5;
	8183	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1011.3;
	8183	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3274.2;
	8183	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10903.5;
	8183	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5935.6;
	8183	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8890.3;
	8183	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1646.7;
	8184	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000.3;
	8184	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	806.5;
	8184	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	965.6;
	8184	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2969;
	8184	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10047.1;
	8184	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5460.4;
	8184	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8293.4;
	8184	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1576.1;
	8185	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989.9;
	8185	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	796.1;
	8185	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	935.8;
	8185	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2756.5;
	8185	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9624.6;
	8185	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5210.5;
	8185	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7876.8;
	8185	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1511.6;
	8186	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	987.2;
	8186	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	792;
	8186	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	927.9;
	8186	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2660.5;
	8186	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9442.2;
	8186	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5075.7;
	8186	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7711;
	8186	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1454.3;
	8187	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986.1;
	8187	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	791.2;
	8187	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	925.9;
	8187	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2614.4;
	8187	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9431.9;
	8187	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5085.2;
	8187	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7633.8;
	8187	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1447.6;
	8188	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	984.8;
	8188	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	799.1;
	8188	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	934.3;
	8188	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2629;
	8188	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9522.3;
	8188	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5161.2;
	8188	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7665.4;
	8188	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1453.3;
	8189	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990.7;
	8189	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	824.7;
	8189	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	952.3;
	8189	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2679.3;
	8189	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10008.3;
	8189	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5401.1;
	8189	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7901.9;
	8189	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1501.1;
	8190	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1006.1;
	8190	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	865.6;
	8190	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1000.6;
	8190	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2877.1;
	8190	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10970.1;
	8190	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5924.6;
	8190	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8444.9;
	8190	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1622.5;
	8191	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1044.4;
	8191	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	945.5;
	8191	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1100;
	8191	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3323.5;
	8191	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12433.2;
	8191	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6756;
	8191	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9375.3;
	8191	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1826.7;
	8192	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1057;
	8192	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	971.3;
	8192	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1132.8;
	8192	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3398.2;
	8192	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12762.7;
	8192	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6927.9;
	8192	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9541.2;
	8192	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1878.8;
	8193	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1042.5;
	8193	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	957.5;
	8193	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1111.3;
	8193	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3323.7;
	8193	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12448.6;
	8193	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6695.2;
	8193	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9569.2;
	8193	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1855.6;
	8194	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1033.6;
	8194	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	954.8;
	8194	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1090.4;
	8194	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3312.7;
	8194	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12150.9;
	8194	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6561.9;
	8194	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9592.9;
	8194	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1848.4;
	8195	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1020.1;
	8195	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	928.2;
	8195	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1075;
	8195	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3337.7;
	8195	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11885.5;
	8195	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6406.7;
	8195	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9609.3;
	8195	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1822.6;
	8196	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009.1;
	8196	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	902.3;
	8196	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1056;
	8196	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3332.5;
	8196	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11608.4;
	8196	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6278.7;
	8196	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9540.3;
	8196	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1783.5;
	8197	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1001.8;
	8197	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	874;
	8197	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1043.7;
	8197	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3324.9;
	8197	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11400.2;
	8197	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6114.4;
	8197	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9493;
	8197	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1750.7;
	8198	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000.9;
	8198	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	860;
	8198	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1038.2;
	8198	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3342.1;
	8198	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11283.9;
	8198	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5966;
	8198	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9519.4;
	8198	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1720.8;
	8199	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.4;
	8199	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	847.6;
	8199	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1018.9;
	8199	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3358.1;
	8199	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11160.5;
	8199	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5845.8;
	8199	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9487.8;
	8199	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1683.8;
	8200	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000.6;
	8200	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	851.8;
	8200	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	996.3;
	8200	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3377;
	8200	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11155.1;
	8200	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5828;
	8200	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9446;
	8200	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1708.6;
	8201	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1017.4;
	8201	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	876.8;
	8201	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1007.9;
	8201	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3414.4;
	8201	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11492.8;
	8201	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5966.2;
	8201	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9552.8;
	8201	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1752.1;
	8202	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1043.8;
	8202	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	934.7;
	8202	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1051.9;
	8202	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3547;
	8202	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12363.2;
	8202	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6389.1;
	8202	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10081;
	8202	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1889.8;
	8203	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1072.2;
	8203	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	963.7;
	8203	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1096.3;
	8203	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3769.8;
	8203	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12771.2;
	8203	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6768.2;
	8203	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10183.2;
	8203	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1924.8;
	8204	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1083.4;
	8204	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	975.5;
	8204	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1098.9;
	8204	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3779.8;
	8204	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12676.4;
	8204	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6737.9;
	8204	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10108.8;
	8204	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1937.6;
	8205	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1091.9;
	8205	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	975.7;
	8205	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1096.5;
	8205	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3748.1;
	8205	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12534.4;
	8205	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6651.7;
	8205	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9968.9;
	8205	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1928;
	8206	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1086.1;
	8206	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	956.3;
	8206	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1062.6;
	8206	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3622.4;
	8206	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12190.8;
	8206	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6399.4;
	8206	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9579.4;
	8206	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1872.2;
	8207	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1067.5;
	8207	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	915.5;
	8207	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1015;
	8207	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3415.4;
	8207	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11502.7;
	8207	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5909.3;
	8207	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8994.4;
	8207	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1745.8;
	8208	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1054.9;
	8208	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	886;
	8208	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	969.2;
	8208	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3117.3;
	8208	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10805.8;
	8208	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5417.2;
	8208	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8441.6;
	8208	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1562.7;
	8209	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1046.2;
	8209	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	861.2;
	8209	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	867.8;
	8209	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2823.2;
	8209	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10688.6;
	8209	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4897.2;
	8209	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8109.6;
	8209	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1564.1;
	8210	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1042.6;
	8210	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	860;
	8210	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	864;
	8210	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2696.8;
	8210	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10646.1;
	8210	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4769.8;
	8210	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7884.8;
	8210	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1520.8;
	8211	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1048.5;
	8211	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	869.7;
	8211	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	869.5;
	8211	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2634.4;
	8211	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10815.1;
	8211	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4746.1;
	8211	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7790.7;
	8211	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1466.6;
	8212	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1053.8;
	8212	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	886.5;
	8212	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878.8;
	8212	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2608.5;
	8212	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11097.7;
	8212	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4822.1;
	8212	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7806.3;
	8212	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1526.8;
	8213	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1069.1;
	8213	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	921.4;
	8213	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	905;
	8213	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2656.1;
	8213	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11739.1;
	8213	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5064.3;
	8213	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8071;
	8213	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1622.4;
	8214	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1092.6;
	8214	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	972.5;
	8214	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	960.5;
	8214	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2843.1;
	8214	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12876.9;
	8214	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5651.1;
	8214	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8714.7;
	8214	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1756;
	8215	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1128.5;
	8215	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1058.1;
	8215	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1064.6;
	8215	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3271.9;
	8215	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14408.3;
	8215	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6557.5;
	8215	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9632.8;
	8215	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2001;
	8216	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1154.6;
	8216	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1090.9;
	8216	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1095.4;
	8216	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3370.8;
	8216	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15059;
	8216	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6903.8;
	8216	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10031.4;
	8216	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2105.7;
	8217	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1151.1;
	8217	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1089.5;
	8217	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1062.6;
	8217	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3327.6;
	8217	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15010.3;
	8217	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6879.2;
	8217	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10233.1;
	8217	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2139.8;
	8218	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1161.2;
	8218	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1086.7;
	8218	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1034;
	8218	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3368.1;
	8218	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15068.6;
	8218	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6898.4;
	8218	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10521.4;
	8218	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2181.2;
	8219	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1163.5;
	8219	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1071.1;
	8219	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1000.2;
	8219	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3411.2;
	8219	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15142.8;
	8219	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6875.5;
	8219	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10741;
	8219	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2178;
	8220	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1162;
	8220	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1057.5;
	8220	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	976.3;
	8220	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3455.9;
	8220	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15107.4;
	8220	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6915.6;
	8220	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10831.1;
	8220	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2204.4;
	8221	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1156.7;
	8221	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1041.2;
	8221	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	951.3;
	8221	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3499.5;
	8221	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15139.9;
	8221	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6905.3;
	8221	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10846.6;
	8221	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2191;
	8222	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1156;
	8222	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1029;
	8222	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940.5;
	8222	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3535.4;
	8222	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15143.8;
	8222	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6940.4;
	8222	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10828.4;
	8222	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2172.4;
	8223	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1150.5;
	8223	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1023.7;
	8223	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	931.4;
	8223	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3593;
	8223	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15097.4;
	8223	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7002.9;
	8223	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10736.4;
	8223	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2173.8;
	8224	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1147.7;
	8224	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1022;
	8224	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	928.1;
	8224	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3631.3;
	8224	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15180.9;
	8224	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7089.2;
	8224	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10790;
	8224	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2155.4;
	8225	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1158;
	8225	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1035.4;
	8225	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	932;
	8225	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3715.5;
	8225	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15499.8;
	8225	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7355.9;
	8225	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11151.1;
	8225	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2271.7;
	8226	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1183.9;
	8226	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1098.1;
	8226	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	966.3;
	8226	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3887.2;
	8226	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16405;
	8226	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7847.3;
	8226	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11898.9;
	8226	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2364.9;
	8227	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1214.4;
	8227	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1137.1;
	8227	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1025.2;
	8227	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4147.1;
	8227	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16739.7;
	8227	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8273.4;
	8227	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12148.4;
	8227	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2426.8;
	8228	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1218;
	8228	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1140;
	8228	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1029.8;
	8228	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4176;
	8228	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16655.9;
	8228	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8305.9;
	8228	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12134.8;
	8228	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2431.3;
	8229	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1223.2;
	8229	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1142.1;
	8229	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1032;
	8229	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4166.3;
	8229	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16540.1;
	8229	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8217.5;
	8229	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12042.7;
	8229	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2491.1;
	8230	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1208.4;
	8230	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1114.4;
	8230	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1009.1;
	8230	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4070.7;
	8230	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16152.8;
	8230	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7914.4;
	8230	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11676.9;
	8230	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2452.5;
	8231	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1176.8;
	8231	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1065.5;
	8231	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	971.9;
	8231	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3856.5;
	8231	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15389.6;
	8231	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7408.2;
	8231	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11073.7;
	8231	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2303.7;
	8232	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1149.6;
	8232	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1021.7;
	8232	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	925.9;
	8232	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3580.2;
	8232	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14617.7;
	8232	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6951.7;
	8232	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10541.1;
	8232	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2184.8;
	8233	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1100;
	8233	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	981.5;
	8233	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1197.3;
	8233	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4214.5;
	8233	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13369.8;
	8233	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7210.6;
	8233	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9721.4;
	8233	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1977.6;
	8234	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1094.4;
	8234	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	973.7;
	8234	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1183.4;
	8234	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4119.3;
	8234	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13235.3;
	8234	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7064.3;
	8234	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9553;
	8234	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1960.7;
	8235	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1091;
	8235	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	972.9;
	8235	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1178.1;
	8235	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4070.9;
	8235	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13218;
	8235	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7037;
	8235	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9490.8;
	8235	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1967.1;
	8236	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1091;
	8236	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	981.4;
	8236	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1180;
	8236	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4069;
	8236	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13333.4;
	8236	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7114.4;
	8236	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9540.4;
	8236	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1995.4;
	8237	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1095;
	8237	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	998.5;
	8237	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1195.6;
	8237	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4153.1;
	8237	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13692.2;
	8237	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7332.4;
	8237	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9818;
	8237	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2059.9;
	8238	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1113.9;
	8238	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1038.4;
	8238	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1243.5;
	8238	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4433.3;
	8238	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14483.8;
	8238	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7841.7;
	8238	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10469;
	8238	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2189.5;
	8239	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1140.9;
	8239	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1102.9;
	8239	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1327.9;
	8239	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4973.8;
	8239	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15602.9;
	8239	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8604.8;
	8239	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11385.4;
	8239	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2416.7;
	8240	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1156.2;
	8240	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1122.2;
	8240	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1352.1;
	8240	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5063.9;
	8240	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15739.1;
	8240	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8756.3;
	8240	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11548.1;
	8240	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2442.1;
	8241	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1147.6;
	8241	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1115.4;
	8241	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1315.7;
	8241	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4985.4;
	8241	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15356.7;
	8241	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8516.9;
	8241	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11602.5;
	8241	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2388.9;
	8242	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1146.2;
	8242	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1096.9;
	8242	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1301.4;
	8242	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4992.9;
	8242	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15114.2;
	8242	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8301.4;
	8242	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11679.1;
	8242	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2328.4;
	8243	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1134.8;
	8243	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1061.4;
	8243	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1277.8;
	8243	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4952.5;
	8243	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14817.2;
	8243	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7986.6;
	8243	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11481.5;
	8243	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2226.4;
	8244	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1120;
	8244	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1032.5;
	8244	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1243.5;
	8244	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4845.9;
	8244	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14400.7;
	8244	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7705;
	8244	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11080.9;
	8244	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2117.1;
	8245	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1092.4;
	8245	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1007.2;
	8245	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1193.3;
	8245	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4680.1;
	8245	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13918;
	8245	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7385.2;
	8245	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10667.4;
	8245	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2011.1;
	8246	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1072.7;
	8246	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	992.7;
	8246	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1154;
	8246	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4563.1;
	8246	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13507.7;
	8246	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7095.2;
	8246	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10378.1;
	8246	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1938.4;
	8247	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1052.6;
	8247	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	979.3;
	8247	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1122.8;
	8247	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4498.9;
	8247	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13119.7;
	8247	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6913.9;
	8247	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10226.3;
	8247	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1879.7;
	8248	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1034.6;
	8248	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	970;
	8248	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1104.3;
	8248	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4487.9;
	8248	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12907.9;
	8248	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6891.2;
	8248	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10223.5;
	8248	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1840.7;
	8249	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1026.2;
	8249	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	981;
	8249	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1113.3;
	8249	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4572.5;
	8249	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13101.2;
	8249	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7052.8;
	8249	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10396.8;
	8249	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1877.3;
	8250	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1038;
	8250	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1017.7;
	8250	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1154;
	8250	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4770.4;
	8250	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13975.1;
	8250	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7565.6;
	8250	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10981.7;
	8250	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2040.4;
	8251	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1078.1;
	8251	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1035.8;
	8251	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1203.5;
	8251	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4995.6;
	8251	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14470.1;
	8251	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7955.9;
	8251	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11172.9;
	8251	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2141.7;
	8252	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1090.7;
	8252	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1025.6;
	8252	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1209.2;
	8252	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4992.9;
	8252	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14457.3;
	8252	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7886.3;
	8252	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11011.9;
	8252	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2152.5;
	8253	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1098.9;
	8253	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1010.2;
	8253	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1208.3;
	8253	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4943.6;
	8253	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14412.5;
	8253	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7790.4;
	8253	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10904.3;
	8253	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2156.1;
	8254	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1096.5;
	8254	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	990;
	8254	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1196.1;
	8254	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4847.6;
	8254	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14176.1;
	8254	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7615.1;
	8254	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10681.9;
	8254	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2122.6;
	8255	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1083.5;
	8255	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	953.1;
	8255	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1164.9;
	8255	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4669.6;
	8255	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13626;
	8255	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7269.4;
	8255	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10302.7;
	8255	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2037.7;
	8256	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1066.1;
	8256	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	915.1;
	8256	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1133.7;
	8256	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4449.2;
	8256	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12938.4;
	8256	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6935.4;
	8256	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9791.6;
	8256	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1941.2;
	8257	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1053.7;
	8257	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	893.2;
	8257	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1095.2;
	8257	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4221.5;
	8257	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12390.6;
	8257	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6726.8;
	8257	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9403.5;
	8257	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1877.4;
	8258	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1046.6;
	8258	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	878.1;
	8258	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1085.7;
	8258	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4076.2;
	8258	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11993.2;
	8258	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6604.8;
	8258	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9224.2;
	8258	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1849.7;
	8259	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1042.5;
	8259	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	868.8;
	8259	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1080.6;
	8259	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3974.9;
	8259	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11707.3;
	8259	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6509.8;
	8259	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9143.9;
	8259	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1839.6;
	8260	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1043;
	8260	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	867.2;
	8260	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1079;
	8260	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3909.3;
	8260	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11594.3;
	8260	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6439.6;
	8260	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9179.7;
	8260	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1842.6;
	8261	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1049;
	8261	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	880.6;
	8261	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1087.5;
	8261	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3918.5;
	8261	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11703.5;
	8261	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6419.2;
	8261	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9367.8;
	8261	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1869.6;
	8262	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1060.1;
	8262	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	896.9;
	8262	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1109.3;
	8262	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4010.2;
	8262	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12065.4;
	8262	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6551.4;
	8262	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9682.2;
	8262	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1919;
	8263	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1071.6;
	8263	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	927.6;
	8263	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1141.9;
	8263	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4137.6;
	8263	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12603.3;
	8263	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6823.4;
	8263	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10061.4;
	8263	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1991;
	8264	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1089.1;
	8264	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	961.9;
	8264	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1179.1;
	8264	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4232.7;
	8264	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13061.9;
	8264	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7064.3;
	8264	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10322.9;
	8264	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2046.8;
	8265	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1085.3;
	8265	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	981.2;
	8265	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1192.2;
	8265	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4321.2;
	8265	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13426.6;
	8265	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7255.6;
	8265	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10594.1;
	8265	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2083.7;
	8266	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1072.3;
	8266	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	985.8;
	8266	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1165.3;
	8266	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4318.9;
	8266	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13615.8;
	8266	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7322;
	8266	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10689.8;
	8266	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2093.2;
	8267	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1052.4;
	8267	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	979.1;
	8267	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1121.3;
	8267	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4202.5;
	8267	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13491.1;
	8267	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7218.6;
	8267	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10584.4;
	8267	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2047.2;
	8268	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1026.5;
	8268	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	953.8;
	8268	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1085.8;
	8268	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4064.8;
	8268	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13189.8;
	8268	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7031.2;
	8268	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10355;
	8268	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1969.3;
	8269	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1003.1;
	8269	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	927.4;
	8269	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1043.2;
	8269	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3913.5;
	8269	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12720.4;
	8269	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6797.5;
	8269	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10083.8;
	8269	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1886.1;
	8270	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	987.6;
	8270	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	894.4;
	8270	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1009.6;
	8270	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3843.6;
	8270	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12253.1;
	8270	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6607.2;
	8270	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9736.6;
	8270	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1816.2;
	8271	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969.5;
	8271	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	872.9;
	8271	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	979.4;
	8271	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3786.1;
	8271	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11972.6;
	8271	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6474.1;
	8271	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9506.9;
	8271	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1761.2;
	8272	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.6;
	8272	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	862.5;
	8272	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	974.2;
	8272	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3773.2;
	8272	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11858.9;
	8272	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6432.6;
	8272	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9423.1;
	8272	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1745;
	8273	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958;
	8273	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	867.2;
	8273	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	983.1;
	8273	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3800.4;
	8273	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11992.5;
	8273	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6522.7;
	8273	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9578.6;
	8273	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1768.3;
	8274	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969.1;
	8274	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	905.8;
	8274	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1023.7;
	8274	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3940.9;
	8274	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12596.5;
	8274	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6835.1;
	8274	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10125.4;
	8274	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1865.3;
	8275	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1004.3;
	8275	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	925.2;
	8275	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1068.8;
	8275	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4109.4;
	8275	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12746.4;
	8275	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6957.2;
	8275	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10377.4;
	8275	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1912.9;
	8276	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1018.1;
	8276	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	920.1;
	8276	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1068.4;
	8276	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4093.5;
	8276	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12554.9;
	8276	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6844.7;
	8276	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10296.1;
	8276	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1884.1;
	8277	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1025.9;
	8277	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	915;
	8277	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1060.6;
	8277	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4052.7;
	8277	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12344.3;
	8277	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6728.1;
	8277	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10131.9;
	8277	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1859;
	8278	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1030.3;
	8278	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	895.6;
	8278	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1040.3;
	8278	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3975.5;
	8278	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11995.6;
	8278	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6521.7;
	8278	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9886.8;
	8278	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1807.5;
	8279	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1025.2;
	8279	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	862.6;
	8279	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	999.8;
	8279	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3831.6;
	8279	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11413.7;
	8279	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6194.9;
	8279	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9490.3;
	8279	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1713.3;
	8280	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.6;
	8280	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	824.7;
	8280	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	959.2;
	8280	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3644.7;
	8280	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10757.2;
	8280	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5793.2;
	8280	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8933.6;
	8280	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1618.6;
	8281	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1014;
	8281	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	789.6;
	8281	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	915.6;
	8281	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3357.2;
	8281	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10132.9;
	8281	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5463.8;
	8281	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8664.2;
	8281	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1529.6;
	8282	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009.1;
	8282	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	768.2;
	8282	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.6;
	8282	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3252.4;
	8282	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9708.8;
	8282	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5216.7;
	8282	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8337.2;
	8282	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1473.6;
	8283	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1003.1;
	8283	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	758.1;
	8283	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	876.6;
	8283	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3166.2;
	8283	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9442.7;
	8283	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5053.6;
	8283	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8091.5;
	8283	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1435.3;
	8284	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1003.9;
	8284	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	749.1;
	8284	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	870.2;
	8284	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3119.5;
	8284	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9299.2;
	8284	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4961.6;
	8284	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7945.9;
	8284	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1420.2;
	8285	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1007.1;
	8285	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	754.3;
	8285	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	869.6;
	8285	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3106.1;
	8285	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9283.8;
	8285	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4945.4;
	8285	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7909.1;
	8285	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1473.2;
	8286	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1011.2;
	8286	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	764.2;
	8286	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884;
	8286	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3164.5;
	8286	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9435.3;
	8286	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5044;
	8286	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7940.1;
	8286	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1508.4;
	8287	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1014.8;
	8287	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	780.6;
	8287	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	907.7;
	8287	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3248.4;
	8287	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9687;
	8287	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5232.7;
	8287	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8042.8;
	8287	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1534.1;
	8288	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1012.4;
	8288	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	794.9;
	8288	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	934.8;
	8288	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3285.4;
	8288	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9977.1;
	8288	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5414.5;
	8288	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8053.6;
	8288	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1563.9;
	8289	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1015.2;
	8289	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	811.8;
	8289	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	953.5;
	8289	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3379.3;
	8289	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10331.5;
	8289	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5598.2;
	8289	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8251.7;
	8289	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1610.4;
	8290	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1017.4;
	8290	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	822.2;
	8290	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	953.9;
	8290	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3429.4;
	8290	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10527.6;
	8290	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5725.8;
	8290	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8520.7;
	8290	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1602.1;
	8291	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1008;
	8291	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	802.9;
	8291	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	935.5;
	8291	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3400.7;
	8291	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10430.4;
	8291	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5728.6;
	8291	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8703.1;
	8291	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1563.5;
	8292	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990.3;
	8292	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	781.7;
	8292	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	912.9;
	8292	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3333;
	8292	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10142.8;
	8292	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5682.2;
	8292	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8835.6;
	8292	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1507.8;
	8293	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	979;
	8293	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	762.8;
	8293	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	895.9;
	8293	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3241.8;
	8293	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9756.7;
	8293	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5606.9;
	8293	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8948.3;
	8293	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1494;
	8294	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.1;
	8294	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742.8;
	8294	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	877.8;
	8294	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3212.6;
	8294	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9450.3;
	8294	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5484.5;
	8294	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9014.9;
	8294	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1473.9;
	8295	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951.4;
	8295	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	729.7;
	8295	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.4;
	8295	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3197.8;
	8295	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9245.5;
	8295	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5415.8;
	8295	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8997.2;
	8295	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1412.7;
	8296	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.3;
	8296	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720.9;
	8296	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851.6;
	8296	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3203.3;
	8296	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9162.1;
	8296	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5382.8;
	8296	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8997.4;
	8296	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1400.5;
	8297	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.8;
	8297	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.1;
	8297	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	858.5;
	8297	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3223.1;
	8297	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9256.1;
	8297	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5467.5;
	8297	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9084.2;
	8297	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1423.5;
	8298	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.6;
	8298	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	778.1;
	8298	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	898.2;
	8298	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3341.7;
	8298	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9916.2;
	8298	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5800.5;
	8298	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9562.3;
	8298	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1524;
	8299	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986.4;
	8299	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	812.3;
	8299	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	958.1;
	8299	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3596;
	8299	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10510.3;
	8299	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6183.2;
	8299	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9942.4;
	8299	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1597.5;
	8300	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.9;
	8300	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	809.4;
	8300	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	965.9;
	8300	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3633.6;
	8300	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10466.3;
	8300	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6207.6;
	8300	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9937.2;
	8300	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1589.2;
	8301	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.2;
	8301	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	800.3;
	8301	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	953;
	8301	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3574.5;
	8301	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10204.3;
	8301	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6078.8;
	8301	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9806.4;
	8301	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1545.1;
	8302	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	987;
	8302	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	787.8;
	8302	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	933.1;
	8302	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3501;
	8302	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9889.2;
	8302	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5827.4;
	8302	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9537.7;
	8302	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1535.5;
	8303	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	973.7;
	8303	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755.4;
	8303	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889.9;
	8303	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3330.8;
	8303	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9279;
	8303	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5412.4;
	8303	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9030.3;
	8303	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1416;
	8304	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.9;
	8304	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.6;
	8304	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	848.2;
	8304	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3095.2;
	8304	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8592.6;
	8304	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4954.1;
	8304	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8452.5;
	8304	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1292;
	8305	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.2;
	8305	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714.3;
	8305	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	828.7;
	8305	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2831.3;
	8305	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8122.8;
	8305	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4589.8;
	8305	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7941;
	8305	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1188.6;
	8306	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.7;
	8306	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	710.5;
	8306	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	821.7;
	8306	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2667.3;
	8306	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7955.3;
	8306	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4421.7;
	8306	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7630.9;
	8306	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1222.8;
	8307	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.6;
	8307	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	722.3;
	8307	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	831.7;
	8307	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2589.8;
	8307	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8038.9;
	8307	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4377.6;
	8307	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7470.1;
	8307	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1195.4;
	8308	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.1;
	8308	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742.3;
	8308	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849;
	8308	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2552.1;
	8308	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8306.9;
	8308	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4434.9;
	8308	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7468.7;
	8308	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1184.4;
	8309	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.5;
	8309	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	773.9;
	8309	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	879.5;
	8309	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2585.4;
	8309	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8909.8;
	8309	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4635.3;
	8309	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7647;
	8309	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1226.4;
	8310	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.8;
	8310	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	834.1;
	8310	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	943.7;
	8310	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2771.9;
	8310	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10050.4;
	8310	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5224.5;
	8310	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8180.5;
	8310	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1331.6;
	8311	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.5;
	8311	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	936.3;
	8311	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1055.7;
	8311	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3166.5;
	8311	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11717.5;
	8311	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6028.7;
	8311	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8956.2;
	8311	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1507.6;
	8312	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1037.4;
	8312	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	978.1;
	8312	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1100.1;
	8312	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3264.2;
	8312	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12369.8;
	8312	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6288;
	8312	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9227.4;
	8312	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1600.9;
	8313	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1024.3;
	8313	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	963.9;
	8313	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1073.7;
	8313	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3288.1;
	8313	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12168;
	8313	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6122.8;
	8313	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9336.3;
	8313	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1613;
	8314	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.4;
	8314	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	935.5;
	8314	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1055.8;
	8314	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3360;
	8314	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11902.6;
	8314	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5977.1;
	8314	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9517;
	8314	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1646.1;
	8315	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000.9;
	8315	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	902.1;
	8315	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1024.3;
	8315	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3341.8;
	8315	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11586.8;
	8315	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5778;
	8315	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9724;
	8315	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1636.1;
	8316	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	978.9;
	8316	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	870.3;
	8316	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	976.6;
	8316	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3402.4;
	8316	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11159.8;
	8316	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5632.2;
	8316	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9812.4;
	8316	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1617.4;
	8317	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.4;
	8317	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	832.4;
	8317	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	938.3;
	8317	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3411;
	8317	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10727.6;
	8317	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5548.7;
	8317	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9838.8;
	8317	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1634.2;
	8318	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.5;
	8318	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	814.6;
	8318	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	908.4;
	8318	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3432.7;
	8318	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10430.4;
	8318	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5448.1;
	8318	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9918.5;
	8318	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1628.2;
	8319	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.2;
	8319	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	799.3;
	8319	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	877.5;
	8319	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3461.5;
	8319	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10160.9;
	8319	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5382.6;
	8319	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9963.1;
	8319	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1605;
	8320	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.9;
	8320	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	792.7;
	8320	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	862.7;
	8320	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3491.4;
	8320	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10013.5;
	8320	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5402.4;
	8320	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9933.3;
	8320	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1585.3;
	8321	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934;
	8321	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	807.8;
	8321	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	864.9;
	8321	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3510.4;
	8321	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10159;
	8321	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5460.3;
	8321	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9948.6;
	8321	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1599.8;
	8322	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.7;
	8322	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	856.6;
	8322	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	903.6;
	8322	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3572.8;
	8322	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11067.9;
	8322	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5785.2;
	8322	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10328.4;
	8322	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1680.1;
	8323	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981.6;
	8323	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	898.3;
	8323	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	954.8;
	8323	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3791.6;
	8323	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11705.7;
	8323	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6260.6;
	8323	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10500.6;
	8323	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1771.6;
	8324	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	991.3;
	8324	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	900.4;
	8324	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	967.8;
	8324	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3793.3;
	8324	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11720.1;
	8324	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6252;
	8324	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10409.9;
	8324	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1790.4;
	8325	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	997.4;
	8325	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	892.3;
	8325	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	970;
	8325	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3747.1;
	8325	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11622.4;
	8325	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6185;
	8325	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10235.7;
	8325	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1757.5;
	8326	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985.2;
	8326	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	871.9;
	8326	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	950;
	8326	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3614.1;
	8326	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11256.9;
	8326	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5930.9;
	8326	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9869.3;
	8326	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1697.6;
	8327	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.2;
	8327	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	829.8;
	8327	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	901.4;
	8327	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3359.3;
	8327	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10467;
	8327	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5474.5;
	8327	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9274.3;
	8327	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1562.3;
	8328	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.1;
	8328	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	784.7;
	8328	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.1;
	8328	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3043.3;
	8328	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9676.3;
	8328	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4981.7;
	8328	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8608.6;
	8328	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1396.4;
	8329	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.6;
	8329	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753.2;
	8329	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	822.8;
	8329	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2811.4;
	8329	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9100.5;
	8329	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4572.4;
	8329	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8152.8;
	8329	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1324.6;
	8330	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925.8;
	8330	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.6;
	8330	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	806.6;
	8330	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2665.5;
	8330	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8865.1;
	8330	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4391.7;
	8330	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7860.1;
	8330	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1312.8;
	8331	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.2;
	8331	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740;
	8331	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	804.2;
	8331	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2594.9;
	8331	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8793.4;
	8331	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4289;
	8331	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7703.5;
	8331	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1215.6;
	8332	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.9;
	8332	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742.2;
	8332	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	809;
	8332	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2566.5;
	8332	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8892.8;
	8332	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4259.6;
	8332	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7687.7;
	8332	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1183.2;
	8333	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933;
	8333	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	759.5;
	8333	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	823;
	8333	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2592;
	8333	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9184.2;
	8333	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4307;
	8333	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7835.5;
	8333	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1260.7;
	8334	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.5;
	8334	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	795.7;
	8334	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	873.7;
	8334	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2759;
	8334	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10028.4;
	8334	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4694.7;
	8334	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8362.7;
	8334	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1362.3;
	8335	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.2;
	8335	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	871.9;
	8335	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	972.4;
	8335	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3162;
	8335	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11306.7;
	8335	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5376.1;
	8335	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9192.2;
	8335	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1489.6;
	8336	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1010.2;
	8336	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	895.5;
	8336	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1012.7;
	8336	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3252.3;
	8336	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11665.9;
	8336	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5582.7;
	8336	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9449.1;
	8336	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1577;
	8337	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	993.3;
	8337	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	893.4;
	8337	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	990.1;
	8337	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3209.3;
	8337	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11412.6;
	8337	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5492.2;
	8337	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9551.4;
	8337	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1620.4;
	8338	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.4;
	8338	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	888.8;
	8338	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	979.3;
	8338	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3308.5;
	8338	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11395.9;
	8338	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5487.4;
	8338	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9845.3;
	8338	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1655.6;
	8339	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	974.1;
	8339	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	874.5;
	8339	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	963.2;
	8339	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3401;
	8339	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11365.1;
	8339	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5481.3;
	8339	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10170.5;
	8339	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1646.7;
	8340	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964;
	8340	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	848.4;
	8340	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	934.3;
	8340	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3480.4;
	8340	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11244.2;
	8340	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5453.7;
	8340	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10352.5;
	8340	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1629.2;
	8341	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.8;
	8341	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	819;
	8341	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	905.8;
	8341	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3580;
	8341	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10923.2;
	8341	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5421.9;
	8341	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10415.6;
	8341	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1628.7;
	8342	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.4;
	8342	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	806;
	8342	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	886.9;
	8342	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3666;
	8342	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10532.3;
	8342	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5442.8;
	8342	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10495.7;
	8342	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1680.7;
	8343	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.2;
	8343	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	799.9;
	8343	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	880.8;
	8343	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3729.7;
	8343	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10200.8;
	8343	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5477.6;
	8343	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10588.1;
	8343	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1625.5;
	8344	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.5;
	8344	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	794.3;
	8344	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	873.5;
	8344	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3726.8;
	8344	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10083;
	8344	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5517.2;
	8344	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10565.4;
	8344	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1580.7;
	8345	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.1;
	8345	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	810.2;
	8345	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.9;
	8345	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3720.8;
	8345	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10278.5;
	8345	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5559.1;
	8345	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10499.5;
	8345	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1606.7;
	8346	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	954.9;
	8346	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	867.5;
	8346	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	926.3;
	8346	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3735;
	8346	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11142.8;
	8346	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5776.7;
	8346	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10701.7;
	8346	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1688.4;
	8347	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	994.7;
	8347	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	917.8;
	8347	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	990.4;
	8347	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3914.3;
	8347	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11907.4;
	8347	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6140.1;
	8347	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10839.9;
	8347	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1781.3;
	8348	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.2;
	8348	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	925.6;
	8348	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1002.6;
	8348	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3876;
	8348	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11999.5;
	8348	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6110.5;
	8348	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10633.6;
	8348	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1708.6;
	8349	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1003.1;
	8349	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	935.4;
	8349	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1010.4;
	8349	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3821.7;
	8349	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12022.2;
	8349	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6013.9;
	8349	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10368.2;
	8349	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1677.9;
	8350	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.9;
	8350	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	924.4;
	8350	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	992.2;
	8350	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3692.7;
	8350	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11722.7;
	8350	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5732;
	8350	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9899.5;
	8350	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1611.7;
	8351	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985.1;
	8351	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	890.4;
	8351	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	947.5;
	8351	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3444.6;
	8351	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11054.3;
	8351	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5323.5;
	8351	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9226;
	8351	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1514.9;
	8352	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.3;
	8352	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	849.3;
	8352	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	908.1;
	8352	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3112.3;
	8352	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10322.7;
	8352	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4859.4;
	8352	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8518.1;
	8352	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1457.3;
	8353	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.7;
	8353	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	838.4;
	8353	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	887;
	8353	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2830.5;
	8353	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9895.4;
	8353	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4509.1;
	8353	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7944.8;
	8353	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1419.7;
	8354	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.8;
	8354	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	835.9;
	8354	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884.2;
	8354	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2685.6;
	8354	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9758.2;
	8354	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4372.2;
	8354	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7672.8;
	8354	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1358.5;
	8355	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.5;
	8355	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	836.5;
	8355	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	896.7;
	8355	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2592.2;
	8355	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9741.6;
	8355	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4393;
	8355	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7496.8;
	8355	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1361;
	8356	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969.2;
	8356	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	848.6;
	8356	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	919;
	8356	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2569.9;
	8356	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9891.3;
	8356	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4459.4;
	8356	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7498.6;
	8356	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1360.5;
	8357	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	978.7;
	8357	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	873.8;
	8357	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	948.8;
	8357	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2598.5;
	8357	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10361.5;
	8357	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4675.5;
	8357	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7678.3;
	8357	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1393;
	8358	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	998.2;
	8358	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	911.4;
	8358	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1008.8;
	8358	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2778.2;
	8358	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11298.3;
	8358	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5202.8;
	8358	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8200.3;
	8358	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1511.1;
	8359	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1033.9;
	8359	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	987.2;
	8359	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1121.6;
	8359	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3197.1;
	8359	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12637;
	8359	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6021.2;
	8359	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8959.6;
	8359	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1705.9;
	8360	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1053.4;
	8360	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1018;
	8360	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1168;
	8360	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3299.9;
	8360	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13101.9;
	8360	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6310.2;
	8360	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9162.8;
	8360	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1767.1;
	8361	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1038.3;
	8361	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1006.1;
	8361	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1145.7;
	8361	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3239.2;
	8361	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12901.4;
	8361	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6180.6;
	8361	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9219.6;
	8361	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1762.4;
	8362	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1031;
	8362	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	999.7;
	8362	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1120.4;
	8362	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3256.9;
	8362	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12815.2;
	8362	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6060.9;
	8362	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9359;
	8362	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1779.4;
	8363	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1011.8;
	8363	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	981.5;
	8363	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1085.5;
	8363	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3284.3;
	8363	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12683.3;
	8363	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5928;
	8363	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9494.5;
	8363	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1769.9;
	8364	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	993;
	8364	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	943.8;
	8364	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1041.1;
	8364	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3284.1;
	8364	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12433.7;
	8364	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5805.6;
	8364	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9565.5;
	8364	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1749.8;
	8365	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	973.9;
	8365	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	901.3;
	8365	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1001;
	8365	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3265.8;
	8365	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11967.8;
	8365	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5688.9;
	8365	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9550.9;
	8365	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1712.1;
	8366	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963.7;
	8366	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	883.3;
	8366	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	961.9;
	8366	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3264.9;
	8366	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11498.5;
	8366	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5601.5;
	8366	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9554.7;
	8366	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1692.7;
	8367	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.9;
	8367	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	867.3;
	8367	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	925.9;
	8367	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3276.6;
	8367	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11143;
	8367	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5511.9;
	8367	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9542.6;
	8367	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1661.8;
	8368	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.3;
	8368	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	860.7;
	8368	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	899.6;
	8368	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3293.6;
	8368	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11083.9;
	8368	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5562.9;
	8368	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9530.1;
	8368	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1676.4;
	8369	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959.9;
	8369	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	884.9;
	8369	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	902.4;
	8369	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3320.3;
	8369	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11405.3;
	8369	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5659.7;
	8369	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9599;
	8369	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1706.3;
	8370	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.7;
	8370	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	952.4;
	8370	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	936.7;
	8370	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3398.9;
	8370	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12350.6;
	8370	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6009.6;
	8370	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9990;
	8370	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1830.6;
	8371	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	993.4;
	8371	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	995.3;
	8371	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	999.4;
	8371	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3624.3;
	8371	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13074.3;
	8371	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6421.2;
	8371	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10140.9;
	8371	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1881.2;
	8372	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000.7;
	8372	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1000.4;
	8372	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1020.7;
	8372	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3644.5;
	8372	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13087.4;
	8372	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6422.5;
	8372	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10039.4;
	8372	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1892;
	8373	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.5;
	8373	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1003.2;
	8373	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1033.5;
	8373	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3637.1;
	8373	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13056.9;
	8373	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6349.9;
	8373	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9923.3;
	8373	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1883.9;
	8374	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1007.6;
	8374	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	978.8;
	8374	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1014.8;
	8374	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3523.7;
	8374	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12715.6;
	8374	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6103.4;
	8374	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9533.3;
	8374	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1817.8;
	8375	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	993.5;
	8375	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	931.2;
	8375	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	976.3;
	8375	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3313.3;
	8375	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11955.8;
	8375	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5650;
	8375	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8938.8;
	8375	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1694.5;
	8376	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.5;
	8376	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	894.1;
	8376	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	934.4;
	8376	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3013;
	8376	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11150.4;
	8376	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5170.5;
	8376	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8326.6;
	8376	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1497.3;
	8377	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.5;
	8377	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	856.1;
	8377	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.9;
	8377	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2827.5;
	8377	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10496.7;
	8377	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4952.3;
	8377	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7887.7;
	8377	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1488.1;
	8378	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.1;
	8378	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	847.8;
	8378	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	876.4;
	8378	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2690.3;
	8378	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10276.2;
	8378	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4794.7;
	8378	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7644.5;
	8378	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1440.7;
	8379	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970.8;
	8379	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	853.8;
	8379	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.1;
	8379	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2611.7;
	8379	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10263.5;
	8379	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4731.2;
	8379	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7541.2;
	8379	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1372.8;
	8380	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.1;
	8380	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	864.9;
	8380	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	886.5;
	8380	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2572;
	8380	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10373.6;
	8380	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4773.4;
	8380	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7550.2;
	8380	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1408.7;
	8381	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	991.1;
	8381	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	885.8;
	8381	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913.5;
	8381	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2612.5;
	8381	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10862;
	8381	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4963.6;
	8381	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7805.3;
	8381	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1497.8;
	8382	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1015;
	8382	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	928;
	8382	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	970.4;
	8382	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2784.8;
	8382	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11821.4;
	8382	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5481.2;
	8382	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8459;
	8382	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1624.5;
	8383	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1052.9;
	8383	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1005.4;
	8383	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1078.1;
	8383	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3192.6;
	8383	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13188.1;
	8383	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6260.8;
	8383	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9415;
	8383	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1852.1;
	8384	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1077.9;
	8384	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1036;
	8384	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1123.7;
	8384	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3301.1;
	8384	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13757.8;
	8384	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6552.4;
	8384	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9744.3;
	8384	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1937.7;
	8385	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1074.7;
	8385	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1034.1;
	8385	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1129.2;
	8385	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3251.7;
	8385	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13670.6;
	8385	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6502.3;
	8385	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9872.7;
	8385	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1934.2;
	8386	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1080.1;
	8386	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1031.1;
	8386	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1151.6;
	8386	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3270.6;
	8386	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13633.7;
	8386	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6557.4;
	8386	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9935;
	8386	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1945.7;
	8387	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1075.1;
	8387	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1024.2;
	8387	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1152;
	8387	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3291.8;
	8387	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13643.2;
	8387	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6555;
	8387	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9928.7;
	8387	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1902.7;
	8388	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1067.2;
	8388	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1001.9;
	8388	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1142.8;
	8388	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3309.7;
	8388	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13489.1;
	8388	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6470.8;
	8388	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9832.2;
	8388	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1897.4;
	8389	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1046.4;
	8389	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	981.7;
	8389	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1110.6;
	8389	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3293.6;
	8389	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13254.6;
	8389	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6355.8;
	8389	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9661.3;
	8389	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1847.6;
	8390	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1037.2;
	8390	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	972.1;
	8390	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1097.8;
	8390	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3275.7;
	8390	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12983.3;
	8390	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6237.7;
	8390	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9577.9;
	8390	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1801.7;
	8391	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1022.1;
	8391	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	959.4;
	8391	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1089.9;
	8391	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3257.1;
	8391	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12741.7;
	8391	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6151.1;
	8391	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9489.5;
	8391	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1768.9;
	8392	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009.5;
	8392	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	956.4;
	8392	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1084.7;
	8392	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3242.2;
	8392	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12631.8;
	8392	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6148.1;
	8392	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9394.9;
	8392	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1708.2;
	8393	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1005.8;
	8393	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	964.2;
	8393	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1095.3;
	8393	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3279.3;
	8393	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12814.5;
	8393	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6275.9;
	8393	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9400.8;
	8393	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1808.6;
	8394	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1019.3;
	8394	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1002.6;
	8394	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1132.2;
	8394	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3414.1;
	8394	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13470.6;
	8394	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6676.3;
	8394	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9876.9;
	8394	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1890.6;
	8395	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1054.6;
	8395	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1017.9;
	8395	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1173.9;
	8395	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3622.5;
	8395	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13783;
	8395	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7008;
	8395	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10280.5;
	8395	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1973;
	8396	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1059.4;
	8396	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1008;
	8396	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1169.6;
	8396	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3638.1;
	8396	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13661.2;
	8396	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6943.1;
	8396	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10313.8;
	8396	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1979.7;
	8397	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1065.4;
	8397	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	994.5;
	8397	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1158.1;
	8397	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3637;
	8397	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13435.2;
	8397	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6823;
	8397	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10333.3;
	8397	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2043.3;
	8398	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1049.5;
	8398	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	958.2;
	8398	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1123.5;
	8398	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3554.3;
	8398	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12988.7;
	8398	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6522.5;
	8398	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10071.6;
	8398	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2012.1;
	8399	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1020.8;
	8399	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	902.1;
	8399	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1058.2;
	8399	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3362.3;
	8399	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12074;
	8399	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6021.3;
	8399	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9571.5;
	8399	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1865.1;
	8400	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	996.3;
	8400	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	854.6;
	8400	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	999.3;
	8400	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3094.5;
	8400	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11154.4;
	8400	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5541;
	8400	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9051.3;
	8400	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1746;
	8401	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1003.7;
	8401	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	839.8;
	8401	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	914.1;
	8401	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2947.5;
	8401	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10415.4;
	8401	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5135.5;
	8401	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8730.6;
	8401	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1563;
	8402	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	996.6;
	8402	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	825.5;
	8402	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	888;
	8402	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2819.8;
	8402	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10073;
	8402	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4929.5;
	8402	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8501.7;
	8402	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1525.5;
	8403	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	991.4;
	8403	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	819.5;
	8403	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.9;
	8403	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2745.6;
	8403	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9931.8;
	8403	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4796.3;
	8403	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8358.9;
	8403	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1510.6;
	8404	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.8;
	8404	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	823.6;
	8404	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.7;
	8404	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2716.3;
	8404	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9931.8;
	8404	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4767.2;
	8404	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8336.4;
	8404	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1510.6;
	8405	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	995.7;
	8405	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	836.5;
	8405	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.6;
	8405	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2754.2;
	8405	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10201;
	8405	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4928.1;
	8405	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8490.3;
	8405	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1537.7;
	8406	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.7;
	8406	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	877.4;
	8406	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	906.9;
	8406	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2916.8;
	8406	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11008;
	8406	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5332.1;
	8406	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9015;
	8406	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1637.1;
	8407	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1040.7;
	8407	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	944.7;
	8407	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	980.8;
	8407	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3301.7;
	8407	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12207.8;
	8407	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6011.5;
	8407	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9751.1;
	8407	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1824.6;
	8408	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1060.5;
	8408	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	964.8;
	8408	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1010.3;
	8408	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3383.7;
	8408	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12566.1;
	8408	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6268.4;
	8408	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9970.2;
	8408	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1877.8;
	8409	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1047.6;
	8409	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	954.1;
	8409	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	990.3;
	8409	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3329;
	8409	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12365.9;
	8409	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6163.3;
	8409	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10006.3;
	8409	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1863.7;
	8410	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1036;
	8410	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	940.9;
	8410	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	979.2;
	8410	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3373.1;
	8410	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12300.3;
	8410	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6145.7;
	8410	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9914.2;
	8410	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1869.5;
	8411	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1027;
	8411	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	921.3;
	8411	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	960.5;
	8411	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3415;
	8411	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12245;
	8411	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6104;
	8411	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9949.9;
	8411	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1848.1;
	8412	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.2;
	8412	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	892.6;
	8412	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	937;
	8412	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3430;
	8412	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12037.5;
	8412	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5969.3;
	8412	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9869.8;
	8412	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1808.8;
	8413	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.6;
	8413	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	864.5;
	8413	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913.1;
	8413	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3428.8;
	8413	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11768.2;
	8413	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5863.7;
	8413	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9815.6;
	8413	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1764.3;
	8414	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	995.2;
	8414	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	852.4;
	8414	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	901.1;
	8414	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3466;
	8414	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11437.1;
	8414	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5755.9;
	8414	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9802.1;
	8414	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1726.6;
	8415	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.4;
	8415	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	843;
	8415	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	890;
	8415	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3504.6;
	8415	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11032;
	8415	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5645.6;
	8415	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9751;
	8415	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1687.2;
	8416	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989.4;
	8416	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	827;
	8416	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	883.7;
	8416	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3542.3;
	8416	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10646.5;
	8416	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5608.3;
	8416	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9709.7;
	8416	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1649.4;
	8417	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981.4;
	8417	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	814;
	8417	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	883;
	8417	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3557.1;
	8417	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10430.3;
	8417	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5650.4;
	8417	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9745.4;
	8417	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1631.2;
	8418	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	976.6;
	8418	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	833.8;
	8418	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897.9;
	8418	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3601.7;
	8418	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10877.5;
	8418	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5955.1;
	8418	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10096.8;
	8418	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1678.7;
	8419	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	997.9;
	8419	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	849.4;
	8419	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	939.9;
	8419	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3777.3;
	8419	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11266.6;
	8419	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6163.5;
	8419	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10205.4;
	8419	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1678.4;
	8420	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	995.5;
	8420	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	829;
	8420	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	935.4;
	8420	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3718.9;
	8420	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11018.4;
	8420	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6027.1;
	8420	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10052;
	8420	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1625.2;
	8421	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	993.9;
	8421	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	810.5;
	8421	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	918.8;
	8421	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3636.5;
	8421	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10745.8;
	8421	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5933;
	8421	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9921.2;
	8421	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1576.9;
	8422	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	991.4;
	8422	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	791.7;
	8422	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	902;
	8422	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3571.3;
	8422	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10419.9;
	8422	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5735.3;
	8422	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9707.7;
	8422	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1522.1;
	8423	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	977.1;
	8423	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.8;
	8423	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	869.4;
	8423	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3438;
	8423	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9811.1;
	8423	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5408.4;
	8423	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9368.5;
	8423	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1432.5;
	8424	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955;
	8424	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	706.6;
	8424	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	830.1;
	8424	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3222.2;
	8424	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9045.1;
	8424	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5015.5;
	8424	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8890.1;
	8424	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1321.5;
	8425	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.8;
	8425	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	675;
	8425	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	815.5;
	8425	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2986;
	8425	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8372;
	8425	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4630.1;
	8425	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8525.6;
	8425	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1231;
	8426	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	930.9;
	8426	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	655.7;
	8426	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	801.2;
	8426	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2839.4;
	8426	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7992.5;
	8426	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4409.8;
	8426	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8270.5;
	8426	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1172.9;
	8427	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.3;
	8427	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	648;
	8427	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	792.8;
	8427	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2749.1;
	8427	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7779.4;
	8427	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4260.6;
	8427	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8092.6;
	8427	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1135.3;
	8428	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.7;
	8428	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	649.7;
	8428	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	789.9;
	8428	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2687.7;
	8428	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7668.8;
	8428	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4166.9;
	8428	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8005;
	8428	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1113.8;
	8429	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.8;
	8429	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	655.1;
	8429	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	797;
	8429	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2668.9;
	8429	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7690.5;
	8429	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4187.2;
	8429	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8024.2;
	8429	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1121.6;
	8430	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950.6;
	8430	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	676.2;
	8430	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	816.7;
	8430	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2716.4;
	8430	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7918.8;
	8430	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4280.7;
	8430	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8171.6;
	8430	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1148.1;
	8431	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967;
	8431	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	709.1;
	8431	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849.2;
	8431	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2832;
	8431	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8346.7;
	8431	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4500.1;
	8431	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8465;
	8431	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1209.7;
	8432	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	982.4;
	8432	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	758.6;
	8432	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	887.1;
	8432	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2913.3;
	8432	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8828.3;
	8432	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4803.4;
	8432	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8688.7;
	8432	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1267.1;
	8433	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989.3;
	8433	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	801.3;
	8433	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	914.6;
	8433	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3097.6;
	8433	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9240.9;
	8433	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5073.2;
	8433	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9126.2;
	8433	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1342;
	8434	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	994.4;
	8434	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	818.7;
	8434	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	910.9;
	8434	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3338.6;
	8434	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9578.4;
	8434	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5351.1;
	8434	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9608.1;
	8434	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1416.4;
	8435	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986.5;
	8435	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	840.3;
	8435	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	894.7;
	8435	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3555.1;
	8435	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9741.4;
	8435	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5505.4;
	8435	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10023.1;
	8435	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1464;
	8436	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	974.7;
	8436	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	854;
	8436	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	894.2;
	8436	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3719.2;
	8436	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9765.9;
	8436	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5535.7;
	8436	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10320.1;
	8436	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1484.1;
	8437	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.2;
	8437	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	866.1;
	8437	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884.3;
	8437	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3866;
	8437	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9686.5;
	8437	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5502.8;
	8437	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10478.4;
	8437	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1483.1;
	8438	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957;
	8438	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	884.6;
	8438	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	879.9;
	8438	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3988.6;
	8438	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9541.7;
	8438	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5480.8;
	8438	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10439.4;
	8438	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1486.7;
	8439	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942;
	8439	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	911.8;
	8439	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.1;
	8439	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4093.2;
	8439	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9454.1;
	8439	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5462.4;
	8439	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10376.6;
	8439	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1480.3;
	8440	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.2;
	8440	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	940.9;
	8440	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	907.7;
	8440	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4129.7;
	8440	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9628.4;
	8440	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5490.8;
	8440	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10323.8;
	8440	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1477;
	8441	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.3;
	8441	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	991.3;
	8441	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	936.8;
	8441	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4075.5;
	8441	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10363.3;
	8441	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5545.1;
	8441	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10385;
	8441	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1491;
	8442	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986.9;
	8442	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1068.2;
	8442	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	989.4;
	8442	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4006.5;
	8442	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11828.2;
	8442	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5733.1;
	8442	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10813.8;
	8442	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1584.4;
	8443	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1049.6;
	8443	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1119.3;
	8443	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1062.2;
	8443	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4092.5;
	8443	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12984.1;
	8443	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6038.6;
	8443	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11081.3;
	8443	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1630.2;
	8444	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1084.7;
	8444	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1140.3;
	8444	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1087.5;
	8444	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3971.3;
	8444	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13632.3;
	8444	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5894.4;
	8444	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10937.1;
	8444	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1614.7;
	8445	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1102.5;
	8445	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1154.1;
	8445	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1113.8;
	8445	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3835.5;
	8445	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14150.6;
	8445	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5768.1;
	8445	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10737.2;
	8445	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1588.2;
	8446	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1121;
	8446	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1160.7;
	8446	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1140.1;
	8446	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3723.5;
	8446	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14672.5;
	8446	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5668.7;
	8446	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10482.9;
	8446	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1590.3;
	8447	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1124.3;
	8447	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1148.1;
	8447	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1147.1;
	8447	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3570.9;
	8447	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14782.9;
	8447	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5665;
	8447	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9740.3;
	8447	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1594;
	8448	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1117.8;
	8448	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1124.2;
	8448	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1151.3;
	8448	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3357.5;
	8448	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14687.7;
	8448	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5625.5;
	8448	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8777;
	8448	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1593.4;
	8449	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1109.2;
	8449	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1110.9;
	8449	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1150.8;
	8449	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3144;
	8449	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14706.6;
	8449	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5778.2;
	8449	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8104.5;
	8449	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1628.5;
	8450	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1106.6;
	8450	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1114.9;
	8450	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1169.9;
	8450	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2985.7;
	8450	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14745.1;
	8450	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6006.4;
	8450	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7898.3;
	8450	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1673.4;
	8451	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1108.9;
	8451	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1119.5;
	8451	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1189;
	8451	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2899.3;
	8451	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14898.6;
	8451	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6343.2;
	8451	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7877.4;
	8451	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1742.2;
	8452	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1115.5;
	8452	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1131.1;
	8452	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1207.8;
	8452	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2877.4;
	8452	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15125.6;
	8452	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6653.1;
	8452	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8003.8;
	8452	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1818.1;
	8453	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1119.1;
	8453	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1156.5;
	8453	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1244.3;
	8453	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2821;
	8453	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15381.3;
	8453	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6992.2;
	8453	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8269;
	8453	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1894.3;
	8454	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1124.1;
	8454	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1179.5;
	8454	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1281.9;
	8454	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2893.4;
	8454	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15803.8;
	8454	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7361.9;
	8454	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8656.4;
	8454	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1992.4;
	8455	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1129.4;
	8455	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1209.4;
	8455	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1329.9;
	8455	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3095.3;
	8455	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16374.1;
	8455	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7827.2;
	8455	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9186.1;
	8455	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2114;
	8456	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1143.7;
	8456	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1245;
	8456	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1381.2;
	8456	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3357;
	8456	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16967;
	8456	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8362.1;
	8456	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9678.5;
	8456	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2241.2;
	8457	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1151.1;
	8457	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1268.6;
	8457	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1414.8;
	8457	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3711.3;
	8457	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17468;
	8457	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8844.6;
	8457	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10211.4;
	8457	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2338.3;
	8458	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1151.7;
	8458	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1263.5;
	8458	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1421.4;
	8458	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4073.1;
	8458	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17684;
	8458	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9171.5;
	8458	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10725.3;
	8458	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2357.8;
	8459	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1135.5;
	8459	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1230.6;
	8459	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1386.4;
	8459	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4275.6;
	8459	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17316.5;
	8459	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9191.8;
	8459	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10981;
	8459	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2290.9;
	8460	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1111.8;
	8460	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1185.9;
	8460	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1339;
	8460	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4353;
	8460	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16781.5;
	8460	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8951.7;
	8460	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10971.9;
	8460	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2211.9;
	8461	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1092.9;
	8461	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1134.9;
	8461	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1302.7;
	8461	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4343;
	8461	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16175.8;
	8461	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8559.7;
	8461	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10879.7;
	8461	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2147.1;
	8462	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1071.8;
	8462	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1084.5;
	8462	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1266.4;
	8462	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4302.5;
	8462	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15606.3;
	8462	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8270.8;
	8462	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10711.9;
	8462	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2083.8;
	8463	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1051.5;
	8463	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1052.3;
	8463	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1230;
	8463	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4283.9;
	8463	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15158.8;
	8463	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8048.8;
	8463	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10531;
	8463	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2041.7;
	8464	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1040.7;
	8464	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1043.9;
	8464	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1210.7;
	8464	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4319.6;
	8464	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15145.6;
	8464	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7971.1;
	8464	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10485.8;
	8464	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2039.7;
	8465	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1047.8;
	8465	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1068.3;
	8465	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1220.7;
	8465	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4387.6;
	8465	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15537.7;
	8465	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8044.6;
	8465	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10670.7;
	8465	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2102.5;
	8466	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1074.3;
	8466	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1140;
	8466	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1287.4;
	8466	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4602.7;
	8466	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16499.7;
	8466	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8660.5;
	8466	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11418.6;
	8466	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2263.4;
	8467	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1131.7;
	8467	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1192.8;
	8467	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1367.4;
	8467	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4973.7;
	8467	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17309.4;
	8467	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9297.1;
	8467	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11978.6;
	8467	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2379.1;
	8468	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1148.2;
	8468	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1194.2;
	8468	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1381.6;
	8468	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5131.7;
	8468	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17465;
	8468	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9436.9;
	8468	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12073.1;
	8468	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2406.8;
	8469	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1154.6;
	8469	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1192.4;
	8469	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1373.1;
	8469	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5192.6;
	8469	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17441.7;
	8469	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9464.9;
	8469	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12038.5;
	8469	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2410.3;
	8470	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1152.1;
	8470	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1177.6;
	8470	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1359.3;
	8470	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5203.8;
	8470	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17210.1;
	8470	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9336.6;
	8470	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11849.1;
	8470	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2424.1;
	8471	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1136.9;
	8471	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1144.3;
	8471	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1322.3;
	8471	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5115;
	8471	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16710.2;
	8471	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9061.4;
	8471	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11450.8;
	8471	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2330.8;
	8472	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1117.3;
	8472	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1113.9;
	8472	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1282.8;
	8472	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4938.2;
	8472	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16158.4;
	8472	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8743.3;
	8472	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11025.2;
	8472	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2247.3;
	8473	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1090.1;
	8473	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1099.1;
	8473	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1305.1;
	8473	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4723.1;
	8473	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15795.5;
	8473	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8527.1;
	8473	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10680.4;
	8473	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2176.8;
	8474	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1088.6;
	8474	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1094.6;
	8474	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1302.5;
	8474	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4626.9;
	8474	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15700.3;
	8474	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8447.6;
	8474	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10530.1;
	8474	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2257.8;
	8475	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1089.7;
	8475	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1100.9;
	8475	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1314.4;
	8475	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4588.6;
	8475	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15730.7;
	8475	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8467.4;
	8475	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10516.1;
	8475	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2268.8;
	8476	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1095.6;
	8476	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1116.9;
	8476	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1333.1;
	8476	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4626.5;
	8476	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15889.2;
	8476	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8552.5;
	8476	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10623.5;
	8476	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2286.4;
	8477	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1103;
	8477	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1141.4;
	8477	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1354.3;
	8477	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4715.6;
	8477	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16331.4;
	8477	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8824.5;
	8477	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10966.5;
	8477	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2358.9;
	8478	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1122.3;
	8478	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1191.9;
	8478	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1397.5;
	8478	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4987.3;
	8478	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17215.8;
	8478	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9335.3;
	8478	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11593.3;
	8478	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2475.2;
	8479	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1136.3;
	8479	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1258.7;
	8479	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1477.7;
	8479	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5410.5;
	8479	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18227.9;
	8479	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9955.5;
	8479	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12351.5;
	8479	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2631.9;
	8480	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1152.9;
	8480	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1296.4;
	8480	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1529.3;
	8480	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5602.6;
	8480	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18632.5;
	8480	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10204.8;
	8480	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12830.9;
	8480	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2714.9;
	8481	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1142.1;
	8481	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1273.7;
	8481	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1511.6;
	8481	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5638.1;
	8481	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	18229.8;
	8481	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	10023.9;
	8481	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12927;
	8481	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2660.9;
	8482	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1121.7;
	8482	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1228.6;
	8482	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1465.4;
	8482	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5614.7;
	8482	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	17527;
	8482	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9673.7;
	8482	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12881.6;
	8482	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2583;
	8483	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1102.2;
	8483	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1173;
	8483	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1408.7;
	8483	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5497.7;
	8483	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16762.2;
	8483	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	9169.3;
	8483	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12821.6;
	8483	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2450.7;
	8484	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1080.6;
	8484	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1112.7;
	8484	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1340.1;
	8484	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5334.8;
	8484	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15857.6;
	8484	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8629.7;
	8484	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12468.7;
	8484	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2310.3;
	8485	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1055.4;
	8485	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1054.3;
	8485	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1274.8;
	8485	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5136.5;
	8485	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15010.6;
	8485	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8079;
	8485	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11948.6;
	8485	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2226.1;
	8486	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1033;
	8486	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1011.5;
	8486	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1222.6;
	8486	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4948.5;
	8486	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14330.3;
	8486	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7598.4;
	8486	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11534.5;
	8486	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2144.8;
	8487	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1023.2;
	8487	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	976.4;
	8487	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1173.6;
	8487	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4808.8;
	8487	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13962.3;
	8487	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7209.7;
	8487	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11229.3;
	8487	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2069;
	8488	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1023.4;
	8488	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	967.2;
	8488	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1149.5;
	8488	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4777;
	8488	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13858.2;
	8488	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7076.6;
	8488	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11023.4;
	8488	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2034.9;
	8489	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1035.9;
	8489	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	985.4;
	8489	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1157.6;
	8489	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4823.9;
	8489	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14178.8;
	8489	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7214.3;
	8489	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11056.8;
	8489	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2070;
	8490	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1062.9;
	8490	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1049.6;
	8490	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1210;
	8490	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5016.7;
	8490	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15260.8;
	8490	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7820.2;
	8490	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11671.1;
	8490	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2219.6;
	8491	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1113.6;
	8491	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1112.3;
	8491	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1277.1;
	8491	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5305.5;
	8491	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15988.5;
	8491	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8506.8;
	8491	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12139.4;
	8491	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2348.7;
	8492	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1125.2;
	8492	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1122.7;
	8492	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1285.8;
	8492	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5330.5;
	8492	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16020;
	8492	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8588.6;
	8492	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12196.8;
	8492	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2391.6;
	8493	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1129;
	8493	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1120.6;
	8493	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1290.7;
	8493	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5311.4;
	8493	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15972.7;
	8493	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8523;
	8493	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	12156.9;
	8493	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2377.1;
	8494	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1120.1;
	8494	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1102.7;
	8494	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1268.8;
	8494	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	5206.6;
	8494	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15655.7;
	8494	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	8286.3;
	8494	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11835.1;
	8494	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2345.9;
	8495	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1099.1;
	8495	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1061.4;
	8495	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1217.4;
	8495	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4983.5;
	8495	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14892.9;
	8495	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7779.7;
	8495	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11282.1;
	8495	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2232.2;
	8496	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1081.8;
	8496	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1025.6;
	8496	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1173.2;
	8496	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4702.7;
	8496	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14191.6;
	8496	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7274.9;
	8496	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10773.4;
	8496	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2113.9;
	8497	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1082.5;
	8497	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1024.6;
	8497	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1076;
	8497	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4520;
	8497	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14026.3;
	8497	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6189.6;
	8497	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10561.9;
	8497	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2152.4;
	8498	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1074.7;
	8498	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1011.7;
	8498	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1067.4;
	8498	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4370.7;
	8498	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13796.5;
	8498	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6046;
	8498	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10331;
	8498	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2171.5;
	8499	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1073.5;
	8499	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1011.3;
	8499	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1073.6;
	8499	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4305.3;
	8499	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13788.8;
	8499	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5980.1;
	8499	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10197.9;
	8499	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2100.1;
	8500	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1076.7;
	8500	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1015.8;
	8500	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1088.2;
	8500	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4293.7;
	8500	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13979;
	8500	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5990.4;
	8500	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10253;
	8500	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2110.3;
	8501	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1085.3;
	8501	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1035.8;
	8501	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1111.8;
	8501	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4352.4;
	8501	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14351.8;
	8501	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6105;
	8501	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10514.2;
	8501	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2201.8;
	8502	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1096.6;
	8502	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1069.3;
	8502	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1159.3;
	8502	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4525.5;
	8502	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15081.3;
	8502	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6558.1;
	8502	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11021.5;
	8502	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2315.2;
	8503	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1116.7;
	8503	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1124.6;
	8503	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1238.5;
	8503	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4890.8;
	8503	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16055.6;
	8503	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7172.1;
	8503	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11657.3;
	8503	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2417.7;
	8504	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1129;
	8504	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1147.1;
	8504	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1276;
	8504	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4936.1;
	8504	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	16385;
	8504	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7367.5;
	8504	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11893.1;
	8504	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2492;
	8505	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1109.1;
	8505	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1127.4;
	8505	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1242.1;
	8505	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4787.1;
	8505	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15908.4;
	8505	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7195;
	8505	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11933.6;
	8505	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2467.8;
	8506	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1079.6;
	8506	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1086.1;
	8506	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1172.7;
	8506	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4597.9;
	8506	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	15025.8;
	8506	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7009.2;
	8506	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11912.4;
	8506	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2347.3;
	8507	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1053.8;
	8507	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1027.1;
	8507	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1100.4;
	8507	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4351.6;
	8507	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	14200.4;
	8507	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6813.2;
	8507	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11623.1;
	8507	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	2163.5;
	8508	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1024.8;
	8508	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	970.9;
	8508	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1028.5;
	8508	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4101.6;
	8508	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13314.8;
	8508	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6568.9;
	8508	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11166;
	8508	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1998.2;
	8509	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	994.3;
	8509	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	921.9;
	8509	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	967.2;
	8509	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3867.4;
	8509	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12460.6;
	8509	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6292.7;
	8509	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10689;
	8509	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1861.9;
	8510	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	971.6;
	8510	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	881.3;
	8510	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	925.9;
	8510	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3698.1;
	8510	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11792.8;
	8510	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6074.9;
	8510	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10360.2;
	8510	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1814.4;
	8511	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.1;
	8511	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	852.7;
	8511	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	902.4;
	8511	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3576.7;
	8511	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11307.8;
	8511	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5870.1;
	8511	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10109.2;
	8511	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1722.1;
	8512	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948.2;
	8512	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	843.3;
	8512	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884.4;
	8512	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3456.4;
	8512	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11104.6;
	8512	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5788.4;
	8512	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9929.7;
	8512	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1685.2;
	8513	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.6;
	8513	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	861.6;
	8513	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	883.3;
	8513	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3415.1;
	8513	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11237.7;
	8513	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5881;
	8513	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9869.5;
	8513	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1722;
	8514	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.9;
	8514	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	910.3;
	8514	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	917.2;
	8514	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3554.4;
	8514	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12121;
	8514	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6279.9;
	8514	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10352.1;
	8514	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1846.5;
	8515	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	996.4;
	8515	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	960.9;
	8515	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	990.6;
	8515	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3921.6;
	8515	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12914;
	8515	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6795.9;
	8515	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10812.2;
	8515	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1975.4;
	8516	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1014.1;
	8516	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	971.8;
	8516	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1008.2;
	8516	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4019.4;
	8516	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13043.2;
	8516	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6821.3;
	8516	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10821.5;
	8516	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1923.8;
	8517	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1018.3;
	8517	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	974.5;
	8517	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1019;
	8517	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4076.5;
	8517	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13049.6;
	8517	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6767.8;
	8517	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10748.9;
	8517	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1907.1;
	8518	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1018.2;
	8518	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	951.7;
	8518	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1008;
	8518	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4061.6;
	8518	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12814.7;
	8518	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6550.3;
	8518	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10515.9;
	8518	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1851.1;
	8519	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1008;
	8519	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	923.6;
	8519	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	970;
	8519	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3936.9;
	8519	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12201.9;
	8519	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6181.7;
	8519	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10084;
	8519	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1753.9;
	8520	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	987.2;
	8520	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	890.8;
	8520	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	932.6;
	8520	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3724.9;
	8520	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11491.2;
	8520	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5764.7;
	8520	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9562.7;
	8520	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1700.3;
	8521	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	976.4;
	8521	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	863.3;
	8521	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	933.6;
	8521	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3505.4;
	8521	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10824.6;
	8521	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5725.6;
	8521	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9081.2;
	8521	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1615.4;
	8522	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.4;
	8522	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	858.1;
	8522	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	927.5;
	8522	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3417.4;
	8522	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10629.2;
	8522	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5619.7;
	8522	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8840.4;
	8522	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1553.5;
	8523	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972;
	8523	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	856.5;
	8523	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	932.1;
	8523	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3387.9;
	8523	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10586.1;
	8523	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5590.2;
	8523	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8676;
	8523	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1567.4;
	8524	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	982.2;
	8524	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	864;
	8524	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	946.9;
	8524	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3405.8;
	8524	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10706.6;
	8524	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5676.7;
	8524	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8679.2;
	8524	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1572.1;
	8525	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	983.5;
	8525	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	882.5;
	8525	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	971;
	8525	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3464.1;
	8525	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11124.6;
	8525	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5907.9;
	8525	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8870;
	8525	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1606.4;
	8526	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	993.1;
	8526	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	913.8;
	8526	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1018;
	8526	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3636.5;
	8526	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11921.7;
	8526	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6393.3;
	8526	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9363.2;
	8526	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1701.5;
	8527	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1009.3;
	8527	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	969.2;
	8527	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1098.7;
	8527	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3926.7;
	8527	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12945.5;
	8527	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6979.1;
	8527	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9956.2;
	8527	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1836.6;
	8528	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1023.4;
	8528	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	1000.4;
	8528	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1141.1;
	8528	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3988.3;
	8528	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13387.7;
	8528	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7241;
	8528	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10207.5;
	8528	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1895.2;
	8529	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1014.1;
	8529	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	988.6;
	8529	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1120.3;
	8529	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3892.5;
	8529	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	13106.5;
	8529	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	7039.8;
	8529	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10194.9;
	8529	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1872.4;
	8530	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1008.9;
	8530	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	960.1;
	8530	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1082.6;
	8530	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3789.5;
	8530	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12524.6;
	8530	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6674.5;
	8530	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10025.3;
	8530	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1824.9;
	8531	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	998.8;
	8531	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	923.2;
	8531	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1037.6;
	8531	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3658.9;
	8531	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11941;
	8531	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6334.7;
	8531	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9837.1;
	8531	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1770.4;
	8532	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990.8;
	8532	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	880.3;
	8532	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	990.9;
	8532	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3537.3;
	8532	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11514;
	8532	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6011.5;
	8532	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9651.6;
	8532	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1710.7;
	8533	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	971.6;
	8533	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	857.2;
	8533	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	948.7;
	8533	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3436.4;
	8533	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11131.3;
	8533	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5766.8;
	8533	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9539.9;
	8533	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1665;
	8534	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.1;
	8534	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	830.9;
	8534	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	921.5;
	8534	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3350.7;
	8534	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10733.5;
	8534	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5559.4;
	8534	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9534.4;
	8534	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1633.7;
	8535	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.6;
	8535	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	802.7;
	8535	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	904.9;
	8535	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3283;
	8535	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10404;
	8535	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5445.5;
	8535	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9527.4;
	8535	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1596.8;
	8536	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961;
	8536	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	784.9;
	8536	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.2;
	8536	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3250.7;
	8536	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10264.2;
	8536	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5433.1;
	8536	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9478.3;
	8536	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1598.6;
	8537	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.7;
	8537	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	787.8;
	8537	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	886.6;
	8537	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3243.8;
	8537	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10393.5;
	8537	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5460.6;
	8537	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9433.4;
	8537	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1587.8;
	8538	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	971.9;
	8538	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	830.4;
	8538	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	905.6;
	8538	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3335.6;
	8538	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11058.4;
	8538	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5749.6;
	8538	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9795.9;
	8538	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1679;
	8539	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1006.1;
	8539	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	868.4;
	8539	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	964.6;
	8539	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3579.8;
	8539	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11642.7;
	8539	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6191.5;
	8539	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10052.1;
	8539	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1723.6;
	8540	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.5;
	8540	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	869.5;
	8540	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	972.2;
	8540	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3565.7;
	8540	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11541.6;
	8540	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6176.7;
	8540	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9936.4;
	8540	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1740.3;
	8541	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1017.1;
	8541	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	867;
	8541	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	974.2;
	8541	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3524.3;
	8541	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11382.3;
	8541	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6097.9;
	8541	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9773.2;
	8541	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1724.5;
	8542	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013;
	8542	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	849;
	8542	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	956.6;
	8542	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3457.2;
	8542	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11111.4;
	8542	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5928.8;
	8542	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9500.6;
	8542	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1679.3;
	8543	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000;
	8543	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	814.5;
	8543	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	921.1;
	8543	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3336.6;
	8543	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10552.1;
	8543	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5612;
	8543	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9079.6;
	8543	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1581.5;
	8544	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986.2;
	8544	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	789.6;
	8544	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	881.2;
	8544	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3105.4;
	8544	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9894.8;
	8544	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5186.9;
	8544	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8535.9;
	8544	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1402.7;
	8545	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.6;
	8545	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	769.3;
	8545	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	858;
	8545	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2917.4;
	8545	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9389.8;
	8545	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4873.2;
	8545	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8066.6;
	8545	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1408.1;
	8546	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.8;
	8546	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	764.7;
	8546	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851.3;
	8546	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2774.9;
	8546	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9170.9;
	8546	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4684.5;
	8546	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7737.1;
	8546	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1351.4;
	8547	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948;
	8547	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	770.9;
	8547	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.6;
	8547	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2679.6;
	8547	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9174.8;
	8547	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4595.4;
	8547	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7536.2;
	8547	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1276.4;
	8548	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	949.3;
	8548	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	780.7;
	8548	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	855.9;
	8548	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2620.7;
	8548	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9310.2;
	8548	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4584.2;
	8548	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7473.8;
	8548	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1307.5;
	8549	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.7;
	8549	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	803.5;
	8549	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	877;
	8549	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2625.6;
	8549	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9757.5;
	8549	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4712.3;
	8549	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7592.1;
	8549	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1385.2;
	8550	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965;
	8550	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	846.5;
	8550	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	923.1;
	8550	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2731.1;
	8550	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10562.4;
	8550	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5078.6;
	8550	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7908.9;
	8550	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1472.5;
	8551	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	983.7;
	8551	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	906.4;
	8551	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1000.1;
	8551	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2985.3;
	8551	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11580.5;
	8551	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5634.7;
	8551	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8380.9;
	8551	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1631.9;
	8552	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1001;
	8552	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	941.4;
	8552	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1042.5;
	8552	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3088.7;
	8552	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12205.3;
	8552	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5973.8;
	8552	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8662.9;
	8552	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1723.9;
	8553	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1005.6;
	8553	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	945.1;
	8553	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1043.5;
	8553	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3136.2;
	8553	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12209.3;
	8553	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6036.1;
	8553	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8918.8;
	8553	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1741.3;
	8554	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1014.1;
	8554	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	937.5;
	8554	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1039.6;
	8554	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3220.9;
	8554	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12057.1;
	8554	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6000.2;
	8554	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9210.5;
	8554	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1730.7;
	8555	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1008.1;
	8555	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	920.3;
	8555	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1013.5;
	8555	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3292.7;
	8555	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11817.2;
	8555	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5887;
	8555	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9415.8;
	8555	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1671.4;
	8556	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	997;
	8556	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	896.4;
	8556	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	986.3;
	8556	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3347;
	8556	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11388.2;
	8556	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5724.8;
	8556	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9534.2;
	8556	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1642.9;
	8557	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	977.5;
	8557	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	878;
	8557	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	950.2;
	8557	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3395.7;
	8557	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10931;
	8557	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5556.9;
	8557	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9578.5;
	8557	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1583.8;
	8558	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970.4;
	8558	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	889.6;
	8558	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	929.8;
	8558	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3445;
	8558	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10625.8;
	8558	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5439;
	8558	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9638.6;
	8558	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1547.9;
	8559	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960;
	8559	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	871.5;
	8559	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	905.2;
	8559	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3449.8;
	8559	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10405.1;
	8559	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5405;
	8559	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9561.9;
	8559	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1525.5;
	8560	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.1;
	8560	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	853.1;
	8560	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	894.8;
	8560	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3430.6;
	8560	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10325.6;
	8560	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5372.9;
	8560	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9486.4;
	8560	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1453.9;
	8561	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.3;
	8561	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	844;
	8561	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.7;
	8561	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3408.6;
	8561	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10434.4;
	8561	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5408.1;
	8561	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9433.4;
	8561	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1509.4;
	8562	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	977.2;
	8562	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	894.2;
	8562	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	925.5;
	8562	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3466.7;
	8562	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11192.6;
	8562	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5742.8;
	8562	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9819.9;
	8562	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1553.6;
	8563	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1011.4;
	8563	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	935.5;
	8563	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	992.1;
	8563	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3642.1;
	8563	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11842.4;
	8563	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6084.1;
	8563	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10000.7;
	8563	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1617.7;
	8564	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1020;
	8564	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	944;
	8564	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1006.7;
	8564	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3619.1;
	8564	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11947.8;
	8564	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6040;
	8564	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9877.1;
	8564	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1620.4;
	8565	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1027.3;
	8565	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	946.9;
	8565	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1007;
	8565	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3565.4;
	8565	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11897.2;
	8565	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5904.7;
	8565	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9714.2;
	8565	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1679.7;
	8566	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1025.2;
	8566	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	927.8;
	8566	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	990.6;
	8566	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3475.9;
	8566	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11763.1;
	8566	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5787.8;
	8566	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9449.5;
	8566	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1669.6;
	8567	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1014.8;
	8567	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	894.5;
	8567	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	954;
	8567	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3329.6;
	8567	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11325.5;
	8567	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5445.3;
	8567	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9058;
	8567	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1577.7;
	8568	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.9;
	8568	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	853.9;
	8568	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	913.9;
	8568	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3118.7;
	8568	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10778.6;
	8568	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5067.3;
	8568	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8551.9;
	8568	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1486.5;
	8569	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000.3;
	8569	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	825.5;
	8569	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	894.6;
	8569	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2959.3;
	8569	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10221.5;
	8569	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4716.2;
	8569	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8098.3;
	8569	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1345.9;
	8570	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.5;
	8570	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	816.2;
	8570	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	883.2;
	8570	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2809.4;
	8570	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9975;
	8570	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4482.6;
	8570	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7778.4;
	8570	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1305.4;
	8571	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985.9;
	8571	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	811.7;
	8571	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	870.7;
	8571	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2704.7;
	8571	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9756.9;
	8571	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4325.8;
	8571	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7579.1;
	8571	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1280.8;
	8572	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	984.3;
	8572	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	806.8;
	8572	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	870.2;
	8572	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2655;
	8572	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9638.8;
	8572	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4298;
	8572	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7506.7;
	8572	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1267.2;
	8573	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986.6;
	8573	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	809.8;
	8573	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	873.2;
	8573	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2648;
	8573	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9751.9;
	8573	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4402.4;
	8573	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7575.6;
	8573	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1281.9;
	8574	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	985.6;
	8574	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	827.7;
	8574	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.8;
	8574	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2708.8;
	8574	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10134;
	8574	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4668.7;
	8574	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7811.7;
	8574	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1328.4;
	8575	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.3;
	8575	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	860.9;
	8575	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	938.6;
	8575	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2863.5;
	8575	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10662.7;
	8575	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5028.7;
	8575	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8185.1;
	8575	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1420.1;
	8576	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1003.4;
	8576	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	891.4;
	8576	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	977.3;
	8576	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2974;
	8576	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11065;
	8576	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5325.2;
	8576	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8477.8;
	8576	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1488.7;
	8577	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1000.8;
	8577	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	905.7;
	8577	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	983.3;
	8577	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3080.6;
	8577	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11346.8;
	8577	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5532.7;
	8577	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8766;
	8577	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1538.6;
	8578	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	998.9;
	8578	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	916;
	8578	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	989.9;
	8578	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3250.1;
	8578	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11572.9;
	8578	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5664.3;
	8578	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9079.2;
	8578	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1583.2;
	8579	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990.7;
	8579	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	912.6;
	8579	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	975.7;
	8579	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3409.2;
	8579	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11647.5;
	8579	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5727.1;
	8579	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9326.9;
	8579	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1578.8;
	8580	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.3;
	8580	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	895.8;
	8580	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	949;
	8580	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3535.1;
	8580	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11554.9;
	8580	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5689.2;
	8580	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9511.5;
	8580	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1566.4;
	8581	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970.5;
	8581	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	874.9;
	8581	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	917.5;
	8581	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3611.4;
	8581	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11326.9;
	8581	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5576.5;
	8581	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9650;
	8581	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1531.3;
	8582	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.4;
	8582	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	857.9;
	8582	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	894.5;
	8582	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3674.7;
	8582	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11081.7;
	8582	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5460;
	8582	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9769.6;
	8582	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1503.7;
	8583	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.9;
	8583	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	842.8;
	8583	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	873.4;
	8583	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3707;
	8583	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10824.9;
	8583	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5364.3;
	8583	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9867.6;
	8583	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1472.7;
	8584	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.3;
	8584	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	834.1;
	8584	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	862.4;
	8584	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3721.6;
	8584	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10678;
	8584	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5363.5;
	8584	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9925.5;
	8584	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1458.8;
	8585	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.4;
	8585	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	835.1;
	8585	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.9;
	8585	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3694.4;
	8585	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10721;
	8585	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5395;
	8585	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9922.7;
	8585	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1472.6;
	8586	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940;
	8586	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	861.1;
	8586	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	888.1;
	8586	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3669.4;
	8586	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11166.7;
	8586	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5593.8;
	8586	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10171;
	8586	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1542.8;
	8587	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.5;
	8587	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	875;
	8587	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	924.7;
	8587	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3811.2;
	8587	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11350.7;
	8587	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5894;
	8587	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10344.1;
	8587	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1571.2;
	8588	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	978.7;
	8588	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	861.7;
	8588	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	923.2;
	8588	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3744.8;
	8588	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11120.6;
	8588	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5817.1;
	8588	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10207.3;
	8588	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1535.9;
	8589	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	983.4;
	8589	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	851.9;
	8589	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	906.1;
	8589	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3655;
	8589	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10887.5;
	8589	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5735.6;
	8589	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10055.1;
	8589	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1506.7;
	8590	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	981;
	8590	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	834.8;
	8590	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	887.4;
	8590	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3573.3;
	8590	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10556.1;
	8590	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5565.9;
	8590	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9829.1;
	8590	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1460.8;
	8591	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	978.7;
	8591	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	804.1;
	8591	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.3;
	8591	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3453.2;
	8591	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10022.2;
	8591	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5310.2;
	8591	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9515.1;
	8591	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1385.7;
	8592	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	968.1;
	8592	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	769.1;
	8592	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	822.7;
	8592	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3272.5;
	8592	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9379.9;
	8592	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4972;
	8592	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9029.7;
	8592	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1295.8;
	8593	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	962.2;
	8593	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	736.1;
	8593	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	771.5;
	8593	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3046.9;
	8593	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8694.8;
	8593	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4617.7;
	8593	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8533.9;
	8593	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1219.1;
	8594	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.7;
	8594	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	718.1;
	8594	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	752.4;
	8594	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2877.1;
	8594	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8253.7;
	8594	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4364;
	8594	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8187.9;
	8594	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1156.4;
	8595	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	950;
	8595	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	707.9;
	8595	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	738.8;
	8595	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2741;
	8595	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7929.2;
	8595	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4192.6;
	8595	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7891.9;
	8595	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1111.4;
	8596	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	952.1;
	8596	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	712.6;
	8596	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	731.5;
	8596	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2655;
	8596	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7776;
	8596	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4076.5;
	8596	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7723.3;
	8596	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1091.2;
	8597	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951.4;
	8597	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	713.8;
	8597	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	735.7;
	8597	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2606.8;
	8597	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7761.4;
	8597	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4066.5;
	8597	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7657.5;
	8597	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1083.9;
	8598	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.8;
	8598	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.6;
	8598	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	751.1;
	8598	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2621.8;
	8598	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7916.7;
	8598	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4148.1;
	8598	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7718.2;
	8598	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1099.5;
	8599	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.2;
	8599	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	753.2;
	8599	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	780.4;
	8599	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2692.5;
	8599	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8222.2;
	8599	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4322.7;
	8599	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7894.1;
	8599	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1148.3;
	8600	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	973.2;
	8600	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	782.6;
	8600	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	810;
	8600	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2750.3;
	8600	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8612.8;
	8600	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4589.7;
	8600	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8005.4;
	8600	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1195;
	8601	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969.5;
	8601	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	809.5;
	8601	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847;
	8601	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2909.7;
	8601	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9055.3;
	8601	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4868.9;
	8601	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8405.3;
	8601	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1278.7;
	8602	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	964.2;
	8602	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	823.9;
	8602	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	862.8;
	8602	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3153.8;
	8602	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9483;
	8602	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5240.8;
	8602	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8994.5;
	8602	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1365.5;
	8603	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951.5;
	8603	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	823.1;
	8603	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.8;
	8603	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3367.7;
	8603	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9685.8;
	8603	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5467.3;
	8603	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9474;
	8603	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1417;
	8604	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.3;
	8604	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	797.2;
	8604	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	858.6;
	8604	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3533.5;
	8604	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9675.1;
	8604	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5634;
	8604	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9876.8;
	8604	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1438.5;
	8605	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.9;
	8605	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	768.8;
	8605	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	850.3;
	8605	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3653.4;
	8605	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9545.6;
	8605	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5705.4;
	8605	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10233.4;
	8605	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1446.4;
	8606	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.6;
	8606	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.1;
	8606	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	841.3;
	8606	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3738.9;
	8606	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9416.4;
	8606	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5733.8;
	8606	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10448.6;
	8606	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1448.2;
	8607	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.8;
	8607	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.2;
	8607	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	833;
	8607	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3792.1;
	8607	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9365.6;
	8607	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5750.8;
	8607	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10569.3;
	8607	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1443;
	8608	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.9;
	8608	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.3;
	8608	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	832;
	8608	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3819.1;
	8608	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9337.6;
	8608	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5775.1;
	8608	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10557.8;
	8608	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1434.9;
	8609	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.5;
	8609	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	724.2;
	8609	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	834.8;
	8609	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3805.4;
	8609	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9374.1;
	8609	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5802.7;
	8609	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10410.4;
	8609	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1433.1;
	8610	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.7;
	8610	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	746;
	8610	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	847.6;
	8610	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3764.9;
	8610	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9777.9;
	8610	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5983.5;
	8610	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10457.9;
	8610	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1491.5;
	8611	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	944.4;
	8611	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755.4;
	8611	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	875.5;
	8611	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3860.1;
	8611	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10052.9;
	8611	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6107.2;
	8611	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10594.5;
	8611	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1510.4;
	8612	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.7;
	8612	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	742.6;
	8612	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	875.1;
	8612	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3722.2;
	8612	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9846.9;
	8612	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5951.2;
	8612	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10281.1;
	8612	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1476.2;
	8613	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.9;
	8613	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	738.2;
	8613	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	873.1;
	8613	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3554.6;
	8613	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9643.9;
	8613	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5788.3;
	8613	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10037.1;
	8613	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1445.9;
	8614	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	932.8;
	8614	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.4;
	8614	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.3;
	8614	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3411.3;
	8614	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9414.5;
	8614	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5633.8;
	8614	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9812;
	8614	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1412.4;
	8615	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	922.2;
	8615	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	708.7;
	8615	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	853.1;
	8615	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3270.6;
	8615	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9089.6;
	8615	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5415;
	8615	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9544.6;
	8615	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1360;
	8616	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.6;
	8616	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	678.1;
	8616	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824.7;
	8616	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3112.4;
	8616	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8621.8;
	8616	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5110.2;
	8616	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9208.3;
	8616	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1285.4;
	8617	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	891.7;
	8617	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	644.9;
	8617	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	768.3;
	8617	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2979.9;
	8617	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8071.8;
	8617	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4765.4;
	8617	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8797.1;
	8617	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1206.4;
	8618	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	887.4;
	8618	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	623.3;
	8618	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	741.7;
	8618	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2862.8;
	8618	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7634.7;
	8618	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4501.2;
	8618	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8459.6;
	8618	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1146.5;
	8619	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878;
	8619	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	605.1;
	8619	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	725.5;
	8619	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2727.9;
	8619	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7299.6;
	8619	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4296.3;
	8619	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8162;
	8619	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1105;
	8620	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	868;
	8620	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	597.6;
	8620	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	715.6;
	8620	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2632.1;
	8620	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7077.5;
	8620	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4161.4;
	8620	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7957.8;
	8620	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1076;
	8621	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	862.6;
	8621	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	599.1;
	8621	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	707.3;
	8621	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2585.6;
	8621	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	6979.8;
	8621	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4095.6;
	8621	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7867;
	8621	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1066.9;
	8622	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	857.8;
	8622	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	602.4;
	8622	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	714.4;
	8622	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2584;
	8622	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7046.7;
	8622	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4105.9;
	8622	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7861.9;
	8622	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1076.6;
	8623	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	858.5;
	8623	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	621.1;
	8623	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	730.7;
	8623	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2629.4;
	8623	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7302.1;
	8623	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4226.2;
	8623	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7990.7;
	8623	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1116;
	8624	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	867.4;
	8624	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	645.3;
	8624	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	755.6;
	8624	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2655.3;
	8624	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7703.4;
	8624	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4416.4;
	8624	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8157.6;
	8624	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1170.8;
	8625	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	869.6;
	8625	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	675.7;
	8625	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	780.5;
	8625	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2812.8;
	8625	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8284.2;
	8625	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4713.4;
	8625	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8508.6;
	8625	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1253.7;
	8626	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.5;
	8626	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	703.7;
	8626	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	807.1;
	8626	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3074.5;
	8626	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8864.7;
	8626	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5066;
	8626	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9034.8;
	8626	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1343.6;
	8627	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	905.9;
	8627	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	713.5;
	8627	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	820.3;
	8627	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3315.5;
	8627	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9209.8;
	8627	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5295.8;
	8627	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9508.6;
	8627	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1381.2;
	8628	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.1;
	8628	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	714.2;
	8628	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	820.8;
	8628	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3508.5;
	8628	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9446;
	8628	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5468.3;
	8628	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9884.4;
	8628	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1403.2;
	8629	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.1;
	8629	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	705;
	8629	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	817.9;
	8629	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3638.4;
	8629	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9506;
	8629	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5565.1;
	8629	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10067;
	8629	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1408.1;
	8630	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	892.5;
	8630	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	689.4;
	8630	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	808;
	8630	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3697.2;
	8630	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9390.7;
	8630	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5567.1;
	8630	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10085.7;
	8630	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1389.1;
	8631	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	884.2;
	8631	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	669.6;
	8631	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	802;
	8631	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3729.5;
	8631	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9241.2;
	8631	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5513.6;
	8631	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10016.3;
	8631	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1367.5;
	8632	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	876.7;
	8632	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	661.5;
	8632	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	800.3;
	8632	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3717.4;
	8632	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9148.9;
	8632	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5487;
	8632	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9871.7;
	8632	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1350.2;
	8633	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	878.5;
	8633	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	660.4;
	8633	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	800.4;
	8633	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3637.4;
	8633	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9093.8;
	8633	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5466.4;
	8633	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9665.5;
	8633	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1340.2;
	8634	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	886.4;
	8634	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	684.7;
	8634	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	813.1;
	8634	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3548;
	8634	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9395;
	8634	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5544.9;
	8634	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9766.8;
	8634	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1389.3;
	8635	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.5;
	8635	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	710.5;
	8635	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.5;
	8635	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3656.3;
	8635	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9753.2;
	8635	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5794.7;
	8635	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10010.2;
	8635	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1450.5;
	8636	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	927.2;
	8636	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	704;
	8636	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	857.1;
	8636	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3611.3;
	8636	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9659.9;
	8636	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5742.3;
	8636	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9936.4;
	8636	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1440;
	8637	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	928.6;
	8637	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	695.3;
	8637	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	849.6;
	8637	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3532.7;
	8637	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9513.2;
	8637	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5628.7;
	8637	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9832.4;
	8637	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1420.6;
	8638	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.5;
	8638	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	681.9;
	8638	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	834.4;
	8638	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3459.2;
	8638	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9294.9;
	8638	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5508.7;
	8638	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9685.8;
	8638	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1441.7;
	8639	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920.2;
	8639	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	661;
	8639	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	809.3;
	8639	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3369.7;
	8639	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8925.2;
	8639	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5291.9;
	8639	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9421.8;
	8639	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1373.2;
	8640	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.1;
	8640	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	634.1;
	8640	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	777;
	8640	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3190.6;
	8640	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8371.2;
	8640	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4975.5;
	8640	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9039.4;
	8640	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1287;
	8641	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	898.1;
	8641	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	613.1;
	8641	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	744.5;
	8641	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3012.5;
	8641	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7856.7;
	8641	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4662.6;
	8641	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8624.5;
	8641	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1173.9;
	8642	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	900.7;
	8642	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	596.8;
	8642	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	722.4;
	8642	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2858.1;
	8642	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7427.5;
	8642	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4417;
	8642	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8307.3;
	8642	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1124.3;
	8643	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.2;
	8643	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	587.5;
	8643	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	709.4;
	8643	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2738.7;
	8643	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7179.1;
	8643	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4265;
	8643	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8116.4;
	8643	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1095.5;
	8644	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	899.3;
	8644	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	586;
	8644	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	705.7;
	8644	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2681.2;
	8644	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7064.9;
	8644	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4195.4;
	8644	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8027.1;
	8644	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1084.2;
	8645	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.3;
	8645	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	595;
	8645	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	707.9;
	8645	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2677.6;
	8645	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7078.5;
	8645	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4211.5;
	8645	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8042.1;
	8645	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1089.7;
	8646	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920;
	8646	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	611.9;
	8646	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	731.7;
	8646	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2739.4;
	8646	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7264.2;
	8646	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4332.4;
	8646	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8165.6;
	8646	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1122.1;
	8647	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.4;
	8647	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	641.8;
	8647	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	760.1;
	8647	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2849.6;
	8647	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7566.7;
	8647	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4506.7;
	8647	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8341.3;
	8647	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1177.6;
	8648	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.5;
	8648	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	655.8;
	8648	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	786.1;
	8648	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2895.2;
	8648	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7843;
	8648	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4697.2;
	8648	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8391;
	8648	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1220.4;
	8649	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.1;
	8649	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	673.9;
	8649	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	804.4;
	8649	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3011.4;
	8649	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8199.3;
	8649	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4918.7;
	8649	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8616.2;
	8649	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1273.8;
	8650	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.2;
	8650	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	695.6;
	8650	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	826.8;
	8650	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3256.1;
	8650	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8708.5;
	8650	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5302;
	8650	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9147.9;
	8650	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1356.7;
	8651	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.5;
	8651	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	711.7;
	8651	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838.9;
	8651	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3491;
	8651	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9120.5;
	8651	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5641.4;
	8651	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9743.6;
	8651	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1427.5;
	8652	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.7;
	8652	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	712.2;
	8652	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	840;
	8652	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3699.9;
	8652	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9348.1;
	8652	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5950.6;
	8652	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10293.9;
	8652	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1481.7;
	8653	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.4;
	8653	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	703;
	8653	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838.6;
	8653	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3865.4;
	8653	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9409.5;
	8653	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6115.8;
	8653	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10607.6;
	8653	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1511.3;
	8654	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	921;
	8654	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	699.9;
	8654	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	838.6;
	8654	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3953;
	8654	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9405.5;
	8654	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6303.4;
	8654	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10865.9;
	8654	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1519.6;
	8655	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.4;
	8655	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	696.4;
	8655	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	840.4;
	8655	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4007.6;
	8655	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9347.4;
	8655	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6424.7;
	8655	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10958.7;
	8655	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1523.7;
	8656	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.4;
	8656	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	695.1;
	8656	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	856.7;
	8656	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4005.8;
	8656	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9303;
	8656	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6464.3;
	8656	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10984;
	8656	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1522.2;
	8657	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920;
	8657	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	700.5;
	8657	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	863.7;
	8657	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3940;
	8657	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9293.4;
	8657	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6440.7;
	8657	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10758.7;
	8657	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1516.1;
	8658	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	925.3;
	8658	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.5;
	8658	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.6;
	8658	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3917.1;
	8658	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9751.4;
	8658	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6500.5;
	8658	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10881.7;
	8658	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1574.5;
	8659	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	951.6;
	8659	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	763.7;
	8659	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	925;
	8659	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4067;
	8659	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10225.8;
	8659	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6767.7;
	8659	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11012.1;
	8659	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1631.1;
	8660	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	955.8;
	8660	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.6;
	8660	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	922.7;
	8660	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3996.5;
	8660	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10066.8;
	8660	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6655.5;
	8660	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10831.2;
	8660	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1600.9;
	8661	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.5;
	8661	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.6;
	8661	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	908.6;
	8661	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3870.8;
	8661	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9778.8;
	8661	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6453.4;
	8661	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10530.3;
	8661	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1549.4;
	8662	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953;
	8662	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.3;
	8662	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884.1;
	8662	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3739.6;
	8662	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9440.5;
	8662	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6165.9;
	8662	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10253.7;
	8662	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1534.6;
	8663	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.9;
	8663	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	696.5;
	8663	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.7;
	8663	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3565.5;
	8663	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8879.4;
	8663	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5742.2;
	8663	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9807.3;
	8663	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1427;
	8664	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.3;
	8664	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	668;
	8664	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	804.3;
	8664	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3343.9;
	8664	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8244.6;
	8664	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5293.4;
	8664	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9302.1;
	8664	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1313.6;
	8665	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.5;
	8665	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	653.9;
	8665	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	784.3;
	8665	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3039.4;
	8665	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7630.5;
	8665	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4850.7;
	8665	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8782.4;
	8665	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1265;
	8666	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	903.6;
	8666	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	645.6;
	8666	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	755.4;
	8666	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2854.3;
	8666	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7253.4;
	8666	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4557.2;
	8666	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8422.1;
	8666	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1244.4;
	8667	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	904.1;
	8667	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	650.7;
	8667	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	737.6;
	8667	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2757.3;
	8667	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7090;
	8667	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4353.1;
	8667	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8186.7;
	8667	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1136.4;
	8668	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	902.9;
	8668	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	660.9;
	8668	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	738.5;
	8668	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2691.1;
	8668	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7096.3;
	8668	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4267.6;
	8668	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8075.7;
	8668	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1099.6;
	8669	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910;
	8669	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	682.5;
	8669	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	753.5;
	8669	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2682.9;
	8669	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7287.2;
	8669	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4271.9;
	8669	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8133.2;
	8669	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1153.8;
	8670	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	923.6;
	8670	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	721.6;
	8670	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	791.7;
	8670	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2759.1;
	8670	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7817.3;
	8670	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4458.3;
	8670	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8422.3;
	8670	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1222.6;
	8671	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	946.4;
	8671	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	783.4;
	8671	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	851.8;
	8671	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2936.4;
	8671	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8588.9;
	8671	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4766.6;
	8671	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8850.8;
	8671	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1274.9;
	8672	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	969.4;
	8672	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	832.5;
	8672	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	907.5;
	8672	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3037.4;
	8672	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9168.7;
	8672	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5019.7;
	8672	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9110.7;
	8672	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1362.9;
	8673	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.5;
	8673	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	849.9;
	8673	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	931;
	8673	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3159.2;
	8673	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9486.3;
	8673	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5165.8;
	8673	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9411;
	8673	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1434;
	8674	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970.2;
	8674	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	844;
	8674	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	940.9;
	8674	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3350.7;
	8674	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9776.2;
	8674	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5406.9;
	8674	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9881.1;
	8674	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1467.8;
	8675	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.1;
	8675	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	827.8;
	8675	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	935.5;
	8675	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3568.7;
	8675	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9926.9;
	8675	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5582.8;
	8675	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10433.6;
	8675	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1467.4;
	8676	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.6;
	8676	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	803.6;
	8676	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	922.3;
	8676	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3785.8;
	8676	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9920.8;
	8676	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5732.6;
	8676	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10936.3;
	8676	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1479;
	8677	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	941.6;
	8677	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	773.5;
	8677	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	896.3;
	8677	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3994;
	8677	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9815.5;
	8677	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5839;
	8677	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11326.6;
	8677	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1504.4;
	8678	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.2;
	8678	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	759.1;
	8678	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	882.5;
	8678	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4139.7;
	8678	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9728.9;
	8678	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5924;
	8678	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11656.9;
	8678	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1583.7;
	8679	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.9;
	8679	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.2;
	8679	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	868.8;
	8679	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4234;
	8679	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9625.9;
	8679	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6008.3;
	8679	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11889.5;
	8679	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1572.3;
	8680	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924;
	8680	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.2;
	8680	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	863.7;
	8680	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4248.7;
	8680	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9575.9;
	8680	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6068.6;
	8680	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11831.9;
	8680	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1564.4;
	8681	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.5;
	8681	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735.4;
	8681	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.3;
	8681	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4161.3;
	8681	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9590.4;
	8681	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6087;
	8681	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11545.7;
	8681	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1569.6;
	8682	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.3;
	8682	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	770.2;
	8682	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.3;
	8682	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4071.8;
	8682	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10040.2;
	8682	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6179.3;
	8682	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11587.4;
	8682	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1616.5;
	8683	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.4;
	8683	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	809.8;
	8683	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	936.7;
	8683	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4195.6;
	8683	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10524.6;
	8683	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6477.6;
	8683	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11644;
	8683	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1702.8;
	8684	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.8;
	8684	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	809.1;
	8684	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	932.3;
	8684	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4105;
	8684	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10409;
	8684	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6332.2;
	8684	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11351.2;
	8684	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1602;
	8685	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.7;
	8685	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	796.4;
	8685	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	918.4;
	8685	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3959.6;
	8685	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10170.7;
	8685	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6120.5;
	8685	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11009.8;
	8685	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1559.7;
	8686	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	948.5;
	8686	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	773.9;
	8686	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	889.8;
	8686	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3797;
	8686	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9765.7;
	8686	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5811.2;
	8686	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10607.1;
	8686	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1484.6;
	8687	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	936.3;
	8687	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.4;
	8687	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	845;
	8687	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3578.1;
	8687	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9170.3;
	8687	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5426.9;
	8687	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10053.1;
	8687	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1406.3;
	8688	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	916.6;
	8688	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	701.4;
	8688	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	798.5;
	8688	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3298.8;
	8688	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8430.9;
	8688	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4975.3;
	8688	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9423.8;
	8688	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1355.4;
	8689	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918;
	8689	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	679;
	8689	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	742.8;
	8689	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3056.5;
	8689	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7843;
	8689	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4574.8;
	8689	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8914.2;
	8689	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1297.2;
	8690	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913;
	8690	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	667.9;
	8690	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	719.7;
	8690	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2884.3;
	8690	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7501.3;
	8690	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4340.6;
	8690	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8549.8;
	8690	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1220;
	8691	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.6;
	8691	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	658.6;
	8691	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	707.6;
	8691	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2785.9;
	8691	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7303.7;
	8691	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4209.8;
	8691	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8297.1;
	8691	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1220.9;
	8692	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.5;
	8692	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	660.1;
	8692	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	707.4;
	8692	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2735.6;
	8692	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7246.6;
	8692	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4159;
	8692	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8218.7;
	8692	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1207.9;
	8693	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	920.7;
	8693	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	669.2;
	8693	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	713.7;
	8693	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2726.1;
	8693	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7438.9;
	8693	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4202.8;
	8693	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8278.8;
	8693	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1228;
	8694	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.9;
	8694	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	694.3;
	8694	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	734.9;
	8694	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2799.9;
	8694	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7911.1;
	8694	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4432.2;
	8694	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8582.1;
	8694	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1292.3;
	8695	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.3;
	8695	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	732.6;
	8695	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	785;
	8695	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2964.8;
	8695	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8573.6;
	8695	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4790;
	8695	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8995.1;
	8695	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1392.2;
	8696	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	959;
	8696	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	760.5;
	8696	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	820.5;
	8696	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3046.9;
	8696	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8982.3;
	8696	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5050;
	8696	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9277.5;
	8696	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1451.9;
	8697	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.8;
	8697	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	765.8;
	8697	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	832;
	8697	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3152.2;
	8697	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9220.5;
	8697	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5250.7;
	8697	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9544.4;
	8697	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1502.4;
	8698	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.5;
	8698	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772.8;
	8698	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	846.2;
	8698	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3322.2;
	8698	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9500.8;
	8698	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5438.4;
	8698	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9938.6;
	8698	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1566.2;
	8699	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	943.7;
	8699	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	764.6;
	8699	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	855.8;
	8699	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3511.2;
	8699	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9768.3;
	8699	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5692.3;
	8699	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10318.7;
	8699	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1616.4;
	8700	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	947.9;
	8700	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	755.9;
	8700	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	860.7;
	8700	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3735.1;
	8700	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10030.7;
	8700	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5928.3;
	8700	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10650.7;
	8700	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1643.9;
	8701	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	934.6;
	8701	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	743.3;
	8701	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	862.7;
	8701	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3897.4;
	8701	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10203.9;
	8701	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6153.2;
	8701	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11032.3;
	8701	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1658.2;
	8702	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	933.2;
	8702	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.6;
	8702	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	870.9;
	8702	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4005.1;
	8702	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10370.1;
	8702	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6393.4;
	8702	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11363.8;
	8702	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1663.7;
	8703	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	937.2;
	8703	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.5;
	8703	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	884.5;
	8703	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4098.4;
	8703	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10474.2;
	8703	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6555.7;
	8703	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11533.8;
	8703	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1677.6;
	8704	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.2;
	8704	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	728.4;
	8704	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892.2;
	8704	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4141.1;
	8704	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10455.6;
	8704	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6633.5;
	8704	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11589.4;
	8704	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1708.9;
	8705	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	942.2;
	8705	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	731.4;
	8705	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	891.9;
	8705	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4096.3;
	8705	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10413.3;
	8705	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6626;
	8705	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11404.7;
	8705	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1695.3;
	8706	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940.3;
	8706	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	762.7;
	8706	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	892;
	8706	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4026.1;
	8706	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10725.5;
	8706	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6626.4;
	8706	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11347.5;
	8706	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1744.7;
	8707	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.4;
	8707	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	792.1;
	8707	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	918.2;
	8707	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4135.9;
	8707	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11071.3;
	8707	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6835.6;
	8707	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11376.4;
	8707	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1767.2;
	8708	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	968.9;
	8708	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	785.9;
	8708	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	908.1;
	8708	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	4034.3;
	8708	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10700.1;
	8708	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6630.7;
	8708	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	11078;
	8708	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1750.9;
	8709	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.6;
	8709	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	781;
	8709	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	887.1;
	8709	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3908.6;
	8709	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10318.6;
	8709	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6389.4;
	8709	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10773.5;
	8709	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1710.6;
	8710	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	958.8;
	8710	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	762;
	8710	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	854.1;
	8710	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3758.8;
	8710	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9871.3;
	8710	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6057.6;
	8710	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10407;
	8710	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1646.2;
	8711	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	945.5;
	8711	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	727.1;
	8711	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	812.5;
	8711	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3563;
	8711	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9196.3;
	8711	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5635.3;
	8711	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9882.2;
	8711	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1544.2;
	8712	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	931.9;
	8712	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	702.8;
	8712	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	769.3;
	8712	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3300.3;
	8712	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8444.7;
	8712	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5143.7;
	8712	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9304;
	8712	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1345;
	8713	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.6;
	8713	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	675.3;
	8713	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	754.8;
	8713	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3089.8;
	8713	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7805;
	8713	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4698.7;
	8713	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8778.7;
	8713	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1317.7;
	8714	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	911.4;
	8714	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	669.1;
	8714	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	741.6;
	8714	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2941.1;
	8714	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7503.7;
	8714	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4413;
	8714	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8492.6;
	8714	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1241.4;
	8715	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913.9;
	8715	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	671.2;
	8715	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	737.8;
	8715	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2827.8;
	8715	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7364;
	8715	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4207.9;
	8715	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8245.2;
	8715	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1141.4;
	8716	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	917.8;
	8716	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	677.9;
	8716	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	734.8;
	8716	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2753;
	8716	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7375.5;
	8716	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4086.9;
	8716	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8144.9;
	8716	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1132.5;
	8717	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	924.3;
	8717	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	695;
	8717	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	742.1;
	8717	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2737.7;
	8717	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	7637.7;
	8717	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4062.2;
	8717	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8203.6;
	8717	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1168.7;
	8718	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	935.9;
	8718	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	733.3;
	8718	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	776.3;
	8718	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2823.1;
	8718	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8229.4;
	8718	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4182.7;
	8718	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8495.3;
	8718	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1218.4;
	8719	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	956.9;
	8719	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	786.4;
	8719	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	841.6;
	8719	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2999.1;
	8719	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9063.5;
	8719	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4500.3;
	8719	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8850.2;
	8719	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1337.6;
	8720	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	972.3;
	8720	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	827.8;
	8720	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	885.4;
	8720	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3084.9;
	8720	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9647.8;
	8720	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4760.9;
	8720	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8867.9;
	8720	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1419.3;
	8721	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	974.2;
	8721	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	834.6;
	8721	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	907.3;
	8721	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3174.2;
	8721	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10033;
	8721	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4920.9;
	8721	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8947.6;
	8721	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1459.2;
	8722	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986;
	8722	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	823.4;
	8722	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	933.2;
	8722	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3254.8;
	8722	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10254.2;
	8722	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5124.8;
	8722	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9164;
	8722	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1511.6;
	8723	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	990.1;
	8723	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	812.4;
	8723	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	944;
	8723	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3358.3;
	8723	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10402.7;
	8723	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5266.7;
	8723	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9391.3;
	8723	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1518.8;
	8724	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	982;
	8724	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	799.8;
	8724	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	936.2;
	8724	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3445;
	8724	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10341.9;
	8724	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5319.6;
	8724	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9491.8;
	8724	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1541;
	8725	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	965.8;
	8725	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	781.9;
	8725	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	920.1;
	8725	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3499.4;
	8725	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10207.7;
	8725	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5303.8;
	8725	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9448.8;
	8725	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1525.9;
	8726	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.8;
	8726	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	771.2;
	8726	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	899.8;
	8726	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3439.1;
	8726	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10008.1;
	8726	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5244.9;
	8726	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9448.9;
	8726	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1501.4;
	8727	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.8;
	8727	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	769.7;
	8727	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	891;
	8727	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3348.5;
	8727	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9801.9;
	8727	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5213.4;
	8727	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9430;
	8727	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1487.6;
	8728	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960.4;
	8728	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	769.6;
	8728	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	888.9;
	8728	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3280.7;
	8728	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9709.5;
	8728	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5203;
	8728	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9288.3;
	8728	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1430.4;
	8729	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.5;
	8729	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	777.7;
	8729	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	897.4;
	8729	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3237.7;
	8729	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9804.6;
	8729	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5280.9;
	8729	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9112.3;
	8729	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1495.5;
	8730	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	992.5;
	8730	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	827.3;
	8730	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	942.1;
	8730	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3312.8;
	8730	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10596.3;
	8730	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5637.7;
	8730	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9387;
	8730	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1545.1;
	8731	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1027.9;
	8731	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	861.7;
	8731	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1002.1;
	8731	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3526;
	8731	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11242.9;
	8731	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6121.2;
	8731	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9675.2;
	8731	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1646.6;
	8732	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1029.3;
	8732	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	860.1;
	8732	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1006.7;
	8732	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3502.1;
	8732	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11263.7;
	8732	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6080.6;
	8732	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9558.4;
	8732	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1662.5;
	8733	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1032.7;
	8733	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	856.6;
	8733	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1006.8;
	8733	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3449.7;
	8733	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11128.4;
	8733	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6037.3;
	8733	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9423.9;
	8733	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1721.6;
	8734	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1022.5;
	8734	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	837;
	8734	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	989.5;
	8734	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3344.1;
	8734	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10876.6;
	8734	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5864.6;
	8734	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9144.5;
	8734	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1712.3;
	8735	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.4;
	8735	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	807.3;
	8735	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	953.2;
	8735	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3197.2;
	8735	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10326;
	8735	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5573.8;
	8735	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8762.9;
	8735	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1613.8;
	8736	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	978.4;
	8736	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	775.5;
	8736	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	909.1;
	8736	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2986.2;
	8736	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9696.8;
	8736	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5199.6;
	8736	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8322.2;
	8736	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1518.8;
	8737	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970;
	8737	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	774.1;
	8737	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878.3;
	8737	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2814.2;
	8737	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9232.9;
	8737	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4850.1;
	8737	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7961.9;
	8737	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1414.8;
	8738	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	963;
	8738	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	772.6;
	8738	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	861.6;
	8738	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2699.2;
	8738	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9026.7;
	8738	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4663;
	8738	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7757.1;
	8738	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1394.1;
	8739	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	961.2;
	8739	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	782.3;
	8739	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	852.2;
	8739	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2633.1;
	8739	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8984.3;
	8739	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4545.2;
	8739	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7653.2;
	8739	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1385.8;
	8740	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	957.9;
	8740	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	797.3;
	8740	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	836.1;
	8740	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2601.5;
	8740	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9076.7;
	8740	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4553.5;
	8740	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7639.6;
	8740	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1391.1;
	8741	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	960;
	8741	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	819.8;
	8741	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	836.1;
	8741	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2618.5;
	8741	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9364.5;
	8741	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4698.5;
	8741	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7759.2;
	8741	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1419.6;
	8742	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	968.9;
	8742	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	857;
	8742	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	865.5;
	8742	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2718;
	8742	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9993;
	8742	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5050.3;
	8742	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8083.8;
	8742	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1490.6;
	8743	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980;
	8743	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	909.8;
	8743	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	915.8;
	8743	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2925.3;
	8743	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10831.5;
	8743	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5545.1;
	8743	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8571.8;
	8743	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1630.6;
	8744	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	998.8;
	8744	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	952.5;
	8744	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	962.2;
	8744	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3077;
	8744	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11469.4;
	8744	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5862.5;
	8744	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8950.9;
	8744	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1729.5;
	8745	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1004.3;
	8745	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	954;
	8745	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	979.9;
	8745	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3161.4;
	8745	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11631.1;
	8745	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6003.6;
	8745	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9241.8;
	8745	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1764.4;
	8746	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1015.9;
	8746	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	925;
	8746	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	998.8;
	8746	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3247.6;
	8746	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11497.5;
	8746	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6125.2;
	8746	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9504.1;
	8746	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1758.4;
	8747	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1024.3;
	8747	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	882.6;
	8747	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1000.2;
	8747	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3285.7;
	8747	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11214.7;
	8747	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6180.7;
	8747	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9665.1;
	8747	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1701.4;
	8748	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1025.7;
	8748	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	844;
	8748	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	996.4;
	8748	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3325.7;
	8748	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10982.3;
	8748	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6119.2;
	8748	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9707.3;
	8748	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1658.8;
	8749	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1013.7;
	8749	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	807.5;
	8749	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	980.6;
	8749	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3307.3;
	8749	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10742.5;
	8749	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5985.4;
	8749	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9638.4;
	8749	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1601.3;
	8750	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1008.4;
	8750	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	785.9;
	8750	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	967;
	8750	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3292.7;
	8750	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10667.9;
	8750	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5855.5;
	8750	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9570.9;
	8750	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1580.1;
	8751	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.7;
	8751	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	783.3;
	8751	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	957.2;
	8751	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3270.1;
	8751	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10729.8;
	8751	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5787.5;
	8751	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9503.5;
	8751	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1580.4;
	8752	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	999.4;
	8752	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	790.3;
	8752	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	946.9;
	8752	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3249.2;
	8752	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10817.6;
	8752	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5801.3;
	8752	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9519.3;
	8752	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1590.5;
	8753	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1004.8;
	8753	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	804.8;
	8753	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	961.1;
	8753	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3238.9;
	8753	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11118.2;
	8753	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5897.6;
	8753	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9635.3;
	8753	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1632.4;
	8754	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1018.1;
	8754	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	846.3;
	8754	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	992;
	8754	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3316.7;
	8754	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11857;
	8754	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6202.6;
	8754	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10017.2;
	8754	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1741.4;
	8755	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1044;
	8755	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	883.1;
	8755	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1027.9;
	8755	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3534.6;
	8755	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12188.5;
	8755	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6517.9;
	8755	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	10075.7;
	8755	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1785.4;
	8756	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1047.7;
	8756	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	880.5;
	8756	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1024.8;
	8756	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3517.1;
	8756	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	12061.2;
	8756	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6467.1;
	8756	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9914.2;
	8756	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1768.3;
	8757	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1047.8;
	8757	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	877.6;
	8757	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	1010.9;
	8757	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3459.9;
	8757	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11827.7;
	8757	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6390.7;
	8757	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9758.5;
	8757	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1741.2;
	8758	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1040.4;
	8758	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	861.8;
	8758	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	991.3;
	8758	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3379;
	8758	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	11487.1;
	8758	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	6156.3;
	8758	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9517.6;
	8758	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1693.4;
	8759	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1032.2;
	8759	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	835.5;
	8759	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	960.7;
	8759	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3261.8;
	8759	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10991.7;
	8759	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5821.5;
	8759	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9194.7;
	8759	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1622.3;
	8760	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	1015.7;
	8760	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	800.6;
	8760	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	921.2;
	8760	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3086.7;
	8760	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10357.1;
	8760	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5435.2;
	8760	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8731.9;
	8760	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1534.7;
	8761	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	989.6;
	8761	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	771.1;
	8761	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	914.8;
	8761	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2945.4;
	8761	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9795.2;
	8761	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5166.2;
	8761	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8344.1;
	8761	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1496.1;
	8762	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980.1;
	8762	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	752.3;
	8762	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	893.3;
	8762	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2812.4;
	8762	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9435.2;
	8762	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4928.7;
	8762	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8063.4;
	8762	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1437.6;
	8763	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	973.3;
	8763	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	741.2;
	8763	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	877.1;
	8763	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2717.9;
	8763	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9094.9;
	8763	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4774.4;
	8763	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7838.2;
	8763	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1363.7;
	8764	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	970.7;
	8764	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735.5;
	8764	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	867.4;
	8764	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2647.4;
	8764	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8952.9;
	8764	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4678.1;
	8764	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7725.2;
	8764	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1337.4;
	8765	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	968.3;
	8765	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	740.5;
	8765	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	866.6;
	8765	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2607;
	8765	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8917.6;
	8765	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4650;
	8765	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7694;
	8765	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1417.3;
	8766	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.7;
	8766	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	754.1;
	8766	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	878.2;
	8766	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2643.9;
	8766	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9121.1;
	8766	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4711.4;
	8766	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7769.2;
	8766	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1399.4;
	8767	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	975.6;
	8767	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	769.7;
	8767	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	899.3;
	8767	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2727.1;
	8767	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9483.3;
	8767	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4884.4;
	8767	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	7943.5;
	8767	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1450.8;
	8768	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	986.8;
	8768	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	787.3;
	8768	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	924.7;
	8768	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2783.5;
	8768	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9865.5;
	8768	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5073;
	8768	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8118.2;
	8768	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1470.3;
	8769	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	980;
	8769	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	800.6;
	8769	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	943.3;
	8769	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2884.6;
	8769	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10139.1;
	8769	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5235.9;
	8769	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8359.7;
	8769	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1469.8;
	8770	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	967.6;
	8770	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	801.4;
	8770	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	937;
	8770	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3017.3;
	8770	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10286.7;
	8770	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5441.4;
	8770	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8653.9;
	8770	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1491.1;
	8771	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	953.5;
	8771	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	782.2;
	8771	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	911.1;
	8771	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3117.8;
	8771	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	10101.3;
	8771	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5514.8;
	8771	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8888.8;
	8771	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1543.9;
	8772	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.8;
	8772	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	751.5;
	8772	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	888.2;
	8772	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3183.8;
	8772	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9740.6;
	8772	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5488.7;
	8772	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9022.2;
	8772	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1524.9;
	8773	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	929.9;
	8773	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	725.7;
	8773	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.9;
	8773	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3209;
	8773	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9372.9;
	8773	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5338.3;
	8773	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9001.9;
	8773	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1499.8;
	8774	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	919.7;
	8774	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	706.7;
	8774	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	840.2;
	8774	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3229;
	8774	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9036.5;
	8774	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5188.8;
	8774	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8854.6;
	8774	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1460.7;
	8775	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	914.1;
	8775	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	690.9;
	8775	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	826.3;
	8775	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3267.2;
	8775	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8859.6;
	8775	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5137.4;
	8775	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8843.4;
	8775	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1438.4;
	8776	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	910.5;
	8776	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	683.6;
	8776	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	823;
	8776	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3312;
	8776	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8800.2;
	8776	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5191.9;
	8776	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8842.2;
	8776	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1436.8;
	8777	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	909.1;
	8777	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	690;
	8777	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	824.6;
	8777	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3347.7;
	8777	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8875.4;
	8777	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5270.9;
	8777	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8884.1;
	8777	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1430;
	8778	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	913;
	8778	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	722;
	8778	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.3;
	8778	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3417.4;
	8778	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9392.4;
	8778	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5503.1;
	8778	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9235.9;
	8778	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1504.1;
	8779	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	940;
	8779	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	754.9;
	8779	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	888.7;
	8779	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3615;
	8779	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9884.3;
	8779	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5858.5;
	8779	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9560.8;
	8779	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1557;
	8780	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	938.8;
	8780	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	747.7;
	8780	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	876.9;
	8780	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3485.8;
	8780	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9641.1;
	8780	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5673.9;
	8780	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9390.2;
	8780	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1526.9;
	8781	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	926.9;
	8781	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	735.6;
	8781	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	859.1;
	8781	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3287.4;
	8781	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	9314.5;
	8781	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5386.7;
	8781	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	9161.7;
	8781	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1485;
	8782	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	918.7;
	8782	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	730.2;
	8782	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	843.1;
	8782	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	3113.7;
	8782	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8982.8;
	8782	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	5137.5;
	8782	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8870.1;
	8782	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1414.4;
	8783	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	906.6;
	8783	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	720.5;
	8783	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	820.9;
	8783	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2972.6;
	8783	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8671.2;
	8783	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4897.5;
	8783	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8561.9;
	8783	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1376;
	8784	0	CT_TAREALOAD	1	CT_LOAD_ALL_P	CT_REP	894.6;
	8784	0	CT_TAREALOAD	2	CT_LOAD_ALL_P	CT_REP	705.6;
	8784	0	CT_TAREALOAD	3	CT_LOAD_ALL_P	CT_REP	805.5;
	8784	0	CT_TAREALOAD	4	CT_LOAD_ALL_P	CT_REP	2846.8;
	8784	0	CT_TAREALOAD	5	CT_LOAD_ALL_P	CT_REP	8404.1;
	8784	0	CT_TAREALOAD	6	CT_LOAD_ALL_P	CT_REP	4675.8;
	8784	0	CT_TAREALOAD	7	CT_LOAD_ALL_P	CT_REP	8284.2;
	8784	0	CT_TAREALOAD	8	CT_LOAD_ALL_P	CT_REP	1316.4;
];
